// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_ASSEMBLER_COND_RD_RN_OPERAND_RM_RN_IS_SP_IN_IT_BLOCK_ADD_T32_H_
#define VIXL_ASSEMBLER_COND_RD_RN_OPERAND_RM_RN_IS_SP_IN_IT_BLOCK_ADD_T32_H_

const byte kInstruction_add_eq_r0_r13_r0[] = {
  0x08, 0xbf, 0x68, 0x44 // It eq; add eq r0 r13 r0
};
const byte kInstruction_add_eq_r1_r13_r1[] = {
  0x08, 0xbf, 0x69, 0x44 // It eq; add eq r1 r13 r1
};
const byte kInstruction_add_eq_r2_r13_r2[] = {
  0x08, 0xbf, 0x6a, 0x44 // It eq; add eq r2 r13 r2
};
const byte kInstruction_add_eq_r3_r13_r3[] = {
  0x08, 0xbf, 0x6b, 0x44 // It eq; add eq r3 r13 r3
};
const byte kInstruction_add_eq_r4_r13_r4[] = {
  0x08, 0xbf, 0x6c, 0x44 // It eq; add eq r4 r13 r4
};
const byte kInstruction_add_eq_r5_r13_r5[] = {
  0x08, 0xbf, 0x6d, 0x44 // It eq; add eq r5 r13 r5
};
const byte kInstruction_add_eq_r6_r13_r6[] = {
  0x08, 0xbf, 0x6e, 0x44 // It eq; add eq r6 r13 r6
};
const byte kInstruction_add_eq_r7_r13_r7[] = {
  0x08, 0xbf, 0x6f, 0x44 // It eq; add eq r7 r13 r7
};
const byte kInstruction_add_ne_r0_r13_r0[] = {
  0x18, 0xbf, 0x68, 0x44 // It ne; add ne r0 r13 r0
};
const byte kInstruction_add_ne_r1_r13_r1[] = {
  0x18, 0xbf, 0x69, 0x44 // It ne; add ne r1 r13 r1
};
const byte kInstruction_add_ne_r2_r13_r2[] = {
  0x18, 0xbf, 0x6a, 0x44 // It ne; add ne r2 r13 r2
};
const byte kInstruction_add_ne_r3_r13_r3[] = {
  0x18, 0xbf, 0x6b, 0x44 // It ne; add ne r3 r13 r3
};
const byte kInstruction_add_ne_r4_r13_r4[] = {
  0x18, 0xbf, 0x6c, 0x44 // It ne; add ne r4 r13 r4
};
const byte kInstruction_add_ne_r5_r13_r5[] = {
  0x18, 0xbf, 0x6d, 0x44 // It ne; add ne r5 r13 r5
};
const byte kInstruction_add_ne_r6_r13_r6[] = {
  0x18, 0xbf, 0x6e, 0x44 // It ne; add ne r6 r13 r6
};
const byte kInstruction_add_ne_r7_r13_r7[] = {
  0x18, 0xbf, 0x6f, 0x44 // It ne; add ne r7 r13 r7
};
const byte kInstruction_add_cs_r0_r13_r0[] = {
  0x28, 0xbf, 0x68, 0x44 // It cs; add cs r0 r13 r0
};
const byte kInstruction_add_cs_r1_r13_r1[] = {
  0x28, 0xbf, 0x69, 0x44 // It cs; add cs r1 r13 r1
};
const byte kInstruction_add_cs_r2_r13_r2[] = {
  0x28, 0xbf, 0x6a, 0x44 // It cs; add cs r2 r13 r2
};
const byte kInstruction_add_cs_r3_r13_r3[] = {
  0x28, 0xbf, 0x6b, 0x44 // It cs; add cs r3 r13 r3
};
const byte kInstruction_add_cs_r4_r13_r4[] = {
  0x28, 0xbf, 0x6c, 0x44 // It cs; add cs r4 r13 r4
};
const byte kInstruction_add_cs_r5_r13_r5[] = {
  0x28, 0xbf, 0x6d, 0x44 // It cs; add cs r5 r13 r5
};
const byte kInstruction_add_cs_r6_r13_r6[] = {
  0x28, 0xbf, 0x6e, 0x44 // It cs; add cs r6 r13 r6
};
const byte kInstruction_add_cs_r7_r13_r7[] = {
  0x28, 0xbf, 0x6f, 0x44 // It cs; add cs r7 r13 r7
};
const byte kInstruction_add_cc_r0_r13_r0[] = {
  0x38, 0xbf, 0x68, 0x44 // It cc; add cc r0 r13 r0
};
const byte kInstruction_add_cc_r1_r13_r1[] = {
  0x38, 0xbf, 0x69, 0x44 // It cc; add cc r1 r13 r1
};
const byte kInstruction_add_cc_r2_r13_r2[] = {
  0x38, 0xbf, 0x6a, 0x44 // It cc; add cc r2 r13 r2
};
const byte kInstruction_add_cc_r3_r13_r3[] = {
  0x38, 0xbf, 0x6b, 0x44 // It cc; add cc r3 r13 r3
};
const byte kInstruction_add_cc_r4_r13_r4[] = {
  0x38, 0xbf, 0x6c, 0x44 // It cc; add cc r4 r13 r4
};
const byte kInstruction_add_cc_r5_r13_r5[] = {
  0x38, 0xbf, 0x6d, 0x44 // It cc; add cc r5 r13 r5
};
const byte kInstruction_add_cc_r6_r13_r6[] = {
  0x38, 0xbf, 0x6e, 0x44 // It cc; add cc r6 r13 r6
};
const byte kInstruction_add_cc_r7_r13_r7[] = {
  0x38, 0xbf, 0x6f, 0x44 // It cc; add cc r7 r13 r7
};
const byte kInstruction_add_mi_r0_r13_r0[] = {
  0x48, 0xbf, 0x68, 0x44 // It mi; add mi r0 r13 r0
};
const byte kInstruction_add_mi_r1_r13_r1[] = {
  0x48, 0xbf, 0x69, 0x44 // It mi; add mi r1 r13 r1
};
const byte kInstruction_add_mi_r2_r13_r2[] = {
  0x48, 0xbf, 0x6a, 0x44 // It mi; add mi r2 r13 r2
};
const byte kInstruction_add_mi_r3_r13_r3[] = {
  0x48, 0xbf, 0x6b, 0x44 // It mi; add mi r3 r13 r3
};
const byte kInstruction_add_mi_r4_r13_r4[] = {
  0x48, 0xbf, 0x6c, 0x44 // It mi; add mi r4 r13 r4
};
const byte kInstruction_add_mi_r5_r13_r5[] = {
  0x48, 0xbf, 0x6d, 0x44 // It mi; add mi r5 r13 r5
};
const byte kInstruction_add_mi_r6_r13_r6[] = {
  0x48, 0xbf, 0x6e, 0x44 // It mi; add mi r6 r13 r6
};
const byte kInstruction_add_mi_r7_r13_r7[] = {
  0x48, 0xbf, 0x6f, 0x44 // It mi; add mi r7 r13 r7
};
const byte kInstruction_add_pl_r0_r13_r0[] = {
  0x58, 0xbf, 0x68, 0x44 // It pl; add pl r0 r13 r0
};
const byte kInstruction_add_pl_r1_r13_r1[] = {
  0x58, 0xbf, 0x69, 0x44 // It pl; add pl r1 r13 r1
};
const byte kInstruction_add_pl_r2_r13_r2[] = {
  0x58, 0xbf, 0x6a, 0x44 // It pl; add pl r2 r13 r2
};
const byte kInstruction_add_pl_r3_r13_r3[] = {
  0x58, 0xbf, 0x6b, 0x44 // It pl; add pl r3 r13 r3
};
const byte kInstruction_add_pl_r4_r13_r4[] = {
  0x58, 0xbf, 0x6c, 0x44 // It pl; add pl r4 r13 r4
};
const byte kInstruction_add_pl_r5_r13_r5[] = {
  0x58, 0xbf, 0x6d, 0x44 // It pl; add pl r5 r13 r5
};
const byte kInstruction_add_pl_r6_r13_r6[] = {
  0x58, 0xbf, 0x6e, 0x44 // It pl; add pl r6 r13 r6
};
const byte kInstruction_add_pl_r7_r13_r7[] = {
  0x58, 0xbf, 0x6f, 0x44 // It pl; add pl r7 r13 r7
};
const byte kInstruction_add_vs_r0_r13_r0[] = {
  0x68, 0xbf, 0x68, 0x44 // It vs; add vs r0 r13 r0
};
const byte kInstruction_add_vs_r1_r13_r1[] = {
  0x68, 0xbf, 0x69, 0x44 // It vs; add vs r1 r13 r1
};
const byte kInstruction_add_vs_r2_r13_r2[] = {
  0x68, 0xbf, 0x6a, 0x44 // It vs; add vs r2 r13 r2
};
const byte kInstruction_add_vs_r3_r13_r3[] = {
  0x68, 0xbf, 0x6b, 0x44 // It vs; add vs r3 r13 r3
};
const byte kInstruction_add_vs_r4_r13_r4[] = {
  0x68, 0xbf, 0x6c, 0x44 // It vs; add vs r4 r13 r4
};
const byte kInstruction_add_vs_r5_r13_r5[] = {
  0x68, 0xbf, 0x6d, 0x44 // It vs; add vs r5 r13 r5
};
const byte kInstruction_add_vs_r6_r13_r6[] = {
  0x68, 0xbf, 0x6e, 0x44 // It vs; add vs r6 r13 r6
};
const byte kInstruction_add_vs_r7_r13_r7[] = {
  0x68, 0xbf, 0x6f, 0x44 // It vs; add vs r7 r13 r7
};
const byte kInstruction_add_vc_r0_r13_r0[] = {
  0x78, 0xbf, 0x68, 0x44 // It vc; add vc r0 r13 r0
};
const byte kInstruction_add_vc_r1_r13_r1[] = {
  0x78, 0xbf, 0x69, 0x44 // It vc; add vc r1 r13 r1
};
const byte kInstruction_add_vc_r2_r13_r2[] = {
  0x78, 0xbf, 0x6a, 0x44 // It vc; add vc r2 r13 r2
};
const byte kInstruction_add_vc_r3_r13_r3[] = {
  0x78, 0xbf, 0x6b, 0x44 // It vc; add vc r3 r13 r3
};
const byte kInstruction_add_vc_r4_r13_r4[] = {
  0x78, 0xbf, 0x6c, 0x44 // It vc; add vc r4 r13 r4
};
const byte kInstruction_add_vc_r5_r13_r5[] = {
  0x78, 0xbf, 0x6d, 0x44 // It vc; add vc r5 r13 r5
};
const byte kInstruction_add_vc_r6_r13_r6[] = {
  0x78, 0xbf, 0x6e, 0x44 // It vc; add vc r6 r13 r6
};
const byte kInstruction_add_vc_r7_r13_r7[] = {
  0x78, 0xbf, 0x6f, 0x44 // It vc; add vc r7 r13 r7
};
const byte kInstruction_add_hi_r0_r13_r0[] = {
  0x88, 0xbf, 0x68, 0x44 // It hi; add hi r0 r13 r0
};
const byte kInstruction_add_hi_r1_r13_r1[] = {
  0x88, 0xbf, 0x69, 0x44 // It hi; add hi r1 r13 r1
};
const byte kInstruction_add_hi_r2_r13_r2[] = {
  0x88, 0xbf, 0x6a, 0x44 // It hi; add hi r2 r13 r2
};
const byte kInstruction_add_hi_r3_r13_r3[] = {
  0x88, 0xbf, 0x6b, 0x44 // It hi; add hi r3 r13 r3
};
const byte kInstruction_add_hi_r4_r13_r4[] = {
  0x88, 0xbf, 0x6c, 0x44 // It hi; add hi r4 r13 r4
};
const byte kInstruction_add_hi_r5_r13_r5[] = {
  0x88, 0xbf, 0x6d, 0x44 // It hi; add hi r5 r13 r5
};
const byte kInstruction_add_hi_r6_r13_r6[] = {
  0x88, 0xbf, 0x6e, 0x44 // It hi; add hi r6 r13 r6
};
const byte kInstruction_add_hi_r7_r13_r7[] = {
  0x88, 0xbf, 0x6f, 0x44 // It hi; add hi r7 r13 r7
};
const byte kInstruction_add_ls_r0_r13_r0[] = {
  0x98, 0xbf, 0x68, 0x44 // It ls; add ls r0 r13 r0
};
const byte kInstruction_add_ls_r1_r13_r1[] = {
  0x98, 0xbf, 0x69, 0x44 // It ls; add ls r1 r13 r1
};
const byte kInstruction_add_ls_r2_r13_r2[] = {
  0x98, 0xbf, 0x6a, 0x44 // It ls; add ls r2 r13 r2
};
const byte kInstruction_add_ls_r3_r13_r3[] = {
  0x98, 0xbf, 0x6b, 0x44 // It ls; add ls r3 r13 r3
};
const byte kInstruction_add_ls_r4_r13_r4[] = {
  0x98, 0xbf, 0x6c, 0x44 // It ls; add ls r4 r13 r4
};
const byte kInstruction_add_ls_r5_r13_r5[] = {
  0x98, 0xbf, 0x6d, 0x44 // It ls; add ls r5 r13 r5
};
const byte kInstruction_add_ls_r6_r13_r6[] = {
  0x98, 0xbf, 0x6e, 0x44 // It ls; add ls r6 r13 r6
};
const byte kInstruction_add_ls_r7_r13_r7[] = {
  0x98, 0xbf, 0x6f, 0x44 // It ls; add ls r7 r13 r7
};
const byte kInstruction_add_ge_r0_r13_r0[] = {
  0xa8, 0xbf, 0x68, 0x44 // It ge; add ge r0 r13 r0
};
const byte kInstruction_add_ge_r1_r13_r1[] = {
  0xa8, 0xbf, 0x69, 0x44 // It ge; add ge r1 r13 r1
};
const byte kInstruction_add_ge_r2_r13_r2[] = {
  0xa8, 0xbf, 0x6a, 0x44 // It ge; add ge r2 r13 r2
};
const byte kInstruction_add_ge_r3_r13_r3[] = {
  0xa8, 0xbf, 0x6b, 0x44 // It ge; add ge r3 r13 r3
};
const byte kInstruction_add_ge_r4_r13_r4[] = {
  0xa8, 0xbf, 0x6c, 0x44 // It ge; add ge r4 r13 r4
};
const byte kInstruction_add_ge_r5_r13_r5[] = {
  0xa8, 0xbf, 0x6d, 0x44 // It ge; add ge r5 r13 r5
};
const byte kInstruction_add_ge_r6_r13_r6[] = {
  0xa8, 0xbf, 0x6e, 0x44 // It ge; add ge r6 r13 r6
};
const byte kInstruction_add_ge_r7_r13_r7[] = {
  0xa8, 0xbf, 0x6f, 0x44 // It ge; add ge r7 r13 r7
};
const byte kInstruction_add_lt_r0_r13_r0[] = {
  0xb8, 0xbf, 0x68, 0x44 // It lt; add lt r0 r13 r0
};
const byte kInstruction_add_lt_r1_r13_r1[] = {
  0xb8, 0xbf, 0x69, 0x44 // It lt; add lt r1 r13 r1
};
const byte kInstruction_add_lt_r2_r13_r2[] = {
  0xb8, 0xbf, 0x6a, 0x44 // It lt; add lt r2 r13 r2
};
const byte kInstruction_add_lt_r3_r13_r3[] = {
  0xb8, 0xbf, 0x6b, 0x44 // It lt; add lt r3 r13 r3
};
const byte kInstruction_add_lt_r4_r13_r4[] = {
  0xb8, 0xbf, 0x6c, 0x44 // It lt; add lt r4 r13 r4
};
const byte kInstruction_add_lt_r5_r13_r5[] = {
  0xb8, 0xbf, 0x6d, 0x44 // It lt; add lt r5 r13 r5
};
const byte kInstruction_add_lt_r6_r13_r6[] = {
  0xb8, 0xbf, 0x6e, 0x44 // It lt; add lt r6 r13 r6
};
const byte kInstruction_add_lt_r7_r13_r7[] = {
  0xb8, 0xbf, 0x6f, 0x44 // It lt; add lt r7 r13 r7
};
const byte kInstruction_add_gt_r0_r13_r0[] = {
  0xc8, 0xbf, 0x68, 0x44 // It gt; add gt r0 r13 r0
};
const byte kInstruction_add_gt_r1_r13_r1[] = {
  0xc8, 0xbf, 0x69, 0x44 // It gt; add gt r1 r13 r1
};
const byte kInstruction_add_gt_r2_r13_r2[] = {
  0xc8, 0xbf, 0x6a, 0x44 // It gt; add gt r2 r13 r2
};
const byte kInstruction_add_gt_r3_r13_r3[] = {
  0xc8, 0xbf, 0x6b, 0x44 // It gt; add gt r3 r13 r3
};
const byte kInstruction_add_gt_r4_r13_r4[] = {
  0xc8, 0xbf, 0x6c, 0x44 // It gt; add gt r4 r13 r4
};
const byte kInstruction_add_gt_r5_r13_r5[] = {
  0xc8, 0xbf, 0x6d, 0x44 // It gt; add gt r5 r13 r5
};
const byte kInstruction_add_gt_r6_r13_r6[] = {
  0xc8, 0xbf, 0x6e, 0x44 // It gt; add gt r6 r13 r6
};
const byte kInstruction_add_gt_r7_r13_r7[] = {
  0xc8, 0xbf, 0x6f, 0x44 // It gt; add gt r7 r13 r7
};
const byte kInstruction_add_le_r0_r13_r0[] = {
  0xd8, 0xbf, 0x68, 0x44 // It le; add le r0 r13 r0
};
const byte kInstruction_add_le_r1_r13_r1[] = {
  0xd8, 0xbf, 0x69, 0x44 // It le; add le r1 r13 r1
};
const byte kInstruction_add_le_r2_r13_r2[] = {
  0xd8, 0xbf, 0x6a, 0x44 // It le; add le r2 r13 r2
};
const byte kInstruction_add_le_r3_r13_r3[] = {
  0xd8, 0xbf, 0x6b, 0x44 // It le; add le r3 r13 r3
};
const byte kInstruction_add_le_r4_r13_r4[] = {
  0xd8, 0xbf, 0x6c, 0x44 // It le; add le r4 r13 r4
};
const byte kInstruction_add_le_r5_r13_r5[] = {
  0xd8, 0xbf, 0x6d, 0x44 // It le; add le r5 r13 r5
};
const byte kInstruction_add_le_r6_r13_r6[] = {
  0xd8, 0xbf, 0x6e, 0x44 // It le; add le r6 r13 r6
};
const byte kInstruction_add_le_r7_r13_r7[] = {
  0xd8, 0xbf, 0x6f, 0x44 // It le; add le r7 r13 r7
};
const TestResult kReferenceadd[] = {
  {
    ARRAY_SIZE(kInstruction_add_eq_r0_r13_r0),
    kInstruction_add_eq_r0_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r1_r13_r1),
    kInstruction_add_eq_r1_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r2_r13_r2),
    kInstruction_add_eq_r2_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r3_r13_r3),
    kInstruction_add_eq_r3_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r4_r13_r4),
    kInstruction_add_eq_r4_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r5_r13_r5),
    kInstruction_add_eq_r5_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r6_r13_r6),
    kInstruction_add_eq_r6_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r7_r13_r7),
    kInstruction_add_eq_r7_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r13_r0),
    kInstruction_add_ne_r0_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r1_r13_r1),
    kInstruction_add_ne_r1_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r2_r13_r2),
    kInstruction_add_ne_r2_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r3_r13_r3),
    kInstruction_add_ne_r3_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r13_r4),
    kInstruction_add_ne_r4_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r5_r13_r5),
    kInstruction_add_ne_r5_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r6_r13_r6),
    kInstruction_add_ne_r6_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r7_r13_r7),
    kInstruction_add_ne_r7_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r0_r13_r0),
    kInstruction_add_cs_r0_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r1_r13_r1),
    kInstruction_add_cs_r1_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r13_r2),
    kInstruction_add_cs_r2_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r3_r13_r3),
    kInstruction_add_cs_r3_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r4_r13_r4),
    kInstruction_add_cs_r4_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r13_r5),
    kInstruction_add_cs_r5_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r6_r13_r6),
    kInstruction_add_cs_r6_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r7_r13_r7),
    kInstruction_add_cs_r7_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r0_r13_r0),
    kInstruction_add_cc_r0_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r1_r13_r1),
    kInstruction_add_cc_r1_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r2_r13_r2),
    kInstruction_add_cc_r2_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r3_r13_r3),
    kInstruction_add_cc_r3_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r4_r13_r4),
    kInstruction_add_cc_r4_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r5_r13_r5),
    kInstruction_add_cc_r5_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r6_r13_r6),
    kInstruction_add_cc_r6_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r7_r13_r7),
    kInstruction_add_cc_r7_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r0_r13_r0),
    kInstruction_add_mi_r0_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r1_r13_r1),
    kInstruction_add_mi_r1_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r2_r13_r2),
    kInstruction_add_mi_r2_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r3_r13_r3),
    kInstruction_add_mi_r3_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r4_r13_r4),
    kInstruction_add_mi_r4_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r5_r13_r5),
    kInstruction_add_mi_r5_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r6_r13_r6),
    kInstruction_add_mi_r6_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r7_r13_r7),
    kInstruction_add_mi_r7_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r0_r13_r0),
    kInstruction_add_pl_r0_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r1_r13_r1),
    kInstruction_add_pl_r1_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r2_r13_r2),
    kInstruction_add_pl_r2_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r3_r13_r3),
    kInstruction_add_pl_r3_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r4_r13_r4),
    kInstruction_add_pl_r4_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r5_r13_r5),
    kInstruction_add_pl_r5_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r6_r13_r6),
    kInstruction_add_pl_r6_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r7_r13_r7),
    kInstruction_add_pl_r7_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r0_r13_r0),
    kInstruction_add_vs_r0_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r13_r1),
    kInstruction_add_vs_r1_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r2_r13_r2),
    kInstruction_add_vs_r2_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r13_r3),
    kInstruction_add_vs_r3_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r4_r13_r4),
    kInstruction_add_vs_r4_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r5_r13_r5),
    kInstruction_add_vs_r5_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r6_r13_r6),
    kInstruction_add_vs_r6_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r7_r13_r7),
    kInstruction_add_vs_r7_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r0_r13_r0),
    kInstruction_add_vc_r0_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r1_r13_r1),
    kInstruction_add_vc_r1_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r13_r2),
    kInstruction_add_vc_r2_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r3_r13_r3),
    kInstruction_add_vc_r3_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r4_r13_r4),
    kInstruction_add_vc_r4_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r5_r13_r5),
    kInstruction_add_vc_r5_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r13_r6),
    kInstruction_add_vc_r6_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r7_r13_r7),
    kInstruction_add_vc_r7_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r0_r13_r0),
    kInstruction_add_hi_r0_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r1_r13_r1),
    kInstruction_add_hi_r1_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r2_r13_r2),
    kInstruction_add_hi_r2_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r3_r13_r3),
    kInstruction_add_hi_r3_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r4_r13_r4),
    kInstruction_add_hi_r4_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r5_r13_r5),
    kInstruction_add_hi_r5_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r6_r13_r6),
    kInstruction_add_hi_r6_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r7_r13_r7),
    kInstruction_add_hi_r7_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r0_r13_r0),
    kInstruction_add_ls_r0_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r1_r13_r1),
    kInstruction_add_ls_r1_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r2_r13_r2),
    kInstruction_add_ls_r2_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r3_r13_r3),
    kInstruction_add_ls_r3_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r4_r13_r4),
    kInstruction_add_ls_r4_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r5_r13_r5),
    kInstruction_add_ls_r5_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r6_r13_r6),
    kInstruction_add_ls_r6_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r7_r13_r7),
    kInstruction_add_ls_r7_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r0_r13_r0),
    kInstruction_add_ge_r0_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r1_r13_r1),
    kInstruction_add_ge_r1_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r2_r13_r2),
    kInstruction_add_ge_r2_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r3_r13_r3),
    kInstruction_add_ge_r3_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r4_r13_r4),
    kInstruction_add_ge_r4_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r5_r13_r5),
    kInstruction_add_ge_r5_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r13_r6),
    kInstruction_add_ge_r6_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r7_r13_r7),
    kInstruction_add_ge_r7_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r13_r0),
    kInstruction_add_lt_r0_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r1_r13_r1),
    kInstruction_add_lt_r1_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r2_r13_r2),
    kInstruction_add_lt_r2_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r3_r13_r3),
    kInstruction_add_lt_r3_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r4_r13_r4),
    kInstruction_add_lt_r4_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r13_r5),
    kInstruction_add_lt_r5_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r6_r13_r6),
    kInstruction_add_lt_r6_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r7_r13_r7),
    kInstruction_add_lt_r7_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r13_r0),
    kInstruction_add_gt_r0_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r1_r13_r1),
    kInstruction_add_gt_r1_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r2_r13_r2),
    kInstruction_add_gt_r2_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r3_r13_r3),
    kInstruction_add_gt_r3_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r4_r13_r4),
    kInstruction_add_gt_r4_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r5_r13_r5),
    kInstruction_add_gt_r5_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r6_r13_r6),
    kInstruction_add_gt_r6_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r7_r13_r7),
    kInstruction_add_gt_r7_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r13_r0),
    kInstruction_add_le_r0_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r1_r13_r1),
    kInstruction_add_le_r1_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r2_r13_r2),
    kInstruction_add_le_r2_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r3_r13_r3),
    kInstruction_add_le_r3_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r4_r13_r4),
    kInstruction_add_le_r4_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r5_r13_r5),
    kInstruction_add_le_r5_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r6_r13_r6),
    kInstruction_add_le_r6_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r7_r13_r7),
    kInstruction_add_le_r7_r13_r7,
  },
};

#endif  // VIXL_ASSEMBLER_COND_RD_RN_OPERAND_RM_RN_IS_SP_IN_IT_BLOCK_ADD_T32_H_
