// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_SIM_SADDLP_4H_TRACE_AARCH64_H_
#define VIXL_SIM_SADDLP_4H_TRACE_AARCH64_H_

const uint16_t kExpected_NEON_saddlp_4H[] = {
  0x0088, 0x00fb, 0xffff, 0xff03,
  0x00d2, 0x00fd, 0xff01, 0xff05,
  0x00fb, 0xffff, 0xff03, 0xff2d,
  0x00fd, 0xff01, 0xff05, 0xff76,
  0xffff, 0xff03, 0xff2d, 0xffc4,
  0xff01, 0xff05, 0xff76, 0xfff5,
  0xff03, 0xff2d, 0xffc4, 0xfffb,
  0xff05, 0xff76, 0xfff5, 0xfffd,
  0xff2d, 0xffc4, 0xfffb, 0xffff,
  0xff76, 0xfff5, 0xfffd, 0x0001,
  0xffc4, 0xfffb, 0xffff, 0x0003,
  0xfff5, 0xfffd, 0x0001, 0x000a,
  0xfffb, 0xffff, 0x0003, 0x003b,
  0xfffd, 0x0001, 0x000a, 0x0088,
  0xffff, 0x0003, 0x003b, 0x00d2,
  0x0001, 0x000a, 0x0088, 0x00fb,
  0x0003, 0x003b, 0x00d2, 0x00fd,
  0x000a, 0x0088, 0x00fb, 0xffff,
  0x003b, 0x00d2, 0x00fd, 0xff01,
};
const unsigned kExpectedCount_NEON_saddlp_4H = 19;

#endif  // VIXL_SIM_SADDLP_4H_TRACE_AARCH64_H_
