// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_SIM_SQSHL_H_2OPIMM_TRACE_AARCH64_H_
#define VIXL_SIM_SQSHL_H_2OPIMM_TRACE_AARCH64_H_

const uint16_t kExpected_NEON_sqshl_H_2OPIMM[] = {
  0xff81,
  0xff02,
  0xfe04,
  0xfc08,
  0xf810,
  0xf020,
  0xe040,
  0xc080,
  0x8100,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0xff82,
  0xff04,
  0xfe08,
  0xfc10,
  0xf820,
  0xf040,
  0xe080,
  0xc100,
  0x8200,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0xff83,
  0xff06,
  0xfe0c,
  0xfc18,
  0xf830,
  0xf060,
  0xe0c0,
  0xc180,
  0x8300,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0xfff0,
  0xffe0,
  0xffc0,
  0xff80,
  0xff00,
  0xfe00,
  0xfc00,
  0xf800,
  0xf000,
  0xe000,
  0xc000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0xfffd,
  0xfffa,
  0xfff4,
  0xffe8,
  0xffd0,
  0xffa0,
  0xff40,
  0xfe80,
  0xfd00,
  0xfa00,
  0xf400,
  0xe800,
  0xd000,
  0xa000,
  0x8000,
  0x8000,
  0xfffe,
  0xfffc,
  0xfff8,
  0xfff0,
  0xffe0,
  0xffc0,
  0xff80,
  0xff00,
  0xfe00,
  0xfc00,
  0xf800,
  0xf000,
  0xe000,
  0xc000,
  0x8000,
  0x8000,
  0xffff,
  0xfffe,
  0xfffc,
  0xfff8,
  0xfff0,
  0xffe0,
  0xffc0,
  0xff80,
  0xff00,
  0xfe00,
  0xfc00,
  0xf800,
  0xf000,
  0xe000,
  0xc000,
  0x8000,
  0x0000,
  0x0000,
  0x0000,
  0x0000,
  0x0000,
  0x0000,
  0x0000,
  0x0000,
  0x0000,
  0x0000,
  0x0000,
  0x0000,
  0x0000,
  0x0000,
  0x0000,
  0x0000,
  0x0001,
  0x0002,
  0x0004,
  0x0008,
  0x0010,
  0x0020,
  0x0040,
  0x0080,
  0x0100,
  0x0200,
  0x0400,
  0x0800,
  0x1000,
  0x2000,
  0x4000,
  0x7fff,
  0x0002,
  0x0004,
  0x0008,
  0x0010,
  0x0020,
  0x0040,
  0x0080,
  0x0100,
  0x0200,
  0x0400,
  0x0800,
  0x1000,
  0x2000,
  0x4000,
  0x7fff,
  0x7fff,
  0x0010,
  0x0020,
  0x0040,
  0x0080,
  0x0100,
  0x0200,
  0x0400,
  0x0800,
  0x1000,
  0x2000,
  0x4000,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x007d,
  0x00fa,
  0x01f4,
  0x03e8,
  0x07d0,
  0x0fa0,
  0x1f40,
  0x3e80,
  0x7d00,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x007e,
  0x00fc,
  0x01f8,
  0x03f0,
  0x07e0,
  0x0fc0,
  0x1f80,
  0x3f00,
  0x7e00,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x007f,
  0x00fe,
  0x01fc,
  0x03f8,
  0x07f0,
  0x0fe0,
  0x1fc0,
  0x3f80,
  0x7f00,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x3333,
  0x6666,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x5555,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7ffd,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7ffe,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x7fff,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8001,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0xaaaa,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0xcccc,
  0x9998,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0xff80,
  0xff00,
  0xfe00,
  0xfc00,
  0xf800,
  0xf000,
  0xe000,
  0xc000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
  0x8000,
};
const unsigned kExpectedCount_NEON_sqshl_H_2OPIMM = 384;

#endif  // VIXL_SIM_SQSHL_H_2OPIMM_TRACE_AARCH64_H_
