// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_ASSEMBLER_COND_SP_SP_OPERAND_IMM7_SUB_T32_H_
#define VIXL_ASSEMBLER_COND_SP_SP_OPERAND_IMM7_SUB_T32_H_

const byte kInstruction_sub_al_sp_sp_0x0[] = {
  0x80, 0xb0 // sub al sp sp 0x0
};
const byte kInstruction_sub_al_sp_sp_0x4[] = {
  0x81, 0xb0 // sub al sp sp 0x4
};
const byte kInstruction_sub_al_sp_sp_0x8[] = {
  0x82, 0xb0 // sub al sp sp 0x8
};
const byte kInstruction_sub_al_sp_sp_0xc[] = {
  0x83, 0xb0 // sub al sp sp 0xc
};
const byte kInstruction_sub_al_sp_sp_0x10[] = {
  0x84, 0xb0 // sub al sp sp 0x10
};
const byte kInstruction_sub_al_sp_sp_0x14[] = {
  0x85, 0xb0 // sub al sp sp 0x14
};
const byte kInstruction_sub_al_sp_sp_0x18[] = {
  0x86, 0xb0 // sub al sp sp 0x18
};
const byte kInstruction_sub_al_sp_sp_0x1c[] = {
  0x87, 0xb0 // sub al sp sp 0x1c
};
const byte kInstruction_sub_al_sp_sp_0x20[] = {
  0x88, 0xb0 // sub al sp sp 0x20
};
const byte kInstruction_sub_al_sp_sp_0x24[] = {
  0x89, 0xb0 // sub al sp sp 0x24
};
const byte kInstruction_sub_al_sp_sp_0x28[] = {
  0x8a, 0xb0 // sub al sp sp 0x28
};
const byte kInstruction_sub_al_sp_sp_0x2c[] = {
  0x8b, 0xb0 // sub al sp sp 0x2c
};
const byte kInstruction_sub_al_sp_sp_0x30[] = {
  0x8c, 0xb0 // sub al sp sp 0x30
};
const byte kInstruction_sub_al_sp_sp_0x34[] = {
  0x8d, 0xb0 // sub al sp sp 0x34
};
const byte kInstruction_sub_al_sp_sp_0x38[] = {
  0x8e, 0xb0 // sub al sp sp 0x38
};
const byte kInstruction_sub_al_sp_sp_0x3c[] = {
  0x8f, 0xb0 // sub al sp sp 0x3c
};
const byte kInstruction_sub_al_sp_sp_0x40[] = {
  0x90, 0xb0 // sub al sp sp 0x40
};
const byte kInstruction_sub_al_sp_sp_0x44[] = {
  0x91, 0xb0 // sub al sp sp 0x44
};
const byte kInstruction_sub_al_sp_sp_0x48[] = {
  0x92, 0xb0 // sub al sp sp 0x48
};
const byte kInstruction_sub_al_sp_sp_0x4c[] = {
  0x93, 0xb0 // sub al sp sp 0x4c
};
const byte kInstruction_sub_al_sp_sp_0x50[] = {
  0x94, 0xb0 // sub al sp sp 0x50
};
const byte kInstruction_sub_al_sp_sp_0x54[] = {
  0x95, 0xb0 // sub al sp sp 0x54
};
const byte kInstruction_sub_al_sp_sp_0x58[] = {
  0x96, 0xb0 // sub al sp sp 0x58
};
const byte kInstruction_sub_al_sp_sp_0x5c[] = {
  0x97, 0xb0 // sub al sp sp 0x5c
};
const byte kInstruction_sub_al_sp_sp_0x60[] = {
  0x98, 0xb0 // sub al sp sp 0x60
};
const byte kInstruction_sub_al_sp_sp_0x64[] = {
  0x99, 0xb0 // sub al sp sp 0x64
};
const byte kInstruction_sub_al_sp_sp_0x68[] = {
  0x9a, 0xb0 // sub al sp sp 0x68
};
const byte kInstruction_sub_al_sp_sp_0x6c[] = {
  0x9b, 0xb0 // sub al sp sp 0x6c
};
const byte kInstruction_sub_al_sp_sp_0x70[] = {
  0x9c, 0xb0 // sub al sp sp 0x70
};
const byte kInstruction_sub_al_sp_sp_0x74[] = {
  0x9d, 0xb0 // sub al sp sp 0x74
};
const byte kInstruction_sub_al_sp_sp_0x78[] = {
  0x9e, 0xb0 // sub al sp sp 0x78
};
const byte kInstruction_sub_al_sp_sp_0x7c[] = {
  0x9f, 0xb0 // sub al sp sp 0x7c
};
const byte kInstruction_sub_al_sp_sp_0x80[] = {
  0xa0, 0xb0 // sub al sp sp 0x80
};
const byte kInstruction_sub_al_sp_sp_0x84[] = {
  0xa1, 0xb0 // sub al sp sp 0x84
};
const byte kInstruction_sub_al_sp_sp_0x88[] = {
  0xa2, 0xb0 // sub al sp sp 0x88
};
const byte kInstruction_sub_al_sp_sp_0x8c[] = {
  0xa3, 0xb0 // sub al sp sp 0x8c
};
const byte kInstruction_sub_al_sp_sp_0x90[] = {
  0xa4, 0xb0 // sub al sp sp 0x90
};
const byte kInstruction_sub_al_sp_sp_0x94[] = {
  0xa5, 0xb0 // sub al sp sp 0x94
};
const byte kInstruction_sub_al_sp_sp_0x98[] = {
  0xa6, 0xb0 // sub al sp sp 0x98
};
const byte kInstruction_sub_al_sp_sp_0x9c[] = {
  0xa7, 0xb0 // sub al sp sp 0x9c
};
const byte kInstruction_sub_al_sp_sp_0xa0[] = {
  0xa8, 0xb0 // sub al sp sp 0xa0
};
const byte kInstruction_sub_al_sp_sp_0xa4[] = {
  0xa9, 0xb0 // sub al sp sp 0xa4
};
const byte kInstruction_sub_al_sp_sp_0xa8[] = {
  0xaa, 0xb0 // sub al sp sp 0xa8
};
const byte kInstruction_sub_al_sp_sp_0xac[] = {
  0xab, 0xb0 // sub al sp sp 0xac
};
const byte kInstruction_sub_al_sp_sp_0xb0[] = {
  0xac, 0xb0 // sub al sp sp 0xb0
};
const byte kInstruction_sub_al_sp_sp_0xb4[] = {
  0xad, 0xb0 // sub al sp sp 0xb4
};
const byte kInstruction_sub_al_sp_sp_0xb8[] = {
  0xae, 0xb0 // sub al sp sp 0xb8
};
const byte kInstruction_sub_al_sp_sp_0xbc[] = {
  0xaf, 0xb0 // sub al sp sp 0xbc
};
const byte kInstruction_sub_al_sp_sp_0xc0[] = {
  0xb0, 0xb0 // sub al sp sp 0xc0
};
const byte kInstruction_sub_al_sp_sp_0xc4[] = {
  0xb1, 0xb0 // sub al sp sp 0xc4
};
const byte kInstruction_sub_al_sp_sp_0xc8[] = {
  0xb2, 0xb0 // sub al sp sp 0xc8
};
const byte kInstruction_sub_al_sp_sp_0xcc[] = {
  0xb3, 0xb0 // sub al sp sp 0xcc
};
const byte kInstruction_sub_al_sp_sp_0xd0[] = {
  0xb4, 0xb0 // sub al sp sp 0xd0
};
const byte kInstruction_sub_al_sp_sp_0xd4[] = {
  0xb5, 0xb0 // sub al sp sp 0xd4
};
const byte kInstruction_sub_al_sp_sp_0xd8[] = {
  0xb6, 0xb0 // sub al sp sp 0xd8
};
const byte kInstruction_sub_al_sp_sp_0xdc[] = {
  0xb7, 0xb0 // sub al sp sp 0xdc
};
const byte kInstruction_sub_al_sp_sp_0xe0[] = {
  0xb8, 0xb0 // sub al sp sp 0xe0
};
const byte kInstruction_sub_al_sp_sp_0xe4[] = {
  0xb9, 0xb0 // sub al sp sp 0xe4
};
const byte kInstruction_sub_al_sp_sp_0xe8[] = {
  0xba, 0xb0 // sub al sp sp 0xe8
};
const byte kInstruction_sub_al_sp_sp_0xec[] = {
  0xbb, 0xb0 // sub al sp sp 0xec
};
const byte kInstruction_sub_al_sp_sp_0xf0[] = {
  0xbc, 0xb0 // sub al sp sp 0xf0
};
const byte kInstruction_sub_al_sp_sp_0xf4[] = {
  0xbd, 0xb0 // sub al sp sp 0xf4
};
const byte kInstruction_sub_al_sp_sp_0xf8[] = {
  0xbe, 0xb0 // sub al sp sp 0xf8
};
const byte kInstruction_sub_al_sp_sp_0xfc[] = {
  0xbf, 0xb0 // sub al sp sp 0xfc
};
const byte kInstruction_sub_al_sp_sp_0x100[] = {
  0xc0, 0xb0 // sub al sp sp 0x100
};
const byte kInstruction_sub_al_sp_sp_0x104[] = {
  0xc1, 0xb0 // sub al sp sp 0x104
};
const byte kInstruction_sub_al_sp_sp_0x108[] = {
  0xc2, 0xb0 // sub al sp sp 0x108
};
const byte kInstruction_sub_al_sp_sp_0x10c[] = {
  0xc3, 0xb0 // sub al sp sp 0x10c
};
const byte kInstruction_sub_al_sp_sp_0x110[] = {
  0xc4, 0xb0 // sub al sp sp 0x110
};
const byte kInstruction_sub_al_sp_sp_0x114[] = {
  0xc5, 0xb0 // sub al sp sp 0x114
};
const byte kInstruction_sub_al_sp_sp_0x118[] = {
  0xc6, 0xb0 // sub al sp sp 0x118
};
const byte kInstruction_sub_al_sp_sp_0x11c[] = {
  0xc7, 0xb0 // sub al sp sp 0x11c
};
const byte kInstruction_sub_al_sp_sp_0x120[] = {
  0xc8, 0xb0 // sub al sp sp 0x120
};
const byte kInstruction_sub_al_sp_sp_0x124[] = {
  0xc9, 0xb0 // sub al sp sp 0x124
};
const byte kInstruction_sub_al_sp_sp_0x128[] = {
  0xca, 0xb0 // sub al sp sp 0x128
};
const byte kInstruction_sub_al_sp_sp_0x12c[] = {
  0xcb, 0xb0 // sub al sp sp 0x12c
};
const byte kInstruction_sub_al_sp_sp_0x130[] = {
  0xcc, 0xb0 // sub al sp sp 0x130
};
const byte kInstruction_sub_al_sp_sp_0x134[] = {
  0xcd, 0xb0 // sub al sp sp 0x134
};
const byte kInstruction_sub_al_sp_sp_0x138[] = {
  0xce, 0xb0 // sub al sp sp 0x138
};
const byte kInstruction_sub_al_sp_sp_0x13c[] = {
  0xcf, 0xb0 // sub al sp sp 0x13c
};
const byte kInstruction_sub_al_sp_sp_0x140[] = {
  0xd0, 0xb0 // sub al sp sp 0x140
};
const byte kInstruction_sub_al_sp_sp_0x144[] = {
  0xd1, 0xb0 // sub al sp sp 0x144
};
const byte kInstruction_sub_al_sp_sp_0x148[] = {
  0xd2, 0xb0 // sub al sp sp 0x148
};
const byte kInstruction_sub_al_sp_sp_0x14c[] = {
  0xd3, 0xb0 // sub al sp sp 0x14c
};
const byte kInstruction_sub_al_sp_sp_0x150[] = {
  0xd4, 0xb0 // sub al sp sp 0x150
};
const byte kInstruction_sub_al_sp_sp_0x154[] = {
  0xd5, 0xb0 // sub al sp sp 0x154
};
const byte kInstruction_sub_al_sp_sp_0x158[] = {
  0xd6, 0xb0 // sub al sp sp 0x158
};
const byte kInstruction_sub_al_sp_sp_0x15c[] = {
  0xd7, 0xb0 // sub al sp sp 0x15c
};
const byte kInstruction_sub_al_sp_sp_0x160[] = {
  0xd8, 0xb0 // sub al sp sp 0x160
};
const byte kInstruction_sub_al_sp_sp_0x164[] = {
  0xd9, 0xb0 // sub al sp sp 0x164
};
const byte kInstruction_sub_al_sp_sp_0x168[] = {
  0xda, 0xb0 // sub al sp sp 0x168
};
const byte kInstruction_sub_al_sp_sp_0x16c[] = {
  0xdb, 0xb0 // sub al sp sp 0x16c
};
const byte kInstruction_sub_al_sp_sp_0x170[] = {
  0xdc, 0xb0 // sub al sp sp 0x170
};
const byte kInstruction_sub_al_sp_sp_0x174[] = {
  0xdd, 0xb0 // sub al sp sp 0x174
};
const byte kInstruction_sub_al_sp_sp_0x178[] = {
  0xde, 0xb0 // sub al sp sp 0x178
};
const byte kInstruction_sub_al_sp_sp_0x17c[] = {
  0xdf, 0xb0 // sub al sp sp 0x17c
};
const byte kInstruction_sub_al_sp_sp_0x180[] = {
  0xe0, 0xb0 // sub al sp sp 0x180
};
const byte kInstruction_sub_al_sp_sp_0x184[] = {
  0xe1, 0xb0 // sub al sp sp 0x184
};
const byte kInstruction_sub_al_sp_sp_0x188[] = {
  0xe2, 0xb0 // sub al sp sp 0x188
};
const byte kInstruction_sub_al_sp_sp_0x18c[] = {
  0xe3, 0xb0 // sub al sp sp 0x18c
};
const byte kInstruction_sub_al_sp_sp_0x190[] = {
  0xe4, 0xb0 // sub al sp sp 0x190
};
const byte kInstruction_sub_al_sp_sp_0x194[] = {
  0xe5, 0xb0 // sub al sp sp 0x194
};
const byte kInstruction_sub_al_sp_sp_0x198[] = {
  0xe6, 0xb0 // sub al sp sp 0x198
};
const byte kInstruction_sub_al_sp_sp_0x19c[] = {
  0xe7, 0xb0 // sub al sp sp 0x19c
};
const byte kInstruction_sub_al_sp_sp_0x1a0[] = {
  0xe8, 0xb0 // sub al sp sp 0x1a0
};
const byte kInstruction_sub_al_sp_sp_0x1a4[] = {
  0xe9, 0xb0 // sub al sp sp 0x1a4
};
const byte kInstruction_sub_al_sp_sp_0x1a8[] = {
  0xea, 0xb0 // sub al sp sp 0x1a8
};
const byte kInstruction_sub_al_sp_sp_0x1ac[] = {
  0xeb, 0xb0 // sub al sp sp 0x1ac
};
const byte kInstruction_sub_al_sp_sp_0x1b0[] = {
  0xec, 0xb0 // sub al sp sp 0x1b0
};
const byte kInstruction_sub_al_sp_sp_0x1b4[] = {
  0xed, 0xb0 // sub al sp sp 0x1b4
};
const byte kInstruction_sub_al_sp_sp_0x1b8[] = {
  0xee, 0xb0 // sub al sp sp 0x1b8
};
const byte kInstruction_sub_al_sp_sp_0x1bc[] = {
  0xef, 0xb0 // sub al sp sp 0x1bc
};
const byte kInstruction_sub_al_sp_sp_0x1c0[] = {
  0xf0, 0xb0 // sub al sp sp 0x1c0
};
const byte kInstruction_sub_al_sp_sp_0x1c4[] = {
  0xf1, 0xb0 // sub al sp sp 0x1c4
};
const byte kInstruction_sub_al_sp_sp_0x1c8[] = {
  0xf2, 0xb0 // sub al sp sp 0x1c8
};
const byte kInstruction_sub_al_sp_sp_0x1cc[] = {
  0xf3, 0xb0 // sub al sp sp 0x1cc
};
const byte kInstruction_sub_al_sp_sp_0x1d0[] = {
  0xf4, 0xb0 // sub al sp sp 0x1d0
};
const byte kInstruction_sub_al_sp_sp_0x1d4[] = {
  0xf5, 0xb0 // sub al sp sp 0x1d4
};
const byte kInstruction_sub_al_sp_sp_0x1d8[] = {
  0xf6, 0xb0 // sub al sp sp 0x1d8
};
const byte kInstruction_sub_al_sp_sp_0x1dc[] = {
  0xf7, 0xb0 // sub al sp sp 0x1dc
};
const byte kInstruction_sub_al_sp_sp_0x1e0[] = {
  0xf8, 0xb0 // sub al sp sp 0x1e0
};
const byte kInstruction_sub_al_sp_sp_0x1e4[] = {
  0xf9, 0xb0 // sub al sp sp 0x1e4
};
const byte kInstruction_sub_al_sp_sp_0x1e8[] = {
  0xfa, 0xb0 // sub al sp sp 0x1e8
};
const byte kInstruction_sub_al_sp_sp_0x1ec[] = {
  0xfb, 0xb0 // sub al sp sp 0x1ec
};
const byte kInstruction_sub_al_sp_sp_0x1f0[] = {
  0xfc, 0xb0 // sub al sp sp 0x1f0
};
const byte kInstruction_sub_al_sp_sp_0x1f4[] = {
  0xfd, 0xb0 // sub al sp sp 0x1f4
};
const byte kInstruction_sub_al_sp_sp_0x1f8[] = {
  0xfe, 0xb0 // sub al sp sp 0x1f8
};
const byte kInstruction_sub_al_sp_sp_0x1fc[] = {
  0xff, 0xb0 // sub al sp sp 0x1fc
};
const TestResult kReferencesub[] = {
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x0),
    kInstruction_sub_al_sp_sp_0x0,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x4),
    kInstruction_sub_al_sp_sp_0x4,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x8),
    kInstruction_sub_al_sp_sp_0x8,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xc),
    kInstruction_sub_al_sp_sp_0xc,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x10),
    kInstruction_sub_al_sp_sp_0x10,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x14),
    kInstruction_sub_al_sp_sp_0x14,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x18),
    kInstruction_sub_al_sp_sp_0x18,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1c),
    kInstruction_sub_al_sp_sp_0x1c,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x20),
    kInstruction_sub_al_sp_sp_0x20,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x24),
    kInstruction_sub_al_sp_sp_0x24,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x28),
    kInstruction_sub_al_sp_sp_0x28,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x2c),
    kInstruction_sub_al_sp_sp_0x2c,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x30),
    kInstruction_sub_al_sp_sp_0x30,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x34),
    kInstruction_sub_al_sp_sp_0x34,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x38),
    kInstruction_sub_al_sp_sp_0x38,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x3c),
    kInstruction_sub_al_sp_sp_0x3c,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x40),
    kInstruction_sub_al_sp_sp_0x40,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x44),
    kInstruction_sub_al_sp_sp_0x44,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x48),
    kInstruction_sub_al_sp_sp_0x48,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x4c),
    kInstruction_sub_al_sp_sp_0x4c,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x50),
    kInstruction_sub_al_sp_sp_0x50,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x54),
    kInstruction_sub_al_sp_sp_0x54,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x58),
    kInstruction_sub_al_sp_sp_0x58,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x5c),
    kInstruction_sub_al_sp_sp_0x5c,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x60),
    kInstruction_sub_al_sp_sp_0x60,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x64),
    kInstruction_sub_al_sp_sp_0x64,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x68),
    kInstruction_sub_al_sp_sp_0x68,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x6c),
    kInstruction_sub_al_sp_sp_0x6c,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x70),
    kInstruction_sub_al_sp_sp_0x70,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x74),
    kInstruction_sub_al_sp_sp_0x74,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x78),
    kInstruction_sub_al_sp_sp_0x78,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x7c),
    kInstruction_sub_al_sp_sp_0x7c,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x80),
    kInstruction_sub_al_sp_sp_0x80,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x84),
    kInstruction_sub_al_sp_sp_0x84,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x88),
    kInstruction_sub_al_sp_sp_0x88,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x8c),
    kInstruction_sub_al_sp_sp_0x8c,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x90),
    kInstruction_sub_al_sp_sp_0x90,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x94),
    kInstruction_sub_al_sp_sp_0x94,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x98),
    kInstruction_sub_al_sp_sp_0x98,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x9c),
    kInstruction_sub_al_sp_sp_0x9c,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xa0),
    kInstruction_sub_al_sp_sp_0xa0,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xa4),
    kInstruction_sub_al_sp_sp_0xa4,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xa8),
    kInstruction_sub_al_sp_sp_0xa8,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xac),
    kInstruction_sub_al_sp_sp_0xac,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xb0),
    kInstruction_sub_al_sp_sp_0xb0,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xb4),
    kInstruction_sub_al_sp_sp_0xb4,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xb8),
    kInstruction_sub_al_sp_sp_0xb8,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xbc),
    kInstruction_sub_al_sp_sp_0xbc,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xc0),
    kInstruction_sub_al_sp_sp_0xc0,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xc4),
    kInstruction_sub_al_sp_sp_0xc4,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xc8),
    kInstruction_sub_al_sp_sp_0xc8,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xcc),
    kInstruction_sub_al_sp_sp_0xcc,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xd0),
    kInstruction_sub_al_sp_sp_0xd0,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xd4),
    kInstruction_sub_al_sp_sp_0xd4,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xd8),
    kInstruction_sub_al_sp_sp_0xd8,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xdc),
    kInstruction_sub_al_sp_sp_0xdc,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xe0),
    kInstruction_sub_al_sp_sp_0xe0,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xe4),
    kInstruction_sub_al_sp_sp_0xe4,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xe8),
    kInstruction_sub_al_sp_sp_0xe8,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xec),
    kInstruction_sub_al_sp_sp_0xec,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xf0),
    kInstruction_sub_al_sp_sp_0xf0,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xf4),
    kInstruction_sub_al_sp_sp_0xf4,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xf8),
    kInstruction_sub_al_sp_sp_0xf8,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0xfc),
    kInstruction_sub_al_sp_sp_0xfc,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x100),
    kInstruction_sub_al_sp_sp_0x100,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x104),
    kInstruction_sub_al_sp_sp_0x104,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x108),
    kInstruction_sub_al_sp_sp_0x108,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x10c),
    kInstruction_sub_al_sp_sp_0x10c,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x110),
    kInstruction_sub_al_sp_sp_0x110,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x114),
    kInstruction_sub_al_sp_sp_0x114,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x118),
    kInstruction_sub_al_sp_sp_0x118,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x11c),
    kInstruction_sub_al_sp_sp_0x11c,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x120),
    kInstruction_sub_al_sp_sp_0x120,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x124),
    kInstruction_sub_al_sp_sp_0x124,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x128),
    kInstruction_sub_al_sp_sp_0x128,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x12c),
    kInstruction_sub_al_sp_sp_0x12c,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x130),
    kInstruction_sub_al_sp_sp_0x130,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x134),
    kInstruction_sub_al_sp_sp_0x134,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x138),
    kInstruction_sub_al_sp_sp_0x138,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x13c),
    kInstruction_sub_al_sp_sp_0x13c,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x140),
    kInstruction_sub_al_sp_sp_0x140,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x144),
    kInstruction_sub_al_sp_sp_0x144,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x148),
    kInstruction_sub_al_sp_sp_0x148,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x14c),
    kInstruction_sub_al_sp_sp_0x14c,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x150),
    kInstruction_sub_al_sp_sp_0x150,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x154),
    kInstruction_sub_al_sp_sp_0x154,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x158),
    kInstruction_sub_al_sp_sp_0x158,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x15c),
    kInstruction_sub_al_sp_sp_0x15c,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x160),
    kInstruction_sub_al_sp_sp_0x160,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x164),
    kInstruction_sub_al_sp_sp_0x164,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x168),
    kInstruction_sub_al_sp_sp_0x168,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x16c),
    kInstruction_sub_al_sp_sp_0x16c,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x170),
    kInstruction_sub_al_sp_sp_0x170,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x174),
    kInstruction_sub_al_sp_sp_0x174,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x178),
    kInstruction_sub_al_sp_sp_0x178,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x17c),
    kInstruction_sub_al_sp_sp_0x17c,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x180),
    kInstruction_sub_al_sp_sp_0x180,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x184),
    kInstruction_sub_al_sp_sp_0x184,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x188),
    kInstruction_sub_al_sp_sp_0x188,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x18c),
    kInstruction_sub_al_sp_sp_0x18c,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x190),
    kInstruction_sub_al_sp_sp_0x190,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x194),
    kInstruction_sub_al_sp_sp_0x194,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x198),
    kInstruction_sub_al_sp_sp_0x198,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x19c),
    kInstruction_sub_al_sp_sp_0x19c,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1a0),
    kInstruction_sub_al_sp_sp_0x1a0,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1a4),
    kInstruction_sub_al_sp_sp_0x1a4,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1a8),
    kInstruction_sub_al_sp_sp_0x1a8,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1ac),
    kInstruction_sub_al_sp_sp_0x1ac,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1b0),
    kInstruction_sub_al_sp_sp_0x1b0,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1b4),
    kInstruction_sub_al_sp_sp_0x1b4,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1b8),
    kInstruction_sub_al_sp_sp_0x1b8,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1bc),
    kInstruction_sub_al_sp_sp_0x1bc,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1c0),
    kInstruction_sub_al_sp_sp_0x1c0,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1c4),
    kInstruction_sub_al_sp_sp_0x1c4,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1c8),
    kInstruction_sub_al_sp_sp_0x1c8,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1cc),
    kInstruction_sub_al_sp_sp_0x1cc,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1d0),
    kInstruction_sub_al_sp_sp_0x1d0,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1d4),
    kInstruction_sub_al_sp_sp_0x1d4,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1d8),
    kInstruction_sub_al_sp_sp_0x1d8,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1dc),
    kInstruction_sub_al_sp_sp_0x1dc,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1e0),
    kInstruction_sub_al_sp_sp_0x1e0,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1e4),
    kInstruction_sub_al_sp_sp_0x1e4,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1e8),
    kInstruction_sub_al_sp_sp_0x1e8,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1ec),
    kInstruction_sub_al_sp_sp_0x1ec,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1f0),
    kInstruction_sub_al_sp_sp_0x1f0,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1f4),
    kInstruction_sub_al_sp_sp_0x1f4,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1f8),
    kInstruction_sub_al_sp_sp_0x1f8,
  },
  {
    ARRAY_SIZE(kInstruction_sub_al_sp_sp_0x1fc),
    kInstruction_sub_al_sp_sp_0x1fc,
  },
};

#endif  // VIXL_ASSEMBLER_COND_SP_SP_OPERAND_IMM7_SUB_T32_H_
