// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_SIM_SHLL_2D_2OPIMM_TRACE_AARCH64_H_
#define VIXL_SIM_SHLL_2D_2OPIMM_TRACE_AARCH64_H_

const uint64_t kExpected_NEON_shll_2D_2OPIMM[] = {
  0xfffffffd00000000, 0xfffffffe00000000,
  0xfffffffe00000000, 0xffffffff00000000,
  0xffffffff00000000, 0x0000000000000000,
  0x0000000000000000, 0x0000000100000000,
  0x0000000100000000, 0x0000000200000000,
  0x0000000200000000, 0x0000002000000000,
  0x0000002000000000, 0x0000007d00000000,
  0x0000007d00000000, 0x0000007e00000000,
  0x0000007e00000000, 0x0000007f00000000,
  0x0000007f00000000, 0x00007ffd00000000,
  0x00007ffd00000000, 0x00007ffe00000000,
  0x00007ffe00000000, 0x00007fff00000000,
  0x00007fff00000000, 0x3333333300000000,
  0x3333333300000000, 0x5555555500000000,
  0x5555555500000000, 0x7ffffffd00000000,
  0x7ffffffd00000000, 0x7ffffffe00000000,
  0x7ffffffe00000000, 0x7fffffff00000000,
  0x7fffffff00000000, 0x8000000000000000,
  0x8000000000000000, 0x8000000100000000,
  0x8000000100000000, 0xaaaaaaaa00000000,
  0xaaaaaaaa00000000, 0xcccccccc00000000,
  0xcccccccc00000000, 0xffff800000000000,
  0xffff800000000000, 0xffff800100000000,
  0xffff800100000000, 0xffff800200000000,
  0xffff800200000000, 0xffff800300000000,
  0xffff800300000000, 0xffffff8000000000,
  0xffffff8000000000, 0xffffff8100000000,
  0xffffff8100000000, 0xffffff8200000000,
  0xffffff8200000000, 0xffffff8300000000,
  0xffffff8300000000, 0xffffffe000000000,
  0xffffffe000000000, 0xfffffffd00000000,
};
const unsigned kExpectedCount_NEON_shll_2D_2OPIMM = 31;

#endif  // VIXL_SIM_SHLL_2D_2OPIMM_TRACE_AARCH64_H_
