// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_ASSEMBLER_COND_RD_RN_OPERAND_RM_RD_IS_RN_IS_SP_IN_IT_BLOCK_ADD_T32_H_
#define VIXL_ASSEMBLER_COND_RD_RN_OPERAND_RM_RD_IS_RN_IS_SP_IN_IT_BLOCK_ADD_T32_H_

const byte kInstruction_add_eq_r13_r13_r0[] = {
  0x08, 0xbf, 0x85, 0x44 // It eq; add eq r13 r13 r0
};
const byte kInstruction_add_eq_r13_r13_r1[] = {
  0x08, 0xbf, 0x8d, 0x44 // It eq; add eq r13 r13 r1
};
const byte kInstruction_add_eq_r13_r13_r2[] = {
  0x08, 0xbf, 0x95, 0x44 // It eq; add eq r13 r13 r2
};
const byte kInstruction_add_eq_r13_r13_r3[] = {
  0x08, 0xbf, 0x9d, 0x44 // It eq; add eq r13 r13 r3
};
const byte kInstruction_add_eq_r13_r13_r4[] = {
  0x08, 0xbf, 0xa5, 0x44 // It eq; add eq r13 r13 r4
};
const byte kInstruction_add_eq_r13_r13_r5[] = {
  0x08, 0xbf, 0xad, 0x44 // It eq; add eq r13 r13 r5
};
const byte kInstruction_add_eq_r13_r13_r6[] = {
  0x08, 0xbf, 0xb5, 0x44 // It eq; add eq r13 r13 r6
};
const byte kInstruction_add_eq_r13_r13_r7[] = {
  0x08, 0xbf, 0xbd, 0x44 // It eq; add eq r13 r13 r7
};
const byte kInstruction_add_eq_r13_r13_r8[] = {
  0x08, 0xbf, 0xc5, 0x44 // It eq; add eq r13 r13 r8
};
const byte kInstruction_add_eq_r13_r13_r9[] = {
  0x08, 0xbf, 0xcd, 0x44 // It eq; add eq r13 r13 r9
};
const byte kInstruction_add_eq_r13_r13_r10[] = {
  0x08, 0xbf, 0xd5, 0x44 // It eq; add eq r13 r13 r10
};
const byte kInstruction_add_eq_r13_r13_r11[] = {
  0x08, 0xbf, 0xdd, 0x44 // It eq; add eq r13 r13 r11
};
const byte kInstruction_add_eq_r13_r13_r12[] = {
  0x08, 0xbf, 0xe5, 0x44 // It eq; add eq r13 r13 r12
};
const byte kInstruction_add_eq_r13_r13_r13[] = {
  0x08, 0xbf, 0xed, 0x44 // It eq; add eq r13 r13 r13
};
const byte kInstruction_add_eq_r13_r13_r14[] = {
  0x08, 0xbf, 0xf5, 0x44 // It eq; add eq r13 r13 r14
};
const byte kInstruction_add_ne_r13_r13_r0[] = {
  0x18, 0xbf, 0x85, 0x44 // It ne; add ne r13 r13 r0
};
const byte kInstruction_add_ne_r13_r13_r1[] = {
  0x18, 0xbf, 0x8d, 0x44 // It ne; add ne r13 r13 r1
};
const byte kInstruction_add_ne_r13_r13_r2[] = {
  0x18, 0xbf, 0x95, 0x44 // It ne; add ne r13 r13 r2
};
const byte kInstruction_add_ne_r13_r13_r3[] = {
  0x18, 0xbf, 0x9d, 0x44 // It ne; add ne r13 r13 r3
};
const byte kInstruction_add_ne_r13_r13_r4[] = {
  0x18, 0xbf, 0xa5, 0x44 // It ne; add ne r13 r13 r4
};
const byte kInstruction_add_ne_r13_r13_r5[] = {
  0x18, 0xbf, 0xad, 0x44 // It ne; add ne r13 r13 r5
};
const byte kInstruction_add_ne_r13_r13_r6[] = {
  0x18, 0xbf, 0xb5, 0x44 // It ne; add ne r13 r13 r6
};
const byte kInstruction_add_ne_r13_r13_r7[] = {
  0x18, 0xbf, 0xbd, 0x44 // It ne; add ne r13 r13 r7
};
const byte kInstruction_add_ne_r13_r13_r8[] = {
  0x18, 0xbf, 0xc5, 0x44 // It ne; add ne r13 r13 r8
};
const byte kInstruction_add_ne_r13_r13_r9[] = {
  0x18, 0xbf, 0xcd, 0x44 // It ne; add ne r13 r13 r9
};
const byte kInstruction_add_ne_r13_r13_r10[] = {
  0x18, 0xbf, 0xd5, 0x44 // It ne; add ne r13 r13 r10
};
const byte kInstruction_add_ne_r13_r13_r11[] = {
  0x18, 0xbf, 0xdd, 0x44 // It ne; add ne r13 r13 r11
};
const byte kInstruction_add_ne_r13_r13_r12[] = {
  0x18, 0xbf, 0xe5, 0x44 // It ne; add ne r13 r13 r12
};
const byte kInstruction_add_ne_r13_r13_r13[] = {
  0x18, 0xbf, 0xed, 0x44 // It ne; add ne r13 r13 r13
};
const byte kInstruction_add_ne_r13_r13_r14[] = {
  0x18, 0xbf, 0xf5, 0x44 // It ne; add ne r13 r13 r14
};
const byte kInstruction_add_cs_r13_r13_r0[] = {
  0x28, 0xbf, 0x85, 0x44 // It cs; add cs r13 r13 r0
};
const byte kInstruction_add_cs_r13_r13_r1[] = {
  0x28, 0xbf, 0x8d, 0x44 // It cs; add cs r13 r13 r1
};
const byte kInstruction_add_cs_r13_r13_r2[] = {
  0x28, 0xbf, 0x95, 0x44 // It cs; add cs r13 r13 r2
};
const byte kInstruction_add_cs_r13_r13_r3[] = {
  0x28, 0xbf, 0x9d, 0x44 // It cs; add cs r13 r13 r3
};
const byte kInstruction_add_cs_r13_r13_r4[] = {
  0x28, 0xbf, 0xa5, 0x44 // It cs; add cs r13 r13 r4
};
const byte kInstruction_add_cs_r13_r13_r5[] = {
  0x28, 0xbf, 0xad, 0x44 // It cs; add cs r13 r13 r5
};
const byte kInstruction_add_cs_r13_r13_r6[] = {
  0x28, 0xbf, 0xb5, 0x44 // It cs; add cs r13 r13 r6
};
const byte kInstruction_add_cs_r13_r13_r7[] = {
  0x28, 0xbf, 0xbd, 0x44 // It cs; add cs r13 r13 r7
};
const byte kInstruction_add_cs_r13_r13_r8[] = {
  0x28, 0xbf, 0xc5, 0x44 // It cs; add cs r13 r13 r8
};
const byte kInstruction_add_cs_r13_r13_r9[] = {
  0x28, 0xbf, 0xcd, 0x44 // It cs; add cs r13 r13 r9
};
const byte kInstruction_add_cs_r13_r13_r10[] = {
  0x28, 0xbf, 0xd5, 0x44 // It cs; add cs r13 r13 r10
};
const byte kInstruction_add_cs_r13_r13_r11[] = {
  0x28, 0xbf, 0xdd, 0x44 // It cs; add cs r13 r13 r11
};
const byte kInstruction_add_cs_r13_r13_r12[] = {
  0x28, 0xbf, 0xe5, 0x44 // It cs; add cs r13 r13 r12
};
const byte kInstruction_add_cs_r13_r13_r13[] = {
  0x28, 0xbf, 0xed, 0x44 // It cs; add cs r13 r13 r13
};
const byte kInstruction_add_cs_r13_r13_r14[] = {
  0x28, 0xbf, 0xf5, 0x44 // It cs; add cs r13 r13 r14
};
const byte kInstruction_add_cc_r13_r13_r0[] = {
  0x38, 0xbf, 0x85, 0x44 // It cc; add cc r13 r13 r0
};
const byte kInstruction_add_cc_r13_r13_r1[] = {
  0x38, 0xbf, 0x8d, 0x44 // It cc; add cc r13 r13 r1
};
const byte kInstruction_add_cc_r13_r13_r2[] = {
  0x38, 0xbf, 0x95, 0x44 // It cc; add cc r13 r13 r2
};
const byte kInstruction_add_cc_r13_r13_r3[] = {
  0x38, 0xbf, 0x9d, 0x44 // It cc; add cc r13 r13 r3
};
const byte kInstruction_add_cc_r13_r13_r4[] = {
  0x38, 0xbf, 0xa5, 0x44 // It cc; add cc r13 r13 r4
};
const byte kInstruction_add_cc_r13_r13_r5[] = {
  0x38, 0xbf, 0xad, 0x44 // It cc; add cc r13 r13 r5
};
const byte kInstruction_add_cc_r13_r13_r6[] = {
  0x38, 0xbf, 0xb5, 0x44 // It cc; add cc r13 r13 r6
};
const byte kInstruction_add_cc_r13_r13_r7[] = {
  0x38, 0xbf, 0xbd, 0x44 // It cc; add cc r13 r13 r7
};
const byte kInstruction_add_cc_r13_r13_r8[] = {
  0x38, 0xbf, 0xc5, 0x44 // It cc; add cc r13 r13 r8
};
const byte kInstruction_add_cc_r13_r13_r9[] = {
  0x38, 0xbf, 0xcd, 0x44 // It cc; add cc r13 r13 r9
};
const byte kInstruction_add_cc_r13_r13_r10[] = {
  0x38, 0xbf, 0xd5, 0x44 // It cc; add cc r13 r13 r10
};
const byte kInstruction_add_cc_r13_r13_r11[] = {
  0x38, 0xbf, 0xdd, 0x44 // It cc; add cc r13 r13 r11
};
const byte kInstruction_add_cc_r13_r13_r12[] = {
  0x38, 0xbf, 0xe5, 0x44 // It cc; add cc r13 r13 r12
};
const byte kInstruction_add_cc_r13_r13_r13[] = {
  0x38, 0xbf, 0xed, 0x44 // It cc; add cc r13 r13 r13
};
const byte kInstruction_add_cc_r13_r13_r14[] = {
  0x38, 0xbf, 0xf5, 0x44 // It cc; add cc r13 r13 r14
};
const byte kInstruction_add_mi_r13_r13_r0[] = {
  0x48, 0xbf, 0x85, 0x44 // It mi; add mi r13 r13 r0
};
const byte kInstruction_add_mi_r13_r13_r1[] = {
  0x48, 0xbf, 0x8d, 0x44 // It mi; add mi r13 r13 r1
};
const byte kInstruction_add_mi_r13_r13_r2[] = {
  0x48, 0xbf, 0x95, 0x44 // It mi; add mi r13 r13 r2
};
const byte kInstruction_add_mi_r13_r13_r3[] = {
  0x48, 0xbf, 0x9d, 0x44 // It mi; add mi r13 r13 r3
};
const byte kInstruction_add_mi_r13_r13_r4[] = {
  0x48, 0xbf, 0xa5, 0x44 // It mi; add mi r13 r13 r4
};
const byte kInstruction_add_mi_r13_r13_r5[] = {
  0x48, 0xbf, 0xad, 0x44 // It mi; add mi r13 r13 r5
};
const byte kInstruction_add_mi_r13_r13_r6[] = {
  0x48, 0xbf, 0xb5, 0x44 // It mi; add mi r13 r13 r6
};
const byte kInstruction_add_mi_r13_r13_r7[] = {
  0x48, 0xbf, 0xbd, 0x44 // It mi; add mi r13 r13 r7
};
const byte kInstruction_add_mi_r13_r13_r8[] = {
  0x48, 0xbf, 0xc5, 0x44 // It mi; add mi r13 r13 r8
};
const byte kInstruction_add_mi_r13_r13_r9[] = {
  0x48, 0xbf, 0xcd, 0x44 // It mi; add mi r13 r13 r9
};
const byte kInstruction_add_mi_r13_r13_r10[] = {
  0x48, 0xbf, 0xd5, 0x44 // It mi; add mi r13 r13 r10
};
const byte kInstruction_add_mi_r13_r13_r11[] = {
  0x48, 0xbf, 0xdd, 0x44 // It mi; add mi r13 r13 r11
};
const byte kInstruction_add_mi_r13_r13_r12[] = {
  0x48, 0xbf, 0xe5, 0x44 // It mi; add mi r13 r13 r12
};
const byte kInstruction_add_mi_r13_r13_r13[] = {
  0x48, 0xbf, 0xed, 0x44 // It mi; add mi r13 r13 r13
};
const byte kInstruction_add_mi_r13_r13_r14[] = {
  0x48, 0xbf, 0xf5, 0x44 // It mi; add mi r13 r13 r14
};
const byte kInstruction_add_pl_r13_r13_r0[] = {
  0x58, 0xbf, 0x85, 0x44 // It pl; add pl r13 r13 r0
};
const byte kInstruction_add_pl_r13_r13_r1[] = {
  0x58, 0xbf, 0x8d, 0x44 // It pl; add pl r13 r13 r1
};
const byte kInstruction_add_pl_r13_r13_r2[] = {
  0x58, 0xbf, 0x95, 0x44 // It pl; add pl r13 r13 r2
};
const byte kInstruction_add_pl_r13_r13_r3[] = {
  0x58, 0xbf, 0x9d, 0x44 // It pl; add pl r13 r13 r3
};
const byte kInstruction_add_pl_r13_r13_r4[] = {
  0x58, 0xbf, 0xa5, 0x44 // It pl; add pl r13 r13 r4
};
const byte kInstruction_add_pl_r13_r13_r5[] = {
  0x58, 0xbf, 0xad, 0x44 // It pl; add pl r13 r13 r5
};
const byte kInstruction_add_pl_r13_r13_r6[] = {
  0x58, 0xbf, 0xb5, 0x44 // It pl; add pl r13 r13 r6
};
const byte kInstruction_add_pl_r13_r13_r7[] = {
  0x58, 0xbf, 0xbd, 0x44 // It pl; add pl r13 r13 r7
};
const byte kInstruction_add_pl_r13_r13_r8[] = {
  0x58, 0xbf, 0xc5, 0x44 // It pl; add pl r13 r13 r8
};
const byte kInstruction_add_pl_r13_r13_r9[] = {
  0x58, 0xbf, 0xcd, 0x44 // It pl; add pl r13 r13 r9
};
const byte kInstruction_add_pl_r13_r13_r10[] = {
  0x58, 0xbf, 0xd5, 0x44 // It pl; add pl r13 r13 r10
};
const byte kInstruction_add_pl_r13_r13_r11[] = {
  0x58, 0xbf, 0xdd, 0x44 // It pl; add pl r13 r13 r11
};
const byte kInstruction_add_pl_r13_r13_r12[] = {
  0x58, 0xbf, 0xe5, 0x44 // It pl; add pl r13 r13 r12
};
const byte kInstruction_add_pl_r13_r13_r13[] = {
  0x58, 0xbf, 0xed, 0x44 // It pl; add pl r13 r13 r13
};
const byte kInstruction_add_pl_r13_r13_r14[] = {
  0x58, 0xbf, 0xf5, 0x44 // It pl; add pl r13 r13 r14
};
const byte kInstruction_add_vs_r13_r13_r0[] = {
  0x68, 0xbf, 0x85, 0x44 // It vs; add vs r13 r13 r0
};
const byte kInstruction_add_vs_r13_r13_r1[] = {
  0x68, 0xbf, 0x8d, 0x44 // It vs; add vs r13 r13 r1
};
const byte kInstruction_add_vs_r13_r13_r2[] = {
  0x68, 0xbf, 0x95, 0x44 // It vs; add vs r13 r13 r2
};
const byte kInstruction_add_vs_r13_r13_r3[] = {
  0x68, 0xbf, 0x9d, 0x44 // It vs; add vs r13 r13 r3
};
const byte kInstruction_add_vs_r13_r13_r4[] = {
  0x68, 0xbf, 0xa5, 0x44 // It vs; add vs r13 r13 r4
};
const byte kInstruction_add_vs_r13_r13_r5[] = {
  0x68, 0xbf, 0xad, 0x44 // It vs; add vs r13 r13 r5
};
const byte kInstruction_add_vs_r13_r13_r6[] = {
  0x68, 0xbf, 0xb5, 0x44 // It vs; add vs r13 r13 r6
};
const byte kInstruction_add_vs_r13_r13_r7[] = {
  0x68, 0xbf, 0xbd, 0x44 // It vs; add vs r13 r13 r7
};
const byte kInstruction_add_vs_r13_r13_r8[] = {
  0x68, 0xbf, 0xc5, 0x44 // It vs; add vs r13 r13 r8
};
const byte kInstruction_add_vs_r13_r13_r9[] = {
  0x68, 0xbf, 0xcd, 0x44 // It vs; add vs r13 r13 r9
};
const byte kInstruction_add_vs_r13_r13_r10[] = {
  0x68, 0xbf, 0xd5, 0x44 // It vs; add vs r13 r13 r10
};
const byte kInstruction_add_vs_r13_r13_r11[] = {
  0x68, 0xbf, 0xdd, 0x44 // It vs; add vs r13 r13 r11
};
const byte kInstruction_add_vs_r13_r13_r12[] = {
  0x68, 0xbf, 0xe5, 0x44 // It vs; add vs r13 r13 r12
};
const byte kInstruction_add_vs_r13_r13_r13[] = {
  0x68, 0xbf, 0xed, 0x44 // It vs; add vs r13 r13 r13
};
const byte kInstruction_add_vs_r13_r13_r14[] = {
  0x68, 0xbf, 0xf5, 0x44 // It vs; add vs r13 r13 r14
};
const byte kInstruction_add_vc_r13_r13_r0[] = {
  0x78, 0xbf, 0x85, 0x44 // It vc; add vc r13 r13 r0
};
const byte kInstruction_add_vc_r13_r13_r1[] = {
  0x78, 0xbf, 0x8d, 0x44 // It vc; add vc r13 r13 r1
};
const byte kInstruction_add_vc_r13_r13_r2[] = {
  0x78, 0xbf, 0x95, 0x44 // It vc; add vc r13 r13 r2
};
const byte kInstruction_add_vc_r13_r13_r3[] = {
  0x78, 0xbf, 0x9d, 0x44 // It vc; add vc r13 r13 r3
};
const byte kInstruction_add_vc_r13_r13_r4[] = {
  0x78, 0xbf, 0xa5, 0x44 // It vc; add vc r13 r13 r4
};
const byte kInstruction_add_vc_r13_r13_r5[] = {
  0x78, 0xbf, 0xad, 0x44 // It vc; add vc r13 r13 r5
};
const byte kInstruction_add_vc_r13_r13_r6[] = {
  0x78, 0xbf, 0xb5, 0x44 // It vc; add vc r13 r13 r6
};
const byte kInstruction_add_vc_r13_r13_r7[] = {
  0x78, 0xbf, 0xbd, 0x44 // It vc; add vc r13 r13 r7
};
const byte kInstruction_add_vc_r13_r13_r8[] = {
  0x78, 0xbf, 0xc5, 0x44 // It vc; add vc r13 r13 r8
};
const byte kInstruction_add_vc_r13_r13_r9[] = {
  0x78, 0xbf, 0xcd, 0x44 // It vc; add vc r13 r13 r9
};
const byte kInstruction_add_vc_r13_r13_r10[] = {
  0x78, 0xbf, 0xd5, 0x44 // It vc; add vc r13 r13 r10
};
const byte kInstruction_add_vc_r13_r13_r11[] = {
  0x78, 0xbf, 0xdd, 0x44 // It vc; add vc r13 r13 r11
};
const byte kInstruction_add_vc_r13_r13_r12[] = {
  0x78, 0xbf, 0xe5, 0x44 // It vc; add vc r13 r13 r12
};
const byte kInstruction_add_vc_r13_r13_r13[] = {
  0x78, 0xbf, 0xed, 0x44 // It vc; add vc r13 r13 r13
};
const byte kInstruction_add_vc_r13_r13_r14[] = {
  0x78, 0xbf, 0xf5, 0x44 // It vc; add vc r13 r13 r14
};
const byte kInstruction_add_hi_r13_r13_r0[] = {
  0x88, 0xbf, 0x85, 0x44 // It hi; add hi r13 r13 r0
};
const byte kInstruction_add_hi_r13_r13_r1[] = {
  0x88, 0xbf, 0x8d, 0x44 // It hi; add hi r13 r13 r1
};
const byte kInstruction_add_hi_r13_r13_r2[] = {
  0x88, 0xbf, 0x95, 0x44 // It hi; add hi r13 r13 r2
};
const byte kInstruction_add_hi_r13_r13_r3[] = {
  0x88, 0xbf, 0x9d, 0x44 // It hi; add hi r13 r13 r3
};
const byte kInstruction_add_hi_r13_r13_r4[] = {
  0x88, 0xbf, 0xa5, 0x44 // It hi; add hi r13 r13 r4
};
const byte kInstruction_add_hi_r13_r13_r5[] = {
  0x88, 0xbf, 0xad, 0x44 // It hi; add hi r13 r13 r5
};
const byte kInstruction_add_hi_r13_r13_r6[] = {
  0x88, 0xbf, 0xb5, 0x44 // It hi; add hi r13 r13 r6
};
const byte kInstruction_add_hi_r13_r13_r7[] = {
  0x88, 0xbf, 0xbd, 0x44 // It hi; add hi r13 r13 r7
};
const byte kInstruction_add_hi_r13_r13_r8[] = {
  0x88, 0xbf, 0xc5, 0x44 // It hi; add hi r13 r13 r8
};
const byte kInstruction_add_hi_r13_r13_r9[] = {
  0x88, 0xbf, 0xcd, 0x44 // It hi; add hi r13 r13 r9
};
const byte kInstruction_add_hi_r13_r13_r10[] = {
  0x88, 0xbf, 0xd5, 0x44 // It hi; add hi r13 r13 r10
};
const byte kInstruction_add_hi_r13_r13_r11[] = {
  0x88, 0xbf, 0xdd, 0x44 // It hi; add hi r13 r13 r11
};
const byte kInstruction_add_hi_r13_r13_r12[] = {
  0x88, 0xbf, 0xe5, 0x44 // It hi; add hi r13 r13 r12
};
const byte kInstruction_add_hi_r13_r13_r13[] = {
  0x88, 0xbf, 0xed, 0x44 // It hi; add hi r13 r13 r13
};
const byte kInstruction_add_hi_r13_r13_r14[] = {
  0x88, 0xbf, 0xf5, 0x44 // It hi; add hi r13 r13 r14
};
const byte kInstruction_add_ls_r13_r13_r0[] = {
  0x98, 0xbf, 0x85, 0x44 // It ls; add ls r13 r13 r0
};
const byte kInstruction_add_ls_r13_r13_r1[] = {
  0x98, 0xbf, 0x8d, 0x44 // It ls; add ls r13 r13 r1
};
const byte kInstruction_add_ls_r13_r13_r2[] = {
  0x98, 0xbf, 0x95, 0x44 // It ls; add ls r13 r13 r2
};
const byte kInstruction_add_ls_r13_r13_r3[] = {
  0x98, 0xbf, 0x9d, 0x44 // It ls; add ls r13 r13 r3
};
const byte kInstruction_add_ls_r13_r13_r4[] = {
  0x98, 0xbf, 0xa5, 0x44 // It ls; add ls r13 r13 r4
};
const byte kInstruction_add_ls_r13_r13_r5[] = {
  0x98, 0xbf, 0xad, 0x44 // It ls; add ls r13 r13 r5
};
const byte kInstruction_add_ls_r13_r13_r6[] = {
  0x98, 0xbf, 0xb5, 0x44 // It ls; add ls r13 r13 r6
};
const byte kInstruction_add_ls_r13_r13_r7[] = {
  0x98, 0xbf, 0xbd, 0x44 // It ls; add ls r13 r13 r7
};
const byte kInstruction_add_ls_r13_r13_r8[] = {
  0x98, 0xbf, 0xc5, 0x44 // It ls; add ls r13 r13 r8
};
const byte kInstruction_add_ls_r13_r13_r9[] = {
  0x98, 0xbf, 0xcd, 0x44 // It ls; add ls r13 r13 r9
};
const byte kInstruction_add_ls_r13_r13_r10[] = {
  0x98, 0xbf, 0xd5, 0x44 // It ls; add ls r13 r13 r10
};
const byte kInstruction_add_ls_r13_r13_r11[] = {
  0x98, 0xbf, 0xdd, 0x44 // It ls; add ls r13 r13 r11
};
const byte kInstruction_add_ls_r13_r13_r12[] = {
  0x98, 0xbf, 0xe5, 0x44 // It ls; add ls r13 r13 r12
};
const byte kInstruction_add_ls_r13_r13_r13[] = {
  0x98, 0xbf, 0xed, 0x44 // It ls; add ls r13 r13 r13
};
const byte kInstruction_add_ls_r13_r13_r14[] = {
  0x98, 0xbf, 0xf5, 0x44 // It ls; add ls r13 r13 r14
};
const byte kInstruction_add_ge_r13_r13_r0[] = {
  0xa8, 0xbf, 0x85, 0x44 // It ge; add ge r13 r13 r0
};
const byte kInstruction_add_ge_r13_r13_r1[] = {
  0xa8, 0xbf, 0x8d, 0x44 // It ge; add ge r13 r13 r1
};
const byte kInstruction_add_ge_r13_r13_r2[] = {
  0xa8, 0xbf, 0x95, 0x44 // It ge; add ge r13 r13 r2
};
const byte kInstruction_add_ge_r13_r13_r3[] = {
  0xa8, 0xbf, 0x9d, 0x44 // It ge; add ge r13 r13 r3
};
const byte kInstruction_add_ge_r13_r13_r4[] = {
  0xa8, 0xbf, 0xa5, 0x44 // It ge; add ge r13 r13 r4
};
const byte kInstruction_add_ge_r13_r13_r5[] = {
  0xa8, 0xbf, 0xad, 0x44 // It ge; add ge r13 r13 r5
};
const byte kInstruction_add_ge_r13_r13_r6[] = {
  0xa8, 0xbf, 0xb5, 0x44 // It ge; add ge r13 r13 r6
};
const byte kInstruction_add_ge_r13_r13_r7[] = {
  0xa8, 0xbf, 0xbd, 0x44 // It ge; add ge r13 r13 r7
};
const byte kInstruction_add_ge_r13_r13_r8[] = {
  0xa8, 0xbf, 0xc5, 0x44 // It ge; add ge r13 r13 r8
};
const byte kInstruction_add_ge_r13_r13_r9[] = {
  0xa8, 0xbf, 0xcd, 0x44 // It ge; add ge r13 r13 r9
};
const byte kInstruction_add_ge_r13_r13_r10[] = {
  0xa8, 0xbf, 0xd5, 0x44 // It ge; add ge r13 r13 r10
};
const byte kInstruction_add_ge_r13_r13_r11[] = {
  0xa8, 0xbf, 0xdd, 0x44 // It ge; add ge r13 r13 r11
};
const byte kInstruction_add_ge_r13_r13_r12[] = {
  0xa8, 0xbf, 0xe5, 0x44 // It ge; add ge r13 r13 r12
};
const byte kInstruction_add_ge_r13_r13_r13[] = {
  0xa8, 0xbf, 0xed, 0x44 // It ge; add ge r13 r13 r13
};
const byte kInstruction_add_ge_r13_r13_r14[] = {
  0xa8, 0xbf, 0xf5, 0x44 // It ge; add ge r13 r13 r14
};
const byte kInstruction_add_lt_r13_r13_r0[] = {
  0xb8, 0xbf, 0x85, 0x44 // It lt; add lt r13 r13 r0
};
const byte kInstruction_add_lt_r13_r13_r1[] = {
  0xb8, 0xbf, 0x8d, 0x44 // It lt; add lt r13 r13 r1
};
const byte kInstruction_add_lt_r13_r13_r2[] = {
  0xb8, 0xbf, 0x95, 0x44 // It lt; add lt r13 r13 r2
};
const byte kInstruction_add_lt_r13_r13_r3[] = {
  0xb8, 0xbf, 0x9d, 0x44 // It lt; add lt r13 r13 r3
};
const byte kInstruction_add_lt_r13_r13_r4[] = {
  0xb8, 0xbf, 0xa5, 0x44 // It lt; add lt r13 r13 r4
};
const byte kInstruction_add_lt_r13_r13_r5[] = {
  0xb8, 0xbf, 0xad, 0x44 // It lt; add lt r13 r13 r5
};
const byte kInstruction_add_lt_r13_r13_r6[] = {
  0xb8, 0xbf, 0xb5, 0x44 // It lt; add lt r13 r13 r6
};
const byte kInstruction_add_lt_r13_r13_r7[] = {
  0xb8, 0xbf, 0xbd, 0x44 // It lt; add lt r13 r13 r7
};
const byte kInstruction_add_lt_r13_r13_r8[] = {
  0xb8, 0xbf, 0xc5, 0x44 // It lt; add lt r13 r13 r8
};
const byte kInstruction_add_lt_r13_r13_r9[] = {
  0xb8, 0xbf, 0xcd, 0x44 // It lt; add lt r13 r13 r9
};
const byte kInstruction_add_lt_r13_r13_r10[] = {
  0xb8, 0xbf, 0xd5, 0x44 // It lt; add lt r13 r13 r10
};
const byte kInstruction_add_lt_r13_r13_r11[] = {
  0xb8, 0xbf, 0xdd, 0x44 // It lt; add lt r13 r13 r11
};
const byte kInstruction_add_lt_r13_r13_r12[] = {
  0xb8, 0xbf, 0xe5, 0x44 // It lt; add lt r13 r13 r12
};
const byte kInstruction_add_lt_r13_r13_r13[] = {
  0xb8, 0xbf, 0xed, 0x44 // It lt; add lt r13 r13 r13
};
const byte kInstruction_add_lt_r13_r13_r14[] = {
  0xb8, 0xbf, 0xf5, 0x44 // It lt; add lt r13 r13 r14
};
const byte kInstruction_add_gt_r13_r13_r0[] = {
  0xc8, 0xbf, 0x85, 0x44 // It gt; add gt r13 r13 r0
};
const byte kInstruction_add_gt_r13_r13_r1[] = {
  0xc8, 0xbf, 0x8d, 0x44 // It gt; add gt r13 r13 r1
};
const byte kInstruction_add_gt_r13_r13_r2[] = {
  0xc8, 0xbf, 0x95, 0x44 // It gt; add gt r13 r13 r2
};
const byte kInstruction_add_gt_r13_r13_r3[] = {
  0xc8, 0xbf, 0x9d, 0x44 // It gt; add gt r13 r13 r3
};
const byte kInstruction_add_gt_r13_r13_r4[] = {
  0xc8, 0xbf, 0xa5, 0x44 // It gt; add gt r13 r13 r4
};
const byte kInstruction_add_gt_r13_r13_r5[] = {
  0xc8, 0xbf, 0xad, 0x44 // It gt; add gt r13 r13 r5
};
const byte kInstruction_add_gt_r13_r13_r6[] = {
  0xc8, 0xbf, 0xb5, 0x44 // It gt; add gt r13 r13 r6
};
const byte kInstruction_add_gt_r13_r13_r7[] = {
  0xc8, 0xbf, 0xbd, 0x44 // It gt; add gt r13 r13 r7
};
const byte kInstruction_add_gt_r13_r13_r8[] = {
  0xc8, 0xbf, 0xc5, 0x44 // It gt; add gt r13 r13 r8
};
const byte kInstruction_add_gt_r13_r13_r9[] = {
  0xc8, 0xbf, 0xcd, 0x44 // It gt; add gt r13 r13 r9
};
const byte kInstruction_add_gt_r13_r13_r10[] = {
  0xc8, 0xbf, 0xd5, 0x44 // It gt; add gt r13 r13 r10
};
const byte kInstruction_add_gt_r13_r13_r11[] = {
  0xc8, 0xbf, 0xdd, 0x44 // It gt; add gt r13 r13 r11
};
const byte kInstruction_add_gt_r13_r13_r12[] = {
  0xc8, 0xbf, 0xe5, 0x44 // It gt; add gt r13 r13 r12
};
const byte kInstruction_add_gt_r13_r13_r13[] = {
  0xc8, 0xbf, 0xed, 0x44 // It gt; add gt r13 r13 r13
};
const byte kInstruction_add_gt_r13_r13_r14[] = {
  0xc8, 0xbf, 0xf5, 0x44 // It gt; add gt r13 r13 r14
};
const byte kInstruction_add_le_r13_r13_r0[] = {
  0xd8, 0xbf, 0x85, 0x44 // It le; add le r13 r13 r0
};
const byte kInstruction_add_le_r13_r13_r1[] = {
  0xd8, 0xbf, 0x8d, 0x44 // It le; add le r13 r13 r1
};
const byte kInstruction_add_le_r13_r13_r2[] = {
  0xd8, 0xbf, 0x95, 0x44 // It le; add le r13 r13 r2
};
const byte kInstruction_add_le_r13_r13_r3[] = {
  0xd8, 0xbf, 0x9d, 0x44 // It le; add le r13 r13 r3
};
const byte kInstruction_add_le_r13_r13_r4[] = {
  0xd8, 0xbf, 0xa5, 0x44 // It le; add le r13 r13 r4
};
const byte kInstruction_add_le_r13_r13_r5[] = {
  0xd8, 0xbf, 0xad, 0x44 // It le; add le r13 r13 r5
};
const byte kInstruction_add_le_r13_r13_r6[] = {
  0xd8, 0xbf, 0xb5, 0x44 // It le; add le r13 r13 r6
};
const byte kInstruction_add_le_r13_r13_r7[] = {
  0xd8, 0xbf, 0xbd, 0x44 // It le; add le r13 r13 r7
};
const byte kInstruction_add_le_r13_r13_r8[] = {
  0xd8, 0xbf, 0xc5, 0x44 // It le; add le r13 r13 r8
};
const byte kInstruction_add_le_r13_r13_r9[] = {
  0xd8, 0xbf, 0xcd, 0x44 // It le; add le r13 r13 r9
};
const byte kInstruction_add_le_r13_r13_r10[] = {
  0xd8, 0xbf, 0xd5, 0x44 // It le; add le r13 r13 r10
};
const byte kInstruction_add_le_r13_r13_r11[] = {
  0xd8, 0xbf, 0xdd, 0x44 // It le; add le r13 r13 r11
};
const byte kInstruction_add_le_r13_r13_r12[] = {
  0xd8, 0xbf, 0xe5, 0x44 // It le; add le r13 r13 r12
};
const byte kInstruction_add_le_r13_r13_r13[] = {
  0xd8, 0xbf, 0xed, 0x44 // It le; add le r13 r13 r13
};
const byte kInstruction_add_le_r13_r13_r14[] = {
  0xd8, 0xbf, 0xf5, 0x44 // It le; add le r13 r13 r14
};
const TestResult kReferenceadd[] = {
  {
    ARRAY_SIZE(kInstruction_add_eq_r13_r13_r0),
    kInstruction_add_eq_r13_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r13_r13_r1),
    kInstruction_add_eq_r13_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r13_r13_r2),
    kInstruction_add_eq_r13_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r13_r13_r3),
    kInstruction_add_eq_r13_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r13_r13_r4),
    kInstruction_add_eq_r13_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r13_r13_r5),
    kInstruction_add_eq_r13_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r13_r13_r6),
    kInstruction_add_eq_r13_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r13_r13_r7),
    kInstruction_add_eq_r13_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r13_r13_r8),
    kInstruction_add_eq_r13_r13_r8,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r13_r13_r9),
    kInstruction_add_eq_r13_r13_r9,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r13_r13_r10),
    kInstruction_add_eq_r13_r13_r10,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r13_r13_r11),
    kInstruction_add_eq_r13_r13_r11,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r13_r13_r12),
    kInstruction_add_eq_r13_r13_r12,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r13_r13_r13),
    kInstruction_add_eq_r13_r13_r13,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r13_r13_r14),
    kInstruction_add_eq_r13_r13_r14,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r13_r13_r0),
    kInstruction_add_ne_r13_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r13_r13_r1),
    kInstruction_add_ne_r13_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r13_r13_r2),
    kInstruction_add_ne_r13_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r13_r13_r3),
    kInstruction_add_ne_r13_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r13_r13_r4),
    kInstruction_add_ne_r13_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r13_r13_r5),
    kInstruction_add_ne_r13_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r13_r13_r6),
    kInstruction_add_ne_r13_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r13_r13_r7),
    kInstruction_add_ne_r13_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r13_r13_r8),
    kInstruction_add_ne_r13_r13_r8,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r13_r13_r9),
    kInstruction_add_ne_r13_r13_r9,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r13_r13_r10),
    kInstruction_add_ne_r13_r13_r10,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r13_r13_r11),
    kInstruction_add_ne_r13_r13_r11,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r13_r13_r12),
    kInstruction_add_ne_r13_r13_r12,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r13_r13_r13),
    kInstruction_add_ne_r13_r13_r13,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r13_r13_r14),
    kInstruction_add_ne_r13_r13_r14,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r13_r13_r0),
    kInstruction_add_cs_r13_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r13_r13_r1),
    kInstruction_add_cs_r13_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r13_r13_r2),
    kInstruction_add_cs_r13_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r13_r13_r3),
    kInstruction_add_cs_r13_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r13_r13_r4),
    kInstruction_add_cs_r13_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r13_r13_r5),
    kInstruction_add_cs_r13_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r13_r13_r6),
    kInstruction_add_cs_r13_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r13_r13_r7),
    kInstruction_add_cs_r13_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r13_r13_r8),
    kInstruction_add_cs_r13_r13_r8,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r13_r13_r9),
    kInstruction_add_cs_r13_r13_r9,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r13_r13_r10),
    kInstruction_add_cs_r13_r13_r10,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r13_r13_r11),
    kInstruction_add_cs_r13_r13_r11,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r13_r13_r12),
    kInstruction_add_cs_r13_r13_r12,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r13_r13_r13),
    kInstruction_add_cs_r13_r13_r13,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r13_r13_r14),
    kInstruction_add_cs_r13_r13_r14,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r13_r13_r0),
    kInstruction_add_cc_r13_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r13_r13_r1),
    kInstruction_add_cc_r13_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r13_r13_r2),
    kInstruction_add_cc_r13_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r13_r13_r3),
    kInstruction_add_cc_r13_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r13_r13_r4),
    kInstruction_add_cc_r13_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r13_r13_r5),
    kInstruction_add_cc_r13_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r13_r13_r6),
    kInstruction_add_cc_r13_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r13_r13_r7),
    kInstruction_add_cc_r13_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r13_r13_r8),
    kInstruction_add_cc_r13_r13_r8,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r13_r13_r9),
    kInstruction_add_cc_r13_r13_r9,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r13_r13_r10),
    kInstruction_add_cc_r13_r13_r10,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r13_r13_r11),
    kInstruction_add_cc_r13_r13_r11,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r13_r13_r12),
    kInstruction_add_cc_r13_r13_r12,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r13_r13_r13),
    kInstruction_add_cc_r13_r13_r13,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r13_r13_r14),
    kInstruction_add_cc_r13_r13_r14,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r13_r13_r0),
    kInstruction_add_mi_r13_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r13_r13_r1),
    kInstruction_add_mi_r13_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r13_r13_r2),
    kInstruction_add_mi_r13_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r13_r13_r3),
    kInstruction_add_mi_r13_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r13_r13_r4),
    kInstruction_add_mi_r13_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r13_r13_r5),
    kInstruction_add_mi_r13_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r13_r13_r6),
    kInstruction_add_mi_r13_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r13_r13_r7),
    kInstruction_add_mi_r13_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r13_r13_r8),
    kInstruction_add_mi_r13_r13_r8,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r13_r13_r9),
    kInstruction_add_mi_r13_r13_r9,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r13_r13_r10),
    kInstruction_add_mi_r13_r13_r10,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r13_r13_r11),
    kInstruction_add_mi_r13_r13_r11,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r13_r13_r12),
    kInstruction_add_mi_r13_r13_r12,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r13_r13_r13),
    kInstruction_add_mi_r13_r13_r13,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r13_r13_r14),
    kInstruction_add_mi_r13_r13_r14,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r13_r13_r0),
    kInstruction_add_pl_r13_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r13_r13_r1),
    kInstruction_add_pl_r13_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r13_r13_r2),
    kInstruction_add_pl_r13_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r13_r13_r3),
    kInstruction_add_pl_r13_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r13_r13_r4),
    kInstruction_add_pl_r13_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r13_r13_r5),
    kInstruction_add_pl_r13_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r13_r13_r6),
    kInstruction_add_pl_r13_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r13_r13_r7),
    kInstruction_add_pl_r13_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r13_r13_r8),
    kInstruction_add_pl_r13_r13_r8,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r13_r13_r9),
    kInstruction_add_pl_r13_r13_r9,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r13_r13_r10),
    kInstruction_add_pl_r13_r13_r10,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r13_r13_r11),
    kInstruction_add_pl_r13_r13_r11,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r13_r13_r12),
    kInstruction_add_pl_r13_r13_r12,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r13_r13_r13),
    kInstruction_add_pl_r13_r13_r13,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r13_r13_r14),
    kInstruction_add_pl_r13_r13_r14,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r13_r13_r0),
    kInstruction_add_vs_r13_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r13_r13_r1),
    kInstruction_add_vs_r13_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r13_r13_r2),
    kInstruction_add_vs_r13_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r13_r13_r3),
    kInstruction_add_vs_r13_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r13_r13_r4),
    kInstruction_add_vs_r13_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r13_r13_r5),
    kInstruction_add_vs_r13_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r13_r13_r6),
    kInstruction_add_vs_r13_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r13_r13_r7),
    kInstruction_add_vs_r13_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r13_r13_r8),
    kInstruction_add_vs_r13_r13_r8,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r13_r13_r9),
    kInstruction_add_vs_r13_r13_r9,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r13_r13_r10),
    kInstruction_add_vs_r13_r13_r10,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r13_r13_r11),
    kInstruction_add_vs_r13_r13_r11,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r13_r13_r12),
    kInstruction_add_vs_r13_r13_r12,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r13_r13_r13),
    kInstruction_add_vs_r13_r13_r13,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r13_r13_r14),
    kInstruction_add_vs_r13_r13_r14,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r13_r13_r0),
    kInstruction_add_vc_r13_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r13_r13_r1),
    kInstruction_add_vc_r13_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r13_r13_r2),
    kInstruction_add_vc_r13_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r13_r13_r3),
    kInstruction_add_vc_r13_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r13_r13_r4),
    kInstruction_add_vc_r13_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r13_r13_r5),
    kInstruction_add_vc_r13_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r13_r13_r6),
    kInstruction_add_vc_r13_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r13_r13_r7),
    kInstruction_add_vc_r13_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r13_r13_r8),
    kInstruction_add_vc_r13_r13_r8,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r13_r13_r9),
    kInstruction_add_vc_r13_r13_r9,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r13_r13_r10),
    kInstruction_add_vc_r13_r13_r10,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r13_r13_r11),
    kInstruction_add_vc_r13_r13_r11,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r13_r13_r12),
    kInstruction_add_vc_r13_r13_r12,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r13_r13_r13),
    kInstruction_add_vc_r13_r13_r13,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r13_r13_r14),
    kInstruction_add_vc_r13_r13_r14,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r13_r13_r0),
    kInstruction_add_hi_r13_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r13_r13_r1),
    kInstruction_add_hi_r13_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r13_r13_r2),
    kInstruction_add_hi_r13_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r13_r13_r3),
    kInstruction_add_hi_r13_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r13_r13_r4),
    kInstruction_add_hi_r13_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r13_r13_r5),
    kInstruction_add_hi_r13_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r13_r13_r6),
    kInstruction_add_hi_r13_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r13_r13_r7),
    kInstruction_add_hi_r13_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r13_r13_r8),
    kInstruction_add_hi_r13_r13_r8,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r13_r13_r9),
    kInstruction_add_hi_r13_r13_r9,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r13_r13_r10),
    kInstruction_add_hi_r13_r13_r10,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r13_r13_r11),
    kInstruction_add_hi_r13_r13_r11,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r13_r13_r12),
    kInstruction_add_hi_r13_r13_r12,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r13_r13_r13),
    kInstruction_add_hi_r13_r13_r13,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r13_r13_r14),
    kInstruction_add_hi_r13_r13_r14,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r13_r13_r0),
    kInstruction_add_ls_r13_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r13_r13_r1),
    kInstruction_add_ls_r13_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r13_r13_r2),
    kInstruction_add_ls_r13_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r13_r13_r3),
    kInstruction_add_ls_r13_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r13_r13_r4),
    kInstruction_add_ls_r13_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r13_r13_r5),
    kInstruction_add_ls_r13_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r13_r13_r6),
    kInstruction_add_ls_r13_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r13_r13_r7),
    kInstruction_add_ls_r13_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r13_r13_r8),
    kInstruction_add_ls_r13_r13_r8,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r13_r13_r9),
    kInstruction_add_ls_r13_r13_r9,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r13_r13_r10),
    kInstruction_add_ls_r13_r13_r10,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r13_r13_r11),
    kInstruction_add_ls_r13_r13_r11,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r13_r13_r12),
    kInstruction_add_ls_r13_r13_r12,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r13_r13_r13),
    kInstruction_add_ls_r13_r13_r13,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r13_r13_r14),
    kInstruction_add_ls_r13_r13_r14,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r13_r13_r0),
    kInstruction_add_ge_r13_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r13_r13_r1),
    kInstruction_add_ge_r13_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r13_r13_r2),
    kInstruction_add_ge_r13_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r13_r13_r3),
    kInstruction_add_ge_r13_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r13_r13_r4),
    kInstruction_add_ge_r13_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r13_r13_r5),
    kInstruction_add_ge_r13_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r13_r13_r6),
    kInstruction_add_ge_r13_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r13_r13_r7),
    kInstruction_add_ge_r13_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r13_r13_r8),
    kInstruction_add_ge_r13_r13_r8,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r13_r13_r9),
    kInstruction_add_ge_r13_r13_r9,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r13_r13_r10),
    kInstruction_add_ge_r13_r13_r10,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r13_r13_r11),
    kInstruction_add_ge_r13_r13_r11,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r13_r13_r12),
    kInstruction_add_ge_r13_r13_r12,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r13_r13_r13),
    kInstruction_add_ge_r13_r13_r13,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r13_r13_r14),
    kInstruction_add_ge_r13_r13_r14,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r13_r13_r0),
    kInstruction_add_lt_r13_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r13_r13_r1),
    kInstruction_add_lt_r13_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r13_r13_r2),
    kInstruction_add_lt_r13_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r13_r13_r3),
    kInstruction_add_lt_r13_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r13_r13_r4),
    kInstruction_add_lt_r13_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r13_r13_r5),
    kInstruction_add_lt_r13_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r13_r13_r6),
    kInstruction_add_lt_r13_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r13_r13_r7),
    kInstruction_add_lt_r13_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r13_r13_r8),
    kInstruction_add_lt_r13_r13_r8,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r13_r13_r9),
    kInstruction_add_lt_r13_r13_r9,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r13_r13_r10),
    kInstruction_add_lt_r13_r13_r10,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r13_r13_r11),
    kInstruction_add_lt_r13_r13_r11,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r13_r13_r12),
    kInstruction_add_lt_r13_r13_r12,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r13_r13_r13),
    kInstruction_add_lt_r13_r13_r13,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r13_r13_r14),
    kInstruction_add_lt_r13_r13_r14,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r13_r13_r0),
    kInstruction_add_gt_r13_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r13_r13_r1),
    kInstruction_add_gt_r13_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r13_r13_r2),
    kInstruction_add_gt_r13_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r13_r13_r3),
    kInstruction_add_gt_r13_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r13_r13_r4),
    kInstruction_add_gt_r13_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r13_r13_r5),
    kInstruction_add_gt_r13_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r13_r13_r6),
    kInstruction_add_gt_r13_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r13_r13_r7),
    kInstruction_add_gt_r13_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r13_r13_r8),
    kInstruction_add_gt_r13_r13_r8,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r13_r13_r9),
    kInstruction_add_gt_r13_r13_r9,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r13_r13_r10),
    kInstruction_add_gt_r13_r13_r10,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r13_r13_r11),
    kInstruction_add_gt_r13_r13_r11,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r13_r13_r12),
    kInstruction_add_gt_r13_r13_r12,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r13_r13_r13),
    kInstruction_add_gt_r13_r13_r13,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r13_r13_r14),
    kInstruction_add_gt_r13_r13_r14,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r13_r13_r0),
    kInstruction_add_le_r13_r13_r0,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r13_r13_r1),
    kInstruction_add_le_r13_r13_r1,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r13_r13_r2),
    kInstruction_add_le_r13_r13_r2,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r13_r13_r3),
    kInstruction_add_le_r13_r13_r3,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r13_r13_r4),
    kInstruction_add_le_r13_r13_r4,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r13_r13_r5),
    kInstruction_add_le_r13_r13_r5,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r13_r13_r6),
    kInstruction_add_le_r13_r13_r6,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r13_r13_r7),
    kInstruction_add_le_r13_r13_r7,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r13_r13_r8),
    kInstruction_add_le_r13_r13_r8,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r13_r13_r9),
    kInstruction_add_le_r13_r13_r9,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r13_r13_r10),
    kInstruction_add_le_r13_r13_r10,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r13_r13_r11),
    kInstruction_add_le_r13_r13_r11,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r13_r13_r12),
    kInstruction_add_le_r13_r13_r12,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r13_r13_r13),
    kInstruction_add_le_r13_r13_r13,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r13_r13_r14),
    kInstruction_add_le_r13_r13_r14,
  },
};

#endif  // VIXL_ASSEMBLER_COND_RD_RN_OPERAND_RM_RD_IS_RN_IS_SP_IN_IT_BLOCK_ADD_T32_H_
