// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_SIM_SQABS_2S_TRACE_AARCH64_H_
#define VIXL_SIM_SQABS_2S_TRACE_AARCH64_H_

const uint32_t kExpected_NEON_sqabs_2S[] = {
  0x00000003, 0x00000002,
  0x00000002, 0x00000001,
  0x00000001, 0x00000000,
  0x00000000, 0x00000001,
  0x00000001, 0x00000002,
  0x00000002, 0x00000020,
  0x00000020, 0x0000007d,
  0x0000007d, 0x0000007e,
  0x0000007e, 0x0000007f,
  0x0000007f, 0x00007ffd,
  0x00007ffd, 0x00007ffe,
  0x00007ffe, 0x00007fff,
  0x00007fff, 0x33333333,
  0x33333333, 0x55555555,
  0x55555555, 0x7ffffffd,
  0x7ffffffd, 0x7ffffffe,
  0x7ffffffe, 0x7fffffff,
  0x7fffffff, 0x7fffffff,
  0x7fffffff, 0x7fffffff,
  0x7fffffff, 0x55555556,
  0x55555556, 0x33333334,
  0x33333334, 0x00008000,
  0x00008000, 0x00007fff,
  0x00007fff, 0x00007ffe,
  0x00007ffe, 0x00007ffd,
  0x00007ffd, 0x00000080,
  0x00000080, 0x0000007f,
  0x0000007f, 0x0000007e,
  0x0000007e, 0x0000007d,
  0x0000007d, 0x00000020,
  0x00000020, 0x00000003,
};
const unsigned kExpectedCount_NEON_sqabs_2S = 31;

#endif  // VIXL_SIM_SQABS_2S_TRACE_AARCH64_H_
