// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_SIM_FCVTMS_4H_TRACE_AARCH64_H_
#define VIXL_SIM_FCVTMS_4H_TRACE_AARCH64_H_

const uint16_t kExpected_NEON_fcvtms_4H[] = {
  0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0001,
  0x0000, 0x0000, 0x0001, 0x0001,
  0x0000, 0x0001, 0x0001, 0x0001,
  0x0001, 0x0001, 0x0001, 0x000a,
  0x0001, 0x0001, 0x000a, 0x7fff,
  0x0001, 0x000a, 0x7fff, 0x7fff,
  0x000a, 0x7fff, 0x7fff, 0x0000,
  0x7fff, 0x7fff, 0x0000, 0x0000,
  0x7fff, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0xffff,
  0x0000, 0x0000, 0xffff, 0xffff,
  0x0000, 0xffff, 0xffff, 0xffff,
  0xffff, 0xffff, 0xffff, 0xffff,
  0xffff, 0xffff, 0xffff, 0xffff,
  0xffff, 0xffff, 0xffff, 0xffff,
  0xffff, 0xffff, 0xffff, 0xfffe,
  0xffff, 0xffff, 0xfffe, 0xfffe,
  0xffff, 0xfffe, 0xfffe, 0xfff6,
  0xfffe, 0xfffe, 0xfff6, 0x8000,
  0xfffe, 0xfff6, 0x8000, 0x8000,
  0xfff6, 0x8000, 0x8000, 0x0000,
  0x8000, 0x8000, 0x0000, 0x0000,
  0x8000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0xffff,
  0x0000, 0x0000, 0xffff, 0xffff,
  0x0000, 0xffff, 0xffff, 0xffff,
  0xffff, 0xffff, 0xffff, 0x0400,
  0xffff, 0xffff, 0x0400, 0x0401,
  0xffff, 0x0400, 0x0401, 0x0402,
  0x0400, 0x0401, 0x0402, 0x0403,
  0x0401, 0x0402, 0x0403, 0x0543,
  0x0402, 0x0403, 0x0543, 0x07fc,
  0x0403, 0x0543, 0x07fc, 0x07fd,
  0x0543, 0x07fc, 0x07fd, 0x07fe,
  0x07fc, 0x07fd, 0x07fe, 0x07ff,
  0x07fd, 0x07fe, 0x07ff, 0x0200,
  0x07fe, 0x07ff, 0x0200, 0x0200,
  0x07ff, 0x0200, 0x0200, 0x0201,
  0x0200, 0x0200, 0x0201, 0x0201,
  0x0200, 0x0201, 0x0201, 0x0390,
  0x0201, 0x0201, 0x0390, 0x03fe,
  0x0201, 0x0390, 0x03fe, 0x03fe,
  0x0390, 0x03fe, 0x03fe, 0x03ff,
  0x03fe, 0x03fe, 0x03ff, 0x03ff,
  0x03fe, 0x03ff, 0x03ff, 0x0100,
  0x03ff, 0x03ff, 0x0100, 0x0100,
  0x03ff, 0x0100, 0x0100, 0x0100,
  0x0100, 0x0100, 0x0100, 0x0100,
  0x0100, 0x0100, 0x0100, 0x014c,
  0x0100, 0x0100, 0x014c, 0x01ff,
  0x0100, 0x014c, 0x01ff, 0x01ff,
  0x014c, 0x01ff, 0x01ff, 0x01ff,
  0x01ff, 0x01ff, 0x01ff, 0x01ff,
  0x01ff, 0x01ff, 0x01ff, 0xfc00,
  0x01ff, 0x01ff, 0xfc00, 0xfbff,
  0x01ff, 0xfc00, 0xfbff, 0xfbfe,
  0xfc00, 0xfbff, 0xfbfe, 0xfbfd,
  0xfbff, 0xfbfe, 0xfbfd, 0xfabd,
  0xfbfe, 0xfbfd, 0xfabd, 0xf804,
  0xfbfd, 0xfabd, 0xf804, 0xf803,
  0xfabd, 0xf804, 0xf803, 0xf802,
  0xf804, 0xf803, 0xf802, 0xf801,
  0xf803, 0xf802, 0xf801, 0xfe00,
  0xf802, 0xf801, 0xfe00, 0xfdff,
  0xf801, 0xfe00, 0xfdff, 0xfdff,
  0xfe00, 0xfdff, 0xfdff, 0xfdfe,
  0xfdff, 0xfdff, 0xfdfe, 0xfc6f,
  0xfdff, 0xfdfe, 0xfc6f, 0xfc02,
  0xfdfe, 0xfc6f, 0xfc02, 0xfc01,
  0xfc6f, 0xfc02, 0xfc01, 0xfc01,
  0xfc02, 0xfc01, 0xfc01, 0xfc00,
  0xfc01, 0xfc01, 0xfc00, 0xff00,
  0xfc01, 0xfc00, 0xff00, 0xfeff,
  0xfc00, 0xff00, 0xfeff, 0xfeff,
  0xff00, 0xfeff, 0xfeff, 0xfeff,
  0xfeff, 0xfeff, 0xfeff, 0xfeb3,
  0xfeff, 0xfeff, 0xfeb3, 0xfe01,
  0xfeff, 0xfeb3, 0xfe01, 0xfe00,
  0xfeb3, 0xfe01, 0xfe00, 0xfe00,
  0xfe01, 0xfe00, 0xfe00, 0xfe00,
  0xfe00, 0xfe00, 0xfe00, 0x0000,
  0xfe00, 0xfe00, 0x0000, 0x0000,
  0xfe00, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000,
};
const unsigned kExpectedCount_NEON_fcvtms_4H = 101;

#endif  // VIXL_SIM_FCVTMS_4H_TRACE_AARCH64_H_
