// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_SIM_FRINTA_S_TRACE_AARCH64_H_
#define VIXL_SIM_FRINTA_S_TRACE_AARCH64_H_

const uint32_t kExpected_frinta_s[] = {
  0x00000000,
  0x00000000,
  0x00000000,
  0x3f800000,
  0x3f800000,
  0x3f800000,
  0x3f800000,
  0x3f800000,
  0x40000000,
  0x41200000,
  0x7fcfffff,
  0x7f800000,
  0x7fd23456,
  0x7fc00000,
  0x7fd23456,
  0x7fc00001,
  0x00000000,
  0x00000000,
  0x00000000,
  0x80000000,
  0x80000000,
  0x80000000,
  0xbf800000,
  0xbf800000,
  0xbf800000,
  0xbf800000,
  0xbf800000,
  0xc0000000,
  0xc1200000,
  0xffcfffff,
  0xff800000,
  0xffd23456,
  0xffc00000,
  0xffd23456,
  0xffc00001,
  0x80000000,
  0x80000000,
  0x80000000,
  0x4b000000,
  0x4b000001,
  0x4b000002,
  0x4b000003,
  0x4b765432,
  0x4b7ffffc,
  0x4b7ffffd,
  0x4b7ffffe,
  0x4b7fffff,
  0x4a800000,
  0x4a800002,
  0x4a800002,
  0x4a800004,
  0x4af65432,
  0x4afffffc,
  0x4afffffe,
  0x4afffffe,
  0x4b000000,
  0x4a000000,
  0x4a000000,
  0x4a000004,
  0x4a000004,
  0x4a765434,
  0x4a7ffffc,
  0x4a7ffffc,
  0x4a800000,
  0x4a800000,
  0xcb000000,
  0xcb000001,
  0xcb000002,
  0xcb000003,
  0xcb765432,
  0xcb7ffffc,
  0xcb7ffffd,
  0xcb7ffffe,
  0xcb7fffff,
  0xca800000,
  0xca800002,
  0xca800002,
  0xca800004,
  0xcaf65432,
  0xcafffffc,
  0xcafffffe,
  0xcafffffe,
  0xcb000000,
  0xca000000,
  0xca000000,
  0xca000004,
  0xca000004,
  0xca765434,
  0xca7ffffc,
  0xca7ffffc,
  0xca800000,
  0xca800000,
  0xdf000001,
  0xdf000000,
  0xdeffffff,
  0x5effffff,
  0x5f000000,
  0x5f7fffff,
  0x5f800000,
  0xcf000001,
  0xcf000000,
  0xceffffff,
  0x4effffff,
  0x4f000000,
};
const unsigned kExpectedCount_frinta_s = 104;

#endif  // VIXL_SIM_FRINTA_S_TRACE_AARCH64_H_
