// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_ASSEMBLER_COND_RD_SP_OPERAND_IMM8_ADD_T32_H_
#define VIXL_ASSEMBLER_COND_RD_SP_OPERAND_IMM8_ADD_T32_H_

const byte kInstruction_add_al_r0_sp_0x0[] = {
  0x00, 0xa8 // add al r0 sp 0x0
};
const byte kInstruction_add_al_r0_sp_0x4[] = {
  0x01, 0xa8 // add al r0 sp 0x4
};
const byte kInstruction_add_al_r0_sp_0x8[] = {
  0x02, 0xa8 // add al r0 sp 0x8
};
const byte kInstruction_add_al_r0_sp_0xc[] = {
  0x03, 0xa8 // add al r0 sp 0xc
};
const byte kInstruction_add_al_r0_sp_0x10[] = {
  0x04, 0xa8 // add al r0 sp 0x10
};
const byte kInstruction_add_al_r0_sp_0x14[] = {
  0x05, 0xa8 // add al r0 sp 0x14
};
const byte kInstruction_add_al_r0_sp_0x18[] = {
  0x06, 0xa8 // add al r0 sp 0x18
};
const byte kInstruction_add_al_r0_sp_0x1c[] = {
  0x07, 0xa8 // add al r0 sp 0x1c
};
const byte kInstruction_add_al_r0_sp_0x20[] = {
  0x08, 0xa8 // add al r0 sp 0x20
};
const byte kInstruction_add_al_r0_sp_0x24[] = {
  0x09, 0xa8 // add al r0 sp 0x24
};
const byte kInstruction_add_al_r0_sp_0x28[] = {
  0x0a, 0xa8 // add al r0 sp 0x28
};
const byte kInstruction_add_al_r0_sp_0x2c[] = {
  0x0b, 0xa8 // add al r0 sp 0x2c
};
const byte kInstruction_add_al_r0_sp_0x30[] = {
  0x0c, 0xa8 // add al r0 sp 0x30
};
const byte kInstruction_add_al_r0_sp_0x34[] = {
  0x0d, 0xa8 // add al r0 sp 0x34
};
const byte kInstruction_add_al_r0_sp_0x38[] = {
  0x0e, 0xa8 // add al r0 sp 0x38
};
const byte kInstruction_add_al_r0_sp_0x3c[] = {
  0x0f, 0xa8 // add al r0 sp 0x3c
};
const byte kInstruction_add_al_r0_sp_0x40[] = {
  0x10, 0xa8 // add al r0 sp 0x40
};
const byte kInstruction_add_al_r0_sp_0x44[] = {
  0x11, 0xa8 // add al r0 sp 0x44
};
const byte kInstruction_add_al_r0_sp_0x48[] = {
  0x12, 0xa8 // add al r0 sp 0x48
};
const byte kInstruction_add_al_r0_sp_0x4c[] = {
  0x13, 0xa8 // add al r0 sp 0x4c
};
const byte kInstruction_add_al_r0_sp_0x50[] = {
  0x14, 0xa8 // add al r0 sp 0x50
};
const byte kInstruction_add_al_r0_sp_0x54[] = {
  0x15, 0xa8 // add al r0 sp 0x54
};
const byte kInstruction_add_al_r0_sp_0x58[] = {
  0x16, 0xa8 // add al r0 sp 0x58
};
const byte kInstruction_add_al_r0_sp_0x5c[] = {
  0x17, 0xa8 // add al r0 sp 0x5c
};
const byte kInstruction_add_al_r0_sp_0x60[] = {
  0x18, 0xa8 // add al r0 sp 0x60
};
const byte kInstruction_add_al_r0_sp_0x64[] = {
  0x19, 0xa8 // add al r0 sp 0x64
};
const byte kInstruction_add_al_r0_sp_0x68[] = {
  0x1a, 0xa8 // add al r0 sp 0x68
};
const byte kInstruction_add_al_r0_sp_0x6c[] = {
  0x1b, 0xa8 // add al r0 sp 0x6c
};
const byte kInstruction_add_al_r0_sp_0x70[] = {
  0x1c, 0xa8 // add al r0 sp 0x70
};
const byte kInstruction_add_al_r0_sp_0x74[] = {
  0x1d, 0xa8 // add al r0 sp 0x74
};
const byte kInstruction_add_al_r0_sp_0x78[] = {
  0x1e, 0xa8 // add al r0 sp 0x78
};
const byte kInstruction_add_al_r0_sp_0x7c[] = {
  0x1f, 0xa8 // add al r0 sp 0x7c
};
const byte kInstruction_add_al_r0_sp_0x80[] = {
  0x20, 0xa8 // add al r0 sp 0x80
};
const byte kInstruction_add_al_r0_sp_0x84[] = {
  0x21, 0xa8 // add al r0 sp 0x84
};
const byte kInstruction_add_al_r0_sp_0x88[] = {
  0x22, 0xa8 // add al r0 sp 0x88
};
const byte kInstruction_add_al_r0_sp_0x8c[] = {
  0x23, 0xa8 // add al r0 sp 0x8c
};
const byte kInstruction_add_al_r0_sp_0x90[] = {
  0x24, 0xa8 // add al r0 sp 0x90
};
const byte kInstruction_add_al_r0_sp_0x94[] = {
  0x25, 0xa8 // add al r0 sp 0x94
};
const byte kInstruction_add_al_r0_sp_0x98[] = {
  0x26, 0xa8 // add al r0 sp 0x98
};
const byte kInstruction_add_al_r0_sp_0x9c[] = {
  0x27, 0xa8 // add al r0 sp 0x9c
};
const byte kInstruction_add_al_r0_sp_0xa0[] = {
  0x28, 0xa8 // add al r0 sp 0xa0
};
const byte kInstruction_add_al_r0_sp_0xa4[] = {
  0x29, 0xa8 // add al r0 sp 0xa4
};
const byte kInstruction_add_al_r0_sp_0xa8[] = {
  0x2a, 0xa8 // add al r0 sp 0xa8
};
const byte kInstruction_add_al_r0_sp_0xac[] = {
  0x2b, 0xa8 // add al r0 sp 0xac
};
const byte kInstruction_add_al_r0_sp_0xb0[] = {
  0x2c, 0xa8 // add al r0 sp 0xb0
};
const byte kInstruction_add_al_r0_sp_0xb4[] = {
  0x2d, 0xa8 // add al r0 sp 0xb4
};
const byte kInstruction_add_al_r0_sp_0xb8[] = {
  0x2e, 0xa8 // add al r0 sp 0xb8
};
const byte kInstruction_add_al_r0_sp_0xbc[] = {
  0x2f, 0xa8 // add al r0 sp 0xbc
};
const byte kInstruction_add_al_r0_sp_0xc0[] = {
  0x30, 0xa8 // add al r0 sp 0xc0
};
const byte kInstruction_add_al_r0_sp_0xc4[] = {
  0x31, 0xa8 // add al r0 sp 0xc4
};
const byte kInstruction_add_al_r0_sp_0xc8[] = {
  0x32, 0xa8 // add al r0 sp 0xc8
};
const byte kInstruction_add_al_r0_sp_0xcc[] = {
  0x33, 0xa8 // add al r0 sp 0xcc
};
const byte kInstruction_add_al_r0_sp_0xd0[] = {
  0x34, 0xa8 // add al r0 sp 0xd0
};
const byte kInstruction_add_al_r0_sp_0xd4[] = {
  0x35, 0xa8 // add al r0 sp 0xd4
};
const byte kInstruction_add_al_r0_sp_0xd8[] = {
  0x36, 0xa8 // add al r0 sp 0xd8
};
const byte kInstruction_add_al_r0_sp_0xdc[] = {
  0x37, 0xa8 // add al r0 sp 0xdc
};
const byte kInstruction_add_al_r0_sp_0xe0[] = {
  0x38, 0xa8 // add al r0 sp 0xe0
};
const byte kInstruction_add_al_r0_sp_0xe4[] = {
  0x39, 0xa8 // add al r0 sp 0xe4
};
const byte kInstruction_add_al_r0_sp_0xe8[] = {
  0x3a, 0xa8 // add al r0 sp 0xe8
};
const byte kInstruction_add_al_r0_sp_0xec[] = {
  0x3b, 0xa8 // add al r0 sp 0xec
};
const byte kInstruction_add_al_r0_sp_0xf0[] = {
  0x3c, 0xa8 // add al r0 sp 0xf0
};
const byte kInstruction_add_al_r0_sp_0xf4[] = {
  0x3d, 0xa8 // add al r0 sp 0xf4
};
const byte kInstruction_add_al_r0_sp_0xf8[] = {
  0x3e, 0xa8 // add al r0 sp 0xf8
};
const byte kInstruction_add_al_r0_sp_0xfc[] = {
  0x3f, 0xa8 // add al r0 sp 0xfc
};
const byte kInstruction_add_al_r0_sp_0x100[] = {
  0x40, 0xa8 // add al r0 sp 0x100
};
const byte kInstruction_add_al_r0_sp_0x104[] = {
  0x41, 0xa8 // add al r0 sp 0x104
};
const byte kInstruction_add_al_r0_sp_0x108[] = {
  0x42, 0xa8 // add al r0 sp 0x108
};
const byte kInstruction_add_al_r0_sp_0x10c[] = {
  0x43, 0xa8 // add al r0 sp 0x10c
};
const byte kInstruction_add_al_r0_sp_0x110[] = {
  0x44, 0xa8 // add al r0 sp 0x110
};
const byte kInstruction_add_al_r0_sp_0x114[] = {
  0x45, 0xa8 // add al r0 sp 0x114
};
const byte kInstruction_add_al_r0_sp_0x118[] = {
  0x46, 0xa8 // add al r0 sp 0x118
};
const byte kInstruction_add_al_r0_sp_0x11c[] = {
  0x47, 0xa8 // add al r0 sp 0x11c
};
const byte kInstruction_add_al_r0_sp_0x120[] = {
  0x48, 0xa8 // add al r0 sp 0x120
};
const byte kInstruction_add_al_r0_sp_0x124[] = {
  0x49, 0xa8 // add al r0 sp 0x124
};
const byte kInstruction_add_al_r0_sp_0x128[] = {
  0x4a, 0xa8 // add al r0 sp 0x128
};
const byte kInstruction_add_al_r0_sp_0x12c[] = {
  0x4b, 0xa8 // add al r0 sp 0x12c
};
const byte kInstruction_add_al_r0_sp_0x130[] = {
  0x4c, 0xa8 // add al r0 sp 0x130
};
const byte kInstruction_add_al_r0_sp_0x134[] = {
  0x4d, 0xa8 // add al r0 sp 0x134
};
const byte kInstruction_add_al_r0_sp_0x138[] = {
  0x4e, 0xa8 // add al r0 sp 0x138
};
const byte kInstruction_add_al_r0_sp_0x13c[] = {
  0x4f, 0xa8 // add al r0 sp 0x13c
};
const byte kInstruction_add_al_r0_sp_0x140[] = {
  0x50, 0xa8 // add al r0 sp 0x140
};
const byte kInstruction_add_al_r0_sp_0x144[] = {
  0x51, 0xa8 // add al r0 sp 0x144
};
const byte kInstruction_add_al_r0_sp_0x148[] = {
  0x52, 0xa8 // add al r0 sp 0x148
};
const byte kInstruction_add_al_r0_sp_0x14c[] = {
  0x53, 0xa8 // add al r0 sp 0x14c
};
const byte kInstruction_add_al_r0_sp_0x150[] = {
  0x54, 0xa8 // add al r0 sp 0x150
};
const byte kInstruction_add_al_r0_sp_0x154[] = {
  0x55, 0xa8 // add al r0 sp 0x154
};
const byte kInstruction_add_al_r0_sp_0x158[] = {
  0x56, 0xa8 // add al r0 sp 0x158
};
const byte kInstruction_add_al_r0_sp_0x15c[] = {
  0x57, 0xa8 // add al r0 sp 0x15c
};
const byte kInstruction_add_al_r0_sp_0x160[] = {
  0x58, 0xa8 // add al r0 sp 0x160
};
const byte kInstruction_add_al_r0_sp_0x164[] = {
  0x59, 0xa8 // add al r0 sp 0x164
};
const byte kInstruction_add_al_r0_sp_0x168[] = {
  0x5a, 0xa8 // add al r0 sp 0x168
};
const byte kInstruction_add_al_r0_sp_0x16c[] = {
  0x5b, 0xa8 // add al r0 sp 0x16c
};
const byte kInstruction_add_al_r0_sp_0x170[] = {
  0x5c, 0xa8 // add al r0 sp 0x170
};
const byte kInstruction_add_al_r0_sp_0x174[] = {
  0x5d, 0xa8 // add al r0 sp 0x174
};
const byte kInstruction_add_al_r0_sp_0x178[] = {
  0x5e, 0xa8 // add al r0 sp 0x178
};
const byte kInstruction_add_al_r0_sp_0x17c[] = {
  0x5f, 0xa8 // add al r0 sp 0x17c
};
const byte kInstruction_add_al_r0_sp_0x180[] = {
  0x60, 0xa8 // add al r0 sp 0x180
};
const byte kInstruction_add_al_r0_sp_0x184[] = {
  0x61, 0xa8 // add al r0 sp 0x184
};
const byte kInstruction_add_al_r0_sp_0x188[] = {
  0x62, 0xa8 // add al r0 sp 0x188
};
const byte kInstruction_add_al_r0_sp_0x18c[] = {
  0x63, 0xa8 // add al r0 sp 0x18c
};
const byte kInstruction_add_al_r0_sp_0x190[] = {
  0x64, 0xa8 // add al r0 sp 0x190
};
const byte kInstruction_add_al_r0_sp_0x194[] = {
  0x65, 0xa8 // add al r0 sp 0x194
};
const byte kInstruction_add_al_r0_sp_0x198[] = {
  0x66, 0xa8 // add al r0 sp 0x198
};
const byte kInstruction_add_al_r0_sp_0x19c[] = {
  0x67, 0xa8 // add al r0 sp 0x19c
};
const byte kInstruction_add_al_r0_sp_0x1a0[] = {
  0x68, 0xa8 // add al r0 sp 0x1a0
};
const byte kInstruction_add_al_r0_sp_0x1a4[] = {
  0x69, 0xa8 // add al r0 sp 0x1a4
};
const byte kInstruction_add_al_r0_sp_0x1a8[] = {
  0x6a, 0xa8 // add al r0 sp 0x1a8
};
const byte kInstruction_add_al_r0_sp_0x1ac[] = {
  0x6b, 0xa8 // add al r0 sp 0x1ac
};
const byte kInstruction_add_al_r0_sp_0x1b0[] = {
  0x6c, 0xa8 // add al r0 sp 0x1b0
};
const byte kInstruction_add_al_r0_sp_0x1b4[] = {
  0x6d, 0xa8 // add al r0 sp 0x1b4
};
const byte kInstruction_add_al_r0_sp_0x1b8[] = {
  0x6e, 0xa8 // add al r0 sp 0x1b8
};
const byte kInstruction_add_al_r0_sp_0x1bc[] = {
  0x6f, 0xa8 // add al r0 sp 0x1bc
};
const byte kInstruction_add_al_r0_sp_0x1c0[] = {
  0x70, 0xa8 // add al r0 sp 0x1c0
};
const byte kInstruction_add_al_r0_sp_0x1c4[] = {
  0x71, 0xa8 // add al r0 sp 0x1c4
};
const byte kInstruction_add_al_r0_sp_0x1c8[] = {
  0x72, 0xa8 // add al r0 sp 0x1c8
};
const byte kInstruction_add_al_r0_sp_0x1cc[] = {
  0x73, 0xa8 // add al r0 sp 0x1cc
};
const byte kInstruction_add_al_r0_sp_0x1d0[] = {
  0x74, 0xa8 // add al r0 sp 0x1d0
};
const byte kInstruction_add_al_r0_sp_0x1d4[] = {
  0x75, 0xa8 // add al r0 sp 0x1d4
};
const byte kInstruction_add_al_r0_sp_0x1d8[] = {
  0x76, 0xa8 // add al r0 sp 0x1d8
};
const byte kInstruction_add_al_r0_sp_0x1dc[] = {
  0x77, 0xa8 // add al r0 sp 0x1dc
};
const byte kInstruction_add_al_r0_sp_0x1e0[] = {
  0x78, 0xa8 // add al r0 sp 0x1e0
};
const byte kInstruction_add_al_r0_sp_0x1e4[] = {
  0x79, 0xa8 // add al r0 sp 0x1e4
};
const byte kInstruction_add_al_r0_sp_0x1e8[] = {
  0x7a, 0xa8 // add al r0 sp 0x1e8
};
const byte kInstruction_add_al_r0_sp_0x1ec[] = {
  0x7b, 0xa8 // add al r0 sp 0x1ec
};
const byte kInstruction_add_al_r0_sp_0x1f0[] = {
  0x7c, 0xa8 // add al r0 sp 0x1f0
};
const byte kInstruction_add_al_r0_sp_0x1f4[] = {
  0x7d, 0xa8 // add al r0 sp 0x1f4
};
const byte kInstruction_add_al_r0_sp_0x1f8[] = {
  0x7e, 0xa8 // add al r0 sp 0x1f8
};
const byte kInstruction_add_al_r0_sp_0x1fc[] = {
  0x7f, 0xa8 // add al r0 sp 0x1fc
};
const byte kInstruction_add_al_r0_sp_0x200[] = {
  0x80, 0xa8 // add al r0 sp 0x200
};
const byte kInstruction_add_al_r0_sp_0x204[] = {
  0x81, 0xa8 // add al r0 sp 0x204
};
const byte kInstruction_add_al_r0_sp_0x208[] = {
  0x82, 0xa8 // add al r0 sp 0x208
};
const byte kInstruction_add_al_r0_sp_0x20c[] = {
  0x83, 0xa8 // add al r0 sp 0x20c
};
const byte kInstruction_add_al_r0_sp_0x210[] = {
  0x84, 0xa8 // add al r0 sp 0x210
};
const byte kInstruction_add_al_r0_sp_0x214[] = {
  0x85, 0xa8 // add al r0 sp 0x214
};
const byte kInstruction_add_al_r0_sp_0x218[] = {
  0x86, 0xa8 // add al r0 sp 0x218
};
const byte kInstruction_add_al_r0_sp_0x21c[] = {
  0x87, 0xa8 // add al r0 sp 0x21c
};
const byte kInstruction_add_al_r0_sp_0x220[] = {
  0x88, 0xa8 // add al r0 sp 0x220
};
const byte kInstruction_add_al_r0_sp_0x224[] = {
  0x89, 0xa8 // add al r0 sp 0x224
};
const byte kInstruction_add_al_r0_sp_0x228[] = {
  0x8a, 0xa8 // add al r0 sp 0x228
};
const byte kInstruction_add_al_r0_sp_0x22c[] = {
  0x8b, 0xa8 // add al r0 sp 0x22c
};
const byte kInstruction_add_al_r0_sp_0x230[] = {
  0x8c, 0xa8 // add al r0 sp 0x230
};
const byte kInstruction_add_al_r0_sp_0x234[] = {
  0x8d, 0xa8 // add al r0 sp 0x234
};
const byte kInstruction_add_al_r0_sp_0x238[] = {
  0x8e, 0xa8 // add al r0 sp 0x238
};
const byte kInstruction_add_al_r0_sp_0x23c[] = {
  0x8f, 0xa8 // add al r0 sp 0x23c
};
const byte kInstruction_add_al_r0_sp_0x240[] = {
  0x90, 0xa8 // add al r0 sp 0x240
};
const byte kInstruction_add_al_r0_sp_0x244[] = {
  0x91, 0xa8 // add al r0 sp 0x244
};
const byte kInstruction_add_al_r0_sp_0x248[] = {
  0x92, 0xa8 // add al r0 sp 0x248
};
const byte kInstruction_add_al_r0_sp_0x24c[] = {
  0x93, 0xa8 // add al r0 sp 0x24c
};
const byte kInstruction_add_al_r0_sp_0x250[] = {
  0x94, 0xa8 // add al r0 sp 0x250
};
const byte kInstruction_add_al_r0_sp_0x254[] = {
  0x95, 0xa8 // add al r0 sp 0x254
};
const byte kInstruction_add_al_r0_sp_0x258[] = {
  0x96, 0xa8 // add al r0 sp 0x258
};
const byte kInstruction_add_al_r0_sp_0x25c[] = {
  0x97, 0xa8 // add al r0 sp 0x25c
};
const byte kInstruction_add_al_r0_sp_0x260[] = {
  0x98, 0xa8 // add al r0 sp 0x260
};
const byte kInstruction_add_al_r0_sp_0x264[] = {
  0x99, 0xa8 // add al r0 sp 0x264
};
const byte kInstruction_add_al_r0_sp_0x268[] = {
  0x9a, 0xa8 // add al r0 sp 0x268
};
const byte kInstruction_add_al_r0_sp_0x26c[] = {
  0x9b, 0xa8 // add al r0 sp 0x26c
};
const byte kInstruction_add_al_r0_sp_0x270[] = {
  0x9c, 0xa8 // add al r0 sp 0x270
};
const byte kInstruction_add_al_r0_sp_0x274[] = {
  0x9d, 0xa8 // add al r0 sp 0x274
};
const byte kInstruction_add_al_r0_sp_0x278[] = {
  0x9e, 0xa8 // add al r0 sp 0x278
};
const byte kInstruction_add_al_r0_sp_0x27c[] = {
  0x9f, 0xa8 // add al r0 sp 0x27c
};
const byte kInstruction_add_al_r0_sp_0x280[] = {
  0xa0, 0xa8 // add al r0 sp 0x280
};
const byte kInstruction_add_al_r0_sp_0x284[] = {
  0xa1, 0xa8 // add al r0 sp 0x284
};
const byte kInstruction_add_al_r0_sp_0x288[] = {
  0xa2, 0xa8 // add al r0 sp 0x288
};
const byte kInstruction_add_al_r0_sp_0x28c[] = {
  0xa3, 0xa8 // add al r0 sp 0x28c
};
const byte kInstruction_add_al_r0_sp_0x290[] = {
  0xa4, 0xa8 // add al r0 sp 0x290
};
const byte kInstruction_add_al_r0_sp_0x294[] = {
  0xa5, 0xa8 // add al r0 sp 0x294
};
const byte kInstruction_add_al_r0_sp_0x298[] = {
  0xa6, 0xa8 // add al r0 sp 0x298
};
const byte kInstruction_add_al_r0_sp_0x29c[] = {
  0xa7, 0xa8 // add al r0 sp 0x29c
};
const byte kInstruction_add_al_r0_sp_0x2a0[] = {
  0xa8, 0xa8 // add al r0 sp 0x2a0
};
const byte kInstruction_add_al_r0_sp_0x2a4[] = {
  0xa9, 0xa8 // add al r0 sp 0x2a4
};
const byte kInstruction_add_al_r0_sp_0x2a8[] = {
  0xaa, 0xa8 // add al r0 sp 0x2a8
};
const byte kInstruction_add_al_r0_sp_0x2ac[] = {
  0xab, 0xa8 // add al r0 sp 0x2ac
};
const byte kInstruction_add_al_r0_sp_0x2b0[] = {
  0xac, 0xa8 // add al r0 sp 0x2b0
};
const byte kInstruction_add_al_r0_sp_0x2b4[] = {
  0xad, 0xa8 // add al r0 sp 0x2b4
};
const byte kInstruction_add_al_r0_sp_0x2b8[] = {
  0xae, 0xa8 // add al r0 sp 0x2b8
};
const byte kInstruction_add_al_r0_sp_0x2bc[] = {
  0xaf, 0xa8 // add al r0 sp 0x2bc
};
const byte kInstruction_add_al_r0_sp_0x2c0[] = {
  0xb0, 0xa8 // add al r0 sp 0x2c0
};
const byte kInstruction_add_al_r0_sp_0x2c4[] = {
  0xb1, 0xa8 // add al r0 sp 0x2c4
};
const byte kInstruction_add_al_r0_sp_0x2c8[] = {
  0xb2, 0xa8 // add al r0 sp 0x2c8
};
const byte kInstruction_add_al_r0_sp_0x2cc[] = {
  0xb3, 0xa8 // add al r0 sp 0x2cc
};
const byte kInstruction_add_al_r0_sp_0x2d0[] = {
  0xb4, 0xa8 // add al r0 sp 0x2d0
};
const byte kInstruction_add_al_r0_sp_0x2d4[] = {
  0xb5, 0xa8 // add al r0 sp 0x2d4
};
const byte kInstruction_add_al_r0_sp_0x2d8[] = {
  0xb6, 0xa8 // add al r0 sp 0x2d8
};
const byte kInstruction_add_al_r0_sp_0x2dc[] = {
  0xb7, 0xa8 // add al r0 sp 0x2dc
};
const byte kInstruction_add_al_r0_sp_0x2e0[] = {
  0xb8, 0xa8 // add al r0 sp 0x2e0
};
const byte kInstruction_add_al_r0_sp_0x2e4[] = {
  0xb9, 0xa8 // add al r0 sp 0x2e4
};
const byte kInstruction_add_al_r0_sp_0x2e8[] = {
  0xba, 0xa8 // add al r0 sp 0x2e8
};
const byte kInstruction_add_al_r0_sp_0x2ec[] = {
  0xbb, 0xa8 // add al r0 sp 0x2ec
};
const byte kInstruction_add_al_r0_sp_0x2f0[] = {
  0xbc, 0xa8 // add al r0 sp 0x2f0
};
const byte kInstruction_add_al_r0_sp_0x2f4[] = {
  0xbd, 0xa8 // add al r0 sp 0x2f4
};
const byte kInstruction_add_al_r0_sp_0x2f8[] = {
  0xbe, 0xa8 // add al r0 sp 0x2f8
};
const byte kInstruction_add_al_r0_sp_0x2fc[] = {
  0xbf, 0xa8 // add al r0 sp 0x2fc
};
const byte kInstruction_add_al_r0_sp_0x300[] = {
  0xc0, 0xa8 // add al r0 sp 0x300
};
const byte kInstruction_add_al_r0_sp_0x304[] = {
  0xc1, 0xa8 // add al r0 sp 0x304
};
const byte kInstruction_add_al_r0_sp_0x308[] = {
  0xc2, 0xa8 // add al r0 sp 0x308
};
const byte kInstruction_add_al_r0_sp_0x30c[] = {
  0xc3, 0xa8 // add al r0 sp 0x30c
};
const byte kInstruction_add_al_r0_sp_0x310[] = {
  0xc4, 0xa8 // add al r0 sp 0x310
};
const byte kInstruction_add_al_r0_sp_0x314[] = {
  0xc5, 0xa8 // add al r0 sp 0x314
};
const byte kInstruction_add_al_r0_sp_0x318[] = {
  0xc6, 0xa8 // add al r0 sp 0x318
};
const byte kInstruction_add_al_r0_sp_0x31c[] = {
  0xc7, 0xa8 // add al r0 sp 0x31c
};
const byte kInstruction_add_al_r0_sp_0x320[] = {
  0xc8, 0xa8 // add al r0 sp 0x320
};
const byte kInstruction_add_al_r0_sp_0x324[] = {
  0xc9, 0xa8 // add al r0 sp 0x324
};
const byte kInstruction_add_al_r0_sp_0x328[] = {
  0xca, 0xa8 // add al r0 sp 0x328
};
const byte kInstruction_add_al_r0_sp_0x32c[] = {
  0xcb, 0xa8 // add al r0 sp 0x32c
};
const byte kInstruction_add_al_r0_sp_0x330[] = {
  0xcc, 0xa8 // add al r0 sp 0x330
};
const byte kInstruction_add_al_r0_sp_0x334[] = {
  0xcd, 0xa8 // add al r0 sp 0x334
};
const byte kInstruction_add_al_r0_sp_0x338[] = {
  0xce, 0xa8 // add al r0 sp 0x338
};
const byte kInstruction_add_al_r0_sp_0x33c[] = {
  0xcf, 0xa8 // add al r0 sp 0x33c
};
const byte kInstruction_add_al_r0_sp_0x340[] = {
  0xd0, 0xa8 // add al r0 sp 0x340
};
const byte kInstruction_add_al_r0_sp_0x344[] = {
  0xd1, 0xa8 // add al r0 sp 0x344
};
const byte kInstruction_add_al_r0_sp_0x348[] = {
  0xd2, 0xa8 // add al r0 sp 0x348
};
const byte kInstruction_add_al_r0_sp_0x34c[] = {
  0xd3, 0xa8 // add al r0 sp 0x34c
};
const byte kInstruction_add_al_r0_sp_0x350[] = {
  0xd4, 0xa8 // add al r0 sp 0x350
};
const byte kInstruction_add_al_r0_sp_0x354[] = {
  0xd5, 0xa8 // add al r0 sp 0x354
};
const byte kInstruction_add_al_r0_sp_0x358[] = {
  0xd6, 0xa8 // add al r0 sp 0x358
};
const byte kInstruction_add_al_r0_sp_0x35c[] = {
  0xd7, 0xa8 // add al r0 sp 0x35c
};
const byte kInstruction_add_al_r0_sp_0x360[] = {
  0xd8, 0xa8 // add al r0 sp 0x360
};
const byte kInstruction_add_al_r0_sp_0x364[] = {
  0xd9, 0xa8 // add al r0 sp 0x364
};
const byte kInstruction_add_al_r0_sp_0x368[] = {
  0xda, 0xa8 // add al r0 sp 0x368
};
const byte kInstruction_add_al_r0_sp_0x36c[] = {
  0xdb, 0xa8 // add al r0 sp 0x36c
};
const byte kInstruction_add_al_r0_sp_0x370[] = {
  0xdc, 0xa8 // add al r0 sp 0x370
};
const byte kInstruction_add_al_r0_sp_0x374[] = {
  0xdd, 0xa8 // add al r0 sp 0x374
};
const byte kInstruction_add_al_r0_sp_0x378[] = {
  0xde, 0xa8 // add al r0 sp 0x378
};
const byte kInstruction_add_al_r0_sp_0x37c[] = {
  0xdf, 0xa8 // add al r0 sp 0x37c
};
const byte kInstruction_add_al_r0_sp_0x380[] = {
  0xe0, 0xa8 // add al r0 sp 0x380
};
const byte kInstruction_add_al_r0_sp_0x384[] = {
  0xe1, 0xa8 // add al r0 sp 0x384
};
const byte kInstruction_add_al_r0_sp_0x388[] = {
  0xe2, 0xa8 // add al r0 sp 0x388
};
const byte kInstruction_add_al_r0_sp_0x38c[] = {
  0xe3, 0xa8 // add al r0 sp 0x38c
};
const byte kInstruction_add_al_r0_sp_0x390[] = {
  0xe4, 0xa8 // add al r0 sp 0x390
};
const byte kInstruction_add_al_r0_sp_0x394[] = {
  0xe5, 0xa8 // add al r0 sp 0x394
};
const byte kInstruction_add_al_r0_sp_0x398[] = {
  0xe6, 0xa8 // add al r0 sp 0x398
};
const byte kInstruction_add_al_r0_sp_0x39c[] = {
  0xe7, 0xa8 // add al r0 sp 0x39c
};
const byte kInstruction_add_al_r0_sp_0x3a0[] = {
  0xe8, 0xa8 // add al r0 sp 0x3a0
};
const byte kInstruction_add_al_r0_sp_0x3a4[] = {
  0xe9, 0xa8 // add al r0 sp 0x3a4
};
const byte kInstruction_add_al_r0_sp_0x3a8[] = {
  0xea, 0xa8 // add al r0 sp 0x3a8
};
const byte kInstruction_add_al_r0_sp_0x3ac[] = {
  0xeb, 0xa8 // add al r0 sp 0x3ac
};
const byte kInstruction_add_al_r0_sp_0x3b0[] = {
  0xec, 0xa8 // add al r0 sp 0x3b0
};
const byte kInstruction_add_al_r0_sp_0x3b4[] = {
  0xed, 0xa8 // add al r0 sp 0x3b4
};
const byte kInstruction_add_al_r0_sp_0x3b8[] = {
  0xee, 0xa8 // add al r0 sp 0x3b8
};
const byte kInstruction_add_al_r0_sp_0x3bc[] = {
  0xef, 0xa8 // add al r0 sp 0x3bc
};
const byte kInstruction_add_al_r0_sp_0x3c0[] = {
  0xf0, 0xa8 // add al r0 sp 0x3c0
};
const byte kInstruction_add_al_r0_sp_0x3c4[] = {
  0xf1, 0xa8 // add al r0 sp 0x3c4
};
const byte kInstruction_add_al_r0_sp_0x3c8[] = {
  0xf2, 0xa8 // add al r0 sp 0x3c8
};
const byte kInstruction_add_al_r0_sp_0x3cc[] = {
  0xf3, 0xa8 // add al r0 sp 0x3cc
};
const byte kInstruction_add_al_r0_sp_0x3d0[] = {
  0xf4, 0xa8 // add al r0 sp 0x3d0
};
const byte kInstruction_add_al_r0_sp_0x3d4[] = {
  0xf5, 0xa8 // add al r0 sp 0x3d4
};
const byte kInstruction_add_al_r0_sp_0x3d8[] = {
  0xf6, 0xa8 // add al r0 sp 0x3d8
};
const byte kInstruction_add_al_r0_sp_0x3dc[] = {
  0xf7, 0xa8 // add al r0 sp 0x3dc
};
const byte kInstruction_add_al_r0_sp_0x3e0[] = {
  0xf8, 0xa8 // add al r0 sp 0x3e0
};
const byte kInstruction_add_al_r0_sp_0x3e4[] = {
  0xf9, 0xa8 // add al r0 sp 0x3e4
};
const byte kInstruction_add_al_r0_sp_0x3e8[] = {
  0xfa, 0xa8 // add al r0 sp 0x3e8
};
const byte kInstruction_add_al_r0_sp_0x3ec[] = {
  0xfb, 0xa8 // add al r0 sp 0x3ec
};
const byte kInstruction_add_al_r0_sp_0x3f0[] = {
  0xfc, 0xa8 // add al r0 sp 0x3f0
};
const byte kInstruction_add_al_r0_sp_0x3f4[] = {
  0xfd, 0xa8 // add al r0 sp 0x3f4
};
const byte kInstruction_add_al_r0_sp_0x3f8[] = {
  0xfe, 0xa8 // add al r0 sp 0x3f8
};
const byte kInstruction_add_al_r0_sp_0x3fc[] = {
  0xff, 0xa8 // add al r0 sp 0x3fc
};
const TestResult kReferenceadd[] = {
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x0),
    kInstruction_add_al_r0_sp_0x0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x4),
    kInstruction_add_al_r0_sp_0x4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x8),
    kInstruction_add_al_r0_sp_0x8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xc),
    kInstruction_add_al_r0_sp_0xc,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x10),
    kInstruction_add_al_r0_sp_0x10,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x14),
    kInstruction_add_al_r0_sp_0x14,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x18),
    kInstruction_add_al_r0_sp_0x18,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1c),
    kInstruction_add_al_r0_sp_0x1c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x20),
    kInstruction_add_al_r0_sp_0x20,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x24),
    kInstruction_add_al_r0_sp_0x24,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x28),
    kInstruction_add_al_r0_sp_0x28,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2c),
    kInstruction_add_al_r0_sp_0x2c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x30),
    kInstruction_add_al_r0_sp_0x30,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x34),
    kInstruction_add_al_r0_sp_0x34,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x38),
    kInstruction_add_al_r0_sp_0x38,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3c),
    kInstruction_add_al_r0_sp_0x3c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x40),
    kInstruction_add_al_r0_sp_0x40,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x44),
    kInstruction_add_al_r0_sp_0x44,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x48),
    kInstruction_add_al_r0_sp_0x48,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x4c),
    kInstruction_add_al_r0_sp_0x4c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x50),
    kInstruction_add_al_r0_sp_0x50,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x54),
    kInstruction_add_al_r0_sp_0x54,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x58),
    kInstruction_add_al_r0_sp_0x58,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x5c),
    kInstruction_add_al_r0_sp_0x5c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x60),
    kInstruction_add_al_r0_sp_0x60,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x64),
    kInstruction_add_al_r0_sp_0x64,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x68),
    kInstruction_add_al_r0_sp_0x68,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x6c),
    kInstruction_add_al_r0_sp_0x6c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x70),
    kInstruction_add_al_r0_sp_0x70,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x74),
    kInstruction_add_al_r0_sp_0x74,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x78),
    kInstruction_add_al_r0_sp_0x78,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x7c),
    kInstruction_add_al_r0_sp_0x7c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x80),
    kInstruction_add_al_r0_sp_0x80,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x84),
    kInstruction_add_al_r0_sp_0x84,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x88),
    kInstruction_add_al_r0_sp_0x88,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x8c),
    kInstruction_add_al_r0_sp_0x8c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x90),
    kInstruction_add_al_r0_sp_0x90,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x94),
    kInstruction_add_al_r0_sp_0x94,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x98),
    kInstruction_add_al_r0_sp_0x98,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x9c),
    kInstruction_add_al_r0_sp_0x9c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xa0),
    kInstruction_add_al_r0_sp_0xa0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xa4),
    kInstruction_add_al_r0_sp_0xa4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xa8),
    kInstruction_add_al_r0_sp_0xa8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xac),
    kInstruction_add_al_r0_sp_0xac,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xb0),
    kInstruction_add_al_r0_sp_0xb0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xb4),
    kInstruction_add_al_r0_sp_0xb4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xb8),
    kInstruction_add_al_r0_sp_0xb8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xbc),
    kInstruction_add_al_r0_sp_0xbc,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xc0),
    kInstruction_add_al_r0_sp_0xc0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xc4),
    kInstruction_add_al_r0_sp_0xc4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xc8),
    kInstruction_add_al_r0_sp_0xc8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xcc),
    kInstruction_add_al_r0_sp_0xcc,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xd0),
    kInstruction_add_al_r0_sp_0xd0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xd4),
    kInstruction_add_al_r0_sp_0xd4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xd8),
    kInstruction_add_al_r0_sp_0xd8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xdc),
    kInstruction_add_al_r0_sp_0xdc,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xe0),
    kInstruction_add_al_r0_sp_0xe0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xe4),
    kInstruction_add_al_r0_sp_0xe4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xe8),
    kInstruction_add_al_r0_sp_0xe8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xec),
    kInstruction_add_al_r0_sp_0xec,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xf0),
    kInstruction_add_al_r0_sp_0xf0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xf4),
    kInstruction_add_al_r0_sp_0xf4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xf8),
    kInstruction_add_al_r0_sp_0xf8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0xfc),
    kInstruction_add_al_r0_sp_0xfc,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x100),
    kInstruction_add_al_r0_sp_0x100,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x104),
    kInstruction_add_al_r0_sp_0x104,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x108),
    kInstruction_add_al_r0_sp_0x108,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x10c),
    kInstruction_add_al_r0_sp_0x10c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x110),
    kInstruction_add_al_r0_sp_0x110,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x114),
    kInstruction_add_al_r0_sp_0x114,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x118),
    kInstruction_add_al_r0_sp_0x118,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x11c),
    kInstruction_add_al_r0_sp_0x11c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x120),
    kInstruction_add_al_r0_sp_0x120,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x124),
    kInstruction_add_al_r0_sp_0x124,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x128),
    kInstruction_add_al_r0_sp_0x128,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x12c),
    kInstruction_add_al_r0_sp_0x12c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x130),
    kInstruction_add_al_r0_sp_0x130,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x134),
    kInstruction_add_al_r0_sp_0x134,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x138),
    kInstruction_add_al_r0_sp_0x138,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x13c),
    kInstruction_add_al_r0_sp_0x13c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x140),
    kInstruction_add_al_r0_sp_0x140,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x144),
    kInstruction_add_al_r0_sp_0x144,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x148),
    kInstruction_add_al_r0_sp_0x148,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x14c),
    kInstruction_add_al_r0_sp_0x14c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x150),
    kInstruction_add_al_r0_sp_0x150,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x154),
    kInstruction_add_al_r0_sp_0x154,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x158),
    kInstruction_add_al_r0_sp_0x158,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x15c),
    kInstruction_add_al_r0_sp_0x15c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x160),
    kInstruction_add_al_r0_sp_0x160,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x164),
    kInstruction_add_al_r0_sp_0x164,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x168),
    kInstruction_add_al_r0_sp_0x168,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x16c),
    kInstruction_add_al_r0_sp_0x16c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x170),
    kInstruction_add_al_r0_sp_0x170,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x174),
    kInstruction_add_al_r0_sp_0x174,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x178),
    kInstruction_add_al_r0_sp_0x178,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x17c),
    kInstruction_add_al_r0_sp_0x17c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x180),
    kInstruction_add_al_r0_sp_0x180,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x184),
    kInstruction_add_al_r0_sp_0x184,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x188),
    kInstruction_add_al_r0_sp_0x188,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x18c),
    kInstruction_add_al_r0_sp_0x18c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x190),
    kInstruction_add_al_r0_sp_0x190,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x194),
    kInstruction_add_al_r0_sp_0x194,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x198),
    kInstruction_add_al_r0_sp_0x198,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x19c),
    kInstruction_add_al_r0_sp_0x19c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1a0),
    kInstruction_add_al_r0_sp_0x1a0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1a4),
    kInstruction_add_al_r0_sp_0x1a4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1a8),
    kInstruction_add_al_r0_sp_0x1a8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1ac),
    kInstruction_add_al_r0_sp_0x1ac,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1b0),
    kInstruction_add_al_r0_sp_0x1b0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1b4),
    kInstruction_add_al_r0_sp_0x1b4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1b8),
    kInstruction_add_al_r0_sp_0x1b8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1bc),
    kInstruction_add_al_r0_sp_0x1bc,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1c0),
    kInstruction_add_al_r0_sp_0x1c0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1c4),
    kInstruction_add_al_r0_sp_0x1c4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1c8),
    kInstruction_add_al_r0_sp_0x1c8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1cc),
    kInstruction_add_al_r0_sp_0x1cc,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1d0),
    kInstruction_add_al_r0_sp_0x1d0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1d4),
    kInstruction_add_al_r0_sp_0x1d4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1d8),
    kInstruction_add_al_r0_sp_0x1d8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1dc),
    kInstruction_add_al_r0_sp_0x1dc,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1e0),
    kInstruction_add_al_r0_sp_0x1e0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1e4),
    kInstruction_add_al_r0_sp_0x1e4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1e8),
    kInstruction_add_al_r0_sp_0x1e8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1ec),
    kInstruction_add_al_r0_sp_0x1ec,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1f0),
    kInstruction_add_al_r0_sp_0x1f0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1f4),
    kInstruction_add_al_r0_sp_0x1f4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1f8),
    kInstruction_add_al_r0_sp_0x1f8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x1fc),
    kInstruction_add_al_r0_sp_0x1fc,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x200),
    kInstruction_add_al_r0_sp_0x200,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x204),
    kInstruction_add_al_r0_sp_0x204,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x208),
    kInstruction_add_al_r0_sp_0x208,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x20c),
    kInstruction_add_al_r0_sp_0x20c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x210),
    kInstruction_add_al_r0_sp_0x210,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x214),
    kInstruction_add_al_r0_sp_0x214,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x218),
    kInstruction_add_al_r0_sp_0x218,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x21c),
    kInstruction_add_al_r0_sp_0x21c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x220),
    kInstruction_add_al_r0_sp_0x220,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x224),
    kInstruction_add_al_r0_sp_0x224,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x228),
    kInstruction_add_al_r0_sp_0x228,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x22c),
    kInstruction_add_al_r0_sp_0x22c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x230),
    kInstruction_add_al_r0_sp_0x230,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x234),
    kInstruction_add_al_r0_sp_0x234,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x238),
    kInstruction_add_al_r0_sp_0x238,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x23c),
    kInstruction_add_al_r0_sp_0x23c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x240),
    kInstruction_add_al_r0_sp_0x240,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x244),
    kInstruction_add_al_r0_sp_0x244,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x248),
    kInstruction_add_al_r0_sp_0x248,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x24c),
    kInstruction_add_al_r0_sp_0x24c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x250),
    kInstruction_add_al_r0_sp_0x250,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x254),
    kInstruction_add_al_r0_sp_0x254,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x258),
    kInstruction_add_al_r0_sp_0x258,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x25c),
    kInstruction_add_al_r0_sp_0x25c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x260),
    kInstruction_add_al_r0_sp_0x260,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x264),
    kInstruction_add_al_r0_sp_0x264,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x268),
    kInstruction_add_al_r0_sp_0x268,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x26c),
    kInstruction_add_al_r0_sp_0x26c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x270),
    kInstruction_add_al_r0_sp_0x270,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x274),
    kInstruction_add_al_r0_sp_0x274,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x278),
    kInstruction_add_al_r0_sp_0x278,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x27c),
    kInstruction_add_al_r0_sp_0x27c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x280),
    kInstruction_add_al_r0_sp_0x280,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x284),
    kInstruction_add_al_r0_sp_0x284,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x288),
    kInstruction_add_al_r0_sp_0x288,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x28c),
    kInstruction_add_al_r0_sp_0x28c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x290),
    kInstruction_add_al_r0_sp_0x290,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x294),
    kInstruction_add_al_r0_sp_0x294,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x298),
    kInstruction_add_al_r0_sp_0x298,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x29c),
    kInstruction_add_al_r0_sp_0x29c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2a0),
    kInstruction_add_al_r0_sp_0x2a0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2a4),
    kInstruction_add_al_r0_sp_0x2a4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2a8),
    kInstruction_add_al_r0_sp_0x2a8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2ac),
    kInstruction_add_al_r0_sp_0x2ac,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2b0),
    kInstruction_add_al_r0_sp_0x2b0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2b4),
    kInstruction_add_al_r0_sp_0x2b4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2b8),
    kInstruction_add_al_r0_sp_0x2b8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2bc),
    kInstruction_add_al_r0_sp_0x2bc,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2c0),
    kInstruction_add_al_r0_sp_0x2c0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2c4),
    kInstruction_add_al_r0_sp_0x2c4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2c8),
    kInstruction_add_al_r0_sp_0x2c8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2cc),
    kInstruction_add_al_r0_sp_0x2cc,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2d0),
    kInstruction_add_al_r0_sp_0x2d0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2d4),
    kInstruction_add_al_r0_sp_0x2d4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2d8),
    kInstruction_add_al_r0_sp_0x2d8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2dc),
    kInstruction_add_al_r0_sp_0x2dc,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2e0),
    kInstruction_add_al_r0_sp_0x2e0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2e4),
    kInstruction_add_al_r0_sp_0x2e4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2e8),
    kInstruction_add_al_r0_sp_0x2e8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2ec),
    kInstruction_add_al_r0_sp_0x2ec,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2f0),
    kInstruction_add_al_r0_sp_0x2f0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2f4),
    kInstruction_add_al_r0_sp_0x2f4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2f8),
    kInstruction_add_al_r0_sp_0x2f8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x2fc),
    kInstruction_add_al_r0_sp_0x2fc,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x300),
    kInstruction_add_al_r0_sp_0x300,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x304),
    kInstruction_add_al_r0_sp_0x304,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x308),
    kInstruction_add_al_r0_sp_0x308,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x30c),
    kInstruction_add_al_r0_sp_0x30c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x310),
    kInstruction_add_al_r0_sp_0x310,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x314),
    kInstruction_add_al_r0_sp_0x314,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x318),
    kInstruction_add_al_r0_sp_0x318,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x31c),
    kInstruction_add_al_r0_sp_0x31c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x320),
    kInstruction_add_al_r0_sp_0x320,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x324),
    kInstruction_add_al_r0_sp_0x324,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x328),
    kInstruction_add_al_r0_sp_0x328,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x32c),
    kInstruction_add_al_r0_sp_0x32c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x330),
    kInstruction_add_al_r0_sp_0x330,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x334),
    kInstruction_add_al_r0_sp_0x334,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x338),
    kInstruction_add_al_r0_sp_0x338,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x33c),
    kInstruction_add_al_r0_sp_0x33c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x340),
    kInstruction_add_al_r0_sp_0x340,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x344),
    kInstruction_add_al_r0_sp_0x344,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x348),
    kInstruction_add_al_r0_sp_0x348,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x34c),
    kInstruction_add_al_r0_sp_0x34c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x350),
    kInstruction_add_al_r0_sp_0x350,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x354),
    kInstruction_add_al_r0_sp_0x354,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x358),
    kInstruction_add_al_r0_sp_0x358,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x35c),
    kInstruction_add_al_r0_sp_0x35c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x360),
    kInstruction_add_al_r0_sp_0x360,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x364),
    kInstruction_add_al_r0_sp_0x364,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x368),
    kInstruction_add_al_r0_sp_0x368,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x36c),
    kInstruction_add_al_r0_sp_0x36c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x370),
    kInstruction_add_al_r0_sp_0x370,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x374),
    kInstruction_add_al_r0_sp_0x374,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x378),
    kInstruction_add_al_r0_sp_0x378,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x37c),
    kInstruction_add_al_r0_sp_0x37c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x380),
    kInstruction_add_al_r0_sp_0x380,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x384),
    kInstruction_add_al_r0_sp_0x384,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x388),
    kInstruction_add_al_r0_sp_0x388,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x38c),
    kInstruction_add_al_r0_sp_0x38c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x390),
    kInstruction_add_al_r0_sp_0x390,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x394),
    kInstruction_add_al_r0_sp_0x394,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x398),
    kInstruction_add_al_r0_sp_0x398,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x39c),
    kInstruction_add_al_r0_sp_0x39c,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3a0),
    kInstruction_add_al_r0_sp_0x3a0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3a4),
    kInstruction_add_al_r0_sp_0x3a4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3a8),
    kInstruction_add_al_r0_sp_0x3a8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3ac),
    kInstruction_add_al_r0_sp_0x3ac,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3b0),
    kInstruction_add_al_r0_sp_0x3b0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3b4),
    kInstruction_add_al_r0_sp_0x3b4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3b8),
    kInstruction_add_al_r0_sp_0x3b8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3bc),
    kInstruction_add_al_r0_sp_0x3bc,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3c0),
    kInstruction_add_al_r0_sp_0x3c0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3c4),
    kInstruction_add_al_r0_sp_0x3c4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3c8),
    kInstruction_add_al_r0_sp_0x3c8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3cc),
    kInstruction_add_al_r0_sp_0x3cc,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3d0),
    kInstruction_add_al_r0_sp_0x3d0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3d4),
    kInstruction_add_al_r0_sp_0x3d4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3d8),
    kInstruction_add_al_r0_sp_0x3d8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3dc),
    kInstruction_add_al_r0_sp_0x3dc,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3e0),
    kInstruction_add_al_r0_sp_0x3e0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3e4),
    kInstruction_add_al_r0_sp_0x3e4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3e8),
    kInstruction_add_al_r0_sp_0x3e8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3ec),
    kInstruction_add_al_r0_sp_0x3ec,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3f0),
    kInstruction_add_al_r0_sp_0x3f0,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3f4),
    kInstruction_add_al_r0_sp_0x3f4,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3f8),
    kInstruction_add_al_r0_sp_0x3f8,
  },
  {
    ARRAY_SIZE(kInstruction_add_al_r0_sp_0x3fc),
    kInstruction_add_al_r0_sp_0x3fc,
  },
};

#endif  // VIXL_ASSEMBLER_COND_RD_SP_OPERAND_IMM8_ADD_T32_H_
