// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_SIM_FRINTM_H_TRACE_AARCH64_H_
#define VIXL_SIM_FRINTM_H_TRACE_AARCH64_H_

const uint16_t kExpected_frintm_h[] = {
  0x0000,
  0x0000,
  0x0000,
  0x0000,
  0x0000,
  0x0000,
  0x3c00,
  0x3c00,
  0x3c00,
  0x4900,
  0x7bff,
  0x7c00,
  0x7f23,
  0x7e00,
  0x7f23,
  0x7e01,
  0x0000,
  0x0000,
  0x0000,
  0x8000,
  0xbc00,
  0xbc00,
  0xbc00,
  0xbc00,
  0xbc00,
  0xbc00,
  0xc000,
  0xc000,
  0xc900,
  0xfbff,
  0xfc00,
  0xff23,
  0xfe00,
  0xff23,
  0xfe01,
  0xbc00,
  0xbc00,
  0xbc00,
  0x6400,
  0x6401,
  0x6402,
  0x6403,
  0x6543,
  0x67fc,
  0x67fd,
  0x67fe,
  0x67ff,
  0x6000,
  0x6000,
  0x6002,
  0x6002,
  0x6320,
  0x63fc,
  0x63fc,
  0x63fe,
  0x63fe,
  0x5c00,
  0x5c00,
  0x5c00,
  0x5c00,
  0x5d30,
  0x5ffc,
  0x5ffc,
  0x5ffc,
  0x5ffc,
  0xe400,
  0xe401,
  0xe402,
  0xe403,
  0xe543,
  0xe7fc,
  0xe7fd,
  0xe7fe,
  0xe7ff,
  0xe000,
  0xe002,
  0xe002,
  0xe004,
  0xe322,
  0xe3fc,
  0xe3fe,
  0xe3fe,
  0xe400,
  0xdc00,
  0xdc04,
  0xdc04,
  0xdc04,
  0xdd34,
  0xdffc,
  0xe000,
  0xe000,
  0xe000,
  0x7e7f,
  0x7e91,
  0x7e00,
  0x7e91,
  0xfe7f,
  0xfe91,
  0xfe00,
  0xfe91,
  0xffff,
};
const unsigned kExpectedCount_frintm_h = 101;

#endif  // VIXL_SIM_FRINTM_H_TRACE_AARCH64_H_
