// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_SIM_FRECPE_4H_TRACE_AARCH64_H_
#define VIXL_SIM_FRECPE_4H_TRACE_AARCH64_H_

const uint16_t kExpected_NEON_frecpe_4H[] = {
  0xff23, 0xfe00, 0xff23, 0xfe01,
  0xfe00, 0xff23, 0xfe01, 0xfc00,
  0xff23, 0xfe01, 0xfc00, 0xf400,
  0xfe01, 0xfc00, 0xf400, 0xfc00,
  0xfc00, 0xf400, 0xfc00, 0x7c00,
  0xf400, 0xfc00, 0x7c00, 0x73fc,
  0xfc00, 0x7c00, 0x73fc, 0x4000,
  0x7c00, 0x73fc, 0x4000, 0x3ffc,
  0x73fc, 0x4000, 0x3ffc, 0x3ffc,
  0x4000, 0x3ffc, 0x3ffc, 0x3c00,
  0x3ffc, 0x3ffc, 0x3c00, 0x3bfc,
  0x3ffc, 0x3c00, 0x3bfc, 0x3bfc,
  0x3c00, 0x3bfc, 0x3bfc, 0x3954,
  0x3bfc, 0x3bfc, 0x3954, 0x2e64,
  0x3bfc, 0x3954, 0x2e64, 0x0100,
  0x3954, 0x2e64, 0x0100, 0x0000,
  0x2e64, 0x0100, 0x0000, 0x7f23,
  0x0100, 0x0000, 0x7f23, 0x7e00,
  0x0000, 0x7f23, 0x7e00, 0x7f23,
  0x7f23, 0x7e00, 0x7f23, 0x7e01,
  0x7e00, 0x7f23, 0x7e01, 0x7c00,
  0x7f23, 0x7e01, 0x7c00, 0x7400,
  0x7e01, 0x7c00, 0x7400, 0x7c00,
  0x7c00, 0x7400, 0x7c00, 0xfc00,
  0x7400, 0x7c00, 0xfc00, 0xf3fc,
  0x7c00, 0xfc00, 0xf3fc, 0xc000,
  0xfc00, 0xf3fc, 0xc000, 0xbffc,
  0xf3fc, 0xc000, 0xbffc, 0xbffc,
  0xc000, 0xbffc, 0xbffc, 0xbc00,
  0xbffc, 0xbffc, 0xbc00, 0xbbfc,
  0xbffc, 0xbc00, 0xbbfc, 0xbbfc,
  0xbc00, 0xbbfc, 0xbbfc, 0xb954,
  0xbbfc, 0xbbfc, 0xb954, 0xae64,
  0xbbfc, 0xb954, 0xae64, 0x8100,
  0xb954, 0xae64, 0x8100, 0x8000,
  0xae64, 0x8100, 0x8000, 0xff23,
  0x8100, 0x8000, 0xff23, 0xfe00,
  0x8000, 0xff23, 0xfe00, 0xff23,
};
const unsigned kExpectedCount_NEON_frecpe_4H = 38;

#endif  // VIXL_SIM_FRECPE_4H_TRACE_AARCH64_H_
