// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_SIM_UADDLP_1D_TRACE_AARCH64_H_
#define VIXL_SIM_UADDLP_1D_TRACE_AARCH64_H_

const uint64_t kExpected_NEON_uaddlp_1D[] = {
  0x00000001fffffffb,
  0x00000001fffffffd,
  0x00000000ffffffff,
  0x0000000000000001,
  0x0000000000000003,
  0x0000000000000022,
  0x000000000000009d,
  0x00000000000000fb,
  0x00000000000000fd,
  0x000000000000807c,
  0x000000000000fffb,
  0x000000000000fffd,
  0x000000003333b332,
  0x0000000088888888,
  0x00000000d5555552,
  0x00000000fffffffb,
  0x00000000fffffffd,
  0x00000000ffffffff,
  0x0000000100000001,
  0x000000012aaaaaab,
  0x0000000177777776,
  0x00000001cccc4ccc,
  0x00000001ffff0001,
  0x00000001ffff0003,
  0x00000001ffff0005,
  0x00000001ffff7f83,
  0x00000001ffffff01,
  0x00000001ffffff03,
  0x00000001ffffff05,
  0x00000001ffffff63,
  0x00000001ffffffdd,
};
const unsigned kExpectedCount_NEON_uaddlp_1D = 31;

#endif  // VIXL_SIM_UADDLP_1D_TRACE_AARCH64_H_
