// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_SIM_SQNEG_16B_TRACE_AARCH64_H_
#define VIXL_SIM_SQNEG_16B_TRACE_AARCH64_H_

const uint8_t kExpected_NEON_sqneg_16B[] = {
  0xcd, 0xab, 0x83, 0x82, 0x81, 0x7f, 0x7f, 0x7e, 0x7d, 0x56, 0x34, 0x08, 0x03, 0x02, 0x01, 0x00,
  0xab, 0x83, 0x82, 0x81, 0x7f, 0x7f, 0x7e, 0x7d, 0x56, 0x34, 0x08, 0x03, 0x02, 0x01, 0x00, 0xff,
  0x83, 0x82, 0x81, 0x7f, 0x7f, 0x7e, 0x7d, 0x56, 0x34, 0x08, 0x03, 0x02, 0x01, 0x00, 0xff, 0xfe,
  0x82, 0x81, 0x7f, 0x7f, 0x7e, 0x7d, 0x56, 0x34, 0x08, 0x03, 0x02, 0x01, 0x00, 0xff, 0xfe, 0xf8,
  0x81, 0x7f, 0x7f, 0x7e, 0x7d, 0x56, 0x34, 0x08, 0x03, 0x02, 0x01, 0x00, 0xff, 0xfe, 0xf8, 0xcd,
  0x7f, 0x7f, 0x7e, 0x7d, 0x56, 0x34, 0x08, 0x03, 0x02, 0x01, 0x00, 0xff, 0xfe, 0xf8, 0xcd, 0xab,
  0x7f, 0x7e, 0x7d, 0x56, 0x34, 0x08, 0x03, 0x02, 0x01, 0x00, 0xff, 0xfe, 0xf8, 0xcd, 0xab, 0x83,
  0x7e, 0x7d, 0x56, 0x34, 0x08, 0x03, 0x02, 0x01, 0x00, 0xff, 0xfe, 0xf8, 0xcd, 0xab, 0x83, 0x82,
  0x7d, 0x56, 0x34, 0x08, 0x03, 0x02, 0x01, 0x00, 0xff, 0xfe, 0xf8, 0xcd, 0xab, 0x83, 0x82, 0x81,
  0x56, 0x34, 0x08, 0x03, 0x02, 0x01, 0x00, 0xff, 0xfe, 0xf8, 0xcd, 0xab, 0x83, 0x82, 0x81, 0x7f,
  0x34, 0x08, 0x03, 0x02, 0x01, 0x00, 0xff, 0xfe, 0xf8, 0xcd, 0xab, 0x83, 0x82, 0x81, 0x7f, 0x7f,
  0x08, 0x03, 0x02, 0x01, 0x00, 0xff, 0xfe, 0xf8, 0xcd, 0xab, 0x83, 0x82, 0x81, 0x7f, 0x7f, 0x7e,
  0x03, 0x02, 0x01, 0x00, 0xff, 0xfe, 0xf8, 0xcd, 0xab, 0x83, 0x82, 0x81, 0x7f, 0x7f, 0x7e, 0x7d,
  0x02, 0x01, 0x00, 0xff, 0xfe, 0xf8, 0xcd, 0xab, 0x83, 0x82, 0x81, 0x7f, 0x7f, 0x7e, 0x7d, 0x56,
  0x01, 0x00, 0xff, 0xfe, 0xf8, 0xcd, 0xab, 0x83, 0x82, 0x81, 0x7f, 0x7f, 0x7e, 0x7d, 0x56, 0x34,
  0x00, 0xff, 0xfe, 0xf8, 0xcd, 0xab, 0x83, 0x82, 0x81, 0x7f, 0x7f, 0x7e, 0x7d, 0x56, 0x34, 0x08,
  0xff, 0xfe, 0xf8, 0xcd, 0xab, 0x83, 0x82, 0x81, 0x7f, 0x7f, 0x7e, 0x7d, 0x56, 0x34, 0x08, 0x03,
  0xfe, 0xf8, 0xcd, 0xab, 0x83, 0x82, 0x81, 0x7f, 0x7f, 0x7e, 0x7d, 0x56, 0x34, 0x08, 0x03, 0x02,
  0xf8, 0xcd, 0xab, 0x83, 0x82, 0x81, 0x7f, 0x7f, 0x7e, 0x7d, 0x56, 0x34, 0x08, 0x03, 0x02, 0x01,
};
const unsigned kExpectedCount_NEON_sqneg_16B = 19;

#endif  // VIXL_SIM_SQNEG_16B_TRACE_AARCH64_H_
