// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_SIM_UQSHL_2S_2OPIMM_TRACE_AARCH64_H_
#define VIXL_SIM_UQSHL_2S_2OPIMM_TRACE_AARCH64_H_

const uint32_t kExpected_NEON_uqshl_2S_2OPIMM[] = {
  0xfffffffd, 0xfffffffe,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xfffffffe, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0xffffffff, 0x00000000,
  0x00000000, 0x00000001,
  0x00000000, 0x00000002,
  0x00000000, 0x00000004,
  0x00000000, 0x00000008,
  0x00000000, 0x00000010,
  0x00000000, 0x00000020,
  0x00000000, 0x00000040,
  0x00000000, 0x00000080,
  0x00000000, 0x00000100,
  0x00000000, 0x00000200,
  0x00000000, 0x00000400,
  0x00000000, 0x00000800,
  0x00000000, 0x00001000,
  0x00000000, 0x00002000,
  0x00000000, 0x00004000,
  0x00000000, 0x00008000,
  0x00000000, 0x00010000,
  0x00000000, 0x00020000,
  0x00000000, 0x00040000,
  0x00000000, 0x00080000,
  0x00000000, 0x00100000,
  0x00000000, 0x00200000,
  0x00000000, 0x00400000,
  0x00000000, 0x00800000,
  0x00000000, 0x01000000,
  0x00000000, 0x02000000,
  0x00000000, 0x04000000,
  0x00000000, 0x08000000,
  0x00000000, 0x10000000,
  0x00000000, 0x20000000,
  0x00000000, 0x40000000,
  0x00000000, 0x80000000,
  0x00000001, 0x00000002,
  0x00000002, 0x00000004,
  0x00000004, 0x00000008,
  0x00000008, 0x00000010,
  0x00000010, 0x00000020,
  0x00000020, 0x00000040,
  0x00000040, 0x00000080,
  0x00000080, 0x00000100,
  0x00000100, 0x00000200,
  0x00000200, 0x00000400,
  0x00000400, 0x00000800,
  0x00000800, 0x00001000,
  0x00001000, 0x00002000,
  0x00002000, 0x00004000,
  0x00004000, 0x00008000,
  0x00008000, 0x00010000,
  0x00010000, 0x00020000,
  0x00020000, 0x00040000,
  0x00040000, 0x00080000,
  0x00080000, 0x00100000,
  0x00100000, 0x00200000,
  0x00200000, 0x00400000,
  0x00400000, 0x00800000,
  0x00800000, 0x01000000,
  0x01000000, 0x02000000,
  0x02000000, 0x04000000,
  0x04000000, 0x08000000,
  0x08000000, 0x10000000,
  0x10000000, 0x20000000,
  0x20000000, 0x40000000,
  0x40000000, 0x80000000,
  0x80000000, 0xffffffff,
  0x00000002, 0x00000020,
  0x00000004, 0x00000040,
  0x00000008, 0x00000080,
  0x00000010, 0x00000100,
  0x00000020, 0x00000200,
  0x00000040, 0x00000400,
  0x00000080, 0x00000800,
  0x00000100, 0x00001000,
  0x00000200, 0x00002000,
  0x00000400, 0x00004000,
  0x00000800, 0x00008000,
  0x00001000, 0x00010000,
  0x00002000, 0x00020000,
  0x00004000, 0x00040000,
  0x00008000, 0x00080000,
  0x00010000, 0x00100000,
  0x00020000, 0x00200000,
  0x00040000, 0x00400000,
  0x00080000, 0x00800000,
  0x00100000, 0x01000000,
  0x00200000, 0x02000000,
  0x00400000, 0x04000000,
  0x00800000, 0x08000000,
  0x01000000, 0x10000000,
  0x02000000, 0x20000000,
  0x04000000, 0x40000000,
  0x08000000, 0x80000000,
  0x10000000, 0xffffffff,
  0x20000000, 0xffffffff,
  0x40000000, 0xffffffff,
  0x80000000, 0xffffffff,
  0xffffffff, 0xffffffff,
  0x00000020, 0x0000007d,
  0x00000040, 0x000000fa,
  0x00000080, 0x000001f4,
  0x00000100, 0x000003e8,
  0x00000200, 0x000007d0,
  0x00000400, 0x00000fa0,
  0x00000800, 0x00001f40,
  0x00001000, 0x00003e80,
  0x00002000, 0x00007d00,
  0x00004000, 0x0000fa00,
  0x00008000, 0x0001f400,
  0x00010000, 0x0003e800,
  0x00020000, 0x0007d000,
  0x00040000, 0x000fa000,
  0x00080000, 0x001f4000,
  0x00100000, 0x003e8000,
  0x00200000, 0x007d0000,
  0x00400000, 0x00fa0000,
  0x00800000, 0x01f40000,
  0x01000000, 0x03e80000,
  0x02000000, 0x07d00000,
  0x04000000, 0x0fa00000,
  0x08000000, 0x1f400000,
  0x10000000, 0x3e800000,
  0x20000000, 0x7d000000,
  0x40000000, 0xfa000000,
  0x80000000, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0x0000007d, 0x0000007e,
  0x000000fa, 0x000000fc,
  0x000001f4, 0x000001f8,
  0x000003e8, 0x000003f0,
  0x000007d0, 0x000007e0,
  0x00000fa0, 0x00000fc0,
  0x00001f40, 0x00001f80,
  0x00003e80, 0x00003f00,
  0x00007d00, 0x00007e00,
  0x0000fa00, 0x0000fc00,
  0x0001f400, 0x0001f800,
  0x0003e800, 0x0003f000,
  0x0007d000, 0x0007e000,
  0x000fa000, 0x000fc000,
  0x001f4000, 0x001f8000,
  0x003e8000, 0x003f0000,
  0x007d0000, 0x007e0000,
  0x00fa0000, 0x00fc0000,
  0x01f40000, 0x01f80000,
  0x03e80000, 0x03f00000,
  0x07d00000, 0x07e00000,
  0x0fa00000, 0x0fc00000,
  0x1f400000, 0x1f800000,
  0x3e800000, 0x3f000000,
  0x7d000000, 0x7e000000,
  0xfa000000, 0xfc000000,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0x0000007e, 0x0000007f,
  0x000000fc, 0x000000fe,
  0x000001f8, 0x000001fc,
  0x000003f0, 0x000003f8,
  0x000007e0, 0x000007f0,
  0x00000fc0, 0x00000fe0,
  0x00001f80, 0x00001fc0,
  0x00003f00, 0x00003f80,
  0x00007e00, 0x00007f00,
  0x0000fc00, 0x0000fe00,
  0x0001f800, 0x0001fc00,
  0x0003f000, 0x0003f800,
  0x0007e000, 0x0007f000,
  0x000fc000, 0x000fe000,
  0x001f8000, 0x001fc000,
  0x003f0000, 0x003f8000,
  0x007e0000, 0x007f0000,
  0x00fc0000, 0x00fe0000,
  0x01f80000, 0x01fc0000,
  0x03f00000, 0x03f80000,
  0x07e00000, 0x07f00000,
  0x0fc00000, 0x0fe00000,
  0x1f800000, 0x1fc00000,
  0x3f000000, 0x3f800000,
  0x7e000000, 0x7f000000,
  0xfc000000, 0xfe000000,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0x0000007f, 0x00007ffd,
  0x000000fe, 0x0000fffa,
  0x000001fc, 0x0001fff4,
  0x000003f8, 0x0003ffe8,
  0x000007f0, 0x0007ffd0,
  0x00000fe0, 0x000fffa0,
  0x00001fc0, 0x001fff40,
  0x00003f80, 0x003ffe80,
  0x00007f00, 0x007ffd00,
  0x0000fe00, 0x00fffa00,
  0x0001fc00, 0x01fff400,
  0x0003f800, 0x03ffe800,
  0x0007f000, 0x07ffd000,
  0x000fe000, 0x0fffa000,
  0x001fc000, 0x1fff4000,
  0x003f8000, 0x3ffe8000,
  0x007f0000, 0x7ffd0000,
  0x00fe0000, 0xfffa0000,
  0x01fc0000, 0xffffffff,
  0x03f80000, 0xffffffff,
  0x07f00000, 0xffffffff,
  0x0fe00000, 0xffffffff,
  0x1fc00000, 0xffffffff,
  0x3f800000, 0xffffffff,
  0x7f000000, 0xffffffff,
  0xfe000000, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0x00007ffd, 0x00007ffe,
  0x0000fffa, 0x0000fffc,
  0x0001fff4, 0x0001fff8,
  0x0003ffe8, 0x0003fff0,
  0x0007ffd0, 0x0007ffe0,
  0x000fffa0, 0x000fffc0,
  0x001fff40, 0x001fff80,
  0x003ffe80, 0x003fff00,
  0x007ffd00, 0x007ffe00,
  0x00fffa00, 0x00fffc00,
  0x01fff400, 0x01fff800,
  0x03ffe800, 0x03fff000,
  0x07ffd000, 0x07ffe000,
  0x0fffa000, 0x0fffc000,
  0x1fff4000, 0x1fff8000,
  0x3ffe8000, 0x3fff0000,
  0x7ffd0000, 0x7ffe0000,
  0xfffa0000, 0xfffc0000,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0x00007ffe, 0x00007fff,
  0x0000fffc, 0x0000fffe,
  0x0001fff8, 0x0001fffc,
  0x0003fff0, 0x0003fff8,
  0x0007ffe0, 0x0007fff0,
  0x000fffc0, 0x000fffe0,
  0x001fff80, 0x001fffc0,
  0x003fff00, 0x003fff80,
  0x007ffe00, 0x007fff00,
  0x00fffc00, 0x00fffe00,
  0x01fff800, 0x01fffc00,
  0x03fff000, 0x03fff800,
  0x07ffe000, 0x07fff000,
  0x0fffc000, 0x0fffe000,
  0x1fff8000, 0x1fffc000,
  0x3fff0000, 0x3fff8000,
  0x7ffe0000, 0x7fff0000,
  0xfffc0000, 0xfffe0000,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0x00007fff, 0x33333333,
  0x0000fffe, 0x66666666,
  0x0001fffc, 0xcccccccc,
  0x0003fff8, 0xffffffff,
  0x0007fff0, 0xffffffff,
  0x000fffe0, 0xffffffff,
  0x001fffc0, 0xffffffff,
  0x003fff80, 0xffffffff,
  0x007fff00, 0xffffffff,
  0x00fffe00, 0xffffffff,
  0x01fffc00, 0xffffffff,
  0x03fff800, 0xffffffff,
  0x07fff000, 0xffffffff,
  0x0fffe000, 0xffffffff,
  0x1fffc000, 0xffffffff,
  0x3fff8000, 0xffffffff,
  0x7fff0000, 0xffffffff,
  0xfffe0000, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0x33333333, 0x55555555,
  0x66666666, 0xaaaaaaaa,
  0xcccccccc, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0x55555555, 0x7ffffffd,
  0xaaaaaaaa, 0xfffffffa,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0x7ffffffd, 0x7ffffffe,
  0xfffffffa, 0xfffffffc,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0x7ffffffe, 0x7fffffff,
  0xfffffffc, 0xfffffffe,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0x7fffffff, 0x80000000,
  0xfffffffe, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0x80000000, 0x80000001,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0x80000001, 0xaaaaaaaa,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xaaaaaaaa, 0xcccccccc,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xcccccccc, 0xffff8000,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffff8000, 0xffff8001,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffff8001, 0xffff8002,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffff8002, 0xffff8003,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffff8003, 0xffffff80,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffff80, 0xffffff81,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffff81, 0xffffff82,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffff82, 0xffffff83,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffff83, 0xffffffe0,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffe0, 0xfffffffd,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff,
};
const unsigned kExpectedCount_NEON_uqshl_2S_2OPIMM = 992;

#endif  // VIXL_SIM_UQSHL_2S_2OPIMM_TRACE_AARCH64_H_
