// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_SIM_SADALP_2D_TRACE_AARCH64_H_
#define VIXL_SIM_SADALP_2D_TRACE_AARCH64_H_

const uint64_t kExpected_NEON_sadalp_2D[] = {
  0xfffffffffffffffb, 0xffffffffffffffff,
  0xfffffffffffffffd, 0x0000000000000001,
  0xffffffffffffffff, 0x0000000000000003,
  0x0000000000000001, 0x0000000000000022,
  0x0000000000000003, 0x000000000000009d,
  0x0000000000000022, 0x00000000000000fb,
  0x000000000000009d, 0x00000000000000fd,
  0x00000000000000fb, 0x000000000000807c,
  0x00000000000000fd, 0x000000000000fffb,
  0x000000000000807c, 0x000000000000fffd,
  0x000000000000fffb, 0x000000003333b332,
  0x000000000000fffd, 0x0000000088888888,
  0x000000003333b332, 0x00000000d5555552,
  0x0000000088888888, 0x00000000fffffffb,
  0x00000000d5555552, 0x00000000fffffffd,
  0x00000000fffffffb, 0xffffffffffffffff,
  0x00000000fffffffd, 0xffffffff00000001,
  0xffffffffffffffff, 0xffffffff2aaaaaab,
  0xffffffff00000001, 0xffffffff77777776,
  0xffffffff2aaaaaab, 0xffffffffcccc4ccc,
  0xffffffff77777776, 0xffffffffffff0001,
  0xffffffffcccc4ccc, 0xffffffffffff0003,
  0xffffffffffff0001, 0xffffffffffff0005,
  0xffffffffffff0003, 0xffffffffffff7f83,
  0xffffffffffff0005, 0xffffffffffffff01,
  0xffffffffffff7f83, 0xffffffffffffff03,
  0xffffffffffffff01, 0xffffffffffffff05,
  0xffffffffffffff03, 0xffffffffffffff63,
  0xffffffffffffff05, 0xffffffffffffffdd,
  0xffffffffffffff63, 0xfffffffffffffffb,
  0xffffffffffffffdd, 0xfffffffffffffffd,
};
const unsigned kExpectedCount_NEON_sadalp_2D = 31;

#endif  // VIXL_SIM_SADALP_2D_TRACE_AARCH64_H_
