// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_SIM_UADALP_2S_TRACE_AARCH64_H_
#define VIXL_SIM_UADALP_2S_TRACE_AARCH64_H_

const uint32_t kExpected_NEON_uadalp_2S[] = {
  0x0001ff03, 0x0001ff73,
  0x0001ff05, 0x0001ffed,
  0x0001ff73, 0x0001fffb,
  0x0001ffed, 0x0001fffd,
  0x0001fffb, 0x0000ffff,
  0x0001fffd, 0x00000001,
  0x0000ffff, 0x00000003,
  0x00000001, 0x00000012,
  0x00000003, 0x0000008d,
  0x00000012, 0x000000fb,
  0x0000008d, 0x000000fd,
  0x000000fb, 0x000033b2,
  0x000000fd, 0x00008888,
  0x000033b2, 0x0000d552,
  0x00008888, 0x0000fffb,
  0x0000d552, 0x0000fffd,
  0x0000fffb, 0x0000ffff,
  0x0000fffd, 0x00010001,
  0x0000ffff, 0x00012aab,
  0x00010001, 0x00017776,
  0x00012aab, 0x0001cc4c,
  0x00017776, 0x0001ff01,
  0x0001cc4c, 0x0001ff03,
  0x0001ff01, 0x0001ff05,
};
const unsigned kExpectedCount_NEON_uadalp_2S = 24;

#endif  // VIXL_SIM_UADALP_2S_TRACE_AARCH64_H_
