// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_SIM_SADDLV_S_4H_TRACE_AARCH64_H_
#define VIXL_SIM_SADDLV_S_4H_TRACE_AARCH64_H_

const uint32_t kExpected_NEON_saddlv_S_4H[] = {
  0xfffffffa,
  0xfffffffe,
  0x00000002,
  0x00000013,
  0x00000090,
  0x0000010d,
  0x0000018a,
  0x000034ad,
  0x00008985,
  0x00010904,
  0x00018883,
  0x0001d54f,
  0x0000fffa,
  0xfffffffe,
  0xffff2aaa,
  0xfffe7777,
  0xfffef6f7,
  0xffff7677,
  0xffffcb4f,
  0xfffffe06,
  0xfffffe76,
  0xfffffef2,
  0xffffff6e,
  0xffffffea,
};
const unsigned kExpectedCount_NEON_saddlv_S_4H = 24;

#endif  // VIXL_SIM_SADDLV_S_4H_TRACE_AARCH64_H_
