// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_ASSEMBLER_COND_RD_OPERAND_RN_SHIFT_AMOUNT_1TO31_MVN_A32_H_
#define VIXL_ASSEMBLER_COND_RD_OPERAND_RN_SHIFT_AMOUNT_1TO31_MVN_A32_H_

const byte kInstruction_mvn_vc_r5_r5_ROR_10[] = {
  0x65, 0x55, 0xe0, 0x71 // mvn vc r5 r5 ROR 10
};
const byte kInstruction_mvn_ne_r3_r4_ROR_17[] = {
  0xe4, 0x38, 0xe0, 0x11 // mvn ne r3 r4 ROR 17
};
const byte kInstruction_mvn_cs_r9_r10_ROR_16[] = {
  0x6a, 0x98, 0xe0, 0x21 // mvn cs r9 r10 ROR 16
};
const byte kInstruction_mvn_lt_r0_r2_ROR_29[] = {
  0xe2, 0x0e, 0xe0, 0xb1 // mvn lt r0 r2 ROR 29
};
const byte kInstruction_mvn_al_r11_r2_ROR_23[] = {
  0xe2, 0xbb, 0xe0, 0xe1 // mvn al r11 r2 ROR 23
};
const byte kInstruction_mvn_hi_r7_r1_LSL_29[] = {
  0x81, 0x7e, 0xe0, 0x81 // mvn hi r7 r1 LSL 29
};
const byte kInstruction_mvn_eq_r5_r3_ROR_21[] = {
  0xe3, 0x5a, 0xe0, 0x01 // mvn eq r5 r3 ROR 21
};
const byte kInstruction_mvn_pl_r2_r10_ROR_13[] = {
  0xea, 0x26, 0xe0, 0x51 // mvn pl r2 r10 ROR 13
};
const byte kInstruction_mvn_lt_r1_r1_LSL_23[] = {
  0x81, 0x1b, 0xe0, 0xb1 // mvn lt r1 r1 LSL 23
};
const byte kInstruction_mvn_mi_r11_r10_ROR_31[] = {
  0xea, 0xbf, 0xe0, 0x41 // mvn mi r11 r10 ROR 31
};
const byte kInstruction_mvn_cs_r3_r5_LSL_8[] = {
  0x05, 0x34, 0xe0, 0x21 // mvn cs r3 r5 LSL 8
};
const byte kInstruction_mvn_gt_r0_r13_LSL_23[] = {
  0x8d, 0x0b, 0xe0, 0xc1 // mvn gt r0 r13 LSL 23
};
const byte kInstruction_mvn_eq_r10_r11_ROR_7[] = {
  0xeb, 0xa3, 0xe0, 0x01 // mvn eq r10 r11 ROR 7
};
const byte kInstruction_mvn_vs_r4_r3_ROR_18[] = {
  0x63, 0x49, 0xe0, 0x61 // mvn vs r4 r3 ROR 18
};
const byte kInstruction_mvn_ge_r6_r0_ROR_16[] = {
  0x60, 0x68, 0xe0, 0xa1 // mvn ge r6 r0 ROR 16
};
const byte kInstruction_mvn_ls_r3_r5_LSL_20[] = {
  0x05, 0x3a, 0xe0, 0x91 // mvn ls r3 r5 LSL 20
};
const byte kInstruction_mvn_ge_r10_r13_ROR_31[] = {
  0xed, 0xaf, 0xe0, 0xa1 // mvn ge r10 r13 ROR 31
};
const byte kInstruction_mvn_cc_r9_r1_ROR_8[] = {
  0x61, 0x94, 0xe0, 0x31 // mvn cc r9 r1 ROR 8
};
const byte kInstruction_mvn_mi_r5_r5_LSL_16[] = {
  0x05, 0x58, 0xe0, 0x41 // mvn mi r5 r5 LSL 16
};
const byte kInstruction_mvn_eq_r14_r6_LSL_4[] = {
  0x06, 0xe2, 0xe0, 0x01 // mvn eq r14 r6 LSL 4
};
const byte kInstruction_mvn_lt_r10_r5_ROR_16[] = {
  0x65, 0xa8, 0xe0, 0xb1 // mvn lt r10 r5 ROR 16
};
const byte kInstruction_mvn_vs_r3_r6_LSL_23[] = {
  0x86, 0x3b, 0xe0, 0x61 // mvn vs r3 r6 LSL 23
};
const byte kInstruction_mvn_ls_r0_r6_LSL_30[] = {
  0x06, 0x0f, 0xe0, 0x91 // mvn ls r0 r6 LSL 30
};
const byte kInstruction_mvn_vc_r0_r5_ROR_19[] = {
  0xe5, 0x09, 0xe0, 0x71 // mvn vc r0 r5 ROR 19
};
const byte kInstruction_mvn_ne_r6_r0_ROR_19[] = {
  0xe0, 0x69, 0xe0, 0x11 // mvn ne r6 r0 ROR 19
};
const byte kInstruction_mvn_vc_r10_r13_LSL_7[] = {
  0x8d, 0xa3, 0xe0, 0x71 // mvn vc r10 r13 LSL 7
};
const byte kInstruction_mvn_pl_r9_r13_ROR_9[] = {
  0xed, 0x94, 0xe0, 0x51 // mvn pl r9 r13 ROR 9
};
const byte kInstruction_mvn_vs_r6_r13_ROR_6[] = {
  0x6d, 0x63, 0xe0, 0x61 // mvn vs r6 r13 ROR 6
};
const byte kInstruction_mvn_mi_r12_r0_ROR_11[] = {
  0xe0, 0xc5, 0xe0, 0x41 // mvn mi r12 r0 ROR 11
};
const byte kInstruction_mvn_vs_r8_r12_LSL_9[] = {
  0x8c, 0x84, 0xe0, 0x61 // mvn vs r8 r12 LSL 9
};
const byte kInstruction_mvn_ls_r2_r3_LSL_10[] = {
  0x03, 0x25, 0xe0, 0x91 // mvn ls r2 r3 LSL 10
};
const byte kInstruction_mvn_gt_r14_r9_ROR_18[] = {
  0x69, 0xe9, 0xe0, 0xc1 // mvn gt r14 r9 ROR 18
};
const byte kInstruction_mvn_pl_r7_r1_LSL_5[] = {
  0x81, 0x72, 0xe0, 0x51 // mvn pl r7 r1 LSL 5
};
const byte kInstruction_mvn_cc_r3_r14_ROR_5[] = {
  0xee, 0x32, 0xe0, 0x31 // mvn cc r3 r14 ROR 5
};
const byte kInstruction_mvn_vc_r3_r10_LSL_13[] = {
  0x8a, 0x36, 0xe0, 0x71 // mvn vc r3 r10 LSL 13
};
const byte kInstruction_mvn_mi_r13_r12_LSL_25[] = {
  0x8c, 0xdc, 0xe0, 0x41 // mvn mi r13 r12 LSL 25
};
const byte kInstruction_mvn_mi_r12_r7_LSL_17[] = {
  0x87, 0xc8, 0xe0, 0x41 // mvn mi r12 r7 LSL 17
};
const byte kInstruction_mvn_al_r10_r14_LSL_18[] = {
  0x0e, 0xa9, 0xe0, 0xe1 // mvn al r10 r14 LSL 18
};
const byte kInstruction_mvn_cc_r7_r4_ROR_30[] = {
  0x64, 0x7f, 0xe0, 0x31 // mvn cc r7 r4 ROR 30
};
const byte kInstruction_mvn_al_r0_r1_ROR_8[] = {
  0x61, 0x04, 0xe0, 0xe1 // mvn al r0 r1 ROR 8
};
const byte kInstruction_mvn_eq_r5_r7_LSL_16[] = {
  0x07, 0x58, 0xe0, 0x01 // mvn eq r5 r7 LSL 16
};
const byte kInstruction_mvn_cc_r1_r2_ROR_25[] = {
  0xe2, 0x1c, 0xe0, 0x31 // mvn cc r1 r2 ROR 25
};
const byte kInstruction_mvn_gt_r9_r6_LSL_18[] = {
  0x06, 0x99, 0xe0, 0xc1 // mvn gt r9 r6 LSL 18
};
const byte kInstruction_mvn_lt_r4_r6_LSL_25[] = {
  0x86, 0x4c, 0xe0, 0xb1 // mvn lt r4 r6 LSL 25
};
const byte kInstruction_mvn_ge_r9_r3_ROR_4[] = {
  0x63, 0x92, 0xe0, 0xa1 // mvn ge r9 r3 ROR 4
};
const byte kInstruction_mvn_le_r8_r7_LSL_2[] = {
  0x07, 0x81, 0xe0, 0xd1 // mvn le r8 r7 LSL 2
};
const byte kInstruction_mvn_eq_r7_r2_LSL_4[] = {
  0x02, 0x72, 0xe0, 0x01 // mvn eq r7 r2 LSL 4
};
const byte kInstruction_mvn_vc_r7_r11_ROR_21[] = {
  0xeb, 0x7a, 0xe0, 0x71 // mvn vc r7 r11 ROR 21
};
const byte kInstruction_mvn_vc_r14_r14_LSL_15[] = {
  0x8e, 0xe7, 0xe0, 0x71 // mvn vc r14 r14 LSL 15
};
const byte kInstruction_mvn_vc_r10_r3_LSL_5[] = {
  0x83, 0xa2, 0xe0, 0x71 // mvn vc r10 r3 LSL 5
};
const byte kInstruction_mvn_vs_r6_r12_ROR_15[] = {
  0xec, 0x67, 0xe0, 0x61 // mvn vs r6 r12 ROR 15
};
const byte kInstruction_mvn_ls_r6_r12_ROR_3[] = {
  0xec, 0x61, 0xe0, 0x91 // mvn ls r6 r12 ROR 3
};
const byte kInstruction_mvn_vs_r12_r10_ROR_17[] = {
  0xea, 0xc8, 0xe0, 0x61 // mvn vs r12 r10 ROR 17
};
const byte kInstruction_mvn_eq_r6_r13_ROR_9[] = {
  0xed, 0x64, 0xe0, 0x01 // mvn eq r6 r13 ROR 9
};
const byte kInstruction_mvn_mi_r10_r9_ROR_18[] = {
  0x69, 0xa9, 0xe0, 0x41 // mvn mi r10 r9 ROR 18
};
const byte kInstruction_mvn_eq_r0_r7_LSL_13[] = {
  0x87, 0x06, 0xe0, 0x01 // mvn eq r0 r7 LSL 13
};
const byte kInstruction_mvn_eq_r9_r13_ROR_16[] = {
  0x6d, 0x98, 0xe0, 0x01 // mvn eq r9 r13 ROR 16
};
const byte kInstruction_mvn_eq_r7_r9_ROR_21[] = {
  0xe9, 0x7a, 0xe0, 0x01 // mvn eq r7 r9 ROR 21
};
const byte kInstruction_mvn_gt_r12_r6_ROR_30[] = {
  0x66, 0xcf, 0xe0, 0xc1 // mvn gt r12 r6 ROR 30
};
const byte kInstruction_mvn_ge_r4_r8_ROR_29[] = {
  0xe8, 0x4e, 0xe0, 0xa1 // mvn ge r4 r8 ROR 29
};
const byte kInstruction_mvn_hi_r7_r9_LSL_21[] = {
  0x89, 0x7a, 0xe0, 0x81 // mvn hi r7 r9 LSL 21
};
const byte kInstruction_mvn_le_r3_r13_LSL_10[] = {
  0x0d, 0x35, 0xe0, 0xd1 // mvn le r3 r13 LSL 10
};
const byte kInstruction_mvn_gt_r9_r13_LSL_18[] = {
  0x0d, 0x99, 0xe0, 0xc1 // mvn gt r9 r13 LSL 18
};
const byte kInstruction_mvn_pl_r3_r11_ROR_20[] = {
  0x6b, 0x3a, 0xe0, 0x51 // mvn pl r3 r11 ROR 20
};
const byte kInstruction_mvn_le_r1_r3_LSL_5[] = {
  0x83, 0x12, 0xe0, 0xd1 // mvn le r1 r3 LSL 5
};
const byte kInstruction_mvn_mi_r4_r6_LSL_12[] = {
  0x06, 0x46, 0xe0, 0x41 // mvn mi r4 r6 LSL 12
};
const byte kInstruction_mvn_pl_r1_r5_ROR_27[] = {
  0xe5, 0x1d, 0xe0, 0x51 // mvn pl r1 r5 ROR 27
};
const byte kInstruction_mvn_gt_r9_r5_LSL_4[] = {
  0x05, 0x92, 0xe0, 0xc1 // mvn gt r9 r5 LSL 4
};
const byte kInstruction_mvn_eq_r4_r8_LSL_11[] = {
  0x88, 0x45, 0xe0, 0x01 // mvn eq r4 r8 LSL 11
};
const byte kInstruction_mvn_mi_r1_r8_ROR_13[] = {
  0xe8, 0x16, 0xe0, 0x41 // mvn mi r1 r8 ROR 13
};
const byte kInstruction_mvn_al_r7_r0_ROR_20[] = {
  0x60, 0x7a, 0xe0, 0xe1 // mvn al r7 r0 ROR 20
};
const byte kInstruction_mvn_vs_r5_r7_ROR_28[] = {
  0x67, 0x5e, 0xe0, 0x61 // mvn vs r5 r7 ROR 28
};
const byte kInstruction_mvn_cs_r0_r12_LSL_31[] = {
  0x8c, 0x0f, 0xe0, 0x21 // mvn cs r0 r12 LSL 31
};
const byte kInstruction_mvn_gt_r11_r12_LSL_25[] = {
  0x8c, 0xbc, 0xe0, 0xc1 // mvn gt r11 r12 LSL 25
};
const byte kInstruction_mvn_gt_r2_r13_ROR_19[] = {
  0xed, 0x29, 0xe0, 0xc1 // mvn gt r2 r13 ROR 19
};
const byte kInstruction_mvn_cs_r4_r7_ROR_7[] = {
  0xe7, 0x43, 0xe0, 0x21 // mvn cs r4 r7 ROR 7
};
const byte kInstruction_mvn_gt_r9_r8_ROR_24[] = {
  0x68, 0x9c, 0xe0, 0xc1 // mvn gt r9 r8 ROR 24
};
const byte kInstruction_mvn_lt_r10_r2_ROR_4[] = {
  0x62, 0xa2, 0xe0, 0xb1 // mvn lt r10 r2 ROR 4
};
const byte kInstruction_mvn_lt_r4_r2_ROR_20[] = {
  0x62, 0x4a, 0xe0, 0xb1 // mvn lt r4 r2 ROR 20
};
const byte kInstruction_mvn_ge_r2_r0_LSL_18[] = {
  0x00, 0x29, 0xe0, 0xa1 // mvn ge r2 r0 LSL 18
};
const byte kInstruction_mvn_hi_r13_r14_LSL_24[] = {
  0x0e, 0xdc, 0xe0, 0x81 // mvn hi r13 r14 LSL 24
};
const byte kInstruction_mvn_al_r8_r0_ROR_3[] = {
  0xe0, 0x81, 0xe0, 0xe1 // mvn al r8 r0 ROR 3
};
const byte kInstruction_mvn_ne_r11_r2_LSL_6[] = {
  0x02, 0xb3, 0xe0, 0x11 // mvn ne r11 r2 LSL 6
};
const byte kInstruction_mvn_cs_r14_r1_LSL_6[] = {
  0x01, 0xe3, 0xe0, 0x21 // mvn cs r14 r1 LSL 6
};
const byte kInstruction_mvn_al_r13_r1_LSL_12[] = {
  0x01, 0xd6, 0xe0, 0xe1 // mvn al r13 r1 LSL 12
};
const byte kInstruction_mvn_ls_r14_r1_ROR_7[] = {
  0xe1, 0xe3, 0xe0, 0x91 // mvn ls r14 r1 ROR 7
};
const byte kInstruction_mvn_cs_r11_r2_ROR_1[] = {
  0xe2, 0xb0, 0xe0, 0x21 // mvn cs r11 r2 ROR 1
};
const byte kInstruction_mvn_al_r9_r1_ROR_19[] = {
  0xe1, 0x99, 0xe0, 0xe1 // mvn al r9 r1 ROR 19
};
const byte kInstruction_mvn_ls_r14_r9_ROR_2[] = {
  0x69, 0xe1, 0xe0, 0x91 // mvn ls r14 r9 ROR 2
};
const byte kInstruction_mvn_ne_r13_r2_ROR_13[] = {
  0xe2, 0xd6, 0xe0, 0x11 // mvn ne r13 r2 ROR 13
};
const byte kInstruction_mvn_vs_r5_r1_LSL_4[] = {
  0x01, 0x52, 0xe0, 0x61 // mvn vs r5 r1 LSL 4
};
const byte kInstruction_mvn_cc_r13_r5_ROR_11[] = {
  0xe5, 0xd5, 0xe0, 0x31 // mvn cc r13 r5 ROR 11
};
const byte kInstruction_mvn_eq_r5_r1_ROR_12[] = {
  0x61, 0x56, 0xe0, 0x01 // mvn eq r5 r1 ROR 12
};
const byte kInstruction_mvn_al_r9_r7_LSL_16[] = {
  0x07, 0x98, 0xe0, 0xe1 // mvn al r9 r7 LSL 16
};
const byte kInstruction_mvn_eq_r9_r10_LSL_20[] = {
  0x0a, 0x9a, 0xe0, 0x01 // mvn eq r9 r10 LSL 20
};
const byte kInstruction_mvn_eq_r4_r13_ROR_3[] = {
  0xed, 0x41, 0xe0, 0x01 // mvn eq r4 r13 ROR 3
};
const byte kInstruction_mvn_cs_r14_r1_LSL_18[] = {
  0x01, 0xe9, 0xe0, 0x21 // mvn cs r14 r1 LSL 18
};
const byte kInstruction_mvn_cc_r5_r7_ROR_1[] = {
  0xe7, 0x50, 0xe0, 0x31 // mvn cc r5 r7 ROR 1
};
const byte kInstruction_mvn_mi_r5_r8_ROR_30[] = {
  0x68, 0x5f, 0xe0, 0x41 // mvn mi r5 r8 ROR 30
};
const byte kInstruction_mvn_eq_r8_r0_LSL_4[] = {
  0x00, 0x82, 0xe0, 0x01 // mvn eq r8 r0 LSL 4
};
const byte kInstruction_mvn_mi_r2_r1_ROR_30[] = {
  0x61, 0x2f, 0xe0, 0x41 // mvn mi r2 r1 ROR 30
};
const byte kInstruction_mvn_vs_r7_r5_LSL_29[] = {
  0x85, 0x7e, 0xe0, 0x61 // mvn vs r7 r5 LSL 29
};
const byte kInstruction_mvn_le_r11_r5_ROR_4[] = {
  0x65, 0xb2, 0xe0, 0xd1 // mvn le r11 r5 ROR 4
};
const byte kInstruction_mvn_gt_r6_r8_LSL_22[] = {
  0x08, 0x6b, 0xe0, 0xc1 // mvn gt r6 r8 LSL 22
};
const byte kInstruction_mvn_al_r13_r4_LSL_8[] = {
  0x04, 0xd4, 0xe0, 0xe1 // mvn al r13 r4 LSL 8
};
const byte kInstruction_mvn_lt_r4_r5_ROR_22[] = {
  0x65, 0x4b, 0xe0, 0xb1 // mvn lt r4 r5 ROR 22
};
const byte kInstruction_mvn_mi_r1_r14_LSL_12[] = {
  0x0e, 0x16, 0xe0, 0x41 // mvn mi r1 r14 LSL 12
};
const byte kInstruction_mvn_gt_r4_r12_LSL_8[] = {
  0x0c, 0x44, 0xe0, 0xc1 // mvn gt r4 r12 LSL 8
};
const byte kInstruction_mvn_gt_r1_r13_ROR_15[] = {
  0xed, 0x17, 0xe0, 0xc1 // mvn gt r1 r13 ROR 15
};
const byte kInstruction_mvn_lt_r14_r5_LSL_23[] = {
  0x85, 0xeb, 0xe0, 0xb1 // mvn lt r14 r5 LSL 23
};
const byte kInstruction_mvn_ls_r2_r14_LSL_21[] = {
  0x8e, 0x2a, 0xe0, 0x91 // mvn ls r2 r14 LSL 21
};
const byte kInstruction_mvn_vc_r12_r0_LSL_12[] = {
  0x00, 0xc6, 0xe0, 0x71 // mvn vc r12 r0 LSL 12
};
const byte kInstruction_mvn_al_r2_r1_ROR_18[] = {
  0x61, 0x29, 0xe0, 0xe1 // mvn al r2 r1 ROR 18
};
const byte kInstruction_mvn_vc_r12_r5_ROR_2[] = {
  0x65, 0xc1, 0xe0, 0x71 // mvn vc r12 r5 ROR 2
};
const byte kInstruction_mvn_mi_r11_r9_LSL_13[] = {
  0x89, 0xb6, 0xe0, 0x41 // mvn mi r11 r9 LSL 13
};
const byte kInstruction_mvn_lt_r8_r12_ROR_3[] = {
  0xec, 0x81, 0xe0, 0xb1 // mvn lt r8 r12 ROR 3
};
const byte kInstruction_mvn_vs_r0_r2_ROR_31[] = {
  0xe2, 0x0f, 0xe0, 0x61 // mvn vs r0 r2 ROR 31
};
const byte kInstruction_mvn_hi_r5_r5_ROR_9[] = {
  0xe5, 0x54, 0xe0, 0x81 // mvn hi r5 r5 ROR 9
};
const byte kInstruction_mvn_ne_r8_r0_LSL_4[] = {
  0x00, 0x82, 0xe0, 0x11 // mvn ne r8 r0 LSL 4
};
const byte kInstruction_mvn_gt_r2_r4_LSL_29[] = {
  0x84, 0x2e, 0xe0, 0xc1 // mvn gt r2 r4 LSL 29
};
const byte kInstruction_mvn_le_r11_r9_LSL_28[] = {
  0x09, 0xbe, 0xe0, 0xd1 // mvn le r11 r9 LSL 28
};
const byte kInstruction_mvn_ge_r13_r8_LSL_26[] = {
  0x08, 0xdd, 0xe0, 0xa1 // mvn ge r13 r8 LSL 26
};
const byte kInstruction_mvn_gt_r5_r13_ROR_25[] = {
  0xed, 0x5c, 0xe0, 0xc1 // mvn gt r5 r13 ROR 25
};
const byte kInstruction_mvn_cc_r9_r11_ROR_28[] = {
  0x6b, 0x9e, 0xe0, 0x31 // mvn cc r9 r11 ROR 28
};
const byte kInstruction_mvn_vs_r12_r14_ROR_18[] = {
  0x6e, 0xc9, 0xe0, 0x61 // mvn vs r12 r14 ROR 18
};
const byte kInstruction_mvn_lt_r6_r11_ROR_2[] = {
  0x6b, 0x61, 0xe0, 0xb1 // mvn lt r6 r11 ROR 2
};
const byte kInstruction_mvn_cs_r12_r6_LSL_7[] = {
  0x86, 0xc3, 0xe0, 0x21 // mvn cs r12 r6 LSL 7
};
const byte kInstruction_mvn_gt_r1_r5_LSL_8[] = {
  0x05, 0x14, 0xe0, 0xc1 // mvn gt r1 r5 LSL 8
};
const byte kInstruction_mvn_vc_r3_r9_ROR_28[] = {
  0x69, 0x3e, 0xe0, 0x71 // mvn vc r3 r9 ROR 28
};
const byte kInstruction_mvn_eq_r1_r6_ROR_13[] = {
  0xe6, 0x16, 0xe0, 0x01 // mvn eq r1 r6 ROR 13
};
const byte kInstruction_mvn_ge_r13_r0_LSL_31[] = {
  0x80, 0xdf, 0xe0, 0xa1 // mvn ge r13 r0 LSL 31
};
const byte kInstruction_mvn_pl_r7_r10_ROR_4[] = {
  0x6a, 0x72, 0xe0, 0x51 // mvn pl r7 r10 ROR 4
};
const byte kInstruction_mvn_eq_r7_r4_ROR_22[] = {
  0x64, 0x7b, 0xe0, 0x01 // mvn eq r7 r4 ROR 22
};
const byte kInstruction_mvn_gt_r14_r12_LSL_25[] = {
  0x8c, 0xec, 0xe0, 0xc1 // mvn gt r14 r12 LSL 25
};
const byte kInstruction_mvn_eq_r2_r11_ROR_22[] = {
  0x6b, 0x2b, 0xe0, 0x01 // mvn eq r2 r11 ROR 22
};
const byte kInstruction_mvn_ge_r4_r11_ROR_17[] = {
  0xeb, 0x48, 0xe0, 0xa1 // mvn ge r4 r11 ROR 17
};
const byte kInstruction_mvn_hi_r1_r9_LSL_12[] = {
  0x09, 0x16, 0xe0, 0x81 // mvn hi r1 r9 LSL 12
};
const byte kInstruction_mvn_mi_r2_r12_ROR_6[] = {
  0x6c, 0x23, 0xe0, 0x41 // mvn mi r2 r12 ROR 6
};
const byte kInstruction_mvn_vs_r6_r9_ROR_15[] = {
  0xe9, 0x67, 0xe0, 0x61 // mvn vs r6 r9 ROR 15
};
const byte kInstruction_mvn_mi_r6_r4_LSL_27[] = {
  0x84, 0x6d, 0xe0, 0x41 // mvn mi r6 r4 LSL 27
};
const byte kInstruction_mvn_vc_r5_r10_LSL_7[] = {
  0x8a, 0x53, 0xe0, 0x71 // mvn vc r5 r10 LSL 7
};
const byte kInstruction_mvn_ls_r4_r12_LSL_6[] = {
  0x0c, 0x43, 0xe0, 0x91 // mvn ls r4 r12 LSL 6
};
const byte kInstruction_mvn_mi_r11_r4_ROR_11[] = {
  0xe4, 0xb5, 0xe0, 0x41 // mvn mi r11 r4 ROR 11
};
const byte kInstruction_mvn_le_r0_r14_ROR_27[] = {
  0xee, 0x0d, 0xe0, 0xd1 // mvn le r0 r14 ROR 27
};
const byte kInstruction_mvn_gt_r4_r9_ROR_18[] = {
  0x69, 0x49, 0xe0, 0xc1 // mvn gt r4 r9 ROR 18
};
const byte kInstruction_mvn_hi_r0_r3_ROR_10[] = {
  0x63, 0x05, 0xe0, 0x81 // mvn hi r0 r3 ROR 10
};
const byte kInstruction_mvn_pl_r13_r3_ROR_25[] = {
  0xe3, 0xdc, 0xe0, 0x51 // mvn pl r13 r3 ROR 25
};
const byte kInstruction_mvn_pl_r1_r14_ROR_1[] = {
  0xee, 0x10, 0xe0, 0x51 // mvn pl r1 r14 ROR 1
};
const byte kInstruction_mvn_ls_r7_r7_ROR_31[] = {
  0xe7, 0x7f, 0xe0, 0x91 // mvn ls r7 r7 ROR 31
};
const byte kInstruction_mvn_gt_r12_r11_LSL_5[] = {
  0x8b, 0xc2, 0xe0, 0xc1 // mvn gt r12 r11 LSL 5
};
const byte kInstruction_mvn_lt_r3_r10_LSL_30[] = {
  0x0a, 0x3f, 0xe0, 0xb1 // mvn lt r3 r10 LSL 30
};
const byte kInstruction_mvn_eq_r14_r3_ROR_24[] = {
  0x63, 0xec, 0xe0, 0x01 // mvn eq r14 r3 ROR 24
};
const byte kInstruction_mvn_le_r14_r5_ROR_18[] = {
  0x65, 0xe9, 0xe0, 0xd1 // mvn le r14 r5 ROR 18
};
const byte kInstruction_mvn_vs_r4_r6_ROR_9[] = {
  0xe6, 0x44, 0xe0, 0x61 // mvn vs r4 r6 ROR 9
};
const byte kInstruction_mvn_vc_r3_r1_ROR_25[] = {
  0xe1, 0x3c, 0xe0, 0x71 // mvn vc r3 r1 ROR 25
};
const byte kInstruction_mvn_vc_r8_r13_ROR_18[] = {
  0x6d, 0x89, 0xe0, 0x71 // mvn vc r8 r13 ROR 18
};
const byte kInstruction_mvn_ls_r7_r13_LSL_28[] = {
  0x0d, 0x7e, 0xe0, 0x91 // mvn ls r7 r13 LSL 28
};
const byte kInstruction_mvn_vs_r6_r11_LSL_18[] = {
  0x0b, 0x69, 0xe0, 0x61 // mvn vs r6 r11 LSL 18
};
const byte kInstruction_mvn_lt_r7_r4_LSL_22[] = {
  0x04, 0x7b, 0xe0, 0xb1 // mvn lt r7 r4 LSL 22
};
const byte kInstruction_mvn_pl_r4_r6_ROR_2[] = {
  0x66, 0x41, 0xe0, 0x51 // mvn pl r4 r6 ROR 2
};
const byte kInstruction_mvn_ls_r2_r2_ROR_20[] = {
  0x62, 0x2a, 0xe0, 0x91 // mvn ls r2 r2 ROR 20
};
const byte kInstruction_mvn_ge_r8_r12_LSL_31[] = {
  0x8c, 0x8f, 0xe0, 0xa1 // mvn ge r8 r12 LSL 31
};
const byte kInstruction_mvn_cc_r9_r9_LSL_26[] = {
  0x09, 0x9d, 0xe0, 0x31 // mvn cc r9 r9 LSL 26
};
const byte kInstruction_mvn_hi_r13_r13_ROR_22[] = {
  0x6d, 0xdb, 0xe0, 0x81 // mvn hi r13 r13 ROR 22
};
const byte kInstruction_mvn_mi_r0_r13_ROR_26[] = {
  0x6d, 0x0d, 0xe0, 0x41 // mvn mi r0 r13 ROR 26
};
const byte kInstruction_mvn_eq_r9_r11_LSL_20[] = {
  0x0b, 0x9a, 0xe0, 0x01 // mvn eq r9 r11 LSL 20
};
const byte kInstruction_mvn_cs_r12_r12_LSL_30[] = {
  0x0c, 0xcf, 0xe0, 0x21 // mvn cs r12 r12 LSL 30
};
const byte kInstruction_mvn_mi_r9_r8_LSL_17[] = {
  0x88, 0x98, 0xe0, 0x41 // mvn mi r9 r8 LSL 17
};
const byte kInstruction_mvn_cc_r0_r10_ROR_28[] = {
  0x6a, 0x0e, 0xe0, 0x31 // mvn cc r0 r10 ROR 28
};
const byte kInstruction_mvn_lt_r3_r13_ROR_19[] = {
  0xed, 0x39, 0xe0, 0xb1 // mvn lt r3 r13 ROR 19
};
const byte kInstruction_mvn_ne_r14_r9_LSL_22[] = {
  0x09, 0xeb, 0xe0, 0x11 // mvn ne r14 r9 LSL 22
};
const byte kInstruction_mvn_ne_r8_r13_LSL_24[] = {
  0x0d, 0x8c, 0xe0, 0x11 // mvn ne r8 r13 LSL 24
};
const byte kInstruction_mvn_eq_r9_r5_LSL_17[] = {
  0x85, 0x98, 0xe0, 0x01 // mvn eq r9 r5 LSL 17
};
const byte kInstruction_mvn_al_r5_r10_LSL_19[] = {
  0x8a, 0x59, 0xe0, 0xe1 // mvn al r5 r10 LSL 19
};
const byte kInstruction_mvn_ge_r13_r13_LSL_7[] = {
  0x8d, 0xd3, 0xe0, 0xa1 // mvn ge r13 r13 LSL 7
};
const byte kInstruction_mvn_vc_r14_r4_LSL_30[] = {
  0x04, 0xef, 0xe0, 0x71 // mvn vc r14 r4 LSL 30
};
const byte kInstruction_mvn_pl_r7_r12_LSL_8[] = {
  0x0c, 0x74, 0xe0, 0x51 // mvn pl r7 r12 LSL 8
};
const byte kInstruction_mvn_eq_r3_r6_LSL_23[] = {
  0x86, 0x3b, 0xe0, 0x01 // mvn eq r3 r6 LSL 23
};
const byte kInstruction_mvn_le_r2_r7_ROR_1[] = {
  0xe7, 0x20, 0xe0, 0xd1 // mvn le r2 r7 ROR 1
};
const byte kInstruction_mvn_al_r0_r8_LSL_25[] = {
  0x88, 0x0c, 0xe0, 0xe1 // mvn al r0 r8 LSL 25
};
const byte kInstruction_mvn_le_r2_r8_LSL_25[] = {
  0x88, 0x2c, 0xe0, 0xd1 // mvn le r2 r8 LSL 25
};
const byte kInstruction_mvn_ge_r5_r4_ROR_7[] = {
  0xe4, 0x53, 0xe0, 0xa1 // mvn ge r5 r4 ROR 7
};
const byte kInstruction_mvn_vc_r1_r13_LSL_13[] = {
  0x8d, 0x16, 0xe0, 0x71 // mvn vc r1 r13 LSL 13
};
const byte kInstruction_mvn_ge_r5_r11_ROR_23[] = {
  0xeb, 0x5b, 0xe0, 0xa1 // mvn ge r5 r11 ROR 23
};
const byte kInstruction_mvn_eq_r13_r0_ROR_29[] = {
  0xe0, 0xde, 0xe0, 0x01 // mvn eq r13 r0 ROR 29
};
const byte kInstruction_mvn_vc_r2_r14_LSL_28[] = {
  0x0e, 0x2e, 0xe0, 0x71 // mvn vc r2 r14 LSL 28
};
const byte kInstruction_mvn_lt_r14_r7_LSL_28[] = {
  0x07, 0xee, 0xe0, 0xb1 // mvn lt r14 r7 LSL 28
};
const byte kInstruction_mvn_gt_r2_r6_ROR_20[] = {
  0x66, 0x2a, 0xe0, 0xc1 // mvn gt r2 r6 ROR 20
};
const byte kInstruction_mvn_pl_r10_r12_ROR_3[] = {
  0xec, 0xa1, 0xe0, 0x51 // mvn pl r10 r12 ROR 3
};
const byte kInstruction_mvn_vc_r6_r3_ROR_3[] = {
  0xe3, 0x61, 0xe0, 0x71 // mvn vc r6 r3 ROR 3
};
const byte kInstruction_mvn_cc_r11_r12_LSL_28[] = {
  0x0c, 0xbe, 0xe0, 0x31 // mvn cc r11 r12 LSL 28
};
const byte kInstruction_mvn_cs_r7_r3_LSL_11[] = {
  0x83, 0x75, 0xe0, 0x21 // mvn cs r7 r3 LSL 11
};
const byte kInstruction_mvn_mi_r2_r7_ROR_1[] = {
  0xe7, 0x20, 0xe0, 0x41 // mvn mi r2 r7 ROR 1
};
const byte kInstruction_mvn_vc_r1_r4_ROR_15[] = {
  0xe4, 0x17, 0xe0, 0x71 // mvn vc r1 r4 ROR 15
};
const byte kInstruction_mvn_le_r11_r11_ROR_5[] = {
  0xeb, 0xb2, 0xe0, 0xd1 // mvn le r11 r11 ROR 5
};
const byte kInstruction_mvn_vc_r9_r2_LSL_10[] = {
  0x02, 0x95, 0xe0, 0x71 // mvn vc r9 r2 LSL 10
};
const byte kInstruction_mvn_hi_r2_r10_LSL_17[] = {
  0x8a, 0x28, 0xe0, 0x81 // mvn hi r2 r10 LSL 17
};
const byte kInstruction_mvn_al_r11_r4_ROR_13[] = {
  0xe4, 0xb6, 0xe0, 0xe1 // mvn al r11 r4 ROR 13
};
const byte kInstruction_mvn_le_r2_r14_LSL_20[] = {
  0x0e, 0x2a, 0xe0, 0xd1 // mvn le r2 r14 LSL 20
};
const byte kInstruction_mvn_cc_r7_r9_LSL_3[] = {
  0x89, 0x71, 0xe0, 0x31 // mvn cc r7 r9 LSL 3
};
const byte kInstruction_mvn_hi_r11_r4_LSL_14[] = {
  0x04, 0xb7, 0xe0, 0x81 // mvn hi r11 r4 LSL 14
};
const byte kInstruction_mvn_gt_r14_r7_LSL_24[] = {
  0x07, 0xec, 0xe0, 0xc1 // mvn gt r14 r7 LSL 24
};
const byte kInstruction_mvn_ne_r9_r0_ROR_25[] = {
  0xe0, 0x9c, 0xe0, 0x11 // mvn ne r9 r0 ROR 25
};
const byte kInstruction_mvn_pl_r13_r2_LSL_4[] = {
  0x02, 0xd2, 0xe0, 0x51 // mvn pl r13 r2 LSL 4
};
const byte kInstruction_mvn_lt_r1_r14_LSL_15[] = {
  0x8e, 0x17, 0xe0, 0xb1 // mvn lt r1 r14 LSL 15
};
const byte kInstruction_mvn_cc_r7_r13_ROR_31[] = {
  0xed, 0x7f, 0xe0, 0x31 // mvn cc r7 r13 ROR 31
};
const byte kInstruction_mvn_ne_r11_r5_LSL_12[] = {
  0x05, 0xb6, 0xe0, 0x11 // mvn ne r11 r5 LSL 12
};
const byte kInstruction_mvn_lt_r14_r3_LSL_4[] = {
  0x03, 0xe2, 0xe0, 0xb1 // mvn lt r14 r3 LSL 4
};
const byte kInstruction_mvn_vs_r0_r14_ROR_29[] = {
  0xee, 0x0e, 0xe0, 0x61 // mvn vs r0 r14 ROR 29
};
const byte kInstruction_mvn_gt_r7_r2_LSL_4[] = {
  0x02, 0x72, 0xe0, 0xc1 // mvn gt r7 r2 LSL 4
};
const byte kInstruction_mvn_vs_r13_r6_ROR_22[] = {
  0x66, 0xdb, 0xe0, 0x61 // mvn vs r13 r6 ROR 22
};
const byte kInstruction_mvn_pl_r14_r2_ROR_24[] = {
  0x62, 0xec, 0xe0, 0x51 // mvn pl r14 r2 ROR 24
};
const byte kInstruction_mvn_cc_r2_r6_ROR_6[] = {
  0x66, 0x23, 0xe0, 0x31 // mvn cc r2 r6 ROR 6
};
const byte kInstruction_mvn_vs_r14_r13_ROR_10[] = {
  0x6d, 0xe5, 0xe0, 0x61 // mvn vs r14 r13 ROR 10
};
const byte kInstruction_mvn_gt_r8_r11_ROR_13[] = {
  0xeb, 0x86, 0xe0, 0xc1 // mvn gt r8 r11 ROR 13
};
const byte kInstruction_mvn_gt_r9_r12_ROR_21[] = {
  0xec, 0x9a, 0xe0, 0xc1 // mvn gt r9 r12 ROR 21
};
const byte kInstruction_mvn_al_r3_r11_ROR_17[] = {
  0xeb, 0x38, 0xe0, 0xe1 // mvn al r3 r11 ROR 17
};
const byte kInstruction_mvn_hi_r0_r0_LSL_25[] = {
  0x80, 0x0c, 0xe0, 0x81 // mvn hi r0 r0 LSL 25
};
const byte kInstruction_mvn_ge_r14_r9_LSL_12[] = {
  0x09, 0xe6, 0xe0, 0xa1 // mvn ge r14 r9 LSL 12
};
const byte kInstruction_mvn_pl_r9_r8_ROR_5[] = {
  0xe8, 0x92, 0xe0, 0x51 // mvn pl r9 r8 ROR 5
};
const byte kInstruction_mvn_ge_r6_r0_ROR_2[] = {
  0x60, 0x61, 0xe0, 0xa1 // mvn ge r6 r0 ROR 2
};
const byte kInstruction_mvn_vs_r2_r6_ROR_21[] = {
  0xe6, 0x2a, 0xe0, 0x61 // mvn vs r2 r6 ROR 21
};
const byte kInstruction_mvn_vs_r14_r2_LSL_18[] = {
  0x02, 0xe9, 0xe0, 0x61 // mvn vs r14 r2 LSL 18
};
const byte kInstruction_mvn_mi_r11_r9_ROR_30[] = {
  0x69, 0xbf, 0xe0, 0x41 // mvn mi r11 r9 ROR 30
};
const byte kInstruction_mvn_cc_r10_r12_ROR_4[] = {
  0x6c, 0xa2, 0xe0, 0x31 // mvn cc r10 r12 ROR 4
};
const byte kInstruction_mvn_vc_r2_r4_ROR_31[] = {
  0xe4, 0x2f, 0xe0, 0x71 // mvn vc r2 r4 ROR 31
};
const byte kInstruction_mvn_ge_r0_r12_ROR_2[] = {
  0x6c, 0x01, 0xe0, 0xa1 // mvn ge r0 r12 ROR 2
};
const byte kInstruction_mvn_cc_r13_r14_LSL_11[] = {
  0x8e, 0xd5, 0xe0, 0x31 // mvn cc r13 r14 LSL 11
};
const byte kInstruction_mvn_cs_r3_r3_ROR_26[] = {
  0x63, 0x3d, 0xe0, 0x21 // mvn cs r3 r3 ROR 26
};
const byte kInstruction_mvn_cc_r0_r0_ROR_21[] = {
  0xe0, 0x0a, 0xe0, 0x31 // mvn cc r0 r0 ROR 21
};
const byte kInstruction_mvn_vc_r13_r11_ROR_19[] = {
  0xeb, 0xd9, 0xe0, 0x71 // mvn vc r13 r11 ROR 19
};
const byte kInstruction_mvn_al_r0_r7_ROR_6[] = {
  0x67, 0x03, 0xe0, 0xe1 // mvn al r0 r7 ROR 6
};
const byte kInstruction_mvn_mi_r0_r8_ROR_19[] = {
  0xe8, 0x09, 0xe0, 0x41 // mvn mi r0 r8 ROR 19
};
const byte kInstruction_mvn_cs_r1_r13_ROR_20[] = {
  0x6d, 0x1a, 0xe0, 0x21 // mvn cs r1 r13 ROR 20
};
const byte kInstruction_mvn_vs_r2_r13_ROR_28[] = {
  0x6d, 0x2e, 0xe0, 0x61 // mvn vs r2 r13 ROR 28
};
const byte kInstruction_mvn_mi_r5_r6_LSL_23[] = {
  0x86, 0x5b, 0xe0, 0x41 // mvn mi r5 r6 LSL 23
};
const byte kInstruction_mvn_ne_r8_r13_ROR_3[] = {
  0xed, 0x81, 0xe0, 0x11 // mvn ne r8 r13 ROR 3
};
const byte kInstruction_mvn_le_r9_r12_ROR_24[] = {
  0x6c, 0x9c, 0xe0, 0xd1 // mvn le r9 r12 ROR 24
};
const byte kInstruction_mvn_le_r4_r0_LSL_10[] = {
  0x00, 0x45, 0xe0, 0xd1 // mvn le r4 r0 LSL 10
};
const byte kInstruction_mvn_hi_r6_r1_ROR_5[] = {
  0xe1, 0x62, 0xe0, 0x81 // mvn hi r6 r1 ROR 5
};
const byte kInstruction_mvn_pl_r1_r5_ROR_31[] = {
  0xe5, 0x1f, 0xe0, 0x51 // mvn pl r1 r5 ROR 31
};
const byte kInstruction_mvn_lt_r1_r13_LSL_7[] = {
  0x8d, 0x13, 0xe0, 0xb1 // mvn lt r1 r13 LSL 7
};
const byte kInstruction_mvn_ge_r8_r10_LSL_24[] = {
  0x0a, 0x8c, 0xe0, 0xa1 // mvn ge r8 r10 LSL 24
};
const byte kInstruction_mvn_eq_r10_r2_ROR_6[] = {
  0x62, 0xa3, 0xe0, 0x01 // mvn eq r10 r2 ROR 6
};
const byte kInstruction_mvn_ls_r7_r2_LSL_3[] = {
  0x82, 0x71, 0xe0, 0x91 // mvn ls r7 r2 LSL 3
};
const byte kInstruction_mvn_al_r13_r6_ROR_19[] = {
  0xe6, 0xd9, 0xe0, 0xe1 // mvn al r13 r6 ROR 19
};
const byte kInstruction_mvn_le_r10_r2_ROR_26[] = {
  0x62, 0xad, 0xe0, 0xd1 // mvn le r10 r2 ROR 26
};
const byte kInstruction_mvn_ls_r11_r7_LSL_25[] = {
  0x87, 0xbc, 0xe0, 0x91 // mvn ls r11 r7 LSL 25
};
const byte kInstruction_mvn_eq_r7_r0_LSL_5[] = {
  0x80, 0x72, 0xe0, 0x01 // mvn eq r7 r0 LSL 5
};
const byte kInstruction_mvn_lt_r3_r11_LSL_23[] = {
  0x8b, 0x3b, 0xe0, 0xb1 // mvn lt r3 r11 LSL 23
};
const byte kInstruction_mvn_ls_r13_r0_ROR_13[] = {
  0xe0, 0xd6, 0xe0, 0x91 // mvn ls r13 r0 ROR 13
};
const byte kInstruction_mvn_mi_r4_r10_LSL_22[] = {
  0x0a, 0x4b, 0xe0, 0x41 // mvn mi r4 r10 LSL 22
};
const byte kInstruction_mvn_pl_r6_r8_LSL_6[] = {
  0x08, 0x63, 0xe0, 0x51 // mvn pl r6 r8 LSL 6
};
const byte kInstruction_mvn_le_r13_r2_ROR_16[] = {
  0x62, 0xd8, 0xe0, 0xd1 // mvn le r13 r2 ROR 16
};
const byte kInstruction_mvn_al_r6_r8_ROR_9[] = {
  0xe8, 0x64, 0xe0, 0xe1 // mvn al r6 r8 ROR 9
};
const byte kInstruction_mvn_lt_r11_r14_ROR_4[] = {
  0x6e, 0xb2, 0xe0, 0xb1 // mvn lt r11 r14 ROR 4
};
const byte kInstruction_mvn_ls_r14_r14_ROR_9[] = {
  0xee, 0xe4, 0xe0, 0x91 // mvn ls r14 r14 ROR 9
};
const byte kInstruction_mvn_gt_r1_r0_LSL_23[] = {
  0x80, 0x1b, 0xe0, 0xc1 // mvn gt r1 r0 LSL 23
};
const byte kInstruction_mvn_lt_r4_r0_ROR_25[] = {
  0xe0, 0x4c, 0xe0, 0xb1 // mvn lt r4 r0 ROR 25
};
const byte kInstruction_mvn_hi_r4_r12_ROR_18[] = {
  0x6c, 0x49, 0xe0, 0x81 // mvn hi r4 r12 ROR 18
};
const byte kInstruction_mvn_ne_r10_r2_ROR_28[] = {
  0x62, 0xae, 0xe0, 0x11 // mvn ne r10 r2 ROR 28
};
const byte kInstruction_mvn_lt_r1_r4_ROR_2[] = {
  0x64, 0x11, 0xe0, 0xb1 // mvn lt r1 r4 ROR 2
};
const byte kInstruction_mvn_mi_r4_r10_ROR_23[] = {
  0xea, 0x4b, 0xe0, 0x41 // mvn mi r4 r10 ROR 23
};
const byte kInstruction_mvn_cs_r12_r7_LSL_9[] = {
  0x87, 0xc4, 0xe0, 0x21 // mvn cs r12 r7 LSL 9
};
const byte kInstruction_mvn_lt_r7_r14_ROR_4[] = {
  0x6e, 0x72, 0xe0, 0xb1 // mvn lt r7 r14 ROR 4
};
const byte kInstruction_mvn_lt_r2_r10_ROR_25[] = {
  0xea, 0x2c, 0xe0, 0xb1 // mvn lt r2 r10 ROR 25
};
const byte kInstruction_mvn_hi_r4_r9_ROR_24[] = {
  0x69, 0x4c, 0xe0, 0x81 // mvn hi r4 r9 ROR 24
};
const byte kInstruction_mvn_hi_r0_r12_ROR_29[] = {
  0xec, 0x0e, 0xe0, 0x81 // mvn hi r0 r12 ROR 29
};
const byte kInstruction_mvn_ge_r8_r9_LSL_10[] = {
  0x09, 0x85, 0xe0, 0xa1 // mvn ge r8 r9 LSL 10
};
const byte kInstruction_mvn_ge_r2_r13_LSL_21[] = {
  0x8d, 0x2a, 0xe0, 0xa1 // mvn ge r2 r13 LSL 21
};
const byte kInstruction_mvn_lt_r8_r12_LSL_28[] = {
  0x0c, 0x8e, 0xe0, 0xb1 // mvn lt r8 r12 LSL 28
};
const byte kInstruction_mvn_hi_r13_r11_LSL_20[] = {
  0x0b, 0xda, 0xe0, 0x81 // mvn hi r13 r11 LSL 20
};
const byte kInstruction_mvn_eq_r6_r0_LSL_17[] = {
  0x80, 0x68, 0xe0, 0x01 // mvn eq r6 r0 LSL 17
};
const byte kInstruction_mvn_vc_r1_r0_ROR_15[] = {
  0xe0, 0x17, 0xe0, 0x71 // mvn vc r1 r0 ROR 15
};
const byte kInstruction_mvn_ge_r13_r11_LSL_7[] = {
  0x8b, 0xd3, 0xe0, 0xa1 // mvn ge r13 r11 LSL 7
};
const byte kInstruction_mvn_mi_r6_r10_LSL_11[] = {
  0x8a, 0x65, 0xe0, 0x41 // mvn mi r6 r10 LSL 11
};
const byte kInstruction_mvn_mi_r1_r8_ROR_26[] = {
  0x68, 0x1d, 0xe0, 0x41 // mvn mi r1 r8 ROR 26
};
const byte kInstruction_mvn_vs_r12_r6_LSL_1[] = {
  0x86, 0xc0, 0xe0, 0x61 // mvn vs r12 r6 LSL 1
};
const byte kInstruction_mvn_lt_r5_r13_LSL_12[] = {
  0x0d, 0x56, 0xe0, 0xb1 // mvn lt r5 r13 LSL 12
};
const byte kInstruction_mvn_pl_r13_r10_LSL_7[] = {
  0x8a, 0xd3, 0xe0, 0x51 // mvn pl r13 r10 LSL 7
};
const byte kInstruction_mvn_hi_r3_r8_LSL_2[] = {
  0x08, 0x31, 0xe0, 0x81 // mvn hi r3 r8 LSL 2
};
const byte kInstruction_mvn_ge_r12_r9_ROR_5[] = {
  0xe9, 0xc2, 0xe0, 0xa1 // mvn ge r12 r9 ROR 5
};
const byte kInstruction_mvn_lt_r11_r11_LSL_21[] = {
  0x8b, 0xba, 0xe0, 0xb1 // mvn lt r11 r11 LSL 21
};
const byte kInstruction_mvn_le_r13_r11_LSL_10[] = {
  0x0b, 0xd5, 0xe0, 0xd1 // mvn le r13 r11 LSL 10
};
const byte kInstruction_mvn_cs_r0_r8_ROR_19[] = {
  0xe8, 0x09, 0xe0, 0x21 // mvn cs r0 r8 ROR 19
};
const byte kInstruction_mvn_ge_r1_r13_ROR_19[] = {
  0xed, 0x19, 0xe0, 0xa1 // mvn ge r1 r13 ROR 19
};
const byte kInstruction_mvn_al_r14_r7_ROR_16[] = {
  0x67, 0xe8, 0xe0, 0xe1 // mvn al r14 r7 ROR 16
};
const byte kInstruction_mvn_lt_r7_r12_LSL_22[] = {
  0x0c, 0x7b, 0xe0, 0xb1 // mvn lt r7 r12 LSL 22
};
const byte kInstruction_mvn_al_r3_r12_LSL_25[] = {
  0x8c, 0x3c, 0xe0, 0xe1 // mvn al r3 r12 LSL 25
};
const byte kInstruction_mvn_cc_r10_r0_ROR_8[] = {
  0x60, 0xa4, 0xe0, 0x31 // mvn cc r10 r0 ROR 8
};
const byte kInstruction_mvn_ls_r10_r4_LSL_26[] = {
  0x04, 0xad, 0xe0, 0x91 // mvn ls r10 r4 LSL 26
};
const byte kInstruction_mvn_hi_r11_r13_ROR_28[] = {
  0x6d, 0xbe, 0xe0, 0x81 // mvn hi r11 r13 ROR 28
};
const byte kInstruction_mvn_le_r14_r14_LSL_28[] = {
  0x0e, 0xee, 0xe0, 0xd1 // mvn le r14 r14 LSL 28
};
const byte kInstruction_mvn_le_r4_r8_LSL_31[] = {
  0x88, 0x4f, 0xe0, 0xd1 // mvn le r4 r8 LSL 31
};
const byte kInstruction_mvn_ge_r8_r6_LSL_1[] = {
  0x86, 0x80, 0xe0, 0xa1 // mvn ge r8 r6 LSL 1
};
const byte kInstruction_mvn_ne_r8_r2_ROR_10[] = {
  0x62, 0x85, 0xe0, 0x11 // mvn ne r8 r2 ROR 10
};
const byte kInstruction_mvn_pl_r8_r2_LSL_18[] = {
  0x02, 0x89, 0xe0, 0x51 // mvn pl r8 r2 LSL 18
};
const byte kInstruction_mvn_ne_r6_r9_LSL_5[] = {
  0x89, 0x62, 0xe0, 0x11 // mvn ne r6 r9 LSL 5
};
const byte kInstruction_mvn_lt_r0_r0_ROR_16[] = {
  0x60, 0x08, 0xe0, 0xb1 // mvn lt r0 r0 ROR 16
};
const byte kInstruction_mvn_le_r6_r1_ROR_13[] = {
  0xe1, 0x66, 0xe0, 0xd1 // mvn le r6 r1 ROR 13
};
const byte kInstruction_mvn_hi_r10_r11_ROR_5[] = {
  0xeb, 0xa2, 0xe0, 0x81 // mvn hi r10 r11 ROR 5
};
const byte kInstruction_mvn_al_r13_r14_ROR_6[] = {
  0x6e, 0xd3, 0xe0, 0xe1 // mvn al r13 r14 ROR 6
};
const byte kInstruction_mvn_vs_r8_r1_LSL_29[] = {
  0x81, 0x8e, 0xe0, 0x61 // mvn vs r8 r1 LSL 29
};
const byte kInstruction_mvn_vc_r12_r5_LSL_9[] = {
  0x85, 0xc4, 0xe0, 0x71 // mvn vc r12 r5 LSL 9
};
const byte kInstruction_mvn_vs_r3_r5_ROR_31[] = {
  0xe5, 0x3f, 0xe0, 0x61 // mvn vs r3 r5 ROR 31
};
const byte kInstruction_mvn_cs_r12_r1_ROR_14[] = {
  0x61, 0xc7, 0xe0, 0x21 // mvn cs r12 r1 ROR 14
};
const byte kInstruction_mvn_vc_r0_r9_ROR_15[] = {
  0xe9, 0x07, 0xe0, 0x71 // mvn vc r0 r9 ROR 15
};
const byte kInstruction_mvn_ls_r7_r0_LSL_8[] = {
  0x00, 0x74, 0xe0, 0x91 // mvn ls r7 r0 LSL 8
};
const byte kInstruction_mvn_ge_r5_r6_LSL_28[] = {
  0x06, 0x5e, 0xe0, 0xa1 // mvn ge r5 r6 LSL 28
};
const byte kInstruction_mvn_cs_r6_r13_LSL_30[] = {
  0x0d, 0x6f, 0xe0, 0x21 // mvn cs r6 r13 LSL 30
};
const byte kInstruction_mvn_mi_r9_r10_ROR_17[] = {
  0xea, 0x98, 0xe0, 0x41 // mvn mi r9 r10 ROR 17
};
const byte kInstruction_mvn_ne_r6_r7_ROR_8[] = {
  0x67, 0x64, 0xe0, 0x11 // mvn ne r6 r7 ROR 8
};
const byte kInstruction_mvn_ge_r12_r6_ROR_11[] = {
  0xe6, 0xc5, 0xe0, 0xa1 // mvn ge r12 r6 ROR 11
};
const byte kInstruction_mvn_eq_r9_r1_LSL_28[] = {
  0x01, 0x9e, 0xe0, 0x01 // mvn eq r9 r1 LSL 28
};
const byte kInstruction_mvn_gt_r14_r11_LSL_30[] = {
  0x0b, 0xef, 0xe0, 0xc1 // mvn gt r14 r11 LSL 30
};
const byte kInstruction_mvn_ls_r11_r10_LSL_24[] = {
  0x0a, 0xbc, 0xe0, 0x91 // mvn ls r11 r10 LSL 24
};
const byte kInstruction_mvn_ls_r13_r12_LSL_29[] = {
  0x8c, 0xde, 0xe0, 0x91 // mvn ls r13 r12 LSL 29
};
const byte kInstruction_mvn_hi_r12_r4_LSL_7[] = {
  0x84, 0xc3, 0xe0, 0x81 // mvn hi r12 r4 LSL 7
};
const byte kInstruction_mvn_ls_r11_r1_ROR_26[] = {
  0x61, 0xbd, 0xe0, 0x91 // mvn ls r11 r1 ROR 26
};
const byte kInstruction_mvn_ls_r14_r6_LSL_10[] = {
  0x06, 0xe5, 0xe0, 0x91 // mvn ls r14 r6 LSL 10
};
const byte kInstruction_mvn_le_r11_r10_ROR_18[] = {
  0x6a, 0xb9, 0xe0, 0xd1 // mvn le r11 r10 ROR 18
};
const byte kInstruction_mvn_cc_r0_r10_LSL_15[] = {
  0x8a, 0x07, 0xe0, 0x31 // mvn cc r0 r10 LSL 15
};
const byte kInstruction_mvn_eq_r7_r14_ROR_29[] = {
  0xee, 0x7e, 0xe0, 0x01 // mvn eq r7 r14 ROR 29
};
const byte kInstruction_mvn_lt_r13_r3_ROR_19[] = {
  0xe3, 0xd9, 0xe0, 0xb1 // mvn lt r13 r3 ROR 19
};
const byte kInstruction_mvn_ge_r7_r3_ROR_4[] = {
  0x63, 0x72, 0xe0, 0xa1 // mvn ge r7 r3 ROR 4
};
const byte kInstruction_mvn_lt_r3_r3_ROR_24[] = {
  0x63, 0x3c, 0xe0, 0xb1 // mvn lt r3 r3 ROR 24
};
const byte kInstruction_mvn_pl_r4_r1_ROR_14[] = {
  0x61, 0x47, 0xe0, 0x51 // mvn pl r4 r1 ROR 14
};
const byte kInstruction_mvn_lt_r1_r9_LSL_4[] = {
  0x09, 0x12, 0xe0, 0xb1 // mvn lt r1 r9 LSL 4
};
const byte kInstruction_mvn_vs_r5_r11_ROR_15[] = {
  0xeb, 0x57, 0xe0, 0x61 // mvn vs r5 r11 ROR 15
};
const byte kInstruction_mvn_gt_r3_r12_LSL_22[] = {
  0x0c, 0x3b, 0xe0, 0xc1 // mvn gt r3 r12 LSL 22
};
const byte kInstruction_mvn_vs_r14_r5_LSL_18[] = {
  0x05, 0xe9, 0xe0, 0x61 // mvn vs r14 r5 LSL 18
};
const byte kInstruction_mvn_ls_r11_r6_ROR_9[] = {
  0xe6, 0xb4, 0xe0, 0x91 // mvn ls r11 r6 ROR 9
};
const byte kInstruction_mvn_mi_r8_r6_LSL_6[] = {
  0x06, 0x83, 0xe0, 0x41 // mvn mi r8 r6 LSL 6
};
const byte kInstruction_mvn_lt_r12_r12_ROR_18[] = {
  0x6c, 0xc9, 0xe0, 0xb1 // mvn lt r12 r12 ROR 18
};
const byte kInstruction_mvn_ls_r11_r5_ROR_1[] = {
  0xe5, 0xb0, 0xe0, 0x91 // mvn ls r11 r5 ROR 1
};
const byte kInstruction_mvn_ne_r5_r9_LSL_26[] = {
  0x09, 0x5d, 0xe0, 0x11 // mvn ne r5 r9 LSL 26
};
const byte kInstruction_mvn_eq_r4_r10_ROR_5[] = {
  0xea, 0x42, 0xe0, 0x01 // mvn eq r4 r10 ROR 5
};
const byte kInstruction_mvn_ne_r14_r11_ROR_7[] = {
  0xeb, 0xe3, 0xe0, 0x11 // mvn ne r14 r11 ROR 7
};
const byte kInstruction_mvn_eq_r3_r10_ROR_28[] = {
  0x6a, 0x3e, 0xe0, 0x01 // mvn eq r3 r10 ROR 28
};
const byte kInstruction_mvn_vc_r1_r11_LSL_13[] = {
  0x8b, 0x16, 0xe0, 0x71 // mvn vc r1 r11 LSL 13
};
const byte kInstruction_mvn_ge_r14_r3_LSL_18[] = {
  0x03, 0xe9, 0xe0, 0xa1 // mvn ge r14 r3 LSL 18
};
const byte kInstruction_mvn_al_r12_r11_LSL_19[] = {
  0x8b, 0xc9, 0xe0, 0xe1 // mvn al r12 r11 LSL 19
};
const byte kInstruction_mvn_al_r8_r11_LSL_10[] = {
  0x0b, 0x85, 0xe0, 0xe1 // mvn al r8 r11 LSL 10
};
const byte kInstruction_mvn_mi_r0_r2_LSL_17[] = {
  0x82, 0x08, 0xe0, 0x41 // mvn mi r0 r2 LSL 17
};
const byte kInstruction_mvn_ls_r8_r4_LSL_30[] = {
  0x04, 0x8f, 0xe0, 0x91 // mvn ls r8 r4 LSL 30
};
const byte kInstruction_mvn_gt_r9_r3_LSL_5[] = {
  0x83, 0x92, 0xe0, 0xc1 // mvn gt r9 r3 LSL 5
};
const byte kInstruction_mvn_ne_r13_r11_ROR_7[] = {
  0xeb, 0xd3, 0xe0, 0x11 // mvn ne r13 r11 ROR 7
};
const byte kInstruction_mvn_gt_r8_r1_ROR_13[] = {
  0xe1, 0x86, 0xe0, 0xc1 // mvn gt r8 r1 ROR 13
};
const byte kInstruction_mvn_ne_r2_r1_LSL_31[] = {
  0x81, 0x2f, 0xe0, 0x11 // mvn ne r2 r1 LSL 31
};
const byte kInstruction_mvn_le_r12_r0_ROR_23[] = {
  0xe0, 0xcb, 0xe0, 0xd1 // mvn le r12 r0 ROR 23
};
const byte kInstruction_mvn_ge_r14_r0_ROR_20[] = {
  0x60, 0xea, 0xe0, 0xa1 // mvn ge r14 r0 ROR 20
};
const byte kInstruction_mvn_ne_r13_r4_ROR_9[] = {
  0xe4, 0xd4, 0xe0, 0x11 // mvn ne r13 r4 ROR 9
};
const byte kInstruction_mvn_lt_r12_r8_ROR_9[] = {
  0xe8, 0xc4, 0xe0, 0xb1 // mvn lt r12 r8 ROR 9
};
const byte kInstruction_mvn_gt_r1_r14_LSL_27[] = {
  0x8e, 0x1d, 0xe0, 0xc1 // mvn gt r1 r14 LSL 27
};
const byte kInstruction_mvn_ls_r0_r14_ROR_5[] = {
  0xee, 0x02, 0xe0, 0x91 // mvn ls r0 r14 ROR 5
};
const byte kInstruction_mvn_lt_r6_r5_ROR_2[] = {
  0x65, 0x61, 0xe0, 0xb1 // mvn lt r6 r5 ROR 2
};
const byte kInstruction_mvn_vs_r1_r13_LSL_7[] = {
  0x8d, 0x13, 0xe0, 0x61 // mvn vs r1 r13 LSL 7
};
const byte kInstruction_mvn_lt_r13_r2_LSL_26[] = {
  0x02, 0xdd, 0xe0, 0xb1 // mvn lt r13 r2 LSL 26
};
const byte kInstruction_mvn_cs_r11_r1_LSL_15[] = {
  0x81, 0xb7, 0xe0, 0x21 // mvn cs r11 r1 LSL 15
};
const byte kInstruction_mvn_cs_r2_r4_ROR_15[] = {
  0xe4, 0x27, 0xe0, 0x21 // mvn cs r2 r4 ROR 15
};
const byte kInstruction_mvn_gt_r9_r10_ROR_28[] = {
  0x6a, 0x9e, 0xe0, 0xc1 // mvn gt r9 r10 ROR 28
};
const byte kInstruction_mvn_ge_r4_r11_LSL_16[] = {
  0x0b, 0x48, 0xe0, 0xa1 // mvn ge r4 r11 LSL 16
};
const byte kInstruction_mvn_mi_r6_r2_LSL_2[] = {
  0x02, 0x61, 0xe0, 0x41 // mvn mi r6 r2 LSL 2
};
const byte kInstruction_mvn_ls_r0_r11_LSL_12[] = {
  0x0b, 0x06, 0xe0, 0x91 // mvn ls r0 r11 LSL 12
};
const byte kInstruction_mvn_mi_r13_r13_LSL_1[] = {
  0x8d, 0xd0, 0xe0, 0x41 // mvn mi r13 r13 LSL 1
};
const byte kInstruction_mvn_ge_r1_r11_LSL_9[] = {
  0x8b, 0x14, 0xe0, 0xa1 // mvn ge r1 r11 LSL 9
};
const byte kInstruction_mvn_gt_r7_r4_ROR_29[] = {
  0xe4, 0x7e, 0xe0, 0xc1 // mvn gt r7 r4 ROR 29
};
const byte kInstruction_mvn_eq_r9_r14_LSL_5[] = {
  0x8e, 0x92, 0xe0, 0x01 // mvn eq r9 r14 LSL 5
};
const byte kInstruction_mvn_eq_r8_r4_ROR_11[] = {
  0xe4, 0x85, 0xe0, 0x01 // mvn eq r8 r4 ROR 11
};
const byte kInstruction_mvn_ge_r5_r6_LSL_23[] = {
  0x86, 0x5b, 0xe0, 0xa1 // mvn ge r5 r6 LSL 23
};
const byte kInstruction_mvn_lt_r5_r4_LSL_23[] = {
  0x84, 0x5b, 0xe0, 0xb1 // mvn lt r5 r4 LSL 23
};
const byte kInstruction_mvn_ge_r6_r3_ROR_21[] = {
  0xe3, 0x6a, 0xe0, 0xa1 // mvn ge r6 r3 ROR 21
};
const byte kInstruction_mvn_lt_r4_r6_ROR_2[] = {
  0x66, 0x41, 0xe0, 0xb1 // mvn lt r4 r6 ROR 2
};
const byte kInstruction_mvn_hi_r7_r9_ROR_14[] = {
  0x69, 0x77, 0xe0, 0x81 // mvn hi r7 r9 ROR 14
};
const byte kInstruction_mvn_pl_r9_r9_LSL_9[] = {
  0x89, 0x94, 0xe0, 0x51 // mvn pl r9 r9 LSL 9
};
const byte kInstruction_mvn_gt_r4_r1_ROR_8[] = {
  0x61, 0x44, 0xe0, 0xc1 // mvn gt r4 r1 ROR 8
};
const byte kInstruction_mvn_pl_r13_r0_ROR_8[] = {
  0x60, 0xd4, 0xe0, 0x51 // mvn pl r13 r0 ROR 8
};
const byte kInstruction_mvn_vc_r1_r12_LSL_12[] = {
  0x0c, 0x16, 0xe0, 0x71 // mvn vc r1 r12 LSL 12
};
const byte kInstruction_mvn_ge_r12_r5_LSL_18[] = {
  0x05, 0xc9, 0xe0, 0xa1 // mvn ge r12 r5 LSL 18
};
const byte kInstruction_mvn_le_r8_r2_LSL_7[] = {
  0x82, 0x83, 0xe0, 0xd1 // mvn le r8 r2 LSL 7
};
const byte kInstruction_mvn_ne_r8_r3_ROR_26[] = {
  0x63, 0x8d, 0xe0, 0x11 // mvn ne r8 r3 ROR 26
};
const byte kInstruction_mvn_lt_r8_r0_LSL_9[] = {
  0x80, 0x84, 0xe0, 0xb1 // mvn lt r8 r0 LSL 9
};
const byte kInstruction_mvn_cc_r11_r8_LSL_28[] = {
  0x08, 0xbe, 0xe0, 0x31 // mvn cc r11 r8 LSL 28
};
const byte kInstruction_mvn_cc_r14_r14_ROR_29[] = {
  0xee, 0xee, 0xe0, 0x31 // mvn cc r14 r14 ROR 29
};
const byte kInstruction_mvn_vc_r4_r9_LSL_31[] = {
  0x89, 0x4f, 0xe0, 0x71 // mvn vc r4 r9 LSL 31
};
const byte kInstruction_mvn_ne_r14_r0_ROR_22[] = {
  0x60, 0xeb, 0xe0, 0x11 // mvn ne r14 r0 ROR 22
};
const byte kInstruction_mvn_al_r14_r5_LSL_2[] = {
  0x05, 0xe1, 0xe0, 0xe1 // mvn al r14 r5 LSL 2
};
const byte kInstruction_mvn_ge_r9_r5_LSL_20[] = {
  0x05, 0x9a, 0xe0, 0xa1 // mvn ge r9 r5 LSL 20
};
const byte kInstruction_mvn_cc_r7_r5_ROR_12[] = {
  0x65, 0x76, 0xe0, 0x31 // mvn cc r7 r5 ROR 12
};
const byte kInstruction_mvn_le_r0_r12_LSL_18[] = {
  0x0c, 0x09, 0xe0, 0xd1 // mvn le r0 r12 LSL 18
};
const byte kInstruction_mvn_ne_r6_r2_ROR_4[] = {
  0x62, 0x62, 0xe0, 0x11 // mvn ne r6 r2 ROR 4
};
const byte kInstruction_mvn_mi_r4_r2_ROR_30[] = {
  0x62, 0x4f, 0xe0, 0x41 // mvn mi r4 r2 ROR 30
};
const byte kInstruction_mvn_ls_r12_r1_ROR_21[] = {
  0xe1, 0xca, 0xe0, 0x91 // mvn ls r12 r1 ROR 21
};
const byte kInstruction_mvn_ne_r11_r6_LSL_30[] = {
  0x06, 0xbf, 0xe0, 0x11 // mvn ne r11 r6 LSL 30
};
const byte kInstruction_mvn_vs_r14_r2_ROR_16[] = {
  0x62, 0xe8, 0xe0, 0x61 // mvn vs r14 r2 ROR 16
};
const byte kInstruction_mvn_le_r2_r1_LSL_5[] = {
  0x81, 0x22, 0xe0, 0xd1 // mvn le r2 r1 LSL 5
};
const byte kInstruction_mvn_vs_r12_r14_ROR_16[] = {
  0x6e, 0xc8, 0xe0, 0x61 // mvn vs r12 r14 ROR 16
};
const byte kInstruction_mvn_hi_r10_r12_ROR_15[] = {
  0xec, 0xa7, 0xe0, 0x81 // mvn hi r10 r12 ROR 15
};
const byte kInstruction_mvn_ls_r10_r5_ROR_11[] = {
  0xe5, 0xa5, 0xe0, 0x91 // mvn ls r10 r5 ROR 11
};
const byte kInstruction_mvn_vc_r6_r0_ROR_27[] = {
  0xe0, 0x6d, 0xe0, 0x71 // mvn vc r6 r0 ROR 27
};
const byte kInstruction_mvn_vs_r8_r13_LSL_15[] = {
  0x8d, 0x87, 0xe0, 0x61 // mvn vs r8 r13 LSL 15
};
const byte kInstruction_mvn_lt_r9_r7_ROR_26[] = {
  0x67, 0x9d, 0xe0, 0xb1 // mvn lt r9 r7 ROR 26
};
const byte kInstruction_mvn_gt_r6_r14_ROR_23[] = {
  0xee, 0x6b, 0xe0, 0xc1 // mvn gt r6 r14 ROR 23
};
const byte kInstruction_mvn_hi_r4_r11_LSL_21[] = {
  0x8b, 0x4a, 0xe0, 0x81 // mvn hi r4 r11 LSL 21
};
const byte kInstruction_mvn_ls_r14_r12_LSL_14[] = {
  0x0c, 0xe7, 0xe0, 0x91 // mvn ls r14 r12 LSL 14
};
const byte kInstruction_mvn_cc_r9_r4_LSL_9[] = {
  0x84, 0x94, 0xe0, 0x31 // mvn cc r9 r4 LSL 9
};
const byte kInstruction_mvn_eq_r5_r1_LSL_26[] = {
  0x01, 0x5d, 0xe0, 0x01 // mvn eq r5 r1 LSL 26
};
const byte kInstruction_mvn_pl_r10_r7_LSL_20[] = {
  0x07, 0xaa, 0xe0, 0x51 // mvn pl r10 r7 LSL 20
};
const byte kInstruction_mvn_gt_r2_r14_LSL_25[] = {
  0x8e, 0x2c, 0xe0, 0xc1 // mvn gt r2 r14 LSL 25
};
const byte kInstruction_mvn_ge_r8_r1_LSL_17[] = {
  0x81, 0x88, 0xe0, 0xa1 // mvn ge r8 r1 LSL 17
};
const byte kInstruction_mvn_ne_r5_r7_ROR_19[] = {
  0xe7, 0x59, 0xe0, 0x11 // mvn ne r5 r7 ROR 19
};
const byte kInstruction_mvn_pl_r4_r9_LSL_20[] = {
  0x09, 0x4a, 0xe0, 0x51 // mvn pl r4 r9 LSL 20
};
const byte kInstruction_mvn_gt_r14_r1_LSL_18[] = {
  0x01, 0xe9, 0xe0, 0xc1 // mvn gt r14 r1 LSL 18
};
const byte kInstruction_mvn_ge_r1_r4_LSL_1[] = {
  0x84, 0x10, 0xe0, 0xa1 // mvn ge r1 r4 LSL 1
};
const byte kInstruction_mvn_cc_r14_r0_LSL_18[] = {
  0x00, 0xe9, 0xe0, 0x31 // mvn cc r14 r0 LSL 18
};
const byte kInstruction_mvn_vc_r12_r13_LSL_16[] = {
  0x0d, 0xc8, 0xe0, 0x71 // mvn vc r12 r13 LSL 16
};
const byte kInstruction_mvn_ls_r10_r8_LSL_15[] = {
  0x88, 0xa7, 0xe0, 0x91 // mvn ls r10 r8 LSL 15
};
const byte kInstruction_mvn_ne_r4_r4_ROR_22[] = {
  0x64, 0x4b, 0xe0, 0x11 // mvn ne r4 r4 ROR 22
};
const byte kInstruction_mvn_le_r4_r5_LSL_29[] = {
  0x85, 0x4e, 0xe0, 0xd1 // mvn le r4 r5 LSL 29
};
const byte kInstruction_mvn_hi_r10_r5_LSL_19[] = {
  0x85, 0xa9, 0xe0, 0x81 // mvn hi r10 r5 LSL 19
};
const byte kInstruction_mvn_gt_r6_r10_LSL_26[] = {
  0x0a, 0x6d, 0xe0, 0xc1 // mvn gt r6 r10 LSL 26
};
const byte kInstruction_mvn_lt_r3_r7_LSL_19[] = {
  0x87, 0x39, 0xe0, 0xb1 // mvn lt r3 r7 LSL 19
};
const byte kInstruction_mvn_vs_r9_r9_ROR_17[] = {
  0xe9, 0x98, 0xe0, 0x61 // mvn vs r9 r9 ROR 17
};
const byte kInstruction_mvn_vc_r0_r0_ROR_21[] = {
  0xe0, 0x0a, 0xe0, 0x71 // mvn vc r0 r0 ROR 21
};
const byte kInstruction_mvn_le_r5_r10_LSL_20[] = {
  0x0a, 0x5a, 0xe0, 0xd1 // mvn le r5 r10 LSL 20
};
const byte kInstruction_mvn_lt_r3_r6_ROR_26[] = {
  0x66, 0x3d, 0xe0, 0xb1 // mvn lt r3 r6 ROR 26
};
const byte kInstruction_mvn_vs_r0_r13_ROR_31[] = {
  0xed, 0x0f, 0xe0, 0x61 // mvn vs r0 r13 ROR 31
};
const byte kInstruction_mvn_le_r9_r7_ROR_18[] = {
  0x67, 0x99, 0xe0, 0xd1 // mvn le r9 r7 ROR 18
};
const byte kInstruction_mvn_le_r2_r8_LSL_10[] = {
  0x08, 0x25, 0xe0, 0xd1 // mvn le r2 r8 LSL 10
};
const byte kInstruction_mvn_eq_r0_r5_LSL_13[] = {
  0x85, 0x06, 0xe0, 0x01 // mvn eq r0 r5 LSL 13
};
const byte kInstruction_mvn_vs_r0_r4_ROR_3[] = {
  0xe4, 0x01, 0xe0, 0x61 // mvn vs r0 r4 ROR 3
};
const byte kInstruction_mvn_cs_r4_r6_LSL_16[] = {
  0x06, 0x48, 0xe0, 0x21 // mvn cs r4 r6 LSL 16
};
const byte kInstruction_mvn_hi_r5_r13_LSL_19[] = {
  0x8d, 0x59, 0xe0, 0x81 // mvn hi r5 r13 LSL 19
};
const byte kInstruction_mvn_cc_r2_r9_ROR_31[] = {
  0xe9, 0x2f, 0xe0, 0x31 // mvn cc r2 r9 ROR 31
};
const byte kInstruction_mvn_ls_r2_r11_ROR_7[] = {
  0xeb, 0x23, 0xe0, 0x91 // mvn ls r2 r11 ROR 7
};
const byte kInstruction_mvn_ne_r9_r6_LSL_21[] = {
  0x86, 0x9a, 0xe0, 0x11 // mvn ne r9 r6 LSL 21
};
const byte kInstruction_mvn_le_r7_r9_ROR_8[] = {
  0x69, 0x74, 0xe0, 0xd1 // mvn le r7 r9 ROR 8
};
const byte kInstruction_mvn_ls_r11_r13_LSL_2[] = {
  0x0d, 0xb1, 0xe0, 0x91 // mvn ls r11 r13 LSL 2
};
const byte kInstruction_mvn_cs_r6_r9_LSL_17[] = {
  0x89, 0x68, 0xe0, 0x21 // mvn cs r6 r9 LSL 17
};
const byte kInstruction_mvn_vs_r12_r1_ROR_21[] = {
  0xe1, 0xca, 0xe0, 0x61 // mvn vs r12 r1 ROR 21
};
const byte kInstruction_mvn_vc_r11_r10_ROR_27[] = {
  0xea, 0xbd, 0xe0, 0x71 // mvn vc r11 r10 ROR 27
};
const byte kInstruction_mvn_ge_r4_r14_LSL_9[] = {
  0x8e, 0x44, 0xe0, 0xa1 // mvn ge r4 r14 LSL 9
};
const byte kInstruction_mvn_eq_r0_r7_LSL_14[] = {
  0x07, 0x07, 0xe0, 0x01 // mvn eq r0 r7 LSL 14
};
const byte kInstruction_mvn_le_r1_r11_LSL_6[] = {
  0x0b, 0x13, 0xe0, 0xd1 // mvn le r1 r11 LSL 6
};
const byte kInstruction_mvn_eq_r8_r7_LSL_7[] = {
  0x87, 0x83, 0xe0, 0x01 // mvn eq r8 r7 LSL 7
};
const byte kInstruction_mvn_ge_r8_r7_ROR_2[] = {
  0x67, 0x81, 0xe0, 0xa1 // mvn ge r8 r7 ROR 2
};
const byte kInstruction_mvn_hi_r3_r4_LSL_17[] = {
  0x84, 0x38, 0xe0, 0x81 // mvn hi r3 r4 LSL 17
};
const byte kInstruction_mvn_mi_r8_r11_ROR_18[] = {
  0x6b, 0x89, 0xe0, 0x41 // mvn mi r8 r11 ROR 18
};
const byte kInstruction_mvn_pl_r4_r10_LSL_11[] = {
  0x8a, 0x45, 0xe0, 0x51 // mvn pl r4 r10 LSL 11
};
const byte kInstruction_mvn_pl_r10_r2_LSL_15[] = {
  0x82, 0xa7, 0xe0, 0x51 // mvn pl r10 r2 LSL 15
};
const byte kInstruction_mvn_ne_r3_r1_LSL_5[] = {
  0x81, 0x32, 0xe0, 0x11 // mvn ne r3 r1 LSL 5
};
const byte kInstruction_mvn_ls_r10_r13_LSL_3[] = {
  0x8d, 0xa1, 0xe0, 0x91 // mvn ls r10 r13 LSL 3
};
const byte kInstruction_mvn_vs_r8_r11_ROR_11[] = {
  0xeb, 0x85, 0xe0, 0x61 // mvn vs r8 r11 ROR 11
};
const byte kInstruction_mvn_gt_r13_r10_LSL_8[] = {
  0x0a, 0xd4, 0xe0, 0xc1 // mvn gt r13 r10 LSL 8
};
const byte kInstruction_mvn_hi_r8_r6_LSL_11[] = {
  0x86, 0x85, 0xe0, 0x81 // mvn hi r8 r6 LSL 11
};
const byte kInstruction_mvn_vc_r8_r3_ROR_13[] = {
  0xe3, 0x86, 0xe0, 0x71 // mvn vc r8 r3 ROR 13
};
const byte kInstruction_mvn_lt_r5_r14_ROR_5[] = {
  0xee, 0x52, 0xe0, 0xb1 // mvn lt r5 r14 ROR 5
};
const byte kInstruction_mvn_hi_r11_r4_ROR_23[] = {
  0xe4, 0xbb, 0xe0, 0x81 // mvn hi r11 r4 ROR 23
};
const byte kInstruction_mvn_al_r5_r6_ROR_3[] = {
  0xe6, 0x51, 0xe0, 0xe1 // mvn al r5 r6 ROR 3
};
const byte kInstruction_mvn_pl_r2_r2_LSL_10[] = {
  0x02, 0x25, 0xe0, 0x51 // mvn pl r2 r2 LSL 10
};
const byte kInstruction_mvn_mi_r0_r7_LSL_9[] = {
  0x87, 0x04, 0xe0, 0x41 // mvn mi r0 r7 LSL 9
};
const byte kInstruction_mvn_gt_r7_r5_LSL_11[] = {
  0x85, 0x75, 0xe0, 0xc1 // mvn gt r7 r5 LSL 11
};
const byte kInstruction_mvn_mi_r10_r11_ROR_18[] = {
  0x6b, 0xa9, 0xe0, 0x41 // mvn mi r10 r11 ROR 18
};
const byte kInstruction_mvn_cc_r7_r10_ROR_10[] = {
  0x6a, 0x75, 0xe0, 0x31 // mvn cc r7 r10 ROR 10
};
const byte kInstruction_mvn_hi_r8_r9_ROR_31[] = {
  0xe9, 0x8f, 0xe0, 0x81 // mvn hi r8 r9 ROR 31
};
const byte kInstruction_mvn_cs_r8_r12_LSL_16[] = {
  0x0c, 0x88, 0xe0, 0x21 // mvn cs r8 r12 LSL 16
};
const byte kInstruction_mvn_gt_r4_r0_LSL_11[] = {
  0x80, 0x45, 0xe0, 0xc1 // mvn gt r4 r0 LSL 11
};
const byte kInstruction_mvn_cc_r4_r12_LSL_3[] = {
  0x8c, 0x41, 0xe0, 0x31 // mvn cc r4 r12 LSL 3
};
const byte kInstruction_mvn_al_r13_r3_LSL_9[] = {
  0x83, 0xd4, 0xe0, 0xe1 // mvn al r13 r3 LSL 9
};
const byte kInstruction_mvn_cs_r0_r4_LSL_4[] = {
  0x04, 0x02, 0xe0, 0x21 // mvn cs r0 r4 LSL 4
};
const byte kInstruction_mvn_hi_r10_r14_LSL_8[] = {
  0x0e, 0xa4, 0xe0, 0x81 // mvn hi r10 r14 LSL 8
};
const byte kInstruction_mvn_ge_r14_r13_ROR_15[] = {
  0xed, 0xe7, 0xe0, 0xa1 // mvn ge r14 r13 ROR 15
};
const byte kInstruction_mvn_vc_r8_r8_LSL_25[] = {
  0x88, 0x8c, 0xe0, 0x71 // mvn vc r8 r8 LSL 25
};
const byte kInstruction_mvn_hi_r11_r3_ROR_23[] = {
  0xe3, 0xbb, 0xe0, 0x81 // mvn hi r11 r3 ROR 23
};
const byte kInstruction_mvn_al_r11_r7_LSL_14[] = {
  0x07, 0xb7, 0xe0, 0xe1 // mvn al r11 r7 LSL 14
};
const byte kInstruction_mvn_ls_r0_r2_LSL_21[] = {
  0x82, 0x0a, 0xe0, 0x91 // mvn ls r0 r2 LSL 21
};
const byte kInstruction_mvn_ne_r2_r3_LSL_24[] = {
  0x03, 0x2c, 0xe0, 0x11 // mvn ne r2 r3 LSL 24
};
const byte kInstruction_mvn_gt_r11_r14_ROR_5[] = {
  0xee, 0xb2, 0xe0, 0xc1 // mvn gt r11 r14 ROR 5
};
const byte kInstruction_mvn_hi_r9_r4_LSL_18[] = {
  0x04, 0x99, 0xe0, 0x81 // mvn hi r9 r4 LSL 18
};
const byte kInstruction_mvn_cs_r10_r6_LSL_3[] = {
  0x86, 0xa1, 0xe0, 0x21 // mvn cs r10 r6 LSL 3
};
const byte kInstruction_mvn_hi_r10_r12_LSL_28[] = {
  0x0c, 0xae, 0xe0, 0x81 // mvn hi r10 r12 LSL 28
};
const byte kInstruction_mvn_pl_r12_r14_LSL_4[] = {
  0x0e, 0xc2, 0xe0, 0x51 // mvn pl r12 r14 LSL 4
};
const byte kInstruction_mvn_ls_r12_r5_ROR_18[] = {
  0x65, 0xc9, 0xe0, 0x91 // mvn ls r12 r5 ROR 18
};
const byte kInstruction_mvn_eq_r0_r5_ROR_3[] = {
  0xe5, 0x01, 0xe0, 0x01 // mvn eq r0 r5 ROR 3
};
const byte kInstruction_mvn_cc_r4_r2_ROR_11[] = {
  0xe2, 0x45, 0xe0, 0x31 // mvn cc r4 r2 ROR 11
};
const byte kInstruction_mvn_lt_r4_r8_ROR_31[] = {
  0xe8, 0x4f, 0xe0, 0xb1 // mvn lt r4 r8 ROR 31
};
const byte kInstruction_mvn_cc_r9_r7_LSL_31[] = {
  0x87, 0x9f, 0xe0, 0x31 // mvn cc r9 r7 LSL 31
};
const byte kInstruction_mvn_cs_r4_r5_ROR_25[] = {
  0xe5, 0x4c, 0xe0, 0x21 // mvn cs r4 r5 ROR 25
};
const byte kInstruction_mvn_mi_r13_r10_LSL_14[] = {
  0x0a, 0xd7, 0xe0, 0x41 // mvn mi r13 r10 LSL 14
};
const byte kInstruction_mvn_gt_r9_r3_ROR_13[] = {
  0xe3, 0x96, 0xe0, 0xc1 // mvn gt r9 r3 ROR 13
};
const byte kInstruction_mvn_lt_r4_r11_LSL_1[] = {
  0x8b, 0x40, 0xe0, 0xb1 // mvn lt r4 r11 LSL 1
};
const byte kInstruction_mvn_le_r10_r4_LSL_19[] = {
  0x84, 0xa9, 0xe0, 0xd1 // mvn le r10 r4 LSL 19
};
const byte kInstruction_mvn_vc_r12_r10_ROR_14[] = {
  0x6a, 0xc7, 0xe0, 0x71 // mvn vc r12 r10 ROR 14
};
const byte kInstruction_mvn_eq_r0_r1_LSL_15[] = {
  0x81, 0x07, 0xe0, 0x01 // mvn eq r0 r1 LSL 15
};
const byte kInstruction_mvn_le_r14_r14_ROR_30[] = {
  0x6e, 0xef, 0xe0, 0xd1 // mvn le r14 r14 ROR 30
};
const byte kInstruction_mvn_gt_r13_r5_LSL_18[] = {
  0x05, 0xd9, 0xe0, 0xc1 // mvn gt r13 r5 LSL 18
};
const byte kInstruction_mvn_lt_r6_r11_LSL_23[] = {
  0x8b, 0x6b, 0xe0, 0xb1 // mvn lt r6 r11 LSL 23
};
const byte kInstruction_mvn_cc_r2_r1_ROR_17[] = {
  0xe1, 0x28, 0xe0, 0x31 // mvn cc r2 r1 ROR 17
};
const byte kInstruction_mvn_mi_r6_r3_LSL_22[] = {
  0x03, 0x6b, 0xe0, 0x41 // mvn mi r6 r3 LSL 22
};
const byte kInstruction_mvn_al_r3_r1_ROR_9[] = {
  0xe1, 0x34, 0xe0, 0xe1 // mvn al r3 r1 ROR 9
};
const byte kInstruction_mvn_hi_r0_r12_LSL_5[] = {
  0x8c, 0x02, 0xe0, 0x81 // mvn hi r0 r12 LSL 5
};
const byte kInstruction_mvn_eq_r6_r0_ROR_11[] = {
  0xe0, 0x65, 0xe0, 0x01 // mvn eq r6 r0 ROR 11
};
const byte kInstruction_mvn_mi_r10_r7_LSL_26[] = {
  0x07, 0xad, 0xe0, 0x41 // mvn mi r10 r7 LSL 26
};
const byte kInstruction_mvn_ne_r7_r3_ROR_12[] = {
  0x63, 0x76, 0xe0, 0x11 // mvn ne r7 r3 ROR 12
};
const byte kInstruction_mvn_le_r0_r8_ROR_30[] = {
  0x68, 0x0f, 0xe0, 0xd1 // mvn le r0 r8 ROR 30
};
const byte kInstruction_mvn_gt_r7_r1_ROR_17[] = {
  0xe1, 0x78, 0xe0, 0xc1 // mvn gt r7 r1 ROR 17
};
const byte kInstruction_mvn_al_r0_r6_ROR_8[] = {
  0x66, 0x04, 0xe0, 0xe1 // mvn al r0 r6 ROR 8
};
const byte kInstruction_mvn_vc_r13_r13_ROR_17[] = {
  0xed, 0xd8, 0xe0, 0x71 // mvn vc r13 r13 ROR 17
};
const byte kInstruction_mvn_ge_r2_r9_ROR_25[] = {
  0xe9, 0x2c, 0xe0, 0xa1 // mvn ge r2 r9 ROR 25
};
const byte kInstruction_mvn_lt_r11_r3_ROR_27[] = {
  0xe3, 0xbd, 0xe0, 0xb1 // mvn lt r11 r3 ROR 27
};
const byte kInstruction_mvn_hi_r0_r3_ROR_14[] = {
  0x63, 0x07, 0xe0, 0x81 // mvn hi r0 r3 ROR 14
};
const byte kInstruction_mvn_vc_r13_r1_ROR_18[] = {
  0x61, 0xd9, 0xe0, 0x71 // mvn vc r13 r1 ROR 18
};
const byte kInstruction_mvn_eq_r1_r13_LSL_31[] = {
  0x8d, 0x1f, 0xe0, 0x01 // mvn eq r1 r13 LSL 31
};
const byte kInstruction_mvn_mi_r12_r9_ROR_22[] = {
  0x69, 0xcb, 0xe0, 0x41 // mvn mi r12 r9 ROR 22
};
const byte kInstruction_mvn_al_r5_r11_ROR_31[] = {
  0xeb, 0x5f, 0xe0, 0xe1 // mvn al r5 r11 ROR 31
};
const byte kInstruction_mvn_ne_r7_r4_ROR_20[] = {
  0x64, 0x7a, 0xe0, 0x11 // mvn ne r7 r4 ROR 20
};
const byte kInstruction_mvn_hi_r0_r1_LSL_10[] = {
  0x01, 0x05, 0xe0, 0x81 // mvn hi r0 r1 LSL 10
};
const byte kInstruction_mvn_ne_r5_r12_ROR_6[] = {
  0x6c, 0x53, 0xe0, 0x11 // mvn ne r5 r12 ROR 6
};
const byte kInstruction_mvn_hi_r14_r14_LSL_6[] = {
  0x0e, 0xe3, 0xe0, 0x81 // mvn hi r14 r14 LSL 6
};
const byte kInstruction_mvn_pl_r12_r9_ROR_28[] = {
  0x69, 0xce, 0xe0, 0x51 // mvn pl r12 r9 ROR 28
};
const byte kInstruction_mvn_al_r7_r2_LSL_2[] = {
  0x02, 0x71, 0xe0, 0xe1 // mvn al r7 r2 LSL 2
};
const byte kInstruction_mvn_le_r12_r9_LSL_5[] = {
  0x89, 0xc2, 0xe0, 0xd1 // mvn le r12 r9 LSL 5
};
const byte kInstruction_mvn_lt_r0_r6_ROR_1[] = {
  0xe6, 0x00, 0xe0, 0xb1 // mvn lt r0 r6 ROR 1
};
const byte kInstruction_mvn_le_r3_r4_LSL_14[] = {
  0x04, 0x37, 0xe0, 0xd1 // mvn le r3 r4 LSL 14
};
const byte kInstruction_mvn_ge_r1_r10_LSL_3[] = {
  0x8a, 0x11, 0xe0, 0xa1 // mvn ge r1 r10 LSL 3
};
const byte kInstruction_mvn_ne_r12_r14_ROR_6[] = {
  0x6e, 0xc3, 0xe0, 0x11 // mvn ne r12 r14 ROR 6
};
const byte kInstruction_mvn_ne_r11_r7_LSL_27[] = {
  0x87, 0xbd, 0xe0, 0x11 // mvn ne r11 r7 LSL 27
};
const byte kInstruction_mvn_mi_r4_r2_LSL_30[] = {
  0x02, 0x4f, 0xe0, 0x41 // mvn mi r4 r2 LSL 30
};
const byte kInstruction_mvn_ls_r8_r2_LSL_20[] = {
  0x02, 0x8a, 0xe0, 0x91 // mvn ls r8 r2 LSL 20
};
const byte kInstruction_mvn_ge_r1_r5_ROR_21[] = {
  0xe5, 0x1a, 0xe0, 0xa1 // mvn ge r1 r5 ROR 21
};
const byte kInstruction_mvn_vs_r3_r10_LSL_13[] = {
  0x8a, 0x36, 0xe0, 0x61 // mvn vs r3 r10 LSL 13
};
const byte kInstruction_mvn_ge_r7_r6_ROR_2[] = {
  0x66, 0x71, 0xe0, 0xa1 // mvn ge r7 r6 ROR 2
};
const byte kInstruction_mvn_ls_r13_r12_ROR_5[] = {
  0xec, 0xd2, 0xe0, 0x91 // mvn ls r13 r12 ROR 5
};
const byte kInstruction_mvn_eq_r9_r10_LSL_19[] = {
  0x8a, 0x99, 0xe0, 0x01 // mvn eq r9 r10 LSL 19
};
const byte kInstruction_mvn_le_r5_r7_ROR_6[] = {
  0x67, 0x53, 0xe0, 0xd1 // mvn le r5 r7 ROR 6
};
const byte kInstruction_mvn_eq_r11_r5_LSL_8[] = {
  0x05, 0xb4, 0xe0, 0x01 // mvn eq r11 r5 LSL 8
};
const byte kInstruction_mvn_mi_r6_r5_ROR_31[] = {
  0xe5, 0x6f, 0xe0, 0x41 // mvn mi r6 r5 ROR 31
};
const byte kInstruction_mvn_cc_r13_r11_ROR_8[] = {
  0x6b, 0xd4, 0xe0, 0x31 // mvn cc r13 r11 ROR 8
};
const byte kInstruction_mvn_mi_r0_r14_LSL_13[] = {
  0x8e, 0x06, 0xe0, 0x41 // mvn mi r0 r14 LSL 13
};
const byte kInstruction_mvn_lt_r7_r12_LSL_2[] = {
  0x0c, 0x71, 0xe0, 0xb1 // mvn lt r7 r12 LSL 2
};
const byte kInstruction_mvn_eq_r6_r8_LSL_22[] = {
  0x08, 0x6b, 0xe0, 0x01 // mvn eq r6 r8 LSL 22
};
const byte kInstruction_mvn_lt_r13_r8_LSL_26[] = {
  0x08, 0xdd, 0xe0, 0xb1 // mvn lt r13 r8 LSL 26
};
const byte kInstruction_mvn_gt_r5_r7_ROR_18[] = {
  0x67, 0x59, 0xe0, 0xc1 // mvn gt r5 r7 ROR 18
};
const byte kInstruction_mvn_al_r9_r4_LSL_25[] = {
  0x84, 0x9c, 0xe0, 0xe1 // mvn al r9 r4 LSL 25
};
const byte kInstruction_mvn_gt_r8_r12_ROR_23[] = {
  0xec, 0x8b, 0xe0, 0xc1 // mvn gt r8 r12 ROR 23
};
const byte kInstruction_mvn_eq_r5_r6_ROR_23[] = {
  0xe6, 0x5b, 0xe0, 0x01 // mvn eq r5 r6 ROR 23
};
const byte kInstruction_mvn_ge_r0_r4_ROR_14[] = {
  0x64, 0x07, 0xe0, 0xa1 // mvn ge r0 r4 ROR 14
};
const byte kInstruction_mvn_le_r1_r1_ROR_12[] = {
  0x61, 0x16, 0xe0, 0xd1 // mvn le r1 r1 ROR 12
};
const byte kInstruction_mvn_le_r12_r12_LSL_29[] = {
  0x8c, 0xce, 0xe0, 0xd1 // mvn le r12 r12 LSL 29
};
const byte kInstruction_mvn_ls_r2_r9_LSL_7[] = {
  0x89, 0x23, 0xe0, 0x91 // mvn ls r2 r9 LSL 7
};
const byte kInstruction_mvn_pl_r4_r12_ROR_10[] = {
  0x6c, 0x45, 0xe0, 0x51 // mvn pl r4 r12 ROR 10
};
const byte kInstruction_mvn_ge_r4_r0_LSL_1[] = {
  0x80, 0x40, 0xe0, 0xa1 // mvn ge r4 r0 LSL 1
};
const byte kInstruction_mvn_mi_r9_r6_LSL_14[] = {
  0x06, 0x97, 0xe0, 0x41 // mvn mi r9 r6 LSL 14
};
const byte kInstruction_mvn_vc_r14_r10_LSL_8[] = {
  0x0a, 0xe4, 0xe0, 0x71 // mvn vc r14 r10 LSL 8
};
const byte kInstruction_mvn_ls_r9_r10_LSL_18[] = {
  0x0a, 0x99, 0xe0, 0x91 // mvn ls r9 r10 LSL 18
};
const byte kInstruction_mvn_cc_r9_r4_ROR_19[] = {
  0xe4, 0x99, 0xe0, 0x31 // mvn cc r9 r4 ROR 19
};
const byte kInstruction_mvn_ne_r10_r2_ROR_5[] = {
  0xe2, 0xa2, 0xe0, 0x11 // mvn ne r10 r2 ROR 5
};
const byte kInstruction_mvn_cc_r14_r7_ROR_31[] = {
  0xe7, 0xef, 0xe0, 0x31 // mvn cc r14 r7 ROR 31
};
const byte kInstruction_mvn_hi_r4_r7_LSL_7[] = {
  0x87, 0x43, 0xe0, 0x81 // mvn hi r4 r7 LSL 7
};
const byte kInstruction_mvn_mi_r5_r9_LSL_14[] = {
  0x09, 0x57, 0xe0, 0x41 // mvn mi r5 r9 LSL 14
};
const byte kInstruction_mvn_cs_r7_r11_ROR_4[] = {
  0x6b, 0x72, 0xe0, 0x21 // mvn cs r7 r11 ROR 4
};
const byte kInstruction_mvn_cc_r9_r8_ROR_9[] = {
  0xe8, 0x94, 0xe0, 0x31 // mvn cc r9 r8 ROR 9
};
const byte kInstruction_mvn_hi_r1_r11_LSL_12[] = {
  0x0b, 0x16, 0xe0, 0x81 // mvn hi r1 r11 LSL 12
};
const byte kInstruction_mvn_pl_r8_r12_ROR_19[] = {
  0xec, 0x89, 0xe0, 0x51 // mvn pl r8 r12 ROR 19
};
const byte kInstruction_mvn_gt_r4_r3_ROR_30[] = {
  0x63, 0x4f, 0xe0, 0xc1 // mvn gt r4 r3 ROR 30
};
const byte kInstruction_mvn_le_r8_r10_ROR_2[] = {
  0x6a, 0x81, 0xe0, 0xd1 // mvn le r8 r10 ROR 2
};
const byte kInstruction_mvn_gt_r2_r1_ROR_1[] = {
  0xe1, 0x20, 0xe0, 0xc1 // mvn gt r2 r1 ROR 1
};
const byte kInstruction_mvn_mi_r7_r5_LSL_11[] = {
  0x85, 0x75, 0xe0, 0x41 // mvn mi r7 r5 LSL 11
};
const byte kInstruction_mvn_eq_r10_r12_ROR_26[] = {
  0x6c, 0xad, 0xe0, 0x01 // mvn eq r10 r12 ROR 26
};
const byte kInstruction_mvn_le_r1_r3_ROR_29[] = {
  0xe3, 0x1e, 0xe0, 0xd1 // mvn le r1 r3 ROR 29
};
const byte kInstruction_mvn_lt_r3_r8_ROR_30[] = {
  0x68, 0x3f, 0xe0, 0xb1 // mvn lt r3 r8 ROR 30
};
const byte kInstruction_mvn_ls_r1_r9_ROR_11[] = {
  0xe9, 0x15, 0xe0, 0x91 // mvn ls r1 r9 ROR 11
};
const byte kInstruction_mvn_vs_r14_r12_ROR_23[] = {
  0xec, 0xeb, 0xe0, 0x61 // mvn vs r14 r12 ROR 23
};
const byte kInstruction_mvn_ge_r9_r14_LSL_4[] = {
  0x0e, 0x92, 0xe0, 0xa1 // mvn ge r9 r14 LSL 4
};
const byte kInstruction_mvn_cc_r1_r8_ROR_2[] = {
  0x68, 0x11, 0xe0, 0x31 // mvn cc r1 r8 ROR 2
};
const byte kInstruction_mvn_ge_r14_r10_ROR_28[] = {
  0x6a, 0xee, 0xe0, 0xa1 // mvn ge r14 r10 ROR 28
};
const byte kInstruction_mvn_cc_r8_r13_ROR_31[] = {
  0xed, 0x8f, 0xe0, 0x31 // mvn cc r8 r13 ROR 31
};
const byte kInstruction_mvn_ls_r10_r8_LSL_28[] = {
  0x08, 0xae, 0xe0, 0x91 // mvn ls r10 r8 LSL 28
};
const byte kInstruction_mvn_ge_r10_r1_LSL_1[] = {
  0x81, 0xa0, 0xe0, 0xa1 // mvn ge r10 r1 LSL 1
};
const byte kInstruction_mvn_vs_r9_r9_ROR_19[] = {
  0xe9, 0x99, 0xe0, 0x61 // mvn vs r9 r9 ROR 19
};
const byte kInstruction_mvn_ne_r6_r7_ROR_19[] = {
  0xe7, 0x69, 0xe0, 0x11 // mvn ne r6 r7 ROR 19
};
const byte kInstruction_mvn_cs_r13_r9_ROR_21[] = {
  0xe9, 0xda, 0xe0, 0x21 // mvn cs r13 r9 ROR 21
};
const byte kInstruction_mvn_ls_r13_r14_ROR_10[] = {
  0x6e, 0xd5, 0xe0, 0x91 // mvn ls r13 r14 ROR 10
};
const byte kInstruction_mvn_hi_r2_r8_LSL_29[] = {
  0x88, 0x2e, 0xe0, 0x81 // mvn hi r2 r8 LSL 29
};
const byte kInstruction_mvn_le_r4_r2_LSL_27[] = {
  0x82, 0x4d, 0xe0, 0xd1 // mvn le r4 r2 LSL 27
};
const byte kInstruction_mvn_gt_r2_r10_ROR_6[] = {
  0x6a, 0x23, 0xe0, 0xc1 // mvn gt r2 r10 ROR 6
};
const byte kInstruction_mvn_hi_r3_r7_ROR_4[] = {
  0x67, 0x32, 0xe0, 0x81 // mvn hi r3 r7 ROR 4
};
const byte kInstruction_mvn_hi_r12_r9_LSL_14[] = {
  0x09, 0xc7, 0xe0, 0x81 // mvn hi r12 r9 LSL 14
};
const byte kInstruction_mvn_lt_r9_r11_ROR_31[] = {
  0xeb, 0x9f, 0xe0, 0xb1 // mvn lt r9 r11 ROR 31
};
const byte kInstruction_mvn_ls_r3_r1_ROR_18[] = {
  0x61, 0x39, 0xe0, 0x91 // mvn ls r3 r1 ROR 18
};
const byte kInstruction_mvn_al_r0_r9_ROR_17[] = {
  0xe9, 0x08, 0xe0, 0xe1 // mvn al r0 r9 ROR 17
};
const byte kInstruction_mvn_cc_r0_r7_LSL_14[] = {
  0x07, 0x07, 0xe0, 0x31 // mvn cc r0 r7 LSL 14
};
const byte kInstruction_mvn_al_r10_r12_LSL_17[] = {
  0x8c, 0xa8, 0xe0, 0xe1 // mvn al r10 r12 LSL 17
};
const byte kInstruction_mvn_lt_r1_r14_LSL_22[] = {
  0x0e, 0x1b, 0xe0, 0xb1 // mvn lt r1 r14 LSL 22
};
const byte kInstruction_mvn_ge_r1_r7_ROR_1[] = {
  0xe7, 0x10, 0xe0, 0xa1 // mvn ge r1 r7 ROR 1
};
const byte kInstruction_mvn_mi_r1_r1_LSL_30[] = {
  0x01, 0x1f, 0xe0, 0x41 // mvn mi r1 r1 LSL 30
};
const byte kInstruction_mvn_cc_r7_r2_LSL_19[] = {
  0x82, 0x79, 0xe0, 0x31 // mvn cc r7 r2 LSL 19
};
const byte kInstruction_mvn_vs_r14_r3_LSL_3[] = {
  0x83, 0xe1, 0xe0, 0x61 // mvn vs r14 r3 LSL 3
};
const byte kInstruction_mvn_ls_r5_r1_ROR_30[] = {
  0x61, 0x5f, 0xe0, 0x91 // mvn ls r5 r1 ROR 30
};
const byte kInstruction_mvn_al_r4_r4_LSL_29[] = {
  0x84, 0x4e, 0xe0, 0xe1 // mvn al r4 r4 LSL 29
};
const byte kInstruction_mvn_mi_r7_r3_ROR_25[] = {
  0xe3, 0x7c, 0xe0, 0x41 // mvn mi r7 r3 ROR 25
};
const byte kInstruction_mvn_eq_r9_r4_ROR_2[] = {
  0x64, 0x91, 0xe0, 0x01 // mvn eq r9 r4 ROR 2
};
const byte kInstruction_mvn_vc_r7_r13_ROR_9[] = {
  0xed, 0x74, 0xe0, 0x71 // mvn vc r7 r13 ROR 9
};
const byte kInstruction_mvn_eq_r7_r1_ROR_21[] = {
  0xe1, 0x7a, 0xe0, 0x01 // mvn eq r7 r1 ROR 21
};
const byte kInstruction_mvn_ge_r2_r6_ROR_12[] = {
  0x66, 0x26, 0xe0, 0xa1 // mvn ge r2 r6 ROR 12
};
const byte kInstruction_mvn_ge_r2_r7_LSL_10[] = {
  0x07, 0x25, 0xe0, 0xa1 // mvn ge r2 r7 LSL 10
};
const byte kInstruction_mvn_ne_r0_r14_LSL_19[] = {
  0x8e, 0x09, 0xe0, 0x11 // mvn ne r0 r14 LSL 19
};
const byte kInstruction_mvn_ge_r0_r7_LSL_19[] = {
  0x87, 0x09, 0xe0, 0xa1 // mvn ge r0 r7 LSL 19
};
const byte kInstruction_mvn_cc_r9_r12_LSL_3[] = {
  0x8c, 0x91, 0xe0, 0x31 // mvn cc r9 r12 LSL 3
};
const byte kInstruction_mvn_gt_r7_r13_ROR_23[] = {
  0xed, 0x7b, 0xe0, 0xc1 // mvn gt r7 r13 ROR 23
};
const byte kInstruction_mvn_mi_r2_r6_LSL_19[] = {
  0x86, 0x29, 0xe0, 0x41 // mvn mi r2 r6 LSL 19
};
const byte kInstruction_mvn_ne_r11_r6_ROR_14[] = {
  0x66, 0xb7, 0xe0, 0x11 // mvn ne r11 r6 ROR 14
};
const byte kInstruction_mvn_ls_r12_r2_LSL_6[] = {
  0x02, 0xc3, 0xe0, 0x91 // mvn ls r12 r2 LSL 6
};
const byte kInstruction_mvn_cc_r4_r14_ROR_11[] = {
  0xee, 0x45, 0xe0, 0x31 // mvn cc r4 r14 ROR 11
};
const byte kInstruction_mvn_lt_r12_r10_LSL_20[] = {
  0x0a, 0xca, 0xe0, 0xb1 // mvn lt r12 r10 LSL 20
};
const byte kInstruction_mvn_le_r10_r14_LSL_3[] = {
  0x8e, 0xa1, 0xe0, 0xd1 // mvn le r10 r14 LSL 3
};
const byte kInstruction_mvn_al_r14_r9_LSL_28[] = {
  0x09, 0xee, 0xe0, 0xe1 // mvn al r14 r9 LSL 28
};
const byte kInstruction_mvn_ls_r9_r3_LSL_17[] = {
  0x83, 0x98, 0xe0, 0x91 // mvn ls r9 r3 LSL 17
};
const byte kInstruction_mvn_eq_r10_r2_ROR_25[] = {
  0xe2, 0xac, 0xe0, 0x01 // mvn eq r10 r2 ROR 25
};
const byte kInstruction_mvn_lt_r6_r7_ROR_2[] = {
  0x67, 0x61, 0xe0, 0xb1 // mvn lt r6 r7 ROR 2
};
const byte kInstruction_mvn_le_r5_r7_ROR_21[] = {
  0xe7, 0x5a, 0xe0, 0xd1 // mvn le r5 r7 ROR 21
};
const byte kInstruction_mvn_eq_r14_r2_LSL_31[] = {
  0x82, 0xef, 0xe0, 0x01 // mvn eq r14 r2 LSL 31
};
const byte kInstruction_mvn_hi_r1_r8_LSL_20[] = {
  0x08, 0x1a, 0xe0, 0x81 // mvn hi r1 r8 LSL 20
};
const byte kInstruction_mvn_hi_r11_r0_ROR_7[] = {
  0xe0, 0xb3, 0xe0, 0x81 // mvn hi r11 r0 ROR 7
};
const byte kInstruction_mvn_lt_r0_r4_ROR_1[] = {
  0xe4, 0x00, 0xe0, 0xb1 // mvn lt r0 r4 ROR 1
};
const byte kInstruction_mvn_ge_r0_r2_ROR_1[] = {
  0xe2, 0x00, 0xe0, 0xa1 // mvn ge r0 r2 ROR 1
};
const byte kInstruction_mvn_ge_r3_r11_ROR_1[] = {
  0xeb, 0x30, 0xe0, 0xa1 // mvn ge r3 r11 ROR 1
};
const byte kInstruction_mvn_le_r4_r12_LSL_27[] = {
  0x8c, 0x4d, 0xe0, 0xd1 // mvn le r4 r12 LSL 27
};
const byte kInstruction_mvn_al_r12_r2_ROR_6[] = {
  0x62, 0xc3, 0xe0, 0xe1 // mvn al r12 r2 ROR 6
};
const byte kInstruction_mvn_al_r11_r13_LSL_2[] = {
  0x0d, 0xb1, 0xe0, 0xe1 // mvn al r11 r13 LSL 2
};
const byte kInstruction_mvn_lt_r6_r8_LSL_3[] = {
  0x88, 0x61, 0xe0, 0xb1 // mvn lt r6 r8 LSL 3
};
const byte kInstruction_mvn_ls_r1_r4_LSL_21[] = {
  0x84, 0x1a, 0xe0, 0x91 // mvn ls r1 r4 LSL 21
};
const byte kInstruction_mvn_al_r12_r13_ROR_7[] = {
  0xed, 0xc3, 0xe0, 0xe1 // mvn al r12 r13 ROR 7
};
const byte kInstruction_mvn_gt_r5_r14_LSL_8[] = {
  0x0e, 0x54, 0xe0, 0xc1 // mvn gt r5 r14 LSL 8
};
const byte kInstruction_mvn_vc_r6_r8_ROR_4[] = {
  0x68, 0x62, 0xe0, 0x71 // mvn vc r6 r8 ROR 4
};
const byte kInstruction_mvn_vc_r0_r14_LSL_1[] = {
  0x8e, 0x00, 0xe0, 0x71 // mvn vc r0 r14 LSL 1
};
const byte kInstruction_mvn_ge_r5_r0_LSL_18[] = {
  0x00, 0x59, 0xe0, 0xa1 // mvn ge r5 r0 LSL 18
};
const byte kInstruction_mvn_lt_r6_r6_ROR_28[] = {
  0x66, 0x6e, 0xe0, 0xb1 // mvn lt r6 r6 ROR 28
};
const byte kInstruction_mvn_lt_r4_r14_LSL_2[] = {
  0x0e, 0x41, 0xe0, 0xb1 // mvn lt r4 r14 LSL 2
};
const byte kInstruction_mvn_lt_r8_r9_ROR_21[] = {
  0xe9, 0x8a, 0xe0, 0xb1 // mvn lt r8 r9 ROR 21
};
const byte kInstruction_mvn_cc_r6_r12_LSL_27[] = {
  0x8c, 0x6d, 0xe0, 0x31 // mvn cc r6 r12 LSL 27
};
const byte kInstruction_mvn_gt_r4_r11_LSL_4[] = {
  0x0b, 0x42, 0xe0, 0xc1 // mvn gt r4 r11 LSL 4
};
const byte kInstruction_mvn_pl_r13_r8_LSL_4[] = {
  0x08, 0xd2, 0xe0, 0x51 // mvn pl r13 r8 LSL 4
};
const byte kInstruction_mvn_eq_r5_r8_LSL_16[] = {
  0x08, 0x58, 0xe0, 0x01 // mvn eq r5 r8 LSL 16
};
const byte kInstruction_mvn_al_r5_r1_ROR_28[] = {
  0x61, 0x5e, 0xe0, 0xe1 // mvn al r5 r1 ROR 28
};
const byte kInstruction_mvn_lt_r2_r12_LSL_9[] = {
  0x8c, 0x24, 0xe0, 0xb1 // mvn lt r2 r12 LSL 9
};
const byte kInstruction_mvn_vc_r10_r4_LSL_28[] = {
  0x04, 0xae, 0xe0, 0x71 // mvn vc r10 r4 LSL 28
};
const byte kInstruction_mvn_hi_r0_r4_ROR_23[] = {
  0xe4, 0x0b, 0xe0, 0x81 // mvn hi r0 r4 ROR 23
};
const byte kInstruction_mvn_gt_r14_r1_ROR_16[] = {
  0x61, 0xe8, 0xe0, 0xc1 // mvn gt r14 r1 ROR 16
};
const byte kInstruction_mvn_ne_r14_r2_ROR_24[] = {
  0x62, 0xec, 0xe0, 0x11 // mvn ne r14 r2 ROR 24
};
const byte kInstruction_mvn_ls_r3_r4_LSL_18[] = {
  0x04, 0x39, 0xe0, 0x91 // mvn ls r3 r4 LSL 18
};
const byte kInstruction_mvn_mi_r9_r6_LSL_13[] = {
  0x86, 0x96, 0xe0, 0x41 // mvn mi r9 r6 LSL 13
};
const byte kInstruction_mvn_ge_r3_r10_ROR_10[] = {
  0x6a, 0x35, 0xe0, 0xa1 // mvn ge r3 r10 ROR 10
};
const byte kInstruction_mvn_ge_r5_r7_LSL_27[] = {
  0x87, 0x5d, 0xe0, 0xa1 // mvn ge r5 r7 LSL 27
};
const byte kInstruction_mvn_vc_r6_r0_ROR_26[] = {
  0x60, 0x6d, 0xe0, 0x71 // mvn vc r6 r0 ROR 26
};
const byte kInstruction_mvn_ne_r11_r3_ROR_6[] = {
  0x63, 0xb3, 0xe0, 0x11 // mvn ne r11 r3 ROR 6
};
const byte kInstruction_mvn_gt_r3_r4_ROR_17[] = {
  0xe4, 0x38, 0xe0, 0xc1 // mvn gt r3 r4 ROR 17
};
const byte kInstruction_mvn_lt_r1_r14_ROR_25[] = {
  0xee, 0x1c, 0xe0, 0xb1 // mvn lt r1 r14 ROR 25
};
const byte kInstruction_mvn_eq_r11_r13_LSL_27[] = {
  0x8d, 0xbd, 0xe0, 0x01 // mvn eq r11 r13 LSL 27
};
const byte kInstruction_mvn_cs_r6_r1_ROR_27[] = {
  0xe1, 0x6d, 0xe0, 0x21 // mvn cs r6 r1 ROR 27
};
const byte kInstruction_mvn_vc_r14_r14_LSL_31[] = {
  0x8e, 0xef, 0xe0, 0x71 // mvn vc r14 r14 LSL 31
};
const byte kInstruction_mvn_vc_r7_r2_LSL_8[] = {
  0x02, 0x74, 0xe0, 0x71 // mvn vc r7 r2 LSL 8
};
const byte kInstruction_mvn_eq_r3_r1_ROR_4[] = {
  0x61, 0x32, 0xe0, 0x01 // mvn eq r3 r1 ROR 4
};
const byte kInstruction_mvn_mi_r13_r14_LSL_12[] = {
  0x0e, 0xd6, 0xe0, 0x41 // mvn mi r13 r14 LSL 12
};
const byte kInstruction_mvn_vc_r10_r14_ROR_28[] = {
  0x6e, 0xae, 0xe0, 0x71 // mvn vc r10 r14 ROR 28
};
const byte kInstruction_mvn_cc_r13_r0_LSL_17[] = {
  0x80, 0xd8, 0xe0, 0x31 // mvn cc r13 r0 LSL 17
};
const byte kInstruction_mvn_vs_r14_r8_LSL_23[] = {
  0x88, 0xeb, 0xe0, 0x61 // mvn vs r14 r8 LSL 23
};
const byte kInstruction_mvn_cc_r14_r10_LSL_16[] = {
  0x0a, 0xe8, 0xe0, 0x31 // mvn cc r14 r10 LSL 16
};
const byte kInstruction_mvn_mi_r8_r7_ROR_23[] = {
  0xe7, 0x8b, 0xe0, 0x41 // mvn mi r8 r7 ROR 23
};
const byte kInstruction_mvn_ge_r5_r12_ROR_10[] = {
  0x6c, 0x55, 0xe0, 0xa1 // mvn ge r5 r12 ROR 10
};
const byte kInstruction_mvn_hi_r13_r5_LSL_19[] = {
  0x85, 0xd9, 0xe0, 0x81 // mvn hi r13 r5 LSL 19
};
const byte kInstruction_mvn_hi_r11_r12_LSL_21[] = {
  0x8c, 0xba, 0xe0, 0x81 // mvn hi r11 r12 LSL 21
};
const byte kInstruction_mvn_cc_r1_r1_LSL_9[] = {
  0x81, 0x14, 0xe0, 0x31 // mvn cc r1 r1 LSL 9
};
const byte kInstruction_mvn_ge_r14_r13_LSL_8[] = {
  0x0d, 0xe4, 0xe0, 0xa1 // mvn ge r14 r13 LSL 8
};
const byte kInstruction_mvn_le_r2_r0_LSL_31[] = {
  0x80, 0x2f, 0xe0, 0xd1 // mvn le r2 r0 LSL 31
};
const byte kInstruction_mvn_gt_r9_r3_LSL_19[] = {
  0x83, 0x99, 0xe0, 0xc1 // mvn gt r9 r3 LSL 19
};
const byte kInstruction_mvn_ls_r9_r3_LSL_22[] = {
  0x03, 0x9b, 0xe0, 0x91 // mvn ls r9 r3 LSL 22
};
const byte kInstruction_mvn_lt_r10_r9_ROR_13[] = {
  0xe9, 0xa6, 0xe0, 0xb1 // mvn lt r10 r9 ROR 13
};
const byte kInstruction_mvn_lt_r1_r8_ROR_11[] = {
  0xe8, 0x15, 0xe0, 0xb1 // mvn lt r1 r8 ROR 11
};
const byte kInstruction_mvn_eq_r1_r2_LSL_3[] = {
  0x82, 0x11, 0xe0, 0x01 // mvn eq r1 r2 LSL 3
};
const byte kInstruction_mvn_eq_r1_r6_LSL_16[] = {
  0x06, 0x18, 0xe0, 0x01 // mvn eq r1 r6 LSL 16
};
const byte kInstruction_mvn_ne_r3_r2_ROR_28[] = {
  0x62, 0x3e, 0xe0, 0x11 // mvn ne r3 r2 ROR 28
};
const byte kInstruction_mvn_ge_r7_r5_ROR_6[] = {
  0x65, 0x73, 0xe0, 0xa1 // mvn ge r7 r5 ROR 6
};
const byte kInstruction_mvn_ne_r3_r1_ROR_12[] = {
  0x61, 0x36, 0xe0, 0x11 // mvn ne r3 r1 ROR 12
};
const byte kInstruction_mvn_gt_r12_r3_ROR_13[] = {
  0xe3, 0xc6, 0xe0, 0xc1 // mvn gt r12 r3 ROR 13
};
const byte kInstruction_mvn_gt_r4_r6_ROR_13[] = {
  0xe6, 0x46, 0xe0, 0xc1 // mvn gt r4 r6 ROR 13
};
const byte kInstruction_mvn_pl_r11_r14_ROR_7[] = {
  0xee, 0xb3, 0xe0, 0x51 // mvn pl r11 r14 ROR 7
};
const byte kInstruction_mvn_gt_r3_r4_LSL_9[] = {
  0x84, 0x34, 0xe0, 0xc1 // mvn gt r3 r4 LSL 9
};
const byte kInstruction_mvn_vc_r14_r14_ROR_6[] = {
  0x6e, 0xe3, 0xe0, 0x71 // mvn vc r14 r14 ROR 6
};
const byte kInstruction_mvn_ne_r1_r14_LSL_26[] = {
  0x0e, 0x1d, 0xe0, 0x11 // mvn ne r1 r14 LSL 26
};
const byte kInstruction_mvn_ge_r4_r0_ROR_4[] = {
  0x60, 0x42, 0xe0, 0xa1 // mvn ge r4 r0 ROR 4
};
const byte kInstruction_mvn_pl_r13_r3_LSL_8[] = {
  0x03, 0xd4, 0xe0, 0x51 // mvn pl r13 r3 LSL 8
};
const byte kInstruction_mvn_vs_r11_r11_ROR_26[] = {
  0x6b, 0xbd, 0xe0, 0x61 // mvn vs r11 r11 ROR 26
};
const byte kInstruction_mvn_le_r9_r3_LSL_23[] = {
  0x83, 0x9b, 0xe0, 0xd1 // mvn le r9 r3 LSL 23
};
const byte kInstruction_mvn_mi_r10_r14_LSL_25[] = {
  0x8e, 0xac, 0xe0, 0x41 // mvn mi r10 r14 LSL 25
};
const byte kInstruction_mvn_mi_r13_r3_LSL_16[] = {
  0x03, 0xd8, 0xe0, 0x41 // mvn mi r13 r3 LSL 16
};
const byte kInstruction_mvn_eq_r5_r5_ROR_8[] = {
  0x65, 0x54, 0xe0, 0x01 // mvn eq r5 r5 ROR 8
};
const byte kInstruction_mvn_lt_r11_r6_ROR_8[] = {
  0x66, 0xb4, 0xe0, 0xb1 // mvn lt r11 r6 ROR 8
};
const byte kInstruction_mvn_cs_r3_r9_ROR_31[] = {
  0xe9, 0x3f, 0xe0, 0x21 // mvn cs r3 r9 ROR 31
};
const byte kInstruction_mvn_cc_r0_r0_LSL_10[] = {
  0x00, 0x05, 0xe0, 0x31 // mvn cc r0 r0 LSL 10
};
const byte kInstruction_mvn_mi_r4_r1_LSL_16[] = {
  0x01, 0x48, 0xe0, 0x41 // mvn mi r4 r1 LSL 16
};
const byte kInstruction_mvn_le_r5_r4_LSL_23[] = {
  0x84, 0x5b, 0xe0, 0xd1 // mvn le r5 r4 LSL 23
};
const byte kInstruction_mvn_lt_r3_r10_LSL_14[] = {
  0x0a, 0x37, 0xe0, 0xb1 // mvn lt r3 r10 LSL 14
};
const byte kInstruction_mvn_al_r10_r8_LSL_4[] = {
  0x08, 0xa2, 0xe0, 0xe1 // mvn al r10 r8 LSL 4
};
const byte kInstruction_mvn_al_r14_r11_LSL_11[] = {
  0x8b, 0xe5, 0xe0, 0xe1 // mvn al r14 r11 LSL 11
};
const byte kInstruction_mvn_mi_r14_r9_LSL_30[] = {
  0x09, 0xef, 0xe0, 0x41 // mvn mi r14 r9 LSL 30
};
const byte kInstruction_mvn_ge_r8_r11_LSL_29[] = {
  0x8b, 0x8e, 0xe0, 0xa1 // mvn ge r8 r11 LSL 29
};
const byte kInstruction_mvn_vs_r14_r9_ROR_13[] = {
  0xe9, 0xe6, 0xe0, 0x61 // mvn vs r14 r9 ROR 13
};
const byte kInstruction_mvn_lt_r5_r5_LSL_9[] = {
  0x85, 0x54, 0xe0, 0xb1 // mvn lt r5 r5 LSL 9
};
const byte kInstruction_mvn_pl_r13_r8_LSL_2[] = {
  0x08, 0xd1, 0xe0, 0x51 // mvn pl r13 r8 LSL 2
};
const byte kInstruction_mvn_al_r7_r14_LSL_15[] = {
  0x8e, 0x77, 0xe0, 0xe1 // mvn al r7 r14 LSL 15
};
const byte kInstruction_mvn_cs_r4_r0_ROR_3[] = {
  0xe0, 0x41, 0xe0, 0x21 // mvn cs r4 r0 ROR 3
};
const byte kInstruction_mvn_gt_r4_r6_ROR_21[] = {
  0xe6, 0x4a, 0xe0, 0xc1 // mvn gt r4 r6 ROR 21
};
const byte kInstruction_mvn_pl_r2_r1_LSL_3[] = {
  0x81, 0x21, 0xe0, 0x51 // mvn pl r2 r1 LSL 3
};
const byte kInstruction_mvn_vc_r5_r11_ROR_1[] = {
  0xeb, 0x50, 0xe0, 0x71 // mvn vc r5 r11 ROR 1
};
const byte kInstruction_mvn_vc_r9_r0_LSL_25[] = {
  0x80, 0x9c, 0xe0, 0x71 // mvn vc r9 r0 LSL 25
};
const byte kInstruction_mvn_gt_r2_r7_LSL_30[] = {
  0x07, 0x2f, 0xe0, 0xc1 // mvn gt r2 r7 LSL 30
};
const byte kInstruction_mvn_mi_r3_r0_LSL_29[] = {
  0x80, 0x3e, 0xe0, 0x41 // mvn mi r3 r0 LSL 29
};
const byte kInstruction_mvn_lt_r6_r3_ROR_18[] = {
  0x63, 0x69, 0xe0, 0xb1 // mvn lt r6 r3 ROR 18
};
const byte kInstruction_mvn_ne_r12_r10_ROR_28[] = {
  0x6a, 0xce, 0xe0, 0x11 // mvn ne r12 r10 ROR 28
};
const byte kInstruction_mvn_vs_r13_r9_LSL_25[] = {
  0x89, 0xdc, 0xe0, 0x61 // mvn vs r13 r9 LSL 25
};
const byte kInstruction_mvn_ge_r5_r12_LSL_13[] = {
  0x8c, 0x56, 0xe0, 0xa1 // mvn ge r5 r12 LSL 13
};
const byte kInstruction_mvn_vs_r2_r4_ROR_18[] = {
  0x64, 0x29, 0xe0, 0x61 // mvn vs r2 r4 ROR 18
};
const byte kInstruction_mvn_pl_r10_r4_LSL_9[] = {
  0x84, 0xa4, 0xe0, 0x51 // mvn pl r10 r4 LSL 9
};
const byte kInstruction_mvn_mi_r14_r13_ROR_27[] = {
  0xed, 0xed, 0xe0, 0x41 // mvn mi r14 r13 ROR 27
};
const byte kInstruction_mvn_lt_r1_r13_LSL_16[] = {
  0x0d, 0x18, 0xe0, 0xb1 // mvn lt r1 r13 LSL 16
};
const byte kInstruction_mvn_vc_r1_r13_LSL_9[] = {
  0x8d, 0x14, 0xe0, 0x71 // mvn vc r1 r13 LSL 9
};
const byte kInstruction_mvn_vs_r1_r5_ROR_24[] = {
  0x65, 0x1c, 0xe0, 0x61 // mvn vs r1 r5 ROR 24
};
const byte kInstruction_mvn_pl_r8_r7_LSL_30[] = {
  0x07, 0x8f, 0xe0, 0x51 // mvn pl r8 r7 LSL 30
};
const byte kInstruction_mvn_eq_r9_r5_LSL_30[] = {
  0x05, 0x9f, 0xe0, 0x01 // mvn eq r9 r5 LSL 30
};
const byte kInstruction_mvn_al_r1_r13_ROR_15[] = {
  0xed, 0x17, 0xe0, 0xe1 // mvn al r1 r13 ROR 15
};
const byte kInstruction_mvn_lt_r3_r6_LSL_24[] = {
  0x06, 0x3c, 0xe0, 0xb1 // mvn lt r3 r6 LSL 24
};
const byte kInstruction_mvn_gt_r3_r5_LSL_6[] = {
  0x05, 0x33, 0xe0, 0xc1 // mvn gt r3 r5 LSL 6
};
const byte kInstruction_mvn_cc_r1_r13_LSL_8[] = {
  0x0d, 0x14, 0xe0, 0x31 // mvn cc r1 r13 LSL 8
};
const byte kInstruction_mvn_cc_r11_r11_ROR_20[] = {
  0x6b, 0xba, 0xe0, 0x31 // mvn cc r11 r11 ROR 20
};
const byte kInstruction_mvn_gt_r4_r8_LSL_22[] = {
  0x08, 0x4b, 0xe0, 0xc1 // mvn gt r4 r8 LSL 22
};
const byte kInstruction_mvn_ge_r4_r12_LSL_3[] = {
  0x8c, 0x41, 0xe0, 0xa1 // mvn ge r4 r12 LSL 3
};
const byte kInstruction_mvn_eq_r10_r13_ROR_10[] = {
  0x6d, 0xa5, 0xe0, 0x01 // mvn eq r10 r13 ROR 10
};
const byte kInstruction_mvn_pl_r2_r9_LSL_1[] = {
  0x89, 0x20, 0xe0, 0x51 // mvn pl r2 r9 LSL 1
};
const byte kInstruction_mvn_cc_r14_r0_ROR_22[] = {
  0x60, 0xeb, 0xe0, 0x31 // mvn cc r14 r0 ROR 22
};
const byte kInstruction_mvn_ge_r9_r8_LSL_6[] = {
  0x08, 0x93, 0xe0, 0xa1 // mvn ge r9 r8 LSL 6
};
const byte kInstruction_mvn_ls_r14_r0_ROR_13[] = {
  0xe0, 0xe6, 0xe0, 0x91 // mvn ls r14 r0 ROR 13
};
const byte kInstruction_mvn_mi_r14_r5_LSL_11[] = {
  0x85, 0xe5, 0xe0, 0x41 // mvn mi r14 r5 LSL 11
};
const byte kInstruction_mvn_hi_r12_r4_LSL_11[] = {
  0x84, 0xc5, 0xe0, 0x81 // mvn hi r12 r4 LSL 11
};
const byte kInstruction_mvn_vc_r11_r10_ROR_22[] = {
  0x6a, 0xbb, 0xe0, 0x71 // mvn vc r11 r10 ROR 22
};
const byte kInstruction_mvn_eq_r12_r4_ROR_10[] = {
  0x64, 0xc5, 0xe0, 0x01 // mvn eq r12 r4 ROR 10
};
const byte kInstruction_mvn_pl_r0_r5_ROR_12[] = {
  0x65, 0x06, 0xe0, 0x51 // mvn pl r0 r5 ROR 12
};
const byte kInstruction_mvn_le_r6_r9_LSL_11[] = {
  0x89, 0x65, 0xe0, 0xd1 // mvn le r6 r9 LSL 11
};
const byte kInstruction_mvn_pl_r0_r5_LSL_13[] = {
  0x85, 0x06, 0xe0, 0x51 // mvn pl r0 r5 LSL 13
};
const byte kInstruction_mvn_hi_r14_r3_ROR_3[] = {
  0xe3, 0xe1, 0xe0, 0x81 // mvn hi r14 r3 ROR 3
};
const byte kInstruction_mvn_hi_r13_r7_LSL_12[] = {
  0x07, 0xd6, 0xe0, 0x81 // mvn hi r13 r7 LSL 12
};
const byte kInstruction_mvn_hi_r1_r4_LSL_3[] = {
  0x84, 0x11, 0xe0, 0x81 // mvn hi r1 r4 LSL 3
};
const byte kInstruction_mvn_vc_r12_r3_LSL_21[] = {
  0x83, 0xca, 0xe0, 0x71 // mvn vc r12 r3 LSL 21
};
const byte kInstruction_mvn_gt_r3_r0_ROR_9[] = {
  0xe0, 0x34, 0xe0, 0xc1 // mvn gt r3 r0 ROR 9
};
const byte kInstruction_mvn_mi_r1_r6_ROR_31[] = {
  0xe6, 0x1f, 0xe0, 0x41 // mvn mi r1 r6 ROR 31
};
const byte kInstruction_mvn_cs_r2_r3_ROR_16[] = {
  0x63, 0x28, 0xe0, 0x21 // mvn cs r2 r3 ROR 16
};
const byte kInstruction_mvn_lt_r6_r3_ROR_7[] = {
  0xe3, 0x63, 0xe0, 0xb1 // mvn lt r6 r3 ROR 7
};
const byte kInstruction_mvn_le_r5_r3_LSL_12[] = {
  0x03, 0x56, 0xe0, 0xd1 // mvn le r5 r3 LSL 12
};
const byte kInstruction_mvn_pl_r14_r12_ROR_3[] = {
  0xec, 0xe1, 0xe0, 0x51 // mvn pl r14 r12 ROR 3
};
const byte kInstruction_mvn_gt_r6_r4_LSL_27[] = {
  0x84, 0x6d, 0xe0, 0xc1 // mvn gt r6 r4 LSL 27
};
const byte kInstruction_mvn_le_r11_r10_ROR_4[] = {
  0x6a, 0xb2, 0xe0, 0xd1 // mvn le r11 r10 ROR 4
};
const byte kInstruction_mvn_gt_r8_r1_LSL_25[] = {
  0x81, 0x8c, 0xe0, 0xc1 // mvn gt r8 r1 LSL 25
};
const byte kInstruction_mvn_pl_r10_r4_LSL_22[] = {
  0x04, 0xab, 0xe0, 0x51 // mvn pl r10 r4 LSL 22
};
const byte kInstruction_mvn_al_r9_r5_ROR_15[] = {
  0xe5, 0x97, 0xe0, 0xe1 // mvn al r9 r5 ROR 15
};
const byte kInstruction_mvn_cs_r1_r5_ROR_3[] = {
  0xe5, 0x11, 0xe0, 0x21 // mvn cs r1 r5 ROR 3
};
const byte kInstruction_mvn_eq_r0_r1_LSL_26[] = {
  0x01, 0x0d, 0xe0, 0x01 // mvn eq r0 r1 LSL 26
};
const byte kInstruction_mvn_hi_r13_r14_LSL_6[] = {
  0x0e, 0xd3, 0xe0, 0x81 // mvn hi r13 r14 LSL 6
};
const byte kInstruction_mvn_vs_r8_r1_LSL_18[] = {
  0x01, 0x89, 0xe0, 0x61 // mvn vs r8 r1 LSL 18
};
const byte kInstruction_mvn_eq_r12_r2_LSL_16[] = {
  0x02, 0xc8, 0xe0, 0x01 // mvn eq r12 r2 LSL 16
};
const byte kInstruction_mvn_cc_r4_r3_LSL_4[] = {
  0x03, 0x42, 0xe0, 0x31 // mvn cc r4 r3 LSL 4
};
const byte kInstruction_mvn_ls_r4_r2_LSL_14[] = {
  0x02, 0x47, 0xe0, 0x91 // mvn ls r4 r2 LSL 14
};
const byte kInstruction_mvn_cs_r8_r12_LSL_27[] = {
  0x8c, 0x8d, 0xe0, 0x21 // mvn cs r8 r12 LSL 27
};
const byte kInstruction_mvn_le_r3_r10_ROR_18[] = {
  0x6a, 0x39, 0xe0, 0xd1 // mvn le r3 r10 ROR 18
};
const byte kInstruction_mvn_vc_r6_r0_LSL_26[] = {
  0x00, 0x6d, 0xe0, 0x71 // mvn vc r6 r0 LSL 26
};
const byte kInstruction_mvn_mi_r8_r13_LSL_3[] = {
  0x8d, 0x81, 0xe0, 0x41 // mvn mi r8 r13 LSL 3
};
const byte kInstruction_mvn_al_r3_r2_ROR_14[] = {
  0x62, 0x37, 0xe0, 0xe1 // mvn al r3 r2 ROR 14
};
const byte kInstruction_mvn_gt_r6_r14_LSL_29[] = {
  0x8e, 0x6e, 0xe0, 0xc1 // mvn gt r6 r14 LSL 29
};
const byte kInstruction_mvn_hi_r9_r12_ROR_25[] = {
  0xec, 0x9c, 0xe0, 0x81 // mvn hi r9 r12 ROR 25
};
const byte kInstruction_mvn_mi_r11_r13_ROR_12[] = {
  0x6d, 0xb6, 0xe0, 0x41 // mvn mi r11 r13 ROR 12
};
const byte kInstruction_mvn_vs_r2_r6_LSL_13[] = {
  0x86, 0x26, 0xe0, 0x61 // mvn vs r2 r6 LSL 13
};
const byte kInstruction_mvn_hi_r14_r4_LSL_29[] = {
  0x84, 0xee, 0xe0, 0x81 // mvn hi r14 r4 LSL 29
};
const byte kInstruction_mvn_hi_r9_r13_LSL_4[] = {
  0x0d, 0x92, 0xe0, 0x81 // mvn hi r9 r13 LSL 4
};
const byte kInstruction_mvn_hi_r8_r1_ROR_12[] = {
  0x61, 0x86, 0xe0, 0x81 // mvn hi r8 r1 ROR 12
};
const byte kInstruction_mvn_vs_r13_r0_ROR_11[] = {
  0xe0, 0xd5, 0xe0, 0x61 // mvn vs r13 r0 ROR 11
};
const byte kInstruction_mvn_le_r4_r11_ROR_31[] = {
  0xeb, 0x4f, 0xe0, 0xd1 // mvn le r4 r11 ROR 31
};
const byte kInstruction_mvn_eq_r7_r1_ROR_9[] = {
  0xe1, 0x74, 0xe0, 0x01 // mvn eq r7 r1 ROR 9
};
const byte kInstruction_mvn_ls_r13_r3_ROR_4[] = {
  0x63, 0xd2, 0xe0, 0x91 // mvn ls r13 r3 ROR 4
};
const byte kInstruction_mvn_hi_r3_r4_ROR_9[] = {
  0xe4, 0x34, 0xe0, 0x81 // mvn hi r3 r4 ROR 9
};
const byte kInstruction_mvn_al_r4_r8_LSL_7[] = {
  0x88, 0x43, 0xe0, 0xe1 // mvn al r4 r8 LSL 7
};
const byte kInstruction_mvn_le_r11_r2_ROR_15[] = {
  0xe2, 0xb7, 0xe0, 0xd1 // mvn le r11 r2 ROR 15
};
const byte kInstruction_mvn_hi_r4_r11_ROR_3[] = {
  0xeb, 0x41, 0xe0, 0x81 // mvn hi r4 r11 ROR 3
};
const byte kInstruction_mvn_mi_r5_r4_LSL_16[] = {
  0x04, 0x58, 0xe0, 0x41 // mvn mi r5 r4 LSL 16
};
const byte kInstruction_mvn_pl_r7_r10_ROR_11[] = {
  0xea, 0x75, 0xe0, 0x51 // mvn pl r7 r10 ROR 11
};
const byte kInstruction_mvn_lt_r11_r14_ROR_5[] = {
  0xee, 0xb2, 0xe0, 0xb1 // mvn lt r11 r14 ROR 5
};
const byte kInstruction_mvn_le_r14_r12_LSL_3[] = {
  0x8c, 0xe1, 0xe0, 0xd1 // mvn le r14 r12 LSL 3
};
const byte kInstruction_mvn_mi_r6_r7_LSL_7[] = {
  0x87, 0x63, 0xe0, 0x41 // mvn mi r6 r7 LSL 7
};
const byte kInstruction_mvn_hi_r6_r12_LSL_13[] = {
  0x8c, 0x66, 0xe0, 0x81 // mvn hi r6 r12 LSL 13
};
const byte kInstruction_mvn_mi_r12_r12_ROR_24[] = {
  0x6c, 0xcc, 0xe0, 0x41 // mvn mi r12 r12 ROR 24
};
const byte kInstruction_mvn_mi_r6_r4_LSL_31[] = {
  0x84, 0x6f, 0xe0, 0x41 // mvn mi r6 r4 LSL 31
};
const byte kInstruction_mvn_vc_r13_r7_LSL_22[] = {
  0x07, 0xdb, 0xe0, 0x71 // mvn vc r13 r7 LSL 22
};
const byte kInstruction_mvn_le_r1_r10_ROR_8[] = {
  0x6a, 0x14, 0xe0, 0xd1 // mvn le r1 r10 ROR 8
};
const byte kInstruction_mvn_gt_r9_r1_LSL_6[] = {
  0x01, 0x93, 0xe0, 0xc1 // mvn gt r9 r1 LSL 6
};
const byte kInstruction_mvn_gt_r14_r2_ROR_26[] = {
  0x62, 0xed, 0xe0, 0xc1 // mvn gt r14 r2 ROR 26
};
const byte kInstruction_mvn_pl_r11_r3_LSL_1[] = {
  0x83, 0xb0, 0xe0, 0x51 // mvn pl r11 r3 LSL 1
};
const byte kInstruction_mvn_ge_r4_r7_LSL_29[] = {
  0x87, 0x4e, 0xe0, 0xa1 // mvn ge r4 r7 LSL 29
};
const byte kInstruction_mvn_gt_r9_r2_LSL_6[] = {
  0x02, 0x93, 0xe0, 0xc1 // mvn gt r9 r2 LSL 6
};
const byte kInstruction_mvn_cs_r4_r7_LSL_21[] = {
  0x87, 0x4a, 0xe0, 0x21 // mvn cs r4 r7 LSL 21
};
const byte kInstruction_mvn_eq_r4_r3_LSL_9[] = {
  0x83, 0x44, 0xe0, 0x01 // mvn eq r4 r3 LSL 9
};
const byte kInstruction_mvn_ne_r14_r3_ROR_10[] = {
  0x63, 0xe5, 0xe0, 0x11 // mvn ne r14 r3 ROR 10
};
const byte kInstruction_mvn_eq_r4_r6_LSL_19[] = {
  0x86, 0x49, 0xe0, 0x01 // mvn eq r4 r6 LSL 19
};
const byte kInstruction_mvn_cs_r6_r1_ROR_2[] = {
  0x61, 0x61, 0xe0, 0x21 // mvn cs r6 r1 ROR 2
};
const byte kInstruction_mvn_ls_r0_r11_LSL_7[] = {
  0x8b, 0x03, 0xe0, 0x91 // mvn ls r0 r11 LSL 7
};
const byte kInstruction_mvn_ge_r10_r10_LSL_28[] = {
  0x0a, 0xae, 0xe0, 0xa1 // mvn ge r10 r10 LSL 28
};
const byte kInstruction_mvn_mi_r0_r9_LSL_27[] = {
  0x89, 0x0d, 0xe0, 0x41 // mvn mi r0 r9 LSL 27
};
const byte kInstruction_mvn_vc_r7_r4_LSL_17[] = {
  0x84, 0x78, 0xe0, 0x71 // mvn vc r7 r4 LSL 17
};
const byte kInstruction_mvn_gt_r4_r4_LSL_21[] = {
  0x84, 0x4a, 0xe0, 0xc1 // mvn gt r4 r4 LSL 21
};
const byte kInstruction_mvn_mi_r5_r9_ROR_28[] = {
  0x69, 0x5e, 0xe0, 0x41 // mvn mi r5 r9 ROR 28
};
const byte kInstruction_mvn_hi_r9_r9_LSL_2[] = {
  0x09, 0x91, 0xe0, 0x81 // mvn hi r9 r9 LSL 2
};
const byte kInstruction_mvn_hi_r2_r2_ROR_11[] = {
  0xe2, 0x25, 0xe0, 0x81 // mvn hi r2 r2 ROR 11
};
const byte kInstruction_mvn_mi_r8_r9_ROR_25[] = {
  0xe9, 0x8c, 0xe0, 0x41 // mvn mi r8 r9 ROR 25
};
const byte kInstruction_mvn_mi_r3_r13_ROR_20[] = {
  0x6d, 0x3a, 0xe0, 0x41 // mvn mi r3 r13 ROR 20
};
const byte kInstruction_mvn_lt_r9_r10_ROR_28[] = {
  0x6a, 0x9e, 0xe0, 0xb1 // mvn lt r9 r10 ROR 28
};
const byte kInstruction_mvn_pl_r8_r5_LSL_16[] = {
  0x05, 0x88, 0xe0, 0x51 // mvn pl r8 r5 LSL 16
};
const byte kInstruction_mvn_al_r14_r1_LSL_11[] = {
  0x81, 0xe5, 0xe0, 0xe1 // mvn al r14 r1 LSL 11
};
const byte kInstruction_mvn_ls_r8_r4_ROR_5[] = {
  0xe4, 0x82, 0xe0, 0x91 // mvn ls r8 r4 ROR 5
};
const byte kInstruction_mvn_cc_r11_r9_ROR_15[] = {
  0xe9, 0xb7, 0xe0, 0x31 // mvn cc r11 r9 ROR 15
};
const byte kInstruction_mvn_gt_r6_r0_LSL_2[] = {
  0x00, 0x61, 0xe0, 0xc1 // mvn gt r6 r0 LSL 2
};
const byte kInstruction_mvn_ne_r12_r8_LSL_15[] = {
  0x88, 0xc7, 0xe0, 0x11 // mvn ne r12 r8 LSL 15
};
const byte kInstruction_mvn_gt_r8_r2_ROR_31[] = {
  0xe2, 0x8f, 0xe0, 0xc1 // mvn gt r8 r2 ROR 31
};
const byte kInstruction_mvn_lt_r7_r12_LSL_19[] = {
  0x8c, 0x79, 0xe0, 0xb1 // mvn lt r7 r12 LSL 19
};
const byte kInstruction_mvn_al_r9_r11_ROR_28[] = {
  0x6b, 0x9e, 0xe0, 0xe1 // mvn al r9 r11 ROR 28
};
const byte kInstruction_mvn_al_r13_r5_ROR_28[] = {
  0x65, 0xde, 0xe0, 0xe1 // mvn al r13 r5 ROR 28
};
const byte kInstruction_mvn_lt_r2_r13_LSL_6[] = {
  0x0d, 0x23, 0xe0, 0xb1 // mvn lt r2 r13 LSL 6
};
const byte kInstruction_mvn_al_r7_r7_LSL_8[] = {
  0x07, 0x74, 0xe0, 0xe1 // mvn al r7 r7 LSL 8
};
const byte kInstruction_mvn_lt_r1_r11_ROR_16[] = {
  0x6b, 0x18, 0xe0, 0xb1 // mvn lt r1 r11 ROR 16
};
const byte kInstruction_mvn_eq_r7_r8_LSL_23[] = {
  0x88, 0x7b, 0xe0, 0x01 // mvn eq r7 r8 LSL 23
};
const byte kInstruction_mvn_cs_r4_r12_LSL_1[] = {
  0x8c, 0x40, 0xe0, 0x21 // mvn cs r4 r12 LSL 1
};
const byte kInstruction_mvn_ls_r2_r3_ROR_26[] = {
  0x63, 0x2d, 0xe0, 0x91 // mvn ls r2 r3 ROR 26
};
const byte kInstruction_mvn_ne_r13_r1_ROR_11[] = {
  0xe1, 0xd5, 0xe0, 0x11 // mvn ne r13 r1 ROR 11
};
const byte kInstruction_mvn_le_r8_r13_ROR_26[] = {
  0x6d, 0x8d, 0xe0, 0xd1 // mvn le r8 r13 ROR 26
};
const byte kInstruction_mvn_al_r2_r4_LSL_8[] = {
  0x04, 0x24, 0xe0, 0xe1 // mvn al r2 r4 LSL 8
};
const byte kInstruction_mvn_gt_r4_r12_ROR_16[] = {
  0x6c, 0x48, 0xe0, 0xc1 // mvn gt r4 r12 ROR 16
};
const byte kInstruction_mvn_lt_r8_r1_ROR_19[] = {
  0xe1, 0x89, 0xe0, 0xb1 // mvn lt r8 r1 ROR 19
};
const byte kInstruction_mvn_ne_r7_r3_ROR_15[] = {
  0xe3, 0x77, 0xe0, 0x11 // mvn ne r7 r3 ROR 15
};
const byte kInstruction_mvn_ge_r11_r8_LSL_23[] = {
  0x88, 0xbb, 0xe0, 0xa1 // mvn ge r11 r8 LSL 23
};
const byte kInstruction_mvn_mi_r11_r12_ROR_10[] = {
  0x6c, 0xb5, 0xe0, 0x41 // mvn mi r11 r12 ROR 10
};
const byte kInstruction_mvn_cs_r1_r4_ROR_15[] = {
  0xe4, 0x17, 0xe0, 0x21 // mvn cs r1 r4 ROR 15
};
const byte kInstruction_mvn_mi_r14_r4_LSL_5[] = {
  0x84, 0xe2, 0xe0, 0x41 // mvn mi r14 r4 LSL 5
};
const byte kInstruction_mvn_ge_r5_r2_ROR_15[] = {
  0xe2, 0x57, 0xe0, 0xa1 // mvn ge r5 r2 ROR 15
};
const byte kInstruction_mvn_lt_r11_r11_LSL_13[] = {
  0x8b, 0xb6, 0xe0, 0xb1 // mvn lt r11 r11 LSL 13
};
const byte kInstruction_mvn_hi_r7_r8_ROR_2[] = {
  0x68, 0x71, 0xe0, 0x81 // mvn hi r7 r8 ROR 2
};
const byte kInstruction_mvn_eq_r14_r0_LSL_4[] = {
  0x00, 0xe2, 0xe0, 0x01 // mvn eq r14 r0 LSL 4
};
const byte kInstruction_mvn_al_r12_r5_LSL_6[] = {
  0x05, 0xc3, 0xe0, 0xe1 // mvn al r12 r5 LSL 6
};
const byte kInstruction_mvn_mi_r5_r6_ROR_14[] = {
  0x66, 0x57, 0xe0, 0x41 // mvn mi r5 r6 ROR 14
};
const byte kInstruction_mvn_cc_r14_r0_ROR_31[] = {
  0xe0, 0xef, 0xe0, 0x31 // mvn cc r14 r0 ROR 31
};
const byte kInstruction_mvn_ne_r4_r12_ROR_22[] = {
  0x6c, 0x4b, 0xe0, 0x11 // mvn ne r4 r12 ROR 22
};
const byte kInstruction_mvn_ne_r1_r3_LSL_24[] = {
  0x03, 0x1c, 0xe0, 0x11 // mvn ne r1 r3 LSL 24
};
const byte kInstruction_mvn_hi_r3_r9_LSL_26[] = {
  0x09, 0x3d, 0xe0, 0x81 // mvn hi r3 r9 LSL 26
};
const byte kInstruction_mvn_vs_r0_r7_ROR_31[] = {
  0xe7, 0x0f, 0xe0, 0x61 // mvn vs r0 r7 ROR 31
};
const byte kInstruction_mvn_le_r3_r1_ROR_12[] = {
  0x61, 0x36, 0xe0, 0xd1 // mvn le r3 r1 ROR 12
};
const byte kInstruction_mvn_le_r5_r8_LSL_19[] = {
  0x88, 0x59, 0xe0, 0xd1 // mvn le r5 r8 LSL 19
};
const byte kInstruction_mvn_lt_r0_r13_ROR_8[] = {
  0x6d, 0x04, 0xe0, 0xb1 // mvn lt r0 r13 ROR 8
};
const byte kInstruction_mvn_ge_r12_r13_LSL_6[] = {
  0x0d, 0xc3, 0xe0, 0xa1 // mvn ge r12 r13 LSL 6
};
const byte kInstruction_mvn_lt_r1_r2_LSL_21[] = {
  0x82, 0x1a, 0xe0, 0xb1 // mvn lt r1 r2 LSL 21
};
const byte kInstruction_mvn_ge_r4_r5_LSL_26[] = {
  0x05, 0x4d, 0xe0, 0xa1 // mvn ge r4 r5 LSL 26
};
const byte kInstruction_mvn_ls_r6_r10_LSL_25[] = {
  0x8a, 0x6c, 0xe0, 0x91 // mvn ls r6 r10 LSL 25
};
const byte kInstruction_mvn_hi_r5_r4_LSL_18[] = {
  0x04, 0x59, 0xe0, 0x81 // mvn hi r5 r4 LSL 18
};
const byte kInstruction_mvn_cs_r7_r14_LSL_23[] = {
  0x8e, 0x7b, 0xe0, 0x21 // mvn cs r7 r14 LSL 23
};
const byte kInstruction_mvn_cc_r14_r4_ROR_3[] = {
  0xe4, 0xe1, 0xe0, 0x31 // mvn cc r14 r4 ROR 3
};
const byte kInstruction_mvn_cs_r14_r4_LSL_17[] = {
  0x84, 0xe8, 0xe0, 0x21 // mvn cs r14 r4 LSL 17
};
const byte kInstruction_mvn_lt_r14_r12_LSL_24[] = {
  0x0c, 0xec, 0xe0, 0xb1 // mvn lt r14 r12 LSL 24
};
const byte kInstruction_mvn_vc_r6_r14_ROR_3[] = {
  0xee, 0x61, 0xe0, 0x71 // mvn vc r6 r14 ROR 3
};
const byte kInstruction_mvn_ls_r11_r5_ROR_18[] = {
  0x65, 0xb9, 0xe0, 0x91 // mvn ls r11 r5 ROR 18
};
const byte kInstruction_mvn_gt_r3_r1_ROR_2[] = {
  0x61, 0x31, 0xe0, 0xc1 // mvn gt r3 r1 ROR 2
};
const byte kInstruction_mvn_al_r11_r14_ROR_5[] = {
  0xee, 0xb2, 0xe0, 0xe1 // mvn al r11 r14 ROR 5
};
const byte kInstruction_mvn_le_r11_r11_LSL_15[] = {
  0x8b, 0xb7, 0xe0, 0xd1 // mvn le r11 r11 LSL 15
};
const byte kInstruction_mvn_gt_r1_r13_LSL_30[] = {
  0x0d, 0x1f, 0xe0, 0xc1 // mvn gt r1 r13 LSL 30
};
const byte kInstruction_mvn_le_r3_r5_LSL_31[] = {
  0x85, 0x3f, 0xe0, 0xd1 // mvn le r3 r5 LSL 31
};
const byte kInstruction_mvn_hi_r4_r10_ROR_16[] = {
  0x6a, 0x48, 0xe0, 0x81 // mvn hi r4 r10 ROR 16
};
const byte kInstruction_mvn_eq_r0_r6_LSL_6[] = {
  0x06, 0x03, 0xe0, 0x01 // mvn eq r0 r6 LSL 6
};
const byte kInstruction_mvn_vs_r0_r14_ROR_3[] = {
  0xee, 0x01, 0xe0, 0x61 // mvn vs r0 r14 ROR 3
};
const byte kInstruction_mvn_eq_r0_r3_ROR_26[] = {
  0x63, 0x0d, 0xe0, 0x01 // mvn eq r0 r3 ROR 26
};
const byte kInstruction_mvn_eq_r1_r10_ROR_30[] = {
  0x6a, 0x1f, 0xe0, 0x01 // mvn eq r1 r10 ROR 30
};
const byte kInstruction_mvn_ls_r9_r2_ROR_12[] = {
  0x62, 0x96, 0xe0, 0x91 // mvn ls r9 r2 ROR 12
};
const byte kInstruction_mvn_gt_r9_r14_ROR_22[] = {
  0x6e, 0x9b, 0xe0, 0xc1 // mvn gt r9 r14 ROR 22
};
const byte kInstruction_mvn_hi_r1_r8_LSL_1[] = {
  0x88, 0x10, 0xe0, 0x81 // mvn hi r1 r8 LSL 1
};
const byte kInstruction_mvn_le_r13_r0_ROR_4[] = {
  0x60, 0xd2, 0xe0, 0xd1 // mvn le r13 r0 ROR 4
};
const byte kInstruction_mvn_lt_r11_r10_ROR_4[] = {
  0x6a, 0xb2, 0xe0, 0xb1 // mvn lt r11 r10 ROR 4
};
const byte kInstruction_mvn_le_r2_r5_LSL_28[] = {
  0x05, 0x2e, 0xe0, 0xd1 // mvn le r2 r5 LSL 28
};
const byte kInstruction_mvn_vc_r8_r12_LSL_27[] = {
  0x8c, 0x8d, 0xe0, 0x71 // mvn vc r8 r12 LSL 27
};
const byte kInstruction_mvn_al_r11_r3_LSL_6[] = {
  0x03, 0xb3, 0xe0, 0xe1 // mvn al r11 r3 LSL 6
};
const byte kInstruction_mvn_al_r12_r1_ROR_2[] = {
  0x61, 0xc1, 0xe0, 0xe1 // mvn al r12 r1 ROR 2
};
const byte kInstruction_mvn_cs_r8_r10_LSL_5[] = {
  0x8a, 0x82, 0xe0, 0x21 // mvn cs r8 r10 LSL 5
};
const byte kInstruction_mvn_hi_r11_r2_ROR_4[] = {
  0x62, 0xb2, 0xe0, 0x81 // mvn hi r11 r2 ROR 4
};
const byte kInstruction_mvn_eq_r1_r2_LSL_15[] = {
  0x82, 0x17, 0xe0, 0x01 // mvn eq r1 r2 LSL 15
};
const byte kInstruction_mvn_lt_r0_r5_ROR_5[] = {
  0xe5, 0x02, 0xe0, 0xb1 // mvn lt r0 r5 ROR 5
};
const byte kInstruction_mvn_mi_r9_r0_ROR_18[] = {
  0x60, 0x99, 0xe0, 0x41 // mvn mi r9 r0 ROR 18
};
const byte kInstruction_mvn_cs_r12_r9_LSL_5[] = {
  0x89, 0xc2, 0xe0, 0x21 // mvn cs r12 r9 LSL 5
};
const byte kInstruction_mvn_cc_r13_r10_LSL_19[] = {
  0x8a, 0xd9, 0xe0, 0x31 // mvn cc r13 r10 LSL 19
};
const byte kInstruction_mvn_eq_r8_r0_LSL_25[] = {
  0x80, 0x8c, 0xe0, 0x01 // mvn eq r8 r0 LSL 25
};
const byte kInstruction_mvn_cc_r9_r7_LSL_4[] = {
  0x07, 0x92, 0xe0, 0x31 // mvn cc r9 r7 LSL 4
};
const byte kInstruction_mvn_ls_r6_r2_LSL_18[] = {
  0x02, 0x69, 0xe0, 0x91 // mvn ls r6 r2 LSL 18
};
const byte kInstruction_mvn_lt_r6_r14_ROR_3[] = {
  0xee, 0x61, 0xe0, 0xb1 // mvn lt r6 r14 ROR 3
};
const byte kInstruction_mvn_al_r10_r6_ROR_13[] = {
  0xe6, 0xa6, 0xe0, 0xe1 // mvn al r10 r6 ROR 13
};
const byte kInstruction_mvn_le_r1_r0_ROR_8[] = {
  0x60, 0x14, 0xe0, 0xd1 // mvn le r1 r0 ROR 8
};
const byte kInstruction_mvn_hi_r1_r3_ROR_5[] = {
  0xe3, 0x12, 0xe0, 0x81 // mvn hi r1 r3 ROR 5
};
const byte kInstruction_mvn_cc_r5_r1_ROR_5[] = {
  0xe1, 0x52, 0xe0, 0x31 // mvn cc r5 r1 ROR 5
};
const byte kInstruction_mvn_vc_r12_r3_LSL_6[] = {
  0x03, 0xc3, 0xe0, 0x71 // mvn vc r12 r3 LSL 6
};
const byte kInstruction_mvn_ls_r11_r13_ROR_28[] = {
  0x6d, 0xbe, 0xe0, 0x91 // mvn ls r11 r13 ROR 28
};
const byte kInstruction_mvn_vs_r9_r13_ROR_10[] = {
  0x6d, 0x95, 0xe0, 0x61 // mvn vs r9 r13 ROR 10
};
const byte kInstruction_mvn_eq_r3_r13_LSL_31[] = {
  0x8d, 0x3f, 0xe0, 0x01 // mvn eq r3 r13 LSL 31
};
const byte kInstruction_mvn_cc_r11_r9_ROR_27[] = {
  0xe9, 0xbd, 0xe0, 0x31 // mvn cc r11 r9 ROR 27
};
const byte kInstruction_mvn_cc_r9_r2_LSL_28[] = {
  0x02, 0x9e, 0xe0, 0x31 // mvn cc r9 r2 LSL 28
};
const byte kInstruction_mvn_ne_r14_r5_LSL_4[] = {
  0x05, 0xe2, 0xe0, 0x11 // mvn ne r14 r5 LSL 4
};
const byte kInstruction_mvn_gt_r4_r2_LSL_31[] = {
  0x82, 0x4f, 0xe0, 0xc1 // mvn gt r4 r2 LSL 31
};
const byte kInstruction_mvn_cc_r9_r1_ROR_17[] = {
  0xe1, 0x98, 0xe0, 0x31 // mvn cc r9 r1 ROR 17
};
const byte kInstruction_mvn_hi_r4_r10_LSL_24[] = {
  0x0a, 0x4c, 0xe0, 0x81 // mvn hi r4 r10 LSL 24
};
const byte kInstruction_mvn_ne_r3_r9_LSL_5[] = {
  0x89, 0x32, 0xe0, 0x11 // mvn ne r3 r9 LSL 5
};
const byte kInstruction_mvn_hi_r8_r5_ROR_10[] = {
  0x65, 0x85, 0xe0, 0x81 // mvn hi r8 r5 ROR 10
};
const byte kInstruction_mvn_pl_r5_r1_ROR_3[] = {
  0xe1, 0x51, 0xe0, 0x51 // mvn pl r5 r1 ROR 3
};
const byte kInstruction_mvn_ge_r14_r0_ROR_14[] = {
  0x60, 0xe7, 0xe0, 0xa1 // mvn ge r14 r0 ROR 14
};
const byte kInstruction_mvn_ge_r11_r7_LSL_15[] = {
  0x87, 0xb7, 0xe0, 0xa1 // mvn ge r11 r7 LSL 15
};
const byte kInstruction_mvn_pl_r10_r11_LSL_15[] = {
  0x8b, 0xa7, 0xe0, 0x51 // mvn pl r10 r11 LSL 15
};
const byte kInstruction_mvn_ne_r6_r11_LSL_3[] = {
  0x8b, 0x61, 0xe0, 0x11 // mvn ne r6 r11 LSL 3
};
const byte kInstruction_mvn_cs_r4_r5_ROR_12[] = {
  0x65, 0x46, 0xe0, 0x21 // mvn cs r4 r5 ROR 12
};
const byte kInstruction_mvn_eq_r8_r10_ROR_4[] = {
  0x6a, 0x82, 0xe0, 0x01 // mvn eq r8 r10 ROR 4
};
const byte kInstruction_mvn_mi_r9_r12_ROR_2[] = {
  0x6c, 0x91, 0xe0, 0x41 // mvn mi r9 r12 ROR 2
};
const byte kInstruction_mvn_pl_r6_r2_LSL_24[] = {
  0x02, 0x6c, 0xe0, 0x51 // mvn pl r6 r2 LSL 24
};
const byte kInstruction_mvn_vs_r6_r8_LSL_12[] = {
  0x08, 0x66, 0xe0, 0x61 // mvn vs r6 r8 LSL 12
};
const byte kInstruction_mvn_vc_r0_r13_ROR_29[] = {
  0xed, 0x0e, 0xe0, 0x71 // mvn vc r0 r13 ROR 29
};
const byte kInstruction_mvn_ne_r1_r4_LSL_19[] = {
  0x84, 0x19, 0xe0, 0x11 // mvn ne r1 r4 LSL 19
};
const byte kInstruction_mvn_cc_r3_r12_ROR_30[] = {
  0x6c, 0x3f, 0xe0, 0x31 // mvn cc r3 r12 ROR 30
};
const byte kInstruction_mvn_gt_r9_r8_LSL_17[] = {
  0x88, 0x98, 0xe0, 0xc1 // mvn gt r9 r8 LSL 17
};
const byte kInstruction_mvn_ne_r3_r9_ROR_14[] = {
  0x69, 0x37, 0xe0, 0x11 // mvn ne r3 r9 ROR 14
};
const byte kInstruction_mvn_ne_r5_r7_ROR_25[] = {
  0xe7, 0x5c, 0xe0, 0x11 // mvn ne r5 r7 ROR 25
};
const byte kInstruction_mvn_pl_r8_r10_LSL_16[] = {
  0x0a, 0x88, 0xe0, 0x51 // mvn pl r8 r10 LSL 16
};
const byte kInstruction_mvn_ls_r10_r10_ROR_22[] = {
  0x6a, 0xab, 0xe0, 0x91 // mvn ls r10 r10 ROR 22
};
const byte kInstruction_mvn_al_r11_r11_ROR_29[] = {
  0xeb, 0xbe, 0xe0, 0xe1 // mvn al r11 r11 ROR 29
};
const byte kInstruction_mvn_ge_r6_r11_LSL_18[] = {
  0x0b, 0x69, 0xe0, 0xa1 // mvn ge r6 r11 LSL 18
};
const byte kInstruction_mvn_hi_r0_r12_LSL_2[] = {
  0x0c, 0x01, 0xe0, 0x81 // mvn hi r0 r12 LSL 2
};
const byte kInstruction_mvn_ge_r3_r12_LSL_25[] = {
  0x8c, 0x3c, 0xe0, 0xa1 // mvn ge r3 r12 LSL 25
};
const byte kInstruction_mvn_pl_r6_r1_LSL_27[] = {
  0x81, 0x6d, 0xe0, 0x51 // mvn pl r6 r1 LSL 27
};
const byte kInstruction_mvn_lt_r7_r8_LSL_19[] = {
  0x88, 0x79, 0xe0, 0xb1 // mvn lt r7 r8 LSL 19
};
const byte kInstruction_mvn_hi_r3_r9_LSL_10[] = {
  0x09, 0x35, 0xe0, 0x81 // mvn hi r3 r9 LSL 10
};
const byte kInstruction_mvn_eq_r3_r13_LSL_27[] = {
  0x8d, 0x3d, 0xe0, 0x01 // mvn eq r3 r13 LSL 27
};
const byte kInstruction_mvn_ne_r3_r1_ROR_6[] = {
  0x61, 0x33, 0xe0, 0x11 // mvn ne r3 r1 ROR 6
};
const byte kInstruction_mvn_cs_r8_r13_ROR_16[] = {
  0x6d, 0x88, 0xe0, 0x21 // mvn cs r8 r13 ROR 16
};
const byte kInstruction_mvn_eq_r1_r4_ROR_23[] = {
  0xe4, 0x1b, 0xe0, 0x01 // mvn eq r1 r4 ROR 23
};
const byte kInstruction_mvn_vc_r5_r8_ROR_5[] = {
  0xe8, 0x52, 0xe0, 0x71 // mvn vc r5 r8 ROR 5
};
const byte kInstruction_mvn_ge_r13_r5_LSL_24[] = {
  0x05, 0xdc, 0xe0, 0xa1 // mvn ge r13 r5 LSL 24
};
const byte kInstruction_mvn_lt_r4_r10_ROR_25[] = {
  0xea, 0x4c, 0xe0, 0xb1 // mvn lt r4 r10 ROR 25
};
const byte kInstruction_mvn_hi_r3_r10_LSL_21[] = {
  0x8a, 0x3a, 0xe0, 0x81 // mvn hi r3 r10 LSL 21
};
const byte kInstruction_mvn_hi_r7_r1_LSL_31[] = {
  0x81, 0x7f, 0xe0, 0x81 // mvn hi r7 r1 LSL 31
};
const byte kInstruction_mvn_gt_r4_r12_ROR_29[] = {
  0xec, 0x4e, 0xe0, 0xc1 // mvn gt r4 r12 ROR 29
};
const byte kInstruction_mvn_cs_r5_r1_ROR_27[] = {
  0xe1, 0x5d, 0xe0, 0x21 // mvn cs r5 r1 ROR 27
};
const byte kInstruction_mvn_al_r4_r7_ROR_8[] = {
  0x67, 0x44, 0xe0, 0xe1 // mvn al r4 r7 ROR 8
};
const byte kInstruction_mvn_ge_r8_r10_LSL_7[] = {
  0x8a, 0x83, 0xe0, 0xa1 // mvn ge r8 r10 LSL 7
};
const byte kInstruction_mvn_vc_r0_r12_LSL_29[] = {
  0x8c, 0x0e, 0xe0, 0x71 // mvn vc r0 r12 LSL 29
};
const byte kInstruction_mvn_lt_r5_r1_LSL_9[] = {
  0x81, 0x54, 0xe0, 0xb1 // mvn lt r5 r1 LSL 9
};
const byte kInstruction_mvn_gt_r6_r10_LSL_30[] = {
  0x0a, 0x6f, 0xe0, 0xc1 // mvn gt r6 r10 LSL 30
};
const byte kInstruction_mvn_eq_r14_r0_LSL_2[] = {
  0x00, 0xe1, 0xe0, 0x01 // mvn eq r14 r0 LSL 2
};
const byte kInstruction_mvn_lt_r0_r6_LSL_21[] = {
  0x86, 0x0a, 0xe0, 0xb1 // mvn lt r0 r6 LSL 21
};
const byte kInstruction_mvn_vs_r2_r12_LSL_13[] = {
  0x8c, 0x26, 0xe0, 0x61 // mvn vs r2 r12 LSL 13
};
const byte kInstruction_mvn_ge_r10_r1_ROR_22[] = {
  0x61, 0xab, 0xe0, 0xa1 // mvn ge r10 r1 ROR 22
};
const byte kInstruction_mvn_ne_r5_r14_ROR_16[] = {
  0x6e, 0x58, 0xe0, 0x11 // mvn ne r5 r14 ROR 16
};
const byte kInstruction_mvn_ge_r13_r11_LSL_21[] = {
  0x8b, 0xda, 0xe0, 0xa1 // mvn ge r13 r11 LSL 21
};
const byte kInstruction_mvn_ge_r5_r9_ROR_29[] = {
  0xe9, 0x5e, 0xe0, 0xa1 // mvn ge r5 r9 ROR 29
};
const byte kInstruction_mvn_al_r3_r13_ROR_18[] = {
  0x6d, 0x39, 0xe0, 0xe1 // mvn al r3 r13 ROR 18
};
const byte kInstruction_mvn_hi_r0_r12_LSL_10[] = {
  0x0c, 0x05, 0xe0, 0x81 // mvn hi r0 r12 LSL 10
};
const byte kInstruction_mvn_vc_r11_r3_ROR_19[] = {
  0xe3, 0xb9, 0xe0, 0x71 // mvn vc r11 r3 ROR 19
};
const byte kInstruction_mvn_mi_r9_r4_LSL_7[] = {
  0x84, 0x93, 0xe0, 0x41 // mvn mi r9 r4 LSL 7
};
const byte kInstruction_mvn_mi_r0_r14_LSL_31[] = {
  0x8e, 0x0f, 0xe0, 0x41 // mvn mi r0 r14 LSL 31
};
const byte kInstruction_mvn_mi_r2_r5_ROR_11[] = {
  0xe5, 0x25, 0xe0, 0x41 // mvn mi r2 r5 ROR 11
};
const byte kInstruction_mvn_hi_r8_r2_LSL_30[] = {
  0x02, 0x8f, 0xe0, 0x81 // mvn hi r8 r2 LSL 30
};
const byte kInstruction_mvn_cs_r6_r13_ROR_25[] = {
  0xed, 0x6c, 0xe0, 0x21 // mvn cs r6 r13 ROR 25
};
const byte kInstruction_mvn_le_r14_r5_ROR_25[] = {
  0xe5, 0xec, 0xe0, 0xd1 // mvn le r14 r5 ROR 25
};
const byte kInstruction_mvn_le_r12_r2_LSL_20[] = {
  0x02, 0xca, 0xe0, 0xd1 // mvn le r12 r2 LSL 20
};
const byte kInstruction_mvn_le_r4_r1_LSL_23[] = {
  0x81, 0x4b, 0xe0, 0xd1 // mvn le r4 r1 LSL 23
};
const byte kInstruction_mvn_al_r9_r11_LSL_6[] = {
  0x0b, 0x93, 0xe0, 0xe1 // mvn al r9 r11 LSL 6
};
const byte kInstruction_mvn_gt_r10_r13_LSL_1[] = {
  0x8d, 0xa0, 0xe0, 0xc1 // mvn gt r10 r13 LSL 1
};
const byte kInstruction_mvn_vs_r8_r6_ROR_27[] = {
  0xe6, 0x8d, 0xe0, 0x61 // mvn vs r8 r6 ROR 27
};
const byte kInstruction_mvn_vs_r3_r5_LSL_28[] = {
  0x05, 0x3e, 0xe0, 0x61 // mvn vs r3 r5 LSL 28
};
const byte kInstruction_mvn_vs_r7_r5_ROR_28[] = {
  0x65, 0x7e, 0xe0, 0x61 // mvn vs r7 r5 ROR 28
};
const byte kInstruction_mvn_vs_r14_r0_ROR_25[] = {
  0xe0, 0xec, 0xe0, 0x61 // mvn vs r14 r0 ROR 25
};
const byte kInstruction_mvn_ge_r9_r14_ROR_20[] = {
  0x6e, 0x9a, 0xe0, 0xa1 // mvn ge r9 r14 ROR 20
};
const byte kInstruction_mvn_ne_r13_r10_LSL_14[] = {
  0x0a, 0xd7, 0xe0, 0x11 // mvn ne r13 r10 LSL 14
};
const byte kInstruction_mvn_hi_r1_r8_ROR_3[] = {
  0xe8, 0x11, 0xe0, 0x81 // mvn hi r1 r8 ROR 3
};
const byte kInstruction_mvn_mi_r14_r7_ROR_18[] = {
  0x67, 0xe9, 0xe0, 0x41 // mvn mi r14 r7 ROR 18
};
const byte kInstruction_mvn_hi_r13_r14_LSL_30[] = {
  0x0e, 0xdf, 0xe0, 0x81 // mvn hi r13 r14 LSL 30
};
const byte kInstruction_mvn_hi_r5_r7_ROR_26[] = {
  0x67, 0x5d, 0xe0, 0x81 // mvn hi r5 r7 ROR 26
};
const byte kInstruction_mvn_eq_r5_r1_ROR_16[] = {
  0x61, 0x58, 0xe0, 0x01 // mvn eq r5 r1 ROR 16
};
const byte kInstruction_mvn_cs_r1_r1_LSL_14[] = {
  0x01, 0x17, 0xe0, 0x21 // mvn cs r1 r1 LSL 14
};
const byte kInstruction_mvn_le_r3_r7_LSL_5[] = {
  0x87, 0x32, 0xe0, 0xd1 // mvn le r3 r7 LSL 5
};
const byte kInstruction_mvn_eq_r2_r3_LSL_2[] = {
  0x03, 0x21, 0xe0, 0x01 // mvn eq r2 r3 LSL 2
};
const byte kInstruction_mvn_vc_r9_r9_ROR_15[] = {
  0xe9, 0x97, 0xe0, 0x71 // mvn vc r9 r9 ROR 15
};
const byte kInstruction_mvn_pl_r4_r14_ROR_10[] = {
  0x6e, 0x45, 0xe0, 0x51 // mvn pl r4 r14 ROR 10
};
const byte kInstruction_mvn_gt_r4_r4_LSL_25[] = {
  0x84, 0x4c, 0xe0, 0xc1 // mvn gt r4 r4 LSL 25
};
const byte kInstruction_mvn_cc_r6_r1_ROR_12[] = {
  0x61, 0x66, 0xe0, 0x31 // mvn cc r6 r1 ROR 12
};
const byte kInstruction_mvn_hi_r12_r3_LSL_23[] = {
  0x83, 0xcb, 0xe0, 0x81 // mvn hi r12 r3 LSL 23
};
const byte kInstruction_mvn_eq_r7_r1_LSL_9[] = {
  0x81, 0x74, 0xe0, 0x01 // mvn eq r7 r1 LSL 9
};
const TestResult kReferencemvn[] = {
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r5_r5_ROR_10),
    kInstruction_mvn_vc_r5_r5_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r3_r4_ROR_17),
    kInstruction_mvn_ne_r3_r4_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r9_r10_ROR_16),
    kInstruction_mvn_cs_r9_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r0_r2_ROR_29),
    kInstruction_mvn_lt_r0_r2_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r11_r2_ROR_23),
    kInstruction_mvn_al_r11_r2_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r7_r1_LSL_29),
    kInstruction_mvn_hi_r7_r1_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r5_r3_ROR_21),
    kInstruction_mvn_eq_r5_r3_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r2_r10_ROR_13),
    kInstruction_mvn_pl_r2_r10_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r1_r1_LSL_23),
    kInstruction_mvn_lt_r1_r1_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r11_r10_ROR_31),
    kInstruction_mvn_mi_r11_r10_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r3_r5_LSL_8),
    kInstruction_mvn_cs_r3_r5_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r0_r13_LSL_23),
    kInstruction_mvn_gt_r0_r13_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r10_r11_ROR_7),
    kInstruction_mvn_eq_r10_r11_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r4_r3_ROR_18),
    kInstruction_mvn_vs_r4_r3_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r6_r0_ROR_16),
    kInstruction_mvn_ge_r6_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r3_r5_LSL_20),
    kInstruction_mvn_ls_r3_r5_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r10_r13_ROR_31),
    kInstruction_mvn_ge_r10_r13_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r9_r1_ROR_8),
    kInstruction_mvn_cc_r9_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r5_r5_LSL_16),
    kInstruction_mvn_mi_r5_r5_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r14_r6_LSL_4),
    kInstruction_mvn_eq_r14_r6_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r10_r5_ROR_16),
    kInstruction_mvn_lt_r10_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r3_r6_LSL_23),
    kInstruction_mvn_vs_r3_r6_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r0_r6_LSL_30),
    kInstruction_mvn_ls_r0_r6_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r0_r5_ROR_19),
    kInstruction_mvn_vc_r0_r5_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r6_r0_ROR_19),
    kInstruction_mvn_ne_r6_r0_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r10_r13_LSL_7),
    kInstruction_mvn_vc_r10_r13_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r9_r13_ROR_9),
    kInstruction_mvn_pl_r9_r13_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r6_r13_ROR_6),
    kInstruction_mvn_vs_r6_r13_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r12_r0_ROR_11),
    kInstruction_mvn_mi_r12_r0_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r8_r12_LSL_9),
    kInstruction_mvn_vs_r8_r12_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r2_r3_LSL_10),
    kInstruction_mvn_ls_r2_r3_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r14_r9_ROR_18),
    kInstruction_mvn_gt_r14_r9_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r7_r1_LSL_5),
    kInstruction_mvn_pl_r7_r1_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r3_r14_ROR_5),
    kInstruction_mvn_cc_r3_r14_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r3_r10_LSL_13),
    kInstruction_mvn_vc_r3_r10_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r13_r12_LSL_25),
    kInstruction_mvn_mi_r13_r12_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r12_r7_LSL_17),
    kInstruction_mvn_mi_r12_r7_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r10_r14_LSL_18),
    kInstruction_mvn_al_r10_r14_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r7_r4_ROR_30),
    kInstruction_mvn_cc_r7_r4_ROR_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r0_r1_ROR_8),
    kInstruction_mvn_al_r0_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r5_r7_LSL_16),
    kInstruction_mvn_eq_r5_r7_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r1_r2_ROR_25),
    kInstruction_mvn_cc_r1_r2_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r9_r6_LSL_18),
    kInstruction_mvn_gt_r9_r6_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r4_r6_LSL_25),
    kInstruction_mvn_lt_r4_r6_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r9_r3_ROR_4),
    kInstruction_mvn_ge_r9_r3_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r8_r7_LSL_2),
    kInstruction_mvn_le_r8_r7_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r7_r2_LSL_4),
    kInstruction_mvn_eq_r7_r2_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r7_r11_ROR_21),
    kInstruction_mvn_vc_r7_r11_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r14_r14_LSL_15),
    kInstruction_mvn_vc_r14_r14_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r10_r3_LSL_5),
    kInstruction_mvn_vc_r10_r3_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r6_r12_ROR_15),
    kInstruction_mvn_vs_r6_r12_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r6_r12_ROR_3),
    kInstruction_mvn_ls_r6_r12_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r12_r10_ROR_17),
    kInstruction_mvn_vs_r12_r10_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r6_r13_ROR_9),
    kInstruction_mvn_eq_r6_r13_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r10_r9_ROR_18),
    kInstruction_mvn_mi_r10_r9_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r0_r7_LSL_13),
    kInstruction_mvn_eq_r0_r7_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r9_r13_ROR_16),
    kInstruction_mvn_eq_r9_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r7_r9_ROR_21),
    kInstruction_mvn_eq_r7_r9_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r12_r6_ROR_30),
    kInstruction_mvn_gt_r12_r6_ROR_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r4_r8_ROR_29),
    kInstruction_mvn_ge_r4_r8_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r7_r9_LSL_21),
    kInstruction_mvn_hi_r7_r9_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r3_r13_LSL_10),
    kInstruction_mvn_le_r3_r13_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r9_r13_LSL_18),
    kInstruction_mvn_gt_r9_r13_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r3_r11_ROR_20),
    kInstruction_mvn_pl_r3_r11_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r1_r3_LSL_5),
    kInstruction_mvn_le_r1_r3_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r4_r6_LSL_12),
    kInstruction_mvn_mi_r4_r6_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r1_r5_ROR_27),
    kInstruction_mvn_pl_r1_r5_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r9_r5_LSL_4),
    kInstruction_mvn_gt_r9_r5_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r4_r8_LSL_11),
    kInstruction_mvn_eq_r4_r8_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r1_r8_ROR_13),
    kInstruction_mvn_mi_r1_r8_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r7_r0_ROR_20),
    kInstruction_mvn_al_r7_r0_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r5_r7_ROR_28),
    kInstruction_mvn_vs_r5_r7_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r0_r12_LSL_31),
    kInstruction_mvn_cs_r0_r12_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r11_r12_LSL_25),
    kInstruction_mvn_gt_r11_r12_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r2_r13_ROR_19),
    kInstruction_mvn_gt_r2_r13_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r4_r7_ROR_7),
    kInstruction_mvn_cs_r4_r7_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r9_r8_ROR_24),
    kInstruction_mvn_gt_r9_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r10_r2_ROR_4),
    kInstruction_mvn_lt_r10_r2_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r4_r2_ROR_20),
    kInstruction_mvn_lt_r4_r2_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r2_r0_LSL_18),
    kInstruction_mvn_ge_r2_r0_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r13_r14_LSL_24),
    kInstruction_mvn_hi_r13_r14_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r8_r0_ROR_3),
    kInstruction_mvn_al_r8_r0_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r11_r2_LSL_6),
    kInstruction_mvn_ne_r11_r2_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r14_r1_LSL_6),
    kInstruction_mvn_cs_r14_r1_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r13_r1_LSL_12),
    kInstruction_mvn_al_r13_r1_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r14_r1_ROR_7),
    kInstruction_mvn_ls_r14_r1_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r11_r2_ROR_1),
    kInstruction_mvn_cs_r11_r2_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r9_r1_ROR_19),
    kInstruction_mvn_al_r9_r1_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r14_r9_ROR_2),
    kInstruction_mvn_ls_r14_r9_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r13_r2_ROR_13),
    kInstruction_mvn_ne_r13_r2_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r5_r1_LSL_4),
    kInstruction_mvn_vs_r5_r1_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r13_r5_ROR_11),
    kInstruction_mvn_cc_r13_r5_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r5_r1_ROR_12),
    kInstruction_mvn_eq_r5_r1_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r9_r7_LSL_16),
    kInstruction_mvn_al_r9_r7_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r9_r10_LSL_20),
    kInstruction_mvn_eq_r9_r10_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r4_r13_ROR_3),
    kInstruction_mvn_eq_r4_r13_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r14_r1_LSL_18),
    kInstruction_mvn_cs_r14_r1_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r5_r7_ROR_1),
    kInstruction_mvn_cc_r5_r7_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r5_r8_ROR_30),
    kInstruction_mvn_mi_r5_r8_ROR_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r8_r0_LSL_4),
    kInstruction_mvn_eq_r8_r0_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r2_r1_ROR_30),
    kInstruction_mvn_mi_r2_r1_ROR_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r7_r5_LSL_29),
    kInstruction_mvn_vs_r7_r5_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r11_r5_ROR_4),
    kInstruction_mvn_le_r11_r5_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r6_r8_LSL_22),
    kInstruction_mvn_gt_r6_r8_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r13_r4_LSL_8),
    kInstruction_mvn_al_r13_r4_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r4_r5_ROR_22),
    kInstruction_mvn_lt_r4_r5_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r1_r14_LSL_12),
    kInstruction_mvn_mi_r1_r14_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r4_r12_LSL_8),
    kInstruction_mvn_gt_r4_r12_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r1_r13_ROR_15),
    kInstruction_mvn_gt_r1_r13_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r14_r5_LSL_23),
    kInstruction_mvn_lt_r14_r5_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r2_r14_LSL_21),
    kInstruction_mvn_ls_r2_r14_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r12_r0_LSL_12),
    kInstruction_mvn_vc_r12_r0_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r2_r1_ROR_18),
    kInstruction_mvn_al_r2_r1_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r12_r5_ROR_2),
    kInstruction_mvn_vc_r12_r5_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r11_r9_LSL_13),
    kInstruction_mvn_mi_r11_r9_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r8_r12_ROR_3),
    kInstruction_mvn_lt_r8_r12_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r0_r2_ROR_31),
    kInstruction_mvn_vs_r0_r2_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r5_r5_ROR_9),
    kInstruction_mvn_hi_r5_r5_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r8_r0_LSL_4),
    kInstruction_mvn_ne_r8_r0_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r2_r4_LSL_29),
    kInstruction_mvn_gt_r2_r4_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r11_r9_LSL_28),
    kInstruction_mvn_le_r11_r9_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r13_r8_LSL_26),
    kInstruction_mvn_ge_r13_r8_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r5_r13_ROR_25),
    kInstruction_mvn_gt_r5_r13_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r9_r11_ROR_28),
    kInstruction_mvn_cc_r9_r11_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r12_r14_ROR_18),
    kInstruction_mvn_vs_r12_r14_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r6_r11_ROR_2),
    kInstruction_mvn_lt_r6_r11_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r12_r6_LSL_7),
    kInstruction_mvn_cs_r12_r6_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r1_r5_LSL_8),
    kInstruction_mvn_gt_r1_r5_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r3_r9_ROR_28),
    kInstruction_mvn_vc_r3_r9_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r1_r6_ROR_13),
    kInstruction_mvn_eq_r1_r6_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r13_r0_LSL_31),
    kInstruction_mvn_ge_r13_r0_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r7_r10_ROR_4),
    kInstruction_mvn_pl_r7_r10_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r7_r4_ROR_22),
    kInstruction_mvn_eq_r7_r4_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r14_r12_LSL_25),
    kInstruction_mvn_gt_r14_r12_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r2_r11_ROR_22),
    kInstruction_mvn_eq_r2_r11_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r4_r11_ROR_17),
    kInstruction_mvn_ge_r4_r11_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r1_r9_LSL_12),
    kInstruction_mvn_hi_r1_r9_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r2_r12_ROR_6),
    kInstruction_mvn_mi_r2_r12_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r6_r9_ROR_15),
    kInstruction_mvn_vs_r6_r9_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r6_r4_LSL_27),
    kInstruction_mvn_mi_r6_r4_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r5_r10_LSL_7),
    kInstruction_mvn_vc_r5_r10_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r4_r12_LSL_6),
    kInstruction_mvn_ls_r4_r12_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r11_r4_ROR_11),
    kInstruction_mvn_mi_r11_r4_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r0_r14_ROR_27),
    kInstruction_mvn_le_r0_r14_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r4_r9_ROR_18),
    kInstruction_mvn_gt_r4_r9_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r0_r3_ROR_10),
    kInstruction_mvn_hi_r0_r3_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r13_r3_ROR_25),
    kInstruction_mvn_pl_r13_r3_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r1_r14_ROR_1),
    kInstruction_mvn_pl_r1_r14_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r7_r7_ROR_31),
    kInstruction_mvn_ls_r7_r7_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r12_r11_LSL_5),
    kInstruction_mvn_gt_r12_r11_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r3_r10_LSL_30),
    kInstruction_mvn_lt_r3_r10_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r14_r3_ROR_24),
    kInstruction_mvn_eq_r14_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r14_r5_ROR_18),
    kInstruction_mvn_le_r14_r5_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r4_r6_ROR_9),
    kInstruction_mvn_vs_r4_r6_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r3_r1_ROR_25),
    kInstruction_mvn_vc_r3_r1_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r8_r13_ROR_18),
    kInstruction_mvn_vc_r8_r13_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r7_r13_LSL_28),
    kInstruction_mvn_ls_r7_r13_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r6_r11_LSL_18),
    kInstruction_mvn_vs_r6_r11_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r7_r4_LSL_22),
    kInstruction_mvn_lt_r7_r4_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r4_r6_ROR_2),
    kInstruction_mvn_pl_r4_r6_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r2_r2_ROR_20),
    kInstruction_mvn_ls_r2_r2_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r8_r12_LSL_31),
    kInstruction_mvn_ge_r8_r12_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r9_r9_LSL_26),
    kInstruction_mvn_cc_r9_r9_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r13_r13_ROR_22),
    kInstruction_mvn_hi_r13_r13_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r0_r13_ROR_26),
    kInstruction_mvn_mi_r0_r13_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r9_r11_LSL_20),
    kInstruction_mvn_eq_r9_r11_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r12_r12_LSL_30),
    kInstruction_mvn_cs_r12_r12_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r9_r8_LSL_17),
    kInstruction_mvn_mi_r9_r8_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r0_r10_ROR_28),
    kInstruction_mvn_cc_r0_r10_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r3_r13_ROR_19),
    kInstruction_mvn_lt_r3_r13_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r14_r9_LSL_22),
    kInstruction_mvn_ne_r14_r9_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r8_r13_LSL_24),
    kInstruction_mvn_ne_r8_r13_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r9_r5_LSL_17),
    kInstruction_mvn_eq_r9_r5_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r5_r10_LSL_19),
    kInstruction_mvn_al_r5_r10_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r13_r13_LSL_7),
    kInstruction_mvn_ge_r13_r13_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r14_r4_LSL_30),
    kInstruction_mvn_vc_r14_r4_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r7_r12_LSL_8),
    kInstruction_mvn_pl_r7_r12_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r3_r6_LSL_23),
    kInstruction_mvn_eq_r3_r6_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r2_r7_ROR_1),
    kInstruction_mvn_le_r2_r7_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r0_r8_LSL_25),
    kInstruction_mvn_al_r0_r8_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r2_r8_LSL_25),
    kInstruction_mvn_le_r2_r8_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r5_r4_ROR_7),
    kInstruction_mvn_ge_r5_r4_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r1_r13_LSL_13),
    kInstruction_mvn_vc_r1_r13_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r5_r11_ROR_23),
    kInstruction_mvn_ge_r5_r11_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r13_r0_ROR_29),
    kInstruction_mvn_eq_r13_r0_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r2_r14_LSL_28),
    kInstruction_mvn_vc_r2_r14_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r14_r7_LSL_28),
    kInstruction_mvn_lt_r14_r7_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r2_r6_ROR_20),
    kInstruction_mvn_gt_r2_r6_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r10_r12_ROR_3),
    kInstruction_mvn_pl_r10_r12_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r6_r3_ROR_3),
    kInstruction_mvn_vc_r6_r3_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r11_r12_LSL_28),
    kInstruction_mvn_cc_r11_r12_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r7_r3_LSL_11),
    kInstruction_mvn_cs_r7_r3_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r2_r7_ROR_1),
    kInstruction_mvn_mi_r2_r7_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r1_r4_ROR_15),
    kInstruction_mvn_vc_r1_r4_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r11_r11_ROR_5),
    kInstruction_mvn_le_r11_r11_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r9_r2_LSL_10),
    kInstruction_mvn_vc_r9_r2_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r2_r10_LSL_17),
    kInstruction_mvn_hi_r2_r10_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r11_r4_ROR_13),
    kInstruction_mvn_al_r11_r4_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r2_r14_LSL_20),
    kInstruction_mvn_le_r2_r14_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r7_r9_LSL_3),
    kInstruction_mvn_cc_r7_r9_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r11_r4_LSL_14),
    kInstruction_mvn_hi_r11_r4_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r14_r7_LSL_24),
    kInstruction_mvn_gt_r14_r7_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r9_r0_ROR_25),
    kInstruction_mvn_ne_r9_r0_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r13_r2_LSL_4),
    kInstruction_mvn_pl_r13_r2_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r1_r14_LSL_15),
    kInstruction_mvn_lt_r1_r14_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r7_r13_ROR_31),
    kInstruction_mvn_cc_r7_r13_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r11_r5_LSL_12),
    kInstruction_mvn_ne_r11_r5_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r14_r3_LSL_4),
    kInstruction_mvn_lt_r14_r3_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r0_r14_ROR_29),
    kInstruction_mvn_vs_r0_r14_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r7_r2_LSL_4),
    kInstruction_mvn_gt_r7_r2_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r13_r6_ROR_22),
    kInstruction_mvn_vs_r13_r6_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r14_r2_ROR_24),
    kInstruction_mvn_pl_r14_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r2_r6_ROR_6),
    kInstruction_mvn_cc_r2_r6_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r14_r13_ROR_10),
    kInstruction_mvn_vs_r14_r13_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r8_r11_ROR_13),
    kInstruction_mvn_gt_r8_r11_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r9_r12_ROR_21),
    kInstruction_mvn_gt_r9_r12_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r3_r11_ROR_17),
    kInstruction_mvn_al_r3_r11_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r0_r0_LSL_25),
    kInstruction_mvn_hi_r0_r0_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r14_r9_LSL_12),
    kInstruction_mvn_ge_r14_r9_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r9_r8_ROR_5),
    kInstruction_mvn_pl_r9_r8_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r6_r0_ROR_2),
    kInstruction_mvn_ge_r6_r0_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r2_r6_ROR_21),
    kInstruction_mvn_vs_r2_r6_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r14_r2_LSL_18),
    kInstruction_mvn_vs_r14_r2_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r11_r9_ROR_30),
    kInstruction_mvn_mi_r11_r9_ROR_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r10_r12_ROR_4),
    kInstruction_mvn_cc_r10_r12_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r2_r4_ROR_31),
    kInstruction_mvn_vc_r2_r4_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r0_r12_ROR_2),
    kInstruction_mvn_ge_r0_r12_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r13_r14_LSL_11),
    kInstruction_mvn_cc_r13_r14_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r3_r3_ROR_26),
    kInstruction_mvn_cs_r3_r3_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r0_r0_ROR_21),
    kInstruction_mvn_cc_r0_r0_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r13_r11_ROR_19),
    kInstruction_mvn_vc_r13_r11_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r0_r7_ROR_6),
    kInstruction_mvn_al_r0_r7_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r0_r8_ROR_19),
    kInstruction_mvn_mi_r0_r8_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r1_r13_ROR_20),
    kInstruction_mvn_cs_r1_r13_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r2_r13_ROR_28),
    kInstruction_mvn_vs_r2_r13_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r5_r6_LSL_23),
    kInstruction_mvn_mi_r5_r6_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r8_r13_ROR_3),
    kInstruction_mvn_ne_r8_r13_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r9_r12_ROR_24),
    kInstruction_mvn_le_r9_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r4_r0_LSL_10),
    kInstruction_mvn_le_r4_r0_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r6_r1_ROR_5),
    kInstruction_mvn_hi_r6_r1_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r1_r5_ROR_31),
    kInstruction_mvn_pl_r1_r5_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r1_r13_LSL_7),
    kInstruction_mvn_lt_r1_r13_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r8_r10_LSL_24),
    kInstruction_mvn_ge_r8_r10_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r10_r2_ROR_6),
    kInstruction_mvn_eq_r10_r2_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r7_r2_LSL_3),
    kInstruction_mvn_ls_r7_r2_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r13_r6_ROR_19),
    kInstruction_mvn_al_r13_r6_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r10_r2_ROR_26),
    kInstruction_mvn_le_r10_r2_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r11_r7_LSL_25),
    kInstruction_mvn_ls_r11_r7_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r7_r0_LSL_5),
    kInstruction_mvn_eq_r7_r0_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r3_r11_LSL_23),
    kInstruction_mvn_lt_r3_r11_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r13_r0_ROR_13),
    kInstruction_mvn_ls_r13_r0_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r4_r10_LSL_22),
    kInstruction_mvn_mi_r4_r10_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r6_r8_LSL_6),
    kInstruction_mvn_pl_r6_r8_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r13_r2_ROR_16),
    kInstruction_mvn_le_r13_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r6_r8_ROR_9),
    kInstruction_mvn_al_r6_r8_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r11_r14_ROR_4),
    kInstruction_mvn_lt_r11_r14_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r14_r14_ROR_9),
    kInstruction_mvn_ls_r14_r14_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r1_r0_LSL_23),
    kInstruction_mvn_gt_r1_r0_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r4_r0_ROR_25),
    kInstruction_mvn_lt_r4_r0_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r4_r12_ROR_18),
    kInstruction_mvn_hi_r4_r12_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r10_r2_ROR_28),
    kInstruction_mvn_ne_r10_r2_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r1_r4_ROR_2),
    kInstruction_mvn_lt_r1_r4_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r4_r10_ROR_23),
    kInstruction_mvn_mi_r4_r10_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r12_r7_LSL_9),
    kInstruction_mvn_cs_r12_r7_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r7_r14_ROR_4),
    kInstruction_mvn_lt_r7_r14_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r2_r10_ROR_25),
    kInstruction_mvn_lt_r2_r10_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r4_r9_ROR_24),
    kInstruction_mvn_hi_r4_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r0_r12_ROR_29),
    kInstruction_mvn_hi_r0_r12_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r8_r9_LSL_10),
    kInstruction_mvn_ge_r8_r9_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r2_r13_LSL_21),
    kInstruction_mvn_ge_r2_r13_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r8_r12_LSL_28),
    kInstruction_mvn_lt_r8_r12_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r13_r11_LSL_20),
    kInstruction_mvn_hi_r13_r11_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r6_r0_LSL_17),
    kInstruction_mvn_eq_r6_r0_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r1_r0_ROR_15),
    kInstruction_mvn_vc_r1_r0_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r13_r11_LSL_7),
    kInstruction_mvn_ge_r13_r11_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r6_r10_LSL_11),
    kInstruction_mvn_mi_r6_r10_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r1_r8_ROR_26),
    kInstruction_mvn_mi_r1_r8_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r12_r6_LSL_1),
    kInstruction_mvn_vs_r12_r6_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r5_r13_LSL_12),
    kInstruction_mvn_lt_r5_r13_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r13_r10_LSL_7),
    kInstruction_mvn_pl_r13_r10_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r3_r8_LSL_2),
    kInstruction_mvn_hi_r3_r8_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r12_r9_ROR_5),
    kInstruction_mvn_ge_r12_r9_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r11_r11_LSL_21),
    kInstruction_mvn_lt_r11_r11_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r13_r11_LSL_10),
    kInstruction_mvn_le_r13_r11_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r0_r8_ROR_19),
    kInstruction_mvn_cs_r0_r8_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r1_r13_ROR_19),
    kInstruction_mvn_ge_r1_r13_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r14_r7_ROR_16),
    kInstruction_mvn_al_r14_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r7_r12_LSL_22),
    kInstruction_mvn_lt_r7_r12_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r3_r12_LSL_25),
    kInstruction_mvn_al_r3_r12_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r10_r0_ROR_8),
    kInstruction_mvn_cc_r10_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r10_r4_LSL_26),
    kInstruction_mvn_ls_r10_r4_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r11_r13_ROR_28),
    kInstruction_mvn_hi_r11_r13_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r14_r14_LSL_28),
    kInstruction_mvn_le_r14_r14_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r4_r8_LSL_31),
    kInstruction_mvn_le_r4_r8_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r8_r6_LSL_1),
    kInstruction_mvn_ge_r8_r6_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r8_r2_ROR_10),
    kInstruction_mvn_ne_r8_r2_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r8_r2_LSL_18),
    kInstruction_mvn_pl_r8_r2_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r6_r9_LSL_5),
    kInstruction_mvn_ne_r6_r9_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r0_r0_ROR_16),
    kInstruction_mvn_lt_r0_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r6_r1_ROR_13),
    kInstruction_mvn_le_r6_r1_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r10_r11_ROR_5),
    kInstruction_mvn_hi_r10_r11_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r13_r14_ROR_6),
    kInstruction_mvn_al_r13_r14_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r8_r1_LSL_29),
    kInstruction_mvn_vs_r8_r1_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r12_r5_LSL_9),
    kInstruction_mvn_vc_r12_r5_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r3_r5_ROR_31),
    kInstruction_mvn_vs_r3_r5_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r12_r1_ROR_14),
    kInstruction_mvn_cs_r12_r1_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r0_r9_ROR_15),
    kInstruction_mvn_vc_r0_r9_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r7_r0_LSL_8),
    kInstruction_mvn_ls_r7_r0_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r5_r6_LSL_28),
    kInstruction_mvn_ge_r5_r6_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r6_r13_LSL_30),
    kInstruction_mvn_cs_r6_r13_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r9_r10_ROR_17),
    kInstruction_mvn_mi_r9_r10_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r6_r7_ROR_8),
    kInstruction_mvn_ne_r6_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r12_r6_ROR_11),
    kInstruction_mvn_ge_r12_r6_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r9_r1_LSL_28),
    kInstruction_mvn_eq_r9_r1_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r14_r11_LSL_30),
    kInstruction_mvn_gt_r14_r11_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r11_r10_LSL_24),
    kInstruction_mvn_ls_r11_r10_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r13_r12_LSL_29),
    kInstruction_mvn_ls_r13_r12_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r12_r4_LSL_7),
    kInstruction_mvn_hi_r12_r4_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r11_r1_ROR_26),
    kInstruction_mvn_ls_r11_r1_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r14_r6_LSL_10),
    kInstruction_mvn_ls_r14_r6_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r11_r10_ROR_18),
    kInstruction_mvn_le_r11_r10_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r0_r10_LSL_15),
    kInstruction_mvn_cc_r0_r10_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r7_r14_ROR_29),
    kInstruction_mvn_eq_r7_r14_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r13_r3_ROR_19),
    kInstruction_mvn_lt_r13_r3_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r7_r3_ROR_4),
    kInstruction_mvn_ge_r7_r3_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r3_r3_ROR_24),
    kInstruction_mvn_lt_r3_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r4_r1_ROR_14),
    kInstruction_mvn_pl_r4_r1_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r1_r9_LSL_4),
    kInstruction_mvn_lt_r1_r9_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r5_r11_ROR_15),
    kInstruction_mvn_vs_r5_r11_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r3_r12_LSL_22),
    kInstruction_mvn_gt_r3_r12_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r14_r5_LSL_18),
    kInstruction_mvn_vs_r14_r5_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r11_r6_ROR_9),
    kInstruction_mvn_ls_r11_r6_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r8_r6_LSL_6),
    kInstruction_mvn_mi_r8_r6_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r12_r12_ROR_18),
    kInstruction_mvn_lt_r12_r12_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r11_r5_ROR_1),
    kInstruction_mvn_ls_r11_r5_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r5_r9_LSL_26),
    kInstruction_mvn_ne_r5_r9_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r4_r10_ROR_5),
    kInstruction_mvn_eq_r4_r10_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r14_r11_ROR_7),
    kInstruction_mvn_ne_r14_r11_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r3_r10_ROR_28),
    kInstruction_mvn_eq_r3_r10_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r1_r11_LSL_13),
    kInstruction_mvn_vc_r1_r11_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r14_r3_LSL_18),
    kInstruction_mvn_ge_r14_r3_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r12_r11_LSL_19),
    kInstruction_mvn_al_r12_r11_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r8_r11_LSL_10),
    kInstruction_mvn_al_r8_r11_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r0_r2_LSL_17),
    kInstruction_mvn_mi_r0_r2_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r8_r4_LSL_30),
    kInstruction_mvn_ls_r8_r4_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r9_r3_LSL_5),
    kInstruction_mvn_gt_r9_r3_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r13_r11_ROR_7),
    kInstruction_mvn_ne_r13_r11_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r8_r1_ROR_13),
    kInstruction_mvn_gt_r8_r1_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r2_r1_LSL_31),
    kInstruction_mvn_ne_r2_r1_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r12_r0_ROR_23),
    kInstruction_mvn_le_r12_r0_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r14_r0_ROR_20),
    kInstruction_mvn_ge_r14_r0_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r13_r4_ROR_9),
    kInstruction_mvn_ne_r13_r4_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r12_r8_ROR_9),
    kInstruction_mvn_lt_r12_r8_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r1_r14_LSL_27),
    kInstruction_mvn_gt_r1_r14_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r0_r14_ROR_5),
    kInstruction_mvn_ls_r0_r14_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r6_r5_ROR_2),
    kInstruction_mvn_lt_r6_r5_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r1_r13_LSL_7),
    kInstruction_mvn_vs_r1_r13_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r13_r2_LSL_26),
    kInstruction_mvn_lt_r13_r2_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r11_r1_LSL_15),
    kInstruction_mvn_cs_r11_r1_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r2_r4_ROR_15),
    kInstruction_mvn_cs_r2_r4_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r9_r10_ROR_28),
    kInstruction_mvn_gt_r9_r10_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r4_r11_LSL_16),
    kInstruction_mvn_ge_r4_r11_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r6_r2_LSL_2),
    kInstruction_mvn_mi_r6_r2_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r0_r11_LSL_12),
    kInstruction_mvn_ls_r0_r11_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r13_r13_LSL_1),
    kInstruction_mvn_mi_r13_r13_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r1_r11_LSL_9),
    kInstruction_mvn_ge_r1_r11_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r7_r4_ROR_29),
    kInstruction_mvn_gt_r7_r4_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r9_r14_LSL_5),
    kInstruction_mvn_eq_r9_r14_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r8_r4_ROR_11),
    kInstruction_mvn_eq_r8_r4_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r5_r6_LSL_23),
    kInstruction_mvn_ge_r5_r6_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r5_r4_LSL_23),
    kInstruction_mvn_lt_r5_r4_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r6_r3_ROR_21),
    kInstruction_mvn_ge_r6_r3_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r4_r6_ROR_2),
    kInstruction_mvn_lt_r4_r6_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r7_r9_ROR_14),
    kInstruction_mvn_hi_r7_r9_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r9_r9_LSL_9),
    kInstruction_mvn_pl_r9_r9_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r4_r1_ROR_8),
    kInstruction_mvn_gt_r4_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r13_r0_ROR_8),
    kInstruction_mvn_pl_r13_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r1_r12_LSL_12),
    kInstruction_mvn_vc_r1_r12_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r12_r5_LSL_18),
    kInstruction_mvn_ge_r12_r5_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r8_r2_LSL_7),
    kInstruction_mvn_le_r8_r2_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r8_r3_ROR_26),
    kInstruction_mvn_ne_r8_r3_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r8_r0_LSL_9),
    kInstruction_mvn_lt_r8_r0_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r11_r8_LSL_28),
    kInstruction_mvn_cc_r11_r8_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r14_r14_ROR_29),
    kInstruction_mvn_cc_r14_r14_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r4_r9_LSL_31),
    kInstruction_mvn_vc_r4_r9_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r14_r0_ROR_22),
    kInstruction_mvn_ne_r14_r0_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r14_r5_LSL_2),
    kInstruction_mvn_al_r14_r5_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r9_r5_LSL_20),
    kInstruction_mvn_ge_r9_r5_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r7_r5_ROR_12),
    kInstruction_mvn_cc_r7_r5_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r0_r12_LSL_18),
    kInstruction_mvn_le_r0_r12_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r6_r2_ROR_4),
    kInstruction_mvn_ne_r6_r2_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r4_r2_ROR_30),
    kInstruction_mvn_mi_r4_r2_ROR_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r12_r1_ROR_21),
    kInstruction_mvn_ls_r12_r1_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r11_r6_LSL_30),
    kInstruction_mvn_ne_r11_r6_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r14_r2_ROR_16),
    kInstruction_mvn_vs_r14_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r2_r1_LSL_5),
    kInstruction_mvn_le_r2_r1_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r12_r14_ROR_16),
    kInstruction_mvn_vs_r12_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r10_r12_ROR_15),
    kInstruction_mvn_hi_r10_r12_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r10_r5_ROR_11),
    kInstruction_mvn_ls_r10_r5_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r6_r0_ROR_27),
    kInstruction_mvn_vc_r6_r0_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r8_r13_LSL_15),
    kInstruction_mvn_vs_r8_r13_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r9_r7_ROR_26),
    kInstruction_mvn_lt_r9_r7_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r6_r14_ROR_23),
    kInstruction_mvn_gt_r6_r14_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r4_r11_LSL_21),
    kInstruction_mvn_hi_r4_r11_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r14_r12_LSL_14),
    kInstruction_mvn_ls_r14_r12_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r9_r4_LSL_9),
    kInstruction_mvn_cc_r9_r4_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r5_r1_LSL_26),
    kInstruction_mvn_eq_r5_r1_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r10_r7_LSL_20),
    kInstruction_mvn_pl_r10_r7_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r2_r14_LSL_25),
    kInstruction_mvn_gt_r2_r14_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r8_r1_LSL_17),
    kInstruction_mvn_ge_r8_r1_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r5_r7_ROR_19),
    kInstruction_mvn_ne_r5_r7_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r4_r9_LSL_20),
    kInstruction_mvn_pl_r4_r9_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r14_r1_LSL_18),
    kInstruction_mvn_gt_r14_r1_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r1_r4_LSL_1),
    kInstruction_mvn_ge_r1_r4_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r14_r0_LSL_18),
    kInstruction_mvn_cc_r14_r0_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r12_r13_LSL_16),
    kInstruction_mvn_vc_r12_r13_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r10_r8_LSL_15),
    kInstruction_mvn_ls_r10_r8_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r4_r4_ROR_22),
    kInstruction_mvn_ne_r4_r4_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r4_r5_LSL_29),
    kInstruction_mvn_le_r4_r5_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r10_r5_LSL_19),
    kInstruction_mvn_hi_r10_r5_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r6_r10_LSL_26),
    kInstruction_mvn_gt_r6_r10_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r3_r7_LSL_19),
    kInstruction_mvn_lt_r3_r7_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r9_r9_ROR_17),
    kInstruction_mvn_vs_r9_r9_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r0_r0_ROR_21),
    kInstruction_mvn_vc_r0_r0_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r5_r10_LSL_20),
    kInstruction_mvn_le_r5_r10_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r3_r6_ROR_26),
    kInstruction_mvn_lt_r3_r6_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r0_r13_ROR_31),
    kInstruction_mvn_vs_r0_r13_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r9_r7_ROR_18),
    kInstruction_mvn_le_r9_r7_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r2_r8_LSL_10),
    kInstruction_mvn_le_r2_r8_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r0_r5_LSL_13),
    kInstruction_mvn_eq_r0_r5_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r0_r4_ROR_3),
    kInstruction_mvn_vs_r0_r4_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r4_r6_LSL_16),
    kInstruction_mvn_cs_r4_r6_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r5_r13_LSL_19),
    kInstruction_mvn_hi_r5_r13_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r2_r9_ROR_31),
    kInstruction_mvn_cc_r2_r9_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r2_r11_ROR_7),
    kInstruction_mvn_ls_r2_r11_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r9_r6_LSL_21),
    kInstruction_mvn_ne_r9_r6_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r7_r9_ROR_8),
    kInstruction_mvn_le_r7_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r11_r13_LSL_2),
    kInstruction_mvn_ls_r11_r13_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r6_r9_LSL_17),
    kInstruction_mvn_cs_r6_r9_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r12_r1_ROR_21),
    kInstruction_mvn_vs_r12_r1_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r11_r10_ROR_27),
    kInstruction_mvn_vc_r11_r10_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r4_r14_LSL_9),
    kInstruction_mvn_ge_r4_r14_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r0_r7_LSL_14),
    kInstruction_mvn_eq_r0_r7_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r1_r11_LSL_6),
    kInstruction_mvn_le_r1_r11_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r8_r7_LSL_7),
    kInstruction_mvn_eq_r8_r7_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r8_r7_ROR_2),
    kInstruction_mvn_ge_r8_r7_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r3_r4_LSL_17),
    kInstruction_mvn_hi_r3_r4_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r8_r11_ROR_18),
    kInstruction_mvn_mi_r8_r11_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r4_r10_LSL_11),
    kInstruction_mvn_pl_r4_r10_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r10_r2_LSL_15),
    kInstruction_mvn_pl_r10_r2_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r3_r1_LSL_5),
    kInstruction_mvn_ne_r3_r1_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r10_r13_LSL_3),
    kInstruction_mvn_ls_r10_r13_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r8_r11_ROR_11),
    kInstruction_mvn_vs_r8_r11_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r13_r10_LSL_8),
    kInstruction_mvn_gt_r13_r10_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r8_r6_LSL_11),
    kInstruction_mvn_hi_r8_r6_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r8_r3_ROR_13),
    kInstruction_mvn_vc_r8_r3_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r5_r14_ROR_5),
    kInstruction_mvn_lt_r5_r14_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r11_r4_ROR_23),
    kInstruction_mvn_hi_r11_r4_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r5_r6_ROR_3),
    kInstruction_mvn_al_r5_r6_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r2_r2_LSL_10),
    kInstruction_mvn_pl_r2_r2_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r0_r7_LSL_9),
    kInstruction_mvn_mi_r0_r7_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r7_r5_LSL_11),
    kInstruction_mvn_gt_r7_r5_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r10_r11_ROR_18),
    kInstruction_mvn_mi_r10_r11_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r7_r10_ROR_10),
    kInstruction_mvn_cc_r7_r10_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r8_r9_ROR_31),
    kInstruction_mvn_hi_r8_r9_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r8_r12_LSL_16),
    kInstruction_mvn_cs_r8_r12_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r4_r0_LSL_11),
    kInstruction_mvn_gt_r4_r0_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r4_r12_LSL_3),
    kInstruction_mvn_cc_r4_r12_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r13_r3_LSL_9),
    kInstruction_mvn_al_r13_r3_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r0_r4_LSL_4),
    kInstruction_mvn_cs_r0_r4_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r10_r14_LSL_8),
    kInstruction_mvn_hi_r10_r14_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r14_r13_ROR_15),
    kInstruction_mvn_ge_r14_r13_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r8_r8_LSL_25),
    kInstruction_mvn_vc_r8_r8_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r11_r3_ROR_23),
    kInstruction_mvn_hi_r11_r3_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r11_r7_LSL_14),
    kInstruction_mvn_al_r11_r7_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r0_r2_LSL_21),
    kInstruction_mvn_ls_r0_r2_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r2_r3_LSL_24),
    kInstruction_mvn_ne_r2_r3_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r11_r14_ROR_5),
    kInstruction_mvn_gt_r11_r14_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r9_r4_LSL_18),
    kInstruction_mvn_hi_r9_r4_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r10_r6_LSL_3),
    kInstruction_mvn_cs_r10_r6_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r10_r12_LSL_28),
    kInstruction_mvn_hi_r10_r12_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r12_r14_LSL_4),
    kInstruction_mvn_pl_r12_r14_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r12_r5_ROR_18),
    kInstruction_mvn_ls_r12_r5_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r0_r5_ROR_3),
    kInstruction_mvn_eq_r0_r5_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r4_r2_ROR_11),
    kInstruction_mvn_cc_r4_r2_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r4_r8_ROR_31),
    kInstruction_mvn_lt_r4_r8_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r9_r7_LSL_31),
    kInstruction_mvn_cc_r9_r7_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r4_r5_ROR_25),
    kInstruction_mvn_cs_r4_r5_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r13_r10_LSL_14),
    kInstruction_mvn_mi_r13_r10_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r9_r3_ROR_13),
    kInstruction_mvn_gt_r9_r3_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r4_r11_LSL_1),
    kInstruction_mvn_lt_r4_r11_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r10_r4_LSL_19),
    kInstruction_mvn_le_r10_r4_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r12_r10_ROR_14),
    kInstruction_mvn_vc_r12_r10_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r0_r1_LSL_15),
    kInstruction_mvn_eq_r0_r1_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r14_r14_ROR_30),
    kInstruction_mvn_le_r14_r14_ROR_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r13_r5_LSL_18),
    kInstruction_mvn_gt_r13_r5_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r6_r11_LSL_23),
    kInstruction_mvn_lt_r6_r11_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r2_r1_ROR_17),
    kInstruction_mvn_cc_r2_r1_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r6_r3_LSL_22),
    kInstruction_mvn_mi_r6_r3_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r3_r1_ROR_9),
    kInstruction_mvn_al_r3_r1_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r0_r12_LSL_5),
    kInstruction_mvn_hi_r0_r12_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r6_r0_ROR_11),
    kInstruction_mvn_eq_r6_r0_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r10_r7_LSL_26),
    kInstruction_mvn_mi_r10_r7_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r7_r3_ROR_12),
    kInstruction_mvn_ne_r7_r3_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r0_r8_ROR_30),
    kInstruction_mvn_le_r0_r8_ROR_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r7_r1_ROR_17),
    kInstruction_mvn_gt_r7_r1_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r0_r6_ROR_8),
    kInstruction_mvn_al_r0_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r13_r13_ROR_17),
    kInstruction_mvn_vc_r13_r13_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r2_r9_ROR_25),
    kInstruction_mvn_ge_r2_r9_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r11_r3_ROR_27),
    kInstruction_mvn_lt_r11_r3_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r0_r3_ROR_14),
    kInstruction_mvn_hi_r0_r3_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r13_r1_ROR_18),
    kInstruction_mvn_vc_r13_r1_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r1_r13_LSL_31),
    kInstruction_mvn_eq_r1_r13_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r12_r9_ROR_22),
    kInstruction_mvn_mi_r12_r9_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r5_r11_ROR_31),
    kInstruction_mvn_al_r5_r11_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r7_r4_ROR_20),
    kInstruction_mvn_ne_r7_r4_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r0_r1_LSL_10),
    kInstruction_mvn_hi_r0_r1_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r5_r12_ROR_6),
    kInstruction_mvn_ne_r5_r12_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r14_r14_LSL_6),
    kInstruction_mvn_hi_r14_r14_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r12_r9_ROR_28),
    kInstruction_mvn_pl_r12_r9_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r7_r2_LSL_2),
    kInstruction_mvn_al_r7_r2_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r12_r9_LSL_5),
    kInstruction_mvn_le_r12_r9_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r0_r6_ROR_1),
    kInstruction_mvn_lt_r0_r6_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r3_r4_LSL_14),
    kInstruction_mvn_le_r3_r4_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r1_r10_LSL_3),
    kInstruction_mvn_ge_r1_r10_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r12_r14_ROR_6),
    kInstruction_mvn_ne_r12_r14_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r11_r7_LSL_27),
    kInstruction_mvn_ne_r11_r7_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r4_r2_LSL_30),
    kInstruction_mvn_mi_r4_r2_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r8_r2_LSL_20),
    kInstruction_mvn_ls_r8_r2_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r1_r5_ROR_21),
    kInstruction_mvn_ge_r1_r5_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r3_r10_LSL_13),
    kInstruction_mvn_vs_r3_r10_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r7_r6_ROR_2),
    kInstruction_mvn_ge_r7_r6_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r13_r12_ROR_5),
    kInstruction_mvn_ls_r13_r12_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r9_r10_LSL_19),
    kInstruction_mvn_eq_r9_r10_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r5_r7_ROR_6),
    kInstruction_mvn_le_r5_r7_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r11_r5_LSL_8),
    kInstruction_mvn_eq_r11_r5_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r6_r5_ROR_31),
    kInstruction_mvn_mi_r6_r5_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r13_r11_ROR_8),
    kInstruction_mvn_cc_r13_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r0_r14_LSL_13),
    kInstruction_mvn_mi_r0_r14_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r7_r12_LSL_2),
    kInstruction_mvn_lt_r7_r12_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r6_r8_LSL_22),
    kInstruction_mvn_eq_r6_r8_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r13_r8_LSL_26),
    kInstruction_mvn_lt_r13_r8_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r5_r7_ROR_18),
    kInstruction_mvn_gt_r5_r7_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r9_r4_LSL_25),
    kInstruction_mvn_al_r9_r4_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r8_r12_ROR_23),
    kInstruction_mvn_gt_r8_r12_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r5_r6_ROR_23),
    kInstruction_mvn_eq_r5_r6_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r0_r4_ROR_14),
    kInstruction_mvn_ge_r0_r4_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r1_r1_ROR_12),
    kInstruction_mvn_le_r1_r1_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r12_r12_LSL_29),
    kInstruction_mvn_le_r12_r12_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r2_r9_LSL_7),
    kInstruction_mvn_ls_r2_r9_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r4_r12_ROR_10),
    kInstruction_mvn_pl_r4_r12_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r4_r0_LSL_1),
    kInstruction_mvn_ge_r4_r0_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r9_r6_LSL_14),
    kInstruction_mvn_mi_r9_r6_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r14_r10_LSL_8),
    kInstruction_mvn_vc_r14_r10_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r9_r10_LSL_18),
    kInstruction_mvn_ls_r9_r10_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r9_r4_ROR_19),
    kInstruction_mvn_cc_r9_r4_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r10_r2_ROR_5),
    kInstruction_mvn_ne_r10_r2_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r14_r7_ROR_31),
    kInstruction_mvn_cc_r14_r7_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r4_r7_LSL_7),
    kInstruction_mvn_hi_r4_r7_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r5_r9_LSL_14),
    kInstruction_mvn_mi_r5_r9_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r7_r11_ROR_4),
    kInstruction_mvn_cs_r7_r11_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r9_r8_ROR_9),
    kInstruction_mvn_cc_r9_r8_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r1_r11_LSL_12),
    kInstruction_mvn_hi_r1_r11_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r8_r12_ROR_19),
    kInstruction_mvn_pl_r8_r12_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r4_r3_ROR_30),
    kInstruction_mvn_gt_r4_r3_ROR_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r8_r10_ROR_2),
    kInstruction_mvn_le_r8_r10_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r2_r1_ROR_1),
    kInstruction_mvn_gt_r2_r1_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r7_r5_LSL_11),
    kInstruction_mvn_mi_r7_r5_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r10_r12_ROR_26),
    kInstruction_mvn_eq_r10_r12_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r1_r3_ROR_29),
    kInstruction_mvn_le_r1_r3_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r3_r8_ROR_30),
    kInstruction_mvn_lt_r3_r8_ROR_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r1_r9_ROR_11),
    kInstruction_mvn_ls_r1_r9_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r14_r12_ROR_23),
    kInstruction_mvn_vs_r14_r12_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r9_r14_LSL_4),
    kInstruction_mvn_ge_r9_r14_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r1_r8_ROR_2),
    kInstruction_mvn_cc_r1_r8_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r14_r10_ROR_28),
    kInstruction_mvn_ge_r14_r10_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r8_r13_ROR_31),
    kInstruction_mvn_cc_r8_r13_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r10_r8_LSL_28),
    kInstruction_mvn_ls_r10_r8_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r10_r1_LSL_1),
    kInstruction_mvn_ge_r10_r1_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r9_r9_ROR_19),
    kInstruction_mvn_vs_r9_r9_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r6_r7_ROR_19),
    kInstruction_mvn_ne_r6_r7_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r13_r9_ROR_21),
    kInstruction_mvn_cs_r13_r9_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r13_r14_ROR_10),
    kInstruction_mvn_ls_r13_r14_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r2_r8_LSL_29),
    kInstruction_mvn_hi_r2_r8_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r4_r2_LSL_27),
    kInstruction_mvn_le_r4_r2_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r2_r10_ROR_6),
    kInstruction_mvn_gt_r2_r10_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r3_r7_ROR_4),
    kInstruction_mvn_hi_r3_r7_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r12_r9_LSL_14),
    kInstruction_mvn_hi_r12_r9_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r9_r11_ROR_31),
    kInstruction_mvn_lt_r9_r11_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r3_r1_ROR_18),
    kInstruction_mvn_ls_r3_r1_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r0_r9_ROR_17),
    kInstruction_mvn_al_r0_r9_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r0_r7_LSL_14),
    kInstruction_mvn_cc_r0_r7_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r10_r12_LSL_17),
    kInstruction_mvn_al_r10_r12_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r1_r14_LSL_22),
    kInstruction_mvn_lt_r1_r14_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r1_r7_ROR_1),
    kInstruction_mvn_ge_r1_r7_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r1_r1_LSL_30),
    kInstruction_mvn_mi_r1_r1_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r7_r2_LSL_19),
    kInstruction_mvn_cc_r7_r2_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r14_r3_LSL_3),
    kInstruction_mvn_vs_r14_r3_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r5_r1_ROR_30),
    kInstruction_mvn_ls_r5_r1_ROR_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r4_r4_LSL_29),
    kInstruction_mvn_al_r4_r4_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r7_r3_ROR_25),
    kInstruction_mvn_mi_r7_r3_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r9_r4_ROR_2),
    kInstruction_mvn_eq_r9_r4_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r7_r13_ROR_9),
    kInstruction_mvn_vc_r7_r13_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r7_r1_ROR_21),
    kInstruction_mvn_eq_r7_r1_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r2_r6_ROR_12),
    kInstruction_mvn_ge_r2_r6_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r2_r7_LSL_10),
    kInstruction_mvn_ge_r2_r7_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r0_r14_LSL_19),
    kInstruction_mvn_ne_r0_r14_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r0_r7_LSL_19),
    kInstruction_mvn_ge_r0_r7_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r9_r12_LSL_3),
    kInstruction_mvn_cc_r9_r12_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r7_r13_ROR_23),
    kInstruction_mvn_gt_r7_r13_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r2_r6_LSL_19),
    kInstruction_mvn_mi_r2_r6_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r11_r6_ROR_14),
    kInstruction_mvn_ne_r11_r6_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r12_r2_LSL_6),
    kInstruction_mvn_ls_r12_r2_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r4_r14_ROR_11),
    kInstruction_mvn_cc_r4_r14_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r12_r10_LSL_20),
    kInstruction_mvn_lt_r12_r10_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r10_r14_LSL_3),
    kInstruction_mvn_le_r10_r14_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r14_r9_LSL_28),
    kInstruction_mvn_al_r14_r9_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r9_r3_LSL_17),
    kInstruction_mvn_ls_r9_r3_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r10_r2_ROR_25),
    kInstruction_mvn_eq_r10_r2_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r6_r7_ROR_2),
    kInstruction_mvn_lt_r6_r7_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r5_r7_ROR_21),
    kInstruction_mvn_le_r5_r7_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r14_r2_LSL_31),
    kInstruction_mvn_eq_r14_r2_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r1_r8_LSL_20),
    kInstruction_mvn_hi_r1_r8_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r11_r0_ROR_7),
    kInstruction_mvn_hi_r11_r0_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r0_r4_ROR_1),
    kInstruction_mvn_lt_r0_r4_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r0_r2_ROR_1),
    kInstruction_mvn_ge_r0_r2_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r3_r11_ROR_1),
    kInstruction_mvn_ge_r3_r11_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r4_r12_LSL_27),
    kInstruction_mvn_le_r4_r12_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r12_r2_ROR_6),
    kInstruction_mvn_al_r12_r2_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r11_r13_LSL_2),
    kInstruction_mvn_al_r11_r13_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r6_r8_LSL_3),
    kInstruction_mvn_lt_r6_r8_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r1_r4_LSL_21),
    kInstruction_mvn_ls_r1_r4_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r12_r13_ROR_7),
    kInstruction_mvn_al_r12_r13_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r5_r14_LSL_8),
    kInstruction_mvn_gt_r5_r14_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r6_r8_ROR_4),
    kInstruction_mvn_vc_r6_r8_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r0_r14_LSL_1),
    kInstruction_mvn_vc_r0_r14_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r5_r0_LSL_18),
    kInstruction_mvn_ge_r5_r0_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r6_r6_ROR_28),
    kInstruction_mvn_lt_r6_r6_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r4_r14_LSL_2),
    kInstruction_mvn_lt_r4_r14_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r8_r9_ROR_21),
    kInstruction_mvn_lt_r8_r9_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r6_r12_LSL_27),
    kInstruction_mvn_cc_r6_r12_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r4_r11_LSL_4),
    kInstruction_mvn_gt_r4_r11_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r13_r8_LSL_4),
    kInstruction_mvn_pl_r13_r8_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r5_r8_LSL_16),
    kInstruction_mvn_eq_r5_r8_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r5_r1_ROR_28),
    kInstruction_mvn_al_r5_r1_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r2_r12_LSL_9),
    kInstruction_mvn_lt_r2_r12_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r10_r4_LSL_28),
    kInstruction_mvn_vc_r10_r4_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r0_r4_ROR_23),
    kInstruction_mvn_hi_r0_r4_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r14_r1_ROR_16),
    kInstruction_mvn_gt_r14_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r14_r2_ROR_24),
    kInstruction_mvn_ne_r14_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r3_r4_LSL_18),
    kInstruction_mvn_ls_r3_r4_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r9_r6_LSL_13),
    kInstruction_mvn_mi_r9_r6_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r3_r10_ROR_10),
    kInstruction_mvn_ge_r3_r10_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r5_r7_LSL_27),
    kInstruction_mvn_ge_r5_r7_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r6_r0_ROR_26),
    kInstruction_mvn_vc_r6_r0_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r11_r3_ROR_6),
    kInstruction_mvn_ne_r11_r3_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r3_r4_ROR_17),
    kInstruction_mvn_gt_r3_r4_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r1_r14_ROR_25),
    kInstruction_mvn_lt_r1_r14_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r11_r13_LSL_27),
    kInstruction_mvn_eq_r11_r13_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r6_r1_ROR_27),
    kInstruction_mvn_cs_r6_r1_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r14_r14_LSL_31),
    kInstruction_mvn_vc_r14_r14_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r7_r2_LSL_8),
    kInstruction_mvn_vc_r7_r2_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r3_r1_ROR_4),
    kInstruction_mvn_eq_r3_r1_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r13_r14_LSL_12),
    kInstruction_mvn_mi_r13_r14_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r10_r14_ROR_28),
    kInstruction_mvn_vc_r10_r14_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r13_r0_LSL_17),
    kInstruction_mvn_cc_r13_r0_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r14_r8_LSL_23),
    kInstruction_mvn_vs_r14_r8_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r14_r10_LSL_16),
    kInstruction_mvn_cc_r14_r10_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r8_r7_ROR_23),
    kInstruction_mvn_mi_r8_r7_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r5_r12_ROR_10),
    kInstruction_mvn_ge_r5_r12_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r13_r5_LSL_19),
    kInstruction_mvn_hi_r13_r5_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r11_r12_LSL_21),
    kInstruction_mvn_hi_r11_r12_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r1_r1_LSL_9),
    kInstruction_mvn_cc_r1_r1_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r14_r13_LSL_8),
    kInstruction_mvn_ge_r14_r13_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r2_r0_LSL_31),
    kInstruction_mvn_le_r2_r0_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r9_r3_LSL_19),
    kInstruction_mvn_gt_r9_r3_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r9_r3_LSL_22),
    kInstruction_mvn_ls_r9_r3_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r10_r9_ROR_13),
    kInstruction_mvn_lt_r10_r9_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r1_r8_ROR_11),
    kInstruction_mvn_lt_r1_r8_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r1_r2_LSL_3),
    kInstruction_mvn_eq_r1_r2_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r1_r6_LSL_16),
    kInstruction_mvn_eq_r1_r6_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r3_r2_ROR_28),
    kInstruction_mvn_ne_r3_r2_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r7_r5_ROR_6),
    kInstruction_mvn_ge_r7_r5_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r3_r1_ROR_12),
    kInstruction_mvn_ne_r3_r1_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r12_r3_ROR_13),
    kInstruction_mvn_gt_r12_r3_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r4_r6_ROR_13),
    kInstruction_mvn_gt_r4_r6_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r11_r14_ROR_7),
    kInstruction_mvn_pl_r11_r14_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r3_r4_LSL_9),
    kInstruction_mvn_gt_r3_r4_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r14_r14_ROR_6),
    kInstruction_mvn_vc_r14_r14_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r1_r14_LSL_26),
    kInstruction_mvn_ne_r1_r14_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r4_r0_ROR_4),
    kInstruction_mvn_ge_r4_r0_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r13_r3_LSL_8),
    kInstruction_mvn_pl_r13_r3_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r11_r11_ROR_26),
    kInstruction_mvn_vs_r11_r11_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r9_r3_LSL_23),
    kInstruction_mvn_le_r9_r3_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r10_r14_LSL_25),
    kInstruction_mvn_mi_r10_r14_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r13_r3_LSL_16),
    kInstruction_mvn_mi_r13_r3_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r5_r5_ROR_8),
    kInstruction_mvn_eq_r5_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r11_r6_ROR_8),
    kInstruction_mvn_lt_r11_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r3_r9_ROR_31),
    kInstruction_mvn_cs_r3_r9_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r0_r0_LSL_10),
    kInstruction_mvn_cc_r0_r0_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r4_r1_LSL_16),
    kInstruction_mvn_mi_r4_r1_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r5_r4_LSL_23),
    kInstruction_mvn_le_r5_r4_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r3_r10_LSL_14),
    kInstruction_mvn_lt_r3_r10_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r10_r8_LSL_4),
    kInstruction_mvn_al_r10_r8_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r14_r11_LSL_11),
    kInstruction_mvn_al_r14_r11_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r14_r9_LSL_30),
    kInstruction_mvn_mi_r14_r9_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r8_r11_LSL_29),
    kInstruction_mvn_ge_r8_r11_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r14_r9_ROR_13),
    kInstruction_mvn_vs_r14_r9_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r5_r5_LSL_9),
    kInstruction_mvn_lt_r5_r5_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r13_r8_LSL_2),
    kInstruction_mvn_pl_r13_r8_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r7_r14_LSL_15),
    kInstruction_mvn_al_r7_r14_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r4_r0_ROR_3),
    kInstruction_mvn_cs_r4_r0_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r4_r6_ROR_21),
    kInstruction_mvn_gt_r4_r6_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r2_r1_LSL_3),
    kInstruction_mvn_pl_r2_r1_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r5_r11_ROR_1),
    kInstruction_mvn_vc_r5_r11_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r9_r0_LSL_25),
    kInstruction_mvn_vc_r9_r0_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r2_r7_LSL_30),
    kInstruction_mvn_gt_r2_r7_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r3_r0_LSL_29),
    kInstruction_mvn_mi_r3_r0_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r6_r3_ROR_18),
    kInstruction_mvn_lt_r6_r3_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r12_r10_ROR_28),
    kInstruction_mvn_ne_r12_r10_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r13_r9_LSL_25),
    kInstruction_mvn_vs_r13_r9_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r5_r12_LSL_13),
    kInstruction_mvn_ge_r5_r12_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r2_r4_ROR_18),
    kInstruction_mvn_vs_r2_r4_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r10_r4_LSL_9),
    kInstruction_mvn_pl_r10_r4_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r14_r13_ROR_27),
    kInstruction_mvn_mi_r14_r13_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r1_r13_LSL_16),
    kInstruction_mvn_lt_r1_r13_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r1_r13_LSL_9),
    kInstruction_mvn_vc_r1_r13_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r1_r5_ROR_24),
    kInstruction_mvn_vs_r1_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r8_r7_LSL_30),
    kInstruction_mvn_pl_r8_r7_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r9_r5_LSL_30),
    kInstruction_mvn_eq_r9_r5_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r1_r13_ROR_15),
    kInstruction_mvn_al_r1_r13_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r3_r6_LSL_24),
    kInstruction_mvn_lt_r3_r6_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r3_r5_LSL_6),
    kInstruction_mvn_gt_r3_r5_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r1_r13_LSL_8),
    kInstruction_mvn_cc_r1_r13_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r11_r11_ROR_20),
    kInstruction_mvn_cc_r11_r11_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r4_r8_LSL_22),
    kInstruction_mvn_gt_r4_r8_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r4_r12_LSL_3),
    kInstruction_mvn_ge_r4_r12_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r10_r13_ROR_10),
    kInstruction_mvn_eq_r10_r13_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r2_r9_LSL_1),
    kInstruction_mvn_pl_r2_r9_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r14_r0_ROR_22),
    kInstruction_mvn_cc_r14_r0_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r9_r8_LSL_6),
    kInstruction_mvn_ge_r9_r8_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r14_r0_ROR_13),
    kInstruction_mvn_ls_r14_r0_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r14_r5_LSL_11),
    kInstruction_mvn_mi_r14_r5_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r12_r4_LSL_11),
    kInstruction_mvn_hi_r12_r4_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r11_r10_ROR_22),
    kInstruction_mvn_vc_r11_r10_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r12_r4_ROR_10),
    kInstruction_mvn_eq_r12_r4_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r0_r5_ROR_12),
    kInstruction_mvn_pl_r0_r5_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r6_r9_LSL_11),
    kInstruction_mvn_le_r6_r9_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r0_r5_LSL_13),
    kInstruction_mvn_pl_r0_r5_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r14_r3_ROR_3),
    kInstruction_mvn_hi_r14_r3_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r13_r7_LSL_12),
    kInstruction_mvn_hi_r13_r7_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r1_r4_LSL_3),
    kInstruction_mvn_hi_r1_r4_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r12_r3_LSL_21),
    kInstruction_mvn_vc_r12_r3_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r3_r0_ROR_9),
    kInstruction_mvn_gt_r3_r0_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r1_r6_ROR_31),
    kInstruction_mvn_mi_r1_r6_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r2_r3_ROR_16),
    kInstruction_mvn_cs_r2_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r6_r3_ROR_7),
    kInstruction_mvn_lt_r6_r3_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r5_r3_LSL_12),
    kInstruction_mvn_le_r5_r3_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r14_r12_ROR_3),
    kInstruction_mvn_pl_r14_r12_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r6_r4_LSL_27),
    kInstruction_mvn_gt_r6_r4_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r11_r10_ROR_4),
    kInstruction_mvn_le_r11_r10_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r8_r1_LSL_25),
    kInstruction_mvn_gt_r8_r1_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r10_r4_LSL_22),
    kInstruction_mvn_pl_r10_r4_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r9_r5_ROR_15),
    kInstruction_mvn_al_r9_r5_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r1_r5_ROR_3),
    kInstruction_mvn_cs_r1_r5_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r0_r1_LSL_26),
    kInstruction_mvn_eq_r0_r1_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r13_r14_LSL_6),
    kInstruction_mvn_hi_r13_r14_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r8_r1_LSL_18),
    kInstruction_mvn_vs_r8_r1_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r12_r2_LSL_16),
    kInstruction_mvn_eq_r12_r2_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r4_r3_LSL_4),
    kInstruction_mvn_cc_r4_r3_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r4_r2_LSL_14),
    kInstruction_mvn_ls_r4_r2_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r8_r12_LSL_27),
    kInstruction_mvn_cs_r8_r12_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r3_r10_ROR_18),
    kInstruction_mvn_le_r3_r10_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r6_r0_LSL_26),
    kInstruction_mvn_vc_r6_r0_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r8_r13_LSL_3),
    kInstruction_mvn_mi_r8_r13_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r3_r2_ROR_14),
    kInstruction_mvn_al_r3_r2_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r6_r14_LSL_29),
    kInstruction_mvn_gt_r6_r14_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r9_r12_ROR_25),
    kInstruction_mvn_hi_r9_r12_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r11_r13_ROR_12),
    kInstruction_mvn_mi_r11_r13_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r2_r6_LSL_13),
    kInstruction_mvn_vs_r2_r6_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r14_r4_LSL_29),
    kInstruction_mvn_hi_r14_r4_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r9_r13_LSL_4),
    kInstruction_mvn_hi_r9_r13_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r8_r1_ROR_12),
    kInstruction_mvn_hi_r8_r1_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r13_r0_ROR_11),
    kInstruction_mvn_vs_r13_r0_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r4_r11_ROR_31),
    kInstruction_mvn_le_r4_r11_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r7_r1_ROR_9),
    kInstruction_mvn_eq_r7_r1_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r13_r3_ROR_4),
    kInstruction_mvn_ls_r13_r3_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r3_r4_ROR_9),
    kInstruction_mvn_hi_r3_r4_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r4_r8_LSL_7),
    kInstruction_mvn_al_r4_r8_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r11_r2_ROR_15),
    kInstruction_mvn_le_r11_r2_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r4_r11_ROR_3),
    kInstruction_mvn_hi_r4_r11_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r5_r4_LSL_16),
    kInstruction_mvn_mi_r5_r4_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r7_r10_ROR_11),
    kInstruction_mvn_pl_r7_r10_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r11_r14_ROR_5),
    kInstruction_mvn_lt_r11_r14_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r14_r12_LSL_3),
    kInstruction_mvn_le_r14_r12_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r6_r7_LSL_7),
    kInstruction_mvn_mi_r6_r7_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r6_r12_LSL_13),
    kInstruction_mvn_hi_r6_r12_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r12_r12_ROR_24),
    kInstruction_mvn_mi_r12_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r6_r4_LSL_31),
    kInstruction_mvn_mi_r6_r4_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r13_r7_LSL_22),
    kInstruction_mvn_vc_r13_r7_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r1_r10_ROR_8),
    kInstruction_mvn_le_r1_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r9_r1_LSL_6),
    kInstruction_mvn_gt_r9_r1_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r14_r2_ROR_26),
    kInstruction_mvn_gt_r14_r2_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r11_r3_LSL_1),
    kInstruction_mvn_pl_r11_r3_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r4_r7_LSL_29),
    kInstruction_mvn_ge_r4_r7_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r9_r2_LSL_6),
    kInstruction_mvn_gt_r9_r2_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r4_r7_LSL_21),
    kInstruction_mvn_cs_r4_r7_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r4_r3_LSL_9),
    kInstruction_mvn_eq_r4_r3_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r14_r3_ROR_10),
    kInstruction_mvn_ne_r14_r3_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r4_r6_LSL_19),
    kInstruction_mvn_eq_r4_r6_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r6_r1_ROR_2),
    kInstruction_mvn_cs_r6_r1_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r0_r11_LSL_7),
    kInstruction_mvn_ls_r0_r11_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r10_r10_LSL_28),
    kInstruction_mvn_ge_r10_r10_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r0_r9_LSL_27),
    kInstruction_mvn_mi_r0_r9_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r7_r4_LSL_17),
    kInstruction_mvn_vc_r7_r4_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r4_r4_LSL_21),
    kInstruction_mvn_gt_r4_r4_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r5_r9_ROR_28),
    kInstruction_mvn_mi_r5_r9_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r9_r9_LSL_2),
    kInstruction_mvn_hi_r9_r9_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r2_r2_ROR_11),
    kInstruction_mvn_hi_r2_r2_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r8_r9_ROR_25),
    kInstruction_mvn_mi_r8_r9_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r3_r13_ROR_20),
    kInstruction_mvn_mi_r3_r13_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r9_r10_ROR_28),
    kInstruction_mvn_lt_r9_r10_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r8_r5_LSL_16),
    kInstruction_mvn_pl_r8_r5_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r14_r1_LSL_11),
    kInstruction_mvn_al_r14_r1_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r8_r4_ROR_5),
    kInstruction_mvn_ls_r8_r4_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r11_r9_ROR_15),
    kInstruction_mvn_cc_r11_r9_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r6_r0_LSL_2),
    kInstruction_mvn_gt_r6_r0_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r12_r8_LSL_15),
    kInstruction_mvn_ne_r12_r8_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r8_r2_ROR_31),
    kInstruction_mvn_gt_r8_r2_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r7_r12_LSL_19),
    kInstruction_mvn_lt_r7_r12_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r9_r11_ROR_28),
    kInstruction_mvn_al_r9_r11_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r13_r5_ROR_28),
    kInstruction_mvn_al_r13_r5_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r2_r13_LSL_6),
    kInstruction_mvn_lt_r2_r13_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r7_r7_LSL_8),
    kInstruction_mvn_al_r7_r7_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r1_r11_ROR_16),
    kInstruction_mvn_lt_r1_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r7_r8_LSL_23),
    kInstruction_mvn_eq_r7_r8_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r4_r12_LSL_1),
    kInstruction_mvn_cs_r4_r12_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r2_r3_ROR_26),
    kInstruction_mvn_ls_r2_r3_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r13_r1_ROR_11),
    kInstruction_mvn_ne_r13_r1_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r8_r13_ROR_26),
    kInstruction_mvn_le_r8_r13_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r2_r4_LSL_8),
    kInstruction_mvn_al_r2_r4_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r4_r12_ROR_16),
    kInstruction_mvn_gt_r4_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r8_r1_ROR_19),
    kInstruction_mvn_lt_r8_r1_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r7_r3_ROR_15),
    kInstruction_mvn_ne_r7_r3_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r11_r8_LSL_23),
    kInstruction_mvn_ge_r11_r8_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r11_r12_ROR_10),
    kInstruction_mvn_mi_r11_r12_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r1_r4_ROR_15),
    kInstruction_mvn_cs_r1_r4_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r14_r4_LSL_5),
    kInstruction_mvn_mi_r14_r4_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r5_r2_ROR_15),
    kInstruction_mvn_ge_r5_r2_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r11_r11_LSL_13),
    kInstruction_mvn_lt_r11_r11_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r7_r8_ROR_2),
    kInstruction_mvn_hi_r7_r8_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r14_r0_LSL_4),
    kInstruction_mvn_eq_r14_r0_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r12_r5_LSL_6),
    kInstruction_mvn_al_r12_r5_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r5_r6_ROR_14),
    kInstruction_mvn_mi_r5_r6_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r14_r0_ROR_31),
    kInstruction_mvn_cc_r14_r0_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r4_r12_ROR_22),
    kInstruction_mvn_ne_r4_r12_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r1_r3_LSL_24),
    kInstruction_mvn_ne_r1_r3_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r3_r9_LSL_26),
    kInstruction_mvn_hi_r3_r9_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r0_r7_ROR_31),
    kInstruction_mvn_vs_r0_r7_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r3_r1_ROR_12),
    kInstruction_mvn_le_r3_r1_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r5_r8_LSL_19),
    kInstruction_mvn_le_r5_r8_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r0_r13_ROR_8),
    kInstruction_mvn_lt_r0_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r12_r13_LSL_6),
    kInstruction_mvn_ge_r12_r13_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r1_r2_LSL_21),
    kInstruction_mvn_lt_r1_r2_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r4_r5_LSL_26),
    kInstruction_mvn_ge_r4_r5_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r6_r10_LSL_25),
    kInstruction_mvn_ls_r6_r10_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r5_r4_LSL_18),
    kInstruction_mvn_hi_r5_r4_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r7_r14_LSL_23),
    kInstruction_mvn_cs_r7_r14_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r14_r4_ROR_3),
    kInstruction_mvn_cc_r14_r4_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r14_r4_LSL_17),
    kInstruction_mvn_cs_r14_r4_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r14_r12_LSL_24),
    kInstruction_mvn_lt_r14_r12_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r6_r14_ROR_3),
    kInstruction_mvn_vc_r6_r14_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r11_r5_ROR_18),
    kInstruction_mvn_ls_r11_r5_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r3_r1_ROR_2),
    kInstruction_mvn_gt_r3_r1_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r11_r14_ROR_5),
    kInstruction_mvn_al_r11_r14_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r11_r11_LSL_15),
    kInstruction_mvn_le_r11_r11_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r1_r13_LSL_30),
    kInstruction_mvn_gt_r1_r13_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r3_r5_LSL_31),
    kInstruction_mvn_le_r3_r5_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r4_r10_ROR_16),
    kInstruction_mvn_hi_r4_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r0_r6_LSL_6),
    kInstruction_mvn_eq_r0_r6_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r0_r14_ROR_3),
    kInstruction_mvn_vs_r0_r14_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r0_r3_ROR_26),
    kInstruction_mvn_eq_r0_r3_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r1_r10_ROR_30),
    kInstruction_mvn_eq_r1_r10_ROR_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r9_r2_ROR_12),
    kInstruction_mvn_ls_r9_r2_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r9_r14_ROR_22),
    kInstruction_mvn_gt_r9_r14_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r1_r8_LSL_1),
    kInstruction_mvn_hi_r1_r8_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r13_r0_ROR_4),
    kInstruction_mvn_le_r13_r0_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r11_r10_ROR_4),
    kInstruction_mvn_lt_r11_r10_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r2_r5_LSL_28),
    kInstruction_mvn_le_r2_r5_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r8_r12_LSL_27),
    kInstruction_mvn_vc_r8_r12_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r11_r3_LSL_6),
    kInstruction_mvn_al_r11_r3_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r12_r1_ROR_2),
    kInstruction_mvn_al_r12_r1_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r8_r10_LSL_5),
    kInstruction_mvn_cs_r8_r10_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r11_r2_ROR_4),
    kInstruction_mvn_hi_r11_r2_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r1_r2_LSL_15),
    kInstruction_mvn_eq_r1_r2_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r0_r5_ROR_5),
    kInstruction_mvn_lt_r0_r5_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r9_r0_ROR_18),
    kInstruction_mvn_mi_r9_r0_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r12_r9_LSL_5),
    kInstruction_mvn_cs_r12_r9_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r13_r10_LSL_19),
    kInstruction_mvn_cc_r13_r10_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r8_r0_LSL_25),
    kInstruction_mvn_eq_r8_r0_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r9_r7_LSL_4),
    kInstruction_mvn_cc_r9_r7_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r6_r2_LSL_18),
    kInstruction_mvn_ls_r6_r2_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r6_r14_ROR_3),
    kInstruction_mvn_lt_r6_r14_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r10_r6_ROR_13),
    kInstruction_mvn_al_r10_r6_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r1_r0_ROR_8),
    kInstruction_mvn_le_r1_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r1_r3_ROR_5),
    kInstruction_mvn_hi_r1_r3_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r5_r1_ROR_5),
    kInstruction_mvn_cc_r5_r1_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r12_r3_LSL_6),
    kInstruction_mvn_vc_r12_r3_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r11_r13_ROR_28),
    kInstruction_mvn_ls_r11_r13_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r9_r13_ROR_10),
    kInstruction_mvn_vs_r9_r13_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r3_r13_LSL_31),
    kInstruction_mvn_eq_r3_r13_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r11_r9_ROR_27),
    kInstruction_mvn_cc_r11_r9_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r9_r2_LSL_28),
    kInstruction_mvn_cc_r9_r2_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r14_r5_LSL_4),
    kInstruction_mvn_ne_r14_r5_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r4_r2_LSL_31),
    kInstruction_mvn_gt_r4_r2_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r9_r1_ROR_17),
    kInstruction_mvn_cc_r9_r1_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r4_r10_LSL_24),
    kInstruction_mvn_hi_r4_r10_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r3_r9_LSL_5),
    kInstruction_mvn_ne_r3_r9_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r8_r5_ROR_10),
    kInstruction_mvn_hi_r8_r5_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r5_r1_ROR_3),
    kInstruction_mvn_pl_r5_r1_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r14_r0_ROR_14),
    kInstruction_mvn_ge_r14_r0_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r11_r7_LSL_15),
    kInstruction_mvn_ge_r11_r7_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r10_r11_LSL_15),
    kInstruction_mvn_pl_r10_r11_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r6_r11_LSL_3),
    kInstruction_mvn_ne_r6_r11_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r4_r5_ROR_12),
    kInstruction_mvn_cs_r4_r5_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r8_r10_ROR_4),
    kInstruction_mvn_eq_r8_r10_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r9_r12_ROR_2),
    kInstruction_mvn_mi_r9_r12_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r6_r2_LSL_24),
    kInstruction_mvn_pl_r6_r2_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r6_r8_LSL_12),
    kInstruction_mvn_vs_r6_r8_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r0_r13_ROR_29),
    kInstruction_mvn_vc_r0_r13_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r1_r4_LSL_19),
    kInstruction_mvn_ne_r1_r4_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r3_r12_ROR_30),
    kInstruction_mvn_cc_r3_r12_ROR_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r9_r8_LSL_17),
    kInstruction_mvn_gt_r9_r8_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r3_r9_ROR_14),
    kInstruction_mvn_ne_r3_r9_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r5_r7_ROR_25),
    kInstruction_mvn_ne_r5_r7_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r8_r10_LSL_16),
    kInstruction_mvn_pl_r8_r10_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ls_r10_r10_ROR_22),
    kInstruction_mvn_ls_r10_r10_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r11_r11_ROR_29),
    kInstruction_mvn_al_r11_r11_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r6_r11_LSL_18),
    kInstruction_mvn_ge_r6_r11_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r0_r12_LSL_2),
    kInstruction_mvn_hi_r0_r12_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r3_r12_LSL_25),
    kInstruction_mvn_ge_r3_r12_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r6_r1_LSL_27),
    kInstruction_mvn_pl_r6_r1_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r7_r8_LSL_19),
    kInstruction_mvn_lt_r7_r8_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r3_r9_LSL_10),
    kInstruction_mvn_hi_r3_r9_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r3_r13_LSL_27),
    kInstruction_mvn_eq_r3_r13_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r3_r1_ROR_6),
    kInstruction_mvn_ne_r3_r1_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r8_r13_ROR_16),
    kInstruction_mvn_cs_r8_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r1_r4_ROR_23),
    kInstruction_mvn_eq_r1_r4_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r5_r8_ROR_5),
    kInstruction_mvn_vc_r5_r8_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r13_r5_LSL_24),
    kInstruction_mvn_ge_r13_r5_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r4_r10_ROR_25),
    kInstruction_mvn_lt_r4_r10_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r3_r10_LSL_21),
    kInstruction_mvn_hi_r3_r10_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r7_r1_LSL_31),
    kInstruction_mvn_hi_r7_r1_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r4_r12_ROR_29),
    kInstruction_mvn_gt_r4_r12_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r5_r1_ROR_27),
    kInstruction_mvn_cs_r5_r1_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r4_r7_ROR_8),
    kInstruction_mvn_al_r4_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r8_r10_LSL_7),
    kInstruction_mvn_ge_r8_r10_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r0_r12_LSL_29),
    kInstruction_mvn_vc_r0_r12_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r5_r1_LSL_9),
    kInstruction_mvn_lt_r5_r1_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r6_r10_LSL_30),
    kInstruction_mvn_gt_r6_r10_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r14_r0_LSL_2),
    kInstruction_mvn_eq_r14_r0_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_lt_r0_r6_LSL_21),
    kInstruction_mvn_lt_r0_r6_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r2_r12_LSL_13),
    kInstruction_mvn_vs_r2_r12_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r10_r1_ROR_22),
    kInstruction_mvn_ge_r10_r1_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r5_r14_ROR_16),
    kInstruction_mvn_ne_r5_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r13_r11_LSL_21),
    kInstruction_mvn_ge_r13_r11_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r5_r9_ROR_29),
    kInstruction_mvn_ge_r5_r9_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r3_r13_ROR_18),
    kInstruction_mvn_al_r3_r13_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r0_r12_LSL_10),
    kInstruction_mvn_hi_r0_r12_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r11_r3_ROR_19),
    kInstruction_mvn_vc_r11_r3_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r9_r4_LSL_7),
    kInstruction_mvn_mi_r9_r4_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r0_r14_LSL_31),
    kInstruction_mvn_mi_r0_r14_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r2_r5_ROR_11),
    kInstruction_mvn_mi_r2_r5_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r8_r2_LSL_30),
    kInstruction_mvn_hi_r8_r2_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r6_r13_ROR_25),
    kInstruction_mvn_cs_r6_r13_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r14_r5_ROR_25),
    kInstruction_mvn_le_r14_r5_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r12_r2_LSL_20),
    kInstruction_mvn_le_r12_r2_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r4_r1_LSL_23),
    kInstruction_mvn_le_r4_r1_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_al_r9_r11_LSL_6),
    kInstruction_mvn_al_r9_r11_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r10_r13_LSL_1),
    kInstruction_mvn_gt_r10_r13_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r8_r6_ROR_27),
    kInstruction_mvn_vs_r8_r6_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r3_r5_LSL_28),
    kInstruction_mvn_vs_r3_r5_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r7_r5_ROR_28),
    kInstruction_mvn_vs_r7_r5_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vs_r14_r0_ROR_25),
    kInstruction_mvn_vs_r14_r0_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ge_r9_r14_ROR_20),
    kInstruction_mvn_ge_r9_r14_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_ne_r13_r10_LSL_14),
    kInstruction_mvn_ne_r13_r10_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r1_r8_ROR_3),
    kInstruction_mvn_hi_r1_r8_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_mi_r14_r7_ROR_18),
    kInstruction_mvn_mi_r14_r7_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r13_r14_LSL_30),
    kInstruction_mvn_hi_r13_r14_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r5_r7_ROR_26),
    kInstruction_mvn_hi_r5_r7_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r5_r1_ROR_16),
    kInstruction_mvn_eq_r5_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cs_r1_r1_LSL_14),
    kInstruction_mvn_cs_r1_r1_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_le_r3_r7_LSL_5),
    kInstruction_mvn_le_r3_r7_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r2_r3_LSL_2),
    kInstruction_mvn_eq_r2_r3_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_vc_r9_r9_ROR_15),
    kInstruction_mvn_vc_r9_r9_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_pl_r4_r14_ROR_10),
    kInstruction_mvn_pl_r4_r14_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_gt_r4_r4_LSL_25),
    kInstruction_mvn_gt_r4_r4_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_cc_r6_r1_ROR_12),
    kInstruction_mvn_cc_r6_r1_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_hi_r12_r3_LSL_23),
    kInstruction_mvn_hi_r12_r3_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mvn_eq_r7_r1_LSL_9),
    kInstruction_mvn_eq_r7_r1_LSL_9,
  },
};

#endif  // VIXL_ASSEMBLER_COND_RD_OPERAND_RN_SHIFT_AMOUNT_1TO31_MVN_A32_H_
