// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_ASSEMBLER_COND_RDLOW_RNLOW_OPERAND_IMMEDIATE_IMM8_IN_IT_BLOCK_ADD_T32_H_
#define VIXL_ASSEMBLER_COND_RDLOW_RNLOW_OPERAND_IMMEDIATE_IMM8_IN_IT_BLOCK_ADD_T32_H_

const byte kInstruction_add_mi_r3_r3_90[] = {
  0x48, 0xbf, 0x5a, 0x33 // It mi; add mi r3 r3 90
};
const byte kInstruction_add_vc_r6_r6_138[] = {
  0x78, 0xbf, 0x8a, 0x36 // It vc; add vc r6 r6 138
};
const byte kInstruction_add_cc_r5_r5_72[] = {
  0x38, 0xbf, 0x48, 0x35 // It cc; add cc r5 r5 72
};
const byte kInstruction_add_ge_r6_r6_181[] = {
  0xa8, 0xbf, 0xb5, 0x36 // It ge; add ge r6 r6 181
};
const byte kInstruction_add_cc_r1_r1_143[] = {
  0x38, 0xbf, 0x8f, 0x31 // It cc; add cc r1 r1 143
};
const byte kInstruction_add_vc_r5_r5_114[] = {
  0x78, 0xbf, 0x72, 0x35 // It vc; add vc r5 r5 114
};
const byte kInstruction_add_ge_r4_r4_195[] = {
  0xa8, 0xbf, 0xc3, 0x34 // It ge; add ge r4 r4 195
};
const byte kInstruction_add_lt_r0_r0_203[] = {
  0xb8, 0xbf, 0xcb, 0x30 // It lt; add lt r0 r0 203
};
const byte kInstruction_add_hi_r1_r1_98[] = {
  0x88, 0xbf, 0x62, 0x31 // It hi; add hi r1 r1 98
};
const byte kInstruction_add_gt_r3_r3_0[] = {
  0xc8, 0xbf, 0x1b, 0x1c // It gt; add gt r3 r3 0
};
const byte kInstruction_add_vs_r5_r5_195[] = {
  0x68, 0xbf, 0xc3, 0x35 // It vs; add vs r5 r5 195
};
const byte kInstruction_add_le_r6_r6_39[] = {
  0xd8, 0xbf, 0x27, 0x36 // It le; add le r6 r6 39
};
const byte kInstruction_add_mi_r0_r0_231[] = {
  0x48, 0xbf, 0xe7, 0x30 // It mi; add mi r0 r0 231
};
const byte kInstruction_add_mi_r5_r5_220[] = {
  0x48, 0xbf, 0xdc, 0x35 // It mi; add mi r5 r5 220
};
const byte kInstruction_add_hi_r4_r4_60[] = {
  0x88, 0xbf, 0x3c, 0x34 // It hi; add hi r4 r4 60
};
const byte kInstruction_add_vc_r0_r0_83[] = {
  0x78, 0xbf, 0x53, 0x30 // It vc; add vc r0 r0 83
};
const byte kInstruction_add_lt_r3_r3_207[] = {
  0xb8, 0xbf, 0xcf, 0x33 // It lt; add lt r3 r3 207
};
const byte kInstruction_add_vs_r3_r3_101[] = {
  0x68, 0xbf, 0x65, 0x33 // It vs; add vs r3 r3 101
};
const byte kInstruction_add_cs_r0_r0_182[] = {
  0x28, 0xbf, 0xb6, 0x30 // It cs; add cs r0 r0 182
};
const byte kInstruction_add_vs_r0_r0_53[] = {
  0x68, 0xbf, 0x35, 0x30 // It vs; add vs r0 r0 53
};
const byte kInstruction_add_hi_r7_r7_142[] = {
  0x88, 0xbf, 0x8e, 0x37 // It hi; add hi r7 r7 142
};
const byte kInstruction_add_ls_r5_r5_89[] = {
  0x98, 0xbf, 0x59, 0x35 // It ls; add ls r5 r5 89
};
const byte kInstruction_add_ls_r5_r5_32[] = {
  0x98, 0xbf, 0x20, 0x35 // It ls; add ls r5 r5 32
};
const byte kInstruction_add_ls_r1_r1_29[] = {
  0x98, 0xbf, 0x1d, 0x31 // It ls; add ls r1 r1 29
};
const byte kInstruction_add_hi_r3_r3_62[] = {
  0x88, 0xbf, 0x3e, 0x33 // It hi; add hi r3 r3 62
};
const byte kInstruction_add_eq_r3_r3_216[] = {
  0x08, 0xbf, 0xd8, 0x33 // It eq; add eq r3 r3 216
};
const byte kInstruction_add_mi_r5_r5_167[] = {
  0x48, 0xbf, 0xa7, 0x35 // It mi; add mi r5 r5 167
};
const byte kInstruction_add_ge_r6_r6_88[] = {
  0xa8, 0xbf, 0x58, 0x36 // It ge; add ge r6 r6 88
};
const byte kInstruction_add_lt_r1_r1_158[] = {
  0xb8, 0xbf, 0x9e, 0x31 // It lt; add lt r1 r1 158
};
const byte kInstruction_add_ge_r5_r5_242[] = {
  0xa8, 0xbf, 0xf2, 0x35 // It ge; add ge r5 r5 242
};
const byte kInstruction_add_ls_r4_r4_225[] = {
  0x98, 0xbf, 0xe1, 0x34 // It ls; add ls r4 r4 225
};
const byte kInstruction_add_eq_r0_r0_4[] = {
  0x08, 0xbf, 0x00, 0x1d // It eq; add eq r0 r0 4
};
const byte kInstruction_add_vs_r1_r1_98[] = {
  0x68, 0xbf, 0x62, 0x31 // It vs; add vs r1 r1 98
};
const byte kInstruction_add_lt_r7_r7_224[] = {
  0xb8, 0xbf, 0xe0, 0x37 // It lt; add lt r7 r7 224
};
const byte kInstruction_add_eq_r5_r5_162[] = {
  0x08, 0xbf, 0xa2, 0x35 // It eq; add eq r5 r5 162
};
const byte kInstruction_add_gt_r5_r5_139[] = {
  0xc8, 0xbf, 0x8b, 0x35 // It gt; add gt r5 r5 139
};
const byte kInstruction_add_cc_r4_r4_68[] = {
  0x38, 0xbf, 0x44, 0x34 // It cc; add cc r4 r4 68
};
const byte kInstruction_add_vc_r0_r0_2[] = {
  0x78, 0xbf, 0x80, 0x1c // It vc; add vc r0 r0 2
};
const byte kInstruction_add_ls_r3_r3_114[] = {
  0x98, 0xbf, 0x72, 0x33 // It ls; add ls r3 r3 114
};
const byte kInstruction_add_vs_r7_r7_220[] = {
  0x68, 0xbf, 0xdc, 0x37 // It vs; add vs r7 r7 220
};
const byte kInstruction_add_cs_r6_r6_78[] = {
  0x28, 0xbf, 0x4e, 0x36 // It cs; add cs r6 r6 78
};
const byte kInstruction_add_cs_r7_r7_129[] = {
  0x28, 0xbf, 0x81, 0x37 // It cs; add cs r7 r7 129
};
const byte kInstruction_add_hi_r3_r3_17[] = {
  0x88, 0xbf, 0x11, 0x33 // It hi; add hi r3 r3 17
};
const byte kInstruction_add_cs_r0_r0_14[] = {
  0x28, 0xbf, 0x0e, 0x30 // It cs; add cs r0 r0 14
};
const byte kInstruction_add_hi_r3_r3_20[] = {
  0x88, 0xbf, 0x14, 0x33 // It hi; add hi r3 r3 20
};
const byte kInstruction_add_ne_r4_r4_210[] = {
  0x18, 0xbf, 0xd2, 0x34 // It ne; add ne r4 r4 210
};
const byte kInstruction_add_eq_r3_r3_172[] = {
  0x08, 0xbf, 0xac, 0x33 // It eq; add eq r3 r3 172
};
const byte kInstruction_add_vc_r3_r3_203[] = {
  0x78, 0xbf, 0xcb, 0x33 // It vc; add vc r3 r3 203
};
const byte kInstruction_add_hi_r3_r3_52[] = {
  0x88, 0xbf, 0x34, 0x33 // It hi; add hi r3 r3 52
};
const byte kInstruction_add_eq_r0_r0_248[] = {
  0x08, 0xbf, 0xf8, 0x30 // It eq; add eq r0 r0 248
};
const byte kInstruction_add_lt_r3_r3_43[] = {
  0xb8, 0xbf, 0x2b, 0x33 // It lt; add lt r3 r3 43
};
const byte kInstruction_add_eq_r4_r4_233[] = {
  0x08, 0xbf, 0xe9, 0x34 // It eq; add eq r4 r4 233
};
const byte kInstruction_add_eq_r6_r6_158[] = {
  0x08, 0xbf, 0x9e, 0x36 // It eq; add eq r6 r6 158
};
const byte kInstruction_add_vs_r7_r7_194[] = {
  0x68, 0xbf, 0xc2, 0x37 // It vs; add vs r7 r7 194
};
const byte kInstruction_add_hi_r5_r5_116[] = {
  0x88, 0xbf, 0x74, 0x35 // It hi; add hi r5 r5 116
};
const byte kInstruction_add_vc_r7_r7_11[] = {
  0x78, 0xbf, 0x0b, 0x37 // It vc; add vc r7 r7 11
};
const byte kInstruction_add_le_r4_r4_0[] = {
  0xd8, 0xbf, 0x24, 0x1c // It le; add le r4 r4 0
};
const byte kInstruction_add_ge_r1_r1_103[] = {
  0xa8, 0xbf, 0x67, 0x31 // It ge; add ge r1 r1 103
};
const byte kInstruction_add_eq_r0_r0_124[] = {
  0x08, 0xbf, 0x7c, 0x30 // It eq; add eq r0 r0 124
};
const byte kInstruction_add_mi_r5_r5_57[] = {
  0x48, 0xbf, 0x39, 0x35 // It mi; add mi r5 r5 57
};
const byte kInstruction_add_ge_r7_r7_22[] = {
  0xa8, 0xbf, 0x16, 0x37 // It ge; add ge r7 r7 22
};
const byte kInstruction_add_mi_r6_r6_99[] = {
  0x48, 0xbf, 0x63, 0x36 // It mi; add mi r6 r6 99
};
const byte kInstruction_add_hi_r7_r7_195[] = {
  0x88, 0xbf, 0xc3, 0x37 // It hi; add hi r7 r7 195
};
const byte kInstruction_add_vs_r4_r4_142[] = {
  0x68, 0xbf, 0x8e, 0x34 // It vs; add vs r4 r4 142
};
const byte kInstruction_add_hi_r6_r6_74[] = {
  0x88, 0xbf, 0x4a, 0x36 // It hi; add hi r6 r6 74
};
const byte kInstruction_add_ne_r1_r1_131[] = {
  0x18, 0xbf, 0x83, 0x31 // It ne; add ne r1 r1 131
};
const byte kInstruction_add_hi_r1_r1_96[] = {
  0x88, 0xbf, 0x60, 0x31 // It hi; add hi r1 r1 96
};
const byte kInstruction_add_eq_r2_r2_248[] = {
  0x08, 0xbf, 0xf8, 0x32 // It eq; add eq r2 r2 248
};
const byte kInstruction_add_le_r2_r2_252[] = {
  0xd8, 0xbf, 0xfc, 0x32 // It le; add le r2 r2 252
};
const byte kInstruction_add_cc_r2_r2_93[] = {
  0x38, 0xbf, 0x5d, 0x32 // It cc; add cc r2 r2 93
};
const byte kInstruction_add_cc_r3_r3_130[] = {
  0x38, 0xbf, 0x82, 0x33 // It cc; add cc r3 r3 130
};
const byte kInstruction_add_pl_r0_r0_0[] = {
  0x58, 0xbf, 0x00, 0x1c // It pl; add pl r0 r0 0
};
const byte kInstruction_add_gt_r3_r3_109[] = {
  0xc8, 0xbf, 0x6d, 0x33 // It gt; add gt r3 r3 109
};
const byte kInstruction_add_vs_r1_r1_192[] = {
  0x68, 0xbf, 0xc0, 0x31 // It vs; add vs r1 r1 192
};
const byte kInstruction_add_gt_r3_r3_53[] = {
  0xc8, 0xbf, 0x35, 0x33 // It gt; add gt r3 r3 53
};
const byte kInstruction_add_lt_r6_r6_229[] = {
  0xb8, 0xbf, 0xe5, 0x36 // It lt; add lt r6 r6 229
};
const byte kInstruction_add_lt_r4_r4_128[] = {
  0xb8, 0xbf, 0x80, 0x34 // It lt; add lt r4 r4 128
};
const byte kInstruction_add_ne_r2_r2_168[] = {
  0x18, 0xbf, 0xa8, 0x32 // It ne; add ne r2 r2 168
};
const byte kInstruction_add_cs_r7_r7_91[] = {
  0x28, 0xbf, 0x5b, 0x37 // It cs; add cs r7 r7 91
};
const byte kInstruction_add_mi_r5_r5_112[] = {
  0x48, 0xbf, 0x70, 0x35 // It mi; add mi r5 r5 112
};
const byte kInstruction_add_vc_r0_r0_200[] = {
  0x78, 0xbf, 0xc8, 0x30 // It vc; add vc r0 r0 200
};
const byte kInstruction_add_cs_r5_r5_82[] = {
  0x28, 0xbf, 0x52, 0x35 // It cs; add cs r5 r5 82
};
const byte kInstruction_add_mi_r4_r4_176[] = {
  0x48, 0xbf, 0xb0, 0x34 // It mi; add mi r4 r4 176
};
const byte kInstruction_add_ls_r2_r2_202[] = {
  0x98, 0xbf, 0xca, 0x32 // It ls; add ls r2 r2 202
};
const byte kInstruction_add_mi_r1_r1_106[] = {
  0x48, 0xbf, 0x6a, 0x31 // It mi; add mi r1 r1 106
};
const byte kInstruction_add_cc_r4_r4_144[] = {
  0x38, 0xbf, 0x90, 0x34 // It cc; add cc r4 r4 144
};
const byte kInstruction_add_cc_r4_r4_241[] = {
  0x38, 0xbf, 0xf1, 0x34 // It cc; add cc r4 r4 241
};
const byte kInstruction_add_vc_r3_r3_107[] = {
  0x78, 0xbf, 0x6b, 0x33 // It vc; add vc r3 r3 107
};
const byte kInstruction_add_ls_r2_r2_21[] = {
  0x98, 0xbf, 0x15, 0x32 // It ls; add ls r2 r2 21
};
const byte kInstruction_add_mi_r2_r2_54[] = {
  0x48, 0xbf, 0x36, 0x32 // It mi; add mi r2 r2 54
};
const byte kInstruction_add_ge_r1_r1_69[] = {
  0xa8, 0xbf, 0x45, 0x31 // It ge; add ge r1 r1 69
};
const byte kInstruction_add_pl_r2_r2_165[] = {
  0x58, 0xbf, 0xa5, 0x32 // It pl; add pl r2 r2 165
};
const byte kInstruction_add_gt_r2_r2_154[] = {
  0xc8, 0xbf, 0x9a, 0x32 // It gt; add gt r2 r2 154
};
const byte kInstruction_add_gt_r4_r4_195[] = {
  0xc8, 0xbf, 0xc3, 0x34 // It gt; add gt r4 r4 195
};
const byte kInstruction_add_ne_r0_r0_223[] = {
  0x18, 0xbf, 0xdf, 0x30 // It ne; add ne r0 r0 223
};
const byte kInstruction_add_vs_r6_r6_79[] = {
  0x68, 0xbf, 0x4f, 0x36 // It vs; add vs r6 r6 79
};
const byte kInstruction_add_lt_r4_r4_57[] = {
  0xb8, 0xbf, 0x39, 0x34 // It lt; add lt r4 r4 57
};
const byte kInstruction_add_vs_r0_r0_157[] = {
  0x68, 0xbf, 0x9d, 0x30 // It vs; add vs r0 r0 157
};
const byte kInstruction_add_vc_r2_r2_69[] = {
  0x78, 0xbf, 0x45, 0x32 // It vc; add vc r2 r2 69
};
const byte kInstruction_add_vc_r7_r7_167[] = {
  0x78, 0xbf, 0xa7, 0x37 // It vc; add vc r7 r7 167
};
const byte kInstruction_add_lt_r4_r4_216[] = {
  0xb8, 0xbf, 0xd8, 0x34 // It lt; add lt r4 r4 216
};
const byte kInstruction_add_le_r1_r1_159[] = {
  0xd8, 0xbf, 0x9f, 0x31 // It le; add le r1 r1 159
};
const byte kInstruction_add_hi_r6_r6_193[] = {
  0x88, 0xbf, 0xc1, 0x36 // It hi; add hi r6 r6 193
};
const byte kInstruction_add_gt_r7_r7_194[] = {
  0xc8, 0xbf, 0xc2, 0x37 // It gt; add gt r7 r7 194
};
const byte kInstruction_add_pl_r1_r1_39[] = {
  0x58, 0xbf, 0x27, 0x31 // It pl; add pl r1 r1 39
};
const byte kInstruction_add_lt_r5_r5_215[] = {
  0xb8, 0xbf, 0xd7, 0x35 // It lt; add lt r5 r5 215
};
const byte kInstruction_add_vc_r1_r1_11[] = {
  0x78, 0xbf, 0x0b, 0x31 // It vc; add vc r1 r1 11
};
const byte kInstruction_add_le_r0_r0_242[] = {
  0xd8, 0xbf, 0xf2, 0x30 // It le; add le r0 r0 242
};
const byte kInstruction_add_eq_r3_r3_215[] = {
  0x08, 0xbf, 0xd7, 0x33 // It eq; add eq r3 r3 215
};
const byte kInstruction_add_vs_r3_r3_37[] = {
  0x68, 0xbf, 0x25, 0x33 // It vs; add vs r3 r3 37
};
const byte kInstruction_add_cs_r5_r5_78[] = {
  0x28, 0xbf, 0x4e, 0x35 // It cs; add cs r5 r5 78
};
const byte kInstruction_add_ne_r1_r1_158[] = {
  0x18, 0xbf, 0x9e, 0x31 // It ne; add ne r1 r1 158
};
const byte kInstruction_add_ne_r0_r0_174[] = {
  0x18, 0xbf, 0xae, 0x30 // It ne; add ne r0 r0 174
};
const byte kInstruction_add_le_r1_r1_81[] = {
  0xd8, 0xbf, 0x51, 0x31 // It le; add le r1 r1 81
};
const byte kInstruction_add_le_r5_r5_250[] = {
  0xd8, 0xbf, 0xfa, 0x35 // It le; add le r5 r5 250
};
const byte kInstruction_add_pl_r7_r7_101[] = {
  0x58, 0xbf, 0x65, 0x37 // It pl; add pl r7 r7 101
};
const byte kInstruction_add_hi_r7_r7_188[] = {
  0x88, 0xbf, 0xbc, 0x37 // It hi; add hi r7 r7 188
};
const byte kInstruction_add_hi_r4_r4_88[] = {
  0x88, 0xbf, 0x58, 0x34 // It hi; add hi r4 r4 88
};
const byte kInstruction_add_ls_r1_r1_42[] = {
  0x98, 0xbf, 0x2a, 0x31 // It ls; add ls r1 r1 42
};
const byte kInstruction_add_vc_r2_r2_19[] = {
  0x78, 0xbf, 0x13, 0x32 // It vc; add vc r2 r2 19
};
const byte kInstruction_add_mi_r6_r6_125[] = {
  0x48, 0xbf, 0x7d, 0x36 // It mi; add mi r6 r6 125
};
const byte kInstruction_add_ls_r2_r2_82[] = {
  0x98, 0xbf, 0x52, 0x32 // It ls; add ls r2 r2 82
};
const byte kInstruction_add_gt_r4_r4_150[] = {
  0xc8, 0xbf, 0x96, 0x34 // It gt; add gt r4 r4 150
};
const byte kInstruction_add_ne_r5_r5_40[] = {
  0x18, 0xbf, 0x28, 0x35 // It ne; add ne r5 r5 40
};
const byte kInstruction_add_cs_r4_r4_221[] = {
  0x28, 0xbf, 0xdd, 0x34 // It cs; add cs r4 r4 221
};
const byte kInstruction_add_cc_r6_r6_120[] = {
  0x38, 0xbf, 0x78, 0x36 // It cc; add cc r6 r6 120
};
const byte kInstruction_add_cc_r2_r2_211[] = {
  0x38, 0xbf, 0xd3, 0x32 // It cc; add cc r2 r2 211
};
const byte kInstruction_add_cs_r6_r6_67[] = {
  0x28, 0xbf, 0x43, 0x36 // It cs; add cs r6 r6 67
};
const byte kInstruction_add_gt_r0_r0_47[] = {
  0xc8, 0xbf, 0x2f, 0x30 // It gt; add gt r0 r0 47
};
const byte kInstruction_add_pl_r7_r7_75[] = {
  0x58, 0xbf, 0x4b, 0x37 // It pl; add pl r7 r7 75
};
const byte kInstruction_add_ls_r1_r1_234[] = {
  0x98, 0xbf, 0xea, 0x31 // It ls; add ls r1 r1 234
};
const byte kInstruction_add_gt_r2_r2_113[] = {
  0xc8, 0xbf, 0x71, 0x32 // It gt; add gt r2 r2 113
};
const byte kInstruction_add_lt_r2_r2_85[] = {
  0xb8, 0xbf, 0x55, 0x32 // It lt; add lt r2 r2 85
};
const byte kInstruction_add_hi_r6_r6_240[] = {
  0x88, 0xbf, 0xf0, 0x36 // It hi; add hi r6 r6 240
};
const byte kInstruction_add_lt_r3_r3_93[] = {
  0xb8, 0xbf, 0x5d, 0x33 // It lt; add lt r3 r3 93
};
const byte kInstruction_add_lt_r4_r4_192[] = {
  0xb8, 0xbf, 0xc0, 0x34 // It lt; add lt r4 r4 192
};
const byte kInstruction_add_gt_r1_r1_148[] = {
  0xc8, 0xbf, 0x94, 0x31 // It gt; add gt r1 r1 148
};
const byte kInstruction_add_cs_r2_r2_122[] = {
  0x28, 0xbf, 0x7a, 0x32 // It cs; add cs r2 r2 122
};
const byte kInstruction_add_ne_r4_r4_111[] = {
  0x18, 0xbf, 0x6f, 0x34 // It ne; add ne r4 r4 111
};
const byte kInstruction_add_cs_r2_r2_47[] = {
  0x28, 0xbf, 0x2f, 0x32 // It cs; add cs r2 r2 47
};
const byte kInstruction_add_cs_r4_r4_186[] = {
  0x28, 0xbf, 0xba, 0x34 // It cs; add cs r4 r4 186
};
const byte kInstruction_add_le_r0_r0_183[] = {
  0xd8, 0xbf, 0xb7, 0x30 // It le; add le r0 r0 183
};
const byte kInstruction_add_vs_r6_r6_232[] = {
  0x68, 0xbf, 0xe8, 0x36 // It vs; add vs r6 r6 232
};
const byte kInstruction_add_cc_r3_r3_99[] = {
  0x38, 0xbf, 0x63, 0x33 // It cc; add cc r3 r3 99
};
const byte kInstruction_add_lt_r4_r4_81[] = {
  0xb8, 0xbf, 0x51, 0x34 // It lt; add lt r4 r4 81
};
const byte kInstruction_add_lt_r3_r3_192[] = {
  0xb8, 0xbf, 0xc0, 0x33 // It lt; add lt r3 r3 192
};
const byte kInstruction_add_ls_r5_r5_219[] = {
  0x98, 0xbf, 0xdb, 0x35 // It ls; add ls r5 r5 219
};
const byte kInstruction_add_lt_r4_r4_153[] = {
  0xb8, 0xbf, 0x99, 0x34 // It lt; add lt r4 r4 153
};
const byte kInstruction_add_vs_r6_r6_76[] = {
  0x68, 0xbf, 0x4c, 0x36 // It vs; add vs r6 r6 76
};
const byte kInstruction_add_eq_r7_r7_37[] = {
  0x08, 0xbf, 0x25, 0x37 // It eq; add eq r7 r7 37
};
const byte kInstruction_add_vs_r6_r6_202[] = {
  0x68, 0xbf, 0xca, 0x36 // It vs; add vs r6 r6 202
};
const byte kInstruction_add_vs_r2_r2_7[] = {
  0x68, 0xbf, 0xd2, 0x1d // It vs; add vs r2 r2 7
};
const byte kInstruction_add_vs_r3_r3_99[] = {
  0x68, 0xbf, 0x63, 0x33 // It vs; add vs r3 r3 99
};
const byte kInstruction_add_vs_r0_r0_224[] = {
  0x68, 0xbf, 0xe0, 0x30 // It vs; add vs r0 r0 224
};
const byte kInstruction_add_le_r5_r5_26[] = {
  0xd8, 0xbf, 0x1a, 0x35 // It le; add le r5 r5 26
};
const byte kInstruction_add_eq_r0_r0_92[] = {
  0x08, 0xbf, 0x5c, 0x30 // It eq; add eq r0 r0 92
};
const byte kInstruction_add_cc_r7_r7_142[] = {
  0x38, 0xbf, 0x8e, 0x37 // It cc; add cc r7 r7 142
};
const byte kInstruction_add_ls_r2_r2_20[] = {
  0x98, 0xbf, 0x14, 0x32 // It ls; add ls r2 r2 20
};
const byte kInstruction_add_hi_r4_r4_0[] = {
  0x88, 0xbf, 0x24, 0x1c // It hi; add hi r4 r4 0
};
const byte kInstruction_add_ne_r0_r0_107[] = {
  0x18, 0xbf, 0x6b, 0x30 // It ne; add ne r0 r0 107
};
const byte kInstruction_add_vs_r6_r6_156[] = {
  0x68, 0xbf, 0x9c, 0x36 // It vs; add vs r6 r6 156
};
const byte kInstruction_add_le_r4_r4_32[] = {
  0xd8, 0xbf, 0x20, 0x34 // It le; add le r4 r4 32
};
const byte kInstruction_add_vc_r6_r6_184[] = {
  0x78, 0xbf, 0xb8, 0x36 // It vc; add vc r6 r6 184
};
const byte kInstruction_add_cs_r7_r7_215[] = {
  0x28, 0xbf, 0xd7, 0x37 // It cs; add cs r7 r7 215
};
const byte kInstruction_add_gt_r7_r7_69[] = {
  0xc8, 0xbf, 0x45, 0x37 // It gt; add gt r7 r7 69
};
const byte kInstruction_add_le_r5_r5_87[] = {
  0xd8, 0xbf, 0x57, 0x35 // It le; add le r5 r5 87
};
const byte kInstruction_add_cs_r7_r7_153[] = {
  0x28, 0xbf, 0x99, 0x37 // It cs; add cs r7 r7 153
};
const byte kInstruction_add_mi_r0_r0_54[] = {
  0x48, 0xbf, 0x36, 0x30 // It mi; add mi r0 r0 54
};
const byte kInstruction_add_ne_r1_r1_215[] = {
  0x18, 0xbf, 0xd7, 0x31 // It ne; add ne r1 r1 215
};
const byte kInstruction_add_pl_r2_r2_175[] = {
  0x58, 0xbf, 0xaf, 0x32 // It pl; add pl r2 r2 175
};
const byte kInstruction_add_ne_r5_r5_46[] = {
  0x18, 0xbf, 0x2e, 0x35 // It ne; add ne r5 r5 46
};
const byte kInstruction_add_le_r3_r3_183[] = {
  0xd8, 0xbf, 0xb7, 0x33 // It le; add le r3 r3 183
};
const byte kInstruction_add_pl_r3_r3_89[] = {
  0x58, 0xbf, 0x59, 0x33 // It pl; add pl r3 r3 89
};
const byte kInstruction_add_ne_r0_r0_229[] = {
  0x18, 0xbf, 0xe5, 0x30 // It ne; add ne r0 r0 229
};
const byte kInstruction_add_cc_r0_r0_142[] = {
  0x38, 0xbf, 0x8e, 0x30 // It cc; add cc r0 r0 142
};
const byte kInstruction_add_ne_r3_r3_101[] = {
  0x18, 0xbf, 0x65, 0x33 // It ne; add ne r3 r3 101
};
const byte kInstruction_add_vc_r3_r3_108[] = {
  0x78, 0xbf, 0x6c, 0x33 // It vc; add vc r3 r3 108
};
const byte kInstruction_add_eq_r7_r7_96[] = {
  0x08, 0xbf, 0x60, 0x37 // It eq; add eq r7 r7 96
};
const byte kInstruction_add_cc_r0_r0_204[] = {
  0x38, 0xbf, 0xcc, 0x30 // It cc; add cc r0 r0 204
};
const byte kInstruction_add_hi_r6_r6_58[] = {
  0x88, 0xbf, 0x3a, 0x36 // It hi; add hi r6 r6 58
};
const byte kInstruction_add_vc_r6_r6_2[] = {
  0x78, 0xbf, 0xb6, 0x1c // It vc; add vc r6 r6 2
};
const byte kInstruction_add_cc_r0_r0_21[] = {
  0x38, 0xbf, 0x15, 0x30 // It cc; add cc r0 r0 21
};
const byte kInstruction_add_ge_r0_r0_223[] = {
  0xa8, 0xbf, 0xdf, 0x30 // It ge; add ge r0 r0 223
};
const byte kInstruction_add_eq_r3_r3_82[] = {
  0x08, 0xbf, 0x52, 0x33 // It eq; add eq r3 r3 82
};
const byte kInstruction_add_hi_r3_r3_209[] = {
  0x88, 0xbf, 0xd1, 0x33 // It hi; add hi r3 r3 209
};
const byte kInstruction_add_vs_r3_r3_89[] = {
  0x68, 0xbf, 0x59, 0x33 // It vs; add vs r3 r3 89
};
const byte kInstruction_add_ls_r2_r2_19[] = {
  0x98, 0xbf, 0x13, 0x32 // It ls; add ls r2 r2 19
};
const byte kInstruction_add_ne_r3_r3_127[] = {
  0x18, 0xbf, 0x7f, 0x33 // It ne; add ne r3 r3 127
};
const byte kInstruction_add_cc_r1_r1_87[] = {
  0x38, 0xbf, 0x57, 0x31 // It cc; add cc r1 r1 87
};
const byte kInstruction_add_lt_r0_r0_36[] = {
  0xb8, 0xbf, 0x24, 0x30 // It lt; add lt r0 r0 36
};
const byte kInstruction_add_ne_r3_r3_217[] = {
  0x18, 0xbf, 0xd9, 0x33 // It ne; add ne r3 r3 217
};
const byte kInstruction_add_hi_r2_r2_92[] = {
  0x88, 0xbf, 0x5c, 0x32 // It hi; add hi r2 r2 92
};
const byte kInstruction_add_vs_r3_r3_231[] = {
  0x68, 0xbf, 0xe7, 0x33 // It vs; add vs r3 r3 231
};
const byte kInstruction_add_vc_r2_r2_8[] = {
  0x78, 0xbf, 0x08, 0x32 // It vc; add vc r2 r2 8
};
const byte kInstruction_add_vs_r3_r3_49[] = {
  0x68, 0xbf, 0x31, 0x33 // It vs; add vs r3 r3 49
};
const byte kInstruction_add_pl_r3_r3_223[] = {
  0x58, 0xbf, 0xdf, 0x33 // It pl; add pl r3 r3 223
};
const byte kInstruction_add_ls_r3_r3_201[] = {
  0x98, 0xbf, 0xc9, 0x33 // It ls; add ls r3 r3 201
};
const byte kInstruction_add_vc_r7_r7_215[] = {
  0x78, 0xbf, 0xd7, 0x37 // It vc; add vc r7 r7 215
};
const byte kInstruction_add_ge_r4_r4_208[] = {
  0xa8, 0xbf, 0xd0, 0x34 // It ge; add ge r4 r4 208
};
const byte kInstruction_add_eq_r4_r4_170[] = {
  0x08, 0xbf, 0xaa, 0x34 // It eq; add eq r4 r4 170
};
const byte kInstruction_add_vs_r0_r0_45[] = {
  0x68, 0xbf, 0x2d, 0x30 // It vs; add vs r0 r0 45
};
const byte kInstruction_add_mi_r4_r4_237[] = {
  0x48, 0xbf, 0xed, 0x34 // It mi; add mi r4 r4 237
};
const byte kInstruction_add_eq_r5_r5_142[] = {
  0x08, 0xbf, 0x8e, 0x35 // It eq; add eq r5 r5 142
};
const byte kInstruction_add_eq_r4_r4_100[] = {
  0x08, 0xbf, 0x64, 0x34 // It eq; add eq r4 r4 100
};
const byte kInstruction_add_mi_r1_r1_73[] = {
  0x48, 0xbf, 0x49, 0x31 // It mi; add mi r1 r1 73
};
const byte kInstruction_add_lt_r0_r0_244[] = {
  0xb8, 0xbf, 0xf4, 0x30 // It lt; add lt r0 r0 244
};
const byte kInstruction_add_ge_r7_r7_233[] = {
  0xa8, 0xbf, 0xe9, 0x37 // It ge; add ge r7 r7 233
};
const byte kInstruction_add_vs_r3_r3_253[] = {
  0x68, 0xbf, 0xfd, 0x33 // It vs; add vs r3 r3 253
};
const byte kInstruction_add_cs_r5_r5_20[] = {
  0x28, 0xbf, 0x14, 0x35 // It cs; add cs r5 r5 20
};
const byte kInstruction_add_pl_r2_r2_84[] = {
  0x58, 0xbf, 0x54, 0x32 // It pl; add pl r2 r2 84
};
const byte kInstruction_add_ne_r4_r4_90[] = {
  0x18, 0xbf, 0x5a, 0x34 // It ne; add ne r4 r4 90
};
const byte kInstruction_add_ne_r6_r6_32[] = {
  0x18, 0xbf, 0x20, 0x36 // It ne; add ne r6 r6 32
};
const byte kInstruction_add_gt_r6_r6_55[] = {
  0xc8, 0xbf, 0x37, 0x36 // It gt; add gt r6 r6 55
};
const byte kInstruction_add_pl_r2_r2_95[] = {
  0x58, 0xbf, 0x5f, 0x32 // It pl; add pl r2 r2 95
};
const byte kInstruction_add_vc_r4_r4_67[] = {
  0x78, 0xbf, 0x43, 0x34 // It vc; add vc r4 r4 67
};
const byte kInstruction_add_vc_r3_r3_236[] = {
  0x78, 0xbf, 0xec, 0x33 // It vc; add vc r3 r3 236
};
const byte kInstruction_add_mi_r7_r7_109[] = {
  0x48, 0xbf, 0x6d, 0x37 // It mi; add mi r7 r7 109
};
const byte kInstruction_add_hi_r0_r0_174[] = {
  0x88, 0xbf, 0xae, 0x30 // It hi; add hi r0 r0 174
};
const byte kInstruction_add_ne_r3_r3_6[] = {
  0x18, 0xbf, 0x9b, 0x1d // It ne; add ne r3 r3 6
};
const byte kInstruction_add_mi_r4_r4_17[] = {
  0x48, 0xbf, 0x11, 0x34 // It mi; add mi r4 r4 17
};
const byte kInstruction_add_ls_r3_r3_20[] = {
  0x98, 0xbf, 0x14, 0x33 // It ls; add ls r3 r3 20
};
const byte kInstruction_add_mi_r7_r7_93[] = {
  0x48, 0xbf, 0x5d, 0x37 // It mi; add mi r7 r7 93
};
const byte kInstruction_add_eq_r5_r5_9[] = {
  0x08, 0xbf, 0x09, 0x35 // It eq; add eq r5 r5 9
};
const byte kInstruction_add_cs_r4_r4_88[] = {
  0x28, 0xbf, 0x58, 0x34 // It cs; add cs r4 r4 88
};
const byte kInstruction_add_cs_r4_r4_92[] = {
  0x28, 0xbf, 0x5c, 0x34 // It cs; add cs r4 r4 92
};
const byte kInstruction_add_gt_r4_r4_69[] = {
  0xc8, 0xbf, 0x45, 0x34 // It gt; add gt r4 r4 69
};
const byte kInstruction_add_ge_r0_r0_219[] = {
  0xa8, 0xbf, 0xdb, 0x30 // It ge; add ge r0 r0 219
};
const byte kInstruction_add_gt_r4_r4_15[] = {
  0xc8, 0xbf, 0x0f, 0x34 // It gt; add gt r4 r4 15
};
const byte kInstruction_add_gt_r5_r5_105[] = {
  0xc8, 0xbf, 0x69, 0x35 // It gt; add gt r5 r5 105
};
const byte kInstruction_add_hi_r2_r2_229[] = {
  0x88, 0xbf, 0xe5, 0x32 // It hi; add hi r2 r2 229
};
const byte kInstruction_add_ls_r7_r7_32[] = {
  0x98, 0xbf, 0x20, 0x37 // It ls; add ls r7 r7 32
};
const byte kInstruction_add_gt_r5_r5_79[] = {
  0xc8, 0xbf, 0x4f, 0x35 // It gt; add gt r5 r5 79
};
const byte kInstruction_add_ls_r7_r7_57[] = {
  0x98, 0xbf, 0x39, 0x37 // It ls; add ls r7 r7 57
};
const byte kInstruction_add_pl_r2_r2_17[] = {
  0x58, 0xbf, 0x11, 0x32 // It pl; add pl r2 r2 17
};
const byte kInstruction_add_cc_r6_r6_67[] = {
  0x38, 0xbf, 0x43, 0x36 // It cc; add cc r6 r6 67
};
const byte kInstruction_add_cs_r7_r7_11[] = {
  0x28, 0xbf, 0x0b, 0x37 // It cs; add cs r7 r7 11
};
const byte kInstruction_add_mi_r6_r6_150[] = {
  0x48, 0xbf, 0x96, 0x36 // It mi; add mi r6 r6 150
};
const byte kInstruction_add_pl_r3_r3_241[] = {
  0x58, 0xbf, 0xf1, 0x33 // It pl; add pl r3 r3 241
};
const byte kInstruction_add_cc_r2_r2_65[] = {
  0x38, 0xbf, 0x41, 0x32 // It cc; add cc r2 r2 65
};
const byte kInstruction_add_ne_r4_r4_81[] = {
  0x18, 0xbf, 0x51, 0x34 // It ne; add ne r4 r4 81
};
const byte kInstruction_add_lt_r4_r4_176[] = {
  0xb8, 0xbf, 0xb0, 0x34 // It lt; add lt r4 r4 176
};
const byte kInstruction_add_cs_r7_r7_58[] = {
  0x28, 0xbf, 0x3a, 0x37 // It cs; add cs r7 r7 58
};
const byte kInstruction_add_gt_r1_r1_114[] = {
  0xc8, 0xbf, 0x72, 0x31 // It gt; add gt r1 r1 114
};
const byte kInstruction_add_vc_r7_r7_4[] = {
  0x78, 0xbf, 0x3f, 0x1d // It vc; add vc r7 r7 4
};
const byte kInstruction_add_le_r3_r3_110[] = {
  0xd8, 0xbf, 0x6e, 0x33 // It le; add le r3 r3 110
};
const byte kInstruction_add_vs_r1_r1_149[] = {
  0x68, 0xbf, 0x95, 0x31 // It vs; add vs r1 r1 149
};
const byte kInstruction_add_cs_r3_r3_179[] = {
  0x28, 0xbf, 0xb3, 0x33 // It cs; add cs r3 r3 179
};
const byte kInstruction_add_vs_r7_r7_199[] = {
  0x68, 0xbf, 0xc7, 0x37 // It vs; add vs r7 r7 199
};
const byte kInstruction_add_ge_r4_r4_46[] = {
  0xa8, 0xbf, 0x2e, 0x34 // It ge; add ge r4 r4 46
};
const byte kInstruction_add_ls_r1_r1_98[] = {
  0x98, 0xbf, 0x62, 0x31 // It ls; add ls r1 r1 98
};
const byte kInstruction_add_cs_r3_r3_60[] = {
  0x28, 0xbf, 0x3c, 0x33 // It cs; add cs r3 r3 60
};
const byte kInstruction_add_vc_r2_r2_201[] = {
  0x78, 0xbf, 0xc9, 0x32 // It vc; add vc r2 r2 201
};
const byte kInstruction_add_le_r5_r5_128[] = {
  0xd8, 0xbf, 0x80, 0x35 // It le; add le r5 r5 128
};
const byte kInstruction_add_vs_r4_r4_39[] = {
  0x68, 0xbf, 0x27, 0x34 // It vs; add vs r4 r4 39
};
const byte kInstruction_add_ls_r4_r4_106[] = {
  0x98, 0xbf, 0x6a, 0x34 // It ls; add ls r4 r4 106
};
const byte kInstruction_add_le_r5_r5_207[] = {
  0xd8, 0xbf, 0xcf, 0x35 // It le; add le r5 r5 207
};
const byte kInstruction_add_cs_r6_r6_7[] = {
  0x28, 0xbf, 0xf6, 0x1d // It cs; add cs r6 r6 7
};
const byte kInstruction_add_ne_r5_r5_133[] = {
  0x18, 0xbf, 0x85, 0x35 // It ne; add ne r5 r5 133
};
const byte kInstruction_add_eq_r7_r7_28[] = {
  0x08, 0xbf, 0x1c, 0x37 // It eq; add eq r7 r7 28
};
const byte kInstruction_add_pl_r7_r7_154[] = {
  0x58, 0xbf, 0x9a, 0x37 // It pl; add pl r7 r7 154
};
const byte kInstruction_add_vs_r3_r3_242[] = {
  0x68, 0xbf, 0xf2, 0x33 // It vs; add vs r3 r3 242
};
const byte kInstruction_add_eq_r6_r6_221[] = {
  0x08, 0xbf, 0xdd, 0x36 // It eq; add eq r6 r6 221
};
const byte kInstruction_add_vs_r5_r5_97[] = {
  0x68, 0xbf, 0x61, 0x35 // It vs; add vs r5 r5 97
};
const byte kInstruction_add_ls_r5_r5_70[] = {
  0x98, 0xbf, 0x46, 0x35 // It ls; add ls r5 r5 70
};
const byte kInstruction_add_vc_r6_r6_236[] = {
  0x78, 0xbf, 0xec, 0x36 // It vc; add vc r6 r6 236
};
const byte kInstruction_add_ne_r2_r2_236[] = {
  0x18, 0xbf, 0xec, 0x32 // It ne; add ne r2 r2 236
};
const byte kInstruction_add_eq_r1_r1_198[] = {
  0x08, 0xbf, 0xc6, 0x31 // It eq; add eq r1 r1 198
};
const byte kInstruction_add_vs_r3_r3_134[] = {
  0x68, 0xbf, 0x86, 0x33 // It vs; add vs r3 r3 134
};
const byte kInstruction_add_ne_r5_r5_42[] = {
  0x18, 0xbf, 0x2a, 0x35 // It ne; add ne r5 r5 42
};
const byte kInstruction_add_ge_r6_r6_223[] = {
  0xa8, 0xbf, 0xdf, 0x36 // It ge; add ge r6 r6 223
};
const byte kInstruction_add_ge_r0_r0_48[] = {
  0xa8, 0xbf, 0x30, 0x30 // It ge; add ge r0 r0 48
};
const byte kInstruction_add_hi_r7_r7_146[] = {
  0x88, 0xbf, 0x92, 0x37 // It hi; add hi r7 r7 146
};
const byte kInstruction_add_ge_r4_r4_175[] = {
  0xa8, 0xbf, 0xaf, 0x34 // It ge; add ge r4 r4 175
};
const byte kInstruction_add_gt_r4_r4_142[] = {
  0xc8, 0xbf, 0x8e, 0x34 // It gt; add gt r4 r4 142
};
const byte kInstruction_add_ne_r6_r6_24[] = {
  0x18, 0xbf, 0x18, 0x36 // It ne; add ne r6 r6 24
};
const byte kInstruction_add_lt_r2_r2_152[] = {
  0xb8, 0xbf, 0x98, 0x32 // It lt; add lt r2 r2 152
};
const byte kInstruction_add_hi_r2_r2_113[] = {
  0x88, 0xbf, 0x71, 0x32 // It hi; add hi r2 r2 113
};
const byte kInstruction_add_vc_r4_r4_135[] = {
  0x78, 0xbf, 0x87, 0x34 // It vc; add vc r4 r4 135
};
const byte kInstruction_add_ls_r4_r4_40[] = {
  0x98, 0xbf, 0x28, 0x34 // It ls; add ls r4 r4 40
};
const byte kInstruction_add_cc_r2_r2_199[] = {
  0x38, 0xbf, 0xc7, 0x32 // It cc; add cc r2 r2 199
};
const byte kInstruction_add_vs_r6_r6_164[] = {
  0x68, 0xbf, 0xa4, 0x36 // It vs; add vs r6 r6 164
};
const byte kInstruction_add_le_r3_r3_184[] = {
  0xd8, 0xbf, 0xb8, 0x33 // It le; add le r3 r3 184
};
const byte kInstruction_add_cs_r4_r4_158[] = {
  0x28, 0xbf, 0x9e, 0x34 // It cs; add cs r4 r4 158
};
const byte kInstruction_add_vc_r5_r5_212[] = {
  0x78, 0xbf, 0xd4, 0x35 // It vc; add vc r5 r5 212
};
const byte kInstruction_add_gt_r5_r5_143[] = {
  0xc8, 0xbf, 0x8f, 0x35 // It gt; add gt r5 r5 143
};
const byte kInstruction_add_ne_r0_r0_186[] = {
  0x18, 0xbf, 0xba, 0x30 // It ne; add ne r0 r0 186
};
const byte kInstruction_add_cs_r5_r5_233[] = {
  0x28, 0xbf, 0xe9, 0x35 // It cs; add cs r5 r5 233
};
const byte kInstruction_add_le_r1_r1_242[] = {
  0xd8, 0xbf, 0xf2, 0x31 // It le; add le r1 r1 242
};
const byte kInstruction_add_ge_r6_r6_70[] = {
  0xa8, 0xbf, 0x46, 0x36 // It ge; add ge r6 r6 70
};
const byte kInstruction_add_cc_r7_r7_234[] = {
  0x38, 0xbf, 0xea, 0x37 // It cc; add cc r7 r7 234
};
const byte kInstruction_add_ls_r0_r0_191[] = {
  0x98, 0xbf, 0xbf, 0x30 // It ls; add ls r0 r0 191
};
const byte kInstruction_add_pl_r3_r3_236[] = {
  0x58, 0xbf, 0xec, 0x33 // It pl; add pl r3 r3 236
};
const byte kInstruction_add_cc_r7_r7_27[] = {
  0x38, 0xbf, 0x1b, 0x37 // It cc; add cc r7 r7 27
};
const byte kInstruction_add_vs_r0_r0_169[] = {
  0x68, 0xbf, 0xa9, 0x30 // It vs; add vs r0 r0 169
};
const byte kInstruction_add_vs_r2_r2_193[] = {
  0x68, 0xbf, 0xc1, 0x32 // It vs; add vs r2 r2 193
};
const byte kInstruction_add_lt_r6_r6_198[] = {
  0xb8, 0xbf, 0xc6, 0x36 // It lt; add lt r6 r6 198
};
const byte kInstruction_add_ge_r7_r7_137[] = {
  0xa8, 0xbf, 0x89, 0x37 // It ge; add ge r7 r7 137
};
const byte kInstruction_add_vs_r1_r1_183[] = {
  0x68, 0xbf, 0xb7, 0x31 // It vs; add vs r1 r1 183
};
const byte kInstruction_add_lt_r5_r5_180[] = {
  0xb8, 0xbf, 0xb4, 0x35 // It lt; add lt r5 r5 180
};
const byte kInstruction_add_le_r3_r3_22[] = {
  0xd8, 0xbf, 0x16, 0x33 // It le; add le r3 r3 22
};
const byte kInstruction_add_ls_r7_r7_41[] = {
  0x98, 0xbf, 0x29, 0x37 // It ls; add ls r7 r7 41
};
const byte kInstruction_add_vs_r6_r6_71[] = {
  0x68, 0xbf, 0x47, 0x36 // It vs; add vs r6 r6 71
};
const byte kInstruction_add_vs_r1_r1_81[] = {
  0x68, 0xbf, 0x51, 0x31 // It vs; add vs r1 r1 81
};
const byte kInstruction_add_vc_r0_r0_181[] = {
  0x78, 0xbf, 0xb5, 0x30 // It vc; add vc r0 r0 181
};
const byte kInstruction_add_lt_r0_r0_1[] = {
  0xb8, 0xbf, 0x40, 0x1c // It lt; add lt r0 r0 1
};
const byte kInstruction_add_le_r1_r1_182[] = {
  0xd8, 0xbf, 0xb6, 0x31 // It le; add le r1 r1 182
};
const byte kInstruction_add_ls_r1_r1_167[] = {
  0x98, 0xbf, 0xa7, 0x31 // It ls; add ls r1 r1 167
};
const byte kInstruction_add_hi_r7_r7_221[] = {
  0x88, 0xbf, 0xdd, 0x37 // It hi; add hi r7 r7 221
};
const byte kInstruction_add_ge_r1_r1_72[] = {
  0xa8, 0xbf, 0x48, 0x31 // It ge; add ge r1 r1 72
};
const byte kInstruction_add_pl_r7_r7_95[] = {
  0x58, 0xbf, 0x5f, 0x37 // It pl; add pl r7 r7 95
};
const byte kInstruction_add_cs_r3_r3_252[] = {
  0x28, 0xbf, 0xfc, 0x33 // It cs; add cs r3 r3 252
};
const byte kInstruction_add_eq_r3_r3_224[] = {
  0x08, 0xbf, 0xe0, 0x33 // It eq; add eq r3 r3 224
};
const byte kInstruction_add_mi_r5_r5_176[] = {
  0x48, 0xbf, 0xb0, 0x35 // It mi; add mi r5 r5 176
};
const byte kInstruction_add_le_r0_r0_213[] = {
  0xd8, 0xbf, 0xd5, 0x30 // It le; add le r0 r0 213
};
const byte kInstruction_add_gt_r1_r1_91[] = {
  0xc8, 0xbf, 0x5b, 0x31 // It gt; add gt r1 r1 91
};
const byte kInstruction_add_vc_r5_r5_236[] = {
  0x78, 0xbf, 0xec, 0x35 // It vc; add vc r5 r5 236
};
const byte kInstruction_add_hi_r4_r4_68[] = {
  0x88, 0xbf, 0x44, 0x34 // It hi; add hi r4 r4 68
};
const byte kInstruction_add_ls_r6_r6_201[] = {
  0x98, 0xbf, 0xc9, 0x36 // It ls; add ls r6 r6 201
};
const byte kInstruction_add_vs_r6_r6_146[] = {
  0x68, 0xbf, 0x92, 0x36 // It vs; add vs r6 r6 146
};
const byte kInstruction_add_le_r5_r5_169[] = {
  0xd8, 0xbf, 0xa9, 0x35 // It le; add le r5 r5 169
};
const byte kInstruction_add_pl_r1_r1_88[] = {
  0x58, 0xbf, 0x58, 0x31 // It pl; add pl r1 r1 88
};
const byte kInstruction_add_cc_r4_r4_26[] = {
  0x38, 0xbf, 0x1a, 0x34 // It cc; add cc r4 r4 26
};
const byte kInstruction_add_gt_r2_r2_228[] = {
  0xc8, 0xbf, 0xe4, 0x32 // It gt; add gt r2 r2 228
};
const byte kInstruction_add_vs_r5_r5_175[] = {
  0x68, 0xbf, 0xaf, 0x35 // It vs; add vs r5 r5 175
};
const byte kInstruction_add_ne_r3_r3_167[] = {
  0x18, 0xbf, 0xa7, 0x33 // It ne; add ne r3 r3 167
};
const byte kInstruction_add_ge_r6_r6_21[] = {
  0xa8, 0xbf, 0x15, 0x36 // It ge; add ge r6 r6 21
};
const byte kInstruction_add_eq_r7_r7_114[] = {
  0x08, 0xbf, 0x72, 0x37 // It eq; add eq r7 r7 114
};
const byte kInstruction_add_cs_r0_r0_26[] = {
  0x28, 0xbf, 0x1a, 0x30 // It cs; add cs r0 r0 26
};
const byte kInstruction_add_pl_r4_r4_29[] = {
  0x58, 0xbf, 0x1d, 0x34 // It pl; add pl r4 r4 29
};
const byte kInstruction_add_le_r1_r1_71[] = {
  0xd8, 0xbf, 0x47, 0x31 // It le; add le r1 r1 71
};
const byte kInstruction_add_cc_r0_r0_62[] = {
  0x38, 0xbf, 0x3e, 0x30 // It cc; add cc r0 r0 62
};
const byte kInstruction_add_eq_r4_r4_138[] = {
  0x08, 0xbf, 0x8a, 0x34 // It eq; add eq r4 r4 138
};
const byte kInstruction_add_cc_r4_r4_232[] = {
  0x38, 0xbf, 0xe8, 0x34 // It cc; add cc r4 r4 232
};
const byte kInstruction_add_vs_r3_r3_111[] = {
  0x68, 0xbf, 0x6f, 0x33 // It vs; add vs r3 r3 111
};
const byte kInstruction_add_ls_r5_r5_80[] = {
  0x98, 0xbf, 0x50, 0x35 // It ls; add ls r5 r5 80
};
const byte kInstruction_add_hi_r4_r4_102[] = {
  0x88, 0xbf, 0x66, 0x34 // It hi; add hi r4 r4 102
};
const byte kInstruction_add_vc_r0_r0_160[] = {
  0x78, 0xbf, 0xa0, 0x30 // It vc; add vc r0 r0 160
};
const byte kInstruction_add_eq_r4_r4_81[] = {
  0x08, 0xbf, 0x51, 0x34 // It eq; add eq r4 r4 81
};
const byte kInstruction_add_le_r7_r7_147[] = {
  0xd8, 0xbf, 0x93, 0x37 // It le; add le r7 r7 147
};
const byte kInstruction_add_ls_r5_r5_122[] = {
  0x98, 0xbf, 0x7a, 0x35 // It ls; add ls r5 r5 122
};
const byte kInstruction_add_lt_r3_r3_19[] = {
  0xb8, 0xbf, 0x13, 0x33 // It lt; add lt r3 r3 19
};
const byte kInstruction_add_mi_r1_r1_149[] = {
  0x48, 0xbf, 0x95, 0x31 // It mi; add mi r1 r1 149
};
const byte kInstruction_add_cc_r1_r1_199[] = {
  0x38, 0xbf, 0xc7, 0x31 // It cc; add cc r1 r1 199
};
const byte kInstruction_add_pl_r3_r3_75[] = {
  0x58, 0xbf, 0x4b, 0x33 // It pl; add pl r3 r3 75
};
const byte kInstruction_add_eq_r6_r6_187[] = {
  0x08, 0xbf, 0xbb, 0x36 // It eq; add eq r6 r6 187
};
const byte kInstruction_add_gt_r3_r3_205[] = {
  0xc8, 0xbf, 0xcd, 0x33 // It gt; add gt r3 r3 205
};
const byte kInstruction_add_lt_r1_r1_24[] = {
  0xb8, 0xbf, 0x18, 0x31 // It lt; add lt r1 r1 24
};
const byte kInstruction_add_mi_r6_r6_55[] = {
  0x48, 0xbf, 0x37, 0x36 // It mi; add mi r6 r6 55
};
const byte kInstruction_add_cc_r3_r3_183[] = {
  0x38, 0xbf, 0xb7, 0x33 // It cc; add cc r3 r3 183
};
const byte kInstruction_add_cc_r3_r3_116[] = {
  0x38, 0xbf, 0x74, 0x33 // It cc; add cc r3 r3 116
};
const byte kInstruction_add_ne_r3_r3_139[] = {
  0x18, 0xbf, 0x8b, 0x33 // It ne; add ne r3 r3 139
};
const byte kInstruction_add_mi_r2_r2_57[] = {
  0x48, 0xbf, 0x39, 0x32 // It mi; add mi r2 r2 57
};
const byte kInstruction_add_pl_r2_r2_67[] = {
  0x58, 0xbf, 0x43, 0x32 // It pl; add pl r2 r2 67
};
const byte kInstruction_add_lt_r1_r1_117[] = {
  0xb8, 0xbf, 0x75, 0x31 // It lt; add lt r1 r1 117
};
const byte kInstruction_add_ge_r6_r6_24[] = {
  0xa8, 0xbf, 0x18, 0x36 // It ge; add ge r6 r6 24
};
const byte kInstruction_add_cs_r2_r2_96[] = {
  0x28, 0xbf, 0x60, 0x32 // It cs; add cs r2 r2 96
};
const byte kInstruction_add_vc_r5_r5_156[] = {
  0x78, 0xbf, 0x9c, 0x35 // It vc; add vc r5 r5 156
};
const byte kInstruction_add_vs_r0_r0_125[] = {
  0x68, 0xbf, 0x7d, 0x30 // It vs; add vs r0 r0 125
};
const byte kInstruction_add_ls_r7_r7_183[] = {
  0x98, 0xbf, 0xb7, 0x37 // It ls; add ls r7 r7 183
};
const byte kInstruction_add_vs_r1_r1_30[] = {
  0x68, 0xbf, 0x1e, 0x31 // It vs; add vs r1 r1 30
};
const byte kInstruction_add_cs_r1_r1_62[] = {
  0x28, 0xbf, 0x3e, 0x31 // It cs; add cs r1 r1 62
};
const byte kInstruction_add_vc_r1_r1_8[] = {
  0x78, 0xbf, 0x08, 0x31 // It vc; add vc r1 r1 8
};
const byte kInstruction_add_vc_r4_r4_184[] = {
  0x78, 0xbf, 0xb8, 0x34 // It vc; add vc r4 r4 184
};
const byte kInstruction_add_vc_r1_r1_235[] = {
  0x78, 0xbf, 0xeb, 0x31 // It vc; add vc r1 r1 235
};
const byte kInstruction_add_gt_r2_r2_78[] = {
  0xc8, 0xbf, 0x4e, 0x32 // It gt; add gt r2 r2 78
};
const byte kInstruction_add_hi_r2_r2_160[] = {
  0x88, 0xbf, 0xa0, 0x32 // It hi; add hi r2 r2 160
};
const byte kInstruction_add_le_r3_r3_243[] = {
  0xd8, 0xbf, 0xf3, 0x33 // It le; add le r3 r3 243
};
const byte kInstruction_add_gt_r5_r5_46[] = {
  0xc8, 0xbf, 0x2e, 0x35 // It gt; add gt r5 r5 46
};
const byte kInstruction_add_ls_r5_r5_148[] = {
  0x98, 0xbf, 0x94, 0x35 // It ls; add ls r5 r5 148
};
const byte kInstruction_add_vs_r2_r2_232[] = {
  0x68, 0xbf, 0xe8, 0x32 // It vs; add vs r2 r2 232
};
const byte kInstruction_add_hi_r0_r0_13[] = {
  0x88, 0xbf, 0x0d, 0x30 // It hi; add hi r0 r0 13
};
const byte kInstruction_add_hi_r2_r2_45[] = {
  0x88, 0xbf, 0x2d, 0x32 // It hi; add hi r2 r2 45
};
const byte kInstruction_add_mi_r1_r1_197[] = {
  0x48, 0xbf, 0xc5, 0x31 // It mi; add mi r1 r1 197
};
const byte kInstruction_add_ls_r7_r7_91[] = {
  0x98, 0xbf, 0x5b, 0x37 // It ls; add ls r7 r7 91
};
const byte kInstruction_add_cs_r2_r2_192[] = {
  0x28, 0xbf, 0xc0, 0x32 // It cs; add cs r2 r2 192
};
const byte kInstruction_add_cs_r4_r4_202[] = {
  0x28, 0xbf, 0xca, 0x34 // It cs; add cs r4 r4 202
};
const byte kInstruction_add_ne_r6_r6_115[] = {
  0x18, 0xbf, 0x73, 0x36 // It ne; add ne r6 r6 115
};
const byte kInstruction_add_mi_r7_r7_173[] = {
  0x48, 0xbf, 0xad, 0x37 // It mi; add mi r7 r7 173
};
const byte kInstruction_add_vs_r1_r1_236[] = {
  0x68, 0xbf, 0xec, 0x31 // It vs; add vs r1 r1 236
};
const byte kInstruction_add_pl_r3_r3_49[] = {
  0x58, 0xbf, 0x31, 0x33 // It pl; add pl r3 r3 49
};
const byte kInstruction_add_cs_r1_r1_193[] = {
  0x28, 0xbf, 0xc1, 0x31 // It cs; add cs r1 r1 193
};
const byte kInstruction_add_gt_r4_r4_200[] = {
  0xc8, 0xbf, 0xc8, 0x34 // It gt; add gt r4 r4 200
};
const byte kInstruction_add_lt_r4_r4_141[] = {
  0xb8, 0xbf, 0x8d, 0x34 // It lt; add lt r4 r4 141
};
const byte kInstruction_add_gt_r6_r6_142[] = {
  0xc8, 0xbf, 0x8e, 0x36 // It gt; add gt r6 r6 142
};
const byte kInstruction_add_le_r2_r2_39[] = {
  0xd8, 0xbf, 0x27, 0x32 // It le; add le r2 r2 39
};
const byte kInstruction_add_cc_r3_r3_98[] = {
  0x38, 0xbf, 0x62, 0x33 // It cc; add cc r3 r3 98
};
const byte kInstruction_add_le_r4_r4_77[] = {
  0xd8, 0xbf, 0x4d, 0x34 // It le; add le r4 r4 77
};
const byte kInstruction_add_ne_r2_r2_140[] = {
  0x18, 0xbf, 0x8c, 0x32 // It ne; add ne r2 r2 140
};
const byte kInstruction_add_cc_r3_r3_117[] = {
  0x38, 0xbf, 0x75, 0x33 // It cc; add cc r3 r3 117
};
const byte kInstruction_add_gt_r3_r3_242[] = {
  0xc8, 0xbf, 0xf2, 0x33 // It gt; add gt r3 r3 242
};
const byte kInstruction_add_ls_r5_r5_90[] = {
  0x98, 0xbf, 0x5a, 0x35 // It ls; add ls r5 r5 90
};
const byte kInstruction_add_gt_r7_r7_16[] = {
  0xc8, 0xbf, 0x10, 0x37 // It gt; add gt r7 r7 16
};
const byte kInstruction_add_vs_r5_r5_131[] = {
  0x68, 0xbf, 0x83, 0x35 // It vs; add vs r5 r5 131
};
const byte kInstruction_add_vc_r6_r6_145[] = {
  0x78, 0xbf, 0x91, 0x36 // It vc; add vc r6 r6 145
};
const byte kInstruction_add_lt_r0_r0_94[] = {
  0xb8, 0xbf, 0x5e, 0x30 // It lt; add lt r0 r0 94
};
const byte kInstruction_add_vs_r3_r3_246[] = {
  0x68, 0xbf, 0xf6, 0x33 // It vs; add vs r3 r3 246
};
const byte kInstruction_add_le_r0_r0_124[] = {
  0xd8, 0xbf, 0x7c, 0x30 // It le; add le r0 r0 124
};
const byte kInstruction_add_eq_r7_r7_60[] = {
  0x08, 0xbf, 0x3c, 0x37 // It eq; add eq r7 r7 60
};
const byte kInstruction_add_mi_r3_r3_244[] = {
  0x48, 0xbf, 0xf4, 0x33 // It mi; add mi r3 r3 244
};
const byte kInstruction_add_ls_r5_r5_110[] = {
  0x98, 0xbf, 0x6e, 0x35 // It ls; add ls r5 r5 110
};
const byte kInstruction_add_lt_r2_r2_168[] = {
  0xb8, 0xbf, 0xa8, 0x32 // It lt; add lt r2 r2 168
};
const byte kInstruction_add_cs_r3_r3_217[] = {
  0x28, 0xbf, 0xd9, 0x33 // It cs; add cs r3 r3 217
};
const byte kInstruction_add_lt_r2_r2_155[] = {
  0xb8, 0xbf, 0x9b, 0x32 // It lt; add lt r2 r2 155
};
const byte kInstruction_add_gt_r2_r2_43[] = {
  0xc8, 0xbf, 0x2b, 0x32 // It gt; add gt r2 r2 43
};
const byte kInstruction_add_pl_r3_r3_247[] = {
  0x58, 0xbf, 0xf7, 0x33 // It pl; add pl r3 r3 247
};
const byte kInstruction_add_mi_r0_r0_193[] = {
  0x48, 0xbf, 0xc1, 0x30 // It mi; add mi r0 r0 193
};
const byte kInstruction_add_pl_r7_r7_22[] = {
  0x58, 0xbf, 0x16, 0x37 // It pl; add pl r7 r7 22
};
const byte kInstruction_add_vs_r7_r7_222[] = {
  0x68, 0xbf, 0xde, 0x37 // It vs; add vs r7 r7 222
};
const byte kInstruction_add_le_r0_r0_205[] = {
  0xd8, 0xbf, 0xcd, 0x30 // It le; add le r0 r0 205
};
const byte kInstruction_add_lt_r4_r4_67[] = {
  0xb8, 0xbf, 0x43, 0x34 // It lt; add lt r4 r4 67
};
const byte kInstruction_add_cc_r1_r1_60[] = {
  0x38, 0xbf, 0x3c, 0x31 // It cc; add cc r1 r1 60
};
const byte kInstruction_add_cc_r4_r4_4[] = {
  0x38, 0xbf, 0x24, 0x1d // It cc; add cc r4 r4 4
};
const byte kInstruction_add_vc_r1_r1_22[] = {
  0x78, 0xbf, 0x16, 0x31 // It vc; add vc r1 r1 22
};
const byte kInstruction_add_vc_r4_r4_94[] = {
  0x78, 0xbf, 0x5e, 0x34 // It vc; add vc r4 r4 94
};
const byte kInstruction_add_ls_r5_r5_180[] = {
  0x98, 0xbf, 0xb4, 0x35 // It ls; add ls r5 r5 180
};
const byte kInstruction_add_cc_r7_r7_87[] = {
  0x38, 0xbf, 0x57, 0x37 // It cc; add cc r7 r7 87
};
const byte kInstruction_add_ls_r3_r3_175[] = {
  0x98, 0xbf, 0xaf, 0x33 // It ls; add ls r3 r3 175
};
const byte kInstruction_add_ge_r7_r7_222[] = {
  0xa8, 0xbf, 0xde, 0x37 // It ge; add ge r7 r7 222
};
const byte kInstruction_add_ne_r2_r2_166[] = {
  0x18, 0xbf, 0xa6, 0x32 // It ne; add ne r2 r2 166
};
const byte kInstruction_add_ls_r1_r1_191[] = {
  0x98, 0xbf, 0xbf, 0x31 // It ls; add ls r1 r1 191
};
const byte kInstruction_add_vs_r0_r0_20[] = {
  0x68, 0xbf, 0x14, 0x30 // It vs; add vs r0 r0 20
};
const byte kInstruction_add_hi_r1_r1_253[] = {
  0x88, 0xbf, 0xfd, 0x31 // It hi; add hi r1 r1 253
};
const byte kInstruction_add_eq_r1_r1_2[] = {
  0x08, 0xbf, 0x89, 0x1c // It eq; add eq r1 r1 2
};
const byte kInstruction_add_ls_r6_r6_74[] = {
  0x98, 0xbf, 0x4a, 0x36 // It ls; add ls r6 r6 74
};
const byte kInstruction_add_mi_r6_r6_242[] = {
  0x48, 0xbf, 0xf2, 0x36 // It mi; add mi r6 r6 242
};
const byte kInstruction_add_le_r6_r6_242[] = {
  0xd8, 0xbf, 0xf2, 0x36 // It le; add le r6 r6 242
};
const byte kInstruction_add_cs_r7_r7_110[] = {
  0x28, 0xbf, 0x6e, 0x37 // It cs; add cs r7 r7 110
};
const byte kInstruction_add_ge_r5_r5_23[] = {
  0xa8, 0xbf, 0x17, 0x35 // It ge; add ge r5 r5 23
};
const byte kInstruction_add_lt_r5_r5_87[] = {
  0xb8, 0xbf, 0x57, 0x35 // It lt; add lt r5 r5 87
};
const byte kInstruction_add_lt_r0_r0_91[] = {
  0xb8, 0xbf, 0x5b, 0x30 // It lt; add lt r0 r0 91
};
const byte kInstruction_add_cc_r2_r2_68[] = {
  0x38, 0xbf, 0x44, 0x32 // It cc; add cc r2 r2 68
};
const byte kInstruction_add_vs_r2_r2_171[] = {
  0x68, 0xbf, 0xab, 0x32 // It vs; add vs r2 r2 171
};
const byte kInstruction_add_pl_r6_r6_118[] = {
  0x58, 0xbf, 0x76, 0x36 // It pl; add pl r6 r6 118
};
const byte kInstruction_add_gt_r6_r6_232[] = {
  0xc8, 0xbf, 0xe8, 0x36 // It gt; add gt r6 r6 232
};
const byte kInstruction_add_ls_r0_r0_255[] = {
  0x98, 0xbf, 0xff, 0x30 // It ls; add ls r0 r0 255
};
const byte kInstruction_add_mi_r4_r4_142[] = {
  0x48, 0xbf, 0x8e, 0x34 // It mi; add mi r4 r4 142
};
const byte kInstruction_add_lt_r5_r5_198[] = {
  0xb8, 0xbf, 0xc6, 0x35 // It lt; add lt r5 r5 198
};
const byte kInstruction_add_lt_r0_r0_114[] = {
  0xb8, 0xbf, 0x72, 0x30 // It lt; add lt r0 r0 114
};
const byte kInstruction_add_lt_r5_r5_129[] = {
  0xb8, 0xbf, 0x81, 0x35 // It lt; add lt r5 r5 129
};
const byte kInstruction_add_pl_r5_r5_145[] = {
  0x58, 0xbf, 0x91, 0x35 // It pl; add pl r5 r5 145
};
const byte kInstruction_add_hi_r7_r7_43[] = {
  0x88, 0xbf, 0x2b, 0x37 // It hi; add hi r7 r7 43
};
const byte kInstruction_add_vs_r1_r1_67[] = {
  0x68, 0xbf, 0x43, 0x31 // It vs; add vs r1 r1 67
};
const byte kInstruction_add_vc_r7_r7_78[] = {
  0x78, 0xbf, 0x4e, 0x37 // It vc; add vc r7 r7 78
};
const byte kInstruction_add_cc_r1_r1_56[] = {
  0x38, 0xbf, 0x38, 0x31 // It cc; add cc r1 r1 56
};
const byte kInstruction_add_vs_r0_r0_12[] = {
  0x68, 0xbf, 0x0c, 0x30 // It vs; add vs r0 r0 12
};
const byte kInstruction_add_pl_r5_r5_35[] = {
  0x58, 0xbf, 0x23, 0x35 // It pl; add pl r5 r5 35
};
const byte kInstruction_add_vs_r5_r5_105[] = {
  0x68, 0xbf, 0x69, 0x35 // It vs; add vs r5 r5 105
};
const byte kInstruction_add_vc_r1_r1_88[] = {
  0x78, 0xbf, 0x58, 0x31 // It vc; add vc r1 r1 88
};
const byte kInstruction_add_lt_r2_r2_69[] = {
  0xb8, 0xbf, 0x45, 0x32 // It lt; add lt r2 r2 69
};
const byte kInstruction_add_cs_r3_r3_255[] = {
  0x28, 0xbf, 0xff, 0x33 // It cs; add cs r3 r3 255
};
const byte kInstruction_add_ls_r6_r6_5[] = {
  0x98, 0xbf, 0x76, 0x1d // It ls; add ls r6 r6 5
};
const byte kInstruction_add_vc_r6_r6_171[] = {
  0x78, 0xbf, 0xab, 0x36 // It vc; add vc r6 r6 171
};
const byte kInstruction_add_pl_r5_r5_166[] = {
  0x58, 0xbf, 0xa6, 0x35 // It pl; add pl r5 r5 166
};
const byte kInstruction_add_ls_r1_r1_121[] = {
  0x98, 0xbf, 0x79, 0x31 // It ls; add ls r1 r1 121
};
const byte kInstruction_add_ne_r4_r4_120[] = {
  0x18, 0xbf, 0x78, 0x34 // It ne; add ne r4 r4 120
};
const byte kInstruction_add_ls_r4_r4_130[] = {
  0x98, 0xbf, 0x82, 0x34 // It ls; add ls r4 r4 130
};
const byte kInstruction_add_ge_r6_r6_104[] = {
  0xa8, 0xbf, 0x68, 0x36 // It ge; add ge r6 r6 104
};
const byte kInstruction_add_ge_r3_r3_227[] = {
  0xa8, 0xbf, 0xe3, 0x33 // It ge; add ge r3 r3 227
};
const byte kInstruction_add_vs_r3_r3_202[] = {
  0x68, 0xbf, 0xca, 0x33 // It vs; add vs r3 r3 202
};
const byte kInstruction_add_ne_r4_r4_198[] = {
  0x18, 0xbf, 0xc6, 0x34 // It ne; add ne r4 r4 198
};
const byte kInstruction_add_cc_r5_r5_78[] = {
  0x38, 0xbf, 0x4e, 0x35 // It cc; add cc r5 r5 78
};
const byte kInstruction_add_cs_r3_r3_7[] = {
  0x28, 0xbf, 0xdb, 0x1d // It cs; add cs r3 r3 7
};
const byte kInstruction_add_mi_r0_r0_35[] = {
  0x48, 0xbf, 0x23, 0x30 // It mi; add mi r0 r0 35
};
const byte kInstruction_add_hi_r3_r3_53[] = {
  0x88, 0xbf, 0x35, 0x33 // It hi; add hi r3 r3 53
};
const byte kInstruction_add_cs_r5_r5_150[] = {
  0x28, 0xbf, 0x96, 0x35 // It cs; add cs r5 r5 150
};
const byte kInstruction_add_ge_r2_r2_143[] = {
  0xa8, 0xbf, 0x8f, 0x32 // It ge; add ge r2 r2 143
};
const byte kInstruction_add_hi_r0_r0_229[] = {
  0x88, 0xbf, 0xe5, 0x30 // It hi; add hi r0 r0 229
};
const byte kInstruction_add_vc_r6_r6_141[] = {
  0x78, 0xbf, 0x8d, 0x36 // It vc; add vc r6 r6 141
};
const byte kInstruction_add_pl_r7_r7_76[] = {
  0x58, 0xbf, 0x4c, 0x37 // It pl; add pl r7 r7 76
};
const byte kInstruction_add_ne_r5_r5_135[] = {
  0x18, 0xbf, 0x87, 0x35 // It ne; add ne r5 r5 135
};
const byte kInstruction_add_hi_r5_r5_51[] = {
  0x88, 0xbf, 0x33, 0x35 // It hi; add hi r5 r5 51
};
const byte kInstruction_add_eq_r6_r6_95[] = {
  0x08, 0xbf, 0x5f, 0x36 // It eq; add eq r6 r6 95
};
const byte kInstruction_add_cs_r6_r6_227[] = {
  0x28, 0xbf, 0xe3, 0x36 // It cs; add cs r6 r6 227
};
const byte kInstruction_add_ge_r6_r6_46[] = {
  0xa8, 0xbf, 0x2e, 0x36 // It ge; add ge r6 r6 46
};
const byte kInstruction_add_ne_r1_r1_150[] = {
  0x18, 0xbf, 0x96, 0x31 // It ne; add ne r1 r1 150
};
const byte kInstruction_add_eq_r1_r1_22[] = {
  0x08, 0xbf, 0x16, 0x31 // It eq; add eq r1 r1 22
};
const byte kInstruction_add_vs_r2_r2_220[] = {
  0x68, 0xbf, 0xdc, 0x32 // It vs; add vs r2 r2 220
};
const byte kInstruction_add_ls_r6_r6_240[] = {
  0x98, 0xbf, 0xf0, 0x36 // It ls; add ls r6 r6 240
};
const byte kInstruction_add_le_r4_r4_43[] = {
  0xd8, 0xbf, 0x2b, 0x34 // It le; add le r4 r4 43
};
const byte kInstruction_add_ls_r7_r7_243[] = {
  0x98, 0xbf, 0xf3, 0x37 // It ls; add ls r7 r7 243
};
const byte kInstruction_add_hi_r3_r3_191[] = {
  0x88, 0xbf, 0xbf, 0x33 // It hi; add hi r3 r3 191
};
const byte kInstruction_add_hi_r0_r0_235[] = {
  0x88, 0xbf, 0xeb, 0x30 // It hi; add hi r0 r0 235
};
const byte kInstruction_add_gt_r1_r1_59[] = {
  0xc8, 0xbf, 0x3b, 0x31 // It gt; add gt r1 r1 59
};
const byte kInstruction_add_vc_r3_r3_165[] = {
  0x78, 0xbf, 0xa5, 0x33 // It vc; add vc r3 r3 165
};
const byte kInstruction_add_vc_r1_r1_46[] = {
  0x78, 0xbf, 0x2e, 0x31 // It vc; add vc r1 r1 46
};
const byte kInstruction_add_eq_r6_r6_232[] = {
  0x08, 0xbf, 0xe8, 0x36 // It eq; add eq r6 r6 232
};
const byte kInstruction_add_ls_r3_r3_132[] = {
  0x98, 0xbf, 0x84, 0x33 // It ls; add ls r3 r3 132
};
const byte kInstruction_add_ls_r7_r7_207[] = {
  0x98, 0xbf, 0xcf, 0x37 // It ls; add ls r7 r7 207
};
const byte kInstruction_add_vs_r7_r7_129[] = {
  0x68, 0xbf, 0x81, 0x37 // It vs; add vs r7 r7 129
};
const byte kInstruction_add_ne_r3_r3_41[] = {
  0x18, 0xbf, 0x29, 0x33 // It ne; add ne r3 r3 41
};
const byte kInstruction_add_ls_r3_r3_153[] = {
  0x98, 0xbf, 0x99, 0x33 // It ls; add ls r3 r3 153
};
const byte kInstruction_add_vc_r4_r4_34[] = {
  0x78, 0xbf, 0x22, 0x34 // It vc; add vc r4 r4 34
};
const byte kInstruction_add_cc_r7_r7_140[] = {
  0x38, 0xbf, 0x8c, 0x37 // It cc; add cc r7 r7 140
};
const byte kInstruction_add_ne_r0_r0_108[] = {
  0x18, 0xbf, 0x6c, 0x30 // It ne; add ne r0 r0 108
};
const byte kInstruction_add_ls_r7_r7_44[] = {
  0x98, 0xbf, 0x2c, 0x37 // It ls; add ls r7 r7 44
};
const byte kInstruction_add_cs_r6_r6_128[] = {
  0x28, 0xbf, 0x80, 0x36 // It cs; add cs r6 r6 128
};
const byte kInstruction_add_ne_r3_r3_195[] = {
  0x18, 0xbf, 0xc3, 0x33 // It ne; add ne r3 r3 195
};
const byte kInstruction_add_hi_r1_r1_239[] = {
  0x88, 0xbf, 0xef, 0x31 // It hi; add hi r1 r1 239
};
const byte kInstruction_add_cc_r3_r3_243[] = {
  0x38, 0xbf, 0xf3, 0x33 // It cc; add cc r3 r3 243
};
const byte kInstruction_add_pl_r1_r1_30[] = {
  0x58, 0xbf, 0x1e, 0x31 // It pl; add pl r1 r1 30
};
const byte kInstruction_add_lt_r4_r4_48[] = {
  0xb8, 0xbf, 0x30, 0x34 // It lt; add lt r4 r4 48
};
const byte kInstruction_add_gt_r5_r5_92[] = {
  0xc8, 0xbf, 0x5c, 0x35 // It gt; add gt r5 r5 92
};
const byte kInstruction_add_pl_r6_r6_167[] = {
  0x58, 0xbf, 0xa7, 0x36 // It pl; add pl r6 r6 167
};
const byte kInstruction_add_lt_r0_r0_8[] = {
  0xb8, 0xbf, 0x08, 0x30 // It lt; add lt r0 r0 8
};
const byte kInstruction_add_vc_r4_r4_49[] = {
  0x78, 0xbf, 0x31, 0x34 // It vc; add vc r4 r4 49
};
const byte kInstruction_add_mi_r3_r3_109[] = {
  0x48, 0xbf, 0x6d, 0x33 // It mi; add mi r3 r3 109
};
const byte kInstruction_add_lt_r7_r7_58[] = {
  0xb8, 0xbf, 0x3a, 0x37 // It lt; add lt r7 r7 58
};
const byte kInstruction_add_lt_r6_r6_54[] = {
  0xb8, 0xbf, 0x36, 0x36 // It lt; add lt r6 r6 54
};
const byte kInstruction_add_eq_r7_r7_36[] = {
  0x08, 0xbf, 0x24, 0x37 // It eq; add eq r7 r7 36
};
const byte kInstruction_add_ls_r0_r0_230[] = {
  0x98, 0xbf, 0xe6, 0x30 // It ls; add ls r0 r0 230
};
const byte kInstruction_add_eq_r7_r7_133[] = {
  0x08, 0xbf, 0x85, 0x37 // It eq; add eq r7 r7 133
};
const byte kInstruction_add_hi_r0_r0_153[] = {
  0x88, 0xbf, 0x99, 0x30 // It hi; add hi r0 r0 153
};
const byte kInstruction_add_cs_r6_r6_154[] = {
  0x28, 0xbf, 0x9a, 0x36 // It cs; add cs r6 r6 154
};
const byte kInstruction_add_eq_r6_r6_178[] = {
  0x08, 0xbf, 0xb2, 0x36 // It eq; add eq r6 r6 178
};
const byte kInstruction_add_hi_r0_r0_206[] = {
  0x88, 0xbf, 0xce, 0x30 // It hi; add hi r0 r0 206
};
const byte kInstruction_add_ls_r4_r4_164[] = {
  0x98, 0xbf, 0xa4, 0x34 // It ls; add ls r4 r4 164
};
const byte kInstruction_add_lt_r3_r3_60[] = {
  0xb8, 0xbf, 0x3c, 0x33 // It lt; add lt r3 r3 60
};
const byte kInstruction_add_cs_r5_r5_44[] = {
  0x28, 0xbf, 0x2c, 0x35 // It cs; add cs r5 r5 44
};
const byte kInstruction_add_le_r5_r5_158[] = {
  0xd8, 0xbf, 0x9e, 0x35 // It le; add le r5 r5 158
};
const byte kInstruction_add_lt_r3_r3_40[] = {
  0xb8, 0xbf, 0x28, 0x33 // It lt; add lt r3 r3 40
};
const byte kInstruction_add_le_r0_r0_53[] = {
  0xd8, 0xbf, 0x35, 0x30 // It le; add le r0 r0 53
};
const byte kInstruction_add_le_r3_r3_189[] = {
  0xd8, 0xbf, 0xbd, 0x33 // It le; add le r3 r3 189
};
const byte kInstruction_add_vs_r3_r3_8[] = {
  0x68, 0xbf, 0x08, 0x33 // It vs; add vs r3 r3 8
};
const byte kInstruction_add_vs_r1_r1_141[] = {
  0x68, 0xbf, 0x8d, 0x31 // It vs; add vs r1 r1 141
};
const byte kInstruction_add_vs_r3_r3_141[] = {
  0x68, 0xbf, 0x8d, 0x33 // It vs; add vs r3 r3 141
};
const byte kInstruction_add_vs_r2_r2_141[] = {
  0x68, 0xbf, 0x8d, 0x32 // It vs; add vs r2 r2 141
};
const byte kInstruction_add_ls_r6_r6_254[] = {
  0x98, 0xbf, 0xfe, 0x36 // It ls; add ls r6 r6 254
};
const byte kInstruction_add_ne_r3_r3_142[] = {
  0x18, 0xbf, 0x8e, 0x33 // It ne; add ne r3 r3 142
};
const byte kInstruction_add_hi_r0_r0_167[] = {
  0x88, 0xbf, 0xa7, 0x30 // It hi; add hi r0 r0 167
};
const byte kInstruction_add_cs_r6_r6_84[] = {
  0x28, 0xbf, 0x54, 0x36 // It cs; add cs r6 r6 84
};
const byte kInstruction_add_cs_r2_r2_234[] = {
  0x28, 0xbf, 0xea, 0x32 // It cs; add cs r2 r2 234
};
const byte kInstruction_add_le_r6_r6_59[] = {
  0xd8, 0xbf, 0x3b, 0x36 // It le; add le r6 r6 59
};
const byte kInstruction_add_hi_r2_r2_135[] = {
  0x88, 0xbf, 0x87, 0x32 // It hi; add hi r2 r2 135
};
const byte kInstruction_add_gt_r7_r7_47[] = {
  0xc8, 0xbf, 0x2f, 0x37 // It gt; add gt r7 r7 47
};
const byte kInstruction_add_eq_r6_r6_122[] = {
  0x08, 0xbf, 0x7a, 0x36 // It eq; add eq r6 r6 122
};
const byte kInstruction_add_vc_r7_r7_118[] = {
  0x78, 0xbf, 0x76, 0x37 // It vc; add vc r7 r7 118
};
const byte kInstruction_add_cs_r2_r2_180[] = {
  0x28, 0xbf, 0xb4, 0x32 // It cs; add cs r2 r2 180
};
const byte kInstruction_add_cs_r2_r2_181[] = {
  0x28, 0xbf, 0xb5, 0x32 // It cs; add cs r2 r2 181
};
const byte kInstruction_add_cc_r1_r1_190[] = {
  0x38, 0xbf, 0xbe, 0x31 // It cc; add cc r1 r1 190
};
const byte kInstruction_add_ge_r5_r5_205[] = {
  0xa8, 0xbf, 0xcd, 0x35 // It ge; add ge r5 r5 205
};
const byte kInstruction_add_gt_r7_r7_26[] = {
  0xc8, 0xbf, 0x1a, 0x37 // It gt; add gt r7 r7 26
};
const byte kInstruction_add_gt_r4_r4_70[] = {
  0xc8, 0xbf, 0x46, 0x34 // It gt; add gt r4 r4 70
};
const byte kInstruction_add_eq_r5_r5_102[] = {
  0x08, 0xbf, 0x66, 0x35 // It eq; add eq r5 r5 102
};
const byte kInstruction_add_ls_r0_r0_57[] = {
  0x98, 0xbf, 0x39, 0x30 // It ls; add ls r0 r0 57
};
const byte kInstruction_add_pl_r1_r1_229[] = {
  0x58, 0xbf, 0xe5, 0x31 // It pl; add pl r1 r1 229
};
const byte kInstruction_add_ge_r2_r2_185[] = {
  0xa8, 0xbf, 0xb9, 0x32 // It ge; add ge r2 r2 185
};
const byte kInstruction_add_ge_r7_r7_80[] = {
  0xa8, 0xbf, 0x50, 0x37 // It ge; add ge r7 r7 80
};
const byte kInstruction_add_cs_r6_r6_123[] = {
  0x28, 0xbf, 0x7b, 0x36 // It cs; add cs r6 r6 123
};
const byte kInstruction_add_pl_r4_r4_137[] = {
  0x58, 0xbf, 0x89, 0x34 // It pl; add pl r4 r4 137
};
const byte kInstruction_add_cc_r1_r1_137[] = {
  0x38, 0xbf, 0x89, 0x31 // It cc; add cc r1 r1 137
};
const byte kInstruction_add_cs_r1_r1_67[] = {
  0x28, 0xbf, 0x43, 0x31 // It cs; add cs r1 r1 67
};
const byte kInstruction_add_vc_r6_r6_207[] = {
  0x78, 0xbf, 0xcf, 0x36 // It vc; add vc r6 r6 207
};
const byte kInstruction_add_cc_r0_r0_50[] = {
  0x38, 0xbf, 0x32, 0x30 // It cc; add cc r0 r0 50
};
const byte kInstruction_add_mi_r3_r3_72[] = {
  0x48, 0xbf, 0x48, 0x33 // It mi; add mi r3 r3 72
};
const byte kInstruction_add_ls_r3_r3_31[] = {
  0x98, 0xbf, 0x1f, 0x33 // It ls; add ls r3 r3 31
};
const byte kInstruction_add_vc_r2_r2_253[] = {
  0x78, 0xbf, 0xfd, 0x32 // It vc; add vc r2 r2 253
};
const byte kInstruction_add_vc_r3_r3_155[] = {
  0x78, 0xbf, 0x9b, 0x33 // It vc; add vc r3 r3 155
};
const byte kInstruction_add_lt_r0_r0_115[] = {
  0xb8, 0xbf, 0x73, 0x30 // It lt; add lt r0 r0 115
};
const byte kInstruction_add_mi_r5_r5_236[] = {
  0x48, 0xbf, 0xec, 0x35 // It mi; add mi r5 r5 236
};
const byte kInstruction_add_eq_r5_r5_151[] = {
  0x08, 0xbf, 0x97, 0x35 // It eq; add eq r5 r5 151
};
const byte kInstruction_add_eq_r5_r5_11[] = {
  0x08, 0xbf, 0x0b, 0x35 // It eq; add eq r5 r5 11
};
const byte kInstruction_add_pl_r5_r5_119[] = {
  0x58, 0xbf, 0x77, 0x35 // It pl; add pl r5 r5 119
};
const byte kInstruction_add_cc_r3_r3_192[] = {
  0x38, 0xbf, 0xc0, 0x33 // It cc; add cc r3 r3 192
};
const byte kInstruction_add_vc_r7_r7_0[] = {
  0x78, 0xbf, 0x3f, 0x1c // It vc; add vc r7 r7 0
};
const byte kInstruction_add_hi_r1_r1_163[] = {
  0x88, 0xbf, 0xa3, 0x31 // It hi; add hi r1 r1 163
};
const byte kInstruction_add_hi_r2_r2_136[] = {
  0x88, 0xbf, 0x88, 0x32 // It hi; add hi r2 r2 136
};
const byte kInstruction_add_cc_r5_r5_235[] = {
  0x38, 0xbf, 0xeb, 0x35 // It cc; add cc r5 r5 235
};
const byte kInstruction_add_pl_r5_r5_96[] = {
  0x58, 0xbf, 0x60, 0x35 // It pl; add pl r5 r5 96
};
const byte kInstruction_add_ge_r3_r3_163[] = {
  0xa8, 0xbf, 0xa3, 0x33 // It ge; add ge r3 r3 163
};
const byte kInstruction_add_cs_r5_r5_239[] = {
  0x28, 0xbf, 0xef, 0x35 // It cs; add cs r5 r5 239
};
const byte kInstruction_add_vs_r5_r5_27[] = {
  0x68, 0xbf, 0x1b, 0x35 // It vs; add vs r5 r5 27
};
const byte kInstruction_add_le_r6_r6_65[] = {
  0xd8, 0xbf, 0x41, 0x36 // It le; add le r6 r6 65
};
const byte kInstruction_add_gt_r0_r0_169[] = {
  0xc8, 0xbf, 0xa9, 0x30 // It gt; add gt r0 r0 169
};
const byte kInstruction_add_ls_r7_r7_190[] = {
  0x98, 0xbf, 0xbe, 0x37 // It ls; add ls r7 r7 190
};
const byte kInstruction_add_lt_r5_r5_205[] = {
  0xb8, 0xbf, 0xcd, 0x35 // It lt; add lt r5 r5 205
};
const byte kInstruction_add_ne_r2_r2_253[] = {
  0x18, 0xbf, 0xfd, 0x32 // It ne; add ne r2 r2 253
};
const byte kInstruction_add_cs_r1_r1_224[] = {
  0x28, 0xbf, 0xe0, 0x31 // It cs; add cs r1 r1 224
};
const byte kInstruction_add_vs_r1_r1_85[] = {
  0x68, 0xbf, 0x55, 0x31 // It vs; add vs r1 r1 85
};
const byte kInstruction_add_le_r0_r0_31[] = {
  0xd8, 0xbf, 0x1f, 0x30 // It le; add le r0 r0 31
};
const byte kInstruction_add_lt_r2_r2_7[] = {
  0xb8, 0xbf, 0xd2, 0x1d // It lt; add lt r2 r2 7
};
const byte kInstruction_add_ne_r0_r0_161[] = {
  0x18, 0xbf, 0xa1, 0x30 // It ne; add ne r0 r0 161
};
const byte kInstruction_add_le_r5_r5_46[] = {
  0xd8, 0xbf, 0x2e, 0x35 // It le; add le r5 r5 46
};
const byte kInstruction_add_pl_r0_r0_235[] = {
  0x58, 0xbf, 0xeb, 0x30 // It pl; add pl r0 r0 235
};
const byte kInstruction_add_eq_r6_r6_113[] = {
  0x08, 0xbf, 0x71, 0x36 // It eq; add eq r6 r6 113
};
const byte kInstruction_add_mi_r6_r6_45[] = {
  0x48, 0xbf, 0x2d, 0x36 // It mi; add mi r6 r6 45
};
const byte kInstruction_add_cs_r0_r0_62[] = {
  0x28, 0xbf, 0x3e, 0x30 // It cs; add cs r0 r0 62
};
const byte kInstruction_add_ge_r2_r2_225[] = {
  0xa8, 0xbf, 0xe1, 0x32 // It ge; add ge r2 r2 225
};
const byte kInstruction_add_vc_r7_r7_47[] = {
  0x78, 0xbf, 0x2f, 0x37 // It vc; add vc r7 r7 47
};
const byte kInstruction_add_vs_r5_r5_125[] = {
  0x68, 0xbf, 0x7d, 0x35 // It vs; add vs r5 r5 125
};
const byte kInstruction_add_ls_r2_r2_184[] = {
  0x98, 0xbf, 0xb8, 0x32 // It ls; add ls r2 r2 184
};
const byte kInstruction_add_ls_r3_r3_113[] = {
  0x98, 0xbf, 0x71, 0x33 // It ls; add ls r3 r3 113
};
const byte kInstruction_add_gt_r3_r3_126[] = {
  0xc8, 0xbf, 0x7e, 0x33 // It gt; add gt r3 r3 126
};
const byte kInstruction_add_gt_r5_r5_101[] = {
  0xc8, 0xbf, 0x65, 0x35 // It gt; add gt r5 r5 101
};
const byte kInstruction_add_lt_r2_r2_147[] = {
  0xb8, 0xbf, 0x93, 0x32 // It lt; add lt r2 r2 147
};
const byte kInstruction_add_le_r5_r5_60[] = {
  0xd8, 0xbf, 0x3c, 0x35 // It le; add le r5 r5 60
};
const byte kInstruction_add_ne_r3_r3_249[] = {
  0x18, 0xbf, 0xf9, 0x33 // It ne; add ne r3 r3 249
};
const byte kInstruction_add_cs_r3_r3_6[] = {
  0x28, 0xbf, 0x9b, 0x1d // It cs; add cs r3 r3 6
};
const byte kInstruction_add_vc_r3_r3_65[] = {
  0x78, 0xbf, 0x41, 0x33 // It vc; add vc r3 r3 65
};
const byte kInstruction_add_ls_r0_r0_137[] = {
  0x98, 0xbf, 0x89, 0x30 // It ls; add ls r0 r0 137
};
const byte kInstruction_add_ls_r0_r0_148[] = {
  0x98, 0xbf, 0x94, 0x30 // It ls; add ls r0 r0 148
};
const byte kInstruction_add_cs_r2_r2_128[] = {
  0x28, 0xbf, 0x80, 0x32 // It cs; add cs r2 r2 128
};
const byte kInstruction_add_eq_r4_r4_38[] = {
  0x08, 0xbf, 0x26, 0x34 // It eq; add eq r4 r4 38
};
const byte kInstruction_add_ne_r2_r2_57[] = {
  0x18, 0xbf, 0x39, 0x32 // It ne; add ne r2 r2 57
};
const byte kInstruction_add_ls_r2_r2_125[] = {
  0x98, 0xbf, 0x7d, 0x32 // It ls; add ls r2 r2 125
};
const byte kInstruction_add_lt_r7_r7_139[] = {
  0xb8, 0xbf, 0x8b, 0x37 // It lt; add lt r7 r7 139
};
const byte kInstruction_add_eq_r2_r2_213[] = {
  0x08, 0xbf, 0xd5, 0x32 // It eq; add eq r2 r2 213
};
const byte kInstruction_add_vc_r0_r0_7[] = {
  0x78, 0xbf, 0xc0, 0x1d // It vc; add vc r0 r0 7
};
const byte kInstruction_add_lt_r0_r0_243[] = {
  0xb8, 0xbf, 0xf3, 0x30 // It lt; add lt r0 r0 243
};
const byte kInstruction_add_lt_r0_r0_227[] = {
  0xb8, 0xbf, 0xe3, 0x30 // It lt; add lt r0 r0 227
};
const byte kInstruction_add_ge_r6_r6_110[] = {
  0xa8, 0xbf, 0x6e, 0x36 // It ge; add ge r6 r6 110
};
const byte kInstruction_add_lt_r3_r3_152[] = {
  0xb8, 0xbf, 0x98, 0x33 // It lt; add lt r3 r3 152
};
const byte kInstruction_add_ge_r7_r7_198[] = {
  0xa8, 0xbf, 0xc6, 0x37 // It ge; add ge r7 r7 198
};
const byte kInstruction_add_mi_r3_r3_158[] = {
  0x48, 0xbf, 0x9e, 0x33 // It mi; add mi r3 r3 158
};
const byte kInstruction_add_cc_r5_r5_171[] = {
  0x38, 0xbf, 0xab, 0x35 // It cc; add cc r5 r5 171
};
const byte kInstruction_add_lt_r5_r5_227[] = {
  0xb8, 0xbf, 0xe3, 0x35 // It lt; add lt r5 r5 227
};
const byte kInstruction_add_gt_r4_r4_197[] = {
  0xc8, 0xbf, 0xc5, 0x34 // It gt; add gt r4 r4 197
};
const byte kInstruction_add_hi_r0_r0_77[] = {
  0x88, 0xbf, 0x4d, 0x30 // It hi; add hi r0 r0 77
};
const byte kInstruction_add_mi_r2_r2_223[] = {
  0x48, 0xbf, 0xdf, 0x32 // It mi; add mi r2 r2 223
};
const byte kInstruction_add_pl_r1_r1_62[] = {
  0x58, 0xbf, 0x3e, 0x31 // It pl; add pl r1 r1 62
};
const byte kInstruction_add_lt_r7_r7_53[] = {
  0xb8, 0xbf, 0x35, 0x37 // It lt; add lt r7 r7 53
};
const byte kInstruction_add_ge_r5_r5_153[] = {
  0xa8, 0xbf, 0x99, 0x35 // It ge; add ge r5 r5 153
};
const byte kInstruction_add_ge_r0_r0_169[] = {
  0xa8, 0xbf, 0xa9, 0x30 // It ge; add ge r0 r0 169
};
const byte kInstruction_add_vc_r2_r2_57[] = {
  0x78, 0xbf, 0x39, 0x32 // It vc; add vc r2 r2 57
};
const byte kInstruction_add_eq_r4_r4_121[] = {
  0x08, 0xbf, 0x79, 0x34 // It eq; add eq r4 r4 121
};
const byte kInstruction_add_lt_r1_r1_26[] = {
  0xb8, 0xbf, 0x1a, 0x31 // It lt; add lt r1 r1 26
};
const byte kInstruction_add_pl_r4_r4_209[] = {
  0x58, 0xbf, 0xd1, 0x34 // It pl; add pl r4 r4 209
};
const byte kInstruction_add_vc_r3_r3_233[] = {
  0x78, 0xbf, 0xe9, 0x33 // It vc; add vc r3 r3 233
};
const byte kInstruction_add_hi_r6_r6_159[] = {
  0x88, 0xbf, 0x9f, 0x36 // It hi; add hi r6 r6 159
};
const byte kInstruction_add_pl_r2_r2_5[] = {
  0x58, 0xbf, 0x52, 0x1d // It pl; add pl r2 r2 5
};
const byte kInstruction_add_ne_r4_r4_227[] = {
  0x18, 0xbf, 0xe3, 0x34 // It ne; add ne r4 r4 227
};
const byte kInstruction_add_cc_r4_r4_116[] = {
  0x38, 0xbf, 0x74, 0x34 // It cc; add cc r4 r4 116
};
const byte kInstruction_add_cs_r3_r3_90[] = {
  0x28, 0xbf, 0x5a, 0x33 // It cs; add cs r3 r3 90
};
const byte kInstruction_add_lt_r5_r5_178[] = {
  0xb8, 0xbf, 0xb2, 0x35 // It lt; add lt r5 r5 178
};
const byte kInstruction_add_cs_r6_r6_144[] = {
  0x28, 0xbf, 0x90, 0x36 // It cs; add cs r6 r6 144
};
const byte kInstruction_add_cs_r2_r2_255[] = {
  0x28, 0xbf, 0xff, 0x32 // It cs; add cs r2 r2 255
};
const byte kInstruction_add_mi_r2_r2_207[] = {
  0x48, 0xbf, 0xcf, 0x32 // It mi; add mi r2 r2 207
};
const byte kInstruction_add_ne_r4_r4_116[] = {
  0x18, 0xbf, 0x74, 0x34 // It ne; add ne r4 r4 116
};
const byte kInstruction_add_pl_r4_r4_215[] = {
  0x58, 0xbf, 0xd7, 0x34 // It pl; add pl r4 r4 215
};
const byte kInstruction_add_cs_r5_r5_24[] = {
  0x28, 0xbf, 0x18, 0x35 // It cs; add cs r5 r5 24
};
const byte kInstruction_add_vc_r4_r4_206[] = {
  0x78, 0xbf, 0xce, 0x34 // It vc; add vc r4 r4 206
};
const byte kInstruction_add_ls_r4_r4_124[] = {
  0x98, 0xbf, 0x7c, 0x34 // It ls; add ls r4 r4 124
};
const byte kInstruction_add_cs_r4_r4_169[] = {
  0x28, 0xbf, 0xa9, 0x34 // It cs; add cs r4 r4 169
};
const byte kInstruction_add_ge_r7_r7_168[] = {
  0xa8, 0xbf, 0xa8, 0x37 // It ge; add ge r7 r7 168
};
const byte kInstruction_add_gt_r5_r5_36[] = {
  0xc8, 0xbf, 0x24, 0x35 // It gt; add gt r5 r5 36
};
const byte kInstruction_add_le_r5_r5_188[] = {
  0xd8, 0xbf, 0xbc, 0x35 // It le; add le r5 r5 188
};
const byte kInstruction_add_ne_r6_r6_157[] = {
  0x18, 0xbf, 0x9d, 0x36 // It ne; add ne r6 r6 157
};
const byte kInstruction_add_eq_r1_r1_236[] = {
  0x08, 0xbf, 0xec, 0x31 // It eq; add eq r1 r1 236
};
const byte kInstruction_add_ls_r1_r1_49[] = {
  0x98, 0xbf, 0x31, 0x31 // It ls; add ls r1 r1 49
};
const byte kInstruction_add_vs_r3_r3_127[] = {
  0x68, 0xbf, 0x7f, 0x33 // It vs; add vs r3 r3 127
};
const byte kInstruction_add_le_r1_r1_232[] = {
  0xd8, 0xbf, 0xe8, 0x31 // It le; add le r1 r1 232
};
const byte kInstruction_add_ge_r3_r3_145[] = {
  0xa8, 0xbf, 0x91, 0x33 // It ge; add ge r3 r3 145
};
const byte kInstruction_add_ne_r0_r0_187[] = {
  0x18, 0xbf, 0xbb, 0x30 // It ne; add ne r0 r0 187
};
const byte kInstruction_add_lt_r0_r0_30[] = {
  0xb8, 0xbf, 0x1e, 0x30 // It lt; add lt r0 r0 30
};
const byte kInstruction_add_mi_r7_r7_236[] = {
  0x48, 0xbf, 0xec, 0x37 // It mi; add mi r7 r7 236
};
const byte kInstruction_add_mi_r3_r3_2[] = {
  0x48, 0xbf, 0x9b, 0x1c // It mi; add mi r3 r3 2
};
const byte kInstruction_add_pl_r0_r0_192[] = {
  0x58, 0xbf, 0xc0, 0x30 // It pl; add pl r0 r0 192
};
const byte kInstruction_add_ne_r0_r0_214[] = {
  0x18, 0xbf, 0xd6, 0x30 // It ne; add ne r0 r0 214
};
const byte kInstruction_add_eq_r5_r5_224[] = {
  0x08, 0xbf, 0xe0, 0x35 // It eq; add eq r5 r5 224
};
const byte kInstruction_add_cs_r7_r7_157[] = {
  0x28, 0xbf, 0x9d, 0x37 // It cs; add cs r7 r7 157
};
const byte kInstruction_add_vc_r1_r1_108[] = {
  0x78, 0xbf, 0x6c, 0x31 // It vc; add vc r1 r1 108
};
const byte kInstruction_add_eq_r3_r3_227[] = {
  0x08, 0xbf, 0xe3, 0x33 // It eq; add eq r3 r3 227
};
const byte kInstruction_add_pl_r0_r0_185[] = {
  0x58, 0xbf, 0xb9, 0x30 // It pl; add pl r0 r0 185
};
const byte kInstruction_add_cs_r1_r1_42[] = {
  0x28, 0xbf, 0x2a, 0x31 // It cs; add cs r1 r1 42
};
const byte kInstruction_add_cs_r1_r1_46[] = {
  0x28, 0xbf, 0x2e, 0x31 // It cs; add cs r1 r1 46
};
const byte kInstruction_add_mi_r5_r5_132[] = {
  0x48, 0xbf, 0x84, 0x35 // It mi; add mi r5 r5 132
};
const byte kInstruction_add_ne_r3_r3_187[] = {
  0x18, 0xbf, 0xbb, 0x33 // It ne; add ne r3 r3 187
};
const byte kInstruction_add_hi_r2_r2_26[] = {
  0x88, 0xbf, 0x1a, 0x32 // It hi; add hi r2 r2 26
};
const byte kInstruction_add_cs_r5_r5_135[] = {
  0x28, 0xbf, 0x87, 0x35 // It cs; add cs r5 r5 135
};
const byte kInstruction_add_vc_r2_r2_125[] = {
  0x78, 0xbf, 0x7d, 0x32 // It vc; add vc r2 r2 125
};
const byte kInstruction_add_lt_r1_r1_74[] = {
  0xb8, 0xbf, 0x4a, 0x31 // It lt; add lt r1 r1 74
};
const byte kInstruction_add_mi_r7_r7_83[] = {
  0x48, 0xbf, 0x53, 0x37 // It mi; add mi r7 r7 83
};
const byte kInstruction_add_cs_r5_r5_168[] = {
  0x28, 0xbf, 0xa8, 0x35 // It cs; add cs r5 r5 168
};
const byte kInstruction_add_eq_r3_r3_207[] = {
  0x08, 0xbf, 0xcf, 0x33 // It eq; add eq r3 r3 207
};
const byte kInstruction_add_vc_r0_r0_44[] = {
  0x78, 0xbf, 0x2c, 0x30 // It vc; add vc r0 r0 44
};
const byte kInstruction_add_ne_r0_r0_182[] = {
  0x18, 0xbf, 0xb6, 0x30 // It ne; add ne r0 r0 182
};
const byte kInstruction_add_cc_r2_r2_206[] = {
  0x38, 0xbf, 0xce, 0x32 // It cc; add cc r2 r2 206
};
const byte kInstruction_add_le_r5_r5_121[] = {
  0xd8, 0xbf, 0x79, 0x35 // It le; add le r5 r5 121
};
const byte kInstruction_add_mi_r4_r4_133[] = {
  0x48, 0xbf, 0x85, 0x34 // It mi; add mi r4 r4 133
};
const byte kInstruction_add_le_r2_r2_73[] = {
  0xd8, 0xbf, 0x49, 0x32 // It le; add le r2 r2 73
};
const byte kInstruction_add_gt_r6_r6_77[] = {
  0xc8, 0xbf, 0x4d, 0x36 // It gt; add gt r6 r6 77
};
const byte kInstruction_add_hi_r6_r6_222[] = {
  0x88, 0xbf, 0xde, 0x36 // It hi; add hi r6 r6 222
};
const byte kInstruction_add_vs_r4_r4_113[] = {
  0x68, 0xbf, 0x71, 0x34 // It vs; add vs r4 r4 113
};
const byte kInstruction_add_cs_r1_r1_225[] = {
  0x28, 0xbf, 0xe1, 0x31 // It cs; add cs r1 r1 225
};
const byte kInstruction_add_cc_r5_r5_143[] = {
  0x38, 0xbf, 0x8f, 0x35 // It cc; add cc r5 r5 143
};
const byte kInstruction_add_vs_r0_r0_117[] = {
  0x68, 0xbf, 0x75, 0x30 // It vs; add vs r0 r0 117
};
const byte kInstruction_add_vc_r7_r7_191[] = {
  0x78, 0xbf, 0xbf, 0x37 // It vc; add vc r7 r7 191
};
const byte kInstruction_add_vs_r3_r3_164[] = {
  0x68, 0xbf, 0xa4, 0x33 // It vs; add vs r3 r3 164
};
const byte kInstruction_add_gt_r0_r0_185[] = {
  0xc8, 0xbf, 0xb9, 0x30 // It gt; add gt r0 r0 185
};
const byte kInstruction_add_gt_r7_r7_33[] = {
  0xc8, 0xbf, 0x21, 0x37 // It gt; add gt r7 r7 33
};
const byte kInstruction_add_le_r1_r1_137[] = {
  0xd8, 0xbf, 0x89, 0x31 // It le; add le r1 r1 137
};
const byte kInstruction_add_mi_r7_r7_192[] = {
  0x48, 0xbf, 0xc0, 0x37 // It mi; add mi r7 r7 192
};
const byte kInstruction_add_ne_r0_r0_82[] = {
  0x18, 0xbf, 0x52, 0x30 // It ne; add ne r0 r0 82
};
const byte kInstruction_add_mi_r3_r3_55[] = {
  0x48, 0xbf, 0x37, 0x33 // It mi; add mi r3 r3 55
};
const byte kInstruction_add_vc_r5_r5_183[] = {
  0x78, 0xbf, 0xb7, 0x35 // It vc; add vc r5 r5 183
};
const byte kInstruction_add_cs_r5_r5_22[] = {
  0x28, 0xbf, 0x16, 0x35 // It cs; add cs r5 r5 22
};
const byte kInstruction_add_cs_r3_r3_136[] = {
  0x28, 0xbf, 0x88, 0x33 // It cs; add cs r3 r3 136
};
const byte kInstruction_add_lt_r5_r5_18[] = {
  0xb8, 0xbf, 0x12, 0x35 // It lt; add lt r5 r5 18
};
const byte kInstruction_add_lt_r5_r5_154[] = {
  0xb8, 0xbf, 0x9a, 0x35 // It lt; add lt r5 r5 154
};
const byte kInstruction_add_le_r4_r4_197[] = {
  0xd8, 0xbf, 0xc5, 0x34 // It le; add le r4 r4 197
};
const byte kInstruction_add_lt_r1_r1_185[] = {
  0xb8, 0xbf, 0xb9, 0x31 // It lt; add lt r1 r1 185
};
const byte kInstruction_add_cs_r5_r5_43[] = {
  0x28, 0xbf, 0x2b, 0x35 // It cs; add cs r5 r5 43
};
const byte kInstruction_add_le_r0_r0_108[] = {
  0xd8, 0xbf, 0x6c, 0x30 // It le; add le r0 r0 108
};
const byte kInstruction_add_mi_r1_r1_227[] = {
  0x48, 0xbf, 0xe3, 0x31 // It mi; add mi r1 r1 227
};
const byte kInstruction_add_cc_r0_r0_69[] = {
  0x38, 0xbf, 0x45, 0x30 // It cc; add cc r0 r0 69
};
const byte kInstruction_add_le_r4_r4_222[] = {
  0xd8, 0xbf, 0xde, 0x34 // It le; add le r4 r4 222
};
const byte kInstruction_add_cs_r2_r2_86[] = {
  0x28, 0xbf, 0x56, 0x32 // It cs; add cs r2 r2 86
};
const byte kInstruction_add_vs_r3_r3_10[] = {
  0x68, 0xbf, 0x0a, 0x33 // It vs; add vs r3 r3 10
};
const byte kInstruction_add_mi_r5_r5_247[] = {
  0x48, 0xbf, 0xf7, 0x35 // It mi; add mi r5 r5 247
};
const byte kInstruction_add_gt_r0_r0_241[] = {
  0xc8, 0xbf, 0xf1, 0x30 // It gt; add gt r0 r0 241
};
const byte kInstruction_add_vs_r3_r3_109[] = {
  0x68, 0xbf, 0x6d, 0x33 // It vs; add vs r3 r3 109
};
const byte kInstruction_add_ls_r1_r1_102[] = {
  0x98, 0xbf, 0x66, 0x31 // It ls; add ls r1 r1 102
};
const byte kInstruction_add_cs_r2_r2_87[] = {
  0x28, 0xbf, 0x57, 0x32 // It cs; add cs r2 r2 87
};
const byte kInstruction_add_lt_r0_r0_253[] = {
  0xb8, 0xbf, 0xfd, 0x30 // It lt; add lt r0 r0 253
};
const byte kInstruction_add_eq_r1_r1_56[] = {
  0x08, 0xbf, 0x38, 0x31 // It eq; add eq r1 r1 56
};
const byte kInstruction_add_vs_r7_r7_147[] = {
  0x68, 0xbf, 0x93, 0x37 // It vs; add vs r7 r7 147
};
const byte kInstruction_add_vs_r5_r5_10[] = {
  0x68, 0xbf, 0x0a, 0x35 // It vs; add vs r5 r5 10
};
const byte kInstruction_add_lt_r1_r1_226[] = {
  0xb8, 0xbf, 0xe2, 0x31 // It lt; add lt r1 r1 226
};
const byte kInstruction_add_gt_r1_r1_102[] = {
  0xc8, 0xbf, 0x66, 0x31 // It gt; add gt r1 r1 102
};
const byte kInstruction_add_ls_r7_r7_66[] = {
  0x98, 0xbf, 0x42, 0x37 // It ls; add ls r7 r7 66
};
const byte kInstruction_add_cs_r5_r5_179[] = {
  0x28, 0xbf, 0xb3, 0x35 // It cs; add cs r5 r5 179
};
const byte kInstruction_add_eq_r5_r5_76[] = {
  0x08, 0xbf, 0x4c, 0x35 // It eq; add eq r5 r5 76
};
const byte kInstruction_add_ls_r2_r2_217[] = {
  0x98, 0xbf, 0xd9, 0x32 // It ls; add ls r2 r2 217
};
const byte kInstruction_add_pl_r1_r1_82[] = {
  0x58, 0xbf, 0x52, 0x31 // It pl; add pl r1 r1 82
};
const byte kInstruction_add_cs_r0_r0_66[] = {
  0x28, 0xbf, 0x42, 0x30 // It cs; add cs r0 r0 66
};
const byte kInstruction_add_ne_r4_r4_69[] = {
  0x18, 0xbf, 0x45, 0x34 // It ne; add ne r4 r4 69
};
const byte kInstruction_add_cs_r7_r7_164[] = {
  0x28, 0xbf, 0xa4, 0x37 // It cs; add cs r7 r7 164
};
const byte kInstruction_add_cs_r7_r7_159[] = {
  0x28, 0xbf, 0x9f, 0x37 // It cs; add cs r7 r7 159
};
const byte kInstruction_add_ge_r6_r6_123[] = {
  0xa8, 0xbf, 0x7b, 0x36 // It ge; add ge r6 r6 123
};
const byte kInstruction_add_ge_r6_r6_3[] = {
  0xa8, 0xbf, 0xf6, 0x1c // It ge; add ge r6 r6 3
};
const byte kInstruction_add_lt_r0_r0_160[] = {
  0xb8, 0xbf, 0xa0, 0x30 // It lt; add lt r0 r0 160
};
const byte kInstruction_add_gt_r4_r4_228[] = {
  0xc8, 0xbf, 0xe4, 0x34 // It gt; add gt r4 r4 228
};
const byte kInstruction_add_hi_r5_r5_156[] = {
  0x88, 0xbf, 0x9c, 0x35 // It hi; add hi r5 r5 156
};
const byte kInstruction_add_gt_r0_r0_191[] = {
  0xc8, 0xbf, 0xbf, 0x30 // It gt; add gt r0 r0 191
};
const byte kInstruction_add_pl_r1_r1_178[] = {
  0x58, 0xbf, 0xb2, 0x31 // It pl; add pl r1 r1 178
};
const byte kInstruction_add_vs_r0_r0_243[] = {
  0x68, 0xbf, 0xf3, 0x30 // It vs; add vs r0 r0 243
};
const byte kInstruction_add_lt_r4_r4_217[] = {
  0xb8, 0xbf, 0xd9, 0x34 // It lt; add lt r4 r4 217
};
const byte kInstruction_add_hi_r3_r3_90[] = {
  0x88, 0xbf, 0x5a, 0x33 // It hi; add hi r3 r3 90
};
const byte kInstruction_add_eq_r7_r7_25[] = {
  0x08, 0xbf, 0x19, 0x37 // It eq; add eq r7 r7 25
};
const byte kInstruction_add_vc_r2_r2_212[] = {
  0x78, 0xbf, 0xd4, 0x32 // It vc; add vc r2 r2 212
};
const byte kInstruction_add_pl_r0_r0_252[] = {
  0x58, 0xbf, 0xfc, 0x30 // It pl; add pl r0 r0 252
};
const byte kInstruction_add_cs_r4_r4_236[] = {
  0x28, 0xbf, 0xec, 0x34 // It cs; add cs r4 r4 236
};
const byte kInstruction_add_cc_r0_r0_105[] = {
  0x38, 0xbf, 0x69, 0x30 // It cc; add cc r0 r0 105
};
const byte kInstruction_add_mi_r3_r3_134[] = {
  0x48, 0xbf, 0x86, 0x33 // It mi; add mi r3 r3 134
};
const byte kInstruction_add_pl_r4_r4_116[] = {
  0x58, 0xbf, 0x74, 0x34 // It pl; add pl r4 r4 116
};
const byte kInstruction_add_lt_r5_r5_223[] = {
  0xb8, 0xbf, 0xdf, 0x35 // It lt; add lt r5 r5 223
};
const byte kInstruction_add_vs_r2_r2_242[] = {
  0x68, 0xbf, 0xf2, 0x32 // It vs; add vs r2 r2 242
};
const byte kInstruction_add_cc_r4_r4_150[] = {
  0x38, 0xbf, 0x96, 0x34 // It cc; add cc r4 r4 150
};
const byte kInstruction_add_ne_r4_r4_126[] = {
  0x18, 0xbf, 0x7e, 0x34 // It ne; add ne r4 r4 126
};
const byte kInstruction_add_le_r4_r4_21[] = {
  0xd8, 0xbf, 0x15, 0x34 // It le; add le r4 r4 21
};
const byte kInstruction_add_gt_r2_r2_159[] = {
  0xc8, 0xbf, 0x9f, 0x32 // It gt; add gt r2 r2 159
};
const byte kInstruction_add_vs_r5_r5_139[] = {
  0x68, 0xbf, 0x8b, 0x35 // It vs; add vs r5 r5 139
};
const byte kInstruction_add_eq_r2_r2_46[] = {
  0x08, 0xbf, 0x2e, 0x32 // It eq; add eq r2 r2 46
};
const byte kInstruction_add_mi_r4_r4_68[] = {
  0x48, 0xbf, 0x44, 0x34 // It mi; add mi r4 r4 68
};
const byte kInstruction_add_mi_r4_r4_112[] = {
  0x48, 0xbf, 0x70, 0x34 // It mi; add mi r4 r4 112
};
const byte kInstruction_add_vs_r5_r5_186[] = {
  0x68, 0xbf, 0xba, 0x35 // It vs; add vs r5 r5 186
};
const byte kInstruction_add_cc_r5_r5_216[] = {
  0x38, 0xbf, 0xd8, 0x35 // It cc; add cc r5 r5 216
};
const byte kInstruction_add_lt_r4_r4_12[] = {
  0xb8, 0xbf, 0x0c, 0x34 // It lt; add lt r4 r4 12
};
const byte kInstruction_add_ne_r1_r1_8[] = {
  0x18, 0xbf, 0x08, 0x31 // It ne; add ne r1 r1 8
};
const byte kInstruction_add_cc_r1_r1_83[] = {
  0x38, 0xbf, 0x53, 0x31 // It cc; add cc r1 r1 83
};
const byte kInstruction_add_eq_r7_r7_109[] = {
  0x08, 0xbf, 0x6d, 0x37 // It eq; add eq r7 r7 109
};
const byte kInstruction_add_ne_r4_r4_3[] = {
  0x18, 0xbf, 0xe4, 0x1c // It ne; add ne r4 r4 3
};
const byte kInstruction_add_pl_r5_r5_62[] = {
  0x58, 0xbf, 0x3e, 0x35 // It pl; add pl r5 r5 62
};
const byte kInstruction_add_gt_r1_r1_205[] = {
  0xc8, 0xbf, 0xcd, 0x31 // It gt; add gt r1 r1 205
};
const byte kInstruction_add_ls_r2_r2_209[] = {
  0x98, 0xbf, 0xd1, 0x32 // It ls; add ls r2 r2 209
};
const byte kInstruction_add_vc_r6_r6_252[] = {
  0x78, 0xbf, 0xfc, 0x36 // It vc; add vc r6 r6 252
};
const byte kInstruction_add_cs_r1_r1_218[] = {
  0x28, 0xbf, 0xda, 0x31 // It cs; add cs r1 r1 218
};
const byte kInstruction_add_vs_r1_r1_71[] = {
  0x68, 0xbf, 0x47, 0x31 // It vs; add vs r1 r1 71
};
const byte kInstruction_add_vs_r2_r2_247[] = {
  0x68, 0xbf, 0xf7, 0x32 // It vs; add vs r2 r2 247
};
const byte kInstruction_add_pl_r0_r0_42[] = {
  0x58, 0xbf, 0x2a, 0x30 // It pl; add pl r0 r0 42
};
const byte kInstruction_add_mi_r7_r7_226[] = {
  0x48, 0xbf, 0xe2, 0x37 // It mi; add mi r7 r7 226
};
const byte kInstruction_add_hi_r4_r4_14[] = {
  0x88, 0xbf, 0x0e, 0x34 // It hi; add hi r4 r4 14
};
const byte kInstruction_add_ls_r7_r7_246[] = {
  0x98, 0xbf, 0xf6, 0x37 // It ls; add ls r7 r7 246
};
const byte kInstruction_add_ge_r4_r4_239[] = {
  0xa8, 0xbf, 0xef, 0x34 // It ge; add ge r4 r4 239
};
const byte kInstruction_add_eq_r1_r1_187[] = {
  0x08, 0xbf, 0xbb, 0x31 // It eq; add eq r1 r1 187
};
const byte kInstruction_add_gt_r4_r4_105[] = {
  0xc8, 0xbf, 0x69, 0x34 // It gt; add gt r4 r4 105
};
const byte kInstruction_add_cs_r5_r5_85[] = {
  0x28, 0xbf, 0x55, 0x35 // It cs; add cs r5 r5 85
};
const byte kInstruction_add_le_r4_r4_47[] = {
  0xd8, 0xbf, 0x2f, 0x34 // It le; add le r4 r4 47
};
const byte kInstruction_add_mi_r4_r4_98[] = {
  0x48, 0xbf, 0x62, 0x34 // It mi; add mi r4 r4 98
};
const byte kInstruction_add_gt_r0_r0_180[] = {
  0xc8, 0xbf, 0xb4, 0x30 // It gt; add gt r0 r0 180
};
const byte kInstruction_add_ge_r5_r5_38[] = {
  0xa8, 0xbf, 0x26, 0x35 // It ge; add ge r5 r5 38
};
const byte kInstruction_add_cc_r2_r2_61[] = {
  0x38, 0xbf, 0x3d, 0x32 // It cc; add cc r2 r2 61
};
const byte kInstruction_add_lt_r7_r7_180[] = {
  0xb8, 0xbf, 0xb4, 0x37 // It lt; add lt r7 r7 180
};
const byte kInstruction_add_gt_r7_r7_149[] = {
  0xc8, 0xbf, 0x95, 0x37 // It gt; add gt r7 r7 149
};
const byte kInstruction_add_pl_r6_r6_146[] = {
  0x58, 0xbf, 0x92, 0x36 // It pl; add pl r6 r6 146
};
const byte kInstruction_add_pl_r4_r4_202[] = {
  0x58, 0xbf, 0xca, 0x34 // It pl; add pl r4 r4 202
};
const byte kInstruction_add_hi_r0_r0_225[] = {
  0x88, 0xbf, 0xe1, 0x30 // It hi; add hi r0 r0 225
};
const byte kInstruction_add_cs_r5_r5_142[] = {
  0x28, 0xbf, 0x8e, 0x35 // It cs; add cs r5 r5 142
};
const byte kInstruction_add_lt_r0_r0_224[] = {
  0xb8, 0xbf, 0xe0, 0x30 // It lt; add lt r0 r0 224
};
const byte kInstruction_add_vc_r5_r5_170[] = {
  0x78, 0xbf, 0xaa, 0x35 // It vc; add vc r5 r5 170
};
const byte kInstruction_add_le_r7_r7_205[] = {
  0xd8, 0xbf, 0xcd, 0x37 // It le; add le r7 r7 205
};
const byte kInstruction_add_cs_r2_r2_228[] = {
  0x28, 0xbf, 0xe4, 0x32 // It cs; add cs r2 r2 228
};
const byte kInstruction_add_cc_r0_r0_96[] = {
  0x38, 0xbf, 0x60, 0x30 // It cc; add cc r0 r0 96
};
const byte kInstruction_add_vc_r0_r0_134[] = {
  0x78, 0xbf, 0x86, 0x30 // It vc; add vc r0 r0 134
};
const byte kInstruction_add_le_r7_r7_5[] = {
  0xd8, 0xbf, 0x7f, 0x1d // It le; add le r7 r7 5
};
const byte kInstruction_add_ne_r0_r0_33[] = {
  0x18, 0xbf, 0x21, 0x30 // It ne; add ne r0 r0 33
};
const byte kInstruction_add_vc_r3_r3_162[] = {
  0x78, 0xbf, 0xa2, 0x33 // It vc; add vc r3 r3 162
};
const byte kInstruction_add_mi_r2_r2_111[] = {
  0x48, 0xbf, 0x6f, 0x32 // It mi; add mi r2 r2 111
};
const byte kInstruction_add_vs_r1_r1_210[] = {
  0x68, 0xbf, 0xd2, 0x31 // It vs; add vs r1 r1 210
};
const byte kInstruction_add_mi_r2_r2_244[] = {
  0x48, 0xbf, 0xf4, 0x32 // It mi; add mi r2 r2 244
};
const byte kInstruction_add_cs_r6_r6_208[] = {
  0x28, 0xbf, 0xd0, 0x36 // It cs; add cs r6 r6 208
};
const byte kInstruction_add_gt_r1_r1_213[] = {
  0xc8, 0xbf, 0xd5, 0x31 // It gt; add gt r1 r1 213
};
const byte kInstruction_add_ge_r7_r7_63[] = {
  0xa8, 0xbf, 0x3f, 0x37 // It ge; add ge r7 r7 63
};
const byte kInstruction_add_mi_r2_r2_214[] = {
  0x48, 0xbf, 0xd6, 0x32 // It mi; add mi r2 r2 214
};
const byte kInstruction_add_ne_r6_r6_224[] = {
  0x18, 0xbf, 0xe0, 0x36 // It ne; add ne r6 r6 224
};
const byte kInstruction_add_vc_r1_r1_169[] = {
  0x78, 0xbf, 0xa9, 0x31 // It vc; add vc r1 r1 169
};
const byte kInstruction_add_vc_r2_r2_228[] = {
  0x78, 0xbf, 0xe4, 0x32 // It vc; add vc r2 r2 228
};
const byte kInstruction_add_ge_r1_r1_161[] = {
  0xa8, 0xbf, 0xa1, 0x31 // It ge; add ge r1 r1 161
};
const byte kInstruction_add_vc_r1_r1_250[] = {
  0x78, 0xbf, 0xfa, 0x31 // It vc; add vc r1 r1 250
};
const byte kInstruction_add_cs_r0_r0_214[] = {
  0x28, 0xbf, 0xd6, 0x30 // It cs; add cs r0 r0 214
};
const byte kInstruction_add_lt_r0_r0_104[] = {
  0xb8, 0xbf, 0x68, 0x30 // It lt; add lt r0 r0 104
};
const byte kInstruction_add_cs_r2_r2_210[] = {
  0x28, 0xbf, 0xd2, 0x32 // It cs; add cs r2 r2 210
};
const byte kInstruction_add_mi_r2_r2_229[] = {
  0x48, 0xbf, 0xe5, 0x32 // It mi; add mi r2 r2 229
};
const byte kInstruction_add_cs_r5_r5_55[] = {
  0x28, 0xbf, 0x37, 0x35 // It cs; add cs r5 r5 55
};
const byte kInstruction_add_lt_r2_r2_17[] = {
  0xb8, 0xbf, 0x11, 0x32 // It lt; add lt r2 r2 17
};
const byte kInstruction_add_ls_r2_r2_169[] = {
  0x98, 0xbf, 0xa9, 0x32 // It ls; add ls r2 r2 169
};
const byte kInstruction_add_vc_r6_r6_74[] = {
  0x78, 0xbf, 0x4a, 0x36 // It vc; add vc r6 r6 74
};
const byte kInstruction_add_ge_r1_r1_242[] = {
  0xa8, 0xbf, 0xf2, 0x31 // It ge; add ge r1 r1 242
};
const byte kInstruction_add_vc_r4_r4_234[] = {
  0x78, 0xbf, 0xea, 0x34 // It vc; add vc r4 r4 234
};
const byte kInstruction_add_eq_r2_r2_147[] = {
  0x08, 0xbf, 0x93, 0x32 // It eq; add eq r2 r2 147
};
const byte kInstruction_add_vc_r1_r1_101[] = {
  0x78, 0xbf, 0x65, 0x31 // It vc; add vc r1 r1 101
};
const byte kInstruction_add_pl_r6_r6_226[] = {
  0x58, 0xbf, 0xe2, 0x36 // It pl; add pl r6 r6 226
};
const byte kInstruction_add_gt_r6_r6_184[] = {
  0xc8, 0xbf, 0xb8, 0x36 // It gt; add gt r6 r6 184
};
const byte kInstruction_add_gt_r3_r3_177[] = {
  0xc8, 0xbf, 0xb1, 0x33 // It gt; add gt r3 r3 177
};
const byte kInstruction_add_ne_r1_r1_126[] = {
  0x18, 0xbf, 0x7e, 0x31 // It ne; add ne r1 r1 126
};
const byte kInstruction_add_ls_r5_r5_222[] = {
  0x98, 0xbf, 0xde, 0x35 // It ls; add ls r5 r5 222
};
const byte kInstruction_add_hi_r4_r4_2[] = {
  0x88, 0xbf, 0xa4, 0x1c // It hi; add hi r4 r4 2
};
const byte kInstruction_add_ls_r2_r2_190[] = {
  0x98, 0xbf, 0xbe, 0x32 // It ls; add ls r2 r2 190
};
const byte kInstruction_add_mi_r4_r4_254[] = {
  0x48, 0xbf, 0xfe, 0x34 // It mi; add mi r4 r4 254
};
const byte kInstruction_add_pl_r4_r4_19[] = {
  0x58, 0xbf, 0x13, 0x34 // It pl; add pl r4 r4 19
};
const byte kInstruction_add_ls_r0_r0_241[] = {
  0x98, 0xbf, 0xf1, 0x30 // It ls; add ls r0 r0 241
};
const byte kInstruction_add_cc_r3_r3_148[] = {
  0x38, 0xbf, 0x94, 0x33 // It cc; add cc r3 r3 148
};
const byte kInstruction_add_gt_r1_r1_195[] = {
  0xc8, 0xbf, 0xc3, 0x31 // It gt; add gt r1 r1 195
};
const byte kInstruction_add_vc_r4_r4_91[] = {
  0x78, 0xbf, 0x5b, 0x34 // It vc; add vc r4 r4 91
};
const byte kInstruction_add_gt_r0_r0_227[] = {
  0xc8, 0xbf, 0xe3, 0x30 // It gt; add gt r0 r0 227
};
const byte kInstruction_add_vs_r6_r6_86[] = {
  0x68, 0xbf, 0x56, 0x36 // It vs; add vs r6 r6 86
};
const byte kInstruction_add_ls_r1_r1_241[] = {
  0x98, 0xbf, 0xf1, 0x31 // It ls; add ls r1 r1 241
};
const byte kInstruction_add_le_r3_r3_241[] = {
  0xd8, 0xbf, 0xf1, 0x33 // It le; add le r3 r3 241
};
const byte kInstruction_add_vs_r2_r2_217[] = {
  0x68, 0xbf, 0xd9, 0x32 // It vs; add vs r2 r2 217
};
const byte kInstruction_add_ls_r7_r7_33[] = {
  0x98, 0xbf, 0x21, 0x37 // It ls; add ls r7 r7 33
};
const byte kInstruction_add_ge_r1_r1_76[] = {
  0xa8, 0xbf, 0x4c, 0x31 // It ge; add ge r1 r1 76
};
const byte kInstruction_add_lt_r2_r2_34[] = {
  0xb8, 0xbf, 0x22, 0x32 // It lt; add lt r2 r2 34
};
const byte kInstruction_add_gt_r7_r7_91[] = {
  0xc8, 0xbf, 0x5b, 0x37 // It gt; add gt r7 r7 91
};
const byte kInstruction_add_le_r3_r3_121[] = {
  0xd8, 0xbf, 0x79, 0x33 // It le; add le r3 r3 121
};
const byte kInstruction_add_ge_r0_r0_128[] = {
  0xa8, 0xbf, 0x80, 0x30 // It ge; add ge r0 r0 128
};
const byte kInstruction_add_vc_r6_r6_199[] = {
  0x78, 0xbf, 0xc7, 0x36 // It vc; add vc r6 r6 199
};
const byte kInstruction_add_hi_r6_r6_138[] = {
  0x88, 0xbf, 0x8a, 0x36 // It hi; add hi r6 r6 138
};
const byte kInstruction_add_pl_r4_r4_241[] = {
  0x58, 0xbf, 0xf1, 0x34 // It pl; add pl r4 r4 241
};
const byte kInstruction_add_vc_r5_r5_246[] = {
  0x78, 0xbf, 0xf6, 0x35 // It vc; add vc r5 r5 246
};
const byte kInstruction_add_cc_r5_r5_133[] = {
  0x38, 0xbf, 0x85, 0x35 // It cc; add cc r5 r5 133
};
const byte kInstruction_add_cs_r5_r5_1[] = {
  0x28, 0xbf, 0x6d, 0x1c // It cs; add cs r5 r5 1
};
const byte kInstruction_add_mi_r1_r1_233[] = {
  0x48, 0xbf, 0xe9, 0x31 // It mi; add mi r1 r1 233
};
const byte kInstruction_add_cs_r3_r3_195[] = {
  0x28, 0xbf, 0xc3, 0x33 // It cs; add cs r3 r3 195
};
const byte kInstruction_add_pl_r0_r0_13[] = {
  0x58, 0xbf, 0x0d, 0x30 // It pl; add pl r0 r0 13
};
const byte kInstruction_add_eq_r5_r5_187[] = {
  0x08, 0xbf, 0xbb, 0x35 // It eq; add eq r5 r5 187
};
const byte kInstruction_add_ls_r2_r2_74[] = {
  0x98, 0xbf, 0x4a, 0x32 // It ls; add ls r2 r2 74
};
const byte kInstruction_add_ge_r4_r4_33[] = {
  0xa8, 0xbf, 0x21, 0x34 // It ge; add ge r4 r4 33
};
const byte kInstruction_add_hi_r3_r3_253[] = {
  0x88, 0xbf, 0xfd, 0x33 // It hi; add hi r3 r3 253
};
const byte kInstruction_add_pl_r1_r1_185[] = {
  0x58, 0xbf, 0xb9, 0x31 // It pl; add pl r1 r1 185
};
const byte kInstruction_add_gt_r2_r2_103[] = {
  0xc8, 0xbf, 0x67, 0x32 // It gt; add gt r2 r2 103
};
const byte kInstruction_add_ls_r3_r3_255[] = {
  0x98, 0xbf, 0xff, 0x33 // It ls; add ls r3 r3 255
};
const byte kInstruction_add_vs_r6_r6_207[] = {
  0x68, 0xbf, 0xcf, 0x36 // It vs; add vs r6 r6 207
};
const byte kInstruction_add_cs_r5_r5_255[] = {
  0x28, 0xbf, 0xff, 0x35 // It cs; add cs r5 r5 255
};
const byte kInstruction_add_lt_r1_r1_107[] = {
  0xb8, 0xbf, 0x6b, 0x31 // It lt; add lt r1 r1 107
};
const byte kInstruction_add_le_r3_r3_122[] = {
  0xd8, 0xbf, 0x7a, 0x33 // It le; add le r3 r3 122
};
const byte kInstruction_add_vc_r0_r0_223[] = {
  0x78, 0xbf, 0xdf, 0x30 // It vc; add vc r0 r0 223
};
const byte kInstruction_add_cc_r7_r7_38[] = {
  0x38, 0xbf, 0x26, 0x37 // It cc; add cc r7 r7 38
};
const byte kInstruction_add_cc_r4_r4_207[] = {
  0x38, 0xbf, 0xcf, 0x34 // It cc; add cc r4 r4 207
};
const byte kInstruction_add_hi_r3_r3_101[] = {
  0x88, 0xbf, 0x65, 0x33 // It hi; add hi r3 r3 101
};
const byte kInstruction_add_gt_r0_r0_237[] = {
  0xc8, 0xbf, 0xed, 0x30 // It gt; add gt r0 r0 237
};
const byte kInstruction_add_mi_r6_r6_13[] = {
  0x48, 0xbf, 0x0d, 0x36 // It mi; add mi r6 r6 13
};
const byte kInstruction_add_cs_r1_r1_109[] = {
  0x28, 0xbf, 0x6d, 0x31 // It cs; add cs r1 r1 109
};
const byte kInstruction_add_le_r4_r4_132[] = {
  0xd8, 0xbf, 0x84, 0x34 // It le; add le r4 r4 132
};
const byte kInstruction_add_le_r2_r2_185[] = {
  0xd8, 0xbf, 0xb9, 0x32 // It le; add le r2 r2 185
};
const byte kInstruction_add_lt_r3_r3_220[] = {
  0xb8, 0xbf, 0xdc, 0x33 // It lt; add lt r3 r3 220
};
const byte kInstruction_add_eq_r7_r7_248[] = {
  0x08, 0xbf, 0xf8, 0x37 // It eq; add eq r7 r7 248
};
const byte kInstruction_add_le_r7_r7_146[] = {
  0xd8, 0xbf, 0x92, 0x37 // It le; add le r7 r7 146
};
const byte kInstruction_add_hi_r1_r1_34[] = {
  0x88, 0xbf, 0x22, 0x31 // It hi; add hi r1 r1 34
};
const byte kInstruction_add_pl_r3_r3_146[] = {
  0x58, 0xbf, 0x92, 0x33 // It pl; add pl r3 r3 146
};
const byte kInstruction_add_cs_r6_r6_22[] = {
  0x28, 0xbf, 0x16, 0x36 // It cs; add cs r6 r6 22
};
const byte kInstruction_add_lt_r2_r2_187[] = {
  0xb8, 0xbf, 0xbb, 0x32 // It lt; add lt r2 r2 187
};
const byte kInstruction_add_cc_r4_r4_176[] = {
  0x38, 0xbf, 0xb0, 0x34 // It cc; add cc r4 r4 176
};
const byte kInstruction_add_ge_r3_r3_233[] = {
  0xa8, 0xbf, 0xe9, 0x33 // It ge; add ge r3 r3 233
};
const byte kInstruction_add_ge_r2_r2_41[] = {
  0xa8, 0xbf, 0x29, 0x32 // It ge; add ge r2 r2 41
};
const byte kInstruction_add_lt_r0_r0_207[] = {
  0xb8, 0xbf, 0xcf, 0x30 // It lt; add lt r0 r0 207
};
const byte kInstruction_add_vc_r0_r0_147[] = {
  0x78, 0xbf, 0x93, 0x30 // It vc; add vc r0 r0 147
};
const byte kInstruction_add_pl_r4_r4_89[] = {
  0x58, 0xbf, 0x59, 0x34 // It pl; add pl r4 r4 89
};
const byte kInstruction_add_cs_r2_r2_57[] = {
  0x28, 0xbf, 0x39, 0x32 // It cs; add cs r2 r2 57
};
const byte kInstruction_add_vc_r0_r0_24[] = {
  0x78, 0xbf, 0x18, 0x30 // It vc; add vc r0 r0 24
};
const byte kInstruction_add_mi_r6_r6_114[] = {
  0x48, 0xbf, 0x72, 0x36 // It mi; add mi r6 r6 114
};
const byte kInstruction_add_pl_r4_r4_109[] = {
  0x58, 0xbf, 0x6d, 0x34 // It pl; add pl r4 r4 109
};
const byte kInstruction_add_mi_r7_r7_98[] = {
  0x48, 0xbf, 0x62, 0x37 // It mi; add mi r7 r7 98
};
const byte kInstruction_add_eq_r5_r5_124[] = {
  0x08, 0xbf, 0x7c, 0x35 // It eq; add eq r5 r5 124
};
const byte kInstruction_add_le_r3_r3_253[] = {
  0xd8, 0xbf, 0xfd, 0x33 // It le; add le r3 r3 253
};
const byte kInstruction_add_hi_r5_r5_149[] = {
  0x88, 0xbf, 0x95, 0x35 // It hi; add hi r5 r5 149
};
const byte kInstruction_add_cc_r5_r5_5[] = {
  0x38, 0xbf, 0x6d, 0x1d // It cc; add cc r5 r5 5
};
const byte kInstruction_add_cs_r1_r1_149[] = {
  0x28, 0xbf, 0x95, 0x31 // It cs; add cs r1 r1 149
};
const byte kInstruction_add_gt_r3_r3_149[] = {
  0xc8, 0xbf, 0x95, 0x33 // It gt; add gt r3 r3 149
};
const byte kInstruction_add_vs_r3_r3_20[] = {
  0x68, 0xbf, 0x14, 0x33 // It vs; add vs r3 r3 20
};
const byte kInstruction_add_lt_r7_r7_241[] = {
  0xb8, 0xbf, 0xf1, 0x37 // It lt; add lt r7 r7 241
};
const byte kInstruction_add_cs_r2_r2_42[] = {
  0x28, 0xbf, 0x2a, 0x32 // It cs; add cs r2 r2 42
};
const byte kInstruction_add_vs_r2_r2_122[] = {
  0x68, 0xbf, 0x7a, 0x32 // It vs; add vs r2 r2 122
};
const byte kInstruction_add_ne_r0_r0_203[] = {
  0x18, 0xbf, 0xcb, 0x30 // It ne; add ne r0 r0 203
};
const byte kInstruction_add_cc_r4_r4_80[] = {
  0x38, 0xbf, 0x50, 0x34 // It cc; add cc r4 r4 80
};
const byte kInstruction_add_gt_r0_r0_162[] = {
  0xc8, 0xbf, 0xa2, 0x30 // It gt; add gt r0 r0 162
};
const byte kInstruction_add_gt_r4_r4_155[] = {
  0xc8, 0xbf, 0x9b, 0x34 // It gt; add gt r4 r4 155
};
const byte kInstruction_add_mi_r1_r1_93[] = {
  0x48, 0xbf, 0x5d, 0x31 // It mi; add mi r1 r1 93
};
const byte kInstruction_add_gt_r2_r2_94[] = {
  0xc8, 0xbf, 0x5e, 0x32 // It gt; add gt r2 r2 94
};
const byte kInstruction_add_eq_r0_r0_87[] = {
  0x08, 0xbf, 0x57, 0x30 // It eq; add eq r0 r0 87
};
const byte kInstruction_add_pl_r1_r1_141[] = {
  0x58, 0xbf, 0x8d, 0x31 // It pl; add pl r1 r1 141
};
const byte kInstruction_add_cc_r6_r6_42[] = {
  0x38, 0xbf, 0x2a, 0x36 // It cc; add cc r6 r6 42
};
const byte kInstruction_add_vc_r0_r0_178[] = {
  0x78, 0xbf, 0xb2, 0x30 // It vc; add vc r0 r0 178
};
const byte kInstruction_add_pl_r6_r6_109[] = {
  0x58, 0xbf, 0x6d, 0x36 // It pl; add pl r6 r6 109
};
const byte kInstruction_add_mi_r0_r0_186[] = {
  0x48, 0xbf, 0xba, 0x30 // It mi; add mi r0 r0 186
};
const byte kInstruction_add_gt_r6_r6_116[] = {
  0xc8, 0xbf, 0x74, 0x36 // It gt; add gt r6 r6 116
};
const byte kInstruction_add_vc_r3_r3_249[] = {
  0x78, 0xbf, 0xf9, 0x33 // It vc; add vc r3 r3 249
};
const byte kInstruction_add_vs_r6_r6_197[] = {
  0x68, 0xbf, 0xc5, 0x36 // It vs; add vs r6 r6 197
};
const byte kInstruction_add_hi_r5_r5_176[] = {
  0x88, 0xbf, 0xb0, 0x35 // It hi; add hi r5 r5 176
};
const byte kInstruction_add_ls_r3_r3_10[] = {
  0x98, 0xbf, 0x0a, 0x33 // It ls; add ls r3 r3 10
};
const byte kInstruction_add_ge_r0_r0_120[] = {
  0xa8, 0xbf, 0x78, 0x30 // It ge; add ge r0 r0 120
};
const byte kInstruction_add_ge_r0_r0_197[] = {
  0xa8, 0xbf, 0xc5, 0x30 // It ge; add ge r0 r0 197
};
const byte kInstruction_add_ls_r7_r7_149[] = {
  0x98, 0xbf, 0x95, 0x37 // It ls; add ls r7 r7 149
};
const byte kInstruction_add_mi_r2_r2_219[] = {
  0x48, 0xbf, 0xdb, 0x32 // It mi; add mi r2 r2 219
};
const byte kInstruction_add_lt_r4_r4_130[] = {
  0xb8, 0xbf, 0x82, 0x34 // It lt; add lt r4 r4 130
};
const byte kInstruction_add_pl_r4_r4_220[] = {
  0x58, 0xbf, 0xdc, 0x34 // It pl; add pl r4 r4 220
};
const byte kInstruction_add_gt_r2_r2_101[] = {
  0xc8, 0xbf, 0x65, 0x32 // It gt; add gt r2 r2 101
};
const byte kInstruction_add_pl_r6_r6_111[] = {
  0x58, 0xbf, 0x6f, 0x36 // It pl; add pl r6 r6 111
};
const byte kInstruction_add_hi_r2_r2_21[] = {
  0x88, 0xbf, 0x15, 0x32 // It hi; add hi r2 r2 21
};
const byte kInstruction_add_lt_r5_r5_174[] = {
  0xb8, 0xbf, 0xae, 0x35 // It lt; add lt r5 r5 174
};
const byte kInstruction_add_hi_r0_r0_190[] = {
  0x88, 0xbf, 0xbe, 0x30 // It hi; add hi r0 r0 190
};
const byte kInstruction_add_ge_r7_r7_166[] = {
  0xa8, 0xbf, 0xa6, 0x37 // It ge; add ge r7 r7 166
};
const byte kInstruction_add_vc_r5_r5_251[] = {
  0x78, 0xbf, 0xfb, 0x35 // It vc; add vc r5 r5 251
};
const byte kInstruction_add_ne_r2_r2_174[] = {
  0x18, 0xbf, 0xae, 0x32 // It ne; add ne r2 r2 174
};
const byte kInstruction_add_eq_r6_r6_255[] = {
  0x08, 0xbf, 0xff, 0x36 // It eq; add eq r6 r6 255
};
const byte kInstruction_add_ls_r5_r5_186[] = {
  0x98, 0xbf, 0xba, 0x35 // It ls; add ls r5 r5 186
};
const byte kInstruction_add_vs_r3_r3_75[] = {
  0x68, 0xbf, 0x4b, 0x33 // It vs; add vs r3 r3 75
};
const byte kInstruction_add_cs_r2_r2_15[] = {
  0x28, 0xbf, 0x0f, 0x32 // It cs; add cs r2 r2 15
};
const byte kInstruction_add_hi_r6_r6_133[] = {
  0x88, 0xbf, 0x85, 0x36 // It hi; add hi r6 r6 133
};
const byte kInstruction_add_vc_r0_r0_188[] = {
  0x78, 0xbf, 0xbc, 0x30 // It vc; add vc r0 r0 188
};
const byte kInstruction_add_ne_r2_r2_85[] = {
  0x18, 0xbf, 0x55, 0x32 // It ne; add ne r2 r2 85
};
const byte kInstruction_add_hi_r2_r2_116[] = {
  0x88, 0xbf, 0x74, 0x32 // It hi; add hi r2 r2 116
};
const byte kInstruction_add_le_r3_r3_91[] = {
  0xd8, 0xbf, 0x5b, 0x33 // It le; add le r3 r3 91
};
const byte kInstruction_add_pl_r1_r1_222[] = {
  0x58, 0xbf, 0xde, 0x31 // It pl; add pl r1 r1 222
};
const byte kInstruction_add_mi_r0_r0_33[] = {
  0x48, 0xbf, 0x21, 0x30 // It mi; add mi r0 r0 33
};
const byte kInstruction_add_vs_r3_r3_26[] = {
  0x68, 0xbf, 0x1a, 0x33 // It vs; add vs r3 r3 26
};
const byte kInstruction_add_cc_r5_r5_182[] = {
  0x38, 0xbf, 0xb6, 0x35 // It cc; add cc r5 r5 182
};
const byte kInstruction_add_gt_r0_r0_62[] = {
  0xc8, 0xbf, 0x3e, 0x30 // It gt; add gt r0 r0 62
};
const byte kInstruction_add_vc_r1_r1_189[] = {
  0x78, 0xbf, 0xbd, 0x31 // It vc; add vc r1 r1 189
};
const byte kInstruction_add_pl_r2_r2_180[] = {
  0x58, 0xbf, 0xb4, 0x32 // It pl; add pl r2 r2 180
};
const byte kInstruction_add_mi_r5_r5_15[] = {
  0x48, 0xbf, 0x0f, 0x35 // It mi; add mi r5 r5 15
};
const byte kInstruction_add_lt_r4_r4_24[] = {
  0xb8, 0xbf, 0x18, 0x34 // It lt; add lt r4 r4 24
};
const byte kInstruction_add_cs_r6_r6_190[] = {
  0x28, 0xbf, 0xbe, 0x36 // It cs; add cs r6 r6 190
};
const byte kInstruction_add_ne_r6_r6_168[] = {
  0x18, 0xbf, 0xa8, 0x36 // It ne; add ne r6 r6 168
};
const byte kInstruction_add_ls_r2_r2_48[] = {
  0x98, 0xbf, 0x30, 0x32 // It ls; add ls r2 r2 48
};
const byte kInstruction_add_vs_r7_r7_6[] = {
  0x68, 0xbf, 0xbf, 0x1d // It vs; add vs r7 r7 6
};
const byte kInstruction_add_cs_r4_r4_80[] = {
  0x28, 0xbf, 0x50, 0x34 // It cs; add cs r4 r4 80
};
const byte kInstruction_add_ls_r3_r3_75[] = {
  0x98, 0xbf, 0x4b, 0x33 // It ls; add ls r3 r3 75
};
const byte kInstruction_add_ge_r2_r2_236[] = {
  0xa8, 0xbf, 0xec, 0x32 // It ge; add ge r2 r2 236
};
const byte kInstruction_add_hi_r1_r1_9[] = {
  0x88, 0xbf, 0x09, 0x31 // It hi; add hi r1 r1 9
};
const byte kInstruction_add_cs_r1_r1_51[] = {
  0x28, 0xbf, 0x33, 0x31 // It cs; add cs r1 r1 51
};
const byte kInstruction_add_cc_r1_r1_139[] = {
  0x38, 0xbf, 0x8b, 0x31 // It cc; add cc r1 r1 139
};
const byte kInstruction_add_cc_r2_r2_204[] = {
  0x38, 0xbf, 0xcc, 0x32 // It cc; add cc r2 r2 204
};
const byte kInstruction_add_gt_r2_r2_216[] = {
  0xc8, 0xbf, 0xd8, 0x32 // It gt; add gt r2 r2 216
};
const byte kInstruction_add_vc_r7_r7_60[] = {
  0x78, 0xbf, 0x3c, 0x37 // It vc; add vc r7 r7 60
};
const byte kInstruction_add_pl_r6_r6_249[] = {
  0x58, 0xbf, 0xf9, 0x36 // It pl; add pl r6 r6 249
};
const byte kInstruction_add_cs_r1_r1_169[] = {
  0x28, 0xbf, 0xa9, 0x31 // It cs; add cs r1 r1 169
};
const byte kInstruction_add_eq_r6_r6_33[] = {
  0x08, 0xbf, 0x21, 0x36 // It eq; add eq r6 r6 33
};
const byte kInstruction_add_ne_r0_r0_91[] = {
  0x18, 0xbf, 0x5b, 0x30 // It ne; add ne r0 r0 91
};
const byte kInstruction_add_vc_r4_r4_56[] = {
  0x78, 0xbf, 0x38, 0x34 // It vc; add vc r4 r4 56
};
const byte kInstruction_add_ge_r5_r5_26[] = {
  0xa8, 0xbf, 0x1a, 0x35 // It ge; add ge r5 r5 26
};
const byte kInstruction_add_lt_r6_r6_210[] = {
  0xb8, 0xbf, 0xd2, 0x36 // It lt; add lt r6 r6 210
};
const byte kInstruction_add_le_r6_r6_122[] = {
  0xd8, 0xbf, 0x7a, 0x36 // It le; add le r6 r6 122
};
const byte kInstruction_add_ls_r4_r4_118[] = {
  0x98, 0xbf, 0x76, 0x34 // It ls; add ls r4 r4 118
};
const byte kInstruction_add_lt_r3_r3_59[] = {
  0xb8, 0xbf, 0x3b, 0x33 // It lt; add lt r3 r3 59
};
const byte kInstruction_add_cc_r3_r3_144[] = {
  0x38, 0xbf, 0x90, 0x33 // It cc; add cc r3 r3 144
};
const byte kInstruction_add_le_r6_r6_159[] = {
  0xd8, 0xbf, 0x9f, 0x36 // It le; add le r6 r6 159
};
const byte kInstruction_add_lt_r0_r0_25[] = {
  0xb8, 0xbf, 0x19, 0x30 // It lt; add lt r0 r0 25
};
const byte kInstruction_add_vs_r0_r0_150[] = {
  0x68, 0xbf, 0x96, 0x30 // It vs; add vs r0 r0 150
};
const byte kInstruction_add_ne_r1_r1_91[] = {
  0x18, 0xbf, 0x5b, 0x31 // It ne; add ne r1 r1 91
};
const byte kInstruction_add_ls_r2_r2_65[] = {
  0x98, 0xbf, 0x41, 0x32 // It ls; add ls r2 r2 65
};
const byte kInstruction_add_lt_r3_r3_80[] = {
  0xb8, 0xbf, 0x50, 0x33 // It lt; add lt r3 r3 80
};
const byte kInstruction_add_eq_r0_r0_37[] = {
  0x08, 0xbf, 0x25, 0x30 // It eq; add eq r0 r0 37
};
const byte kInstruction_add_ne_r4_r4_71[] = {
  0x18, 0xbf, 0x47, 0x34 // It ne; add ne r4 r4 71
};
const byte kInstruction_add_ls_r7_r7_169[] = {
  0x98, 0xbf, 0xa9, 0x37 // It ls; add ls r7 r7 169
};
const byte kInstruction_add_cs_r0_r0_141[] = {
  0x28, 0xbf, 0x8d, 0x30 // It cs; add cs r0 r0 141
};
const byte kInstruction_add_pl_r3_r3_190[] = {
  0x58, 0xbf, 0xbe, 0x33 // It pl; add pl r3 r3 190
};
const byte kInstruction_add_pl_r7_r7_53[] = {
  0x58, 0xbf, 0x35, 0x37 // It pl; add pl r7 r7 53
};
const byte kInstruction_add_vc_r3_r3_60[] = {
  0x78, 0xbf, 0x3c, 0x33 // It vc; add vc r3 r3 60
};
const byte kInstruction_add_ge_r3_r3_104[] = {
  0xa8, 0xbf, 0x68, 0x33 // It ge; add ge r3 r3 104
};
const byte kInstruction_add_pl_r1_r1_93[] = {
  0x58, 0xbf, 0x5d, 0x31 // It pl; add pl r1 r1 93
};
const byte kInstruction_add_pl_r7_r7_234[] = {
  0x58, 0xbf, 0xea, 0x37 // It pl; add pl r7 r7 234
};
const byte kInstruction_add_lt_r0_r0_226[] = {
  0xb8, 0xbf, 0xe2, 0x30 // It lt; add lt r0 r0 226
};
const byte kInstruction_add_mi_r1_r1_207[] = {
  0x48, 0xbf, 0xcf, 0x31 // It mi; add mi r1 r1 207
};
const byte kInstruction_add_le_r1_r1_152[] = {
  0xd8, 0xbf, 0x98, 0x31 // It le; add le r1 r1 152
};
const byte kInstruction_add_gt_r6_r6_151[] = {
  0xc8, 0xbf, 0x97, 0x36 // It gt; add gt r6 r6 151
};
const byte kInstruction_add_ge_r4_r4_127[] = {
  0xa8, 0xbf, 0x7f, 0x34 // It ge; add ge r4 r4 127
};
const byte kInstruction_add_lt_r1_r1_167[] = {
  0xb8, 0xbf, 0xa7, 0x31 // It lt; add lt r1 r1 167
};
const byte kInstruction_add_pl_r2_r2_206[] = {
  0x58, 0xbf, 0xce, 0x32 // It pl; add pl r2 r2 206
};
const byte kInstruction_add_ge_r1_r1_241[] = {
  0xa8, 0xbf, 0xf1, 0x31 // It ge; add ge r1 r1 241
};
const byte kInstruction_add_vs_r3_r3_156[] = {
  0x68, 0xbf, 0x9c, 0x33 // It vs; add vs r3 r3 156
};
const byte kInstruction_add_gt_r0_r0_125[] = {
  0xc8, 0xbf, 0x7d, 0x30 // It gt; add gt r0 r0 125
};
const TestResult kReferenceadd[] = {
  {
    ARRAY_SIZE(kInstruction_add_mi_r3_r3_90),
    kInstruction_add_mi_r3_r3_90,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r6_138),
    kInstruction_add_vc_r6_r6_138,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r5_r5_72),
    kInstruction_add_cc_r5_r5_72,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r6_181),
    kInstruction_add_ge_r6_r6_181,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r1_r1_143),
    kInstruction_add_cc_r1_r1_143,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r5_r5_114),
    kInstruction_add_vc_r5_r5_114,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r4_r4_195),
    kInstruction_add_ge_r4_r4_195,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r0_203),
    kInstruction_add_lt_r0_r0_203,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r1_r1_98),
    kInstruction_add_hi_r1_r1_98,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r3_r3_0),
    kInstruction_add_gt_r3_r3_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r5_r5_195),
    kInstruction_add_vs_r5_r5_195,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r6_r6_39),
    kInstruction_add_le_r6_r6_39,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r0_r0_231),
    kInstruction_add_mi_r0_r0_231,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r5_r5_220),
    kInstruction_add_mi_r5_r5_220,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r4_r4_60),
    kInstruction_add_hi_r4_r4_60,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r0_r0_83),
    kInstruction_add_vc_r0_r0_83,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r3_r3_207),
    kInstruction_add_lt_r3_r3_207,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_101),
    kInstruction_add_vs_r3_r3_101,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r0_r0_182),
    kInstruction_add_cs_r0_r0_182,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r0_r0_53),
    kInstruction_add_vs_r0_r0_53,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r7_r7_142),
    kInstruction_add_hi_r7_r7_142,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r5_r5_89),
    kInstruction_add_ls_r5_r5_89,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r5_r5_32),
    kInstruction_add_ls_r5_r5_32,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r1_r1_29),
    kInstruction_add_ls_r1_r1_29,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r3_r3_62),
    kInstruction_add_hi_r3_r3_62,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r3_r3_216),
    kInstruction_add_eq_r3_r3_216,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r5_r5_167),
    kInstruction_add_mi_r5_r5_167,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r6_88),
    kInstruction_add_ge_r6_r6_88,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r1_r1_158),
    kInstruction_add_lt_r1_r1_158,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r5_r5_242),
    kInstruction_add_ge_r5_r5_242,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r4_r4_225),
    kInstruction_add_ls_r4_r4_225,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r0_r0_4),
    kInstruction_add_eq_r0_r0_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r1_98),
    kInstruction_add_vs_r1_r1_98,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r7_r7_224),
    kInstruction_add_lt_r7_r7_224,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r5_r5_162),
    kInstruction_add_eq_r5_r5_162,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r5_r5_139),
    kInstruction_add_gt_r5_r5_139,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r4_r4_68),
    kInstruction_add_cc_r4_r4_68,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r0_r0_2),
    kInstruction_add_vc_r0_r0_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r3_r3_114),
    kInstruction_add_ls_r3_r3_114,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r7_r7_220),
    kInstruction_add_vs_r7_r7_220,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r6_r6_78),
    kInstruction_add_cs_r6_r6_78,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r7_r7_129),
    kInstruction_add_cs_r7_r7_129,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r3_r3_17),
    kInstruction_add_hi_r3_r3_17,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r0_r0_14),
    kInstruction_add_cs_r0_r0_14,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r3_r3_20),
    kInstruction_add_hi_r3_r3_20,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r4_210),
    kInstruction_add_ne_r4_r4_210,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r3_r3_172),
    kInstruction_add_eq_r3_r3_172,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r3_r3_203),
    kInstruction_add_vc_r3_r3_203,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r3_r3_52),
    kInstruction_add_hi_r3_r3_52,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r0_r0_248),
    kInstruction_add_eq_r0_r0_248,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r3_r3_43),
    kInstruction_add_lt_r3_r3_43,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r4_r4_233),
    kInstruction_add_eq_r4_r4_233,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r6_r6_158),
    kInstruction_add_eq_r6_r6_158,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r7_r7_194),
    kInstruction_add_vs_r7_r7_194,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r5_r5_116),
    kInstruction_add_hi_r5_r5_116,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r7_r7_11),
    kInstruction_add_vc_r7_r7_11,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r4_r4_0),
    kInstruction_add_le_r4_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r1_r1_103),
    kInstruction_add_ge_r1_r1_103,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r0_r0_124),
    kInstruction_add_eq_r0_r0_124,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r5_r5_57),
    kInstruction_add_mi_r5_r5_57,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r7_r7_22),
    kInstruction_add_ge_r7_r7_22,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r6_r6_99),
    kInstruction_add_mi_r6_r6_99,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r7_r7_195),
    kInstruction_add_hi_r7_r7_195,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r4_r4_142),
    kInstruction_add_vs_r4_r4_142,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r6_r6_74),
    kInstruction_add_hi_r6_r6_74,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r1_r1_131),
    kInstruction_add_ne_r1_r1_131,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r1_r1_96),
    kInstruction_add_hi_r1_r1_96,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r2_r2_248),
    kInstruction_add_eq_r2_r2_248,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r2_r2_252),
    kInstruction_add_le_r2_r2_252,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r2_r2_93),
    kInstruction_add_cc_r2_r2_93,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r3_r3_130),
    kInstruction_add_cc_r3_r3_130,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r0_r0_0),
    kInstruction_add_pl_r0_r0_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r3_r3_109),
    kInstruction_add_gt_r3_r3_109,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r1_192),
    kInstruction_add_vs_r1_r1_192,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r3_r3_53),
    kInstruction_add_gt_r3_r3_53,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r6_r6_229),
    kInstruction_add_lt_r6_r6_229,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r4_r4_128),
    kInstruction_add_lt_r4_r4_128,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r2_r2_168),
    kInstruction_add_ne_r2_r2_168,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r7_r7_91),
    kInstruction_add_cs_r7_r7_91,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r5_r5_112),
    kInstruction_add_mi_r5_r5_112,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r0_r0_200),
    kInstruction_add_vc_r0_r0_200,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r5_82),
    kInstruction_add_cs_r5_r5_82,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r4_r4_176),
    kInstruction_add_mi_r4_r4_176,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r2_r2_202),
    kInstruction_add_ls_r2_r2_202,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r1_r1_106),
    kInstruction_add_mi_r1_r1_106,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r4_r4_144),
    kInstruction_add_cc_r4_r4_144,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r4_r4_241),
    kInstruction_add_cc_r4_r4_241,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r3_r3_107),
    kInstruction_add_vc_r3_r3_107,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r2_r2_21),
    kInstruction_add_ls_r2_r2_21,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r2_r2_54),
    kInstruction_add_mi_r2_r2_54,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r1_r1_69),
    kInstruction_add_ge_r1_r1_69,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r2_r2_165),
    kInstruction_add_pl_r2_r2_165,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r2_r2_154),
    kInstruction_add_gt_r2_r2_154,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r4_r4_195),
    kInstruction_add_gt_r4_r4_195,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r0_223),
    kInstruction_add_ne_r0_r0_223,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r6_r6_79),
    kInstruction_add_vs_r6_r6_79,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r4_r4_57),
    kInstruction_add_lt_r4_r4_57,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r0_r0_157),
    kInstruction_add_vs_r0_r0_157,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r2_69),
    kInstruction_add_vc_r2_r2_69,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r7_r7_167),
    kInstruction_add_vc_r7_r7_167,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r4_r4_216),
    kInstruction_add_lt_r4_r4_216,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r1_r1_159),
    kInstruction_add_le_r1_r1_159,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r6_r6_193),
    kInstruction_add_hi_r6_r6_193,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r7_r7_194),
    kInstruction_add_gt_r7_r7_194,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r1_r1_39),
    kInstruction_add_pl_r1_r1_39,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r5_215),
    kInstruction_add_lt_r5_r5_215,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r1_r1_11),
    kInstruction_add_vc_r1_r1_11,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r0_242),
    kInstruction_add_le_r0_r0_242,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r3_r3_215),
    kInstruction_add_eq_r3_r3_215,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_37),
    kInstruction_add_vs_r3_r3_37,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r5_78),
    kInstruction_add_cs_r5_r5_78,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r1_r1_158),
    kInstruction_add_ne_r1_r1_158,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r0_174),
    kInstruction_add_ne_r0_r0_174,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r1_r1_81),
    kInstruction_add_le_r1_r1_81,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r5_r5_250),
    kInstruction_add_le_r5_r5_250,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r7_r7_101),
    kInstruction_add_pl_r7_r7_101,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r7_r7_188),
    kInstruction_add_hi_r7_r7_188,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r4_r4_88),
    kInstruction_add_hi_r4_r4_88,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r1_r1_42),
    kInstruction_add_ls_r1_r1_42,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r2_19),
    kInstruction_add_vc_r2_r2_19,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r6_r6_125),
    kInstruction_add_mi_r6_r6_125,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r2_r2_82),
    kInstruction_add_ls_r2_r2_82,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r4_r4_150),
    kInstruction_add_gt_r4_r4_150,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r5_r5_40),
    kInstruction_add_ne_r5_r5_40,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r4_r4_221),
    kInstruction_add_cs_r4_r4_221,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r6_r6_120),
    kInstruction_add_cc_r6_r6_120,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r2_r2_211),
    kInstruction_add_cc_r2_r2_211,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r6_r6_67),
    kInstruction_add_cs_r6_r6_67,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r0_47),
    kInstruction_add_gt_r0_r0_47,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r7_r7_75),
    kInstruction_add_pl_r7_r7_75,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r1_r1_234),
    kInstruction_add_ls_r1_r1_234,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r2_r2_113),
    kInstruction_add_gt_r2_r2_113,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r2_r2_85),
    kInstruction_add_lt_r2_r2_85,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r6_r6_240),
    kInstruction_add_hi_r6_r6_240,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r3_r3_93),
    kInstruction_add_lt_r3_r3_93,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r4_r4_192),
    kInstruction_add_lt_r4_r4_192,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r1_r1_148),
    kInstruction_add_gt_r1_r1_148,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r2_122),
    kInstruction_add_cs_r2_r2_122,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r4_111),
    kInstruction_add_ne_r4_r4_111,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r2_47),
    kInstruction_add_cs_r2_r2_47,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r4_r4_186),
    kInstruction_add_cs_r4_r4_186,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r0_183),
    kInstruction_add_le_r0_r0_183,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r6_r6_232),
    kInstruction_add_vs_r6_r6_232,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r3_r3_99),
    kInstruction_add_cc_r3_r3_99,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r4_r4_81),
    kInstruction_add_lt_r4_r4_81,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r3_r3_192),
    kInstruction_add_lt_r3_r3_192,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r5_r5_219),
    kInstruction_add_ls_r5_r5_219,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r4_r4_153),
    kInstruction_add_lt_r4_r4_153,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r6_r6_76),
    kInstruction_add_vs_r6_r6_76,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r7_r7_37),
    kInstruction_add_eq_r7_r7_37,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r6_r6_202),
    kInstruction_add_vs_r6_r6_202,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r2_r2_7),
    kInstruction_add_vs_r2_r2_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_99),
    kInstruction_add_vs_r3_r3_99,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r0_r0_224),
    kInstruction_add_vs_r0_r0_224,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r5_r5_26),
    kInstruction_add_le_r5_r5_26,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r0_r0_92),
    kInstruction_add_eq_r0_r0_92,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r7_r7_142),
    kInstruction_add_cc_r7_r7_142,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r2_r2_20),
    kInstruction_add_ls_r2_r2_20,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r4_r4_0),
    kInstruction_add_hi_r4_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r0_107),
    kInstruction_add_ne_r0_r0_107,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r6_r6_156),
    kInstruction_add_vs_r6_r6_156,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r4_r4_32),
    kInstruction_add_le_r4_r4_32,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r6_184),
    kInstruction_add_vc_r6_r6_184,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r7_r7_215),
    kInstruction_add_cs_r7_r7_215,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r7_r7_69),
    kInstruction_add_gt_r7_r7_69,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r5_r5_87),
    kInstruction_add_le_r5_r5_87,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r7_r7_153),
    kInstruction_add_cs_r7_r7_153,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r0_r0_54),
    kInstruction_add_mi_r0_r0_54,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r1_r1_215),
    kInstruction_add_ne_r1_r1_215,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r2_r2_175),
    kInstruction_add_pl_r2_r2_175,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r5_r5_46),
    kInstruction_add_ne_r5_r5_46,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r3_r3_183),
    kInstruction_add_le_r3_r3_183,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r3_r3_89),
    kInstruction_add_pl_r3_r3_89,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r0_229),
    kInstruction_add_ne_r0_r0_229,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r0_r0_142),
    kInstruction_add_cc_r0_r0_142,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r3_r3_101),
    kInstruction_add_ne_r3_r3_101,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r3_r3_108),
    kInstruction_add_vc_r3_r3_108,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r7_r7_96),
    kInstruction_add_eq_r7_r7_96,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r0_r0_204),
    kInstruction_add_cc_r0_r0_204,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r6_r6_58),
    kInstruction_add_hi_r6_r6_58,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r6_2),
    kInstruction_add_vc_r6_r6_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r0_r0_21),
    kInstruction_add_cc_r0_r0_21,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r0_r0_223),
    kInstruction_add_ge_r0_r0_223,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r3_r3_82),
    kInstruction_add_eq_r3_r3_82,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r3_r3_209),
    kInstruction_add_hi_r3_r3_209,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_89),
    kInstruction_add_vs_r3_r3_89,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r2_r2_19),
    kInstruction_add_ls_r2_r2_19,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r3_r3_127),
    kInstruction_add_ne_r3_r3_127,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r1_r1_87),
    kInstruction_add_cc_r1_r1_87,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r0_36),
    kInstruction_add_lt_r0_r0_36,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r3_r3_217),
    kInstruction_add_ne_r3_r3_217,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r2_r2_92),
    kInstruction_add_hi_r2_r2_92,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_231),
    kInstruction_add_vs_r3_r3_231,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r2_8),
    kInstruction_add_vc_r2_r2_8,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_49),
    kInstruction_add_vs_r3_r3_49,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r3_r3_223),
    kInstruction_add_pl_r3_r3_223,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r3_r3_201),
    kInstruction_add_ls_r3_r3_201,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r7_r7_215),
    kInstruction_add_vc_r7_r7_215,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r4_r4_208),
    kInstruction_add_ge_r4_r4_208,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r4_r4_170),
    kInstruction_add_eq_r4_r4_170,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r0_r0_45),
    kInstruction_add_vs_r0_r0_45,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r4_r4_237),
    kInstruction_add_mi_r4_r4_237,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r5_r5_142),
    kInstruction_add_eq_r5_r5_142,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r4_r4_100),
    kInstruction_add_eq_r4_r4_100,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r1_r1_73),
    kInstruction_add_mi_r1_r1_73,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r0_244),
    kInstruction_add_lt_r0_r0_244,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r7_r7_233),
    kInstruction_add_ge_r7_r7_233,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_253),
    kInstruction_add_vs_r3_r3_253,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r5_20),
    kInstruction_add_cs_r5_r5_20,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r2_r2_84),
    kInstruction_add_pl_r2_r2_84,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r4_90),
    kInstruction_add_ne_r4_r4_90,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r6_r6_32),
    kInstruction_add_ne_r6_r6_32,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r6_r6_55),
    kInstruction_add_gt_r6_r6_55,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r2_r2_95),
    kInstruction_add_pl_r2_r2_95,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r4_r4_67),
    kInstruction_add_vc_r4_r4_67,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r3_r3_236),
    kInstruction_add_vc_r3_r3_236,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r7_r7_109),
    kInstruction_add_mi_r7_r7_109,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r0_r0_174),
    kInstruction_add_hi_r0_r0_174,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r3_r3_6),
    kInstruction_add_ne_r3_r3_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r4_r4_17),
    kInstruction_add_mi_r4_r4_17,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r3_r3_20),
    kInstruction_add_ls_r3_r3_20,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r7_r7_93),
    kInstruction_add_mi_r7_r7_93,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r5_r5_9),
    kInstruction_add_eq_r5_r5_9,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r4_r4_88),
    kInstruction_add_cs_r4_r4_88,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r4_r4_92),
    kInstruction_add_cs_r4_r4_92,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r4_r4_69),
    kInstruction_add_gt_r4_r4_69,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r0_r0_219),
    kInstruction_add_ge_r0_r0_219,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r4_r4_15),
    kInstruction_add_gt_r4_r4_15,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r5_r5_105),
    kInstruction_add_gt_r5_r5_105,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r2_r2_229),
    kInstruction_add_hi_r2_r2_229,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r7_r7_32),
    kInstruction_add_ls_r7_r7_32,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r5_r5_79),
    kInstruction_add_gt_r5_r5_79,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r7_r7_57),
    kInstruction_add_ls_r7_r7_57,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r2_r2_17),
    kInstruction_add_pl_r2_r2_17,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r6_r6_67),
    kInstruction_add_cc_r6_r6_67,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r7_r7_11),
    kInstruction_add_cs_r7_r7_11,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r6_r6_150),
    kInstruction_add_mi_r6_r6_150,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r3_r3_241),
    kInstruction_add_pl_r3_r3_241,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r2_r2_65),
    kInstruction_add_cc_r2_r2_65,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r4_81),
    kInstruction_add_ne_r4_r4_81,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r4_r4_176),
    kInstruction_add_lt_r4_r4_176,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r7_r7_58),
    kInstruction_add_cs_r7_r7_58,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r1_r1_114),
    kInstruction_add_gt_r1_r1_114,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r7_r7_4),
    kInstruction_add_vc_r7_r7_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r3_r3_110),
    kInstruction_add_le_r3_r3_110,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r1_149),
    kInstruction_add_vs_r1_r1_149,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r3_r3_179),
    kInstruction_add_cs_r3_r3_179,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r7_r7_199),
    kInstruction_add_vs_r7_r7_199,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r4_r4_46),
    kInstruction_add_ge_r4_r4_46,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r1_r1_98),
    kInstruction_add_ls_r1_r1_98,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r3_r3_60),
    kInstruction_add_cs_r3_r3_60,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r2_201),
    kInstruction_add_vc_r2_r2_201,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r5_r5_128),
    kInstruction_add_le_r5_r5_128,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r4_r4_39),
    kInstruction_add_vs_r4_r4_39,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r4_r4_106),
    kInstruction_add_ls_r4_r4_106,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r5_r5_207),
    kInstruction_add_le_r5_r5_207,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r6_r6_7),
    kInstruction_add_cs_r6_r6_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r5_r5_133),
    kInstruction_add_ne_r5_r5_133,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r7_r7_28),
    kInstruction_add_eq_r7_r7_28,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r7_r7_154),
    kInstruction_add_pl_r7_r7_154,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_242),
    kInstruction_add_vs_r3_r3_242,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r6_r6_221),
    kInstruction_add_eq_r6_r6_221,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r5_r5_97),
    kInstruction_add_vs_r5_r5_97,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r5_r5_70),
    kInstruction_add_ls_r5_r5_70,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r6_236),
    kInstruction_add_vc_r6_r6_236,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r2_r2_236),
    kInstruction_add_ne_r2_r2_236,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r1_r1_198),
    kInstruction_add_eq_r1_r1_198,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_134),
    kInstruction_add_vs_r3_r3_134,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r5_r5_42),
    kInstruction_add_ne_r5_r5_42,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r6_223),
    kInstruction_add_ge_r6_r6_223,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r0_r0_48),
    kInstruction_add_ge_r0_r0_48,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r7_r7_146),
    kInstruction_add_hi_r7_r7_146,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r4_r4_175),
    kInstruction_add_ge_r4_r4_175,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r4_r4_142),
    kInstruction_add_gt_r4_r4_142,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r6_r6_24),
    kInstruction_add_ne_r6_r6_24,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r2_r2_152),
    kInstruction_add_lt_r2_r2_152,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r2_r2_113),
    kInstruction_add_hi_r2_r2_113,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r4_r4_135),
    kInstruction_add_vc_r4_r4_135,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r4_r4_40),
    kInstruction_add_ls_r4_r4_40,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r2_r2_199),
    kInstruction_add_cc_r2_r2_199,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r6_r6_164),
    kInstruction_add_vs_r6_r6_164,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r3_r3_184),
    kInstruction_add_le_r3_r3_184,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r4_r4_158),
    kInstruction_add_cs_r4_r4_158,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r5_r5_212),
    kInstruction_add_vc_r5_r5_212,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r5_r5_143),
    kInstruction_add_gt_r5_r5_143,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r0_186),
    kInstruction_add_ne_r0_r0_186,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r5_233),
    kInstruction_add_cs_r5_r5_233,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r1_r1_242),
    kInstruction_add_le_r1_r1_242,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r6_70),
    kInstruction_add_ge_r6_r6_70,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r7_r7_234),
    kInstruction_add_cc_r7_r7_234,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r0_r0_191),
    kInstruction_add_ls_r0_r0_191,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r3_r3_236),
    kInstruction_add_pl_r3_r3_236,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r7_r7_27),
    kInstruction_add_cc_r7_r7_27,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r0_r0_169),
    kInstruction_add_vs_r0_r0_169,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r2_r2_193),
    kInstruction_add_vs_r2_r2_193,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r6_r6_198),
    kInstruction_add_lt_r6_r6_198,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r7_r7_137),
    kInstruction_add_ge_r7_r7_137,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r1_183),
    kInstruction_add_vs_r1_r1_183,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r5_180),
    kInstruction_add_lt_r5_r5_180,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r3_r3_22),
    kInstruction_add_le_r3_r3_22,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r7_r7_41),
    kInstruction_add_ls_r7_r7_41,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r6_r6_71),
    kInstruction_add_vs_r6_r6_71,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r1_81),
    kInstruction_add_vs_r1_r1_81,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r0_r0_181),
    kInstruction_add_vc_r0_r0_181,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r0_1),
    kInstruction_add_lt_r0_r0_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r1_r1_182),
    kInstruction_add_le_r1_r1_182,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r1_r1_167),
    kInstruction_add_ls_r1_r1_167,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r7_r7_221),
    kInstruction_add_hi_r7_r7_221,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r1_r1_72),
    kInstruction_add_ge_r1_r1_72,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r7_r7_95),
    kInstruction_add_pl_r7_r7_95,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r3_r3_252),
    kInstruction_add_cs_r3_r3_252,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r3_r3_224),
    kInstruction_add_eq_r3_r3_224,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r5_r5_176),
    kInstruction_add_mi_r5_r5_176,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r0_213),
    kInstruction_add_le_r0_r0_213,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r1_r1_91),
    kInstruction_add_gt_r1_r1_91,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r5_r5_236),
    kInstruction_add_vc_r5_r5_236,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r4_r4_68),
    kInstruction_add_hi_r4_r4_68,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r6_r6_201),
    kInstruction_add_ls_r6_r6_201,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r6_r6_146),
    kInstruction_add_vs_r6_r6_146,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r5_r5_169),
    kInstruction_add_le_r5_r5_169,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r1_r1_88),
    kInstruction_add_pl_r1_r1_88,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r4_r4_26),
    kInstruction_add_cc_r4_r4_26,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r2_r2_228),
    kInstruction_add_gt_r2_r2_228,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r5_r5_175),
    kInstruction_add_vs_r5_r5_175,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r3_r3_167),
    kInstruction_add_ne_r3_r3_167,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r6_21),
    kInstruction_add_ge_r6_r6_21,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r7_r7_114),
    kInstruction_add_eq_r7_r7_114,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r0_r0_26),
    kInstruction_add_cs_r0_r0_26,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r4_r4_29),
    kInstruction_add_pl_r4_r4_29,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r1_r1_71),
    kInstruction_add_le_r1_r1_71,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r0_r0_62),
    kInstruction_add_cc_r0_r0_62,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r4_r4_138),
    kInstruction_add_eq_r4_r4_138,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r4_r4_232),
    kInstruction_add_cc_r4_r4_232,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_111),
    kInstruction_add_vs_r3_r3_111,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r5_r5_80),
    kInstruction_add_ls_r5_r5_80,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r4_r4_102),
    kInstruction_add_hi_r4_r4_102,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r0_r0_160),
    kInstruction_add_vc_r0_r0_160,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r4_r4_81),
    kInstruction_add_eq_r4_r4_81,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r7_r7_147),
    kInstruction_add_le_r7_r7_147,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r5_r5_122),
    kInstruction_add_ls_r5_r5_122,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r3_r3_19),
    kInstruction_add_lt_r3_r3_19,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r1_r1_149),
    kInstruction_add_mi_r1_r1_149,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r1_r1_199),
    kInstruction_add_cc_r1_r1_199,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r3_r3_75),
    kInstruction_add_pl_r3_r3_75,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r6_r6_187),
    kInstruction_add_eq_r6_r6_187,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r3_r3_205),
    kInstruction_add_gt_r3_r3_205,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r1_r1_24),
    kInstruction_add_lt_r1_r1_24,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r6_r6_55),
    kInstruction_add_mi_r6_r6_55,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r3_r3_183),
    kInstruction_add_cc_r3_r3_183,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r3_r3_116),
    kInstruction_add_cc_r3_r3_116,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r3_r3_139),
    kInstruction_add_ne_r3_r3_139,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r2_r2_57),
    kInstruction_add_mi_r2_r2_57,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r2_r2_67),
    kInstruction_add_pl_r2_r2_67,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r1_r1_117),
    kInstruction_add_lt_r1_r1_117,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r6_24),
    kInstruction_add_ge_r6_r6_24,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r2_96),
    kInstruction_add_cs_r2_r2_96,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r5_r5_156),
    kInstruction_add_vc_r5_r5_156,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r0_r0_125),
    kInstruction_add_vs_r0_r0_125,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r7_r7_183),
    kInstruction_add_ls_r7_r7_183,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r1_30),
    kInstruction_add_vs_r1_r1_30,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r1_r1_62),
    kInstruction_add_cs_r1_r1_62,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r1_r1_8),
    kInstruction_add_vc_r1_r1_8,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r4_r4_184),
    kInstruction_add_vc_r4_r4_184,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r1_r1_235),
    kInstruction_add_vc_r1_r1_235,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r2_r2_78),
    kInstruction_add_gt_r2_r2_78,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r2_r2_160),
    kInstruction_add_hi_r2_r2_160,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r3_r3_243),
    kInstruction_add_le_r3_r3_243,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r5_r5_46),
    kInstruction_add_gt_r5_r5_46,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r5_r5_148),
    kInstruction_add_ls_r5_r5_148,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r2_r2_232),
    kInstruction_add_vs_r2_r2_232,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r0_r0_13),
    kInstruction_add_hi_r0_r0_13,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r2_r2_45),
    kInstruction_add_hi_r2_r2_45,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r1_r1_197),
    kInstruction_add_mi_r1_r1_197,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r7_r7_91),
    kInstruction_add_ls_r7_r7_91,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r2_192),
    kInstruction_add_cs_r2_r2_192,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r4_r4_202),
    kInstruction_add_cs_r4_r4_202,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r6_r6_115),
    kInstruction_add_ne_r6_r6_115,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r7_r7_173),
    kInstruction_add_mi_r7_r7_173,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r1_236),
    kInstruction_add_vs_r1_r1_236,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r3_r3_49),
    kInstruction_add_pl_r3_r3_49,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r1_r1_193),
    kInstruction_add_cs_r1_r1_193,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r4_r4_200),
    kInstruction_add_gt_r4_r4_200,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r4_r4_141),
    kInstruction_add_lt_r4_r4_141,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r6_r6_142),
    kInstruction_add_gt_r6_r6_142,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r2_r2_39),
    kInstruction_add_le_r2_r2_39,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r3_r3_98),
    kInstruction_add_cc_r3_r3_98,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r4_r4_77),
    kInstruction_add_le_r4_r4_77,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r2_r2_140),
    kInstruction_add_ne_r2_r2_140,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r3_r3_117),
    kInstruction_add_cc_r3_r3_117,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r3_r3_242),
    kInstruction_add_gt_r3_r3_242,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r5_r5_90),
    kInstruction_add_ls_r5_r5_90,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r7_r7_16),
    kInstruction_add_gt_r7_r7_16,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r5_r5_131),
    kInstruction_add_vs_r5_r5_131,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r6_145),
    kInstruction_add_vc_r6_r6_145,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r0_94),
    kInstruction_add_lt_r0_r0_94,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_246),
    kInstruction_add_vs_r3_r3_246,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r0_124),
    kInstruction_add_le_r0_r0_124,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r7_r7_60),
    kInstruction_add_eq_r7_r7_60,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r3_r3_244),
    kInstruction_add_mi_r3_r3_244,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r5_r5_110),
    kInstruction_add_ls_r5_r5_110,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r2_r2_168),
    kInstruction_add_lt_r2_r2_168,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r3_r3_217),
    kInstruction_add_cs_r3_r3_217,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r2_r2_155),
    kInstruction_add_lt_r2_r2_155,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r2_r2_43),
    kInstruction_add_gt_r2_r2_43,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r3_r3_247),
    kInstruction_add_pl_r3_r3_247,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r0_r0_193),
    kInstruction_add_mi_r0_r0_193,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r7_r7_22),
    kInstruction_add_pl_r7_r7_22,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r7_r7_222),
    kInstruction_add_vs_r7_r7_222,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r0_205),
    kInstruction_add_le_r0_r0_205,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r4_r4_67),
    kInstruction_add_lt_r4_r4_67,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r1_r1_60),
    kInstruction_add_cc_r1_r1_60,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r4_r4_4),
    kInstruction_add_cc_r4_r4_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r1_r1_22),
    kInstruction_add_vc_r1_r1_22,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r4_r4_94),
    kInstruction_add_vc_r4_r4_94,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r5_r5_180),
    kInstruction_add_ls_r5_r5_180,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r7_r7_87),
    kInstruction_add_cc_r7_r7_87,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r3_r3_175),
    kInstruction_add_ls_r3_r3_175,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r7_r7_222),
    kInstruction_add_ge_r7_r7_222,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r2_r2_166),
    kInstruction_add_ne_r2_r2_166,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r1_r1_191),
    kInstruction_add_ls_r1_r1_191,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r0_r0_20),
    kInstruction_add_vs_r0_r0_20,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r1_r1_253),
    kInstruction_add_hi_r1_r1_253,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r1_r1_2),
    kInstruction_add_eq_r1_r1_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r6_r6_74),
    kInstruction_add_ls_r6_r6_74,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r6_r6_242),
    kInstruction_add_mi_r6_r6_242,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r6_r6_242),
    kInstruction_add_le_r6_r6_242,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r7_r7_110),
    kInstruction_add_cs_r7_r7_110,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r5_r5_23),
    kInstruction_add_ge_r5_r5_23,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r5_87),
    kInstruction_add_lt_r5_r5_87,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r0_91),
    kInstruction_add_lt_r0_r0_91,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r2_r2_68),
    kInstruction_add_cc_r2_r2_68,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r2_r2_171),
    kInstruction_add_vs_r2_r2_171,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r6_r6_118),
    kInstruction_add_pl_r6_r6_118,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r6_r6_232),
    kInstruction_add_gt_r6_r6_232,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r0_r0_255),
    kInstruction_add_ls_r0_r0_255,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r4_r4_142),
    kInstruction_add_mi_r4_r4_142,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r5_198),
    kInstruction_add_lt_r5_r5_198,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r0_114),
    kInstruction_add_lt_r0_r0_114,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r5_129),
    kInstruction_add_lt_r5_r5_129,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r5_r5_145),
    kInstruction_add_pl_r5_r5_145,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r7_r7_43),
    kInstruction_add_hi_r7_r7_43,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r1_67),
    kInstruction_add_vs_r1_r1_67,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r7_r7_78),
    kInstruction_add_vc_r7_r7_78,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r1_r1_56),
    kInstruction_add_cc_r1_r1_56,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r0_r0_12),
    kInstruction_add_vs_r0_r0_12,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r5_r5_35),
    kInstruction_add_pl_r5_r5_35,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r5_r5_105),
    kInstruction_add_vs_r5_r5_105,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r1_r1_88),
    kInstruction_add_vc_r1_r1_88,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r2_r2_69),
    kInstruction_add_lt_r2_r2_69,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r3_r3_255),
    kInstruction_add_cs_r3_r3_255,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r6_r6_5),
    kInstruction_add_ls_r6_r6_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r6_171),
    kInstruction_add_vc_r6_r6_171,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r5_r5_166),
    kInstruction_add_pl_r5_r5_166,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r1_r1_121),
    kInstruction_add_ls_r1_r1_121,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r4_120),
    kInstruction_add_ne_r4_r4_120,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r4_r4_130),
    kInstruction_add_ls_r4_r4_130,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r6_104),
    kInstruction_add_ge_r6_r6_104,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r3_r3_227),
    kInstruction_add_ge_r3_r3_227,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_202),
    kInstruction_add_vs_r3_r3_202,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r4_198),
    kInstruction_add_ne_r4_r4_198,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r5_r5_78),
    kInstruction_add_cc_r5_r5_78,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r3_r3_7),
    kInstruction_add_cs_r3_r3_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r0_r0_35),
    kInstruction_add_mi_r0_r0_35,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r3_r3_53),
    kInstruction_add_hi_r3_r3_53,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r5_150),
    kInstruction_add_cs_r5_r5_150,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r2_r2_143),
    kInstruction_add_ge_r2_r2_143,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r0_r0_229),
    kInstruction_add_hi_r0_r0_229,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r6_141),
    kInstruction_add_vc_r6_r6_141,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r7_r7_76),
    kInstruction_add_pl_r7_r7_76,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r5_r5_135),
    kInstruction_add_ne_r5_r5_135,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r5_r5_51),
    kInstruction_add_hi_r5_r5_51,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r6_r6_95),
    kInstruction_add_eq_r6_r6_95,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r6_r6_227),
    kInstruction_add_cs_r6_r6_227,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r6_46),
    kInstruction_add_ge_r6_r6_46,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r1_r1_150),
    kInstruction_add_ne_r1_r1_150,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r1_r1_22),
    kInstruction_add_eq_r1_r1_22,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r2_r2_220),
    kInstruction_add_vs_r2_r2_220,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r6_r6_240),
    kInstruction_add_ls_r6_r6_240,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r4_r4_43),
    kInstruction_add_le_r4_r4_43,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r7_r7_243),
    kInstruction_add_ls_r7_r7_243,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r3_r3_191),
    kInstruction_add_hi_r3_r3_191,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r0_r0_235),
    kInstruction_add_hi_r0_r0_235,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r1_r1_59),
    kInstruction_add_gt_r1_r1_59,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r3_r3_165),
    kInstruction_add_vc_r3_r3_165,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r1_r1_46),
    kInstruction_add_vc_r1_r1_46,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r6_r6_232),
    kInstruction_add_eq_r6_r6_232,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r3_r3_132),
    kInstruction_add_ls_r3_r3_132,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r7_r7_207),
    kInstruction_add_ls_r7_r7_207,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r7_r7_129),
    kInstruction_add_vs_r7_r7_129,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r3_r3_41),
    kInstruction_add_ne_r3_r3_41,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r3_r3_153),
    kInstruction_add_ls_r3_r3_153,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r4_r4_34),
    kInstruction_add_vc_r4_r4_34,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r7_r7_140),
    kInstruction_add_cc_r7_r7_140,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r0_108),
    kInstruction_add_ne_r0_r0_108,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r7_r7_44),
    kInstruction_add_ls_r7_r7_44,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r6_r6_128),
    kInstruction_add_cs_r6_r6_128,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r3_r3_195),
    kInstruction_add_ne_r3_r3_195,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r1_r1_239),
    kInstruction_add_hi_r1_r1_239,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r3_r3_243),
    kInstruction_add_cc_r3_r3_243,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r1_r1_30),
    kInstruction_add_pl_r1_r1_30,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r4_r4_48),
    kInstruction_add_lt_r4_r4_48,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r5_r5_92),
    kInstruction_add_gt_r5_r5_92,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r6_r6_167),
    kInstruction_add_pl_r6_r6_167,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r0_8),
    kInstruction_add_lt_r0_r0_8,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r4_r4_49),
    kInstruction_add_vc_r4_r4_49,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r3_r3_109),
    kInstruction_add_mi_r3_r3_109,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r7_r7_58),
    kInstruction_add_lt_r7_r7_58,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r6_r6_54),
    kInstruction_add_lt_r6_r6_54,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r7_r7_36),
    kInstruction_add_eq_r7_r7_36,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r0_r0_230),
    kInstruction_add_ls_r0_r0_230,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r7_r7_133),
    kInstruction_add_eq_r7_r7_133,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r0_r0_153),
    kInstruction_add_hi_r0_r0_153,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r6_r6_154),
    kInstruction_add_cs_r6_r6_154,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r6_r6_178),
    kInstruction_add_eq_r6_r6_178,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r0_r0_206),
    kInstruction_add_hi_r0_r0_206,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r4_r4_164),
    kInstruction_add_ls_r4_r4_164,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r3_r3_60),
    kInstruction_add_lt_r3_r3_60,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r5_44),
    kInstruction_add_cs_r5_r5_44,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r5_r5_158),
    kInstruction_add_le_r5_r5_158,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r3_r3_40),
    kInstruction_add_lt_r3_r3_40,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r0_53),
    kInstruction_add_le_r0_r0_53,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r3_r3_189),
    kInstruction_add_le_r3_r3_189,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_8),
    kInstruction_add_vs_r3_r3_8,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r1_141),
    kInstruction_add_vs_r1_r1_141,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_141),
    kInstruction_add_vs_r3_r3_141,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r2_r2_141),
    kInstruction_add_vs_r2_r2_141,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r6_r6_254),
    kInstruction_add_ls_r6_r6_254,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r3_r3_142),
    kInstruction_add_ne_r3_r3_142,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r0_r0_167),
    kInstruction_add_hi_r0_r0_167,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r6_r6_84),
    kInstruction_add_cs_r6_r6_84,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r2_234),
    kInstruction_add_cs_r2_r2_234,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r6_r6_59),
    kInstruction_add_le_r6_r6_59,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r2_r2_135),
    kInstruction_add_hi_r2_r2_135,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r7_r7_47),
    kInstruction_add_gt_r7_r7_47,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r6_r6_122),
    kInstruction_add_eq_r6_r6_122,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r7_r7_118),
    kInstruction_add_vc_r7_r7_118,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r2_180),
    kInstruction_add_cs_r2_r2_180,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r2_181),
    kInstruction_add_cs_r2_r2_181,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r1_r1_190),
    kInstruction_add_cc_r1_r1_190,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r5_r5_205),
    kInstruction_add_ge_r5_r5_205,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r7_r7_26),
    kInstruction_add_gt_r7_r7_26,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r4_r4_70),
    kInstruction_add_gt_r4_r4_70,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r5_r5_102),
    kInstruction_add_eq_r5_r5_102,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r0_r0_57),
    kInstruction_add_ls_r0_r0_57,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r1_r1_229),
    kInstruction_add_pl_r1_r1_229,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r2_r2_185),
    kInstruction_add_ge_r2_r2_185,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r7_r7_80),
    kInstruction_add_ge_r7_r7_80,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r6_r6_123),
    kInstruction_add_cs_r6_r6_123,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r4_r4_137),
    kInstruction_add_pl_r4_r4_137,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r1_r1_137),
    kInstruction_add_cc_r1_r1_137,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r1_r1_67),
    kInstruction_add_cs_r1_r1_67,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r6_207),
    kInstruction_add_vc_r6_r6_207,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r0_r0_50),
    kInstruction_add_cc_r0_r0_50,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r3_r3_72),
    kInstruction_add_mi_r3_r3_72,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r3_r3_31),
    kInstruction_add_ls_r3_r3_31,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r2_253),
    kInstruction_add_vc_r2_r2_253,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r3_r3_155),
    kInstruction_add_vc_r3_r3_155,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r0_115),
    kInstruction_add_lt_r0_r0_115,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r5_r5_236),
    kInstruction_add_mi_r5_r5_236,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r5_r5_151),
    kInstruction_add_eq_r5_r5_151,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r5_r5_11),
    kInstruction_add_eq_r5_r5_11,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r5_r5_119),
    kInstruction_add_pl_r5_r5_119,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r3_r3_192),
    kInstruction_add_cc_r3_r3_192,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r7_r7_0),
    kInstruction_add_vc_r7_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r1_r1_163),
    kInstruction_add_hi_r1_r1_163,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r2_r2_136),
    kInstruction_add_hi_r2_r2_136,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r5_r5_235),
    kInstruction_add_cc_r5_r5_235,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r5_r5_96),
    kInstruction_add_pl_r5_r5_96,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r3_r3_163),
    kInstruction_add_ge_r3_r3_163,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r5_239),
    kInstruction_add_cs_r5_r5_239,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r5_r5_27),
    kInstruction_add_vs_r5_r5_27,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r6_r6_65),
    kInstruction_add_le_r6_r6_65,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r0_169),
    kInstruction_add_gt_r0_r0_169,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r7_r7_190),
    kInstruction_add_ls_r7_r7_190,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r5_205),
    kInstruction_add_lt_r5_r5_205,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r2_r2_253),
    kInstruction_add_ne_r2_r2_253,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r1_r1_224),
    kInstruction_add_cs_r1_r1_224,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r1_85),
    kInstruction_add_vs_r1_r1_85,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r0_31),
    kInstruction_add_le_r0_r0_31,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r2_r2_7),
    kInstruction_add_lt_r2_r2_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r0_161),
    kInstruction_add_ne_r0_r0_161,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r5_r5_46),
    kInstruction_add_le_r5_r5_46,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r0_r0_235),
    kInstruction_add_pl_r0_r0_235,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r6_r6_113),
    kInstruction_add_eq_r6_r6_113,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r6_r6_45),
    kInstruction_add_mi_r6_r6_45,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r0_r0_62),
    kInstruction_add_cs_r0_r0_62,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r2_r2_225),
    kInstruction_add_ge_r2_r2_225,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r7_r7_47),
    kInstruction_add_vc_r7_r7_47,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r5_r5_125),
    kInstruction_add_vs_r5_r5_125,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r2_r2_184),
    kInstruction_add_ls_r2_r2_184,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r3_r3_113),
    kInstruction_add_ls_r3_r3_113,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r3_r3_126),
    kInstruction_add_gt_r3_r3_126,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r5_r5_101),
    kInstruction_add_gt_r5_r5_101,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r2_r2_147),
    kInstruction_add_lt_r2_r2_147,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r5_r5_60),
    kInstruction_add_le_r5_r5_60,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r3_r3_249),
    kInstruction_add_ne_r3_r3_249,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r3_r3_6),
    kInstruction_add_cs_r3_r3_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r3_r3_65),
    kInstruction_add_vc_r3_r3_65,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r0_r0_137),
    kInstruction_add_ls_r0_r0_137,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r0_r0_148),
    kInstruction_add_ls_r0_r0_148,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r2_128),
    kInstruction_add_cs_r2_r2_128,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r4_r4_38),
    kInstruction_add_eq_r4_r4_38,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r2_r2_57),
    kInstruction_add_ne_r2_r2_57,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r2_r2_125),
    kInstruction_add_ls_r2_r2_125,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r7_r7_139),
    kInstruction_add_lt_r7_r7_139,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r2_r2_213),
    kInstruction_add_eq_r2_r2_213,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r0_r0_7),
    kInstruction_add_vc_r0_r0_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r0_243),
    kInstruction_add_lt_r0_r0_243,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r0_227),
    kInstruction_add_lt_r0_r0_227,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r6_110),
    kInstruction_add_ge_r6_r6_110,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r3_r3_152),
    kInstruction_add_lt_r3_r3_152,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r7_r7_198),
    kInstruction_add_ge_r7_r7_198,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r3_r3_158),
    kInstruction_add_mi_r3_r3_158,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r5_r5_171),
    kInstruction_add_cc_r5_r5_171,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r5_227),
    kInstruction_add_lt_r5_r5_227,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r4_r4_197),
    kInstruction_add_gt_r4_r4_197,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r0_r0_77),
    kInstruction_add_hi_r0_r0_77,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r2_r2_223),
    kInstruction_add_mi_r2_r2_223,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r1_r1_62),
    kInstruction_add_pl_r1_r1_62,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r7_r7_53),
    kInstruction_add_lt_r7_r7_53,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r5_r5_153),
    kInstruction_add_ge_r5_r5_153,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r0_r0_169),
    kInstruction_add_ge_r0_r0_169,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r2_57),
    kInstruction_add_vc_r2_r2_57,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r4_r4_121),
    kInstruction_add_eq_r4_r4_121,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r1_r1_26),
    kInstruction_add_lt_r1_r1_26,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r4_r4_209),
    kInstruction_add_pl_r4_r4_209,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r3_r3_233),
    kInstruction_add_vc_r3_r3_233,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r6_r6_159),
    kInstruction_add_hi_r6_r6_159,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r2_r2_5),
    kInstruction_add_pl_r2_r2_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r4_227),
    kInstruction_add_ne_r4_r4_227,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r4_r4_116),
    kInstruction_add_cc_r4_r4_116,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r3_r3_90),
    kInstruction_add_cs_r3_r3_90,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r5_178),
    kInstruction_add_lt_r5_r5_178,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r6_r6_144),
    kInstruction_add_cs_r6_r6_144,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r2_255),
    kInstruction_add_cs_r2_r2_255,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r2_r2_207),
    kInstruction_add_mi_r2_r2_207,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r4_116),
    kInstruction_add_ne_r4_r4_116,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r4_r4_215),
    kInstruction_add_pl_r4_r4_215,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r5_24),
    kInstruction_add_cs_r5_r5_24,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r4_r4_206),
    kInstruction_add_vc_r4_r4_206,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r4_r4_124),
    kInstruction_add_ls_r4_r4_124,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r4_r4_169),
    kInstruction_add_cs_r4_r4_169,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r7_r7_168),
    kInstruction_add_ge_r7_r7_168,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r5_r5_36),
    kInstruction_add_gt_r5_r5_36,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r5_r5_188),
    kInstruction_add_le_r5_r5_188,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r6_r6_157),
    kInstruction_add_ne_r6_r6_157,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r1_r1_236),
    kInstruction_add_eq_r1_r1_236,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r1_r1_49),
    kInstruction_add_ls_r1_r1_49,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_127),
    kInstruction_add_vs_r3_r3_127,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r1_r1_232),
    kInstruction_add_le_r1_r1_232,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r3_r3_145),
    kInstruction_add_ge_r3_r3_145,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r0_187),
    kInstruction_add_ne_r0_r0_187,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r0_30),
    kInstruction_add_lt_r0_r0_30,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r7_r7_236),
    kInstruction_add_mi_r7_r7_236,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r3_r3_2),
    kInstruction_add_mi_r3_r3_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r0_r0_192),
    kInstruction_add_pl_r0_r0_192,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r0_214),
    kInstruction_add_ne_r0_r0_214,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r5_r5_224),
    kInstruction_add_eq_r5_r5_224,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r7_r7_157),
    kInstruction_add_cs_r7_r7_157,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r1_r1_108),
    kInstruction_add_vc_r1_r1_108,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r3_r3_227),
    kInstruction_add_eq_r3_r3_227,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r0_r0_185),
    kInstruction_add_pl_r0_r0_185,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r1_r1_42),
    kInstruction_add_cs_r1_r1_42,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r1_r1_46),
    kInstruction_add_cs_r1_r1_46,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r5_r5_132),
    kInstruction_add_mi_r5_r5_132,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r3_r3_187),
    kInstruction_add_ne_r3_r3_187,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r2_r2_26),
    kInstruction_add_hi_r2_r2_26,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r5_135),
    kInstruction_add_cs_r5_r5_135,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r2_125),
    kInstruction_add_vc_r2_r2_125,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r1_r1_74),
    kInstruction_add_lt_r1_r1_74,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r7_r7_83),
    kInstruction_add_mi_r7_r7_83,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r5_168),
    kInstruction_add_cs_r5_r5_168,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r3_r3_207),
    kInstruction_add_eq_r3_r3_207,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r0_r0_44),
    kInstruction_add_vc_r0_r0_44,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r0_182),
    kInstruction_add_ne_r0_r0_182,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r2_r2_206),
    kInstruction_add_cc_r2_r2_206,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r5_r5_121),
    kInstruction_add_le_r5_r5_121,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r4_r4_133),
    kInstruction_add_mi_r4_r4_133,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r2_r2_73),
    kInstruction_add_le_r2_r2_73,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r6_r6_77),
    kInstruction_add_gt_r6_r6_77,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r6_r6_222),
    kInstruction_add_hi_r6_r6_222,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r4_r4_113),
    kInstruction_add_vs_r4_r4_113,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r1_r1_225),
    kInstruction_add_cs_r1_r1_225,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r5_r5_143),
    kInstruction_add_cc_r5_r5_143,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r0_r0_117),
    kInstruction_add_vs_r0_r0_117,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r7_r7_191),
    kInstruction_add_vc_r7_r7_191,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_164),
    kInstruction_add_vs_r3_r3_164,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r0_185),
    kInstruction_add_gt_r0_r0_185,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r7_r7_33),
    kInstruction_add_gt_r7_r7_33,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r1_r1_137),
    kInstruction_add_le_r1_r1_137,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r7_r7_192),
    kInstruction_add_mi_r7_r7_192,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r0_82),
    kInstruction_add_ne_r0_r0_82,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r3_r3_55),
    kInstruction_add_mi_r3_r3_55,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r5_r5_183),
    kInstruction_add_vc_r5_r5_183,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r5_22),
    kInstruction_add_cs_r5_r5_22,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r3_r3_136),
    kInstruction_add_cs_r3_r3_136,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r5_18),
    kInstruction_add_lt_r5_r5_18,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r5_154),
    kInstruction_add_lt_r5_r5_154,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r4_r4_197),
    kInstruction_add_le_r4_r4_197,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r1_r1_185),
    kInstruction_add_lt_r1_r1_185,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r5_43),
    kInstruction_add_cs_r5_r5_43,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r0_108),
    kInstruction_add_le_r0_r0_108,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r1_r1_227),
    kInstruction_add_mi_r1_r1_227,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r0_r0_69),
    kInstruction_add_cc_r0_r0_69,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r4_r4_222),
    kInstruction_add_le_r4_r4_222,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r2_86),
    kInstruction_add_cs_r2_r2_86,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_10),
    kInstruction_add_vs_r3_r3_10,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r5_r5_247),
    kInstruction_add_mi_r5_r5_247,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r0_241),
    kInstruction_add_gt_r0_r0_241,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_109),
    kInstruction_add_vs_r3_r3_109,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r1_r1_102),
    kInstruction_add_ls_r1_r1_102,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r2_87),
    kInstruction_add_cs_r2_r2_87,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r0_253),
    kInstruction_add_lt_r0_r0_253,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r1_r1_56),
    kInstruction_add_eq_r1_r1_56,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r7_r7_147),
    kInstruction_add_vs_r7_r7_147,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r5_r5_10),
    kInstruction_add_vs_r5_r5_10,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r1_r1_226),
    kInstruction_add_lt_r1_r1_226,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r1_r1_102),
    kInstruction_add_gt_r1_r1_102,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r7_r7_66),
    kInstruction_add_ls_r7_r7_66,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r5_179),
    kInstruction_add_cs_r5_r5_179,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r5_r5_76),
    kInstruction_add_eq_r5_r5_76,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r2_r2_217),
    kInstruction_add_ls_r2_r2_217,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r1_r1_82),
    kInstruction_add_pl_r1_r1_82,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r0_r0_66),
    kInstruction_add_cs_r0_r0_66,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r4_69),
    kInstruction_add_ne_r4_r4_69,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r7_r7_164),
    kInstruction_add_cs_r7_r7_164,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r7_r7_159),
    kInstruction_add_cs_r7_r7_159,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r6_123),
    kInstruction_add_ge_r6_r6_123,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r6_3),
    kInstruction_add_ge_r6_r6_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r0_160),
    kInstruction_add_lt_r0_r0_160,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r4_r4_228),
    kInstruction_add_gt_r4_r4_228,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r5_r5_156),
    kInstruction_add_hi_r5_r5_156,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r0_191),
    kInstruction_add_gt_r0_r0_191,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r1_r1_178),
    kInstruction_add_pl_r1_r1_178,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r0_r0_243),
    kInstruction_add_vs_r0_r0_243,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r4_r4_217),
    kInstruction_add_lt_r4_r4_217,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r3_r3_90),
    kInstruction_add_hi_r3_r3_90,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r7_r7_25),
    kInstruction_add_eq_r7_r7_25,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r2_212),
    kInstruction_add_vc_r2_r2_212,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r0_r0_252),
    kInstruction_add_pl_r0_r0_252,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r4_r4_236),
    kInstruction_add_cs_r4_r4_236,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r0_r0_105),
    kInstruction_add_cc_r0_r0_105,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r3_r3_134),
    kInstruction_add_mi_r3_r3_134,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r4_r4_116),
    kInstruction_add_pl_r4_r4_116,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r5_223),
    kInstruction_add_lt_r5_r5_223,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r2_r2_242),
    kInstruction_add_vs_r2_r2_242,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r4_r4_150),
    kInstruction_add_cc_r4_r4_150,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r4_126),
    kInstruction_add_ne_r4_r4_126,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r4_r4_21),
    kInstruction_add_le_r4_r4_21,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r2_r2_159),
    kInstruction_add_gt_r2_r2_159,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r5_r5_139),
    kInstruction_add_vs_r5_r5_139,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r2_r2_46),
    kInstruction_add_eq_r2_r2_46,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r4_r4_68),
    kInstruction_add_mi_r4_r4_68,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r4_r4_112),
    kInstruction_add_mi_r4_r4_112,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r5_r5_186),
    kInstruction_add_vs_r5_r5_186,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r5_r5_216),
    kInstruction_add_cc_r5_r5_216,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r4_r4_12),
    kInstruction_add_lt_r4_r4_12,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r1_r1_8),
    kInstruction_add_ne_r1_r1_8,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r1_r1_83),
    kInstruction_add_cc_r1_r1_83,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r7_r7_109),
    kInstruction_add_eq_r7_r7_109,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r4_3),
    kInstruction_add_ne_r4_r4_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r5_r5_62),
    kInstruction_add_pl_r5_r5_62,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r1_r1_205),
    kInstruction_add_gt_r1_r1_205,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r2_r2_209),
    kInstruction_add_ls_r2_r2_209,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r6_252),
    kInstruction_add_vc_r6_r6_252,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r1_r1_218),
    kInstruction_add_cs_r1_r1_218,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r1_71),
    kInstruction_add_vs_r1_r1_71,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r2_r2_247),
    kInstruction_add_vs_r2_r2_247,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r0_r0_42),
    kInstruction_add_pl_r0_r0_42,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r7_r7_226),
    kInstruction_add_mi_r7_r7_226,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r4_r4_14),
    kInstruction_add_hi_r4_r4_14,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r7_r7_246),
    kInstruction_add_ls_r7_r7_246,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r4_r4_239),
    kInstruction_add_ge_r4_r4_239,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r1_r1_187),
    kInstruction_add_eq_r1_r1_187,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r4_r4_105),
    kInstruction_add_gt_r4_r4_105,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r5_85),
    kInstruction_add_cs_r5_r5_85,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r4_r4_47),
    kInstruction_add_le_r4_r4_47,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r4_r4_98),
    kInstruction_add_mi_r4_r4_98,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r0_180),
    kInstruction_add_gt_r0_r0_180,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r5_r5_38),
    kInstruction_add_ge_r5_r5_38,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r2_r2_61),
    kInstruction_add_cc_r2_r2_61,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r7_r7_180),
    kInstruction_add_lt_r7_r7_180,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r7_r7_149),
    kInstruction_add_gt_r7_r7_149,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r6_r6_146),
    kInstruction_add_pl_r6_r6_146,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r4_r4_202),
    kInstruction_add_pl_r4_r4_202,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r0_r0_225),
    kInstruction_add_hi_r0_r0_225,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r5_142),
    kInstruction_add_cs_r5_r5_142,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r0_224),
    kInstruction_add_lt_r0_r0_224,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r5_r5_170),
    kInstruction_add_vc_r5_r5_170,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r7_r7_205),
    kInstruction_add_le_r7_r7_205,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r2_228),
    kInstruction_add_cs_r2_r2_228,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r0_r0_96),
    kInstruction_add_cc_r0_r0_96,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r0_r0_134),
    kInstruction_add_vc_r0_r0_134,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r7_r7_5),
    kInstruction_add_le_r7_r7_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r0_33),
    kInstruction_add_ne_r0_r0_33,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r3_r3_162),
    kInstruction_add_vc_r3_r3_162,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r2_r2_111),
    kInstruction_add_mi_r2_r2_111,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r1_210),
    kInstruction_add_vs_r1_r1_210,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r2_r2_244),
    kInstruction_add_mi_r2_r2_244,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r6_r6_208),
    kInstruction_add_cs_r6_r6_208,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r1_r1_213),
    kInstruction_add_gt_r1_r1_213,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r7_r7_63),
    kInstruction_add_ge_r7_r7_63,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r2_r2_214),
    kInstruction_add_mi_r2_r2_214,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r6_r6_224),
    kInstruction_add_ne_r6_r6_224,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r1_r1_169),
    kInstruction_add_vc_r1_r1_169,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r2_228),
    kInstruction_add_vc_r2_r2_228,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r1_r1_161),
    kInstruction_add_ge_r1_r1_161,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r1_r1_250),
    kInstruction_add_vc_r1_r1_250,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r0_r0_214),
    kInstruction_add_cs_r0_r0_214,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r0_104),
    kInstruction_add_lt_r0_r0_104,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r2_210),
    kInstruction_add_cs_r2_r2_210,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r2_r2_229),
    kInstruction_add_mi_r2_r2_229,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r5_55),
    kInstruction_add_cs_r5_r5_55,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r2_r2_17),
    kInstruction_add_lt_r2_r2_17,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r2_r2_169),
    kInstruction_add_ls_r2_r2_169,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r6_74),
    kInstruction_add_vc_r6_r6_74,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r1_r1_242),
    kInstruction_add_ge_r1_r1_242,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r4_r4_234),
    kInstruction_add_vc_r4_r4_234,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r2_r2_147),
    kInstruction_add_eq_r2_r2_147,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r1_r1_101),
    kInstruction_add_vc_r1_r1_101,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r6_r6_226),
    kInstruction_add_pl_r6_r6_226,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r6_r6_184),
    kInstruction_add_gt_r6_r6_184,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r3_r3_177),
    kInstruction_add_gt_r3_r3_177,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r1_r1_126),
    kInstruction_add_ne_r1_r1_126,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r5_r5_222),
    kInstruction_add_ls_r5_r5_222,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r4_r4_2),
    kInstruction_add_hi_r4_r4_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r2_r2_190),
    kInstruction_add_ls_r2_r2_190,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r4_r4_254),
    kInstruction_add_mi_r4_r4_254,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r4_r4_19),
    kInstruction_add_pl_r4_r4_19,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r0_r0_241),
    kInstruction_add_ls_r0_r0_241,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r3_r3_148),
    kInstruction_add_cc_r3_r3_148,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r1_r1_195),
    kInstruction_add_gt_r1_r1_195,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r4_r4_91),
    kInstruction_add_vc_r4_r4_91,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r0_227),
    kInstruction_add_gt_r0_r0_227,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r6_r6_86),
    kInstruction_add_vs_r6_r6_86,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r1_r1_241),
    kInstruction_add_ls_r1_r1_241,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r3_r3_241),
    kInstruction_add_le_r3_r3_241,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r2_r2_217),
    kInstruction_add_vs_r2_r2_217,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r7_r7_33),
    kInstruction_add_ls_r7_r7_33,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r1_r1_76),
    kInstruction_add_ge_r1_r1_76,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r2_r2_34),
    kInstruction_add_lt_r2_r2_34,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r7_r7_91),
    kInstruction_add_gt_r7_r7_91,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r3_r3_121),
    kInstruction_add_le_r3_r3_121,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r0_r0_128),
    kInstruction_add_ge_r0_r0_128,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r6_199),
    kInstruction_add_vc_r6_r6_199,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r6_r6_138),
    kInstruction_add_hi_r6_r6_138,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r4_r4_241),
    kInstruction_add_pl_r4_r4_241,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r5_r5_246),
    kInstruction_add_vc_r5_r5_246,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r5_r5_133),
    kInstruction_add_cc_r5_r5_133,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r5_1),
    kInstruction_add_cs_r5_r5_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r1_r1_233),
    kInstruction_add_mi_r1_r1_233,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r3_r3_195),
    kInstruction_add_cs_r3_r3_195,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r0_r0_13),
    kInstruction_add_pl_r0_r0_13,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r5_r5_187),
    kInstruction_add_eq_r5_r5_187,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r2_r2_74),
    kInstruction_add_ls_r2_r2_74,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r4_r4_33),
    kInstruction_add_ge_r4_r4_33,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r3_r3_253),
    kInstruction_add_hi_r3_r3_253,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r1_r1_185),
    kInstruction_add_pl_r1_r1_185,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r2_r2_103),
    kInstruction_add_gt_r2_r2_103,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r3_r3_255),
    kInstruction_add_ls_r3_r3_255,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r6_r6_207),
    kInstruction_add_vs_r6_r6_207,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r5_255),
    kInstruction_add_cs_r5_r5_255,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r1_r1_107),
    kInstruction_add_lt_r1_r1_107,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r3_r3_122),
    kInstruction_add_le_r3_r3_122,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r0_r0_223),
    kInstruction_add_vc_r0_r0_223,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r7_r7_38),
    kInstruction_add_cc_r7_r7_38,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r4_r4_207),
    kInstruction_add_cc_r4_r4_207,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r3_r3_101),
    kInstruction_add_hi_r3_r3_101,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r0_237),
    kInstruction_add_gt_r0_r0_237,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r6_r6_13),
    kInstruction_add_mi_r6_r6_13,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r1_r1_109),
    kInstruction_add_cs_r1_r1_109,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r4_r4_132),
    kInstruction_add_le_r4_r4_132,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r2_r2_185),
    kInstruction_add_le_r2_r2_185,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r3_r3_220),
    kInstruction_add_lt_r3_r3_220,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r7_r7_248),
    kInstruction_add_eq_r7_r7_248,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r7_r7_146),
    kInstruction_add_le_r7_r7_146,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r1_r1_34),
    kInstruction_add_hi_r1_r1_34,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r3_r3_146),
    kInstruction_add_pl_r3_r3_146,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r6_r6_22),
    kInstruction_add_cs_r6_r6_22,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r2_r2_187),
    kInstruction_add_lt_r2_r2_187,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r4_r4_176),
    kInstruction_add_cc_r4_r4_176,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r3_r3_233),
    kInstruction_add_ge_r3_r3_233,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r2_r2_41),
    kInstruction_add_ge_r2_r2_41,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r0_207),
    kInstruction_add_lt_r0_r0_207,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r0_r0_147),
    kInstruction_add_vc_r0_r0_147,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r4_r4_89),
    kInstruction_add_pl_r4_r4_89,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r2_57),
    kInstruction_add_cs_r2_r2_57,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r0_r0_24),
    kInstruction_add_vc_r0_r0_24,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r6_r6_114),
    kInstruction_add_mi_r6_r6_114,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r4_r4_109),
    kInstruction_add_pl_r4_r4_109,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r7_r7_98),
    kInstruction_add_mi_r7_r7_98,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r5_r5_124),
    kInstruction_add_eq_r5_r5_124,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r3_r3_253),
    kInstruction_add_le_r3_r3_253,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r5_r5_149),
    kInstruction_add_hi_r5_r5_149,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r5_r5_5),
    kInstruction_add_cc_r5_r5_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r1_r1_149),
    kInstruction_add_cs_r1_r1_149,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r3_r3_149),
    kInstruction_add_gt_r3_r3_149,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_20),
    kInstruction_add_vs_r3_r3_20,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r7_r7_241),
    kInstruction_add_lt_r7_r7_241,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r2_42),
    kInstruction_add_cs_r2_r2_42,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r2_r2_122),
    kInstruction_add_vs_r2_r2_122,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r0_203),
    kInstruction_add_ne_r0_r0_203,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r4_r4_80),
    kInstruction_add_cc_r4_r4_80,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r0_162),
    kInstruction_add_gt_r0_r0_162,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r4_r4_155),
    kInstruction_add_gt_r4_r4_155,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r1_r1_93),
    kInstruction_add_mi_r1_r1_93,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r2_r2_94),
    kInstruction_add_gt_r2_r2_94,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r0_r0_87),
    kInstruction_add_eq_r0_r0_87,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r1_r1_141),
    kInstruction_add_pl_r1_r1_141,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r6_r6_42),
    kInstruction_add_cc_r6_r6_42,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r0_r0_178),
    kInstruction_add_vc_r0_r0_178,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r6_r6_109),
    kInstruction_add_pl_r6_r6_109,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r0_r0_186),
    kInstruction_add_mi_r0_r0_186,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r6_r6_116),
    kInstruction_add_gt_r6_r6_116,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r3_r3_249),
    kInstruction_add_vc_r3_r3_249,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r6_r6_197),
    kInstruction_add_vs_r6_r6_197,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r5_r5_176),
    kInstruction_add_hi_r5_r5_176,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r3_r3_10),
    kInstruction_add_ls_r3_r3_10,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r0_r0_120),
    kInstruction_add_ge_r0_r0_120,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r0_r0_197),
    kInstruction_add_ge_r0_r0_197,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r7_r7_149),
    kInstruction_add_ls_r7_r7_149,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r2_r2_219),
    kInstruction_add_mi_r2_r2_219,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r4_r4_130),
    kInstruction_add_lt_r4_r4_130,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r4_r4_220),
    kInstruction_add_pl_r4_r4_220,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r2_r2_101),
    kInstruction_add_gt_r2_r2_101,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r6_r6_111),
    kInstruction_add_pl_r6_r6_111,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r2_r2_21),
    kInstruction_add_hi_r2_r2_21,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r5_174),
    kInstruction_add_lt_r5_r5_174,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r0_r0_190),
    kInstruction_add_hi_r0_r0_190,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r7_r7_166),
    kInstruction_add_ge_r7_r7_166,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r5_r5_251),
    kInstruction_add_vc_r5_r5_251,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r2_r2_174),
    kInstruction_add_ne_r2_r2_174,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r6_r6_255),
    kInstruction_add_eq_r6_r6_255,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r5_r5_186),
    kInstruction_add_ls_r5_r5_186,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_75),
    kInstruction_add_vs_r3_r3_75,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r2_15),
    kInstruction_add_cs_r2_r2_15,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r6_r6_133),
    kInstruction_add_hi_r6_r6_133,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r0_r0_188),
    kInstruction_add_vc_r0_r0_188,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r2_r2_85),
    kInstruction_add_ne_r2_r2_85,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r2_r2_116),
    kInstruction_add_hi_r2_r2_116,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r3_r3_91),
    kInstruction_add_le_r3_r3_91,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r1_r1_222),
    kInstruction_add_pl_r1_r1_222,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r0_r0_33),
    kInstruction_add_mi_r0_r0_33,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_26),
    kInstruction_add_vs_r3_r3_26,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r5_r5_182),
    kInstruction_add_cc_r5_r5_182,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r0_62),
    kInstruction_add_gt_r0_r0_62,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r1_r1_189),
    kInstruction_add_vc_r1_r1_189,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r2_r2_180),
    kInstruction_add_pl_r2_r2_180,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r5_r5_15),
    kInstruction_add_mi_r5_r5_15,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r4_r4_24),
    kInstruction_add_lt_r4_r4_24,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r6_r6_190),
    kInstruction_add_cs_r6_r6_190,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r6_r6_168),
    kInstruction_add_ne_r6_r6_168,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r2_r2_48),
    kInstruction_add_ls_r2_r2_48,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r7_r7_6),
    kInstruction_add_vs_r7_r7_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r4_r4_80),
    kInstruction_add_cs_r4_r4_80,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r3_r3_75),
    kInstruction_add_ls_r3_r3_75,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r2_r2_236),
    kInstruction_add_ge_r2_r2_236,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r1_r1_9),
    kInstruction_add_hi_r1_r1_9,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r1_r1_51),
    kInstruction_add_cs_r1_r1_51,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r1_r1_139),
    kInstruction_add_cc_r1_r1_139,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r2_r2_204),
    kInstruction_add_cc_r2_r2_204,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r2_r2_216),
    kInstruction_add_gt_r2_r2_216,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r7_r7_60),
    kInstruction_add_vc_r7_r7_60,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r6_r6_249),
    kInstruction_add_pl_r6_r6_249,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r1_r1_169),
    kInstruction_add_cs_r1_r1_169,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r6_r6_33),
    kInstruction_add_eq_r6_r6_33,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r0_91),
    kInstruction_add_ne_r0_r0_91,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r4_r4_56),
    kInstruction_add_vc_r4_r4_56,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r5_r5_26),
    kInstruction_add_ge_r5_r5_26,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r6_r6_210),
    kInstruction_add_lt_r6_r6_210,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r6_r6_122),
    kInstruction_add_le_r6_r6_122,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r4_r4_118),
    kInstruction_add_ls_r4_r4_118,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r3_r3_59),
    kInstruction_add_lt_r3_r3_59,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r3_r3_144),
    kInstruction_add_cc_r3_r3_144,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r6_r6_159),
    kInstruction_add_le_r6_r6_159,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r0_25),
    kInstruction_add_lt_r0_r0_25,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r0_r0_150),
    kInstruction_add_vs_r0_r0_150,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r1_r1_91),
    kInstruction_add_ne_r1_r1_91,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r2_r2_65),
    kInstruction_add_ls_r2_r2_65,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r3_r3_80),
    kInstruction_add_lt_r3_r3_80,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r0_r0_37),
    kInstruction_add_eq_r0_r0_37,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r4_71),
    kInstruction_add_ne_r4_r4_71,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r7_r7_169),
    kInstruction_add_ls_r7_r7_169,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r0_r0_141),
    kInstruction_add_cs_r0_r0_141,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r3_r3_190),
    kInstruction_add_pl_r3_r3_190,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r7_r7_53),
    kInstruction_add_pl_r7_r7_53,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r3_r3_60),
    kInstruction_add_vc_r3_r3_60,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r3_r3_104),
    kInstruction_add_ge_r3_r3_104,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r1_r1_93),
    kInstruction_add_pl_r1_r1_93,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r7_r7_234),
    kInstruction_add_pl_r7_r7_234,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r0_226),
    kInstruction_add_lt_r0_r0_226,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r1_r1_207),
    kInstruction_add_mi_r1_r1_207,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r1_r1_152),
    kInstruction_add_le_r1_r1_152,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r6_r6_151),
    kInstruction_add_gt_r6_r6_151,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r4_r4_127),
    kInstruction_add_ge_r4_r4_127,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r1_r1_167),
    kInstruction_add_lt_r1_r1_167,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r2_r2_206),
    kInstruction_add_pl_r2_r2_206,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r1_r1_241),
    kInstruction_add_ge_r1_r1_241,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_156),
    kInstruction_add_vs_r3_r3_156,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r0_125),
    kInstruction_add_gt_r0_r0_125,
  },
};

#endif  // VIXL_ASSEMBLER_COND_RDLOW_RNLOW_OPERAND_IMMEDIATE_IMM8_IN_IT_BLOCK_ADD_T32_H_
