// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_ASSEMBLER_COND_RD_OPERAND_RN_SHIFT_AMOUNT_1TO31_CMN_T32_H_
#define VIXL_ASSEMBLER_COND_RD_OPERAND_RN_SHIFT_AMOUNT_1TO31_CMN_T32_H_

const byte kInstruction_cmn_al_r14_r8_ROR_21[] = {
  0x1e, 0xeb, 0x78, 0x5f // cmn al r14 r8 ROR 21
};
const byte kInstruction_cmn_al_r5_r13_ROR_4[] = {
  0x15, 0xeb, 0x3d, 0x1f // cmn al r5 r13 ROR 4
};
const byte kInstruction_cmn_al_r0_r3_ROR_4[] = {
  0x10, 0xeb, 0x33, 0x1f // cmn al r0 r3 ROR 4
};
const byte kInstruction_cmn_al_r3_r14_ROR_7[] = {
  0x13, 0xeb, 0xfe, 0x1f // cmn al r3 r14 ROR 7
};
const byte kInstruction_cmn_al_r2_r6_LSL_30[] = {
  0x12, 0xeb, 0x86, 0x7f // cmn al r2 r6 LSL 30
};
const byte kInstruction_cmn_al_r11_r4_LSL_26[] = {
  0x1b, 0xeb, 0x84, 0x6f // cmn al r11 r4 LSL 26
};
const byte kInstruction_cmn_al_r7_r0_LSL_30[] = {
  0x17, 0xeb, 0x80, 0x7f // cmn al r7 r0 LSL 30
};
const byte kInstruction_cmn_al_r0_r9_ROR_17[] = {
  0x10, 0xeb, 0x79, 0x4f // cmn al r0 r9 ROR 17
};
const byte kInstruction_cmn_al_r11_r3_ROR_31[] = {
  0x1b, 0xeb, 0xf3, 0x7f // cmn al r11 r3 ROR 31
};
const byte kInstruction_cmn_al_r8_r8_ROR_20[] = {
  0x18, 0xeb, 0x38, 0x5f // cmn al r8 r8 ROR 20
};
const byte kInstruction_cmn_al_r11_r13_LSL_24[] = {
  0x1b, 0xeb, 0x0d, 0x6f // cmn al r11 r13 LSL 24
};
const byte kInstruction_cmn_al_r1_r3_ROR_27[] = {
  0x11, 0xeb, 0xf3, 0x6f // cmn al r1 r3 ROR 27
};
const byte kInstruction_cmn_al_r9_r10_ROR_5[] = {
  0x19, 0xeb, 0x7a, 0x1f // cmn al r9 r10 ROR 5
};
const byte kInstruction_cmn_al_r3_r11_ROR_25[] = {
  0x13, 0xeb, 0x7b, 0x6f // cmn al r3 r11 ROR 25
};
const byte kInstruction_cmn_al_r2_r6_LSL_7[] = {
  0x12, 0xeb, 0xc6, 0x1f // cmn al r2 r6 LSL 7
};
const byte kInstruction_cmn_al_r3_r12_ROR_8[] = {
  0x13, 0xeb, 0x3c, 0x2f // cmn al r3 r12 ROR 8
};
const byte kInstruction_cmn_al_r0_r0_LSL_14[] = {
  0x10, 0xeb, 0x80, 0x3f // cmn al r0 r0 LSL 14
};
const byte kInstruction_cmn_al_r8_r10_ROR_2[] = {
  0x18, 0xeb, 0xba, 0x0f // cmn al r8 r10 ROR 2
};
const byte kInstruction_cmn_al_r0_r8_LSL_31[] = {
  0x10, 0xeb, 0xc8, 0x7f // cmn al r0 r8 LSL 31
};
const byte kInstruction_cmn_al_r1_r14_ROR_27[] = {
  0x11, 0xeb, 0xfe, 0x6f // cmn al r1 r14 ROR 27
};
const byte kInstruction_cmn_al_r14_r2_ROR_27[] = {
  0x1e, 0xeb, 0xf2, 0x6f // cmn al r14 r2 ROR 27
};
const byte kInstruction_cmn_al_r3_r10_LSL_25[] = {
  0x13, 0xeb, 0x4a, 0x6f // cmn al r3 r10 LSL 25
};
const byte kInstruction_cmn_al_r13_r8_ROR_6[] = {
  0x1d, 0xeb, 0xb8, 0x1f // cmn al r13 r8 ROR 6
};
const byte kInstruction_cmn_al_r5_r12_ROR_7[] = {
  0x15, 0xeb, 0xfc, 0x1f // cmn al r5 r12 ROR 7
};
const byte kInstruction_cmn_al_r7_r8_LSL_10[] = {
  0x17, 0xeb, 0x88, 0x2f // cmn al r7 r8 LSL 10
};
const byte kInstruction_cmn_al_r9_r12_ROR_10[] = {
  0x19, 0xeb, 0xbc, 0x2f // cmn al r9 r12 ROR 10
};
const byte kInstruction_cmn_al_r8_r6_ROR_12[] = {
  0x18, 0xeb, 0x36, 0x3f // cmn al r8 r6 ROR 12
};
const byte kInstruction_cmn_al_r8_r10_LSL_7[] = {
  0x18, 0xeb, 0xca, 0x1f // cmn al r8 r10 LSL 7
};
const byte kInstruction_cmn_al_r5_r12_ROR_8[] = {
  0x15, 0xeb, 0x3c, 0x2f // cmn al r5 r12 ROR 8
};
const byte kInstruction_cmn_al_r0_r13_LSL_21[] = {
  0x10, 0xeb, 0x4d, 0x5f // cmn al r0 r13 LSL 21
};
const byte kInstruction_cmn_al_r2_r0_LSL_7[] = {
  0x12, 0xeb, 0xc0, 0x1f // cmn al r2 r0 LSL 7
};
const byte kInstruction_cmn_al_r10_r2_LSL_22[] = {
  0x1a, 0xeb, 0x82, 0x5f // cmn al r10 r2 LSL 22
};
const byte kInstruction_cmn_al_r4_r11_ROR_2[] = {
  0x14, 0xeb, 0xbb, 0x0f // cmn al r4 r11 ROR 2
};
const byte kInstruction_cmn_al_r11_r14_LSL_14[] = {
  0x1b, 0xeb, 0x8e, 0x3f // cmn al r11 r14 LSL 14
};
const byte kInstruction_cmn_al_r8_r0_LSL_23[] = {
  0x18, 0xeb, 0xc0, 0x5f // cmn al r8 r0 LSL 23
};
const byte kInstruction_cmn_al_r10_r4_LSL_12[] = {
  0x1a, 0xeb, 0x04, 0x3f // cmn al r10 r4 LSL 12
};
const byte kInstruction_cmn_al_r4_r5_ROR_28[] = {
  0x14, 0xeb, 0x35, 0x7f // cmn al r4 r5 ROR 28
};
const byte kInstruction_cmn_al_r1_r5_LSL_25[] = {
  0x11, 0xeb, 0x45, 0x6f // cmn al r1 r5 LSL 25
};
const byte kInstruction_cmn_al_r7_r10_ROR_16[] = {
  0x17, 0xeb, 0x3a, 0x4f // cmn al r7 r10 ROR 16
};
const byte kInstruction_cmn_al_r3_r0_LSL_31[] = {
  0x13, 0xeb, 0xc0, 0x7f // cmn al r3 r0 LSL 31
};
const byte kInstruction_cmn_al_r2_r2_LSL_2[] = {
  0x12, 0xeb, 0x82, 0x0f // cmn al r2 r2 LSL 2
};
const byte kInstruction_cmn_al_r8_r1_LSL_9[] = {
  0x18, 0xeb, 0x41, 0x2f // cmn al r8 r1 LSL 9
};
const byte kInstruction_cmn_al_r13_r6_LSL_6[] = {
  0x1d, 0xeb, 0x86, 0x1f // cmn al r13 r6 LSL 6
};
const byte kInstruction_cmn_al_r0_r8_LSL_13[] = {
  0x10, 0xeb, 0x48, 0x3f // cmn al r0 r8 LSL 13
};
const byte kInstruction_cmn_al_r11_r8_ROR_3[] = {
  0x1b, 0xeb, 0xf8, 0x0f // cmn al r11 r8 ROR 3
};
const byte kInstruction_cmn_al_r8_r1_LSL_13[] = {
  0x18, 0xeb, 0x41, 0x3f // cmn al r8 r1 LSL 13
};
const byte kInstruction_cmn_al_r6_r2_LSL_13[] = {
  0x16, 0xeb, 0x42, 0x3f // cmn al r6 r2 LSL 13
};
const byte kInstruction_cmn_al_r4_r1_LSL_16[] = {
  0x14, 0xeb, 0x01, 0x4f // cmn al r4 r1 LSL 16
};
const byte kInstruction_cmn_al_r0_r4_LSL_9[] = {
  0x10, 0xeb, 0x44, 0x2f // cmn al r0 r4 LSL 9
};
const byte kInstruction_cmn_al_r11_r8_ROR_8[] = {
  0x1b, 0xeb, 0x38, 0x2f // cmn al r11 r8 ROR 8
};
const byte kInstruction_cmn_al_r9_r12_LSL_5[] = {
  0x19, 0xeb, 0x4c, 0x1f // cmn al r9 r12 LSL 5
};
const byte kInstruction_cmn_al_r13_r9_ROR_13[] = {
  0x1d, 0xeb, 0x79, 0x3f // cmn al r13 r9 ROR 13
};
const byte kInstruction_cmn_al_r2_r0_ROR_23[] = {
  0x12, 0xeb, 0xf0, 0x5f // cmn al r2 r0 ROR 23
};
const byte kInstruction_cmn_al_r7_r8_ROR_6[] = {
  0x17, 0xeb, 0xb8, 0x1f // cmn al r7 r8 ROR 6
};
const byte kInstruction_cmn_al_r2_r6_LSL_11[] = {
  0x12, 0xeb, 0xc6, 0x2f // cmn al r2 r6 LSL 11
};
const byte kInstruction_cmn_al_r2_r13_LSL_12[] = {
  0x12, 0xeb, 0x0d, 0x3f // cmn al r2 r13 LSL 12
};
const byte kInstruction_cmn_al_r7_r4_LSL_13[] = {
  0x17, 0xeb, 0x44, 0x3f // cmn al r7 r4 LSL 13
};
const byte kInstruction_cmn_al_r8_r4_LSL_1[] = {
  0x18, 0xeb, 0x44, 0x0f // cmn al r8 r4 LSL 1
};
const byte kInstruction_cmn_al_r14_r4_LSL_8[] = {
  0x1e, 0xeb, 0x04, 0x2f // cmn al r14 r4 LSL 8
};
const byte kInstruction_cmn_al_r7_r9_ROR_20[] = {
  0x17, 0xeb, 0x39, 0x5f // cmn al r7 r9 ROR 20
};
const byte kInstruction_cmn_al_r4_r3_ROR_6[] = {
  0x14, 0xeb, 0xb3, 0x1f // cmn al r4 r3 ROR 6
};
const byte kInstruction_cmn_al_r3_r12_LSL_2[] = {
  0x13, 0xeb, 0x8c, 0x0f // cmn al r3 r12 LSL 2
};
const byte kInstruction_cmn_al_r12_r9_ROR_7[] = {
  0x1c, 0xeb, 0xf9, 0x1f // cmn al r12 r9 ROR 7
};
const byte kInstruction_cmn_al_r0_r2_LSL_17[] = {
  0x10, 0xeb, 0x42, 0x4f // cmn al r0 r2 LSL 17
};
const byte kInstruction_cmn_al_r14_r3_LSL_28[] = {
  0x1e, 0xeb, 0x03, 0x7f // cmn al r14 r3 LSL 28
};
const byte kInstruction_cmn_al_r14_r12_ROR_15[] = {
  0x1e, 0xeb, 0xfc, 0x3f // cmn al r14 r12 ROR 15
};
const byte kInstruction_cmn_al_r8_r0_ROR_8[] = {
  0x18, 0xeb, 0x30, 0x2f // cmn al r8 r0 ROR 8
};
const byte kInstruction_cmn_al_r2_r11_LSL_20[] = {
  0x12, 0xeb, 0x0b, 0x5f // cmn al r2 r11 LSL 20
};
const byte kInstruction_cmn_al_r8_r13_LSL_30[] = {
  0x18, 0xeb, 0x8d, 0x7f // cmn al r8 r13 LSL 30
};
const byte kInstruction_cmn_al_r2_r9_LSL_24[] = {
  0x12, 0xeb, 0x09, 0x6f // cmn al r2 r9 LSL 24
};
const byte kInstruction_cmn_al_r12_r1_LSL_2[] = {
  0x1c, 0xeb, 0x81, 0x0f // cmn al r12 r1 LSL 2
};
const byte kInstruction_cmn_al_r7_r1_ROR_7[] = {
  0x17, 0xeb, 0xf1, 0x1f // cmn al r7 r1 ROR 7
};
const byte kInstruction_cmn_al_r11_r7_ROR_30[] = {
  0x1b, 0xeb, 0xb7, 0x7f // cmn al r11 r7 ROR 30
};
const byte kInstruction_cmn_al_r0_r9_ROR_22[] = {
  0x10, 0xeb, 0xb9, 0x5f // cmn al r0 r9 ROR 22
};
const byte kInstruction_cmn_al_r14_r9_ROR_17[] = {
  0x1e, 0xeb, 0x79, 0x4f // cmn al r14 r9 ROR 17
};
const byte kInstruction_cmn_al_r3_r14_LSL_3[] = {
  0x13, 0xeb, 0xce, 0x0f // cmn al r3 r14 LSL 3
};
const byte kInstruction_cmn_al_r0_r0_ROR_30[] = {
  0x10, 0xeb, 0xb0, 0x7f // cmn al r0 r0 ROR 30
};
const byte kInstruction_cmn_al_r9_r8_LSL_4[] = {
  0x19, 0xeb, 0x08, 0x1f // cmn al r9 r8 LSL 4
};
const byte kInstruction_cmn_al_r8_r13_LSL_16[] = {
  0x18, 0xeb, 0x0d, 0x4f // cmn al r8 r13 LSL 16
};
const byte kInstruction_cmn_al_r0_r8_ROR_28[] = {
  0x10, 0xeb, 0x38, 0x7f // cmn al r0 r8 ROR 28
};
const byte kInstruction_cmn_al_r11_r10_LSL_23[] = {
  0x1b, 0xeb, 0xca, 0x5f // cmn al r11 r10 LSL 23
};
const byte kInstruction_cmn_al_r11_r4_LSL_27[] = {
  0x1b, 0xeb, 0xc4, 0x6f // cmn al r11 r4 LSL 27
};
const byte kInstruction_cmn_al_r2_r2_LSL_30[] = {
  0x12, 0xeb, 0x82, 0x7f // cmn al r2 r2 LSL 30
};
const byte kInstruction_cmn_al_r5_r4_LSL_17[] = {
  0x15, 0xeb, 0x44, 0x4f // cmn al r5 r4 LSL 17
};
const byte kInstruction_cmn_al_r11_r12_LSL_1[] = {
  0x1b, 0xeb, 0x4c, 0x0f // cmn al r11 r12 LSL 1
};
const byte kInstruction_cmn_al_r0_r14_ROR_21[] = {
  0x10, 0xeb, 0x7e, 0x5f // cmn al r0 r14 ROR 21
};
const byte kInstruction_cmn_al_r11_r5_ROR_21[] = {
  0x1b, 0xeb, 0x75, 0x5f // cmn al r11 r5 ROR 21
};
const byte kInstruction_cmn_al_r6_r14_LSL_14[] = {
  0x16, 0xeb, 0x8e, 0x3f // cmn al r6 r14 LSL 14
};
const byte kInstruction_cmn_al_r1_r9_LSL_24[] = {
  0x11, 0xeb, 0x09, 0x6f // cmn al r1 r9 LSL 24
};
const byte kInstruction_cmn_al_r5_r2_LSL_22[] = {
  0x15, 0xeb, 0x82, 0x5f // cmn al r5 r2 LSL 22
};
const byte kInstruction_cmn_al_r3_r5_LSL_4[] = {
  0x13, 0xeb, 0x05, 0x1f // cmn al r3 r5 LSL 4
};
const byte kInstruction_cmn_al_r8_r0_LSL_5[] = {
  0x18, 0xeb, 0x40, 0x1f // cmn al r8 r0 LSL 5
};
const byte kInstruction_cmn_al_r5_r14_ROR_13[] = {
  0x15, 0xeb, 0x7e, 0x3f // cmn al r5 r14 ROR 13
};
const byte kInstruction_cmn_al_r12_r11_LSL_27[] = {
  0x1c, 0xeb, 0xcb, 0x6f // cmn al r12 r11 LSL 27
};
const byte kInstruction_cmn_al_r10_r14_LSL_20[] = {
  0x1a, 0xeb, 0x0e, 0x5f // cmn al r10 r14 LSL 20
};
const byte kInstruction_cmn_al_r5_r9_ROR_4[] = {
  0x15, 0xeb, 0x39, 0x1f // cmn al r5 r9 ROR 4
};
const byte kInstruction_cmn_al_r8_r11_ROR_11[] = {
  0x18, 0xeb, 0xfb, 0x2f // cmn al r8 r11 ROR 11
};
const byte kInstruction_cmn_al_r4_r1_LSL_24[] = {
  0x14, 0xeb, 0x01, 0x6f // cmn al r4 r1 LSL 24
};
const byte kInstruction_cmn_al_r7_r10_ROR_4[] = {
  0x17, 0xeb, 0x3a, 0x1f // cmn al r7 r10 ROR 4
};
const byte kInstruction_cmn_al_r3_r0_ROR_11[] = {
  0x13, 0xeb, 0xf0, 0x2f // cmn al r3 r0 ROR 11
};
const byte kInstruction_cmn_al_r3_r10_ROR_23[] = {
  0x13, 0xeb, 0xfa, 0x5f // cmn al r3 r10 ROR 23
};
const byte kInstruction_cmn_al_r2_r8_LSL_15[] = {
  0x12, 0xeb, 0xc8, 0x3f // cmn al r2 r8 LSL 15
};
const byte kInstruction_cmn_al_r3_r6_LSL_18[] = {
  0x13, 0xeb, 0x86, 0x4f // cmn al r3 r6 LSL 18
};
const byte kInstruction_cmn_al_r10_r2_ROR_7[] = {
  0x1a, 0xeb, 0xf2, 0x1f // cmn al r10 r2 ROR 7
};
const byte kInstruction_cmn_al_r6_r6_ROR_14[] = {
  0x16, 0xeb, 0xb6, 0x3f // cmn al r6 r6 ROR 14
};
const byte kInstruction_cmn_al_r1_r8_LSL_25[] = {
  0x11, 0xeb, 0x48, 0x6f // cmn al r1 r8 LSL 25
};
const byte kInstruction_cmn_al_r0_r14_LSL_13[] = {
  0x10, 0xeb, 0x4e, 0x3f // cmn al r0 r14 LSL 13
};
const byte kInstruction_cmn_al_r14_r1_LSL_13[] = {
  0x1e, 0xeb, 0x41, 0x3f // cmn al r14 r1 LSL 13
};
const byte kInstruction_cmn_al_r8_r11_ROR_8[] = {
  0x18, 0xeb, 0x3b, 0x2f // cmn al r8 r11 ROR 8
};
const byte kInstruction_cmn_al_r0_r12_ROR_6[] = {
  0x10, 0xeb, 0xbc, 0x1f // cmn al r0 r12 ROR 6
};
const byte kInstruction_cmn_al_r0_r12_LSL_26[] = {
  0x10, 0xeb, 0x8c, 0x6f // cmn al r0 r12 LSL 26
};
const byte kInstruction_cmn_al_r11_r2_LSL_10[] = {
  0x1b, 0xeb, 0x82, 0x2f // cmn al r11 r2 LSL 10
};
const byte kInstruction_cmn_al_r12_r7_LSL_3[] = {
  0x1c, 0xeb, 0xc7, 0x0f // cmn al r12 r7 LSL 3
};
const byte kInstruction_cmn_al_r6_r10_LSL_25[] = {
  0x16, 0xeb, 0x4a, 0x6f // cmn al r6 r10 LSL 25
};
const byte kInstruction_cmn_al_r12_r7_ROR_8[] = {
  0x1c, 0xeb, 0x37, 0x2f // cmn al r12 r7 ROR 8
};
const byte kInstruction_cmn_al_r3_r3_ROR_24[] = {
  0x13, 0xeb, 0x33, 0x6f // cmn al r3 r3 ROR 24
};
const byte kInstruction_cmn_al_r9_r12_LSL_24[] = {
  0x19, 0xeb, 0x0c, 0x6f // cmn al r9 r12 LSL 24
};
const byte kInstruction_cmn_al_r14_r10_LSL_27[] = {
  0x1e, 0xeb, 0xca, 0x6f // cmn al r14 r10 LSL 27
};
const byte kInstruction_cmn_al_r6_r5_ROR_4[] = {
  0x16, 0xeb, 0x35, 0x1f // cmn al r6 r5 ROR 4
};
const byte kInstruction_cmn_al_r7_r4_LSL_21[] = {
  0x17, 0xeb, 0x44, 0x5f // cmn al r7 r4 LSL 21
};
const byte kInstruction_cmn_al_r2_r4_ROR_31[] = {
  0x12, 0xeb, 0xf4, 0x7f // cmn al r2 r4 ROR 31
};
const byte kInstruction_cmn_al_r11_r7_ROR_8[] = {
  0x1b, 0xeb, 0x37, 0x2f // cmn al r11 r7 ROR 8
};
const byte kInstruction_cmn_al_r5_r2_ROR_17[] = {
  0x15, 0xeb, 0x72, 0x4f // cmn al r5 r2 ROR 17
};
const byte kInstruction_cmn_al_r1_r11_LSL_13[] = {
  0x11, 0xeb, 0x4b, 0x3f // cmn al r1 r11 LSL 13
};
const byte kInstruction_cmn_al_r7_r11_ROR_2[] = {
  0x17, 0xeb, 0xbb, 0x0f // cmn al r7 r11 ROR 2
};
const byte kInstruction_cmn_al_r1_r7_ROR_29[] = {
  0x11, 0xeb, 0x77, 0x7f // cmn al r1 r7 ROR 29
};
const byte kInstruction_cmn_al_r4_r1_LSL_31[] = {
  0x14, 0xeb, 0xc1, 0x7f // cmn al r4 r1 LSL 31
};
const byte kInstruction_cmn_al_r11_r13_ROR_11[] = {
  0x1b, 0xeb, 0xfd, 0x2f // cmn al r11 r13 ROR 11
};
const byte kInstruction_cmn_al_r13_r1_ROR_14[] = {
  0x1d, 0xeb, 0xb1, 0x3f // cmn al r13 r1 ROR 14
};
const byte kInstruction_cmn_al_r1_r14_ROR_21[] = {
  0x11, 0xeb, 0x7e, 0x5f // cmn al r1 r14 ROR 21
};
const byte kInstruction_cmn_al_r7_r12_LSL_5[] = {
  0x17, 0xeb, 0x4c, 0x1f // cmn al r7 r12 LSL 5
};
const byte kInstruction_cmn_al_r10_r3_LSL_2[] = {
  0x1a, 0xeb, 0x83, 0x0f // cmn al r10 r3 LSL 2
};
const byte kInstruction_cmn_al_r12_r10_ROR_25[] = {
  0x1c, 0xeb, 0x7a, 0x6f // cmn al r12 r10 ROR 25
};
const byte kInstruction_cmn_al_r2_r7_LSL_6[] = {
  0x12, 0xeb, 0x87, 0x1f // cmn al r2 r7 LSL 6
};
const byte kInstruction_cmn_al_r5_r0_LSL_23[] = {
  0x15, 0xeb, 0xc0, 0x5f // cmn al r5 r0 LSL 23
};
const byte kInstruction_cmn_al_r2_r8_ROR_18[] = {
  0x12, 0xeb, 0xb8, 0x4f // cmn al r2 r8 ROR 18
};
const byte kInstruction_cmn_al_r0_r1_ROR_15[] = {
  0x10, 0xeb, 0xf1, 0x3f // cmn al r0 r1 ROR 15
};
const byte kInstruction_cmn_al_r14_r4_ROR_20[] = {
  0x1e, 0xeb, 0x34, 0x5f // cmn al r14 r4 ROR 20
};
const byte kInstruction_cmn_al_r5_r10_LSL_15[] = {
  0x15, 0xeb, 0xca, 0x3f // cmn al r5 r10 LSL 15
};
const byte kInstruction_cmn_al_r0_r1_ROR_28[] = {
  0x10, 0xeb, 0x31, 0x7f // cmn al r0 r1 ROR 28
};
const byte kInstruction_cmn_al_r11_r2_LSL_9[] = {
  0x1b, 0xeb, 0x42, 0x2f // cmn al r11 r2 LSL 9
};
const byte kInstruction_cmn_al_r9_r10_LSL_19[] = {
  0x19, 0xeb, 0xca, 0x4f // cmn al r9 r10 LSL 19
};
const byte kInstruction_cmn_al_r4_r11_LSL_8[] = {
  0x14, 0xeb, 0x0b, 0x2f // cmn al r4 r11 LSL 8
};
const byte kInstruction_cmn_al_r12_r14_LSL_12[] = {
  0x1c, 0xeb, 0x0e, 0x3f // cmn al r12 r14 LSL 12
};
const byte kInstruction_cmn_al_r5_r9_LSL_8[] = {
  0x15, 0xeb, 0x09, 0x2f // cmn al r5 r9 LSL 8
};
const byte kInstruction_cmn_al_r12_r14_LSL_16[] = {
  0x1c, 0xeb, 0x0e, 0x4f // cmn al r12 r14 LSL 16
};
const byte kInstruction_cmn_al_r13_r13_LSL_23[] = {
  0x1d, 0xeb, 0xcd, 0x5f // cmn al r13 r13 LSL 23
};
const byte kInstruction_cmn_al_r3_r12_ROR_19[] = {
  0x13, 0xeb, 0xfc, 0x4f // cmn al r3 r12 ROR 19
};
const byte kInstruction_cmn_al_r2_r5_ROR_24[] = {
  0x12, 0xeb, 0x35, 0x6f // cmn al r2 r5 ROR 24
};
const byte kInstruction_cmn_al_r3_r2_ROR_11[] = {
  0x13, 0xeb, 0xf2, 0x2f // cmn al r3 r2 ROR 11
};
const byte kInstruction_cmn_al_r1_r13_ROR_1[] = {
  0x11, 0xeb, 0x7d, 0x0f // cmn al r1 r13 ROR 1
};
const byte kInstruction_cmn_al_r6_r7_LSL_11[] = {
  0x16, 0xeb, 0xc7, 0x2f // cmn al r6 r7 LSL 11
};
const byte kInstruction_cmn_al_r3_r14_LSL_2[] = {
  0x13, 0xeb, 0x8e, 0x0f // cmn al r3 r14 LSL 2
};
const byte kInstruction_cmn_al_r0_r10_LSL_20[] = {
  0x10, 0xeb, 0x0a, 0x5f // cmn al r0 r10 LSL 20
};
const byte kInstruction_cmn_al_r3_r0_LSL_19[] = {
  0x13, 0xeb, 0xc0, 0x4f // cmn al r3 r0 LSL 19
};
const byte kInstruction_cmn_al_r14_r0_ROR_21[] = {
  0x1e, 0xeb, 0x70, 0x5f // cmn al r14 r0 ROR 21
};
const byte kInstruction_cmn_al_r12_r13_LSL_16[] = {
  0x1c, 0xeb, 0x0d, 0x4f // cmn al r12 r13 LSL 16
};
const byte kInstruction_cmn_al_r8_r5_ROR_28[] = {
  0x18, 0xeb, 0x35, 0x7f // cmn al r8 r5 ROR 28
};
const byte kInstruction_cmn_al_r8_r1_LSL_25[] = {
  0x18, 0xeb, 0x41, 0x6f // cmn al r8 r1 LSL 25
};
const byte kInstruction_cmn_al_r6_r4_LSL_25[] = {
  0x16, 0xeb, 0x44, 0x6f // cmn al r6 r4 LSL 25
};
const byte kInstruction_cmn_al_r7_r12_ROR_26[] = {
  0x17, 0xeb, 0xbc, 0x6f // cmn al r7 r12 ROR 26
};
const byte kInstruction_cmn_al_r8_r14_LSL_12[] = {
  0x18, 0xeb, 0x0e, 0x3f // cmn al r8 r14 LSL 12
};
const byte kInstruction_cmn_al_r6_r6_ROR_2[] = {
  0x16, 0xeb, 0xb6, 0x0f // cmn al r6 r6 ROR 2
};
const byte kInstruction_cmn_al_r11_r11_LSL_13[] = {
  0x1b, 0xeb, 0x4b, 0x3f // cmn al r11 r11 LSL 13
};
const byte kInstruction_cmn_al_r1_r12_ROR_25[] = {
  0x11, 0xeb, 0x7c, 0x6f // cmn al r1 r12 ROR 25
};
const byte kInstruction_cmn_al_r9_r5_LSL_20[] = {
  0x19, 0xeb, 0x05, 0x5f // cmn al r9 r5 LSL 20
};
const byte kInstruction_cmn_al_r12_r11_LSL_20[] = {
  0x1c, 0xeb, 0x0b, 0x5f // cmn al r12 r11 LSL 20
};
const byte kInstruction_cmn_al_r8_r3_ROR_31[] = {
  0x18, 0xeb, 0xf3, 0x7f // cmn al r8 r3 ROR 31
};
const byte kInstruction_cmn_al_r0_r14_ROR_3[] = {
  0x10, 0xeb, 0xfe, 0x0f // cmn al r0 r14 ROR 3
};
const byte kInstruction_cmn_al_r13_r14_LSL_30[] = {
  0x1d, 0xeb, 0x8e, 0x7f // cmn al r13 r14 LSL 30
};
const byte kInstruction_cmn_al_r3_r6_ROR_24[] = {
  0x13, 0xeb, 0x36, 0x6f // cmn al r3 r6 ROR 24
};
const byte kInstruction_cmn_al_r10_r10_LSL_23[] = {
  0x1a, 0xeb, 0xca, 0x5f // cmn al r10 r10 LSL 23
};
const byte kInstruction_cmn_al_r9_r1_LSL_7[] = {
  0x19, 0xeb, 0xc1, 0x1f // cmn al r9 r1 LSL 7
};
const byte kInstruction_cmn_al_r6_r13_ROR_27[] = {
  0x16, 0xeb, 0xfd, 0x6f // cmn al r6 r13 ROR 27
};
const byte kInstruction_cmn_al_r1_r14_LSL_21[] = {
  0x11, 0xeb, 0x4e, 0x5f // cmn al r1 r14 LSL 21
};
const byte kInstruction_cmn_al_r9_r14_ROR_12[] = {
  0x19, 0xeb, 0x3e, 0x3f // cmn al r9 r14 ROR 12
};
const byte kInstruction_cmn_al_r14_r2_LSL_17[] = {
  0x1e, 0xeb, 0x42, 0x4f // cmn al r14 r2 LSL 17
};
const byte kInstruction_cmn_al_r11_r14_LSL_11[] = {
  0x1b, 0xeb, 0xce, 0x2f // cmn al r11 r14 LSL 11
};
const byte kInstruction_cmn_al_r11_r7_ROR_2[] = {
  0x1b, 0xeb, 0xb7, 0x0f // cmn al r11 r7 ROR 2
};
const byte kInstruction_cmn_al_r8_r7_ROR_20[] = {
  0x18, 0xeb, 0x37, 0x5f // cmn al r8 r7 ROR 20
};
const byte kInstruction_cmn_al_r10_r11_LSL_5[] = {
  0x1a, 0xeb, 0x4b, 0x1f // cmn al r10 r11 LSL 5
};
const byte kInstruction_cmn_al_r5_r11_ROR_16[] = {
  0x15, 0xeb, 0x3b, 0x4f // cmn al r5 r11 ROR 16
};
const byte kInstruction_cmn_al_r8_r3_LSL_23[] = {
  0x18, 0xeb, 0xc3, 0x5f // cmn al r8 r3 LSL 23
};
const byte kInstruction_cmn_al_r2_r8_LSL_23[] = {
  0x12, 0xeb, 0xc8, 0x5f // cmn al r2 r8 LSL 23
};
const byte kInstruction_cmn_al_r2_r13_LSL_28[] = {
  0x12, 0xeb, 0x0d, 0x7f // cmn al r2 r13 LSL 28
};
const byte kInstruction_cmn_al_r1_r0_LSL_20[] = {
  0x11, 0xeb, 0x00, 0x5f // cmn al r1 r0 LSL 20
};
const byte kInstruction_cmn_al_r9_r8_ROR_21[] = {
  0x19, 0xeb, 0x78, 0x5f // cmn al r9 r8 ROR 21
};
const byte kInstruction_cmn_al_r5_r14_LSL_22[] = {
  0x15, 0xeb, 0x8e, 0x5f // cmn al r5 r14 LSL 22
};
const byte kInstruction_cmn_al_r4_r9_ROR_7[] = {
  0x14, 0xeb, 0xf9, 0x1f // cmn al r4 r9 ROR 7
};
const byte kInstruction_cmn_al_r2_r12_ROR_9[] = {
  0x12, 0xeb, 0x7c, 0x2f // cmn al r2 r12 ROR 9
};
const byte kInstruction_cmn_al_r4_r10_LSL_31[] = {
  0x14, 0xeb, 0xca, 0x7f // cmn al r4 r10 LSL 31
};
const byte kInstruction_cmn_al_r4_r2_LSL_8[] = {
  0x14, 0xeb, 0x02, 0x2f // cmn al r4 r2 LSL 8
};
const byte kInstruction_cmn_al_r10_r6_LSL_27[] = {
  0x1a, 0xeb, 0xc6, 0x6f // cmn al r10 r6 LSL 27
};
const byte kInstruction_cmn_al_r12_r7_ROR_7[] = {
  0x1c, 0xeb, 0xf7, 0x1f // cmn al r12 r7 ROR 7
};
const byte kInstruction_cmn_al_r5_r1_ROR_9[] = {
  0x15, 0xeb, 0x71, 0x2f // cmn al r5 r1 ROR 9
};
const byte kInstruction_cmn_al_r13_r13_LSL_29[] = {
  0x1d, 0xeb, 0x4d, 0x7f // cmn al r13 r13 LSL 29
};
const byte kInstruction_cmn_al_r0_r12_ROR_28[] = {
  0x10, 0xeb, 0x3c, 0x7f // cmn al r0 r12 ROR 28
};
const byte kInstruction_cmn_al_r9_r6_ROR_26[] = {
  0x19, 0xeb, 0xb6, 0x6f // cmn al r9 r6 ROR 26
};
const byte kInstruction_cmn_al_r1_r8_ROR_14[] = {
  0x11, 0xeb, 0xb8, 0x3f // cmn al r1 r8 ROR 14
};
const byte kInstruction_cmn_al_r9_r5_LSL_19[] = {
  0x19, 0xeb, 0xc5, 0x4f // cmn al r9 r5 LSL 19
};
const byte kInstruction_cmn_al_r10_r5_LSL_23[] = {
  0x1a, 0xeb, 0xc5, 0x5f // cmn al r10 r5 LSL 23
};
const byte kInstruction_cmn_al_r2_r12_ROR_2[] = {
  0x12, 0xeb, 0xbc, 0x0f // cmn al r2 r12 ROR 2
};
const byte kInstruction_cmn_al_r5_r10_ROR_18[] = {
  0x15, 0xeb, 0xba, 0x4f // cmn al r5 r10 ROR 18
};
const byte kInstruction_cmn_al_r13_r12_ROR_12[] = {
  0x1d, 0xeb, 0x3c, 0x3f // cmn al r13 r12 ROR 12
};
const byte kInstruction_cmn_al_r9_r10_LSL_15[] = {
  0x19, 0xeb, 0xca, 0x3f // cmn al r9 r10 LSL 15
};
const byte kInstruction_cmn_al_r9_r2_LSL_26[] = {
  0x19, 0xeb, 0x82, 0x6f // cmn al r9 r2 LSL 26
};
const byte kInstruction_cmn_al_r2_r3_LSL_19[] = {
  0x12, 0xeb, 0xc3, 0x4f // cmn al r2 r3 LSL 19
};
const byte kInstruction_cmn_al_r7_r10_ROR_28[] = {
  0x17, 0xeb, 0x3a, 0x7f // cmn al r7 r10 ROR 28
};
const byte kInstruction_cmn_al_r14_r6_ROR_6[] = {
  0x1e, 0xeb, 0xb6, 0x1f // cmn al r14 r6 ROR 6
};
const byte kInstruction_cmn_al_r5_r1_ROR_16[] = {
  0x15, 0xeb, 0x31, 0x4f // cmn al r5 r1 ROR 16
};
const byte kInstruction_cmn_al_r4_r6_ROR_10[] = {
  0x14, 0xeb, 0xb6, 0x2f // cmn al r4 r6 ROR 10
};
const byte kInstruction_cmn_al_r7_r12_LSL_10[] = {
  0x17, 0xeb, 0x8c, 0x2f // cmn al r7 r12 LSL 10
};
const byte kInstruction_cmn_al_r0_r14_LSL_28[] = {
  0x10, 0xeb, 0x0e, 0x7f // cmn al r0 r14 LSL 28
};
const byte kInstruction_cmn_al_r8_r2_LSL_27[] = {
  0x18, 0xeb, 0xc2, 0x6f // cmn al r8 r2 LSL 27
};
const byte kInstruction_cmn_al_r8_r12_ROR_1[] = {
  0x18, 0xeb, 0x7c, 0x0f // cmn al r8 r12 ROR 1
};
const byte kInstruction_cmn_al_r8_r4_ROR_4[] = {
  0x18, 0xeb, 0x34, 0x1f // cmn al r8 r4 ROR 4
};
const byte kInstruction_cmn_al_r3_r5_ROR_20[] = {
  0x13, 0xeb, 0x35, 0x5f // cmn al r3 r5 ROR 20
};
const byte kInstruction_cmn_al_r6_r13_LSL_5[] = {
  0x16, 0xeb, 0x4d, 0x1f // cmn al r6 r13 LSL 5
};
const byte kInstruction_cmn_al_r9_r9_LSL_4[] = {
  0x19, 0xeb, 0x09, 0x1f // cmn al r9 r9 LSL 4
};
const byte kInstruction_cmn_al_r13_r13_ROR_7[] = {
  0x1d, 0xeb, 0xfd, 0x1f // cmn al r13 r13 ROR 7
};
const byte kInstruction_cmn_al_r3_r7_LSL_11[] = {
  0x13, 0xeb, 0xc7, 0x2f // cmn al r3 r7 LSL 11
};
const byte kInstruction_cmn_al_r1_r3_ROR_15[] = {
  0x11, 0xeb, 0xf3, 0x3f // cmn al r1 r3 ROR 15
};
const byte kInstruction_cmn_al_r2_r0_LSL_30[] = {
  0x12, 0xeb, 0x80, 0x7f // cmn al r2 r0 LSL 30
};
const byte kInstruction_cmn_al_r11_r13_ROR_16[] = {
  0x1b, 0xeb, 0x3d, 0x4f // cmn al r11 r13 ROR 16
};
const byte kInstruction_cmn_al_r3_r0_LSL_15[] = {
  0x13, 0xeb, 0xc0, 0x3f // cmn al r3 r0 LSL 15
};
const byte kInstruction_cmn_al_r2_r0_LSL_8[] = {
  0x12, 0xeb, 0x00, 0x2f // cmn al r2 r0 LSL 8
};
const byte kInstruction_cmn_al_r4_r13_ROR_5[] = {
  0x14, 0xeb, 0x7d, 0x1f // cmn al r4 r13 ROR 5
};
const byte kInstruction_cmn_al_r13_r0_ROR_29[] = {
  0x1d, 0xeb, 0x70, 0x7f // cmn al r13 r0 ROR 29
};
const byte kInstruction_cmn_al_r6_r2_ROR_18[] = {
  0x16, 0xeb, 0xb2, 0x4f // cmn al r6 r2 ROR 18
};
const byte kInstruction_cmn_al_r1_r2_LSL_2[] = {
  0x11, 0xeb, 0x82, 0x0f // cmn al r1 r2 LSL 2
};
const byte kInstruction_cmn_al_r7_r4_ROR_12[] = {
  0x17, 0xeb, 0x34, 0x3f // cmn al r7 r4 ROR 12
};
const byte kInstruction_cmn_al_r9_r11_ROR_6[] = {
  0x19, 0xeb, 0xbb, 0x1f // cmn al r9 r11 ROR 6
};
const byte kInstruction_cmn_al_r8_r2_ROR_12[] = {
  0x18, 0xeb, 0x32, 0x3f // cmn al r8 r2 ROR 12
};
const byte kInstruction_cmn_al_r12_r2_LSL_19[] = {
  0x1c, 0xeb, 0xc2, 0x4f // cmn al r12 r2 LSL 19
};
const byte kInstruction_cmn_al_r13_r0_ROR_30[] = {
  0x1d, 0xeb, 0xb0, 0x7f // cmn al r13 r0 ROR 30
};
const byte kInstruction_cmn_al_r7_r6_ROR_16[] = {
  0x17, 0xeb, 0x36, 0x4f // cmn al r7 r6 ROR 16
};
const byte kInstruction_cmn_al_r6_r13_ROR_19[] = {
  0x16, 0xeb, 0xfd, 0x4f // cmn al r6 r13 ROR 19
};
const byte kInstruction_cmn_al_r14_r3_LSL_19[] = {
  0x1e, 0xeb, 0xc3, 0x4f // cmn al r14 r3 LSL 19
};
const byte kInstruction_cmn_al_r14_r5_ROR_24[] = {
  0x1e, 0xeb, 0x35, 0x6f // cmn al r14 r5 ROR 24
};
const byte kInstruction_cmn_al_r10_r12_LSL_8[] = {
  0x1a, 0xeb, 0x0c, 0x2f // cmn al r10 r12 LSL 8
};
const byte kInstruction_cmn_al_r14_r7_LSL_11[] = {
  0x1e, 0xeb, 0xc7, 0x2f // cmn al r14 r7 LSL 11
};
const byte kInstruction_cmn_al_r10_r14_LSL_30[] = {
  0x1a, 0xeb, 0x8e, 0x7f // cmn al r10 r14 LSL 30
};
const byte kInstruction_cmn_al_r7_r13_ROR_7[] = {
  0x17, 0xeb, 0xfd, 0x1f // cmn al r7 r13 ROR 7
};
const byte kInstruction_cmn_al_r11_r0_LSL_23[] = {
  0x1b, 0xeb, 0xc0, 0x5f // cmn al r11 r0 LSL 23
};
const byte kInstruction_cmn_al_r0_r12_LSL_30[] = {
  0x10, 0xeb, 0x8c, 0x7f // cmn al r0 r12 LSL 30
};
const byte kInstruction_cmn_al_r5_r12_LSL_7[] = {
  0x15, 0xeb, 0xcc, 0x1f // cmn al r5 r12 LSL 7
};
const byte kInstruction_cmn_al_r12_r12_ROR_14[] = {
  0x1c, 0xeb, 0xbc, 0x3f // cmn al r12 r12 ROR 14
};
const byte kInstruction_cmn_al_r8_r11_LSL_4[] = {
  0x18, 0xeb, 0x0b, 0x1f // cmn al r8 r11 LSL 4
};
const byte kInstruction_cmn_al_r14_r2_ROR_24[] = {
  0x1e, 0xeb, 0x32, 0x6f // cmn al r14 r2 ROR 24
};
const byte kInstruction_cmn_al_r10_r2_ROR_31[] = {
  0x1a, 0xeb, 0xf2, 0x7f // cmn al r10 r2 ROR 31
};
const byte kInstruction_cmn_al_r11_r5_LSL_1[] = {
  0x1b, 0xeb, 0x45, 0x0f // cmn al r11 r5 LSL 1
};
const byte kInstruction_cmn_al_r8_r6_LSL_24[] = {
  0x18, 0xeb, 0x06, 0x6f // cmn al r8 r6 LSL 24
};
const byte kInstruction_cmn_al_r14_r1_ROR_8[] = {
  0x1e, 0xeb, 0x31, 0x2f // cmn al r14 r1 ROR 8
};
const byte kInstruction_cmn_al_r11_r2_ROR_19[] = {
  0x1b, 0xeb, 0xf2, 0x4f // cmn al r11 r2 ROR 19
};
const byte kInstruction_cmn_al_r1_r12_LSL_14[] = {
  0x11, 0xeb, 0x8c, 0x3f // cmn al r1 r12 LSL 14
};
const byte kInstruction_cmn_al_r6_r10_LSL_30[] = {
  0x16, 0xeb, 0x8a, 0x7f // cmn al r6 r10 LSL 30
};
const byte kInstruction_cmn_al_r8_r1_ROR_6[] = {
  0x18, 0xeb, 0xb1, 0x1f // cmn al r8 r1 ROR 6
};
const byte kInstruction_cmn_al_r13_r11_ROR_6[] = {
  0x1d, 0xeb, 0xbb, 0x1f // cmn al r13 r11 ROR 6
};
const byte kInstruction_cmn_al_r5_r13_LSL_30[] = {
  0x15, 0xeb, 0x8d, 0x7f // cmn al r5 r13 LSL 30
};
const byte kInstruction_cmn_al_r0_r13_ROR_26[] = {
  0x10, 0xeb, 0xbd, 0x6f // cmn al r0 r13 ROR 26
};
const byte kInstruction_cmn_al_r4_r10_LSL_30[] = {
  0x14, 0xeb, 0x8a, 0x7f // cmn al r4 r10 LSL 30
};
const byte kInstruction_cmn_al_r8_r4_LSL_16[] = {
  0x18, 0xeb, 0x04, 0x4f // cmn al r8 r4 LSL 16
};
const byte kInstruction_cmn_al_r11_r12_LSL_11[] = {
  0x1b, 0xeb, 0xcc, 0x2f // cmn al r11 r12 LSL 11
};
const byte kInstruction_cmn_al_r9_r3_ROR_2[] = {
  0x19, 0xeb, 0xb3, 0x0f // cmn al r9 r3 ROR 2
};
const byte kInstruction_cmn_al_r11_r7_LSL_26[] = {
  0x1b, 0xeb, 0x87, 0x6f // cmn al r11 r7 LSL 26
};
const byte kInstruction_cmn_al_r14_r2_ROR_22[] = {
  0x1e, 0xeb, 0xb2, 0x5f // cmn al r14 r2 ROR 22
};
const byte kInstruction_cmn_al_r8_r2_ROR_10[] = {
  0x18, 0xeb, 0xb2, 0x2f // cmn al r8 r2 ROR 10
};
const byte kInstruction_cmn_al_r5_r2_ROR_28[] = {
  0x15, 0xeb, 0x32, 0x7f // cmn al r5 r2 ROR 28
};
const byte kInstruction_cmn_al_r14_r3_ROR_11[] = {
  0x1e, 0xeb, 0xf3, 0x2f // cmn al r14 r3 ROR 11
};
const byte kInstruction_cmn_al_r3_r14_LSL_5[] = {
  0x13, 0xeb, 0x4e, 0x1f // cmn al r3 r14 LSL 5
};
const byte kInstruction_cmn_al_r12_r3_ROR_15[] = {
  0x1c, 0xeb, 0xf3, 0x3f // cmn al r12 r3 ROR 15
};
const byte kInstruction_cmn_al_r4_r1_ROR_30[] = {
  0x14, 0xeb, 0xb1, 0x7f // cmn al r4 r1 ROR 30
};
const byte kInstruction_cmn_al_r13_r8_ROR_11[] = {
  0x1d, 0xeb, 0xf8, 0x2f // cmn al r13 r8 ROR 11
};
const byte kInstruction_cmn_al_r6_r9_ROR_9[] = {
  0x16, 0xeb, 0x79, 0x2f // cmn al r6 r9 ROR 9
};
const byte kInstruction_cmn_al_r8_r13_ROR_15[] = {
  0x18, 0xeb, 0xfd, 0x3f // cmn al r8 r13 ROR 15
};
const byte kInstruction_cmn_al_r0_r14_ROR_12[] = {
  0x10, 0xeb, 0x3e, 0x3f // cmn al r0 r14 ROR 12
};
const byte kInstruction_cmn_al_r0_r10_LSL_6[] = {
  0x10, 0xeb, 0x8a, 0x1f // cmn al r0 r10 LSL 6
};
const byte kInstruction_cmn_al_r8_r7_LSL_5[] = {
  0x18, 0xeb, 0x47, 0x1f // cmn al r8 r7 LSL 5
};
const byte kInstruction_cmn_al_r3_r7_ROR_11[] = {
  0x13, 0xeb, 0xf7, 0x2f // cmn al r3 r7 ROR 11
};
const byte kInstruction_cmn_al_r5_r1_LSL_20[] = {
  0x15, 0xeb, 0x01, 0x5f // cmn al r5 r1 LSL 20
};
const byte kInstruction_cmn_al_r6_r7_LSL_10[] = {
  0x16, 0xeb, 0x87, 0x2f // cmn al r6 r7 LSL 10
};
const byte kInstruction_cmn_al_r14_r0_ROR_27[] = {
  0x1e, 0xeb, 0xf0, 0x6f // cmn al r14 r0 ROR 27
};
const byte kInstruction_cmn_al_r6_r12_LSL_5[] = {
  0x16, 0xeb, 0x4c, 0x1f // cmn al r6 r12 LSL 5
};
const byte kInstruction_cmn_al_r4_r8_LSL_28[] = {
  0x14, 0xeb, 0x08, 0x7f // cmn al r4 r8 LSL 28
};
const byte kInstruction_cmn_al_r7_r6_ROR_15[] = {
  0x17, 0xeb, 0xf6, 0x3f // cmn al r7 r6 ROR 15
};
const byte kInstruction_cmn_al_r0_r6_LSL_27[] = {
  0x10, 0xeb, 0xc6, 0x6f // cmn al r0 r6 LSL 27
};
const byte kInstruction_cmn_al_r1_r6_ROR_19[] = {
  0x11, 0xeb, 0xf6, 0x4f // cmn al r1 r6 ROR 19
};
const byte kInstruction_cmn_al_r11_r14_ROR_21[] = {
  0x1b, 0xeb, 0x7e, 0x5f // cmn al r11 r14 ROR 21
};
const byte kInstruction_cmn_al_r4_r13_LSL_5[] = {
  0x14, 0xeb, 0x4d, 0x1f // cmn al r4 r13 LSL 5
};
const byte kInstruction_cmn_al_r4_r7_ROR_2[] = {
  0x14, 0xeb, 0xb7, 0x0f // cmn al r4 r7 ROR 2
};
const byte kInstruction_cmn_al_r8_r1_ROR_29[] = {
  0x18, 0xeb, 0x71, 0x7f // cmn al r8 r1 ROR 29
};
const byte kInstruction_cmn_al_r14_r2_LSL_24[] = {
  0x1e, 0xeb, 0x02, 0x6f // cmn al r14 r2 LSL 24
};
const byte kInstruction_cmn_al_r5_r12_ROR_27[] = {
  0x15, 0xeb, 0xfc, 0x6f // cmn al r5 r12 ROR 27
};
const byte kInstruction_cmn_al_r11_r4_ROR_14[] = {
  0x1b, 0xeb, 0xb4, 0x3f // cmn al r11 r4 ROR 14
};
const byte kInstruction_cmn_al_r4_r7_LSL_28[] = {
  0x14, 0xeb, 0x07, 0x7f // cmn al r4 r7 LSL 28
};
const byte kInstruction_cmn_al_r3_r6_LSL_9[] = {
  0x13, 0xeb, 0x46, 0x2f // cmn al r3 r6 LSL 9
};
const byte kInstruction_cmn_al_r11_r13_LSL_12[] = {
  0x1b, 0xeb, 0x0d, 0x3f // cmn al r11 r13 LSL 12
};
const byte kInstruction_cmn_al_r10_r7_ROR_20[] = {
  0x1a, 0xeb, 0x37, 0x5f // cmn al r10 r7 ROR 20
};
const byte kInstruction_cmn_al_r2_r8_LSL_12[] = {
  0x12, 0xeb, 0x08, 0x3f // cmn al r2 r8 LSL 12
};
const byte kInstruction_cmn_al_r4_r13_LSL_28[] = {
  0x14, 0xeb, 0x0d, 0x7f // cmn al r4 r13 LSL 28
};
const byte kInstruction_cmn_al_r1_r12_LSL_28[] = {
  0x11, 0xeb, 0x0c, 0x7f // cmn al r1 r12 LSL 28
};
const byte kInstruction_cmn_al_r7_r14_LSL_6[] = {
  0x17, 0xeb, 0x8e, 0x1f // cmn al r7 r14 LSL 6
};
const byte kInstruction_cmn_al_r11_r2_ROR_10[] = {
  0x1b, 0xeb, 0xb2, 0x2f // cmn al r11 r2 ROR 10
};
const byte kInstruction_cmn_al_r3_r9_ROR_7[] = {
  0x13, 0xeb, 0xf9, 0x1f // cmn al r3 r9 ROR 7
};
const byte kInstruction_cmn_al_r2_r12_LSL_11[] = {
  0x12, 0xeb, 0xcc, 0x2f // cmn al r2 r12 LSL 11
};
const byte kInstruction_cmn_al_r4_r2_ROR_19[] = {
  0x14, 0xeb, 0xf2, 0x4f // cmn al r4 r2 ROR 19
};
const byte kInstruction_cmn_al_r13_r0_LSL_5[] = {
  0x1d, 0xeb, 0x40, 0x1f // cmn al r13 r0 LSL 5
};
const byte kInstruction_cmn_al_r14_r11_ROR_29[] = {
  0x1e, 0xeb, 0x7b, 0x7f // cmn al r14 r11 ROR 29
};
const byte kInstruction_cmn_al_r9_r6_LSL_11[] = {
  0x19, 0xeb, 0xc6, 0x2f // cmn al r9 r6 LSL 11
};
const byte kInstruction_cmn_al_r2_r10_ROR_30[] = {
  0x12, 0xeb, 0xba, 0x7f // cmn al r2 r10 ROR 30
};
const byte kInstruction_cmn_al_r3_r14_ROR_4[] = {
  0x13, 0xeb, 0x3e, 0x1f // cmn al r3 r14 ROR 4
};
const byte kInstruction_cmn_al_r9_r6_LSL_24[] = {
  0x19, 0xeb, 0x06, 0x6f // cmn al r9 r6 LSL 24
};
const byte kInstruction_cmn_al_r9_r8_LSL_2[] = {
  0x19, 0xeb, 0x88, 0x0f // cmn al r9 r8 LSL 2
};
const byte kInstruction_cmn_al_r12_r5_ROR_7[] = {
  0x1c, 0xeb, 0xf5, 0x1f // cmn al r12 r5 ROR 7
};
const byte kInstruction_cmn_al_r1_r11_ROR_20[] = {
  0x11, 0xeb, 0x3b, 0x5f // cmn al r1 r11 ROR 20
};
const byte kInstruction_cmn_al_r6_r12_LSL_6[] = {
  0x16, 0xeb, 0x8c, 0x1f // cmn al r6 r12 LSL 6
};
const byte kInstruction_cmn_al_r14_r11_ROR_26[] = {
  0x1e, 0xeb, 0xbb, 0x6f // cmn al r14 r11 ROR 26
};
const byte kInstruction_cmn_al_r5_r8_ROR_6[] = {
  0x15, 0xeb, 0xb8, 0x1f // cmn al r5 r8 ROR 6
};
const byte kInstruction_cmn_al_r10_r4_LSL_8[] = {
  0x1a, 0xeb, 0x04, 0x2f // cmn al r10 r4 LSL 8
};
const byte kInstruction_cmn_al_r5_r0_ROR_24[] = {
  0x15, 0xeb, 0x30, 0x6f // cmn al r5 r0 ROR 24
};
const byte kInstruction_cmn_al_r10_r3_LSL_20[] = {
  0x1a, 0xeb, 0x03, 0x5f // cmn al r10 r3 LSL 20
};
const byte kInstruction_cmn_al_r14_r4_LSL_22[] = {
  0x1e, 0xeb, 0x84, 0x5f // cmn al r14 r4 LSL 22
};
const byte kInstruction_cmn_al_r8_r8_ROR_1[] = {
  0x18, 0xeb, 0x78, 0x0f // cmn al r8 r8 ROR 1
};
const byte kInstruction_cmn_al_r12_r9_ROR_8[] = {
  0x1c, 0xeb, 0x39, 0x2f // cmn al r12 r9 ROR 8
};
const byte kInstruction_cmn_al_r6_r13_ROR_15[] = {
  0x16, 0xeb, 0xfd, 0x3f // cmn al r6 r13 ROR 15
};
const byte kInstruction_cmn_al_r9_r5_LSL_2[] = {
  0x19, 0xeb, 0x85, 0x0f // cmn al r9 r5 LSL 2
};
const byte kInstruction_cmn_al_r13_r8_LSL_17[] = {
  0x1d, 0xeb, 0x48, 0x4f // cmn al r13 r8 LSL 17
};
const byte kInstruction_cmn_al_r2_r5_ROR_27[] = {
  0x12, 0xeb, 0xf5, 0x6f // cmn al r2 r5 ROR 27
};
const byte kInstruction_cmn_al_r13_r12_ROR_10[] = {
  0x1d, 0xeb, 0xbc, 0x2f // cmn al r13 r12 ROR 10
};
const byte kInstruction_cmn_al_r13_r2_LSL_30[] = {
  0x1d, 0xeb, 0x82, 0x7f // cmn al r13 r2 LSL 30
};
const byte kInstruction_cmn_al_r10_r11_LSL_20[] = {
  0x1a, 0xeb, 0x0b, 0x5f // cmn al r10 r11 LSL 20
};
const byte kInstruction_cmn_al_r6_r1_ROR_22[] = {
  0x16, 0xeb, 0xb1, 0x5f // cmn al r6 r1 ROR 22
};
const byte kInstruction_cmn_al_r0_r4_ROR_18[] = {
  0x10, 0xeb, 0xb4, 0x4f // cmn al r0 r4 ROR 18
};
const byte kInstruction_cmn_al_r11_r14_ROR_19[] = {
  0x1b, 0xeb, 0xfe, 0x4f // cmn al r11 r14 ROR 19
};
const byte kInstruction_cmn_al_r0_r7_ROR_4[] = {
  0x10, 0xeb, 0x37, 0x1f // cmn al r0 r7 ROR 4
};
const byte kInstruction_cmn_al_r1_r5_LSL_13[] = {
  0x11, 0xeb, 0x45, 0x3f // cmn al r1 r5 LSL 13
};
const byte kInstruction_cmn_al_r11_r14_LSL_6[] = {
  0x1b, 0xeb, 0x8e, 0x1f // cmn al r11 r14 LSL 6
};
const byte kInstruction_cmn_al_r6_r11_LSL_10[] = {
  0x16, 0xeb, 0x8b, 0x2f // cmn al r6 r11 LSL 10
};
const byte kInstruction_cmn_al_r10_r14_LSL_1[] = {
  0x1a, 0xeb, 0x4e, 0x0f // cmn al r10 r14 LSL 1
};
const byte kInstruction_cmn_al_r13_r4_ROR_29[] = {
  0x1d, 0xeb, 0x74, 0x7f // cmn al r13 r4 ROR 29
};
const byte kInstruction_cmn_al_r9_r8_ROR_28[] = {
  0x19, 0xeb, 0x38, 0x7f // cmn al r9 r8 ROR 28
};
const byte kInstruction_cmn_al_r7_r12_ROR_22[] = {
  0x17, 0xeb, 0xbc, 0x5f // cmn al r7 r12 ROR 22
};
const byte kInstruction_cmn_al_r1_r12_ROR_10[] = {
  0x11, 0xeb, 0xbc, 0x2f // cmn al r1 r12 ROR 10
};
const byte kInstruction_cmn_al_r7_r8_ROR_31[] = {
  0x17, 0xeb, 0xf8, 0x7f // cmn al r7 r8 ROR 31
};
const byte kInstruction_cmn_al_r0_r2_ROR_25[] = {
  0x10, 0xeb, 0x72, 0x6f // cmn al r0 r2 ROR 25
};
const byte kInstruction_cmn_al_r3_r5_LSL_24[] = {
  0x13, 0xeb, 0x05, 0x6f // cmn al r3 r5 LSL 24
};
const byte kInstruction_cmn_al_r11_r10_LSL_25[] = {
  0x1b, 0xeb, 0x4a, 0x6f // cmn al r11 r10 LSL 25
};
const byte kInstruction_cmn_al_r11_r1_LSL_9[] = {
  0x1b, 0xeb, 0x41, 0x2f // cmn al r11 r1 LSL 9
};
const byte kInstruction_cmn_al_r2_r7_LSL_3[] = {
  0x12, 0xeb, 0xc7, 0x0f // cmn al r2 r7 LSL 3
};
const byte kInstruction_cmn_al_r0_r10_ROR_20[] = {
  0x10, 0xeb, 0x3a, 0x5f // cmn al r0 r10 ROR 20
};
const byte kInstruction_cmn_al_r7_r0_LSL_2[] = {
  0x17, 0xeb, 0x80, 0x0f // cmn al r7 r0 LSL 2
};
const byte kInstruction_cmn_al_r12_r5_LSL_8[] = {
  0x1c, 0xeb, 0x05, 0x2f // cmn al r12 r5 LSL 8
};
const byte kInstruction_cmn_al_r11_r1_ROR_16[] = {
  0x1b, 0xeb, 0x31, 0x4f // cmn al r11 r1 ROR 16
};
const byte kInstruction_cmn_al_r3_r3_LSL_30[] = {
  0x13, 0xeb, 0x83, 0x7f // cmn al r3 r3 LSL 30
};
const byte kInstruction_cmn_al_r4_r9_LSL_16[] = {
  0x14, 0xeb, 0x09, 0x4f // cmn al r4 r9 LSL 16
};
const byte kInstruction_cmn_al_r9_r8_LSL_27[] = {
  0x19, 0xeb, 0xc8, 0x6f // cmn al r9 r8 LSL 27
};
const byte kInstruction_cmn_al_r4_r2_LSL_15[] = {
  0x14, 0xeb, 0xc2, 0x3f // cmn al r4 r2 LSL 15
};
const byte kInstruction_cmn_al_r12_r13_LSL_3[] = {
  0x1c, 0xeb, 0xcd, 0x0f // cmn al r12 r13 LSL 3
};
const byte kInstruction_cmn_al_r11_r3_LSL_29[] = {
  0x1b, 0xeb, 0x43, 0x7f // cmn al r11 r3 LSL 29
};
const byte kInstruction_cmn_al_r4_r6_ROR_23[] = {
  0x14, 0xeb, 0xf6, 0x5f // cmn al r4 r6 ROR 23
};
const byte kInstruction_cmn_al_r5_r8_LSL_3[] = {
  0x15, 0xeb, 0xc8, 0x0f // cmn al r5 r8 LSL 3
};
const byte kInstruction_cmn_al_r1_r9_ROR_4[] = {
  0x11, 0xeb, 0x39, 0x1f // cmn al r1 r9 ROR 4
};
const byte kInstruction_cmn_al_r13_r6_LSL_12[] = {
  0x1d, 0xeb, 0x06, 0x3f // cmn al r13 r6 LSL 12
};
const byte kInstruction_cmn_al_r12_r6_ROR_17[] = {
  0x1c, 0xeb, 0x76, 0x4f // cmn al r12 r6 ROR 17
};
const byte kInstruction_cmn_al_r2_r1_LSL_22[] = {
  0x12, 0xeb, 0x81, 0x5f // cmn al r2 r1 LSL 22
};
const byte kInstruction_cmn_al_r9_r2_LSL_10[] = {
  0x19, 0xeb, 0x82, 0x2f // cmn al r9 r2 LSL 10
};
const byte kInstruction_cmn_al_r11_r10_LSL_21[] = {
  0x1b, 0xeb, 0x4a, 0x5f // cmn al r11 r10 LSL 21
};
const byte kInstruction_cmn_al_r8_r12_LSL_31[] = {
  0x18, 0xeb, 0xcc, 0x7f // cmn al r8 r12 LSL 31
};
const byte kInstruction_cmn_al_r5_r2_LSL_11[] = {
  0x15, 0xeb, 0xc2, 0x2f // cmn al r5 r2 LSL 11
};
const byte kInstruction_cmn_al_r9_r6_LSL_31[] = {
  0x19, 0xeb, 0xc6, 0x7f // cmn al r9 r6 LSL 31
};
const byte kInstruction_cmn_al_r6_r6_ROR_23[] = {
  0x16, 0xeb, 0xf6, 0x5f // cmn al r6 r6 ROR 23
};
const byte kInstruction_cmn_al_r14_r3_LSL_5[] = {
  0x1e, 0xeb, 0x43, 0x1f // cmn al r14 r3 LSL 5
};
const byte kInstruction_cmn_al_r14_r9_ROR_26[] = {
  0x1e, 0xeb, 0xb9, 0x6f // cmn al r14 r9 ROR 26
};
const byte kInstruction_cmn_al_r13_r1_ROR_15[] = {
  0x1d, 0xeb, 0xf1, 0x3f // cmn al r13 r1 ROR 15
};
const byte kInstruction_cmn_al_r2_r13_LSL_13[] = {
  0x12, 0xeb, 0x4d, 0x3f // cmn al r2 r13 LSL 13
};
const byte kInstruction_cmn_al_r14_r7_LSL_8[] = {
  0x1e, 0xeb, 0x07, 0x2f // cmn al r14 r7 LSL 8
};
const byte kInstruction_cmn_al_r9_r4_LSL_8[] = {
  0x19, 0xeb, 0x04, 0x2f // cmn al r9 r4 LSL 8
};
const byte kInstruction_cmn_al_r0_r13_LSL_26[] = {
  0x10, 0xeb, 0x8d, 0x6f // cmn al r0 r13 LSL 26
};
const byte kInstruction_cmn_al_r11_r1_LSL_12[] = {
  0x1b, 0xeb, 0x01, 0x3f // cmn al r11 r1 LSL 12
};
const byte kInstruction_cmn_al_r7_r6_LSL_5[] = {
  0x17, 0xeb, 0x46, 0x1f // cmn al r7 r6 LSL 5
};
const byte kInstruction_cmn_al_r4_r9_LSL_15[] = {
  0x14, 0xeb, 0xc9, 0x3f // cmn al r4 r9 LSL 15
};
const byte kInstruction_cmn_al_r2_r14_ROR_19[] = {
  0x12, 0xeb, 0xfe, 0x4f // cmn al r2 r14 ROR 19
};
const byte kInstruction_cmn_al_r12_r4_LSL_21[] = {
  0x1c, 0xeb, 0x44, 0x5f // cmn al r12 r4 LSL 21
};
const byte kInstruction_cmn_al_r0_r9_ROR_23[] = {
  0x10, 0xeb, 0xf9, 0x5f // cmn al r0 r9 ROR 23
};
const byte kInstruction_cmn_al_r5_r13_ROR_31[] = {
  0x15, 0xeb, 0xfd, 0x7f // cmn al r5 r13 ROR 31
};
const byte kInstruction_cmn_al_r8_r1_ROR_19[] = {
  0x18, 0xeb, 0xf1, 0x4f // cmn al r8 r1 ROR 19
};
const byte kInstruction_cmn_al_r1_r11_ROR_10[] = {
  0x11, 0xeb, 0xbb, 0x2f // cmn al r1 r11 ROR 10
};
const byte kInstruction_cmn_al_r10_r7_ROR_22[] = {
  0x1a, 0xeb, 0xb7, 0x5f // cmn al r10 r7 ROR 22
};
const byte kInstruction_cmn_al_r10_r11_LSL_26[] = {
  0x1a, 0xeb, 0x8b, 0x6f // cmn al r10 r11 LSL 26
};
const byte kInstruction_cmn_al_r0_r14_ROR_15[] = {
  0x10, 0xeb, 0xfe, 0x3f // cmn al r0 r14 ROR 15
};
const byte kInstruction_cmn_al_r1_r9_LSL_20[] = {
  0x11, 0xeb, 0x09, 0x5f // cmn al r1 r9 LSL 20
};
const byte kInstruction_cmn_al_r7_r0_LSL_11[] = {
  0x17, 0xeb, 0xc0, 0x2f // cmn al r7 r0 LSL 11
};
const byte kInstruction_cmn_al_r10_r14_ROR_15[] = {
  0x1a, 0xeb, 0xfe, 0x3f // cmn al r10 r14 ROR 15
};
const byte kInstruction_cmn_al_r1_r11_LSL_2[] = {
  0x11, 0xeb, 0x8b, 0x0f // cmn al r1 r11 LSL 2
};
const byte kInstruction_cmn_al_r1_r13_ROR_2[] = {
  0x11, 0xeb, 0xbd, 0x0f // cmn al r1 r13 ROR 2
};
const byte kInstruction_cmn_al_r12_r1_ROR_13[] = {
  0x1c, 0xeb, 0x71, 0x3f // cmn al r12 r1 ROR 13
};
const byte kInstruction_cmn_al_r1_r4_LSL_17[] = {
  0x11, 0xeb, 0x44, 0x4f // cmn al r1 r4 LSL 17
};
const byte kInstruction_cmn_al_r11_r4_ROR_16[] = {
  0x1b, 0xeb, 0x34, 0x4f // cmn al r11 r4 ROR 16
};
const byte kInstruction_cmn_al_r1_r1_LSL_31[] = {
  0x11, 0xeb, 0xc1, 0x7f // cmn al r1 r1 LSL 31
};
const byte kInstruction_cmn_al_r11_r0_ROR_15[] = {
  0x1b, 0xeb, 0xf0, 0x3f // cmn al r11 r0 ROR 15
};
const byte kInstruction_cmn_al_r1_r12_LSL_10[] = {
  0x11, 0xeb, 0x8c, 0x2f // cmn al r1 r12 LSL 10
};
const byte kInstruction_cmn_al_r0_r12_LSL_11[] = {
  0x10, 0xeb, 0xcc, 0x2f // cmn al r0 r12 LSL 11
};
const byte kInstruction_cmn_al_r10_r10_ROR_8[] = {
  0x1a, 0xeb, 0x3a, 0x2f // cmn al r10 r10 ROR 8
};
const byte kInstruction_cmn_al_r2_r5_ROR_29[] = {
  0x12, 0xeb, 0x75, 0x7f // cmn al r2 r5 ROR 29
};
const byte kInstruction_cmn_al_r7_r7_LSL_9[] = {
  0x17, 0xeb, 0x47, 0x2f // cmn al r7 r7 LSL 9
};
const byte kInstruction_cmn_al_r9_r1_LSL_11[] = {
  0x19, 0xeb, 0xc1, 0x2f // cmn al r9 r1 LSL 11
};
const byte kInstruction_cmn_al_r5_r8_ROR_14[] = {
  0x15, 0xeb, 0xb8, 0x3f // cmn al r5 r8 ROR 14
};
const byte kInstruction_cmn_al_r1_r6_ROR_25[] = {
  0x11, 0xeb, 0x76, 0x6f // cmn al r1 r6 ROR 25
};
const byte kInstruction_cmn_al_r13_r2_LSL_7[] = {
  0x1d, 0xeb, 0xc2, 0x1f // cmn al r13 r2 LSL 7
};
const byte kInstruction_cmn_al_r0_r3_ROR_24[] = {
  0x10, 0xeb, 0x33, 0x6f // cmn al r0 r3 ROR 24
};
const byte kInstruction_cmn_al_r4_r5_ROR_31[] = {
  0x14, 0xeb, 0xf5, 0x7f // cmn al r4 r5 ROR 31
};
const byte kInstruction_cmn_al_r0_r13_ROR_28[] = {
  0x10, 0xeb, 0x3d, 0x7f // cmn al r0 r13 ROR 28
};
const byte kInstruction_cmn_al_r11_r3_LSL_5[] = {
  0x1b, 0xeb, 0x43, 0x1f // cmn al r11 r3 LSL 5
};
const byte kInstruction_cmn_al_r3_r1_LSL_1[] = {
  0x13, 0xeb, 0x41, 0x0f // cmn al r3 r1 LSL 1
};
const byte kInstruction_cmn_al_r10_r7_ROR_8[] = {
  0x1a, 0xeb, 0x37, 0x2f // cmn al r10 r7 ROR 8
};
const byte kInstruction_cmn_al_r0_r13_LSL_17[] = {
  0x10, 0xeb, 0x4d, 0x4f // cmn al r0 r13 LSL 17
};
const byte kInstruction_cmn_al_r10_r3_LSL_16[] = {
  0x1a, 0xeb, 0x03, 0x4f // cmn al r10 r3 LSL 16
};
const byte kInstruction_cmn_al_r14_r14_LSL_21[] = {
  0x1e, 0xeb, 0x4e, 0x5f // cmn al r14 r14 LSL 21
};
const byte kInstruction_cmn_al_r0_r7_ROR_12[] = {
  0x10, 0xeb, 0x37, 0x3f // cmn al r0 r7 ROR 12
};
const byte kInstruction_cmn_al_r4_r11_LSL_1[] = {
  0x14, 0xeb, 0x4b, 0x0f // cmn al r4 r11 LSL 1
};
const byte kInstruction_cmn_al_r4_r10_ROR_25[] = {
  0x14, 0xeb, 0x7a, 0x6f // cmn al r4 r10 ROR 25
};
const byte kInstruction_cmn_al_r11_r7_LSL_10[] = {
  0x1b, 0xeb, 0x87, 0x2f // cmn al r11 r7 LSL 10
};
const byte kInstruction_cmn_al_r4_r13_LSL_13[] = {
  0x14, 0xeb, 0x4d, 0x3f // cmn al r4 r13 LSL 13
};
const byte kInstruction_cmn_al_r11_r4_ROR_9[] = {
  0x1b, 0xeb, 0x74, 0x2f // cmn al r11 r4 ROR 9
};
const byte kInstruction_cmn_al_r8_r3_LSL_4[] = {
  0x18, 0xeb, 0x03, 0x1f // cmn al r8 r3 LSL 4
};
const byte kInstruction_cmn_al_r0_r3_LSL_22[] = {
  0x10, 0xeb, 0x83, 0x5f // cmn al r0 r3 LSL 22
};
const byte kInstruction_cmn_al_r9_r3_LSL_16[] = {
  0x19, 0xeb, 0x03, 0x4f // cmn al r9 r3 LSL 16
};
const byte kInstruction_cmn_al_r1_r11_LSL_24[] = {
  0x11, 0xeb, 0x0b, 0x6f // cmn al r1 r11 LSL 24
};
const byte kInstruction_cmn_al_r0_r7_LSL_11[] = {
  0x10, 0xeb, 0xc7, 0x2f // cmn al r0 r7 LSL 11
};
const byte kInstruction_cmn_al_r13_r8_LSL_29[] = {
  0x1d, 0xeb, 0x48, 0x7f // cmn al r13 r8 LSL 29
};
const byte kInstruction_cmn_al_r8_r8_LSL_1[] = {
  0x18, 0xeb, 0x48, 0x0f // cmn al r8 r8 LSL 1
};
const byte kInstruction_cmn_al_r8_r1_LSL_21[] = {
  0x18, 0xeb, 0x41, 0x5f // cmn al r8 r1 LSL 21
};
const byte kInstruction_cmn_al_r1_r3_LSL_27[] = {
  0x11, 0xeb, 0xc3, 0x6f // cmn al r1 r3 LSL 27
};
const byte kInstruction_cmn_al_r1_r8_ROR_2[] = {
  0x11, 0xeb, 0xb8, 0x0f // cmn al r1 r8 ROR 2
};
const byte kInstruction_cmn_al_r9_r4_ROR_9[] = {
  0x19, 0xeb, 0x74, 0x2f // cmn al r9 r4 ROR 9
};
const byte kInstruction_cmn_al_r10_r3_ROR_28[] = {
  0x1a, 0xeb, 0x33, 0x7f // cmn al r10 r3 ROR 28
};
const byte kInstruction_cmn_al_r4_r8_ROR_11[] = {
  0x14, 0xeb, 0xf8, 0x2f // cmn al r4 r8 ROR 11
};
const byte kInstruction_cmn_al_r2_r3_LSL_17[] = {
  0x12, 0xeb, 0x43, 0x4f // cmn al r2 r3 LSL 17
};
const byte kInstruction_cmn_al_r5_r6_ROR_17[] = {
  0x15, 0xeb, 0x76, 0x4f // cmn al r5 r6 ROR 17
};
const byte kInstruction_cmn_al_r8_r9_LSL_6[] = {
  0x18, 0xeb, 0x89, 0x1f // cmn al r8 r9 LSL 6
};
const byte kInstruction_cmn_al_r9_r11_ROR_29[] = {
  0x19, 0xeb, 0x7b, 0x7f // cmn al r9 r11 ROR 29
};
const byte kInstruction_cmn_al_r11_r12_ROR_20[] = {
  0x1b, 0xeb, 0x3c, 0x5f // cmn al r11 r12 ROR 20
};
const byte kInstruction_cmn_al_r8_r13_LSL_21[] = {
  0x18, 0xeb, 0x4d, 0x5f // cmn al r8 r13 LSL 21
};
const byte kInstruction_cmn_al_r8_r9_LSL_22[] = {
  0x18, 0xeb, 0x89, 0x5f // cmn al r8 r9 LSL 22
};
const byte kInstruction_cmn_al_r11_r1_ROR_14[] = {
  0x1b, 0xeb, 0xb1, 0x3f // cmn al r11 r1 ROR 14
};
const byte kInstruction_cmn_al_r13_r3_LSL_15[] = {
  0x1d, 0xeb, 0xc3, 0x3f // cmn al r13 r3 LSL 15
};
const byte kInstruction_cmn_al_r3_r7_LSL_9[] = {
  0x13, 0xeb, 0x47, 0x2f // cmn al r3 r7 LSL 9
};
const byte kInstruction_cmn_al_r5_r4_ROR_10[] = {
  0x15, 0xeb, 0xb4, 0x2f // cmn al r5 r4 ROR 10
};
const byte kInstruction_cmn_al_r11_r10_LSL_24[] = {
  0x1b, 0xeb, 0x0a, 0x6f // cmn al r11 r10 LSL 24
};
const byte kInstruction_cmn_al_r1_r9_ROR_25[] = {
  0x11, 0xeb, 0x79, 0x6f // cmn al r1 r9 ROR 25
};
const byte kInstruction_cmn_al_r9_r2_ROR_5[] = {
  0x19, 0xeb, 0x72, 0x1f // cmn al r9 r2 ROR 5
};
const byte kInstruction_cmn_al_r5_r4_ROR_19[] = {
  0x15, 0xeb, 0xf4, 0x4f // cmn al r5 r4 ROR 19
};
const byte kInstruction_cmn_al_r1_r14_ROR_19[] = {
  0x11, 0xeb, 0xfe, 0x4f // cmn al r1 r14 ROR 19
};
const byte kInstruction_cmn_al_r10_r9_LSL_30[] = {
  0x1a, 0xeb, 0x89, 0x7f // cmn al r10 r9 LSL 30
};
const byte kInstruction_cmn_al_r9_r4_ROR_24[] = {
  0x19, 0xeb, 0x34, 0x6f // cmn al r9 r4 ROR 24
};
const byte kInstruction_cmn_al_r6_r6_LSL_23[] = {
  0x16, 0xeb, 0xc6, 0x5f // cmn al r6 r6 LSL 23
};
const byte kInstruction_cmn_al_r5_r7_LSL_29[] = {
  0x15, 0xeb, 0x47, 0x7f // cmn al r5 r7 LSL 29
};
const byte kInstruction_cmn_al_r10_r5_LSL_10[] = {
  0x1a, 0xeb, 0x85, 0x2f // cmn al r10 r5 LSL 10
};
const byte kInstruction_cmn_al_r13_r8_LSL_15[] = {
  0x1d, 0xeb, 0xc8, 0x3f // cmn al r13 r8 LSL 15
};
const byte kInstruction_cmn_al_r5_r2_ROR_14[] = {
  0x15, 0xeb, 0xb2, 0x3f // cmn al r5 r2 ROR 14
};
const byte kInstruction_cmn_al_r11_r3_ROR_6[] = {
  0x1b, 0xeb, 0xb3, 0x1f // cmn al r11 r3 ROR 6
};
const byte kInstruction_cmn_al_r12_r7_ROR_9[] = {
  0x1c, 0xeb, 0x77, 0x2f // cmn al r12 r7 ROR 9
};
const byte kInstruction_cmn_al_r1_r3_ROR_23[] = {
  0x11, 0xeb, 0xf3, 0x5f // cmn al r1 r3 ROR 23
};
const byte kInstruction_cmn_al_r10_r13_LSL_30[] = {
  0x1a, 0xeb, 0x8d, 0x7f // cmn al r10 r13 LSL 30
};
const byte kInstruction_cmn_al_r3_r2_LSL_15[] = {
  0x13, 0xeb, 0xc2, 0x3f // cmn al r3 r2 LSL 15
};
const byte kInstruction_cmn_al_r6_r0_LSL_31[] = {
  0x16, 0xeb, 0xc0, 0x7f // cmn al r6 r0 LSL 31
};
const byte kInstruction_cmn_al_r13_r12_LSL_12[] = {
  0x1d, 0xeb, 0x0c, 0x3f // cmn al r13 r12 LSL 12
};
const byte kInstruction_cmn_al_r7_r8_ROR_9[] = {
  0x17, 0xeb, 0x78, 0x2f // cmn al r7 r8 ROR 9
};
const byte kInstruction_cmn_al_r7_r13_LSL_16[] = {
  0x17, 0xeb, 0x0d, 0x4f // cmn al r7 r13 LSL 16
};
const byte kInstruction_cmn_al_r3_r6_LSL_17[] = {
  0x13, 0xeb, 0x46, 0x4f // cmn al r3 r6 LSL 17
};
const byte kInstruction_cmn_al_r13_r4_ROR_14[] = {
  0x1d, 0xeb, 0xb4, 0x3f // cmn al r13 r4 ROR 14
};
const byte kInstruction_cmn_al_r2_r5_ROR_7[] = {
  0x12, 0xeb, 0xf5, 0x1f // cmn al r2 r5 ROR 7
};
const byte kInstruction_cmn_al_r7_r10_ROR_18[] = {
  0x17, 0xeb, 0xba, 0x4f // cmn al r7 r10 ROR 18
};
const byte kInstruction_cmn_al_r7_r12_LSL_26[] = {
  0x17, 0xeb, 0x8c, 0x6f // cmn al r7 r12 LSL 26
};
const byte kInstruction_cmn_al_r13_r13_ROR_25[] = {
  0x1d, 0xeb, 0x7d, 0x6f // cmn al r13 r13 ROR 25
};
const byte kInstruction_cmn_al_r11_r2_ROR_5[] = {
  0x1b, 0xeb, 0x72, 0x1f // cmn al r11 r2 ROR 5
};
const byte kInstruction_cmn_al_r0_r14_ROR_8[] = {
  0x10, 0xeb, 0x3e, 0x2f // cmn al r0 r14 ROR 8
};
const byte kInstruction_cmn_al_r6_r2_ROR_27[] = {
  0x16, 0xeb, 0xf2, 0x6f // cmn al r6 r2 ROR 27
};
const byte kInstruction_cmn_al_r4_r14_LSL_31[] = {
  0x14, 0xeb, 0xce, 0x7f // cmn al r4 r14 LSL 31
};
const byte kInstruction_cmn_al_r1_r0_LSL_7[] = {
  0x11, 0xeb, 0xc0, 0x1f // cmn al r1 r0 LSL 7
};
const byte kInstruction_cmn_al_r0_r8_LSL_10[] = {
  0x10, 0xeb, 0x88, 0x2f // cmn al r0 r8 LSL 10
};
const byte kInstruction_cmn_al_r9_r0_LSL_13[] = {
  0x19, 0xeb, 0x40, 0x3f // cmn al r9 r0 LSL 13
};
const byte kInstruction_cmn_al_r11_r11_ROR_11[] = {
  0x1b, 0xeb, 0xfb, 0x2f // cmn al r11 r11 ROR 11
};
const byte kInstruction_cmn_al_r11_r4_ROR_7[] = {
  0x1b, 0xeb, 0xf4, 0x1f // cmn al r11 r4 ROR 7
};
const byte kInstruction_cmn_al_r10_r4_ROR_19[] = {
  0x1a, 0xeb, 0xf4, 0x4f // cmn al r10 r4 ROR 19
};
const byte kInstruction_cmn_al_r1_r13_ROR_4[] = {
  0x11, 0xeb, 0x3d, 0x1f // cmn al r1 r13 ROR 4
};
const byte kInstruction_cmn_al_r9_r14_ROR_5[] = {
  0x19, 0xeb, 0x7e, 0x1f // cmn al r9 r14 ROR 5
};
const byte kInstruction_cmn_al_r14_r11_LSL_22[] = {
  0x1e, 0xeb, 0x8b, 0x5f // cmn al r14 r11 LSL 22
};
const byte kInstruction_cmn_al_r2_r4_LSL_17[] = {
  0x12, 0xeb, 0x44, 0x4f // cmn al r2 r4 LSL 17
};
const byte kInstruction_cmn_al_r2_r6_LSL_15[] = {
  0x12, 0xeb, 0xc6, 0x3f // cmn al r2 r6 LSL 15
};
const byte kInstruction_cmn_al_r10_r5_ROR_28[] = {
  0x1a, 0xeb, 0x35, 0x7f // cmn al r10 r5 ROR 28
};
const byte kInstruction_cmn_al_r6_r4_ROR_13[] = {
  0x16, 0xeb, 0x74, 0x3f // cmn al r6 r4 ROR 13
};
const byte kInstruction_cmn_al_r3_r4_ROR_10[] = {
  0x13, 0xeb, 0xb4, 0x2f // cmn al r3 r4 ROR 10
};
const byte kInstruction_cmn_al_r3_r7_ROR_15[] = {
  0x13, 0xeb, 0xf7, 0x3f // cmn al r3 r7 ROR 15
};
const byte kInstruction_cmn_al_r7_r14_LSL_28[] = {
  0x17, 0xeb, 0x0e, 0x7f // cmn al r7 r14 LSL 28
};
const byte kInstruction_cmn_al_r10_r6_ROR_23[] = {
  0x1a, 0xeb, 0xf6, 0x5f // cmn al r10 r6 ROR 23
};
const byte kInstruction_cmn_al_r4_r5_ROR_4[] = {
  0x14, 0xeb, 0x35, 0x1f // cmn al r4 r5 ROR 4
};
const byte kInstruction_cmn_al_r9_r3_ROR_8[] = {
  0x19, 0xeb, 0x33, 0x2f // cmn al r9 r3 ROR 8
};
const byte kInstruction_cmn_al_r2_r1_LSL_14[] = {
  0x12, 0xeb, 0x81, 0x3f // cmn al r2 r1 LSL 14
};
const byte kInstruction_cmn_al_r10_r7_LSL_4[] = {
  0x1a, 0xeb, 0x07, 0x1f // cmn al r10 r7 LSL 4
};
const byte kInstruction_cmn_al_r0_r0_LSL_18[] = {
  0x10, 0xeb, 0x80, 0x4f // cmn al r0 r0 LSL 18
};
const byte kInstruction_cmn_al_r12_r3_ROR_18[] = {
  0x1c, 0xeb, 0xb3, 0x4f // cmn al r12 r3 ROR 18
};
const byte kInstruction_cmn_al_r5_r5_LSL_4[] = {
  0x15, 0xeb, 0x05, 0x1f // cmn al r5 r5 LSL 4
};
const byte kInstruction_cmn_al_r11_r4_ROR_12[] = {
  0x1b, 0xeb, 0x34, 0x3f // cmn al r11 r4 ROR 12
};
const byte kInstruction_cmn_al_r5_r14_ROR_23[] = {
  0x15, 0xeb, 0xfe, 0x5f // cmn al r5 r14 ROR 23
};
const byte kInstruction_cmn_al_r7_r6_LSL_31[] = {
  0x17, 0xeb, 0xc6, 0x7f // cmn al r7 r6 LSL 31
};
const byte kInstruction_cmn_al_r14_r2_ROR_7[] = {
  0x1e, 0xeb, 0xf2, 0x1f // cmn al r14 r2 ROR 7
};
const byte kInstruction_cmn_al_r10_r0_ROR_23[] = {
  0x1a, 0xeb, 0xf0, 0x5f // cmn al r10 r0 ROR 23
};
const byte kInstruction_cmn_al_r6_r0_LSL_25[] = {
  0x16, 0xeb, 0x40, 0x6f // cmn al r6 r0 LSL 25
};
const byte kInstruction_cmn_al_r7_r4_ROR_16[] = {
  0x17, 0xeb, 0x34, 0x4f // cmn al r7 r4 ROR 16
};
const byte kInstruction_cmn_al_r9_r0_LSL_19[] = {
  0x19, 0xeb, 0xc0, 0x4f // cmn al r9 r0 LSL 19
};
const byte kInstruction_cmn_al_r7_r2_LSL_23[] = {
  0x17, 0xeb, 0xc2, 0x5f // cmn al r7 r2 LSL 23
};
const byte kInstruction_cmn_al_r7_r10_LSL_17[] = {
  0x17, 0xeb, 0x4a, 0x4f // cmn al r7 r10 LSL 17
};
const byte kInstruction_cmn_al_r8_r10_ROR_19[] = {
  0x18, 0xeb, 0xfa, 0x4f // cmn al r8 r10 ROR 19
};
const byte kInstruction_cmn_al_r6_r9_LSL_19[] = {
  0x16, 0xeb, 0xc9, 0x4f // cmn al r6 r9 LSL 19
};
const byte kInstruction_cmn_al_r12_r6_LSL_14[] = {
  0x1c, 0xeb, 0x86, 0x3f // cmn al r12 r6 LSL 14
};
const byte kInstruction_cmn_al_r4_r12_ROR_24[] = {
  0x14, 0xeb, 0x3c, 0x6f // cmn al r4 r12 ROR 24
};
const byte kInstruction_cmn_al_r3_r6_LSL_6[] = {
  0x13, 0xeb, 0x86, 0x1f // cmn al r3 r6 LSL 6
};
const byte kInstruction_cmn_al_r11_r6_ROR_4[] = {
  0x1b, 0xeb, 0x36, 0x1f // cmn al r11 r6 ROR 4
};
const byte kInstruction_cmn_al_r14_r3_LSL_10[] = {
  0x1e, 0xeb, 0x83, 0x2f // cmn al r14 r3 LSL 10
};
const byte kInstruction_cmn_al_r1_r1_LSL_12[] = {
  0x11, 0xeb, 0x01, 0x3f // cmn al r1 r1 LSL 12
};
const byte kInstruction_cmn_al_r7_r8_LSL_19[] = {
  0x17, 0xeb, 0xc8, 0x4f // cmn al r7 r8 LSL 19
};
const byte kInstruction_cmn_al_r5_r5_LSL_22[] = {
  0x15, 0xeb, 0x85, 0x5f // cmn al r5 r5 LSL 22
};
const byte kInstruction_cmn_al_r14_r8_ROR_29[] = {
  0x1e, 0xeb, 0x78, 0x7f // cmn al r14 r8 ROR 29
};
const byte kInstruction_cmn_al_r3_r14_LSL_24[] = {
  0x13, 0xeb, 0x0e, 0x6f // cmn al r3 r14 LSL 24
};
const byte kInstruction_cmn_al_r11_r14_ROR_5[] = {
  0x1b, 0xeb, 0x7e, 0x1f // cmn al r11 r14 ROR 5
};
const byte kInstruction_cmn_al_r12_r4_ROR_15[] = {
  0x1c, 0xeb, 0xf4, 0x3f // cmn al r12 r4 ROR 15
};
const byte kInstruction_cmn_al_r3_r5_LSL_12[] = {
  0x13, 0xeb, 0x05, 0x3f // cmn al r3 r5 LSL 12
};
const byte kInstruction_cmn_al_r0_r12_ROR_26[] = {
  0x10, 0xeb, 0xbc, 0x6f // cmn al r0 r12 ROR 26
};
const byte kInstruction_cmn_al_r5_r7_ROR_12[] = {
  0x15, 0xeb, 0x37, 0x3f // cmn al r5 r7 ROR 12
};
const byte kInstruction_cmn_al_r8_r9_LSL_17[] = {
  0x18, 0xeb, 0x49, 0x4f // cmn al r8 r9 LSL 17
};
const byte kInstruction_cmn_al_r1_r2_ROR_13[] = {
  0x11, 0xeb, 0x72, 0x3f // cmn al r1 r2 ROR 13
};
const byte kInstruction_cmn_al_r0_r13_ROR_8[] = {
  0x10, 0xeb, 0x3d, 0x2f // cmn al r0 r13 ROR 8
};
const byte kInstruction_cmn_al_r4_r0_LSL_9[] = {
  0x14, 0xeb, 0x40, 0x2f // cmn al r4 r0 LSL 9
};
const byte kInstruction_cmn_al_r6_r2_LSL_21[] = {
  0x16, 0xeb, 0x42, 0x5f // cmn al r6 r2 LSL 21
};
const byte kInstruction_cmn_al_r5_r3_ROR_29[] = {
  0x15, 0xeb, 0x73, 0x7f // cmn al r5 r3 ROR 29
};
const byte kInstruction_cmn_al_r14_r6_LSL_7[] = {
  0x1e, 0xeb, 0xc6, 0x1f // cmn al r14 r6 LSL 7
};
const byte kInstruction_cmn_al_r9_r4_ROR_11[] = {
  0x19, 0xeb, 0xf4, 0x2f // cmn al r9 r4 ROR 11
};
const byte kInstruction_cmn_al_r4_r0_LSL_14[] = {
  0x14, 0xeb, 0x80, 0x3f // cmn al r4 r0 LSL 14
};
const byte kInstruction_cmn_al_r10_r14_ROR_3[] = {
  0x1a, 0xeb, 0xfe, 0x0f // cmn al r10 r14 ROR 3
};
const byte kInstruction_cmn_al_r0_r11_LSL_31[] = {
  0x10, 0xeb, 0xcb, 0x7f // cmn al r0 r11 LSL 31
};
const byte kInstruction_cmn_al_r5_r2_LSL_17[] = {
  0x15, 0xeb, 0x42, 0x4f // cmn al r5 r2 LSL 17
};
const byte kInstruction_cmn_al_r2_r6_ROR_26[] = {
  0x12, 0xeb, 0xb6, 0x6f // cmn al r2 r6 ROR 26
};
const byte kInstruction_cmn_al_r9_r9_LSL_8[] = {
  0x19, 0xeb, 0x09, 0x2f // cmn al r9 r9 LSL 8
};
const byte kInstruction_cmn_al_r12_r12_LSL_19[] = {
  0x1c, 0xeb, 0xcc, 0x4f // cmn al r12 r12 LSL 19
};
const byte kInstruction_cmn_al_r2_r11_LSL_10[] = {
  0x12, 0xeb, 0x8b, 0x2f // cmn al r2 r11 LSL 10
};
const byte kInstruction_cmn_al_r4_r7_ROR_6[] = {
  0x14, 0xeb, 0xb7, 0x1f // cmn al r4 r7 ROR 6
};
const byte kInstruction_cmn_al_r2_r3_ROR_14[] = {
  0x12, 0xeb, 0xb3, 0x3f // cmn al r2 r3 ROR 14
};
const byte kInstruction_cmn_al_r4_r12_ROR_16[] = {
  0x14, 0xeb, 0x3c, 0x4f // cmn al r4 r12 ROR 16
};
const byte kInstruction_cmn_al_r12_r12_LSL_2[] = {
  0x1c, 0xeb, 0x8c, 0x0f // cmn al r12 r12 LSL 2
};
const byte kInstruction_cmn_al_r10_r5_ROR_23[] = {
  0x1a, 0xeb, 0xf5, 0x5f // cmn al r10 r5 ROR 23
};
const byte kInstruction_cmn_al_r8_r14_ROR_24[] = {
  0x18, 0xeb, 0x3e, 0x6f // cmn al r8 r14 ROR 24
};
const byte kInstruction_cmn_al_r7_r1_LSL_15[] = {
  0x17, 0xeb, 0xc1, 0x3f // cmn al r7 r1 LSL 15
};
const byte kInstruction_cmn_al_r13_r10_LSL_8[] = {
  0x1d, 0xeb, 0x0a, 0x2f // cmn al r13 r10 LSL 8
};
const byte kInstruction_cmn_al_r1_r9_ROR_5[] = {
  0x11, 0xeb, 0x79, 0x1f // cmn al r1 r9 ROR 5
};
const byte kInstruction_cmn_al_r1_r2_ROR_17[] = {
  0x11, 0xeb, 0x72, 0x4f // cmn al r1 r2 ROR 17
};
const byte kInstruction_cmn_al_r12_r3_ROR_30[] = {
  0x1c, 0xeb, 0xb3, 0x7f // cmn al r12 r3 ROR 30
};
const byte kInstruction_cmn_al_r0_r5_LSL_18[] = {
  0x10, 0xeb, 0x85, 0x4f // cmn al r0 r5 LSL 18
};
const byte kInstruction_cmn_al_r5_r7_LSL_2[] = {
  0x15, 0xeb, 0x87, 0x0f // cmn al r5 r7 LSL 2
};
const byte kInstruction_cmn_al_r13_r4_LSL_30[] = {
  0x1d, 0xeb, 0x84, 0x7f // cmn al r13 r4 LSL 30
};
const byte kInstruction_cmn_al_r2_r14_ROR_31[] = {
  0x12, 0xeb, 0xfe, 0x7f // cmn al r2 r14 ROR 31
};
const byte kInstruction_cmn_al_r12_r8_LSL_30[] = {
  0x1c, 0xeb, 0x88, 0x7f // cmn al r12 r8 LSL 30
};
const byte kInstruction_cmn_al_r3_r5_ROR_4[] = {
  0x13, 0xeb, 0x35, 0x1f // cmn al r3 r5 ROR 4
};
const byte kInstruction_cmn_al_r11_r11_ROR_3[] = {
  0x1b, 0xeb, 0xfb, 0x0f // cmn al r11 r11 ROR 3
};
const byte kInstruction_cmn_al_r4_r2_ROR_17[] = {
  0x14, 0xeb, 0x72, 0x4f // cmn al r4 r2 ROR 17
};
const byte kInstruction_cmn_al_r6_r9_LSL_15[] = {
  0x16, 0xeb, 0xc9, 0x3f // cmn al r6 r9 LSL 15
};
const byte kInstruction_cmn_al_r5_r6_ROR_25[] = {
  0x15, 0xeb, 0x76, 0x6f // cmn al r5 r6 ROR 25
};
const byte kInstruction_cmn_al_r6_r3_LSL_19[] = {
  0x16, 0xeb, 0xc3, 0x4f // cmn al r6 r3 LSL 19
};
const byte kInstruction_cmn_al_r3_r7_LSL_6[] = {
  0x13, 0xeb, 0x87, 0x1f // cmn al r3 r7 LSL 6
};
const byte kInstruction_cmn_al_r12_r1_LSL_13[] = {
  0x1c, 0xeb, 0x41, 0x3f // cmn al r12 r1 LSL 13
};
const byte kInstruction_cmn_al_r2_r1_ROR_9[] = {
  0x12, 0xeb, 0x71, 0x2f // cmn al r2 r1 ROR 9
};
const byte kInstruction_cmn_al_r12_r4_LSL_28[] = {
  0x1c, 0xeb, 0x04, 0x7f // cmn al r12 r4 LSL 28
};
const byte kInstruction_cmn_al_r14_r13_ROR_12[] = {
  0x1e, 0xeb, 0x3d, 0x3f // cmn al r14 r13 ROR 12
};
const byte kInstruction_cmn_al_r11_r12_LSL_16[] = {
  0x1b, 0xeb, 0x0c, 0x4f // cmn al r11 r12 LSL 16
};
const byte kInstruction_cmn_al_r8_r7_LSL_29[] = {
  0x18, 0xeb, 0x47, 0x7f // cmn al r8 r7 LSL 29
};
const byte kInstruction_cmn_al_r0_r10_ROR_25[] = {
  0x10, 0xeb, 0x7a, 0x6f // cmn al r0 r10 ROR 25
};
const byte kInstruction_cmn_al_r9_r6_LSL_3[] = {
  0x19, 0xeb, 0xc6, 0x0f // cmn al r9 r6 LSL 3
};
const byte kInstruction_cmn_al_r9_r12_LSL_18[] = {
  0x19, 0xeb, 0x8c, 0x4f // cmn al r9 r12 LSL 18
};
const byte kInstruction_cmn_al_r1_r14_LSL_9[] = {
  0x11, 0xeb, 0x4e, 0x2f // cmn al r1 r14 LSL 9
};
const byte kInstruction_cmn_al_r3_r2_ROR_7[] = {
  0x13, 0xeb, 0xf2, 0x1f // cmn al r3 r2 ROR 7
};
const byte kInstruction_cmn_al_r14_r9_ROR_16[] = {
  0x1e, 0xeb, 0x39, 0x4f // cmn al r14 r9 ROR 16
};
const byte kInstruction_cmn_al_r6_r12_LSL_2[] = {
  0x16, 0xeb, 0x8c, 0x0f // cmn al r6 r12 LSL 2
};
const byte kInstruction_cmn_al_r12_r7_LSL_14[] = {
  0x1c, 0xeb, 0x87, 0x3f // cmn al r12 r7 LSL 14
};
const byte kInstruction_cmn_al_r12_r10_LSL_8[] = {
  0x1c, 0xeb, 0x0a, 0x2f // cmn al r12 r10 LSL 8
};
const byte kInstruction_cmn_al_r4_r13_ROR_27[] = {
  0x14, 0xeb, 0xfd, 0x6f // cmn al r4 r13 ROR 27
};
const byte kInstruction_cmn_al_r5_r5_ROR_20[] = {
  0x15, 0xeb, 0x35, 0x5f // cmn al r5 r5 ROR 20
};
const byte kInstruction_cmn_al_r1_r5_ROR_14[] = {
  0x11, 0xeb, 0xb5, 0x3f // cmn al r1 r5 ROR 14
};
const byte kInstruction_cmn_al_r6_r2_ROR_28[] = {
  0x16, 0xeb, 0x32, 0x7f // cmn al r6 r2 ROR 28
};
const byte kInstruction_cmn_al_r1_r8_LSL_9[] = {
  0x11, 0xeb, 0x48, 0x2f // cmn al r1 r8 LSL 9
};
const byte kInstruction_cmn_al_r2_r10_LSL_16[] = {
  0x12, 0xeb, 0x0a, 0x4f // cmn al r2 r10 LSL 16
};
const byte kInstruction_cmn_al_r5_r1_LSL_30[] = {
  0x15, 0xeb, 0x81, 0x7f // cmn al r5 r1 LSL 30
};
const byte kInstruction_cmn_al_r5_r3_ROR_19[] = {
  0x15, 0xeb, 0xf3, 0x4f // cmn al r5 r3 ROR 19
};
const byte kInstruction_cmn_al_r1_r2_LSL_7[] = {
  0x11, 0xeb, 0xc2, 0x1f // cmn al r1 r2 LSL 7
};
const byte kInstruction_cmn_al_r7_r10_ROR_11[] = {
  0x17, 0xeb, 0xfa, 0x2f // cmn al r7 r10 ROR 11
};
const byte kInstruction_cmn_al_r0_r10_LSL_10[] = {
  0x10, 0xeb, 0x8a, 0x2f // cmn al r0 r10 LSL 10
};
const byte kInstruction_cmn_al_r7_r5_ROR_1[] = {
  0x17, 0xeb, 0x75, 0x0f // cmn al r7 r5 ROR 1
};
const byte kInstruction_cmn_al_r0_r1_LSL_3[] = {
  0x10, 0xeb, 0xc1, 0x0f // cmn al r0 r1 LSL 3
};
const byte kInstruction_cmn_al_r11_r0_LSL_24[] = {
  0x1b, 0xeb, 0x00, 0x6f // cmn al r11 r0 LSL 24
};
const byte kInstruction_cmn_al_r3_r7_LSL_29[] = {
  0x13, 0xeb, 0x47, 0x7f // cmn al r3 r7 LSL 29
};
const byte kInstruction_cmn_al_r5_r3_ROR_28[] = {
  0x15, 0xeb, 0x33, 0x7f // cmn al r5 r3 ROR 28
};
const byte kInstruction_cmn_al_r3_r4_LSL_26[] = {
  0x13, 0xeb, 0x84, 0x6f // cmn al r3 r4 LSL 26
};
const byte kInstruction_cmn_al_r8_r10_LSL_30[] = {
  0x18, 0xeb, 0x8a, 0x7f // cmn al r8 r10 LSL 30
};
const byte kInstruction_cmn_al_r5_r5_ROR_7[] = {
  0x15, 0xeb, 0xf5, 0x1f // cmn al r5 r5 ROR 7
};
const byte kInstruction_cmn_al_r6_r7_ROR_13[] = {
  0x16, 0xeb, 0x77, 0x3f // cmn al r6 r7 ROR 13
};
const byte kInstruction_cmn_al_r7_r1_ROR_20[] = {
  0x17, 0xeb, 0x31, 0x5f // cmn al r7 r1 ROR 20
};
const byte kInstruction_cmn_al_r11_r11_LSL_1[] = {
  0x1b, 0xeb, 0x4b, 0x0f // cmn al r11 r11 LSL 1
};
const byte kInstruction_cmn_al_r10_r13_LSL_1[] = {
  0x1a, 0xeb, 0x4d, 0x0f // cmn al r10 r13 LSL 1
};
const byte kInstruction_cmn_al_r6_r11_LSL_9[] = {
  0x16, 0xeb, 0x4b, 0x2f // cmn al r6 r11 LSL 9
};
const byte kInstruction_cmn_al_r2_r3_LSL_18[] = {
  0x12, 0xeb, 0x83, 0x4f // cmn al r2 r3 LSL 18
};
const byte kInstruction_cmn_al_r3_r4_ROR_16[] = {
  0x13, 0xeb, 0x34, 0x4f // cmn al r3 r4 ROR 16
};
const byte kInstruction_cmn_al_r4_r10_LSL_1[] = {
  0x14, 0xeb, 0x4a, 0x0f // cmn al r4 r10 LSL 1
};
const byte kInstruction_cmn_al_r1_r5_LSL_31[] = {
  0x11, 0xeb, 0xc5, 0x7f // cmn al r1 r5 LSL 31
};
const byte kInstruction_cmn_al_r6_r6_LSL_29[] = {
  0x16, 0xeb, 0x46, 0x7f // cmn al r6 r6 LSL 29
};
const byte kInstruction_cmn_al_r10_r13_LSL_5[] = {
  0x1a, 0xeb, 0x4d, 0x1f // cmn al r10 r13 LSL 5
};
const byte kInstruction_cmn_al_r14_r6_LSL_24[] = {
  0x1e, 0xeb, 0x06, 0x6f // cmn al r14 r6 LSL 24
};
const byte kInstruction_cmn_al_r4_r12_LSL_31[] = {
  0x14, 0xeb, 0xcc, 0x7f // cmn al r4 r12 LSL 31
};
const byte kInstruction_cmn_al_r12_r7_ROR_6[] = {
  0x1c, 0xeb, 0xb7, 0x1f // cmn al r12 r7 ROR 6
};
const byte kInstruction_cmn_al_r14_r10_LSL_26[] = {
  0x1e, 0xeb, 0x8a, 0x6f // cmn al r14 r10 LSL 26
};
const byte kInstruction_cmn_al_r9_r9_ROR_24[] = {
  0x19, 0xeb, 0x39, 0x6f // cmn al r9 r9 ROR 24
};
const byte kInstruction_cmn_al_r8_r13_LSL_5[] = {
  0x18, 0xeb, 0x4d, 0x1f // cmn al r8 r13 LSL 5
};
const byte kInstruction_cmn_al_r4_r11_LSL_17[] = {
  0x14, 0xeb, 0x4b, 0x4f // cmn al r4 r11 LSL 17
};
const byte kInstruction_cmn_al_r2_r14_LSL_9[] = {
  0x12, 0xeb, 0x4e, 0x2f // cmn al r2 r14 LSL 9
};
const byte kInstruction_cmn_al_r1_r0_ROR_29[] = {
  0x11, 0xeb, 0x70, 0x7f // cmn al r1 r0 ROR 29
};
const byte kInstruction_cmn_al_r10_r2_ROR_9[] = {
  0x1a, 0xeb, 0x72, 0x2f // cmn al r10 r2 ROR 9
};
const byte kInstruction_cmn_al_r3_r7_ROR_29[] = {
  0x13, 0xeb, 0x77, 0x7f // cmn al r3 r7 ROR 29
};
const byte kInstruction_cmn_al_r1_r12_ROR_1[] = {
  0x11, 0xeb, 0x7c, 0x0f // cmn al r1 r12 ROR 1
};
const byte kInstruction_cmn_al_r7_r2_ROR_8[] = {
  0x17, 0xeb, 0x32, 0x2f // cmn al r7 r2 ROR 8
};
const byte kInstruction_cmn_al_r7_r1_LSL_24[] = {
  0x17, 0xeb, 0x01, 0x6f // cmn al r7 r1 LSL 24
};
const byte kInstruction_cmn_al_r4_r1_LSL_17[] = {
  0x14, 0xeb, 0x41, 0x4f // cmn al r4 r1 LSL 17
};
const byte kInstruction_cmn_al_r12_r11_ROR_22[] = {
  0x1c, 0xeb, 0xbb, 0x5f // cmn al r12 r11 ROR 22
};
const byte kInstruction_cmn_al_r0_r4_ROR_20[] = {
  0x10, 0xeb, 0x34, 0x5f // cmn al r0 r4 ROR 20
};
const byte kInstruction_cmn_al_r5_r12_LSL_11[] = {
  0x15, 0xeb, 0xcc, 0x2f // cmn al r5 r12 LSL 11
};
const byte kInstruction_cmn_al_r11_r2_ROR_31[] = {
  0x1b, 0xeb, 0xf2, 0x7f // cmn al r11 r2 ROR 31
};
const byte kInstruction_cmn_al_r14_r11_LSL_4[] = {
  0x1e, 0xeb, 0x0b, 0x1f // cmn al r14 r11 LSL 4
};
const byte kInstruction_cmn_al_r6_r8_LSL_12[] = {
  0x16, 0xeb, 0x08, 0x3f // cmn al r6 r8 LSL 12
};
const byte kInstruction_cmn_al_r14_r0_LSL_30[] = {
  0x1e, 0xeb, 0x80, 0x7f // cmn al r14 r0 LSL 30
};
const byte kInstruction_cmn_al_r6_r11_LSL_29[] = {
  0x16, 0xeb, 0x4b, 0x7f // cmn al r6 r11 LSL 29
};
const byte kInstruction_cmn_al_r6_r12_LSL_21[] = {
  0x16, 0xeb, 0x4c, 0x5f // cmn al r6 r12 LSL 21
};
const byte kInstruction_cmn_al_r9_r6_LSL_29[] = {
  0x19, 0xeb, 0x46, 0x7f // cmn al r9 r6 LSL 29
};
const byte kInstruction_cmn_al_r10_r13_ROR_29[] = {
  0x1a, 0xeb, 0x7d, 0x7f // cmn al r10 r13 ROR 29
};
const byte kInstruction_cmn_al_r5_r7_ROR_16[] = {
  0x15, 0xeb, 0x37, 0x4f // cmn al r5 r7 ROR 16
};
const byte kInstruction_cmn_al_r0_r14_LSL_27[] = {
  0x10, 0xeb, 0xce, 0x6f // cmn al r0 r14 LSL 27
};
const byte kInstruction_cmn_al_r14_r4_ROR_18[] = {
  0x1e, 0xeb, 0xb4, 0x4f // cmn al r14 r4 ROR 18
};
const byte kInstruction_cmn_al_r14_r2_LSL_28[] = {
  0x1e, 0xeb, 0x02, 0x7f // cmn al r14 r2 LSL 28
};
const byte kInstruction_cmn_al_r13_r3_LSL_29[] = {
  0x1d, 0xeb, 0x43, 0x7f // cmn al r13 r3 LSL 29
};
const byte kInstruction_cmn_al_r8_r7_LSL_19[] = {
  0x18, 0xeb, 0xc7, 0x4f // cmn al r8 r7 LSL 19
};
const byte kInstruction_cmn_al_r14_r1_LSL_1[] = {
  0x1e, 0xeb, 0x41, 0x0f // cmn al r14 r1 LSL 1
};
const byte kInstruction_cmn_al_r1_r11_ROR_2[] = {
  0x11, 0xeb, 0xbb, 0x0f // cmn al r1 r11 ROR 2
};
const byte kInstruction_cmn_al_r7_r4_ROR_14[] = {
  0x17, 0xeb, 0xb4, 0x3f // cmn al r7 r4 ROR 14
};
const byte kInstruction_cmn_al_r6_r3_ROR_22[] = {
  0x16, 0xeb, 0xb3, 0x5f // cmn al r6 r3 ROR 22
};
const byte kInstruction_cmn_al_r5_r4_LSL_1[] = {
  0x15, 0xeb, 0x44, 0x0f // cmn al r5 r4 LSL 1
};
const byte kInstruction_cmn_al_r11_r11_LSL_6[] = {
  0x1b, 0xeb, 0x8b, 0x1f // cmn al r11 r11 LSL 6
};
const byte kInstruction_cmn_al_r13_r5_LSL_21[] = {
  0x1d, 0xeb, 0x45, 0x5f // cmn al r13 r5 LSL 21
};
const byte kInstruction_cmn_al_r7_r5_ROR_10[] = {
  0x17, 0xeb, 0xb5, 0x2f // cmn al r7 r5 ROR 10
};
const byte kInstruction_cmn_al_r13_r10_LSL_17[] = {
  0x1d, 0xeb, 0x4a, 0x4f // cmn al r13 r10 LSL 17
};
const byte kInstruction_cmn_al_r7_r1_LSL_5[] = {
  0x17, 0xeb, 0x41, 0x1f // cmn al r7 r1 LSL 5
};
const byte kInstruction_cmn_al_r0_r11_LSL_25[] = {
  0x10, 0xeb, 0x4b, 0x6f // cmn al r0 r11 LSL 25
};
const byte kInstruction_cmn_al_r3_r2_LSL_9[] = {
  0x13, 0xeb, 0x42, 0x2f // cmn al r3 r2 LSL 9
};
const byte kInstruction_cmn_al_r8_r11_ROR_12[] = {
  0x18, 0xeb, 0x3b, 0x3f // cmn al r8 r11 ROR 12
};
const byte kInstruction_cmn_al_r12_r9_LSL_21[] = {
  0x1c, 0xeb, 0x49, 0x5f // cmn al r12 r9 LSL 21
};
const byte kInstruction_cmn_al_r14_r3_ROR_13[] = {
  0x1e, 0xeb, 0x73, 0x3f // cmn al r14 r3 ROR 13
};
const byte kInstruction_cmn_al_r13_r8_ROR_26[] = {
  0x1d, 0xeb, 0xb8, 0x6f // cmn al r13 r8 ROR 26
};
const byte kInstruction_cmn_al_r4_r11_ROR_11[] = {
  0x14, 0xeb, 0xfb, 0x2f // cmn al r4 r11 ROR 11
};
const byte kInstruction_cmn_al_r10_r3_ROR_16[] = {
  0x1a, 0xeb, 0x33, 0x4f // cmn al r10 r3 ROR 16
};
const byte kInstruction_cmn_al_r14_r0_ROR_12[] = {
  0x1e, 0xeb, 0x30, 0x3f // cmn al r14 r0 ROR 12
};
const byte kInstruction_cmn_al_r11_r4_LSL_18[] = {
  0x1b, 0xeb, 0x84, 0x4f // cmn al r11 r4 LSL 18
};
const byte kInstruction_cmn_al_r13_r13_ROR_3[] = {
  0x1d, 0xeb, 0xfd, 0x0f // cmn al r13 r13 ROR 3
};
const byte kInstruction_cmn_al_r10_r14_ROR_10[] = {
  0x1a, 0xeb, 0xbe, 0x2f // cmn al r10 r14 ROR 10
};
const byte kInstruction_cmn_al_r12_r10_LSL_12[] = {
  0x1c, 0xeb, 0x0a, 0x3f // cmn al r12 r10 LSL 12
};
const byte kInstruction_cmn_al_r0_r0_ROR_29[] = {
  0x10, 0xeb, 0x70, 0x7f // cmn al r0 r0 ROR 29
};
const byte kInstruction_cmn_al_r13_r3_LSL_4[] = {
  0x1d, 0xeb, 0x03, 0x1f // cmn al r13 r3 LSL 4
};
const byte kInstruction_cmn_al_r1_r10_LSL_15[] = {
  0x11, 0xeb, 0xca, 0x3f // cmn al r1 r10 LSL 15
};
const byte kInstruction_cmn_al_r3_r10_LSL_18[] = {
  0x13, 0xeb, 0x8a, 0x4f // cmn al r3 r10 LSL 18
};
const byte kInstruction_cmn_al_r11_r14_ROR_13[] = {
  0x1b, 0xeb, 0x7e, 0x3f // cmn al r11 r14 ROR 13
};
const byte kInstruction_cmn_al_r5_r8_ROR_9[] = {
  0x15, 0xeb, 0x78, 0x2f // cmn al r5 r8 ROR 9
};
const byte kInstruction_cmn_al_r1_r7_LSL_3[] = {
  0x11, 0xeb, 0xc7, 0x0f // cmn al r1 r7 LSL 3
};
const byte kInstruction_cmn_al_r8_r9_LSL_16[] = {
  0x18, 0xeb, 0x09, 0x4f // cmn al r8 r9 LSL 16
};
const byte kInstruction_cmn_al_r6_r6_ROR_29[] = {
  0x16, 0xeb, 0x76, 0x7f // cmn al r6 r6 ROR 29
};
const byte kInstruction_cmn_al_r4_r10_ROR_6[] = {
  0x14, 0xeb, 0xba, 0x1f // cmn al r4 r10 ROR 6
};
const byte kInstruction_cmn_al_r13_r7_LSL_8[] = {
  0x1d, 0xeb, 0x07, 0x2f // cmn al r13 r7 LSL 8
};
const byte kInstruction_cmn_al_r0_r0_LSL_26[] = {
  0x10, 0xeb, 0x80, 0x6f // cmn al r0 r0 LSL 26
};
const byte kInstruction_cmn_al_r6_r3_ROR_14[] = {
  0x16, 0xeb, 0xb3, 0x3f // cmn al r6 r3 ROR 14
};
const byte kInstruction_cmn_al_r13_r12_ROR_16[] = {
  0x1d, 0xeb, 0x3c, 0x4f // cmn al r13 r12 ROR 16
};
const byte kInstruction_cmn_al_r0_r14_ROR_16[] = {
  0x10, 0xeb, 0x3e, 0x4f // cmn al r0 r14 ROR 16
};
const byte kInstruction_cmn_al_r2_r13_ROR_3[] = {
  0x12, 0xeb, 0xfd, 0x0f // cmn al r2 r13 ROR 3
};
const byte kInstruction_cmn_al_r3_r5_LSL_31[] = {
  0x13, 0xeb, 0xc5, 0x7f // cmn al r3 r5 LSL 31
};
const byte kInstruction_cmn_al_r12_r10_LSL_14[] = {
  0x1c, 0xeb, 0x8a, 0x3f // cmn al r12 r10 LSL 14
};
const byte kInstruction_cmn_al_r5_r10_LSL_24[] = {
  0x15, 0xeb, 0x0a, 0x6f // cmn al r5 r10 LSL 24
};
const byte kInstruction_cmn_al_r5_r5_LSL_21[] = {
  0x15, 0xeb, 0x45, 0x5f // cmn al r5 r5 LSL 21
};
const byte kInstruction_cmn_al_r1_r1_LSL_23[] = {
  0x11, 0xeb, 0xc1, 0x5f // cmn al r1 r1 LSL 23
};
const byte kInstruction_cmn_al_r6_r0_LSL_12[] = {
  0x16, 0xeb, 0x00, 0x3f // cmn al r6 r0 LSL 12
};
const byte kInstruction_cmn_al_r6_r2_ROR_22[] = {
  0x16, 0xeb, 0xb2, 0x5f // cmn al r6 r2 ROR 22
};
const byte kInstruction_cmn_al_r10_r0_LSL_15[] = {
  0x1a, 0xeb, 0xc0, 0x3f // cmn al r10 r0 LSL 15
};
const byte kInstruction_cmn_al_r14_r9_ROR_28[] = {
  0x1e, 0xeb, 0x39, 0x7f // cmn al r14 r9 ROR 28
};
const byte kInstruction_cmn_al_r0_r2_ROR_6[] = {
  0x10, 0xeb, 0xb2, 0x1f // cmn al r0 r2 ROR 6
};
const byte kInstruction_cmn_al_r9_r4_LSL_11[] = {
  0x19, 0xeb, 0xc4, 0x2f // cmn al r9 r4 LSL 11
};
const byte kInstruction_cmn_al_r5_r11_LSL_5[] = {
  0x15, 0xeb, 0x4b, 0x1f // cmn al r5 r11 LSL 5
};
const byte kInstruction_cmn_al_r7_r4_ROR_6[] = {
  0x17, 0xeb, 0xb4, 0x1f // cmn al r7 r4 ROR 6
};
const byte kInstruction_cmn_al_r2_r10_ROR_1[] = {
  0x12, 0xeb, 0x7a, 0x0f // cmn al r2 r10 ROR 1
};
const byte kInstruction_cmn_al_r2_r3_LSL_27[] = {
  0x12, 0xeb, 0xc3, 0x6f // cmn al r2 r3 LSL 27
};
const byte kInstruction_cmn_al_r5_r13_LSL_10[] = {
  0x15, 0xeb, 0x8d, 0x2f // cmn al r5 r13 LSL 10
};
const byte kInstruction_cmn_al_r10_r3_LSL_14[] = {
  0x1a, 0xeb, 0x83, 0x3f // cmn al r10 r3 LSL 14
};
const byte kInstruction_cmn_al_r12_r7_ROR_29[] = {
  0x1c, 0xeb, 0x77, 0x7f // cmn al r12 r7 ROR 29
};
const byte kInstruction_cmn_al_r0_r0_ROR_1[] = {
  0x10, 0xeb, 0x70, 0x0f // cmn al r0 r0 ROR 1
};
const byte kInstruction_cmn_al_r11_r13_LSL_21[] = {
  0x1b, 0xeb, 0x4d, 0x5f // cmn al r11 r13 LSL 21
};
const byte kInstruction_cmn_al_r7_r2_LSL_12[] = {
  0x17, 0xeb, 0x02, 0x3f // cmn al r7 r2 LSL 12
};
const byte kInstruction_cmn_al_r1_r10_LSL_2[] = {
  0x11, 0xeb, 0x8a, 0x0f // cmn al r1 r10 LSL 2
};
const byte kInstruction_cmn_al_r11_r5_LSL_19[] = {
  0x1b, 0xeb, 0xc5, 0x4f // cmn al r11 r5 LSL 19
};
const byte kInstruction_cmn_al_r13_r2_ROR_2[] = {
  0x1d, 0xeb, 0xb2, 0x0f // cmn al r13 r2 ROR 2
};
const byte kInstruction_cmn_al_r7_r9_LSL_8[] = {
  0x17, 0xeb, 0x09, 0x2f // cmn al r7 r9 LSL 8
};
const byte kInstruction_cmn_al_r5_r9_ROR_8[] = {
  0x15, 0xeb, 0x39, 0x2f // cmn al r5 r9 ROR 8
};
const byte kInstruction_cmn_al_r10_r14_LSL_31[] = {
  0x1a, 0xeb, 0xce, 0x7f // cmn al r10 r14 LSL 31
};
const byte kInstruction_cmn_al_r0_r2_ROR_23[] = {
  0x10, 0xeb, 0xf2, 0x5f // cmn al r0 r2 ROR 23
};
const byte kInstruction_cmn_al_r14_r13_LSL_28[] = {
  0x1e, 0xeb, 0x0d, 0x7f // cmn al r14 r13 LSL 28
};
const byte kInstruction_cmn_al_r1_r2_LSL_25[] = {
  0x11, 0xeb, 0x42, 0x6f // cmn al r1 r2 LSL 25
};
const byte kInstruction_cmn_al_r3_r5_LSL_29[] = {
  0x13, 0xeb, 0x45, 0x7f // cmn al r3 r5 LSL 29
};
const byte kInstruction_cmn_al_r3_r4_ROR_13[] = {
  0x13, 0xeb, 0x74, 0x3f // cmn al r3 r4 ROR 13
};
const byte kInstruction_cmn_al_r4_r6_ROR_4[] = {
  0x14, 0xeb, 0x36, 0x1f // cmn al r4 r6 ROR 4
};
const byte kInstruction_cmn_al_r6_r8_ROR_3[] = {
  0x16, 0xeb, 0xf8, 0x0f // cmn al r6 r8 ROR 3
};
const byte kInstruction_cmn_al_r11_r2_ROR_2[] = {
  0x1b, 0xeb, 0xb2, 0x0f // cmn al r11 r2 ROR 2
};
const byte kInstruction_cmn_al_r14_r5_LSL_29[] = {
  0x1e, 0xeb, 0x45, 0x7f // cmn al r14 r5 LSL 29
};
const byte kInstruction_cmn_al_r0_r7_ROR_23[] = {
  0x10, 0xeb, 0xf7, 0x5f // cmn al r0 r7 ROR 23
};
const byte kInstruction_cmn_al_r2_r10_LSL_15[] = {
  0x12, 0xeb, 0xca, 0x3f // cmn al r2 r10 LSL 15
};
const byte kInstruction_cmn_al_r6_r7_LSL_5[] = {
  0x16, 0xeb, 0x47, 0x1f // cmn al r6 r7 LSL 5
};
const byte kInstruction_cmn_al_r11_r13_ROR_3[] = {
  0x1b, 0xeb, 0xfd, 0x0f // cmn al r11 r13 ROR 3
};
const byte kInstruction_cmn_al_r5_r12_ROR_13[] = {
  0x15, 0xeb, 0x7c, 0x3f // cmn al r5 r12 ROR 13
};
const byte kInstruction_cmn_al_r9_r12_LSL_26[] = {
  0x19, 0xeb, 0x8c, 0x6f // cmn al r9 r12 LSL 26
};
const byte kInstruction_cmn_al_r7_r3_ROR_12[] = {
  0x17, 0xeb, 0x33, 0x3f // cmn al r7 r3 ROR 12
};
const byte kInstruction_cmn_al_r9_r13_LSL_7[] = {
  0x19, 0xeb, 0xcd, 0x1f // cmn al r9 r13 LSL 7
};
const byte kInstruction_cmn_al_r6_r14_LSL_25[] = {
  0x16, 0xeb, 0x4e, 0x6f // cmn al r6 r14 LSL 25
};
const byte kInstruction_cmn_al_r1_r12_ROR_17[] = {
  0x11, 0xeb, 0x7c, 0x4f // cmn al r1 r12 ROR 17
};
const byte kInstruction_cmn_al_r1_r10_LSL_24[] = {
  0x11, 0xeb, 0x0a, 0x6f // cmn al r1 r10 LSL 24
};
const byte kInstruction_cmn_al_r0_r11_LSL_17[] = {
  0x10, 0xeb, 0x4b, 0x4f // cmn al r0 r11 LSL 17
};
const byte kInstruction_cmn_al_r8_r3_ROR_11[] = {
  0x18, 0xeb, 0xf3, 0x2f // cmn al r8 r3 ROR 11
};
const byte kInstruction_cmn_al_r11_r12_ROR_22[] = {
  0x1b, 0xeb, 0xbc, 0x5f // cmn al r11 r12 ROR 22
};
const byte kInstruction_cmn_al_r0_r9_ROR_28[] = {
  0x10, 0xeb, 0x39, 0x7f // cmn al r0 r9 ROR 28
};
const byte kInstruction_cmn_al_r9_r3_ROR_7[] = {
  0x19, 0xeb, 0xf3, 0x1f // cmn al r9 r3 ROR 7
};
const byte kInstruction_cmn_al_r9_r2_ROR_16[] = {
  0x19, 0xeb, 0x32, 0x4f // cmn al r9 r2 ROR 16
};
const byte kInstruction_cmn_al_r0_r12_LSL_28[] = {
  0x10, 0xeb, 0x0c, 0x7f // cmn al r0 r12 LSL 28
};
const byte kInstruction_cmn_al_r1_r4_ROR_22[] = {
  0x11, 0xeb, 0xb4, 0x5f // cmn al r1 r4 ROR 22
};
const byte kInstruction_cmn_al_r12_r7_LSL_16[] = {
  0x1c, 0xeb, 0x07, 0x4f // cmn al r12 r7 LSL 16
};
const byte kInstruction_cmn_al_r7_r9_ROR_3[] = {
  0x17, 0xeb, 0xf9, 0x0f // cmn al r7 r9 ROR 3
};
const byte kInstruction_cmn_al_r8_r2_LSL_13[] = {
  0x18, 0xeb, 0x42, 0x3f // cmn al r8 r2 LSL 13
};
const byte kInstruction_cmn_al_r10_r11_LSL_10[] = {
  0x1a, 0xeb, 0x8b, 0x2f // cmn al r10 r11 LSL 10
};
const byte kInstruction_cmn_al_r10_r12_ROR_6[] = {
  0x1a, 0xeb, 0xbc, 0x1f // cmn al r10 r12 ROR 6
};
const byte kInstruction_cmn_al_r3_r12_LSL_27[] = {
  0x13, 0xeb, 0xcc, 0x6f // cmn al r3 r12 LSL 27
};
const byte kInstruction_cmn_al_r10_r10_ROR_27[] = {
  0x1a, 0xeb, 0xfa, 0x6f // cmn al r10 r10 ROR 27
};
const byte kInstruction_cmn_al_r8_r8_LSL_8[] = {
  0x18, 0xeb, 0x08, 0x2f // cmn al r8 r8 LSL 8
};
const byte kInstruction_cmn_al_r3_r0_ROR_28[] = {
  0x13, 0xeb, 0x30, 0x7f // cmn al r3 r0 ROR 28
};
const byte kInstruction_cmn_al_r10_r3_ROR_15[] = {
  0x1a, 0xeb, 0xf3, 0x3f // cmn al r10 r3 ROR 15
};
const byte kInstruction_cmn_al_r1_r14_LSL_27[] = {
  0x11, 0xeb, 0xce, 0x6f // cmn al r1 r14 LSL 27
};
const byte kInstruction_cmn_al_r10_r1_LSL_31[] = {
  0x1a, 0xeb, 0xc1, 0x7f // cmn al r10 r1 LSL 31
};
const byte kInstruction_cmn_al_r12_r11_LSL_24[] = {
  0x1c, 0xeb, 0x0b, 0x6f // cmn al r12 r11 LSL 24
};
const byte kInstruction_cmn_al_r5_r11_ROR_11[] = {
  0x15, 0xeb, 0xfb, 0x2f // cmn al r5 r11 ROR 11
};
const byte kInstruction_cmn_al_r13_r8_ROR_12[] = {
  0x1d, 0xeb, 0x38, 0x3f // cmn al r13 r8 ROR 12
};
const byte kInstruction_cmn_al_r1_r14_LSL_28[] = {
  0x11, 0xeb, 0x0e, 0x7f // cmn al r1 r14 LSL 28
};
const byte kInstruction_cmn_al_r6_r2_LSL_6[] = {
  0x16, 0xeb, 0x82, 0x1f // cmn al r6 r2 LSL 6
};
const byte kInstruction_cmn_al_r14_r13_ROR_22[] = {
  0x1e, 0xeb, 0xbd, 0x5f // cmn al r14 r13 ROR 22
};
const byte kInstruction_cmn_al_r12_r12_ROR_18[] = {
  0x1c, 0xeb, 0xbc, 0x4f // cmn al r12 r12 ROR 18
};
const byte kInstruction_cmn_al_r8_r7_ROR_2[] = {
  0x18, 0xeb, 0xb7, 0x0f // cmn al r8 r7 ROR 2
};
const byte kInstruction_cmn_al_r6_r4_LSL_2[] = {
  0x16, 0xeb, 0x84, 0x0f // cmn al r6 r4 LSL 2
};
const byte kInstruction_cmn_al_r2_r6_ROR_17[] = {
  0x12, 0xeb, 0x76, 0x4f // cmn al r2 r6 ROR 17
};
const byte kInstruction_cmn_al_r11_r14_ROR_16[] = {
  0x1b, 0xeb, 0x3e, 0x4f // cmn al r11 r14 ROR 16
};
const byte kInstruction_cmn_al_r1_r5_ROR_22[] = {
  0x11, 0xeb, 0xb5, 0x5f // cmn al r1 r5 ROR 22
};
const byte kInstruction_cmn_al_r2_r2_ROR_11[] = {
  0x12, 0xeb, 0xf2, 0x2f // cmn al r2 r2 ROR 11
};
const byte kInstruction_cmn_al_r12_r9_ROR_1[] = {
  0x1c, 0xeb, 0x79, 0x0f // cmn al r12 r9 ROR 1
};
const byte kInstruction_cmn_al_r13_r5_LSL_13[] = {
  0x1d, 0xeb, 0x45, 0x3f // cmn al r13 r5 LSL 13
};
const byte kInstruction_cmn_al_r0_r0_LSL_3[] = {
  0x10, 0xeb, 0xc0, 0x0f // cmn al r0 r0 LSL 3
};
const byte kInstruction_cmn_al_r0_r11_ROR_9[] = {
  0x10, 0xeb, 0x7b, 0x2f // cmn al r0 r11 ROR 9
};
const byte kInstruction_cmn_al_r6_r1_LSL_18[] = {
  0x16, 0xeb, 0x81, 0x4f // cmn al r6 r1 LSL 18
};
const byte kInstruction_cmn_al_r6_r3_LSL_21[] = {
  0x16, 0xeb, 0x43, 0x5f // cmn al r6 r3 LSL 21
};
const byte kInstruction_cmn_al_r7_r6_LSL_30[] = {
  0x17, 0xeb, 0x86, 0x7f // cmn al r7 r6 LSL 30
};
const byte kInstruction_cmn_al_r11_r3_ROR_28[] = {
  0x1b, 0xeb, 0x33, 0x7f // cmn al r11 r3 ROR 28
};
const byte kInstruction_cmn_al_r0_r4_ROR_14[] = {
  0x10, 0xeb, 0xb4, 0x3f // cmn al r0 r4 ROR 14
};
const byte kInstruction_cmn_al_r1_r14_ROR_22[] = {
  0x11, 0xeb, 0xbe, 0x5f // cmn al r1 r14 ROR 22
};
const byte kInstruction_cmn_al_r3_r13_LSL_30[] = {
  0x13, 0xeb, 0x8d, 0x7f // cmn al r3 r13 LSL 30
};
const byte kInstruction_cmn_al_r14_r6_LSL_14[] = {
  0x1e, 0xeb, 0x86, 0x3f // cmn al r14 r6 LSL 14
};
const byte kInstruction_cmn_al_r11_r10_ROR_1[] = {
  0x1b, 0xeb, 0x7a, 0x0f // cmn al r11 r10 ROR 1
};
const byte kInstruction_cmn_al_r11_r13_ROR_15[] = {
  0x1b, 0xeb, 0xfd, 0x3f // cmn al r11 r13 ROR 15
};
const byte kInstruction_cmn_al_r5_r11_LSL_28[] = {
  0x15, 0xeb, 0x0b, 0x7f // cmn al r5 r11 LSL 28
};
const byte kInstruction_cmn_al_r9_r12_ROR_28[] = {
  0x19, 0xeb, 0x3c, 0x7f // cmn al r9 r12 ROR 28
};
const byte kInstruction_cmn_al_r14_r9_ROR_31[] = {
  0x1e, 0xeb, 0xf9, 0x7f // cmn al r14 r9 ROR 31
};
const byte kInstruction_cmn_al_r7_r4_LSL_24[] = {
  0x17, 0xeb, 0x04, 0x6f // cmn al r7 r4 LSL 24
};
const byte kInstruction_cmn_al_r11_r14_ROR_6[] = {
  0x1b, 0xeb, 0xbe, 0x1f // cmn al r11 r14 ROR 6
};
const byte kInstruction_cmn_al_r2_r12_LSL_5[] = {
  0x12, 0xeb, 0x4c, 0x1f // cmn al r2 r12 LSL 5
};
const byte kInstruction_cmn_al_r1_r14_ROR_6[] = {
  0x11, 0xeb, 0xbe, 0x1f // cmn al r1 r14 ROR 6
};
const byte kInstruction_cmn_al_r6_r4_LSL_20[] = {
  0x16, 0xeb, 0x04, 0x5f // cmn al r6 r4 LSL 20
};
const byte kInstruction_cmn_al_r10_r7_LSL_26[] = {
  0x1a, 0xeb, 0x87, 0x6f // cmn al r10 r7 LSL 26
};
const byte kInstruction_cmn_al_r5_r10_LSL_23[] = {
  0x15, 0xeb, 0xca, 0x5f // cmn al r5 r10 LSL 23
};
const byte kInstruction_cmn_al_r9_r5_ROR_27[] = {
  0x19, 0xeb, 0xf5, 0x6f // cmn al r9 r5 ROR 27
};
const byte kInstruction_cmn_al_r6_r9_ROR_19[] = {
  0x16, 0xeb, 0xf9, 0x4f // cmn al r6 r9 ROR 19
};
const byte kInstruction_cmn_al_r2_r0_LSL_29[] = {
  0x12, 0xeb, 0x40, 0x7f // cmn al r2 r0 LSL 29
};
const byte kInstruction_cmn_al_r3_r11_ROR_9[] = {
  0x13, 0xeb, 0x7b, 0x2f // cmn al r3 r11 ROR 9
};
const byte kInstruction_cmn_al_r0_r14_LSL_7[] = {
  0x10, 0xeb, 0xce, 0x1f // cmn al r0 r14 LSL 7
};
const byte kInstruction_cmn_al_r5_r11_LSL_10[] = {
  0x15, 0xeb, 0x8b, 0x2f // cmn al r5 r11 LSL 10
};
const byte kInstruction_cmn_al_r12_r9_ROR_25[] = {
  0x1c, 0xeb, 0x79, 0x6f // cmn al r12 r9 ROR 25
};
const byte kInstruction_cmn_al_r11_r1_ROR_2[] = {
  0x1b, 0xeb, 0xb1, 0x0f // cmn al r11 r1 ROR 2
};
const byte kInstruction_cmn_al_r8_r7_ROR_14[] = {
  0x18, 0xeb, 0xb7, 0x3f // cmn al r8 r7 ROR 14
};
const byte kInstruction_cmn_al_r7_r0_ROR_1[] = {
  0x17, 0xeb, 0x70, 0x0f // cmn al r7 r0 ROR 1
};
const byte kInstruction_cmn_al_r3_r2_ROR_18[] = {
  0x13, 0xeb, 0xb2, 0x4f // cmn al r3 r2 ROR 18
};
const byte kInstruction_cmn_al_r14_r12_LSL_4[] = {
  0x1e, 0xeb, 0x0c, 0x1f // cmn al r14 r12 LSL 4
};
const byte kInstruction_cmn_al_r12_r5_ROR_8[] = {
  0x1c, 0xeb, 0x35, 0x2f // cmn al r12 r5 ROR 8
};
const byte kInstruction_cmn_al_r6_r9_ROR_27[] = {
  0x16, 0xeb, 0xf9, 0x6f // cmn al r6 r9 ROR 27
};
const byte kInstruction_cmn_al_r10_r13_LSL_8[] = {
  0x1a, 0xeb, 0x0d, 0x2f // cmn al r10 r13 LSL 8
};
const byte kInstruction_cmn_al_r10_r11_ROR_9[] = {
  0x1a, 0xeb, 0x7b, 0x2f // cmn al r10 r11 ROR 9
};
const byte kInstruction_cmn_al_r2_r6_ROR_5[] = {
  0x12, 0xeb, 0x76, 0x1f // cmn al r2 r6 ROR 5
};
const byte kInstruction_cmn_al_r9_r3_LSL_17[] = {
  0x19, 0xeb, 0x43, 0x4f // cmn al r9 r3 LSL 17
};
const byte kInstruction_cmn_al_r12_r7_ROR_19[] = {
  0x1c, 0xeb, 0xf7, 0x4f // cmn al r12 r7 ROR 19
};
const byte kInstruction_cmn_al_r9_r5_LSL_11[] = {
  0x19, 0xeb, 0xc5, 0x2f // cmn al r9 r5 LSL 11
};
const byte kInstruction_cmn_al_r14_r9_LSL_20[] = {
  0x1e, 0xeb, 0x09, 0x5f // cmn al r14 r9 LSL 20
};
const byte kInstruction_cmn_al_r6_r2_ROR_4[] = {
  0x16, 0xeb, 0x32, 0x1f // cmn al r6 r2 ROR 4
};
const byte kInstruction_cmn_al_r8_r4_LSL_3[] = {
  0x18, 0xeb, 0xc4, 0x0f // cmn al r8 r4 LSL 3
};
const byte kInstruction_cmn_al_r5_r0_ROR_22[] = {
  0x15, 0xeb, 0xb0, 0x5f // cmn al r5 r0 ROR 22
};
const byte kInstruction_cmn_al_r10_r6_LSL_13[] = {
  0x1a, 0xeb, 0x46, 0x3f // cmn al r10 r6 LSL 13
};
const byte kInstruction_cmn_al_r10_r4_ROR_7[] = {
  0x1a, 0xeb, 0xf4, 0x1f // cmn al r10 r4 ROR 7
};
const byte kInstruction_cmn_al_r6_r2_ROR_17[] = {
  0x16, 0xeb, 0x72, 0x4f // cmn al r6 r2 ROR 17
};
const byte kInstruction_cmn_al_r9_r8_LSL_26[] = {
  0x19, 0xeb, 0x88, 0x6f // cmn al r9 r8 LSL 26
};
const byte kInstruction_cmn_al_r6_r11_ROR_15[] = {
  0x16, 0xeb, 0xfb, 0x3f // cmn al r6 r11 ROR 15
};
const byte kInstruction_cmn_al_r12_r14_LSL_8[] = {
  0x1c, 0xeb, 0x0e, 0x2f // cmn al r12 r14 LSL 8
};
const byte kInstruction_cmn_al_r4_r3_ROR_3[] = {
  0x14, 0xeb, 0xf3, 0x0f // cmn al r4 r3 ROR 3
};
const byte kInstruction_cmn_al_r9_r5_LSL_12[] = {
  0x19, 0xeb, 0x05, 0x3f // cmn al r9 r5 LSL 12
};
const byte kInstruction_cmn_al_r6_r3_ROR_13[] = {
  0x16, 0xeb, 0x73, 0x3f // cmn al r6 r3 ROR 13
};
const byte kInstruction_cmn_al_r10_r13_LSL_21[] = {
  0x1a, 0xeb, 0x4d, 0x5f // cmn al r10 r13 LSL 21
};
const byte kInstruction_cmn_al_r4_r0_ROR_3[] = {
  0x14, 0xeb, 0xf0, 0x0f // cmn al r4 r0 ROR 3
};
const byte kInstruction_cmn_al_r9_r11_LSL_8[] = {
  0x19, 0xeb, 0x0b, 0x2f // cmn al r9 r11 LSL 8
};
const byte kInstruction_cmn_al_r0_r14_ROR_14[] = {
  0x10, 0xeb, 0xbe, 0x3f // cmn al r0 r14 ROR 14
};
const byte kInstruction_cmn_al_r9_r9_ROR_14[] = {
  0x19, 0xeb, 0xb9, 0x3f // cmn al r9 r9 ROR 14
};
const byte kInstruction_cmn_al_r8_r14_ROR_18[] = {
  0x18, 0xeb, 0xbe, 0x4f // cmn al r8 r14 ROR 18
};
const byte kInstruction_cmn_al_r13_r2_ROR_13[] = {
  0x1d, 0xeb, 0x72, 0x3f // cmn al r13 r2 ROR 13
};
const byte kInstruction_cmn_al_r13_r13_LSL_12[] = {
  0x1d, 0xeb, 0x0d, 0x3f // cmn al r13 r13 LSL 12
};
const byte kInstruction_cmn_al_r14_r9_LSL_25[] = {
  0x1e, 0xeb, 0x49, 0x6f // cmn al r14 r9 LSL 25
};
const byte kInstruction_cmn_al_r1_r0_ROR_9[] = {
  0x11, 0xeb, 0x70, 0x2f // cmn al r1 r0 ROR 9
};
const byte kInstruction_cmn_al_r12_r6_ROR_20[] = {
  0x1c, 0xeb, 0x36, 0x5f // cmn al r12 r6 ROR 20
};
const byte kInstruction_cmn_al_r9_r2_ROR_23[] = {
  0x19, 0xeb, 0xf2, 0x5f // cmn al r9 r2 ROR 23
};
const byte kInstruction_cmn_al_r2_r10_LSL_1[] = {
  0x12, 0xeb, 0x4a, 0x0f // cmn al r2 r10 LSL 1
};
const byte kInstruction_cmn_al_r8_r7_ROR_15[] = {
  0x18, 0xeb, 0xf7, 0x3f // cmn al r8 r7 ROR 15
};
const byte kInstruction_cmn_al_r5_r3_ROR_14[] = {
  0x15, 0xeb, 0xb3, 0x3f // cmn al r5 r3 ROR 14
};
const byte kInstruction_cmn_al_r4_r8_ROR_1[] = {
  0x14, 0xeb, 0x78, 0x0f // cmn al r4 r8 ROR 1
};
const byte kInstruction_cmn_al_r1_r2_ROR_5[] = {
  0x11, 0xeb, 0x72, 0x1f // cmn al r1 r2 ROR 5
};
const byte kInstruction_cmn_al_r6_r2_ROR_16[] = {
  0x16, 0xeb, 0x32, 0x4f // cmn al r6 r2 ROR 16
};
const byte kInstruction_cmn_al_r5_r4_LSL_15[] = {
  0x15, 0xeb, 0xc4, 0x3f // cmn al r5 r4 LSL 15
};
const byte kInstruction_cmn_al_r4_r2_LSL_14[] = {
  0x14, 0xeb, 0x82, 0x3f // cmn al r4 r2 LSL 14
};
const byte kInstruction_cmn_al_r2_r1_LSL_7[] = {
  0x12, 0xeb, 0xc1, 0x1f // cmn al r2 r1 LSL 7
};
const byte kInstruction_cmn_al_r0_r8_ROR_30[] = {
  0x10, 0xeb, 0xb8, 0x7f // cmn al r0 r8 ROR 30
};
const byte kInstruction_cmn_al_r9_r5_LSL_22[] = {
  0x19, 0xeb, 0x85, 0x5f // cmn al r9 r5 LSL 22
};
const byte kInstruction_cmn_al_r9_r10_LSL_25[] = {
  0x19, 0xeb, 0x4a, 0x6f // cmn al r9 r10 LSL 25
};
const byte kInstruction_cmn_al_r4_r6_ROR_15[] = {
  0x14, 0xeb, 0xf6, 0x3f // cmn al r4 r6 ROR 15
};
const byte kInstruction_cmn_al_r2_r9_LSL_29[] = {
  0x12, 0xeb, 0x49, 0x7f // cmn al r2 r9 LSL 29
};
const byte kInstruction_cmn_al_r6_r6_ROR_27[] = {
  0x16, 0xeb, 0xf6, 0x6f // cmn al r6 r6 ROR 27
};
const byte kInstruction_cmn_al_r11_r1_ROR_26[] = {
  0x1b, 0xeb, 0xb1, 0x6f // cmn al r11 r1 ROR 26
};
const byte kInstruction_cmn_al_r5_r10_ROR_23[] = {
  0x15, 0xeb, 0xfa, 0x5f // cmn al r5 r10 ROR 23
};
const byte kInstruction_cmn_al_r13_r5_LSL_29[] = {
  0x1d, 0xeb, 0x45, 0x7f // cmn al r13 r5 LSL 29
};
const byte kInstruction_cmn_al_r14_r1_ROR_27[] = {
  0x1e, 0xeb, 0xf1, 0x6f // cmn al r14 r1 ROR 27
};
const byte kInstruction_cmn_al_r0_r10_LSL_27[] = {
  0x10, 0xeb, 0xca, 0x6f // cmn al r0 r10 LSL 27
};
const byte kInstruction_cmn_al_r2_r4_ROR_14[] = {
  0x12, 0xeb, 0xb4, 0x3f // cmn al r2 r4 ROR 14
};
const byte kInstruction_cmn_al_r1_r10_LSL_31[] = {
  0x11, 0xeb, 0xca, 0x7f // cmn al r1 r10 LSL 31
};
const byte kInstruction_cmn_al_r2_r3_ROR_5[] = {
  0x12, 0xeb, 0x73, 0x1f // cmn al r2 r3 ROR 5
};
const byte kInstruction_cmn_al_r0_r6_ROR_14[] = {
  0x10, 0xeb, 0xb6, 0x3f // cmn al r0 r6 ROR 14
};
const byte kInstruction_cmn_al_r11_r9_ROR_25[] = {
  0x1b, 0xeb, 0x79, 0x6f // cmn al r11 r9 ROR 25
};
const byte kInstruction_cmn_al_r2_r11_ROR_29[] = {
  0x12, 0xeb, 0x7b, 0x7f // cmn al r2 r11 ROR 29
};
const byte kInstruction_cmn_al_r7_r6_ROR_2[] = {
  0x17, 0xeb, 0xb6, 0x0f // cmn al r7 r6 ROR 2
};
const byte kInstruction_cmn_al_r8_r3_ROR_3[] = {
  0x18, 0xeb, 0xf3, 0x0f // cmn al r8 r3 ROR 3
};
const byte kInstruction_cmn_al_r8_r13_LSL_18[] = {
  0x18, 0xeb, 0x8d, 0x4f // cmn al r8 r13 LSL 18
};
const byte kInstruction_cmn_al_r3_r13_ROR_27[] = {
  0x13, 0xeb, 0xfd, 0x6f // cmn al r3 r13 ROR 27
};
const byte kInstruction_cmn_al_r5_r5_LSL_18[] = {
  0x15, 0xeb, 0x85, 0x4f // cmn al r5 r5 LSL 18
};
const byte kInstruction_cmn_al_r14_r8_ROR_17[] = {
  0x1e, 0xeb, 0x78, 0x4f // cmn al r14 r8 ROR 17
};
const byte kInstruction_cmn_al_r10_r6_LSL_18[] = {
  0x1a, 0xeb, 0x86, 0x4f // cmn al r10 r6 LSL 18
};
const byte kInstruction_cmn_al_r7_r1_LSL_2[] = {
  0x17, 0xeb, 0x81, 0x0f // cmn al r7 r1 LSL 2
};
const byte kInstruction_cmn_al_r9_r7_ROR_13[] = {
  0x19, 0xeb, 0x77, 0x3f // cmn al r9 r7 ROR 13
};
const byte kInstruction_cmn_al_r5_r9_LSL_29[] = {
  0x15, 0xeb, 0x49, 0x7f // cmn al r5 r9 LSL 29
};
const byte kInstruction_cmn_al_r14_r12_LSL_22[] = {
  0x1e, 0xeb, 0x8c, 0x5f // cmn al r14 r12 LSL 22
};
const byte kInstruction_cmn_al_r0_r8_LSL_12[] = {
  0x10, 0xeb, 0x08, 0x3f // cmn al r0 r8 LSL 12
};
const byte kInstruction_cmn_al_r12_r12_ROR_28[] = {
  0x1c, 0xeb, 0x3c, 0x7f // cmn al r12 r12 ROR 28
};
const byte kInstruction_cmn_al_r12_r3_LSL_2[] = {
  0x1c, 0xeb, 0x83, 0x0f // cmn al r12 r3 LSL 2
};
const byte kInstruction_cmn_al_r6_r11_ROR_3[] = {
  0x16, 0xeb, 0xfb, 0x0f // cmn al r6 r11 ROR 3
};
const byte kInstruction_cmn_al_r1_r5_ROR_21[] = {
  0x11, 0xeb, 0x75, 0x5f // cmn al r1 r5 ROR 21
};
const byte kInstruction_cmn_al_r0_r14_ROR_29[] = {
  0x10, 0xeb, 0x7e, 0x7f // cmn al r0 r14 ROR 29
};
const byte kInstruction_cmn_al_r11_r8_ROR_16[] = {
  0x1b, 0xeb, 0x38, 0x4f // cmn al r11 r8 ROR 16
};
const byte kInstruction_cmn_al_r9_r10_ROR_31[] = {
  0x19, 0xeb, 0xfa, 0x7f // cmn al r9 r10 ROR 31
};
const byte kInstruction_cmn_al_r10_r1_LSL_25[] = {
  0x1a, 0xeb, 0x41, 0x6f // cmn al r10 r1 LSL 25
};
const byte kInstruction_cmn_al_r9_r4_ROR_1[] = {
  0x19, 0xeb, 0x74, 0x0f // cmn al r9 r4 ROR 1
};
const byte kInstruction_cmn_al_r5_r12_LSL_4[] = {
  0x15, 0xeb, 0x0c, 0x1f // cmn al r5 r12 LSL 4
};
const byte kInstruction_cmn_al_r10_r7_ROR_25[] = {
  0x1a, 0xeb, 0x77, 0x6f // cmn al r10 r7 ROR 25
};
const byte kInstruction_cmn_al_r9_r3_LSL_6[] = {
  0x19, 0xeb, 0x83, 0x1f // cmn al r9 r3 LSL 6
};
const byte kInstruction_cmn_al_r7_r4_ROR_26[] = {
  0x17, 0xeb, 0xb4, 0x6f // cmn al r7 r4 ROR 26
};
const byte kInstruction_cmn_al_r13_r0_LSL_31[] = {
  0x1d, 0xeb, 0xc0, 0x7f // cmn al r13 r0 LSL 31
};
const byte kInstruction_cmn_al_r9_r2_LSL_22[] = {
  0x19, 0xeb, 0x82, 0x5f // cmn al r9 r2 LSL 22
};
const byte kInstruction_cmn_al_r6_r8_ROR_21[] = {
  0x16, 0xeb, 0x78, 0x5f // cmn al r6 r8 ROR 21
};
const byte kInstruction_cmn_al_r12_r7_LSL_11[] = {
  0x1c, 0xeb, 0xc7, 0x2f // cmn al r12 r7 LSL 11
};
const byte kInstruction_cmn_al_r10_r0_LSL_17[] = {
  0x1a, 0xeb, 0x40, 0x4f // cmn al r10 r0 LSL 17
};
const byte kInstruction_cmn_al_r2_r9_ROR_23[] = {
  0x12, 0xeb, 0xf9, 0x5f // cmn al r2 r9 ROR 23
};
const byte kInstruction_cmn_al_r2_r4_LSL_6[] = {
  0x12, 0xeb, 0x84, 0x1f // cmn al r2 r4 LSL 6
};
const byte kInstruction_cmn_al_r7_r8_LSL_24[] = {
  0x17, 0xeb, 0x08, 0x6f // cmn al r7 r8 LSL 24
};
const byte kInstruction_cmn_al_r8_r5_LSL_25[] = {
  0x18, 0xeb, 0x45, 0x6f // cmn al r8 r5 LSL 25
};
const byte kInstruction_cmn_al_r10_r11_ROR_23[] = {
  0x1a, 0xeb, 0xfb, 0x5f // cmn al r10 r11 ROR 23
};
const byte kInstruction_cmn_al_r0_r14_LSL_16[] = {
  0x10, 0xeb, 0x0e, 0x4f // cmn al r0 r14 LSL 16
};
const byte kInstruction_cmn_al_r2_r3_LSL_16[] = {
  0x12, 0xeb, 0x03, 0x4f // cmn al r2 r3 LSL 16
};
const byte kInstruction_cmn_al_r0_r11_ROR_13[] = {
  0x10, 0xeb, 0x7b, 0x3f // cmn al r0 r11 ROR 13
};
const byte kInstruction_cmn_al_r6_r4_ROR_15[] = {
  0x16, 0xeb, 0xf4, 0x3f // cmn al r6 r4 ROR 15
};
const byte kInstruction_cmn_al_r8_r9_LSL_19[] = {
  0x18, 0xeb, 0xc9, 0x4f // cmn al r8 r9 LSL 19
};
const byte kInstruction_cmn_al_r1_r13_LSL_23[] = {
  0x11, 0xeb, 0xcd, 0x5f // cmn al r1 r13 LSL 23
};
const byte kInstruction_cmn_al_r4_r11_LSL_6[] = {
  0x14, 0xeb, 0x8b, 0x1f // cmn al r4 r11 LSL 6
};
const byte kInstruction_cmn_al_r9_r0_ROR_15[] = {
  0x19, 0xeb, 0xf0, 0x3f // cmn al r9 r0 ROR 15
};
const byte kInstruction_cmn_al_r7_r0_ROR_7[] = {
  0x17, 0xeb, 0xf0, 0x1f // cmn al r7 r0 ROR 7
};
const byte kInstruction_cmn_al_r5_r7_LSL_26[] = {
  0x15, 0xeb, 0x87, 0x6f // cmn al r5 r7 LSL 26
};
const byte kInstruction_cmn_al_r9_r4_LSL_6[] = {
  0x19, 0xeb, 0x84, 0x1f // cmn al r9 r4 LSL 6
};
const byte kInstruction_cmn_al_r4_r9_LSL_9[] = {
  0x14, 0xeb, 0x49, 0x2f // cmn al r4 r9 LSL 9
};
const byte kInstruction_cmn_al_r3_r14_ROR_24[] = {
  0x13, 0xeb, 0x3e, 0x6f // cmn al r3 r14 ROR 24
};
const byte kInstruction_cmn_al_r6_r6_ROR_4[] = {
  0x16, 0xeb, 0x36, 0x1f // cmn al r6 r6 ROR 4
};
const byte kInstruction_cmn_al_r2_r1_ROR_19[] = {
  0x12, 0xeb, 0xf1, 0x4f // cmn al r2 r1 ROR 19
};
const byte kInstruction_cmn_al_r13_r1_LSL_2[] = {
  0x1d, 0xeb, 0x81, 0x0f // cmn al r13 r1 LSL 2
};
const byte kInstruction_cmn_al_r5_r9_ROR_2[] = {
  0x15, 0xeb, 0xb9, 0x0f // cmn al r5 r9 ROR 2
};
const byte kInstruction_cmn_al_r13_r14_ROR_25[] = {
  0x1d, 0xeb, 0x7e, 0x6f // cmn al r13 r14 ROR 25
};
const byte kInstruction_cmn_al_r2_r11_ROR_18[] = {
  0x12, 0xeb, 0xbb, 0x4f // cmn al r2 r11 ROR 18
};
const byte kInstruction_cmn_al_r10_r12_ROR_5[] = {
  0x1a, 0xeb, 0x7c, 0x1f // cmn al r10 r12 ROR 5
};
const byte kInstruction_cmn_al_r2_r4_ROR_2[] = {
  0x12, 0xeb, 0xb4, 0x0f // cmn al r2 r4 ROR 2
};
const byte kInstruction_cmn_al_r12_r14_LSL_15[] = {
  0x1c, 0xeb, 0xce, 0x3f // cmn al r12 r14 LSL 15
};
const byte kInstruction_cmn_al_r13_r2_LSL_27[] = {
  0x1d, 0xeb, 0xc2, 0x6f // cmn al r13 r2 LSL 27
};
const byte kInstruction_cmn_al_r8_r13_ROR_27[] = {
  0x18, 0xeb, 0xfd, 0x6f // cmn al r8 r13 ROR 27
};
const byte kInstruction_cmn_al_r12_r11_LSL_22[] = {
  0x1c, 0xeb, 0x8b, 0x5f // cmn al r12 r11 LSL 22
};
const byte kInstruction_cmn_al_r13_r5_ROR_12[] = {
  0x1d, 0xeb, 0x35, 0x3f // cmn al r13 r5 ROR 12
};
const byte kInstruction_cmn_al_r13_r7_ROR_17[] = {
  0x1d, 0xeb, 0x77, 0x4f // cmn al r13 r7 ROR 17
};
const byte kInstruction_cmn_al_r4_r10_LSL_26[] = {
  0x14, 0xeb, 0x8a, 0x6f // cmn al r4 r10 LSL 26
};
const byte kInstruction_cmn_al_r6_r9_LSL_24[] = {
  0x16, 0xeb, 0x09, 0x6f // cmn al r6 r9 LSL 24
};
const byte kInstruction_cmn_al_r14_r5_LSL_13[] = {
  0x1e, 0xeb, 0x45, 0x3f // cmn al r14 r5 LSL 13
};
const byte kInstruction_cmn_al_r12_r0_ROR_8[] = {
  0x1c, 0xeb, 0x30, 0x2f // cmn al r12 r0 ROR 8
};
const byte kInstruction_cmn_al_r1_r2_LSL_30[] = {
  0x11, 0xeb, 0x82, 0x7f // cmn al r1 r2 LSL 30
};
const byte kInstruction_cmn_al_r10_r7_LSL_15[] = {
  0x1a, 0xeb, 0xc7, 0x3f // cmn al r10 r7 LSL 15
};
const byte kInstruction_cmn_al_r0_r7_LSL_31[] = {
  0x10, 0xeb, 0xc7, 0x7f // cmn al r0 r7 LSL 31
};
const byte kInstruction_cmn_al_r14_r11_LSL_9[] = {
  0x1e, 0xeb, 0x4b, 0x2f // cmn al r14 r11 LSL 9
};
const byte kInstruction_cmn_al_r1_r6_ROR_2[] = {
  0x11, 0xeb, 0xb6, 0x0f // cmn al r1 r6 ROR 2
};
const byte kInstruction_cmn_al_r5_r2_ROR_26[] = {
  0x15, 0xeb, 0xb2, 0x6f // cmn al r5 r2 ROR 26
};
const byte kInstruction_cmn_al_r3_r5_LSL_2[] = {
  0x13, 0xeb, 0x85, 0x0f // cmn al r3 r5 LSL 2
};
const byte kInstruction_cmn_al_r8_r3_LSL_3[] = {
  0x18, 0xeb, 0xc3, 0x0f // cmn al r8 r3 LSL 3
};
const byte kInstruction_cmn_al_r6_r11_ROR_6[] = {
  0x16, 0xeb, 0xbb, 0x1f // cmn al r6 r11 ROR 6
};
const byte kInstruction_cmn_al_r4_r4_ROR_13[] = {
  0x14, 0xeb, 0x74, 0x3f // cmn al r4 r4 ROR 13
};
const byte kInstruction_cmn_al_r4_r2_LSL_20[] = {
  0x14, 0xeb, 0x02, 0x5f // cmn al r4 r2 LSL 20
};
const byte kInstruction_cmn_al_r13_r8_ROR_25[] = {
  0x1d, 0xeb, 0x78, 0x6f // cmn al r13 r8 ROR 25
};
const byte kInstruction_cmn_al_r3_r13_LSL_18[] = {
  0x13, 0xeb, 0x8d, 0x4f // cmn al r3 r13 LSL 18
};
const byte kInstruction_cmn_al_r1_r0_ROR_23[] = {
  0x11, 0xeb, 0xf0, 0x5f // cmn al r1 r0 ROR 23
};
const byte kInstruction_cmn_al_r14_r14_LSL_25[] = {
  0x1e, 0xeb, 0x4e, 0x6f // cmn al r14 r14 LSL 25
};
const byte kInstruction_cmn_al_r0_r8_LSL_11[] = {
  0x10, 0xeb, 0xc8, 0x2f // cmn al r0 r8 LSL 11
};
const byte kInstruction_cmn_al_r10_r1_LSL_13[] = {
  0x1a, 0xeb, 0x41, 0x3f // cmn al r10 r1 LSL 13
};
const byte kInstruction_cmn_al_r13_r10_LSL_9[] = {
  0x1d, 0xeb, 0x4a, 0x2f // cmn al r13 r10 LSL 9
};
const byte kInstruction_cmn_al_r6_r13_LSL_16[] = {
  0x16, 0xeb, 0x0d, 0x4f // cmn al r6 r13 LSL 16
};
const byte kInstruction_cmn_al_r13_r14_LSL_5[] = {
  0x1d, 0xeb, 0x4e, 0x1f // cmn al r13 r14 LSL 5
};
const byte kInstruction_cmn_al_r2_r6_LSL_14[] = {
  0x12, 0xeb, 0x86, 0x3f // cmn al r2 r6 LSL 14
};
const byte kInstruction_cmn_al_r13_r7_LSL_13[] = {
  0x1d, 0xeb, 0x47, 0x3f // cmn al r13 r7 LSL 13
};
const byte kInstruction_cmn_al_r7_r5_ROR_26[] = {
  0x17, 0xeb, 0xb5, 0x6f // cmn al r7 r5 ROR 26
};
const byte kInstruction_cmn_al_r12_r11_LSL_12[] = {
  0x1c, 0xeb, 0x0b, 0x3f // cmn al r12 r11 LSL 12
};
const byte kInstruction_cmn_al_r5_r13_ROR_8[] = {
  0x15, 0xeb, 0x3d, 0x2f // cmn al r5 r13 ROR 8
};
const byte kInstruction_cmn_al_r2_r8_ROR_2[] = {
  0x12, 0xeb, 0xb8, 0x0f // cmn al r2 r8 ROR 2
};
const byte kInstruction_cmn_al_r0_r9_ROR_24[] = {
  0x10, 0xeb, 0x39, 0x6f // cmn al r0 r9 ROR 24
};
const byte kInstruction_cmn_al_r8_r6_LSL_29[] = {
  0x18, 0xeb, 0x46, 0x7f // cmn al r8 r6 LSL 29
};
const byte kInstruction_cmn_al_r9_r10_LSL_18[] = {
  0x19, 0xeb, 0x8a, 0x4f // cmn al r9 r10 LSL 18
};
const byte kInstruction_cmn_al_r6_r0_LSL_24[] = {
  0x16, 0xeb, 0x00, 0x6f // cmn al r6 r0 LSL 24
};
const byte kInstruction_cmn_al_r9_r12_LSL_20[] = {
  0x19, 0xeb, 0x0c, 0x5f // cmn al r9 r12 LSL 20
};
const byte kInstruction_cmn_al_r14_r11_LSL_3[] = {
  0x1e, 0xeb, 0xcb, 0x0f // cmn al r14 r11 LSL 3
};
const byte kInstruction_cmn_al_r12_r11_LSL_26[] = {
  0x1c, 0xeb, 0x8b, 0x6f // cmn al r12 r11 LSL 26
};
const byte kInstruction_cmn_al_r10_r0_LSL_4[] = {
  0x1a, 0xeb, 0x00, 0x1f // cmn al r10 r0 LSL 4
};
const byte kInstruction_cmn_al_r10_r12_ROR_7[] = {
  0x1a, 0xeb, 0xfc, 0x1f // cmn al r10 r12 ROR 7
};
const byte kInstruction_cmn_al_r12_r3_LSL_1[] = {
  0x1c, 0xeb, 0x43, 0x0f // cmn al r12 r3 LSL 1
};
const byte kInstruction_cmn_al_r12_r4_ROR_12[] = {
  0x1c, 0xeb, 0x34, 0x3f // cmn al r12 r4 ROR 12
};
const byte kInstruction_cmn_al_r8_r0_ROR_12[] = {
  0x18, 0xeb, 0x30, 0x3f // cmn al r8 r0 ROR 12
};
const byte kInstruction_cmn_al_r3_r3_ROR_6[] = {
  0x13, 0xeb, 0xb3, 0x1f // cmn al r3 r3 ROR 6
};
const byte kInstruction_cmn_al_r11_r0_LSL_19[] = {
  0x1b, 0xeb, 0xc0, 0x4f // cmn al r11 r0 LSL 19
};
const byte kInstruction_cmn_al_r8_r14_LSL_24[] = {
  0x18, 0xeb, 0x0e, 0x6f // cmn al r8 r14 LSL 24
};
const byte kInstruction_cmn_al_r0_r11_LSL_1[] = {
  0x10, 0xeb, 0x4b, 0x0f // cmn al r0 r11 LSL 1
};
const byte kInstruction_cmn_al_r13_r5_LSL_6[] = {
  0x1d, 0xeb, 0x85, 0x1f // cmn al r13 r5 LSL 6
};
const byte kInstruction_cmn_al_r14_r14_ROR_31[] = {
  0x1e, 0xeb, 0xfe, 0x7f // cmn al r14 r14 ROR 31
};
const byte kInstruction_cmn_al_r10_r14_ROR_6[] = {
  0x1a, 0xeb, 0xbe, 0x1f // cmn al r10 r14 ROR 6
};
const byte kInstruction_cmn_al_r14_r4_LSL_1[] = {
  0x1e, 0xeb, 0x44, 0x0f // cmn al r14 r4 LSL 1
};
const byte kInstruction_cmn_al_r0_r3_ROR_7[] = {
  0x10, 0xeb, 0xf3, 0x1f // cmn al r0 r3 ROR 7
};
const byte kInstruction_cmn_al_r2_r10_LSL_17[] = {
  0x12, 0xeb, 0x4a, 0x4f // cmn al r2 r10 LSL 17
};
const byte kInstruction_cmn_al_r11_r6_ROR_19[] = {
  0x1b, 0xeb, 0xf6, 0x4f // cmn al r11 r6 ROR 19
};
const byte kInstruction_cmn_al_r9_r13_ROR_11[] = {
  0x19, 0xeb, 0xfd, 0x2f // cmn al r9 r13 ROR 11
};
const byte kInstruction_cmn_al_r11_r3_LSL_2[] = {
  0x1b, 0xeb, 0x83, 0x0f // cmn al r11 r3 LSL 2
};
const byte kInstruction_cmn_al_r9_r1_ROR_4[] = {
  0x19, 0xeb, 0x31, 0x1f // cmn al r9 r1 ROR 4
};
const byte kInstruction_cmn_al_r8_r11_LSL_16[] = {
  0x18, 0xeb, 0x0b, 0x4f // cmn al r8 r11 LSL 16
};
const byte kInstruction_cmn_al_r1_r12_LSL_11[] = {
  0x11, 0xeb, 0xcc, 0x2f // cmn al r1 r12 LSL 11
};
const byte kInstruction_cmn_al_r0_r8_ROR_6[] = {
  0x10, 0xeb, 0xb8, 0x1f // cmn al r0 r8 ROR 6
};
const byte kInstruction_cmn_al_r5_r3_LSL_2[] = {
  0x15, 0xeb, 0x83, 0x0f // cmn al r5 r3 LSL 2
};
const byte kInstruction_cmn_al_r6_r3_LSL_17[] = {
  0x16, 0xeb, 0x43, 0x4f // cmn al r6 r3 LSL 17
};
const byte kInstruction_cmn_al_r3_r12_ROR_27[] = {
  0x13, 0xeb, 0xfc, 0x6f // cmn al r3 r12 ROR 27
};
const byte kInstruction_cmn_al_r6_r8_ROR_15[] = {
  0x16, 0xeb, 0xf8, 0x3f // cmn al r6 r8 ROR 15
};
const byte kInstruction_cmn_al_r7_r11_LSL_10[] = {
  0x17, 0xeb, 0x8b, 0x2f // cmn al r7 r11 LSL 10
};
const byte kInstruction_cmn_al_r2_r12_LSL_15[] = {
  0x12, 0xeb, 0xcc, 0x3f // cmn al r2 r12 LSL 15
};
const byte kInstruction_cmn_al_r2_r2_ROR_8[] = {
  0x12, 0xeb, 0x32, 0x2f // cmn al r2 r2 ROR 8
};
const byte kInstruction_cmn_al_r12_r0_ROR_26[] = {
  0x1c, 0xeb, 0xb0, 0x6f // cmn al r12 r0 ROR 26
};
const byte kInstruction_cmn_al_r11_r13_ROR_19[] = {
  0x1b, 0xeb, 0xfd, 0x4f // cmn al r11 r13 ROR 19
};
const byte kInstruction_cmn_al_r0_r9_ROR_2[] = {
  0x10, 0xeb, 0xb9, 0x0f // cmn al r0 r9 ROR 2
};
const byte kInstruction_cmn_al_r9_r7_ROR_25[] = {
  0x19, 0xeb, 0x77, 0x6f // cmn al r9 r7 ROR 25
};
const byte kInstruction_cmn_al_r9_r8_LSL_28[] = {
  0x19, 0xeb, 0x08, 0x7f // cmn al r9 r8 LSL 28
};
const byte kInstruction_cmn_al_r11_r12_ROR_17[] = {
  0x1b, 0xeb, 0x7c, 0x4f // cmn al r11 r12 ROR 17
};
const byte kInstruction_cmn_al_r12_r2_ROR_20[] = {
  0x1c, 0xeb, 0x32, 0x5f // cmn al r12 r2 ROR 20
};
const byte kInstruction_cmn_al_r11_r9_ROR_16[] = {
  0x1b, 0xeb, 0x39, 0x4f // cmn al r11 r9 ROR 16
};
const byte kInstruction_cmn_al_r13_r11_LSL_14[] = {
  0x1d, 0xeb, 0x8b, 0x3f // cmn al r13 r11 LSL 14
};
const byte kInstruction_cmn_al_r2_r4_ROR_12[] = {
  0x12, 0xeb, 0x34, 0x3f // cmn al r2 r4 ROR 12
};
const byte kInstruction_cmn_al_r3_r12_ROR_17[] = {
  0x13, 0xeb, 0x7c, 0x4f // cmn al r3 r12 ROR 17
};
const byte kInstruction_cmn_al_r14_r11_ROR_27[] = {
  0x1e, 0xeb, 0xfb, 0x6f // cmn al r14 r11 ROR 27
};
const TestResult kReferencecmn[] = {
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r8_ROR_21),
    kInstruction_cmn_al_r14_r8_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r13_ROR_4),
    kInstruction_cmn_al_r5_r13_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r3_ROR_4),
    kInstruction_cmn_al_r0_r3_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r14_ROR_7),
    kInstruction_cmn_al_r3_r14_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r6_LSL_30),
    kInstruction_cmn_al_r2_r6_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r4_LSL_26),
    kInstruction_cmn_al_r11_r4_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r0_LSL_30),
    kInstruction_cmn_al_r7_r0_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r9_ROR_17),
    kInstruction_cmn_al_r0_r9_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r3_ROR_31),
    kInstruction_cmn_al_r11_r3_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r8_ROR_20),
    kInstruction_cmn_al_r8_r8_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r13_LSL_24),
    kInstruction_cmn_al_r11_r13_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r3_ROR_27),
    kInstruction_cmn_al_r1_r3_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r10_ROR_5),
    kInstruction_cmn_al_r9_r10_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r11_ROR_25),
    kInstruction_cmn_al_r3_r11_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r6_LSL_7),
    kInstruction_cmn_al_r2_r6_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r12_ROR_8),
    kInstruction_cmn_al_r3_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r0_LSL_14),
    kInstruction_cmn_al_r0_r0_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r10_ROR_2),
    kInstruction_cmn_al_r8_r10_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r8_LSL_31),
    kInstruction_cmn_al_r0_r8_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r14_ROR_27),
    kInstruction_cmn_al_r1_r14_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r2_ROR_27),
    kInstruction_cmn_al_r14_r2_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r10_LSL_25),
    kInstruction_cmn_al_r3_r10_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r8_ROR_6),
    kInstruction_cmn_al_r13_r8_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r12_ROR_7),
    kInstruction_cmn_al_r5_r12_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r8_LSL_10),
    kInstruction_cmn_al_r7_r8_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r12_ROR_10),
    kInstruction_cmn_al_r9_r12_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r6_ROR_12),
    kInstruction_cmn_al_r8_r6_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r10_LSL_7),
    kInstruction_cmn_al_r8_r10_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r12_ROR_8),
    kInstruction_cmn_al_r5_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r13_LSL_21),
    kInstruction_cmn_al_r0_r13_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r0_LSL_7),
    kInstruction_cmn_al_r2_r0_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r2_LSL_22),
    kInstruction_cmn_al_r10_r2_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r11_ROR_2),
    kInstruction_cmn_al_r4_r11_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r14_LSL_14),
    kInstruction_cmn_al_r11_r14_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r0_LSL_23),
    kInstruction_cmn_al_r8_r0_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r4_LSL_12),
    kInstruction_cmn_al_r10_r4_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r5_ROR_28),
    kInstruction_cmn_al_r4_r5_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r5_LSL_25),
    kInstruction_cmn_al_r1_r5_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r10_ROR_16),
    kInstruction_cmn_al_r7_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r0_LSL_31),
    kInstruction_cmn_al_r3_r0_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r2_LSL_2),
    kInstruction_cmn_al_r2_r2_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r1_LSL_9),
    kInstruction_cmn_al_r8_r1_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r6_LSL_6),
    kInstruction_cmn_al_r13_r6_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r8_LSL_13),
    kInstruction_cmn_al_r0_r8_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r8_ROR_3),
    kInstruction_cmn_al_r11_r8_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r1_LSL_13),
    kInstruction_cmn_al_r8_r1_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r2_LSL_13),
    kInstruction_cmn_al_r6_r2_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r1_LSL_16),
    kInstruction_cmn_al_r4_r1_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r4_LSL_9),
    kInstruction_cmn_al_r0_r4_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r8_ROR_8),
    kInstruction_cmn_al_r11_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r12_LSL_5),
    kInstruction_cmn_al_r9_r12_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r9_ROR_13),
    kInstruction_cmn_al_r13_r9_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r0_ROR_23),
    kInstruction_cmn_al_r2_r0_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r8_ROR_6),
    kInstruction_cmn_al_r7_r8_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r6_LSL_11),
    kInstruction_cmn_al_r2_r6_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r13_LSL_12),
    kInstruction_cmn_al_r2_r13_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r4_LSL_13),
    kInstruction_cmn_al_r7_r4_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r4_LSL_1),
    kInstruction_cmn_al_r8_r4_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r4_LSL_8),
    kInstruction_cmn_al_r14_r4_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r9_ROR_20),
    kInstruction_cmn_al_r7_r9_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r3_ROR_6),
    kInstruction_cmn_al_r4_r3_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r12_LSL_2),
    kInstruction_cmn_al_r3_r12_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r9_ROR_7),
    kInstruction_cmn_al_r12_r9_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r2_LSL_17),
    kInstruction_cmn_al_r0_r2_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r3_LSL_28),
    kInstruction_cmn_al_r14_r3_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r12_ROR_15),
    kInstruction_cmn_al_r14_r12_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r0_ROR_8),
    kInstruction_cmn_al_r8_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r11_LSL_20),
    kInstruction_cmn_al_r2_r11_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r13_LSL_30),
    kInstruction_cmn_al_r8_r13_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r9_LSL_24),
    kInstruction_cmn_al_r2_r9_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r1_LSL_2),
    kInstruction_cmn_al_r12_r1_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r1_ROR_7),
    kInstruction_cmn_al_r7_r1_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r7_ROR_30),
    kInstruction_cmn_al_r11_r7_ROR_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r9_ROR_22),
    kInstruction_cmn_al_r0_r9_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r9_ROR_17),
    kInstruction_cmn_al_r14_r9_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r14_LSL_3),
    kInstruction_cmn_al_r3_r14_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r0_ROR_30),
    kInstruction_cmn_al_r0_r0_ROR_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r8_LSL_4),
    kInstruction_cmn_al_r9_r8_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r13_LSL_16),
    kInstruction_cmn_al_r8_r13_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r8_ROR_28),
    kInstruction_cmn_al_r0_r8_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r10_LSL_23),
    kInstruction_cmn_al_r11_r10_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r4_LSL_27),
    kInstruction_cmn_al_r11_r4_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r2_LSL_30),
    kInstruction_cmn_al_r2_r2_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r4_LSL_17),
    kInstruction_cmn_al_r5_r4_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r12_LSL_1),
    kInstruction_cmn_al_r11_r12_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r14_ROR_21),
    kInstruction_cmn_al_r0_r14_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r5_ROR_21),
    kInstruction_cmn_al_r11_r5_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r14_LSL_14),
    kInstruction_cmn_al_r6_r14_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r9_LSL_24),
    kInstruction_cmn_al_r1_r9_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r2_LSL_22),
    kInstruction_cmn_al_r5_r2_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r5_LSL_4),
    kInstruction_cmn_al_r3_r5_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r0_LSL_5),
    kInstruction_cmn_al_r8_r0_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r14_ROR_13),
    kInstruction_cmn_al_r5_r14_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r11_LSL_27),
    kInstruction_cmn_al_r12_r11_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r14_LSL_20),
    kInstruction_cmn_al_r10_r14_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r9_ROR_4),
    kInstruction_cmn_al_r5_r9_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r11_ROR_11),
    kInstruction_cmn_al_r8_r11_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r1_LSL_24),
    kInstruction_cmn_al_r4_r1_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r10_ROR_4),
    kInstruction_cmn_al_r7_r10_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r0_ROR_11),
    kInstruction_cmn_al_r3_r0_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r10_ROR_23),
    kInstruction_cmn_al_r3_r10_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r8_LSL_15),
    kInstruction_cmn_al_r2_r8_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r6_LSL_18),
    kInstruction_cmn_al_r3_r6_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r2_ROR_7),
    kInstruction_cmn_al_r10_r2_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r6_ROR_14),
    kInstruction_cmn_al_r6_r6_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r8_LSL_25),
    kInstruction_cmn_al_r1_r8_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r14_LSL_13),
    kInstruction_cmn_al_r0_r14_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r1_LSL_13),
    kInstruction_cmn_al_r14_r1_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r11_ROR_8),
    kInstruction_cmn_al_r8_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r12_ROR_6),
    kInstruction_cmn_al_r0_r12_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r12_LSL_26),
    kInstruction_cmn_al_r0_r12_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r2_LSL_10),
    kInstruction_cmn_al_r11_r2_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r7_LSL_3),
    kInstruction_cmn_al_r12_r7_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r10_LSL_25),
    kInstruction_cmn_al_r6_r10_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r7_ROR_8),
    kInstruction_cmn_al_r12_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r3_ROR_24),
    kInstruction_cmn_al_r3_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r12_LSL_24),
    kInstruction_cmn_al_r9_r12_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r10_LSL_27),
    kInstruction_cmn_al_r14_r10_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r5_ROR_4),
    kInstruction_cmn_al_r6_r5_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r4_LSL_21),
    kInstruction_cmn_al_r7_r4_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r4_ROR_31),
    kInstruction_cmn_al_r2_r4_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r7_ROR_8),
    kInstruction_cmn_al_r11_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r2_ROR_17),
    kInstruction_cmn_al_r5_r2_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r11_LSL_13),
    kInstruction_cmn_al_r1_r11_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r11_ROR_2),
    kInstruction_cmn_al_r7_r11_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r7_ROR_29),
    kInstruction_cmn_al_r1_r7_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r1_LSL_31),
    kInstruction_cmn_al_r4_r1_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r13_ROR_11),
    kInstruction_cmn_al_r11_r13_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r1_ROR_14),
    kInstruction_cmn_al_r13_r1_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r14_ROR_21),
    kInstruction_cmn_al_r1_r14_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r12_LSL_5),
    kInstruction_cmn_al_r7_r12_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r3_LSL_2),
    kInstruction_cmn_al_r10_r3_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r10_ROR_25),
    kInstruction_cmn_al_r12_r10_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r7_LSL_6),
    kInstruction_cmn_al_r2_r7_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r0_LSL_23),
    kInstruction_cmn_al_r5_r0_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r8_ROR_18),
    kInstruction_cmn_al_r2_r8_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r1_ROR_15),
    kInstruction_cmn_al_r0_r1_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r4_ROR_20),
    kInstruction_cmn_al_r14_r4_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r10_LSL_15),
    kInstruction_cmn_al_r5_r10_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r1_ROR_28),
    kInstruction_cmn_al_r0_r1_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r2_LSL_9),
    kInstruction_cmn_al_r11_r2_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r10_LSL_19),
    kInstruction_cmn_al_r9_r10_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r11_LSL_8),
    kInstruction_cmn_al_r4_r11_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r14_LSL_12),
    kInstruction_cmn_al_r12_r14_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r9_LSL_8),
    kInstruction_cmn_al_r5_r9_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r14_LSL_16),
    kInstruction_cmn_al_r12_r14_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r13_LSL_23),
    kInstruction_cmn_al_r13_r13_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r12_ROR_19),
    kInstruction_cmn_al_r3_r12_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r5_ROR_24),
    kInstruction_cmn_al_r2_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r2_ROR_11),
    kInstruction_cmn_al_r3_r2_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r13_ROR_1),
    kInstruction_cmn_al_r1_r13_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r7_LSL_11),
    kInstruction_cmn_al_r6_r7_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r14_LSL_2),
    kInstruction_cmn_al_r3_r14_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r10_LSL_20),
    kInstruction_cmn_al_r0_r10_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r0_LSL_19),
    kInstruction_cmn_al_r3_r0_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r0_ROR_21),
    kInstruction_cmn_al_r14_r0_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r13_LSL_16),
    kInstruction_cmn_al_r12_r13_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r5_ROR_28),
    kInstruction_cmn_al_r8_r5_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r1_LSL_25),
    kInstruction_cmn_al_r8_r1_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r4_LSL_25),
    kInstruction_cmn_al_r6_r4_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r12_ROR_26),
    kInstruction_cmn_al_r7_r12_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r14_LSL_12),
    kInstruction_cmn_al_r8_r14_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r6_ROR_2),
    kInstruction_cmn_al_r6_r6_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r11_LSL_13),
    kInstruction_cmn_al_r11_r11_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r12_ROR_25),
    kInstruction_cmn_al_r1_r12_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r5_LSL_20),
    kInstruction_cmn_al_r9_r5_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r11_LSL_20),
    kInstruction_cmn_al_r12_r11_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r3_ROR_31),
    kInstruction_cmn_al_r8_r3_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r14_ROR_3),
    kInstruction_cmn_al_r0_r14_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r14_LSL_30),
    kInstruction_cmn_al_r13_r14_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r6_ROR_24),
    kInstruction_cmn_al_r3_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r10_LSL_23),
    kInstruction_cmn_al_r10_r10_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r1_LSL_7),
    kInstruction_cmn_al_r9_r1_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r13_ROR_27),
    kInstruction_cmn_al_r6_r13_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r14_LSL_21),
    kInstruction_cmn_al_r1_r14_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r14_ROR_12),
    kInstruction_cmn_al_r9_r14_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r2_LSL_17),
    kInstruction_cmn_al_r14_r2_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r14_LSL_11),
    kInstruction_cmn_al_r11_r14_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r7_ROR_2),
    kInstruction_cmn_al_r11_r7_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r7_ROR_20),
    kInstruction_cmn_al_r8_r7_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r11_LSL_5),
    kInstruction_cmn_al_r10_r11_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r11_ROR_16),
    kInstruction_cmn_al_r5_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r3_LSL_23),
    kInstruction_cmn_al_r8_r3_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r8_LSL_23),
    kInstruction_cmn_al_r2_r8_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r13_LSL_28),
    kInstruction_cmn_al_r2_r13_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r0_LSL_20),
    kInstruction_cmn_al_r1_r0_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r8_ROR_21),
    kInstruction_cmn_al_r9_r8_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r14_LSL_22),
    kInstruction_cmn_al_r5_r14_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r9_ROR_7),
    kInstruction_cmn_al_r4_r9_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r12_ROR_9),
    kInstruction_cmn_al_r2_r12_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r10_LSL_31),
    kInstruction_cmn_al_r4_r10_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r2_LSL_8),
    kInstruction_cmn_al_r4_r2_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r6_LSL_27),
    kInstruction_cmn_al_r10_r6_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r7_ROR_7),
    kInstruction_cmn_al_r12_r7_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r1_ROR_9),
    kInstruction_cmn_al_r5_r1_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r13_LSL_29),
    kInstruction_cmn_al_r13_r13_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r12_ROR_28),
    kInstruction_cmn_al_r0_r12_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r6_ROR_26),
    kInstruction_cmn_al_r9_r6_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r8_ROR_14),
    kInstruction_cmn_al_r1_r8_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r5_LSL_19),
    kInstruction_cmn_al_r9_r5_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r5_LSL_23),
    kInstruction_cmn_al_r10_r5_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r12_ROR_2),
    kInstruction_cmn_al_r2_r12_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r10_ROR_18),
    kInstruction_cmn_al_r5_r10_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r12_ROR_12),
    kInstruction_cmn_al_r13_r12_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r10_LSL_15),
    kInstruction_cmn_al_r9_r10_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r2_LSL_26),
    kInstruction_cmn_al_r9_r2_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r3_LSL_19),
    kInstruction_cmn_al_r2_r3_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r10_ROR_28),
    kInstruction_cmn_al_r7_r10_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r6_ROR_6),
    kInstruction_cmn_al_r14_r6_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r1_ROR_16),
    kInstruction_cmn_al_r5_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r6_ROR_10),
    kInstruction_cmn_al_r4_r6_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r12_LSL_10),
    kInstruction_cmn_al_r7_r12_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r14_LSL_28),
    kInstruction_cmn_al_r0_r14_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r2_LSL_27),
    kInstruction_cmn_al_r8_r2_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r12_ROR_1),
    kInstruction_cmn_al_r8_r12_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r4_ROR_4),
    kInstruction_cmn_al_r8_r4_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r5_ROR_20),
    kInstruction_cmn_al_r3_r5_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r13_LSL_5),
    kInstruction_cmn_al_r6_r13_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r9_LSL_4),
    kInstruction_cmn_al_r9_r9_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r13_ROR_7),
    kInstruction_cmn_al_r13_r13_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r7_LSL_11),
    kInstruction_cmn_al_r3_r7_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r3_ROR_15),
    kInstruction_cmn_al_r1_r3_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r0_LSL_30),
    kInstruction_cmn_al_r2_r0_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r13_ROR_16),
    kInstruction_cmn_al_r11_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r0_LSL_15),
    kInstruction_cmn_al_r3_r0_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r0_LSL_8),
    kInstruction_cmn_al_r2_r0_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r13_ROR_5),
    kInstruction_cmn_al_r4_r13_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r0_ROR_29),
    kInstruction_cmn_al_r13_r0_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r2_ROR_18),
    kInstruction_cmn_al_r6_r2_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r2_LSL_2),
    kInstruction_cmn_al_r1_r2_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r4_ROR_12),
    kInstruction_cmn_al_r7_r4_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r11_ROR_6),
    kInstruction_cmn_al_r9_r11_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r2_ROR_12),
    kInstruction_cmn_al_r8_r2_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r2_LSL_19),
    kInstruction_cmn_al_r12_r2_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r0_ROR_30),
    kInstruction_cmn_al_r13_r0_ROR_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r6_ROR_16),
    kInstruction_cmn_al_r7_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r13_ROR_19),
    kInstruction_cmn_al_r6_r13_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r3_LSL_19),
    kInstruction_cmn_al_r14_r3_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r5_ROR_24),
    kInstruction_cmn_al_r14_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r12_LSL_8),
    kInstruction_cmn_al_r10_r12_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r7_LSL_11),
    kInstruction_cmn_al_r14_r7_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r14_LSL_30),
    kInstruction_cmn_al_r10_r14_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r13_ROR_7),
    kInstruction_cmn_al_r7_r13_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r0_LSL_23),
    kInstruction_cmn_al_r11_r0_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r12_LSL_30),
    kInstruction_cmn_al_r0_r12_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r12_LSL_7),
    kInstruction_cmn_al_r5_r12_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r12_ROR_14),
    kInstruction_cmn_al_r12_r12_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r11_LSL_4),
    kInstruction_cmn_al_r8_r11_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r2_ROR_24),
    kInstruction_cmn_al_r14_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r2_ROR_31),
    kInstruction_cmn_al_r10_r2_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r5_LSL_1),
    kInstruction_cmn_al_r11_r5_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r6_LSL_24),
    kInstruction_cmn_al_r8_r6_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r1_ROR_8),
    kInstruction_cmn_al_r14_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r2_ROR_19),
    kInstruction_cmn_al_r11_r2_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r12_LSL_14),
    kInstruction_cmn_al_r1_r12_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r10_LSL_30),
    kInstruction_cmn_al_r6_r10_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r1_ROR_6),
    kInstruction_cmn_al_r8_r1_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r11_ROR_6),
    kInstruction_cmn_al_r13_r11_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r13_LSL_30),
    kInstruction_cmn_al_r5_r13_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r13_ROR_26),
    kInstruction_cmn_al_r0_r13_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r10_LSL_30),
    kInstruction_cmn_al_r4_r10_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r4_LSL_16),
    kInstruction_cmn_al_r8_r4_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r12_LSL_11),
    kInstruction_cmn_al_r11_r12_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r3_ROR_2),
    kInstruction_cmn_al_r9_r3_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r7_LSL_26),
    kInstruction_cmn_al_r11_r7_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r2_ROR_22),
    kInstruction_cmn_al_r14_r2_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r2_ROR_10),
    kInstruction_cmn_al_r8_r2_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r2_ROR_28),
    kInstruction_cmn_al_r5_r2_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r3_ROR_11),
    kInstruction_cmn_al_r14_r3_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r14_LSL_5),
    kInstruction_cmn_al_r3_r14_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r3_ROR_15),
    kInstruction_cmn_al_r12_r3_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r1_ROR_30),
    kInstruction_cmn_al_r4_r1_ROR_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r8_ROR_11),
    kInstruction_cmn_al_r13_r8_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r9_ROR_9),
    kInstruction_cmn_al_r6_r9_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r13_ROR_15),
    kInstruction_cmn_al_r8_r13_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r14_ROR_12),
    kInstruction_cmn_al_r0_r14_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r10_LSL_6),
    kInstruction_cmn_al_r0_r10_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r7_LSL_5),
    kInstruction_cmn_al_r8_r7_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r7_ROR_11),
    kInstruction_cmn_al_r3_r7_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r1_LSL_20),
    kInstruction_cmn_al_r5_r1_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r7_LSL_10),
    kInstruction_cmn_al_r6_r7_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r0_ROR_27),
    kInstruction_cmn_al_r14_r0_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r12_LSL_5),
    kInstruction_cmn_al_r6_r12_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r8_LSL_28),
    kInstruction_cmn_al_r4_r8_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r6_ROR_15),
    kInstruction_cmn_al_r7_r6_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r6_LSL_27),
    kInstruction_cmn_al_r0_r6_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r6_ROR_19),
    kInstruction_cmn_al_r1_r6_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r14_ROR_21),
    kInstruction_cmn_al_r11_r14_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r13_LSL_5),
    kInstruction_cmn_al_r4_r13_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r7_ROR_2),
    kInstruction_cmn_al_r4_r7_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r1_ROR_29),
    kInstruction_cmn_al_r8_r1_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r2_LSL_24),
    kInstruction_cmn_al_r14_r2_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r12_ROR_27),
    kInstruction_cmn_al_r5_r12_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r4_ROR_14),
    kInstruction_cmn_al_r11_r4_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r7_LSL_28),
    kInstruction_cmn_al_r4_r7_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r6_LSL_9),
    kInstruction_cmn_al_r3_r6_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r13_LSL_12),
    kInstruction_cmn_al_r11_r13_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r7_ROR_20),
    kInstruction_cmn_al_r10_r7_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r8_LSL_12),
    kInstruction_cmn_al_r2_r8_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r13_LSL_28),
    kInstruction_cmn_al_r4_r13_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r12_LSL_28),
    kInstruction_cmn_al_r1_r12_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r14_LSL_6),
    kInstruction_cmn_al_r7_r14_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r2_ROR_10),
    kInstruction_cmn_al_r11_r2_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r9_ROR_7),
    kInstruction_cmn_al_r3_r9_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r12_LSL_11),
    kInstruction_cmn_al_r2_r12_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r2_ROR_19),
    kInstruction_cmn_al_r4_r2_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r0_LSL_5),
    kInstruction_cmn_al_r13_r0_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r11_ROR_29),
    kInstruction_cmn_al_r14_r11_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r6_LSL_11),
    kInstruction_cmn_al_r9_r6_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r10_ROR_30),
    kInstruction_cmn_al_r2_r10_ROR_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r14_ROR_4),
    kInstruction_cmn_al_r3_r14_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r6_LSL_24),
    kInstruction_cmn_al_r9_r6_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r8_LSL_2),
    kInstruction_cmn_al_r9_r8_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r5_ROR_7),
    kInstruction_cmn_al_r12_r5_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r11_ROR_20),
    kInstruction_cmn_al_r1_r11_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r12_LSL_6),
    kInstruction_cmn_al_r6_r12_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r11_ROR_26),
    kInstruction_cmn_al_r14_r11_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r8_ROR_6),
    kInstruction_cmn_al_r5_r8_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r4_LSL_8),
    kInstruction_cmn_al_r10_r4_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r0_ROR_24),
    kInstruction_cmn_al_r5_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r3_LSL_20),
    kInstruction_cmn_al_r10_r3_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r4_LSL_22),
    kInstruction_cmn_al_r14_r4_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r8_ROR_1),
    kInstruction_cmn_al_r8_r8_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r9_ROR_8),
    kInstruction_cmn_al_r12_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r13_ROR_15),
    kInstruction_cmn_al_r6_r13_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r5_LSL_2),
    kInstruction_cmn_al_r9_r5_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r8_LSL_17),
    kInstruction_cmn_al_r13_r8_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r5_ROR_27),
    kInstruction_cmn_al_r2_r5_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r12_ROR_10),
    kInstruction_cmn_al_r13_r12_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r2_LSL_30),
    kInstruction_cmn_al_r13_r2_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r11_LSL_20),
    kInstruction_cmn_al_r10_r11_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r1_ROR_22),
    kInstruction_cmn_al_r6_r1_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r4_ROR_18),
    kInstruction_cmn_al_r0_r4_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r14_ROR_19),
    kInstruction_cmn_al_r11_r14_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r7_ROR_4),
    kInstruction_cmn_al_r0_r7_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r5_LSL_13),
    kInstruction_cmn_al_r1_r5_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r14_LSL_6),
    kInstruction_cmn_al_r11_r14_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r11_LSL_10),
    kInstruction_cmn_al_r6_r11_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r14_LSL_1),
    kInstruction_cmn_al_r10_r14_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r4_ROR_29),
    kInstruction_cmn_al_r13_r4_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r8_ROR_28),
    kInstruction_cmn_al_r9_r8_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r12_ROR_22),
    kInstruction_cmn_al_r7_r12_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r12_ROR_10),
    kInstruction_cmn_al_r1_r12_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r8_ROR_31),
    kInstruction_cmn_al_r7_r8_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r2_ROR_25),
    kInstruction_cmn_al_r0_r2_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r5_LSL_24),
    kInstruction_cmn_al_r3_r5_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r10_LSL_25),
    kInstruction_cmn_al_r11_r10_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r1_LSL_9),
    kInstruction_cmn_al_r11_r1_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r7_LSL_3),
    kInstruction_cmn_al_r2_r7_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r10_ROR_20),
    kInstruction_cmn_al_r0_r10_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r0_LSL_2),
    kInstruction_cmn_al_r7_r0_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r5_LSL_8),
    kInstruction_cmn_al_r12_r5_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r1_ROR_16),
    kInstruction_cmn_al_r11_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r3_LSL_30),
    kInstruction_cmn_al_r3_r3_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r9_LSL_16),
    kInstruction_cmn_al_r4_r9_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r8_LSL_27),
    kInstruction_cmn_al_r9_r8_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r2_LSL_15),
    kInstruction_cmn_al_r4_r2_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r13_LSL_3),
    kInstruction_cmn_al_r12_r13_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r3_LSL_29),
    kInstruction_cmn_al_r11_r3_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r6_ROR_23),
    kInstruction_cmn_al_r4_r6_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r8_LSL_3),
    kInstruction_cmn_al_r5_r8_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r9_ROR_4),
    kInstruction_cmn_al_r1_r9_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r6_LSL_12),
    kInstruction_cmn_al_r13_r6_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r6_ROR_17),
    kInstruction_cmn_al_r12_r6_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r1_LSL_22),
    kInstruction_cmn_al_r2_r1_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r2_LSL_10),
    kInstruction_cmn_al_r9_r2_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r10_LSL_21),
    kInstruction_cmn_al_r11_r10_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r12_LSL_31),
    kInstruction_cmn_al_r8_r12_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r2_LSL_11),
    kInstruction_cmn_al_r5_r2_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r6_LSL_31),
    kInstruction_cmn_al_r9_r6_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r6_ROR_23),
    kInstruction_cmn_al_r6_r6_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r3_LSL_5),
    kInstruction_cmn_al_r14_r3_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r9_ROR_26),
    kInstruction_cmn_al_r14_r9_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r1_ROR_15),
    kInstruction_cmn_al_r13_r1_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r13_LSL_13),
    kInstruction_cmn_al_r2_r13_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r7_LSL_8),
    kInstruction_cmn_al_r14_r7_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r4_LSL_8),
    kInstruction_cmn_al_r9_r4_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r13_LSL_26),
    kInstruction_cmn_al_r0_r13_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r1_LSL_12),
    kInstruction_cmn_al_r11_r1_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r6_LSL_5),
    kInstruction_cmn_al_r7_r6_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r9_LSL_15),
    kInstruction_cmn_al_r4_r9_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r14_ROR_19),
    kInstruction_cmn_al_r2_r14_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r4_LSL_21),
    kInstruction_cmn_al_r12_r4_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r9_ROR_23),
    kInstruction_cmn_al_r0_r9_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r13_ROR_31),
    kInstruction_cmn_al_r5_r13_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r1_ROR_19),
    kInstruction_cmn_al_r8_r1_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r11_ROR_10),
    kInstruction_cmn_al_r1_r11_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r7_ROR_22),
    kInstruction_cmn_al_r10_r7_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r11_LSL_26),
    kInstruction_cmn_al_r10_r11_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r14_ROR_15),
    kInstruction_cmn_al_r0_r14_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r9_LSL_20),
    kInstruction_cmn_al_r1_r9_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r0_LSL_11),
    kInstruction_cmn_al_r7_r0_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r14_ROR_15),
    kInstruction_cmn_al_r10_r14_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r11_LSL_2),
    kInstruction_cmn_al_r1_r11_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r13_ROR_2),
    kInstruction_cmn_al_r1_r13_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r1_ROR_13),
    kInstruction_cmn_al_r12_r1_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r4_LSL_17),
    kInstruction_cmn_al_r1_r4_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r4_ROR_16),
    kInstruction_cmn_al_r11_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r1_LSL_31),
    kInstruction_cmn_al_r1_r1_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r0_ROR_15),
    kInstruction_cmn_al_r11_r0_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r12_LSL_10),
    kInstruction_cmn_al_r1_r12_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r12_LSL_11),
    kInstruction_cmn_al_r0_r12_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r10_ROR_8),
    kInstruction_cmn_al_r10_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r5_ROR_29),
    kInstruction_cmn_al_r2_r5_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r7_LSL_9),
    kInstruction_cmn_al_r7_r7_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r1_LSL_11),
    kInstruction_cmn_al_r9_r1_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r8_ROR_14),
    kInstruction_cmn_al_r5_r8_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r6_ROR_25),
    kInstruction_cmn_al_r1_r6_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r2_LSL_7),
    kInstruction_cmn_al_r13_r2_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r3_ROR_24),
    kInstruction_cmn_al_r0_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r5_ROR_31),
    kInstruction_cmn_al_r4_r5_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r13_ROR_28),
    kInstruction_cmn_al_r0_r13_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r3_LSL_5),
    kInstruction_cmn_al_r11_r3_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r1_LSL_1),
    kInstruction_cmn_al_r3_r1_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r7_ROR_8),
    kInstruction_cmn_al_r10_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r13_LSL_17),
    kInstruction_cmn_al_r0_r13_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r3_LSL_16),
    kInstruction_cmn_al_r10_r3_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r14_LSL_21),
    kInstruction_cmn_al_r14_r14_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r7_ROR_12),
    kInstruction_cmn_al_r0_r7_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r11_LSL_1),
    kInstruction_cmn_al_r4_r11_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r10_ROR_25),
    kInstruction_cmn_al_r4_r10_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r7_LSL_10),
    kInstruction_cmn_al_r11_r7_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r13_LSL_13),
    kInstruction_cmn_al_r4_r13_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r4_ROR_9),
    kInstruction_cmn_al_r11_r4_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r3_LSL_4),
    kInstruction_cmn_al_r8_r3_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r3_LSL_22),
    kInstruction_cmn_al_r0_r3_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r3_LSL_16),
    kInstruction_cmn_al_r9_r3_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r11_LSL_24),
    kInstruction_cmn_al_r1_r11_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r7_LSL_11),
    kInstruction_cmn_al_r0_r7_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r8_LSL_29),
    kInstruction_cmn_al_r13_r8_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r8_LSL_1),
    kInstruction_cmn_al_r8_r8_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r1_LSL_21),
    kInstruction_cmn_al_r8_r1_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r3_LSL_27),
    kInstruction_cmn_al_r1_r3_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r8_ROR_2),
    kInstruction_cmn_al_r1_r8_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r4_ROR_9),
    kInstruction_cmn_al_r9_r4_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r3_ROR_28),
    kInstruction_cmn_al_r10_r3_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r8_ROR_11),
    kInstruction_cmn_al_r4_r8_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r3_LSL_17),
    kInstruction_cmn_al_r2_r3_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r6_ROR_17),
    kInstruction_cmn_al_r5_r6_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r9_LSL_6),
    kInstruction_cmn_al_r8_r9_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r11_ROR_29),
    kInstruction_cmn_al_r9_r11_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r12_ROR_20),
    kInstruction_cmn_al_r11_r12_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r13_LSL_21),
    kInstruction_cmn_al_r8_r13_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r9_LSL_22),
    kInstruction_cmn_al_r8_r9_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r1_ROR_14),
    kInstruction_cmn_al_r11_r1_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r3_LSL_15),
    kInstruction_cmn_al_r13_r3_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r7_LSL_9),
    kInstruction_cmn_al_r3_r7_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r4_ROR_10),
    kInstruction_cmn_al_r5_r4_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r10_LSL_24),
    kInstruction_cmn_al_r11_r10_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r9_ROR_25),
    kInstruction_cmn_al_r1_r9_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r2_ROR_5),
    kInstruction_cmn_al_r9_r2_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r4_ROR_19),
    kInstruction_cmn_al_r5_r4_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r14_ROR_19),
    kInstruction_cmn_al_r1_r14_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r9_LSL_30),
    kInstruction_cmn_al_r10_r9_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r4_ROR_24),
    kInstruction_cmn_al_r9_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r6_LSL_23),
    kInstruction_cmn_al_r6_r6_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r7_LSL_29),
    kInstruction_cmn_al_r5_r7_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r5_LSL_10),
    kInstruction_cmn_al_r10_r5_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r8_LSL_15),
    kInstruction_cmn_al_r13_r8_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r2_ROR_14),
    kInstruction_cmn_al_r5_r2_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r3_ROR_6),
    kInstruction_cmn_al_r11_r3_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r7_ROR_9),
    kInstruction_cmn_al_r12_r7_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r3_ROR_23),
    kInstruction_cmn_al_r1_r3_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r13_LSL_30),
    kInstruction_cmn_al_r10_r13_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r2_LSL_15),
    kInstruction_cmn_al_r3_r2_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r0_LSL_31),
    kInstruction_cmn_al_r6_r0_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r12_LSL_12),
    kInstruction_cmn_al_r13_r12_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r8_ROR_9),
    kInstruction_cmn_al_r7_r8_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r13_LSL_16),
    kInstruction_cmn_al_r7_r13_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r6_LSL_17),
    kInstruction_cmn_al_r3_r6_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r4_ROR_14),
    kInstruction_cmn_al_r13_r4_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r5_ROR_7),
    kInstruction_cmn_al_r2_r5_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r10_ROR_18),
    kInstruction_cmn_al_r7_r10_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r12_LSL_26),
    kInstruction_cmn_al_r7_r12_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r13_ROR_25),
    kInstruction_cmn_al_r13_r13_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r2_ROR_5),
    kInstruction_cmn_al_r11_r2_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r14_ROR_8),
    kInstruction_cmn_al_r0_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r2_ROR_27),
    kInstruction_cmn_al_r6_r2_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r14_LSL_31),
    kInstruction_cmn_al_r4_r14_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r0_LSL_7),
    kInstruction_cmn_al_r1_r0_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r8_LSL_10),
    kInstruction_cmn_al_r0_r8_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r0_LSL_13),
    kInstruction_cmn_al_r9_r0_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r11_ROR_11),
    kInstruction_cmn_al_r11_r11_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r4_ROR_7),
    kInstruction_cmn_al_r11_r4_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r4_ROR_19),
    kInstruction_cmn_al_r10_r4_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r13_ROR_4),
    kInstruction_cmn_al_r1_r13_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r14_ROR_5),
    kInstruction_cmn_al_r9_r14_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r11_LSL_22),
    kInstruction_cmn_al_r14_r11_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r4_LSL_17),
    kInstruction_cmn_al_r2_r4_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r6_LSL_15),
    kInstruction_cmn_al_r2_r6_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r5_ROR_28),
    kInstruction_cmn_al_r10_r5_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r4_ROR_13),
    kInstruction_cmn_al_r6_r4_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r4_ROR_10),
    kInstruction_cmn_al_r3_r4_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r7_ROR_15),
    kInstruction_cmn_al_r3_r7_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r14_LSL_28),
    kInstruction_cmn_al_r7_r14_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r6_ROR_23),
    kInstruction_cmn_al_r10_r6_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r5_ROR_4),
    kInstruction_cmn_al_r4_r5_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r3_ROR_8),
    kInstruction_cmn_al_r9_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r1_LSL_14),
    kInstruction_cmn_al_r2_r1_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r7_LSL_4),
    kInstruction_cmn_al_r10_r7_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r0_LSL_18),
    kInstruction_cmn_al_r0_r0_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r3_ROR_18),
    kInstruction_cmn_al_r12_r3_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r5_LSL_4),
    kInstruction_cmn_al_r5_r5_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r4_ROR_12),
    kInstruction_cmn_al_r11_r4_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r14_ROR_23),
    kInstruction_cmn_al_r5_r14_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r6_LSL_31),
    kInstruction_cmn_al_r7_r6_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r2_ROR_7),
    kInstruction_cmn_al_r14_r2_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r0_ROR_23),
    kInstruction_cmn_al_r10_r0_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r0_LSL_25),
    kInstruction_cmn_al_r6_r0_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r4_ROR_16),
    kInstruction_cmn_al_r7_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r0_LSL_19),
    kInstruction_cmn_al_r9_r0_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r2_LSL_23),
    kInstruction_cmn_al_r7_r2_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r10_LSL_17),
    kInstruction_cmn_al_r7_r10_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r10_ROR_19),
    kInstruction_cmn_al_r8_r10_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r9_LSL_19),
    kInstruction_cmn_al_r6_r9_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r6_LSL_14),
    kInstruction_cmn_al_r12_r6_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r12_ROR_24),
    kInstruction_cmn_al_r4_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r6_LSL_6),
    kInstruction_cmn_al_r3_r6_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r6_ROR_4),
    kInstruction_cmn_al_r11_r6_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r3_LSL_10),
    kInstruction_cmn_al_r14_r3_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r1_LSL_12),
    kInstruction_cmn_al_r1_r1_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r8_LSL_19),
    kInstruction_cmn_al_r7_r8_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r5_LSL_22),
    kInstruction_cmn_al_r5_r5_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r8_ROR_29),
    kInstruction_cmn_al_r14_r8_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r14_LSL_24),
    kInstruction_cmn_al_r3_r14_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r14_ROR_5),
    kInstruction_cmn_al_r11_r14_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r4_ROR_15),
    kInstruction_cmn_al_r12_r4_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r5_LSL_12),
    kInstruction_cmn_al_r3_r5_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r12_ROR_26),
    kInstruction_cmn_al_r0_r12_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r7_ROR_12),
    kInstruction_cmn_al_r5_r7_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r9_LSL_17),
    kInstruction_cmn_al_r8_r9_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r2_ROR_13),
    kInstruction_cmn_al_r1_r2_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r13_ROR_8),
    kInstruction_cmn_al_r0_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r0_LSL_9),
    kInstruction_cmn_al_r4_r0_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r2_LSL_21),
    kInstruction_cmn_al_r6_r2_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r3_ROR_29),
    kInstruction_cmn_al_r5_r3_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r6_LSL_7),
    kInstruction_cmn_al_r14_r6_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r4_ROR_11),
    kInstruction_cmn_al_r9_r4_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r0_LSL_14),
    kInstruction_cmn_al_r4_r0_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r14_ROR_3),
    kInstruction_cmn_al_r10_r14_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r11_LSL_31),
    kInstruction_cmn_al_r0_r11_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r2_LSL_17),
    kInstruction_cmn_al_r5_r2_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r6_ROR_26),
    kInstruction_cmn_al_r2_r6_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r9_LSL_8),
    kInstruction_cmn_al_r9_r9_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r12_LSL_19),
    kInstruction_cmn_al_r12_r12_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r11_LSL_10),
    kInstruction_cmn_al_r2_r11_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r7_ROR_6),
    kInstruction_cmn_al_r4_r7_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r3_ROR_14),
    kInstruction_cmn_al_r2_r3_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r12_ROR_16),
    kInstruction_cmn_al_r4_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r12_LSL_2),
    kInstruction_cmn_al_r12_r12_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r5_ROR_23),
    kInstruction_cmn_al_r10_r5_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r14_ROR_24),
    kInstruction_cmn_al_r8_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r1_LSL_15),
    kInstruction_cmn_al_r7_r1_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r10_LSL_8),
    kInstruction_cmn_al_r13_r10_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r9_ROR_5),
    kInstruction_cmn_al_r1_r9_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r2_ROR_17),
    kInstruction_cmn_al_r1_r2_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r3_ROR_30),
    kInstruction_cmn_al_r12_r3_ROR_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r5_LSL_18),
    kInstruction_cmn_al_r0_r5_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r7_LSL_2),
    kInstruction_cmn_al_r5_r7_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r4_LSL_30),
    kInstruction_cmn_al_r13_r4_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r14_ROR_31),
    kInstruction_cmn_al_r2_r14_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r8_LSL_30),
    kInstruction_cmn_al_r12_r8_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r5_ROR_4),
    kInstruction_cmn_al_r3_r5_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r11_ROR_3),
    kInstruction_cmn_al_r11_r11_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r2_ROR_17),
    kInstruction_cmn_al_r4_r2_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r9_LSL_15),
    kInstruction_cmn_al_r6_r9_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r6_ROR_25),
    kInstruction_cmn_al_r5_r6_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r3_LSL_19),
    kInstruction_cmn_al_r6_r3_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r7_LSL_6),
    kInstruction_cmn_al_r3_r7_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r1_LSL_13),
    kInstruction_cmn_al_r12_r1_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r1_ROR_9),
    kInstruction_cmn_al_r2_r1_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r4_LSL_28),
    kInstruction_cmn_al_r12_r4_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r13_ROR_12),
    kInstruction_cmn_al_r14_r13_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r12_LSL_16),
    kInstruction_cmn_al_r11_r12_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r7_LSL_29),
    kInstruction_cmn_al_r8_r7_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r10_ROR_25),
    kInstruction_cmn_al_r0_r10_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r6_LSL_3),
    kInstruction_cmn_al_r9_r6_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r12_LSL_18),
    kInstruction_cmn_al_r9_r12_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r14_LSL_9),
    kInstruction_cmn_al_r1_r14_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r2_ROR_7),
    kInstruction_cmn_al_r3_r2_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r9_ROR_16),
    kInstruction_cmn_al_r14_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r12_LSL_2),
    kInstruction_cmn_al_r6_r12_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r7_LSL_14),
    kInstruction_cmn_al_r12_r7_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r10_LSL_8),
    kInstruction_cmn_al_r12_r10_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r13_ROR_27),
    kInstruction_cmn_al_r4_r13_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r5_ROR_20),
    kInstruction_cmn_al_r5_r5_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r5_ROR_14),
    kInstruction_cmn_al_r1_r5_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r2_ROR_28),
    kInstruction_cmn_al_r6_r2_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r8_LSL_9),
    kInstruction_cmn_al_r1_r8_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r10_LSL_16),
    kInstruction_cmn_al_r2_r10_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r1_LSL_30),
    kInstruction_cmn_al_r5_r1_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r3_ROR_19),
    kInstruction_cmn_al_r5_r3_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r2_LSL_7),
    kInstruction_cmn_al_r1_r2_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r10_ROR_11),
    kInstruction_cmn_al_r7_r10_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r10_LSL_10),
    kInstruction_cmn_al_r0_r10_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r5_ROR_1),
    kInstruction_cmn_al_r7_r5_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r1_LSL_3),
    kInstruction_cmn_al_r0_r1_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r0_LSL_24),
    kInstruction_cmn_al_r11_r0_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r7_LSL_29),
    kInstruction_cmn_al_r3_r7_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r3_ROR_28),
    kInstruction_cmn_al_r5_r3_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r4_LSL_26),
    kInstruction_cmn_al_r3_r4_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r10_LSL_30),
    kInstruction_cmn_al_r8_r10_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r5_ROR_7),
    kInstruction_cmn_al_r5_r5_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r7_ROR_13),
    kInstruction_cmn_al_r6_r7_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r1_ROR_20),
    kInstruction_cmn_al_r7_r1_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r11_LSL_1),
    kInstruction_cmn_al_r11_r11_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r13_LSL_1),
    kInstruction_cmn_al_r10_r13_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r11_LSL_9),
    kInstruction_cmn_al_r6_r11_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r3_LSL_18),
    kInstruction_cmn_al_r2_r3_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r4_ROR_16),
    kInstruction_cmn_al_r3_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r10_LSL_1),
    kInstruction_cmn_al_r4_r10_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r5_LSL_31),
    kInstruction_cmn_al_r1_r5_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r6_LSL_29),
    kInstruction_cmn_al_r6_r6_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r13_LSL_5),
    kInstruction_cmn_al_r10_r13_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r6_LSL_24),
    kInstruction_cmn_al_r14_r6_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r12_LSL_31),
    kInstruction_cmn_al_r4_r12_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r7_ROR_6),
    kInstruction_cmn_al_r12_r7_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r10_LSL_26),
    kInstruction_cmn_al_r14_r10_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r9_ROR_24),
    kInstruction_cmn_al_r9_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r13_LSL_5),
    kInstruction_cmn_al_r8_r13_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r11_LSL_17),
    kInstruction_cmn_al_r4_r11_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r14_LSL_9),
    kInstruction_cmn_al_r2_r14_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r0_ROR_29),
    kInstruction_cmn_al_r1_r0_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r2_ROR_9),
    kInstruction_cmn_al_r10_r2_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r7_ROR_29),
    kInstruction_cmn_al_r3_r7_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r12_ROR_1),
    kInstruction_cmn_al_r1_r12_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r2_ROR_8),
    kInstruction_cmn_al_r7_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r1_LSL_24),
    kInstruction_cmn_al_r7_r1_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r1_LSL_17),
    kInstruction_cmn_al_r4_r1_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r11_ROR_22),
    kInstruction_cmn_al_r12_r11_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r4_ROR_20),
    kInstruction_cmn_al_r0_r4_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r12_LSL_11),
    kInstruction_cmn_al_r5_r12_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r2_ROR_31),
    kInstruction_cmn_al_r11_r2_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r11_LSL_4),
    kInstruction_cmn_al_r14_r11_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r8_LSL_12),
    kInstruction_cmn_al_r6_r8_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r0_LSL_30),
    kInstruction_cmn_al_r14_r0_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r11_LSL_29),
    kInstruction_cmn_al_r6_r11_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r12_LSL_21),
    kInstruction_cmn_al_r6_r12_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r6_LSL_29),
    kInstruction_cmn_al_r9_r6_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r13_ROR_29),
    kInstruction_cmn_al_r10_r13_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r7_ROR_16),
    kInstruction_cmn_al_r5_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r14_LSL_27),
    kInstruction_cmn_al_r0_r14_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r4_ROR_18),
    kInstruction_cmn_al_r14_r4_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r2_LSL_28),
    kInstruction_cmn_al_r14_r2_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r3_LSL_29),
    kInstruction_cmn_al_r13_r3_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r7_LSL_19),
    kInstruction_cmn_al_r8_r7_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r1_LSL_1),
    kInstruction_cmn_al_r14_r1_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r11_ROR_2),
    kInstruction_cmn_al_r1_r11_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r4_ROR_14),
    kInstruction_cmn_al_r7_r4_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r3_ROR_22),
    kInstruction_cmn_al_r6_r3_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r4_LSL_1),
    kInstruction_cmn_al_r5_r4_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r11_LSL_6),
    kInstruction_cmn_al_r11_r11_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r5_LSL_21),
    kInstruction_cmn_al_r13_r5_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r5_ROR_10),
    kInstruction_cmn_al_r7_r5_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r10_LSL_17),
    kInstruction_cmn_al_r13_r10_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r1_LSL_5),
    kInstruction_cmn_al_r7_r1_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r11_LSL_25),
    kInstruction_cmn_al_r0_r11_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r2_LSL_9),
    kInstruction_cmn_al_r3_r2_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r11_ROR_12),
    kInstruction_cmn_al_r8_r11_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r9_LSL_21),
    kInstruction_cmn_al_r12_r9_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r3_ROR_13),
    kInstruction_cmn_al_r14_r3_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r8_ROR_26),
    kInstruction_cmn_al_r13_r8_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r11_ROR_11),
    kInstruction_cmn_al_r4_r11_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r3_ROR_16),
    kInstruction_cmn_al_r10_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r0_ROR_12),
    kInstruction_cmn_al_r14_r0_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r4_LSL_18),
    kInstruction_cmn_al_r11_r4_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r13_ROR_3),
    kInstruction_cmn_al_r13_r13_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r14_ROR_10),
    kInstruction_cmn_al_r10_r14_ROR_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r10_LSL_12),
    kInstruction_cmn_al_r12_r10_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r0_ROR_29),
    kInstruction_cmn_al_r0_r0_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r3_LSL_4),
    kInstruction_cmn_al_r13_r3_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r10_LSL_15),
    kInstruction_cmn_al_r1_r10_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r10_LSL_18),
    kInstruction_cmn_al_r3_r10_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r14_ROR_13),
    kInstruction_cmn_al_r11_r14_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r8_ROR_9),
    kInstruction_cmn_al_r5_r8_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r7_LSL_3),
    kInstruction_cmn_al_r1_r7_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r9_LSL_16),
    kInstruction_cmn_al_r8_r9_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r6_ROR_29),
    kInstruction_cmn_al_r6_r6_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r10_ROR_6),
    kInstruction_cmn_al_r4_r10_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r7_LSL_8),
    kInstruction_cmn_al_r13_r7_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r0_LSL_26),
    kInstruction_cmn_al_r0_r0_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r3_ROR_14),
    kInstruction_cmn_al_r6_r3_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r12_ROR_16),
    kInstruction_cmn_al_r13_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r14_ROR_16),
    kInstruction_cmn_al_r0_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r13_ROR_3),
    kInstruction_cmn_al_r2_r13_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r5_LSL_31),
    kInstruction_cmn_al_r3_r5_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r10_LSL_14),
    kInstruction_cmn_al_r12_r10_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r10_LSL_24),
    kInstruction_cmn_al_r5_r10_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r5_LSL_21),
    kInstruction_cmn_al_r5_r5_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r1_LSL_23),
    kInstruction_cmn_al_r1_r1_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r0_LSL_12),
    kInstruction_cmn_al_r6_r0_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r2_ROR_22),
    kInstruction_cmn_al_r6_r2_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r0_LSL_15),
    kInstruction_cmn_al_r10_r0_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r9_ROR_28),
    kInstruction_cmn_al_r14_r9_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r2_ROR_6),
    kInstruction_cmn_al_r0_r2_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r4_LSL_11),
    kInstruction_cmn_al_r9_r4_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r11_LSL_5),
    kInstruction_cmn_al_r5_r11_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r4_ROR_6),
    kInstruction_cmn_al_r7_r4_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r10_ROR_1),
    kInstruction_cmn_al_r2_r10_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r3_LSL_27),
    kInstruction_cmn_al_r2_r3_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r13_LSL_10),
    kInstruction_cmn_al_r5_r13_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r3_LSL_14),
    kInstruction_cmn_al_r10_r3_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r7_ROR_29),
    kInstruction_cmn_al_r12_r7_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r0_ROR_1),
    kInstruction_cmn_al_r0_r0_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r13_LSL_21),
    kInstruction_cmn_al_r11_r13_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r2_LSL_12),
    kInstruction_cmn_al_r7_r2_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r10_LSL_2),
    kInstruction_cmn_al_r1_r10_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r5_LSL_19),
    kInstruction_cmn_al_r11_r5_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r2_ROR_2),
    kInstruction_cmn_al_r13_r2_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r9_LSL_8),
    kInstruction_cmn_al_r7_r9_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r9_ROR_8),
    kInstruction_cmn_al_r5_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r14_LSL_31),
    kInstruction_cmn_al_r10_r14_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r2_ROR_23),
    kInstruction_cmn_al_r0_r2_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r13_LSL_28),
    kInstruction_cmn_al_r14_r13_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r2_LSL_25),
    kInstruction_cmn_al_r1_r2_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r5_LSL_29),
    kInstruction_cmn_al_r3_r5_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r4_ROR_13),
    kInstruction_cmn_al_r3_r4_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r6_ROR_4),
    kInstruction_cmn_al_r4_r6_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r8_ROR_3),
    kInstruction_cmn_al_r6_r8_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r2_ROR_2),
    kInstruction_cmn_al_r11_r2_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r5_LSL_29),
    kInstruction_cmn_al_r14_r5_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r7_ROR_23),
    kInstruction_cmn_al_r0_r7_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r10_LSL_15),
    kInstruction_cmn_al_r2_r10_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r7_LSL_5),
    kInstruction_cmn_al_r6_r7_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r13_ROR_3),
    kInstruction_cmn_al_r11_r13_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r12_ROR_13),
    kInstruction_cmn_al_r5_r12_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r12_LSL_26),
    kInstruction_cmn_al_r9_r12_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r3_ROR_12),
    kInstruction_cmn_al_r7_r3_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r13_LSL_7),
    kInstruction_cmn_al_r9_r13_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r14_LSL_25),
    kInstruction_cmn_al_r6_r14_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r12_ROR_17),
    kInstruction_cmn_al_r1_r12_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r10_LSL_24),
    kInstruction_cmn_al_r1_r10_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r11_LSL_17),
    kInstruction_cmn_al_r0_r11_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r3_ROR_11),
    kInstruction_cmn_al_r8_r3_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r12_ROR_22),
    kInstruction_cmn_al_r11_r12_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r9_ROR_28),
    kInstruction_cmn_al_r0_r9_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r3_ROR_7),
    kInstruction_cmn_al_r9_r3_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r2_ROR_16),
    kInstruction_cmn_al_r9_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r12_LSL_28),
    kInstruction_cmn_al_r0_r12_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r4_ROR_22),
    kInstruction_cmn_al_r1_r4_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r7_LSL_16),
    kInstruction_cmn_al_r12_r7_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r9_ROR_3),
    kInstruction_cmn_al_r7_r9_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r2_LSL_13),
    kInstruction_cmn_al_r8_r2_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r11_LSL_10),
    kInstruction_cmn_al_r10_r11_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r12_ROR_6),
    kInstruction_cmn_al_r10_r12_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r12_LSL_27),
    kInstruction_cmn_al_r3_r12_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r10_ROR_27),
    kInstruction_cmn_al_r10_r10_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r8_LSL_8),
    kInstruction_cmn_al_r8_r8_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r0_ROR_28),
    kInstruction_cmn_al_r3_r0_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r3_ROR_15),
    kInstruction_cmn_al_r10_r3_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r14_LSL_27),
    kInstruction_cmn_al_r1_r14_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r1_LSL_31),
    kInstruction_cmn_al_r10_r1_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r11_LSL_24),
    kInstruction_cmn_al_r12_r11_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r11_ROR_11),
    kInstruction_cmn_al_r5_r11_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r8_ROR_12),
    kInstruction_cmn_al_r13_r8_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r14_LSL_28),
    kInstruction_cmn_al_r1_r14_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r2_LSL_6),
    kInstruction_cmn_al_r6_r2_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r13_ROR_22),
    kInstruction_cmn_al_r14_r13_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r12_ROR_18),
    kInstruction_cmn_al_r12_r12_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r7_ROR_2),
    kInstruction_cmn_al_r8_r7_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r4_LSL_2),
    kInstruction_cmn_al_r6_r4_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r6_ROR_17),
    kInstruction_cmn_al_r2_r6_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r14_ROR_16),
    kInstruction_cmn_al_r11_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r5_ROR_22),
    kInstruction_cmn_al_r1_r5_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r2_ROR_11),
    kInstruction_cmn_al_r2_r2_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r9_ROR_1),
    kInstruction_cmn_al_r12_r9_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r5_LSL_13),
    kInstruction_cmn_al_r13_r5_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r0_LSL_3),
    kInstruction_cmn_al_r0_r0_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r11_ROR_9),
    kInstruction_cmn_al_r0_r11_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r1_LSL_18),
    kInstruction_cmn_al_r6_r1_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r3_LSL_21),
    kInstruction_cmn_al_r6_r3_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r6_LSL_30),
    kInstruction_cmn_al_r7_r6_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r3_ROR_28),
    kInstruction_cmn_al_r11_r3_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r4_ROR_14),
    kInstruction_cmn_al_r0_r4_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r14_ROR_22),
    kInstruction_cmn_al_r1_r14_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r13_LSL_30),
    kInstruction_cmn_al_r3_r13_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r6_LSL_14),
    kInstruction_cmn_al_r14_r6_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r10_ROR_1),
    kInstruction_cmn_al_r11_r10_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r13_ROR_15),
    kInstruction_cmn_al_r11_r13_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r11_LSL_28),
    kInstruction_cmn_al_r5_r11_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r12_ROR_28),
    kInstruction_cmn_al_r9_r12_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r9_ROR_31),
    kInstruction_cmn_al_r14_r9_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r4_LSL_24),
    kInstruction_cmn_al_r7_r4_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r14_ROR_6),
    kInstruction_cmn_al_r11_r14_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r12_LSL_5),
    kInstruction_cmn_al_r2_r12_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r14_ROR_6),
    kInstruction_cmn_al_r1_r14_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r4_LSL_20),
    kInstruction_cmn_al_r6_r4_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r7_LSL_26),
    kInstruction_cmn_al_r10_r7_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r10_LSL_23),
    kInstruction_cmn_al_r5_r10_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r5_ROR_27),
    kInstruction_cmn_al_r9_r5_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r9_ROR_19),
    kInstruction_cmn_al_r6_r9_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r0_LSL_29),
    kInstruction_cmn_al_r2_r0_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r11_ROR_9),
    kInstruction_cmn_al_r3_r11_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r14_LSL_7),
    kInstruction_cmn_al_r0_r14_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r11_LSL_10),
    kInstruction_cmn_al_r5_r11_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r9_ROR_25),
    kInstruction_cmn_al_r12_r9_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r1_ROR_2),
    kInstruction_cmn_al_r11_r1_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r7_ROR_14),
    kInstruction_cmn_al_r8_r7_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r0_ROR_1),
    kInstruction_cmn_al_r7_r0_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r2_ROR_18),
    kInstruction_cmn_al_r3_r2_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r12_LSL_4),
    kInstruction_cmn_al_r14_r12_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r5_ROR_8),
    kInstruction_cmn_al_r12_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r9_ROR_27),
    kInstruction_cmn_al_r6_r9_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r13_LSL_8),
    kInstruction_cmn_al_r10_r13_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r11_ROR_9),
    kInstruction_cmn_al_r10_r11_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r6_ROR_5),
    kInstruction_cmn_al_r2_r6_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r3_LSL_17),
    kInstruction_cmn_al_r9_r3_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r7_ROR_19),
    kInstruction_cmn_al_r12_r7_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r5_LSL_11),
    kInstruction_cmn_al_r9_r5_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r9_LSL_20),
    kInstruction_cmn_al_r14_r9_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r2_ROR_4),
    kInstruction_cmn_al_r6_r2_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r4_LSL_3),
    kInstruction_cmn_al_r8_r4_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r0_ROR_22),
    kInstruction_cmn_al_r5_r0_ROR_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r6_LSL_13),
    kInstruction_cmn_al_r10_r6_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r4_ROR_7),
    kInstruction_cmn_al_r10_r4_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r2_ROR_17),
    kInstruction_cmn_al_r6_r2_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r8_LSL_26),
    kInstruction_cmn_al_r9_r8_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r11_ROR_15),
    kInstruction_cmn_al_r6_r11_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r14_LSL_8),
    kInstruction_cmn_al_r12_r14_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r3_ROR_3),
    kInstruction_cmn_al_r4_r3_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r5_LSL_12),
    kInstruction_cmn_al_r9_r5_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r3_ROR_13),
    kInstruction_cmn_al_r6_r3_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r13_LSL_21),
    kInstruction_cmn_al_r10_r13_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r0_ROR_3),
    kInstruction_cmn_al_r4_r0_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r11_LSL_8),
    kInstruction_cmn_al_r9_r11_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r14_ROR_14),
    kInstruction_cmn_al_r0_r14_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r9_ROR_14),
    kInstruction_cmn_al_r9_r9_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r14_ROR_18),
    kInstruction_cmn_al_r8_r14_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r2_ROR_13),
    kInstruction_cmn_al_r13_r2_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r13_LSL_12),
    kInstruction_cmn_al_r13_r13_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r9_LSL_25),
    kInstruction_cmn_al_r14_r9_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r0_ROR_9),
    kInstruction_cmn_al_r1_r0_ROR_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r6_ROR_20),
    kInstruction_cmn_al_r12_r6_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r2_ROR_23),
    kInstruction_cmn_al_r9_r2_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r10_LSL_1),
    kInstruction_cmn_al_r2_r10_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r7_ROR_15),
    kInstruction_cmn_al_r8_r7_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r3_ROR_14),
    kInstruction_cmn_al_r5_r3_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r8_ROR_1),
    kInstruction_cmn_al_r4_r8_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r2_ROR_5),
    kInstruction_cmn_al_r1_r2_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r2_ROR_16),
    kInstruction_cmn_al_r6_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r4_LSL_15),
    kInstruction_cmn_al_r5_r4_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r2_LSL_14),
    kInstruction_cmn_al_r4_r2_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r1_LSL_7),
    kInstruction_cmn_al_r2_r1_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r8_ROR_30),
    kInstruction_cmn_al_r0_r8_ROR_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r5_LSL_22),
    kInstruction_cmn_al_r9_r5_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r10_LSL_25),
    kInstruction_cmn_al_r9_r10_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r6_ROR_15),
    kInstruction_cmn_al_r4_r6_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r9_LSL_29),
    kInstruction_cmn_al_r2_r9_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r6_ROR_27),
    kInstruction_cmn_al_r6_r6_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r1_ROR_26),
    kInstruction_cmn_al_r11_r1_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r10_ROR_23),
    kInstruction_cmn_al_r5_r10_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r5_LSL_29),
    kInstruction_cmn_al_r13_r5_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r1_ROR_27),
    kInstruction_cmn_al_r14_r1_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r10_LSL_27),
    kInstruction_cmn_al_r0_r10_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r4_ROR_14),
    kInstruction_cmn_al_r2_r4_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r10_LSL_31),
    kInstruction_cmn_al_r1_r10_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r3_ROR_5),
    kInstruction_cmn_al_r2_r3_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r6_ROR_14),
    kInstruction_cmn_al_r0_r6_ROR_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r9_ROR_25),
    kInstruction_cmn_al_r11_r9_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r11_ROR_29),
    kInstruction_cmn_al_r2_r11_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r6_ROR_2),
    kInstruction_cmn_al_r7_r6_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r3_ROR_3),
    kInstruction_cmn_al_r8_r3_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r13_LSL_18),
    kInstruction_cmn_al_r8_r13_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r13_ROR_27),
    kInstruction_cmn_al_r3_r13_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r5_LSL_18),
    kInstruction_cmn_al_r5_r5_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r8_ROR_17),
    kInstruction_cmn_al_r14_r8_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r6_LSL_18),
    kInstruction_cmn_al_r10_r6_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r1_LSL_2),
    kInstruction_cmn_al_r7_r1_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r7_ROR_13),
    kInstruction_cmn_al_r9_r7_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r9_LSL_29),
    kInstruction_cmn_al_r5_r9_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r12_LSL_22),
    kInstruction_cmn_al_r14_r12_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r8_LSL_12),
    kInstruction_cmn_al_r0_r8_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r12_ROR_28),
    kInstruction_cmn_al_r12_r12_ROR_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r3_LSL_2),
    kInstruction_cmn_al_r12_r3_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r11_ROR_3),
    kInstruction_cmn_al_r6_r11_ROR_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r5_ROR_21),
    kInstruction_cmn_al_r1_r5_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r14_ROR_29),
    kInstruction_cmn_al_r0_r14_ROR_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r8_ROR_16),
    kInstruction_cmn_al_r11_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r10_ROR_31),
    kInstruction_cmn_al_r9_r10_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r1_LSL_25),
    kInstruction_cmn_al_r10_r1_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r4_ROR_1),
    kInstruction_cmn_al_r9_r4_ROR_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r12_LSL_4),
    kInstruction_cmn_al_r5_r12_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r7_ROR_25),
    kInstruction_cmn_al_r10_r7_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r3_LSL_6),
    kInstruction_cmn_al_r9_r3_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r4_ROR_26),
    kInstruction_cmn_al_r7_r4_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r0_LSL_31),
    kInstruction_cmn_al_r13_r0_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r2_LSL_22),
    kInstruction_cmn_al_r9_r2_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r8_ROR_21),
    kInstruction_cmn_al_r6_r8_ROR_21,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r7_LSL_11),
    kInstruction_cmn_al_r12_r7_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r0_LSL_17),
    kInstruction_cmn_al_r10_r0_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r9_ROR_23),
    kInstruction_cmn_al_r2_r9_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r4_LSL_6),
    kInstruction_cmn_al_r2_r4_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r8_LSL_24),
    kInstruction_cmn_al_r7_r8_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r5_LSL_25),
    kInstruction_cmn_al_r8_r5_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r11_ROR_23),
    kInstruction_cmn_al_r10_r11_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r14_LSL_16),
    kInstruction_cmn_al_r0_r14_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r3_LSL_16),
    kInstruction_cmn_al_r2_r3_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r11_ROR_13),
    kInstruction_cmn_al_r0_r11_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r4_ROR_15),
    kInstruction_cmn_al_r6_r4_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r9_LSL_19),
    kInstruction_cmn_al_r8_r9_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r13_LSL_23),
    kInstruction_cmn_al_r1_r13_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r11_LSL_6),
    kInstruction_cmn_al_r4_r11_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r0_ROR_15),
    kInstruction_cmn_al_r9_r0_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r0_ROR_7),
    kInstruction_cmn_al_r7_r0_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r7_LSL_26),
    kInstruction_cmn_al_r5_r7_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r4_LSL_6),
    kInstruction_cmn_al_r9_r4_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r9_LSL_9),
    kInstruction_cmn_al_r4_r9_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r14_ROR_24),
    kInstruction_cmn_al_r3_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r6_ROR_4),
    kInstruction_cmn_al_r6_r6_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r1_ROR_19),
    kInstruction_cmn_al_r2_r1_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r1_LSL_2),
    kInstruction_cmn_al_r13_r1_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r9_ROR_2),
    kInstruction_cmn_al_r5_r9_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r14_ROR_25),
    kInstruction_cmn_al_r13_r14_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r11_ROR_18),
    kInstruction_cmn_al_r2_r11_ROR_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r12_ROR_5),
    kInstruction_cmn_al_r10_r12_ROR_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r4_ROR_2),
    kInstruction_cmn_al_r2_r4_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r14_LSL_15),
    kInstruction_cmn_al_r12_r14_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r2_LSL_27),
    kInstruction_cmn_al_r13_r2_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r13_ROR_27),
    kInstruction_cmn_al_r8_r13_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r11_LSL_22),
    kInstruction_cmn_al_r12_r11_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r5_ROR_12),
    kInstruction_cmn_al_r13_r5_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r7_ROR_17),
    kInstruction_cmn_al_r13_r7_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r10_LSL_26),
    kInstruction_cmn_al_r4_r10_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r9_LSL_24),
    kInstruction_cmn_al_r6_r9_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r5_LSL_13),
    kInstruction_cmn_al_r14_r5_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r0_ROR_8),
    kInstruction_cmn_al_r12_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r2_LSL_30),
    kInstruction_cmn_al_r1_r2_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r7_LSL_15),
    kInstruction_cmn_al_r10_r7_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r7_LSL_31),
    kInstruction_cmn_al_r0_r7_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r11_LSL_9),
    kInstruction_cmn_al_r14_r11_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r6_ROR_2),
    kInstruction_cmn_al_r1_r6_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r2_ROR_26),
    kInstruction_cmn_al_r5_r2_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r5_LSL_2),
    kInstruction_cmn_al_r3_r5_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r3_LSL_3),
    kInstruction_cmn_al_r8_r3_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r11_ROR_6),
    kInstruction_cmn_al_r6_r11_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r4_ROR_13),
    kInstruction_cmn_al_r4_r4_ROR_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r4_r2_LSL_20),
    kInstruction_cmn_al_r4_r2_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r8_ROR_25),
    kInstruction_cmn_al_r13_r8_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r13_LSL_18),
    kInstruction_cmn_al_r3_r13_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r0_ROR_23),
    kInstruction_cmn_al_r1_r0_ROR_23,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r14_LSL_25),
    kInstruction_cmn_al_r14_r14_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r8_LSL_11),
    kInstruction_cmn_al_r0_r8_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r1_LSL_13),
    kInstruction_cmn_al_r10_r1_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r10_LSL_9),
    kInstruction_cmn_al_r13_r10_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r13_LSL_16),
    kInstruction_cmn_al_r6_r13_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r14_LSL_5),
    kInstruction_cmn_al_r13_r14_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r6_LSL_14),
    kInstruction_cmn_al_r2_r6_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r7_LSL_13),
    kInstruction_cmn_al_r13_r7_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r5_ROR_26),
    kInstruction_cmn_al_r7_r5_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r11_LSL_12),
    kInstruction_cmn_al_r12_r11_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r13_ROR_8),
    kInstruction_cmn_al_r5_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r8_ROR_2),
    kInstruction_cmn_al_r2_r8_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r9_ROR_24),
    kInstruction_cmn_al_r0_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r6_LSL_29),
    kInstruction_cmn_al_r8_r6_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r10_LSL_18),
    kInstruction_cmn_al_r9_r10_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r0_LSL_24),
    kInstruction_cmn_al_r6_r0_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r12_LSL_20),
    kInstruction_cmn_al_r9_r12_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r11_LSL_3),
    kInstruction_cmn_al_r14_r11_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r11_LSL_26),
    kInstruction_cmn_al_r12_r11_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r0_LSL_4),
    kInstruction_cmn_al_r10_r0_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r12_ROR_7),
    kInstruction_cmn_al_r10_r12_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r3_LSL_1),
    kInstruction_cmn_al_r12_r3_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r4_ROR_12),
    kInstruction_cmn_al_r12_r4_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r0_ROR_12),
    kInstruction_cmn_al_r8_r0_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r3_ROR_6),
    kInstruction_cmn_al_r3_r3_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r0_LSL_19),
    kInstruction_cmn_al_r11_r0_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r14_LSL_24),
    kInstruction_cmn_al_r8_r14_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r11_LSL_1),
    kInstruction_cmn_al_r0_r11_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r5_LSL_6),
    kInstruction_cmn_al_r13_r5_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r14_ROR_31),
    kInstruction_cmn_al_r14_r14_ROR_31,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r10_r14_ROR_6),
    kInstruction_cmn_al_r10_r14_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r4_LSL_1),
    kInstruction_cmn_al_r14_r4_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r3_ROR_7),
    kInstruction_cmn_al_r0_r3_ROR_7,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r10_LSL_17),
    kInstruction_cmn_al_r2_r10_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r6_ROR_19),
    kInstruction_cmn_al_r11_r6_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r13_ROR_11),
    kInstruction_cmn_al_r9_r13_ROR_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r3_LSL_2),
    kInstruction_cmn_al_r11_r3_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r1_ROR_4),
    kInstruction_cmn_al_r9_r1_ROR_4,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r8_r11_LSL_16),
    kInstruction_cmn_al_r8_r11_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r1_r12_LSL_11),
    kInstruction_cmn_al_r1_r12_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r8_ROR_6),
    kInstruction_cmn_al_r0_r8_ROR_6,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r5_r3_LSL_2),
    kInstruction_cmn_al_r5_r3_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r3_LSL_17),
    kInstruction_cmn_al_r6_r3_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r12_ROR_27),
    kInstruction_cmn_al_r3_r12_ROR_27,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r6_r8_ROR_15),
    kInstruction_cmn_al_r6_r8_ROR_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r7_r11_LSL_10),
    kInstruction_cmn_al_r7_r11_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r12_LSL_15),
    kInstruction_cmn_al_r2_r12_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r2_ROR_8),
    kInstruction_cmn_al_r2_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r0_ROR_26),
    kInstruction_cmn_al_r12_r0_ROR_26,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r13_ROR_19),
    kInstruction_cmn_al_r11_r13_ROR_19,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r0_r9_ROR_2),
    kInstruction_cmn_al_r0_r9_ROR_2,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r7_ROR_25),
    kInstruction_cmn_al_r9_r7_ROR_25,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r9_r8_LSL_28),
    kInstruction_cmn_al_r9_r8_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r12_ROR_17),
    kInstruction_cmn_al_r11_r12_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r12_r2_ROR_20),
    kInstruction_cmn_al_r12_r2_ROR_20,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r11_r9_ROR_16),
    kInstruction_cmn_al_r11_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r13_r11_LSL_14),
    kInstruction_cmn_al_r13_r11_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r2_r4_ROR_12),
    kInstruction_cmn_al_r2_r4_ROR_12,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r3_r12_ROR_17),
    kInstruction_cmn_al_r3_r12_ROR_17,
  },
  {
    ARRAY_SIZE(kInstruction_cmn_al_r14_r11_ROR_27),
    kInstruction_cmn_al_r14_r11_ROR_27,
  },
};

#endif  // VIXL_ASSEMBLER_COND_RD_OPERAND_RN_SHIFT_AMOUNT_1TO31_CMN_T32_H_
