// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_ASSEMBLER_COND_RD_RN_OPERAND_RM_ROR_AMOUNT_SXTAB_A32_H_
#define VIXL_ASSEMBLER_COND_RD_RN_OPERAND_RM_ROR_AMOUNT_SXTAB_A32_H_

const byte kInstruction_sxtab_ls_r3_r3_r13_ROR_0[] = {
  0x7d, 0x30, 0xa3, 0x96 // sxtab ls r3 r3 r13 ROR 0
};
const byte kInstruction_sxtab_cs_r2_r7_r1_ROR_16[] = {
  0x71, 0x28, 0xa7, 0x26 // sxtab cs r2 r7 r1 ROR 16
};
const byte kInstruction_sxtab_mi_r13_r0_r2_ROR_8[] = {
  0x72, 0xd4, 0xa0, 0x46 // sxtab mi r13 r0 r2 ROR 8
};
const byte kInstruction_sxtab_lt_r0_r6_r1_ROR_8[] = {
  0x71, 0x04, 0xa6, 0xb6 // sxtab lt r0 r6 r1 ROR 8
};
const byte kInstruction_sxtab_al_r6_r4_r8_ROR_16[] = {
  0x78, 0x68, 0xa4, 0xe6 // sxtab al r6 r4 r8 ROR 16
};
const byte kInstruction_sxtab_gt_r1_r7_r10_ROR_16[] = {
  0x7a, 0x18, 0xa7, 0xc6 // sxtab gt r1 r7 r10 ROR 16
};
const byte kInstruction_sxtab_cc_r12_r9_r11_ROR_16[] = {
  0x7b, 0xc8, 0xa9, 0x36 // sxtab cc r12 r9 r11 ROR 16
};
const byte kInstruction_sxtab_mi_r6_r8_r13_ROR_24[] = {
  0x7d, 0x6c, 0xa8, 0x46 // sxtab mi r6 r8 r13 ROR 24
};
const byte kInstruction_sxtab_mi_r9_r1_r6_ROR_16[] = {
  0x76, 0x98, 0xa1, 0x46 // sxtab mi r9 r1 r6 ROR 16
};
const byte kInstruction_sxtab_le_r8_r9_r14_ROR_8[] = {
  0x7e, 0x84, 0xa9, 0xd6 // sxtab le r8 r9 r14 ROR 8
};
const byte kInstruction_sxtab_mi_r6_r0_r1_ROR_8[] = {
  0x71, 0x64, 0xa0, 0x46 // sxtab mi r6 r0 r1 ROR 8
};
const byte kInstruction_sxtab_cs_r14_r2_r2_ROR_24[] = {
  0x72, 0xec, 0xa2, 0x26 // sxtab cs r14 r2 r2 ROR 24
};
const byte kInstruction_sxtab_ge_r1_r11_r8_ROR_8[] = {
  0x78, 0x14, 0xab, 0xa6 // sxtab ge r1 r11 r8 ROR 8
};
const byte kInstruction_sxtab_al_r6_r8_r9_ROR_16[] = {
  0x79, 0x68, 0xa8, 0xe6 // sxtab al r6 r8 r9 ROR 16
};
const byte kInstruction_sxtab_ls_r12_r8_r2_ROR_24[] = {
  0x72, 0xcc, 0xa8, 0x96 // sxtab ls r12 r8 r2 ROR 24
};
const byte kInstruction_sxtab_vc_r2_r8_r8_ROR_0[] = {
  0x78, 0x20, 0xa8, 0x76 // sxtab vc r2 r8 r8 ROR 0
};
const byte kInstruction_sxtab_le_r14_r6_r4_ROR_16[] = {
  0x74, 0xe8, 0xa6, 0xd6 // sxtab le r14 r6 r4 ROR 16
};
const byte kInstruction_sxtab_ge_r0_r4_r12_ROR_0[] = {
  0x7c, 0x00, 0xa4, 0xa6 // sxtab ge r0 r4 r12 ROR 0
};
const byte kInstruction_sxtab_cc_r1_r3_r9_ROR_24[] = {
  0x79, 0x1c, 0xa3, 0x36 // sxtab cc r1 r3 r9 ROR 24
};
const byte kInstruction_sxtab_mi_r9_r5_r4_ROR_8[] = {
  0x74, 0x94, 0xa5, 0x46 // sxtab mi r9 r5 r4 ROR 8
};
const byte kInstruction_sxtab_al_r4_r10_r4_ROR_8[] = {
  0x74, 0x44, 0xaa, 0xe6 // sxtab al r4 r10 r4 ROR 8
};
const byte kInstruction_sxtab_vs_r8_r13_r1_ROR_24[] = {
  0x71, 0x8c, 0xad, 0x66 // sxtab vs r8 r13 r1 ROR 24
};
const byte kInstruction_sxtab_eq_r4_r12_r2_ROR_0[] = {
  0x72, 0x40, 0xac, 0x06 // sxtab eq r4 r12 r2 ROR 0
};
const byte kInstruction_sxtab_pl_r7_r5_r6_ROR_24[] = {
  0x76, 0x7c, 0xa5, 0x56 // sxtab pl r7 r5 r6 ROR 24
};
const byte kInstruction_sxtab_lt_r9_r14_r14_ROR_24[] = {
  0x7e, 0x9c, 0xae, 0xb6 // sxtab lt r9 r14 r14 ROR 24
};
const byte kInstruction_sxtab_ge_r8_r6_r2_ROR_0[] = {
  0x72, 0x80, 0xa6, 0xa6 // sxtab ge r8 r6 r2 ROR 0
};
const byte kInstruction_sxtab_lt_r11_r6_r6_ROR_0[] = {
  0x76, 0xb0, 0xa6, 0xb6 // sxtab lt r11 r6 r6 ROR 0
};
const byte kInstruction_sxtab_le_r12_r2_r2_ROR_16[] = {
  0x72, 0xc8, 0xa2, 0xd6 // sxtab le r12 r2 r2 ROR 16
};
const byte kInstruction_sxtab_hi_r10_r2_r2_ROR_16[] = {
  0x72, 0xa8, 0xa2, 0x86 // sxtab hi r10 r2 r2 ROR 16
};
const byte kInstruction_sxtab_mi_r0_r8_r6_ROR_0[] = {
  0x76, 0x00, 0xa8, 0x46 // sxtab mi r0 r8 r6 ROR 0
};
const byte kInstruction_sxtab_pl_r0_r6_r11_ROR_8[] = {
  0x7b, 0x04, 0xa6, 0x56 // sxtab pl r0 r6 r11 ROR 8
};
const byte kInstruction_sxtab_hi_r0_r0_r3_ROR_0[] = {
  0x73, 0x00, 0xa0, 0x86 // sxtab hi r0 r0 r3 ROR 0
};
const byte kInstruction_sxtab_gt_r4_r9_r13_ROR_16[] = {
  0x7d, 0x48, 0xa9, 0xc6 // sxtab gt r4 r9 r13 ROR 16
};
const byte kInstruction_sxtab_ne_r4_r7_r10_ROR_16[] = {
  0x7a, 0x48, 0xa7, 0x16 // sxtab ne r4 r7 r10 ROR 16
};
const byte kInstruction_sxtab_vc_r4_r13_r9_ROR_16[] = {
  0x79, 0x48, 0xad, 0x76 // sxtab vc r4 r13 r9 ROR 16
};
const byte kInstruction_sxtab_hi_r8_r9_r4_ROR_8[] = {
  0x74, 0x84, 0xa9, 0x86 // sxtab hi r8 r9 r4 ROR 8
};
const byte kInstruction_sxtab_hi_r2_r6_r0_ROR_8[] = {
  0x70, 0x24, 0xa6, 0x86 // sxtab hi r2 r6 r0 ROR 8
};
const byte kInstruction_sxtab_cs_r13_r7_r6_ROR_0[] = {
  0x76, 0xd0, 0xa7, 0x26 // sxtab cs r13 r7 r6 ROR 0
};
const byte kInstruction_sxtab_gt_r11_r1_r14_ROR_8[] = {
  0x7e, 0xb4, 0xa1, 0xc6 // sxtab gt r11 r1 r14 ROR 8
};
const byte kInstruction_sxtab_eq_r0_r1_r4_ROR_8[] = {
  0x74, 0x04, 0xa1, 0x06 // sxtab eq r0 r1 r4 ROR 8
};
const byte kInstruction_sxtab_eq_r11_r12_r1_ROR_16[] = {
  0x71, 0xb8, 0xac, 0x06 // sxtab eq r11 r12 r1 ROR 16
};
const byte kInstruction_sxtab_pl_r2_r12_r9_ROR_16[] = {
  0x79, 0x28, 0xac, 0x56 // sxtab pl r2 r12 r9 ROR 16
};
const byte kInstruction_sxtab_ls_r1_r5_r11_ROR_16[] = {
  0x7b, 0x18, 0xa5, 0x96 // sxtab ls r1 r5 r11 ROR 16
};
const byte kInstruction_sxtab_pl_r0_r13_r9_ROR_24[] = {
  0x79, 0x0c, 0xad, 0x56 // sxtab pl r0 r13 r9 ROR 24
};
const byte kInstruction_sxtab_pl_r6_r7_r11_ROR_16[] = {
  0x7b, 0x68, 0xa7, 0x56 // sxtab pl r6 r7 r11 ROR 16
};
const byte kInstruction_sxtab_le_r0_r10_r9_ROR_0[] = {
  0x79, 0x00, 0xaa, 0xd6 // sxtab le r0 r10 r9 ROR 0
};
const byte kInstruction_sxtab_vc_r9_r14_r5_ROR_0[] = {
  0x75, 0x90, 0xae, 0x76 // sxtab vc r9 r14 r5 ROR 0
};
const byte kInstruction_sxtab_ls_r10_r4_r1_ROR_16[] = {
  0x71, 0xa8, 0xa4, 0x96 // sxtab ls r10 r4 r1 ROR 16
};
const byte kInstruction_sxtab_ls_r3_r7_r2_ROR_0[] = {
  0x72, 0x30, 0xa7, 0x96 // sxtab ls r3 r7 r2 ROR 0
};
const byte kInstruction_sxtab_mi_r13_r14_r3_ROR_0[] = {
  0x73, 0xd0, 0xae, 0x46 // sxtab mi r13 r14 r3 ROR 0
};
const byte kInstruction_sxtab_lt_r2_r14_r4_ROR_8[] = {
  0x74, 0x24, 0xae, 0xb6 // sxtab lt r2 r14 r4 ROR 8
};
const byte kInstruction_sxtab_cc_r1_r9_r5_ROR_16[] = {
  0x75, 0x18, 0xa9, 0x36 // sxtab cc r1 r9 r5 ROR 16
};
const byte kInstruction_sxtab_pl_r12_r8_r4_ROR_16[] = {
  0x74, 0xc8, 0xa8, 0x56 // sxtab pl r12 r8 r4 ROR 16
};
const byte kInstruction_sxtab_vs_r10_r13_r3_ROR_16[] = {
  0x73, 0xa8, 0xad, 0x66 // sxtab vs r10 r13 r3 ROR 16
};
const byte kInstruction_sxtab_hi_r10_r13_r14_ROR_0[] = {
  0x7e, 0xa0, 0xad, 0x86 // sxtab hi r10 r13 r14 ROR 0
};
const byte kInstruction_sxtab_vc_r1_r7_r6_ROR_0[] = {
  0x76, 0x10, 0xa7, 0x76 // sxtab vc r1 r7 r6 ROR 0
};
const byte kInstruction_sxtab_vs_r4_r3_r11_ROR_16[] = {
  0x7b, 0x48, 0xa3, 0x66 // sxtab vs r4 r3 r11 ROR 16
};
const byte kInstruction_sxtab_lt_r3_r12_r3_ROR_24[] = {
  0x73, 0x3c, 0xac, 0xb6 // sxtab lt r3 r12 r3 ROR 24
};
const byte kInstruction_sxtab_ge_r4_r0_r6_ROR_24[] = {
  0x76, 0x4c, 0xa0, 0xa6 // sxtab ge r4 r0 r6 ROR 24
};
const byte kInstruction_sxtab_vc_r12_r4_r0_ROR_0[] = {
  0x70, 0xc0, 0xa4, 0x76 // sxtab vc r12 r4 r0 ROR 0
};
const byte kInstruction_sxtab_ne_r2_r2_r9_ROR_8[] = {
  0x79, 0x24, 0xa2, 0x16 // sxtab ne r2 r2 r9 ROR 8
};
const byte kInstruction_sxtab_le_r9_r4_r13_ROR_8[] = {
  0x7d, 0x94, 0xa4, 0xd6 // sxtab le r9 r4 r13 ROR 8
};
const byte kInstruction_sxtab_lt_r13_r1_r14_ROR_24[] = {
  0x7e, 0xdc, 0xa1, 0xb6 // sxtab lt r13 r1 r14 ROR 24
};
const byte kInstruction_sxtab_ge_r7_r3_r9_ROR_0[] = {
  0x79, 0x70, 0xa3, 0xa6 // sxtab ge r7 r3 r9 ROR 0
};
const byte kInstruction_sxtab_mi_r6_r2_r12_ROR_0[] = {
  0x7c, 0x60, 0xa2, 0x46 // sxtab mi r6 r2 r12 ROR 0
};
const byte kInstruction_sxtab_vs_r2_r13_r10_ROR_0[] = {
  0x7a, 0x20, 0xad, 0x66 // sxtab vs r2 r13 r10 ROR 0
};
const byte kInstruction_sxtab_lt_r3_r12_r3_ROR_16[] = {
  0x73, 0x38, 0xac, 0xb6 // sxtab lt r3 r12 r3 ROR 16
};
const byte kInstruction_sxtab_ne_r3_r6_r14_ROR_16[] = {
  0x7e, 0x38, 0xa6, 0x16 // sxtab ne r3 r6 r14 ROR 16
};
const byte kInstruction_sxtab_le_r6_r7_r0_ROR_0[] = {
  0x70, 0x60, 0xa7, 0xd6 // sxtab le r6 r7 r0 ROR 0
};
const byte kInstruction_sxtab_vs_r3_r1_r1_ROR_0[] = {
  0x71, 0x30, 0xa1, 0x66 // sxtab vs r3 r1 r1 ROR 0
};
const byte kInstruction_sxtab_ge_r0_r6_r0_ROR_8[] = {
  0x70, 0x04, 0xa6, 0xa6 // sxtab ge r0 r6 r0 ROR 8
};
const byte kInstruction_sxtab_ne_r10_r0_r13_ROR_16[] = {
  0x7d, 0xa8, 0xa0, 0x16 // sxtab ne r10 r0 r13 ROR 16
};
const byte kInstruction_sxtab_ne_r11_r8_r11_ROR_16[] = {
  0x7b, 0xb8, 0xa8, 0x16 // sxtab ne r11 r8 r11 ROR 16
};
const byte kInstruction_sxtab_lt_r4_r5_r11_ROR_16[] = {
  0x7b, 0x48, 0xa5, 0xb6 // sxtab lt r4 r5 r11 ROR 16
};
const byte kInstruction_sxtab_gt_r2_r13_r4_ROR_8[] = {
  0x74, 0x24, 0xad, 0xc6 // sxtab gt r2 r13 r4 ROR 8
};
const byte kInstruction_sxtab_ls_r14_r7_r9_ROR_24[] = {
  0x79, 0xec, 0xa7, 0x96 // sxtab ls r14 r7 r9 ROR 24
};
const byte kInstruction_sxtab_vc_r4_r12_r9_ROR_16[] = {
  0x79, 0x48, 0xac, 0x76 // sxtab vc r4 r12 r9 ROR 16
};
const byte kInstruction_sxtab_vc_r7_r9_r3_ROR_0[] = {
  0x73, 0x70, 0xa9, 0x76 // sxtab vc r7 r9 r3 ROR 0
};
const byte kInstruction_sxtab_eq_r10_r3_r11_ROR_0[] = {
  0x7b, 0xa0, 0xa3, 0x06 // sxtab eq r10 r3 r11 ROR 0
};
const byte kInstruction_sxtab_ne_r3_r6_r11_ROR_16[] = {
  0x7b, 0x38, 0xa6, 0x16 // sxtab ne r3 r6 r11 ROR 16
};
const byte kInstruction_sxtab_cs_r1_r2_r3_ROR_16[] = {
  0x73, 0x18, 0xa2, 0x26 // sxtab cs r1 r2 r3 ROR 16
};
const byte kInstruction_sxtab_gt_r0_r1_r12_ROR_0[] = {
  0x7c, 0x00, 0xa1, 0xc6 // sxtab gt r0 r1 r12 ROR 0
};
const byte kInstruction_sxtab_mi_r11_r8_r11_ROR_16[] = {
  0x7b, 0xb8, 0xa8, 0x46 // sxtab mi r11 r8 r11 ROR 16
};
const byte kInstruction_sxtab_mi_r6_r9_r0_ROR_24[] = {
  0x70, 0x6c, 0xa9, 0x46 // sxtab mi r6 r9 r0 ROR 24
};
const byte kInstruction_sxtab_hi_r3_r12_r12_ROR_0[] = {
  0x7c, 0x30, 0xac, 0x86 // sxtab hi r3 r12 r12 ROR 0
};
const byte kInstruction_sxtab_lt_r8_r2_r2_ROR_16[] = {
  0x72, 0x88, 0xa2, 0xb6 // sxtab lt r8 r2 r2 ROR 16
};
const byte kInstruction_sxtab_le_r4_r12_r5_ROR_0[] = {
  0x75, 0x40, 0xac, 0xd6 // sxtab le r4 r12 r5 ROR 0
};
const byte kInstruction_sxtab_cs_r12_r13_r0_ROR_24[] = {
  0x70, 0xcc, 0xad, 0x26 // sxtab cs r12 r13 r0 ROR 24
};
const byte kInstruction_sxtab_al_r11_r12_r3_ROR_16[] = {
  0x73, 0xb8, 0xac, 0xe6 // sxtab al r11 r12 r3 ROR 16
};
const byte kInstruction_sxtab_hi_r0_r14_r14_ROR_8[] = {
  0x7e, 0x04, 0xae, 0x86 // sxtab hi r0 r14 r14 ROR 8
};
const byte kInstruction_sxtab_gt_r3_r4_r4_ROR_0[] = {
  0x74, 0x30, 0xa4, 0xc6 // sxtab gt r3 r4 r4 ROR 0
};
const byte kInstruction_sxtab_mi_r13_r10_r13_ROR_24[] = {
  0x7d, 0xdc, 0xaa, 0x46 // sxtab mi r13 r10 r13 ROR 24
};
const byte kInstruction_sxtab_eq_r0_r5_r8_ROR_0[] = {
  0x78, 0x00, 0xa5, 0x06 // sxtab eq r0 r5 r8 ROR 0
};
const byte kInstruction_sxtab_mi_r0_r5_r11_ROR_16[] = {
  0x7b, 0x08, 0xa5, 0x46 // sxtab mi r0 r5 r11 ROR 16
};
const byte kInstruction_sxtab_eq_r0_r9_r2_ROR_16[] = {
  0x72, 0x08, 0xa9, 0x06 // sxtab eq r0 r9 r2 ROR 16
};
const byte kInstruction_sxtab_lt_r14_r10_r12_ROR_24[] = {
  0x7c, 0xec, 0xaa, 0xb6 // sxtab lt r14 r10 r12 ROR 24
};
const byte kInstruction_sxtab_eq_r14_r6_r0_ROR_8[] = {
  0x70, 0xe4, 0xa6, 0x06 // sxtab eq r14 r6 r0 ROR 8
};
const byte kInstruction_sxtab_eq_r12_r13_r8_ROR_24[] = {
  0x78, 0xcc, 0xad, 0x06 // sxtab eq r12 r13 r8 ROR 24
};
const byte kInstruction_sxtab_hi_r5_r7_r12_ROR_16[] = {
  0x7c, 0x58, 0xa7, 0x86 // sxtab hi r5 r7 r12 ROR 16
};
const byte kInstruction_sxtab_mi_r7_r0_r10_ROR_8[] = {
  0x7a, 0x74, 0xa0, 0x46 // sxtab mi r7 r0 r10 ROR 8
};
const byte kInstruction_sxtab_cc_r9_r8_r12_ROR_24[] = {
  0x7c, 0x9c, 0xa8, 0x36 // sxtab cc r9 r8 r12 ROR 24
};
const byte kInstruction_sxtab_gt_r13_r14_r6_ROR_0[] = {
  0x76, 0xd0, 0xae, 0xc6 // sxtab gt r13 r14 r6 ROR 0
};
const byte kInstruction_sxtab_al_r5_r4_r2_ROR_16[] = {
  0x72, 0x58, 0xa4, 0xe6 // sxtab al r5 r4 r2 ROR 16
};
const byte kInstruction_sxtab_pl_r8_r0_r3_ROR_24[] = {
  0x73, 0x8c, 0xa0, 0x56 // sxtab pl r8 r0 r3 ROR 24
};
const byte kInstruction_sxtab_eq_r1_r4_r1_ROR_8[] = {
  0x71, 0x14, 0xa4, 0x06 // sxtab eq r1 r4 r1 ROR 8
};
const byte kInstruction_sxtab_lt_r5_r9_r12_ROR_8[] = {
  0x7c, 0x54, 0xa9, 0xb6 // sxtab lt r5 r9 r12 ROR 8
};
const byte kInstruction_sxtab_vs_r8_r8_r2_ROR_0[] = {
  0x72, 0x80, 0xa8, 0x66 // sxtab vs r8 r8 r2 ROR 0
};
const byte kInstruction_sxtab_pl_r11_r3_r6_ROR_0[] = {
  0x76, 0xb0, 0xa3, 0x56 // sxtab pl r11 r3 r6 ROR 0
};
const byte kInstruction_sxtab_al_r2_r9_r11_ROR_8[] = {
  0x7b, 0x24, 0xa9, 0xe6 // sxtab al r2 r9 r11 ROR 8
};
const byte kInstruction_sxtab_pl_r0_r4_r10_ROR_8[] = {
  0x7a, 0x04, 0xa4, 0x56 // sxtab pl r0 r4 r10 ROR 8
};
const byte kInstruction_sxtab_pl_r9_r14_r2_ROR_16[] = {
  0x72, 0x98, 0xae, 0x56 // sxtab pl r9 r14 r2 ROR 16
};
const byte kInstruction_sxtab_ge_r13_r11_r10_ROR_24[] = {
  0x7a, 0xdc, 0xab, 0xa6 // sxtab ge r13 r11 r10 ROR 24
};
const byte kInstruction_sxtab_al_r10_r8_r4_ROR_8[] = {
  0x74, 0xa4, 0xa8, 0xe6 // sxtab al r10 r8 r4 ROR 8
};
const byte kInstruction_sxtab_cc_r1_r2_r4_ROR_8[] = {
  0x74, 0x14, 0xa2, 0x36 // sxtab cc r1 r2 r4 ROR 8
};
const byte kInstruction_sxtab_mi_r8_r7_r9_ROR_0[] = {
  0x79, 0x80, 0xa7, 0x46 // sxtab mi r8 r7 r9 ROR 0
};
const byte kInstruction_sxtab_pl_r8_r9_r6_ROR_8[] = {
  0x76, 0x84, 0xa9, 0x56 // sxtab pl r8 r9 r6 ROR 8
};
const byte kInstruction_sxtab_mi_r2_r7_r3_ROR_16[] = {
  0x73, 0x28, 0xa7, 0x46 // sxtab mi r2 r7 r3 ROR 16
};
const byte kInstruction_sxtab_vc_r12_r11_r2_ROR_0[] = {
  0x72, 0xc0, 0xab, 0x76 // sxtab vc r12 r11 r2 ROR 0
};
const byte kInstruction_sxtab_ge_r3_r6_r1_ROR_8[] = {
  0x71, 0x34, 0xa6, 0xa6 // sxtab ge r3 r6 r1 ROR 8
};
const byte kInstruction_sxtab_ls_r6_r0_r2_ROR_16[] = {
  0x72, 0x68, 0xa0, 0x96 // sxtab ls r6 r0 r2 ROR 16
};
const byte kInstruction_sxtab_cs_r11_r10_r9_ROR_8[] = {
  0x79, 0xb4, 0xaa, 0x26 // sxtab cs r11 r10 r9 ROR 8
};
const byte kInstruction_sxtab_pl_r2_r4_r7_ROR_0[] = {
  0x77, 0x20, 0xa4, 0x56 // sxtab pl r2 r4 r7 ROR 0
};
const byte kInstruction_sxtab_eq_r9_r2_r11_ROR_8[] = {
  0x7b, 0x94, 0xa2, 0x06 // sxtab eq r9 r2 r11 ROR 8
};
const byte kInstruction_sxtab_hi_r5_r11_r8_ROR_0[] = {
  0x78, 0x50, 0xab, 0x86 // sxtab hi r5 r11 r8 ROR 0
};
const byte kInstruction_sxtab_hi_r5_r7_r3_ROR_8[] = {
  0x73, 0x54, 0xa7, 0x86 // sxtab hi r5 r7 r3 ROR 8
};
const byte kInstruction_sxtab_lt_r0_r1_r9_ROR_8[] = {
  0x79, 0x04, 0xa1, 0xb6 // sxtab lt r0 r1 r9 ROR 8
};
const byte kInstruction_sxtab_le_r6_r3_r8_ROR_24[] = {
  0x78, 0x6c, 0xa3, 0xd6 // sxtab le r6 r3 r8 ROR 24
};
const byte kInstruction_sxtab_hi_r5_r7_r2_ROR_0[] = {
  0x72, 0x50, 0xa7, 0x86 // sxtab hi r5 r7 r2 ROR 0
};
const byte kInstruction_sxtab_eq_r0_r5_r3_ROR_24[] = {
  0x73, 0x0c, 0xa5, 0x06 // sxtab eq r0 r5 r3 ROR 24
};
const byte kInstruction_sxtab_mi_r13_r13_r4_ROR_8[] = {
  0x74, 0xd4, 0xad, 0x46 // sxtab mi r13 r13 r4 ROR 8
};
const byte kInstruction_sxtab_eq_r4_r13_r0_ROR_8[] = {
  0x70, 0x44, 0xad, 0x06 // sxtab eq r4 r13 r0 ROR 8
};
const byte kInstruction_sxtab_ge_r11_r4_r2_ROR_8[] = {
  0x72, 0xb4, 0xa4, 0xa6 // sxtab ge r11 r4 r2 ROR 8
};
const byte kInstruction_sxtab_pl_r5_r1_r9_ROR_24[] = {
  0x79, 0x5c, 0xa1, 0x56 // sxtab pl r5 r1 r9 ROR 24
};
const byte kInstruction_sxtab_pl_r6_r4_r0_ROR_16[] = {
  0x70, 0x68, 0xa4, 0x56 // sxtab pl r6 r4 r0 ROR 16
};
const byte kInstruction_sxtab_ge_r6_r0_r11_ROR_16[] = {
  0x7b, 0x68, 0xa0, 0xa6 // sxtab ge r6 r0 r11 ROR 16
};
const byte kInstruction_sxtab_cc_r4_r0_r14_ROR_16[] = {
  0x7e, 0x48, 0xa0, 0x36 // sxtab cc r4 r0 r14 ROR 16
};
const byte kInstruction_sxtab_ge_r10_r10_r12_ROR_8[] = {
  0x7c, 0xa4, 0xaa, 0xa6 // sxtab ge r10 r10 r12 ROR 8
};
const byte kInstruction_sxtab_eq_r1_r11_r2_ROR_8[] = {
  0x72, 0x14, 0xab, 0x06 // sxtab eq r1 r11 r2 ROR 8
};
const byte kInstruction_sxtab_ne_r2_r14_r6_ROR_16[] = {
  0x76, 0x28, 0xae, 0x16 // sxtab ne r2 r14 r6 ROR 16
};
const byte kInstruction_sxtab_al_r4_r10_r9_ROR_8[] = {
  0x79, 0x44, 0xaa, 0xe6 // sxtab al r4 r10 r9 ROR 8
};
const byte kInstruction_sxtab_al_r11_r12_r10_ROR_0[] = {
  0x7a, 0xb0, 0xac, 0xe6 // sxtab al r11 r12 r10 ROR 0
};
const byte kInstruction_sxtab_ne_r0_r13_r10_ROR_16[] = {
  0x7a, 0x08, 0xad, 0x16 // sxtab ne r0 r13 r10 ROR 16
};
const byte kInstruction_sxtab_pl_r4_r2_r5_ROR_0[] = {
  0x75, 0x40, 0xa2, 0x56 // sxtab pl r4 r2 r5 ROR 0
};
const byte kInstruction_sxtab_al_r14_r8_r8_ROR_16[] = {
  0x78, 0xe8, 0xa8, 0xe6 // sxtab al r14 r8 r8 ROR 16
};
const byte kInstruction_sxtab_pl_r3_r5_r14_ROR_0[] = {
  0x7e, 0x30, 0xa5, 0x56 // sxtab pl r3 r5 r14 ROR 0
};
const byte kInstruction_sxtab_le_r6_r14_r11_ROR_0[] = {
  0x7b, 0x60, 0xae, 0xd6 // sxtab le r6 r14 r11 ROR 0
};
const byte kInstruction_sxtab_cc_r9_r12_r12_ROR_8[] = {
  0x7c, 0x94, 0xac, 0x36 // sxtab cc r9 r12 r12 ROR 8
};
const byte kInstruction_sxtab_lt_r1_r4_r1_ROR_16[] = {
  0x71, 0x18, 0xa4, 0xb6 // sxtab lt r1 r4 r1 ROR 16
};
const byte kInstruction_sxtab_ge_r10_r11_r10_ROR_0[] = {
  0x7a, 0xa0, 0xab, 0xa6 // sxtab ge r10 r11 r10 ROR 0
};
const byte kInstruction_sxtab_gt_r0_r11_r12_ROR_8[] = {
  0x7c, 0x04, 0xab, 0xc6 // sxtab gt r0 r11 r12 ROR 8
};
const byte kInstruction_sxtab_vc_r13_r1_r12_ROR_16[] = {
  0x7c, 0xd8, 0xa1, 0x76 // sxtab vc r13 r1 r12 ROR 16
};
const byte kInstruction_sxtab_vs_r8_r5_r13_ROR_16[] = {
  0x7d, 0x88, 0xa5, 0x66 // sxtab vs r8 r5 r13 ROR 16
};
const byte kInstruction_sxtab_ge_r5_r7_r12_ROR_8[] = {
  0x7c, 0x54, 0xa7, 0xa6 // sxtab ge r5 r7 r12 ROR 8
};
const byte kInstruction_sxtab_eq_r7_r3_r6_ROR_8[] = {
  0x76, 0x74, 0xa3, 0x06 // sxtab eq r7 r3 r6 ROR 8
};
const byte kInstruction_sxtab_hi_r14_r13_r3_ROR_8[] = {
  0x73, 0xe4, 0xad, 0x86 // sxtab hi r14 r13 r3 ROR 8
};
const byte kInstruction_sxtab_hi_r3_r7_r1_ROR_8[] = {
  0x71, 0x34, 0xa7, 0x86 // sxtab hi r3 r7 r1 ROR 8
};
const byte kInstruction_sxtab_al_r2_r1_r12_ROR_8[] = {
  0x7c, 0x24, 0xa1, 0xe6 // sxtab al r2 r1 r12 ROR 8
};
const byte kInstruction_sxtab_mi_r5_r6_r4_ROR_16[] = {
  0x74, 0x58, 0xa6, 0x46 // sxtab mi r5 r6 r4 ROR 16
};
const byte kInstruction_sxtab_eq_r2_r12_r0_ROR_24[] = {
  0x70, 0x2c, 0xac, 0x06 // sxtab eq r2 r12 r0 ROR 24
};
const byte kInstruction_sxtab_cs_r14_r8_r12_ROR_0[] = {
  0x7c, 0xe0, 0xa8, 0x26 // sxtab cs r14 r8 r12 ROR 0
};
const byte kInstruction_sxtab_hi_r10_r7_r11_ROR_24[] = {
  0x7b, 0xac, 0xa7, 0x86 // sxtab hi r10 r7 r11 ROR 24
};
const byte kInstruction_sxtab_le_r9_r8_r0_ROR_8[] = {
  0x70, 0x94, 0xa8, 0xd6 // sxtab le r9 r8 r0 ROR 8
};
const byte kInstruction_sxtab_vs_r6_r8_r7_ROR_0[] = {
  0x77, 0x60, 0xa8, 0x66 // sxtab vs r6 r8 r7 ROR 0
};
const byte kInstruction_sxtab_le_r0_r8_r13_ROR_0[] = {
  0x7d, 0x00, 0xa8, 0xd6 // sxtab le r0 r8 r13 ROR 0
};
const byte kInstruction_sxtab_cc_r4_r4_r6_ROR_16[] = {
  0x76, 0x48, 0xa4, 0x36 // sxtab cc r4 r4 r6 ROR 16
};
const byte kInstruction_sxtab_ge_r0_r9_r3_ROR_8[] = {
  0x73, 0x04, 0xa9, 0xa6 // sxtab ge r0 r9 r3 ROR 8
};
const byte kInstruction_sxtab_cs_r5_r4_r2_ROR_0[] = {
  0x72, 0x50, 0xa4, 0x26 // sxtab cs r5 r4 r2 ROR 0
};
const byte kInstruction_sxtab_gt_r12_r9_r8_ROR_16[] = {
  0x78, 0xc8, 0xa9, 0xc6 // sxtab gt r12 r9 r8 ROR 16
};
const byte kInstruction_sxtab_vs_r4_r9_r3_ROR_24[] = {
  0x73, 0x4c, 0xa9, 0x66 // sxtab vs r4 r9 r3 ROR 24
};
const byte kInstruction_sxtab_cc_r8_r11_r8_ROR_8[] = {
  0x78, 0x84, 0xab, 0x36 // sxtab cc r8 r11 r8 ROR 8
};
const byte kInstruction_sxtab_lt_r5_r6_r11_ROR_24[] = {
  0x7b, 0x5c, 0xa6, 0xb6 // sxtab lt r5 r6 r11 ROR 24
};
const byte kInstruction_sxtab_cc_r8_r11_r1_ROR_24[] = {
  0x71, 0x8c, 0xab, 0x36 // sxtab cc r8 r11 r1 ROR 24
};
const byte kInstruction_sxtab_ls_r0_r0_r14_ROR_16[] = {
  0x7e, 0x08, 0xa0, 0x96 // sxtab ls r0 r0 r14 ROR 16
};
const byte kInstruction_sxtab_lt_r10_r14_r7_ROR_8[] = {
  0x77, 0xa4, 0xae, 0xb6 // sxtab lt r10 r14 r7 ROR 8
};
const byte kInstruction_sxtab_pl_r10_r13_r6_ROR_16[] = {
  0x76, 0xa8, 0xad, 0x56 // sxtab pl r10 r13 r6 ROR 16
};
const byte kInstruction_sxtab_lt_r11_r13_r4_ROR_8[] = {
  0x74, 0xb4, 0xad, 0xb6 // sxtab lt r11 r13 r4 ROR 8
};
const byte kInstruction_sxtab_cs_r5_r14_r0_ROR_0[] = {
  0x70, 0x50, 0xae, 0x26 // sxtab cs r5 r14 r0 ROR 0
};
const byte kInstruction_sxtab_lt_r3_r2_r10_ROR_0[] = {
  0x7a, 0x30, 0xa2, 0xb6 // sxtab lt r3 r2 r10 ROR 0
};
const byte kInstruction_sxtab_ls_r7_r6_r7_ROR_8[] = {
  0x77, 0x74, 0xa6, 0x96 // sxtab ls r7 r6 r7 ROR 8
};
const byte kInstruction_sxtab_ge_r2_r14_r14_ROR_0[] = {
  0x7e, 0x20, 0xae, 0xa6 // sxtab ge r2 r14 r14 ROR 0
};
const byte kInstruction_sxtab_al_r9_r5_r9_ROR_0[] = {
  0x79, 0x90, 0xa5, 0xe6 // sxtab al r9 r5 r9 ROR 0
};
const byte kInstruction_sxtab_vc_r14_r2_r5_ROR_24[] = {
  0x75, 0xec, 0xa2, 0x76 // sxtab vc r14 r2 r5 ROR 24
};
const byte kInstruction_sxtab_hi_r1_r9_r3_ROR_16[] = {
  0x73, 0x18, 0xa9, 0x86 // sxtab hi r1 r9 r3 ROR 16
};
const byte kInstruction_sxtab_lt_r1_r3_r10_ROR_8[] = {
  0x7a, 0x14, 0xa3, 0xb6 // sxtab lt r1 r3 r10 ROR 8
};
const byte kInstruction_sxtab_lt_r12_r1_r5_ROR_16[] = {
  0x75, 0xc8, 0xa1, 0xb6 // sxtab lt r12 r1 r5 ROR 16
};
const byte kInstruction_sxtab_ge_r3_r4_r10_ROR_0[] = {
  0x7a, 0x30, 0xa4, 0xa6 // sxtab ge r3 r4 r10 ROR 0
};
const byte kInstruction_sxtab_ge_r7_r13_r8_ROR_16[] = {
  0x78, 0x78, 0xad, 0xa6 // sxtab ge r7 r13 r8 ROR 16
};
const byte kInstruction_sxtab_eq_r12_r8_r7_ROR_16[] = {
  0x77, 0xc8, 0xa8, 0x06 // sxtab eq r12 r8 r7 ROR 16
};
const byte kInstruction_sxtab_vs_r6_r12_r0_ROR_16[] = {
  0x70, 0x68, 0xac, 0x66 // sxtab vs r6 r12 r0 ROR 16
};
const byte kInstruction_sxtab_al_r4_r2_r9_ROR_24[] = {
  0x79, 0x4c, 0xa2, 0xe6 // sxtab al r4 r2 r9 ROR 24
};
const byte kInstruction_sxtab_ne_r4_r12_r6_ROR_8[] = {
  0x76, 0x44, 0xac, 0x16 // sxtab ne r4 r12 r6 ROR 8
};
const byte kInstruction_sxtab_pl_r10_r5_r11_ROR_8[] = {
  0x7b, 0xa4, 0xa5, 0x56 // sxtab pl r10 r5 r11 ROR 8
};
const byte kInstruction_sxtab_cc_r12_r5_r1_ROR_0[] = {
  0x71, 0xc0, 0xa5, 0x36 // sxtab cc r12 r5 r1 ROR 0
};
const byte kInstruction_sxtab_lt_r11_r7_r8_ROR_16[] = {
  0x78, 0xb8, 0xa7, 0xb6 // sxtab lt r11 r7 r8 ROR 16
};
const byte kInstruction_sxtab_ls_r1_r7_r5_ROR_0[] = {
  0x75, 0x10, 0xa7, 0x96 // sxtab ls r1 r7 r5 ROR 0
};
const byte kInstruction_sxtab_mi_r13_r2_r12_ROR_16[] = {
  0x7c, 0xd8, 0xa2, 0x46 // sxtab mi r13 r2 r12 ROR 16
};
const byte kInstruction_sxtab_ls_r1_r7_r2_ROR_16[] = {
  0x72, 0x18, 0xa7, 0x96 // sxtab ls r1 r7 r2 ROR 16
};
const byte kInstruction_sxtab_al_r14_r8_r13_ROR_8[] = {
  0x7d, 0xe4, 0xa8, 0xe6 // sxtab al r14 r8 r13 ROR 8
};
const byte kInstruction_sxtab_lt_r1_r0_r10_ROR_0[] = {
  0x7a, 0x10, 0xa0, 0xb6 // sxtab lt r1 r0 r10 ROR 0
};
const byte kInstruction_sxtab_al_r10_r7_r5_ROR_24[] = {
  0x75, 0xac, 0xa7, 0xe6 // sxtab al r10 r7 r5 ROR 24
};
const byte kInstruction_sxtab_vs_r12_r14_r10_ROR_0[] = {
  0x7a, 0xc0, 0xae, 0x66 // sxtab vs r12 r14 r10 ROR 0
};
const byte kInstruction_sxtab_vs_r7_r9_r1_ROR_16[] = {
  0x71, 0x78, 0xa9, 0x66 // sxtab vs r7 r9 r1 ROR 16
};
const byte kInstruction_sxtab_pl_r8_r0_r0_ROR_24[] = {
  0x70, 0x8c, 0xa0, 0x56 // sxtab pl r8 r0 r0 ROR 24
};
const byte kInstruction_sxtab_le_r12_r12_r5_ROR_8[] = {
  0x75, 0xc4, 0xac, 0xd6 // sxtab le r12 r12 r5 ROR 8
};
const byte kInstruction_sxtab_gt_r2_r7_r11_ROR_24[] = {
  0x7b, 0x2c, 0xa7, 0xc6 // sxtab gt r2 r7 r11 ROR 24
};
const byte kInstruction_sxtab_ne_r2_r0_r4_ROR_24[] = {
  0x74, 0x2c, 0xa0, 0x16 // sxtab ne r2 r0 r4 ROR 24
};
const byte kInstruction_sxtab_le_r7_r3_r7_ROR_16[] = {
  0x77, 0x78, 0xa3, 0xd6 // sxtab le r7 r3 r7 ROR 16
};
const byte kInstruction_sxtab_vc_r3_r13_r8_ROR_16[] = {
  0x78, 0x38, 0xad, 0x76 // sxtab vc r3 r13 r8 ROR 16
};
const byte kInstruction_sxtab_al_r12_r14_r0_ROR_8[] = {
  0x70, 0xc4, 0xae, 0xe6 // sxtab al r12 r14 r0 ROR 8
};
const byte kInstruction_sxtab_le_r4_r9_r1_ROR_0[] = {
  0x71, 0x40, 0xa9, 0xd6 // sxtab le r4 r9 r1 ROR 0
};
const byte kInstruction_sxtab_ge_r0_r1_r1_ROR_8[] = {
  0x71, 0x04, 0xa1, 0xa6 // sxtab ge r0 r1 r1 ROR 8
};
const byte kInstruction_sxtab_pl_r0_r11_r13_ROR_16[] = {
  0x7d, 0x08, 0xab, 0x56 // sxtab pl r0 r11 r13 ROR 16
};
const byte kInstruction_sxtab_hi_r7_r14_r0_ROR_0[] = {
  0x70, 0x70, 0xae, 0x86 // sxtab hi r7 r14 r0 ROR 0
};
const byte kInstruction_sxtab_ls_r11_r14_r6_ROR_24[] = {
  0x76, 0xbc, 0xae, 0x96 // sxtab ls r11 r14 r6 ROR 24
};
const byte kInstruction_sxtab_ge_r2_r9_r12_ROR_0[] = {
  0x7c, 0x20, 0xa9, 0xa6 // sxtab ge r2 r9 r12 ROR 0
};
const byte kInstruction_sxtab_lt_r12_r1_r7_ROR_16[] = {
  0x77, 0xc8, 0xa1, 0xb6 // sxtab lt r12 r1 r7 ROR 16
};
const byte kInstruction_sxtab_eq_r7_r7_r12_ROR_16[] = {
  0x7c, 0x78, 0xa7, 0x06 // sxtab eq r7 r7 r12 ROR 16
};
const byte kInstruction_sxtab_ls_r2_r9_r14_ROR_24[] = {
  0x7e, 0x2c, 0xa9, 0x96 // sxtab ls r2 r9 r14 ROR 24
};
const byte kInstruction_sxtab_lt_r10_r14_r14_ROR_24[] = {
  0x7e, 0xac, 0xae, 0xb6 // sxtab lt r10 r14 r14 ROR 24
};
const byte kInstruction_sxtab_pl_r8_r10_r8_ROR_16[] = {
  0x78, 0x88, 0xaa, 0x56 // sxtab pl r8 r10 r8 ROR 16
};
const byte kInstruction_sxtab_ls_r8_r12_r2_ROR_8[] = {
  0x72, 0x84, 0xac, 0x96 // sxtab ls r8 r12 r2 ROR 8
};
const byte kInstruction_sxtab_ne_r11_r10_r3_ROR_0[] = {
  0x73, 0xb0, 0xaa, 0x16 // sxtab ne r11 r10 r3 ROR 0
};
const byte kInstruction_sxtab_ne_r6_r2_r10_ROR_8[] = {
  0x7a, 0x64, 0xa2, 0x16 // sxtab ne r6 r2 r10 ROR 8
};
const byte kInstruction_sxtab_mi_r10_r7_r12_ROR_16[] = {
  0x7c, 0xa8, 0xa7, 0x46 // sxtab mi r10 r7 r12 ROR 16
};
const byte kInstruction_sxtab_vc_r8_r14_r7_ROR_24[] = {
  0x77, 0x8c, 0xae, 0x76 // sxtab vc r8 r14 r7 ROR 24
};
const byte kInstruction_sxtab_gt_r12_r9_r4_ROR_8[] = {
  0x74, 0xc4, 0xa9, 0xc6 // sxtab gt r12 r9 r4 ROR 8
};
const byte kInstruction_sxtab_pl_r1_r4_r11_ROR_16[] = {
  0x7b, 0x18, 0xa4, 0x56 // sxtab pl r1 r4 r11 ROR 16
};
const byte kInstruction_sxtab_mi_r3_r10_r6_ROR_16[] = {
  0x76, 0x38, 0xaa, 0x46 // sxtab mi r3 r10 r6 ROR 16
};
const byte kInstruction_sxtab_eq_r14_r10_r12_ROR_24[] = {
  0x7c, 0xec, 0xaa, 0x06 // sxtab eq r14 r10 r12 ROR 24
};
const byte kInstruction_sxtab_eq_r7_r9_r13_ROR_8[] = {
  0x7d, 0x74, 0xa9, 0x06 // sxtab eq r7 r9 r13 ROR 8
};
const byte kInstruction_sxtab_eq_r11_r7_r4_ROR_16[] = {
  0x74, 0xb8, 0xa7, 0x06 // sxtab eq r11 r7 r4 ROR 16
};
const byte kInstruction_sxtab_vc_r12_r1_r6_ROR_0[] = {
  0x76, 0xc0, 0xa1, 0x76 // sxtab vc r12 r1 r6 ROR 0
};
const byte kInstruction_sxtab_pl_r6_r0_r2_ROR_0[] = {
  0x72, 0x60, 0xa0, 0x56 // sxtab pl r6 r0 r2 ROR 0
};
const byte kInstruction_sxtab_hi_r11_r14_r5_ROR_8[] = {
  0x75, 0xb4, 0xae, 0x86 // sxtab hi r11 r14 r5 ROR 8
};
const byte kInstruction_sxtab_pl_r4_r14_r5_ROR_24[] = {
  0x75, 0x4c, 0xae, 0x56 // sxtab pl r4 r14 r5 ROR 24
};
const byte kInstruction_sxtab_eq_r5_r1_r14_ROR_0[] = {
  0x7e, 0x50, 0xa1, 0x06 // sxtab eq r5 r1 r14 ROR 0
};
const byte kInstruction_sxtab_gt_r2_r8_r9_ROR_24[] = {
  0x79, 0x2c, 0xa8, 0xc6 // sxtab gt r2 r8 r9 ROR 24
};
const byte kInstruction_sxtab_pl_r11_r8_r11_ROR_8[] = {
  0x7b, 0xb4, 0xa8, 0x56 // sxtab pl r11 r8 r11 ROR 8
};
const byte kInstruction_sxtab_cc_r7_r7_r8_ROR_16[] = {
  0x78, 0x78, 0xa7, 0x36 // sxtab cc r7 r7 r8 ROR 16
};
const byte kInstruction_sxtab_le_r8_r3_r0_ROR_16[] = {
  0x70, 0x88, 0xa3, 0xd6 // sxtab le r8 r3 r0 ROR 16
};
const byte kInstruction_sxtab_hi_r4_r5_r9_ROR_8[] = {
  0x79, 0x44, 0xa5, 0x86 // sxtab hi r4 r5 r9 ROR 8
};
const byte kInstruction_sxtab_pl_r2_r9_r7_ROR_8[] = {
  0x77, 0x24, 0xa9, 0x56 // sxtab pl r2 r9 r7 ROR 8
};
const byte kInstruction_sxtab_al_r11_r11_r2_ROR_16[] = {
  0x72, 0xb8, 0xab, 0xe6 // sxtab al r11 r11 r2 ROR 16
};
const byte kInstruction_sxtab_ne_r12_r5_r3_ROR_24[] = {
  0x73, 0xcc, 0xa5, 0x16 // sxtab ne r12 r5 r3 ROR 24
};
const byte kInstruction_sxtab_mi_r6_r8_r5_ROR_16[] = {
  0x75, 0x68, 0xa8, 0x46 // sxtab mi r6 r8 r5 ROR 16
};
const byte kInstruction_sxtab_le_r14_r0_r7_ROR_0[] = {
  0x77, 0xe0, 0xa0, 0xd6 // sxtab le r14 r0 r7 ROR 0
};
const byte kInstruction_sxtab_cc_r8_r4_r13_ROR_8[] = {
  0x7d, 0x84, 0xa4, 0x36 // sxtab cc r8 r4 r13 ROR 8
};
const byte kInstruction_sxtab_al_r1_r11_r3_ROR_8[] = {
  0x73, 0x14, 0xab, 0xe6 // sxtab al r1 r11 r3 ROR 8
};
const byte kInstruction_sxtab_vc_r3_r2_r13_ROR_16[] = {
  0x7d, 0x38, 0xa2, 0x76 // sxtab vc r3 r2 r13 ROR 16
};
const byte kInstruction_sxtab_vs_r11_r14_r8_ROR_16[] = {
  0x78, 0xb8, 0xae, 0x66 // sxtab vs r11 r14 r8 ROR 16
};
const byte kInstruction_sxtab_eq_r3_r4_r12_ROR_24[] = {
  0x7c, 0x3c, 0xa4, 0x06 // sxtab eq r3 r4 r12 ROR 24
};
const byte kInstruction_sxtab_gt_r10_r10_r5_ROR_8[] = {
  0x75, 0xa4, 0xaa, 0xc6 // sxtab gt r10 r10 r5 ROR 8
};
const byte kInstruction_sxtab_hi_r6_r10_r4_ROR_16[] = {
  0x74, 0x68, 0xaa, 0x86 // sxtab hi r6 r10 r4 ROR 16
};
const byte kInstruction_sxtab_eq_r12_r3_r13_ROR_8[] = {
  0x7d, 0xc4, 0xa3, 0x06 // sxtab eq r12 r3 r13 ROR 8
};
const byte kInstruction_sxtab_le_r7_r10_r2_ROR_8[] = {
  0x72, 0x74, 0xaa, 0xd6 // sxtab le r7 r10 r2 ROR 8
};
const byte kInstruction_sxtab_cs_r4_r9_r14_ROR_0[] = {
  0x7e, 0x40, 0xa9, 0x26 // sxtab cs r4 r9 r14 ROR 0
};
const byte kInstruction_sxtab_ge_r10_r3_r5_ROR_16[] = {
  0x75, 0xa8, 0xa3, 0xa6 // sxtab ge r10 r3 r5 ROR 16
};
const byte kInstruction_sxtab_cc_r1_r10_r11_ROR_0[] = {
  0x7b, 0x10, 0xaa, 0x36 // sxtab cc r1 r10 r11 ROR 0
};
const byte kInstruction_sxtab_ls_r10_r11_r6_ROR_24[] = {
  0x76, 0xac, 0xab, 0x96 // sxtab ls r10 r11 r6 ROR 24
};
const byte kInstruction_sxtab_ne_r12_r10_r1_ROR_8[] = {
  0x71, 0xc4, 0xaa, 0x16 // sxtab ne r12 r10 r1 ROR 8
};
const byte kInstruction_sxtab_lt_r10_r4_r7_ROR_24[] = {
  0x77, 0xac, 0xa4, 0xb6 // sxtab lt r10 r4 r7 ROR 24
};
const byte kInstruction_sxtab_ge_r7_r0_r10_ROR_0[] = {
  0x7a, 0x70, 0xa0, 0xa6 // sxtab ge r7 r0 r10 ROR 0
};
const byte kInstruction_sxtab_ls_r7_r6_r5_ROR_0[] = {
  0x75, 0x70, 0xa6, 0x96 // sxtab ls r7 r6 r5 ROR 0
};
const byte kInstruction_sxtab_cc_r8_r1_r8_ROR_8[] = {
  0x78, 0x84, 0xa1, 0x36 // sxtab cc r8 r1 r8 ROR 8
};
const byte kInstruction_sxtab_cs_r7_r0_r13_ROR_24[] = {
  0x7d, 0x7c, 0xa0, 0x26 // sxtab cs r7 r0 r13 ROR 24
};
const byte kInstruction_sxtab_cc_r10_r6_r0_ROR_0[] = {
  0x70, 0xa0, 0xa6, 0x36 // sxtab cc r10 r6 r0 ROR 0
};
const byte kInstruction_sxtab_mi_r0_r13_r6_ROR_8[] = {
  0x76, 0x04, 0xad, 0x46 // sxtab mi r0 r13 r6 ROR 8
};
const byte kInstruction_sxtab_hi_r12_r5_r3_ROR_16[] = {
  0x73, 0xc8, 0xa5, 0x86 // sxtab hi r12 r5 r3 ROR 16
};
const byte kInstruction_sxtab_vs_r4_r9_r2_ROR_8[] = {
  0x72, 0x44, 0xa9, 0x66 // sxtab vs r4 r9 r2 ROR 8
};
const byte kInstruction_sxtab_vs_r9_r7_r12_ROR_24[] = {
  0x7c, 0x9c, 0xa7, 0x66 // sxtab vs r9 r7 r12 ROR 24
};
const byte kInstruction_sxtab_cc_r7_r13_r8_ROR_0[] = {
  0x78, 0x70, 0xad, 0x36 // sxtab cc r7 r13 r8 ROR 0
};
const byte kInstruction_sxtab_mi_r14_r3_r2_ROR_8[] = {
  0x72, 0xe4, 0xa3, 0x46 // sxtab mi r14 r3 r2 ROR 8
};
const byte kInstruction_sxtab_vc_r11_r14_r2_ROR_8[] = {
  0x72, 0xb4, 0xae, 0x76 // sxtab vc r11 r14 r2 ROR 8
};
const byte kInstruction_sxtab_pl_r9_r7_r0_ROR_16[] = {
  0x70, 0x98, 0xa7, 0x56 // sxtab pl r9 r7 r0 ROR 16
};
const byte kInstruction_sxtab_vs_r13_r10_r4_ROR_24[] = {
  0x74, 0xdc, 0xaa, 0x66 // sxtab vs r13 r10 r4 ROR 24
};
const byte kInstruction_sxtab_vs_r13_r8_r10_ROR_16[] = {
  0x7a, 0xd8, 0xa8, 0x66 // sxtab vs r13 r8 r10 ROR 16
};
const byte kInstruction_sxtab_hi_r9_r8_r8_ROR_0[] = {
  0x78, 0x90, 0xa8, 0x86 // sxtab hi r9 r8 r8 ROR 0
};
const byte kInstruction_sxtab_mi_r10_r14_r9_ROR_16[] = {
  0x79, 0xa8, 0xae, 0x46 // sxtab mi r10 r14 r9 ROR 16
};
const byte kInstruction_sxtab_pl_r4_r2_r5_ROR_24[] = {
  0x75, 0x4c, 0xa2, 0x56 // sxtab pl r4 r2 r5 ROR 24
};
const byte kInstruction_sxtab_cc_r13_r10_r4_ROR_8[] = {
  0x74, 0xd4, 0xaa, 0x36 // sxtab cc r13 r10 r4 ROR 8
};
const byte kInstruction_sxtab_hi_r4_r9_r2_ROR_16[] = {
  0x72, 0x48, 0xa9, 0x86 // sxtab hi r4 r9 r2 ROR 16
};
const byte kInstruction_sxtab_cs_r13_r5_r0_ROR_16[] = {
  0x70, 0xd8, 0xa5, 0x26 // sxtab cs r13 r5 r0 ROR 16
};
const byte kInstruction_sxtab_le_r10_r5_r0_ROR_0[] = {
  0x70, 0xa0, 0xa5, 0xd6 // sxtab le r10 r5 r0 ROR 0
};
const byte kInstruction_sxtab_al_r10_r7_r7_ROR_24[] = {
  0x77, 0xac, 0xa7, 0xe6 // sxtab al r10 r7 r7 ROR 24
};
const byte kInstruction_sxtab_pl_r2_r1_r10_ROR_16[] = {
  0x7a, 0x28, 0xa1, 0x56 // sxtab pl r2 r1 r10 ROR 16
};
const byte kInstruction_sxtab_le_r7_r11_r0_ROR_24[] = {
  0x70, 0x7c, 0xab, 0xd6 // sxtab le r7 r11 r0 ROR 24
};
const byte kInstruction_sxtab_vs_r6_r8_r0_ROR_8[] = {
  0x70, 0x64, 0xa8, 0x66 // sxtab vs r6 r8 r0 ROR 8
};
const byte kInstruction_sxtab_al_r6_r9_r2_ROR_8[] = {
  0x72, 0x64, 0xa9, 0xe6 // sxtab al r6 r9 r2 ROR 8
};
const byte kInstruction_sxtab_vs_r3_r12_r1_ROR_8[] = {
  0x71, 0x34, 0xac, 0x66 // sxtab vs r3 r12 r1 ROR 8
};
const byte kInstruction_sxtab_le_r14_r5_r11_ROR_16[] = {
  0x7b, 0xe8, 0xa5, 0xd6 // sxtab le r14 r5 r11 ROR 16
};
const byte kInstruction_sxtab_cs_r1_r12_r14_ROR_0[] = {
  0x7e, 0x10, 0xac, 0x26 // sxtab cs r1 r12 r14 ROR 0
};
const byte kInstruction_sxtab_ne_r14_r2_r2_ROR_24[] = {
  0x72, 0xec, 0xa2, 0x16 // sxtab ne r14 r2 r2 ROR 24
};
const byte kInstruction_sxtab_eq_r14_r0_r8_ROR_16[] = {
  0x78, 0xe8, 0xa0, 0x06 // sxtab eq r14 r0 r8 ROR 16
};
const byte kInstruction_sxtab_ge_r13_r1_r2_ROR_24[] = {
  0x72, 0xdc, 0xa1, 0xa6 // sxtab ge r13 r1 r2 ROR 24
};
const byte kInstruction_sxtab_lt_r2_r1_r10_ROR_8[] = {
  0x7a, 0x24, 0xa1, 0xb6 // sxtab lt r2 r1 r10 ROR 8
};
const byte kInstruction_sxtab_vc_r9_r3_r14_ROR_0[] = {
  0x7e, 0x90, 0xa3, 0x76 // sxtab vc r9 r3 r14 ROR 0
};
const byte kInstruction_sxtab_gt_r6_r0_r4_ROR_24[] = {
  0x74, 0x6c, 0xa0, 0xc6 // sxtab gt r6 r0 r4 ROR 24
};
const byte kInstruction_sxtab_cs_r4_r12_r2_ROR_0[] = {
  0x72, 0x40, 0xac, 0x26 // sxtab cs r4 r12 r2 ROR 0
};
const byte kInstruction_sxtab_vc_r1_r7_r5_ROR_8[] = {
  0x75, 0x14, 0xa7, 0x76 // sxtab vc r1 r7 r5 ROR 8
};
const byte kInstruction_sxtab_cs_r6_r7_r1_ROR_8[] = {
  0x71, 0x64, 0xa7, 0x26 // sxtab cs r6 r7 r1 ROR 8
};
const byte kInstruction_sxtab_cs_r1_r10_r7_ROR_8[] = {
  0x77, 0x14, 0xaa, 0x26 // sxtab cs r1 r10 r7 ROR 8
};
const byte kInstruction_sxtab_eq_r13_r5_r0_ROR_16[] = {
  0x70, 0xd8, 0xa5, 0x06 // sxtab eq r13 r5 r0 ROR 16
};
const byte kInstruction_sxtab_gt_r11_r12_r12_ROR_16[] = {
  0x7c, 0xb8, 0xac, 0xc6 // sxtab gt r11 r12 r12 ROR 16
};
const byte kInstruction_sxtab_le_r7_r12_r9_ROR_16[] = {
  0x79, 0x78, 0xac, 0xd6 // sxtab le r7 r12 r9 ROR 16
};
const byte kInstruction_sxtab_le_r3_r7_r6_ROR_8[] = {
  0x76, 0x34, 0xa7, 0xd6 // sxtab le r3 r7 r6 ROR 8
};
const byte kInstruction_sxtab_pl_r4_r7_r6_ROR_8[] = {
  0x76, 0x44, 0xa7, 0x56 // sxtab pl r4 r7 r6 ROR 8
};
const byte kInstruction_sxtab_ge_r1_r7_r8_ROR_16[] = {
  0x78, 0x18, 0xa7, 0xa6 // sxtab ge r1 r7 r8 ROR 16
};
const byte kInstruction_sxtab_vs_r4_r6_r4_ROR_0[] = {
  0x74, 0x40, 0xa6, 0x66 // sxtab vs r4 r6 r4 ROR 0
};
const byte kInstruction_sxtab_hi_r11_r11_r4_ROR_8[] = {
  0x74, 0xb4, 0xab, 0x86 // sxtab hi r11 r11 r4 ROR 8
};
const byte kInstruction_sxtab_hi_r13_r0_r1_ROR_16[] = {
  0x71, 0xd8, 0xa0, 0x86 // sxtab hi r13 r0 r1 ROR 16
};
const byte kInstruction_sxtab_lt_r0_r3_r1_ROR_0[] = {
  0x71, 0x00, 0xa3, 0xb6 // sxtab lt r0 r3 r1 ROR 0
};
const byte kInstruction_sxtab_vs_r2_r14_r1_ROR_16[] = {
  0x71, 0x28, 0xae, 0x66 // sxtab vs r2 r14 r1 ROR 16
};
const byte kInstruction_sxtab_lt_r0_r2_r3_ROR_16[] = {
  0x73, 0x08, 0xa2, 0xb6 // sxtab lt r0 r2 r3 ROR 16
};
const byte kInstruction_sxtab_cc_r8_r14_r2_ROR_0[] = {
  0x72, 0x80, 0xae, 0x36 // sxtab cc r8 r14 r2 ROR 0
};
const byte kInstruction_sxtab_mi_r1_r4_r5_ROR_8[] = {
  0x75, 0x14, 0xa4, 0x46 // sxtab mi r1 r4 r5 ROR 8
};
const byte kInstruction_sxtab_cc_r13_r5_r9_ROR_24[] = {
  0x79, 0xdc, 0xa5, 0x36 // sxtab cc r13 r5 r9 ROR 24
};
const byte kInstruction_sxtab_cc_r0_r14_r3_ROR_8[] = {
  0x73, 0x04, 0xae, 0x36 // sxtab cc r0 r14 r3 ROR 8
};
const byte kInstruction_sxtab_vc_r6_r5_r10_ROR_8[] = {
  0x7a, 0x64, 0xa5, 0x76 // sxtab vc r6 r5 r10 ROR 8
};
const byte kInstruction_sxtab_vs_r6_r10_r9_ROR_8[] = {
  0x79, 0x64, 0xaa, 0x66 // sxtab vs r6 r10 r9 ROR 8
};
const byte kInstruction_sxtab_gt_r5_r14_r3_ROR_16[] = {
  0x73, 0x58, 0xae, 0xc6 // sxtab gt r5 r14 r3 ROR 16
};
const byte kInstruction_sxtab_hi_r12_r7_r5_ROR_0[] = {
  0x75, 0xc0, 0xa7, 0x86 // sxtab hi r12 r7 r5 ROR 0
};
const byte kInstruction_sxtab_hi_r6_r3_r0_ROR_8[] = {
  0x70, 0x64, 0xa3, 0x86 // sxtab hi r6 r3 r0 ROR 8
};
const byte kInstruction_sxtab_le_r12_r3_r1_ROR_8[] = {
  0x71, 0xc4, 0xa3, 0xd6 // sxtab le r12 r3 r1 ROR 8
};
const byte kInstruction_sxtab_cc_r13_r14_r0_ROR_16[] = {
  0x70, 0xd8, 0xae, 0x36 // sxtab cc r13 r14 r0 ROR 16
};
const byte kInstruction_sxtab_ls_r14_r14_r11_ROR_24[] = {
  0x7b, 0xec, 0xae, 0x96 // sxtab ls r14 r14 r11 ROR 24
};
const byte kInstruction_sxtab_lt_r0_r13_r0_ROR_0[] = {
  0x70, 0x00, 0xad, 0xb6 // sxtab lt r0 r13 r0 ROR 0
};
const byte kInstruction_sxtab_pl_r12_r7_r4_ROR_24[] = {
  0x74, 0xcc, 0xa7, 0x56 // sxtab pl r12 r7 r4 ROR 24
};
const byte kInstruction_sxtab_mi_r11_r1_r11_ROR_16[] = {
  0x7b, 0xb8, 0xa1, 0x46 // sxtab mi r11 r1 r11 ROR 16
};
const byte kInstruction_sxtab_al_r5_r2_r11_ROR_24[] = {
  0x7b, 0x5c, 0xa2, 0xe6 // sxtab al r5 r2 r11 ROR 24
};
const byte kInstruction_sxtab_mi_r3_r3_r4_ROR_24[] = {
  0x74, 0x3c, 0xa3, 0x46 // sxtab mi r3 r3 r4 ROR 24
};
const byte kInstruction_sxtab_vc_r12_r12_r8_ROR_24[] = {
  0x78, 0xcc, 0xac, 0x76 // sxtab vc r12 r12 r8 ROR 24
};
const byte kInstruction_sxtab_ls_r3_r12_r10_ROR_24[] = {
  0x7a, 0x3c, 0xac, 0x96 // sxtab ls r3 r12 r10 ROR 24
};
const byte kInstruction_sxtab_ne_r0_r0_r4_ROR_0[] = {
  0x74, 0x00, 0xa0, 0x16 // sxtab ne r0 r0 r4 ROR 0
};
const byte kInstruction_sxtab_ne_r12_r8_r13_ROR_8[] = {
  0x7d, 0xc4, 0xa8, 0x16 // sxtab ne r12 r8 r13 ROR 8
};
const byte kInstruction_sxtab_mi_r4_r4_r2_ROR_16[] = {
  0x72, 0x48, 0xa4, 0x46 // sxtab mi r4 r4 r2 ROR 16
};
const byte kInstruction_sxtab_ge_r8_r2_r10_ROR_16[] = {
  0x7a, 0x88, 0xa2, 0xa6 // sxtab ge r8 r2 r10 ROR 16
};
const byte kInstruction_sxtab_cs_r6_r5_r12_ROR_8[] = {
  0x7c, 0x64, 0xa5, 0x26 // sxtab cs r6 r5 r12 ROR 8
};
const byte kInstruction_sxtab_ne_r7_r13_r5_ROR_0[] = {
  0x75, 0x70, 0xad, 0x16 // sxtab ne r7 r13 r5 ROR 0
};
const byte kInstruction_sxtab_ne_r4_r3_r1_ROR_24[] = {
  0x71, 0x4c, 0xa3, 0x16 // sxtab ne r4 r3 r1 ROR 24
};
const byte kInstruction_sxtab_ls_r4_r4_r4_ROR_24[] = {
  0x74, 0x4c, 0xa4, 0x96 // sxtab ls r4 r4 r4 ROR 24
};
const byte kInstruction_sxtab_ge_r14_r10_r14_ROR_24[] = {
  0x7e, 0xec, 0xaa, 0xa6 // sxtab ge r14 r10 r14 ROR 24
};
const byte kInstruction_sxtab_lt_r4_r7_r2_ROR_24[] = {
  0x72, 0x4c, 0xa7, 0xb6 // sxtab lt r4 r7 r2 ROR 24
};
const byte kInstruction_sxtab_vs_r10_r3_r3_ROR_16[] = {
  0x73, 0xa8, 0xa3, 0x66 // sxtab vs r10 r3 r3 ROR 16
};
const byte kInstruction_sxtab_mi_r13_r2_r5_ROR_0[] = {
  0x75, 0xd0, 0xa2, 0x46 // sxtab mi r13 r2 r5 ROR 0
};
const byte kInstruction_sxtab_cc_r14_r0_r0_ROR_0[] = {
  0x70, 0xe0, 0xa0, 0x36 // sxtab cc r14 r0 r0 ROR 0
};
const byte kInstruction_sxtab_ne_r7_r11_r10_ROR_16[] = {
  0x7a, 0x78, 0xab, 0x16 // sxtab ne r7 r11 r10 ROR 16
};
const byte kInstruction_sxtab_al_r12_r13_r4_ROR_24[] = {
  0x74, 0xcc, 0xad, 0xe6 // sxtab al r12 r13 r4 ROR 24
};
const byte kInstruction_sxtab_lt_r6_r8_r1_ROR_0[] = {
  0x71, 0x60, 0xa8, 0xb6 // sxtab lt r6 r8 r1 ROR 0
};
const byte kInstruction_sxtab_ne_r10_r6_r6_ROR_24[] = {
  0x76, 0xac, 0xa6, 0x16 // sxtab ne r10 r6 r6 ROR 24
};
const byte kInstruction_sxtab_le_r12_r5_r0_ROR_0[] = {
  0x70, 0xc0, 0xa5, 0xd6 // sxtab le r12 r5 r0 ROR 0
};
const byte kInstruction_sxtab_ne_r6_r9_r11_ROR_0[] = {
  0x7b, 0x60, 0xa9, 0x16 // sxtab ne r6 r9 r11 ROR 0
};
const byte kInstruction_sxtab_hi_r10_r5_r10_ROR_16[] = {
  0x7a, 0xa8, 0xa5, 0x86 // sxtab hi r10 r5 r10 ROR 16
};
const byte kInstruction_sxtab_hi_r7_r14_r3_ROR_24[] = {
  0x73, 0x7c, 0xae, 0x86 // sxtab hi r7 r14 r3 ROR 24
};
const byte kInstruction_sxtab_ls_r7_r10_r5_ROR_16[] = {
  0x75, 0x78, 0xaa, 0x96 // sxtab ls r7 r10 r5 ROR 16
};
const byte kInstruction_sxtab_vs_r11_r4_r4_ROR_0[] = {
  0x74, 0xb0, 0xa4, 0x66 // sxtab vs r11 r4 r4 ROR 0
};
const byte kInstruction_sxtab_ls_r13_r2_r0_ROR_0[] = {
  0x70, 0xd0, 0xa2, 0x96 // sxtab ls r13 r2 r0 ROR 0
};
const byte kInstruction_sxtab_lt_r4_r13_r11_ROR_8[] = {
  0x7b, 0x44, 0xad, 0xb6 // sxtab lt r4 r13 r11 ROR 8
};
const byte kInstruction_sxtab_ne_r2_r12_r12_ROR_0[] = {
  0x7c, 0x20, 0xac, 0x16 // sxtab ne r2 r12 r12 ROR 0
};
const byte kInstruction_sxtab_cc_r5_r7_r5_ROR_0[] = {
  0x75, 0x50, 0xa7, 0x36 // sxtab cc r5 r7 r5 ROR 0
};
const byte kInstruction_sxtab_vc_r6_r14_r9_ROR_24[] = {
  0x79, 0x6c, 0xae, 0x76 // sxtab vc r6 r14 r9 ROR 24
};
const byte kInstruction_sxtab_ne_r10_r11_r12_ROR_8[] = {
  0x7c, 0xa4, 0xab, 0x16 // sxtab ne r10 r11 r12 ROR 8
};
const byte kInstruction_sxtab_gt_r10_r6_r7_ROR_16[] = {
  0x77, 0xa8, 0xa6, 0xc6 // sxtab gt r10 r6 r7 ROR 16
};
const byte kInstruction_sxtab_pl_r7_r13_r4_ROR_8[] = {
  0x74, 0x74, 0xad, 0x56 // sxtab pl r7 r13 r4 ROR 8
};
const byte kInstruction_sxtab_ne_r4_r8_r1_ROR_0[] = {
  0x71, 0x40, 0xa8, 0x16 // sxtab ne r4 r8 r1 ROR 0
};
const byte kInstruction_sxtab_mi_r5_r6_r9_ROR_16[] = {
  0x79, 0x58, 0xa6, 0x46 // sxtab mi r5 r6 r9 ROR 16
};
const byte kInstruction_sxtab_mi_r14_r8_r3_ROR_16[] = {
  0x73, 0xe8, 0xa8, 0x46 // sxtab mi r14 r8 r3 ROR 16
};
const byte kInstruction_sxtab_lt_r1_r14_r14_ROR_16[] = {
  0x7e, 0x18, 0xae, 0xb6 // sxtab lt r1 r14 r14 ROR 16
};
const byte kInstruction_sxtab_le_r9_r12_r9_ROR_8[] = {
  0x79, 0x94, 0xac, 0xd6 // sxtab le r9 r12 r9 ROR 8
};
const byte kInstruction_sxtab_le_r10_r2_r2_ROR_16[] = {
  0x72, 0xa8, 0xa2, 0xd6 // sxtab le r10 r2 r2 ROR 16
};
const byte kInstruction_sxtab_vc_r11_r12_r0_ROR_16[] = {
  0x70, 0xb8, 0xac, 0x76 // sxtab vc r11 r12 r0 ROR 16
};
const byte kInstruction_sxtab_vc_r5_r12_r1_ROR_16[] = {
  0x71, 0x58, 0xac, 0x76 // sxtab vc r5 r12 r1 ROR 16
};
const byte kInstruction_sxtab_hi_r2_r9_r2_ROR_0[] = {
  0x72, 0x20, 0xa9, 0x86 // sxtab hi r2 r9 r2 ROR 0
};
const byte kInstruction_sxtab_vs_r11_r7_r14_ROR_0[] = {
  0x7e, 0xb0, 0xa7, 0x66 // sxtab vs r11 r7 r14 ROR 0
};
const byte kInstruction_sxtab_eq_r11_r2_r14_ROR_8[] = {
  0x7e, 0xb4, 0xa2, 0x06 // sxtab eq r11 r2 r14 ROR 8
};
const byte kInstruction_sxtab_cs_r11_r2_r3_ROR_16[] = {
  0x73, 0xb8, 0xa2, 0x26 // sxtab cs r11 r2 r3 ROR 16
};
const byte kInstruction_sxtab_vc_r1_r9_r7_ROR_16[] = {
  0x77, 0x18, 0xa9, 0x76 // sxtab vc r1 r9 r7 ROR 16
};
const byte kInstruction_sxtab_mi_r14_r11_r9_ROR_16[] = {
  0x79, 0xe8, 0xab, 0x46 // sxtab mi r14 r11 r9 ROR 16
};
const byte kInstruction_sxtab_ge_r11_r8_r12_ROR_16[] = {
  0x7c, 0xb8, 0xa8, 0xa6 // sxtab ge r11 r8 r12 ROR 16
};
const byte kInstruction_sxtab_vs_r11_r12_r11_ROR_16[] = {
  0x7b, 0xb8, 0xac, 0x66 // sxtab vs r11 r12 r11 ROR 16
};
const byte kInstruction_sxtab_al_r6_r5_r14_ROR_8[] = {
  0x7e, 0x64, 0xa5, 0xe6 // sxtab al r6 r5 r14 ROR 8
};
const byte kInstruction_sxtab_mi_r9_r8_r0_ROR_24[] = {
  0x70, 0x9c, 0xa8, 0x46 // sxtab mi r9 r8 r0 ROR 24
};
const byte kInstruction_sxtab_vs_r4_r2_r11_ROR_24[] = {
  0x7b, 0x4c, 0xa2, 0x66 // sxtab vs r4 r2 r11 ROR 24
};
const byte kInstruction_sxtab_ne_r5_r7_r5_ROR_8[] = {
  0x75, 0x54, 0xa7, 0x16 // sxtab ne r5 r7 r5 ROR 8
};
const byte kInstruction_sxtab_cs_r2_r4_r2_ROR_0[] = {
  0x72, 0x20, 0xa4, 0x26 // sxtab cs r2 r4 r2 ROR 0
};
const byte kInstruction_sxtab_ne_r10_r4_r8_ROR_0[] = {
  0x78, 0xa0, 0xa4, 0x16 // sxtab ne r10 r4 r8 ROR 0
};
const byte kInstruction_sxtab_vs_r4_r7_r0_ROR_16[] = {
  0x70, 0x48, 0xa7, 0x66 // sxtab vs r4 r7 r0 ROR 16
};
const byte kInstruction_sxtab_gt_r1_r1_r2_ROR_24[] = {
  0x72, 0x1c, 0xa1, 0xc6 // sxtab gt r1 r1 r2 ROR 24
};
const byte kInstruction_sxtab_pl_r4_r13_r10_ROR_24[] = {
  0x7a, 0x4c, 0xad, 0x56 // sxtab pl r4 r13 r10 ROR 24
};
const byte kInstruction_sxtab_vs_r2_r3_r8_ROR_0[] = {
  0x78, 0x20, 0xa3, 0x66 // sxtab vs r2 r3 r8 ROR 0
};
const byte kInstruction_sxtab_lt_r10_r0_r10_ROR_16[] = {
  0x7a, 0xa8, 0xa0, 0xb6 // sxtab lt r10 r0 r10 ROR 16
};
const byte kInstruction_sxtab_cs_r7_r12_r4_ROR_0[] = {
  0x74, 0x70, 0xac, 0x26 // sxtab cs r7 r12 r4 ROR 0
};
const byte kInstruction_sxtab_lt_r4_r6_r5_ROR_8[] = {
  0x75, 0x44, 0xa6, 0xb6 // sxtab lt r4 r6 r5 ROR 8
};
const byte kInstruction_sxtab_pl_r14_r11_r2_ROR_24[] = {
  0x72, 0xec, 0xab, 0x56 // sxtab pl r14 r11 r2 ROR 24
};
const byte kInstruction_sxtab_ge_r10_r7_r6_ROR_0[] = {
  0x76, 0xa0, 0xa7, 0xa6 // sxtab ge r10 r7 r6 ROR 0
};
const byte kInstruction_sxtab_pl_r8_r5_r10_ROR_24[] = {
  0x7a, 0x8c, 0xa5, 0x56 // sxtab pl r8 r5 r10 ROR 24
};
const byte kInstruction_sxtab_mi_r2_r5_r5_ROR_0[] = {
  0x75, 0x20, 0xa5, 0x46 // sxtab mi r2 r5 r5 ROR 0
};
const byte kInstruction_sxtab_mi_r6_r13_r2_ROR_24[] = {
  0x72, 0x6c, 0xad, 0x46 // sxtab mi r6 r13 r2 ROR 24
};
const byte kInstruction_sxtab_vs_r5_r8_r4_ROR_8[] = {
  0x74, 0x54, 0xa8, 0x66 // sxtab vs r5 r8 r4 ROR 8
};
const byte kInstruction_sxtab_mi_r14_r13_r10_ROR_24[] = {
  0x7a, 0xec, 0xad, 0x46 // sxtab mi r14 r13 r10 ROR 24
};
const byte kInstruction_sxtab_vs_r12_r0_r7_ROR_24[] = {
  0x77, 0xcc, 0xa0, 0x66 // sxtab vs r12 r0 r7 ROR 24
};
const byte kInstruction_sxtab_lt_r2_r11_r1_ROR_24[] = {
  0x71, 0x2c, 0xab, 0xb6 // sxtab lt r2 r11 r1 ROR 24
};
const byte kInstruction_sxtab_ls_r8_r1_r2_ROR_16[] = {
  0x72, 0x88, 0xa1, 0x96 // sxtab ls r8 r1 r2 ROR 16
};
const byte kInstruction_sxtab_vs_r9_r14_r7_ROR_16[] = {
  0x77, 0x98, 0xae, 0x66 // sxtab vs r9 r14 r7 ROR 16
};
const byte kInstruction_sxtab_ls_r12_r1_r14_ROR_16[] = {
  0x7e, 0xc8, 0xa1, 0x96 // sxtab ls r12 r1 r14 ROR 16
};
const byte kInstruction_sxtab_le_r8_r12_r7_ROR_24[] = {
  0x77, 0x8c, 0xac, 0xd6 // sxtab le r8 r12 r7 ROR 24
};
const byte kInstruction_sxtab_pl_r6_r6_r0_ROR_8[] = {
  0x70, 0x64, 0xa6, 0x56 // sxtab pl r6 r6 r0 ROR 8
};
const byte kInstruction_sxtab_ne_r5_r13_r6_ROR_24[] = {
  0x76, 0x5c, 0xad, 0x16 // sxtab ne r5 r13 r6 ROR 24
};
const byte kInstruction_sxtab_lt_r3_r10_r5_ROR_16[] = {
  0x75, 0x38, 0xaa, 0xb6 // sxtab lt r3 r10 r5 ROR 16
};
const byte kInstruction_sxtab_eq_r6_r11_r7_ROR_16[] = {
  0x77, 0x68, 0xab, 0x06 // sxtab eq r6 r11 r7 ROR 16
};
const byte kInstruction_sxtab_cs_r10_r13_r4_ROR_0[] = {
  0x74, 0xa0, 0xad, 0x26 // sxtab cs r10 r13 r4 ROR 0
};
const byte kInstruction_sxtab_le_r13_r13_r0_ROR_24[] = {
  0x70, 0xdc, 0xad, 0xd6 // sxtab le r13 r13 r0 ROR 24
};
const byte kInstruction_sxtab_vc_r4_r12_r9_ROR_0[] = {
  0x79, 0x40, 0xac, 0x76 // sxtab vc r4 r12 r9 ROR 0
};
const byte kInstruction_sxtab_gt_r3_r2_r4_ROR_16[] = {
  0x74, 0x38, 0xa2, 0xc6 // sxtab gt r3 r2 r4 ROR 16
};
const byte kInstruction_sxtab_cs_r14_r11_r9_ROR_8[] = {
  0x79, 0xe4, 0xab, 0x26 // sxtab cs r14 r11 r9 ROR 8
};
const byte kInstruction_sxtab_lt_r14_r1_r13_ROR_16[] = {
  0x7d, 0xe8, 0xa1, 0xb6 // sxtab lt r14 r1 r13 ROR 16
};
const byte kInstruction_sxtab_cs_r6_r4_r12_ROR_8[] = {
  0x7c, 0x64, 0xa4, 0x26 // sxtab cs r6 r4 r12 ROR 8
};
const byte kInstruction_sxtab_lt_r10_r13_r5_ROR_8[] = {
  0x75, 0xa4, 0xad, 0xb6 // sxtab lt r10 r13 r5 ROR 8
};
const byte kInstruction_sxtab_mi_r6_r12_r12_ROR_0[] = {
  0x7c, 0x60, 0xac, 0x46 // sxtab mi r6 r12 r12 ROR 0
};
const byte kInstruction_sxtab_ge_r12_r1_r12_ROR_16[] = {
  0x7c, 0xc8, 0xa1, 0xa6 // sxtab ge r12 r1 r12 ROR 16
};
const byte kInstruction_sxtab_hi_r7_r11_r13_ROR_8[] = {
  0x7d, 0x74, 0xab, 0x86 // sxtab hi r7 r11 r13 ROR 8
};
const byte kInstruction_sxtab_vs_r3_r11_r11_ROR_0[] = {
  0x7b, 0x30, 0xab, 0x66 // sxtab vs r3 r11 r11 ROR 0
};
const byte kInstruction_sxtab_vc_r6_r12_r12_ROR_0[] = {
  0x7c, 0x60, 0xac, 0x76 // sxtab vc r6 r12 r12 ROR 0
};
const byte kInstruction_sxtab_gt_r10_r0_r13_ROR_8[] = {
  0x7d, 0xa4, 0xa0, 0xc6 // sxtab gt r10 r0 r13 ROR 8
};
const byte kInstruction_sxtab_pl_r4_r7_r5_ROR_16[] = {
  0x75, 0x48, 0xa7, 0x56 // sxtab pl r4 r7 r5 ROR 16
};
const byte kInstruction_sxtab_cc_r11_r4_r13_ROR_0[] = {
  0x7d, 0xb0, 0xa4, 0x36 // sxtab cc r11 r4 r13 ROR 0
};
const byte kInstruction_sxtab_gt_r14_r13_r13_ROR_24[] = {
  0x7d, 0xec, 0xad, 0xc6 // sxtab gt r14 r13 r13 ROR 24
};
const byte kInstruction_sxtab_lt_r7_r11_r11_ROR_8[] = {
  0x7b, 0x74, 0xab, 0xb6 // sxtab lt r7 r11 r11 ROR 8
};
const byte kInstruction_sxtab_ge_r12_r7_r0_ROR_16[] = {
  0x70, 0xc8, 0xa7, 0xa6 // sxtab ge r12 r7 r0 ROR 16
};
const byte kInstruction_sxtab_ne_r10_r12_r13_ROR_0[] = {
  0x7d, 0xa0, 0xac, 0x16 // sxtab ne r10 r12 r13 ROR 0
};
const byte kInstruction_sxtab_gt_r12_r9_r0_ROR_8[] = {
  0x70, 0xc4, 0xa9, 0xc6 // sxtab gt r12 r9 r0 ROR 8
};
const byte kInstruction_sxtab_pl_r10_r14_r6_ROR_16[] = {
  0x76, 0xa8, 0xae, 0x56 // sxtab pl r10 r14 r6 ROR 16
};
const byte kInstruction_sxtab_vc_r6_r5_r2_ROR_24[] = {
  0x72, 0x6c, 0xa5, 0x76 // sxtab vc r6 r5 r2 ROR 24
};
const byte kInstruction_sxtab_gt_r12_r7_r5_ROR_16[] = {
  0x75, 0xc8, 0xa7, 0xc6 // sxtab gt r12 r7 r5 ROR 16
};
const byte kInstruction_sxtab_eq_r11_r9_r0_ROR_16[] = {
  0x70, 0xb8, 0xa9, 0x06 // sxtab eq r11 r9 r0 ROR 16
};
const byte kInstruction_sxtab_lt_r7_r7_r4_ROR_16[] = {
  0x74, 0x78, 0xa7, 0xb6 // sxtab lt r7 r7 r4 ROR 16
};
const byte kInstruction_sxtab_gt_r5_r4_r10_ROR_8[] = {
  0x7a, 0x54, 0xa4, 0xc6 // sxtab gt r5 r4 r10 ROR 8
};
const byte kInstruction_sxtab_hi_r10_r2_r13_ROR_16[] = {
  0x7d, 0xa8, 0xa2, 0x86 // sxtab hi r10 r2 r13 ROR 16
};
const byte kInstruction_sxtab_al_r13_r5_r5_ROR_24[] = {
  0x75, 0xdc, 0xa5, 0xe6 // sxtab al r13 r5 r5 ROR 24
};
const byte kInstruction_sxtab_cs_r12_r0_r7_ROR_24[] = {
  0x77, 0xcc, 0xa0, 0x26 // sxtab cs r12 r0 r7 ROR 24
};
const byte kInstruction_sxtab_hi_r1_r13_r10_ROR_16[] = {
  0x7a, 0x18, 0xad, 0x86 // sxtab hi r1 r13 r10 ROR 16
};
const byte kInstruction_sxtab_eq_r13_r8_r10_ROR_8[] = {
  0x7a, 0xd4, 0xa8, 0x06 // sxtab eq r13 r8 r10 ROR 8
};
const byte kInstruction_sxtab_le_r6_r8_r3_ROR_8[] = {
  0x73, 0x64, 0xa8, 0xd6 // sxtab le r6 r8 r3 ROR 8
};
const byte kInstruction_sxtab_lt_r13_r12_r12_ROR_0[] = {
  0x7c, 0xd0, 0xac, 0xb6 // sxtab lt r13 r12 r12 ROR 0
};
const byte kInstruction_sxtab_ls_r9_r10_r9_ROR_0[] = {
  0x79, 0x90, 0xaa, 0x96 // sxtab ls r9 r10 r9 ROR 0
};
const byte kInstruction_sxtab_eq_r1_r3_r13_ROR_16[] = {
  0x7d, 0x18, 0xa3, 0x06 // sxtab eq r1 r3 r13 ROR 16
};
const byte kInstruction_sxtab_vs_r12_r12_r9_ROR_8[] = {
  0x79, 0xc4, 0xac, 0x66 // sxtab vs r12 r12 r9 ROR 8
};
const byte kInstruction_sxtab_vc_r7_r8_r7_ROR_8[] = {
  0x77, 0x74, 0xa8, 0x76 // sxtab vc r7 r8 r7 ROR 8
};
const byte kInstruction_sxtab_eq_r6_r0_r14_ROR_24[] = {
  0x7e, 0x6c, 0xa0, 0x06 // sxtab eq r6 r0 r14 ROR 24
};
const byte kInstruction_sxtab_le_r0_r13_r10_ROR_8[] = {
  0x7a, 0x04, 0xad, 0xd6 // sxtab le r0 r13 r10 ROR 8
};
const byte kInstruction_sxtab_ge_r11_r6_r3_ROR_8[] = {
  0x73, 0xb4, 0xa6, 0xa6 // sxtab ge r11 r6 r3 ROR 8
};
const byte kInstruction_sxtab_gt_r2_r2_r8_ROR_0[] = {
  0x78, 0x20, 0xa2, 0xc6 // sxtab gt r2 r2 r8 ROR 0
};
const byte kInstruction_sxtab_le_r6_r4_r0_ROR_8[] = {
  0x70, 0x64, 0xa4, 0xd6 // sxtab le r6 r4 r0 ROR 8
};
const byte kInstruction_sxtab_le_r2_r8_r12_ROR_16[] = {
  0x7c, 0x28, 0xa8, 0xd6 // sxtab le r2 r8 r12 ROR 16
};
const byte kInstruction_sxtab_le_r10_r1_r4_ROR_24[] = {
  0x74, 0xac, 0xa1, 0xd6 // sxtab le r10 r1 r4 ROR 24
};
const byte kInstruction_sxtab_cc_r4_r1_r10_ROR_16[] = {
  0x7a, 0x48, 0xa1, 0x36 // sxtab cc r4 r1 r10 ROR 16
};
const byte kInstruction_sxtab_pl_r9_r4_r9_ROR_16[] = {
  0x79, 0x98, 0xa4, 0x56 // sxtab pl r9 r4 r9 ROR 16
};
const byte kInstruction_sxtab_mi_r8_r4_r12_ROR_0[] = {
  0x7c, 0x80, 0xa4, 0x46 // sxtab mi r8 r4 r12 ROR 0
};
const byte kInstruction_sxtab_lt_r5_r0_r10_ROR_24[] = {
  0x7a, 0x5c, 0xa0, 0xb6 // sxtab lt r5 r0 r10 ROR 24
};
const byte kInstruction_sxtab_hi_r5_r0_r8_ROR_0[] = {
  0x78, 0x50, 0xa0, 0x86 // sxtab hi r5 r0 r8 ROR 0
};
const byte kInstruction_sxtab_cs_r4_r9_r3_ROR_8[] = {
  0x73, 0x44, 0xa9, 0x26 // sxtab cs r4 r9 r3 ROR 8
};
const byte kInstruction_sxtab_vs_r2_r5_r1_ROR_8[] = {
  0x71, 0x24, 0xa5, 0x66 // sxtab vs r2 r5 r1 ROR 8
};
const byte kInstruction_sxtab_ne_r10_r13_r9_ROR_8[] = {
  0x79, 0xa4, 0xad, 0x16 // sxtab ne r10 r13 r9 ROR 8
};
const byte kInstruction_sxtab_cs_r0_r5_r11_ROR_8[] = {
  0x7b, 0x04, 0xa5, 0x26 // sxtab cs r0 r5 r11 ROR 8
};
const byte kInstruction_sxtab_mi_r8_r9_r8_ROR_8[] = {
  0x78, 0x84, 0xa9, 0x46 // sxtab mi r8 r9 r8 ROR 8
};
const byte kInstruction_sxtab_al_r3_r14_r2_ROR_0[] = {
  0x72, 0x30, 0xae, 0xe6 // sxtab al r3 r14 r2 ROR 0
};
const byte kInstruction_sxtab_hi_r9_r10_r9_ROR_24[] = {
  0x79, 0x9c, 0xaa, 0x86 // sxtab hi r9 r10 r9 ROR 24
};
const byte kInstruction_sxtab_lt_r14_r3_r1_ROR_8[] = {
  0x71, 0xe4, 0xa3, 0xb6 // sxtab lt r14 r3 r1 ROR 8
};
const byte kInstruction_sxtab_pl_r11_r12_r14_ROR_16[] = {
  0x7e, 0xb8, 0xac, 0x56 // sxtab pl r11 r12 r14 ROR 16
};
const byte kInstruction_sxtab_eq_r12_r1_r6_ROR_0[] = {
  0x76, 0xc0, 0xa1, 0x06 // sxtab eq r12 r1 r6 ROR 0
};
const byte kInstruction_sxtab_ls_r9_r12_r1_ROR_8[] = {
  0x71, 0x94, 0xac, 0x96 // sxtab ls r9 r12 r1 ROR 8
};
const byte kInstruction_sxtab_pl_r8_r6_r10_ROR_24[] = {
  0x7a, 0x8c, 0xa6, 0x56 // sxtab pl r8 r6 r10 ROR 24
};
const byte kInstruction_sxtab_pl_r1_r10_r14_ROR_8[] = {
  0x7e, 0x14, 0xaa, 0x56 // sxtab pl r1 r10 r14 ROR 8
};
const byte kInstruction_sxtab_mi_r10_r7_r12_ROR_0[] = {
  0x7c, 0xa0, 0xa7, 0x46 // sxtab mi r10 r7 r12 ROR 0
};
const byte kInstruction_sxtab_al_r9_r2_r7_ROR_24[] = {
  0x77, 0x9c, 0xa2, 0xe6 // sxtab al r9 r2 r7 ROR 24
};
const byte kInstruction_sxtab_ls_r2_r9_r8_ROR_16[] = {
  0x78, 0x28, 0xa9, 0x96 // sxtab ls r2 r9 r8 ROR 16
};
const byte kInstruction_sxtab_ls_r5_r12_r11_ROR_16[] = {
  0x7b, 0x58, 0xac, 0x96 // sxtab ls r5 r12 r11 ROR 16
};
const byte kInstruction_sxtab_mi_r9_r10_r11_ROR_24[] = {
  0x7b, 0x9c, 0xaa, 0x46 // sxtab mi r9 r10 r11 ROR 24
};
const byte kInstruction_sxtab_mi_r10_r2_r10_ROR_8[] = {
  0x7a, 0xa4, 0xa2, 0x46 // sxtab mi r10 r2 r10 ROR 8
};
const byte kInstruction_sxtab_gt_r7_r5_r4_ROR_24[] = {
  0x74, 0x7c, 0xa5, 0xc6 // sxtab gt r7 r5 r4 ROR 24
};
const byte kInstruction_sxtab_le_r11_r9_r6_ROR_16[] = {
  0x76, 0xb8, 0xa9, 0xd6 // sxtab le r11 r9 r6 ROR 16
};
const byte kInstruction_sxtab_ne_r13_r2_r10_ROR_16[] = {
  0x7a, 0xd8, 0xa2, 0x16 // sxtab ne r13 r2 r10 ROR 16
};
const byte kInstruction_sxtab_ge_r8_r8_r13_ROR_0[] = {
  0x7d, 0x80, 0xa8, 0xa6 // sxtab ge r8 r8 r13 ROR 0
};
const byte kInstruction_sxtab_al_r0_r14_r11_ROR_8[] = {
  0x7b, 0x04, 0xae, 0xe6 // sxtab al r0 r14 r11 ROR 8
};
const byte kInstruction_sxtab_mi_r3_r5_r14_ROR_0[] = {
  0x7e, 0x30, 0xa5, 0x46 // sxtab mi r3 r5 r14 ROR 0
};
const byte kInstruction_sxtab_ne_r3_r12_r10_ROR_16[] = {
  0x7a, 0x38, 0xac, 0x16 // sxtab ne r3 r12 r10 ROR 16
};
const byte kInstruction_sxtab_le_r4_r14_r12_ROR_0[] = {
  0x7c, 0x40, 0xae, 0xd6 // sxtab le r4 r14 r12 ROR 0
};
const byte kInstruction_sxtab_vs_r14_r9_r6_ROR_16[] = {
  0x76, 0xe8, 0xa9, 0x66 // sxtab vs r14 r9 r6 ROR 16
};
const byte kInstruction_sxtab_cs_r9_r3_r11_ROR_8[] = {
  0x7b, 0x94, 0xa3, 0x26 // sxtab cs r9 r3 r11 ROR 8
};
const byte kInstruction_sxtab_le_r12_r14_r13_ROR_8[] = {
  0x7d, 0xc4, 0xae, 0xd6 // sxtab le r12 r14 r13 ROR 8
};
const byte kInstruction_sxtab_eq_r6_r12_r5_ROR_24[] = {
  0x75, 0x6c, 0xac, 0x06 // sxtab eq r6 r12 r5 ROR 24
};
const byte kInstruction_sxtab_cc_r2_r11_r11_ROR_8[] = {
  0x7b, 0x24, 0xab, 0x36 // sxtab cc r2 r11 r11 ROR 8
};
const byte kInstruction_sxtab_vs_r9_r3_r7_ROR_0[] = {
  0x77, 0x90, 0xa3, 0x66 // sxtab vs r9 r3 r7 ROR 0
};
const byte kInstruction_sxtab_le_r5_r6_r11_ROR_8[] = {
  0x7b, 0x54, 0xa6, 0xd6 // sxtab le r5 r6 r11 ROR 8
};
const byte kInstruction_sxtab_cc_r5_r14_r9_ROR_16[] = {
  0x79, 0x58, 0xae, 0x36 // sxtab cc r5 r14 r9 ROR 16
};
const byte kInstruction_sxtab_ne_r11_r14_r4_ROR_24[] = {
  0x74, 0xbc, 0xae, 0x16 // sxtab ne r11 r14 r4 ROR 24
};
const byte kInstruction_sxtab_gt_r0_r14_r13_ROR_16[] = {
  0x7d, 0x08, 0xae, 0xc6 // sxtab gt r0 r14 r13 ROR 16
};
const byte kInstruction_sxtab_cc_r0_r12_r13_ROR_24[] = {
  0x7d, 0x0c, 0xac, 0x36 // sxtab cc r0 r12 r13 ROR 24
};
const byte kInstruction_sxtab_vs_r14_r11_r8_ROR_24[] = {
  0x78, 0xec, 0xab, 0x66 // sxtab vs r14 r11 r8 ROR 24
};
const byte kInstruction_sxtab_ge_r12_r4_r6_ROR_0[] = {
  0x76, 0xc0, 0xa4, 0xa6 // sxtab ge r12 r4 r6 ROR 0
};
const byte kInstruction_sxtab_gt_r4_r0_r4_ROR_0[] = {
  0x74, 0x40, 0xa0, 0xc6 // sxtab gt r4 r0 r4 ROR 0
};
const byte kInstruction_sxtab_hi_r2_r3_r6_ROR_0[] = {
  0x76, 0x20, 0xa3, 0x86 // sxtab hi r2 r3 r6 ROR 0
};
const byte kInstruction_sxtab_gt_r3_r13_r6_ROR_0[] = {
  0x76, 0x30, 0xad, 0xc6 // sxtab gt r3 r13 r6 ROR 0
};
const byte kInstruction_sxtab_cc_r2_r5_r10_ROR_0[] = {
  0x7a, 0x20, 0xa5, 0x36 // sxtab cc r2 r5 r10 ROR 0
};
const byte kInstruction_sxtab_ne_r3_r10_r9_ROR_8[] = {
  0x79, 0x34, 0xaa, 0x16 // sxtab ne r3 r10 r9 ROR 8
};
const byte kInstruction_sxtab_al_r8_r4_r12_ROR_8[] = {
  0x7c, 0x84, 0xa4, 0xe6 // sxtab al r8 r4 r12 ROR 8
};
const byte kInstruction_sxtab_cc_r14_r0_r2_ROR_0[] = {
  0x72, 0xe0, 0xa0, 0x36 // sxtab cc r14 r0 r2 ROR 0
};
const byte kInstruction_sxtab_hi_r5_r5_r12_ROR_24[] = {
  0x7c, 0x5c, 0xa5, 0x86 // sxtab hi r5 r5 r12 ROR 24
};
const byte kInstruction_sxtab_cc_r0_r8_r8_ROR_8[] = {
  0x78, 0x04, 0xa8, 0x36 // sxtab cc r0 r8 r8 ROR 8
};
const byte kInstruction_sxtab_vc_r0_r9_r12_ROR_24[] = {
  0x7c, 0x0c, 0xa9, 0x76 // sxtab vc r0 r9 r12 ROR 24
};
const byte kInstruction_sxtab_vc_r8_r2_r11_ROR_16[] = {
  0x7b, 0x88, 0xa2, 0x76 // sxtab vc r8 r2 r11 ROR 16
};
const byte kInstruction_sxtab_ne_r13_r13_r7_ROR_24[] = {
  0x77, 0xdc, 0xad, 0x16 // sxtab ne r13 r13 r7 ROR 24
};
const byte kInstruction_sxtab_al_r4_r4_r1_ROR_0[] = {
  0x71, 0x40, 0xa4, 0xe6 // sxtab al r4 r4 r1 ROR 0
};
const byte kInstruction_sxtab_cc_r1_r9_r10_ROR_16[] = {
  0x7a, 0x18, 0xa9, 0x36 // sxtab cc r1 r9 r10 ROR 16
};
const byte kInstruction_sxtab_cs_r2_r0_r5_ROR_24[] = {
  0x75, 0x2c, 0xa0, 0x26 // sxtab cs r2 r0 r5 ROR 24
};
const byte kInstruction_sxtab_hi_r11_r11_r9_ROR_24[] = {
  0x79, 0xbc, 0xab, 0x86 // sxtab hi r11 r11 r9 ROR 24
};
const byte kInstruction_sxtab_ls_r12_r2_r7_ROR_24[] = {
  0x77, 0xcc, 0xa2, 0x96 // sxtab ls r12 r2 r7 ROR 24
};
const byte kInstruction_sxtab_le_r4_r2_r6_ROR_16[] = {
  0x76, 0x48, 0xa2, 0xd6 // sxtab le r4 r2 r6 ROR 16
};
const byte kInstruction_sxtab_ls_r8_r0_r14_ROR_8[] = {
  0x7e, 0x84, 0xa0, 0x96 // sxtab ls r8 r0 r14 ROR 8
};
const byte kInstruction_sxtab_ne_r5_r9_r0_ROR_8[] = {
  0x70, 0x54, 0xa9, 0x16 // sxtab ne r5 r9 r0 ROR 8
};
const byte kInstruction_sxtab_ge_r7_r8_r14_ROR_8[] = {
  0x7e, 0x74, 0xa8, 0xa6 // sxtab ge r7 r8 r14 ROR 8
};
const byte kInstruction_sxtab_ne_r14_r6_r13_ROR_8[] = {
  0x7d, 0xe4, 0xa6, 0x16 // sxtab ne r14 r6 r13 ROR 8
};
const byte kInstruction_sxtab_cs_r13_r2_r9_ROR_0[] = {
  0x79, 0xd0, 0xa2, 0x26 // sxtab cs r13 r2 r9 ROR 0
};
const byte kInstruction_sxtab_hi_r9_r0_r0_ROR_8[] = {
  0x70, 0x94, 0xa0, 0x86 // sxtab hi r9 r0 r0 ROR 8
};
const byte kInstruction_sxtab_vs_r12_r9_r3_ROR_16[] = {
  0x73, 0xc8, 0xa9, 0x66 // sxtab vs r12 r9 r3 ROR 16
};
const byte kInstruction_sxtab_ne_r2_r14_r8_ROR_24[] = {
  0x78, 0x2c, 0xae, 0x16 // sxtab ne r2 r14 r8 ROR 24
};
const byte kInstruction_sxtab_le_r13_r14_r2_ROR_8[] = {
  0x72, 0xd4, 0xae, 0xd6 // sxtab le r13 r14 r2 ROR 8
};
const byte kInstruction_sxtab_gt_r9_r7_r12_ROR_16[] = {
  0x7c, 0x98, 0xa7, 0xc6 // sxtab gt r9 r7 r12 ROR 16
};
const byte kInstruction_sxtab_ne_r2_r13_r7_ROR_16[] = {
  0x77, 0x28, 0xad, 0x16 // sxtab ne r2 r13 r7 ROR 16
};
const byte kInstruction_sxtab_pl_r10_r11_r13_ROR_24[] = {
  0x7d, 0xac, 0xab, 0x56 // sxtab pl r10 r11 r13 ROR 24
};
const byte kInstruction_sxtab_mi_r5_r3_r8_ROR_24[] = {
  0x78, 0x5c, 0xa3, 0x46 // sxtab mi r5 r3 r8 ROR 24
};
const byte kInstruction_sxtab_le_r12_r8_r2_ROR_16[] = {
  0x72, 0xc8, 0xa8, 0xd6 // sxtab le r12 r8 r2 ROR 16
};
const byte kInstruction_sxtab_ls_r4_r10_r9_ROR_8[] = {
  0x79, 0x44, 0xaa, 0x96 // sxtab ls r4 r10 r9 ROR 8
};
const byte kInstruction_sxtab_eq_r6_r11_r8_ROR_24[] = {
  0x78, 0x6c, 0xab, 0x06 // sxtab eq r6 r11 r8 ROR 24
};
const byte kInstruction_sxtab_le_r11_r12_r5_ROR_24[] = {
  0x75, 0xbc, 0xac, 0xd6 // sxtab le r11 r12 r5 ROR 24
};
const byte kInstruction_sxtab_gt_r0_r10_r0_ROR_8[] = {
  0x70, 0x04, 0xaa, 0xc6 // sxtab gt r0 r10 r0 ROR 8
};
const byte kInstruction_sxtab_pl_r9_r4_r9_ROR_0[] = {
  0x79, 0x90, 0xa4, 0x56 // sxtab pl r9 r4 r9 ROR 0
};
const byte kInstruction_sxtab_vc_r14_r2_r8_ROR_24[] = {
  0x78, 0xec, 0xa2, 0x76 // sxtab vc r14 r2 r8 ROR 24
};
const byte kInstruction_sxtab_gt_r7_r0_r7_ROR_0[] = {
  0x77, 0x70, 0xa0, 0xc6 // sxtab gt r7 r0 r7 ROR 0
};
const byte kInstruction_sxtab_vc_r12_r12_r9_ROR_0[] = {
  0x79, 0xc0, 0xac, 0x76 // sxtab vc r12 r12 r9 ROR 0
};
const byte kInstruction_sxtab_pl_r2_r12_r3_ROR_0[] = {
  0x73, 0x20, 0xac, 0x56 // sxtab pl r2 r12 r3 ROR 0
};
const byte kInstruction_sxtab_cs_r10_r2_r0_ROR_16[] = {
  0x70, 0xa8, 0xa2, 0x26 // sxtab cs r10 r2 r0 ROR 16
};
const byte kInstruction_sxtab_vs_r6_r5_r10_ROR_16[] = {
  0x7a, 0x68, 0xa5, 0x66 // sxtab vs r6 r5 r10 ROR 16
};
const byte kInstruction_sxtab_pl_r5_r14_r6_ROR_24[] = {
  0x76, 0x5c, 0xae, 0x56 // sxtab pl r5 r14 r6 ROR 24
};
const byte kInstruction_sxtab_al_r12_r5_r9_ROR_16[] = {
  0x79, 0xc8, 0xa5, 0xe6 // sxtab al r12 r5 r9 ROR 16
};
const byte kInstruction_sxtab_ge_r3_r0_r3_ROR_24[] = {
  0x73, 0x3c, 0xa0, 0xa6 // sxtab ge r3 r0 r3 ROR 24
};
const byte kInstruction_sxtab_le_r4_r11_r12_ROR_8[] = {
  0x7c, 0x44, 0xab, 0xd6 // sxtab le r4 r11 r12 ROR 8
};
const byte kInstruction_sxtab_pl_r10_r10_r13_ROR_16[] = {
  0x7d, 0xa8, 0xaa, 0x56 // sxtab pl r10 r10 r13 ROR 16
};
const byte kInstruction_sxtab_al_r1_r8_r3_ROR_8[] = {
  0x73, 0x14, 0xa8, 0xe6 // sxtab al r1 r8 r3 ROR 8
};
const byte kInstruction_sxtab_cc_r8_r6_r12_ROR_16[] = {
  0x7c, 0x88, 0xa6, 0x36 // sxtab cc r8 r6 r12 ROR 16
};
const byte kInstruction_sxtab_vs_r13_r5_r6_ROR_16[] = {
  0x76, 0xd8, 0xa5, 0x66 // sxtab vs r13 r5 r6 ROR 16
};
const byte kInstruction_sxtab_eq_r10_r6_r2_ROR_8[] = {
  0x72, 0xa4, 0xa6, 0x06 // sxtab eq r10 r6 r2 ROR 8
};
const byte kInstruction_sxtab_gt_r3_r3_r9_ROR_24[] = {
  0x79, 0x3c, 0xa3, 0xc6 // sxtab gt r3 r3 r9 ROR 24
};
const byte kInstruction_sxtab_hi_r4_r6_r9_ROR_0[] = {
  0x79, 0x40, 0xa6, 0x86 // sxtab hi r4 r6 r9 ROR 0
};
const byte kInstruction_sxtab_vs_r7_r6_r8_ROR_16[] = {
  0x78, 0x78, 0xa6, 0x66 // sxtab vs r7 r6 r8 ROR 16
};
const byte kInstruction_sxtab_hi_r6_r11_r0_ROR_24[] = {
  0x70, 0x6c, 0xab, 0x86 // sxtab hi r6 r11 r0 ROR 24
};
const byte kInstruction_sxtab_ge_r2_r6_r4_ROR_24[] = {
  0x74, 0x2c, 0xa6, 0xa6 // sxtab ge r2 r6 r4 ROR 24
};
const byte kInstruction_sxtab_pl_r5_r2_r10_ROR_16[] = {
  0x7a, 0x58, 0xa2, 0x56 // sxtab pl r5 r2 r10 ROR 16
};
const byte kInstruction_sxtab_hi_r3_r13_r5_ROR_16[] = {
  0x75, 0x38, 0xad, 0x86 // sxtab hi r3 r13 r5 ROR 16
};
const byte kInstruction_sxtab_lt_r6_r7_r7_ROR_24[] = {
  0x77, 0x6c, 0xa7, 0xb6 // sxtab lt r6 r7 r7 ROR 24
};
const byte kInstruction_sxtab_ne_r13_r9_r10_ROR_24[] = {
  0x7a, 0xdc, 0xa9, 0x16 // sxtab ne r13 r9 r10 ROR 24
};
const byte kInstruction_sxtab_mi_r6_r0_r11_ROR_0[] = {
  0x7b, 0x60, 0xa0, 0x46 // sxtab mi r6 r0 r11 ROR 0
};
const byte kInstruction_sxtab_ge_r9_r3_r3_ROR_24[] = {
  0x73, 0x9c, 0xa3, 0xa6 // sxtab ge r9 r3 r3 ROR 24
};
const byte kInstruction_sxtab_ls_r10_r11_r3_ROR_16[] = {
  0x73, 0xa8, 0xab, 0x96 // sxtab ls r10 r11 r3 ROR 16
};
const byte kInstruction_sxtab_al_r13_r13_r7_ROR_8[] = {
  0x77, 0xd4, 0xad, 0xe6 // sxtab al r13 r13 r7 ROR 8
};
const byte kInstruction_sxtab_lt_r9_r11_r2_ROR_16[] = {
  0x72, 0x98, 0xab, 0xb6 // sxtab lt r9 r11 r2 ROR 16
};
const byte kInstruction_sxtab_ls_r10_r9_r8_ROR_24[] = {
  0x78, 0xac, 0xa9, 0x96 // sxtab ls r10 r9 r8 ROR 24
};
const byte kInstruction_sxtab_cs_r6_r12_r9_ROR_8[] = {
  0x79, 0x64, 0xac, 0x26 // sxtab cs r6 r12 r9 ROR 8
};
const byte kInstruction_sxtab_le_r14_r5_r0_ROR_0[] = {
  0x70, 0xe0, 0xa5, 0xd6 // sxtab le r14 r5 r0 ROR 0
};
const byte kInstruction_sxtab_le_r8_r1_r1_ROR_8[] = {
  0x71, 0x84, 0xa1, 0xd6 // sxtab le r8 r1 r1 ROR 8
};
const byte kInstruction_sxtab_hi_r2_r0_r3_ROR_8[] = {
  0x73, 0x24, 0xa0, 0x86 // sxtab hi r2 r0 r3 ROR 8
};
const byte kInstruction_sxtab_ne_r4_r8_r1_ROR_8[] = {
  0x71, 0x44, 0xa8, 0x16 // sxtab ne r4 r8 r1 ROR 8
};
const byte kInstruction_sxtab_al_r13_r8_r9_ROR_8[] = {
  0x79, 0xd4, 0xa8, 0xe6 // sxtab al r13 r8 r9 ROR 8
};
const byte kInstruction_sxtab_mi_r4_r10_r13_ROR_16[] = {
  0x7d, 0x48, 0xaa, 0x46 // sxtab mi r4 r10 r13 ROR 16
};
const byte kInstruction_sxtab_cc_r10_r5_r5_ROR_24[] = {
  0x75, 0xac, 0xa5, 0x36 // sxtab cc r10 r5 r5 ROR 24
};
const byte kInstruction_sxtab_ls_r0_r12_r3_ROR_24[] = {
  0x73, 0x0c, 0xac, 0x96 // sxtab ls r0 r12 r3 ROR 24
};
const byte kInstruction_sxtab_vc_r4_r11_r11_ROR_24[] = {
  0x7b, 0x4c, 0xab, 0x76 // sxtab vc r4 r11 r11 ROR 24
};
const byte kInstruction_sxtab_hi_r5_r12_r0_ROR_24[] = {
  0x70, 0x5c, 0xac, 0x86 // sxtab hi r5 r12 r0 ROR 24
};
const byte kInstruction_sxtab_le_r7_r3_r9_ROR_0[] = {
  0x79, 0x70, 0xa3, 0xd6 // sxtab le r7 r3 r9 ROR 0
};
const byte kInstruction_sxtab_vs_r14_r7_r9_ROR_24[] = {
  0x79, 0xec, 0xa7, 0x66 // sxtab vs r14 r7 r9 ROR 24
};
const byte kInstruction_sxtab_ne_r2_r2_r9_ROR_24[] = {
  0x79, 0x2c, 0xa2, 0x16 // sxtab ne r2 r2 r9 ROR 24
};
const byte kInstruction_sxtab_ls_r5_r7_r11_ROR_16[] = {
  0x7b, 0x58, 0xa7, 0x96 // sxtab ls r5 r7 r11 ROR 16
};
const byte kInstruction_sxtab_ls_r0_r8_r4_ROR_0[] = {
  0x74, 0x00, 0xa8, 0x96 // sxtab ls r0 r8 r4 ROR 0
};
const byte kInstruction_sxtab_hi_r5_r13_r0_ROR_0[] = {
  0x70, 0x50, 0xad, 0x86 // sxtab hi r5 r13 r0 ROR 0
};
const byte kInstruction_sxtab_mi_r7_r12_r7_ROR_8[] = {
  0x77, 0x74, 0xac, 0x46 // sxtab mi r7 r12 r7 ROR 8
};
const byte kInstruction_sxtab_ne_r3_r1_r3_ROR_0[] = {
  0x73, 0x30, 0xa1, 0x16 // sxtab ne r3 r1 r3 ROR 0
};
const byte kInstruction_sxtab_ne_r9_r11_r8_ROR_16[] = {
  0x78, 0x98, 0xab, 0x16 // sxtab ne r9 r11 r8 ROR 16
};
const byte kInstruction_sxtab_ge_r12_r4_r14_ROR_0[] = {
  0x7e, 0xc0, 0xa4, 0xa6 // sxtab ge r12 r4 r14 ROR 0
};
const byte kInstruction_sxtab_ne_r11_r1_r4_ROR_24[] = {
  0x74, 0xbc, 0xa1, 0x16 // sxtab ne r11 r1 r4 ROR 24
};
const byte kInstruction_sxtab_ge_r9_r9_r13_ROR_0[] = {
  0x7d, 0x90, 0xa9, 0xa6 // sxtab ge r9 r9 r13 ROR 0
};
const byte kInstruction_sxtab_gt_r13_r9_r14_ROR_16[] = {
  0x7e, 0xd8, 0xa9, 0xc6 // sxtab gt r13 r9 r14 ROR 16
};
const byte kInstruction_sxtab_vs_r10_r0_r12_ROR_16[] = {
  0x7c, 0xa8, 0xa0, 0x66 // sxtab vs r10 r0 r12 ROR 16
};
const byte kInstruction_sxtab_pl_r10_r7_r1_ROR_24[] = {
  0x71, 0xac, 0xa7, 0x56 // sxtab pl r10 r7 r1 ROR 24
};
const byte kInstruction_sxtab_al_r14_r12_r10_ROR_0[] = {
  0x7a, 0xe0, 0xac, 0xe6 // sxtab al r14 r12 r10 ROR 0
};
const byte kInstruction_sxtab_cc_r10_r8_r10_ROR_8[] = {
  0x7a, 0xa4, 0xa8, 0x36 // sxtab cc r10 r8 r10 ROR 8
};
const byte kInstruction_sxtab_eq_r14_r2_r14_ROR_16[] = {
  0x7e, 0xe8, 0xa2, 0x06 // sxtab eq r14 r2 r14 ROR 16
};
const byte kInstruction_sxtab_vc_r3_r12_r8_ROR_16[] = {
  0x78, 0x38, 0xac, 0x76 // sxtab vc r3 r12 r8 ROR 16
};
const byte kInstruction_sxtab_gt_r13_r4_r7_ROR_0[] = {
  0x77, 0xd0, 0xa4, 0xc6 // sxtab gt r13 r4 r7 ROR 0
};
const byte kInstruction_sxtab_ne_r12_r6_r11_ROR_16[] = {
  0x7b, 0xc8, 0xa6, 0x16 // sxtab ne r12 r6 r11 ROR 16
};
const byte kInstruction_sxtab_vs_r1_r7_r12_ROR_0[] = {
  0x7c, 0x10, 0xa7, 0x66 // sxtab vs r1 r7 r12 ROR 0
};
const byte kInstruction_sxtab_hi_r14_r3_r7_ROR_0[] = {
  0x77, 0xe0, 0xa3, 0x86 // sxtab hi r14 r3 r7 ROR 0
};
const byte kInstruction_sxtab_ne_r4_r2_r0_ROR_16[] = {
  0x70, 0x48, 0xa2, 0x16 // sxtab ne r4 r2 r0 ROR 16
};
const byte kInstruction_sxtab_cs_r11_r3_r11_ROR_24[] = {
  0x7b, 0xbc, 0xa3, 0x26 // sxtab cs r11 r3 r11 ROR 24
};
const byte kInstruction_sxtab_le_r10_r2_r0_ROR_24[] = {
  0x70, 0xac, 0xa2, 0xd6 // sxtab le r10 r2 r0 ROR 24
};
const byte kInstruction_sxtab_vs_r8_r11_r0_ROR_8[] = {
  0x70, 0x84, 0xab, 0x66 // sxtab vs r8 r11 r0 ROR 8
};
const byte kInstruction_sxtab_le_r12_r7_r7_ROR_8[] = {
  0x77, 0xc4, 0xa7, 0xd6 // sxtab le r12 r7 r7 ROR 8
};
const byte kInstruction_sxtab_eq_r8_r6_r13_ROR_16[] = {
  0x7d, 0x88, 0xa6, 0x06 // sxtab eq r8 r6 r13 ROR 16
};
const byte kInstruction_sxtab_hi_r1_r12_r7_ROR_24[] = {
  0x77, 0x1c, 0xac, 0x86 // sxtab hi r1 r12 r7 ROR 24
};
const byte kInstruction_sxtab_gt_r9_r7_r14_ROR_8[] = {
  0x7e, 0x94, 0xa7, 0xc6 // sxtab gt r9 r7 r14 ROR 8
};
const byte kInstruction_sxtab_pl_r10_r2_r7_ROR_8[] = {
  0x77, 0xa4, 0xa2, 0x56 // sxtab pl r10 r2 r7 ROR 8
};
const byte kInstruction_sxtab_cc_r0_r12_r3_ROR_0[] = {
  0x73, 0x00, 0xac, 0x36 // sxtab cc r0 r12 r3 ROR 0
};
const byte kInstruction_sxtab_ge_r3_r0_r0_ROR_8[] = {
  0x70, 0x34, 0xa0, 0xa6 // sxtab ge r3 r0 r0 ROR 8
};
const byte kInstruction_sxtab_lt_r10_r14_r1_ROR_16[] = {
  0x71, 0xa8, 0xae, 0xb6 // sxtab lt r10 r14 r1 ROR 16
};
const byte kInstruction_sxtab_vc_r0_r0_r13_ROR_8[] = {
  0x7d, 0x04, 0xa0, 0x76 // sxtab vc r0 r0 r13 ROR 8
};
const byte kInstruction_sxtab_vc_r2_r12_r0_ROR_8[] = {
  0x70, 0x24, 0xac, 0x76 // sxtab vc r2 r12 r0 ROR 8
};
const byte kInstruction_sxtab_gt_r1_r7_r7_ROR_24[] = {
  0x77, 0x1c, 0xa7, 0xc6 // sxtab gt r1 r7 r7 ROR 24
};
const byte kInstruction_sxtab_cc_r8_r12_r0_ROR_24[] = {
  0x70, 0x8c, 0xac, 0x36 // sxtab cc r8 r12 r0 ROR 24
};
const byte kInstruction_sxtab_vs_r0_r10_r12_ROR_8[] = {
  0x7c, 0x04, 0xaa, 0x66 // sxtab vs r0 r10 r12 ROR 8
};
const byte kInstruction_sxtab_le_r0_r1_r11_ROR_8[] = {
  0x7b, 0x04, 0xa1, 0xd6 // sxtab le r0 r1 r11 ROR 8
};
const byte kInstruction_sxtab_vs_r9_r12_r9_ROR_8[] = {
  0x79, 0x94, 0xac, 0x66 // sxtab vs r9 r12 r9 ROR 8
};
const byte kInstruction_sxtab_mi_r7_r7_r13_ROR_24[] = {
  0x7d, 0x7c, 0xa7, 0x46 // sxtab mi r7 r7 r13 ROR 24
};
const byte kInstruction_sxtab_cc_r1_r4_r3_ROR_0[] = {
  0x73, 0x10, 0xa4, 0x36 // sxtab cc r1 r4 r3 ROR 0
};
const byte kInstruction_sxtab_pl_r1_r12_r4_ROR_16[] = {
  0x74, 0x18, 0xac, 0x56 // sxtab pl r1 r12 r4 ROR 16
};
const byte kInstruction_sxtab_mi_r4_r9_r13_ROR_24[] = {
  0x7d, 0x4c, 0xa9, 0x46 // sxtab mi r4 r9 r13 ROR 24
};
const byte kInstruction_sxtab_eq_r12_r14_r14_ROR_16[] = {
  0x7e, 0xc8, 0xae, 0x06 // sxtab eq r12 r14 r14 ROR 16
};
const byte kInstruction_sxtab_ls_r10_r0_r4_ROR_0[] = {
  0x74, 0xa0, 0xa0, 0x96 // sxtab ls r10 r0 r4 ROR 0
};
const byte kInstruction_sxtab_hi_r3_r6_r12_ROR_8[] = {
  0x7c, 0x34, 0xa6, 0x86 // sxtab hi r3 r6 r12 ROR 8
};
const byte kInstruction_sxtab_le_r10_r0_r4_ROR_8[] = {
  0x74, 0xa4, 0xa0, 0xd6 // sxtab le r10 r0 r4 ROR 8
};
const byte kInstruction_sxtab_lt_r12_r3_r2_ROR_8[] = {
  0x72, 0xc4, 0xa3, 0xb6 // sxtab lt r12 r3 r2 ROR 8
};
const byte kInstruction_sxtab_pl_r13_r11_r10_ROR_16[] = {
  0x7a, 0xd8, 0xab, 0x56 // sxtab pl r13 r11 r10 ROR 16
};
const byte kInstruction_sxtab_ne_r13_r14_r14_ROR_24[] = {
  0x7e, 0xdc, 0xae, 0x16 // sxtab ne r13 r14 r14 ROR 24
};
const byte kInstruction_sxtab_eq_r9_r9_r0_ROR_8[] = {
  0x70, 0x94, 0xa9, 0x06 // sxtab eq r9 r9 r0 ROR 8
};
const byte kInstruction_sxtab_eq_r12_r10_r0_ROR_8[] = {
  0x70, 0xc4, 0xaa, 0x06 // sxtab eq r12 r10 r0 ROR 8
};
const byte kInstruction_sxtab_ne_r0_r0_r3_ROR_0[] = {
  0x73, 0x00, 0xa0, 0x16 // sxtab ne r0 r0 r3 ROR 0
};
const byte kInstruction_sxtab_gt_r12_r14_r4_ROR_0[] = {
  0x74, 0xc0, 0xae, 0xc6 // sxtab gt r12 r14 r4 ROR 0
};
const byte kInstruction_sxtab_pl_r1_r2_r14_ROR_0[] = {
  0x7e, 0x10, 0xa2, 0x56 // sxtab pl r1 r2 r14 ROR 0
};
const byte kInstruction_sxtab_lt_r1_r8_r8_ROR_8[] = {
  0x78, 0x14, 0xa8, 0xb6 // sxtab lt r1 r8 r8 ROR 8
};
const byte kInstruction_sxtab_cc_r9_r0_r2_ROR_16[] = {
  0x72, 0x98, 0xa0, 0x36 // sxtab cc r9 r0 r2 ROR 16
};
const byte kInstruction_sxtab_cs_r9_r0_r5_ROR_8[] = {
  0x75, 0x94, 0xa0, 0x26 // sxtab cs r9 r0 r5 ROR 8
};
const byte kInstruction_sxtab_mi_r13_r8_r5_ROR_24[] = {
  0x75, 0xdc, 0xa8, 0x46 // sxtab mi r13 r8 r5 ROR 24
};
const byte kInstruction_sxtab_vc_r1_r1_r7_ROR_8[] = {
  0x77, 0x14, 0xa1, 0x76 // sxtab vc r1 r1 r7 ROR 8
};
const byte kInstruction_sxtab_cs_r6_r5_r5_ROR_16[] = {
  0x75, 0x68, 0xa5, 0x26 // sxtab cs r6 r5 r5 ROR 16
};
const byte kInstruction_sxtab_le_r1_r13_r3_ROR_24[] = {
  0x73, 0x1c, 0xad, 0xd6 // sxtab le r1 r13 r3 ROR 24
};
const byte kInstruction_sxtab_gt_r11_r7_r3_ROR_24[] = {
  0x73, 0xbc, 0xa7, 0xc6 // sxtab gt r11 r7 r3 ROR 24
};
const byte kInstruction_sxtab_al_r3_r3_r5_ROR_0[] = {
  0x75, 0x30, 0xa3, 0xe6 // sxtab al r3 r3 r5 ROR 0
};
const byte kInstruction_sxtab_hi_r5_r7_r14_ROR_24[] = {
  0x7e, 0x5c, 0xa7, 0x86 // sxtab hi r5 r7 r14 ROR 24
};
const byte kInstruction_sxtab_ne_r1_r9_r4_ROR_24[] = {
  0x74, 0x1c, 0xa9, 0x16 // sxtab ne r1 r9 r4 ROR 24
};
const byte kInstruction_sxtab_hi_r7_r3_r11_ROR_8[] = {
  0x7b, 0x74, 0xa3, 0x86 // sxtab hi r7 r3 r11 ROR 8
};
const byte kInstruction_sxtab_al_r1_r8_r8_ROR_8[] = {
  0x78, 0x14, 0xa8, 0xe6 // sxtab al r1 r8 r8 ROR 8
};
const byte kInstruction_sxtab_hi_r3_r8_r1_ROR_16[] = {
  0x71, 0x38, 0xa8, 0x86 // sxtab hi r3 r8 r1 ROR 16
};
const byte kInstruction_sxtab_ge_r9_r9_r10_ROR_16[] = {
  0x7a, 0x98, 0xa9, 0xa6 // sxtab ge r9 r9 r10 ROR 16
};
const byte kInstruction_sxtab_le_r11_r1_r0_ROR_24[] = {
  0x70, 0xbc, 0xa1, 0xd6 // sxtab le r11 r1 r0 ROR 24
};
const byte kInstruction_sxtab_ls_r13_r12_r9_ROR_0[] = {
  0x79, 0xd0, 0xac, 0x96 // sxtab ls r13 r12 r9 ROR 0
};
const byte kInstruction_sxtab_hi_r8_r6_r7_ROR_24[] = {
  0x77, 0x8c, 0xa6, 0x86 // sxtab hi r8 r6 r7 ROR 24
};
const byte kInstruction_sxtab_lt_r8_r13_r11_ROR_16[] = {
  0x7b, 0x88, 0xad, 0xb6 // sxtab lt r8 r13 r11 ROR 16
};
const byte kInstruction_sxtab_ge_r3_r9_r9_ROR_0[] = {
  0x79, 0x30, 0xa9, 0xa6 // sxtab ge r3 r9 r9 ROR 0
};
const byte kInstruction_sxtab_ne_r5_r7_r4_ROR_24[] = {
  0x74, 0x5c, 0xa7, 0x16 // sxtab ne r5 r7 r4 ROR 24
};
const byte kInstruction_sxtab_al_r3_r12_r2_ROR_24[] = {
  0x72, 0x3c, 0xac, 0xe6 // sxtab al r3 r12 r2 ROR 24
};
const byte kInstruction_sxtab_ne_r0_r4_r13_ROR_0[] = {
  0x7d, 0x00, 0xa4, 0x16 // sxtab ne r0 r4 r13 ROR 0
};
const byte kInstruction_sxtab_ge_r14_r8_r9_ROR_0[] = {
  0x79, 0xe0, 0xa8, 0xa6 // sxtab ge r14 r8 r9 ROR 0
};
const byte kInstruction_sxtab_ne_r9_r13_r12_ROR_16[] = {
  0x7c, 0x98, 0xad, 0x16 // sxtab ne r9 r13 r12 ROR 16
};
const byte kInstruction_sxtab_pl_r11_r1_r4_ROR_24[] = {
  0x74, 0xbc, 0xa1, 0x56 // sxtab pl r11 r1 r4 ROR 24
};
const byte kInstruction_sxtab_ne_r2_r12_r10_ROR_16[] = {
  0x7a, 0x28, 0xac, 0x16 // sxtab ne r2 r12 r10 ROR 16
};
const byte kInstruction_sxtab_gt_r1_r4_r11_ROR_0[] = {
  0x7b, 0x10, 0xa4, 0xc6 // sxtab gt r1 r4 r11 ROR 0
};
const byte kInstruction_sxtab_lt_r7_r6_r13_ROR_16[] = {
  0x7d, 0x78, 0xa6, 0xb6 // sxtab lt r7 r6 r13 ROR 16
};
const byte kInstruction_sxtab_hi_r2_r4_r6_ROR_8[] = {
  0x76, 0x24, 0xa4, 0x86 // sxtab hi r2 r4 r6 ROR 8
};
const byte kInstruction_sxtab_al_r12_r0_r9_ROR_0[] = {
  0x79, 0xc0, 0xa0, 0xe6 // sxtab al r12 r0 r9 ROR 0
};
const byte kInstruction_sxtab_lt_r0_r14_r3_ROR_24[] = {
  0x73, 0x0c, 0xae, 0xb6 // sxtab lt r0 r14 r3 ROR 24
};
const byte kInstruction_sxtab_mi_r5_r8_r10_ROR_16[] = {
  0x7a, 0x58, 0xa8, 0x46 // sxtab mi r5 r8 r10 ROR 16
};
const byte kInstruction_sxtab_vc_r4_r1_r4_ROR_8[] = {
  0x74, 0x44, 0xa1, 0x76 // sxtab vc r4 r1 r4 ROR 8
};
const byte kInstruction_sxtab_cc_r3_r4_r8_ROR_24[] = {
  0x78, 0x3c, 0xa4, 0x36 // sxtab cc r3 r4 r8 ROR 24
};
const byte kInstruction_sxtab_cs_r11_r0_r0_ROR_16[] = {
  0x70, 0xb8, 0xa0, 0x26 // sxtab cs r11 r0 r0 ROR 16
};
const byte kInstruction_sxtab_le_r9_r2_r11_ROR_0[] = {
  0x7b, 0x90, 0xa2, 0xd6 // sxtab le r9 r2 r11 ROR 0
};
const byte kInstruction_sxtab_vc_r10_r7_r8_ROR_8[] = {
  0x78, 0xa4, 0xa7, 0x76 // sxtab vc r10 r7 r8 ROR 8
};
const byte kInstruction_sxtab_le_r0_r3_r1_ROR_24[] = {
  0x71, 0x0c, 0xa3, 0xd6 // sxtab le r0 r3 r1 ROR 24
};
const byte kInstruction_sxtab_ne_r4_r8_r2_ROR_0[] = {
  0x72, 0x40, 0xa8, 0x16 // sxtab ne r4 r8 r2 ROR 0
};
const byte kInstruction_sxtab_gt_r6_r4_r13_ROR_0[] = {
  0x7d, 0x60, 0xa4, 0xc6 // sxtab gt r6 r4 r13 ROR 0
};
const byte kInstruction_sxtab_ge_r10_r5_r11_ROR_8[] = {
  0x7b, 0xa4, 0xa5, 0xa6 // sxtab ge r10 r5 r11 ROR 8
};
const byte kInstruction_sxtab_ge_r11_r14_r5_ROR_24[] = {
  0x75, 0xbc, 0xae, 0xa6 // sxtab ge r11 r14 r5 ROR 24
};
const byte kInstruction_sxtab_gt_r14_r8_r3_ROR_8[] = {
  0x73, 0xe4, 0xa8, 0xc6 // sxtab gt r14 r8 r3 ROR 8
};
const byte kInstruction_sxtab_al_r2_r7_r3_ROR_24[] = {
  0x73, 0x2c, 0xa7, 0xe6 // sxtab al r2 r7 r3 ROR 24
};
const byte kInstruction_sxtab_le_r5_r3_r4_ROR_8[] = {
  0x74, 0x54, 0xa3, 0xd6 // sxtab le r5 r3 r4 ROR 8
};
const byte kInstruction_sxtab_ls_r14_r7_r2_ROR_16[] = {
  0x72, 0xe8, 0xa7, 0x96 // sxtab ls r14 r7 r2 ROR 16
};
const byte kInstruction_sxtab_vc_r11_r7_r0_ROR_24[] = {
  0x70, 0xbc, 0xa7, 0x76 // sxtab vc r11 r7 r0 ROR 24
};
const byte kInstruction_sxtab_al_r8_r6_r12_ROR_8[] = {
  0x7c, 0x84, 0xa6, 0xe6 // sxtab al r8 r6 r12 ROR 8
};
const byte kInstruction_sxtab_ne_r3_r7_r11_ROR_16[] = {
  0x7b, 0x38, 0xa7, 0x16 // sxtab ne r3 r7 r11 ROR 16
};
const byte kInstruction_sxtab_ls_r3_r0_r9_ROR_24[] = {
  0x79, 0x3c, 0xa0, 0x96 // sxtab ls r3 r0 r9 ROR 24
};
const byte kInstruction_sxtab_vs_r7_r12_r11_ROR_8[] = {
  0x7b, 0x74, 0xac, 0x66 // sxtab vs r7 r12 r11 ROR 8
};
const byte kInstruction_sxtab_ge_r2_r5_r2_ROR_16[] = {
  0x72, 0x28, 0xa5, 0xa6 // sxtab ge r2 r5 r2 ROR 16
};
const byte kInstruction_sxtab_vs_r14_r14_r4_ROR_24[] = {
  0x74, 0xec, 0xae, 0x66 // sxtab vs r14 r14 r4 ROR 24
};
const byte kInstruction_sxtab_pl_r3_r8_r12_ROR_24[] = {
  0x7c, 0x3c, 0xa8, 0x56 // sxtab pl r3 r8 r12 ROR 24
};
const byte kInstruction_sxtab_hi_r1_r6_r3_ROR_16[] = {
  0x73, 0x18, 0xa6, 0x86 // sxtab hi r1 r6 r3 ROR 16
};
const byte kInstruction_sxtab_pl_r2_r5_r11_ROR_24[] = {
  0x7b, 0x2c, 0xa5, 0x56 // sxtab pl r2 r5 r11 ROR 24
};
const byte kInstruction_sxtab_vs_r5_r2_r3_ROR_0[] = {
  0x73, 0x50, 0xa2, 0x66 // sxtab vs r5 r2 r3 ROR 0
};
const byte kInstruction_sxtab_al_r3_r14_r10_ROR_0[] = {
  0x7a, 0x30, 0xae, 0xe6 // sxtab al r3 r14 r10 ROR 0
};
const byte kInstruction_sxtab_cs_r12_r8_r1_ROR_24[] = {
  0x71, 0xcc, 0xa8, 0x26 // sxtab cs r12 r8 r1 ROR 24
};
const byte kInstruction_sxtab_mi_r4_r12_r7_ROR_8[] = {
  0x77, 0x44, 0xac, 0x46 // sxtab mi r4 r12 r7 ROR 8
};
const byte kInstruction_sxtab_ls_r12_r2_r5_ROR_8[] = {
  0x75, 0xc4, 0xa2, 0x96 // sxtab ls r12 r2 r5 ROR 8
};
const byte kInstruction_sxtab_ge_r7_r11_r9_ROR_0[] = {
  0x79, 0x70, 0xab, 0xa6 // sxtab ge r7 r11 r9 ROR 0
};
const byte kInstruction_sxtab_lt_r4_r12_r14_ROR_24[] = {
  0x7e, 0x4c, 0xac, 0xb6 // sxtab lt r4 r12 r14 ROR 24
};
const byte kInstruction_sxtab_vs_r5_r11_r2_ROR_8[] = {
  0x72, 0x54, 0xab, 0x66 // sxtab vs r5 r11 r2 ROR 8
};
const byte kInstruction_sxtab_lt_r6_r6_r2_ROR_16[] = {
  0x72, 0x68, 0xa6, 0xb6 // sxtab lt r6 r6 r2 ROR 16
};
const byte kInstruction_sxtab_lt_r10_r10_r14_ROR_8[] = {
  0x7e, 0xa4, 0xaa, 0xb6 // sxtab lt r10 r10 r14 ROR 8
};
const byte kInstruction_sxtab_gt_r6_r5_r5_ROR_24[] = {
  0x75, 0x6c, 0xa5, 0xc6 // sxtab gt r6 r5 r5 ROR 24
};
const byte kInstruction_sxtab_ls_r13_r14_r4_ROR_24[] = {
  0x74, 0xdc, 0xae, 0x96 // sxtab ls r13 r14 r4 ROR 24
};
const byte kInstruction_sxtab_ls_r12_r13_r10_ROR_24[] = {
  0x7a, 0xcc, 0xad, 0x96 // sxtab ls r12 r13 r10 ROR 24
};
const byte kInstruction_sxtab_pl_r8_r1_r8_ROR_16[] = {
  0x78, 0x88, 0xa1, 0x56 // sxtab pl r8 r1 r8 ROR 16
};
const byte kInstruction_sxtab_ge_r12_r2_r2_ROR_24[] = {
  0x72, 0xcc, 0xa2, 0xa6 // sxtab ge r12 r2 r2 ROR 24
};
const byte kInstruction_sxtab_pl_r6_r13_r13_ROR_8[] = {
  0x7d, 0x64, 0xad, 0x56 // sxtab pl r6 r13 r13 ROR 8
};
const byte kInstruction_sxtab_pl_r9_r10_r6_ROR_8[] = {
  0x76, 0x94, 0xaa, 0x56 // sxtab pl r9 r10 r6 ROR 8
};
const byte kInstruction_sxtab_hi_r0_r2_r1_ROR_16[] = {
  0x71, 0x08, 0xa2, 0x86 // sxtab hi r0 r2 r1 ROR 16
};
const byte kInstruction_sxtab_pl_r13_r1_r8_ROR_8[] = {
  0x78, 0xd4, 0xa1, 0x56 // sxtab pl r13 r1 r8 ROR 8
};
const byte kInstruction_sxtab_le_r9_r13_r3_ROR_8[] = {
  0x73, 0x94, 0xad, 0xd6 // sxtab le r9 r13 r3 ROR 8
};
const byte kInstruction_sxtab_ne_r8_r5_r8_ROR_16[] = {
  0x78, 0x88, 0xa5, 0x16 // sxtab ne r8 r5 r8 ROR 16
};
const byte kInstruction_sxtab_pl_r1_r9_r5_ROR_24[] = {
  0x75, 0x1c, 0xa9, 0x56 // sxtab pl r1 r9 r5 ROR 24
};
const byte kInstruction_sxtab_hi_r1_r9_r5_ROR_16[] = {
  0x75, 0x18, 0xa9, 0x86 // sxtab hi r1 r9 r5 ROR 16
};
const byte kInstruction_sxtab_mi_r3_r7_r12_ROR_16[] = {
  0x7c, 0x38, 0xa7, 0x46 // sxtab mi r3 r7 r12 ROR 16
};
const byte kInstruction_sxtab_al_r0_r10_r11_ROR_8[] = {
  0x7b, 0x04, 0xaa, 0xe6 // sxtab al r0 r10 r11 ROR 8
};
const byte kInstruction_sxtab_lt_r14_r2_r13_ROR_16[] = {
  0x7d, 0xe8, 0xa2, 0xb6 // sxtab lt r14 r2 r13 ROR 16
};
const byte kInstruction_sxtab_al_r8_r7_r0_ROR_0[] = {
  0x70, 0x80, 0xa7, 0xe6 // sxtab al r8 r7 r0 ROR 0
};
const byte kInstruction_sxtab_mi_r13_r14_r9_ROR_0[] = {
  0x79, 0xd0, 0xae, 0x46 // sxtab mi r13 r14 r9 ROR 0
};
const byte kInstruction_sxtab_lt_r12_r13_r1_ROR_16[] = {
  0x71, 0xc8, 0xad, 0xb6 // sxtab lt r12 r13 r1 ROR 16
};
const byte kInstruction_sxtab_mi_r14_r7_r2_ROR_16[] = {
  0x72, 0xe8, 0xa7, 0x46 // sxtab mi r14 r7 r2 ROR 16
};
const byte kInstruction_sxtab_hi_r2_r4_r7_ROR_8[] = {
  0x77, 0x24, 0xa4, 0x86 // sxtab hi r2 r4 r7 ROR 8
};
const byte kInstruction_sxtab_le_r13_r12_r7_ROR_24[] = {
  0x77, 0xdc, 0xac, 0xd6 // sxtab le r13 r12 r7 ROR 24
};
const byte kInstruction_sxtab_mi_r1_r5_r7_ROR_24[] = {
  0x77, 0x1c, 0xa5, 0x46 // sxtab mi r1 r5 r7 ROR 24
};
const byte kInstruction_sxtab_cc_r9_r10_r10_ROR_16[] = {
  0x7a, 0x98, 0xaa, 0x36 // sxtab cc r9 r10 r10 ROR 16
};
const byte kInstruction_sxtab_eq_r8_r8_r11_ROR_24[] = {
  0x7b, 0x8c, 0xa8, 0x06 // sxtab eq r8 r8 r11 ROR 24
};
const byte kInstruction_sxtab_hi_r4_r13_r1_ROR_24[] = {
  0x71, 0x4c, 0xad, 0x86 // sxtab hi r4 r13 r1 ROR 24
};
const byte kInstruction_sxtab_vs_r3_r14_r3_ROR_8[] = {
  0x73, 0x34, 0xae, 0x66 // sxtab vs r3 r14 r3 ROR 8
};
const byte kInstruction_sxtab_lt_r8_r6_r11_ROR_8[] = {
  0x7b, 0x84, 0xa6, 0xb6 // sxtab lt r8 r6 r11 ROR 8
};
const byte kInstruction_sxtab_ls_r5_r11_r1_ROR_8[] = {
  0x71, 0x54, 0xab, 0x96 // sxtab ls r5 r11 r1 ROR 8
};
const byte kInstruction_sxtab_vs_r13_r5_r10_ROR_0[] = {
  0x7a, 0xd0, 0xa5, 0x66 // sxtab vs r13 r5 r10 ROR 0
};
const byte kInstruction_sxtab_al_r9_r5_r4_ROR_0[] = {
  0x74, 0x90, 0xa5, 0xe6 // sxtab al r9 r5 r4 ROR 0
};
const byte kInstruction_sxtab_le_r8_r6_r5_ROR_24[] = {
  0x75, 0x8c, 0xa6, 0xd6 // sxtab le r8 r6 r5 ROR 24
};
const byte kInstruction_sxtab_hi_r3_r14_r10_ROR_8[] = {
  0x7a, 0x34, 0xae, 0x86 // sxtab hi r3 r14 r10 ROR 8
};
const byte kInstruction_sxtab_mi_r9_r2_r11_ROR_16[] = {
  0x7b, 0x98, 0xa2, 0x46 // sxtab mi r9 r2 r11 ROR 16
};
const byte kInstruction_sxtab_cs_r5_r6_r6_ROR_24[] = {
  0x76, 0x5c, 0xa6, 0x26 // sxtab cs r5 r6 r6 ROR 24
};
const byte kInstruction_sxtab_ls_r2_r10_r4_ROR_16[] = {
  0x74, 0x28, 0xaa, 0x96 // sxtab ls r2 r10 r4 ROR 16
};
const byte kInstruction_sxtab_al_r2_r11_r1_ROR_0[] = {
  0x71, 0x20, 0xab, 0xe6 // sxtab al r2 r11 r1 ROR 0
};
const byte kInstruction_sxtab_al_r4_r5_r5_ROR_0[] = {
  0x75, 0x40, 0xa5, 0xe6 // sxtab al r4 r5 r5 ROR 0
};
const byte kInstruction_sxtab_cc_r3_r0_r3_ROR_8[] = {
  0x73, 0x34, 0xa0, 0x36 // sxtab cc r3 r0 r3 ROR 8
};
const byte kInstruction_sxtab_hi_r14_r9_r11_ROR_8[] = {
  0x7b, 0xe4, 0xa9, 0x86 // sxtab hi r14 r9 r11 ROR 8
};
const byte kInstruction_sxtab_cs_r0_r1_r13_ROR_8[] = {
  0x7d, 0x04, 0xa1, 0x26 // sxtab cs r0 r1 r13 ROR 8
};
const byte kInstruction_sxtab_vs_r9_r6_r2_ROR_24[] = {
  0x72, 0x9c, 0xa6, 0x66 // sxtab vs r9 r6 r2 ROR 24
};
const byte kInstruction_sxtab_lt_r9_r12_r2_ROR_0[] = {
  0x72, 0x90, 0xac, 0xb6 // sxtab lt r9 r12 r2 ROR 0
};
const byte kInstruction_sxtab_hi_r14_r12_r2_ROR_24[] = {
  0x72, 0xec, 0xac, 0x86 // sxtab hi r14 r12 r2 ROR 24
};
const byte kInstruction_sxtab_lt_r0_r10_r1_ROR_8[] = {
  0x71, 0x04, 0xaa, 0xb6 // sxtab lt r0 r10 r1 ROR 8
};
const byte kInstruction_sxtab_vs_r0_r12_r0_ROR_24[] = {
  0x70, 0x0c, 0xac, 0x66 // sxtab vs r0 r12 r0 ROR 24
};
const byte kInstruction_sxtab_mi_r4_r2_r9_ROR_16[] = {
  0x79, 0x48, 0xa2, 0x46 // sxtab mi r4 r2 r9 ROR 16
};
const byte kInstruction_sxtab_pl_r5_r4_r13_ROR_24[] = {
  0x7d, 0x5c, 0xa4, 0x56 // sxtab pl r5 r4 r13 ROR 24
};
const byte kInstruction_sxtab_cs_r9_r9_r2_ROR_8[] = {
  0x72, 0x94, 0xa9, 0x26 // sxtab cs r9 r9 r2 ROR 8
};
const byte kInstruction_sxtab_vs_r8_r6_r6_ROR_8[] = {
  0x76, 0x84, 0xa6, 0x66 // sxtab vs r8 r6 r6 ROR 8
};
const byte kInstruction_sxtab_cc_r12_r2_r8_ROR_16[] = {
  0x78, 0xc8, 0xa2, 0x36 // sxtab cc r12 r2 r8 ROR 16
};
const byte kInstruction_sxtab_ge_r4_r0_r11_ROR_0[] = {
  0x7b, 0x40, 0xa0, 0xa6 // sxtab ge r4 r0 r11 ROR 0
};
const byte kInstruction_sxtab_hi_r13_r3_r4_ROR_8[] = {
  0x74, 0xd4, 0xa3, 0x86 // sxtab hi r13 r3 r4 ROR 8
};
const byte kInstruction_sxtab_cs_r5_r8_r9_ROR_24[] = {
  0x79, 0x5c, 0xa8, 0x26 // sxtab cs r5 r8 r9 ROR 24
};
const byte kInstruction_sxtab_cc_r10_r2_r14_ROR_24[] = {
  0x7e, 0xac, 0xa2, 0x36 // sxtab cc r10 r2 r14 ROR 24
};
const byte kInstruction_sxtab_pl_r10_r1_r13_ROR_0[] = {
  0x7d, 0xa0, 0xa1, 0x56 // sxtab pl r10 r1 r13 ROR 0
};
const byte kInstruction_sxtab_ne_r4_r13_r13_ROR_8[] = {
  0x7d, 0x44, 0xad, 0x16 // sxtab ne r4 r13 r13 ROR 8
};
const byte kInstruction_sxtab_pl_r13_r11_r14_ROR_16[] = {
  0x7e, 0xd8, 0xab, 0x56 // sxtab pl r13 r11 r14 ROR 16
};
const byte kInstruction_sxtab_cs_r10_r6_r11_ROR_24[] = {
  0x7b, 0xac, 0xa6, 0x26 // sxtab cs r10 r6 r11 ROR 24
};
const byte kInstruction_sxtab_hi_r10_r14_r14_ROR_16[] = {
  0x7e, 0xa8, 0xae, 0x86 // sxtab hi r10 r14 r14 ROR 16
};
const byte kInstruction_sxtab_eq_r5_r1_r1_ROR_24[] = {
  0x71, 0x5c, 0xa1, 0x06 // sxtab eq r5 r1 r1 ROR 24
};
const byte kInstruction_sxtab_lt_r11_r5_r1_ROR_8[] = {
  0x71, 0xb4, 0xa5, 0xb6 // sxtab lt r11 r5 r1 ROR 8
};
const byte kInstruction_sxtab_gt_r7_r11_r13_ROR_16[] = {
  0x7d, 0x78, 0xab, 0xc6 // sxtab gt r7 r11 r13 ROR 16
};
const byte kInstruction_sxtab_mi_r1_r5_r10_ROR_16[] = {
  0x7a, 0x18, 0xa5, 0x46 // sxtab mi r1 r5 r10 ROR 16
};
const byte kInstruction_sxtab_gt_r10_r1_r13_ROR_0[] = {
  0x7d, 0xa0, 0xa1, 0xc6 // sxtab gt r10 r1 r13 ROR 0
};
const byte kInstruction_sxtab_cs_r14_r2_r13_ROR_0[] = {
  0x7d, 0xe0, 0xa2, 0x26 // sxtab cs r14 r2 r13 ROR 0
};
const byte kInstruction_sxtab_hi_r10_r14_r6_ROR_16[] = {
  0x76, 0xa8, 0xae, 0x86 // sxtab hi r10 r14 r6 ROR 16
};
const byte kInstruction_sxtab_hi_r0_r6_r9_ROR_16[] = {
  0x79, 0x08, 0xa6, 0x86 // sxtab hi r0 r6 r9 ROR 16
};
const byte kInstruction_sxtab_gt_r14_r4_r9_ROR_8[] = {
  0x79, 0xe4, 0xa4, 0xc6 // sxtab gt r14 r4 r9 ROR 8
};
const byte kInstruction_sxtab_mi_r1_r6_r11_ROR_8[] = {
  0x7b, 0x14, 0xa6, 0x46 // sxtab mi r1 r6 r11 ROR 8
};
const byte kInstruction_sxtab_gt_r7_r2_r8_ROR_16[] = {
  0x78, 0x78, 0xa2, 0xc6 // sxtab gt r7 r2 r8 ROR 16
};
const byte kInstruction_sxtab_cc_r6_r9_r0_ROR_8[] = {
  0x70, 0x64, 0xa9, 0x36 // sxtab cc r6 r9 r0 ROR 8
};
const byte kInstruction_sxtab_ls_r5_r3_r3_ROR_16[] = {
  0x73, 0x58, 0xa3, 0x96 // sxtab ls r5 r3 r3 ROR 16
};
const byte kInstruction_sxtab_al_r0_r6_r7_ROR_0[] = {
  0x77, 0x00, 0xa6, 0xe6 // sxtab al r0 r6 r7 ROR 0
};
const byte kInstruction_sxtab_lt_r10_r0_r2_ROR_0[] = {
  0x72, 0xa0, 0xa0, 0xb6 // sxtab lt r10 r0 r2 ROR 0
};
const byte kInstruction_sxtab_le_r13_r8_r0_ROR_0[] = {
  0x70, 0xd0, 0xa8, 0xd6 // sxtab le r13 r8 r0 ROR 0
};
const byte kInstruction_sxtab_al_r0_r8_r0_ROR_24[] = {
  0x70, 0x0c, 0xa8, 0xe6 // sxtab al r0 r8 r0 ROR 24
};
const byte kInstruction_sxtab_hi_r8_r8_r5_ROR_16[] = {
  0x75, 0x88, 0xa8, 0x86 // sxtab hi r8 r8 r5 ROR 16
};
const byte kInstruction_sxtab_mi_r9_r10_r13_ROR_24[] = {
  0x7d, 0x9c, 0xaa, 0x46 // sxtab mi r9 r10 r13 ROR 24
};
const byte kInstruction_sxtab_eq_r13_r12_r4_ROR_24[] = {
  0x74, 0xdc, 0xac, 0x06 // sxtab eq r13 r12 r4 ROR 24
};
const byte kInstruction_sxtab_cs_r1_r14_r14_ROR_16[] = {
  0x7e, 0x18, 0xae, 0x26 // sxtab cs r1 r14 r14 ROR 16
};
const byte kInstruction_sxtab_cs_r8_r6_r11_ROR_0[] = {
  0x7b, 0x80, 0xa6, 0x26 // sxtab cs r8 r6 r11 ROR 0
};
const byte kInstruction_sxtab_cc_r11_r4_r13_ROR_16[] = {
  0x7d, 0xb8, 0xa4, 0x36 // sxtab cc r11 r4 r13 ROR 16
};
const byte kInstruction_sxtab_hi_r3_r4_r5_ROR_0[] = {
  0x75, 0x30, 0xa4, 0x86 // sxtab hi r3 r4 r5 ROR 0
};
const byte kInstruction_sxtab_pl_r7_r3_r9_ROR_16[] = {
  0x79, 0x78, 0xa3, 0x56 // sxtab pl r7 r3 r9 ROR 16
};
const byte kInstruction_sxtab_lt_r5_r1_r0_ROR_16[] = {
  0x70, 0x58, 0xa1, 0xb6 // sxtab lt r5 r1 r0 ROR 16
};
const byte kInstruction_sxtab_gt_r9_r7_r9_ROR_24[] = {
  0x79, 0x9c, 0xa7, 0xc6 // sxtab gt r9 r7 r9 ROR 24
};
const byte kInstruction_sxtab_ls_r7_r14_r5_ROR_24[] = {
  0x75, 0x7c, 0xae, 0x96 // sxtab ls r7 r14 r5 ROR 24
};
const byte kInstruction_sxtab_le_r3_r0_r8_ROR_0[] = {
  0x78, 0x30, 0xa0, 0xd6 // sxtab le r3 r0 r8 ROR 0
};
const byte kInstruction_sxtab_pl_r8_r1_r0_ROR_24[] = {
  0x70, 0x8c, 0xa1, 0x56 // sxtab pl r8 r1 r0 ROR 24
};
const byte kInstruction_sxtab_ge_r3_r13_r14_ROR_0[] = {
  0x7e, 0x30, 0xad, 0xa6 // sxtab ge r3 r13 r14 ROR 0
};
const byte kInstruction_sxtab_gt_r4_r12_r6_ROR_16[] = {
  0x76, 0x48, 0xac, 0xc6 // sxtab gt r4 r12 r6 ROR 16
};
const byte kInstruction_sxtab_cc_r9_r5_r11_ROR_0[] = {
  0x7b, 0x90, 0xa5, 0x36 // sxtab cc r9 r5 r11 ROR 0
};
const byte kInstruction_sxtab_vs_r8_r3_r4_ROR_16[] = {
  0x74, 0x88, 0xa3, 0x66 // sxtab vs r8 r3 r4 ROR 16
};
const byte kInstruction_sxtab_gt_r0_r13_r14_ROR_24[] = {
  0x7e, 0x0c, 0xad, 0xc6 // sxtab gt r0 r13 r14 ROR 24
};
const byte kInstruction_sxtab_vs_r8_r11_r14_ROR_24[] = {
  0x7e, 0x8c, 0xab, 0x66 // sxtab vs r8 r11 r14 ROR 24
};
const byte kInstruction_sxtab_ge_r5_r11_r11_ROR_8[] = {
  0x7b, 0x54, 0xab, 0xa6 // sxtab ge r5 r11 r11 ROR 8
};
const byte kInstruction_sxtab_gt_r14_r2_r9_ROR_8[] = {
  0x79, 0xe4, 0xa2, 0xc6 // sxtab gt r14 r2 r9 ROR 8
};
const byte kInstruction_sxtab_vc_r2_r9_r4_ROR_16[] = {
  0x74, 0x28, 0xa9, 0x76 // sxtab vc r2 r9 r4 ROR 16
};
const byte kInstruction_sxtab_eq_r8_r3_r7_ROR_8[] = {
  0x77, 0x84, 0xa3, 0x06 // sxtab eq r8 r3 r7 ROR 8
};
const byte kInstruction_sxtab_pl_r8_r14_r14_ROR_8[] = {
  0x7e, 0x84, 0xae, 0x56 // sxtab pl r8 r14 r14 ROR 8
};
const byte kInstruction_sxtab_mi_r13_r3_r7_ROR_24[] = {
  0x77, 0xdc, 0xa3, 0x46 // sxtab mi r13 r3 r7 ROR 24
};
const byte kInstruction_sxtab_pl_r1_r12_r4_ROR_0[] = {
  0x74, 0x10, 0xac, 0x56 // sxtab pl r1 r12 r4 ROR 0
};
const byte kInstruction_sxtab_eq_r1_r12_r12_ROR_8[] = {
  0x7c, 0x14, 0xac, 0x06 // sxtab eq r1 r12 r12 ROR 8
};
const byte kInstruction_sxtab_cs_r10_r9_r9_ROR_0[] = {
  0x79, 0xa0, 0xa9, 0x26 // sxtab cs r10 r9 r9 ROR 0
};
const byte kInstruction_sxtab_lt_r14_r12_r1_ROR_8[] = {
  0x71, 0xe4, 0xac, 0xb6 // sxtab lt r14 r12 r1 ROR 8
};
const byte kInstruction_sxtab_hi_r8_r5_r10_ROR_0[] = {
  0x7a, 0x80, 0xa5, 0x86 // sxtab hi r8 r5 r10 ROR 0
};
const byte kInstruction_sxtab_lt_r0_r8_r1_ROR_16[] = {
  0x71, 0x08, 0xa8, 0xb6 // sxtab lt r0 r8 r1 ROR 16
};
const byte kInstruction_sxtab_vc_r13_r4_r9_ROR_24[] = {
  0x79, 0xdc, 0xa4, 0x76 // sxtab vc r13 r4 r9 ROR 24
};
const byte kInstruction_sxtab_cc_r3_r4_r1_ROR_24[] = {
  0x71, 0x3c, 0xa4, 0x36 // sxtab cc r3 r4 r1 ROR 24
};
const byte kInstruction_sxtab_vc_r0_r5_r3_ROR_24[] = {
  0x73, 0x0c, 0xa5, 0x76 // sxtab vc r0 r5 r3 ROR 24
};
const byte kInstruction_sxtab_lt_r2_r7_r3_ROR_8[] = {
  0x73, 0x24, 0xa7, 0xb6 // sxtab lt r2 r7 r3 ROR 8
};
const byte kInstruction_sxtab_pl_r5_r7_r14_ROR_8[] = {
  0x7e, 0x54, 0xa7, 0x56 // sxtab pl r5 r7 r14 ROR 8
};
const byte kInstruction_sxtab_ne_r11_r11_r13_ROR_24[] = {
  0x7d, 0xbc, 0xab, 0x16 // sxtab ne r11 r11 r13 ROR 24
};
const byte kInstruction_sxtab_ge_r6_r14_r8_ROR_24[] = {
  0x78, 0x6c, 0xae, 0xa6 // sxtab ge r6 r14 r8 ROR 24
};
const byte kInstruction_sxtab_le_r6_r7_r1_ROR_0[] = {
  0x71, 0x60, 0xa7, 0xd6 // sxtab le r6 r7 r1 ROR 0
};
const byte kInstruction_sxtab_vc_r12_r4_r10_ROR_0[] = {
  0x7a, 0xc0, 0xa4, 0x76 // sxtab vc r12 r4 r10 ROR 0
};
const byte kInstruction_sxtab_ne_r12_r7_r13_ROR_8[] = {
  0x7d, 0xc4, 0xa7, 0x16 // sxtab ne r12 r7 r13 ROR 8
};
const byte kInstruction_sxtab_cc_r3_r11_r3_ROR_16[] = {
  0x73, 0x38, 0xab, 0x36 // sxtab cc r3 r11 r3 ROR 16
};
const byte kInstruction_sxtab_gt_r0_r2_r5_ROR_0[] = {
  0x75, 0x00, 0xa2, 0xc6 // sxtab gt r0 r2 r5 ROR 0
};
const byte kInstruction_sxtab_ge_r6_r14_r5_ROR_16[] = {
  0x75, 0x68, 0xae, 0xa6 // sxtab ge r6 r14 r5 ROR 16
};
const byte kInstruction_sxtab_eq_r11_r6_r11_ROR_16[] = {
  0x7b, 0xb8, 0xa6, 0x06 // sxtab eq r11 r6 r11 ROR 16
};
const byte kInstruction_sxtab_vc_r11_r7_r14_ROR_8[] = {
  0x7e, 0xb4, 0xa7, 0x76 // sxtab vc r11 r7 r14 ROR 8
};
const byte kInstruction_sxtab_ls_r14_r4_r3_ROR_24[] = {
  0x73, 0xec, 0xa4, 0x96 // sxtab ls r14 r4 r3 ROR 24
};
const byte kInstruction_sxtab_lt_r0_r5_r13_ROR_16[] = {
  0x7d, 0x08, 0xa5, 0xb6 // sxtab lt r0 r5 r13 ROR 16
};
const byte kInstruction_sxtab_hi_r11_r1_r10_ROR_24[] = {
  0x7a, 0xbc, 0xa1, 0x86 // sxtab hi r11 r1 r10 ROR 24
};
const byte kInstruction_sxtab_ls_r7_r7_r2_ROR_16[] = {
  0x72, 0x78, 0xa7, 0x96 // sxtab ls r7 r7 r2 ROR 16
};
const byte kInstruction_sxtab_hi_r1_r2_r6_ROR_0[] = {
  0x76, 0x10, 0xa2, 0x86 // sxtab hi r1 r2 r6 ROR 0
};
const byte kInstruction_sxtab_hi_r2_r0_r11_ROR_8[] = {
  0x7b, 0x24, 0xa0, 0x86 // sxtab hi r2 r0 r11 ROR 8
};
const byte kInstruction_sxtab_lt_r7_r9_r5_ROR_0[] = {
  0x75, 0x70, 0xa9, 0xb6 // sxtab lt r7 r9 r5 ROR 0
};
const byte kInstruction_sxtab_hi_r12_r14_r13_ROR_8[] = {
  0x7d, 0xc4, 0xae, 0x86 // sxtab hi r12 r14 r13 ROR 8
};
const byte kInstruction_sxtab_hi_r4_r6_r2_ROR_8[] = {
  0x72, 0x44, 0xa6, 0x86 // sxtab hi r4 r6 r2 ROR 8
};
const byte kInstruction_sxtab_eq_r3_r6_r10_ROR_16[] = {
  0x7a, 0x38, 0xa6, 0x06 // sxtab eq r3 r6 r10 ROR 16
};
const byte kInstruction_sxtab_al_r10_r13_r6_ROR_16[] = {
  0x76, 0xa8, 0xad, 0xe6 // sxtab al r10 r13 r6 ROR 16
};
const byte kInstruction_sxtab_vs_r3_r12_r10_ROR_24[] = {
  0x7a, 0x3c, 0xac, 0x66 // sxtab vs r3 r12 r10 ROR 24
};
const byte kInstruction_sxtab_le_r1_r4_r14_ROR_24[] = {
  0x7e, 0x1c, 0xa4, 0xd6 // sxtab le r1 r4 r14 ROR 24
};
const byte kInstruction_sxtab_vc_r2_r1_r3_ROR_24[] = {
  0x73, 0x2c, 0xa1, 0x76 // sxtab vc r2 r1 r3 ROR 24
};
const byte kInstruction_sxtab_hi_r11_r3_r2_ROR_24[] = {
  0x72, 0xbc, 0xa3, 0x86 // sxtab hi r11 r3 r2 ROR 24
};
const byte kInstruction_sxtab_al_r6_r11_r13_ROR_8[] = {
  0x7d, 0x64, 0xab, 0xe6 // sxtab al r6 r11 r13 ROR 8
};
const byte kInstruction_sxtab_lt_r5_r13_r12_ROR_16[] = {
  0x7c, 0x58, 0xad, 0xb6 // sxtab lt r5 r13 r12 ROR 16
};
const byte kInstruction_sxtab_cc_r10_r2_r0_ROR_16[] = {
  0x70, 0xa8, 0xa2, 0x36 // sxtab cc r10 r2 r0 ROR 16
};
const byte kInstruction_sxtab_ge_r2_r14_r3_ROR_24[] = {
  0x73, 0x2c, 0xae, 0xa6 // sxtab ge r2 r14 r3 ROR 24
};
const byte kInstruction_sxtab_gt_r14_r0_r9_ROR_8[] = {
  0x79, 0xe4, 0xa0, 0xc6 // sxtab gt r14 r0 r9 ROR 8
};
const byte kInstruction_sxtab_ne_r7_r5_r12_ROR_8[] = {
  0x7c, 0x74, 0xa5, 0x16 // sxtab ne r7 r5 r12 ROR 8
};
const byte kInstruction_sxtab_gt_r2_r13_r13_ROR_16[] = {
  0x7d, 0x28, 0xad, 0xc6 // sxtab gt r2 r13 r13 ROR 16
};
const byte kInstruction_sxtab_lt_r0_r5_r10_ROR_24[] = {
  0x7a, 0x0c, 0xa5, 0xb6 // sxtab lt r0 r5 r10 ROR 24
};
const byte kInstruction_sxtab_ls_r13_r2_r6_ROR_16[] = {
  0x76, 0xd8, 0xa2, 0x96 // sxtab ls r13 r2 r6 ROR 16
};
const byte kInstruction_sxtab_vc_r8_r12_r11_ROR_16[] = {
  0x7b, 0x88, 0xac, 0x76 // sxtab vc r8 r12 r11 ROR 16
};
const byte kInstruction_sxtab_vs_r1_r3_r7_ROR_24[] = {
  0x77, 0x1c, 0xa3, 0x66 // sxtab vs r1 r3 r7 ROR 24
};
const byte kInstruction_sxtab_hi_r10_r3_r7_ROR_0[] = {
  0x77, 0xa0, 0xa3, 0x86 // sxtab hi r10 r3 r7 ROR 0
};
const byte kInstruction_sxtab_ls_r11_r9_r7_ROR_8[] = {
  0x77, 0xb4, 0xa9, 0x96 // sxtab ls r11 r9 r7 ROR 8
};
const byte kInstruction_sxtab_al_r10_r6_r1_ROR_16[] = {
  0x71, 0xa8, 0xa6, 0xe6 // sxtab al r10 r6 r1 ROR 16
};
const byte kInstruction_sxtab_cs_r1_r6_r11_ROR_8[] = {
  0x7b, 0x14, 0xa6, 0x26 // sxtab cs r1 r6 r11 ROR 8
};
const byte kInstruction_sxtab_lt_r6_r6_r10_ROR_24[] = {
  0x7a, 0x6c, 0xa6, 0xb6 // sxtab lt r6 r6 r10 ROR 24
};
const byte kInstruction_sxtab_le_r10_r10_r5_ROR_8[] = {
  0x75, 0xa4, 0xaa, 0xd6 // sxtab le r10 r10 r5 ROR 8
};
const byte kInstruction_sxtab_mi_r3_r10_r1_ROR_24[] = {
  0x71, 0x3c, 0xaa, 0x46 // sxtab mi r3 r10 r1 ROR 24
};
const byte kInstruction_sxtab_le_r9_r4_r11_ROR_8[] = {
  0x7b, 0x94, 0xa4, 0xd6 // sxtab le r9 r4 r11 ROR 8
};
const byte kInstruction_sxtab_ge_r8_r4_r12_ROR_8[] = {
  0x7c, 0x84, 0xa4, 0xa6 // sxtab ge r8 r4 r12 ROR 8
};
const byte kInstruction_sxtab_gt_r8_r9_r10_ROR_8[] = {
  0x7a, 0x84, 0xa9, 0xc6 // sxtab gt r8 r9 r10 ROR 8
};
const byte kInstruction_sxtab_cc_r1_r10_r1_ROR_16[] = {
  0x71, 0x18, 0xaa, 0x36 // sxtab cc r1 r10 r1 ROR 16
};
const byte kInstruction_sxtab_vc_r10_r10_r13_ROR_0[] = {
  0x7d, 0xa0, 0xaa, 0x76 // sxtab vc r10 r10 r13 ROR 0
};
const byte kInstruction_sxtab_vs_r7_r13_r0_ROR_16[] = {
  0x70, 0x78, 0xad, 0x66 // sxtab vs r7 r13 r0 ROR 16
};
const byte kInstruction_sxtab_le_r1_r10_r13_ROR_8[] = {
  0x7d, 0x14, 0xaa, 0xd6 // sxtab le r1 r10 r13 ROR 8
};
const byte kInstruction_sxtab_ls_r8_r3_r12_ROR_8[] = {
  0x7c, 0x84, 0xa3, 0x96 // sxtab ls r8 r3 r12 ROR 8
};
const byte kInstruction_sxtab_vc_r13_r9_r12_ROR_8[] = {
  0x7c, 0xd4, 0xa9, 0x76 // sxtab vc r13 r9 r12 ROR 8
};
const byte kInstruction_sxtab_vs_r11_r8_r9_ROR_0[] = {
  0x79, 0xb0, 0xa8, 0x66 // sxtab vs r11 r8 r9 ROR 0
};
const byte kInstruction_sxtab_pl_r8_r0_r7_ROR_24[] = {
  0x77, 0x8c, 0xa0, 0x56 // sxtab pl r8 r0 r7 ROR 24
};
const byte kInstruction_sxtab_cs_r7_r8_r4_ROR_24[] = {
  0x74, 0x7c, 0xa8, 0x26 // sxtab cs r7 r8 r4 ROR 24
};
const byte kInstruction_sxtab_gt_r12_r12_r0_ROR_0[] = {
  0x70, 0xc0, 0xac, 0xc6 // sxtab gt r12 r12 r0 ROR 0
};
const byte kInstruction_sxtab_cc_r5_r8_r4_ROR_8[] = {
  0x74, 0x54, 0xa8, 0x36 // sxtab cc r5 r8 r4 ROR 8
};
const byte kInstruction_sxtab_cc_r4_r11_r11_ROR_16[] = {
  0x7b, 0x48, 0xab, 0x36 // sxtab cc r4 r11 r11 ROR 16
};
const byte kInstruction_sxtab_mi_r13_r9_r7_ROR_24[] = {
  0x77, 0xdc, 0xa9, 0x46 // sxtab mi r13 r9 r7 ROR 24
};
const byte kInstruction_sxtab_lt_r5_r7_r13_ROR_24[] = {
  0x7d, 0x5c, 0xa7, 0xb6 // sxtab lt r5 r7 r13 ROR 24
};
const byte kInstruction_sxtab_ne_r6_r5_r0_ROR_8[] = {
  0x70, 0x64, 0xa5, 0x16 // sxtab ne r6 r5 r0 ROR 8
};
const byte kInstruction_sxtab_vc_r14_r6_r1_ROR_16[] = {
  0x71, 0xe8, 0xa6, 0x76 // sxtab vc r14 r6 r1 ROR 16
};
const byte kInstruction_sxtab_lt_r8_r10_r0_ROR_8[] = {
  0x70, 0x84, 0xaa, 0xb6 // sxtab lt r8 r10 r0 ROR 8
};
const byte kInstruction_sxtab_gt_r2_r1_r14_ROR_0[] = {
  0x7e, 0x20, 0xa1, 0xc6 // sxtab gt r2 r1 r14 ROR 0
};
const byte kInstruction_sxtab_mi_r4_r3_r0_ROR_8[] = {
  0x70, 0x44, 0xa3, 0x46 // sxtab mi r4 r3 r0 ROR 8
};
const byte kInstruction_sxtab_gt_r1_r3_r14_ROR_24[] = {
  0x7e, 0x1c, 0xa3, 0xc6 // sxtab gt r1 r3 r14 ROR 24
};
const byte kInstruction_sxtab_cc_r3_r9_r1_ROR_0[] = {
  0x71, 0x30, 0xa9, 0x36 // sxtab cc r3 r9 r1 ROR 0
};
const byte kInstruction_sxtab_ne_r3_r1_r6_ROR_24[] = {
  0x76, 0x3c, 0xa1, 0x16 // sxtab ne r3 r1 r6 ROR 24
};
const byte kInstruction_sxtab_gt_r3_r11_r6_ROR_24[] = {
  0x76, 0x3c, 0xab, 0xc6 // sxtab gt r3 r11 r6 ROR 24
};
const byte kInstruction_sxtab_mi_r9_r7_r7_ROR_0[] = {
  0x77, 0x90, 0xa7, 0x46 // sxtab mi r9 r7 r7 ROR 0
};
const byte kInstruction_sxtab_le_r0_r11_r11_ROR_0[] = {
  0x7b, 0x00, 0xab, 0xd6 // sxtab le r0 r11 r11 ROR 0
};
const byte kInstruction_sxtab_ge_r6_r10_r6_ROR_16[] = {
  0x76, 0x68, 0xaa, 0xa6 // sxtab ge r6 r10 r6 ROR 16
};
const byte kInstruction_sxtab_ls_r12_r9_r6_ROR_8[] = {
  0x76, 0xc4, 0xa9, 0x96 // sxtab ls r12 r9 r6 ROR 8
};
const byte kInstruction_sxtab_al_r1_r5_r11_ROR_8[] = {
  0x7b, 0x14, 0xa5, 0xe6 // sxtab al r1 r5 r11 ROR 8
};
const byte kInstruction_sxtab_cs_r9_r10_r9_ROR_8[] = {
  0x79, 0x94, 0xaa, 0x26 // sxtab cs r9 r10 r9 ROR 8
};
const byte kInstruction_sxtab_mi_r11_r14_r9_ROR_24[] = {
  0x79, 0xbc, 0xae, 0x46 // sxtab mi r11 r14 r9 ROR 24
};
const byte kInstruction_sxtab_ne_r12_r11_r12_ROR_16[] = {
  0x7c, 0xc8, 0xab, 0x16 // sxtab ne r12 r11 r12 ROR 16
};
const byte kInstruction_sxtab_ge_r7_r5_r1_ROR_0[] = {
  0x71, 0x70, 0xa5, 0xa6 // sxtab ge r7 r5 r1 ROR 0
};
const byte kInstruction_sxtab_vc_r6_r13_r2_ROR_16[] = {
  0x72, 0x68, 0xad, 0x76 // sxtab vc r6 r13 r2 ROR 16
};
const byte kInstruction_sxtab_mi_r8_r2_r3_ROR_8[] = {
  0x73, 0x84, 0xa2, 0x46 // sxtab mi r8 r2 r3 ROR 8
};
const byte kInstruction_sxtab_lt_r10_r0_r2_ROR_16[] = {
  0x72, 0xa8, 0xa0, 0xb6 // sxtab lt r10 r0 r2 ROR 16
};
const byte kInstruction_sxtab_ge_r0_r14_r7_ROR_8[] = {
  0x77, 0x04, 0xae, 0xa6 // sxtab ge r0 r14 r7 ROR 8
};
const byte kInstruction_sxtab_mi_r5_r11_r6_ROR_0[] = {
  0x76, 0x50, 0xab, 0x46 // sxtab mi r5 r11 r6 ROR 0
};
const byte kInstruction_sxtab_eq_r10_r11_r0_ROR_24[] = {
  0x70, 0xac, 0xab, 0x06 // sxtab eq r10 r11 r0 ROR 24
};
const byte kInstruction_sxtab_gt_r10_r3_r3_ROR_16[] = {
  0x73, 0xa8, 0xa3, 0xc6 // sxtab gt r10 r3 r3 ROR 16
};
const byte kInstruction_sxtab_cs_r5_r1_r1_ROR_0[] = {
  0x71, 0x50, 0xa1, 0x26 // sxtab cs r5 r1 r1 ROR 0
};
const byte kInstruction_sxtab_ls_r6_r13_r2_ROR_0[] = {
  0x72, 0x60, 0xad, 0x96 // sxtab ls r6 r13 r2 ROR 0
};
const byte kInstruction_sxtab_al_r0_r2_r7_ROR_0[] = {
  0x77, 0x00, 0xa2, 0xe6 // sxtab al r0 r2 r7 ROR 0
};
const byte kInstruction_sxtab_le_r9_r13_r12_ROR_24[] = {
  0x7c, 0x9c, 0xad, 0xd6 // sxtab le r9 r13 r12 ROR 24
};
const byte kInstruction_sxtab_eq_r11_r0_r4_ROR_0[] = {
  0x74, 0xb0, 0xa0, 0x06 // sxtab eq r11 r0 r4 ROR 0
};
const byte kInstruction_sxtab_ne_r10_r10_r12_ROR_24[] = {
  0x7c, 0xac, 0xaa, 0x16 // sxtab ne r10 r10 r12 ROR 24
};
const byte kInstruction_sxtab_ls_r0_r6_r7_ROR_16[] = {
  0x77, 0x08, 0xa6, 0x96 // sxtab ls r0 r6 r7 ROR 16
};
const byte kInstruction_sxtab_ls_r5_r4_r6_ROR_16[] = {
  0x76, 0x58, 0xa4, 0x96 // sxtab ls r5 r4 r6 ROR 16
};
const byte kInstruction_sxtab_eq_r7_r3_r12_ROR_8[] = {
  0x7c, 0x74, 0xa3, 0x06 // sxtab eq r7 r3 r12 ROR 8
};
const byte kInstruction_sxtab_cs_r8_r0_r3_ROR_0[] = {
  0x73, 0x80, 0xa0, 0x26 // sxtab cs r8 r0 r3 ROR 0
};
const byte kInstruction_sxtab_ls_r1_r3_r2_ROR_0[] = {
  0x72, 0x10, 0xa3, 0x96 // sxtab ls r1 r3 r2 ROR 0
};
const byte kInstruction_sxtab_lt_r2_r0_r2_ROR_24[] = {
  0x72, 0x2c, 0xa0, 0xb6 // sxtab lt r2 r0 r2 ROR 24
};
const byte kInstruction_sxtab_eq_r6_r9_r10_ROR_8[] = {
  0x7a, 0x64, 0xa9, 0x06 // sxtab eq r6 r9 r10 ROR 8
};
const byte kInstruction_sxtab_vs_r12_r4_r10_ROR_16[] = {
  0x7a, 0xc8, 0xa4, 0x66 // sxtab vs r12 r4 r10 ROR 16
};
const byte kInstruction_sxtab_cc_r0_r14_r1_ROR_24[] = {
  0x71, 0x0c, 0xae, 0x36 // sxtab cc r0 r14 r1 ROR 24
};
const byte kInstruction_sxtab_al_r6_r11_r10_ROR_0[] = {
  0x7a, 0x60, 0xab, 0xe6 // sxtab al r6 r11 r10 ROR 0
};
const byte kInstruction_sxtab_le_r4_r3_r4_ROR_16[] = {
  0x74, 0x48, 0xa3, 0xd6 // sxtab le r4 r3 r4 ROR 16
};
const byte kInstruction_sxtab_vs_r5_r14_r1_ROR_8[] = {
  0x71, 0x54, 0xae, 0x66 // sxtab vs r5 r14 r1 ROR 8
};
const byte kInstruction_sxtab_eq_r8_r10_r4_ROR_24[] = {
  0x74, 0x8c, 0xaa, 0x06 // sxtab eq r8 r10 r4 ROR 24
};
const byte kInstruction_sxtab_ne_r10_r11_r5_ROR_16[] = {
  0x75, 0xa8, 0xab, 0x16 // sxtab ne r10 r11 r5 ROR 16
};
const byte kInstruction_sxtab_vc_r5_r1_r9_ROR_8[] = {
  0x79, 0x54, 0xa1, 0x76 // sxtab vc r5 r1 r9 ROR 8
};
const byte kInstruction_sxtab_vc_r2_r9_r7_ROR_16[] = {
  0x77, 0x28, 0xa9, 0x76 // sxtab vc r2 r9 r7 ROR 16
};
const byte kInstruction_sxtab_ne_r12_r3_r4_ROR_24[] = {
  0x74, 0xcc, 0xa3, 0x16 // sxtab ne r12 r3 r4 ROR 24
};
const byte kInstruction_sxtab_mi_r1_r9_r11_ROR_16[] = {
  0x7b, 0x18, 0xa9, 0x46 // sxtab mi r1 r9 r11 ROR 16
};
const byte kInstruction_sxtab_hi_r12_r5_r2_ROR_0[] = {
  0x72, 0xc0, 0xa5, 0x86 // sxtab hi r12 r5 r2 ROR 0
};
const byte kInstruction_sxtab_ge_r14_r3_r7_ROR_16[] = {
  0x77, 0xe8, 0xa3, 0xa6 // sxtab ge r14 r3 r7 ROR 16
};
const byte kInstruction_sxtab_lt_r11_r14_r7_ROR_0[] = {
  0x77, 0xb0, 0xae, 0xb6 // sxtab lt r11 r14 r7 ROR 0
};
const byte kInstruction_sxtab_ge_r5_r2_r5_ROR_0[] = {
  0x75, 0x50, 0xa2, 0xa6 // sxtab ge r5 r2 r5 ROR 0
};
const byte kInstruction_sxtab_ne_r10_r6_r11_ROR_8[] = {
  0x7b, 0xa4, 0xa6, 0x16 // sxtab ne r10 r6 r11 ROR 8
};
const byte kInstruction_sxtab_vs_r11_r5_r13_ROR_8[] = {
  0x7d, 0xb4, 0xa5, 0x66 // sxtab vs r11 r5 r13 ROR 8
};
const byte kInstruction_sxtab_cc_r10_r10_r11_ROR_16[] = {
  0x7b, 0xa8, 0xaa, 0x36 // sxtab cc r10 r10 r11 ROR 16
};
const byte kInstruction_sxtab_hi_r6_r13_r5_ROR_24[] = {
  0x75, 0x6c, 0xad, 0x86 // sxtab hi r6 r13 r5 ROR 24
};
const byte kInstruction_sxtab_gt_r13_r14_r2_ROR_0[] = {
  0x72, 0xd0, 0xae, 0xc6 // sxtab gt r13 r14 r2 ROR 0
};
const byte kInstruction_sxtab_mi_r9_r1_r14_ROR_16[] = {
  0x7e, 0x98, 0xa1, 0x46 // sxtab mi r9 r1 r14 ROR 16
};
const byte kInstruction_sxtab_eq_r4_r1_r9_ROR_16[] = {
  0x79, 0x48, 0xa1, 0x06 // sxtab eq r4 r1 r9 ROR 16
};
const byte kInstruction_sxtab_ge_r10_r0_r14_ROR_24[] = {
  0x7e, 0xac, 0xa0, 0xa6 // sxtab ge r10 r0 r14 ROR 24
};
const byte kInstruction_sxtab_ls_r12_r12_r11_ROR_24[] = {
  0x7b, 0xcc, 0xac, 0x96 // sxtab ls r12 r12 r11 ROR 24
};
const byte kInstruction_sxtab_ls_r14_r7_r14_ROR_24[] = {
  0x7e, 0xec, 0xa7, 0x96 // sxtab ls r14 r7 r14 ROR 24
};
const byte kInstruction_sxtab_ne_r10_r2_r11_ROR_8[] = {
  0x7b, 0xa4, 0xa2, 0x16 // sxtab ne r10 r2 r11 ROR 8
};
const byte kInstruction_sxtab_lt_r14_r2_r9_ROR_0[] = {
  0x79, 0xe0, 0xa2, 0xb6 // sxtab lt r14 r2 r9 ROR 0
};
const byte kInstruction_sxtab_eq_r4_r4_r1_ROR_24[] = {
  0x71, 0x4c, 0xa4, 0x06 // sxtab eq r4 r4 r1 ROR 24
};
const byte kInstruction_sxtab_cc_r7_r0_r9_ROR_24[] = {
  0x79, 0x7c, 0xa0, 0x36 // sxtab cc r7 r0 r9 ROR 24
};
const byte kInstruction_sxtab_al_r3_r3_r11_ROR_8[] = {
  0x7b, 0x34, 0xa3, 0xe6 // sxtab al r3 r3 r11 ROR 8
};
const byte kInstruction_sxtab_le_r8_r9_r14_ROR_0[] = {
  0x7e, 0x80, 0xa9, 0xd6 // sxtab le r8 r9 r14 ROR 0
};
const byte kInstruction_sxtab_gt_r2_r9_r4_ROR_16[] = {
  0x74, 0x28, 0xa9, 0xc6 // sxtab gt r2 r9 r4 ROR 16
};
const byte kInstruction_sxtab_ne_r11_r11_r12_ROR_0[] = {
  0x7c, 0xb0, 0xab, 0x16 // sxtab ne r11 r11 r12 ROR 0
};
const byte kInstruction_sxtab_hi_r13_r11_r3_ROR_0[] = {
  0x73, 0xd0, 0xab, 0x86 // sxtab hi r13 r11 r3 ROR 0
};
const byte kInstruction_sxtab_eq_r10_r13_r14_ROR_24[] = {
  0x7e, 0xac, 0xad, 0x06 // sxtab eq r10 r13 r14 ROR 24
};
const byte kInstruction_sxtab_ne_r3_r5_r3_ROR_0[] = {
  0x73, 0x30, 0xa5, 0x16 // sxtab ne r3 r5 r3 ROR 0
};
const byte kInstruction_sxtab_eq_r9_r4_r14_ROR_0[] = {
  0x7e, 0x90, 0xa4, 0x06 // sxtab eq r9 r4 r14 ROR 0
};
const byte kInstruction_sxtab_al_r14_r8_r7_ROR_24[] = {
  0x77, 0xec, 0xa8, 0xe6 // sxtab al r14 r8 r7 ROR 24
};
const byte kInstruction_sxtab_mi_r0_r0_r13_ROR_8[] = {
  0x7d, 0x04, 0xa0, 0x46 // sxtab mi r0 r0 r13 ROR 8
};
const byte kInstruction_sxtab_cc_r9_r4_r13_ROR_16[] = {
  0x7d, 0x98, 0xa4, 0x36 // sxtab cc r9 r4 r13 ROR 16
};
const byte kInstruction_sxtab_cc_r10_r0_r4_ROR_24[] = {
  0x74, 0xac, 0xa0, 0x36 // sxtab cc r10 r0 r4 ROR 24
};
const byte kInstruction_sxtab_ge_r7_r5_r10_ROR_24[] = {
  0x7a, 0x7c, 0xa5, 0xa6 // sxtab ge r7 r5 r10 ROR 24
};
const byte kInstruction_sxtab_vs_r7_r0_r14_ROR_24[] = {
  0x7e, 0x7c, 0xa0, 0x66 // sxtab vs r7 r0 r14 ROR 24
};
const byte kInstruction_sxtab_cc_r12_r5_r9_ROR_16[] = {
  0x79, 0xc8, 0xa5, 0x36 // sxtab cc r12 r5 r9 ROR 16
};
const byte kInstruction_sxtab_le_r6_r8_r2_ROR_0[] = {
  0x72, 0x60, 0xa8, 0xd6 // sxtab le r6 r8 r2 ROR 0
};
const byte kInstruction_sxtab_ne_r13_r13_r3_ROR_24[] = {
  0x73, 0xdc, 0xad, 0x16 // sxtab ne r13 r13 r3 ROR 24
};
const byte kInstruction_sxtab_eq_r1_r10_r1_ROR_8[] = {
  0x71, 0x14, 0xaa, 0x06 // sxtab eq r1 r10 r1 ROR 8
};
const byte kInstruction_sxtab_eq_r14_r10_r6_ROR_16[] = {
  0x76, 0xe8, 0xaa, 0x06 // sxtab eq r14 r10 r6 ROR 16
};
const byte kInstruction_sxtab_al_r14_r14_r10_ROR_8[] = {
  0x7a, 0xe4, 0xae, 0xe6 // sxtab al r14 r14 r10 ROR 8
};
const byte kInstruction_sxtab_al_r7_r8_r4_ROR_24[] = {
  0x74, 0x7c, 0xa8, 0xe6 // sxtab al r7 r8 r4 ROR 24
};
const byte kInstruction_sxtab_al_r11_r6_r6_ROR_8[] = {
  0x76, 0xb4, 0xa6, 0xe6 // sxtab al r11 r6 r6 ROR 8
};
const byte kInstruction_sxtab_lt_r5_r3_r13_ROR_16[] = {
  0x7d, 0x58, 0xa3, 0xb6 // sxtab lt r5 r3 r13 ROR 16
};
const byte kInstruction_sxtab_cc_r8_r3_r2_ROR_24[] = {
  0x72, 0x8c, 0xa3, 0x36 // sxtab cc r8 r3 r2 ROR 24
};
const byte kInstruction_sxtab_ge_r11_r0_r10_ROR_0[] = {
  0x7a, 0xb0, 0xa0, 0xa6 // sxtab ge r11 r0 r10 ROR 0
};
const byte kInstruction_sxtab_ge_r7_r12_r12_ROR_16[] = {
  0x7c, 0x78, 0xac, 0xa6 // sxtab ge r7 r12 r12 ROR 16
};
const byte kInstruction_sxtab_ls_r7_r5_r2_ROR_8[] = {
  0x72, 0x74, 0xa5, 0x96 // sxtab ls r7 r5 r2 ROR 8
};
const byte kInstruction_sxtab_hi_r14_r4_r3_ROR_8[] = {
  0x73, 0xe4, 0xa4, 0x86 // sxtab hi r14 r4 r3 ROR 8
};
const byte kInstruction_sxtab_le_r3_r5_r2_ROR_0[] = {
  0x72, 0x30, 0xa5, 0xd6 // sxtab le r3 r5 r2 ROR 0
};
const byte kInstruction_sxtab_cs_r13_r9_r13_ROR_16[] = {
  0x7d, 0xd8, 0xa9, 0x26 // sxtab cs r13 r9 r13 ROR 16
};
const byte kInstruction_sxtab_ge_r1_r14_r2_ROR_16[] = {
  0x72, 0x18, 0xae, 0xa6 // sxtab ge r1 r14 r2 ROR 16
};
const byte kInstruction_sxtab_gt_r4_r6_r2_ROR_16[] = {
  0x72, 0x48, 0xa6, 0xc6 // sxtab gt r4 r6 r2 ROR 16
};
const byte kInstruction_sxtab_al_r6_r8_r13_ROR_0[] = {
  0x7d, 0x60, 0xa8, 0xe6 // sxtab al r6 r8 r13 ROR 0
};
const byte kInstruction_sxtab_gt_r6_r8_r3_ROR_16[] = {
  0x73, 0x68, 0xa8, 0xc6 // sxtab gt r6 r8 r3 ROR 16
};
const byte kInstruction_sxtab_ge_r5_r11_r14_ROR_24[] = {
  0x7e, 0x5c, 0xab, 0xa6 // sxtab ge r5 r11 r14 ROR 24
};
const byte kInstruction_sxtab_cc_r9_r10_r11_ROR_16[] = {
  0x7b, 0x98, 0xaa, 0x36 // sxtab cc r9 r10 r11 ROR 16
};
const byte kInstruction_sxtab_vs_r11_r13_r3_ROR_0[] = {
  0x73, 0xb0, 0xad, 0x66 // sxtab vs r11 r13 r3 ROR 0
};
const byte kInstruction_sxtab_cc_r2_r5_r12_ROR_24[] = {
  0x7c, 0x2c, 0xa5, 0x36 // sxtab cc r2 r5 r12 ROR 24
};
const byte kInstruction_sxtab_ge_r3_r9_r11_ROR_0[] = {
  0x7b, 0x30, 0xa9, 0xa6 // sxtab ge r3 r9 r11 ROR 0
};
const byte kInstruction_sxtab_ne_r8_r3_r1_ROR_8[] = {
  0x71, 0x84, 0xa3, 0x16 // sxtab ne r8 r3 r1 ROR 8
};
const byte kInstruction_sxtab_ls_r1_r6_r6_ROR_16[] = {
  0x76, 0x18, 0xa6, 0x96 // sxtab ls r1 r6 r6 ROR 16
};
const byte kInstruction_sxtab_pl_r8_r13_r4_ROR_16[] = {
  0x74, 0x88, 0xad, 0x56 // sxtab pl r8 r13 r4 ROR 16
};
const byte kInstruction_sxtab_ge_r7_r3_r0_ROR_24[] = {
  0x70, 0x7c, 0xa3, 0xa6 // sxtab ge r7 r3 r0 ROR 24
};
const byte kInstruction_sxtab_ls_r12_r9_r7_ROR_16[] = {
  0x77, 0xc8, 0xa9, 0x96 // sxtab ls r12 r9 r7 ROR 16
};
const byte kInstruction_sxtab_ls_r14_r6_r7_ROR_24[] = {
  0x77, 0xec, 0xa6, 0x96 // sxtab ls r14 r6 r7 ROR 24
};
const byte kInstruction_sxtab_al_r7_r0_r10_ROR_8[] = {
  0x7a, 0x74, 0xa0, 0xe6 // sxtab al r7 r0 r10 ROR 8
};
const byte kInstruction_sxtab_mi_r5_r14_r11_ROR_0[] = {
  0x7b, 0x50, 0xae, 0x46 // sxtab mi r5 r14 r11 ROR 0
};
const byte kInstruction_sxtab_ls_r12_r8_r2_ROR_0[] = {
  0x72, 0xc0, 0xa8, 0x96 // sxtab ls r12 r8 r2 ROR 0
};
const byte kInstruction_sxtab_gt_r2_r2_r10_ROR_8[] = {
  0x7a, 0x24, 0xa2, 0xc6 // sxtab gt r2 r2 r10 ROR 8
};
const byte kInstruction_sxtab_ls_r3_r2_r5_ROR_0[] = {
  0x75, 0x30, 0xa2, 0x96 // sxtab ls r3 r2 r5 ROR 0
};
const byte kInstruction_sxtab_ge_r4_r14_r3_ROR_24[] = {
  0x73, 0x4c, 0xae, 0xa6 // sxtab ge r4 r14 r3 ROR 24
};
const byte kInstruction_sxtab_ls_r5_r2_r6_ROR_16[] = {
  0x76, 0x58, 0xa2, 0x96 // sxtab ls r5 r2 r6 ROR 16
};
const byte kInstruction_sxtab_le_r10_r1_r7_ROR_0[] = {
  0x77, 0xa0, 0xa1, 0xd6 // sxtab le r10 r1 r7 ROR 0
};
const byte kInstruction_sxtab_gt_r13_r2_r9_ROR_16[] = {
  0x79, 0xd8, 0xa2, 0xc6 // sxtab gt r13 r2 r9 ROR 16
};
const byte kInstruction_sxtab_cs_r12_r9_r7_ROR_16[] = {
  0x77, 0xc8, 0xa9, 0x26 // sxtab cs r12 r9 r7 ROR 16
};
const byte kInstruction_sxtab_al_r10_r12_r1_ROR_24[] = {
  0x71, 0xac, 0xac, 0xe6 // sxtab al r10 r12 r1 ROR 24
};
const byte kInstruction_sxtab_le_r9_r3_r14_ROR_16[] = {
  0x7e, 0x98, 0xa3, 0xd6 // sxtab le r9 r3 r14 ROR 16
};
const byte kInstruction_sxtab_le_r1_r4_r6_ROR_16[] = {
  0x76, 0x18, 0xa4, 0xd6 // sxtab le r1 r4 r6 ROR 16
};
const TestResult kReferencesxtab[] = {
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r3_r3_r13_ROR_0),
    kInstruction_sxtab_ls_r3_r3_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r2_r7_r1_ROR_16),
    kInstruction_sxtab_cs_r2_r7_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r13_r0_r2_ROR_8),
    kInstruction_sxtab_mi_r13_r0_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r0_r6_r1_ROR_8),
    kInstruction_sxtab_lt_r0_r6_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r6_r4_r8_ROR_16),
    kInstruction_sxtab_al_r6_r4_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r1_r7_r10_ROR_16),
    kInstruction_sxtab_gt_r1_r7_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r12_r9_r11_ROR_16),
    kInstruction_sxtab_cc_r12_r9_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r6_r8_r13_ROR_24),
    kInstruction_sxtab_mi_r6_r8_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r9_r1_r6_ROR_16),
    kInstruction_sxtab_mi_r9_r1_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r8_r9_r14_ROR_8),
    kInstruction_sxtab_le_r8_r9_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r6_r0_r1_ROR_8),
    kInstruction_sxtab_mi_r6_r0_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r14_r2_r2_ROR_24),
    kInstruction_sxtab_cs_r14_r2_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r1_r11_r8_ROR_8),
    kInstruction_sxtab_ge_r1_r11_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r6_r8_r9_ROR_16),
    kInstruction_sxtab_al_r6_r8_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r12_r8_r2_ROR_24),
    kInstruction_sxtab_ls_r12_r8_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r2_r8_r8_ROR_0),
    kInstruction_sxtab_vc_r2_r8_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r14_r6_r4_ROR_16),
    kInstruction_sxtab_le_r14_r6_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r0_r4_r12_ROR_0),
    kInstruction_sxtab_ge_r0_r4_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r1_r3_r9_ROR_24),
    kInstruction_sxtab_cc_r1_r3_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r9_r5_r4_ROR_8),
    kInstruction_sxtab_mi_r9_r5_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r4_r10_r4_ROR_8),
    kInstruction_sxtab_al_r4_r10_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r8_r13_r1_ROR_24),
    kInstruction_sxtab_vs_r8_r13_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r4_r12_r2_ROR_0),
    kInstruction_sxtab_eq_r4_r12_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r7_r5_r6_ROR_24),
    kInstruction_sxtab_pl_r7_r5_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r9_r14_r14_ROR_24),
    kInstruction_sxtab_lt_r9_r14_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r8_r6_r2_ROR_0),
    kInstruction_sxtab_ge_r8_r6_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r11_r6_r6_ROR_0),
    kInstruction_sxtab_lt_r11_r6_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r12_r2_r2_ROR_16),
    kInstruction_sxtab_le_r12_r2_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r10_r2_r2_ROR_16),
    kInstruction_sxtab_hi_r10_r2_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r0_r8_r6_ROR_0),
    kInstruction_sxtab_mi_r0_r8_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r0_r6_r11_ROR_8),
    kInstruction_sxtab_pl_r0_r6_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r0_r0_r3_ROR_0),
    kInstruction_sxtab_hi_r0_r0_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r4_r9_r13_ROR_16),
    kInstruction_sxtab_gt_r4_r9_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r4_r7_r10_ROR_16),
    kInstruction_sxtab_ne_r4_r7_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r4_r13_r9_ROR_16),
    kInstruction_sxtab_vc_r4_r13_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r8_r9_r4_ROR_8),
    kInstruction_sxtab_hi_r8_r9_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r2_r6_r0_ROR_8),
    kInstruction_sxtab_hi_r2_r6_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r13_r7_r6_ROR_0),
    kInstruction_sxtab_cs_r13_r7_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r11_r1_r14_ROR_8),
    kInstruction_sxtab_gt_r11_r1_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r0_r1_r4_ROR_8),
    kInstruction_sxtab_eq_r0_r1_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r11_r12_r1_ROR_16),
    kInstruction_sxtab_eq_r11_r12_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r2_r12_r9_ROR_16),
    kInstruction_sxtab_pl_r2_r12_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r1_r5_r11_ROR_16),
    kInstruction_sxtab_ls_r1_r5_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r0_r13_r9_ROR_24),
    kInstruction_sxtab_pl_r0_r13_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r6_r7_r11_ROR_16),
    kInstruction_sxtab_pl_r6_r7_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r0_r10_r9_ROR_0),
    kInstruction_sxtab_le_r0_r10_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r9_r14_r5_ROR_0),
    kInstruction_sxtab_vc_r9_r14_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r10_r4_r1_ROR_16),
    kInstruction_sxtab_ls_r10_r4_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r3_r7_r2_ROR_0),
    kInstruction_sxtab_ls_r3_r7_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r13_r14_r3_ROR_0),
    kInstruction_sxtab_mi_r13_r14_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r2_r14_r4_ROR_8),
    kInstruction_sxtab_lt_r2_r14_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r1_r9_r5_ROR_16),
    kInstruction_sxtab_cc_r1_r9_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r12_r8_r4_ROR_16),
    kInstruction_sxtab_pl_r12_r8_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r10_r13_r3_ROR_16),
    kInstruction_sxtab_vs_r10_r13_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r10_r13_r14_ROR_0),
    kInstruction_sxtab_hi_r10_r13_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r1_r7_r6_ROR_0),
    kInstruction_sxtab_vc_r1_r7_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r4_r3_r11_ROR_16),
    kInstruction_sxtab_vs_r4_r3_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r3_r12_r3_ROR_24),
    kInstruction_sxtab_lt_r3_r12_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r4_r0_r6_ROR_24),
    kInstruction_sxtab_ge_r4_r0_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r12_r4_r0_ROR_0),
    kInstruction_sxtab_vc_r12_r4_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r2_r2_r9_ROR_8),
    kInstruction_sxtab_ne_r2_r2_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r9_r4_r13_ROR_8),
    kInstruction_sxtab_le_r9_r4_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r13_r1_r14_ROR_24),
    kInstruction_sxtab_lt_r13_r1_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r7_r3_r9_ROR_0),
    kInstruction_sxtab_ge_r7_r3_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r6_r2_r12_ROR_0),
    kInstruction_sxtab_mi_r6_r2_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r2_r13_r10_ROR_0),
    kInstruction_sxtab_vs_r2_r13_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r3_r12_r3_ROR_16),
    kInstruction_sxtab_lt_r3_r12_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r3_r6_r14_ROR_16),
    kInstruction_sxtab_ne_r3_r6_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r6_r7_r0_ROR_0),
    kInstruction_sxtab_le_r6_r7_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r3_r1_r1_ROR_0),
    kInstruction_sxtab_vs_r3_r1_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r0_r6_r0_ROR_8),
    kInstruction_sxtab_ge_r0_r6_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r10_r0_r13_ROR_16),
    kInstruction_sxtab_ne_r10_r0_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r11_r8_r11_ROR_16),
    kInstruction_sxtab_ne_r11_r8_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r4_r5_r11_ROR_16),
    kInstruction_sxtab_lt_r4_r5_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r2_r13_r4_ROR_8),
    kInstruction_sxtab_gt_r2_r13_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r14_r7_r9_ROR_24),
    kInstruction_sxtab_ls_r14_r7_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r4_r12_r9_ROR_16),
    kInstruction_sxtab_vc_r4_r12_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r7_r9_r3_ROR_0),
    kInstruction_sxtab_vc_r7_r9_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r10_r3_r11_ROR_0),
    kInstruction_sxtab_eq_r10_r3_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r3_r6_r11_ROR_16),
    kInstruction_sxtab_ne_r3_r6_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r1_r2_r3_ROR_16),
    kInstruction_sxtab_cs_r1_r2_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r0_r1_r12_ROR_0),
    kInstruction_sxtab_gt_r0_r1_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r11_r8_r11_ROR_16),
    kInstruction_sxtab_mi_r11_r8_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r6_r9_r0_ROR_24),
    kInstruction_sxtab_mi_r6_r9_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r3_r12_r12_ROR_0),
    kInstruction_sxtab_hi_r3_r12_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r8_r2_r2_ROR_16),
    kInstruction_sxtab_lt_r8_r2_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r4_r12_r5_ROR_0),
    kInstruction_sxtab_le_r4_r12_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r12_r13_r0_ROR_24),
    kInstruction_sxtab_cs_r12_r13_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r11_r12_r3_ROR_16),
    kInstruction_sxtab_al_r11_r12_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r0_r14_r14_ROR_8),
    kInstruction_sxtab_hi_r0_r14_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r3_r4_r4_ROR_0),
    kInstruction_sxtab_gt_r3_r4_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r13_r10_r13_ROR_24),
    kInstruction_sxtab_mi_r13_r10_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r0_r5_r8_ROR_0),
    kInstruction_sxtab_eq_r0_r5_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r0_r5_r11_ROR_16),
    kInstruction_sxtab_mi_r0_r5_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r0_r9_r2_ROR_16),
    kInstruction_sxtab_eq_r0_r9_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r14_r10_r12_ROR_24),
    kInstruction_sxtab_lt_r14_r10_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r14_r6_r0_ROR_8),
    kInstruction_sxtab_eq_r14_r6_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r12_r13_r8_ROR_24),
    kInstruction_sxtab_eq_r12_r13_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r5_r7_r12_ROR_16),
    kInstruction_sxtab_hi_r5_r7_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r7_r0_r10_ROR_8),
    kInstruction_sxtab_mi_r7_r0_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r9_r8_r12_ROR_24),
    kInstruction_sxtab_cc_r9_r8_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r13_r14_r6_ROR_0),
    kInstruction_sxtab_gt_r13_r14_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r5_r4_r2_ROR_16),
    kInstruction_sxtab_al_r5_r4_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r8_r0_r3_ROR_24),
    kInstruction_sxtab_pl_r8_r0_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r1_r4_r1_ROR_8),
    kInstruction_sxtab_eq_r1_r4_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r5_r9_r12_ROR_8),
    kInstruction_sxtab_lt_r5_r9_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r8_r8_r2_ROR_0),
    kInstruction_sxtab_vs_r8_r8_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r11_r3_r6_ROR_0),
    kInstruction_sxtab_pl_r11_r3_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r2_r9_r11_ROR_8),
    kInstruction_sxtab_al_r2_r9_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r0_r4_r10_ROR_8),
    kInstruction_sxtab_pl_r0_r4_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r9_r14_r2_ROR_16),
    kInstruction_sxtab_pl_r9_r14_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r13_r11_r10_ROR_24),
    kInstruction_sxtab_ge_r13_r11_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r10_r8_r4_ROR_8),
    kInstruction_sxtab_al_r10_r8_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r1_r2_r4_ROR_8),
    kInstruction_sxtab_cc_r1_r2_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r8_r7_r9_ROR_0),
    kInstruction_sxtab_mi_r8_r7_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r8_r9_r6_ROR_8),
    kInstruction_sxtab_pl_r8_r9_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r2_r7_r3_ROR_16),
    kInstruction_sxtab_mi_r2_r7_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r12_r11_r2_ROR_0),
    kInstruction_sxtab_vc_r12_r11_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r3_r6_r1_ROR_8),
    kInstruction_sxtab_ge_r3_r6_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r6_r0_r2_ROR_16),
    kInstruction_sxtab_ls_r6_r0_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r11_r10_r9_ROR_8),
    kInstruction_sxtab_cs_r11_r10_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r2_r4_r7_ROR_0),
    kInstruction_sxtab_pl_r2_r4_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r9_r2_r11_ROR_8),
    kInstruction_sxtab_eq_r9_r2_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r5_r11_r8_ROR_0),
    kInstruction_sxtab_hi_r5_r11_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r5_r7_r3_ROR_8),
    kInstruction_sxtab_hi_r5_r7_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r0_r1_r9_ROR_8),
    kInstruction_sxtab_lt_r0_r1_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r6_r3_r8_ROR_24),
    kInstruction_sxtab_le_r6_r3_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r5_r7_r2_ROR_0),
    kInstruction_sxtab_hi_r5_r7_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r0_r5_r3_ROR_24),
    kInstruction_sxtab_eq_r0_r5_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r13_r13_r4_ROR_8),
    kInstruction_sxtab_mi_r13_r13_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r4_r13_r0_ROR_8),
    kInstruction_sxtab_eq_r4_r13_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r11_r4_r2_ROR_8),
    kInstruction_sxtab_ge_r11_r4_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r5_r1_r9_ROR_24),
    kInstruction_sxtab_pl_r5_r1_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r6_r4_r0_ROR_16),
    kInstruction_sxtab_pl_r6_r4_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r6_r0_r11_ROR_16),
    kInstruction_sxtab_ge_r6_r0_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r4_r0_r14_ROR_16),
    kInstruction_sxtab_cc_r4_r0_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r10_r10_r12_ROR_8),
    kInstruction_sxtab_ge_r10_r10_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r1_r11_r2_ROR_8),
    kInstruction_sxtab_eq_r1_r11_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r2_r14_r6_ROR_16),
    kInstruction_sxtab_ne_r2_r14_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r4_r10_r9_ROR_8),
    kInstruction_sxtab_al_r4_r10_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r11_r12_r10_ROR_0),
    kInstruction_sxtab_al_r11_r12_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r0_r13_r10_ROR_16),
    kInstruction_sxtab_ne_r0_r13_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r4_r2_r5_ROR_0),
    kInstruction_sxtab_pl_r4_r2_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r14_r8_r8_ROR_16),
    kInstruction_sxtab_al_r14_r8_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r3_r5_r14_ROR_0),
    kInstruction_sxtab_pl_r3_r5_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r6_r14_r11_ROR_0),
    kInstruction_sxtab_le_r6_r14_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r9_r12_r12_ROR_8),
    kInstruction_sxtab_cc_r9_r12_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r1_r4_r1_ROR_16),
    kInstruction_sxtab_lt_r1_r4_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r10_r11_r10_ROR_0),
    kInstruction_sxtab_ge_r10_r11_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r0_r11_r12_ROR_8),
    kInstruction_sxtab_gt_r0_r11_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r13_r1_r12_ROR_16),
    kInstruction_sxtab_vc_r13_r1_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r8_r5_r13_ROR_16),
    kInstruction_sxtab_vs_r8_r5_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r5_r7_r12_ROR_8),
    kInstruction_sxtab_ge_r5_r7_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r7_r3_r6_ROR_8),
    kInstruction_sxtab_eq_r7_r3_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r14_r13_r3_ROR_8),
    kInstruction_sxtab_hi_r14_r13_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r3_r7_r1_ROR_8),
    kInstruction_sxtab_hi_r3_r7_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r2_r1_r12_ROR_8),
    kInstruction_sxtab_al_r2_r1_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r5_r6_r4_ROR_16),
    kInstruction_sxtab_mi_r5_r6_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r2_r12_r0_ROR_24),
    kInstruction_sxtab_eq_r2_r12_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r14_r8_r12_ROR_0),
    kInstruction_sxtab_cs_r14_r8_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r10_r7_r11_ROR_24),
    kInstruction_sxtab_hi_r10_r7_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r9_r8_r0_ROR_8),
    kInstruction_sxtab_le_r9_r8_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r6_r8_r7_ROR_0),
    kInstruction_sxtab_vs_r6_r8_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r0_r8_r13_ROR_0),
    kInstruction_sxtab_le_r0_r8_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r4_r4_r6_ROR_16),
    kInstruction_sxtab_cc_r4_r4_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r0_r9_r3_ROR_8),
    kInstruction_sxtab_ge_r0_r9_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r5_r4_r2_ROR_0),
    kInstruction_sxtab_cs_r5_r4_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r12_r9_r8_ROR_16),
    kInstruction_sxtab_gt_r12_r9_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r4_r9_r3_ROR_24),
    kInstruction_sxtab_vs_r4_r9_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r8_r11_r8_ROR_8),
    kInstruction_sxtab_cc_r8_r11_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r5_r6_r11_ROR_24),
    kInstruction_sxtab_lt_r5_r6_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r8_r11_r1_ROR_24),
    kInstruction_sxtab_cc_r8_r11_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r0_r0_r14_ROR_16),
    kInstruction_sxtab_ls_r0_r0_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r10_r14_r7_ROR_8),
    kInstruction_sxtab_lt_r10_r14_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r10_r13_r6_ROR_16),
    kInstruction_sxtab_pl_r10_r13_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r11_r13_r4_ROR_8),
    kInstruction_sxtab_lt_r11_r13_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r5_r14_r0_ROR_0),
    kInstruction_sxtab_cs_r5_r14_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r3_r2_r10_ROR_0),
    kInstruction_sxtab_lt_r3_r2_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r7_r6_r7_ROR_8),
    kInstruction_sxtab_ls_r7_r6_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r2_r14_r14_ROR_0),
    kInstruction_sxtab_ge_r2_r14_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r9_r5_r9_ROR_0),
    kInstruction_sxtab_al_r9_r5_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r14_r2_r5_ROR_24),
    kInstruction_sxtab_vc_r14_r2_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r1_r9_r3_ROR_16),
    kInstruction_sxtab_hi_r1_r9_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r1_r3_r10_ROR_8),
    kInstruction_sxtab_lt_r1_r3_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r12_r1_r5_ROR_16),
    kInstruction_sxtab_lt_r12_r1_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r3_r4_r10_ROR_0),
    kInstruction_sxtab_ge_r3_r4_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r7_r13_r8_ROR_16),
    kInstruction_sxtab_ge_r7_r13_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r12_r8_r7_ROR_16),
    kInstruction_sxtab_eq_r12_r8_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r6_r12_r0_ROR_16),
    kInstruction_sxtab_vs_r6_r12_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r4_r2_r9_ROR_24),
    kInstruction_sxtab_al_r4_r2_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r4_r12_r6_ROR_8),
    kInstruction_sxtab_ne_r4_r12_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r10_r5_r11_ROR_8),
    kInstruction_sxtab_pl_r10_r5_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r12_r5_r1_ROR_0),
    kInstruction_sxtab_cc_r12_r5_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r11_r7_r8_ROR_16),
    kInstruction_sxtab_lt_r11_r7_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r1_r7_r5_ROR_0),
    kInstruction_sxtab_ls_r1_r7_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r13_r2_r12_ROR_16),
    kInstruction_sxtab_mi_r13_r2_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r1_r7_r2_ROR_16),
    kInstruction_sxtab_ls_r1_r7_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r14_r8_r13_ROR_8),
    kInstruction_sxtab_al_r14_r8_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r1_r0_r10_ROR_0),
    kInstruction_sxtab_lt_r1_r0_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r10_r7_r5_ROR_24),
    kInstruction_sxtab_al_r10_r7_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r12_r14_r10_ROR_0),
    kInstruction_sxtab_vs_r12_r14_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r7_r9_r1_ROR_16),
    kInstruction_sxtab_vs_r7_r9_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r8_r0_r0_ROR_24),
    kInstruction_sxtab_pl_r8_r0_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r12_r12_r5_ROR_8),
    kInstruction_sxtab_le_r12_r12_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r2_r7_r11_ROR_24),
    kInstruction_sxtab_gt_r2_r7_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r2_r0_r4_ROR_24),
    kInstruction_sxtab_ne_r2_r0_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r7_r3_r7_ROR_16),
    kInstruction_sxtab_le_r7_r3_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r3_r13_r8_ROR_16),
    kInstruction_sxtab_vc_r3_r13_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r12_r14_r0_ROR_8),
    kInstruction_sxtab_al_r12_r14_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r4_r9_r1_ROR_0),
    kInstruction_sxtab_le_r4_r9_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r0_r1_r1_ROR_8),
    kInstruction_sxtab_ge_r0_r1_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r0_r11_r13_ROR_16),
    kInstruction_sxtab_pl_r0_r11_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r7_r14_r0_ROR_0),
    kInstruction_sxtab_hi_r7_r14_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r11_r14_r6_ROR_24),
    kInstruction_sxtab_ls_r11_r14_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r2_r9_r12_ROR_0),
    kInstruction_sxtab_ge_r2_r9_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r12_r1_r7_ROR_16),
    kInstruction_sxtab_lt_r12_r1_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r7_r7_r12_ROR_16),
    kInstruction_sxtab_eq_r7_r7_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r2_r9_r14_ROR_24),
    kInstruction_sxtab_ls_r2_r9_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r10_r14_r14_ROR_24),
    kInstruction_sxtab_lt_r10_r14_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r8_r10_r8_ROR_16),
    kInstruction_sxtab_pl_r8_r10_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r8_r12_r2_ROR_8),
    kInstruction_sxtab_ls_r8_r12_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r11_r10_r3_ROR_0),
    kInstruction_sxtab_ne_r11_r10_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r6_r2_r10_ROR_8),
    kInstruction_sxtab_ne_r6_r2_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r10_r7_r12_ROR_16),
    kInstruction_sxtab_mi_r10_r7_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r8_r14_r7_ROR_24),
    kInstruction_sxtab_vc_r8_r14_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r12_r9_r4_ROR_8),
    kInstruction_sxtab_gt_r12_r9_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r1_r4_r11_ROR_16),
    kInstruction_sxtab_pl_r1_r4_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r3_r10_r6_ROR_16),
    kInstruction_sxtab_mi_r3_r10_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r14_r10_r12_ROR_24),
    kInstruction_sxtab_eq_r14_r10_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r7_r9_r13_ROR_8),
    kInstruction_sxtab_eq_r7_r9_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r11_r7_r4_ROR_16),
    kInstruction_sxtab_eq_r11_r7_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r12_r1_r6_ROR_0),
    kInstruction_sxtab_vc_r12_r1_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r6_r0_r2_ROR_0),
    kInstruction_sxtab_pl_r6_r0_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r11_r14_r5_ROR_8),
    kInstruction_sxtab_hi_r11_r14_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r4_r14_r5_ROR_24),
    kInstruction_sxtab_pl_r4_r14_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r5_r1_r14_ROR_0),
    kInstruction_sxtab_eq_r5_r1_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r2_r8_r9_ROR_24),
    kInstruction_sxtab_gt_r2_r8_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r11_r8_r11_ROR_8),
    kInstruction_sxtab_pl_r11_r8_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r7_r7_r8_ROR_16),
    kInstruction_sxtab_cc_r7_r7_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r8_r3_r0_ROR_16),
    kInstruction_sxtab_le_r8_r3_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r4_r5_r9_ROR_8),
    kInstruction_sxtab_hi_r4_r5_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r2_r9_r7_ROR_8),
    kInstruction_sxtab_pl_r2_r9_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r11_r11_r2_ROR_16),
    kInstruction_sxtab_al_r11_r11_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r12_r5_r3_ROR_24),
    kInstruction_sxtab_ne_r12_r5_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r6_r8_r5_ROR_16),
    kInstruction_sxtab_mi_r6_r8_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r14_r0_r7_ROR_0),
    kInstruction_sxtab_le_r14_r0_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r8_r4_r13_ROR_8),
    kInstruction_sxtab_cc_r8_r4_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r1_r11_r3_ROR_8),
    kInstruction_sxtab_al_r1_r11_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r3_r2_r13_ROR_16),
    kInstruction_sxtab_vc_r3_r2_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r11_r14_r8_ROR_16),
    kInstruction_sxtab_vs_r11_r14_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r3_r4_r12_ROR_24),
    kInstruction_sxtab_eq_r3_r4_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r10_r10_r5_ROR_8),
    kInstruction_sxtab_gt_r10_r10_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r6_r10_r4_ROR_16),
    kInstruction_sxtab_hi_r6_r10_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r12_r3_r13_ROR_8),
    kInstruction_sxtab_eq_r12_r3_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r7_r10_r2_ROR_8),
    kInstruction_sxtab_le_r7_r10_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r4_r9_r14_ROR_0),
    kInstruction_sxtab_cs_r4_r9_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r10_r3_r5_ROR_16),
    kInstruction_sxtab_ge_r10_r3_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r1_r10_r11_ROR_0),
    kInstruction_sxtab_cc_r1_r10_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r10_r11_r6_ROR_24),
    kInstruction_sxtab_ls_r10_r11_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r12_r10_r1_ROR_8),
    kInstruction_sxtab_ne_r12_r10_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r10_r4_r7_ROR_24),
    kInstruction_sxtab_lt_r10_r4_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r7_r0_r10_ROR_0),
    kInstruction_sxtab_ge_r7_r0_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r7_r6_r5_ROR_0),
    kInstruction_sxtab_ls_r7_r6_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r8_r1_r8_ROR_8),
    kInstruction_sxtab_cc_r8_r1_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r7_r0_r13_ROR_24),
    kInstruction_sxtab_cs_r7_r0_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r10_r6_r0_ROR_0),
    kInstruction_sxtab_cc_r10_r6_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r0_r13_r6_ROR_8),
    kInstruction_sxtab_mi_r0_r13_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r12_r5_r3_ROR_16),
    kInstruction_sxtab_hi_r12_r5_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r4_r9_r2_ROR_8),
    kInstruction_sxtab_vs_r4_r9_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r9_r7_r12_ROR_24),
    kInstruction_sxtab_vs_r9_r7_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r7_r13_r8_ROR_0),
    kInstruction_sxtab_cc_r7_r13_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r14_r3_r2_ROR_8),
    kInstruction_sxtab_mi_r14_r3_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r11_r14_r2_ROR_8),
    kInstruction_sxtab_vc_r11_r14_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r9_r7_r0_ROR_16),
    kInstruction_sxtab_pl_r9_r7_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r13_r10_r4_ROR_24),
    kInstruction_sxtab_vs_r13_r10_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r13_r8_r10_ROR_16),
    kInstruction_sxtab_vs_r13_r8_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r9_r8_r8_ROR_0),
    kInstruction_sxtab_hi_r9_r8_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r10_r14_r9_ROR_16),
    kInstruction_sxtab_mi_r10_r14_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r4_r2_r5_ROR_24),
    kInstruction_sxtab_pl_r4_r2_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r13_r10_r4_ROR_8),
    kInstruction_sxtab_cc_r13_r10_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r4_r9_r2_ROR_16),
    kInstruction_sxtab_hi_r4_r9_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r13_r5_r0_ROR_16),
    kInstruction_sxtab_cs_r13_r5_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r10_r5_r0_ROR_0),
    kInstruction_sxtab_le_r10_r5_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r10_r7_r7_ROR_24),
    kInstruction_sxtab_al_r10_r7_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r2_r1_r10_ROR_16),
    kInstruction_sxtab_pl_r2_r1_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r7_r11_r0_ROR_24),
    kInstruction_sxtab_le_r7_r11_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r6_r8_r0_ROR_8),
    kInstruction_sxtab_vs_r6_r8_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r6_r9_r2_ROR_8),
    kInstruction_sxtab_al_r6_r9_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r3_r12_r1_ROR_8),
    kInstruction_sxtab_vs_r3_r12_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r14_r5_r11_ROR_16),
    kInstruction_sxtab_le_r14_r5_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r1_r12_r14_ROR_0),
    kInstruction_sxtab_cs_r1_r12_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r14_r2_r2_ROR_24),
    kInstruction_sxtab_ne_r14_r2_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r14_r0_r8_ROR_16),
    kInstruction_sxtab_eq_r14_r0_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r13_r1_r2_ROR_24),
    kInstruction_sxtab_ge_r13_r1_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r2_r1_r10_ROR_8),
    kInstruction_sxtab_lt_r2_r1_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r9_r3_r14_ROR_0),
    kInstruction_sxtab_vc_r9_r3_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r6_r0_r4_ROR_24),
    kInstruction_sxtab_gt_r6_r0_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r4_r12_r2_ROR_0),
    kInstruction_sxtab_cs_r4_r12_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r1_r7_r5_ROR_8),
    kInstruction_sxtab_vc_r1_r7_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r6_r7_r1_ROR_8),
    kInstruction_sxtab_cs_r6_r7_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r1_r10_r7_ROR_8),
    kInstruction_sxtab_cs_r1_r10_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r13_r5_r0_ROR_16),
    kInstruction_sxtab_eq_r13_r5_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r11_r12_r12_ROR_16),
    kInstruction_sxtab_gt_r11_r12_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r7_r12_r9_ROR_16),
    kInstruction_sxtab_le_r7_r12_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r3_r7_r6_ROR_8),
    kInstruction_sxtab_le_r3_r7_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r4_r7_r6_ROR_8),
    kInstruction_sxtab_pl_r4_r7_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r1_r7_r8_ROR_16),
    kInstruction_sxtab_ge_r1_r7_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r4_r6_r4_ROR_0),
    kInstruction_sxtab_vs_r4_r6_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r11_r11_r4_ROR_8),
    kInstruction_sxtab_hi_r11_r11_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r13_r0_r1_ROR_16),
    kInstruction_sxtab_hi_r13_r0_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r0_r3_r1_ROR_0),
    kInstruction_sxtab_lt_r0_r3_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r2_r14_r1_ROR_16),
    kInstruction_sxtab_vs_r2_r14_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r0_r2_r3_ROR_16),
    kInstruction_sxtab_lt_r0_r2_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r8_r14_r2_ROR_0),
    kInstruction_sxtab_cc_r8_r14_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r1_r4_r5_ROR_8),
    kInstruction_sxtab_mi_r1_r4_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r13_r5_r9_ROR_24),
    kInstruction_sxtab_cc_r13_r5_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r0_r14_r3_ROR_8),
    kInstruction_sxtab_cc_r0_r14_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r6_r5_r10_ROR_8),
    kInstruction_sxtab_vc_r6_r5_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r6_r10_r9_ROR_8),
    kInstruction_sxtab_vs_r6_r10_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r5_r14_r3_ROR_16),
    kInstruction_sxtab_gt_r5_r14_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r12_r7_r5_ROR_0),
    kInstruction_sxtab_hi_r12_r7_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r6_r3_r0_ROR_8),
    kInstruction_sxtab_hi_r6_r3_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r12_r3_r1_ROR_8),
    kInstruction_sxtab_le_r12_r3_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r13_r14_r0_ROR_16),
    kInstruction_sxtab_cc_r13_r14_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r14_r14_r11_ROR_24),
    kInstruction_sxtab_ls_r14_r14_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r0_r13_r0_ROR_0),
    kInstruction_sxtab_lt_r0_r13_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r12_r7_r4_ROR_24),
    kInstruction_sxtab_pl_r12_r7_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r11_r1_r11_ROR_16),
    kInstruction_sxtab_mi_r11_r1_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r5_r2_r11_ROR_24),
    kInstruction_sxtab_al_r5_r2_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r3_r3_r4_ROR_24),
    kInstruction_sxtab_mi_r3_r3_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r12_r12_r8_ROR_24),
    kInstruction_sxtab_vc_r12_r12_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r3_r12_r10_ROR_24),
    kInstruction_sxtab_ls_r3_r12_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r0_r0_r4_ROR_0),
    kInstruction_sxtab_ne_r0_r0_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r12_r8_r13_ROR_8),
    kInstruction_sxtab_ne_r12_r8_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r4_r4_r2_ROR_16),
    kInstruction_sxtab_mi_r4_r4_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r8_r2_r10_ROR_16),
    kInstruction_sxtab_ge_r8_r2_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r6_r5_r12_ROR_8),
    kInstruction_sxtab_cs_r6_r5_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r7_r13_r5_ROR_0),
    kInstruction_sxtab_ne_r7_r13_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r4_r3_r1_ROR_24),
    kInstruction_sxtab_ne_r4_r3_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r4_r4_r4_ROR_24),
    kInstruction_sxtab_ls_r4_r4_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r14_r10_r14_ROR_24),
    kInstruction_sxtab_ge_r14_r10_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r4_r7_r2_ROR_24),
    kInstruction_sxtab_lt_r4_r7_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r10_r3_r3_ROR_16),
    kInstruction_sxtab_vs_r10_r3_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r13_r2_r5_ROR_0),
    kInstruction_sxtab_mi_r13_r2_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r14_r0_r0_ROR_0),
    kInstruction_sxtab_cc_r14_r0_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r7_r11_r10_ROR_16),
    kInstruction_sxtab_ne_r7_r11_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r12_r13_r4_ROR_24),
    kInstruction_sxtab_al_r12_r13_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r6_r8_r1_ROR_0),
    kInstruction_sxtab_lt_r6_r8_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r10_r6_r6_ROR_24),
    kInstruction_sxtab_ne_r10_r6_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r12_r5_r0_ROR_0),
    kInstruction_sxtab_le_r12_r5_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r6_r9_r11_ROR_0),
    kInstruction_sxtab_ne_r6_r9_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r10_r5_r10_ROR_16),
    kInstruction_sxtab_hi_r10_r5_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r7_r14_r3_ROR_24),
    kInstruction_sxtab_hi_r7_r14_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r7_r10_r5_ROR_16),
    kInstruction_sxtab_ls_r7_r10_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r11_r4_r4_ROR_0),
    kInstruction_sxtab_vs_r11_r4_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r13_r2_r0_ROR_0),
    kInstruction_sxtab_ls_r13_r2_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r4_r13_r11_ROR_8),
    kInstruction_sxtab_lt_r4_r13_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r2_r12_r12_ROR_0),
    kInstruction_sxtab_ne_r2_r12_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r5_r7_r5_ROR_0),
    kInstruction_sxtab_cc_r5_r7_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r6_r14_r9_ROR_24),
    kInstruction_sxtab_vc_r6_r14_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r10_r11_r12_ROR_8),
    kInstruction_sxtab_ne_r10_r11_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r10_r6_r7_ROR_16),
    kInstruction_sxtab_gt_r10_r6_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r7_r13_r4_ROR_8),
    kInstruction_sxtab_pl_r7_r13_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r4_r8_r1_ROR_0),
    kInstruction_sxtab_ne_r4_r8_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r5_r6_r9_ROR_16),
    kInstruction_sxtab_mi_r5_r6_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r14_r8_r3_ROR_16),
    kInstruction_sxtab_mi_r14_r8_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r1_r14_r14_ROR_16),
    kInstruction_sxtab_lt_r1_r14_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r9_r12_r9_ROR_8),
    kInstruction_sxtab_le_r9_r12_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r10_r2_r2_ROR_16),
    kInstruction_sxtab_le_r10_r2_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r11_r12_r0_ROR_16),
    kInstruction_sxtab_vc_r11_r12_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r5_r12_r1_ROR_16),
    kInstruction_sxtab_vc_r5_r12_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r2_r9_r2_ROR_0),
    kInstruction_sxtab_hi_r2_r9_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r11_r7_r14_ROR_0),
    kInstruction_sxtab_vs_r11_r7_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r11_r2_r14_ROR_8),
    kInstruction_sxtab_eq_r11_r2_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r11_r2_r3_ROR_16),
    kInstruction_sxtab_cs_r11_r2_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r1_r9_r7_ROR_16),
    kInstruction_sxtab_vc_r1_r9_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r14_r11_r9_ROR_16),
    kInstruction_sxtab_mi_r14_r11_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r11_r8_r12_ROR_16),
    kInstruction_sxtab_ge_r11_r8_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r11_r12_r11_ROR_16),
    kInstruction_sxtab_vs_r11_r12_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r6_r5_r14_ROR_8),
    kInstruction_sxtab_al_r6_r5_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r9_r8_r0_ROR_24),
    kInstruction_sxtab_mi_r9_r8_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r4_r2_r11_ROR_24),
    kInstruction_sxtab_vs_r4_r2_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r5_r7_r5_ROR_8),
    kInstruction_sxtab_ne_r5_r7_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r2_r4_r2_ROR_0),
    kInstruction_sxtab_cs_r2_r4_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r10_r4_r8_ROR_0),
    kInstruction_sxtab_ne_r10_r4_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r4_r7_r0_ROR_16),
    kInstruction_sxtab_vs_r4_r7_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r1_r1_r2_ROR_24),
    kInstruction_sxtab_gt_r1_r1_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r4_r13_r10_ROR_24),
    kInstruction_sxtab_pl_r4_r13_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r2_r3_r8_ROR_0),
    kInstruction_sxtab_vs_r2_r3_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r10_r0_r10_ROR_16),
    kInstruction_sxtab_lt_r10_r0_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r7_r12_r4_ROR_0),
    kInstruction_sxtab_cs_r7_r12_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r4_r6_r5_ROR_8),
    kInstruction_sxtab_lt_r4_r6_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r14_r11_r2_ROR_24),
    kInstruction_sxtab_pl_r14_r11_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r10_r7_r6_ROR_0),
    kInstruction_sxtab_ge_r10_r7_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r8_r5_r10_ROR_24),
    kInstruction_sxtab_pl_r8_r5_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r2_r5_r5_ROR_0),
    kInstruction_sxtab_mi_r2_r5_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r6_r13_r2_ROR_24),
    kInstruction_sxtab_mi_r6_r13_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r5_r8_r4_ROR_8),
    kInstruction_sxtab_vs_r5_r8_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r14_r13_r10_ROR_24),
    kInstruction_sxtab_mi_r14_r13_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r12_r0_r7_ROR_24),
    kInstruction_sxtab_vs_r12_r0_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r2_r11_r1_ROR_24),
    kInstruction_sxtab_lt_r2_r11_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r8_r1_r2_ROR_16),
    kInstruction_sxtab_ls_r8_r1_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r9_r14_r7_ROR_16),
    kInstruction_sxtab_vs_r9_r14_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r12_r1_r14_ROR_16),
    kInstruction_sxtab_ls_r12_r1_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r8_r12_r7_ROR_24),
    kInstruction_sxtab_le_r8_r12_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r6_r6_r0_ROR_8),
    kInstruction_sxtab_pl_r6_r6_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r5_r13_r6_ROR_24),
    kInstruction_sxtab_ne_r5_r13_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r3_r10_r5_ROR_16),
    kInstruction_sxtab_lt_r3_r10_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r6_r11_r7_ROR_16),
    kInstruction_sxtab_eq_r6_r11_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r10_r13_r4_ROR_0),
    kInstruction_sxtab_cs_r10_r13_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r13_r13_r0_ROR_24),
    kInstruction_sxtab_le_r13_r13_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r4_r12_r9_ROR_0),
    kInstruction_sxtab_vc_r4_r12_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r3_r2_r4_ROR_16),
    kInstruction_sxtab_gt_r3_r2_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r14_r11_r9_ROR_8),
    kInstruction_sxtab_cs_r14_r11_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r14_r1_r13_ROR_16),
    kInstruction_sxtab_lt_r14_r1_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r6_r4_r12_ROR_8),
    kInstruction_sxtab_cs_r6_r4_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r10_r13_r5_ROR_8),
    kInstruction_sxtab_lt_r10_r13_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r6_r12_r12_ROR_0),
    kInstruction_sxtab_mi_r6_r12_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r12_r1_r12_ROR_16),
    kInstruction_sxtab_ge_r12_r1_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r7_r11_r13_ROR_8),
    kInstruction_sxtab_hi_r7_r11_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r3_r11_r11_ROR_0),
    kInstruction_sxtab_vs_r3_r11_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r6_r12_r12_ROR_0),
    kInstruction_sxtab_vc_r6_r12_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r10_r0_r13_ROR_8),
    kInstruction_sxtab_gt_r10_r0_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r4_r7_r5_ROR_16),
    kInstruction_sxtab_pl_r4_r7_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r11_r4_r13_ROR_0),
    kInstruction_sxtab_cc_r11_r4_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r14_r13_r13_ROR_24),
    kInstruction_sxtab_gt_r14_r13_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r7_r11_r11_ROR_8),
    kInstruction_sxtab_lt_r7_r11_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r12_r7_r0_ROR_16),
    kInstruction_sxtab_ge_r12_r7_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r10_r12_r13_ROR_0),
    kInstruction_sxtab_ne_r10_r12_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r12_r9_r0_ROR_8),
    kInstruction_sxtab_gt_r12_r9_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r10_r14_r6_ROR_16),
    kInstruction_sxtab_pl_r10_r14_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r6_r5_r2_ROR_24),
    kInstruction_sxtab_vc_r6_r5_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r12_r7_r5_ROR_16),
    kInstruction_sxtab_gt_r12_r7_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r11_r9_r0_ROR_16),
    kInstruction_sxtab_eq_r11_r9_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r7_r7_r4_ROR_16),
    kInstruction_sxtab_lt_r7_r7_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r5_r4_r10_ROR_8),
    kInstruction_sxtab_gt_r5_r4_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r10_r2_r13_ROR_16),
    kInstruction_sxtab_hi_r10_r2_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r13_r5_r5_ROR_24),
    kInstruction_sxtab_al_r13_r5_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r12_r0_r7_ROR_24),
    kInstruction_sxtab_cs_r12_r0_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r1_r13_r10_ROR_16),
    kInstruction_sxtab_hi_r1_r13_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r13_r8_r10_ROR_8),
    kInstruction_sxtab_eq_r13_r8_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r6_r8_r3_ROR_8),
    kInstruction_sxtab_le_r6_r8_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r13_r12_r12_ROR_0),
    kInstruction_sxtab_lt_r13_r12_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r9_r10_r9_ROR_0),
    kInstruction_sxtab_ls_r9_r10_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r1_r3_r13_ROR_16),
    kInstruction_sxtab_eq_r1_r3_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r12_r12_r9_ROR_8),
    kInstruction_sxtab_vs_r12_r12_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r7_r8_r7_ROR_8),
    kInstruction_sxtab_vc_r7_r8_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r6_r0_r14_ROR_24),
    kInstruction_sxtab_eq_r6_r0_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r0_r13_r10_ROR_8),
    kInstruction_sxtab_le_r0_r13_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r11_r6_r3_ROR_8),
    kInstruction_sxtab_ge_r11_r6_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r2_r2_r8_ROR_0),
    kInstruction_sxtab_gt_r2_r2_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r6_r4_r0_ROR_8),
    kInstruction_sxtab_le_r6_r4_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r2_r8_r12_ROR_16),
    kInstruction_sxtab_le_r2_r8_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r10_r1_r4_ROR_24),
    kInstruction_sxtab_le_r10_r1_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r4_r1_r10_ROR_16),
    kInstruction_sxtab_cc_r4_r1_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r9_r4_r9_ROR_16),
    kInstruction_sxtab_pl_r9_r4_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r8_r4_r12_ROR_0),
    kInstruction_sxtab_mi_r8_r4_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r5_r0_r10_ROR_24),
    kInstruction_sxtab_lt_r5_r0_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r5_r0_r8_ROR_0),
    kInstruction_sxtab_hi_r5_r0_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r4_r9_r3_ROR_8),
    kInstruction_sxtab_cs_r4_r9_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r2_r5_r1_ROR_8),
    kInstruction_sxtab_vs_r2_r5_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r10_r13_r9_ROR_8),
    kInstruction_sxtab_ne_r10_r13_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r0_r5_r11_ROR_8),
    kInstruction_sxtab_cs_r0_r5_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r8_r9_r8_ROR_8),
    kInstruction_sxtab_mi_r8_r9_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r3_r14_r2_ROR_0),
    kInstruction_sxtab_al_r3_r14_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r9_r10_r9_ROR_24),
    kInstruction_sxtab_hi_r9_r10_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r14_r3_r1_ROR_8),
    kInstruction_sxtab_lt_r14_r3_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r11_r12_r14_ROR_16),
    kInstruction_sxtab_pl_r11_r12_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r12_r1_r6_ROR_0),
    kInstruction_sxtab_eq_r12_r1_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r9_r12_r1_ROR_8),
    kInstruction_sxtab_ls_r9_r12_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r8_r6_r10_ROR_24),
    kInstruction_sxtab_pl_r8_r6_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r1_r10_r14_ROR_8),
    kInstruction_sxtab_pl_r1_r10_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r10_r7_r12_ROR_0),
    kInstruction_sxtab_mi_r10_r7_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r9_r2_r7_ROR_24),
    kInstruction_sxtab_al_r9_r2_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r2_r9_r8_ROR_16),
    kInstruction_sxtab_ls_r2_r9_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r5_r12_r11_ROR_16),
    kInstruction_sxtab_ls_r5_r12_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r9_r10_r11_ROR_24),
    kInstruction_sxtab_mi_r9_r10_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r10_r2_r10_ROR_8),
    kInstruction_sxtab_mi_r10_r2_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r7_r5_r4_ROR_24),
    kInstruction_sxtab_gt_r7_r5_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r11_r9_r6_ROR_16),
    kInstruction_sxtab_le_r11_r9_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r13_r2_r10_ROR_16),
    kInstruction_sxtab_ne_r13_r2_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r8_r8_r13_ROR_0),
    kInstruction_sxtab_ge_r8_r8_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r0_r14_r11_ROR_8),
    kInstruction_sxtab_al_r0_r14_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r3_r5_r14_ROR_0),
    kInstruction_sxtab_mi_r3_r5_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r3_r12_r10_ROR_16),
    kInstruction_sxtab_ne_r3_r12_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r4_r14_r12_ROR_0),
    kInstruction_sxtab_le_r4_r14_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r14_r9_r6_ROR_16),
    kInstruction_sxtab_vs_r14_r9_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r9_r3_r11_ROR_8),
    kInstruction_sxtab_cs_r9_r3_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r12_r14_r13_ROR_8),
    kInstruction_sxtab_le_r12_r14_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r6_r12_r5_ROR_24),
    kInstruction_sxtab_eq_r6_r12_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r2_r11_r11_ROR_8),
    kInstruction_sxtab_cc_r2_r11_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r9_r3_r7_ROR_0),
    kInstruction_sxtab_vs_r9_r3_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r5_r6_r11_ROR_8),
    kInstruction_sxtab_le_r5_r6_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r5_r14_r9_ROR_16),
    kInstruction_sxtab_cc_r5_r14_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r11_r14_r4_ROR_24),
    kInstruction_sxtab_ne_r11_r14_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r0_r14_r13_ROR_16),
    kInstruction_sxtab_gt_r0_r14_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r0_r12_r13_ROR_24),
    kInstruction_sxtab_cc_r0_r12_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r14_r11_r8_ROR_24),
    kInstruction_sxtab_vs_r14_r11_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r12_r4_r6_ROR_0),
    kInstruction_sxtab_ge_r12_r4_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r4_r0_r4_ROR_0),
    kInstruction_sxtab_gt_r4_r0_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r2_r3_r6_ROR_0),
    kInstruction_sxtab_hi_r2_r3_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r3_r13_r6_ROR_0),
    kInstruction_sxtab_gt_r3_r13_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r2_r5_r10_ROR_0),
    kInstruction_sxtab_cc_r2_r5_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r3_r10_r9_ROR_8),
    kInstruction_sxtab_ne_r3_r10_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r8_r4_r12_ROR_8),
    kInstruction_sxtab_al_r8_r4_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r14_r0_r2_ROR_0),
    kInstruction_sxtab_cc_r14_r0_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r5_r5_r12_ROR_24),
    kInstruction_sxtab_hi_r5_r5_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r0_r8_r8_ROR_8),
    kInstruction_sxtab_cc_r0_r8_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r0_r9_r12_ROR_24),
    kInstruction_sxtab_vc_r0_r9_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r8_r2_r11_ROR_16),
    kInstruction_sxtab_vc_r8_r2_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r13_r13_r7_ROR_24),
    kInstruction_sxtab_ne_r13_r13_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r4_r4_r1_ROR_0),
    kInstruction_sxtab_al_r4_r4_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r1_r9_r10_ROR_16),
    kInstruction_sxtab_cc_r1_r9_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r2_r0_r5_ROR_24),
    kInstruction_sxtab_cs_r2_r0_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r11_r11_r9_ROR_24),
    kInstruction_sxtab_hi_r11_r11_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r12_r2_r7_ROR_24),
    kInstruction_sxtab_ls_r12_r2_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r4_r2_r6_ROR_16),
    kInstruction_sxtab_le_r4_r2_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r8_r0_r14_ROR_8),
    kInstruction_sxtab_ls_r8_r0_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r5_r9_r0_ROR_8),
    kInstruction_sxtab_ne_r5_r9_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r7_r8_r14_ROR_8),
    kInstruction_sxtab_ge_r7_r8_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r14_r6_r13_ROR_8),
    kInstruction_sxtab_ne_r14_r6_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r13_r2_r9_ROR_0),
    kInstruction_sxtab_cs_r13_r2_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r9_r0_r0_ROR_8),
    kInstruction_sxtab_hi_r9_r0_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r12_r9_r3_ROR_16),
    kInstruction_sxtab_vs_r12_r9_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r2_r14_r8_ROR_24),
    kInstruction_sxtab_ne_r2_r14_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r13_r14_r2_ROR_8),
    kInstruction_sxtab_le_r13_r14_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r9_r7_r12_ROR_16),
    kInstruction_sxtab_gt_r9_r7_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r2_r13_r7_ROR_16),
    kInstruction_sxtab_ne_r2_r13_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r10_r11_r13_ROR_24),
    kInstruction_sxtab_pl_r10_r11_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r5_r3_r8_ROR_24),
    kInstruction_sxtab_mi_r5_r3_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r12_r8_r2_ROR_16),
    kInstruction_sxtab_le_r12_r8_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r4_r10_r9_ROR_8),
    kInstruction_sxtab_ls_r4_r10_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r6_r11_r8_ROR_24),
    kInstruction_sxtab_eq_r6_r11_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r11_r12_r5_ROR_24),
    kInstruction_sxtab_le_r11_r12_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r0_r10_r0_ROR_8),
    kInstruction_sxtab_gt_r0_r10_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r9_r4_r9_ROR_0),
    kInstruction_sxtab_pl_r9_r4_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r14_r2_r8_ROR_24),
    kInstruction_sxtab_vc_r14_r2_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r7_r0_r7_ROR_0),
    kInstruction_sxtab_gt_r7_r0_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r12_r12_r9_ROR_0),
    kInstruction_sxtab_vc_r12_r12_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r2_r12_r3_ROR_0),
    kInstruction_sxtab_pl_r2_r12_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r10_r2_r0_ROR_16),
    kInstruction_sxtab_cs_r10_r2_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r6_r5_r10_ROR_16),
    kInstruction_sxtab_vs_r6_r5_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r5_r14_r6_ROR_24),
    kInstruction_sxtab_pl_r5_r14_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r12_r5_r9_ROR_16),
    kInstruction_sxtab_al_r12_r5_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r3_r0_r3_ROR_24),
    kInstruction_sxtab_ge_r3_r0_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r4_r11_r12_ROR_8),
    kInstruction_sxtab_le_r4_r11_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r10_r10_r13_ROR_16),
    kInstruction_sxtab_pl_r10_r10_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r1_r8_r3_ROR_8),
    kInstruction_sxtab_al_r1_r8_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r8_r6_r12_ROR_16),
    kInstruction_sxtab_cc_r8_r6_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r13_r5_r6_ROR_16),
    kInstruction_sxtab_vs_r13_r5_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r10_r6_r2_ROR_8),
    kInstruction_sxtab_eq_r10_r6_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r3_r3_r9_ROR_24),
    kInstruction_sxtab_gt_r3_r3_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r4_r6_r9_ROR_0),
    kInstruction_sxtab_hi_r4_r6_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r7_r6_r8_ROR_16),
    kInstruction_sxtab_vs_r7_r6_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r6_r11_r0_ROR_24),
    kInstruction_sxtab_hi_r6_r11_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r2_r6_r4_ROR_24),
    kInstruction_sxtab_ge_r2_r6_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r5_r2_r10_ROR_16),
    kInstruction_sxtab_pl_r5_r2_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r3_r13_r5_ROR_16),
    kInstruction_sxtab_hi_r3_r13_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r6_r7_r7_ROR_24),
    kInstruction_sxtab_lt_r6_r7_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r13_r9_r10_ROR_24),
    kInstruction_sxtab_ne_r13_r9_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r6_r0_r11_ROR_0),
    kInstruction_sxtab_mi_r6_r0_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r9_r3_r3_ROR_24),
    kInstruction_sxtab_ge_r9_r3_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r10_r11_r3_ROR_16),
    kInstruction_sxtab_ls_r10_r11_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r13_r13_r7_ROR_8),
    kInstruction_sxtab_al_r13_r13_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r9_r11_r2_ROR_16),
    kInstruction_sxtab_lt_r9_r11_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r10_r9_r8_ROR_24),
    kInstruction_sxtab_ls_r10_r9_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r6_r12_r9_ROR_8),
    kInstruction_sxtab_cs_r6_r12_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r14_r5_r0_ROR_0),
    kInstruction_sxtab_le_r14_r5_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r8_r1_r1_ROR_8),
    kInstruction_sxtab_le_r8_r1_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r2_r0_r3_ROR_8),
    kInstruction_sxtab_hi_r2_r0_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r4_r8_r1_ROR_8),
    kInstruction_sxtab_ne_r4_r8_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r13_r8_r9_ROR_8),
    kInstruction_sxtab_al_r13_r8_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r4_r10_r13_ROR_16),
    kInstruction_sxtab_mi_r4_r10_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r10_r5_r5_ROR_24),
    kInstruction_sxtab_cc_r10_r5_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r0_r12_r3_ROR_24),
    kInstruction_sxtab_ls_r0_r12_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r4_r11_r11_ROR_24),
    kInstruction_sxtab_vc_r4_r11_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r5_r12_r0_ROR_24),
    kInstruction_sxtab_hi_r5_r12_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r7_r3_r9_ROR_0),
    kInstruction_sxtab_le_r7_r3_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r14_r7_r9_ROR_24),
    kInstruction_sxtab_vs_r14_r7_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r2_r2_r9_ROR_24),
    kInstruction_sxtab_ne_r2_r2_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r5_r7_r11_ROR_16),
    kInstruction_sxtab_ls_r5_r7_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r0_r8_r4_ROR_0),
    kInstruction_sxtab_ls_r0_r8_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r5_r13_r0_ROR_0),
    kInstruction_sxtab_hi_r5_r13_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r7_r12_r7_ROR_8),
    kInstruction_sxtab_mi_r7_r12_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r3_r1_r3_ROR_0),
    kInstruction_sxtab_ne_r3_r1_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r9_r11_r8_ROR_16),
    kInstruction_sxtab_ne_r9_r11_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r12_r4_r14_ROR_0),
    kInstruction_sxtab_ge_r12_r4_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r11_r1_r4_ROR_24),
    kInstruction_sxtab_ne_r11_r1_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r9_r9_r13_ROR_0),
    kInstruction_sxtab_ge_r9_r9_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r13_r9_r14_ROR_16),
    kInstruction_sxtab_gt_r13_r9_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r10_r0_r12_ROR_16),
    kInstruction_sxtab_vs_r10_r0_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r10_r7_r1_ROR_24),
    kInstruction_sxtab_pl_r10_r7_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r14_r12_r10_ROR_0),
    kInstruction_sxtab_al_r14_r12_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r10_r8_r10_ROR_8),
    kInstruction_sxtab_cc_r10_r8_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r14_r2_r14_ROR_16),
    kInstruction_sxtab_eq_r14_r2_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r3_r12_r8_ROR_16),
    kInstruction_sxtab_vc_r3_r12_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r13_r4_r7_ROR_0),
    kInstruction_sxtab_gt_r13_r4_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r12_r6_r11_ROR_16),
    kInstruction_sxtab_ne_r12_r6_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r1_r7_r12_ROR_0),
    kInstruction_sxtab_vs_r1_r7_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r14_r3_r7_ROR_0),
    kInstruction_sxtab_hi_r14_r3_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r4_r2_r0_ROR_16),
    kInstruction_sxtab_ne_r4_r2_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r11_r3_r11_ROR_24),
    kInstruction_sxtab_cs_r11_r3_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r10_r2_r0_ROR_24),
    kInstruction_sxtab_le_r10_r2_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r8_r11_r0_ROR_8),
    kInstruction_sxtab_vs_r8_r11_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r12_r7_r7_ROR_8),
    kInstruction_sxtab_le_r12_r7_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r8_r6_r13_ROR_16),
    kInstruction_sxtab_eq_r8_r6_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r1_r12_r7_ROR_24),
    kInstruction_sxtab_hi_r1_r12_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r9_r7_r14_ROR_8),
    kInstruction_sxtab_gt_r9_r7_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r10_r2_r7_ROR_8),
    kInstruction_sxtab_pl_r10_r2_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r0_r12_r3_ROR_0),
    kInstruction_sxtab_cc_r0_r12_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r3_r0_r0_ROR_8),
    kInstruction_sxtab_ge_r3_r0_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r10_r14_r1_ROR_16),
    kInstruction_sxtab_lt_r10_r14_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r0_r0_r13_ROR_8),
    kInstruction_sxtab_vc_r0_r0_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r2_r12_r0_ROR_8),
    kInstruction_sxtab_vc_r2_r12_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r1_r7_r7_ROR_24),
    kInstruction_sxtab_gt_r1_r7_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r8_r12_r0_ROR_24),
    kInstruction_sxtab_cc_r8_r12_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r0_r10_r12_ROR_8),
    kInstruction_sxtab_vs_r0_r10_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r0_r1_r11_ROR_8),
    kInstruction_sxtab_le_r0_r1_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r9_r12_r9_ROR_8),
    kInstruction_sxtab_vs_r9_r12_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r7_r7_r13_ROR_24),
    kInstruction_sxtab_mi_r7_r7_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r1_r4_r3_ROR_0),
    kInstruction_sxtab_cc_r1_r4_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r1_r12_r4_ROR_16),
    kInstruction_sxtab_pl_r1_r12_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r4_r9_r13_ROR_24),
    kInstruction_sxtab_mi_r4_r9_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r12_r14_r14_ROR_16),
    kInstruction_sxtab_eq_r12_r14_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r10_r0_r4_ROR_0),
    kInstruction_sxtab_ls_r10_r0_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r3_r6_r12_ROR_8),
    kInstruction_sxtab_hi_r3_r6_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r10_r0_r4_ROR_8),
    kInstruction_sxtab_le_r10_r0_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r12_r3_r2_ROR_8),
    kInstruction_sxtab_lt_r12_r3_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r13_r11_r10_ROR_16),
    kInstruction_sxtab_pl_r13_r11_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r13_r14_r14_ROR_24),
    kInstruction_sxtab_ne_r13_r14_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r9_r9_r0_ROR_8),
    kInstruction_sxtab_eq_r9_r9_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r12_r10_r0_ROR_8),
    kInstruction_sxtab_eq_r12_r10_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r0_r0_r3_ROR_0),
    kInstruction_sxtab_ne_r0_r0_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r12_r14_r4_ROR_0),
    kInstruction_sxtab_gt_r12_r14_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r1_r2_r14_ROR_0),
    kInstruction_sxtab_pl_r1_r2_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r1_r8_r8_ROR_8),
    kInstruction_sxtab_lt_r1_r8_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r9_r0_r2_ROR_16),
    kInstruction_sxtab_cc_r9_r0_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r9_r0_r5_ROR_8),
    kInstruction_sxtab_cs_r9_r0_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r13_r8_r5_ROR_24),
    kInstruction_sxtab_mi_r13_r8_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r1_r1_r7_ROR_8),
    kInstruction_sxtab_vc_r1_r1_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r6_r5_r5_ROR_16),
    kInstruction_sxtab_cs_r6_r5_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r1_r13_r3_ROR_24),
    kInstruction_sxtab_le_r1_r13_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r11_r7_r3_ROR_24),
    kInstruction_sxtab_gt_r11_r7_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r3_r3_r5_ROR_0),
    kInstruction_sxtab_al_r3_r3_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r5_r7_r14_ROR_24),
    kInstruction_sxtab_hi_r5_r7_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r1_r9_r4_ROR_24),
    kInstruction_sxtab_ne_r1_r9_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r7_r3_r11_ROR_8),
    kInstruction_sxtab_hi_r7_r3_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r1_r8_r8_ROR_8),
    kInstruction_sxtab_al_r1_r8_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r3_r8_r1_ROR_16),
    kInstruction_sxtab_hi_r3_r8_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r9_r9_r10_ROR_16),
    kInstruction_sxtab_ge_r9_r9_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r11_r1_r0_ROR_24),
    kInstruction_sxtab_le_r11_r1_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r13_r12_r9_ROR_0),
    kInstruction_sxtab_ls_r13_r12_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r8_r6_r7_ROR_24),
    kInstruction_sxtab_hi_r8_r6_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r8_r13_r11_ROR_16),
    kInstruction_sxtab_lt_r8_r13_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r3_r9_r9_ROR_0),
    kInstruction_sxtab_ge_r3_r9_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r5_r7_r4_ROR_24),
    kInstruction_sxtab_ne_r5_r7_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r3_r12_r2_ROR_24),
    kInstruction_sxtab_al_r3_r12_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r0_r4_r13_ROR_0),
    kInstruction_sxtab_ne_r0_r4_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r14_r8_r9_ROR_0),
    kInstruction_sxtab_ge_r14_r8_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r9_r13_r12_ROR_16),
    kInstruction_sxtab_ne_r9_r13_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r11_r1_r4_ROR_24),
    kInstruction_sxtab_pl_r11_r1_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r2_r12_r10_ROR_16),
    kInstruction_sxtab_ne_r2_r12_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r1_r4_r11_ROR_0),
    kInstruction_sxtab_gt_r1_r4_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r7_r6_r13_ROR_16),
    kInstruction_sxtab_lt_r7_r6_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r2_r4_r6_ROR_8),
    kInstruction_sxtab_hi_r2_r4_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r12_r0_r9_ROR_0),
    kInstruction_sxtab_al_r12_r0_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r0_r14_r3_ROR_24),
    kInstruction_sxtab_lt_r0_r14_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r5_r8_r10_ROR_16),
    kInstruction_sxtab_mi_r5_r8_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r4_r1_r4_ROR_8),
    kInstruction_sxtab_vc_r4_r1_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r3_r4_r8_ROR_24),
    kInstruction_sxtab_cc_r3_r4_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r11_r0_r0_ROR_16),
    kInstruction_sxtab_cs_r11_r0_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r9_r2_r11_ROR_0),
    kInstruction_sxtab_le_r9_r2_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r10_r7_r8_ROR_8),
    kInstruction_sxtab_vc_r10_r7_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r0_r3_r1_ROR_24),
    kInstruction_sxtab_le_r0_r3_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r4_r8_r2_ROR_0),
    kInstruction_sxtab_ne_r4_r8_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r6_r4_r13_ROR_0),
    kInstruction_sxtab_gt_r6_r4_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r10_r5_r11_ROR_8),
    kInstruction_sxtab_ge_r10_r5_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r11_r14_r5_ROR_24),
    kInstruction_sxtab_ge_r11_r14_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r14_r8_r3_ROR_8),
    kInstruction_sxtab_gt_r14_r8_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r2_r7_r3_ROR_24),
    kInstruction_sxtab_al_r2_r7_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r5_r3_r4_ROR_8),
    kInstruction_sxtab_le_r5_r3_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r14_r7_r2_ROR_16),
    kInstruction_sxtab_ls_r14_r7_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r11_r7_r0_ROR_24),
    kInstruction_sxtab_vc_r11_r7_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r8_r6_r12_ROR_8),
    kInstruction_sxtab_al_r8_r6_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r3_r7_r11_ROR_16),
    kInstruction_sxtab_ne_r3_r7_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r3_r0_r9_ROR_24),
    kInstruction_sxtab_ls_r3_r0_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r7_r12_r11_ROR_8),
    kInstruction_sxtab_vs_r7_r12_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r2_r5_r2_ROR_16),
    kInstruction_sxtab_ge_r2_r5_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r14_r14_r4_ROR_24),
    kInstruction_sxtab_vs_r14_r14_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r3_r8_r12_ROR_24),
    kInstruction_sxtab_pl_r3_r8_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r1_r6_r3_ROR_16),
    kInstruction_sxtab_hi_r1_r6_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r2_r5_r11_ROR_24),
    kInstruction_sxtab_pl_r2_r5_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r5_r2_r3_ROR_0),
    kInstruction_sxtab_vs_r5_r2_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r3_r14_r10_ROR_0),
    kInstruction_sxtab_al_r3_r14_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r12_r8_r1_ROR_24),
    kInstruction_sxtab_cs_r12_r8_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r4_r12_r7_ROR_8),
    kInstruction_sxtab_mi_r4_r12_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r12_r2_r5_ROR_8),
    kInstruction_sxtab_ls_r12_r2_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r7_r11_r9_ROR_0),
    kInstruction_sxtab_ge_r7_r11_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r4_r12_r14_ROR_24),
    kInstruction_sxtab_lt_r4_r12_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r5_r11_r2_ROR_8),
    kInstruction_sxtab_vs_r5_r11_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r6_r6_r2_ROR_16),
    kInstruction_sxtab_lt_r6_r6_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r10_r10_r14_ROR_8),
    kInstruction_sxtab_lt_r10_r10_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r6_r5_r5_ROR_24),
    kInstruction_sxtab_gt_r6_r5_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r13_r14_r4_ROR_24),
    kInstruction_sxtab_ls_r13_r14_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r12_r13_r10_ROR_24),
    kInstruction_sxtab_ls_r12_r13_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r8_r1_r8_ROR_16),
    kInstruction_sxtab_pl_r8_r1_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r12_r2_r2_ROR_24),
    kInstruction_sxtab_ge_r12_r2_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r6_r13_r13_ROR_8),
    kInstruction_sxtab_pl_r6_r13_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r9_r10_r6_ROR_8),
    kInstruction_sxtab_pl_r9_r10_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r0_r2_r1_ROR_16),
    kInstruction_sxtab_hi_r0_r2_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r13_r1_r8_ROR_8),
    kInstruction_sxtab_pl_r13_r1_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r9_r13_r3_ROR_8),
    kInstruction_sxtab_le_r9_r13_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r8_r5_r8_ROR_16),
    kInstruction_sxtab_ne_r8_r5_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r1_r9_r5_ROR_24),
    kInstruction_sxtab_pl_r1_r9_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r1_r9_r5_ROR_16),
    kInstruction_sxtab_hi_r1_r9_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r3_r7_r12_ROR_16),
    kInstruction_sxtab_mi_r3_r7_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r0_r10_r11_ROR_8),
    kInstruction_sxtab_al_r0_r10_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r14_r2_r13_ROR_16),
    kInstruction_sxtab_lt_r14_r2_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r8_r7_r0_ROR_0),
    kInstruction_sxtab_al_r8_r7_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r13_r14_r9_ROR_0),
    kInstruction_sxtab_mi_r13_r14_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r12_r13_r1_ROR_16),
    kInstruction_sxtab_lt_r12_r13_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r14_r7_r2_ROR_16),
    kInstruction_sxtab_mi_r14_r7_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r2_r4_r7_ROR_8),
    kInstruction_sxtab_hi_r2_r4_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r13_r12_r7_ROR_24),
    kInstruction_sxtab_le_r13_r12_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r1_r5_r7_ROR_24),
    kInstruction_sxtab_mi_r1_r5_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r9_r10_r10_ROR_16),
    kInstruction_sxtab_cc_r9_r10_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r8_r8_r11_ROR_24),
    kInstruction_sxtab_eq_r8_r8_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r4_r13_r1_ROR_24),
    kInstruction_sxtab_hi_r4_r13_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r3_r14_r3_ROR_8),
    kInstruction_sxtab_vs_r3_r14_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r8_r6_r11_ROR_8),
    kInstruction_sxtab_lt_r8_r6_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r5_r11_r1_ROR_8),
    kInstruction_sxtab_ls_r5_r11_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r13_r5_r10_ROR_0),
    kInstruction_sxtab_vs_r13_r5_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r9_r5_r4_ROR_0),
    kInstruction_sxtab_al_r9_r5_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r8_r6_r5_ROR_24),
    kInstruction_sxtab_le_r8_r6_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r3_r14_r10_ROR_8),
    kInstruction_sxtab_hi_r3_r14_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r9_r2_r11_ROR_16),
    kInstruction_sxtab_mi_r9_r2_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r5_r6_r6_ROR_24),
    kInstruction_sxtab_cs_r5_r6_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r2_r10_r4_ROR_16),
    kInstruction_sxtab_ls_r2_r10_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r2_r11_r1_ROR_0),
    kInstruction_sxtab_al_r2_r11_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r4_r5_r5_ROR_0),
    kInstruction_sxtab_al_r4_r5_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r3_r0_r3_ROR_8),
    kInstruction_sxtab_cc_r3_r0_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r14_r9_r11_ROR_8),
    kInstruction_sxtab_hi_r14_r9_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r0_r1_r13_ROR_8),
    kInstruction_sxtab_cs_r0_r1_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r9_r6_r2_ROR_24),
    kInstruction_sxtab_vs_r9_r6_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r9_r12_r2_ROR_0),
    kInstruction_sxtab_lt_r9_r12_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r14_r12_r2_ROR_24),
    kInstruction_sxtab_hi_r14_r12_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r0_r10_r1_ROR_8),
    kInstruction_sxtab_lt_r0_r10_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r0_r12_r0_ROR_24),
    kInstruction_sxtab_vs_r0_r12_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r4_r2_r9_ROR_16),
    kInstruction_sxtab_mi_r4_r2_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r5_r4_r13_ROR_24),
    kInstruction_sxtab_pl_r5_r4_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r9_r9_r2_ROR_8),
    kInstruction_sxtab_cs_r9_r9_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r8_r6_r6_ROR_8),
    kInstruction_sxtab_vs_r8_r6_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r12_r2_r8_ROR_16),
    kInstruction_sxtab_cc_r12_r2_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r4_r0_r11_ROR_0),
    kInstruction_sxtab_ge_r4_r0_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r13_r3_r4_ROR_8),
    kInstruction_sxtab_hi_r13_r3_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r5_r8_r9_ROR_24),
    kInstruction_sxtab_cs_r5_r8_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r10_r2_r14_ROR_24),
    kInstruction_sxtab_cc_r10_r2_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r10_r1_r13_ROR_0),
    kInstruction_sxtab_pl_r10_r1_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r4_r13_r13_ROR_8),
    kInstruction_sxtab_ne_r4_r13_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r13_r11_r14_ROR_16),
    kInstruction_sxtab_pl_r13_r11_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r10_r6_r11_ROR_24),
    kInstruction_sxtab_cs_r10_r6_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r10_r14_r14_ROR_16),
    kInstruction_sxtab_hi_r10_r14_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r5_r1_r1_ROR_24),
    kInstruction_sxtab_eq_r5_r1_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r11_r5_r1_ROR_8),
    kInstruction_sxtab_lt_r11_r5_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r7_r11_r13_ROR_16),
    kInstruction_sxtab_gt_r7_r11_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r1_r5_r10_ROR_16),
    kInstruction_sxtab_mi_r1_r5_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r10_r1_r13_ROR_0),
    kInstruction_sxtab_gt_r10_r1_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r14_r2_r13_ROR_0),
    kInstruction_sxtab_cs_r14_r2_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r10_r14_r6_ROR_16),
    kInstruction_sxtab_hi_r10_r14_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r0_r6_r9_ROR_16),
    kInstruction_sxtab_hi_r0_r6_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r14_r4_r9_ROR_8),
    kInstruction_sxtab_gt_r14_r4_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r1_r6_r11_ROR_8),
    kInstruction_sxtab_mi_r1_r6_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r7_r2_r8_ROR_16),
    kInstruction_sxtab_gt_r7_r2_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r6_r9_r0_ROR_8),
    kInstruction_sxtab_cc_r6_r9_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r5_r3_r3_ROR_16),
    kInstruction_sxtab_ls_r5_r3_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r0_r6_r7_ROR_0),
    kInstruction_sxtab_al_r0_r6_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r10_r0_r2_ROR_0),
    kInstruction_sxtab_lt_r10_r0_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r13_r8_r0_ROR_0),
    kInstruction_sxtab_le_r13_r8_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r0_r8_r0_ROR_24),
    kInstruction_sxtab_al_r0_r8_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r8_r8_r5_ROR_16),
    kInstruction_sxtab_hi_r8_r8_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r9_r10_r13_ROR_24),
    kInstruction_sxtab_mi_r9_r10_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r13_r12_r4_ROR_24),
    kInstruction_sxtab_eq_r13_r12_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r1_r14_r14_ROR_16),
    kInstruction_sxtab_cs_r1_r14_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r8_r6_r11_ROR_0),
    kInstruction_sxtab_cs_r8_r6_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r11_r4_r13_ROR_16),
    kInstruction_sxtab_cc_r11_r4_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r3_r4_r5_ROR_0),
    kInstruction_sxtab_hi_r3_r4_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r7_r3_r9_ROR_16),
    kInstruction_sxtab_pl_r7_r3_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r5_r1_r0_ROR_16),
    kInstruction_sxtab_lt_r5_r1_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r9_r7_r9_ROR_24),
    kInstruction_sxtab_gt_r9_r7_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r7_r14_r5_ROR_24),
    kInstruction_sxtab_ls_r7_r14_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r3_r0_r8_ROR_0),
    kInstruction_sxtab_le_r3_r0_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r8_r1_r0_ROR_24),
    kInstruction_sxtab_pl_r8_r1_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r3_r13_r14_ROR_0),
    kInstruction_sxtab_ge_r3_r13_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r4_r12_r6_ROR_16),
    kInstruction_sxtab_gt_r4_r12_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r9_r5_r11_ROR_0),
    kInstruction_sxtab_cc_r9_r5_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r8_r3_r4_ROR_16),
    kInstruction_sxtab_vs_r8_r3_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r0_r13_r14_ROR_24),
    kInstruction_sxtab_gt_r0_r13_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r8_r11_r14_ROR_24),
    kInstruction_sxtab_vs_r8_r11_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r5_r11_r11_ROR_8),
    kInstruction_sxtab_ge_r5_r11_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r14_r2_r9_ROR_8),
    kInstruction_sxtab_gt_r14_r2_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r2_r9_r4_ROR_16),
    kInstruction_sxtab_vc_r2_r9_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r8_r3_r7_ROR_8),
    kInstruction_sxtab_eq_r8_r3_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r8_r14_r14_ROR_8),
    kInstruction_sxtab_pl_r8_r14_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r13_r3_r7_ROR_24),
    kInstruction_sxtab_mi_r13_r3_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r1_r12_r4_ROR_0),
    kInstruction_sxtab_pl_r1_r12_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r1_r12_r12_ROR_8),
    kInstruction_sxtab_eq_r1_r12_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r10_r9_r9_ROR_0),
    kInstruction_sxtab_cs_r10_r9_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r14_r12_r1_ROR_8),
    kInstruction_sxtab_lt_r14_r12_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r8_r5_r10_ROR_0),
    kInstruction_sxtab_hi_r8_r5_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r0_r8_r1_ROR_16),
    kInstruction_sxtab_lt_r0_r8_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r13_r4_r9_ROR_24),
    kInstruction_sxtab_vc_r13_r4_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r3_r4_r1_ROR_24),
    kInstruction_sxtab_cc_r3_r4_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r0_r5_r3_ROR_24),
    kInstruction_sxtab_vc_r0_r5_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r2_r7_r3_ROR_8),
    kInstruction_sxtab_lt_r2_r7_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r5_r7_r14_ROR_8),
    kInstruction_sxtab_pl_r5_r7_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r11_r11_r13_ROR_24),
    kInstruction_sxtab_ne_r11_r11_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r6_r14_r8_ROR_24),
    kInstruction_sxtab_ge_r6_r14_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r6_r7_r1_ROR_0),
    kInstruction_sxtab_le_r6_r7_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r12_r4_r10_ROR_0),
    kInstruction_sxtab_vc_r12_r4_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r12_r7_r13_ROR_8),
    kInstruction_sxtab_ne_r12_r7_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r3_r11_r3_ROR_16),
    kInstruction_sxtab_cc_r3_r11_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r0_r2_r5_ROR_0),
    kInstruction_sxtab_gt_r0_r2_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r6_r14_r5_ROR_16),
    kInstruction_sxtab_ge_r6_r14_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r11_r6_r11_ROR_16),
    kInstruction_sxtab_eq_r11_r6_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r11_r7_r14_ROR_8),
    kInstruction_sxtab_vc_r11_r7_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r14_r4_r3_ROR_24),
    kInstruction_sxtab_ls_r14_r4_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r0_r5_r13_ROR_16),
    kInstruction_sxtab_lt_r0_r5_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r11_r1_r10_ROR_24),
    kInstruction_sxtab_hi_r11_r1_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r7_r7_r2_ROR_16),
    kInstruction_sxtab_ls_r7_r7_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r1_r2_r6_ROR_0),
    kInstruction_sxtab_hi_r1_r2_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r2_r0_r11_ROR_8),
    kInstruction_sxtab_hi_r2_r0_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r7_r9_r5_ROR_0),
    kInstruction_sxtab_lt_r7_r9_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r12_r14_r13_ROR_8),
    kInstruction_sxtab_hi_r12_r14_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r4_r6_r2_ROR_8),
    kInstruction_sxtab_hi_r4_r6_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r3_r6_r10_ROR_16),
    kInstruction_sxtab_eq_r3_r6_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r10_r13_r6_ROR_16),
    kInstruction_sxtab_al_r10_r13_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r3_r12_r10_ROR_24),
    kInstruction_sxtab_vs_r3_r12_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r1_r4_r14_ROR_24),
    kInstruction_sxtab_le_r1_r4_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r2_r1_r3_ROR_24),
    kInstruction_sxtab_vc_r2_r1_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r11_r3_r2_ROR_24),
    kInstruction_sxtab_hi_r11_r3_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r6_r11_r13_ROR_8),
    kInstruction_sxtab_al_r6_r11_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r5_r13_r12_ROR_16),
    kInstruction_sxtab_lt_r5_r13_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r10_r2_r0_ROR_16),
    kInstruction_sxtab_cc_r10_r2_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r2_r14_r3_ROR_24),
    kInstruction_sxtab_ge_r2_r14_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r14_r0_r9_ROR_8),
    kInstruction_sxtab_gt_r14_r0_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r7_r5_r12_ROR_8),
    kInstruction_sxtab_ne_r7_r5_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r2_r13_r13_ROR_16),
    kInstruction_sxtab_gt_r2_r13_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r0_r5_r10_ROR_24),
    kInstruction_sxtab_lt_r0_r5_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r13_r2_r6_ROR_16),
    kInstruction_sxtab_ls_r13_r2_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r8_r12_r11_ROR_16),
    kInstruction_sxtab_vc_r8_r12_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r1_r3_r7_ROR_24),
    kInstruction_sxtab_vs_r1_r3_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r10_r3_r7_ROR_0),
    kInstruction_sxtab_hi_r10_r3_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r11_r9_r7_ROR_8),
    kInstruction_sxtab_ls_r11_r9_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r10_r6_r1_ROR_16),
    kInstruction_sxtab_al_r10_r6_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r1_r6_r11_ROR_8),
    kInstruction_sxtab_cs_r1_r6_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r6_r6_r10_ROR_24),
    kInstruction_sxtab_lt_r6_r6_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r10_r10_r5_ROR_8),
    kInstruction_sxtab_le_r10_r10_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r3_r10_r1_ROR_24),
    kInstruction_sxtab_mi_r3_r10_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r9_r4_r11_ROR_8),
    kInstruction_sxtab_le_r9_r4_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r8_r4_r12_ROR_8),
    kInstruction_sxtab_ge_r8_r4_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r8_r9_r10_ROR_8),
    kInstruction_sxtab_gt_r8_r9_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r1_r10_r1_ROR_16),
    kInstruction_sxtab_cc_r1_r10_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r10_r10_r13_ROR_0),
    kInstruction_sxtab_vc_r10_r10_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r7_r13_r0_ROR_16),
    kInstruction_sxtab_vs_r7_r13_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r1_r10_r13_ROR_8),
    kInstruction_sxtab_le_r1_r10_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r8_r3_r12_ROR_8),
    kInstruction_sxtab_ls_r8_r3_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r13_r9_r12_ROR_8),
    kInstruction_sxtab_vc_r13_r9_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r11_r8_r9_ROR_0),
    kInstruction_sxtab_vs_r11_r8_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r8_r0_r7_ROR_24),
    kInstruction_sxtab_pl_r8_r0_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r7_r8_r4_ROR_24),
    kInstruction_sxtab_cs_r7_r8_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r12_r12_r0_ROR_0),
    kInstruction_sxtab_gt_r12_r12_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r5_r8_r4_ROR_8),
    kInstruction_sxtab_cc_r5_r8_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r4_r11_r11_ROR_16),
    kInstruction_sxtab_cc_r4_r11_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r13_r9_r7_ROR_24),
    kInstruction_sxtab_mi_r13_r9_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r5_r7_r13_ROR_24),
    kInstruction_sxtab_lt_r5_r7_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r6_r5_r0_ROR_8),
    kInstruction_sxtab_ne_r6_r5_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r14_r6_r1_ROR_16),
    kInstruction_sxtab_vc_r14_r6_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r8_r10_r0_ROR_8),
    kInstruction_sxtab_lt_r8_r10_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r2_r1_r14_ROR_0),
    kInstruction_sxtab_gt_r2_r1_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r4_r3_r0_ROR_8),
    kInstruction_sxtab_mi_r4_r3_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r1_r3_r14_ROR_24),
    kInstruction_sxtab_gt_r1_r3_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r3_r9_r1_ROR_0),
    kInstruction_sxtab_cc_r3_r9_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r3_r1_r6_ROR_24),
    kInstruction_sxtab_ne_r3_r1_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r3_r11_r6_ROR_24),
    kInstruction_sxtab_gt_r3_r11_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r9_r7_r7_ROR_0),
    kInstruction_sxtab_mi_r9_r7_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r0_r11_r11_ROR_0),
    kInstruction_sxtab_le_r0_r11_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r6_r10_r6_ROR_16),
    kInstruction_sxtab_ge_r6_r10_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r12_r9_r6_ROR_8),
    kInstruction_sxtab_ls_r12_r9_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r1_r5_r11_ROR_8),
    kInstruction_sxtab_al_r1_r5_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r9_r10_r9_ROR_8),
    kInstruction_sxtab_cs_r9_r10_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r11_r14_r9_ROR_24),
    kInstruction_sxtab_mi_r11_r14_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r12_r11_r12_ROR_16),
    kInstruction_sxtab_ne_r12_r11_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r7_r5_r1_ROR_0),
    kInstruction_sxtab_ge_r7_r5_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r6_r13_r2_ROR_16),
    kInstruction_sxtab_vc_r6_r13_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r8_r2_r3_ROR_8),
    kInstruction_sxtab_mi_r8_r2_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r10_r0_r2_ROR_16),
    kInstruction_sxtab_lt_r10_r0_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r0_r14_r7_ROR_8),
    kInstruction_sxtab_ge_r0_r14_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r5_r11_r6_ROR_0),
    kInstruction_sxtab_mi_r5_r11_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r10_r11_r0_ROR_24),
    kInstruction_sxtab_eq_r10_r11_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r10_r3_r3_ROR_16),
    kInstruction_sxtab_gt_r10_r3_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r5_r1_r1_ROR_0),
    kInstruction_sxtab_cs_r5_r1_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r6_r13_r2_ROR_0),
    kInstruction_sxtab_ls_r6_r13_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r0_r2_r7_ROR_0),
    kInstruction_sxtab_al_r0_r2_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r9_r13_r12_ROR_24),
    kInstruction_sxtab_le_r9_r13_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r11_r0_r4_ROR_0),
    kInstruction_sxtab_eq_r11_r0_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r10_r10_r12_ROR_24),
    kInstruction_sxtab_ne_r10_r10_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r0_r6_r7_ROR_16),
    kInstruction_sxtab_ls_r0_r6_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r5_r4_r6_ROR_16),
    kInstruction_sxtab_ls_r5_r4_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r7_r3_r12_ROR_8),
    kInstruction_sxtab_eq_r7_r3_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r8_r0_r3_ROR_0),
    kInstruction_sxtab_cs_r8_r0_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r1_r3_r2_ROR_0),
    kInstruction_sxtab_ls_r1_r3_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r2_r0_r2_ROR_24),
    kInstruction_sxtab_lt_r2_r0_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r6_r9_r10_ROR_8),
    kInstruction_sxtab_eq_r6_r9_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r12_r4_r10_ROR_16),
    kInstruction_sxtab_vs_r12_r4_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r0_r14_r1_ROR_24),
    kInstruction_sxtab_cc_r0_r14_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r6_r11_r10_ROR_0),
    kInstruction_sxtab_al_r6_r11_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r4_r3_r4_ROR_16),
    kInstruction_sxtab_le_r4_r3_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r5_r14_r1_ROR_8),
    kInstruction_sxtab_vs_r5_r14_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r8_r10_r4_ROR_24),
    kInstruction_sxtab_eq_r8_r10_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r10_r11_r5_ROR_16),
    kInstruction_sxtab_ne_r10_r11_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r5_r1_r9_ROR_8),
    kInstruction_sxtab_vc_r5_r1_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vc_r2_r9_r7_ROR_16),
    kInstruction_sxtab_vc_r2_r9_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r12_r3_r4_ROR_24),
    kInstruction_sxtab_ne_r12_r3_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r1_r9_r11_ROR_16),
    kInstruction_sxtab_mi_r1_r9_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r12_r5_r2_ROR_0),
    kInstruction_sxtab_hi_r12_r5_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r14_r3_r7_ROR_16),
    kInstruction_sxtab_ge_r14_r3_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r11_r14_r7_ROR_0),
    kInstruction_sxtab_lt_r11_r14_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r5_r2_r5_ROR_0),
    kInstruction_sxtab_ge_r5_r2_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r10_r6_r11_ROR_8),
    kInstruction_sxtab_ne_r10_r6_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r11_r5_r13_ROR_8),
    kInstruction_sxtab_vs_r11_r5_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r10_r10_r11_ROR_16),
    kInstruction_sxtab_cc_r10_r10_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r6_r13_r5_ROR_24),
    kInstruction_sxtab_hi_r6_r13_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r13_r14_r2_ROR_0),
    kInstruction_sxtab_gt_r13_r14_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r9_r1_r14_ROR_16),
    kInstruction_sxtab_mi_r9_r1_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r4_r1_r9_ROR_16),
    kInstruction_sxtab_eq_r4_r1_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r10_r0_r14_ROR_24),
    kInstruction_sxtab_ge_r10_r0_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r12_r12_r11_ROR_24),
    kInstruction_sxtab_ls_r12_r12_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r14_r7_r14_ROR_24),
    kInstruction_sxtab_ls_r14_r7_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r10_r2_r11_ROR_8),
    kInstruction_sxtab_ne_r10_r2_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r14_r2_r9_ROR_0),
    kInstruction_sxtab_lt_r14_r2_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r4_r4_r1_ROR_24),
    kInstruction_sxtab_eq_r4_r4_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r7_r0_r9_ROR_24),
    kInstruction_sxtab_cc_r7_r0_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r3_r3_r11_ROR_8),
    kInstruction_sxtab_al_r3_r3_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r8_r9_r14_ROR_0),
    kInstruction_sxtab_le_r8_r9_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r2_r9_r4_ROR_16),
    kInstruction_sxtab_gt_r2_r9_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r11_r11_r12_ROR_0),
    kInstruction_sxtab_ne_r11_r11_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r13_r11_r3_ROR_0),
    kInstruction_sxtab_hi_r13_r11_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r10_r13_r14_ROR_24),
    kInstruction_sxtab_eq_r10_r13_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r3_r5_r3_ROR_0),
    kInstruction_sxtab_ne_r3_r5_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r9_r4_r14_ROR_0),
    kInstruction_sxtab_eq_r9_r4_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r14_r8_r7_ROR_24),
    kInstruction_sxtab_al_r14_r8_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r0_r0_r13_ROR_8),
    kInstruction_sxtab_mi_r0_r0_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r9_r4_r13_ROR_16),
    kInstruction_sxtab_cc_r9_r4_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r10_r0_r4_ROR_24),
    kInstruction_sxtab_cc_r10_r0_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r7_r5_r10_ROR_24),
    kInstruction_sxtab_ge_r7_r5_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r7_r0_r14_ROR_24),
    kInstruction_sxtab_vs_r7_r0_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r12_r5_r9_ROR_16),
    kInstruction_sxtab_cc_r12_r5_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r6_r8_r2_ROR_0),
    kInstruction_sxtab_le_r6_r8_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r13_r13_r3_ROR_24),
    kInstruction_sxtab_ne_r13_r13_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r1_r10_r1_ROR_8),
    kInstruction_sxtab_eq_r1_r10_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_eq_r14_r10_r6_ROR_16),
    kInstruction_sxtab_eq_r14_r10_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r14_r14_r10_ROR_8),
    kInstruction_sxtab_al_r14_r14_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r7_r8_r4_ROR_24),
    kInstruction_sxtab_al_r7_r8_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r11_r6_r6_ROR_8),
    kInstruction_sxtab_al_r11_r6_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_lt_r5_r3_r13_ROR_16),
    kInstruction_sxtab_lt_r5_r3_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r8_r3_r2_ROR_24),
    kInstruction_sxtab_cc_r8_r3_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r11_r0_r10_ROR_0),
    kInstruction_sxtab_ge_r11_r0_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r7_r12_r12_ROR_16),
    kInstruction_sxtab_ge_r7_r12_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r7_r5_r2_ROR_8),
    kInstruction_sxtab_ls_r7_r5_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_hi_r14_r4_r3_ROR_8),
    kInstruction_sxtab_hi_r14_r4_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r3_r5_r2_ROR_0),
    kInstruction_sxtab_le_r3_r5_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r13_r9_r13_ROR_16),
    kInstruction_sxtab_cs_r13_r9_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r1_r14_r2_ROR_16),
    kInstruction_sxtab_ge_r1_r14_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r4_r6_r2_ROR_16),
    kInstruction_sxtab_gt_r4_r6_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r6_r8_r13_ROR_0),
    kInstruction_sxtab_al_r6_r8_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r6_r8_r3_ROR_16),
    kInstruction_sxtab_gt_r6_r8_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r5_r11_r14_ROR_24),
    kInstruction_sxtab_ge_r5_r11_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r9_r10_r11_ROR_16),
    kInstruction_sxtab_cc_r9_r10_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_vs_r11_r13_r3_ROR_0),
    kInstruction_sxtab_vs_r11_r13_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cc_r2_r5_r12_ROR_24),
    kInstruction_sxtab_cc_r2_r5_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r3_r9_r11_ROR_0),
    kInstruction_sxtab_ge_r3_r9_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ne_r8_r3_r1_ROR_8),
    kInstruction_sxtab_ne_r8_r3_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r1_r6_r6_ROR_16),
    kInstruction_sxtab_ls_r1_r6_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_pl_r8_r13_r4_ROR_16),
    kInstruction_sxtab_pl_r8_r13_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r7_r3_r0_ROR_24),
    kInstruction_sxtab_ge_r7_r3_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r12_r9_r7_ROR_16),
    kInstruction_sxtab_ls_r12_r9_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r14_r6_r7_ROR_24),
    kInstruction_sxtab_ls_r14_r6_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r7_r0_r10_ROR_8),
    kInstruction_sxtab_al_r7_r0_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_mi_r5_r14_r11_ROR_0),
    kInstruction_sxtab_mi_r5_r14_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r12_r8_r2_ROR_0),
    kInstruction_sxtab_ls_r12_r8_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r2_r2_r10_ROR_8),
    kInstruction_sxtab_gt_r2_r2_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r3_r2_r5_ROR_0),
    kInstruction_sxtab_ls_r3_r2_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ge_r4_r14_r3_ROR_24),
    kInstruction_sxtab_ge_r4_r14_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_ls_r5_r2_r6_ROR_16),
    kInstruction_sxtab_ls_r5_r2_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r10_r1_r7_ROR_0),
    kInstruction_sxtab_le_r10_r1_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_gt_r13_r2_r9_ROR_16),
    kInstruction_sxtab_gt_r13_r2_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_cs_r12_r9_r7_ROR_16),
    kInstruction_sxtab_cs_r12_r9_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_al_r10_r12_r1_ROR_24),
    kInstruction_sxtab_al_r10_r12_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r9_r3_r14_ROR_16),
    kInstruction_sxtab_le_r9_r3_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtab_le_r1_r4_r6_ROR_16),
    kInstruction_sxtab_le_r1_r4_r6_ROR_16,
  },
};

#endif  // VIXL_ASSEMBLER_COND_RD_RN_OPERAND_RM_ROR_AMOUNT_SXTAB_A32_H_
