// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_ASSEMBLER_COND_RD_OPERAND_RN_SHIFT_RS_MOVS_A32_H_
#define VIXL_ASSEMBLER_COND_RD_OPERAND_RN_SHIFT_RS_MOVS_A32_H_

const byte kInstruction_movs_gt_r13_r11_LSL_r12[] = {
  0x1b, 0xdc, 0xb0, 0xc1 // movs gt r13 r11 LSL r12
};
const byte kInstruction_movs_gt_r12_r4_LSR_r6[] = {
  0x34, 0xc6, 0xb0, 0xc1 // movs gt r12 r4 LSR r6
};
const byte kInstruction_movs_ge_r11_r13_ROR_r2[] = {
  0x7d, 0xb2, 0xb0, 0xa1 // movs ge r11 r13 ROR r2
};
const byte kInstruction_movs_hi_r9_r8_ASR_r10[] = {
  0x58, 0x9a, 0xb0, 0x81 // movs hi r9 r8 ASR r10
};
const byte kInstruction_movs_al_r9_r10_LSL_r4[] = {
  0x1a, 0x94, 0xb0, 0xe1 // movs al r9 r10 LSL r4
};
const byte kInstruction_movs_gt_r4_r11_LSR_r4[] = {
  0x3b, 0x44, 0xb0, 0xc1 // movs gt r4 r11 LSR r4
};
const byte kInstruction_movs_vc_r4_r6_ASR_r11[] = {
  0x56, 0x4b, 0xb0, 0x71 // movs vc r4 r6 ASR r11
};
const byte kInstruction_movs_mi_r9_r14_LSR_r9[] = {
  0x3e, 0x99, 0xb0, 0x41 // movs mi r9 r14 LSR r9
};
const byte kInstruction_movs_ne_r5_r10_LSR_r8[] = {
  0x3a, 0x58, 0xb0, 0x11 // movs ne r5 r10 LSR r8
};
const byte kInstruction_movs_eq_r3_r0_ROR_r11[] = {
  0x70, 0x3b, 0xb0, 0x01 // movs eq r3 r0 ROR r11
};
const byte kInstruction_movs_gt_r9_r11_LSL_r12[] = {
  0x1b, 0x9c, 0xb0, 0xc1 // movs gt r9 r11 LSL r12
};
const byte kInstruction_movs_ge_r14_r6_ROR_r13[] = {
  0x76, 0xed, 0xb0, 0xa1 // movs ge r14 r6 ROR r13
};
const byte kInstruction_movs_vc_r14_r14_LSR_r11[] = {
  0x3e, 0xeb, 0xb0, 0x71 // movs vc r14 r14 LSR r11
};
const byte kInstruction_movs_cc_r6_r3_LSL_r2[] = {
  0x13, 0x62, 0xb0, 0x31 // movs cc r6 r3 LSL r2
};
const byte kInstruction_movs_ge_r4_r6_ROR_r7[] = {
  0x76, 0x47, 0xb0, 0xa1 // movs ge r4 r6 ROR r7
};
const byte kInstruction_movs_vs_r13_r3_LSR_r12[] = {
  0x33, 0xdc, 0xb0, 0x61 // movs vs r13 r3 LSR r12
};
const byte kInstruction_movs_ge_r8_r11_LSR_r13[] = {
  0x3b, 0x8d, 0xb0, 0xa1 // movs ge r8 r11 LSR r13
};
const byte kInstruction_movs_cc_r5_r0_LSR_r10[] = {
  0x30, 0x5a, 0xb0, 0x31 // movs cc r5 r0 LSR r10
};
const byte kInstruction_movs_lt_r7_r12_ASR_r7[] = {
  0x5c, 0x77, 0xb0, 0xb1 // movs lt r7 r12 ASR r7
};
const byte kInstruction_movs_cc_r14_r1_LSR_r14[] = {
  0x31, 0xee, 0xb0, 0x31 // movs cc r14 r1 LSR r14
};
const byte kInstruction_movs_vs_r10_r1_LSR_r1[] = {
  0x31, 0xa1, 0xb0, 0x61 // movs vs r10 r1 LSR r1
};
const byte kInstruction_movs_lt_r5_r0_ASR_r2[] = {
  0x50, 0x52, 0xb0, 0xb1 // movs lt r5 r0 ASR r2
};
const byte kInstruction_movs_ne_r6_r3_LSL_r2[] = {
  0x13, 0x62, 0xb0, 0x11 // movs ne r6 r3 LSL r2
};
const byte kInstruction_movs_vs_r5_r5_LSL_r13[] = {
  0x15, 0x5d, 0xb0, 0x61 // movs vs r5 r5 LSL r13
};
const byte kInstruction_movs_hi_r9_r11_ROR_r13[] = {
  0x7b, 0x9d, 0xb0, 0x81 // movs hi r9 r11 ROR r13
};
const byte kInstruction_movs_vs_r0_r2_LSL_r6[] = {
  0x12, 0x06, 0xb0, 0x61 // movs vs r0 r2 LSL r6
};
const byte kInstruction_movs_le_r7_r1_ASR_r0[] = {
  0x51, 0x70, 0xb0, 0xd1 // movs le r7 r1 ASR r0
};
const byte kInstruction_movs_ge_r2_r12_LSL_r10[] = {
  0x1c, 0x2a, 0xb0, 0xa1 // movs ge r2 r12 LSL r10
};
const byte kInstruction_movs_vc_r14_r1_ASR_r9[] = {
  0x51, 0xe9, 0xb0, 0x71 // movs vc r14 r1 ASR r9
};
const byte kInstruction_movs_lt_r5_r1_ROR_r5[] = {
  0x71, 0x55, 0xb0, 0xb1 // movs lt r5 r1 ROR r5
};
const byte kInstruction_movs_eq_r5_r12_ROR_r0[] = {
  0x7c, 0x50, 0xb0, 0x01 // movs eq r5 r12 ROR r0
};
const byte kInstruction_movs_pl_r7_r7_ASR_r9[] = {
  0x57, 0x79, 0xb0, 0x51 // movs pl r7 r7 ASR r9
};
const byte kInstruction_movs_eq_r3_r10_LSR_r9[] = {
  0x3a, 0x39, 0xb0, 0x01 // movs eq r3 r10 LSR r9
};
const byte kInstruction_movs_ge_r0_r5_LSR_r13[] = {
  0x35, 0x0d, 0xb0, 0xa1 // movs ge r0 r5 LSR r13
};
const byte kInstruction_movs_cc_r13_r1_ASR_r6[] = {
  0x51, 0xd6, 0xb0, 0x31 // movs cc r13 r1 ASR r6
};
const byte kInstruction_movs_cs_r12_r3_ASR_r12[] = {
  0x53, 0xcc, 0xb0, 0x21 // movs cs r12 r3 ASR r12
};
const byte kInstruction_movs_al_r12_r10_ROR_r0[] = {
  0x7a, 0xc0, 0xb0, 0xe1 // movs al r12 r10 ROR r0
};
const byte kInstruction_movs_al_r6_r13_LSL_r0[] = {
  0x1d, 0x60, 0xb0, 0xe1 // movs al r6 r13 LSL r0
};
const byte kInstruction_movs_cc_r14_r14_ROR_r0[] = {
  0x7e, 0xe0, 0xb0, 0x31 // movs cc r14 r14 ROR r0
};
const byte kInstruction_movs_ge_r2_r14_ROR_r9[] = {
  0x7e, 0x29, 0xb0, 0xa1 // movs ge r2 r14 ROR r9
};
const byte kInstruction_movs_hi_r5_r10_LSL_r8[] = {
  0x1a, 0x58, 0xb0, 0x81 // movs hi r5 r10 LSL r8
};
const byte kInstruction_movs_ge_r13_r5_LSL_r7[] = {
  0x15, 0xd7, 0xb0, 0xa1 // movs ge r13 r5 LSL r7
};
const byte kInstruction_movs_cc_r12_r10_ASR_r4[] = {
  0x5a, 0xc4, 0xb0, 0x31 // movs cc r12 r10 ASR r4
};
const byte kInstruction_movs_ls_r12_r3_LSL_r7[] = {
  0x13, 0xc7, 0xb0, 0x91 // movs ls r12 r3 LSL r7
};
const byte kInstruction_movs_cs_r2_r9_ASR_r8[] = {
  0x59, 0x28, 0xb0, 0x21 // movs cs r2 r9 ASR r8
};
const byte kInstruction_movs_vs_r1_r5_LSR_r1[] = {
  0x35, 0x11, 0xb0, 0x61 // movs vs r1 r5 LSR r1
};
const byte kInstruction_movs_cs_r7_r14_ASR_r9[] = {
  0x5e, 0x79, 0xb0, 0x21 // movs cs r7 r14 ASR r9
};
const byte kInstruction_movs_hi_r6_r10_ASR_r0[] = {
  0x5a, 0x60, 0xb0, 0x81 // movs hi r6 r10 ASR r0
};
const byte kInstruction_movs_pl_r13_r8_ASR_r13[] = {
  0x58, 0xdd, 0xb0, 0x51 // movs pl r13 r8 ASR r13
};
const byte kInstruction_movs_vs_r1_r0_ROR_r2[] = {
  0x70, 0x12, 0xb0, 0x61 // movs vs r1 r0 ROR r2
};
const byte kInstruction_movs_lt_r14_r0_LSL_r10[] = {
  0x10, 0xea, 0xb0, 0xb1 // movs lt r14 r0 LSL r10
};
const byte kInstruction_movs_ls_r8_r12_ROR_r14[] = {
  0x7c, 0x8e, 0xb0, 0x91 // movs ls r8 r12 ROR r14
};
const byte kInstruction_movs_al_r11_r2_LSL_r0[] = {
  0x12, 0xb0, 0xb0, 0xe1 // movs al r11 r2 LSL r0
};
const byte kInstruction_movs_mi_r0_r8_LSR_r7[] = {
  0x38, 0x07, 0xb0, 0x41 // movs mi r0 r8 LSR r7
};
const byte kInstruction_movs_cc_r3_r5_LSL_r9[] = {
  0x15, 0x39, 0xb0, 0x31 // movs cc r3 r5 LSL r9
};
const byte kInstruction_movs_vc_r14_r4_ROR_r4[] = {
  0x74, 0xe4, 0xb0, 0x71 // movs vc r14 r4 ROR r4
};
const byte kInstruction_movs_cs_r1_r8_LSR_r7[] = {
  0x38, 0x17, 0xb0, 0x21 // movs cs r1 r8 LSR r7
};
const byte kInstruction_movs_ls_r6_r5_ASR_r13[] = {
  0x55, 0x6d, 0xb0, 0x91 // movs ls r6 r5 ASR r13
};
const byte kInstruction_movs_pl_r0_r12_LSR_r5[] = {
  0x3c, 0x05, 0xb0, 0x51 // movs pl r0 r12 LSR r5
};
const byte kInstruction_movs_lt_r9_r7_ROR_r2[] = {
  0x77, 0x92, 0xb0, 0xb1 // movs lt r9 r7 ROR r2
};
const byte kInstruction_movs_ne_r6_r13_ASR_r13[] = {
  0x5d, 0x6d, 0xb0, 0x11 // movs ne r6 r13 ASR r13
};
const byte kInstruction_movs_cc_r9_r1_LSR_r5[] = {
  0x31, 0x95, 0xb0, 0x31 // movs cc r9 r1 LSR r5
};
const byte kInstruction_movs_ls_r8_r0_ROR_r10[] = {
  0x70, 0x8a, 0xb0, 0x91 // movs ls r8 r0 ROR r10
};
const byte kInstruction_movs_mi_r9_r8_LSR_r2[] = {
  0x38, 0x92, 0xb0, 0x41 // movs mi r9 r8 LSR r2
};
const byte kInstruction_movs_pl_r10_r14_ROR_r8[] = {
  0x7e, 0xa8, 0xb0, 0x51 // movs pl r10 r14 ROR r8
};
const byte kInstruction_movs_hi_r4_r6_ROR_r10[] = {
  0x76, 0x4a, 0xb0, 0x81 // movs hi r4 r6 ROR r10
};
const byte kInstruction_movs_vc_r0_r8_ROR_r10[] = {
  0x78, 0x0a, 0xb0, 0x71 // movs vc r0 r8 ROR r10
};
const byte kInstruction_movs_ne_r0_r4_LSR_r11[] = {
  0x34, 0x0b, 0xb0, 0x11 // movs ne r0 r4 LSR r11
};
const byte kInstruction_movs_pl_r0_r13_ROR_r14[] = {
  0x7d, 0x0e, 0xb0, 0x51 // movs pl r0 r13 ROR r14
};
const byte kInstruction_movs_ne_r5_r6_LSL_r5[] = {
  0x16, 0x55, 0xb0, 0x11 // movs ne r5 r6 LSL r5
};
const byte kInstruction_movs_ge_r8_r3_ROR_r14[] = {
  0x73, 0x8e, 0xb0, 0xa1 // movs ge r8 r3 ROR r14
};
const byte kInstruction_movs_ne_r8_r7_LSL_r10[] = {
  0x17, 0x8a, 0xb0, 0x11 // movs ne r8 r7 LSL r10
};
const byte kInstruction_movs_eq_r13_r9_ROR_r13[] = {
  0x79, 0xdd, 0xb0, 0x01 // movs eq r13 r9 ROR r13
};
const byte kInstruction_movs_eq_r10_r8_ROR_r14[] = {
  0x78, 0xae, 0xb0, 0x01 // movs eq r10 r8 ROR r14
};
const byte kInstruction_movs_mi_r1_r8_ROR_r9[] = {
  0x78, 0x19, 0xb0, 0x41 // movs mi r1 r8 ROR r9
};
const byte kInstruction_movs_le_r10_r8_ROR_r10[] = {
  0x78, 0xaa, 0xb0, 0xd1 // movs le r10 r8 ROR r10
};
const byte kInstruction_movs_le_r7_r2_LSR_r11[] = {
  0x32, 0x7b, 0xb0, 0xd1 // movs le r7 r2 LSR r11
};
const byte kInstruction_movs_le_r11_r9_LSL_r11[] = {
  0x19, 0xbb, 0xb0, 0xd1 // movs le r11 r9 LSL r11
};
const byte kInstruction_movs_cc_r5_r5_LSR_r4[] = {
  0x35, 0x54, 0xb0, 0x31 // movs cc r5 r5 LSR r4
};
const byte kInstruction_movs_al_r9_r5_LSL_r8[] = {
  0x15, 0x98, 0xb0, 0xe1 // movs al r9 r5 LSL r8
};
const byte kInstruction_movs_cs_r7_r14_LSL_r8[] = {
  0x1e, 0x78, 0xb0, 0x21 // movs cs r7 r14 LSL r8
};
const byte kInstruction_movs_vc_r9_r9_LSR_r7[] = {
  0x39, 0x97, 0xb0, 0x71 // movs vc r9 r9 LSR r7
};
const byte kInstruction_movs_al_r13_r9_LSL_r1[] = {
  0x19, 0xd1, 0xb0, 0xe1 // movs al r13 r9 LSL r1
};
const byte kInstruction_movs_cc_r8_r4_LSR_r14[] = {
  0x34, 0x8e, 0xb0, 0x31 // movs cc r8 r4 LSR r14
};
const byte kInstruction_movs_cc_r14_r3_LSL_r10[] = {
  0x13, 0xea, 0xb0, 0x31 // movs cc r14 r3 LSL r10
};
const byte kInstruction_movs_ge_r8_r6_ROR_r14[] = {
  0x76, 0x8e, 0xb0, 0xa1 // movs ge r8 r6 ROR r14
};
const byte kInstruction_movs_le_r4_r5_ASR_r5[] = {
  0x55, 0x45, 0xb0, 0xd1 // movs le r4 r5 ASR r5
};
const byte kInstruction_movs_vc_r4_r8_LSR_r6[] = {
  0x38, 0x46, 0xb0, 0x71 // movs vc r4 r8 LSR r6
};
const byte kInstruction_movs_vc_r9_r8_ROR_r14[] = {
  0x78, 0x9e, 0xb0, 0x71 // movs vc r9 r8 ROR r14
};
const byte kInstruction_movs_ne_r7_r5_LSL_r11[] = {
  0x15, 0x7b, 0xb0, 0x11 // movs ne r7 r5 LSL r11
};
const byte kInstruction_movs_vs_r12_r12_ASR_r8[] = {
  0x5c, 0xc8, 0xb0, 0x61 // movs vs r12 r12 ASR r8
};
const byte kInstruction_movs_pl_r2_r14_ASR_r7[] = {
  0x5e, 0x27, 0xb0, 0x51 // movs pl r2 r14 ASR r7
};
const byte kInstruction_movs_cc_r7_r10_ASR_r9[] = {
  0x5a, 0x79, 0xb0, 0x31 // movs cc r7 r10 ASR r9
};
const byte kInstruction_movs_ne_r7_r10_LSL_r11[] = {
  0x1a, 0x7b, 0xb0, 0x11 // movs ne r7 r10 LSL r11
};
const byte kInstruction_movs_vs_r13_r14_ROR_r0[] = {
  0x7e, 0xd0, 0xb0, 0x61 // movs vs r13 r14 ROR r0
};
const byte kInstruction_movs_gt_r4_r12_LSL_r6[] = {
  0x1c, 0x46, 0xb0, 0xc1 // movs gt r4 r12 LSL r6
};
const byte kInstruction_movs_ge_r14_r6_ROR_r11[] = {
  0x76, 0xeb, 0xb0, 0xa1 // movs ge r14 r6 ROR r11
};
const byte kInstruction_movs_pl_r2_r14_ASR_r0[] = {
  0x5e, 0x20, 0xb0, 0x51 // movs pl r2 r14 ASR r0
};
const byte kInstruction_movs_gt_r7_r6_LSL_r11[] = {
  0x16, 0x7b, 0xb0, 0xc1 // movs gt r7 r6 LSL r11
};
const byte kInstruction_movs_cc_r11_r9_LSL_r11[] = {
  0x19, 0xbb, 0xb0, 0x31 // movs cc r11 r9 LSL r11
};
const byte kInstruction_movs_ne_r1_r5_ROR_r8[] = {
  0x75, 0x18, 0xb0, 0x11 // movs ne r1 r5 ROR r8
};
const byte kInstruction_movs_cs_r6_r2_ROR_r6[] = {
  0x72, 0x66, 0xb0, 0x21 // movs cs r6 r2 ROR r6
};
const byte kInstruction_movs_vc_r1_r14_LSL_r11[] = {
  0x1e, 0x1b, 0xb0, 0x71 // movs vc r1 r14 LSL r11
};
const byte kInstruction_movs_cc_r10_r13_LSL_r11[] = {
  0x1d, 0xab, 0xb0, 0x31 // movs cc r10 r13 LSL r11
};
const byte kInstruction_movs_mi_r12_r10_ROR_r5[] = {
  0x7a, 0xc5, 0xb0, 0x41 // movs mi r12 r10 ROR r5
};
const byte kInstruction_movs_ls_r2_r11_LSL_r3[] = {
  0x1b, 0x23, 0xb0, 0x91 // movs ls r2 r11 LSL r3
};
const byte kInstruction_movs_lt_r6_r13_LSL_r12[] = {
  0x1d, 0x6c, 0xb0, 0xb1 // movs lt r6 r13 LSL r12
};
const byte kInstruction_movs_eq_r2_r14_ASR_r5[] = {
  0x5e, 0x25, 0xb0, 0x01 // movs eq r2 r14 ASR r5
};
const byte kInstruction_movs_eq_r0_r14_LSR_r5[] = {
  0x3e, 0x05, 0xb0, 0x01 // movs eq r0 r14 LSR r5
};
const byte kInstruction_movs_ls_r7_r11_LSL_r1[] = {
  0x1b, 0x71, 0xb0, 0x91 // movs ls r7 r11 LSL r1
};
const byte kInstruction_movs_vs_r14_r13_LSR_r14[] = {
  0x3d, 0xee, 0xb0, 0x61 // movs vs r14 r13 LSR r14
};
const byte kInstruction_movs_vs_r13_r10_ASR_r3[] = {
  0x5a, 0xd3, 0xb0, 0x61 // movs vs r13 r10 ASR r3
};
const byte kInstruction_movs_gt_r5_r4_LSL_r2[] = {
  0x14, 0x52, 0xb0, 0xc1 // movs gt r5 r4 LSL r2
};
const byte kInstruction_movs_hi_r5_r12_LSR_r10[] = {
  0x3c, 0x5a, 0xb0, 0x81 // movs hi r5 r12 LSR r10
};
const byte kInstruction_movs_mi_r11_r7_ROR_r11[] = {
  0x77, 0xbb, 0xb0, 0x41 // movs mi r11 r7 ROR r11
};
const byte kInstruction_movs_ne_r10_r7_ROR_r9[] = {
  0x77, 0xa9, 0xb0, 0x11 // movs ne r10 r7 ROR r9
};
const byte kInstruction_movs_ls_r8_r6_ASR_r10[] = {
  0x56, 0x8a, 0xb0, 0x91 // movs ls r8 r6 ASR r10
};
const byte kInstruction_movs_vc_r7_r10_ROR_r9[] = {
  0x7a, 0x79, 0xb0, 0x71 // movs vc r7 r10 ROR r9
};
const byte kInstruction_movs_mi_r8_r3_LSR_r3[] = {
  0x33, 0x83, 0xb0, 0x41 // movs mi r8 r3 LSR r3
};
const byte kInstruction_movs_lt_r8_r2_ROR_r12[] = {
  0x72, 0x8c, 0xb0, 0xb1 // movs lt r8 r2 ROR r12
};
const byte kInstruction_movs_le_r10_r14_LSL_r11[] = {
  0x1e, 0xab, 0xb0, 0xd1 // movs le r10 r14 LSL r11
};
const byte kInstruction_movs_pl_r6_r6_LSR_r5[] = {
  0x36, 0x65, 0xb0, 0x51 // movs pl r6 r6 LSR r5
};
const byte kInstruction_movs_cc_r8_r4_ROR_r2[] = {
  0x74, 0x82, 0xb0, 0x31 // movs cc r8 r4 ROR r2
};
const byte kInstruction_movs_al_r7_r6_ASR_r2[] = {
  0x56, 0x72, 0xb0, 0xe1 // movs al r7 r6 ASR r2
};
const byte kInstruction_movs_ne_r2_r13_ASR_r8[] = {
  0x5d, 0x28, 0xb0, 0x11 // movs ne r2 r13 ASR r8
};
const byte kInstruction_movs_ne_r1_r5_ROR_r12[] = {
  0x75, 0x1c, 0xb0, 0x11 // movs ne r1 r5 ROR r12
};
const byte kInstruction_movs_lt_r10_r6_ROR_r8[] = {
  0x76, 0xa8, 0xb0, 0xb1 // movs lt r10 r6 ROR r8
};
const byte kInstruction_movs_cs_r2_r7_LSL_r0[] = {
  0x17, 0x20, 0xb0, 0x21 // movs cs r2 r7 LSL r0
};
const byte kInstruction_movs_ne_r3_r2_ASR_r4[] = {
  0x52, 0x34, 0xb0, 0x11 // movs ne r3 r2 ASR r4
};
const byte kInstruction_movs_mi_r4_r1_LSR_r0[] = {
  0x31, 0x40, 0xb0, 0x41 // movs mi r4 r1 LSR r0
};
const byte kInstruction_movs_lt_r1_r6_LSL_r10[] = {
  0x16, 0x1a, 0xb0, 0xb1 // movs lt r1 r6 LSL r10
};
const byte kInstruction_movs_gt_r6_r9_LSL_r13[] = {
  0x19, 0x6d, 0xb0, 0xc1 // movs gt r6 r9 LSL r13
};
const byte kInstruction_movs_vc_r5_r2_LSL_r13[] = {
  0x12, 0x5d, 0xb0, 0x71 // movs vc r5 r2 LSL r13
};
const byte kInstruction_movs_ne_r11_r8_ROR_r5[] = {
  0x78, 0xb5, 0xb0, 0x11 // movs ne r11 r8 ROR r5
};
const byte kInstruction_movs_eq_r10_r2_ROR_r12[] = {
  0x72, 0xac, 0xb0, 0x01 // movs eq r10 r2 ROR r12
};
const byte kInstruction_movs_ge_r0_r5_LSR_r12[] = {
  0x35, 0x0c, 0xb0, 0xa1 // movs ge r0 r5 LSR r12
};
const byte kInstruction_movs_vs_r5_r13_ROR_r9[] = {
  0x7d, 0x59, 0xb0, 0x61 // movs vs r5 r13 ROR r9
};
const byte kInstruction_movs_gt_r0_r6_ASR_r9[] = {
  0x56, 0x09, 0xb0, 0xc1 // movs gt r0 r6 ASR r9
};
const byte kInstruction_movs_hi_r6_r0_LSR_r9[] = {
  0x30, 0x69, 0xb0, 0x81 // movs hi r6 r0 LSR r9
};
const byte kInstruction_movs_lt_r7_r4_ASR_r9[] = {
  0x54, 0x79, 0xb0, 0xb1 // movs lt r7 r4 ASR r9
};
const byte kInstruction_movs_ge_r5_r0_ROR_r2[] = {
  0x70, 0x52, 0xb0, 0xa1 // movs ge r5 r0 ROR r2
};
const byte kInstruction_movs_ls_r10_r11_ROR_r7[] = {
  0x7b, 0xa7, 0xb0, 0x91 // movs ls r10 r11 ROR r7
};
const byte kInstruction_movs_cs_r7_r1_ASR_r13[] = {
  0x51, 0x7d, 0xb0, 0x21 // movs cs r7 r1 ASR r13
};
const byte kInstruction_movs_pl_r3_r9_ASR_r11[] = {
  0x59, 0x3b, 0xb0, 0x51 // movs pl r3 r9 ASR r11
};
const byte kInstruction_movs_ne_r13_r5_LSR_r2[] = {
  0x35, 0xd2, 0xb0, 0x11 // movs ne r13 r5 LSR r2
};
const byte kInstruction_movs_vc_r5_r8_ASR_r6[] = {
  0x58, 0x56, 0xb0, 0x71 // movs vc r5 r8 ASR r6
};
const byte kInstruction_movs_ge_r8_r11_ROR_r13[] = {
  0x7b, 0x8d, 0xb0, 0xa1 // movs ge r8 r11 ROR r13
};
const byte kInstruction_movs_lt_r0_r10_LSR_r5[] = {
  0x3a, 0x05, 0xb0, 0xb1 // movs lt r0 r10 LSR r5
};
const byte kInstruction_movs_vc_r8_r1_ASR_r11[] = {
  0x51, 0x8b, 0xb0, 0x71 // movs vc r8 r1 ASR r11
};
const byte kInstruction_movs_pl_r11_r9_ASR_r9[] = {
  0x59, 0xb9, 0xb0, 0x51 // movs pl r11 r9 ASR r9
};
const byte kInstruction_movs_ls_r3_r1_LSL_r11[] = {
  0x11, 0x3b, 0xb0, 0x91 // movs ls r3 r1 LSL r11
};
const byte kInstruction_movs_hi_r8_r12_ASR_r7[] = {
  0x5c, 0x87, 0xb0, 0x81 // movs hi r8 r12 ASR r7
};
const byte kInstruction_movs_ne_r10_r12_LSR_r9[] = {
  0x3c, 0xa9, 0xb0, 0x11 // movs ne r10 r12 LSR r9
};
const byte kInstruction_movs_ge_r12_r4_ASR_r14[] = {
  0x54, 0xce, 0xb0, 0xa1 // movs ge r12 r4 ASR r14
};
const byte kInstruction_movs_mi_r6_r2_ROR_r1[] = {
  0x72, 0x61, 0xb0, 0x41 // movs mi r6 r2 ROR r1
};
const byte kInstruction_movs_ls_r13_r13_LSL_r0[] = {
  0x1d, 0xd0, 0xb0, 0x91 // movs ls r13 r13 LSL r0
};
const byte kInstruction_movs_eq_r8_r10_LSR_r0[] = {
  0x3a, 0x80, 0xb0, 0x01 // movs eq r8 r10 LSR r0
};
const byte kInstruction_movs_gt_r5_r0_ROR_r14[] = {
  0x70, 0x5e, 0xb0, 0xc1 // movs gt r5 r0 ROR r14
};
const byte kInstruction_movs_vs_r3_r4_ROR_r3[] = {
  0x74, 0x33, 0xb0, 0x61 // movs vs r3 r4 ROR r3
};
const byte kInstruction_movs_lt_r5_r13_LSL_r6[] = {
  0x1d, 0x56, 0xb0, 0xb1 // movs lt r5 r13 LSL r6
};
const byte kInstruction_movs_cc_r5_r1_ASR_r13[] = {
  0x51, 0x5d, 0xb0, 0x31 // movs cc r5 r1 ASR r13
};
const byte kInstruction_movs_lt_r11_r3_ROR_r7[] = {
  0x73, 0xb7, 0xb0, 0xb1 // movs lt r11 r3 ROR r7
};
const byte kInstruction_movs_cs_r12_r2_LSR_r4[] = {
  0x32, 0xc4, 0xb0, 0x21 // movs cs r12 r2 LSR r4
};
const byte kInstruction_movs_vc_r3_r12_LSL_r4[] = {
  0x1c, 0x34, 0xb0, 0x71 // movs vc r3 r12 LSL r4
};
const byte kInstruction_movs_hi_r14_r9_LSL_r5[] = {
  0x19, 0xe5, 0xb0, 0x81 // movs hi r14 r9 LSL r5
};
const byte kInstruction_movs_ls_r10_r4_LSR_r5[] = {
  0x34, 0xa5, 0xb0, 0x91 // movs ls r10 r4 LSR r5
};
const byte kInstruction_movs_ne_r11_r14_ROR_r14[] = {
  0x7e, 0xbe, 0xb0, 0x11 // movs ne r11 r14 ROR r14
};
const byte kInstruction_movs_vc_r11_r12_ASR_r7[] = {
  0x5c, 0xb7, 0xb0, 0x71 // movs vc r11 r12 ASR r7
};
const byte kInstruction_movs_cc_r7_r0_LSR_r3[] = {
  0x30, 0x73, 0xb0, 0x31 // movs cc r7 r0 LSR r3
};
const byte kInstruction_movs_cs_r1_r8_LSR_r3[] = {
  0x38, 0x13, 0xb0, 0x21 // movs cs r1 r8 LSR r3
};
const byte kInstruction_movs_mi_r7_r6_LSL_r13[] = {
  0x16, 0x7d, 0xb0, 0x41 // movs mi r7 r6 LSL r13
};
const byte kInstruction_movs_hi_r10_r3_ROR_r7[] = {
  0x73, 0xa7, 0xb0, 0x81 // movs hi r10 r3 ROR r7
};
const byte kInstruction_movs_vs_r2_r11_LSR_r13[] = {
  0x3b, 0x2d, 0xb0, 0x61 // movs vs r2 r11 LSR r13
};
const byte kInstruction_movs_eq_r12_r14_LSL_r2[] = {
  0x1e, 0xc2, 0xb0, 0x01 // movs eq r12 r14 LSL r2
};
const byte kInstruction_movs_vs_r14_r13_LSL_r6[] = {
  0x1d, 0xe6, 0xb0, 0x61 // movs vs r14 r13 LSL r6
};
const byte kInstruction_movs_cs_r1_r0_LSL_r10[] = {
  0x10, 0x1a, 0xb0, 0x21 // movs cs r1 r0 LSL r10
};
const byte kInstruction_movs_ge_r2_r14_LSR_r10[] = {
  0x3e, 0x2a, 0xb0, 0xa1 // movs ge r2 r14 LSR r10
};
const byte kInstruction_movs_mi_r14_r10_ROR_r0[] = {
  0x7a, 0xe0, 0xb0, 0x41 // movs mi r14 r10 ROR r0
};
const byte kInstruction_movs_eq_r8_r4_ROR_r4[] = {
  0x74, 0x84, 0xb0, 0x01 // movs eq r8 r4 ROR r4
};
const byte kInstruction_movs_gt_r4_r9_LSR_r3[] = {
  0x39, 0x43, 0xb0, 0xc1 // movs gt r4 r9 LSR r3
};
const byte kInstruction_movs_ls_r13_r0_ASR_r5[] = {
  0x50, 0xd5, 0xb0, 0x91 // movs ls r13 r0 ASR r5
};
const byte kInstruction_movs_ge_r3_r13_ASR_r12[] = {
  0x5d, 0x3c, 0xb0, 0xa1 // movs ge r3 r13 ASR r12
};
const byte kInstruction_movs_vc_r9_r7_LSR_r12[] = {
  0x37, 0x9c, 0xb0, 0x71 // movs vc r9 r7 LSR r12
};
const byte kInstruction_movs_al_r1_r0_LSL_r8[] = {
  0x10, 0x18, 0xb0, 0xe1 // movs al r1 r0 LSL r8
};
const byte kInstruction_movs_ls_r11_r7_ROR_r5[] = {
  0x77, 0xb5, 0xb0, 0x91 // movs ls r11 r7 ROR r5
};
const byte kInstruction_movs_vs_r11_r8_LSR_r13[] = {
  0x38, 0xbd, 0xb0, 0x61 // movs vs r11 r8 LSR r13
};
const byte kInstruction_movs_le_r6_r5_LSR_r3[] = {
  0x35, 0x63, 0xb0, 0xd1 // movs le r6 r5 LSR r3
};
const byte kInstruction_movs_cs_r4_r1_LSL_r7[] = {
  0x11, 0x47, 0xb0, 0x21 // movs cs r4 r1 LSL r7
};
const byte kInstruction_movs_lt_r4_r9_LSL_r2[] = {
  0x19, 0x42, 0xb0, 0xb1 // movs lt r4 r9 LSL r2
};
const byte kInstruction_movs_mi_r12_r1_LSR_r10[] = {
  0x31, 0xca, 0xb0, 0x41 // movs mi r12 r1 LSR r10
};
const byte kInstruction_movs_al_r14_r2_LSR_r3[] = {
  0x32, 0xe3, 0xb0, 0xe1 // movs al r14 r2 LSR r3
};
const byte kInstruction_movs_al_r11_r9_LSL_r4[] = {
  0x19, 0xb4, 0xb0, 0xe1 // movs al r11 r9 LSL r4
};
const byte kInstruction_movs_hi_r4_r4_ROR_r3[] = {
  0x74, 0x43, 0xb0, 0x81 // movs hi r4 r4 ROR r3
};
const byte kInstruction_movs_al_r3_r5_ROR_r6[] = {
  0x75, 0x36, 0xb0, 0xe1 // movs al r3 r5 ROR r6
};
const byte kInstruction_movs_al_r8_r4_LSR_r7[] = {
  0x34, 0x87, 0xb0, 0xe1 // movs al r8 r4 LSR r7
};
const byte kInstruction_movs_vs_r7_r5_LSL_r6[] = {
  0x15, 0x76, 0xb0, 0x61 // movs vs r7 r5 LSL r6
};
const byte kInstruction_movs_ls_r6_r2_LSL_r8[] = {
  0x12, 0x68, 0xb0, 0x91 // movs ls r6 r2 LSL r8
};
const byte kInstruction_movs_pl_r4_r12_LSL_r6[] = {
  0x1c, 0x46, 0xb0, 0x51 // movs pl r4 r12 LSL r6
};
const byte kInstruction_movs_ge_r8_r12_LSR_r4[] = {
  0x3c, 0x84, 0xb0, 0xa1 // movs ge r8 r12 LSR r4
};
const byte kInstruction_movs_le_r11_r1_ROR_r9[] = {
  0x71, 0xb9, 0xb0, 0xd1 // movs le r11 r1 ROR r9
};
const byte kInstruction_movs_hi_r1_r13_LSR_r9[] = {
  0x3d, 0x19, 0xb0, 0x81 // movs hi r1 r13 LSR r9
};
const byte kInstruction_movs_cc_r3_r0_LSR_r1[] = {
  0x30, 0x31, 0xb0, 0x31 // movs cc r3 r0 LSR r1
};
const byte kInstruction_movs_ge_r10_r11_ROR_r0[] = {
  0x7b, 0xa0, 0xb0, 0xa1 // movs ge r10 r11 ROR r0
};
const byte kInstruction_movs_pl_r6_r5_ROR_r3[] = {
  0x75, 0x63, 0xb0, 0x51 // movs pl r6 r5 ROR r3
};
const byte kInstruction_movs_cs_r10_r1_LSR_r12[] = {
  0x31, 0xac, 0xb0, 0x21 // movs cs r10 r1 LSR r12
};
const byte kInstruction_movs_ne_r10_r4_LSR_r1[] = {
  0x34, 0xa1, 0xb0, 0x11 // movs ne r10 r4 LSR r1
};
const byte kInstruction_movs_vs_r5_r1_LSL_r7[] = {
  0x11, 0x57, 0xb0, 0x61 // movs vs r5 r1 LSL r7
};
const byte kInstruction_movs_ge_r10_r2_LSL_r4[] = {
  0x12, 0xa4, 0xb0, 0xa1 // movs ge r10 r2 LSL r4
};
const byte kInstruction_movs_vc_r3_r1_ASR_r2[] = {
  0x51, 0x32, 0xb0, 0x71 // movs vc r3 r1 ASR r2
};
const byte kInstruction_movs_ls_r2_r3_ASR_r14[] = {
  0x53, 0x2e, 0xb0, 0x91 // movs ls r2 r3 ASR r14
};
const byte kInstruction_movs_vs_r1_r1_LSL_r7[] = {
  0x11, 0x17, 0xb0, 0x61 // movs vs r1 r1 LSL r7
};
const byte kInstruction_movs_gt_r2_r1_LSL_r14[] = {
  0x11, 0x2e, 0xb0, 0xc1 // movs gt r2 r1 LSL r14
};
const byte kInstruction_movs_al_r11_r12_ROR_r7[] = {
  0x7c, 0xb7, 0xb0, 0xe1 // movs al r11 r12 ROR r7
};
const byte kInstruction_movs_eq_r2_r12_LSL_r10[] = {
  0x1c, 0x2a, 0xb0, 0x01 // movs eq r2 r12 LSL r10
};
const byte kInstruction_movs_ls_r7_r2_LSR_r7[] = {
  0x32, 0x77, 0xb0, 0x91 // movs ls r7 r2 LSR r7
};
const byte kInstruction_movs_ne_r9_r5_ROR_r12[] = {
  0x75, 0x9c, 0xb0, 0x11 // movs ne r9 r5 ROR r12
};
const byte kInstruction_movs_ge_r13_r8_LSR_r12[] = {
  0x38, 0xdc, 0xb0, 0xa1 // movs ge r13 r8 LSR r12
};
const byte kInstruction_movs_ls_r12_r11_ROR_r9[] = {
  0x7b, 0xc9, 0xb0, 0x91 // movs ls r12 r11 ROR r9
};
const byte kInstruction_movs_cc_r13_r5_LSL_r8[] = {
  0x15, 0xd8, 0xb0, 0x31 // movs cc r13 r5 LSL r8
};
const byte kInstruction_movs_ne_r7_r12_LSL_r7[] = {
  0x1c, 0x77, 0xb0, 0x11 // movs ne r7 r12 LSL r7
};
const byte kInstruction_movs_ge_r6_r13_ROR_r2[] = {
  0x7d, 0x62, 0xb0, 0xa1 // movs ge r6 r13 ROR r2
};
const byte kInstruction_movs_ne_r8_r7_ROR_r6[] = {
  0x77, 0x86, 0xb0, 0x11 // movs ne r8 r7 ROR r6
};
const byte kInstruction_movs_cc_r7_r10_ROR_r0[] = {
  0x7a, 0x70, 0xb0, 0x31 // movs cc r7 r10 ROR r0
};
const byte kInstruction_movs_le_r5_r5_ASR_r5[] = {
  0x55, 0x55, 0xb0, 0xd1 // movs le r5 r5 ASR r5
};
const byte kInstruction_movs_hi_r14_r7_ROR_r0[] = {
  0x77, 0xe0, 0xb0, 0x81 // movs hi r14 r7 ROR r0
};
const byte kInstruction_movs_vs_r9_r3_LSL_r11[] = {
  0x13, 0x9b, 0xb0, 0x61 // movs vs r9 r3 LSL r11
};
const byte kInstruction_movs_ls_r0_r5_LSL_r11[] = {
  0x15, 0x0b, 0xb0, 0x91 // movs ls r0 r5 LSL r11
};
const byte kInstruction_movs_ne_r12_r3_LSL_r14[] = {
  0x13, 0xce, 0xb0, 0x11 // movs ne r12 r3 LSL r14
};
const byte kInstruction_movs_lt_r12_r12_ROR_r4[] = {
  0x7c, 0xc4, 0xb0, 0xb1 // movs lt r12 r12 ROR r4
};
const byte kInstruction_movs_gt_r13_r10_LSR_r3[] = {
  0x3a, 0xd3, 0xb0, 0xc1 // movs gt r13 r10 LSR r3
};
const byte kInstruction_movs_ls_r11_r2_ASR_r10[] = {
  0x52, 0xba, 0xb0, 0x91 // movs ls r11 r2 ASR r10
};
const byte kInstruction_movs_vc_r7_r8_ASR_r13[] = {
  0x58, 0x7d, 0xb0, 0x71 // movs vc r7 r8 ASR r13
};
const byte kInstruction_movs_eq_r8_r3_LSR_r13[] = {
  0x33, 0x8d, 0xb0, 0x01 // movs eq r8 r3 LSR r13
};
const byte kInstruction_movs_mi_r10_r6_LSL_r8[] = {
  0x16, 0xa8, 0xb0, 0x41 // movs mi r10 r6 LSL r8
};
const byte kInstruction_movs_le_r11_r11_LSR_r1[] = {
  0x3b, 0xb1, 0xb0, 0xd1 // movs le r11 r11 LSR r1
};
const byte kInstruction_movs_vc_r11_r2_ROR_r13[] = {
  0x72, 0xbd, 0xb0, 0x71 // movs vc r11 r2 ROR r13
};
const byte kInstruction_movs_vs_r8_r9_ASR_r2[] = {
  0x59, 0x82, 0xb0, 0x61 // movs vs r8 r9 ASR r2
};
const byte kInstruction_movs_pl_r11_r11_ASR_r13[] = {
  0x5b, 0xbd, 0xb0, 0x51 // movs pl r11 r11 ASR r13
};
const byte kInstruction_movs_mi_r14_r1_LSL_r7[] = {
  0x11, 0xe7, 0xb0, 0x41 // movs mi r14 r1 LSL r7
};
const byte kInstruction_movs_mi_r7_r7_LSR_r6[] = {
  0x37, 0x76, 0xb0, 0x41 // movs mi r7 r7 LSR r6
};
const byte kInstruction_movs_eq_r5_r13_LSR_r8[] = {
  0x3d, 0x58, 0xb0, 0x01 // movs eq r5 r13 LSR r8
};
const byte kInstruction_movs_cs_r8_r7_ASR_r0[] = {
  0x57, 0x80, 0xb0, 0x21 // movs cs r8 r7 ASR r0
};
const byte kInstruction_movs_pl_r8_r4_LSL_r5[] = {
  0x14, 0x85, 0xb0, 0x51 // movs pl r8 r4 LSL r5
};
const byte kInstruction_movs_pl_r1_r6_ROR_r2[] = {
  0x76, 0x12, 0xb0, 0x51 // movs pl r1 r6 ROR r2
};
const byte kInstruction_movs_cc_r13_r14_ASR_r0[] = {
  0x5e, 0xd0, 0xb0, 0x31 // movs cc r13 r14 ASR r0
};
const byte kInstruction_movs_al_r3_r12_LSL_r3[] = {
  0x1c, 0x33, 0xb0, 0xe1 // movs al r3 r12 LSL r3
};
const byte kInstruction_movs_vs_r12_r10_LSL_r14[] = {
  0x1a, 0xce, 0xb0, 0x61 // movs vs r12 r10 LSL r14
};
const byte kInstruction_movs_lt_r8_r3_ASR_r0[] = {
  0x53, 0x80, 0xb0, 0xb1 // movs lt r8 r3 ASR r0
};
const byte kInstruction_movs_pl_r0_r13_LSL_r2[] = {
  0x1d, 0x02, 0xb0, 0x51 // movs pl r0 r13 LSL r2
};
const byte kInstruction_movs_gt_r8_r2_LSL_r4[] = {
  0x12, 0x84, 0xb0, 0xc1 // movs gt r8 r2 LSL r4
};
const byte kInstruction_movs_mi_r13_r0_LSL_r8[] = {
  0x10, 0xd8, 0xb0, 0x41 // movs mi r13 r0 LSL r8
};
const byte kInstruction_movs_eq_r1_r10_LSL_r14[] = {
  0x1a, 0x1e, 0xb0, 0x01 // movs eq r1 r10 LSL r14
};
const byte kInstruction_movs_hi_r8_r11_ASR_r10[] = {
  0x5b, 0x8a, 0xb0, 0x81 // movs hi r8 r11 ASR r10
};
const byte kInstruction_movs_ne_r11_r9_ASR_r0[] = {
  0x59, 0xb0, 0xb0, 0x11 // movs ne r11 r9 ASR r0
};
const byte kInstruction_movs_cc_r0_r13_ROR_r5[] = {
  0x7d, 0x05, 0xb0, 0x31 // movs cc r0 r13 ROR r5
};
const byte kInstruction_movs_ge_r13_r7_ROR_r5[] = {
  0x77, 0xd5, 0xb0, 0xa1 // movs ge r13 r7 ROR r5
};
const byte kInstruction_movs_eq_r4_r5_LSL_r2[] = {
  0x15, 0x42, 0xb0, 0x01 // movs eq r4 r5 LSL r2
};
const byte kInstruction_movs_ge_r5_r12_ASR_r3[] = {
  0x5c, 0x53, 0xb0, 0xa1 // movs ge r5 r12 ASR r3
};
const byte kInstruction_movs_ge_r9_r7_ROR_r6[] = {
  0x77, 0x96, 0xb0, 0xa1 // movs ge r9 r7 ROR r6
};
const byte kInstruction_movs_cc_r2_r2_LSR_r0[] = {
  0x32, 0x20, 0xb0, 0x31 // movs cc r2 r2 LSR r0
};
const byte kInstruction_movs_cs_r9_r7_ASR_r11[] = {
  0x57, 0x9b, 0xb0, 0x21 // movs cs r9 r7 ASR r11
};
const byte kInstruction_movs_al_r10_r6_ASR_r10[] = {
  0x56, 0xaa, 0xb0, 0xe1 // movs al r10 r6 ASR r10
};
const byte kInstruction_movs_ls_r13_r10_LSL_r10[] = {
  0x1a, 0xda, 0xb0, 0x91 // movs ls r13 r10 LSL r10
};
const byte kInstruction_movs_ne_r14_r13_ASR_r8[] = {
  0x5d, 0xe8, 0xb0, 0x11 // movs ne r14 r13 ASR r8
};
const byte kInstruction_movs_lt_r12_r3_ROR_r10[] = {
  0x73, 0xca, 0xb0, 0xb1 // movs lt r12 r3 ROR r10
};
const byte kInstruction_movs_al_r12_r8_ROR_r8[] = {
  0x78, 0xc8, 0xb0, 0xe1 // movs al r12 r8 ROR r8
};
const byte kInstruction_movs_ls_r5_r8_ASR_r8[] = {
  0x58, 0x58, 0xb0, 0x91 // movs ls r5 r8 ASR r8
};
const byte kInstruction_movs_mi_r13_r11_LSR_r2[] = {
  0x3b, 0xd2, 0xb0, 0x41 // movs mi r13 r11 LSR r2
};
const byte kInstruction_movs_le_r7_r10_ROR_r1[] = {
  0x7a, 0x71, 0xb0, 0xd1 // movs le r7 r10 ROR r1
};
const byte kInstruction_movs_ne_r5_r2_LSL_r11[] = {
  0x12, 0x5b, 0xb0, 0x11 // movs ne r5 r2 LSL r11
};
const byte kInstruction_movs_ne_r14_r7_LSL_r10[] = {
  0x17, 0xea, 0xb0, 0x11 // movs ne r14 r7 LSL r10
};
const byte kInstruction_movs_al_r0_r1_LSR_r14[] = {
  0x31, 0x0e, 0xb0, 0xe1 // movs al r0 r1 LSR r14
};
const byte kInstruction_movs_mi_r5_r9_LSR_r11[] = {
  0x39, 0x5b, 0xb0, 0x41 // movs mi r5 r9 LSR r11
};
const byte kInstruction_movs_eq_r8_r3_LSR_r11[] = {
  0x33, 0x8b, 0xb0, 0x01 // movs eq r8 r3 LSR r11
};
const byte kInstruction_movs_eq_r4_r14_LSR_r11[] = {
  0x3e, 0x4b, 0xb0, 0x01 // movs eq r4 r14 LSR r11
};
const byte kInstruction_movs_le_r0_r7_LSL_r11[] = {
  0x17, 0x0b, 0xb0, 0xd1 // movs le r0 r7 LSL r11
};
const byte kInstruction_movs_le_r2_r2_LSL_r10[] = {
  0x12, 0x2a, 0xb0, 0xd1 // movs le r2 r2 LSL r10
};
const byte kInstruction_movs_le_r3_r5_ROR_r0[] = {
  0x75, 0x30, 0xb0, 0xd1 // movs le r3 r5 ROR r0
};
const byte kInstruction_movs_mi_r8_r10_LSL_r10[] = {
  0x1a, 0x8a, 0xb0, 0x41 // movs mi r8 r10 LSL r10
};
const byte kInstruction_movs_ls_r11_r7_ROR_r8[] = {
  0x77, 0xb8, 0xb0, 0x91 // movs ls r11 r7 ROR r8
};
const byte kInstruction_movs_hi_r14_r11_LSR_r7[] = {
  0x3b, 0xe7, 0xb0, 0x81 // movs hi r14 r11 LSR r7
};
const byte kInstruction_movs_mi_r10_r2_LSR_r1[] = {
  0x32, 0xa1, 0xb0, 0x41 // movs mi r10 r2 LSR r1
};
const byte kInstruction_movs_le_r3_r14_ASR_r10[] = {
  0x5e, 0x3a, 0xb0, 0xd1 // movs le r3 r14 ASR r10
};
const byte kInstruction_movs_vs_r3_r11_ROR_r2[] = {
  0x7b, 0x32, 0xb0, 0x61 // movs vs r3 r11 ROR r2
};
const byte kInstruction_movs_lt_r2_r8_ASR_r2[] = {
  0x58, 0x22, 0xb0, 0xb1 // movs lt r2 r8 ASR r2
};
const byte kInstruction_movs_pl_r7_r4_ROR_r6[] = {
  0x74, 0x76, 0xb0, 0x51 // movs pl r7 r4 ROR r6
};
const byte kInstruction_movs_al_r7_r12_ASR_r12[] = {
  0x5c, 0x7c, 0xb0, 0xe1 // movs al r7 r12 ASR r12
};
const byte kInstruction_movs_ge_r8_r9_ROR_r0[] = {
  0x79, 0x80, 0xb0, 0xa1 // movs ge r8 r9 ROR r0
};
const byte kInstruction_movs_mi_r9_r5_LSL_r7[] = {
  0x15, 0x97, 0xb0, 0x41 // movs mi r9 r5 LSL r7
};
const byte kInstruction_movs_cs_r4_r13_LSL_r9[] = {
  0x1d, 0x49, 0xb0, 0x21 // movs cs r4 r13 LSL r9
};
const byte kInstruction_movs_eq_r5_r3_LSR_r1[] = {
  0x33, 0x51, 0xb0, 0x01 // movs eq r5 r3 LSR r1
};
const byte kInstruction_movs_lt_r6_r14_LSR_r5[] = {
  0x3e, 0x65, 0xb0, 0xb1 // movs lt r6 r14 LSR r5
};
const byte kInstruction_movs_eq_r11_r9_LSL_r11[] = {
  0x19, 0xbb, 0xb0, 0x01 // movs eq r11 r9 LSL r11
};
const byte kInstruction_movs_cs_r4_r0_ROR_r0[] = {
  0x70, 0x40, 0xb0, 0x21 // movs cs r4 r0 ROR r0
};
const byte kInstruction_movs_pl_r11_r13_LSL_r4[] = {
  0x1d, 0xb4, 0xb0, 0x51 // movs pl r11 r13 LSL r4
};
const byte kInstruction_movs_al_r1_r2_LSR_r7[] = {
  0x32, 0x17, 0xb0, 0xe1 // movs al r1 r2 LSR r7
};
const byte kInstruction_movs_mi_r7_r2_ROR_r2[] = {
  0x72, 0x72, 0xb0, 0x41 // movs mi r7 r2 ROR r2
};
const byte kInstruction_movs_ls_r2_r10_LSL_r10[] = {
  0x1a, 0x2a, 0xb0, 0x91 // movs ls r2 r10 LSL r10
};
const byte kInstruction_movs_cs_r11_r8_LSL_r3[] = {
  0x18, 0xb3, 0xb0, 0x21 // movs cs r11 r8 LSL r3
};
const byte kInstruction_movs_cc_r11_r5_ROR_r1[] = {
  0x75, 0xb1, 0xb0, 0x31 // movs cc r11 r5 ROR r1
};
const byte kInstruction_movs_le_r11_r0_ASR_r12[] = {
  0x50, 0xbc, 0xb0, 0xd1 // movs le r11 r0 ASR r12
};
const byte kInstruction_movs_pl_r5_r8_LSL_r6[] = {
  0x18, 0x56, 0xb0, 0x51 // movs pl r5 r8 LSL r6
};
const byte kInstruction_movs_ls_r9_r13_LSL_r10[] = {
  0x1d, 0x9a, 0xb0, 0x91 // movs ls r9 r13 LSL r10
};
const byte kInstruction_movs_ge_r14_r5_LSL_r10[] = {
  0x15, 0xea, 0xb0, 0xa1 // movs ge r14 r5 LSL r10
};
const byte kInstruction_movs_vs_r8_r10_ASR_r2[] = {
  0x5a, 0x82, 0xb0, 0x61 // movs vs r8 r10 ASR r2
};
const byte kInstruction_movs_vs_r11_r11_LSR_r4[] = {
  0x3b, 0xb4, 0xb0, 0x61 // movs vs r11 r11 LSR r4
};
const byte kInstruction_movs_ls_r7_r0_ASR_r7[] = {
  0x50, 0x77, 0xb0, 0x91 // movs ls r7 r0 ASR r7
};
const byte kInstruction_movs_cc_r10_r5_ASR_r13[] = {
  0x55, 0xad, 0xb0, 0x31 // movs cc r10 r5 ASR r13
};
const byte kInstruction_movs_hi_r12_r13_ROR_r5[] = {
  0x7d, 0xc5, 0xb0, 0x81 // movs hi r12 r13 ROR r5
};
const byte kInstruction_movs_le_r12_r14_LSR_r5[] = {
  0x3e, 0xc5, 0xb0, 0xd1 // movs le r12 r14 LSR r5
};
const byte kInstruction_movs_le_r0_r0_LSL_r14[] = {
  0x10, 0x0e, 0xb0, 0xd1 // movs le r0 r0 LSL r14
};
const byte kInstruction_movs_ge_r3_r7_ROR_r14[] = {
  0x77, 0x3e, 0xb0, 0xa1 // movs ge r3 r7 ROR r14
};
const byte kInstruction_movs_eq_r3_r1_LSL_r8[] = {
  0x11, 0x38, 0xb0, 0x01 // movs eq r3 r1 LSL r8
};
const byte kInstruction_movs_lt_r7_r1_LSR_r11[] = {
  0x31, 0x7b, 0xb0, 0xb1 // movs lt r7 r1 LSR r11
};
const byte kInstruction_movs_ls_r0_r10_ROR_r11[] = {
  0x7a, 0x0b, 0xb0, 0x91 // movs ls r0 r10 ROR r11
};
const byte kInstruction_movs_le_r0_r1_LSL_r8[] = {
  0x11, 0x08, 0xb0, 0xd1 // movs le r0 r1 LSL r8
};
const byte kInstruction_movs_mi_r13_r3_LSR_r8[] = {
  0x33, 0xd8, 0xb0, 0x41 // movs mi r13 r3 LSR r8
};
const byte kInstruction_movs_ls_r9_r6_LSR_r0[] = {
  0x36, 0x90, 0xb0, 0x91 // movs ls r9 r6 LSR r0
};
const byte kInstruction_movs_mi_r11_r6_LSR_r3[] = {
  0x36, 0xb3, 0xb0, 0x41 // movs mi r11 r6 LSR r3
};
const byte kInstruction_movs_lt_r11_r6_LSL_r14[] = {
  0x16, 0xbe, 0xb0, 0xb1 // movs lt r11 r6 LSL r14
};
const byte kInstruction_movs_pl_r4_r14_ROR_r5[] = {
  0x7e, 0x45, 0xb0, 0x51 // movs pl r4 r14 ROR r5
};
const byte kInstruction_movs_eq_r7_r6_LSR_r6[] = {
  0x36, 0x76, 0xb0, 0x01 // movs eq r7 r6 LSR r6
};
const byte kInstruction_movs_hi_r11_r7_ASR_r8[] = {
  0x57, 0xb8, 0xb0, 0x81 // movs hi r11 r7 ASR r8
};
const byte kInstruction_movs_al_r7_r14_ASR_r14[] = {
  0x5e, 0x7e, 0xb0, 0xe1 // movs al r7 r14 ASR r14
};
const byte kInstruction_movs_cc_r3_r1_ASR_r10[] = {
  0x51, 0x3a, 0xb0, 0x31 // movs cc r3 r1 ASR r10
};
const byte kInstruction_movs_pl_r9_r3_ROR_r2[] = {
  0x73, 0x92, 0xb0, 0x51 // movs pl r9 r3 ROR r2
};
const byte kInstruction_movs_vc_r6_r9_LSR_r14[] = {
  0x39, 0x6e, 0xb0, 0x71 // movs vc r6 r9 LSR r14
};
const byte kInstruction_movs_le_r3_r14_ASR_r7[] = {
  0x5e, 0x37, 0xb0, 0xd1 // movs le r3 r14 ASR r7
};
const byte kInstruction_movs_le_r12_r14_ROR_r14[] = {
  0x7e, 0xce, 0xb0, 0xd1 // movs le r12 r14 ROR r14
};
const byte kInstruction_movs_pl_r7_r11_ASR_r9[] = {
  0x5b, 0x79, 0xb0, 0x51 // movs pl r7 r11 ASR r9
};
const byte kInstruction_movs_le_r1_r12_ASR_r4[] = {
  0x5c, 0x14, 0xb0, 0xd1 // movs le r1 r12 ASR r4
};
const byte kInstruction_movs_eq_r7_r12_ASR_r6[] = {
  0x5c, 0x76, 0xb0, 0x01 // movs eq r7 r12 ASR r6
};
const byte kInstruction_movs_mi_r12_r12_ROR_r14[] = {
  0x7c, 0xce, 0xb0, 0x41 // movs mi r12 r12 ROR r14
};
const byte kInstruction_movs_pl_r1_r3_ROR_r3[] = {
  0x73, 0x13, 0xb0, 0x51 // movs pl r1 r3 ROR r3
};
const byte kInstruction_movs_hi_r1_r12_ASR_r2[] = {
  0x5c, 0x12, 0xb0, 0x81 // movs hi r1 r12 ASR r2
};
const byte kInstruction_movs_ls_r7_r5_LSR_r14[] = {
  0x35, 0x7e, 0xb0, 0x91 // movs ls r7 r5 LSR r14
};
const byte kInstruction_movs_al_r6_r6_LSR_r2[] = {
  0x36, 0x62, 0xb0, 0xe1 // movs al r6 r6 LSR r2
};
const byte kInstruction_movs_al_r8_r3_ROR_r13[] = {
  0x73, 0x8d, 0xb0, 0xe1 // movs al r8 r3 ROR r13
};
const byte kInstruction_movs_ls_r12_r14_LSL_r11[] = {
  0x1e, 0xcb, 0xb0, 0x91 // movs ls r12 r14 LSL r11
};
const byte kInstruction_movs_ne_r9_r9_ROR_r1[] = {
  0x79, 0x91, 0xb0, 0x11 // movs ne r9 r9 ROR r1
};
const byte kInstruction_movs_hi_r9_r13_ROR_r1[] = {
  0x7d, 0x91, 0xb0, 0x81 // movs hi r9 r13 ROR r1
};
const byte kInstruction_movs_vc_r10_r13_ASR_r13[] = {
  0x5d, 0xad, 0xb0, 0x71 // movs vc r10 r13 ASR r13
};
const byte kInstruction_movs_al_r13_r13_ROR_r7[] = {
  0x7d, 0xd7, 0xb0, 0xe1 // movs al r13 r13 ROR r7
};
const byte kInstruction_movs_ls_r0_r14_LSL_r10[] = {
  0x1e, 0x0a, 0xb0, 0x91 // movs ls r0 r14 LSL r10
};
const byte kInstruction_movs_vc_r9_r14_ROR_r11[] = {
  0x7e, 0x9b, 0xb0, 0x71 // movs vc r9 r14 ROR r11
};
const byte kInstruction_movs_vc_r10_r3_LSR_r14[] = {
  0x33, 0xae, 0xb0, 0x71 // movs vc r10 r3 LSR r14
};
const byte kInstruction_movs_cc_r11_r14_LSR_r3[] = {
  0x3e, 0xb3, 0xb0, 0x31 // movs cc r11 r14 LSR r3
};
const byte kInstruction_movs_ge_r1_r2_LSL_r1[] = {
  0x12, 0x11, 0xb0, 0xa1 // movs ge r1 r2 LSL r1
};
const byte kInstruction_movs_al_r10_r8_LSR_r5[] = {
  0x38, 0xa5, 0xb0, 0xe1 // movs al r10 r8 LSR r5
};
const byte kInstruction_movs_vs_r2_r5_ASR_r8[] = {
  0x55, 0x28, 0xb0, 0x61 // movs vs r2 r5 ASR r8
};
const byte kInstruction_movs_cs_r10_r5_LSL_r11[] = {
  0x15, 0xab, 0xb0, 0x21 // movs cs r10 r5 LSL r11
};
const byte kInstruction_movs_gt_r9_r0_LSR_r6[] = {
  0x30, 0x96, 0xb0, 0xc1 // movs gt r9 r0 LSR r6
};
const byte kInstruction_movs_le_r8_r2_ROR_r0[] = {
  0x72, 0x80, 0xb0, 0xd1 // movs le r8 r2 ROR r0
};
const byte kInstruction_movs_cc_r0_r9_ASR_r4[] = {
  0x59, 0x04, 0xb0, 0x31 // movs cc r0 r9 ASR r4
};
const byte kInstruction_movs_pl_r7_r4_ASR_r4[] = {
  0x54, 0x74, 0xb0, 0x51 // movs pl r7 r4 ASR r4
};
const byte kInstruction_movs_cs_r6_r5_ASR_r6[] = {
  0x55, 0x66, 0xb0, 0x21 // movs cs r6 r5 ASR r6
};
const byte kInstruction_movs_al_r14_r6_LSL_r2[] = {
  0x16, 0xe2, 0xb0, 0xe1 // movs al r14 r6 LSL r2
};
const byte kInstruction_movs_pl_r11_r14_ASR_r4[] = {
  0x5e, 0xb4, 0xb0, 0x51 // movs pl r11 r14 ASR r4
};
const byte kInstruction_movs_pl_r4_r9_LSL_r10[] = {
  0x19, 0x4a, 0xb0, 0x51 // movs pl r4 r9 LSL r10
};
const byte kInstruction_movs_le_r7_r1_ROR_r8[] = {
  0x71, 0x78, 0xb0, 0xd1 // movs le r7 r1 ROR r8
};
const byte kInstruction_movs_al_r3_r1_LSR_r10[] = {
  0x31, 0x3a, 0xb0, 0xe1 // movs al r3 r1 LSR r10
};
const byte kInstruction_movs_ge_r8_r14_LSL_r11[] = {
  0x1e, 0x8b, 0xb0, 0xa1 // movs ge r8 r14 LSL r11
};
const byte kInstruction_movs_le_r11_r5_LSL_r2[] = {
  0x15, 0xb2, 0xb0, 0xd1 // movs le r11 r5 LSL r2
};
const byte kInstruction_movs_le_r13_r2_ROR_r0[] = {
  0x72, 0xd0, 0xb0, 0xd1 // movs le r13 r2 ROR r0
};
const byte kInstruction_movs_eq_r4_r13_ASR_r10[] = {
  0x5d, 0x4a, 0xb0, 0x01 // movs eq r4 r13 ASR r10
};
const byte kInstruction_movs_lt_r14_r5_ASR_r14[] = {
  0x55, 0xee, 0xb0, 0xb1 // movs lt r14 r5 ASR r14
};
const byte kInstruction_movs_gt_r2_r6_ASR_r12[] = {
  0x56, 0x2c, 0xb0, 0xc1 // movs gt r2 r6 ASR r12
};
const byte kInstruction_movs_gt_r9_r1_ASR_r8[] = {
  0x51, 0x98, 0xb0, 0xc1 // movs gt r9 r1 ASR r8
};
const byte kInstruction_movs_vs_r11_r6_LSL_r13[] = {
  0x16, 0xbd, 0xb0, 0x61 // movs vs r11 r6 LSL r13
};
const byte kInstruction_movs_ge_r2_r13_LSL_r3[] = {
  0x1d, 0x23, 0xb0, 0xa1 // movs ge r2 r13 LSL r3
};
const byte kInstruction_movs_hi_r4_r1_ROR_r14[] = {
  0x71, 0x4e, 0xb0, 0x81 // movs hi r4 r1 ROR r14
};
const byte kInstruction_movs_gt_r8_r13_LSL_r2[] = {
  0x1d, 0x82, 0xb0, 0xc1 // movs gt r8 r13 LSL r2
};
const byte kInstruction_movs_pl_r13_r1_LSR_r3[] = {
  0x31, 0xd3, 0xb0, 0x51 // movs pl r13 r1 LSR r3
};
const byte kInstruction_movs_cc_r13_r6_LSL_r10[] = {
  0x16, 0xda, 0xb0, 0x31 // movs cc r13 r6 LSL r10
};
const byte kInstruction_movs_ls_r10_r1_LSR_r1[] = {
  0x31, 0xa1, 0xb0, 0x91 // movs ls r10 r1 LSR r1
};
const byte kInstruction_movs_hi_r12_r2_LSR_r12[] = {
  0x32, 0xcc, 0xb0, 0x81 // movs hi r12 r2 LSR r12
};
const byte kInstruction_movs_ge_r1_r1_ROR_r12[] = {
  0x71, 0x1c, 0xb0, 0xa1 // movs ge r1 r1 ROR r12
};
const byte kInstruction_movs_vc_r12_r0_LSL_r10[] = {
  0x10, 0xca, 0xb0, 0x71 // movs vc r12 r0 LSL r10
};
const byte kInstruction_movs_eq_r5_r8_ROR_r7[] = {
  0x78, 0x57, 0xb0, 0x01 // movs eq r5 r8 ROR r7
};
const byte kInstruction_movs_cc_r5_r1_LSR_r1[] = {
  0x31, 0x51, 0xb0, 0x31 // movs cc r5 r1 LSR r1
};
const byte kInstruction_movs_pl_r7_r14_ASR_r4[] = {
  0x5e, 0x74, 0xb0, 0x51 // movs pl r7 r14 ASR r4
};
const byte kInstruction_movs_ge_r0_r5_LSR_r4[] = {
  0x35, 0x04, 0xb0, 0xa1 // movs ge r0 r5 LSR r4
};
const byte kInstruction_movs_ge_r3_r10_LSL_r11[] = {
  0x1a, 0x3b, 0xb0, 0xa1 // movs ge r3 r10 LSL r11
};
const byte kInstruction_movs_cs_r13_r11_LSL_r9[] = {
  0x1b, 0xd9, 0xb0, 0x21 // movs cs r13 r11 LSL r9
};
const byte kInstruction_movs_hi_r11_r14_LSL_r10[] = {
  0x1e, 0xba, 0xb0, 0x81 // movs hi r11 r14 LSL r10
};
const byte kInstruction_movs_mi_r7_r5_ROR_r7[] = {
  0x75, 0x77, 0xb0, 0x41 // movs mi r7 r5 ROR r7
};
const byte kInstruction_movs_ls_r1_r10_ROR_r13[] = {
  0x7a, 0x1d, 0xb0, 0x91 // movs ls r1 r10 ROR r13
};
const byte kInstruction_movs_mi_r10_r10_LSR_r12[] = {
  0x3a, 0xac, 0xb0, 0x41 // movs mi r10 r10 LSR r12
};
const byte kInstruction_movs_le_r14_r7_LSR_r11[] = {
  0x37, 0xeb, 0xb0, 0xd1 // movs le r14 r7 LSR r11
};
const byte kInstruction_movs_al_r3_r12_LSR_r14[] = {
  0x3c, 0x3e, 0xb0, 0xe1 // movs al r3 r12 LSR r14
};
const byte kInstruction_movs_vs_r1_r3_ROR_r1[] = {
  0x73, 0x11, 0xb0, 0x61 // movs vs r1 r3 ROR r1
};
const byte kInstruction_movs_ge_r4_r10_LSL_r13[] = {
  0x1a, 0x4d, 0xb0, 0xa1 // movs ge r4 r10 LSL r13
};
const byte kInstruction_movs_cs_r4_r3_LSL_r13[] = {
  0x13, 0x4d, 0xb0, 0x21 // movs cs r4 r3 LSL r13
};
const byte kInstruction_movs_ge_r9_r5_ASR_r11[] = {
  0x55, 0x9b, 0xb0, 0xa1 // movs ge r9 r5 ASR r11
};
const byte kInstruction_movs_pl_r12_r4_ROR_r14[] = {
  0x74, 0xce, 0xb0, 0x51 // movs pl r12 r4 ROR r14
};
const byte kInstruction_movs_mi_r11_r5_LSR_r3[] = {
  0x35, 0xb3, 0xb0, 0x41 // movs mi r11 r5 LSR r3
};
const byte kInstruction_movs_mi_r13_r5_LSR_r4[] = {
  0x35, 0xd4, 0xb0, 0x41 // movs mi r13 r5 LSR r4
};
const byte kInstruction_movs_ls_r13_r0_ASR_r14[] = {
  0x50, 0xde, 0xb0, 0x91 // movs ls r13 r0 ASR r14
};
const byte kInstruction_movs_cc_r1_r9_ROR_r8[] = {
  0x79, 0x18, 0xb0, 0x31 // movs cc r1 r9 ROR r8
};
const byte kInstruction_movs_mi_r2_r9_LSL_r7[] = {
  0x19, 0x27, 0xb0, 0x41 // movs mi r2 r9 LSL r7
};
const byte kInstruction_movs_gt_r12_r13_LSR_r1[] = {
  0x3d, 0xc1, 0xb0, 0xc1 // movs gt r12 r13 LSR r1
};
const byte kInstruction_movs_le_r6_r3_LSL_r9[] = {
  0x13, 0x69, 0xb0, 0xd1 // movs le r6 r3 LSL r9
};
const byte kInstruction_movs_hi_r11_r2_LSR_r6[] = {
  0x32, 0xb6, 0xb0, 0x81 // movs hi r11 r2 LSR r6
};
const byte kInstruction_movs_vc_r2_r7_ROR_r10[] = {
  0x77, 0x2a, 0xb0, 0x71 // movs vc r2 r7 ROR r10
};
const byte kInstruction_movs_ge_r1_r1_ASR_r9[] = {
  0x51, 0x19, 0xb0, 0xa1 // movs ge r1 r1 ASR r9
};
const byte kInstruction_movs_le_r8_r14_ROR_r2[] = {
  0x7e, 0x82, 0xb0, 0xd1 // movs le r8 r14 ROR r2
};
const byte kInstruction_movs_cs_r5_r4_LSR_r12[] = {
  0x34, 0x5c, 0xb0, 0x21 // movs cs r5 r4 LSR r12
};
const byte kInstruction_movs_ne_r6_r7_ASR_r10[] = {
  0x57, 0x6a, 0xb0, 0x11 // movs ne r6 r7 ASR r10
};
const byte kInstruction_movs_lt_r8_r3_LSL_r7[] = {
  0x13, 0x87, 0xb0, 0xb1 // movs lt r8 r3 LSL r7
};
const byte kInstruction_movs_ls_r6_r2_LSL_r6[] = {
  0x12, 0x66, 0xb0, 0x91 // movs ls r6 r2 LSL r6
};
const byte kInstruction_movs_hi_r6_r3_ROR_r1[] = {
  0x73, 0x61, 0xb0, 0x81 // movs hi r6 r3 ROR r1
};
const byte kInstruction_movs_le_r10_r9_LSL_r14[] = {
  0x19, 0xae, 0xb0, 0xd1 // movs le r10 r9 LSL r14
};
const byte kInstruction_movs_le_r5_r6_ROR_r11[] = {
  0x76, 0x5b, 0xb0, 0xd1 // movs le r5 r6 ROR r11
};
const byte kInstruction_movs_ne_r7_r12_ASR_r7[] = {
  0x5c, 0x77, 0xb0, 0x11 // movs ne r7 r12 ASR r7
};
const byte kInstruction_movs_gt_r11_r5_ASR_r1[] = {
  0x55, 0xb1, 0xb0, 0xc1 // movs gt r11 r5 ASR r1
};
const byte kInstruction_movs_al_r9_r4_ROR_r10[] = {
  0x74, 0x9a, 0xb0, 0xe1 // movs al r9 r4 ROR r10
};
const byte kInstruction_movs_vc_r5_r9_ROR_r12[] = {
  0x79, 0x5c, 0xb0, 0x71 // movs vc r5 r9 ROR r12
};
const byte kInstruction_movs_hi_r1_r0_LSR_r3[] = {
  0x30, 0x13, 0xb0, 0x81 // movs hi r1 r0 LSR r3
};
const byte kInstruction_movs_mi_r2_r12_ROR_r8[] = {
  0x7c, 0x28, 0xb0, 0x41 // movs mi r2 r12 ROR r8
};
const byte kInstruction_movs_eq_r14_r9_ROR_r10[] = {
  0x79, 0xea, 0xb0, 0x01 // movs eq r14 r9 ROR r10
};
const byte kInstruction_movs_le_r8_r13_LSL_r1[] = {
  0x1d, 0x81, 0xb0, 0xd1 // movs le r8 r13 LSL r1
};
const byte kInstruction_movs_ne_r13_r4_ROR_r4[] = {
  0x74, 0xd4, 0xb0, 0x11 // movs ne r13 r4 ROR r4
};
const byte kInstruction_movs_ge_r4_r13_ROR_r14[] = {
  0x7d, 0x4e, 0xb0, 0xa1 // movs ge r4 r13 ROR r14
};
const byte kInstruction_movs_vc_r10_r10_LSL_r1[] = {
  0x1a, 0xa1, 0xb0, 0x71 // movs vc r10 r10 LSL r1
};
const byte kInstruction_movs_gt_r10_r2_ROR_r10[] = {
  0x72, 0xaa, 0xb0, 0xc1 // movs gt r10 r2 ROR r10
};
const byte kInstruction_movs_eq_r3_r10_ROR_r12[] = {
  0x7a, 0x3c, 0xb0, 0x01 // movs eq r3 r10 ROR r12
};
const byte kInstruction_movs_pl_r8_r8_ASR_r14[] = {
  0x58, 0x8e, 0xb0, 0x51 // movs pl r8 r8 ASR r14
};
const byte kInstruction_movs_hi_r6_r11_LSR_r0[] = {
  0x3b, 0x60, 0xb0, 0x81 // movs hi r6 r11 LSR r0
};
const byte kInstruction_movs_pl_r2_r13_LSR_r13[] = {
  0x3d, 0x2d, 0xb0, 0x51 // movs pl r2 r13 LSR r13
};
const byte kInstruction_movs_cs_r3_r1_ASR_r4[] = {
  0x51, 0x34, 0xb0, 0x21 // movs cs r3 r1 ASR r4
};
const byte kInstruction_movs_ge_r13_r8_ROR_r5[] = {
  0x78, 0xd5, 0xb0, 0xa1 // movs ge r13 r8 ROR r5
};
const byte kInstruction_movs_ne_r3_r9_ASR_r10[] = {
  0x59, 0x3a, 0xb0, 0x11 // movs ne r3 r9 ASR r10
};
const byte kInstruction_movs_cc_r6_r12_ASR_r9[] = {
  0x5c, 0x69, 0xb0, 0x31 // movs cc r6 r12 ASR r9
};
const byte kInstruction_movs_cc_r10_r8_LSR_r10[] = {
  0x38, 0xaa, 0xb0, 0x31 // movs cc r10 r8 LSR r10
};
const byte kInstruction_movs_vc_r7_r0_ASR_r7[] = {
  0x50, 0x77, 0xb0, 0x71 // movs vc r7 r0 ASR r7
};
const byte kInstruction_movs_lt_r2_r5_LSL_r2[] = {
  0x15, 0x22, 0xb0, 0xb1 // movs lt r2 r5 LSL r2
};
const byte kInstruction_movs_gt_r13_r1_ASR_r8[] = {
  0x51, 0xd8, 0xb0, 0xc1 // movs gt r13 r1 ASR r8
};
const byte kInstruction_movs_hi_r11_r14_LSR_r0[] = {
  0x3e, 0xb0, 0xb0, 0x81 // movs hi r11 r14 LSR r0
};
const byte kInstruction_movs_ls_r3_r2_LSL_r2[] = {
  0x12, 0x32, 0xb0, 0x91 // movs ls r3 r2 LSL r2
};
const byte kInstruction_movs_mi_r5_r2_ROR_r13[] = {
  0x72, 0x5d, 0xb0, 0x41 // movs mi r5 r2 ROR r13
};
const byte kInstruction_movs_ne_r14_r2_ASR_r0[] = {
  0x52, 0xe0, 0xb0, 0x11 // movs ne r14 r2 ASR r0
};
const byte kInstruction_movs_pl_r4_r8_LSL_r2[] = {
  0x18, 0x42, 0xb0, 0x51 // movs pl r4 r8 LSL r2
};
const byte kInstruction_movs_ge_r8_r10_LSR_r13[] = {
  0x3a, 0x8d, 0xb0, 0xa1 // movs ge r8 r10 LSR r13
};
const byte kInstruction_movs_mi_r14_r14_ASR_r12[] = {
  0x5e, 0xec, 0xb0, 0x41 // movs mi r14 r14 ASR r12
};
const byte kInstruction_movs_ge_r3_r1_LSL_r9[] = {
  0x11, 0x39, 0xb0, 0xa1 // movs ge r3 r1 LSL r9
};
const byte kInstruction_movs_mi_r12_r4_ASR_r6[] = {
  0x54, 0xc6, 0xb0, 0x41 // movs mi r12 r4 ASR r6
};
const byte kInstruction_movs_le_r1_r4_LSL_r6[] = {
  0x14, 0x16, 0xb0, 0xd1 // movs le r1 r4 LSL r6
};
const byte kInstruction_movs_ge_r14_r9_ROR_r7[] = {
  0x79, 0xe7, 0xb0, 0xa1 // movs ge r14 r9 ROR r7
};
const byte kInstruction_movs_gt_r6_r14_LSL_r7[] = {
  0x1e, 0x67, 0xb0, 0xc1 // movs gt r6 r14 LSL r7
};
const byte kInstruction_movs_vc_r12_r7_ASR_r3[] = {
  0x57, 0xc3, 0xb0, 0x71 // movs vc r12 r7 ASR r3
};
const byte kInstruction_movs_ne_r7_r5_ROR_r0[] = {
  0x75, 0x70, 0xb0, 0x11 // movs ne r7 r5 ROR r0
};
const byte kInstruction_movs_ne_r13_r2_ROR_r12[] = {
  0x72, 0xdc, 0xb0, 0x11 // movs ne r13 r2 ROR r12
};
const byte kInstruction_movs_cc_r9_r2_ASR_r10[] = {
  0x52, 0x9a, 0xb0, 0x31 // movs cc r9 r2 ASR r10
};
const byte kInstruction_movs_ls_r7_r6_LSR_r5[] = {
  0x36, 0x75, 0xb0, 0x91 // movs ls r7 r6 LSR r5
};
const byte kInstruction_movs_gt_r14_r1_ASR_r12[] = {
  0x51, 0xec, 0xb0, 0xc1 // movs gt r14 r1 ASR r12
};
const byte kInstruction_movs_vs_r11_r4_ROR_r4[] = {
  0x74, 0xb4, 0xb0, 0x61 // movs vs r11 r4 ROR r4
};
const byte kInstruction_movs_le_r13_r13_ROR_r14[] = {
  0x7d, 0xde, 0xb0, 0xd1 // movs le r13 r13 ROR r14
};
const byte kInstruction_movs_eq_r12_r3_LSR_r2[] = {
  0x33, 0xc2, 0xb0, 0x01 // movs eq r12 r3 LSR r2
};
const byte kInstruction_movs_vs_r9_r14_LSL_r1[] = {
  0x1e, 0x91, 0xb0, 0x61 // movs vs r9 r14 LSL r1
};
const byte kInstruction_movs_le_r8_r9_LSL_r9[] = {
  0x19, 0x89, 0xb0, 0xd1 // movs le r8 r9 LSL r9
};
const byte kInstruction_movs_ne_r2_r13_ROR_r12[] = {
  0x7d, 0x2c, 0xb0, 0x11 // movs ne r2 r13 ROR r12
};
const byte kInstruction_movs_eq_r0_r14_ROR_r14[] = {
  0x7e, 0x0e, 0xb0, 0x01 // movs eq r0 r14 ROR r14
};
const byte kInstruction_movs_cc_r8_r0_ROR_r9[] = {
  0x70, 0x89, 0xb0, 0x31 // movs cc r8 r0 ROR r9
};
const byte kInstruction_movs_gt_r11_r0_LSR_r9[] = {
  0x30, 0xb9, 0xb0, 0xc1 // movs gt r11 r0 LSR r9
};
const byte kInstruction_movs_vc_r1_r8_LSL_r4[] = {
  0x18, 0x14, 0xb0, 0x71 // movs vc r1 r8 LSL r4
};
const byte kInstruction_movs_ne_r4_r4_ROR_r3[] = {
  0x74, 0x43, 0xb0, 0x11 // movs ne r4 r4 ROR r3
};
const byte kInstruction_movs_cs_r12_r6_LSR_r0[] = {
  0x36, 0xc0, 0xb0, 0x21 // movs cs r12 r6 LSR r0
};
const byte kInstruction_movs_pl_r8_r14_ROR_r7[] = {
  0x7e, 0x87, 0xb0, 0x51 // movs pl r8 r14 ROR r7
};
const byte kInstruction_movs_hi_r7_r11_LSR_r14[] = {
  0x3b, 0x7e, 0xb0, 0x81 // movs hi r7 r11 LSR r14
};
const byte kInstruction_movs_vs_r1_r9_ASR_r14[] = {
  0x59, 0x1e, 0xb0, 0x61 // movs vs r1 r9 ASR r14
};
const byte kInstruction_movs_ge_r11_r4_LSR_r14[] = {
  0x34, 0xbe, 0xb0, 0xa1 // movs ge r11 r4 LSR r14
};
const byte kInstruction_movs_gt_r9_r13_LSL_r0[] = {
  0x1d, 0x90, 0xb0, 0xc1 // movs gt r9 r13 LSL r0
};
const byte kInstruction_movs_cc_r11_r1_LSR_r7[] = {
  0x31, 0xb7, 0xb0, 0x31 // movs cc r11 r1 LSR r7
};
const byte kInstruction_movs_al_r14_r7_ROR_r12[] = {
  0x77, 0xec, 0xb0, 0xe1 // movs al r14 r7 ROR r12
};
const byte kInstruction_movs_vc_r11_r5_ROR_r1[] = {
  0x75, 0xb1, 0xb0, 0x71 // movs vc r11 r5 ROR r1
};
const byte kInstruction_movs_eq_r14_r9_ROR_r2[] = {
  0x79, 0xe2, 0xb0, 0x01 // movs eq r14 r9 ROR r2
};
const byte kInstruction_movs_eq_r6_r10_LSR_r8[] = {
  0x3a, 0x68, 0xb0, 0x01 // movs eq r6 r10 LSR r8
};
const byte kInstruction_movs_cc_r5_r7_LSL_r10[] = {
  0x17, 0x5a, 0xb0, 0x31 // movs cc r5 r7 LSL r10
};
const byte kInstruction_movs_vc_r0_r6_ROR_r5[] = {
  0x76, 0x05, 0xb0, 0x71 // movs vc r0 r6 ROR r5
};
const byte kInstruction_movs_ls_r3_r2_ASR_r5[] = {
  0x52, 0x35, 0xb0, 0x91 // movs ls r3 r2 ASR r5
};
const byte kInstruction_movs_eq_r0_r5_LSL_r13[] = {
  0x15, 0x0d, 0xb0, 0x01 // movs eq r0 r5 LSL r13
};
const byte kInstruction_movs_hi_r3_r2_ASR_r2[] = {
  0x52, 0x32, 0xb0, 0x81 // movs hi r3 r2 ASR r2
};
const byte kInstruction_movs_vs_r6_r1_ASR_r1[] = {
  0x51, 0x61, 0xb0, 0x61 // movs vs r6 r1 ASR r1
};
const byte kInstruction_movs_al_r7_r6_ASR_r11[] = {
  0x56, 0x7b, 0xb0, 0xe1 // movs al r7 r6 ASR r11
};
const byte kInstruction_movs_al_r0_r7_LSR_r6[] = {
  0x37, 0x06, 0xb0, 0xe1 // movs al r0 r7 LSR r6
};
const byte kInstruction_movs_vc_r6_r11_LSL_r13[] = {
  0x1b, 0x6d, 0xb0, 0x71 // movs vc r6 r11 LSL r13
};
const byte kInstruction_movs_mi_r11_r0_LSR_r11[] = {
  0x30, 0xbb, 0xb0, 0x41 // movs mi r11 r0 LSR r11
};
const byte kInstruction_movs_ne_r10_r10_ROR_r7[] = {
  0x7a, 0xa7, 0xb0, 0x11 // movs ne r10 r10 ROR r7
};
const byte kInstruction_movs_eq_r9_r10_ROR_r8[] = {
  0x7a, 0x98, 0xb0, 0x01 // movs eq r9 r10 ROR r8
};
const byte kInstruction_movs_vc_r0_r11_ROR_r6[] = {
  0x7b, 0x06, 0xb0, 0x71 // movs vc r0 r11 ROR r6
};
const byte kInstruction_movs_eq_r3_r2_LSR_r10[] = {
  0x32, 0x3a, 0xb0, 0x01 // movs eq r3 r2 LSR r10
};
const byte kInstruction_movs_gt_r13_r4_LSR_r7[] = {
  0x34, 0xd7, 0xb0, 0xc1 // movs gt r13 r4 LSR r7
};
const byte kInstruction_movs_lt_r1_r12_LSR_r11[] = {
  0x3c, 0x1b, 0xb0, 0xb1 // movs lt r1 r12 LSR r11
};
const byte kInstruction_movs_cs_r12_r7_LSR_r4[] = {
  0x37, 0xc4, 0xb0, 0x21 // movs cs r12 r7 LSR r4
};
const byte kInstruction_movs_cc_r1_r13_ASR_r13[] = {
  0x5d, 0x1d, 0xb0, 0x31 // movs cc r1 r13 ASR r13
};
const byte kInstruction_movs_mi_r4_r9_LSL_r0[] = {
  0x19, 0x40, 0xb0, 0x41 // movs mi r4 r9 LSL r0
};
const byte kInstruction_movs_gt_r14_r7_ROR_r0[] = {
  0x77, 0xe0, 0xb0, 0xc1 // movs gt r14 r7 ROR r0
};
const byte kInstruction_movs_gt_r1_r0_ASR_r6[] = {
  0x50, 0x16, 0xb0, 0xc1 // movs gt r1 r0 ASR r6
};
const byte kInstruction_movs_ne_r14_r10_ASR_r10[] = {
  0x5a, 0xea, 0xb0, 0x11 // movs ne r14 r10 ASR r10
};
const byte kInstruction_movs_pl_r14_r9_LSR_r8[] = {
  0x39, 0xe8, 0xb0, 0x51 // movs pl r14 r9 LSR r8
};
const byte kInstruction_movs_vs_r3_r7_LSL_r6[] = {
  0x17, 0x36, 0xb0, 0x61 // movs vs r3 r7 LSL r6
};
const byte kInstruction_movs_lt_r12_r0_LSL_r3[] = {
  0x10, 0xc3, 0xb0, 0xb1 // movs lt r12 r0 LSL r3
};
const byte kInstruction_movs_hi_r5_r11_ROR_r13[] = {
  0x7b, 0x5d, 0xb0, 0x81 // movs hi r5 r11 ROR r13
};
const byte kInstruction_movs_vc_r5_r2_LSR_r2[] = {
  0x32, 0x52, 0xb0, 0x71 // movs vc r5 r2 LSR r2
};
const byte kInstruction_movs_le_r13_r8_ROR_r0[] = {
  0x78, 0xd0, 0xb0, 0xd1 // movs le r13 r8 ROR r0
};
const byte kInstruction_movs_cc_r13_r9_LSL_r10[] = {
  0x19, 0xda, 0xb0, 0x31 // movs cc r13 r9 LSL r10
};
const byte kInstruction_movs_ne_r0_r3_ASR_r9[] = {
  0x53, 0x09, 0xb0, 0x11 // movs ne r0 r3 ASR r9
};
const byte kInstruction_movs_mi_r0_r8_ASR_r10[] = {
  0x58, 0x0a, 0xb0, 0x41 // movs mi r0 r8 ASR r10
};
const byte kInstruction_movs_cc_r4_r4_ASR_r4[] = {
  0x54, 0x44, 0xb0, 0x31 // movs cc r4 r4 ASR r4
};
const byte kInstruction_movs_vc_r5_r10_LSL_r11[] = {
  0x1a, 0x5b, 0xb0, 0x71 // movs vc r5 r10 LSL r11
};
const byte kInstruction_movs_gt_r11_r10_ROR_r2[] = {
  0x7a, 0xb2, 0xb0, 0xc1 // movs gt r11 r10 ROR r2
};
const byte kInstruction_movs_ne_r5_r5_LSR_r11[] = {
  0x35, 0x5b, 0xb0, 0x11 // movs ne r5 r5 LSR r11
};
const byte kInstruction_movs_ne_r6_r11_LSR_r13[] = {
  0x3b, 0x6d, 0xb0, 0x11 // movs ne r6 r11 LSR r13
};
const byte kInstruction_movs_lt_r3_r1_ROR_r1[] = {
  0x71, 0x31, 0xb0, 0xb1 // movs lt r3 r1 ROR r1
};
const byte kInstruction_movs_cc_r1_r14_ASR_r12[] = {
  0x5e, 0x1c, 0xb0, 0x31 // movs cc r1 r14 ASR r12
};
const byte kInstruction_movs_mi_r2_r10_ROR_r10[] = {
  0x7a, 0x2a, 0xb0, 0x41 // movs mi r2 r10 ROR r10
};
const byte kInstruction_movs_eq_r5_r8_ROR_r10[] = {
  0x78, 0x5a, 0xb0, 0x01 // movs eq r5 r8 ROR r10
};
const byte kInstruction_movs_al_r3_r9_LSL_r2[] = {
  0x19, 0x32, 0xb0, 0xe1 // movs al r3 r9 LSL r2
};
const byte kInstruction_movs_cs_r1_r14_LSR_r7[] = {
  0x3e, 0x17, 0xb0, 0x21 // movs cs r1 r14 LSR r7
};
const byte kInstruction_movs_cc_r14_r9_ROR_r1[] = {
  0x79, 0xe1, 0xb0, 0x31 // movs cc r14 r9 ROR r1
};
const byte kInstruction_movs_al_r3_r8_ROR_r10[] = {
  0x78, 0x3a, 0xb0, 0xe1 // movs al r3 r8 ROR r10
};
const byte kInstruction_movs_vs_r2_r10_ASR_r5[] = {
  0x5a, 0x25, 0xb0, 0x61 // movs vs r2 r10 ASR r5
};
const byte kInstruction_movs_al_r7_r8_ROR_r11[] = {
  0x78, 0x7b, 0xb0, 0xe1 // movs al r7 r8 ROR r11
};
const byte kInstruction_movs_cc_r11_r12_ASR_r13[] = {
  0x5c, 0xbd, 0xb0, 0x31 // movs cc r11 r12 ASR r13
};
const byte kInstruction_movs_cs_r2_r7_LSR_r12[] = {
  0x37, 0x2c, 0xb0, 0x21 // movs cs r2 r7 LSR r12
};
const byte kInstruction_movs_vs_r7_r4_ROR_r14[] = {
  0x74, 0x7e, 0xb0, 0x61 // movs vs r7 r4 ROR r14
};
const byte kInstruction_movs_ne_r12_r1_ROR_r3[] = {
  0x71, 0xc3, 0xb0, 0x11 // movs ne r12 r1 ROR r3
};
const byte kInstruction_movs_ne_r5_r11_LSL_r2[] = {
  0x1b, 0x52, 0xb0, 0x11 // movs ne r5 r11 LSL r2
};
const byte kInstruction_movs_cc_r5_r12_ASR_r13[] = {
  0x5c, 0x5d, 0xb0, 0x31 // movs cc r5 r12 ASR r13
};
const byte kInstruction_movs_gt_r8_r0_LSL_r14[] = {
  0x10, 0x8e, 0xb0, 0xc1 // movs gt r8 r0 LSL r14
};
const byte kInstruction_movs_cc_r5_r4_LSR_r12[] = {
  0x34, 0x5c, 0xb0, 0x31 // movs cc r5 r4 LSR r12
};
const byte kInstruction_movs_ge_r6_r7_LSR_r14[] = {
  0x37, 0x6e, 0xb0, 0xa1 // movs ge r6 r7 LSR r14
};
const byte kInstruction_movs_vs_r2_r0_LSL_r13[] = {
  0x10, 0x2d, 0xb0, 0x61 // movs vs r2 r0 LSL r13
};
const byte kInstruction_movs_gt_r7_r8_LSL_r0[] = {
  0x18, 0x70, 0xb0, 0xc1 // movs gt r7 r8 LSL r0
};
const byte kInstruction_movs_ne_r3_r11_ROR_r8[] = {
  0x7b, 0x38, 0xb0, 0x11 // movs ne r3 r11 ROR r8
};
const byte kInstruction_movs_ne_r0_r8_LSR_r4[] = {
  0x38, 0x04, 0xb0, 0x11 // movs ne r0 r8 LSR r4
};
const byte kInstruction_movs_eq_r1_r14_ASR_r13[] = {
  0x5e, 0x1d, 0xb0, 0x01 // movs eq r1 r14 ASR r13
};
const byte kInstruction_movs_pl_r13_r10_LSL_r4[] = {
  0x1a, 0xd4, 0xb0, 0x51 // movs pl r13 r10 LSL r4
};
const byte kInstruction_movs_mi_r12_r14_LSL_r11[] = {
  0x1e, 0xcb, 0xb0, 0x41 // movs mi r12 r14 LSL r11
};
const byte kInstruction_movs_vc_r6_r4_LSL_r7[] = {
  0x14, 0x67, 0xb0, 0x71 // movs vc r6 r4 LSL r7
};
const byte kInstruction_movs_ge_r7_r8_LSR_r0[] = {
  0x38, 0x70, 0xb0, 0xa1 // movs ge r7 r8 LSR r0
};
const byte kInstruction_movs_ls_r2_r12_LSR_r6[] = {
  0x3c, 0x26, 0xb0, 0x91 // movs ls r2 r12 LSR r6
};
const byte kInstruction_movs_lt_r0_r13_LSL_r1[] = {
  0x1d, 0x01, 0xb0, 0xb1 // movs lt r0 r13 LSL r1
};
const byte kInstruction_movs_vs_r5_r7_LSR_r0[] = {
  0x37, 0x50, 0xb0, 0x61 // movs vs r5 r7 LSR r0
};
const byte kInstruction_movs_ne_r10_r8_ROR_r4[] = {
  0x78, 0xa4, 0xb0, 0x11 // movs ne r10 r8 ROR r4
};
const byte kInstruction_movs_vc_r4_r1_LSL_r7[] = {
  0x11, 0x47, 0xb0, 0x71 // movs vc r4 r1 LSL r7
};
const byte kInstruction_movs_mi_r14_r1_ROR_r10[] = {
  0x71, 0xea, 0xb0, 0x41 // movs mi r14 r1 ROR r10
};
const byte kInstruction_movs_le_r4_r5_LSL_r12[] = {
  0x15, 0x4c, 0xb0, 0xd1 // movs le r4 r5 LSL r12
};
const byte kInstruction_movs_cs_r9_r3_ROR_r10[] = {
  0x73, 0x9a, 0xb0, 0x21 // movs cs r9 r3 ROR r10
};
const byte kInstruction_movs_al_r5_r9_ROR_r7[] = {
  0x79, 0x57, 0xb0, 0xe1 // movs al r5 r9 ROR r7
};
const byte kInstruction_movs_cs_r14_r14_ROR_r1[] = {
  0x7e, 0xe1, 0xb0, 0x21 // movs cs r14 r14 ROR r1
};
const byte kInstruction_movs_vc_r5_r11_LSR_r1[] = {
  0x3b, 0x51, 0xb0, 0x71 // movs vc r5 r11 LSR r1
};
const byte kInstruction_movs_hi_r12_r7_ASR_r3[] = {
  0x57, 0xc3, 0xb0, 0x81 // movs hi r12 r7 ASR r3
};
const byte kInstruction_movs_gt_r13_r3_LSL_r10[] = {
  0x13, 0xda, 0xb0, 0xc1 // movs gt r13 r3 LSL r10
};
const byte kInstruction_movs_pl_r12_r14_ASR_r14[] = {
  0x5e, 0xce, 0xb0, 0x51 // movs pl r12 r14 ASR r14
};
const byte kInstruction_movs_al_r3_r1_ROR_r6[] = {
  0x71, 0x36, 0xb0, 0xe1 // movs al r3 r1 ROR r6
};
const byte kInstruction_movs_le_r13_r2_ROR_r9[] = {
  0x72, 0xd9, 0xb0, 0xd1 // movs le r13 r2 ROR r9
};
const byte kInstruction_movs_cs_r12_r4_LSL_r9[] = {
  0x14, 0xc9, 0xb0, 0x21 // movs cs r12 r4 LSL r9
};
const byte kInstruction_movs_gt_r4_r7_ASR_r14[] = {
  0x57, 0x4e, 0xb0, 0xc1 // movs gt r4 r7 ASR r14
};
const byte kInstruction_movs_vc_r13_r1_ROR_r10[] = {
  0x71, 0xda, 0xb0, 0x71 // movs vc r13 r1 ROR r10
};
const byte kInstruction_movs_ls_r10_r12_LSR_r1[] = {
  0x3c, 0xa1, 0xb0, 0x91 // movs ls r10 r12 LSR r1
};
const byte kInstruction_movs_hi_r8_r0_LSL_r0[] = {
  0x10, 0x80, 0xb0, 0x81 // movs hi r8 r0 LSL r0
};
const byte kInstruction_movs_al_r4_r9_LSL_r0[] = {
  0x19, 0x40, 0xb0, 0xe1 // movs al r4 r9 LSL r0
};
const byte kInstruction_movs_eq_r12_r10_LSL_r14[] = {
  0x1a, 0xce, 0xb0, 0x01 // movs eq r12 r10 LSL r14
};
const byte kInstruction_movs_le_r4_r9_LSL_r12[] = {
  0x19, 0x4c, 0xb0, 0xd1 // movs le r4 r9 LSL r12
};
const byte kInstruction_movs_cc_r1_r6_ASR_r14[] = {
  0x56, 0x1e, 0xb0, 0x31 // movs cc r1 r6 ASR r14
};
const byte kInstruction_movs_pl_r3_r7_ASR_r2[] = {
  0x57, 0x32, 0xb0, 0x51 // movs pl r3 r7 ASR r2
};
const byte kInstruction_movs_eq_r14_r9_ROR_r0[] = {
  0x79, 0xe0, 0xb0, 0x01 // movs eq r14 r9 ROR r0
};
const byte kInstruction_movs_hi_r8_r6_LSR_r5[] = {
  0x36, 0x85, 0xb0, 0x81 // movs hi r8 r6 LSR r5
};
const byte kInstruction_movs_hi_r10_r1_LSL_r0[] = {
  0x11, 0xa0, 0xb0, 0x81 // movs hi r10 r1 LSL r0
};
const byte kInstruction_movs_ls_r0_r13_LSR_r0[] = {
  0x3d, 0x00, 0xb0, 0x91 // movs ls r0 r13 LSR r0
};
const byte kInstruction_movs_vs_r10_r7_ROR_r7[] = {
  0x77, 0xa7, 0xb0, 0x61 // movs vs r10 r7 ROR r7
};
const byte kInstruction_movs_eq_r1_r3_LSR_r6[] = {
  0x33, 0x16, 0xb0, 0x01 // movs eq r1 r3 LSR r6
};
const byte kInstruction_movs_eq_r8_r6_ASR_r0[] = {
  0x56, 0x80, 0xb0, 0x01 // movs eq r8 r6 ASR r0
};
const byte kInstruction_movs_eq_r2_r5_LSR_r10[] = {
  0x35, 0x2a, 0xb0, 0x01 // movs eq r2 r5 LSR r10
};
const byte kInstruction_movs_al_r0_r4_LSL_r13[] = {
  0x14, 0x0d, 0xb0, 0xe1 // movs al r0 r4 LSL r13
};
const byte kInstruction_movs_ge_r0_r12_ROR_r2[] = {
  0x7c, 0x02, 0xb0, 0xa1 // movs ge r0 r12 ROR r2
};
const byte kInstruction_movs_le_r0_r13_ROR_r4[] = {
  0x7d, 0x04, 0xb0, 0xd1 // movs le r0 r13 ROR r4
};
const byte kInstruction_movs_ge_r3_r11_ASR_r6[] = {
  0x5b, 0x36, 0xb0, 0xa1 // movs ge r3 r11 ASR r6
};
const byte kInstruction_movs_ne_r7_r11_ROR_r0[] = {
  0x7b, 0x70, 0xb0, 0x11 // movs ne r7 r11 ROR r0
};
const byte kInstruction_movs_hi_r14_r5_ASR_r0[] = {
  0x55, 0xe0, 0xb0, 0x81 // movs hi r14 r5 ASR r0
};
const byte kInstruction_movs_cs_r12_r9_LSL_r2[] = {
  0x19, 0xc2, 0xb0, 0x21 // movs cs r12 r9 LSL r2
};
const byte kInstruction_movs_cs_r12_r8_ASR_r10[] = {
  0x58, 0xca, 0xb0, 0x21 // movs cs r12 r8 ASR r10
};
const byte kInstruction_movs_cc_r0_r2_LSR_r13[] = {
  0x32, 0x0d, 0xb0, 0x31 // movs cc r0 r2 LSR r13
};
const byte kInstruction_movs_cc_r5_r0_LSR_r4[] = {
  0x30, 0x54, 0xb0, 0x31 // movs cc r5 r0 LSR r4
};
const byte kInstruction_movs_ne_r10_r14_ROR_r3[] = {
  0x7e, 0xa3, 0xb0, 0x11 // movs ne r10 r14 ROR r3
};
const byte kInstruction_movs_hi_r5_r1_LSL_r10[] = {
  0x11, 0x5a, 0xb0, 0x81 // movs hi r5 r1 LSL r10
};
const byte kInstruction_movs_lt_r14_r12_LSR_r13[] = {
  0x3c, 0xed, 0xb0, 0xb1 // movs lt r14 r12 LSR r13
};
const byte kInstruction_movs_hi_r14_r9_LSR_r5[] = {
  0x39, 0xe5, 0xb0, 0x81 // movs hi r14 r9 LSR r5
};
const byte kInstruction_movs_cc_r4_r4_LSR_r3[] = {
  0x34, 0x43, 0xb0, 0x31 // movs cc r4 r4 LSR r3
};
const byte kInstruction_movs_hi_r9_r4_ROR_r13[] = {
  0x74, 0x9d, 0xb0, 0x81 // movs hi r9 r4 ROR r13
};
const byte kInstruction_movs_cc_r7_r6_ASR_r0[] = {
  0x56, 0x70, 0xb0, 0x31 // movs cc r7 r6 ASR r0
};
const byte kInstruction_movs_ne_r6_r12_ROR_r8[] = {
  0x7c, 0x68, 0xb0, 0x11 // movs ne r6 r12 ROR r8
};
const byte kInstruction_movs_cc_r4_r1_LSL_r9[] = {
  0x11, 0x49, 0xb0, 0x31 // movs cc r4 r1 LSL r9
};
const byte kInstruction_movs_ls_r6_r10_LSR_r9[] = {
  0x3a, 0x69, 0xb0, 0x91 // movs ls r6 r10 LSR r9
};
const byte kInstruction_movs_ge_r13_r6_ASR_r4[] = {
  0x56, 0xd4, 0xb0, 0xa1 // movs ge r13 r6 ASR r4
};
const byte kInstruction_movs_pl_r0_r6_ROR_r12[] = {
  0x76, 0x0c, 0xb0, 0x51 // movs pl r0 r6 ROR r12
};
const byte kInstruction_movs_eq_r13_r2_LSL_r6[] = {
  0x12, 0xd6, 0xb0, 0x01 // movs eq r13 r2 LSL r6
};
const byte kInstruction_movs_vc_r5_r14_LSR_r2[] = {
  0x3e, 0x52, 0xb0, 0x71 // movs vc r5 r14 LSR r2
};
const byte kInstruction_movs_ls_r3_r7_LSL_r14[] = {
  0x17, 0x3e, 0xb0, 0x91 // movs ls r3 r7 LSL r14
};
const byte kInstruction_movs_le_r13_r7_ROR_r1[] = {
  0x77, 0xd1, 0xb0, 0xd1 // movs le r13 r7 ROR r1
};
const byte kInstruction_movs_ge_r6_r1_ROR_r2[] = {
  0x71, 0x62, 0xb0, 0xa1 // movs ge r6 r1 ROR r2
};
const byte kInstruction_movs_le_r5_r13_LSL_r8[] = {
  0x1d, 0x58, 0xb0, 0xd1 // movs le r5 r13 LSL r8
};
const byte kInstruction_movs_eq_r13_r14_ROR_r6[] = {
  0x7e, 0xd6, 0xb0, 0x01 // movs eq r13 r14 ROR r6
};
const byte kInstruction_movs_vs_r10_r9_ASR_r2[] = {
  0x59, 0xa2, 0xb0, 0x61 // movs vs r10 r9 ASR r2
};
const byte kInstruction_movs_ls_r7_r10_LSL_r10[] = {
  0x1a, 0x7a, 0xb0, 0x91 // movs ls r7 r10 LSL r10
};
const byte kInstruction_movs_mi_r0_r8_LSL_r13[] = {
  0x18, 0x0d, 0xb0, 0x41 // movs mi r0 r8 LSL r13
};
const byte kInstruction_movs_cc_r6_r5_ROR_r6[] = {
  0x75, 0x66, 0xb0, 0x31 // movs cc r6 r5 ROR r6
};
const byte kInstruction_movs_cc_r5_r14_ASR_r9[] = {
  0x5e, 0x59, 0xb0, 0x31 // movs cc r5 r14 ASR r9
};
const byte kInstruction_movs_ge_r8_r3_ROR_r4[] = {
  0x73, 0x84, 0xb0, 0xa1 // movs ge r8 r3 ROR r4
};
const byte kInstruction_movs_ge_r4_r3_ROR_r0[] = {
  0x73, 0x40, 0xb0, 0xa1 // movs ge r4 r3 ROR r0
};
const byte kInstruction_movs_lt_r12_r13_ROR_r5[] = {
  0x7d, 0xc5, 0xb0, 0xb1 // movs lt r12 r13 ROR r5
};
const byte kInstruction_movs_cs_r14_r14_LSL_r8[] = {
  0x1e, 0xe8, 0xb0, 0x21 // movs cs r14 r14 LSL r8
};
const byte kInstruction_movs_mi_r6_r3_ASR_r12[] = {
  0x53, 0x6c, 0xb0, 0x41 // movs mi r6 r3 ASR r12
};
const byte kInstruction_movs_gt_r2_r1_ASR_r3[] = {
  0x51, 0x23, 0xb0, 0xc1 // movs gt r2 r1 ASR r3
};
const byte kInstruction_movs_vc_r14_r14_LSL_r11[] = {
  0x1e, 0xeb, 0xb0, 0x71 // movs vc r14 r14 LSL r11
};
const byte kInstruction_movs_pl_r12_r14_ASR_r5[] = {
  0x5e, 0xc5, 0xb0, 0x51 // movs pl r12 r14 ASR r5
};
const byte kInstruction_movs_cs_r7_r12_LSL_r14[] = {
  0x1c, 0x7e, 0xb0, 0x21 // movs cs r7 r12 LSL r14
};
const byte kInstruction_movs_eq_r3_r1_LSR_r3[] = {
  0x31, 0x33, 0xb0, 0x01 // movs eq r3 r1 LSR r3
};
const byte kInstruction_movs_cs_r6_r7_ASR_r12[] = {
  0x57, 0x6c, 0xb0, 0x21 // movs cs r6 r7 ASR r12
};
const byte kInstruction_movs_le_r6_r4_ROR_r4[] = {
  0x74, 0x64, 0xb0, 0xd1 // movs le r6 r4 ROR r4
};
const byte kInstruction_movs_hi_r1_r6_LSR_r4[] = {
  0x36, 0x14, 0xb0, 0x81 // movs hi r1 r6 LSR r4
};
const byte kInstruction_movs_mi_r6_r3_ASR_r4[] = {
  0x53, 0x64, 0xb0, 0x41 // movs mi r6 r3 ASR r4
};
const byte kInstruction_movs_vs_r9_r10_ASR_r12[] = {
  0x5a, 0x9c, 0xb0, 0x61 // movs vs r9 r10 ASR r12
};
const byte kInstruction_movs_cc_r9_r10_LSL_r1[] = {
  0x1a, 0x91, 0xb0, 0x31 // movs cc r9 r10 LSL r1
};
const byte kInstruction_movs_ls_r12_r9_ASR_r4[] = {
  0x59, 0xc4, 0xb0, 0x91 // movs ls r12 r9 ASR r4
};
const byte kInstruction_movs_vc_r10_r7_ASR_r12[] = {
  0x57, 0xac, 0xb0, 0x71 // movs vc r10 r7 ASR r12
};
const byte kInstruction_movs_cs_r8_r1_LSR_r5[] = {
  0x31, 0x85, 0xb0, 0x21 // movs cs r8 r1 LSR r5
};
const byte kInstruction_movs_eq_r10_r10_ASR_r4[] = {
  0x5a, 0xa4, 0xb0, 0x01 // movs eq r10 r10 ASR r4
};
const byte kInstruction_movs_ls_r7_r7_LSL_r5[] = {
  0x17, 0x75, 0xb0, 0x91 // movs ls r7 r7 LSL r5
};
const byte kInstruction_movs_vs_r8_r11_ROR_r12[] = {
  0x7b, 0x8c, 0xb0, 0x61 // movs vs r8 r11 ROR r12
};
const byte kInstruction_movs_vs_r7_r6_ASR_r4[] = {
  0x56, 0x74, 0xb0, 0x61 // movs vs r7 r6 ASR r4
};
const byte kInstruction_movs_ge_r5_r3_LSL_r6[] = {
  0x13, 0x56, 0xb0, 0xa1 // movs ge r5 r3 LSL r6
};
const byte kInstruction_movs_vc_r5_r5_LSL_r12[] = {
  0x15, 0x5c, 0xb0, 0x71 // movs vc r5 r5 LSL r12
};
const byte kInstruction_movs_le_r0_r13_LSL_r12[] = {
  0x1d, 0x0c, 0xb0, 0xd1 // movs le r0 r13 LSL r12
};
const byte kInstruction_movs_ls_r4_r0_ASR_r11[] = {
  0x50, 0x4b, 0xb0, 0x91 // movs ls r4 r0 ASR r11
};
const byte kInstruction_movs_ge_r3_r7_ASR_r5[] = {
  0x57, 0x35, 0xb0, 0xa1 // movs ge r3 r7 ASR r5
};
const byte kInstruction_movs_ge_r11_r4_ROR_r11[] = {
  0x74, 0xbb, 0xb0, 0xa1 // movs ge r11 r4 ROR r11
};
const byte kInstruction_movs_al_r6_r7_ROR_r7[] = {
  0x77, 0x67, 0xb0, 0xe1 // movs al r6 r7 ROR r7
};
const byte kInstruction_movs_gt_r4_r4_LSR_r6[] = {
  0x34, 0x46, 0xb0, 0xc1 // movs gt r4 r4 LSR r6
};
const byte kInstruction_movs_pl_r14_r2_LSR_r1[] = {
  0x32, 0xe1, 0xb0, 0x51 // movs pl r14 r2 LSR r1
};
const byte kInstruction_movs_le_r10_r9_ROR_r9[] = {
  0x79, 0xa9, 0xb0, 0xd1 // movs le r10 r9 ROR r9
};
const byte kInstruction_movs_ne_r5_r3_ASR_r2[] = {
  0x53, 0x52, 0xb0, 0x11 // movs ne r5 r3 ASR r2
};
const byte kInstruction_movs_lt_r12_r4_LSL_r11[] = {
  0x14, 0xcb, 0xb0, 0xb1 // movs lt r12 r4 LSL r11
};
const byte kInstruction_movs_mi_r1_r1_LSR_r13[] = {
  0x31, 0x1d, 0xb0, 0x41 // movs mi r1 r1 LSR r13
};
const byte kInstruction_movs_hi_r7_r4_ROR_r9[] = {
  0x74, 0x79, 0xb0, 0x81 // movs hi r7 r4 ROR r9
};
const byte kInstruction_movs_lt_r8_r14_LSR_r10[] = {
  0x3e, 0x8a, 0xb0, 0xb1 // movs lt r8 r14 LSR r10
};
const byte kInstruction_movs_ge_r7_r5_ROR_r11[] = {
  0x75, 0x7b, 0xb0, 0xa1 // movs ge r7 r5 ROR r11
};
const byte kInstruction_movs_ls_r12_r14_LSL_r7[] = {
  0x1e, 0xc7, 0xb0, 0x91 // movs ls r12 r14 LSL r7
};
const byte kInstruction_movs_hi_r14_r0_LSL_r3[] = {
  0x10, 0xe3, 0xb0, 0x81 // movs hi r14 r0 LSL r3
};
const byte kInstruction_movs_ge_r8_r10_ROR_r2[] = {
  0x7a, 0x82, 0xb0, 0xa1 // movs ge r8 r10 ROR r2
};
const byte kInstruction_movs_hi_r6_r3_LSR_r0[] = {
  0x33, 0x60, 0xb0, 0x81 // movs hi r6 r3 LSR r0
};
const byte kInstruction_movs_ls_r4_r1_ROR_r3[] = {
  0x71, 0x43, 0xb0, 0x91 // movs ls r4 r1 ROR r3
};
const byte kInstruction_movs_ls_r9_r13_ASR_r1[] = {
  0x5d, 0x91, 0xb0, 0x91 // movs ls r9 r13 ASR r1
};
const byte kInstruction_movs_lt_r10_r1_LSR_r5[] = {
  0x31, 0xa5, 0xb0, 0xb1 // movs lt r10 r1 LSR r5
};
const byte kInstruction_movs_vc_r10_r13_ASR_r3[] = {
  0x5d, 0xa3, 0xb0, 0x71 // movs vc r10 r13 ASR r3
};
const byte kInstruction_movs_gt_r1_r0_LSR_r9[] = {
  0x30, 0x19, 0xb0, 0xc1 // movs gt r1 r0 LSR r9
};
const byte kInstruction_movs_vs_r13_r11_LSL_r14[] = {
  0x1b, 0xde, 0xb0, 0x61 // movs vs r13 r11 LSL r14
};
const byte kInstruction_movs_eq_r4_r7_LSL_r9[] = {
  0x17, 0x49, 0xb0, 0x01 // movs eq r4 r7 LSL r9
};
const byte kInstruction_movs_lt_r14_r9_LSL_r13[] = {
  0x19, 0xed, 0xb0, 0xb1 // movs lt r14 r9 LSL r13
};
const byte kInstruction_movs_le_r8_r8_ASR_r0[] = {
  0x58, 0x80, 0xb0, 0xd1 // movs le r8 r8 ASR r0
};
const byte kInstruction_movs_vs_r14_r9_LSR_r3[] = {
  0x39, 0xe3, 0xb0, 0x61 // movs vs r14 r9 LSR r3
};
const byte kInstruction_movs_al_r10_r6_ROR_r13[] = {
  0x76, 0xad, 0xb0, 0xe1 // movs al r10 r6 ROR r13
};
const byte kInstruction_movs_vs_r5_r2_ROR_r7[] = {
  0x72, 0x57, 0xb0, 0x61 // movs vs r5 r2 ROR r7
};
const byte kInstruction_movs_vs_r14_r12_ROR_r14[] = {
  0x7c, 0xee, 0xb0, 0x61 // movs vs r14 r12 ROR r14
};
const byte kInstruction_movs_lt_r11_r2_ASR_r0[] = {
  0x52, 0xb0, 0xb0, 0xb1 // movs lt r11 r2 ASR r0
};
const byte kInstruction_movs_vc_r3_r2_LSL_r9[] = {
  0x12, 0x39, 0xb0, 0x71 // movs vc r3 r2 LSL r9
};
const byte kInstruction_movs_hi_r14_r4_LSL_r2[] = {
  0x14, 0xe2, 0xb0, 0x81 // movs hi r14 r4 LSL r2
};
const byte kInstruction_movs_gt_r8_r9_ASR_r0[] = {
  0x59, 0x80, 0xb0, 0xc1 // movs gt r8 r9 ASR r0
};
const byte kInstruction_movs_gt_r4_r4_LSL_r11[] = {
  0x14, 0x4b, 0xb0, 0xc1 // movs gt r4 r4 LSL r11
};
const byte kInstruction_movs_cc_r5_r14_LSL_r9[] = {
  0x1e, 0x59, 0xb0, 0x31 // movs cc r5 r14 LSL r9
};
const byte kInstruction_movs_lt_r10_r10_ASR_r5[] = {
  0x5a, 0xa5, 0xb0, 0xb1 // movs lt r10 r10 ASR r5
};
const byte kInstruction_movs_eq_r1_r12_LSR_r7[] = {
  0x3c, 0x17, 0xb0, 0x01 // movs eq r1 r12 LSR r7
};
const byte kInstruction_movs_eq_r11_r9_ROR_r7[] = {
  0x79, 0xb7, 0xb0, 0x01 // movs eq r11 r9 ROR r7
};
const byte kInstruction_movs_gt_r5_r7_ASR_r6[] = {
  0x57, 0x56, 0xb0, 0xc1 // movs gt r5 r7 ASR r6
};
const byte kInstruction_movs_cc_r7_r2_LSL_r13[] = {
  0x12, 0x7d, 0xb0, 0x31 // movs cc r7 r2 LSL r13
};
const byte kInstruction_movs_cs_r12_r12_LSL_r4[] = {
  0x1c, 0xc4, 0xb0, 0x21 // movs cs r12 r12 LSL r4
};
const byte kInstruction_movs_ls_r4_r4_ROR_r7[] = {
  0x74, 0x47, 0xb0, 0x91 // movs ls r4 r4 ROR r7
};
const byte kInstruction_movs_al_r13_r8_ASR_r10[] = {
  0x58, 0xda, 0xb0, 0xe1 // movs al r13 r8 ASR r10
};
const byte kInstruction_movs_cs_r1_r0_ASR_r1[] = {
  0x50, 0x11, 0xb0, 0x21 // movs cs r1 r0 ASR r1
};
const byte kInstruction_movs_ne_r6_r7_LSL_r0[] = {
  0x17, 0x60, 0xb0, 0x11 // movs ne r6 r7 LSL r0
};
const byte kInstruction_movs_cc_r5_r13_LSR_r9[] = {
  0x3d, 0x59, 0xb0, 0x31 // movs cc r5 r13 LSR r9
};
const byte kInstruction_movs_cc_r9_r6_LSL_r14[] = {
  0x16, 0x9e, 0xb0, 0x31 // movs cc r9 r6 LSL r14
};
const byte kInstruction_movs_al_r12_r13_ASR_r2[] = {
  0x5d, 0xc2, 0xb0, 0xe1 // movs al r12 r13 ASR r2
};
const byte kInstruction_movs_gt_r0_r6_LSR_r6[] = {
  0x36, 0x06, 0xb0, 0xc1 // movs gt r0 r6 LSR r6
};
const byte kInstruction_movs_al_r6_r10_ROR_r10[] = {
  0x7a, 0x6a, 0xb0, 0xe1 // movs al r6 r10 ROR r10
};
const byte kInstruction_movs_lt_r4_r13_LSL_r6[] = {
  0x1d, 0x46, 0xb0, 0xb1 // movs lt r4 r13 LSL r6
};
const byte kInstruction_movs_hi_r12_r0_LSL_r10[] = {
  0x10, 0xca, 0xb0, 0x81 // movs hi r12 r0 LSL r10
};
const byte kInstruction_movs_ge_r5_r9_ROR_r11[] = {
  0x79, 0x5b, 0xb0, 0xa1 // movs ge r5 r9 ROR r11
};
const byte kInstruction_movs_ge_r4_r5_ASR_r10[] = {
  0x55, 0x4a, 0xb0, 0xa1 // movs ge r4 r5 ASR r10
};
const byte kInstruction_movs_al_r0_r5_LSL_r11[] = {
  0x15, 0x0b, 0xb0, 0xe1 // movs al r0 r5 LSL r11
};
const byte kInstruction_movs_mi_r6_r3_LSR_r8[] = {
  0x33, 0x68, 0xb0, 0x41 // movs mi r6 r3 LSR r8
};
const byte kInstruction_movs_gt_r2_r6_LSR_r6[] = {
  0x36, 0x26, 0xb0, 0xc1 // movs gt r2 r6 LSR r6
};
const byte kInstruction_movs_le_r7_r3_ROR_r10[] = {
  0x73, 0x7a, 0xb0, 0xd1 // movs le r7 r3 ROR r10
};
const byte kInstruction_movs_vs_r8_r5_LSR_r13[] = {
  0x35, 0x8d, 0xb0, 0x61 // movs vs r8 r5 LSR r13
};
const byte kInstruction_movs_al_r10_r7_ASR_r14[] = {
  0x57, 0xae, 0xb0, 0xe1 // movs al r10 r7 ASR r14
};
const byte kInstruction_movs_lt_r10_r11_LSL_r5[] = {
  0x1b, 0xa5, 0xb0, 0xb1 // movs lt r10 r11 LSL r5
};
const byte kInstruction_movs_lt_r11_r3_LSR_r4[] = {
  0x33, 0xb4, 0xb0, 0xb1 // movs lt r11 r3 LSR r4
};
const byte kInstruction_movs_vc_r11_r10_LSR_r8[] = {
  0x3a, 0xb8, 0xb0, 0x71 // movs vc r11 r10 LSR r8
};
const byte kInstruction_movs_cs_r11_r5_LSL_r9[] = {
  0x15, 0xb9, 0xb0, 0x21 // movs cs r11 r5 LSL r9
};
const byte kInstruction_movs_cs_r6_r5_LSR_r7[] = {
  0x35, 0x67, 0xb0, 0x21 // movs cs r6 r5 LSR r7
};
const byte kInstruction_movs_lt_r7_r7_ASR_r1[] = {
  0x57, 0x71, 0xb0, 0xb1 // movs lt r7 r7 ASR r1
};
const byte kInstruction_movs_cc_r7_r4_ROR_r7[] = {
  0x74, 0x77, 0xb0, 0x31 // movs cc r7 r4 ROR r7
};
const byte kInstruction_movs_ls_r8_r12_LSR_r9[] = {
  0x3c, 0x89, 0xb0, 0x91 // movs ls r8 r12 LSR r9
};
const byte kInstruction_movs_le_r5_r13_ROR_r11[] = {
  0x7d, 0x5b, 0xb0, 0xd1 // movs le r5 r13 ROR r11
};
const byte kInstruction_movs_eq_r4_r5_ROR_r12[] = {
  0x75, 0x4c, 0xb0, 0x01 // movs eq r4 r5 ROR r12
};
const byte kInstruction_movs_ls_r0_r0_LSL_r1[] = {
  0x10, 0x01, 0xb0, 0x91 // movs ls r0 r0 LSL r1
};
const byte kInstruction_movs_ge_r7_r2_ROR_r9[] = {
  0x72, 0x79, 0xb0, 0xa1 // movs ge r7 r2 ROR r9
};
const byte kInstruction_movs_cc_r5_r6_LSR_r12[] = {
  0x36, 0x5c, 0xb0, 0x31 // movs cc r5 r6 LSR r12
};
const byte kInstruction_movs_cs_r7_r4_ASR_r10[] = {
  0x54, 0x7a, 0xb0, 0x21 // movs cs r7 r4 ASR r10
};
const byte kInstruction_movs_al_r5_r4_ROR_r14[] = {
  0x74, 0x5e, 0xb0, 0xe1 // movs al r5 r4 ROR r14
};
const byte kInstruction_movs_eq_r7_r3_ROR_r12[] = {
  0x73, 0x7c, 0xb0, 0x01 // movs eq r7 r3 ROR r12
};
const byte kInstruction_movs_cc_r9_r4_LSL_r14[] = {
  0x14, 0x9e, 0xb0, 0x31 // movs cc r9 r4 LSL r14
};
const byte kInstruction_movs_le_r5_r2_ASR_r13[] = {
  0x52, 0x5d, 0xb0, 0xd1 // movs le r5 r2 ASR r13
};
const byte kInstruction_movs_eq_r14_r12_ROR_r4[] = {
  0x7c, 0xe4, 0xb0, 0x01 // movs eq r14 r12 ROR r4
};
const byte kInstruction_movs_eq_r5_r2_ASR_r8[] = {
  0x52, 0x58, 0xb0, 0x01 // movs eq r5 r2 ASR r8
};
const byte kInstruction_movs_gt_r12_r5_LSR_r7[] = {
  0x35, 0xc7, 0xb0, 0xc1 // movs gt r12 r5 LSR r7
};
const byte kInstruction_movs_cc_r11_r14_ROR_r1[] = {
  0x7e, 0xb1, 0xb0, 0x31 // movs cc r11 r14 ROR r1
};
const byte kInstruction_movs_ge_r0_r11_LSL_r9[] = {
  0x1b, 0x09, 0xb0, 0xa1 // movs ge r0 r11 LSL r9
};
const byte kInstruction_movs_ls_r4_r4_ROR_r0[] = {
  0x74, 0x40, 0xb0, 0x91 // movs ls r4 r4 ROR r0
};
const byte kInstruction_movs_cs_r7_r9_ROR_r10[] = {
  0x79, 0x7a, 0xb0, 0x21 // movs cs r7 r9 ROR r10
};
const byte kInstruction_movs_le_r9_r4_LSR_r5[] = {
  0x34, 0x95, 0xb0, 0xd1 // movs le r9 r4 LSR r5
};
const byte kInstruction_movs_ls_r1_r4_LSL_r10[] = {
  0x14, 0x1a, 0xb0, 0x91 // movs ls r1 r4 LSL r10
};
const byte kInstruction_movs_lt_r9_r13_LSL_r2[] = {
  0x1d, 0x92, 0xb0, 0xb1 // movs lt r9 r13 LSL r2
};
const byte kInstruction_movs_vs_r13_r6_ROR_r13[] = {
  0x76, 0xdd, 0xb0, 0x61 // movs vs r13 r6 ROR r13
};
const byte kInstruction_movs_cs_r4_r0_LSL_r9[] = {
  0x10, 0x49, 0xb0, 0x21 // movs cs r4 r0 LSL r9
};
const byte kInstruction_movs_le_r8_r13_LSR_r14[] = {
  0x3d, 0x8e, 0xb0, 0xd1 // movs le r8 r13 LSR r14
};
const byte kInstruction_movs_al_r9_r7_LSL_r10[] = {
  0x17, 0x9a, 0xb0, 0xe1 // movs al r9 r7 LSL r10
};
const byte kInstruction_movs_mi_r6_r8_LSL_r2[] = {
  0x18, 0x62, 0xb0, 0x41 // movs mi r6 r8 LSL r2
};
const byte kInstruction_movs_ne_r6_r5_ASR_r5[] = {
  0x55, 0x65, 0xb0, 0x11 // movs ne r6 r5 ASR r5
};
const byte kInstruction_movs_ls_r1_r8_LSR_r12[] = {
  0x38, 0x1c, 0xb0, 0x91 // movs ls r1 r8 LSR r12
};
const byte kInstruction_movs_cs_r9_r7_LSL_r10[] = {
  0x17, 0x9a, 0xb0, 0x21 // movs cs r9 r7 LSL r10
};
const byte kInstruction_movs_cc_r3_r0_ROR_r2[] = {
  0x70, 0x32, 0xb0, 0x31 // movs cc r3 r0 ROR r2
};
const byte kInstruction_movs_vc_r5_r6_ASR_r11[] = {
  0x56, 0x5b, 0xb0, 0x71 // movs vc r5 r6 ASR r11
};
const byte kInstruction_movs_le_r11_r9_ROR_r10[] = {
  0x79, 0xba, 0xb0, 0xd1 // movs le r11 r9 ROR r10
};
const byte kInstruction_movs_gt_r4_r10_ROR_r13[] = {
  0x7a, 0x4d, 0xb0, 0xc1 // movs gt r4 r10 ROR r13
};
const byte kInstruction_movs_lt_r13_r14_LSL_r11[] = {
  0x1e, 0xdb, 0xb0, 0xb1 // movs lt r13 r14 LSL r11
};
const byte kInstruction_movs_ls_r10_r8_LSL_r11[] = {
  0x18, 0xab, 0xb0, 0x91 // movs ls r10 r8 LSL r11
};
const byte kInstruction_movs_gt_r7_r9_ROR_r12[] = {
  0x79, 0x7c, 0xb0, 0xc1 // movs gt r7 r9 ROR r12
};
const byte kInstruction_movs_le_r11_r8_LSR_r0[] = {
  0x38, 0xb0, 0xb0, 0xd1 // movs le r11 r8 LSR r0
};
const byte kInstruction_movs_vs_r3_r1_LSR_r3[] = {
  0x31, 0x33, 0xb0, 0x61 // movs vs r3 r1 LSR r3
};
const byte kInstruction_movs_cs_r14_r13_ROR_r5[] = {
  0x7d, 0xe5, 0xb0, 0x21 // movs cs r14 r13 ROR r5
};
const byte kInstruction_movs_pl_r1_r0_LSL_r6[] = {
  0x10, 0x16, 0xb0, 0x51 // movs pl r1 r0 LSL r6
};
const byte kInstruction_movs_hi_r13_r8_ROR_r12[] = {
  0x78, 0xdc, 0xb0, 0x81 // movs hi r13 r8 ROR r12
};
const byte kInstruction_movs_vc_r6_r7_LSR_r0[] = {
  0x37, 0x60, 0xb0, 0x71 // movs vc r6 r7 LSR r0
};
const byte kInstruction_movs_gt_r4_r10_LSL_r4[] = {
  0x1a, 0x44, 0xb0, 0xc1 // movs gt r4 r10 LSL r4
};
const byte kInstruction_movs_ge_r4_r12_LSR_r7[] = {
  0x3c, 0x47, 0xb0, 0xa1 // movs ge r4 r12 LSR r7
};
const byte kInstruction_movs_pl_r1_r8_LSR_r3[] = {
  0x38, 0x13, 0xb0, 0x51 // movs pl r1 r8 LSR r3
};
const byte kInstruction_movs_hi_r0_r0_ASR_r7[] = {
  0x50, 0x07, 0xb0, 0x81 // movs hi r0 r0 ASR r7
};
const byte kInstruction_movs_cs_r0_r7_ROR_r12[] = {
  0x77, 0x0c, 0xb0, 0x21 // movs cs r0 r7 ROR r12
};
const byte kInstruction_movs_eq_r10_r9_LSR_r0[] = {
  0x39, 0xa0, 0xb0, 0x01 // movs eq r10 r9 LSR r0
};
const byte kInstruction_movs_cc_r5_r10_ROR_r10[] = {
  0x7a, 0x5a, 0xb0, 0x31 // movs cc r5 r10 ROR r10
};
const byte kInstruction_movs_cs_r0_r14_ROR_r8[] = {
  0x7e, 0x08, 0xb0, 0x21 // movs cs r0 r14 ROR r8
};
const byte kInstruction_movs_eq_r14_r11_LSR_r5[] = {
  0x3b, 0xe5, 0xb0, 0x01 // movs eq r14 r11 LSR r5
};
const byte kInstruction_movs_le_r14_r4_LSR_r12[] = {
  0x34, 0xec, 0xb0, 0xd1 // movs le r14 r4 LSR r12
};
const byte kInstruction_movs_lt_r10_r8_LSL_r10[] = {
  0x18, 0xaa, 0xb0, 0xb1 // movs lt r10 r8 LSL r10
};
const byte kInstruction_movs_le_r13_r4_LSR_r0[] = {
  0x34, 0xd0, 0xb0, 0xd1 // movs le r13 r4 LSR r0
};
const byte kInstruction_movs_vc_r9_r9_LSL_r14[] = {
  0x19, 0x9e, 0xb0, 0x71 // movs vc r9 r9 LSL r14
};
const byte kInstruction_movs_vs_r1_r2_LSR_r7[] = {
  0x32, 0x17, 0xb0, 0x61 // movs vs r1 r2 LSR r7
};
const byte kInstruction_movs_al_r14_r4_ROR_r4[] = {
  0x74, 0xe4, 0xb0, 0xe1 // movs al r14 r4 ROR r4
};
const byte kInstruction_movs_vs_r10_r6_ROR_r10[] = {
  0x76, 0xaa, 0xb0, 0x61 // movs vs r10 r6 ROR r10
};
const byte kInstruction_movs_cc_r11_r13_LSL_r11[] = {
  0x1d, 0xbb, 0xb0, 0x31 // movs cc r11 r13 LSL r11
};
const byte kInstruction_movs_lt_r4_r9_ROR_r2[] = {
  0x79, 0x42, 0xb0, 0xb1 // movs lt r4 r9 ROR r2
};
const byte kInstruction_movs_gt_r4_r12_ASR_r8[] = {
  0x5c, 0x48, 0xb0, 0xc1 // movs gt r4 r12 ASR r8
};
const byte kInstruction_movs_hi_r3_r6_LSR_r14[] = {
  0x36, 0x3e, 0xb0, 0x81 // movs hi r3 r6 LSR r14
};
const byte kInstruction_movs_vs_r11_r6_LSL_r5[] = {
  0x16, 0xb5, 0xb0, 0x61 // movs vs r11 r6 LSL r5
};
const byte kInstruction_movs_ne_r14_r12_LSR_r9[] = {
  0x3c, 0xe9, 0xb0, 0x11 // movs ne r14 r12 LSR r9
};
const byte kInstruction_movs_cc_r12_r5_ROR_r0[] = {
  0x75, 0xc0, 0xb0, 0x31 // movs cc r12 r5 ROR r0
};
const byte kInstruction_movs_al_r11_r1_ASR_r14[] = {
  0x51, 0xbe, 0xb0, 0xe1 // movs al r11 r1 ASR r14
};
const byte kInstruction_movs_ge_r9_r1_LSR_r12[] = {
  0x31, 0x9c, 0xb0, 0xa1 // movs ge r9 r1 LSR r12
};
const byte kInstruction_movs_pl_r12_r1_LSR_r11[] = {
  0x31, 0xcb, 0xb0, 0x51 // movs pl r12 r1 LSR r11
};
const byte kInstruction_movs_pl_r0_r1_ROR_r4[] = {
  0x71, 0x04, 0xb0, 0x51 // movs pl r0 r1 ROR r4
};
const byte kInstruction_movs_mi_r10_r12_ROR_r10[] = {
  0x7c, 0xaa, 0xb0, 0x41 // movs mi r10 r12 ROR r10
};
const byte kInstruction_movs_cs_r4_r5_ASR_r0[] = {
  0x55, 0x40, 0xb0, 0x21 // movs cs r4 r5 ASR r0
};
const byte kInstruction_movs_lt_r6_r3_ASR_r11[] = {
  0x53, 0x6b, 0xb0, 0xb1 // movs lt r6 r3 ASR r11
};
const byte kInstruction_movs_cs_r5_r13_ASR_r6[] = {
  0x5d, 0x56, 0xb0, 0x21 // movs cs r5 r13 ASR r6
};
const byte kInstruction_movs_vs_r10_r3_LSR_r10[] = {
  0x33, 0xaa, 0xb0, 0x61 // movs vs r10 r3 LSR r10
};
const byte kInstruction_movs_vc_r5_r12_LSR_r8[] = {
  0x3c, 0x58, 0xb0, 0x71 // movs vc r5 r12 LSR r8
};
const byte kInstruction_movs_al_r3_r10_ROR_r8[] = {
  0x7a, 0x38, 0xb0, 0xe1 // movs al r3 r10 ROR r8
};
const byte kInstruction_movs_vc_r11_r4_ASR_r0[] = {
  0x54, 0xb0, 0xb0, 0x71 // movs vc r11 r4 ASR r0
};
const byte kInstruction_movs_ge_r10_r1_LSL_r11[] = {
  0x11, 0xab, 0xb0, 0xa1 // movs ge r10 r1 LSL r11
};
const byte kInstruction_movs_eq_r1_r3_LSR_r8[] = {
  0x33, 0x18, 0xb0, 0x01 // movs eq r1 r3 LSR r8
};
const byte kInstruction_movs_vc_r14_r4_LSL_r9[] = {
  0x14, 0xe9, 0xb0, 0x71 // movs vc r14 r4 LSL r9
};
const byte kInstruction_movs_gt_r2_r0_ROR_r3[] = {
  0x70, 0x23, 0xb0, 0xc1 // movs gt r2 r0 ROR r3
};
const byte kInstruction_movs_mi_r10_r3_LSR_r4[] = {
  0x33, 0xa4, 0xb0, 0x41 // movs mi r10 r3 LSR r4
};
const byte kInstruction_movs_vs_r7_r9_ROR_r13[] = {
  0x79, 0x7d, 0xb0, 0x61 // movs vs r7 r9 ROR r13
};
const byte kInstruction_movs_eq_r4_r3_LSR_r1[] = {
  0x33, 0x41, 0xb0, 0x01 // movs eq r4 r3 LSR r1
};
const byte kInstruction_movs_mi_r7_r0_LSL_r0[] = {
  0x10, 0x70, 0xb0, 0x41 // movs mi r7 r0 LSL r0
};
const byte kInstruction_movs_ls_r11_r6_LSR_r11[] = {
  0x36, 0xbb, 0xb0, 0x91 // movs ls r11 r6 LSR r11
};
const byte kInstruction_movs_al_r8_r4_LSL_r12[] = {
  0x14, 0x8c, 0xb0, 0xe1 // movs al r8 r4 LSL r12
};
const byte kInstruction_movs_pl_r7_r13_LSR_r8[] = {
  0x3d, 0x78, 0xb0, 0x51 // movs pl r7 r13 LSR r8
};
const byte kInstruction_movs_eq_r4_r4_LSR_r13[] = {
  0x34, 0x4d, 0xb0, 0x01 // movs eq r4 r4 LSR r13
};
const byte kInstruction_movs_eq_r3_r4_ROR_r6[] = {
  0x74, 0x36, 0xb0, 0x01 // movs eq r3 r4 ROR r6
};
const byte kInstruction_movs_ne_r8_r11_LSR_r14[] = {
  0x3b, 0x8e, 0xb0, 0x11 // movs ne r8 r11 LSR r14
};
const byte kInstruction_movs_ge_r12_r7_LSL_r1[] = {
  0x17, 0xc1, 0xb0, 0xa1 // movs ge r12 r7 LSL r1
};
const byte kInstruction_movs_gt_r12_r7_LSL_r14[] = {
  0x17, 0xce, 0xb0, 0xc1 // movs gt r12 r7 LSL r14
};
const byte kInstruction_movs_ne_r14_r13_LSR_r2[] = {
  0x3d, 0xe2, 0xb0, 0x11 // movs ne r14 r13 LSR r2
};
const byte kInstruction_movs_ne_r14_r2_LSR_r14[] = {
  0x32, 0xee, 0xb0, 0x11 // movs ne r14 r2 LSR r14
};
const byte kInstruction_movs_vs_r14_r5_ROR_r1[] = {
  0x75, 0xe1, 0xb0, 0x61 // movs vs r14 r5 ROR r1
};
const byte kInstruction_movs_ne_r2_r6_ROR_r4[] = {
  0x76, 0x24, 0xb0, 0x11 // movs ne r2 r6 ROR r4
};
const byte kInstruction_movs_lt_r4_r11_ROR_r13[] = {
  0x7b, 0x4d, 0xb0, 0xb1 // movs lt r4 r11 ROR r13
};
const byte kInstruction_movs_pl_r9_r3_ASR_r2[] = {
  0x53, 0x92, 0xb0, 0x51 // movs pl r9 r3 ASR r2
};
const byte kInstruction_movs_cs_r5_r5_LSR_r2[] = {
  0x35, 0x52, 0xb0, 0x21 // movs cs r5 r5 LSR r2
};
const byte kInstruction_movs_cc_r3_r1_ASR_r5[] = {
  0x51, 0x35, 0xb0, 0x31 // movs cc r3 r1 ASR r5
};
const byte kInstruction_movs_ne_r0_r10_LSL_r1[] = {
  0x1a, 0x01, 0xb0, 0x11 // movs ne r0 r10 LSL r1
};
const byte kInstruction_movs_pl_r5_r9_LSR_r0[] = {
  0x39, 0x50, 0xb0, 0x51 // movs pl r5 r9 LSR r0
};
const byte kInstruction_movs_gt_r11_r7_LSL_r11[] = {
  0x17, 0xbb, 0xb0, 0xc1 // movs gt r11 r7 LSL r11
};
const byte kInstruction_movs_al_r5_r13_LSR_r0[] = {
  0x3d, 0x50, 0xb0, 0xe1 // movs al r5 r13 LSR r0
};
const byte kInstruction_movs_al_r0_r6_ROR_r9[] = {
  0x76, 0x09, 0xb0, 0xe1 // movs al r0 r6 ROR r9
};
const byte kInstruction_movs_cc_r1_r1_LSR_r11[] = {
  0x31, 0x1b, 0xb0, 0x31 // movs cc r1 r1 LSR r11
};
const byte kInstruction_movs_cs_r5_r5_ASR_r3[] = {
  0x55, 0x53, 0xb0, 0x21 // movs cs r5 r5 ASR r3
};
const byte kInstruction_movs_lt_r4_r5_ASR_r3[] = {
  0x55, 0x43, 0xb0, 0xb1 // movs lt r4 r5 ASR r3
};
const byte kInstruction_movs_mi_r2_r5_LSL_r5[] = {
  0x15, 0x25, 0xb0, 0x41 // movs mi r2 r5 LSL r5
};
const byte kInstruction_movs_al_r5_r3_ASR_r1[] = {
  0x53, 0x51, 0xb0, 0xe1 // movs al r5 r3 ASR r1
};
const byte kInstruction_movs_gt_r8_r2_LSR_r3[] = {
  0x32, 0x83, 0xb0, 0xc1 // movs gt r8 r2 LSR r3
};
const byte kInstruction_movs_ne_r5_r0_ASR_r3[] = {
  0x50, 0x53, 0xb0, 0x11 // movs ne r5 r0 ASR r3
};
const byte kInstruction_movs_ge_r12_r14_LSR_r6[] = {
  0x3e, 0xc6, 0xb0, 0xa1 // movs ge r12 r14 LSR r6
};
const byte kInstruction_movs_al_r13_r7_ASR_r10[] = {
  0x57, 0xda, 0xb0, 0xe1 // movs al r13 r7 ASR r10
};
const byte kInstruction_movs_vs_r5_r12_ASR_r3[] = {
  0x5c, 0x53, 0xb0, 0x61 // movs vs r5 r12 ASR r3
};
const byte kInstruction_movs_vc_r10_r10_ROR_r8[] = {
  0x7a, 0xa8, 0xb0, 0x71 // movs vc r10 r10 ROR r8
};
const byte kInstruction_movs_vc_r4_r4_ASR_r4[] = {
  0x54, 0x44, 0xb0, 0x71 // movs vc r4 r4 ASR r4
};
const byte kInstruction_movs_hi_r1_r1_LSR_r3[] = {
  0x31, 0x13, 0xb0, 0x81 // movs hi r1 r1 LSR r3
};
const byte kInstruction_movs_ge_r14_r8_LSR_r8[] = {
  0x38, 0xe8, 0xb0, 0xa1 // movs ge r14 r8 LSR r8
};
const byte kInstruction_movs_le_r5_r5_LSL_r1[] = {
  0x15, 0x51, 0xb0, 0xd1 // movs le r5 r5 LSL r1
};
const byte kInstruction_movs_vc_r2_r8_LSL_r8[] = {
  0x18, 0x28, 0xb0, 0x71 // movs vc r2 r8 LSL r8
};
const byte kInstruction_movs_hi_r0_r6_LSL_r5[] = {
  0x16, 0x05, 0xb0, 0x81 // movs hi r0 r6 LSL r5
};
const byte kInstruction_movs_le_r8_r7_LSR_r3[] = {
  0x37, 0x83, 0xb0, 0xd1 // movs le r8 r7 LSR r3
};
const byte kInstruction_movs_cc_r10_r4_LSR_r4[] = {
  0x34, 0xa4, 0xb0, 0x31 // movs cc r10 r4 LSR r4
};
const byte kInstruction_movs_cc_r5_r9_LSL_r7[] = {
  0x19, 0x57, 0xb0, 0x31 // movs cc r5 r9 LSL r7
};
const byte kInstruction_movs_lt_r1_r14_ROR_r13[] = {
  0x7e, 0x1d, 0xb0, 0xb1 // movs lt r1 r14 ROR r13
};
const byte kInstruction_movs_mi_r9_r9_ROR_r10[] = {
  0x79, 0x9a, 0xb0, 0x41 // movs mi r9 r9 ROR r10
};
const byte kInstruction_movs_cc_r0_r5_ROR_r7[] = {
  0x75, 0x07, 0xb0, 0x31 // movs cc r0 r5 ROR r7
};
const byte kInstruction_movs_vs_r6_r5_LSR_r10[] = {
  0x35, 0x6a, 0xb0, 0x61 // movs vs r6 r5 LSR r10
};
const byte kInstruction_movs_pl_r14_r13_ASR_r4[] = {
  0x5d, 0xe4, 0xb0, 0x51 // movs pl r14 r13 ASR r4
};
const byte kInstruction_movs_ne_r7_r9_LSL_r11[] = {
  0x19, 0x7b, 0xb0, 0x11 // movs ne r7 r9 LSL r11
};
const byte kInstruction_movs_ne_r13_r3_ROR_r9[] = {
  0x73, 0xd9, 0xb0, 0x11 // movs ne r13 r3 ROR r9
};
const byte kInstruction_movs_mi_r6_r13_ROR_r5[] = {
  0x7d, 0x65, 0xb0, 0x41 // movs mi r6 r13 ROR r5
};
const byte kInstruction_movs_vc_r10_r0_ASR_r6[] = {
  0x50, 0xa6, 0xb0, 0x71 // movs vc r10 r0 ASR r6
};
const byte kInstruction_movs_ge_r5_r4_LSR_r12[] = {
  0x34, 0x5c, 0xb0, 0xa1 // movs ge r5 r4 LSR r12
};
const byte kInstruction_movs_vc_r10_r3_LSR_r8[] = {
  0x33, 0xa8, 0xb0, 0x71 // movs vc r10 r3 LSR r8
};
const byte kInstruction_movs_hi_r8_r1_LSR_r7[] = {
  0x31, 0x87, 0xb0, 0x81 // movs hi r8 r1 LSR r7
};
const byte kInstruction_movs_al_r3_r6_ASR_r11[] = {
  0x56, 0x3b, 0xb0, 0xe1 // movs al r3 r6 ASR r11
};
const byte kInstruction_movs_cc_r1_r5_ASR_r0[] = {
  0x55, 0x10, 0xb0, 0x31 // movs cc r1 r5 ASR r0
};
const byte kInstruction_movs_mi_r11_r3_LSR_r8[] = {
  0x33, 0xb8, 0xb0, 0x41 // movs mi r11 r3 LSR r8
};
const byte kInstruction_movs_gt_r4_r8_ASR_r12[] = {
  0x58, 0x4c, 0xb0, 0xc1 // movs gt r4 r8 ASR r12
};
const byte kInstruction_movs_hi_r0_r8_ROR_r10[] = {
  0x78, 0x0a, 0xb0, 0x81 // movs hi r0 r8 ROR r10
};
const byte kInstruction_movs_gt_r14_r8_LSR_r9[] = {
  0x38, 0xe9, 0xb0, 0xc1 // movs gt r14 r8 LSR r9
};
const byte kInstruction_movs_pl_r14_r13_LSR_r9[] = {
  0x3d, 0xe9, 0xb0, 0x51 // movs pl r14 r13 LSR r9
};
const byte kInstruction_movs_cs_r11_r0_LSR_r8[] = {
  0x30, 0xb8, 0xb0, 0x21 // movs cs r11 r0 LSR r8
};
const byte kInstruction_movs_cs_r10_r3_LSR_r4[] = {
  0x33, 0xa4, 0xb0, 0x21 // movs cs r10 r3 LSR r4
};
const byte kInstruction_movs_ge_r9_r0_ASR_r3[] = {
  0x50, 0x93, 0xb0, 0xa1 // movs ge r9 r0 ASR r3
};
const byte kInstruction_movs_eq_r12_r14_LSL_r9[] = {
  0x1e, 0xc9, 0xb0, 0x01 // movs eq r12 r14 LSL r9
};
const byte kInstruction_movs_eq_r2_r11_ASR_r8[] = {
  0x5b, 0x28, 0xb0, 0x01 // movs eq r2 r11 ASR r8
};
const byte kInstruction_movs_le_r9_r12_ASR_r4[] = {
  0x5c, 0x94, 0xb0, 0xd1 // movs le r9 r12 ASR r4
};
const byte kInstruction_movs_lt_r6_r12_LSL_r0[] = {
  0x1c, 0x60, 0xb0, 0xb1 // movs lt r6 r12 LSL r0
};
const byte kInstruction_movs_vc_r14_r4_ROR_r6[] = {
  0x74, 0xe6, 0xb0, 0x71 // movs vc r14 r4 ROR r6
};
const byte kInstruction_movs_hi_r6_r1_LSL_r8[] = {
  0x11, 0x68, 0xb0, 0x81 // movs hi r6 r1 LSL r8
};
const byte kInstruction_movs_le_r10_r12_ROR_r14[] = {
  0x7c, 0xae, 0xb0, 0xd1 // movs le r10 r12 ROR r14
};
const byte kInstruction_movs_ls_r9_r1_ROR_r5[] = {
  0x71, 0x95, 0xb0, 0x91 // movs ls r9 r1 ROR r5
};
const byte kInstruction_movs_mi_r3_r5_LSL_r9[] = {
  0x15, 0x39, 0xb0, 0x41 // movs mi r3 r5 LSL r9
};
const byte kInstruction_movs_hi_r5_r4_LSR_r10[] = {
  0x34, 0x5a, 0xb0, 0x81 // movs hi r5 r4 LSR r10
};
const byte kInstruction_movs_ge_r2_r5_LSR_r9[] = {
  0x35, 0x29, 0xb0, 0xa1 // movs ge r2 r5 LSR r9
};
const byte kInstruction_movs_ge_r11_r14_LSL_r9[] = {
  0x1e, 0xb9, 0xb0, 0xa1 // movs ge r11 r14 LSL r9
};
const byte kInstruction_movs_vc_r7_r2_LSR_r6[] = {
  0x32, 0x76, 0xb0, 0x71 // movs vc r7 r2 LSR r6
};
const byte kInstruction_movs_ls_r12_r1_ASR_r6[] = {
  0x51, 0xc6, 0xb0, 0x91 // movs ls r12 r1 ASR r6
};
const byte kInstruction_movs_cs_r3_r2_ROR_r4[] = {
  0x72, 0x34, 0xb0, 0x21 // movs cs r3 r2 ROR r4
};
const byte kInstruction_movs_lt_r13_r14_ROR_r7[] = {
  0x7e, 0xd7, 0xb0, 0xb1 // movs lt r13 r14 ROR r7
};
const byte kInstruction_movs_le_r13_r9_LSR_r2[] = {
  0x39, 0xd2, 0xb0, 0xd1 // movs le r13 r9 LSR r2
};
const byte kInstruction_movs_ls_r0_r4_LSL_r8[] = {
  0x14, 0x08, 0xb0, 0x91 // movs ls r0 r4 LSL r8
};
const byte kInstruction_movs_pl_r14_r5_ROR_r5[] = {
  0x75, 0xe5, 0xb0, 0x51 // movs pl r14 r5 ROR r5
};
const byte kInstruction_movs_le_r7_r8_ASR_r13[] = {
  0x58, 0x7d, 0xb0, 0xd1 // movs le r7 r8 ASR r13
};
const byte kInstruction_movs_vc_r4_r5_ASR_r5[] = {
  0x55, 0x45, 0xb0, 0x71 // movs vc r4 r5 ASR r5
};
const byte kInstruction_movs_gt_r14_r2_ROR_r3[] = {
  0x72, 0xe3, 0xb0, 0xc1 // movs gt r14 r2 ROR r3
};
const byte kInstruction_movs_eq_r13_r12_LSL_r5[] = {
  0x1c, 0xd5, 0xb0, 0x01 // movs eq r13 r12 LSL r5
};
const byte kInstruction_movs_pl_r5_r12_LSR_r2[] = {
  0x3c, 0x52, 0xb0, 0x51 // movs pl r5 r12 LSR r2
};
const byte kInstruction_movs_cc_r4_r13_ROR_r14[] = {
  0x7d, 0x4e, 0xb0, 0x31 // movs cc r4 r13 ROR r14
};
const byte kInstruction_movs_ne_r10_r9_LSL_r9[] = {
  0x19, 0xa9, 0xb0, 0x11 // movs ne r10 r9 LSL r9
};
const byte kInstruction_movs_eq_r1_r1_ROR_r14[] = {
  0x71, 0x1e, 0xb0, 0x01 // movs eq r1 r1 ROR r14
};
const byte kInstruction_movs_eq_r13_r0_LSL_r13[] = {
  0x10, 0xdd, 0xb0, 0x01 // movs eq r13 r0 LSL r13
};
const byte kInstruction_movs_vc_r0_r11_ASR_r3[] = {
  0x5b, 0x03, 0xb0, 0x71 // movs vc r0 r11 ASR r3
};
const byte kInstruction_movs_vc_r1_r4_LSR_r3[] = {
  0x34, 0x13, 0xb0, 0x71 // movs vc r1 r4 LSR r3
};
const byte kInstruction_movs_cc_r8_r3_ASR_r12[] = {
  0x53, 0x8c, 0xb0, 0x31 // movs cc r8 r3 ASR r12
};
const byte kInstruction_movs_vs_r3_r10_LSL_r4[] = {
  0x1a, 0x34, 0xb0, 0x61 // movs vs r3 r10 LSL r4
};
const byte kInstruction_movs_gt_r10_r9_ASR_r10[] = {
  0x59, 0xaa, 0xb0, 0xc1 // movs gt r10 r9 ASR r10
};
const byte kInstruction_movs_cc_r10_r2_LSL_r10[] = {
  0x12, 0xaa, 0xb0, 0x31 // movs cc r10 r2 LSL r10
};
const byte kInstruction_movs_cs_r0_r2_LSL_r1[] = {
  0x12, 0x01, 0xb0, 0x21 // movs cs r0 r2 LSL r1
};
const byte kInstruction_movs_hi_r14_r14_ASR_r13[] = {
  0x5e, 0xed, 0xb0, 0x81 // movs hi r14 r14 ASR r13
};
const byte kInstruction_movs_mi_r4_r3_LSL_r1[] = {
  0x13, 0x41, 0xb0, 0x41 // movs mi r4 r3 LSL r1
};
const byte kInstruction_movs_gt_r5_r14_ROR_r2[] = {
  0x7e, 0x52, 0xb0, 0xc1 // movs gt r5 r14 ROR r2
};
const byte kInstruction_movs_cc_r10_r13_ASR_r12[] = {
  0x5d, 0xac, 0xb0, 0x31 // movs cc r10 r13 ASR r12
};
const byte kInstruction_movs_cs_r2_r4_LSL_r12[] = {
  0x14, 0x2c, 0xb0, 0x21 // movs cs r2 r4 LSL r12
};
const byte kInstruction_movs_vs_r4_r2_ROR_r11[] = {
  0x72, 0x4b, 0xb0, 0x61 // movs vs r4 r2 ROR r11
};
const byte kInstruction_movs_cs_r1_r13_LSR_r7[] = {
  0x3d, 0x17, 0xb0, 0x21 // movs cs r1 r13 LSR r7
};
const byte kInstruction_movs_al_r3_r6_ASR_r5[] = {
  0x56, 0x35, 0xb0, 0xe1 // movs al r3 r6 ASR r5
};
const byte kInstruction_movs_ge_r12_r4_LSL_r2[] = {
  0x14, 0xc2, 0xb0, 0xa1 // movs ge r12 r4 LSL r2
};
const byte kInstruction_movs_cc_r8_r13_ROR_r5[] = {
  0x7d, 0x85, 0xb0, 0x31 // movs cc r8 r13 ROR r5
};
const byte kInstruction_movs_vc_r0_r10_LSL_r7[] = {
  0x1a, 0x07, 0xb0, 0x71 // movs vc r0 r10 LSL r7
};
const byte kInstruction_movs_ge_r13_r0_LSL_r4[] = {
  0x10, 0xd4, 0xb0, 0xa1 // movs ge r13 r0 LSL r4
};
const byte kInstruction_movs_cc_r14_r12_LSL_r10[] = {
  0x1c, 0xea, 0xb0, 0x31 // movs cc r14 r12 LSL r10
};
const byte kInstruction_movs_mi_r9_r9_ASR_r9[] = {
  0x59, 0x99, 0xb0, 0x41 // movs mi r9 r9 ASR r9
};
const byte kInstruction_movs_al_r1_r13_ROR_r14[] = {
  0x7d, 0x1e, 0xb0, 0xe1 // movs al r1 r13 ROR r14
};
const byte kInstruction_movs_ls_r5_r10_ROR_r1[] = {
  0x7a, 0x51, 0xb0, 0x91 // movs ls r5 r10 ROR r1
};
const byte kInstruction_movs_mi_r7_r9_LSL_r7[] = {
  0x19, 0x77, 0xb0, 0x41 // movs mi r7 r9 LSL r7
};
const byte kInstruction_movs_ne_r11_r13_LSR_r5[] = {
  0x3d, 0xb5, 0xb0, 0x11 // movs ne r11 r13 LSR r5
};
const byte kInstruction_movs_mi_r1_r6_ROR_r9[] = {
  0x76, 0x19, 0xb0, 0x41 // movs mi r1 r6 ROR r9
};
const byte kInstruction_movs_le_r1_r2_LSR_r2[] = {
  0x32, 0x12, 0xb0, 0xd1 // movs le r1 r2 LSR r2
};
const byte kInstruction_movs_vc_r12_r9_LSR_r6[] = {
  0x39, 0xc6, 0xb0, 0x71 // movs vc r12 r9 LSR r6
};
const byte kInstruction_movs_cc_r11_r2_ROR_r9[] = {
  0x72, 0xb9, 0xb0, 0x31 // movs cc r11 r2 ROR r9
};
const byte kInstruction_movs_mi_r6_r11_LSL_r10[] = {
  0x1b, 0x6a, 0xb0, 0x41 // movs mi r6 r11 LSL r10
};
const byte kInstruction_movs_cc_r6_r1_ROR_r6[] = {
  0x71, 0x66, 0xb0, 0x31 // movs cc r6 r1 ROR r6
};
const byte kInstruction_movs_gt_r1_r13_LSL_r10[] = {
  0x1d, 0x1a, 0xb0, 0xc1 // movs gt r1 r13 LSL r10
};
const byte kInstruction_movs_eq_r10_r12_ROR_r11[] = {
  0x7c, 0xab, 0xb0, 0x01 // movs eq r10 r12 ROR r11
};
const byte kInstruction_movs_cs_r5_r7_LSL_r5[] = {
  0x17, 0x55, 0xb0, 0x21 // movs cs r5 r7 LSL r5
};
const byte kInstruction_movs_cs_r2_r14_ROR_r6[] = {
  0x7e, 0x26, 0xb0, 0x21 // movs cs r2 r14 ROR r6
};
const byte kInstruction_movs_cc_r4_r6_ASR_r7[] = {
  0x56, 0x47, 0xb0, 0x31 // movs cc r4 r6 ASR r7
};
const byte kInstruction_movs_mi_r5_r3_ASR_r8[] = {
  0x53, 0x58, 0xb0, 0x41 // movs mi r5 r3 ASR r8
};
const byte kInstruction_movs_vs_r5_r0_LSR_r4[] = {
  0x30, 0x54, 0xb0, 0x61 // movs vs r5 r0 LSR r4
};
const byte kInstruction_movs_ge_r4_r7_ASR_r5[] = {
  0x57, 0x45, 0xb0, 0xa1 // movs ge r4 r7 ASR r5
};
const byte kInstruction_movs_lt_r4_r9_ASR_r7[] = {
  0x59, 0x47, 0xb0, 0xb1 // movs lt r4 r9 ASR r7
};
const byte kInstruction_movs_cs_r3_r13_ASR_r6[] = {
  0x5d, 0x36, 0xb0, 0x21 // movs cs r3 r13 ASR r6
};
const byte kInstruction_movs_vs_r3_r7_LSR_r2[] = {
  0x37, 0x32, 0xb0, 0x61 // movs vs r3 r7 LSR r2
};
const byte kInstruction_movs_pl_r0_r3_ASR_r7[] = {
  0x53, 0x07, 0xb0, 0x51 // movs pl r0 r3 ASR r7
};
const byte kInstruction_movs_lt_r7_r10_LSL_r0[] = {
  0x1a, 0x70, 0xb0, 0xb1 // movs lt r7 r10 LSL r0
};
const byte kInstruction_movs_pl_r5_r1_LSR_r4[] = {
  0x31, 0x54, 0xb0, 0x51 // movs pl r5 r1 LSR r4
};
const byte kInstruction_movs_gt_r10_r7_LSL_r10[] = {
  0x17, 0xaa, 0xb0, 0xc1 // movs gt r10 r7 LSL r10
};
const byte kInstruction_movs_ls_r14_r6_ROR_r0[] = {
  0x76, 0xe0, 0xb0, 0x91 // movs ls r14 r6 ROR r0
};
const byte kInstruction_movs_le_r1_r5_ROR_r11[] = {
  0x75, 0x1b, 0xb0, 0xd1 // movs le r1 r5 ROR r11
};
const byte kInstruction_movs_ls_r6_r14_LSL_r8[] = {
  0x1e, 0x68, 0xb0, 0x91 // movs ls r6 r14 LSL r8
};
const byte kInstruction_movs_mi_r12_r11_LSL_r5[] = {
  0x1b, 0xc5, 0xb0, 0x41 // movs mi r12 r11 LSL r5
};
const byte kInstruction_movs_le_r13_r5_ASR_r3[] = {
  0x55, 0xd3, 0xb0, 0xd1 // movs le r13 r5 ASR r3
};
const byte kInstruction_movs_ls_r14_r10_ROR_r14[] = {
  0x7a, 0xee, 0xb0, 0x91 // movs ls r14 r10 ROR r14
};
const byte kInstruction_movs_eq_r5_r12_ASR_r5[] = {
  0x5c, 0x55, 0xb0, 0x01 // movs eq r5 r12 ASR r5
};
const byte kInstruction_movs_gt_r14_r4_ASR_r14[] = {
  0x54, 0xee, 0xb0, 0xc1 // movs gt r14 r4 ASR r14
};
const byte kInstruction_movs_ls_r0_r5_ASR_r13[] = {
  0x55, 0x0d, 0xb0, 0x91 // movs ls r0 r5 ASR r13
};
const byte kInstruction_movs_pl_r6_r4_LSR_r1[] = {
  0x34, 0x61, 0xb0, 0x51 // movs pl r6 r4 LSR r1
};
const byte kInstruction_movs_pl_r7_r11_ROR_r3[] = {
  0x7b, 0x73, 0xb0, 0x51 // movs pl r7 r11 ROR r3
};
const byte kInstruction_movs_lt_r11_r9_ASR_r13[] = {
  0x59, 0xbd, 0xb0, 0xb1 // movs lt r11 r9 ASR r13
};
const byte kInstruction_movs_cc_r3_r9_ASR_r4[] = {
  0x59, 0x34, 0xb0, 0x31 // movs cc r3 r9 ASR r4
};
const byte kInstruction_movs_eq_r2_r1_ASR_r7[] = {
  0x51, 0x27, 0xb0, 0x01 // movs eq r2 r1 ASR r7
};
const byte kInstruction_movs_le_r8_r10_ASR_r9[] = {
  0x5a, 0x89, 0xb0, 0xd1 // movs le r8 r10 ASR r9
};
const byte kInstruction_movs_hi_r14_r2_LSL_r6[] = {
  0x12, 0xe6, 0xb0, 0x81 // movs hi r14 r2 LSL r6
};
const byte kInstruction_movs_mi_r12_r4_LSR_r8[] = {
  0x34, 0xc8, 0xb0, 0x41 // movs mi r12 r4 LSR r8
};
const byte kInstruction_movs_pl_r4_r1_ROR_r2[] = {
  0x71, 0x42, 0xb0, 0x51 // movs pl r4 r1 ROR r2
};
const byte kInstruction_movs_hi_r5_r9_ROR_r7[] = {
  0x79, 0x57, 0xb0, 0x81 // movs hi r5 r9 ROR r7
};
const byte kInstruction_movs_le_r4_r2_LSR_r1[] = {
  0x32, 0x41, 0xb0, 0xd1 // movs le r4 r2 LSR r1
};
const byte kInstruction_movs_al_r8_r1_ROR_r9[] = {
  0x71, 0x89, 0xb0, 0xe1 // movs al r8 r1 ROR r9
};
const byte kInstruction_movs_mi_r3_r6_ASR_r6[] = {
  0x56, 0x36, 0xb0, 0x41 // movs mi r3 r6 ASR r6
};
const byte kInstruction_movs_ls_r1_r12_LSR_r9[] = {
  0x3c, 0x19, 0xb0, 0x91 // movs ls r1 r12 LSR r9
};
const byte kInstruction_movs_le_r13_r5_LSR_r13[] = {
  0x35, 0xdd, 0xb0, 0xd1 // movs le r13 r5 LSR r13
};
const byte kInstruction_movs_ne_r3_r12_ROR_r7[] = {
  0x7c, 0x37, 0xb0, 0x11 // movs ne r3 r12 ROR r7
};
const byte kInstruction_movs_mi_r0_r9_LSR_r5[] = {
  0x39, 0x05, 0xb0, 0x41 // movs mi r0 r9 LSR r5
};
const byte kInstruction_movs_al_r4_r4_ROR_r3[] = {
  0x74, 0x43, 0xb0, 0xe1 // movs al r4 r4 ROR r3
};
const byte kInstruction_movs_cc_r1_r1_ASR_r3[] = {
  0x51, 0x13, 0xb0, 0x31 // movs cc r1 r1 ASR r3
};
const byte kInstruction_movs_lt_r12_r0_LSL_r10[] = {
  0x10, 0xca, 0xb0, 0xb1 // movs lt r12 r0 LSL r10
};
const byte kInstruction_movs_ge_r1_r10_LSL_r4[] = {
  0x1a, 0x14, 0xb0, 0xa1 // movs ge r1 r10 LSL r4
};
const byte kInstruction_movs_hi_r2_r7_LSL_r4[] = {
  0x17, 0x24, 0xb0, 0x81 // movs hi r2 r7 LSL r4
};
const byte kInstruction_movs_le_r1_r6_ASR_r13[] = {
  0x56, 0x1d, 0xb0, 0xd1 // movs le r1 r6 ASR r13
};
const byte kInstruction_movs_eq_r0_r2_ROR_r8[] = {
  0x72, 0x08, 0xb0, 0x01 // movs eq r0 r2 ROR r8
};
const byte kInstruction_movs_vc_r6_r0_LSL_r8[] = {
  0x10, 0x68, 0xb0, 0x71 // movs vc r6 r0 LSL r8
};
const byte kInstruction_movs_hi_r11_r7_LSL_r5[] = {
  0x17, 0xb5, 0xb0, 0x81 // movs hi r11 r7 LSL r5
};
const byte kInstruction_movs_ne_r0_r1_ASR_r2[] = {
  0x51, 0x02, 0xb0, 0x11 // movs ne r0 r1 ASR r2
};
const byte kInstruction_movs_gt_r7_r10_ROR_r8[] = {
  0x7a, 0x78, 0xb0, 0xc1 // movs gt r7 r10 ROR r8
};
const byte kInstruction_movs_vc_r5_r2_ASR_r8[] = {
  0x52, 0x58, 0xb0, 0x71 // movs vc r5 r2 ASR r8
};
const byte kInstruction_movs_ls_r4_r9_LSL_r14[] = {
  0x19, 0x4e, 0xb0, 0x91 // movs ls r4 r9 LSL r14
};
const byte kInstruction_movs_vs_r4_r4_LSR_r5[] = {
  0x34, 0x45, 0xb0, 0x61 // movs vs r4 r4 LSR r5
};
const byte kInstruction_movs_hi_r4_r14_LSL_r13[] = {
  0x1e, 0x4d, 0xb0, 0x81 // movs hi r4 r14 LSL r13
};
const byte kInstruction_movs_cc_r8_r11_ROR_r13[] = {
  0x7b, 0x8d, 0xb0, 0x31 // movs cc r8 r11 ROR r13
};
const byte kInstruction_movs_ls_r2_r2_LSR_r14[] = {
  0x32, 0x2e, 0xb0, 0x91 // movs ls r2 r2 LSR r14
};
const byte kInstruction_movs_lt_r1_r4_ROR_r12[] = {
  0x74, 0x1c, 0xb0, 0xb1 // movs lt r1 r4 ROR r12
};
const byte kInstruction_movs_mi_r2_r6_ROR_r14[] = {
  0x76, 0x2e, 0xb0, 0x41 // movs mi r2 r6 ROR r14
};
const byte kInstruction_movs_hi_r7_r2_ASR_r9[] = {
  0x52, 0x79, 0xb0, 0x81 // movs hi r7 r2 ASR r9
};
const byte kInstruction_movs_le_r10_r14_ROR_r13[] = {
  0x7e, 0xad, 0xb0, 0xd1 // movs le r10 r14 ROR r13
};
const byte kInstruction_movs_vc_r7_r14_ASR_r11[] = {
  0x5e, 0x7b, 0xb0, 0x71 // movs vc r7 r14 ASR r11
};
const byte kInstruction_movs_mi_r4_r5_ASR_r6[] = {
  0x55, 0x46, 0xb0, 0x41 // movs mi r4 r5 ASR r6
};
const byte kInstruction_movs_al_r11_r14_ROR_r9[] = {
  0x7e, 0xb9, 0xb0, 0xe1 // movs al r11 r14 ROR r9
};
const byte kInstruction_movs_vc_r0_r10_ASR_r13[] = {
  0x5a, 0x0d, 0xb0, 0x71 // movs vc r0 r10 ASR r13
};
const byte kInstruction_movs_al_r10_r2_ROR_r13[] = {
  0x72, 0xad, 0xb0, 0xe1 // movs al r10 r2 ROR r13
};
const byte kInstruction_movs_hi_r9_r2_LSR_r9[] = {
  0x32, 0x99, 0xb0, 0x81 // movs hi r9 r2 LSR r9
};
const byte kInstruction_movs_gt_r2_r0_ASR_r10[] = {
  0x50, 0x2a, 0xb0, 0xc1 // movs gt r2 r0 ASR r10
};
const byte kInstruction_movs_hi_r4_r12_ASR_r13[] = {
  0x5c, 0x4d, 0xb0, 0x81 // movs hi r4 r12 ASR r13
};
const byte kInstruction_movs_vc_r10_r14_ASR_r0[] = {
  0x5e, 0xa0, 0xb0, 0x71 // movs vc r10 r14 ASR r0
};
const byte kInstruction_movs_ne_r13_r9_ASR_r4[] = {
  0x59, 0xd4, 0xb0, 0x11 // movs ne r13 r9 ASR r4
};
const byte kInstruction_movs_eq_r5_r8_ROR_r5[] = {
  0x78, 0x55, 0xb0, 0x01 // movs eq r5 r8 ROR r5
};
const byte kInstruction_movs_lt_r0_r13_ROR_r3[] = {
  0x7d, 0x03, 0xb0, 0xb1 // movs lt r0 r13 ROR r3
};
const byte kInstruction_movs_hi_r0_r1_LSL_r1[] = {
  0x11, 0x01, 0xb0, 0x81 // movs hi r0 r1 LSL r1
};
const byte kInstruction_movs_pl_r13_r6_ASR_r10[] = {
  0x56, 0xda, 0xb0, 0x51 // movs pl r13 r6 ASR r10
};
const byte kInstruction_movs_hi_r0_r4_LSR_r2[] = {
  0x34, 0x02, 0xb0, 0x81 // movs hi r0 r4 LSR r2
};
const byte kInstruction_movs_eq_r5_r3_ASR_r8[] = {
  0x53, 0x58, 0xb0, 0x01 // movs eq r5 r3 ASR r8
};
const byte kInstruction_movs_al_r1_r5_LSR_r4[] = {
  0x35, 0x14, 0xb0, 0xe1 // movs al r1 r5 LSR r4
};
const byte kInstruction_movs_eq_r8_r5_LSR_r14[] = {
  0x35, 0x8e, 0xb0, 0x01 // movs eq r8 r5 LSR r14
};
const byte kInstruction_movs_ne_r12_r13_ROR_r0[] = {
  0x7d, 0xc0, 0xb0, 0x11 // movs ne r12 r13 ROR r0
};
const byte kInstruction_movs_pl_r0_r2_ROR_r11[] = {
  0x72, 0x0b, 0xb0, 0x51 // movs pl r0 r2 ROR r11
};
const byte kInstruction_movs_cs_r10_r6_ROR_r9[] = {
  0x76, 0xa9, 0xb0, 0x21 // movs cs r10 r6 ROR r9
};
const byte kInstruction_movs_eq_r10_r4_ASR_r14[] = {
  0x54, 0xae, 0xb0, 0x01 // movs eq r10 r4 ASR r14
};
const byte kInstruction_movs_ls_r9_r4_LSR_r14[] = {
  0x34, 0x9e, 0xb0, 0x91 // movs ls r9 r4 LSR r14
};
const byte kInstruction_movs_cs_r14_r2_ROR_r5[] = {
  0x72, 0xe5, 0xb0, 0x21 // movs cs r14 r2 ROR r5
};
const byte kInstruction_movs_vc_r9_r9_LSL_r7[] = {
  0x19, 0x97, 0xb0, 0x71 // movs vc r9 r9 LSL r7
};
const byte kInstruction_movs_mi_r7_r0_LSL_r4[] = {
  0x10, 0x74, 0xb0, 0x41 // movs mi r7 r0 LSL r4
};
const byte kInstruction_movs_cc_r2_r10_ASR_r7[] = {
  0x5a, 0x27, 0xb0, 0x31 // movs cc r2 r10 ASR r7
};
const byte kInstruction_movs_cs_r5_r10_LSR_r9[] = {
  0x3a, 0x59, 0xb0, 0x21 // movs cs r5 r10 LSR r9
};
const TestResult kReferencemovs[] = {
  {
    ARRAY_SIZE(kInstruction_movs_gt_r13_r11_LSL_r12),
    kInstruction_movs_gt_r13_r11_LSL_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r12_r4_LSR_r6),
    kInstruction_movs_gt_r12_r4_LSR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r11_r13_ROR_r2),
    kInstruction_movs_ge_r11_r13_ROR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r9_r8_ASR_r10),
    kInstruction_movs_hi_r9_r8_ASR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r9_r10_LSL_r4),
    kInstruction_movs_al_r9_r10_LSL_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r4_r11_LSR_r4),
    kInstruction_movs_gt_r4_r11_LSR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r4_r6_ASR_r11),
    kInstruction_movs_vc_r4_r6_ASR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r9_r14_LSR_r9),
    kInstruction_movs_mi_r9_r14_LSR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r5_r10_LSR_r8),
    kInstruction_movs_ne_r5_r10_LSR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r3_r0_ROR_r11),
    kInstruction_movs_eq_r3_r0_ROR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r9_r11_LSL_r12),
    kInstruction_movs_gt_r9_r11_LSL_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r14_r6_ROR_r13),
    kInstruction_movs_ge_r14_r6_ROR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r14_r14_LSR_r11),
    kInstruction_movs_vc_r14_r14_LSR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r6_r3_LSL_r2),
    kInstruction_movs_cc_r6_r3_LSL_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r4_r6_ROR_r7),
    kInstruction_movs_ge_r4_r6_ROR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r13_r3_LSR_r12),
    kInstruction_movs_vs_r13_r3_LSR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r8_r11_LSR_r13),
    kInstruction_movs_ge_r8_r11_LSR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r5_r0_LSR_r10),
    kInstruction_movs_cc_r5_r0_LSR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r7_r12_ASR_r7),
    kInstruction_movs_lt_r7_r12_ASR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r14_r1_LSR_r14),
    kInstruction_movs_cc_r14_r1_LSR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r10_r1_LSR_r1),
    kInstruction_movs_vs_r10_r1_LSR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r5_r0_ASR_r2),
    kInstruction_movs_lt_r5_r0_ASR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r6_r3_LSL_r2),
    kInstruction_movs_ne_r6_r3_LSL_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r5_r5_LSL_r13),
    kInstruction_movs_vs_r5_r5_LSL_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r9_r11_ROR_r13),
    kInstruction_movs_hi_r9_r11_ROR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r0_r2_LSL_r6),
    kInstruction_movs_vs_r0_r2_LSL_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r7_r1_ASR_r0),
    kInstruction_movs_le_r7_r1_ASR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r2_r12_LSL_r10),
    kInstruction_movs_ge_r2_r12_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r14_r1_ASR_r9),
    kInstruction_movs_vc_r14_r1_ASR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r5_r1_ROR_r5),
    kInstruction_movs_lt_r5_r1_ROR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r5_r12_ROR_r0),
    kInstruction_movs_eq_r5_r12_ROR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r7_r7_ASR_r9),
    kInstruction_movs_pl_r7_r7_ASR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r3_r10_LSR_r9),
    kInstruction_movs_eq_r3_r10_LSR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r0_r5_LSR_r13),
    kInstruction_movs_ge_r0_r5_LSR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r13_r1_ASR_r6),
    kInstruction_movs_cc_r13_r1_ASR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r12_r3_ASR_r12),
    kInstruction_movs_cs_r12_r3_ASR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r12_r10_ROR_r0),
    kInstruction_movs_al_r12_r10_ROR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r6_r13_LSL_r0),
    kInstruction_movs_al_r6_r13_LSL_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r14_r14_ROR_r0),
    kInstruction_movs_cc_r14_r14_ROR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r2_r14_ROR_r9),
    kInstruction_movs_ge_r2_r14_ROR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r5_r10_LSL_r8),
    kInstruction_movs_hi_r5_r10_LSL_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r13_r5_LSL_r7),
    kInstruction_movs_ge_r13_r5_LSL_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r12_r10_ASR_r4),
    kInstruction_movs_cc_r12_r10_ASR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r12_r3_LSL_r7),
    kInstruction_movs_ls_r12_r3_LSL_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r2_r9_ASR_r8),
    kInstruction_movs_cs_r2_r9_ASR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r1_r5_LSR_r1),
    kInstruction_movs_vs_r1_r5_LSR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r7_r14_ASR_r9),
    kInstruction_movs_cs_r7_r14_ASR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r6_r10_ASR_r0),
    kInstruction_movs_hi_r6_r10_ASR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r13_r8_ASR_r13),
    kInstruction_movs_pl_r13_r8_ASR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r1_r0_ROR_r2),
    kInstruction_movs_vs_r1_r0_ROR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r14_r0_LSL_r10),
    kInstruction_movs_lt_r14_r0_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r8_r12_ROR_r14),
    kInstruction_movs_ls_r8_r12_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r11_r2_LSL_r0),
    kInstruction_movs_al_r11_r2_LSL_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r0_r8_LSR_r7),
    kInstruction_movs_mi_r0_r8_LSR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r3_r5_LSL_r9),
    kInstruction_movs_cc_r3_r5_LSL_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r14_r4_ROR_r4),
    kInstruction_movs_vc_r14_r4_ROR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r1_r8_LSR_r7),
    kInstruction_movs_cs_r1_r8_LSR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r6_r5_ASR_r13),
    kInstruction_movs_ls_r6_r5_ASR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r0_r12_LSR_r5),
    kInstruction_movs_pl_r0_r12_LSR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r9_r7_ROR_r2),
    kInstruction_movs_lt_r9_r7_ROR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r6_r13_ASR_r13),
    kInstruction_movs_ne_r6_r13_ASR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r9_r1_LSR_r5),
    kInstruction_movs_cc_r9_r1_LSR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r8_r0_ROR_r10),
    kInstruction_movs_ls_r8_r0_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r9_r8_LSR_r2),
    kInstruction_movs_mi_r9_r8_LSR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r10_r14_ROR_r8),
    kInstruction_movs_pl_r10_r14_ROR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r4_r6_ROR_r10),
    kInstruction_movs_hi_r4_r6_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r0_r8_ROR_r10),
    kInstruction_movs_vc_r0_r8_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r0_r4_LSR_r11),
    kInstruction_movs_ne_r0_r4_LSR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r0_r13_ROR_r14),
    kInstruction_movs_pl_r0_r13_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r5_r6_LSL_r5),
    kInstruction_movs_ne_r5_r6_LSL_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r8_r3_ROR_r14),
    kInstruction_movs_ge_r8_r3_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r8_r7_LSL_r10),
    kInstruction_movs_ne_r8_r7_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r13_r9_ROR_r13),
    kInstruction_movs_eq_r13_r9_ROR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r10_r8_ROR_r14),
    kInstruction_movs_eq_r10_r8_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r1_r8_ROR_r9),
    kInstruction_movs_mi_r1_r8_ROR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r10_r8_ROR_r10),
    kInstruction_movs_le_r10_r8_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r7_r2_LSR_r11),
    kInstruction_movs_le_r7_r2_LSR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r11_r9_LSL_r11),
    kInstruction_movs_le_r11_r9_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r5_r5_LSR_r4),
    kInstruction_movs_cc_r5_r5_LSR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r9_r5_LSL_r8),
    kInstruction_movs_al_r9_r5_LSL_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r7_r14_LSL_r8),
    kInstruction_movs_cs_r7_r14_LSL_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r9_r9_LSR_r7),
    kInstruction_movs_vc_r9_r9_LSR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r13_r9_LSL_r1),
    kInstruction_movs_al_r13_r9_LSL_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r8_r4_LSR_r14),
    kInstruction_movs_cc_r8_r4_LSR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r14_r3_LSL_r10),
    kInstruction_movs_cc_r14_r3_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r8_r6_ROR_r14),
    kInstruction_movs_ge_r8_r6_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r4_r5_ASR_r5),
    kInstruction_movs_le_r4_r5_ASR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r4_r8_LSR_r6),
    kInstruction_movs_vc_r4_r8_LSR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r9_r8_ROR_r14),
    kInstruction_movs_vc_r9_r8_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r7_r5_LSL_r11),
    kInstruction_movs_ne_r7_r5_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r12_r12_ASR_r8),
    kInstruction_movs_vs_r12_r12_ASR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r2_r14_ASR_r7),
    kInstruction_movs_pl_r2_r14_ASR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r7_r10_ASR_r9),
    kInstruction_movs_cc_r7_r10_ASR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r7_r10_LSL_r11),
    kInstruction_movs_ne_r7_r10_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r13_r14_ROR_r0),
    kInstruction_movs_vs_r13_r14_ROR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r4_r12_LSL_r6),
    kInstruction_movs_gt_r4_r12_LSL_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r14_r6_ROR_r11),
    kInstruction_movs_ge_r14_r6_ROR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r2_r14_ASR_r0),
    kInstruction_movs_pl_r2_r14_ASR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r7_r6_LSL_r11),
    kInstruction_movs_gt_r7_r6_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r11_r9_LSL_r11),
    kInstruction_movs_cc_r11_r9_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r1_r5_ROR_r8),
    kInstruction_movs_ne_r1_r5_ROR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r6_r2_ROR_r6),
    kInstruction_movs_cs_r6_r2_ROR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r1_r14_LSL_r11),
    kInstruction_movs_vc_r1_r14_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r10_r13_LSL_r11),
    kInstruction_movs_cc_r10_r13_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r12_r10_ROR_r5),
    kInstruction_movs_mi_r12_r10_ROR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r2_r11_LSL_r3),
    kInstruction_movs_ls_r2_r11_LSL_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r6_r13_LSL_r12),
    kInstruction_movs_lt_r6_r13_LSL_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r2_r14_ASR_r5),
    kInstruction_movs_eq_r2_r14_ASR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r0_r14_LSR_r5),
    kInstruction_movs_eq_r0_r14_LSR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r7_r11_LSL_r1),
    kInstruction_movs_ls_r7_r11_LSL_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r14_r13_LSR_r14),
    kInstruction_movs_vs_r14_r13_LSR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r13_r10_ASR_r3),
    kInstruction_movs_vs_r13_r10_ASR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r5_r4_LSL_r2),
    kInstruction_movs_gt_r5_r4_LSL_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r5_r12_LSR_r10),
    kInstruction_movs_hi_r5_r12_LSR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r11_r7_ROR_r11),
    kInstruction_movs_mi_r11_r7_ROR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r10_r7_ROR_r9),
    kInstruction_movs_ne_r10_r7_ROR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r8_r6_ASR_r10),
    kInstruction_movs_ls_r8_r6_ASR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r7_r10_ROR_r9),
    kInstruction_movs_vc_r7_r10_ROR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r8_r3_LSR_r3),
    kInstruction_movs_mi_r8_r3_LSR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r8_r2_ROR_r12),
    kInstruction_movs_lt_r8_r2_ROR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r10_r14_LSL_r11),
    kInstruction_movs_le_r10_r14_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r6_r6_LSR_r5),
    kInstruction_movs_pl_r6_r6_LSR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r8_r4_ROR_r2),
    kInstruction_movs_cc_r8_r4_ROR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r7_r6_ASR_r2),
    kInstruction_movs_al_r7_r6_ASR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r2_r13_ASR_r8),
    kInstruction_movs_ne_r2_r13_ASR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r1_r5_ROR_r12),
    kInstruction_movs_ne_r1_r5_ROR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r10_r6_ROR_r8),
    kInstruction_movs_lt_r10_r6_ROR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r2_r7_LSL_r0),
    kInstruction_movs_cs_r2_r7_LSL_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r3_r2_ASR_r4),
    kInstruction_movs_ne_r3_r2_ASR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r4_r1_LSR_r0),
    kInstruction_movs_mi_r4_r1_LSR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r1_r6_LSL_r10),
    kInstruction_movs_lt_r1_r6_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r6_r9_LSL_r13),
    kInstruction_movs_gt_r6_r9_LSL_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r5_r2_LSL_r13),
    kInstruction_movs_vc_r5_r2_LSL_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r11_r8_ROR_r5),
    kInstruction_movs_ne_r11_r8_ROR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r10_r2_ROR_r12),
    kInstruction_movs_eq_r10_r2_ROR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r0_r5_LSR_r12),
    kInstruction_movs_ge_r0_r5_LSR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r5_r13_ROR_r9),
    kInstruction_movs_vs_r5_r13_ROR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r0_r6_ASR_r9),
    kInstruction_movs_gt_r0_r6_ASR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r6_r0_LSR_r9),
    kInstruction_movs_hi_r6_r0_LSR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r7_r4_ASR_r9),
    kInstruction_movs_lt_r7_r4_ASR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r5_r0_ROR_r2),
    kInstruction_movs_ge_r5_r0_ROR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r10_r11_ROR_r7),
    kInstruction_movs_ls_r10_r11_ROR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r7_r1_ASR_r13),
    kInstruction_movs_cs_r7_r1_ASR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r3_r9_ASR_r11),
    kInstruction_movs_pl_r3_r9_ASR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r13_r5_LSR_r2),
    kInstruction_movs_ne_r13_r5_LSR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r5_r8_ASR_r6),
    kInstruction_movs_vc_r5_r8_ASR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r8_r11_ROR_r13),
    kInstruction_movs_ge_r8_r11_ROR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r0_r10_LSR_r5),
    kInstruction_movs_lt_r0_r10_LSR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r8_r1_ASR_r11),
    kInstruction_movs_vc_r8_r1_ASR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r11_r9_ASR_r9),
    kInstruction_movs_pl_r11_r9_ASR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r3_r1_LSL_r11),
    kInstruction_movs_ls_r3_r1_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r8_r12_ASR_r7),
    kInstruction_movs_hi_r8_r12_ASR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r10_r12_LSR_r9),
    kInstruction_movs_ne_r10_r12_LSR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r12_r4_ASR_r14),
    kInstruction_movs_ge_r12_r4_ASR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r6_r2_ROR_r1),
    kInstruction_movs_mi_r6_r2_ROR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r13_r13_LSL_r0),
    kInstruction_movs_ls_r13_r13_LSL_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r8_r10_LSR_r0),
    kInstruction_movs_eq_r8_r10_LSR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r5_r0_ROR_r14),
    kInstruction_movs_gt_r5_r0_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r3_r4_ROR_r3),
    kInstruction_movs_vs_r3_r4_ROR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r5_r13_LSL_r6),
    kInstruction_movs_lt_r5_r13_LSL_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r5_r1_ASR_r13),
    kInstruction_movs_cc_r5_r1_ASR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r11_r3_ROR_r7),
    kInstruction_movs_lt_r11_r3_ROR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r12_r2_LSR_r4),
    kInstruction_movs_cs_r12_r2_LSR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r3_r12_LSL_r4),
    kInstruction_movs_vc_r3_r12_LSL_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r14_r9_LSL_r5),
    kInstruction_movs_hi_r14_r9_LSL_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r10_r4_LSR_r5),
    kInstruction_movs_ls_r10_r4_LSR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r11_r14_ROR_r14),
    kInstruction_movs_ne_r11_r14_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r11_r12_ASR_r7),
    kInstruction_movs_vc_r11_r12_ASR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r7_r0_LSR_r3),
    kInstruction_movs_cc_r7_r0_LSR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r1_r8_LSR_r3),
    kInstruction_movs_cs_r1_r8_LSR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r7_r6_LSL_r13),
    kInstruction_movs_mi_r7_r6_LSL_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r10_r3_ROR_r7),
    kInstruction_movs_hi_r10_r3_ROR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r2_r11_LSR_r13),
    kInstruction_movs_vs_r2_r11_LSR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r12_r14_LSL_r2),
    kInstruction_movs_eq_r12_r14_LSL_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r14_r13_LSL_r6),
    kInstruction_movs_vs_r14_r13_LSL_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r1_r0_LSL_r10),
    kInstruction_movs_cs_r1_r0_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r2_r14_LSR_r10),
    kInstruction_movs_ge_r2_r14_LSR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r14_r10_ROR_r0),
    kInstruction_movs_mi_r14_r10_ROR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r8_r4_ROR_r4),
    kInstruction_movs_eq_r8_r4_ROR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r4_r9_LSR_r3),
    kInstruction_movs_gt_r4_r9_LSR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r13_r0_ASR_r5),
    kInstruction_movs_ls_r13_r0_ASR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r3_r13_ASR_r12),
    kInstruction_movs_ge_r3_r13_ASR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r9_r7_LSR_r12),
    kInstruction_movs_vc_r9_r7_LSR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r1_r0_LSL_r8),
    kInstruction_movs_al_r1_r0_LSL_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r11_r7_ROR_r5),
    kInstruction_movs_ls_r11_r7_ROR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r11_r8_LSR_r13),
    kInstruction_movs_vs_r11_r8_LSR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r6_r5_LSR_r3),
    kInstruction_movs_le_r6_r5_LSR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r4_r1_LSL_r7),
    kInstruction_movs_cs_r4_r1_LSL_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r4_r9_LSL_r2),
    kInstruction_movs_lt_r4_r9_LSL_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r12_r1_LSR_r10),
    kInstruction_movs_mi_r12_r1_LSR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r14_r2_LSR_r3),
    kInstruction_movs_al_r14_r2_LSR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r11_r9_LSL_r4),
    kInstruction_movs_al_r11_r9_LSL_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r4_r4_ROR_r3),
    kInstruction_movs_hi_r4_r4_ROR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r3_r5_ROR_r6),
    kInstruction_movs_al_r3_r5_ROR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r8_r4_LSR_r7),
    kInstruction_movs_al_r8_r4_LSR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r7_r5_LSL_r6),
    kInstruction_movs_vs_r7_r5_LSL_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r6_r2_LSL_r8),
    kInstruction_movs_ls_r6_r2_LSL_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r4_r12_LSL_r6),
    kInstruction_movs_pl_r4_r12_LSL_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r8_r12_LSR_r4),
    kInstruction_movs_ge_r8_r12_LSR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r11_r1_ROR_r9),
    kInstruction_movs_le_r11_r1_ROR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r1_r13_LSR_r9),
    kInstruction_movs_hi_r1_r13_LSR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r3_r0_LSR_r1),
    kInstruction_movs_cc_r3_r0_LSR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r10_r11_ROR_r0),
    kInstruction_movs_ge_r10_r11_ROR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r6_r5_ROR_r3),
    kInstruction_movs_pl_r6_r5_ROR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r10_r1_LSR_r12),
    kInstruction_movs_cs_r10_r1_LSR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r10_r4_LSR_r1),
    kInstruction_movs_ne_r10_r4_LSR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r5_r1_LSL_r7),
    kInstruction_movs_vs_r5_r1_LSL_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r10_r2_LSL_r4),
    kInstruction_movs_ge_r10_r2_LSL_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r3_r1_ASR_r2),
    kInstruction_movs_vc_r3_r1_ASR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r2_r3_ASR_r14),
    kInstruction_movs_ls_r2_r3_ASR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r1_r1_LSL_r7),
    kInstruction_movs_vs_r1_r1_LSL_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r2_r1_LSL_r14),
    kInstruction_movs_gt_r2_r1_LSL_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r11_r12_ROR_r7),
    kInstruction_movs_al_r11_r12_ROR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r2_r12_LSL_r10),
    kInstruction_movs_eq_r2_r12_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r7_r2_LSR_r7),
    kInstruction_movs_ls_r7_r2_LSR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r9_r5_ROR_r12),
    kInstruction_movs_ne_r9_r5_ROR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r13_r8_LSR_r12),
    kInstruction_movs_ge_r13_r8_LSR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r12_r11_ROR_r9),
    kInstruction_movs_ls_r12_r11_ROR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r13_r5_LSL_r8),
    kInstruction_movs_cc_r13_r5_LSL_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r7_r12_LSL_r7),
    kInstruction_movs_ne_r7_r12_LSL_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r6_r13_ROR_r2),
    kInstruction_movs_ge_r6_r13_ROR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r8_r7_ROR_r6),
    kInstruction_movs_ne_r8_r7_ROR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r7_r10_ROR_r0),
    kInstruction_movs_cc_r7_r10_ROR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r5_r5_ASR_r5),
    kInstruction_movs_le_r5_r5_ASR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r14_r7_ROR_r0),
    kInstruction_movs_hi_r14_r7_ROR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r9_r3_LSL_r11),
    kInstruction_movs_vs_r9_r3_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r0_r5_LSL_r11),
    kInstruction_movs_ls_r0_r5_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r12_r3_LSL_r14),
    kInstruction_movs_ne_r12_r3_LSL_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r12_r12_ROR_r4),
    kInstruction_movs_lt_r12_r12_ROR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r13_r10_LSR_r3),
    kInstruction_movs_gt_r13_r10_LSR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r11_r2_ASR_r10),
    kInstruction_movs_ls_r11_r2_ASR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r7_r8_ASR_r13),
    kInstruction_movs_vc_r7_r8_ASR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r8_r3_LSR_r13),
    kInstruction_movs_eq_r8_r3_LSR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r10_r6_LSL_r8),
    kInstruction_movs_mi_r10_r6_LSL_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r11_r11_LSR_r1),
    kInstruction_movs_le_r11_r11_LSR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r11_r2_ROR_r13),
    kInstruction_movs_vc_r11_r2_ROR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r8_r9_ASR_r2),
    kInstruction_movs_vs_r8_r9_ASR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r11_r11_ASR_r13),
    kInstruction_movs_pl_r11_r11_ASR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r14_r1_LSL_r7),
    kInstruction_movs_mi_r14_r1_LSL_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r7_r7_LSR_r6),
    kInstruction_movs_mi_r7_r7_LSR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r5_r13_LSR_r8),
    kInstruction_movs_eq_r5_r13_LSR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r8_r7_ASR_r0),
    kInstruction_movs_cs_r8_r7_ASR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r8_r4_LSL_r5),
    kInstruction_movs_pl_r8_r4_LSL_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r1_r6_ROR_r2),
    kInstruction_movs_pl_r1_r6_ROR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r13_r14_ASR_r0),
    kInstruction_movs_cc_r13_r14_ASR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r3_r12_LSL_r3),
    kInstruction_movs_al_r3_r12_LSL_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r12_r10_LSL_r14),
    kInstruction_movs_vs_r12_r10_LSL_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r8_r3_ASR_r0),
    kInstruction_movs_lt_r8_r3_ASR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r0_r13_LSL_r2),
    kInstruction_movs_pl_r0_r13_LSL_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r8_r2_LSL_r4),
    kInstruction_movs_gt_r8_r2_LSL_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r13_r0_LSL_r8),
    kInstruction_movs_mi_r13_r0_LSL_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r1_r10_LSL_r14),
    kInstruction_movs_eq_r1_r10_LSL_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r8_r11_ASR_r10),
    kInstruction_movs_hi_r8_r11_ASR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r11_r9_ASR_r0),
    kInstruction_movs_ne_r11_r9_ASR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r0_r13_ROR_r5),
    kInstruction_movs_cc_r0_r13_ROR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r13_r7_ROR_r5),
    kInstruction_movs_ge_r13_r7_ROR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r4_r5_LSL_r2),
    kInstruction_movs_eq_r4_r5_LSL_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r5_r12_ASR_r3),
    kInstruction_movs_ge_r5_r12_ASR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r9_r7_ROR_r6),
    kInstruction_movs_ge_r9_r7_ROR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r2_r2_LSR_r0),
    kInstruction_movs_cc_r2_r2_LSR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r9_r7_ASR_r11),
    kInstruction_movs_cs_r9_r7_ASR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r10_r6_ASR_r10),
    kInstruction_movs_al_r10_r6_ASR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r13_r10_LSL_r10),
    kInstruction_movs_ls_r13_r10_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r14_r13_ASR_r8),
    kInstruction_movs_ne_r14_r13_ASR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r12_r3_ROR_r10),
    kInstruction_movs_lt_r12_r3_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r12_r8_ROR_r8),
    kInstruction_movs_al_r12_r8_ROR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r5_r8_ASR_r8),
    kInstruction_movs_ls_r5_r8_ASR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r13_r11_LSR_r2),
    kInstruction_movs_mi_r13_r11_LSR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r7_r10_ROR_r1),
    kInstruction_movs_le_r7_r10_ROR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r5_r2_LSL_r11),
    kInstruction_movs_ne_r5_r2_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r14_r7_LSL_r10),
    kInstruction_movs_ne_r14_r7_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r0_r1_LSR_r14),
    kInstruction_movs_al_r0_r1_LSR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r5_r9_LSR_r11),
    kInstruction_movs_mi_r5_r9_LSR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r8_r3_LSR_r11),
    kInstruction_movs_eq_r8_r3_LSR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r4_r14_LSR_r11),
    kInstruction_movs_eq_r4_r14_LSR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r0_r7_LSL_r11),
    kInstruction_movs_le_r0_r7_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r2_r2_LSL_r10),
    kInstruction_movs_le_r2_r2_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r3_r5_ROR_r0),
    kInstruction_movs_le_r3_r5_ROR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r8_r10_LSL_r10),
    kInstruction_movs_mi_r8_r10_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r11_r7_ROR_r8),
    kInstruction_movs_ls_r11_r7_ROR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r14_r11_LSR_r7),
    kInstruction_movs_hi_r14_r11_LSR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r10_r2_LSR_r1),
    kInstruction_movs_mi_r10_r2_LSR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r3_r14_ASR_r10),
    kInstruction_movs_le_r3_r14_ASR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r3_r11_ROR_r2),
    kInstruction_movs_vs_r3_r11_ROR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r2_r8_ASR_r2),
    kInstruction_movs_lt_r2_r8_ASR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r7_r4_ROR_r6),
    kInstruction_movs_pl_r7_r4_ROR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r7_r12_ASR_r12),
    kInstruction_movs_al_r7_r12_ASR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r8_r9_ROR_r0),
    kInstruction_movs_ge_r8_r9_ROR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r9_r5_LSL_r7),
    kInstruction_movs_mi_r9_r5_LSL_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r4_r13_LSL_r9),
    kInstruction_movs_cs_r4_r13_LSL_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r5_r3_LSR_r1),
    kInstruction_movs_eq_r5_r3_LSR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r6_r14_LSR_r5),
    kInstruction_movs_lt_r6_r14_LSR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r11_r9_LSL_r11),
    kInstruction_movs_eq_r11_r9_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r4_r0_ROR_r0),
    kInstruction_movs_cs_r4_r0_ROR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r11_r13_LSL_r4),
    kInstruction_movs_pl_r11_r13_LSL_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r1_r2_LSR_r7),
    kInstruction_movs_al_r1_r2_LSR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r7_r2_ROR_r2),
    kInstruction_movs_mi_r7_r2_ROR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r2_r10_LSL_r10),
    kInstruction_movs_ls_r2_r10_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r11_r8_LSL_r3),
    kInstruction_movs_cs_r11_r8_LSL_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r11_r5_ROR_r1),
    kInstruction_movs_cc_r11_r5_ROR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r11_r0_ASR_r12),
    kInstruction_movs_le_r11_r0_ASR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r5_r8_LSL_r6),
    kInstruction_movs_pl_r5_r8_LSL_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r9_r13_LSL_r10),
    kInstruction_movs_ls_r9_r13_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r14_r5_LSL_r10),
    kInstruction_movs_ge_r14_r5_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r8_r10_ASR_r2),
    kInstruction_movs_vs_r8_r10_ASR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r11_r11_LSR_r4),
    kInstruction_movs_vs_r11_r11_LSR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r7_r0_ASR_r7),
    kInstruction_movs_ls_r7_r0_ASR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r10_r5_ASR_r13),
    kInstruction_movs_cc_r10_r5_ASR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r12_r13_ROR_r5),
    kInstruction_movs_hi_r12_r13_ROR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r12_r14_LSR_r5),
    kInstruction_movs_le_r12_r14_LSR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r0_r0_LSL_r14),
    kInstruction_movs_le_r0_r0_LSL_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r3_r7_ROR_r14),
    kInstruction_movs_ge_r3_r7_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r3_r1_LSL_r8),
    kInstruction_movs_eq_r3_r1_LSL_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r7_r1_LSR_r11),
    kInstruction_movs_lt_r7_r1_LSR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r0_r10_ROR_r11),
    kInstruction_movs_ls_r0_r10_ROR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r0_r1_LSL_r8),
    kInstruction_movs_le_r0_r1_LSL_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r13_r3_LSR_r8),
    kInstruction_movs_mi_r13_r3_LSR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r9_r6_LSR_r0),
    kInstruction_movs_ls_r9_r6_LSR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r11_r6_LSR_r3),
    kInstruction_movs_mi_r11_r6_LSR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r11_r6_LSL_r14),
    kInstruction_movs_lt_r11_r6_LSL_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r4_r14_ROR_r5),
    kInstruction_movs_pl_r4_r14_ROR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r7_r6_LSR_r6),
    kInstruction_movs_eq_r7_r6_LSR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r11_r7_ASR_r8),
    kInstruction_movs_hi_r11_r7_ASR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r7_r14_ASR_r14),
    kInstruction_movs_al_r7_r14_ASR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r3_r1_ASR_r10),
    kInstruction_movs_cc_r3_r1_ASR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r9_r3_ROR_r2),
    kInstruction_movs_pl_r9_r3_ROR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r6_r9_LSR_r14),
    kInstruction_movs_vc_r6_r9_LSR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r3_r14_ASR_r7),
    kInstruction_movs_le_r3_r14_ASR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r12_r14_ROR_r14),
    kInstruction_movs_le_r12_r14_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r7_r11_ASR_r9),
    kInstruction_movs_pl_r7_r11_ASR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r1_r12_ASR_r4),
    kInstruction_movs_le_r1_r12_ASR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r7_r12_ASR_r6),
    kInstruction_movs_eq_r7_r12_ASR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r12_r12_ROR_r14),
    kInstruction_movs_mi_r12_r12_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r1_r3_ROR_r3),
    kInstruction_movs_pl_r1_r3_ROR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r1_r12_ASR_r2),
    kInstruction_movs_hi_r1_r12_ASR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r7_r5_LSR_r14),
    kInstruction_movs_ls_r7_r5_LSR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r6_r6_LSR_r2),
    kInstruction_movs_al_r6_r6_LSR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r8_r3_ROR_r13),
    kInstruction_movs_al_r8_r3_ROR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r12_r14_LSL_r11),
    kInstruction_movs_ls_r12_r14_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r9_r9_ROR_r1),
    kInstruction_movs_ne_r9_r9_ROR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r9_r13_ROR_r1),
    kInstruction_movs_hi_r9_r13_ROR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r10_r13_ASR_r13),
    kInstruction_movs_vc_r10_r13_ASR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r13_r13_ROR_r7),
    kInstruction_movs_al_r13_r13_ROR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r0_r14_LSL_r10),
    kInstruction_movs_ls_r0_r14_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r9_r14_ROR_r11),
    kInstruction_movs_vc_r9_r14_ROR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r10_r3_LSR_r14),
    kInstruction_movs_vc_r10_r3_LSR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r11_r14_LSR_r3),
    kInstruction_movs_cc_r11_r14_LSR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r1_r2_LSL_r1),
    kInstruction_movs_ge_r1_r2_LSL_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r10_r8_LSR_r5),
    kInstruction_movs_al_r10_r8_LSR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r2_r5_ASR_r8),
    kInstruction_movs_vs_r2_r5_ASR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r10_r5_LSL_r11),
    kInstruction_movs_cs_r10_r5_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r9_r0_LSR_r6),
    kInstruction_movs_gt_r9_r0_LSR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r8_r2_ROR_r0),
    kInstruction_movs_le_r8_r2_ROR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r0_r9_ASR_r4),
    kInstruction_movs_cc_r0_r9_ASR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r7_r4_ASR_r4),
    kInstruction_movs_pl_r7_r4_ASR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r6_r5_ASR_r6),
    kInstruction_movs_cs_r6_r5_ASR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r14_r6_LSL_r2),
    kInstruction_movs_al_r14_r6_LSL_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r11_r14_ASR_r4),
    kInstruction_movs_pl_r11_r14_ASR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r4_r9_LSL_r10),
    kInstruction_movs_pl_r4_r9_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r7_r1_ROR_r8),
    kInstruction_movs_le_r7_r1_ROR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r3_r1_LSR_r10),
    kInstruction_movs_al_r3_r1_LSR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r8_r14_LSL_r11),
    kInstruction_movs_ge_r8_r14_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r11_r5_LSL_r2),
    kInstruction_movs_le_r11_r5_LSL_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r13_r2_ROR_r0),
    kInstruction_movs_le_r13_r2_ROR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r4_r13_ASR_r10),
    kInstruction_movs_eq_r4_r13_ASR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r14_r5_ASR_r14),
    kInstruction_movs_lt_r14_r5_ASR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r2_r6_ASR_r12),
    kInstruction_movs_gt_r2_r6_ASR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r9_r1_ASR_r8),
    kInstruction_movs_gt_r9_r1_ASR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r11_r6_LSL_r13),
    kInstruction_movs_vs_r11_r6_LSL_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r2_r13_LSL_r3),
    kInstruction_movs_ge_r2_r13_LSL_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r4_r1_ROR_r14),
    kInstruction_movs_hi_r4_r1_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r8_r13_LSL_r2),
    kInstruction_movs_gt_r8_r13_LSL_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r13_r1_LSR_r3),
    kInstruction_movs_pl_r13_r1_LSR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r13_r6_LSL_r10),
    kInstruction_movs_cc_r13_r6_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r10_r1_LSR_r1),
    kInstruction_movs_ls_r10_r1_LSR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r12_r2_LSR_r12),
    kInstruction_movs_hi_r12_r2_LSR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r1_r1_ROR_r12),
    kInstruction_movs_ge_r1_r1_ROR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r12_r0_LSL_r10),
    kInstruction_movs_vc_r12_r0_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r5_r8_ROR_r7),
    kInstruction_movs_eq_r5_r8_ROR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r5_r1_LSR_r1),
    kInstruction_movs_cc_r5_r1_LSR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r7_r14_ASR_r4),
    kInstruction_movs_pl_r7_r14_ASR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r0_r5_LSR_r4),
    kInstruction_movs_ge_r0_r5_LSR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r3_r10_LSL_r11),
    kInstruction_movs_ge_r3_r10_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r13_r11_LSL_r9),
    kInstruction_movs_cs_r13_r11_LSL_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r11_r14_LSL_r10),
    kInstruction_movs_hi_r11_r14_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r7_r5_ROR_r7),
    kInstruction_movs_mi_r7_r5_ROR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r1_r10_ROR_r13),
    kInstruction_movs_ls_r1_r10_ROR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r10_r10_LSR_r12),
    kInstruction_movs_mi_r10_r10_LSR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r14_r7_LSR_r11),
    kInstruction_movs_le_r14_r7_LSR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r3_r12_LSR_r14),
    kInstruction_movs_al_r3_r12_LSR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r1_r3_ROR_r1),
    kInstruction_movs_vs_r1_r3_ROR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r4_r10_LSL_r13),
    kInstruction_movs_ge_r4_r10_LSL_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r4_r3_LSL_r13),
    kInstruction_movs_cs_r4_r3_LSL_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r9_r5_ASR_r11),
    kInstruction_movs_ge_r9_r5_ASR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r12_r4_ROR_r14),
    kInstruction_movs_pl_r12_r4_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r11_r5_LSR_r3),
    kInstruction_movs_mi_r11_r5_LSR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r13_r5_LSR_r4),
    kInstruction_movs_mi_r13_r5_LSR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r13_r0_ASR_r14),
    kInstruction_movs_ls_r13_r0_ASR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r1_r9_ROR_r8),
    kInstruction_movs_cc_r1_r9_ROR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r2_r9_LSL_r7),
    kInstruction_movs_mi_r2_r9_LSL_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r12_r13_LSR_r1),
    kInstruction_movs_gt_r12_r13_LSR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r6_r3_LSL_r9),
    kInstruction_movs_le_r6_r3_LSL_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r11_r2_LSR_r6),
    kInstruction_movs_hi_r11_r2_LSR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r2_r7_ROR_r10),
    kInstruction_movs_vc_r2_r7_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r1_r1_ASR_r9),
    kInstruction_movs_ge_r1_r1_ASR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r8_r14_ROR_r2),
    kInstruction_movs_le_r8_r14_ROR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r5_r4_LSR_r12),
    kInstruction_movs_cs_r5_r4_LSR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r6_r7_ASR_r10),
    kInstruction_movs_ne_r6_r7_ASR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r8_r3_LSL_r7),
    kInstruction_movs_lt_r8_r3_LSL_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r6_r2_LSL_r6),
    kInstruction_movs_ls_r6_r2_LSL_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r6_r3_ROR_r1),
    kInstruction_movs_hi_r6_r3_ROR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r10_r9_LSL_r14),
    kInstruction_movs_le_r10_r9_LSL_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r5_r6_ROR_r11),
    kInstruction_movs_le_r5_r6_ROR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r7_r12_ASR_r7),
    kInstruction_movs_ne_r7_r12_ASR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r11_r5_ASR_r1),
    kInstruction_movs_gt_r11_r5_ASR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r9_r4_ROR_r10),
    kInstruction_movs_al_r9_r4_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r5_r9_ROR_r12),
    kInstruction_movs_vc_r5_r9_ROR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r1_r0_LSR_r3),
    kInstruction_movs_hi_r1_r0_LSR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r2_r12_ROR_r8),
    kInstruction_movs_mi_r2_r12_ROR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r14_r9_ROR_r10),
    kInstruction_movs_eq_r14_r9_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r8_r13_LSL_r1),
    kInstruction_movs_le_r8_r13_LSL_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r13_r4_ROR_r4),
    kInstruction_movs_ne_r13_r4_ROR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r4_r13_ROR_r14),
    kInstruction_movs_ge_r4_r13_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r10_r10_LSL_r1),
    kInstruction_movs_vc_r10_r10_LSL_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r10_r2_ROR_r10),
    kInstruction_movs_gt_r10_r2_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r3_r10_ROR_r12),
    kInstruction_movs_eq_r3_r10_ROR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r8_r8_ASR_r14),
    kInstruction_movs_pl_r8_r8_ASR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r6_r11_LSR_r0),
    kInstruction_movs_hi_r6_r11_LSR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r2_r13_LSR_r13),
    kInstruction_movs_pl_r2_r13_LSR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r3_r1_ASR_r4),
    kInstruction_movs_cs_r3_r1_ASR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r13_r8_ROR_r5),
    kInstruction_movs_ge_r13_r8_ROR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r3_r9_ASR_r10),
    kInstruction_movs_ne_r3_r9_ASR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r6_r12_ASR_r9),
    kInstruction_movs_cc_r6_r12_ASR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r10_r8_LSR_r10),
    kInstruction_movs_cc_r10_r8_LSR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r7_r0_ASR_r7),
    kInstruction_movs_vc_r7_r0_ASR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r2_r5_LSL_r2),
    kInstruction_movs_lt_r2_r5_LSL_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r13_r1_ASR_r8),
    kInstruction_movs_gt_r13_r1_ASR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r11_r14_LSR_r0),
    kInstruction_movs_hi_r11_r14_LSR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r3_r2_LSL_r2),
    kInstruction_movs_ls_r3_r2_LSL_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r5_r2_ROR_r13),
    kInstruction_movs_mi_r5_r2_ROR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r14_r2_ASR_r0),
    kInstruction_movs_ne_r14_r2_ASR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r4_r8_LSL_r2),
    kInstruction_movs_pl_r4_r8_LSL_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r8_r10_LSR_r13),
    kInstruction_movs_ge_r8_r10_LSR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r14_r14_ASR_r12),
    kInstruction_movs_mi_r14_r14_ASR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r3_r1_LSL_r9),
    kInstruction_movs_ge_r3_r1_LSL_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r12_r4_ASR_r6),
    kInstruction_movs_mi_r12_r4_ASR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r1_r4_LSL_r6),
    kInstruction_movs_le_r1_r4_LSL_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r14_r9_ROR_r7),
    kInstruction_movs_ge_r14_r9_ROR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r6_r14_LSL_r7),
    kInstruction_movs_gt_r6_r14_LSL_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r12_r7_ASR_r3),
    kInstruction_movs_vc_r12_r7_ASR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r7_r5_ROR_r0),
    kInstruction_movs_ne_r7_r5_ROR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r13_r2_ROR_r12),
    kInstruction_movs_ne_r13_r2_ROR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r9_r2_ASR_r10),
    kInstruction_movs_cc_r9_r2_ASR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r7_r6_LSR_r5),
    kInstruction_movs_ls_r7_r6_LSR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r14_r1_ASR_r12),
    kInstruction_movs_gt_r14_r1_ASR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r11_r4_ROR_r4),
    kInstruction_movs_vs_r11_r4_ROR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r13_r13_ROR_r14),
    kInstruction_movs_le_r13_r13_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r12_r3_LSR_r2),
    kInstruction_movs_eq_r12_r3_LSR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r9_r14_LSL_r1),
    kInstruction_movs_vs_r9_r14_LSL_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r8_r9_LSL_r9),
    kInstruction_movs_le_r8_r9_LSL_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r2_r13_ROR_r12),
    kInstruction_movs_ne_r2_r13_ROR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r0_r14_ROR_r14),
    kInstruction_movs_eq_r0_r14_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r8_r0_ROR_r9),
    kInstruction_movs_cc_r8_r0_ROR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r11_r0_LSR_r9),
    kInstruction_movs_gt_r11_r0_LSR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r1_r8_LSL_r4),
    kInstruction_movs_vc_r1_r8_LSL_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r4_r4_ROR_r3),
    kInstruction_movs_ne_r4_r4_ROR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r12_r6_LSR_r0),
    kInstruction_movs_cs_r12_r6_LSR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r8_r14_ROR_r7),
    kInstruction_movs_pl_r8_r14_ROR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r7_r11_LSR_r14),
    kInstruction_movs_hi_r7_r11_LSR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r1_r9_ASR_r14),
    kInstruction_movs_vs_r1_r9_ASR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r11_r4_LSR_r14),
    kInstruction_movs_ge_r11_r4_LSR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r9_r13_LSL_r0),
    kInstruction_movs_gt_r9_r13_LSL_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r11_r1_LSR_r7),
    kInstruction_movs_cc_r11_r1_LSR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r14_r7_ROR_r12),
    kInstruction_movs_al_r14_r7_ROR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r11_r5_ROR_r1),
    kInstruction_movs_vc_r11_r5_ROR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r14_r9_ROR_r2),
    kInstruction_movs_eq_r14_r9_ROR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r6_r10_LSR_r8),
    kInstruction_movs_eq_r6_r10_LSR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r5_r7_LSL_r10),
    kInstruction_movs_cc_r5_r7_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r0_r6_ROR_r5),
    kInstruction_movs_vc_r0_r6_ROR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r3_r2_ASR_r5),
    kInstruction_movs_ls_r3_r2_ASR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r0_r5_LSL_r13),
    kInstruction_movs_eq_r0_r5_LSL_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r3_r2_ASR_r2),
    kInstruction_movs_hi_r3_r2_ASR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r6_r1_ASR_r1),
    kInstruction_movs_vs_r6_r1_ASR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r7_r6_ASR_r11),
    kInstruction_movs_al_r7_r6_ASR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r0_r7_LSR_r6),
    kInstruction_movs_al_r0_r7_LSR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r6_r11_LSL_r13),
    kInstruction_movs_vc_r6_r11_LSL_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r11_r0_LSR_r11),
    kInstruction_movs_mi_r11_r0_LSR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r10_r10_ROR_r7),
    kInstruction_movs_ne_r10_r10_ROR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r9_r10_ROR_r8),
    kInstruction_movs_eq_r9_r10_ROR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r0_r11_ROR_r6),
    kInstruction_movs_vc_r0_r11_ROR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r3_r2_LSR_r10),
    kInstruction_movs_eq_r3_r2_LSR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r13_r4_LSR_r7),
    kInstruction_movs_gt_r13_r4_LSR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r1_r12_LSR_r11),
    kInstruction_movs_lt_r1_r12_LSR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r12_r7_LSR_r4),
    kInstruction_movs_cs_r12_r7_LSR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r1_r13_ASR_r13),
    kInstruction_movs_cc_r1_r13_ASR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r4_r9_LSL_r0),
    kInstruction_movs_mi_r4_r9_LSL_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r14_r7_ROR_r0),
    kInstruction_movs_gt_r14_r7_ROR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r1_r0_ASR_r6),
    kInstruction_movs_gt_r1_r0_ASR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r14_r10_ASR_r10),
    kInstruction_movs_ne_r14_r10_ASR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r14_r9_LSR_r8),
    kInstruction_movs_pl_r14_r9_LSR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r3_r7_LSL_r6),
    kInstruction_movs_vs_r3_r7_LSL_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r12_r0_LSL_r3),
    kInstruction_movs_lt_r12_r0_LSL_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r5_r11_ROR_r13),
    kInstruction_movs_hi_r5_r11_ROR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r5_r2_LSR_r2),
    kInstruction_movs_vc_r5_r2_LSR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r13_r8_ROR_r0),
    kInstruction_movs_le_r13_r8_ROR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r13_r9_LSL_r10),
    kInstruction_movs_cc_r13_r9_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r0_r3_ASR_r9),
    kInstruction_movs_ne_r0_r3_ASR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r0_r8_ASR_r10),
    kInstruction_movs_mi_r0_r8_ASR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r4_r4_ASR_r4),
    kInstruction_movs_cc_r4_r4_ASR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r5_r10_LSL_r11),
    kInstruction_movs_vc_r5_r10_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r11_r10_ROR_r2),
    kInstruction_movs_gt_r11_r10_ROR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r5_r5_LSR_r11),
    kInstruction_movs_ne_r5_r5_LSR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r6_r11_LSR_r13),
    kInstruction_movs_ne_r6_r11_LSR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r3_r1_ROR_r1),
    kInstruction_movs_lt_r3_r1_ROR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r1_r14_ASR_r12),
    kInstruction_movs_cc_r1_r14_ASR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r2_r10_ROR_r10),
    kInstruction_movs_mi_r2_r10_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r5_r8_ROR_r10),
    kInstruction_movs_eq_r5_r8_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r3_r9_LSL_r2),
    kInstruction_movs_al_r3_r9_LSL_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r1_r14_LSR_r7),
    kInstruction_movs_cs_r1_r14_LSR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r14_r9_ROR_r1),
    kInstruction_movs_cc_r14_r9_ROR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r3_r8_ROR_r10),
    kInstruction_movs_al_r3_r8_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r2_r10_ASR_r5),
    kInstruction_movs_vs_r2_r10_ASR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r7_r8_ROR_r11),
    kInstruction_movs_al_r7_r8_ROR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r11_r12_ASR_r13),
    kInstruction_movs_cc_r11_r12_ASR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r2_r7_LSR_r12),
    kInstruction_movs_cs_r2_r7_LSR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r7_r4_ROR_r14),
    kInstruction_movs_vs_r7_r4_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r12_r1_ROR_r3),
    kInstruction_movs_ne_r12_r1_ROR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r5_r11_LSL_r2),
    kInstruction_movs_ne_r5_r11_LSL_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r5_r12_ASR_r13),
    kInstruction_movs_cc_r5_r12_ASR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r8_r0_LSL_r14),
    kInstruction_movs_gt_r8_r0_LSL_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r5_r4_LSR_r12),
    kInstruction_movs_cc_r5_r4_LSR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r6_r7_LSR_r14),
    kInstruction_movs_ge_r6_r7_LSR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r2_r0_LSL_r13),
    kInstruction_movs_vs_r2_r0_LSL_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r7_r8_LSL_r0),
    kInstruction_movs_gt_r7_r8_LSL_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r3_r11_ROR_r8),
    kInstruction_movs_ne_r3_r11_ROR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r0_r8_LSR_r4),
    kInstruction_movs_ne_r0_r8_LSR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r1_r14_ASR_r13),
    kInstruction_movs_eq_r1_r14_ASR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r13_r10_LSL_r4),
    kInstruction_movs_pl_r13_r10_LSL_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r12_r14_LSL_r11),
    kInstruction_movs_mi_r12_r14_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r6_r4_LSL_r7),
    kInstruction_movs_vc_r6_r4_LSL_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r7_r8_LSR_r0),
    kInstruction_movs_ge_r7_r8_LSR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r2_r12_LSR_r6),
    kInstruction_movs_ls_r2_r12_LSR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r0_r13_LSL_r1),
    kInstruction_movs_lt_r0_r13_LSL_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r5_r7_LSR_r0),
    kInstruction_movs_vs_r5_r7_LSR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r10_r8_ROR_r4),
    kInstruction_movs_ne_r10_r8_ROR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r4_r1_LSL_r7),
    kInstruction_movs_vc_r4_r1_LSL_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r14_r1_ROR_r10),
    kInstruction_movs_mi_r14_r1_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r4_r5_LSL_r12),
    kInstruction_movs_le_r4_r5_LSL_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r9_r3_ROR_r10),
    kInstruction_movs_cs_r9_r3_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r5_r9_ROR_r7),
    kInstruction_movs_al_r5_r9_ROR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r14_r14_ROR_r1),
    kInstruction_movs_cs_r14_r14_ROR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r5_r11_LSR_r1),
    kInstruction_movs_vc_r5_r11_LSR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r12_r7_ASR_r3),
    kInstruction_movs_hi_r12_r7_ASR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r13_r3_LSL_r10),
    kInstruction_movs_gt_r13_r3_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r12_r14_ASR_r14),
    kInstruction_movs_pl_r12_r14_ASR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r3_r1_ROR_r6),
    kInstruction_movs_al_r3_r1_ROR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r13_r2_ROR_r9),
    kInstruction_movs_le_r13_r2_ROR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r12_r4_LSL_r9),
    kInstruction_movs_cs_r12_r4_LSL_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r4_r7_ASR_r14),
    kInstruction_movs_gt_r4_r7_ASR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r13_r1_ROR_r10),
    kInstruction_movs_vc_r13_r1_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r10_r12_LSR_r1),
    kInstruction_movs_ls_r10_r12_LSR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r8_r0_LSL_r0),
    kInstruction_movs_hi_r8_r0_LSL_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r4_r9_LSL_r0),
    kInstruction_movs_al_r4_r9_LSL_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r12_r10_LSL_r14),
    kInstruction_movs_eq_r12_r10_LSL_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r4_r9_LSL_r12),
    kInstruction_movs_le_r4_r9_LSL_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r1_r6_ASR_r14),
    kInstruction_movs_cc_r1_r6_ASR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r3_r7_ASR_r2),
    kInstruction_movs_pl_r3_r7_ASR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r14_r9_ROR_r0),
    kInstruction_movs_eq_r14_r9_ROR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r8_r6_LSR_r5),
    kInstruction_movs_hi_r8_r6_LSR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r10_r1_LSL_r0),
    kInstruction_movs_hi_r10_r1_LSL_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r0_r13_LSR_r0),
    kInstruction_movs_ls_r0_r13_LSR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r10_r7_ROR_r7),
    kInstruction_movs_vs_r10_r7_ROR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r1_r3_LSR_r6),
    kInstruction_movs_eq_r1_r3_LSR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r8_r6_ASR_r0),
    kInstruction_movs_eq_r8_r6_ASR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r2_r5_LSR_r10),
    kInstruction_movs_eq_r2_r5_LSR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r0_r4_LSL_r13),
    kInstruction_movs_al_r0_r4_LSL_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r0_r12_ROR_r2),
    kInstruction_movs_ge_r0_r12_ROR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r0_r13_ROR_r4),
    kInstruction_movs_le_r0_r13_ROR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r3_r11_ASR_r6),
    kInstruction_movs_ge_r3_r11_ASR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r7_r11_ROR_r0),
    kInstruction_movs_ne_r7_r11_ROR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r14_r5_ASR_r0),
    kInstruction_movs_hi_r14_r5_ASR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r12_r9_LSL_r2),
    kInstruction_movs_cs_r12_r9_LSL_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r12_r8_ASR_r10),
    kInstruction_movs_cs_r12_r8_ASR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r0_r2_LSR_r13),
    kInstruction_movs_cc_r0_r2_LSR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r5_r0_LSR_r4),
    kInstruction_movs_cc_r5_r0_LSR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r10_r14_ROR_r3),
    kInstruction_movs_ne_r10_r14_ROR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r5_r1_LSL_r10),
    kInstruction_movs_hi_r5_r1_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r14_r12_LSR_r13),
    kInstruction_movs_lt_r14_r12_LSR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r14_r9_LSR_r5),
    kInstruction_movs_hi_r14_r9_LSR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r4_r4_LSR_r3),
    kInstruction_movs_cc_r4_r4_LSR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r9_r4_ROR_r13),
    kInstruction_movs_hi_r9_r4_ROR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r7_r6_ASR_r0),
    kInstruction_movs_cc_r7_r6_ASR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r6_r12_ROR_r8),
    kInstruction_movs_ne_r6_r12_ROR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r4_r1_LSL_r9),
    kInstruction_movs_cc_r4_r1_LSL_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r6_r10_LSR_r9),
    kInstruction_movs_ls_r6_r10_LSR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r13_r6_ASR_r4),
    kInstruction_movs_ge_r13_r6_ASR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r0_r6_ROR_r12),
    kInstruction_movs_pl_r0_r6_ROR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r13_r2_LSL_r6),
    kInstruction_movs_eq_r13_r2_LSL_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r5_r14_LSR_r2),
    kInstruction_movs_vc_r5_r14_LSR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r3_r7_LSL_r14),
    kInstruction_movs_ls_r3_r7_LSL_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r13_r7_ROR_r1),
    kInstruction_movs_le_r13_r7_ROR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r6_r1_ROR_r2),
    kInstruction_movs_ge_r6_r1_ROR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r5_r13_LSL_r8),
    kInstruction_movs_le_r5_r13_LSL_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r13_r14_ROR_r6),
    kInstruction_movs_eq_r13_r14_ROR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r10_r9_ASR_r2),
    kInstruction_movs_vs_r10_r9_ASR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r7_r10_LSL_r10),
    kInstruction_movs_ls_r7_r10_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r0_r8_LSL_r13),
    kInstruction_movs_mi_r0_r8_LSL_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r6_r5_ROR_r6),
    kInstruction_movs_cc_r6_r5_ROR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r5_r14_ASR_r9),
    kInstruction_movs_cc_r5_r14_ASR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r8_r3_ROR_r4),
    kInstruction_movs_ge_r8_r3_ROR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r4_r3_ROR_r0),
    kInstruction_movs_ge_r4_r3_ROR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r12_r13_ROR_r5),
    kInstruction_movs_lt_r12_r13_ROR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r14_r14_LSL_r8),
    kInstruction_movs_cs_r14_r14_LSL_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r6_r3_ASR_r12),
    kInstruction_movs_mi_r6_r3_ASR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r2_r1_ASR_r3),
    kInstruction_movs_gt_r2_r1_ASR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r14_r14_LSL_r11),
    kInstruction_movs_vc_r14_r14_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r12_r14_ASR_r5),
    kInstruction_movs_pl_r12_r14_ASR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r7_r12_LSL_r14),
    kInstruction_movs_cs_r7_r12_LSL_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r3_r1_LSR_r3),
    kInstruction_movs_eq_r3_r1_LSR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r6_r7_ASR_r12),
    kInstruction_movs_cs_r6_r7_ASR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r6_r4_ROR_r4),
    kInstruction_movs_le_r6_r4_ROR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r1_r6_LSR_r4),
    kInstruction_movs_hi_r1_r6_LSR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r6_r3_ASR_r4),
    kInstruction_movs_mi_r6_r3_ASR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r9_r10_ASR_r12),
    kInstruction_movs_vs_r9_r10_ASR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r9_r10_LSL_r1),
    kInstruction_movs_cc_r9_r10_LSL_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r12_r9_ASR_r4),
    kInstruction_movs_ls_r12_r9_ASR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r10_r7_ASR_r12),
    kInstruction_movs_vc_r10_r7_ASR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r8_r1_LSR_r5),
    kInstruction_movs_cs_r8_r1_LSR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r10_r10_ASR_r4),
    kInstruction_movs_eq_r10_r10_ASR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r7_r7_LSL_r5),
    kInstruction_movs_ls_r7_r7_LSL_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r8_r11_ROR_r12),
    kInstruction_movs_vs_r8_r11_ROR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r7_r6_ASR_r4),
    kInstruction_movs_vs_r7_r6_ASR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r5_r3_LSL_r6),
    kInstruction_movs_ge_r5_r3_LSL_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r5_r5_LSL_r12),
    kInstruction_movs_vc_r5_r5_LSL_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r0_r13_LSL_r12),
    kInstruction_movs_le_r0_r13_LSL_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r4_r0_ASR_r11),
    kInstruction_movs_ls_r4_r0_ASR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r3_r7_ASR_r5),
    kInstruction_movs_ge_r3_r7_ASR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r11_r4_ROR_r11),
    kInstruction_movs_ge_r11_r4_ROR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r6_r7_ROR_r7),
    kInstruction_movs_al_r6_r7_ROR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r4_r4_LSR_r6),
    kInstruction_movs_gt_r4_r4_LSR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r14_r2_LSR_r1),
    kInstruction_movs_pl_r14_r2_LSR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r10_r9_ROR_r9),
    kInstruction_movs_le_r10_r9_ROR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r5_r3_ASR_r2),
    kInstruction_movs_ne_r5_r3_ASR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r12_r4_LSL_r11),
    kInstruction_movs_lt_r12_r4_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r1_r1_LSR_r13),
    kInstruction_movs_mi_r1_r1_LSR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r7_r4_ROR_r9),
    kInstruction_movs_hi_r7_r4_ROR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r8_r14_LSR_r10),
    kInstruction_movs_lt_r8_r14_LSR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r7_r5_ROR_r11),
    kInstruction_movs_ge_r7_r5_ROR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r12_r14_LSL_r7),
    kInstruction_movs_ls_r12_r14_LSL_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r14_r0_LSL_r3),
    kInstruction_movs_hi_r14_r0_LSL_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r8_r10_ROR_r2),
    kInstruction_movs_ge_r8_r10_ROR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r6_r3_LSR_r0),
    kInstruction_movs_hi_r6_r3_LSR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r4_r1_ROR_r3),
    kInstruction_movs_ls_r4_r1_ROR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r9_r13_ASR_r1),
    kInstruction_movs_ls_r9_r13_ASR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r10_r1_LSR_r5),
    kInstruction_movs_lt_r10_r1_LSR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r10_r13_ASR_r3),
    kInstruction_movs_vc_r10_r13_ASR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r1_r0_LSR_r9),
    kInstruction_movs_gt_r1_r0_LSR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r13_r11_LSL_r14),
    kInstruction_movs_vs_r13_r11_LSL_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r4_r7_LSL_r9),
    kInstruction_movs_eq_r4_r7_LSL_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r14_r9_LSL_r13),
    kInstruction_movs_lt_r14_r9_LSL_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r8_r8_ASR_r0),
    kInstruction_movs_le_r8_r8_ASR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r14_r9_LSR_r3),
    kInstruction_movs_vs_r14_r9_LSR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r10_r6_ROR_r13),
    kInstruction_movs_al_r10_r6_ROR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r5_r2_ROR_r7),
    kInstruction_movs_vs_r5_r2_ROR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r14_r12_ROR_r14),
    kInstruction_movs_vs_r14_r12_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r11_r2_ASR_r0),
    kInstruction_movs_lt_r11_r2_ASR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r3_r2_LSL_r9),
    kInstruction_movs_vc_r3_r2_LSL_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r14_r4_LSL_r2),
    kInstruction_movs_hi_r14_r4_LSL_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r8_r9_ASR_r0),
    kInstruction_movs_gt_r8_r9_ASR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r4_r4_LSL_r11),
    kInstruction_movs_gt_r4_r4_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r5_r14_LSL_r9),
    kInstruction_movs_cc_r5_r14_LSL_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r10_r10_ASR_r5),
    kInstruction_movs_lt_r10_r10_ASR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r1_r12_LSR_r7),
    kInstruction_movs_eq_r1_r12_LSR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r11_r9_ROR_r7),
    kInstruction_movs_eq_r11_r9_ROR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r5_r7_ASR_r6),
    kInstruction_movs_gt_r5_r7_ASR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r7_r2_LSL_r13),
    kInstruction_movs_cc_r7_r2_LSL_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r12_r12_LSL_r4),
    kInstruction_movs_cs_r12_r12_LSL_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r4_r4_ROR_r7),
    kInstruction_movs_ls_r4_r4_ROR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r13_r8_ASR_r10),
    kInstruction_movs_al_r13_r8_ASR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r1_r0_ASR_r1),
    kInstruction_movs_cs_r1_r0_ASR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r6_r7_LSL_r0),
    kInstruction_movs_ne_r6_r7_LSL_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r5_r13_LSR_r9),
    kInstruction_movs_cc_r5_r13_LSR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r9_r6_LSL_r14),
    kInstruction_movs_cc_r9_r6_LSL_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r12_r13_ASR_r2),
    kInstruction_movs_al_r12_r13_ASR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r0_r6_LSR_r6),
    kInstruction_movs_gt_r0_r6_LSR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r6_r10_ROR_r10),
    kInstruction_movs_al_r6_r10_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r4_r13_LSL_r6),
    kInstruction_movs_lt_r4_r13_LSL_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r12_r0_LSL_r10),
    kInstruction_movs_hi_r12_r0_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r5_r9_ROR_r11),
    kInstruction_movs_ge_r5_r9_ROR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r4_r5_ASR_r10),
    kInstruction_movs_ge_r4_r5_ASR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r0_r5_LSL_r11),
    kInstruction_movs_al_r0_r5_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r6_r3_LSR_r8),
    kInstruction_movs_mi_r6_r3_LSR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r2_r6_LSR_r6),
    kInstruction_movs_gt_r2_r6_LSR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r7_r3_ROR_r10),
    kInstruction_movs_le_r7_r3_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r8_r5_LSR_r13),
    kInstruction_movs_vs_r8_r5_LSR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r10_r7_ASR_r14),
    kInstruction_movs_al_r10_r7_ASR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r10_r11_LSL_r5),
    kInstruction_movs_lt_r10_r11_LSL_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r11_r3_LSR_r4),
    kInstruction_movs_lt_r11_r3_LSR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r11_r10_LSR_r8),
    kInstruction_movs_vc_r11_r10_LSR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r11_r5_LSL_r9),
    kInstruction_movs_cs_r11_r5_LSL_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r6_r5_LSR_r7),
    kInstruction_movs_cs_r6_r5_LSR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r7_r7_ASR_r1),
    kInstruction_movs_lt_r7_r7_ASR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r7_r4_ROR_r7),
    kInstruction_movs_cc_r7_r4_ROR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r8_r12_LSR_r9),
    kInstruction_movs_ls_r8_r12_LSR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r5_r13_ROR_r11),
    kInstruction_movs_le_r5_r13_ROR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r4_r5_ROR_r12),
    kInstruction_movs_eq_r4_r5_ROR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r0_r0_LSL_r1),
    kInstruction_movs_ls_r0_r0_LSL_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r7_r2_ROR_r9),
    kInstruction_movs_ge_r7_r2_ROR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r5_r6_LSR_r12),
    kInstruction_movs_cc_r5_r6_LSR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r7_r4_ASR_r10),
    kInstruction_movs_cs_r7_r4_ASR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r5_r4_ROR_r14),
    kInstruction_movs_al_r5_r4_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r7_r3_ROR_r12),
    kInstruction_movs_eq_r7_r3_ROR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r9_r4_LSL_r14),
    kInstruction_movs_cc_r9_r4_LSL_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r5_r2_ASR_r13),
    kInstruction_movs_le_r5_r2_ASR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r14_r12_ROR_r4),
    kInstruction_movs_eq_r14_r12_ROR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r5_r2_ASR_r8),
    kInstruction_movs_eq_r5_r2_ASR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r12_r5_LSR_r7),
    kInstruction_movs_gt_r12_r5_LSR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r11_r14_ROR_r1),
    kInstruction_movs_cc_r11_r14_ROR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r0_r11_LSL_r9),
    kInstruction_movs_ge_r0_r11_LSL_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r4_r4_ROR_r0),
    kInstruction_movs_ls_r4_r4_ROR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r7_r9_ROR_r10),
    kInstruction_movs_cs_r7_r9_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r9_r4_LSR_r5),
    kInstruction_movs_le_r9_r4_LSR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r1_r4_LSL_r10),
    kInstruction_movs_ls_r1_r4_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r9_r13_LSL_r2),
    kInstruction_movs_lt_r9_r13_LSL_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r13_r6_ROR_r13),
    kInstruction_movs_vs_r13_r6_ROR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r4_r0_LSL_r9),
    kInstruction_movs_cs_r4_r0_LSL_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r8_r13_LSR_r14),
    kInstruction_movs_le_r8_r13_LSR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r9_r7_LSL_r10),
    kInstruction_movs_al_r9_r7_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r6_r8_LSL_r2),
    kInstruction_movs_mi_r6_r8_LSL_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r6_r5_ASR_r5),
    kInstruction_movs_ne_r6_r5_ASR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r1_r8_LSR_r12),
    kInstruction_movs_ls_r1_r8_LSR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r9_r7_LSL_r10),
    kInstruction_movs_cs_r9_r7_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r3_r0_ROR_r2),
    kInstruction_movs_cc_r3_r0_ROR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r5_r6_ASR_r11),
    kInstruction_movs_vc_r5_r6_ASR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r11_r9_ROR_r10),
    kInstruction_movs_le_r11_r9_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r4_r10_ROR_r13),
    kInstruction_movs_gt_r4_r10_ROR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r13_r14_LSL_r11),
    kInstruction_movs_lt_r13_r14_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r10_r8_LSL_r11),
    kInstruction_movs_ls_r10_r8_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r7_r9_ROR_r12),
    kInstruction_movs_gt_r7_r9_ROR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r11_r8_LSR_r0),
    kInstruction_movs_le_r11_r8_LSR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r3_r1_LSR_r3),
    kInstruction_movs_vs_r3_r1_LSR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r14_r13_ROR_r5),
    kInstruction_movs_cs_r14_r13_ROR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r1_r0_LSL_r6),
    kInstruction_movs_pl_r1_r0_LSL_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r13_r8_ROR_r12),
    kInstruction_movs_hi_r13_r8_ROR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r6_r7_LSR_r0),
    kInstruction_movs_vc_r6_r7_LSR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r4_r10_LSL_r4),
    kInstruction_movs_gt_r4_r10_LSL_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r4_r12_LSR_r7),
    kInstruction_movs_ge_r4_r12_LSR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r1_r8_LSR_r3),
    kInstruction_movs_pl_r1_r8_LSR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r0_r0_ASR_r7),
    kInstruction_movs_hi_r0_r0_ASR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r0_r7_ROR_r12),
    kInstruction_movs_cs_r0_r7_ROR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r10_r9_LSR_r0),
    kInstruction_movs_eq_r10_r9_LSR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r5_r10_ROR_r10),
    kInstruction_movs_cc_r5_r10_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r0_r14_ROR_r8),
    kInstruction_movs_cs_r0_r14_ROR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r14_r11_LSR_r5),
    kInstruction_movs_eq_r14_r11_LSR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r14_r4_LSR_r12),
    kInstruction_movs_le_r14_r4_LSR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r10_r8_LSL_r10),
    kInstruction_movs_lt_r10_r8_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r13_r4_LSR_r0),
    kInstruction_movs_le_r13_r4_LSR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r9_r9_LSL_r14),
    kInstruction_movs_vc_r9_r9_LSL_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r1_r2_LSR_r7),
    kInstruction_movs_vs_r1_r2_LSR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r14_r4_ROR_r4),
    kInstruction_movs_al_r14_r4_ROR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r10_r6_ROR_r10),
    kInstruction_movs_vs_r10_r6_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r11_r13_LSL_r11),
    kInstruction_movs_cc_r11_r13_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r4_r9_ROR_r2),
    kInstruction_movs_lt_r4_r9_ROR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r4_r12_ASR_r8),
    kInstruction_movs_gt_r4_r12_ASR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r3_r6_LSR_r14),
    kInstruction_movs_hi_r3_r6_LSR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r11_r6_LSL_r5),
    kInstruction_movs_vs_r11_r6_LSL_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r14_r12_LSR_r9),
    kInstruction_movs_ne_r14_r12_LSR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r12_r5_ROR_r0),
    kInstruction_movs_cc_r12_r5_ROR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r11_r1_ASR_r14),
    kInstruction_movs_al_r11_r1_ASR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r9_r1_LSR_r12),
    kInstruction_movs_ge_r9_r1_LSR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r12_r1_LSR_r11),
    kInstruction_movs_pl_r12_r1_LSR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r0_r1_ROR_r4),
    kInstruction_movs_pl_r0_r1_ROR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r10_r12_ROR_r10),
    kInstruction_movs_mi_r10_r12_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r4_r5_ASR_r0),
    kInstruction_movs_cs_r4_r5_ASR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r6_r3_ASR_r11),
    kInstruction_movs_lt_r6_r3_ASR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r5_r13_ASR_r6),
    kInstruction_movs_cs_r5_r13_ASR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r10_r3_LSR_r10),
    kInstruction_movs_vs_r10_r3_LSR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r5_r12_LSR_r8),
    kInstruction_movs_vc_r5_r12_LSR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r3_r10_ROR_r8),
    kInstruction_movs_al_r3_r10_ROR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r11_r4_ASR_r0),
    kInstruction_movs_vc_r11_r4_ASR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r10_r1_LSL_r11),
    kInstruction_movs_ge_r10_r1_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r1_r3_LSR_r8),
    kInstruction_movs_eq_r1_r3_LSR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r14_r4_LSL_r9),
    kInstruction_movs_vc_r14_r4_LSL_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r2_r0_ROR_r3),
    kInstruction_movs_gt_r2_r0_ROR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r10_r3_LSR_r4),
    kInstruction_movs_mi_r10_r3_LSR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r7_r9_ROR_r13),
    kInstruction_movs_vs_r7_r9_ROR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r4_r3_LSR_r1),
    kInstruction_movs_eq_r4_r3_LSR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r7_r0_LSL_r0),
    kInstruction_movs_mi_r7_r0_LSL_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r11_r6_LSR_r11),
    kInstruction_movs_ls_r11_r6_LSR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r8_r4_LSL_r12),
    kInstruction_movs_al_r8_r4_LSL_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r7_r13_LSR_r8),
    kInstruction_movs_pl_r7_r13_LSR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r4_r4_LSR_r13),
    kInstruction_movs_eq_r4_r4_LSR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r3_r4_ROR_r6),
    kInstruction_movs_eq_r3_r4_ROR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r8_r11_LSR_r14),
    kInstruction_movs_ne_r8_r11_LSR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r12_r7_LSL_r1),
    kInstruction_movs_ge_r12_r7_LSL_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r12_r7_LSL_r14),
    kInstruction_movs_gt_r12_r7_LSL_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r14_r13_LSR_r2),
    kInstruction_movs_ne_r14_r13_LSR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r14_r2_LSR_r14),
    kInstruction_movs_ne_r14_r2_LSR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r14_r5_ROR_r1),
    kInstruction_movs_vs_r14_r5_ROR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r2_r6_ROR_r4),
    kInstruction_movs_ne_r2_r6_ROR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r4_r11_ROR_r13),
    kInstruction_movs_lt_r4_r11_ROR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r9_r3_ASR_r2),
    kInstruction_movs_pl_r9_r3_ASR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r5_r5_LSR_r2),
    kInstruction_movs_cs_r5_r5_LSR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r3_r1_ASR_r5),
    kInstruction_movs_cc_r3_r1_ASR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r0_r10_LSL_r1),
    kInstruction_movs_ne_r0_r10_LSL_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r5_r9_LSR_r0),
    kInstruction_movs_pl_r5_r9_LSR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r11_r7_LSL_r11),
    kInstruction_movs_gt_r11_r7_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r5_r13_LSR_r0),
    kInstruction_movs_al_r5_r13_LSR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r0_r6_ROR_r9),
    kInstruction_movs_al_r0_r6_ROR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r1_r1_LSR_r11),
    kInstruction_movs_cc_r1_r1_LSR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r5_r5_ASR_r3),
    kInstruction_movs_cs_r5_r5_ASR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r4_r5_ASR_r3),
    kInstruction_movs_lt_r4_r5_ASR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r2_r5_LSL_r5),
    kInstruction_movs_mi_r2_r5_LSL_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r5_r3_ASR_r1),
    kInstruction_movs_al_r5_r3_ASR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r8_r2_LSR_r3),
    kInstruction_movs_gt_r8_r2_LSR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r5_r0_ASR_r3),
    kInstruction_movs_ne_r5_r0_ASR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r12_r14_LSR_r6),
    kInstruction_movs_ge_r12_r14_LSR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r13_r7_ASR_r10),
    kInstruction_movs_al_r13_r7_ASR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r5_r12_ASR_r3),
    kInstruction_movs_vs_r5_r12_ASR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r10_r10_ROR_r8),
    kInstruction_movs_vc_r10_r10_ROR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r4_r4_ASR_r4),
    kInstruction_movs_vc_r4_r4_ASR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r1_r1_LSR_r3),
    kInstruction_movs_hi_r1_r1_LSR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r14_r8_LSR_r8),
    kInstruction_movs_ge_r14_r8_LSR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r5_r5_LSL_r1),
    kInstruction_movs_le_r5_r5_LSL_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r2_r8_LSL_r8),
    kInstruction_movs_vc_r2_r8_LSL_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r0_r6_LSL_r5),
    kInstruction_movs_hi_r0_r6_LSL_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r8_r7_LSR_r3),
    kInstruction_movs_le_r8_r7_LSR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r10_r4_LSR_r4),
    kInstruction_movs_cc_r10_r4_LSR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r5_r9_LSL_r7),
    kInstruction_movs_cc_r5_r9_LSL_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r1_r14_ROR_r13),
    kInstruction_movs_lt_r1_r14_ROR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r9_r9_ROR_r10),
    kInstruction_movs_mi_r9_r9_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r0_r5_ROR_r7),
    kInstruction_movs_cc_r0_r5_ROR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r6_r5_LSR_r10),
    kInstruction_movs_vs_r6_r5_LSR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r14_r13_ASR_r4),
    kInstruction_movs_pl_r14_r13_ASR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r7_r9_LSL_r11),
    kInstruction_movs_ne_r7_r9_LSL_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r13_r3_ROR_r9),
    kInstruction_movs_ne_r13_r3_ROR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r6_r13_ROR_r5),
    kInstruction_movs_mi_r6_r13_ROR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r10_r0_ASR_r6),
    kInstruction_movs_vc_r10_r0_ASR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r5_r4_LSR_r12),
    kInstruction_movs_ge_r5_r4_LSR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r10_r3_LSR_r8),
    kInstruction_movs_vc_r10_r3_LSR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r8_r1_LSR_r7),
    kInstruction_movs_hi_r8_r1_LSR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r3_r6_ASR_r11),
    kInstruction_movs_al_r3_r6_ASR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r1_r5_ASR_r0),
    kInstruction_movs_cc_r1_r5_ASR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r11_r3_LSR_r8),
    kInstruction_movs_mi_r11_r3_LSR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r4_r8_ASR_r12),
    kInstruction_movs_gt_r4_r8_ASR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r0_r8_ROR_r10),
    kInstruction_movs_hi_r0_r8_ROR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r14_r8_LSR_r9),
    kInstruction_movs_gt_r14_r8_LSR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r14_r13_LSR_r9),
    kInstruction_movs_pl_r14_r13_LSR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r11_r0_LSR_r8),
    kInstruction_movs_cs_r11_r0_LSR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r10_r3_LSR_r4),
    kInstruction_movs_cs_r10_r3_LSR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r9_r0_ASR_r3),
    kInstruction_movs_ge_r9_r0_ASR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r12_r14_LSL_r9),
    kInstruction_movs_eq_r12_r14_LSL_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r2_r11_ASR_r8),
    kInstruction_movs_eq_r2_r11_ASR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r9_r12_ASR_r4),
    kInstruction_movs_le_r9_r12_ASR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r6_r12_LSL_r0),
    kInstruction_movs_lt_r6_r12_LSL_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r14_r4_ROR_r6),
    kInstruction_movs_vc_r14_r4_ROR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r6_r1_LSL_r8),
    kInstruction_movs_hi_r6_r1_LSL_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r10_r12_ROR_r14),
    kInstruction_movs_le_r10_r12_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r9_r1_ROR_r5),
    kInstruction_movs_ls_r9_r1_ROR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r3_r5_LSL_r9),
    kInstruction_movs_mi_r3_r5_LSL_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r5_r4_LSR_r10),
    kInstruction_movs_hi_r5_r4_LSR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r2_r5_LSR_r9),
    kInstruction_movs_ge_r2_r5_LSR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r11_r14_LSL_r9),
    kInstruction_movs_ge_r11_r14_LSL_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r7_r2_LSR_r6),
    kInstruction_movs_vc_r7_r2_LSR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r12_r1_ASR_r6),
    kInstruction_movs_ls_r12_r1_ASR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r3_r2_ROR_r4),
    kInstruction_movs_cs_r3_r2_ROR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r13_r14_ROR_r7),
    kInstruction_movs_lt_r13_r14_ROR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r13_r9_LSR_r2),
    kInstruction_movs_le_r13_r9_LSR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r0_r4_LSL_r8),
    kInstruction_movs_ls_r0_r4_LSL_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r14_r5_ROR_r5),
    kInstruction_movs_pl_r14_r5_ROR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r7_r8_ASR_r13),
    kInstruction_movs_le_r7_r8_ASR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r4_r5_ASR_r5),
    kInstruction_movs_vc_r4_r5_ASR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r14_r2_ROR_r3),
    kInstruction_movs_gt_r14_r2_ROR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r13_r12_LSL_r5),
    kInstruction_movs_eq_r13_r12_LSL_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r5_r12_LSR_r2),
    kInstruction_movs_pl_r5_r12_LSR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r4_r13_ROR_r14),
    kInstruction_movs_cc_r4_r13_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r10_r9_LSL_r9),
    kInstruction_movs_ne_r10_r9_LSL_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r1_r1_ROR_r14),
    kInstruction_movs_eq_r1_r1_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r13_r0_LSL_r13),
    kInstruction_movs_eq_r13_r0_LSL_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r0_r11_ASR_r3),
    kInstruction_movs_vc_r0_r11_ASR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r1_r4_LSR_r3),
    kInstruction_movs_vc_r1_r4_LSR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r8_r3_ASR_r12),
    kInstruction_movs_cc_r8_r3_ASR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r3_r10_LSL_r4),
    kInstruction_movs_vs_r3_r10_LSL_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r10_r9_ASR_r10),
    kInstruction_movs_gt_r10_r9_ASR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r10_r2_LSL_r10),
    kInstruction_movs_cc_r10_r2_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r0_r2_LSL_r1),
    kInstruction_movs_cs_r0_r2_LSL_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r14_r14_ASR_r13),
    kInstruction_movs_hi_r14_r14_ASR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r4_r3_LSL_r1),
    kInstruction_movs_mi_r4_r3_LSL_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r5_r14_ROR_r2),
    kInstruction_movs_gt_r5_r14_ROR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r10_r13_ASR_r12),
    kInstruction_movs_cc_r10_r13_ASR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r2_r4_LSL_r12),
    kInstruction_movs_cs_r2_r4_LSL_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r4_r2_ROR_r11),
    kInstruction_movs_vs_r4_r2_ROR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r1_r13_LSR_r7),
    kInstruction_movs_cs_r1_r13_LSR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r3_r6_ASR_r5),
    kInstruction_movs_al_r3_r6_ASR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r12_r4_LSL_r2),
    kInstruction_movs_ge_r12_r4_LSL_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r8_r13_ROR_r5),
    kInstruction_movs_cc_r8_r13_ROR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r0_r10_LSL_r7),
    kInstruction_movs_vc_r0_r10_LSL_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r13_r0_LSL_r4),
    kInstruction_movs_ge_r13_r0_LSL_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r14_r12_LSL_r10),
    kInstruction_movs_cc_r14_r12_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r9_r9_ASR_r9),
    kInstruction_movs_mi_r9_r9_ASR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r1_r13_ROR_r14),
    kInstruction_movs_al_r1_r13_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r5_r10_ROR_r1),
    kInstruction_movs_ls_r5_r10_ROR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r7_r9_LSL_r7),
    kInstruction_movs_mi_r7_r9_LSL_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r11_r13_LSR_r5),
    kInstruction_movs_ne_r11_r13_LSR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r1_r6_ROR_r9),
    kInstruction_movs_mi_r1_r6_ROR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r1_r2_LSR_r2),
    kInstruction_movs_le_r1_r2_LSR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r12_r9_LSR_r6),
    kInstruction_movs_vc_r12_r9_LSR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r11_r2_ROR_r9),
    kInstruction_movs_cc_r11_r2_ROR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r6_r11_LSL_r10),
    kInstruction_movs_mi_r6_r11_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r6_r1_ROR_r6),
    kInstruction_movs_cc_r6_r1_ROR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r1_r13_LSL_r10),
    kInstruction_movs_gt_r1_r13_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r10_r12_ROR_r11),
    kInstruction_movs_eq_r10_r12_ROR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r5_r7_LSL_r5),
    kInstruction_movs_cs_r5_r7_LSL_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r2_r14_ROR_r6),
    kInstruction_movs_cs_r2_r14_ROR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r4_r6_ASR_r7),
    kInstruction_movs_cc_r4_r6_ASR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r5_r3_ASR_r8),
    kInstruction_movs_mi_r5_r3_ASR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r5_r0_LSR_r4),
    kInstruction_movs_vs_r5_r0_LSR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r4_r7_ASR_r5),
    kInstruction_movs_ge_r4_r7_ASR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r4_r9_ASR_r7),
    kInstruction_movs_lt_r4_r9_ASR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r3_r13_ASR_r6),
    kInstruction_movs_cs_r3_r13_ASR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r3_r7_LSR_r2),
    kInstruction_movs_vs_r3_r7_LSR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r0_r3_ASR_r7),
    kInstruction_movs_pl_r0_r3_ASR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r7_r10_LSL_r0),
    kInstruction_movs_lt_r7_r10_LSL_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r5_r1_LSR_r4),
    kInstruction_movs_pl_r5_r1_LSR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r10_r7_LSL_r10),
    kInstruction_movs_gt_r10_r7_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r14_r6_ROR_r0),
    kInstruction_movs_ls_r14_r6_ROR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r1_r5_ROR_r11),
    kInstruction_movs_le_r1_r5_ROR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r6_r14_LSL_r8),
    kInstruction_movs_ls_r6_r14_LSL_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r12_r11_LSL_r5),
    kInstruction_movs_mi_r12_r11_LSL_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r13_r5_ASR_r3),
    kInstruction_movs_le_r13_r5_ASR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r14_r10_ROR_r14),
    kInstruction_movs_ls_r14_r10_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r5_r12_ASR_r5),
    kInstruction_movs_eq_r5_r12_ASR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r14_r4_ASR_r14),
    kInstruction_movs_gt_r14_r4_ASR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r0_r5_ASR_r13),
    kInstruction_movs_ls_r0_r5_ASR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r6_r4_LSR_r1),
    kInstruction_movs_pl_r6_r4_LSR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r7_r11_ROR_r3),
    kInstruction_movs_pl_r7_r11_ROR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r11_r9_ASR_r13),
    kInstruction_movs_lt_r11_r9_ASR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r3_r9_ASR_r4),
    kInstruction_movs_cc_r3_r9_ASR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r2_r1_ASR_r7),
    kInstruction_movs_eq_r2_r1_ASR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r8_r10_ASR_r9),
    kInstruction_movs_le_r8_r10_ASR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r14_r2_LSL_r6),
    kInstruction_movs_hi_r14_r2_LSL_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r12_r4_LSR_r8),
    kInstruction_movs_mi_r12_r4_LSR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r4_r1_ROR_r2),
    kInstruction_movs_pl_r4_r1_ROR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r5_r9_ROR_r7),
    kInstruction_movs_hi_r5_r9_ROR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r4_r2_LSR_r1),
    kInstruction_movs_le_r4_r2_LSR_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r8_r1_ROR_r9),
    kInstruction_movs_al_r8_r1_ROR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r3_r6_ASR_r6),
    kInstruction_movs_mi_r3_r6_ASR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r1_r12_LSR_r9),
    kInstruction_movs_ls_r1_r12_LSR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r13_r5_LSR_r13),
    kInstruction_movs_le_r13_r5_LSR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r3_r12_ROR_r7),
    kInstruction_movs_ne_r3_r12_ROR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r0_r9_LSR_r5),
    kInstruction_movs_mi_r0_r9_LSR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r4_r4_ROR_r3),
    kInstruction_movs_al_r4_r4_ROR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r1_r1_ASR_r3),
    kInstruction_movs_cc_r1_r1_ASR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r12_r0_LSL_r10),
    kInstruction_movs_lt_r12_r0_LSL_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ge_r1_r10_LSL_r4),
    kInstruction_movs_ge_r1_r10_LSL_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r2_r7_LSL_r4),
    kInstruction_movs_hi_r2_r7_LSL_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r1_r6_ASR_r13),
    kInstruction_movs_le_r1_r6_ASR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r0_r2_ROR_r8),
    kInstruction_movs_eq_r0_r2_ROR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r6_r0_LSL_r8),
    kInstruction_movs_vc_r6_r0_LSL_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r11_r7_LSL_r5),
    kInstruction_movs_hi_r11_r7_LSL_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r0_r1_ASR_r2),
    kInstruction_movs_ne_r0_r1_ASR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r7_r10_ROR_r8),
    kInstruction_movs_gt_r7_r10_ROR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r5_r2_ASR_r8),
    kInstruction_movs_vc_r5_r2_ASR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r4_r9_LSL_r14),
    kInstruction_movs_ls_r4_r9_LSL_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vs_r4_r4_LSR_r5),
    kInstruction_movs_vs_r4_r4_LSR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r4_r14_LSL_r13),
    kInstruction_movs_hi_r4_r14_LSL_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r8_r11_ROR_r13),
    kInstruction_movs_cc_r8_r11_ROR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r2_r2_LSR_r14),
    kInstruction_movs_ls_r2_r2_LSR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r1_r4_ROR_r12),
    kInstruction_movs_lt_r1_r4_ROR_r12,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r2_r6_ROR_r14),
    kInstruction_movs_mi_r2_r6_ROR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r7_r2_ASR_r9),
    kInstruction_movs_hi_r7_r2_ASR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_le_r10_r14_ROR_r13),
    kInstruction_movs_le_r10_r14_ROR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r7_r14_ASR_r11),
    kInstruction_movs_vc_r7_r14_ASR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r4_r5_ASR_r6),
    kInstruction_movs_mi_r4_r5_ASR_r6,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r11_r14_ROR_r9),
    kInstruction_movs_al_r11_r14_ROR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r0_r10_ASR_r13),
    kInstruction_movs_vc_r0_r10_ASR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r10_r2_ROR_r13),
    kInstruction_movs_al_r10_r2_ROR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r9_r2_LSR_r9),
    kInstruction_movs_hi_r9_r2_LSR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_gt_r2_r0_ASR_r10),
    kInstruction_movs_gt_r2_r0_ASR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r4_r12_ASR_r13),
    kInstruction_movs_hi_r4_r12_ASR_r13,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r10_r14_ASR_r0),
    kInstruction_movs_vc_r10_r14_ASR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r13_r9_ASR_r4),
    kInstruction_movs_ne_r13_r9_ASR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r5_r8_ROR_r5),
    kInstruction_movs_eq_r5_r8_ROR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_lt_r0_r13_ROR_r3),
    kInstruction_movs_lt_r0_r13_ROR_r3,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r0_r1_LSL_r1),
    kInstruction_movs_hi_r0_r1_LSL_r1,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r13_r6_ASR_r10),
    kInstruction_movs_pl_r13_r6_ASR_r10,
  },
  {
    ARRAY_SIZE(kInstruction_movs_hi_r0_r4_LSR_r2),
    kInstruction_movs_hi_r0_r4_LSR_r2,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r5_r3_ASR_r8),
    kInstruction_movs_eq_r5_r3_ASR_r8,
  },
  {
    ARRAY_SIZE(kInstruction_movs_al_r1_r5_LSR_r4),
    kInstruction_movs_al_r1_r5_LSR_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r8_r5_LSR_r14),
    kInstruction_movs_eq_r8_r5_LSR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ne_r12_r13_ROR_r0),
    kInstruction_movs_ne_r12_r13_ROR_r0,
  },
  {
    ARRAY_SIZE(kInstruction_movs_pl_r0_r2_ROR_r11),
    kInstruction_movs_pl_r0_r2_ROR_r11,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r10_r6_ROR_r9),
    kInstruction_movs_cs_r10_r6_ROR_r9,
  },
  {
    ARRAY_SIZE(kInstruction_movs_eq_r10_r4_ASR_r14),
    kInstruction_movs_eq_r10_r4_ASR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_ls_r9_r4_LSR_r14),
    kInstruction_movs_ls_r9_r4_LSR_r14,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r14_r2_ROR_r5),
    kInstruction_movs_cs_r14_r2_ROR_r5,
  },
  {
    ARRAY_SIZE(kInstruction_movs_vc_r9_r9_LSL_r7),
    kInstruction_movs_vc_r9_r9_LSL_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_mi_r7_r0_LSL_r4),
    kInstruction_movs_mi_r7_r0_LSL_r4,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cc_r2_r10_ASR_r7),
    kInstruction_movs_cc_r2_r10_ASR_r7,
  },
  {
    ARRAY_SIZE(kInstruction_movs_cs_r5_r10_LSR_r9),
    kInstruction_movs_cs_r5_r10_LSR_r9,
  },
};

#endif  // VIXL_ASSEMBLER_COND_RD_OPERAND_RN_SHIFT_RS_MOVS_A32_H_
