// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_ASSEMBLER_COND_RDLOW_RNLOW_OPERAND_IMMEDIATE_IMM3_IN_IT_BLOCK_ADD_T32_H_
#define VIXL_ASSEMBLER_COND_RDLOW_RNLOW_OPERAND_IMMEDIATE_IMM3_IN_IT_BLOCK_ADD_T32_H_

const byte kInstruction_add_pl_r0_r0_7[] = {
  0x58, 0xbf, 0xc0, 0x1d // It pl; add pl r0 r0 7
};
const byte kInstruction_add_cs_r0_r2_5[] = {
  0x28, 0xbf, 0x50, 0x1d // It cs; add cs r0 r2 5
};
const byte kInstruction_add_ls_r1_r6_0[] = {
  0x98, 0xbf, 0x31, 0x1c // It ls; add ls r1 r6 0
};
const byte kInstruction_add_cc_r5_r1_6[] = {
  0x38, 0xbf, 0x8d, 0x1d // It cc; add cc r5 r1 6
};
const byte kInstruction_add_cs_r5_r2_0[] = {
  0x28, 0xbf, 0x15, 0x1c // It cs; add cs r5 r2 0
};
const byte kInstruction_add_vs_r5_r6_7[] = {
  0x68, 0xbf, 0xf5, 0x1d // It vs; add vs r5 r6 7
};
const byte kInstruction_add_ls_r2_r0_5[] = {
  0x98, 0xbf, 0x42, 0x1d // It ls; add ls r2 r0 5
};
const byte kInstruction_add_eq_r3_r4_5[] = {
  0x08, 0xbf, 0x63, 0x1d // It eq; add eq r3 r4 5
};
const byte kInstruction_add_lt_r4_r5_7[] = {
  0xb8, 0xbf, 0xec, 0x1d // It lt; add lt r4 r5 7
};
const byte kInstruction_add_lt_r6_r7_1[] = {
  0xb8, 0xbf, 0x7e, 0x1c // It lt; add lt r6 r7 1
};
const byte kInstruction_add_eq_r1_r0_3[] = {
  0x08, 0xbf, 0xc1, 0x1c // It eq; add eq r1 r0 3
};
const byte kInstruction_add_le_r4_r1_0[] = {
  0xd8, 0xbf, 0x0c, 0x1c // It le; add le r4 r1 0
};
const byte kInstruction_add_vc_r5_r2_5[] = {
  0x78, 0xbf, 0x55, 0x1d // It vc; add vc r5 r2 5
};
const byte kInstruction_add_cs_r1_r1_5[] = {
  0x28, 0xbf, 0x49, 0x1d // It cs; add cs r1 r1 5
};
const byte kInstruction_add_cs_r2_r5_4[] = {
  0x28, 0xbf, 0x2a, 0x1d // It cs; add cs r2 r5 4
};
const byte kInstruction_add_ge_r7_r5_7[] = {
  0xa8, 0xbf, 0xef, 0x1d // It ge; add ge r7 r5 7
};
const byte kInstruction_add_eq_r3_r4_4[] = {
  0x08, 0xbf, 0x23, 0x1d // It eq; add eq r3 r4 4
};
const byte kInstruction_add_ls_r1_r0_2[] = {
  0x98, 0xbf, 0x81, 0x1c // It ls; add ls r1 r0 2
};
const byte kInstruction_add_vc_r1_r2_0[] = {
  0x78, 0xbf, 0x11, 0x1c // It vc; add vc r1 r2 0
};
const byte kInstruction_add_ls_r6_r6_4[] = {
  0x98, 0xbf, 0x36, 0x1d // It ls; add ls r6 r6 4
};
const byte kInstruction_add_hi_r3_r1_0[] = {
  0x88, 0xbf, 0x0b, 0x1c // It hi; add hi r3 r1 0
};
const byte kInstruction_add_ge_r7_r1_0[] = {
  0xa8, 0xbf, 0x0f, 0x1c // It ge; add ge r7 r1 0
};
const byte kInstruction_add_ge_r2_r0_0[] = {
  0xa8, 0xbf, 0x02, 0x1c // It ge; add ge r2 r0 0
};
const byte kInstruction_add_ge_r1_r7_0[] = {
  0xa8, 0xbf, 0x39, 0x1c // It ge; add ge r1 r7 0
};
const byte kInstruction_add_lt_r5_r2_0[] = {
  0xb8, 0xbf, 0x15, 0x1c // It lt; add lt r5 r2 0
};
const byte kInstruction_add_ls_r6_r3_0[] = {
  0x98, 0xbf, 0x1e, 0x1c // It ls; add ls r6 r3 0
};
const byte kInstruction_add_ne_r5_r7_4[] = {
  0x18, 0xbf, 0x3d, 0x1d // It ne; add ne r5 r7 4
};
const byte kInstruction_add_le_r5_r3_2[] = {
  0xd8, 0xbf, 0x9d, 0x1c // It le; add le r5 r3 2
};
const byte kInstruction_add_eq_r6_r1_4[] = {
  0x08, 0xbf, 0x0e, 0x1d // It eq; add eq r6 r1 4
};
const byte kInstruction_add_gt_r6_r0_1[] = {
  0xc8, 0xbf, 0x46, 0x1c // It gt; add gt r6 r0 1
};
const byte kInstruction_add_pl_r5_r2_2[] = {
  0x58, 0xbf, 0x95, 0x1c // It pl; add pl r5 r2 2
};
const byte kInstruction_add_ls_r7_r2_0[] = {
  0x98, 0xbf, 0x17, 0x1c // It ls; add ls r7 r2 0
};
const byte kInstruction_add_hi_r4_r7_6[] = {
  0x88, 0xbf, 0xbc, 0x1d // It hi; add hi r4 r7 6
};
const byte kInstruction_add_hi_r1_r2_5[] = {
  0x88, 0xbf, 0x51, 0x1d // It hi; add hi r1 r2 5
};
const byte kInstruction_add_ls_r3_r0_3[] = {
  0x98, 0xbf, 0xc3, 0x1c // It ls; add ls r3 r0 3
};
const byte kInstruction_add_le_r5_r5_7[] = {
  0xd8, 0xbf, 0xed, 0x1d // It le; add le r5 r5 7
};
const byte kInstruction_add_pl_r5_r0_2[] = {
  0x58, 0xbf, 0x85, 0x1c // It pl; add pl r5 r0 2
};
const byte kInstruction_add_cs_r3_r1_7[] = {
  0x28, 0xbf, 0xcb, 0x1d // It cs; add cs r3 r1 7
};
const byte kInstruction_add_le_r4_r6_6[] = {
  0xd8, 0xbf, 0xb4, 0x1d // It le; add le r4 r6 6
};
const byte kInstruction_add_lt_r7_r5_2[] = {
  0xb8, 0xbf, 0xaf, 0x1c // It lt; add lt r7 r5 2
};
const byte kInstruction_add_le_r5_r4_3[] = {
  0xd8, 0xbf, 0xe5, 0x1c // It le; add le r5 r4 3
};
const byte kInstruction_add_ne_r4_r7_0[] = {
  0x18, 0xbf, 0x3c, 0x1c // It ne; add ne r4 r7 0
};
const byte kInstruction_add_gt_r0_r3_5[] = {
  0xc8, 0xbf, 0x58, 0x1d // It gt; add gt r0 r3 5
};
const byte kInstruction_add_hi_r0_r4_5[] = {
  0x88, 0xbf, 0x60, 0x1d // It hi; add hi r0 r4 5
};
const byte kInstruction_add_cs_r4_r0_6[] = {
  0x28, 0xbf, 0x84, 0x1d // It cs; add cs r4 r0 6
};
const byte kInstruction_add_ne_r6_r3_0[] = {
  0x18, 0xbf, 0x1e, 0x1c // It ne; add ne r6 r3 0
};
const byte kInstruction_add_ge_r0_r5_3[] = {
  0xa8, 0xbf, 0xe8, 0x1c // It ge; add ge r0 r5 3
};
const byte kInstruction_add_vc_r4_r2_7[] = {
  0x78, 0xbf, 0xd4, 0x1d // It vc; add vc r4 r2 7
};
const byte kInstruction_add_le_r3_r0_6[] = {
  0xd8, 0xbf, 0x83, 0x1d // It le; add le r3 r0 6
};
const byte kInstruction_add_lt_r0_r4_5[] = {
  0xb8, 0xbf, 0x60, 0x1d // It lt; add lt r0 r4 5
};
const byte kInstruction_add_cs_r7_r0_4[] = {
  0x28, 0xbf, 0x07, 0x1d // It cs; add cs r7 r0 4
};
const byte kInstruction_add_ge_r0_r0_6[] = {
  0xa8, 0xbf, 0x80, 0x1d // It ge; add ge r0 r0 6
};
const byte kInstruction_add_hi_r2_r6_0[] = {
  0x88, 0xbf, 0x32, 0x1c // It hi; add hi r2 r6 0
};
const byte kInstruction_add_hi_r1_r4_0[] = {
  0x88, 0xbf, 0x21, 0x1c // It hi; add hi r1 r4 0
};
const byte kInstruction_add_lt_r1_r5_5[] = {
  0xb8, 0xbf, 0x69, 0x1d // It lt; add lt r1 r5 5
};
const byte kInstruction_add_eq_r0_r6_7[] = {
  0x08, 0xbf, 0xf0, 0x1d // It eq; add eq r0 r6 7
};
const byte kInstruction_add_hi_r2_r4_6[] = {
  0x88, 0xbf, 0xa2, 0x1d // It hi; add hi r2 r4 6
};
const byte kInstruction_add_ls_r4_r4_2[] = {
  0x98, 0xbf, 0xa4, 0x1c // It ls; add ls r4 r4 2
};
const byte kInstruction_add_gt_r1_r5_5[] = {
  0xc8, 0xbf, 0x69, 0x1d // It gt; add gt r1 r5 5
};
const byte kInstruction_add_gt_r5_r4_2[] = {
  0xc8, 0xbf, 0xa5, 0x1c // It gt; add gt r5 r4 2
};
const byte kInstruction_add_gt_r0_r3_6[] = {
  0xc8, 0xbf, 0x98, 0x1d // It gt; add gt r0 r3 6
};
const byte kInstruction_add_hi_r6_r2_6[] = {
  0x88, 0xbf, 0x96, 0x1d // It hi; add hi r6 r2 6
};
const byte kInstruction_add_le_r0_r7_5[] = {
  0xd8, 0xbf, 0x78, 0x1d // It le; add le r0 r7 5
};
const byte kInstruction_add_mi_r2_r7_6[] = {
  0x48, 0xbf, 0xba, 0x1d // It mi; add mi r2 r7 6
};
const byte kInstruction_add_le_r5_r6_2[] = {
  0xd8, 0xbf, 0xb5, 0x1c // It le; add le r5 r6 2
};
const byte kInstruction_add_eq_r0_r4_5[] = {
  0x08, 0xbf, 0x60, 0x1d // It eq; add eq r0 r4 5
};
const byte kInstruction_add_hi_r4_r5_2[] = {
  0x88, 0xbf, 0xac, 0x1c // It hi; add hi r4 r5 2
};
const byte kInstruction_add_gt_r0_r7_0[] = {
  0xc8, 0xbf, 0x38, 0x1c // It gt; add gt r0 r7 0
};
const byte kInstruction_add_eq_r6_r4_0[] = {
  0x08, 0xbf, 0x26, 0x1c // It eq; add eq r6 r4 0
};
const byte kInstruction_add_mi_r2_r2_4[] = {
  0x48, 0xbf, 0x12, 0x1d // It mi; add mi r2 r2 4
};
const byte kInstruction_add_vs_r7_r4_2[] = {
  0x68, 0xbf, 0xa7, 0x1c // It vs; add vs r7 r4 2
};
const byte kInstruction_add_ne_r5_r3_6[] = {
  0x18, 0xbf, 0x9d, 0x1d // It ne; add ne r5 r3 6
};
const byte kInstruction_add_ge_r0_r3_2[] = {
  0xa8, 0xbf, 0x98, 0x1c // It ge; add ge r0 r3 2
};
const byte kInstruction_add_cc_r1_r5_5[] = {
  0x38, 0xbf, 0x69, 0x1d // It cc; add cc r1 r5 5
};
const byte kInstruction_add_ge_r7_r3_5[] = {
  0xa8, 0xbf, 0x5f, 0x1d // It ge; add ge r7 r3 5
};
const byte kInstruction_add_ls_r6_r4_0[] = {
  0x98, 0xbf, 0x26, 0x1c // It ls; add ls r6 r4 0
};
const byte kInstruction_add_cs_r5_r1_4[] = {
  0x28, 0xbf, 0x0d, 0x1d // It cs; add cs r5 r1 4
};
const byte kInstruction_add_ls_r6_r7_0[] = {
  0x98, 0xbf, 0x3e, 0x1c // It ls; add ls r6 r7 0
};
const byte kInstruction_add_cc_r5_r7_5[] = {
  0x38, 0xbf, 0x7d, 0x1d // It cc; add cc r5 r7 5
};
const byte kInstruction_add_eq_r0_r2_5[] = {
  0x08, 0xbf, 0x50, 0x1d // It eq; add eq r0 r2 5
};
const byte kInstruction_add_hi_r1_r6_6[] = {
  0x88, 0xbf, 0xb1, 0x1d // It hi; add hi r1 r6 6
};
const byte kInstruction_add_vs_r4_r6_7[] = {
  0x68, 0xbf, 0xf4, 0x1d // It vs; add vs r4 r6 7
};
const byte kInstruction_add_lt_r1_r5_4[] = {
  0xb8, 0xbf, 0x29, 0x1d // It lt; add lt r1 r5 4
};
const byte kInstruction_add_gt_r1_r2_7[] = {
  0xc8, 0xbf, 0xd1, 0x1d // It gt; add gt r1 r2 7
};
const byte kInstruction_add_cc_r4_r5_4[] = {
  0x38, 0xbf, 0x2c, 0x1d // It cc; add cc r4 r5 4
};
const byte kInstruction_add_pl_r6_r5_2[] = {
  0x58, 0xbf, 0xae, 0x1c // It pl; add pl r6 r5 2
};
const byte kInstruction_add_lt_r5_r1_0[] = {
  0xb8, 0xbf, 0x0d, 0x1c // It lt; add lt r5 r1 0
};
const byte kInstruction_add_vc_r1_r7_1[] = {
  0x78, 0xbf, 0x79, 0x1c // It vc; add vc r1 r7 1
};
const byte kInstruction_add_eq_r1_r2_6[] = {
  0x08, 0xbf, 0x91, 0x1d // It eq; add eq r1 r2 6
};
const byte kInstruction_add_ls_r0_r4_2[] = {
  0x98, 0xbf, 0xa0, 0x1c // It ls; add ls r0 r4 2
};
const byte kInstruction_add_vs_r2_r6_0[] = {
  0x68, 0xbf, 0x32, 0x1c // It vs; add vs r2 r6 0
};
const byte kInstruction_add_le_r6_r3_6[] = {
  0xd8, 0xbf, 0x9e, 0x1d // It le; add le r6 r3 6
};
const byte kInstruction_add_mi_r6_r7_1[] = {
  0x48, 0xbf, 0x7e, 0x1c // It mi; add mi r6 r7 1
};
const byte kInstruction_add_lt_r2_r0_5[] = {
  0xb8, 0xbf, 0x42, 0x1d // It lt; add lt r2 r0 5
};
const byte kInstruction_add_le_r0_r6_1[] = {
  0xd8, 0xbf, 0x70, 0x1c // It le; add le r0 r6 1
};
const byte kInstruction_add_hi_r6_r6_6[] = {
  0x88, 0xbf, 0xb6, 0x1d // It hi; add hi r6 r6 6
};
const byte kInstruction_add_eq_r5_r6_1[] = {
  0x08, 0xbf, 0x75, 0x1c // It eq; add eq r5 r6 1
};
const byte kInstruction_add_eq_r2_r1_4[] = {
  0x08, 0xbf, 0x0a, 0x1d // It eq; add eq r2 r1 4
};
const byte kInstruction_add_gt_r5_r6_5[] = {
  0xc8, 0xbf, 0x75, 0x1d // It gt; add gt r5 r6 5
};
const byte kInstruction_add_vc_r3_r4_3[] = {
  0x78, 0xbf, 0xe3, 0x1c // It vc; add vc r3 r4 3
};
const byte kInstruction_add_cc_r0_r1_4[] = {
  0x38, 0xbf, 0x08, 0x1d // It cc; add cc r0 r1 4
};
const byte kInstruction_add_hi_r3_r1_7[] = {
  0x88, 0xbf, 0xcb, 0x1d // It hi; add hi r3 r1 7
};
const byte kInstruction_add_ge_r2_r3_5[] = {
  0xa8, 0xbf, 0x5a, 0x1d // It ge; add ge r2 r3 5
};
const byte kInstruction_add_mi_r1_r0_5[] = {
  0x48, 0xbf, 0x41, 0x1d // It mi; add mi r1 r0 5
};
const byte kInstruction_add_vc_r3_r0_3[] = {
  0x78, 0xbf, 0xc3, 0x1c // It vc; add vc r3 r0 3
};
const byte kInstruction_add_vs_r0_r5_7[] = {
  0x68, 0xbf, 0xe8, 0x1d // It vs; add vs r0 r5 7
};
const byte kInstruction_add_eq_r2_r1_5[] = {
  0x08, 0xbf, 0x4a, 0x1d // It eq; add eq r2 r1 5
};
const byte kInstruction_add_ne_r3_r0_0[] = {
  0x18, 0xbf, 0x03, 0x1c // It ne; add ne r3 r0 0
};
const byte kInstruction_add_vs_r0_r6_1[] = {
  0x68, 0xbf, 0x70, 0x1c // It vs; add vs r0 r6 1
};
const byte kInstruction_add_cs_r6_r1_4[] = {
  0x28, 0xbf, 0x0e, 0x1d // It cs; add cs r6 r1 4
};
const byte kInstruction_add_cs_r5_r2_4[] = {
  0x28, 0xbf, 0x15, 0x1d // It cs; add cs r5 r2 4
};
const byte kInstruction_add_cs_r4_r4_7[] = {
  0x28, 0xbf, 0xe4, 0x1d // It cs; add cs r4 r4 7
};
const byte kInstruction_add_vs_r1_r2_4[] = {
  0x68, 0xbf, 0x11, 0x1d // It vs; add vs r1 r2 4
};
const byte kInstruction_add_ne_r0_r1_6[] = {
  0x18, 0xbf, 0x88, 0x1d // It ne; add ne r0 r1 6
};
const byte kInstruction_add_ge_r2_r4_0[] = {
  0xa8, 0xbf, 0x22, 0x1c // It ge; add ge r2 r4 0
};
const byte kInstruction_add_pl_r0_r4_4[] = {
  0x58, 0xbf, 0x20, 0x1d // It pl; add pl r0 r4 4
};
const byte kInstruction_add_vs_r6_r2_6[] = {
  0x68, 0xbf, 0x96, 0x1d // It vs; add vs r6 r2 6
};
const byte kInstruction_add_lt_r4_r0_6[] = {
  0xb8, 0xbf, 0x84, 0x1d // It lt; add lt r4 r0 6
};
const byte kInstruction_add_cc_r3_r1_2[] = {
  0x38, 0xbf, 0x8b, 0x1c // It cc; add cc r3 r1 2
};
const byte kInstruction_add_hi_r6_r1_2[] = {
  0x88, 0xbf, 0x8e, 0x1c // It hi; add hi r6 r1 2
};
const byte kInstruction_add_cc_r0_r5_4[] = {
  0x38, 0xbf, 0x28, 0x1d // It cc; add cc r0 r5 4
};
const byte kInstruction_add_lt_r3_r3_2[] = {
  0xb8, 0xbf, 0x9b, 0x1c // It lt; add lt r3 r3 2
};
const byte kInstruction_add_vc_r0_r2_3[] = {
  0x78, 0xbf, 0xd0, 0x1c // It vc; add vc r0 r2 3
};
const byte kInstruction_add_ne_r5_r3_5[] = {
  0x18, 0xbf, 0x5d, 0x1d // It ne; add ne r5 r3 5
};
const byte kInstruction_add_gt_r4_r3_6[] = {
  0xc8, 0xbf, 0x9c, 0x1d // It gt; add gt r4 r3 6
};
const byte kInstruction_add_eq_r5_r5_7[] = {
  0x08, 0xbf, 0xed, 0x1d // It eq; add eq r5 r5 7
};
const byte kInstruction_add_cc_r6_r1_6[] = {
  0x38, 0xbf, 0x8e, 0x1d // It cc; add cc r6 r1 6
};
const byte kInstruction_add_ls_r4_r4_4[] = {
  0x98, 0xbf, 0x24, 0x1d // It ls; add ls r4 r4 4
};
const byte kInstruction_add_vs_r7_r3_2[] = {
  0x68, 0xbf, 0x9f, 0x1c // It vs; add vs r7 r3 2
};
const byte kInstruction_add_cc_r0_r4_1[] = {
  0x38, 0xbf, 0x60, 0x1c // It cc; add cc r0 r4 1
};
const byte kInstruction_add_ge_r4_r2_3[] = {
  0xa8, 0xbf, 0xd4, 0x1c // It ge; add ge r4 r2 3
};
const byte kInstruction_add_pl_r1_r6_0[] = {
  0x58, 0xbf, 0x31, 0x1c // It pl; add pl r1 r6 0
};
const byte kInstruction_add_eq_r1_r5_4[] = {
  0x08, 0xbf, 0x29, 0x1d // It eq; add eq r1 r5 4
};
const byte kInstruction_add_pl_r0_r1_7[] = {
  0x58, 0xbf, 0xc8, 0x1d // It pl; add pl r0 r1 7
};
const byte kInstruction_add_cs_r5_r6_2[] = {
  0x28, 0xbf, 0xb5, 0x1c // It cs; add cs r5 r6 2
};
const byte kInstruction_add_mi_r1_r2_3[] = {
  0x48, 0xbf, 0xd1, 0x1c // It mi; add mi r1 r2 3
};
const byte kInstruction_add_mi_r3_r4_0[] = {
  0x48, 0xbf, 0x23, 0x1c // It mi; add mi r3 r4 0
};
const byte kInstruction_add_ne_r5_r1_1[] = {
  0x18, 0xbf, 0x4d, 0x1c // It ne; add ne r5 r1 1
};
const byte kInstruction_add_cc_r3_r5_4[] = {
  0x38, 0xbf, 0x2b, 0x1d // It cc; add cc r3 r5 4
};
const byte kInstruction_add_ne_r4_r1_4[] = {
  0x18, 0xbf, 0x0c, 0x1d // It ne; add ne r4 r1 4
};
const byte kInstruction_add_vs_r4_r4_4[] = {
  0x68, 0xbf, 0x24, 0x1d // It vs; add vs r4 r4 4
};
const byte kInstruction_add_mi_r4_r2_6[] = {
  0x48, 0xbf, 0x94, 0x1d // It mi; add mi r4 r2 6
};
const byte kInstruction_add_hi_r4_r5_3[] = {
  0x88, 0xbf, 0xec, 0x1c // It hi; add hi r4 r5 3
};
const byte kInstruction_add_pl_r2_r5_7[] = {
  0x58, 0xbf, 0xea, 0x1d // It pl; add pl r2 r5 7
};
const byte kInstruction_add_cs_r1_r4_1[] = {
  0x28, 0xbf, 0x61, 0x1c // It cs; add cs r1 r4 1
};
const byte kInstruction_add_hi_r7_r5_0[] = {
  0x88, 0xbf, 0x2f, 0x1c // It hi; add hi r7 r5 0
};
const byte kInstruction_add_mi_r5_r6_4[] = {
  0x48, 0xbf, 0x35, 0x1d // It mi; add mi r5 r6 4
};
const byte kInstruction_add_pl_r3_r2_1[] = {
  0x58, 0xbf, 0x53, 0x1c // It pl; add pl r3 r2 1
};
const byte kInstruction_add_le_r1_r5_2[] = {
  0xd8, 0xbf, 0xa9, 0x1c // It le; add le r1 r5 2
};
const byte kInstruction_add_le_r1_r0_0[] = {
  0xd8, 0xbf, 0x01, 0x1c // It le; add le r1 r0 0
};
const byte kInstruction_add_vc_r5_r5_4[] = {
  0x78, 0xbf, 0x2d, 0x1d // It vc; add vc r5 r5 4
};
const byte kInstruction_add_vc_r7_r2_5[] = {
  0x78, 0xbf, 0x57, 0x1d // It vc; add vc r7 r2 5
};
const byte kInstruction_add_hi_r5_r4_2[] = {
  0x88, 0xbf, 0xa5, 0x1c // It hi; add hi r5 r4 2
};
const byte kInstruction_add_ls_r4_r4_1[] = {
  0x98, 0xbf, 0x64, 0x1c // It ls; add ls r4 r4 1
};
const byte kInstruction_add_cc_r6_r6_4[] = {
  0x38, 0xbf, 0x36, 0x1d // It cc; add cc r6 r6 4
};
const byte kInstruction_add_mi_r6_r7_0[] = {
  0x48, 0xbf, 0x3e, 0x1c // It mi; add mi r6 r7 0
};
const byte kInstruction_add_eq_r0_r1_2[] = {
  0x08, 0xbf, 0x88, 0x1c // It eq; add eq r0 r1 2
};
const byte kInstruction_add_cc_r7_r2_5[] = {
  0x38, 0xbf, 0x57, 0x1d // It cc; add cc r7 r2 5
};
const byte kInstruction_add_ls_r6_r7_7[] = {
  0x98, 0xbf, 0xfe, 0x1d // It ls; add ls r6 r7 7
};
const byte kInstruction_add_cs_r1_r1_3[] = {
  0x28, 0xbf, 0xc9, 0x1c // It cs; add cs r1 r1 3
};
const byte kInstruction_add_ls_r7_r1_2[] = {
  0x98, 0xbf, 0x8f, 0x1c // It ls; add ls r7 r1 2
};
const byte kInstruction_add_gt_r3_r1_3[] = {
  0xc8, 0xbf, 0xcb, 0x1c // It gt; add gt r3 r1 3
};
const byte kInstruction_add_gt_r4_r7_0[] = {
  0xc8, 0xbf, 0x3c, 0x1c // It gt; add gt r4 r7 0
};
const byte kInstruction_add_pl_r4_r5_2[] = {
  0x58, 0xbf, 0xac, 0x1c // It pl; add pl r4 r5 2
};
const byte kInstruction_add_vs_r4_r7_6[] = {
  0x68, 0xbf, 0xbc, 0x1d // It vs; add vs r4 r7 6
};
const byte kInstruction_add_vc_r5_r1_6[] = {
  0x78, 0xbf, 0x8d, 0x1d // It vc; add vc r5 r1 6
};
const byte kInstruction_add_vs_r6_r5_5[] = {
  0x68, 0xbf, 0x6e, 0x1d // It vs; add vs r6 r5 5
};
const byte kInstruction_add_eq_r5_r0_1[] = {
  0x08, 0xbf, 0x45, 0x1c // It eq; add eq r5 r0 1
};
const byte kInstruction_add_mi_r4_r6_1[] = {
  0x48, 0xbf, 0x74, 0x1c // It mi; add mi r4 r6 1
};
const byte kInstruction_add_cs_r1_r5_7[] = {
  0x28, 0xbf, 0xe9, 0x1d // It cs; add cs r1 r5 7
};
const byte kInstruction_add_lt_r0_r3_6[] = {
  0xb8, 0xbf, 0x98, 0x1d // It lt; add lt r0 r3 6
};
const byte kInstruction_add_eq_r7_r0_6[] = {
  0x08, 0xbf, 0x87, 0x1d // It eq; add eq r7 r0 6
};
const byte kInstruction_add_vc_r3_r5_3[] = {
  0x78, 0xbf, 0xeb, 0x1c // It vc; add vc r3 r5 3
};
const byte kInstruction_add_gt_r4_r7_7[] = {
  0xc8, 0xbf, 0xfc, 0x1d // It gt; add gt r4 r7 7
};
const byte kInstruction_add_ne_r4_r1_1[] = {
  0x18, 0xbf, 0x4c, 0x1c // It ne; add ne r4 r1 1
};
const byte kInstruction_add_ge_r6_r7_4[] = {
  0xa8, 0xbf, 0x3e, 0x1d // It ge; add ge r6 r7 4
};
const byte kInstruction_add_le_r2_r0_2[] = {
  0xd8, 0xbf, 0x82, 0x1c // It le; add le r2 r0 2
};
const byte kInstruction_add_mi_r7_r5_7[] = {
  0x48, 0xbf, 0xef, 0x1d // It mi; add mi r7 r5 7
};
const byte kInstruction_add_lt_r5_r3_6[] = {
  0xb8, 0xbf, 0x9d, 0x1d // It lt; add lt r5 r3 6
};
const byte kInstruction_add_vs_r1_r4_7[] = {
  0x68, 0xbf, 0xe1, 0x1d // It vs; add vs r1 r4 7
};
const byte kInstruction_add_cs_r2_r2_4[] = {
  0x28, 0xbf, 0x12, 0x1d // It cs; add cs r2 r2 4
};
const byte kInstruction_add_cs_r1_r0_1[] = {
  0x28, 0xbf, 0x41, 0x1c // It cs; add cs r1 r0 1
};
const byte kInstruction_add_ne_r0_r7_4[] = {
  0x18, 0xbf, 0x38, 0x1d // It ne; add ne r0 r7 4
};
const byte kInstruction_add_vs_r6_r0_4[] = {
  0x68, 0xbf, 0x06, 0x1d // It vs; add vs r6 r0 4
};
const byte kInstruction_add_vc_r7_r1_2[] = {
  0x78, 0xbf, 0x8f, 0x1c // It vc; add vc r7 r1 2
};
const byte kInstruction_add_ls_r5_r0_4[] = {
  0x98, 0xbf, 0x05, 0x1d // It ls; add ls r5 r0 4
};
const byte kInstruction_add_le_r0_r1_1[] = {
  0xd8, 0xbf, 0x48, 0x1c // It le; add le r0 r1 1
};
const byte kInstruction_add_cs_r7_r3_1[] = {
  0x28, 0xbf, 0x5f, 0x1c // It cs; add cs r7 r3 1
};
const byte kInstruction_add_cs_r7_r5_3[] = {
  0x28, 0xbf, 0xef, 0x1c // It cs; add cs r7 r5 3
};
const byte kInstruction_add_gt_r3_r7_2[] = {
  0xc8, 0xbf, 0xbb, 0x1c // It gt; add gt r3 r7 2
};
const byte kInstruction_add_eq_r3_r4_1[] = {
  0x08, 0xbf, 0x63, 0x1c // It eq; add eq r3 r4 1
};
const byte kInstruction_add_mi_r3_r4_6[] = {
  0x48, 0xbf, 0xa3, 0x1d // It mi; add mi r3 r4 6
};
const byte kInstruction_add_eq_r0_r4_3[] = {
  0x08, 0xbf, 0xe0, 0x1c // It eq; add eq r0 r4 3
};
const byte kInstruction_add_le_r0_r7_6[] = {
  0xd8, 0xbf, 0xb8, 0x1d // It le; add le r0 r7 6
};
const byte kInstruction_add_le_r5_r6_3[] = {
  0xd8, 0xbf, 0xf5, 0x1c // It le; add le r5 r6 3
};
const byte kInstruction_add_vs_r3_r5_3[] = {
  0x68, 0xbf, 0xeb, 0x1c // It vs; add vs r3 r5 3
};
const byte kInstruction_add_pl_r3_r4_3[] = {
  0x58, 0xbf, 0xe3, 0x1c // It pl; add pl r3 r4 3
};
const byte kInstruction_add_le_r2_r3_0[] = {
  0xd8, 0xbf, 0x1a, 0x1c // It le; add le r2 r3 0
};
const byte kInstruction_add_vc_r5_r4_4[] = {
  0x78, 0xbf, 0x25, 0x1d // It vc; add vc r5 r4 4
};
const byte kInstruction_add_le_r5_r0_6[] = {
  0xd8, 0xbf, 0x85, 0x1d // It le; add le r5 r0 6
};
const byte kInstruction_add_vs_r6_r5_6[] = {
  0x68, 0xbf, 0xae, 0x1d // It vs; add vs r6 r5 6
};
const byte kInstruction_add_ge_r2_r2_3[] = {
  0xa8, 0xbf, 0xd2, 0x1c // It ge; add ge r2 r2 3
};
const byte kInstruction_add_vs_r5_r4_5[] = {
  0x68, 0xbf, 0x65, 0x1d // It vs; add vs r5 r4 5
};
const byte kInstruction_add_vc_r6_r4_2[] = {
  0x78, 0xbf, 0xa6, 0x1c // It vc; add vc r6 r4 2
};
const byte kInstruction_add_ne_r4_r2_2[] = {
  0x18, 0xbf, 0x94, 0x1c // It ne; add ne r4 r2 2
};
const byte kInstruction_add_ge_r5_r7_7[] = {
  0xa8, 0xbf, 0xfd, 0x1d // It ge; add ge r5 r7 7
};
const byte kInstruction_add_ne_r1_r7_7[] = {
  0x18, 0xbf, 0xf9, 0x1d // It ne; add ne r1 r7 7
};
const byte kInstruction_add_vc_r6_r1_2[] = {
  0x78, 0xbf, 0x8e, 0x1c // It vc; add vc r6 r1 2
};
const byte kInstruction_add_mi_r2_r4_0[] = {
  0x48, 0xbf, 0x22, 0x1c // It mi; add mi r2 r4 0
};
const byte kInstruction_add_hi_r6_r3_1[] = {
  0x88, 0xbf, 0x5e, 0x1c // It hi; add hi r6 r3 1
};
const byte kInstruction_add_cc_r5_r1_5[] = {
  0x38, 0xbf, 0x4d, 0x1d // It cc; add cc r5 r1 5
};
const byte kInstruction_add_mi_r3_r5_1[] = {
  0x48, 0xbf, 0x6b, 0x1c // It mi; add mi r3 r5 1
};
const byte kInstruction_add_vs_r0_r5_3[] = {
  0x68, 0xbf, 0xe8, 0x1c // It vs; add vs r0 r5 3
};
const byte kInstruction_add_gt_r0_r1_4[] = {
  0xc8, 0xbf, 0x08, 0x1d // It gt; add gt r0 r1 4
};
const byte kInstruction_add_vs_r7_r5_4[] = {
  0x68, 0xbf, 0x2f, 0x1d // It vs; add vs r7 r5 4
};
const byte kInstruction_add_mi_r1_r5_3[] = {
  0x48, 0xbf, 0xe9, 0x1c // It mi; add mi r1 r5 3
};
const byte kInstruction_add_hi_r6_r6_0[] = {
  0x88, 0xbf, 0x36, 0x1c // It hi; add hi r6 r6 0
};
const byte kInstruction_add_gt_r6_r4_3[] = {
  0xc8, 0xbf, 0xe6, 0x1c // It gt; add gt r6 r4 3
};
const byte kInstruction_add_gt_r3_r2_6[] = {
  0xc8, 0xbf, 0x93, 0x1d // It gt; add gt r3 r2 6
};
const byte kInstruction_add_mi_r3_r7_7[] = {
  0x48, 0xbf, 0xfb, 0x1d // It mi; add mi r3 r7 7
};
const byte kInstruction_add_gt_r4_r6_0[] = {
  0xc8, 0xbf, 0x34, 0x1c // It gt; add gt r4 r6 0
};
const byte kInstruction_add_cs_r6_r4_0[] = {
  0x28, 0xbf, 0x26, 0x1c // It cs; add cs r6 r4 0
};
const byte kInstruction_add_ne_r6_r2_0[] = {
  0x18, 0xbf, 0x16, 0x1c // It ne; add ne r6 r2 0
};
const byte kInstruction_add_ne_r3_r3_0[] = {
  0x18, 0xbf, 0x1b, 0x1c // It ne; add ne r3 r3 0
};
const byte kInstruction_add_gt_r2_r3_5[] = {
  0xc8, 0xbf, 0x5a, 0x1d // It gt; add gt r2 r3 5
};
const byte kInstruction_add_ne_r5_r0_5[] = {
  0x18, 0xbf, 0x45, 0x1d // It ne; add ne r5 r0 5
};
const byte kInstruction_add_le_r7_r7_4[] = {
  0xd8, 0xbf, 0x3f, 0x1d // It le; add le r7 r7 4
};
const byte kInstruction_add_ge_r7_r3_0[] = {
  0xa8, 0xbf, 0x1f, 0x1c // It ge; add ge r7 r3 0
};
const byte kInstruction_add_le_r4_r5_6[] = {
  0xd8, 0xbf, 0xac, 0x1d // It le; add le r4 r5 6
};
const byte kInstruction_add_ls_r3_r2_6[] = {
  0x98, 0xbf, 0x93, 0x1d // It ls; add ls r3 r2 6
};
const byte kInstruction_add_gt_r4_r5_7[] = {
  0xc8, 0xbf, 0xec, 0x1d // It gt; add gt r4 r5 7
};
const byte kInstruction_add_eq_r2_r1_3[] = {
  0x08, 0xbf, 0xca, 0x1c // It eq; add eq r2 r1 3
};
const byte kInstruction_add_lt_r3_r0_6[] = {
  0xb8, 0xbf, 0x83, 0x1d // It lt; add lt r3 r0 6
};
const byte kInstruction_add_ne_r4_r0_2[] = {
  0x18, 0xbf, 0x84, 0x1c // It ne; add ne r4 r0 2
};
const byte kInstruction_add_lt_r7_r6_5[] = {
  0xb8, 0xbf, 0x77, 0x1d // It lt; add lt r7 r6 5
};
const byte kInstruction_add_hi_r6_r0_6[] = {
  0x88, 0xbf, 0x86, 0x1d // It hi; add hi r6 r0 6
};
const byte kInstruction_add_vs_r1_r6_7[] = {
  0x68, 0xbf, 0xf1, 0x1d // It vs; add vs r1 r6 7
};
const byte kInstruction_add_cc_r0_r6_6[] = {
  0x38, 0xbf, 0xb0, 0x1d // It cc; add cc r0 r6 6
};
const byte kInstruction_add_ne_r1_r7_6[] = {
  0x18, 0xbf, 0xb9, 0x1d // It ne; add ne r1 r7 6
};
const byte kInstruction_add_cs_r2_r3_1[] = {
  0x28, 0xbf, 0x5a, 0x1c // It cs; add cs r2 r3 1
};
const byte kInstruction_add_lt_r3_r7_7[] = {
  0xb8, 0xbf, 0xfb, 0x1d // It lt; add lt r3 r7 7
};
const byte kInstruction_add_cc_r3_r3_0[] = {
  0x38, 0xbf, 0x1b, 0x1c // It cc; add cc r3 r3 0
};
const byte kInstruction_add_eq_r4_r3_6[] = {
  0x08, 0xbf, 0x9c, 0x1d // It eq; add eq r4 r3 6
};
const byte kInstruction_add_cs_r0_r5_7[] = {
  0x28, 0xbf, 0xe8, 0x1d // It cs; add cs r0 r5 7
};
const byte kInstruction_add_gt_r1_r2_4[] = {
  0xc8, 0xbf, 0x11, 0x1d // It gt; add gt r1 r2 4
};
const byte kInstruction_add_cc_r2_r1_5[] = {
  0x38, 0xbf, 0x4a, 0x1d // It cc; add cc r2 r1 5
};
const byte kInstruction_add_pl_r7_r5_5[] = {
  0x58, 0xbf, 0x6f, 0x1d // It pl; add pl r7 r5 5
};
const byte kInstruction_add_pl_r5_r7_6[] = {
  0x58, 0xbf, 0xbd, 0x1d // It pl; add pl r5 r7 6
};
const byte kInstruction_add_vc_r2_r2_7[] = {
  0x78, 0xbf, 0xd2, 0x1d // It vc; add vc r2 r2 7
};
const byte kInstruction_add_vs_r4_r4_0[] = {
  0x68, 0xbf, 0x24, 0x1c // It vs; add vs r4 r4 0
};
const byte kInstruction_add_vc_r2_r6_5[] = {
  0x78, 0xbf, 0x72, 0x1d // It vc; add vc r2 r6 5
};
const byte kInstruction_add_gt_r1_r7_0[] = {
  0xc8, 0xbf, 0x39, 0x1c // It gt; add gt r1 r7 0
};
const byte kInstruction_add_pl_r4_r1_7[] = {
  0x58, 0xbf, 0xcc, 0x1d // It pl; add pl r4 r1 7
};
const byte kInstruction_add_lt_r4_r3_2[] = {
  0xb8, 0xbf, 0x9c, 0x1c // It lt; add lt r4 r3 2
};
const byte kInstruction_add_ge_r0_r2_4[] = {
  0xa8, 0xbf, 0x10, 0x1d // It ge; add ge r0 r2 4
};
const byte kInstruction_add_ne_r7_r5_7[] = {
  0x18, 0xbf, 0xef, 0x1d // It ne; add ne r7 r5 7
};
const byte kInstruction_add_gt_r2_r1_6[] = {
  0xc8, 0xbf, 0x8a, 0x1d // It gt; add gt r2 r1 6
};
const byte kInstruction_add_eq_r4_r1_6[] = {
  0x08, 0xbf, 0x8c, 0x1d // It eq; add eq r4 r1 6
};
const byte kInstruction_add_vs_r3_r7_3[] = {
  0x68, 0xbf, 0xfb, 0x1c // It vs; add vs r3 r7 3
};
const byte kInstruction_add_mi_r0_r7_3[] = {
  0x48, 0xbf, 0xf8, 0x1c // It mi; add mi r0 r7 3
};
const byte kInstruction_add_eq_r3_r5_6[] = {
  0x08, 0xbf, 0xab, 0x1d // It eq; add eq r3 r5 6
};
const byte kInstruction_add_lt_r6_r2_3[] = {
  0xb8, 0xbf, 0xd6, 0x1c // It lt; add lt r6 r2 3
};
const byte kInstruction_add_lt_r7_r5_5[] = {
  0xb8, 0xbf, 0x6f, 0x1d // It lt; add lt r7 r5 5
};
const byte kInstruction_add_vc_r6_r5_6[] = {
  0x78, 0xbf, 0xae, 0x1d // It vc; add vc r6 r5 6
};
const byte kInstruction_add_cs_r2_r0_7[] = {
  0x28, 0xbf, 0xc2, 0x1d // It cs; add cs r2 r0 7
};
const byte kInstruction_add_hi_r4_r2_3[] = {
  0x88, 0xbf, 0xd4, 0x1c // It hi; add hi r4 r2 3
};
const byte kInstruction_add_vs_r5_r4_7[] = {
  0x68, 0xbf, 0xe5, 0x1d // It vs; add vs r5 r4 7
};
const byte kInstruction_add_ge_r7_r1_2[] = {
  0xa8, 0xbf, 0x8f, 0x1c // It ge; add ge r7 r1 2
};
const byte kInstruction_add_cc_r6_r4_6[] = {
  0x38, 0xbf, 0xa6, 0x1d // It cc; add cc r6 r4 6
};
const byte kInstruction_add_ls_r4_r1_0[] = {
  0x98, 0xbf, 0x0c, 0x1c // It ls; add ls r4 r1 0
};
const byte kInstruction_add_cc_r1_r1_1[] = {
  0x38, 0xbf, 0x49, 0x1c // It cc; add cc r1 r1 1
};
const byte kInstruction_add_ls_r2_r3_5[] = {
  0x98, 0xbf, 0x5a, 0x1d // It ls; add ls r2 r3 5
};
const byte kInstruction_add_cc_r2_r0_6[] = {
  0x38, 0xbf, 0x82, 0x1d // It cc; add cc r2 r0 6
};
const byte kInstruction_add_gt_r6_r5_4[] = {
  0xc8, 0xbf, 0x2e, 0x1d // It gt; add gt r6 r5 4
};
const byte kInstruction_add_le_r3_r3_3[] = {
  0xd8, 0xbf, 0xdb, 0x1c // It le; add le r3 r3 3
};
const byte kInstruction_add_ls_r4_r5_7[] = {
  0x98, 0xbf, 0xec, 0x1d // It ls; add ls r4 r5 7
};
const byte kInstruction_add_ls_r3_r7_0[] = {
  0x98, 0xbf, 0x3b, 0x1c // It ls; add ls r3 r7 0
};
const byte kInstruction_add_lt_r5_r4_1[] = {
  0xb8, 0xbf, 0x65, 0x1c // It lt; add lt r5 r4 1
};
const byte kInstruction_add_hi_r0_r2_3[] = {
  0x88, 0xbf, 0xd0, 0x1c // It hi; add hi r0 r2 3
};
const byte kInstruction_add_ne_r7_r2_2[] = {
  0x18, 0xbf, 0x97, 0x1c // It ne; add ne r7 r2 2
};
const byte kInstruction_add_gt_r6_r4_4[] = {
  0xc8, 0xbf, 0x26, 0x1d // It gt; add gt r6 r4 4
};
const byte kInstruction_add_cc_r5_r1_0[] = {
  0x38, 0xbf, 0x0d, 0x1c // It cc; add cc r5 r1 0
};
const byte kInstruction_add_le_r6_r5_3[] = {
  0xd8, 0xbf, 0xee, 0x1c // It le; add le r6 r5 3
};
const byte kInstruction_add_cs_r3_r4_4[] = {
  0x28, 0xbf, 0x23, 0x1d // It cs; add cs r3 r4 4
};
const byte kInstruction_add_mi_r4_r7_5[] = {
  0x48, 0xbf, 0x7c, 0x1d // It mi; add mi r4 r7 5
};
const byte kInstruction_add_mi_r1_r3_6[] = {
  0x48, 0xbf, 0x99, 0x1d // It mi; add mi r1 r3 6
};
const byte kInstruction_add_hi_r0_r7_5[] = {
  0x88, 0xbf, 0x78, 0x1d // It hi; add hi r0 r7 5
};
const byte kInstruction_add_ls_r4_r3_2[] = {
  0x98, 0xbf, 0x9c, 0x1c // It ls; add ls r4 r3 2
};
const byte kInstruction_add_ne_r4_r7_6[] = {
  0x18, 0xbf, 0xbc, 0x1d // It ne; add ne r4 r7 6
};
const byte kInstruction_add_cs_r4_r5_5[] = {
  0x28, 0xbf, 0x6c, 0x1d // It cs; add cs r4 r5 5
};
const byte kInstruction_add_mi_r3_r5_0[] = {
  0x48, 0xbf, 0x2b, 0x1c // It mi; add mi r3 r5 0
};
const byte kInstruction_add_cs_r5_r0_1[] = {
  0x28, 0xbf, 0x45, 0x1c // It cs; add cs r5 r0 1
};
const byte kInstruction_add_pl_r3_r2_6[] = {
  0x58, 0xbf, 0x93, 0x1d // It pl; add pl r3 r2 6
};
const byte kInstruction_add_vs_r1_r6_5[] = {
  0x68, 0xbf, 0x71, 0x1d // It vs; add vs r1 r6 5
};
const byte kInstruction_add_le_r4_r0_2[] = {
  0xd8, 0xbf, 0x84, 0x1c // It le; add le r4 r0 2
};
const byte kInstruction_add_eq_r3_r7_7[] = {
  0x08, 0xbf, 0xfb, 0x1d // It eq; add eq r3 r7 7
};
const byte kInstruction_add_lt_r6_r4_7[] = {
  0xb8, 0xbf, 0xe6, 0x1d // It lt; add lt r6 r4 7
};
const byte kInstruction_add_hi_r2_r0_5[] = {
  0x88, 0xbf, 0x42, 0x1d // It hi; add hi r2 r0 5
};
const byte kInstruction_add_ge_r1_r3_3[] = {
  0xa8, 0xbf, 0xd9, 0x1c // It ge; add ge r1 r3 3
};
const byte kInstruction_add_pl_r6_r3_0[] = {
  0x58, 0xbf, 0x1e, 0x1c // It pl; add pl r6 r3 0
};
const byte kInstruction_add_hi_r1_r0_4[] = {
  0x88, 0xbf, 0x01, 0x1d // It hi; add hi r1 r0 4
};
const byte kInstruction_add_lt_r3_r5_3[] = {
  0xb8, 0xbf, 0xeb, 0x1c // It lt; add lt r3 r5 3
};
const byte kInstruction_add_ge_r2_r7_3[] = {
  0xa8, 0xbf, 0xfa, 0x1c // It ge; add ge r2 r7 3
};
const byte kInstruction_add_mi_r0_r7_4[] = {
  0x48, 0xbf, 0x38, 0x1d // It mi; add mi r0 r7 4
};
const byte kInstruction_add_pl_r7_r5_1[] = {
  0x58, 0xbf, 0x6f, 0x1c // It pl; add pl r7 r5 1
};
const byte kInstruction_add_lt_r0_r3_4[] = {
  0xb8, 0xbf, 0x18, 0x1d // It lt; add lt r0 r3 4
};
const byte kInstruction_add_mi_r6_r4_2[] = {
  0x48, 0xbf, 0xa6, 0x1c // It mi; add mi r6 r4 2
};
const byte kInstruction_add_vc_r4_r4_1[] = {
  0x78, 0xbf, 0x64, 0x1c // It vc; add vc r4 r4 1
};
const byte kInstruction_add_gt_r1_r5_3[] = {
  0xc8, 0xbf, 0xe9, 0x1c // It gt; add gt r1 r5 3
};
const byte kInstruction_add_cc_r2_r0_7[] = {
  0x38, 0xbf, 0xc2, 0x1d // It cc; add cc r2 r0 7
};
const byte kInstruction_add_le_r6_r4_0[] = {
  0xd8, 0xbf, 0x26, 0x1c // It le; add le r6 r4 0
};
const byte kInstruction_add_le_r0_r6_2[] = {
  0xd8, 0xbf, 0xb0, 0x1c // It le; add le r0 r6 2
};
const byte kInstruction_add_le_r1_r1_7[] = {
  0xd8, 0xbf, 0xc9, 0x1d // It le; add le r1 r1 7
};
const byte kInstruction_add_vs_r1_r1_5[] = {
  0x68, 0xbf, 0x49, 0x1d // It vs; add vs r1 r1 5
};
const byte kInstruction_add_cs_r6_r0_2[] = {
  0x28, 0xbf, 0x86, 0x1c // It cs; add cs r6 r0 2
};
const byte kInstruction_add_vs_r3_r0_7[] = {
  0x68, 0xbf, 0xc3, 0x1d // It vs; add vs r3 r0 7
};
const byte kInstruction_add_vs_r3_r3_4[] = {
  0x68, 0xbf, 0x1b, 0x1d // It vs; add vs r3 r3 4
};
const byte kInstruction_add_cs_r4_r2_7[] = {
  0x28, 0xbf, 0xd4, 0x1d // It cs; add cs r4 r2 7
};
const byte kInstruction_add_mi_r4_r0_1[] = {
  0x48, 0xbf, 0x44, 0x1c // It mi; add mi r4 r0 1
};
const byte kInstruction_add_gt_r2_r4_1[] = {
  0xc8, 0xbf, 0x62, 0x1c // It gt; add gt r2 r4 1
};
const byte kInstruction_add_ne_r4_r6_0[] = {
  0x18, 0xbf, 0x34, 0x1c // It ne; add ne r4 r6 0
};
const byte kInstruction_add_cs_r4_r1_5[] = {
  0x28, 0xbf, 0x4c, 0x1d // It cs; add cs r4 r1 5
};
const byte kInstruction_add_cc_r2_r6_3[] = {
  0x38, 0xbf, 0xf2, 0x1c // It cc; add cc r2 r6 3
};
const byte kInstruction_add_ge_r6_r1_0[] = {
  0xa8, 0xbf, 0x0e, 0x1c // It ge; add ge r6 r1 0
};
const byte kInstruction_add_pl_r2_r5_3[] = {
  0x58, 0xbf, 0xea, 0x1c // It pl; add pl r2 r5 3
};
const byte kInstruction_add_ne_r1_r7_4[] = {
  0x18, 0xbf, 0x39, 0x1d // It ne; add ne r1 r7 4
};
const byte kInstruction_add_le_r2_r6_0[] = {
  0xd8, 0xbf, 0x32, 0x1c // It le; add le r2 r6 0
};
const byte kInstruction_add_gt_r7_r1_6[] = {
  0xc8, 0xbf, 0x8f, 0x1d // It gt; add gt r7 r1 6
};
const byte kInstruction_add_cs_r0_r3_7[] = {
  0x28, 0xbf, 0xd8, 0x1d // It cs; add cs r0 r3 7
};
const byte kInstruction_add_mi_r1_r1_0[] = {
  0x48, 0xbf, 0x09, 0x1c // It mi; add mi r1 r1 0
};
const byte kInstruction_add_eq_r4_r0_5[] = {
  0x08, 0xbf, 0x44, 0x1d // It eq; add eq r4 r0 5
};
const byte kInstruction_add_lt_r5_r0_6[] = {
  0xb8, 0xbf, 0x85, 0x1d // It lt; add lt r5 r0 6
};
const byte kInstruction_add_vc_r2_r6_6[] = {
  0x78, 0xbf, 0xb2, 0x1d // It vc; add vc r2 r6 6
};
const byte kInstruction_add_cs_r1_r2_7[] = {
  0x28, 0xbf, 0xd1, 0x1d // It cs; add cs r1 r2 7
};
const byte kInstruction_add_ls_r6_r6_3[] = {
  0x98, 0xbf, 0xf6, 0x1c // It ls; add ls r6 r6 3
};
const byte kInstruction_add_lt_r5_r0_4[] = {
  0xb8, 0xbf, 0x05, 0x1d // It lt; add lt r5 r0 4
};
const byte kInstruction_add_gt_r1_r6_0[] = {
  0xc8, 0xbf, 0x31, 0x1c // It gt; add gt r1 r6 0
};
const byte kInstruction_add_vs_r7_r2_2[] = {
  0x68, 0xbf, 0x97, 0x1c // It vs; add vs r7 r2 2
};
const byte kInstruction_add_gt_r1_r3_5[] = {
  0xc8, 0xbf, 0x59, 0x1d // It gt; add gt r1 r3 5
};
const byte kInstruction_add_mi_r2_r6_5[] = {
  0x48, 0xbf, 0x72, 0x1d // It mi; add mi r2 r6 5
};
const byte kInstruction_add_cs_r3_r6_2[] = {
  0x28, 0xbf, 0xb3, 0x1c // It cs; add cs r3 r6 2
};
const byte kInstruction_add_cs_r2_r6_2[] = {
  0x28, 0xbf, 0xb2, 0x1c // It cs; add cs r2 r6 2
};
const byte kInstruction_add_cs_r7_r3_6[] = {
  0x28, 0xbf, 0x9f, 0x1d // It cs; add cs r7 r3 6
};
const byte kInstruction_add_cs_r7_r7_6[] = {
  0x28, 0xbf, 0xbf, 0x1d // It cs; add cs r7 r7 6
};
const byte kInstruction_add_eq_r1_r1_4[] = {
  0x08, 0xbf, 0x09, 0x1d // It eq; add eq r1 r1 4
};
const byte kInstruction_add_mi_r3_r2_2[] = {
  0x48, 0xbf, 0x93, 0x1c // It mi; add mi r3 r2 2
};
const byte kInstruction_add_vc_r5_r7_0[] = {
  0x78, 0xbf, 0x3d, 0x1c // It vc; add vc r5 r7 0
};
const byte kInstruction_add_hi_r1_r6_7[] = {
  0x88, 0xbf, 0xf1, 0x1d // It hi; add hi r1 r6 7
};
const byte kInstruction_add_vs_r5_r7_3[] = {
  0x68, 0xbf, 0xfd, 0x1c // It vs; add vs r5 r7 3
};
const byte kInstruction_add_gt_r1_r7_2[] = {
  0xc8, 0xbf, 0xb9, 0x1c // It gt; add gt r1 r7 2
};
const byte kInstruction_add_vc_r4_r5_0[] = {
  0x78, 0xbf, 0x2c, 0x1c // It vc; add vc r4 r5 0
};
const byte kInstruction_add_le_r6_r7_0[] = {
  0xd8, 0xbf, 0x3e, 0x1c // It le; add le r6 r7 0
};
const byte kInstruction_add_ge_r6_r1_5[] = {
  0xa8, 0xbf, 0x4e, 0x1d // It ge; add ge r6 r1 5
};
const byte kInstruction_add_cc_r0_r2_1[] = {
  0x38, 0xbf, 0x50, 0x1c // It cc; add cc r0 r2 1
};
const byte kInstruction_add_le_r7_r3_5[] = {
  0xd8, 0xbf, 0x5f, 0x1d // It le; add le r7 r3 5
};
const byte kInstruction_add_hi_r7_r1_2[] = {
  0x88, 0xbf, 0x8f, 0x1c // It hi; add hi r7 r1 2
};
const byte kInstruction_add_cc_r7_r5_0[] = {
  0x38, 0xbf, 0x2f, 0x1c // It cc; add cc r7 r5 0
};
const byte kInstruction_add_gt_r0_r6_0[] = {
  0xc8, 0xbf, 0x30, 0x1c // It gt; add gt r0 r6 0
};
const byte kInstruction_add_ne_r6_r5_4[] = {
  0x18, 0xbf, 0x2e, 0x1d // It ne; add ne r6 r5 4
};
const byte kInstruction_add_cc_r7_r0_1[] = {
  0x38, 0xbf, 0x47, 0x1c // It cc; add cc r7 r0 1
};
const byte kInstruction_add_eq_r6_r6_5[] = {
  0x08, 0xbf, 0x76, 0x1d // It eq; add eq r6 r6 5
};
const byte kInstruction_add_pl_r4_r3_0[] = {
  0x58, 0xbf, 0x1c, 0x1c // It pl; add pl r4 r3 0
};
const byte kInstruction_add_ne_r2_r3_6[] = {
  0x18, 0xbf, 0x9a, 0x1d // It ne; add ne r2 r3 6
};
const byte kInstruction_add_le_r0_r2_1[] = {
  0xd8, 0xbf, 0x50, 0x1c // It le; add le r0 r2 1
};
const byte kInstruction_add_eq_r0_r1_5[] = {
  0x08, 0xbf, 0x48, 0x1d // It eq; add eq r0 r1 5
};
const byte kInstruction_add_ls_r5_r6_2[] = {
  0x98, 0xbf, 0xb5, 0x1c // It ls; add ls r5 r6 2
};
const byte kInstruction_add_ne_r3_r4_5[] = {
  0x18, 0xbf, 0x63, 0x1d // It ne; add ne r3 r4 5
};
const byte kInstruction_add_gt_r1_r4_2[] = {
  0xc8, 0xbf, 0xa1, 0x1c // It gt; add gt r1 r4 2
};
const byte kInstruction_add_le_r3_r5_2[] = {
  0xd8, 0xbf, 0xab, 0x1c // It le; add le r3 r5 2
};
const byte kInstruction_add_vc_r0_r5_5[] = {
  0x78, 0xbf, 0x68, 0x1d // It vc; add vc r0 r5 5
};
const byte kInstruction_add_pl_r5_r3_6[] = {
  0x58, 0xbf, 0x9d, 0x1d // It pl; add pl r5 r3 6
};
const byte kInstruction_add_eq_r0_r1_4[] = {
  0x08, 0xbf, 0x08, 0x1d // It eq; add eq r0 r1 4
};
const byte kInstruction_add_cs_r0_r0_7[] = {
  0x28, 0xbf, 0xc0, 0x1d // It cs; add cs r0 r0 7
};
const byte kInstruction_add_lt_r6_r5_6[] = {
  0xb8, 0xbf, 0xae, 0x1d // It lt; add lt r6 r5 6
};
const byte kInstruction_add_vs_r1_r1_2[] = {
  0x68, 0xbf, 0x89, 0x1c // It vs; add vs r1 r1 2
};
const byte kInstruction_add_mi_r4_r5_0[] = {
  0x48, 0xbf, 0x2c, 0x1c // It mi; add mi r4 r5 0
};
const byte kInstruction_add_vc_r3_r0_4[] = {
  0x78, 0xbf, 0x03, 0x1d // It vc; add vc r3 r0 4
};
const byte kInstruction_add_le_r2_r2_3[] = {
  0xd8, 0xbf, 0xd2, 0x1c // It le; add le r2 r2 3
};
const byte kInstruction_add_gt_r3_r7_5[] = {
  0xc8, 0xbf, 0x7b, 0x1d // It gt; add gt r3 r7 5
};
const byte kInstruction_add_cs_r0_r4_0[] = {
  0x28, 0xbf, 0x20, 0x1c // It cs; add cs r0 r4 0
};
const byte kInstruction_add_gt_r5_r0_2[] = {
  0xc8, 0xbf, 0x85, 0x1c // It gt; add gt r5 r0 2
};
const byte kInstruction_add_vs_r4_r1_1[] = {
  0x68, 0xbf, 0x4c, 0x1c // It vs; add vs r4 r1 1
};
const byte kInstruction_add_cc_r7_r3_5[] = {
  0x38, 0xbf, 0x5f, 0x1d // It cc; add cc r7 r3 5
};
const byte kInstruction_add_ls_r5_r0_3[] = {
  0x98, 0xbf, 0xc5, 0x1c // It ls; add ls r5 r0 3
};
const byte kInstruction_add_vs_r2_r4_3[] = {
  0x68, 0xbf, 0xe2, 0x1c // It vs; add vs r2 r4 3
};
const byte kInstruction_add_vc_r3_r3_4[] = {
  0x78, 0xbf, 0x1b, 0x1d // It vc; add vc r3 r3 4
};
const byte kInstruction_add_gt_r0_r6_1[] = {
  0xc8, 0xbf, 0x70, 0x1c // It gt; add gt r0 r6 1
};
const byte kInstruction_add_hi_r1_r7_1[] = {
  0x88, 0xbf, 0x79, 0x1c // It hi; add hi r1 r7 1
};
const byte kInstruction_add_ne_r2_r4_5[] = {
  0x18, 0xbf, 0x62, 0x1d // It ne; add ne r2 r4 5
};
const byte kInstruction_add_hi_r0_r7_0[] = {
  0x88, 0xbf, 0x38, 0x1c // It hi; add hi r0 r7 0
};
const byte kInstruction_add_ne_r1_r1_5[] = {
  0x18, 0xbf, 0x49, 0x1d // It ne; add ne r1 r1 5
};
const byte kInstruction_add_cc_r1_r0_2[] = {
  0x38, 0xbf, 0x81, 0x1c // It cc; add cc r1 r0 2
};
const byte kInstruction_add_cc_r7_r3_3[] = {
  0x38, 0xbf, 0xdf, 0x1c // It cc; add cc r7 r3 3
};
const byte kInstruction_add_lt_r0_r0_3[] = {
  0xb8, 0xbf, 0xc0, 0x1c // It lt; add lt r0 r0 3
};
const byte kInstruction_add_hi_r1_r1_1[] = {
  0x88, 0xbf, 0x49, 0x1c // It hi; add hi r1 r1 1
};
const byte kInstruction_add_vs_r7_r2_4[] = {
  0x68, 0xbf, 0x17, 0x1d // It vs; add vs r7 r2 4
};
const byte kInstruction_add_cs_r2_r1_3[] = {
  0x28, 0xbf, 0xca, 0x1c // It cs; add cs r2 r1 3
};
const byte kInstruction_add_ge_r4_r0_7[] = {
  0xa8, 0xbf, 0xc4, 0x1d // It ge; add ge r4 r0 7
};
const byte kInstruction_add_gt_r0_r2_2[] = {
  0xc8, 0xbf, 0x90, 0x1c // It gt; add gt r0 r2 2
};
const byte kInstruction_add_lt_r6_r4_4[] = {
  0xb8, 0xbf, 0x26, 0x1d // It lt; add lt r6 r4 4
};
const byte kInstruction_add_lt_r2_r4_1[] = {
  0xb8, 0xbf, 0x62, 0x1c // It lt; add lt r2 r4 1
};
const byte kInstruction_add_cc_r5_r5_3[] = {
  0x38, 0xbf, 0xed, 0x1c // It cc; add cc r5 r5 3
};
const byte kInstruction_add_eq_r7_r3_0[] = {
  0x08, 0xbf, 0x1f, 0x1c // It eq; add eq r7 r3 0
};
const byte kInstruction_add_cc_r0_r2_2[] = {
  0x38, 0xbf, 0x90, 0x1c // It cc; add cc r0 r2 2
};
const byte kInstruction_add_mi_r5_r7_7[] = {
  0x48, 0xbf, 0xfd, 0x1d // It mi; add mi r5 r7 7
};
const byte kInstruction_add_eq_r2_r4_5[] = {
  0x08, 0xbf, 0x62, 0x1d // It eq; add eq r2 r4 5
};
const byte kInstruction_add_pl_r1_r7_0[] = {
  0x58, 0xbf, 0x39, 0x1c // It pl; add pl r1 r7 0
};
const byte kInstruction_add_vs_r6_r2_0[] = {
  0x68, 0xbf, 0x16, 0x1c // It vs; add vs r6 r2 0
};
const byte kInstruction_add_cc_r0_r1_6[] = {
  0x38, 0xbf, 0x88, 0x1d // It cc; add cc r0 r1 6
};
const byte kInstruction_add_cs_r2_r2_5[] = {
  0x28, 0xbf, 0x52, 0x1d // It cs; add cs r2 r2 5
};
const byte kInstruction_add_le_r0_r6_7[] = {
  0xd8, 0xbf, 0xf0, 0x1d // It le; add le r0 r6 7
};
const byte kInstruction_add_hi_r5_r0_7[] = {
  0x88, 0xbf, 0xc5, 0x1d // It hi; add hi r5 r0 7
};
const byte kInstruction_add_gt_r2_r5_1[] = {
  0xc8, 0xbf, 0x6a, 0x1c // It gt; add gt r2 r5 1
};
const byte kInstruction_add_gt_r2_r1_2[] = {
  0xc8, 0xbf, 0x8a, 0x1c // It gt; add gt r2 r1 2
};
const byte kInstruction_add_le_r6_r2_5[] = {
  0xd8, 0xbf, 0x56, 0x1d // It le; add le r6 r2 5
};
const byte kInstruction_add_ls_r4_r3_3[] = {
  0x98, 0xbf, 0xdc, 0x1c // It ls; add ls r4 r3 3
};
const byte kInstruction_add_le_r0_r2_6[] = {
  0xd8, 0xbf, 0x90, 0x1d // It le; add le r0 r2 6
};
const byte kInstruction_add_pl_r0_r7_6[] = {
  0x58, 0xbf, 0xb8, 0x1d // It pl; add pl r0 r7 6
};
const byte kInstruction_add_pl_r4_r5_4[] = {
  0x58, 0xbf, 0x2c, 0x1d // It pl; add pl r4 r5 4
};
const byte kInstruction_add_vs_r3_r2_2[] = {
  0x68, 0xbf, 0x93, 0x1c // It vs; add vs r3 r2 2
};
const byte kInstruction_add_vc_r2_r4_3[] = {
  0x78, 0xbf, 0xe2, 0x1c // It vc; add vc r2 r4 3
};
const byte kInstruction_add_ge_r1_r5_7[] = {
  0xa8, 0xbf, 0xe9, 0x1d // It ge; add ge r1 r5 7
};
const byte kInstruction_add_pl_r1_r6_3[] = {
  0x58, 0xbf, 0xf1, 0x1c // It pl; add pl r1 r6 3
};
const byte kInstruction_add_hi_r3_r6_0[] = {
  0x88, 0xbf, 0x33, 0x1c // It hi; add hi r3 r6 0
};
const byte kInstruction_add_ne_r0_r4_3[] = {
  0x18, 0xbf, 0xe0, 0x1c // It ne; add ne r0 r4 3
};
const byte kInstruction_add_ne_r4_r5_0[] = {
  0x18, 0xbf, 0x2c, 0x1c // It ne; add ne r4 r5 0
};
const byte kInstruction_add_cc_r7_r6_2[] = {
  0x38, 0xbf, 0xb7, 0x1c // It cc; add cc r7 r6 2
};
const byte kInstruction_add_cc_r2_r0_5[] = {
  0x38, 0xbf, 0x42, 0x1d // It cc; add cc r2 r0 5
};
const byte kInstruction_add_ge_r3_r1_2[] = {
  0xa8, 0xbf, 0x8b, 0x1c // It ge; add ge r3 r1 2
};
const byte kInstruction_add_ge_r1_r1_2[] = {
  0xa8, 0xbf, 0x89, 0x1c // It ge; add ge r1 r1 2
};
const byte kInstruction_add_hi_r5_r6_4[] = {
  0x88, 0xbf, 0x35, 0x1d // It hi; add hi r5 r6 4
};
const byte kInstruction_add_ne_r6_r4_3[] = {
  0x18, 0xbf, 0xe6, 0x1c // It ne; add ne r6 r4 3
};
const byte kInstruction_add_ne_r3_r1_0[] = {
  0x18, 0xbf, 0x0b, 0x1c // It ne; add ne r3 r1 0
};
const byte kInstruction_add_cs_r3_r4_0[] = {
  0x28, 0xbf, 0x23, 0x1c // It cs; add cs r3 r4 0
};
const byte kInstruction_add_eq_r5_r0_6[] = {
  0x08, 0xbf, 0x85, 0x1d // It eq; add eq r5 r0 6
};
const byte kInstruction_add_pl_r1_r5_1[] = {
  0x58, 0xbf, 0x69, 0x1c // It pl; add pl r1 r5 1
};
const byte kInstruction_add_ls_r7_r7_4[] = {
  0x98, 0xbf, 0x3f, 0x1d // It ls; add ls r7 r7 4
};
const byte kInstruction_add_le_r2_r2_2[] = {
  0xd8, 0xbf, 0x92, 0x1c // It le; add le r2 r2 2
};
const byte kInstruction_add_eq_r6_r4_5[] = {
  0x08, 0xbf, 0x66, 0x1d // It eq; add eq r6 r4 5
};
const byte kInstruction_add_cc_r1_r3_0[] = {
  0x38, 0xbf, 0x19, 0x1c // It cc; add cc r1 r3 0
};
const byte kInstruction_add_gt_r3_r5_5[] = {
  0xc8, 0xbf, 0x6b, 0x1d // It gt; add gt r3 r5 5
};
const byte kInstruction_add_ge_r7_r2_2[] = {
  0xa8, 0xbf, 0x97, 0x1c // It ge; add ge r7 r2 2
};
const byte kInstruction_add_gt_r2_r7_4[] = {
  0xc8, 0xbf, 0x3a, 0x1d // It gt; add gt r2 r7 4
};
const byte kInstruction_add_vs_r7_r3_3[] = {
  0x68, 0xbf, 0xdf, 0x1c // It vs; add vs r7 r3 3
};
const byte kInstruction_add_ge_r2_r5_4[] = {
  0xa8, 0xbf, 0x2a, 0x1d // It ge; add ge r2 r5 4
};
const byte kInstruction_add_ge_r0_r3_0[] = {
  0xa8, 0xbf, 0x18, 0x1c // It ge; add ge r0 r3 0
};
const byte kInstruction_add_hi_r3_r1_4[] = {
  0x88, 0xbf, 0x0b, 0x1d // It hi; add hi r3 r1 4
};
const byte kInstruction_add_pl_r7_r7_2[] = {
  0x58, 0xbf, 0xbf, 0x1c // It pl; add pl r7 r7 2
};
const byte kInstruction_add_pl_r6_r0_0[] = {
  0x58, 0xbf, 0x06, 0x1c // It pl; add pl r6 r0 0
};
const byte kInstruction_add_ls_r0_r6_4[] = {
  0x98, 0xbf, 0x30, 0x1d // It ls; add ls r0 r6 4
};
const byte kInstruction_add_pl_r3_r4_0[] = {
  0x58, 0xbf, 0x23, 0x1c // It pl; add pl r3 r4 0
};
const byte kInstruction_add_ge_r5_r3_5[] = {
  0xa8, 0xbf, 0x5d, 0x1d // It ge; add ge r5 r3 5
};
const byte kInstruction_add_ls_r3_r2_5[] = {
  0x98, 0xbf, 0x53, 0x1d // It ls; add ls r3 r2 5
};
const byte kInstruction_add_vs_r3_r2_4[] = {
  0x68, 0xbf, 0x13, 0x1d // It vs; add vs r3 r2 4
};
const byte kInstruction_add_gt_r7_r6_7[] = {
  0xc8, 0xbf, 0xf7, 0x1d // It gt; add gt r7 r6 7
};
const byte kInstruction_add_hi_r0_r2_6[] = {
  0x88, 0xbf, 0x90, 0x1d // It hi; add hi r0 r2 6
};
const byte kInstruction_add_lt_r2_r6_4[] = {
  0xb8, 0xbf, 0x32, 0x1d // It lt; add lt r2 r6 4
};
const byte kInstruction_add_mi_r1_r0_3[] = {
  0x48, 0xbf, 0xc1, 0x1c // It mi; add mi r1 r0 3
};
const byte kInstruction_add_le_r5_r4_7[] = {
  0xd8, 0xbf, 0xe5, 0x1d // It le; add le r5 r4 7
};
const byte kInstruction_add_ge_r0_r7_6[] = {
  0xa8, 0xbf, 0xb8, 0x1d // It ge; add ge r0 r7 6
};
const byte kInstruction_add_hi_r7_r3_3[] = {
  0x88, 0xbf, 0xdf, 0x1c // It hi; add hi r7 r3 3
};
const byte kInstruction_add_cc_r4_r4_7[] = {
  0x38, 0xbf, 0xe4, 0x1d // It cc; add cc r4 r4 7
};
const byte kInstruction_add_le_r4_r2_5[] = {
  0xd8, 0xbf, 0x54, 0x1d // It le; add le r4 r2 5
};
const byte kInstruction_add_gt_r0_r0_3[] = {
  0xc8, 0xbf, 0xc0, 0x1c // It gt; add gt r0 r0 3
};
const byte kInstruction_add_pl_r0_r4_5[] = {
  0x58, 0xbf, 0x60, 0x1d // It pl; add pl r0 r4 5
};
const byte kInstruction_add_ge_r0_r5_6[] = {
  0xa8, 0xbf, 0xa8, 0x1d // It ge; add ge r0 r5 6
};
const byte kInstruction_add_lt_r7_r3_1[] = {
  0xb8, 0xbf, 0x5f, 0x1c // It lt; add lt r7 r3 1
};
const byte kInstruction_add_cs_r4_r2_0[] = {
  0x28, 0xbf, 0x14, 0x1c // It cs; add cs r4 r2 0
};
const byte kInstruction_add_vs_r2_r5_5[] = {
  0x68, 0xbf, 0x6a, 0x1d // It vs; add vs r2 r5 5
};
const byte kInstruction_add_le_r5_r4_1[] = {
  0xd8, 0xbf, 0x65, 0x1c // It le; add le r5 r4 1
};
const byte kInstruction_add_ge_r5_r5_1[] = {
  0xa8, 0xbf, 0x6d, 0x1c // It ge; add ge r5 r5 1
};
const byte kInstruction_add_ls_r2_r4_6[] = {
  0x98, 0xbf, 0xa2, 0x1d // It ls; add ls r2 r4 6
};
const byte kInstruction_add_lt_r4_r4_0[] = {
  0xb8, 0xbf, 0x24, 0x1c // It lt; add lt r4 r4 0
};
const byte kInstruction_add_vs_r4_r6_1[] = {
  0x68, 0xbf, 0x74, 0x1c // It vs; add vs r4 r6 1
};
const byte kInstruction_add_hi_r4_r1_0[] = {
  0x88, 0xbf, 0x0c, 0x1c // It hi; add hi r4 r1 0
};
const byte kInstruction_add_vc_r0_r7_1[] = {
  0x78, 0xbf, 0x78, 0x1c // It vc; add vc r0 r7 1
};
const byte kInstruction_add_lt_r2_r7_2[] = {
  0xb8, 0xbf, 0xba, 0x1c // It lt; add lt r2 r7 2
};
const byte kInstruction_add_ls_r7_r5_6[] = {
  0x98, 0xbf, 0xaf, 0x1d // It ls; add ls r7 r5 6
};
const byte kInstruction_add_vs_r3_r2_5[] = {
  0x68, 0xbf, 0x53, 0x1d // It vs; add vs r3 r2 5
};
const byte kInstruction_add_vs_r6_r3_4[] = {
  0x68, 0xbf, 0x1e, 0x1d // It vs; add vs r6 r3 4
};
const byte kInstruction_add_le_r1_r2_6[] = {
  0xd8, 0xbf, 0x91, 0x1d // It le; add le r1 r2 6
};
const byte kInstruction_add_vc_r3_r7_7[] = {
  0x78, 0xbf, 0xfb, 0x1d // It vc; add vc r3 r7 7
};
const byte kInstruction_add_le_r7_r7_0[] = {
  0xd8, 0xbf, 0x3f, 0x1c // It le; add le r7 r7 0
};
const byte kInstruction_add_lt_r0_r0_6[] = {
  0xb8, 0xbf, 0x80, 0x1d // It lt; add lt r0 r0 6
};
const byte kInstruction_add_ne_r4_r5_2[] = {
  0x18, 0xbf, 0xac, 0x1c // It ne; add ne r4 r5 2
};
const byte kInstruction_add_mi_r4_r1_6[] = {
  0x48, 0xbf, 0x8c, 0x1d // It mi; add mi r4 r1 6
};
const byte kInstruction_add_ge_r1_r4_2[] = {
  0xa8, 0xbf, 0xa1, 0x1c // It ge; add ge r1 r4 2
};
const byte kInstruction_add_mi_r4_r3_3[] = {
  0x48, 0xbf, 0xdc, 0x1c // It mi; add mi r4 r3 3
};
const byte kInstruction_add_lt_r2_r5_0[] = {
  0xb8, 0xbf, 0x2a, 0x1c // It lt; add lt r2 r5 0
};
const byte kInstruction_add_gt_r3_r4_1[] = {
  0xc8, 0xbf, 0x63, 0x1c // It gt; add gt r3 r4 1
};
const byte kInstruction_add_le_r6_r3_3[] = {
  0xd8, 0xbf, 0xde, 0x1c // It le; add le r6 r3 3
};
const byte kInstruction_add_ge_r2_r6_6[] = {
  0xa8, 0xbf, 0xb2, 0x1d // It ge; add ge r2 r6 6
};
const byte kInstruction_add_ne_r2_r1_5[] = {
  0x18, 0xbf, 0x4a, 0x1d // It ne; add ne r2 r1 5
};
const byte kInstruction_add_cc_r1_r6_0[] = {
  0x38, 0xbf, 0x31, 0x1c // It cc; add cc r1 r6 0
};
const byte kInstruction_add_lt_r1_r6_1[] = {
  0xb8, 0xbf, 0x71, 0x1c // It lt; add lt r1 r6 1
};
const byte kInstruction_add_vc_r2_r7_2[] = {
  0x78, 0xbf, 0xba, 0x1c // It vc; add vc r2 r7 2
};
const byte kInstruction_add_le_r5_r1_1[] = {
  0xd8, 0xbf, 0x4d, 0x1c // It le; add le r5 r1 1
};
const byte kInstruction_add_ge_r1_r7_3[] = {
  0xa8, 0xbf, 0xf9, 0x1c // It ge; add ge r1 r7 3
};
const byte kInstruction_add_gt_r5_r3_5[] = {
  0xc8, 0xbf, 0x5d, 0x1d // It gt; add gt r5 r3 5
};
const byte kInstruction_add_ne_r1_r5_3[] = {
  0x18, 0xbf, 0xe9, 0x1c // It ne; add ne r1 r5 3
};
const byte kInstruction_add_gt_r5_r6_4[] = {
  0xc8, 0xbf, 0x35, 0x1d // It gt; add gt r5 r6 4
};
const byte kInstruction_add_eq_r3_r2_0[] = {
  0x08, 0xbf, 0x13, 0x1c // It eq; add eq r3 r2 0
};
const byte kInstruction_add_lt_r0_r0_1[] = {
  0xb8, 0xbf, 0x40, 0x1c // It lt; add lt r0 r0 1
};
const byte kInstruction_add_cc_r0_r3_1[] = {
  0x38, 0xbf, 0x58, 0x1c // It cc; add cc r0 r3 1
};
const byte kInstruction_add_cc_r0_r1_5[] = {
  0x38, 0xbf, 0x48, 0x1d // It cc; add cc r0 r1 5
};
const byte kInstruction_add_ne_r4_r4_0[] = {
  0x18, 0xbf, 0x24, 0x1c // It ne; add ne r4 r4 0
};
const byte kInstruction_add_ge_r0_r6_4[] = {
  0xa8, 0xbf, 0x30, 0x1d // It ge; add ge r0 r6 4
};
const byte kInstruction_add_lt_r2_r6_0[] = {
  0xb8, 0xbf, 0x32, 0x1c // It lt; add lt r2 r6 0
};
const byte kInstruction_add_cs_r0_r6_5[] = {
  0x28, 0xbf, 0x70, 0x1d // It cs; add cs r0 r6 5
};
const byte kInstruction_add_lt_r3_r5_1[] = {
  0xb8, 0xbf, 0x6b, 0x1c // It lt; add lt r3 r5 1
};
const byte kInstruction_add_lt_r7_r1_0[] = {
  0xb8, 0xbf, 0x0f, 0x1c // It lt; add lt r7 r1 0
};
const byte kInstruction_add_cc_r6_r3_6[] = {
  0x38, 0xbf, 0x9e, 0x1d // It cc; add cc r6 r3 6
};
const byte kInstruction_add_gt_r1_r0_0[] = {
  0xc8, 0xbf, 0x01, 0x1c // It gt; add gt r1 r0 0
};
const byte kInstruction_add_ls_r0_r7_5[] = {
  0x98, 0xbf, 0x78, 0x1d // It ls; add ls r0 r7 5
};
const byte kInstruction_add_hi_r5_r2_6[] = {
  0x88, 0xbf, 0x95, 0x1d // It hi; add hi r5 r2 6
};
const byte kInstruction_add_pl_r5_r2_7[] = {
  0x58, 0xbf, 0xd5, 0x1d // It pl; add pl r5 r2 7
};
const byte kInstruction_add_mi_r7_r0_5[] = {
  0x48, 0xbf, 0x47, 0x1d // It mi; add mi r7 r0 5
};
const byte kInstruction_add_cc_r3_r0_1[] = {
  0x38, 0xbf, 0x43, 0x1c // It cc; add cc r3 r0 1
};
const byte kInstruction_add_lt_r2_r2_0[] = {
  0xb8, 0xbf, 0x12, 0x1c // It lt; add lt r2 r2 0
};
const byte kInstruction_add_vc_r2_r1_6[] = {
  0x78, 0xbf, 0x8a, 0x1d // It vc; add vc r2 r1 6
};
const byte kInstruction_add_le_r2_r3_3[] = {
  0xd8, 0xbf, 0xda, 0x1c // It le; add le r2 r3 3
};
const byte kInstruction_add_le_r1_r3_1[] = {
  0xd8, 0xbf, 0x59, 0x1c // It le; add le r1 r3 1
};
const byte kInstruction_add_ls_r5_r6_4[] = {
  0x98, 0xbf, 0x35, 0x1d // It ls; add ls r5 r6 4
};
const byte kInstruction_add_ne_r5_r1_0[] = {
  0x18, 0xbf, 0x0d, 0x1c // It ne; add ne r5 r1 0
};
const byte kInstruction_add_le_r0_r0_5[] = {
  0xd8, 0xbf, 0x40, 0x1d // It le; add le r0 r0 5
};
const byte kInstruction_add_cc_r4_r7_0[] = {
  0x38, 0xbf, 0x3c, 0x1c // It cc; add cc r4 r7 0
};
const byte kInstruction_add_ne_r1_r4_0[] = {
  0x18, 0xbf, 0x21, 0x1c // It ne; add ne r1 r4 0
};
const byte kInstruction_add_hi_r5_r3_7[] = {
  0x88, 0xbf, 0xdd, 0x1d // It hi; add hi r5 r3 7
};
const byte kInstruction_add_ne_r0_r3_1[] = {
  0x18, 0xbf, 0x58, 0x1c // It ne; add ne r0 r3 1
};
const byte kInstruction_add_vs_r1_r3_0[] = {
  0x68, 0xbf, 0x19, 0x1c // It vs; add vs r1 r3 0
};
const byte kInstruction_add_gt_r6_r5_7[] = {
  0xc8, 0xbf, 0xee, 0x1d // It gt; add gt r6 r5 7
};
const byte kInstruction_add_cs_r3_r0_1[] = {
  0x28, 0xbf, 0x43, 0x1c // It cs; add cs r3 r0 1
};
const byte kInstruction_add_cc_r7_r0_5[] = {
  0x38, 0xbf, 0x47, 0x1d // It cc; add cc r7 r0 5
};
const byte kInstruction_add_pl_r5_r2_5[] = {
  0x58, 0xbf, 0x55, 0x1d // It pl; add pl r5 r2 5
};
const byte kInstruction_add_vs_r5_r1_6[] = {
  0x68, 0xbf, 0x8d, 0x1d // It vs; add vs r5 r1 6
};
const byte kInstruction_add_ge_r1_r1_0[] = {
  0xa8, 0xbf, 0x09, 0x1c // It ge; add ge r1 r1 0
};
const byte kInstruction_add_eq_r6_r1_2[] = {
  0x08, 0xbf, 0x8e, 0x1c // It eq; add eq r6 r1 2
};
const byte kInstruction_add_lt_r0_r5_5[] = {
  0xb8, 0xbf, 0x68, 0x1d // It lt; add lt r0 r5 5
};
const byte kInstruction_add_vc_r2_r2_1[] = {
  0x78, 0xbf, 0x52, 0x1c // It vc; add vc r2 r2 1
};
const byte kInstruction_add_vc_r0_r4_4[] = {
  0x78, 0xbf, 0x20, 0x1d // It vc; add vc r0 r4 4
};
const byte kInstruction_add_pl_r2_r6_4[] = {
  0x58, 0xbf, 0x32, 0x1d // It pl; add pl r2 r6 4
};
const byte kInstruction_add_hi_r1_r3_2[] = {
  0x88, 0xbf, 0x99, 0x1c // It hi; add hi r1 r3 2
};
const byte kInstruction_add_hi_r7_r2_1[] = {
  0x88, 0xbf, 0x57, 0x1c // It hi; add hi r7 r2 1
};
const byte kInstruction_add_lt_r0_r1_4[] = {
  0xb8, 0xbf, 0x08, 0x1d // It lt; add lt r0 r1 4
};
const byte kInstruction_add_cc_r6_r2_0[] = {
  0x38, 0xbf, 0x16, 0x1c // It cc; add cc r6 r2 0
};
const byte kInstruction_add_vs_r5_r4_2[] = {
  0x68, 0xbf, 0xa5, 0x1c // It vs; add vs r5 r4 2
};
const byte kInstruction_add_pl_r0_r5_3[] = {
  0x58, 0xbf, 0xe8, 0x1c // It pl; add pl r0 r5 3
};
const byte kInstruction_add_le_r0_r3_4[] = {
  0xd8, 0xbf, 0x18, 0x1d // It le; add le r0 r3 4
};
const byte kInstruction_add_cc_r6_r2_5[] = {
  0x38, 0xbf, 0x56, 0x1d // It cc; add cc r6 r2 5
};
const byte kInstruction_add_ls_r6_r1_3[] = {
  0x98, 0xbf, 0xce, 0x1c // It ls; add ls r6 r1 3
};
const byte kInstruction_add_ne_r6_r6_0[] = {
  0x18, 0xbf, 0x36, 0x1c // It ne; add ne r6 r6 0
};
const byte kInstruction_add_vc_r1_r3_1[] = {
  0x78, 0xbf, 0x59, 0x1c // It vc; add vc r1 r3 1
};
const byte kInstruction_add_lt_r1_r7_6[] = {
  0xb8, 0xbf, 0xb9, 0x1d // It lt; add lt r1 r7 6
};
const byte kInstruction_add_hi_r5_r3_5[] = {
  0x88, 0xbf, 0x5d, 0x1d // It hi; add hi r5 r3 5
};
const byte kInstruction_add_gt_r6_r4_6[] = {
  0xc8, 0xbf, 0xa6, 0x1d // It gt; add gt r6 r4 6
};
const byte kInstruction_add_ne_r6_r6_3[] = {
  0x18, 0xbf, 0xf6, 0x1c // It ne; add ne r6 r6 3
};
const byte kInstruction_add_pl_r2_r4_1[] = {
  0x58, 0xbf, 0x62, 0x1c // It pl; add pl r2 r4 1
};
const byte kInstruction_add_cc_r3_r6_7[] = {
  0x38, 0xbf, 0xf3, 0x1d // It cc; add cc r3 r6 7
};
const byte kInstruction_add_cs_r3_r7_0[] = {
  0x28, 0xbf, 0x3b, 0x1c // It cs; add cs r3 r7 0
};
const byte kInstruction_add_cc_r7_r7_5[] = {
  0x38, 0xbf, 0x7f, 0x1d // It cc; add cc r7 r7 5
};
const byte kInstruction_add_vc_r6_r0_7[] = {
  0x78, 0xbf, 0xc6, 0x1d // It vc; add vc r6 r0 7
};
const byte kInstruction_add_lt_r2_r0_3[] = {
  0xb8, 0xbf, 0xc2, 0x1c // It lt; add lt r2 r0 3
};
const byte kInstruction_add_ne_r5_r4_7[] = {
  0x18, 0xbf, 0xe5, 0x1d // It ne; add ne r5 r4 7
};
const byte kInstruction_add_vc_r2_r7_6[] = {
  0x78, 0xbf, 0xba, 0x1d // It vc; add vc r2 r7 6
};
const byte kInstruction_add_cc_r4_r3_2[] = {
  0x38, 0xbf, 0x9c, 0x1c // It cc; add cc r4 r3 2
};
const byte kInstruction_add_lt_r3_r7_6[] = {
  0xb8, 0xbf, 0xbb, 0x1d // It lt; add lt r3 r7 6
};
const byte kInstruction_add_gt_r3_r7_7[] = {
  0xc8, 0xbf, 0xfb, 0x1d // It gt; add gt r3 r7 7
};
const byte kInstruction_add_ge_r6_r7_5[] = {
  0xa8, 0xbf, 0x7e, 0x1d // It ge; add ge r6 r7 5
};
const byte kInstruction_add_ls_r1_r4_5[] = {
  0x98, 0xbf, 0x61, 0x1d // It ls; add ls r1 r4 5
};
const byte kInstruction_add_eq_r1_r7_3[] = {
  0x08, 0xbf, 0xf9, 0x1c // It eq; add eq r1 r7 3
};
const byte kInstruction_add_le_r1_r4_6[] = {
  0xd8, 0xbf, 0xa1, 0x1d // It le; add le r1 r4 6
};
const byte kInstruction_add_ne_r4_r3_1[] = {
  0x18, 0xbf, 0x5c, 0x1c // It ne; add ne r4 r3 1
};
const byte kInstruction_add_pl_r7_r3_6[] = {
  0x58, 0xbf, 0x9f, 0x1d // It pl; add pl r7 r3 6
};
const byte kInstruction_add_ne_r7_r7_0[] = {
  0x18, 0xbf, 0x3f, 0x1c // It ne; add ne r7 r7 0
};
const byte kInstruction_add_lt_r1_r2_6[] = {
  0xb8, 0xbf, 0x91, 0x1d // It lt; add lt r1 r2 6
};
const byte kInstruction_add_le_r0_r7_2[] = {
  0xd8, 0xbf, 0xb8, 0x1c // It le; add le r0 r7 2
};
const byte kInstruction_add_pl_r2_r2_0[] = {
  0x58, 0xbf, 0x12, 0x1c // It pl; add pl r2 r2 0
};
const byte kInstruction_add_vs_r2_r6_2[] = {
  0x68, 0xbf, 0xb2, 0x1c // It vs; add vs r2 r6 2
};
const byte kInstruction_add_pl_r0_r7_7[] = {
  0x58, 0xbf, 0xf8, 0x1d // It pl; add pl r0 r7 7
};
const byte kInstruction_add_gt_r6_r2_5[] = {
  0xc8, 0xbf, 0x56, 0x1d // It gt; add gt r6 r2 5
};
const byte kInstruction_add_cs_r4_r3_1[] = {
  0x28, 0xbf, 0x5c, 0x1c // It cs; add cs r4 r3 1
};
const byte kInstruction_add_ne_r3_r5_4[] = {
  0x18, 0xbf, 0x2b, 0x1d // It ne; add ne r3 r5 4
};
const byte kInstruction_add_mi_r0_r0_0[] = {
  0x48, 0xbf, 0x00, 0x1c // It mi; add mi r0 r0 0
};
const byte kInstruction_add_ge_r2_r0_1[] = {
  0xa8, 0xbf, 0x42, 0x1c // It ge; add ge r2 r0 1
};
const byte kInstruction_add_mi_r7_r3_7[] = {
  0x48, 0xbf, 0xdf, 0x1d // It mi; add mi r7 r3 7
};
const byte kInstruction_add_ne_r0_r7_2[] = {
  0x18, 0xbf, 0xb8, 0x1c // It ne; add ne r0 r7 2
};
const byte kInstruction_add_ge_r0_r6_2[] = {
  0xa8, 0xbf, 0xb0, 0x1c // It ge; add ge r0 r6 2
};
const byte kInstruction_add_hi_r2_r7_2[] = {
  0x88, 0xbf, 0xba, 0x1c // It hi; add hi r2 r7 2
};
const byte kInstruction_add_vs_r0_r4_0[] = {
  0x68, 0xbf, 0x20, 0x1c // It vs; add vs r0 r4 0
};
const byte kInstruction_add_hi_r2_r1_0[] = {
  0x88, 0xbf, 0x0a, 0x1c // It hi; add hi r2 r1 0
};
const byte kInstruction_add_hi_r7_r0_3[] = {
  0x88, 0xbf, 0xc7, 0x1c // It hi; add hi r7 r0 3
};
const byte kInstruction_add_ge_r5_r4_0[] = {
  0xa8, 0xbf, 0x25, 0x1c // It ge; add ge r5 r4 0
};
const byte kInstruction_add_mi_r4_r1_3[] = {
  0x48, 0xbf, 0xcc, 0x1c // It mi; add mi r4 r1 3
};
const byte kInstruction_add_le_r1_r6_3[] = {
  0xd8, 0xbf, 0xf1, 0x1c // It le; add le r1 r6 3
};
const byte kInstruction_add_le_r7_r6_2[] = {
  0xd8, 0xbf, 0xb7, 0x1c // It le; add le r7 r6 2
};
const byte kInstruction_add_mi_r6_r3_4[] = {
  0x48, 0xbf, 0x1e, 0x1d // It mi; add mi r6 r3 4
};
const byte kInstruction_add_cs_r4_r4_0[] = {
  0x28, 0xbf, 0x24, 0x1c // It cs; add cs r4 r4 0
};
const byte kInstruction_add_cc_r2_r3_2[] = {
  0x38, 0xbf, 0x9a, 0x1c // It cc; add cc r2 r3 2
};
const byte kInstruction_add_mi_r1_r5_4[] = {
  0x48, 0xbf, 0x29, 0x1d // It mi; add mi r1 r5 4
};
const byte kInstruction_add_cc_r5_r3_2[] = {
  0x38, 0xbf, 0x9d, 0x1c // It cc; add cc r5 r3 2
};
const byte kInstruction_add_gt_r2_r0_2[] = {
  0xc8, 0xbf, 0x82, 0x1c // It gt; add gt r2 r0 2
};
const byte kInstruction_add_le_r2_r4_7[] = {
  0xd8, 0xbf, 0xe2, 0x1d // It le; add le r2 r4 7
};
const byte kInstruction_add_vc_r2_r3_3[] = {
  0x78, 0xbf, 0xda, 0x1c // It vc; add vc r2 r3 3
};
const byte kInstruction_add_eq_r4_r2_3[] = {
  0x08, 0xbf, 0xd4, 0x1c // It eq; add eq r4 r2 3
};
const byte kInstruction_add_ge_r6_r0_7[] = {
  0xa8, 0xbf, 0xc6, 0x1d // It ge; add ge r6 r0 7
};
const byte kInstruction_add_eq_r0_r7_7[] = {
  0x08, 0xbf, 0xf8, 0x1d // It eq; add eq r0 r7 7
};
const byte kInstruction_add_gt_r5_r6_2[] = {
  0xc8, 0xbf, 0xb5, 0x1c // It gt; add gt r5 r6 2
};
const byte kInstruction_add_ne_r6_r0_7[] = {
  0x18, 0xbf, 0xc6, 0x1d // It ne; add ne r6 r0 7
};
const byte kInstruction_add_ne_r7_r0_4[] = {
  0x18, 0xbf, 0x07, 0x1d // It ne; add ne r7 r0 4
};
const byte kInstruction_add_eq_r7_r7_1[] = {
  0x08, 0xbf, 0x7f, 0x1c // It eq; add eq r7 r7 1
};
const byte kInstruction_add_le_r6_r5_0[] = {
  0xd8, 0xbf, 0x2e, 0x1c // It le; add le r6 r5 0
};
const byte kInstruction_add_vc_r3_r6_7[] = {
  0x78, 0xbf, 0xf3, 0x1d // It vc; add vc r3 r6 7
};
const byte kInstruction_add_pl_r0_r3_7[] = {
  0x58, 0xbf, 0xd8, 0x1d // It pl; add pl r0 r3 7
};
const byte kInstruction_add_ls_r5_r5_0[] = {
  0x98, 0xbf, 0x2d, 0x1c // It ls; add ls r5 r5 0
};
const byte kInstruction_add_ge_r6_r3_7[] = {
  0xa8, 0xbf, 0xde, 0x1d // It ge; add ge r6 r3 7
};
const byte kInstruction_add_eq_r3_r2_6[] = {
  0x08, 0xbf, 0x93, 0x1d // It eq; add eq r3 r2 6
};
const byte kInstruction_add_cs_r5_r3_7[] = {
  0x28, 0xbf, 0xdd, 0x1d // It cs; add cs r5 r3 7
};
const byte kInstruction_add_cc_r3_r5_7[] = {
  0x38, 0xbf, 0xeb, 0x1d // It cc; add cc r3 r5 7
};
const byte kInstruction_add_ge_r6_r1_4[] = {
  0xa8, 0xbf, 0x0e, 0x1d // It ge; add ge r6 r1 4
};
const byte kInstruction_add_mi_r5_r4_5[] = {
  0x48, 0xbf, 0x65, 0x1d // It mi; add mi r5 r4 5
};
const byte kInstruction_add_eq_r7_r6_6[] = {
  0x08, 0xbf, 0xb7, 0x1d // It eq; add eq r7 r6 6
};
const byte kInstruction_add_gt_r2_r2_2[] = {
  0xc8, 0xbf, 0x92, 0x1c // It gt; add gt r2 r2 2
};
const byte kInstruction_add_mi_r1_r0_2[] = {
  0x48, 0xbf, 0x81, 0x1c // It mi; add mi r1 r0 2
};
const byte kInstruction_add_eq_r2_r2_1[] = {
  0x08, 0xbf, 0x52, 0x1c // It eq; add eq r2 r2 1
};
const byte kInstruction_add_lt_r1_r2_1[] = {
  0xb8, 0xbf, 0x51, 0x1c // It lt; add lt r1 r2 1
};
const byte kInstruction_add_cc_r5_r1_3[] = {
  0x38, 0xbf, 0xcd, 0x1c // It cc; add cc r5 r1 3
};
const byte kInstruction_add_ge_r1_r7_5[] = {
  0xa8, 0xbf, 0x79, 0x1d // It ge; add ge r1 r7 5
};
const byte kInstruction_add_lt_r6_r3_3[] = {
  0xb8, 0xbf, 0xde, 0x1c // It lt; add lt r6 r3 3
};
const byte kInstruction_add_vc_r3_r1_7[] = {
  0x78, 0xbf, 0xcb, 0x1d // It vc; add vc r3 r1 7
};
const byte kInstruction_add_mi_r7_r2_7[] = {
  0x48, 0xbf, 0xd7, 0x1d // It mi; add mi r7 r2 7
};
const byte kInstruction_add_lt_r0_r3_1[] = {
  0xb8, 0xbf, 0x58, 0x1c // It lt; add lt r0 r3 1
};
const byte kInstruction_add_cs_r1_r7_1[] = {
  0x28, 0xbf, 0x79, 0x1c // It cs; add cs r1 r7 1
};
const byte kInstruction_add_cc_r3_r4_5[] = {
  0x38, 0xbf, 0x63, 0x1d // It cc; add cc r3 r4 5
};
const byte kInstruction_add_lt_r5_r0_2[] = {
  0xb8, 0xbf, 0x85, 0x1c // It lt; add lt r5 r0 2
};
const byte kInstruction_add_hi_r1_r5_7[] = {
  0x88, 0xbf, 0xe9, 0x1d // It hi; add hi r1 r5 7
};
const byte kInstruction_add_ne_r4_r6_5[] = {
  0x18, 0xbf, 0x74, 0x1d // It ne; add ne r4 r6 5
};
const byte kInstruction_add_vc_r2_r3_6[] = {
  0x78, 0xbf, 0x9a, 0x1d // It vc; add vc r2 r3 6
};
const byte kInstruction_add_mi_r1_r7_0[] = {
  0x48, 0xbf, 0x39, 0x1c // It mi; add mi r1 r7 0
};
const byte kInstruction_add_hi_r0_r2_4[] = {
  0x88, 0xbf, 0x10, 0x1d // It hi; add hi r0 r2 4
};
const byte kInstruction_add_cs_r5_r2_5[] = {
  0x28, 0xbf, 0x55, 0x1d // It cs; add cs r5 r2 5
};
const byte kInstruction_add_ge_r2_r6_3[] = {
  0xa8, 0xbf, 0xf2, 0x1c // It ge; add ge r2 r6 3
};
const byte kInstruction_add_cs_r2_r7_5[] = {
  0x28, 0xbf, 0x7a, 0x1d // It cs; add cs r2 r7 5
};
const byte kInstruction_add_eq_r1_r6_5[] = {
  0x08, 0xbf, 0x71, 0x1d // It eq; add eq r1 r6 5
};
const byte kInstruction_add_vc_r7_r0_6[] = {
  0x78, 0xbf, 0x87, 0x1d // It vc; add vc r7 r0 6
};
const byte kInstruction_add_vs_r1_r0_2[] = {
  0x68, 0xbf, 0x81, 0x1c // It vs; add vs r1 r0 2
};
const byte kInstruction_add_eq_r3_r3_5[] = {
  0x08, 0xbf, 0x5b, 0x1d // It eq; add eq r3 r3 5
};
const byte kInstruction_add_cc_r0_r4_5[] = {
  0x38, 0xbf, 0x60, 0x1d // It cc; add cc r0 r4 5
};
const byte kInstruction_add_lt_r7_r2_5[] = {
  0xb8, 0xbf, 0x57, 0x1d // It lt; add lt r7 r2 5
};
const byte kInstruction_add_pl_r1_r1_0[] = {
  0x58, 0xbf, 0x09, 0x1c // It pl; add pl r1 r1 0
};
const byte kInstruction_add_hi_r7_r2_5[] = {
  0x88, 0xbf, 0x57, 0x1d // It hi; add hi r7 r2 5
};
const byte kInstruction_add_cc_r0_r4_0[] = {
  0x38, 0xbf, 0x20, 0x1c // It cc; add cc r0 r4 0
};
const byte kInstruction_add_eq_r0_r3_7[] = {
  0x08, 0xbf, 0xd8, 0x1d // It eq; add eq r0 r3 7
};
const byte kInstruction_add_lt_r4_r5_0[] = {
  0xb8, 0xbf, 0x2c, 0x1c // It lt; add lt r4 r5 0
};
const byte kInstruction_add_vc_r5_r4_7[] = {
  0x78, 0xbf, 0xe5, 0x1d // It vc; add vc r5 r4 7
};
const byte kInstruction_add_gt_r7_r7_0[] = {
  0xc8, 0xbf, 0x3f, 0x1c // It gt; add gt r7 r7 0
};
const byte kInstruction_add_le_r1_r2_7[] = {
  0xd8, 0xbf, 0xd1, 0x1d // It le; add le r1 r2 7
};
const byte kInstruction_add_cc_r0_r4_6[] = {
  0x38, 0xbf, 0xa0, 0x1d // It cc; add cc r0 r4 6
};
const byte kInstruction_add_eq_r1_r5_5[] = {
  0x08, 0xbf, 0x69, 0x1d // It eq; add eq r1 r5 5
};
const byte kInstruction_add_hi_r1_r5_2[] = {
  0x88, 0xbf, 0xa9, 0x1c // It hi; add hi r1 r5 2
};
const byte kInstruction_add_ne_r2_r4_1[] = {
  0x18, 0xbf, 0x62, 0x1c // It ne; add ne r2 r4 1
};
const byte kInstruction_add_cc_r3_r6_6[] = {
  0x38, 0xbf, 0xb3, 0x1d // It cc; add cc r3 r6 6
};
const byte kInstruction_add_eq_r7_r7_6[] = {
  0x08, 0xbf, 0xbf, 0x1d // It eq; add eq r7 r7 6
};
const byte kInstruction_add_ne_r5_r4_4[] = {
  0x18, 0xbf, 0x25, 0x1d // It ne; add ne r5 r4 4
};
const byte kInstruction_add_eq_r2_r4_2[] = {
  0x08, 0xbf, 0xa2, 0x1c // It eq; add eq r2 r4 2
};
const byte kInstruction_add_vc_r2_r1_2[] = {
  0x78, 0xbf, 0x8a, 0x1c // It vc; add vc r2 r1 2
};
const byte kInstruction_add_ge_r5_r1_3[] = {
  0xa8, 0xbf, 0xcd, 0x1c // It ge; add ge r5 r1 3
};
const byte kInstruction_add_hi_r0_r0_5[] = {
  0x88, 0xbf, 0x40, 0x1d // It hi; add hi r0 r0 5
};
const byte kInstruction_add_ne_r6_r2_6[] = {
  0x18, 0xbf, 0x96, 0x1d // It ne; add ne r6 r2 6
};
const byte kInstruction_add_vc_r2_r5_0[] = {
  0x78, 0xbf, 0x2a, 0x1c // It vc; add vc r2 r5 0
};
const byte kInstruction_add_le_r1_r3_3[] = {
  0xd8, 0xbf, 0xd9, 0x1c // It le; add le r1 r3 3
};
const byte kInstruction_add_mi_r0_r2_0[] = {
  0x48, 0xbf, 0x10, 0x1c // It mi; add mi r0 r2 0
};
const byte kInstruction_add_mi_r1_r2_7[] = {
  0x48, 0xbf, 0xd1, 0x1d // It mi; add mi r1 r2 7
};
const byte kInstruction_add_vc_r6_r5_5[] = {
  0x78, 0xbf, 0x6e, 0x1d // It vc; add vc r6 r5 5
};
const byte kInstruction_add_ge_r2_r0_3[] = {
  0xa8, 0xbf, 0xc2, 0x1c // It ge; add ge r2 r0 3
};
const byte kInstruction_add_le_r5_r0_7[] = {
  0xd8, 0xbf, 0xc5, 0x1d // It le; add le r5 r0 7
};
const byte kInstruction_add_lt_r1_r4_0[] = {
  0xb8, 0xbf, 0x21, 0x1c // It lt; add lt r1 r4 0
};
const byte kInstruction_add_le_r4_r4_3[] = {
  0xd8, 0xbf, 0xe4, 0x1c // It le; add le r4 r4 3
};
const byte kInstruction_add_vs_r0_r6_5[] = {
  0x68, 0xbf, 0x70, 0x1d // It vs; add vs r0 r6 5
};
const byte kInstruction_add_vs_r0_r0_2[] = {
  0x68, 0xbf, 0x80, 0x1c // It vs; add vs r0 r0 2
};
const byte kInstruction_add_ne_r4_r3_3[] = {
  0x18, 0xbf, 0xdc, 0x1c // It ne; add ne r4 r3 3
};
const byte kInstruction_add_ge_r0_r6_5[] = {
  0xa8, 0xbf, 0x70, 0x1d // It ge; add ge r0 r6 5
};
const byte kInstruction_add_pl_r6_r0_5[] = {
  0x58, 0xbf, 0x46, 0x1d // It pl; add pl r6 r0 5
};
const byte kInstruction_add_ls_r5_r3_4[] = {
  0x98, 0xbf, 0x1d, 0x1d // It ls; add ls r5 r3 4
};
const byte kInstruction_add_vs_r3_r4_5[] = {
  0x68, 0xbf, 0x63, 0x1d // It vs; add vs r3 r4 5
};
const byte kInstruction_add_gt_r1_r4_6[] = {
  0xc8, 0xbf, 0xa1, 0x1d // It gt; add gt r1 r4 6
};
const byte kInstruction_add_vc_r1_r1_2[] = {
  0x78, 0xbf, 0x89, 0x1c // It vc; add vc r1 r1 2
};
const byte kInstruction_add_lt_r7_r1_5[] = {
  0xb8, 0xbf, 0x4f, 0x1d // It lt; add lt r7 r1 5
};
const byte kInstruction_add_eq_r0_r4_1[] = {
  0x08, 0xbf, 0x60, 0x1c // It eq; add eq r0 r4 1
};
const byte kInstruction_add_le_r0_r4_2[] = {
  0xd8, 0xbf, 0xa0, 0x1c // It le; add le r0 r4 2
};
const byte kInstruction_add_vc_r0_r0_4[] = {
  0x78, 0xbf, 0x00, 0x1d // It vc; add vc r0 r0 4
};
const byte kInstruction_add_eq_r4_r6_3[] = {
  0x08, 0xbf, 0xf4, 0x1c // It eq; add eq r4 r6 3
};
const byte kInstruction_add_eq_r7_r1_4[] = {
  0x08, 0xbf, 0x0f, 0x1d // It eq; add eq r7 r1 4
};
const byte kInstruction_add_pl_r5_r4_7[] = {
  0x58, 0xbf, 0xe5, 0x1d // It pl; add pl r5 r4 7
};
const byte kInstruction_add_ge_r6_r0_3[] = {
  0xa8, 0xbf, 0xc6, 0x1c // It ge; add ge r6 r0 3
};
const byte kInstruction_add_gt_r5_r5_4[] = {
  0xc8, 0xbf, 0x2d, 0x1d // It gt; add gt r5 r5 4
};
const byte kInstruction_add_mi_r6_r0_2[] = {
  0x48, 0xbf, 0x86, 0x1c // It mi; add mi r6 r0 2
};
const byte kInstruction_add_gt_r7_r5_2[] = {
  0xc8, 0xbf, 0xaf, 0x1c // It gt; add gt r7 r5 2
};
const byte kInstruction_add_le_r0_r6_5[] = {
  0xd8, 0xbf, 0x70, 0x1d // It le; add le r0 r6 5
};
const byte kInstruction_add_vc_r1_r5_6[] = {
  0x78, 0xbf, 0xa9, 0x1d // It vc; add vc r1 r5 6
};
const byte kInstruction_add_eq_r5_r5_5[] = {
  0x08, 0xbf, 0x6d, 0x1d // It eq; add eq r5 r5 5
};
const byte kInstruction_add_ge_r2_r6_2[] = {
  0xa8, 0xbf, 0xb2, 0x1c // It ge; add ge r2 r6 2
};
const byte kInstruction_add_eq_r4_r4_7[] = {
  0x08, 0xbf, 0xe4, 0x1d // It eq; add eq r4 r4 7
};
const byte kInstruction_add_ne_r5_r5_4[] = {
  0x18, 0xbf, 0x2d, 0x1d // It ne; add ne r5 r5 4
};
const byte kInstruction_add_lt_r2_r1_0[] = {
  0xb8, 0xbf, 0x0a, 0x1c // It lt; add lt r2 r1 0
};
const byte kInstruction_add_gt_r4_r0_4[] = {
  0xc8, 0xbf, 0x04, 0x1d // It gt; add gt r4 r0 4
};
const byte kInstruction_add_le_r0_r0_1[] = {
  0xd8, 0xbf, 0x40, 0x1c // It le; add le r0 r0 1
};
const byte kInstruction_add_pl_r1_r3_1[] = {
  0x58, 0xbf, 0x59, 0x1c // It pl; add pl r1 r3 1
};
const byte kInstruction_add_ge_r6_r2_0[] = {
  0xa8, 0xbf, 0x16, 0x1c // It ge; add ge r6 r2 0
};
const byte kInstruction_add_ne_r2_r6_0[] = {
  0x18, 0xbf, 0x32, 0x1c // It ne; add ne r2 r6 0
};
const byte kInstruction_add_cs_r3_r2_5[] = {
  0x28, 0xbf, 0x53, 0x1d // It cs; add cs r3 r2 5
};
const byte kInstruction_add_eq_r3_r3_2[] = {
  0x08, 0xbf, 0x9b, 0x1c // It eq; add eq r3 r3 2
};
const byte kInstruction_add_ne_r6_r6_5[] = {
  0x18, 0xbf, 0x76, 0x1d // It ne; add ne r6 r6 5
};
const byte kInstruction_add_vc_r5_r2_2[] = {
  0x78, 0xbf, 0x95, 0x1c // It vc; add vc r5 r2 2
};
const byte kInstruction_add_pl_r3_r6_4[] = {
  0x58, 0xbf, 0x33, 0x1d // It pl; add pl r3 r6 4
};
const byte kInstruction_add_cs_r5_r7_6[] = {
  0x28, 0xbf, 0xbd, 0x1d // It cs; add cs r5 r7 6
};
const byte kInstruction_add_cs_r2_r1_7[] = {
  0x28, 0xbf, 0xca, 0x1d // It cs; add cs r2 r1 7
};
const byte kInstruction_add_pl_r4_r5_5[] = {
  0x58, 0xbf, 0x6c, 0x1d // It pl; add pl r4 r5 5
};
const byte kInstruction_add_pl_r1_r5_7[] = {
  0x58, 0xbf, 0xe9, 0x1d // It pl; add pl r1 r5 7
};
const byte kInstruction_add_mi_r4_r3_5[] = {
  0x48, 0xbf, 0x5c, 0x1d // It mi; add mi r4 r3 5
};
const byte kInstruction_add_vc_r0_r4_0[] = {
  0x78, 0xbf, 0x20, 0x1c // It vc; add vc r0 r4 0
};
const byte kInstruction_add_le_r2_r7_0[] = {
  0xd8, 0xbf, 0x3a, 0x1c // It le; add le r2 r7 0
};
const byte kInstruction_add_cc_r7_r2_6[] = {
  0x38, 0xbf, 0x97, 0x1d // It cc; add cc r7 r2 6
};
const byte kInstruction_add_vs_r7_r1_1[] = {
  0x68, 0xbf, 0x4f, 0x1c // It vs; add vs r7 r1 1
};
const byte kInstruction_add_gt_r3_r3_5[] = {
  0xc8, 0xbf, 0x5b, 0x1d // It gt; add gt r3 r3 5
};
const byte kInstruction_add_eq_r7_r1_1[] = {
  0x08, 0xbf, 0x4f, 0x1c // It eq; add eq r7 r1 1
};
const byte kInstruction_add_pl_r5_r7_5[] = {
  0x58, 0xbf, 0x7d, 0x1d // It pl; add pl r5 r7 5
};
const byte kInstruction_add_hi_r5_r6_1[] = {
  0x88, 0xbf, 0x75, 0x1c // It hi; add hi r5 r6 1
};
const byte kInstruction_add_cc_r2_r0_0[] = {
  0x38, 0xbf, 0x02, 0x1c // It cc; add cc r2 r0 0
};
const byte kInstruction_add_cs_r7_r7_2[] = {
  0x28, 0xbf, 0xbf, 0x1c // It cs; add cs r7 r7 2
};
const byte kInstruction_add_ne_r0_r3_6[] = {
  0x18, 0xbf, 0x98, 0x1d // It ne; add ne r0 r3 6
};
const byte kInstruction_add_mi_r4_r7_2[] = {
  0x48, 0xbf, 0xbc, 0x1c // It mi; add mi r4 r7 2
};
const byte kInstruction_add_le_r4_r4_7[] = {
  0xd8, 0xbf, 0xe4, 0x1d // It le; add le r4 r4 7
};
const byte kInstruction_add_gt_r1_r1_0[] = {
  0xc8, 0xbf, 0x09, 0x1c // It gt; add gt r1 r1 0
};
const byte kInstruction_add_vs_r2_r5_1[] = {
  0x68, 0xbf, 0x6a, 0x1c // It vs; add vs r2 r5 1
};
const byte kInstruction_add_ls_r7_r2_4[] = {
  0x98, 0xbf, 0x17, 0x1d // It ls; add ls r7 r2 4
};
const byte kInstruction_add_vs_r2_r1_4[] = {
  0x68, 0xbf, 0x0a, 0x1d // It vs; add vs r2 r1 4
};
const byte kInstruction_add_eq_r5_r6_5[] = {
  0x08, 0xbf, 0x75, 0x1d // It eq; add eq r5 r6 5
};
const byte kInstruction_add_vs_r4_r3_4[] = {
  0x68, 0xbf, 0x1c, 0x1d // It vs; add vs r4 r3 4
};
const byte kInstruction_add_vs_r2_r1_2[] = {
  0x68, 0xbf, 0x8a, 0x1c // It vs; add vs r2 r1 2
};
const byte kInstruction_add_hi_r0_r3_3[] = {
  0x88, 0xbf, 0xd8, 0x1c // It hi; add hi r0 r3 3
};
const byte kInstruction_add_hi_r4_r1_5[] = {
  0x88, 0xbf, 0x4c, 0x1d // It hi; add hi r4 r1 5
};
const byte kInstruction_add_pl_r6_r6_7[] = {
  0x58, 0xbf, 0xf6, 0x1d // It pl; add pl r6 r6 7
};
const byte kInstruction_add_gt_r0_r5_0[] = {
  0xc8, 0xbf, 0x28, 0x1c // It gt; add gt r0 r5 0
};
const byte kInstruction_add_cc_r5_r0_7[] = {
  0x38, 0xbf, 0xc5, 0x1d // It cc; add cc r5 r0 7
};
const byte kInstruction_add_ls_r7_r7_7[] = {
  0x98, 0xbf, 0xff, 0x1d // It ls; add ls r7 r7 7
};
const byte kInstruction_add_vc_r6_r3_0[] = {
  0x78, 0xbf, 0x1e, 0x1c // It vc; add vc r6 r3 0
};
const byte kInstruction_add_cc_r6_r4_4[] = {
  0x38, 0xbf, 0x26, 0x1d // It cc; add cc r6 r4 4
};
const byte kInstruction_add_cc_r6_r5_0[] = {
  0x38, 0xbf, 0x2e, 0x1c // It cc; add cc r6 r5 0
};
const byte kInstruction_add_le_r2_r0_6[] = {
  0xd8, 0xbf, 0x82, 0x1d // It le; add le r2 r0 6
};
const byte kInstruction_add_gt_r5_r6_3[] = {
  0xc8, 0xbf, 0xf5, 0x1c // It gt; add gt r5 r6 3
};
const byte kInstruction_add_vs_r3_r4_0[] = {
  0x68, 0xbf, 0x23, 0x1c // It vs; add vs r3 r4 0
};
const byte kInstruction_add_le_r4_r4_1[] = {
  0xd8, 0xbf, 0x64, 0x1c // It le; add le r4 r4 1
};
const byte kInstruction_add_ge_r4_r0_6[] = {
  0xa8, 0xbf, 0x84, 0x1d // It ge; add ge r4 r0 6
};
const byte kInstruction_add_eq_r2_r3_2[] = {
  0x08, 0xbf, 0x9a, 0x1c // It eq; add eq r2 r3 2
};
const byte kInstruction_add_lt_r5_r1_6[] = {
  0xb8, 0xbf, 0x8d, 0x1d // It lt; add lt r5 r1 6
};
const byte kInstruction_add_le_r7_r2_0[] = {
  0xd8, 0xbf, 0x17, 0x1c // It le; add le r7 r2 0
};
const byte kInstruction_add_mi_r2_r4_6[] = {
  0x48, 0xbf, 0xa2, 0x1d // It mi; add mi r2 r4 6
};
const byte kInstruction_add_vc_r6_r5_4[] = {
  0x78, 0xbf, 0x2e, 0x1d // It vc; add vc r6 r5 4
};
const byte kInstruction_add_gt_r3_r6_5[] = {
  0xc8, 0xbf, 0x73, 0x1d // It gt; add gt r3 r6 5
};
const byte kInstruction_add_pl_r3_r2_0[] = {
  0x58, 0xbf, 0x13, 0x1c // It pl; add pl r3 r2 0
};
const byte kInstruction_add_ls_r3_r2_2[] = {
  0x98, 0xbf, 0x93, 0x1c // It ls; add ls r3 r2 2
};
const byte kInstruction_add_ge_r2_r7_7[] = {
  0xa8, 0xbf, 0xfa, 0x1d // It ge; add ge r2 r7 7
};
const byte kInstruction_add_vc_r3_r5_0[] = {
  0x78, 0xbf, 0x2b, 0x1c // It vc; add vc r3 r5 0
};
const byte kInstruction_add_cc_r4_r0_7[] = {
  0x38, 0xbf, 0xc4, 0x1d // It cc; add cc r4 r0 7
};
const byte kInstruction_add_ls_r3_r5_2[] = {
  0x98, 0xbf, 0xab, 0x1c // It ls; add ls r3 r5 2
};
const byte kInstruction_add_vc_r6_r7_6[] = {
  0x78, 0xbf, 0xbe, 0x1d // It vc; add vc r6 r7 6
};
const byte kInstruction_add_ls_r0_r3_6[] = {
  0x98, 0xbf, 0x98, 0x1d // It ls; add ls r0 r3 6
};
const byte kInstruction_add_hi_r6_r0_1[] = {
  0x88, 0xbf, 0x46, 0x1c // It hi; add hi r6 r0 1
};
const byte kInstruction_add_mi_r5_r0_6[] = {
  0x48, 0xbf, 0x85, 0x1d // It mi; add mi r5 r0 6
};
const byte kInstruction_add_mi_r2_r5_3[] = {
  0x48, 0xbf, 0xea, 0x1c // It mi; add mi r2 r5 3
};
const byte kInstruction_add_ge_r0_r7_0[] = {
  0xa8, 0xbf, 0x38, 0x1c // It ge; add ge r0 r7 0
};
const byte kInstruction_add_vc_r7_r3_3[] = {
  0x78, 0xbf, 0xdf, 0x1c // It vc; add vc r7 r3 3
};
const byte kInstruction_add_mi_r0_r2_5[] = {
  0x48, 0xbf, 0x50, 0x1d // It mi; add mi r0 r2 5
};
const byte kInstruction_add_cc_r4_r3_5[] = {
  0x38, 0xbf, 0x5c, 0x1d // It cc; add cc r4 r3 5
};
const byte kInstruction_add_pl_r0_r3_0[] = {
  0x58, 0xbf, 0x18, 0x1c // It pl; add pl r0 r3 0
};
const byte kInstruction_add_gt_r6_r3_1[] = {
  0xc8, 0xbf, 0x5e, 0x1c // It gt; add gt r6 r3 1
};
const byte kInstruction_add_lt_r3_r3_0[] = {
  0xb8, 0xbf, 0x1b, 0x1c // It lt; add lt r3 r3 0
};
const byte kInstruction_add_hi_r1_r5_6[] = {
  0x88, 0xbf, 0xa9, 0x1d // It hi; add hi r1 r5 6
};
const byte kInstruction_add_hi_r4_r7_0[] = {
  0x88, 0xbf, 0x3c, 0x1c // It hi; add hi r4 r7 0
};
const byte kInstruction_add_ge_r6_r2_2[] = {
  0xa8, 0xbf, 0x96, 0x1c // It ge; add ge r6 r2 2
};
const byte kInstruction_add_pl_r7_r4_4[] = {
  0x58, 0xbf, 0x27, 0x1d // It pl; add pl r7 r4 4
};
const byte kInstruction_add_cc_r5_r5_5[] = {
  0x38, 0xbf, 0x6d, 0x1d // It cc; add cc r5 r5 5
};
const byte kInstruction_add_pl_r6_r1_1[] = {
  0x58, 0xbf, 0x4e, 0x1c // It pl; add pl r6 r1 1
};
const byte kInstruction_add_le_r7_r6_3[] = {
  0xd8, 0xbf, 0xf7, 0x1c // It le; add le r7 r6 3
};
const byte kInstruction_add_lt_r4_r7_1[] = {
  0xb8, 0xbf, 0x7c, 0x1c // It lt; add lt r4 r7 1
};
const byte kInstruction_add_pl_r2_r4_7[] = {
  0x58, 0xbf, 0xe2, 0x1d // It pl; add pl r2 r4 7
};
const byte kInstruction_add_vc_r1_r3_0[] = {
  0x78, 0xbf, 0x19, 0x1c // It vc; add vc r1 r3 0
};
const byte kInstruction_add_pl_r7_r7_3[] = {
  0x58, 0xbf, 0xff, 0x1c // It pl; add pl r7 r7 3
};
const byte kInstruction_add_ls_r4_r1_3[] = {
  0x98, 0xbf, 0xcc, 0x1c // It ls; add ls r4 r1 3
};
const byte kInstruction_add_lt_r6_r6_2[] = {
  0xb8, 0xbf, 0xb6, 0x1c // It lt; add lt r6 r6 2
};
const byte kInstruction_add_mi_r4_r2_4[] = {
  0x48, 0xbf, 0x14, 0x1d // It mi; add mi r4 r2 4
};
const byte kInstruction_add_pl_r3_r4_7[] = {
  0x58, 0xbf, 0xe3, 0x1d // It pl; add pl r3 r4 7
};
const byte kInstruction_add_ls_r2_r6_6[] = {
  0x98, 0xbf, 0xb2, 0x1d // It ls; add ls r2 r6 6
};
const byte kInstruction_add_hi_r5_r7_6[] = {
  0x88, 0xbf, 0xbd, 0x1d // It hi; add hi r5 r7 6
};
const byte kInstruction_add_lt_r0_r4_1[] = {
  0xb8, 0xbf, 0x60, 0x1c // It lt; add lt r0 r4 1
};
const byte kInstruction_add_ge_r5_r5_6[] = {
  0xa8, 0xbf, 0xad, 0x1d // It ge; add ge r5 r5 6
};
const byte kInstruction_add_pl_r1_r7_3[] = {
  0x58, 0xbf, 0xf9, 0x1c // It pl; add pl r1 r7 3
};
const byte kInstruction_add_cs_r6_r2_1[] = {
  0x28, 0xbf, 0x56, 0x1c // It cs; add cs r6 r2 1
};
const byte kInstruction_add_ne_r1_r3_0[] = {
  0x18, 0xbf, 0x19, 0x1c // It ne; add ne r1 r3 0
};
const byte kInstruction_add_mi_r6_r5_7[] = {
  0x48, 0xbf, 0xee, 0x1d // It mi; add mi r6 r5 7
};
const byte kInstruction_add_gt_r2_r2_0[] = {
  0xc8, 0xbf, 0x12, 0x1c // It gt; add gt r2 r2 0
};
const byte kInstruction_add_ls_r6_r4_6[] = {
  0x98, 0xbf, 0xa6, 0x1d // It ls; add ls r6 r4 6
};
const byte kInstruction_add_pl_r2_r6_7[] = {
  0x58, 0xbf, 0xf2, 0x1d // It pl; add pl r2 r6 7
};
const byte kInstruction_add_ne_r1_r5_4[] = {
  0x18, 0xbf, 0x29, 0x1d // It ne; add ne r1 r5 4
};
const byte kInstruction_add_mi_r0_r0_5[] = {
  0x48, 0xbf, 0x40, 0x1d // It mi; add mi r0 r0 5
};
const byte kInstruction_add_eq_r5_r5_2[] = {
  0x08, 0xbf, 0xad, 0x1c // It eq; add eq r5 r5 2
};
const byte kInstruction_add_mi_r7_r7_3[] = {
  0x48, 0xbf, 0xff, 0x1c // It mi; add mi r7 r7 3
};
const byte kInstruction_add_vc_r7_r7_1[] = {
  0x78, 0xbf, 0x7f, 0x1c // It vc; add vc r7 r7 1
};
const byte kInstruction_add_vc_r5_r4_3[] = {
  0x78, 0xbf, 0xe5, 0x1c // It vc; add vc r5 r4 3
};
const byte kInstruction_add_mi_r5_r7_6[] = {
  0x48, 0xbf, 0xbd, 0x1d // It mi; add mi r5 r7 6
};
const byte kInstruction_add_vs_r2_r3_0[] = {
  0x68, 0xbf, 0x1a, 0x1c // It vs; add vs r2 r3 0
};
const byte kInstruction_add_ne_r2_r4_2[] = {
  0x18, 0xbf, 0xa2, 0x1c // It ne; add ne r2 r4 2
};
const byte kInstruction_add_eq_r6_r7_3[] = {
  0x08, 0xbf, 0xfe, 0x1c // It eq; add eq r6 r7 3
};
const byte kInstruction_add_ne_r0_r1_5[] = {
  0x18, 0xbf, 0x48, 0x1d // It ne; add ne r0 r1 5
};
const byte kInstruction_add_lt_r3_r2_6[] = {
  0xb8, 0xbf, 0x93, 0x1d // It lt; add lt r3 r2 6
};
const byte kInstruction_add_gt_r4_r2_1[] = {
  0xc8, 0xbf, 0x54, 0x1c // It gt; add gt r4 r2 1
};
const byte kInstruction_add_ls_r6_r1_0[] = {
  0x98, 0xbf, 0x0e, 0x1c // It ls; add ls r6 r1 0
};
const byte kInstruction_add_cc_r6_r2_3[] = {
  0x38, 0xbf, 0xd6, 0x1c // It cc; add cc r6 r2 3
};
const byte kInstruction_add_hi_r3_r2_7[] = {
  0x88, 0xbf, 0xd3, 0x1d // It hi; add hi r3 r2 7
};
const byte kInstruction_add_cc_r6_r7_3[] = {
  0x38, 0xbf, 0xfe, 0x1c // It cc; add cc r6 r7 3
};
const byte kInstruction_add_vs_r7_r6_4[] = {
  0x68, 0xbf, 0x37, 0x1d // It vs; add vs r7 r6 4
};
const byte kInstruction_add_gt_r5_r4_5[] = {
  0xc8, 0xbf, 0x65, 0x1d // It gt; add gt r5 r4 5
};
const byte kInstruction_add_vc_r5_r7_6[] = {
  0x78, 0xbf, 0xbd, 0x1d // It vc; add vc r5 r7 6
};
const byte kInstruction_add_hi_r0_r6_5[] = {
  0x88, 0xbf, 0x70, 0x1d // It hi; add hi r0 r6 5
};
const byte kInstruction_add_eq_r2_r7_3[] = {
  0x08, 0xbf, 0xfa, 0x1c // It eq; add eq r2 r7 3
};
const byte kInstruction_add_le_r6_r5_7[] = {
  0xd8, 0xbf, 0xee, 0x1d // It le; add le r6 r5 7
};
const byte kInstruction_add_ge_r0_r5_5[] = {
  0xa8, 0xbf, 0x68, 0x1d // It ge; add ge r0 r5 5
};
const byte kInstruction_add_pl_r1_r6_4[] = {
  0x58, 0xbf, 0x31, 0x1d // It pl; add pl r1 r6 4
};
const byte kInstruction_add_mi_r3_r0_6[] = {
  0x48, 0xbf, 0x83, 0x1d // It mi; add mi r3 r0 6
};
const byte kInstruction_add_cs_r7_r6_1[] = {
  0x28, 0xbf, 0x77, 0x1c // It cs; add cs r7 r6 1
};
const byte kInstruction_add_vc_r3_r6_3[] = {
  0x78, 0xbf, 0xf3, 0x1c // It vc; add vc r3 r6 3
};
const byte kInstruction_add_ls_r5_r1_4[] = {
  0x98, 0xbf, 0x0d, 0x1d // It ls; add ls r5 r1 4
};
const byte kInstruction_add_le_r6_r2_3[] = {
  0xd8, 0xbf, 0xd6, 0x1c // It le; add le r6 r2 3
};
const byte kInstruction_add_cc_r1_r6_4[] = {
  0x38, 0xbf, 0x31, 0x1d // It cc; add cc r1 r6 4
};
const byte kInstruction_add_mi_r0_r7_0[] = {
  0x48, 0xbf, 0x38, 0x1c // It mi; add mi r0 r7 0
};
const byte kInstruction_add_vc_r6_r2_2[] = {
  0x78, 0xbf, 0x96, 0x1c // It vc; add vc r6 r2 2
};
const byte kInstruction_add_gt_r3_r0_4[] = {
  0xc8, 0xbf, 0x03, 0x1d // It gt; add gt r3 r0 4
};
const byte kInstruction_add_cs_r4_r0_5[] = {
  0x28, 0xbf, 0x44, 0x1d // It cs; add cs r4 r0 5
};
const byte kInstruction_add_hi_r4_r4_6[] = {
  0x88, 0xbf, 0xa4, 0x1d // It hi; add hi r4 r4 6
};
const byte kInstruction_add_cc_r5_r5_1[] = {
  0x38, 0xbf, 0x6d, 0x1c // It cc; add cc r5 r5 1
};
const byte kInstruction_add_pl_r6_r3_4[] = {
  0x58, 0xbf, 0x1e, 0x1d // It pl; add pl r6 r3 4
};
const byte kInstruction_add_ge_r7_r4_0[] = {
  0xa8, 0xbf, 0x27, 0x1c // It ge; add ge r7 r4 0
};
const byte kInstruction_add_hi_r2_r1_3[] = {
  0x88, 0xbf, 0xca, 0x1c // It hi; add hi r2 r1 3
};
const byte kInstruction_add_le_r4_r3_1[] = {
  0xd8, 0xbf, 0x5c, 0x1c // It le; add le r4 r3 1
};
const byte kInstruction_add_gt_r0_r0_7[] = {
  0xc8, 0xbf, 0xc0, 0x1d // It gt; add gt r0 r0 7
};
const byte kInstruction_add_gt_r2_r0_6[] = {
  0xc8, 0xbf, 0x82, 0x1d // It gt; add gt r2 r0 6
};
const byte kInstruction_add_ge_r4_r3_5[] = {
  0xa8, 0xbf, 0x5c, 0x1d // It ge; add ge r4 r3 5
};
const byte kInstruction_add_le_r2_r0_7[] = {
  0xd8, 0xbf, 0xc2, 0x1d // It le; add le r2 r0 7
};
const byte kInstruction_add_vc_r7_r7_4[] = {
  0x78, 0xbf, 0x3f, 0x1d // It vc; add vc r7 r7 4
};
const byte kInstruction_add_lt_r6_r5_1[] = {
  0xb8, 0xbf, 0x6e, 0x1c // It lt; add lt r6 r5 1
};
const byte kInstruction_add_vs_r1_r0_4[] = {
  0x68, 0xbf, 0x01, 0x1d // It vs; add vs r1 r0 4
};
const byte kInstruction_add_vs_r1_r1_6[] = {
  0x68, 0xbf, 0x89, 0x1d // It vs; add vs r1 r1 6
};
const byte kInstruction_add_ge_r1_r0_6[] = {
  0xa8, 0xbf, 0x81, 0x1d // It ge; add ge r1 r0 6
};
const byte kInstruction_add_eq_r1_r6_0[] = {
  0x08, 0xbf, 0x31, 0x1c // It eq; add eq r1 r6 0
};
const byte kInstruction_add_cs_r0_r5_5[] = {
  0x28, 0xbf, 0x68, 0x1d // It cs; add cs r0 r5 5
};
const byte kInstruction_add_mi_r7_r6_5[] = {
  0x48, 0xbf, 0x77, 0x1d // It mi; add mi r7 r6 5
};
const byte kInstruction_add_vc_r1_r4_3[] = {
  0x78, 0xbf, 0xe1, 0x1c // It vc; add vc r1 r4 3
};
const byte kInstruction_add_hi_r1_r3_4[] = {
  0x88, 0xbf, 0x19, 0x1d // It hi; add hi r1 r3 4
};
const byte kInstruction_add_vs_r3_r6_3[] = {
  0x68, 0xbf, 0xf3, 0x1c // It vs; add vs r3 r6 3
};
const byte kInstruction_add_vc_r1_r6_1[] = {
  0x78, 0xbf, 0x71, 0x1c // It vc; add vc r1 r6 1
};
const byte kInstruction_add_mi_r3_r7_1[] = {
  0x48, 0xbf, 0x7b, 0x1c // It mi; add mi r3 r7 1
};
const byte kInstruction_add_eq_r7_r0_1[] = {
  0x08, 0xbf, 0x47, 0x1c // It eq; add eq r7 r0 1
};
const byte kInstruction_add_hi_r3_r7_7[] = {
  0x88, 0xbf, 0xfb, 0x1d // It hi; add hi r3 r7 7
};
const byte kInstruction_add_mi_r3_r4_3[] = {
  0x48, 0xbf, 0xe3, 0x1c // It mi; add mi r3 r4 3
};
const byte kInstruction_add_le_r3_r1_2[] = {
  0xd8, 0xbf, 0x8b, 0x1c // It le; add le r3 r1 2
};
const byte kInstruction_add_mi_r1_r3_5[] = {
  0x48, 0xbf, 0x59, 0x1d // It mi; add mi r1 r3 5
};
const byte kInstruction_add_vc_r2_r2_4[] = {
  0x78, 0xbf, 0x12, 0x1d // It vc; add vc r2 r2 4
};
const byte kInstruction_add_vc_r4_r7_2[] = {
  0x78, 0xbf, 0xbc, 0x1c // It vc; add vc r4 r7 2
};
const byte kInstruction_add_vs_r3_r7_0[] = {
  0x68, 0xbf, 0x3b, 0x1c // It vs; add vs r3 r7 0
};
const byte kInstruction_add_gt_r7_r1_3[] = {
  0xc8, 0xbf, 0xcf, 0x1c // It gt; add gt r7 r1 3
};
const byte kInstruction_add_hi_r7_r4_7[] = {
  0x88, 0xbf, 0xe7, 0x1d // It hi; add hi r7 r4 7
};
const byte kInstruction_add_hi_r1_r0_3[] = {
  0x88, 0xbf, 0xc1, 0x1c // It hi; add hi r1 r0 3
};
const byte kInstruction_add_ne_r0_r4_5[] = {
  0x18, 0xbf, 0x60, 0x1d // It ne; add ne r0 r4 5
};
const byte kInstruction_add_hi_r7_r6_3[] = {
  0x88, 0xbf, 0xf7, 0x1c // It hi; add hi r7 r6 3
};
const byte kInstruction_add_ne_r3_r4_4[] = {
  0x18, 0xbf, 0x23, 0x1d // It ne; add ne r3 r4 4
};
const byte kInstruction_add_pl_r2_r5_5[] = {
  0x58, 0xbf, 0x6a, 0x1d // It pl; add pl r2 r5 5
};
const byte kInstruction_add_ge_r6_r7_0[] = {
  0xa8, 0xbf, 0x3e, 0x1c // It ge; add ge r6 r7 0
};
const byte kInstruction_add_ne_r7_r3_4[] = {
  0x18, 0xbf, 0x1f, 0x1d // It ne; add ne r7 r3 4
};
const byte kInstruction_add_ls_r5_r2_6[] = {
  0x98, 0xbf, 0x95, 0x1d // It ls; add ls r5 r2 6
};
const byte kInstruction_add_cs_r6_r3_2[] = {
  0x28, 0xbf, 0x9e, 0x1c // It cs; add cs r6 r3 2
};
const byte kInstruction_add_vs_r0_r1_6[] = {
  0x68, 0xbf, 0x88, 0x1d // It vs; add vs r0 r1 6
};
const byte kInstruction_add_vc_r6_r3_7[] = {
  0x78, 0xbf, 0xde, 0x1d // It vc; add vc r6 r3 7
};
const byte kInstruction_add_eq_r3_r0_5[] = {
  0x08, 0xbf, 0x43, 0x1d // It eq; add eq r3 r0 5
};
const byte kInstruction_add_cc_r4_r5_0[] = {
  0x38, 0xbf, 0x2c, 0x1c // It cc; add cc r4 r5 0
};
const byte kInstruction_add_hi_r3_r1_2[] = {
  0x88, 0xbf, 0x8b, 0x1c // It hi; add hi r3 r1 2
};
const byte kInstruction_add_lt_r2_r5_5[] = {
  0xb8, 0xbf, 0x6a, 0x1d // It lt; add lt r2 r5 5
};
const byte kInstruction_add_le_r0_r3_3[] = {
  0xd8, 0xbf, 0xd8, 0x1c // It le; add le r0 r3 3
};
const byte kInstruction_add_vc_r2_r5_7[] = {
  0x78, 0xbf, 0xea, 0x1d // It vc; add vc r2 r5 7
};
const byte kInstruction_add_gt_r7_r1_7[] = {
  0xc8, 0xbf, 0xcf, 0x1d // It gt; add gt r7 r1 7
};
const byte kInstruction_add_le_r2_r1_5[] = {
  0xd8, 0xbf, 0x4a, 0x1d // It le; add le r2 r1 5
};
const byte kInstruction_add_mi_r4_r2_0[] = {
  0x48, 0xbf, 0x14, 0x1c // It mi; add mi r4 r2 0
};
const byte kInstruction_add_ne_r4_r3_2[] = {
  0x18, 0xbf, 0x9c, 0x1c // It ne; add ne r4 r3 2
};
const byte kInstruction_add_le_r3_r0_7[] = {
  0xd8, 0xbf, 0xc3, 0x1d // It le; add le r3 r0 7
};
const byte kInstruction_add_vc_r6_r5_7[] = {
  0x78, 0xbf, 0xee, 0x1d // It vc; add vc r6 r5 7
};
const byte kInstruction_add_lt_r5_r5_0[] = {
  0xb8, 0xbf, 0x2d, 0x1c // It lt; add lt r5 r5 0
};
const byte kInstruction_add_le_r5_r7_2[] = {
  0xd8, 0xbf, 0xbd, 0x1c // It le; add le r5 r7 2
};
const byte kInstruction_add_mi_r2_r6_4[] = {
  0x48, 0xbf, 0x32, 0x1d // It mi; add mi r2 r6 4
};
const byte kInstruction_add_ne_r5_r4_5[] = {
  0x18, 0xbf, 0x65, 0x1d // It ne; add ne r5 r4 5
};
const byte kInstruction_add_mi_r5_r7_2[] = {
  0x48, 0xbf, 0xbd, 0x1c // It mi; add mi r5 r7 2
};
const byte kInstruction_add_lt_r7_r1_1[] = {
  0xb8, 0xbf, 0x4f, 0x1c // It lt; add lt r7 r1 1
};
const byte kInstruction_add_cc_r3_r0_5[] = {
  0x38, 0xbf, 0x43, 0x1d // It cc; add cc r3 r0 5
};
const byte kInstruction_add_pl_r2_r2_7[] = {
  0x58, 0xbf, 0xd2, 0x1d // It pl; add pl r2 r2 7
};
const byte kInstruction_add_lt_r3_r5_0[] = {
  0xb8, 0xbf, 0x2b, 0x1c // It lt; add lt r3 r5 0
};
const byte kInstruction_add_cs_r7_r7_5[] = {
  0x28, 0xbf, 0x7f, 0x1d // It cs; add cs r7 r7 5
};
const byte kInstruction_add_ge_r6_r5_2[] = {
  0xa8, 0xbf, 0xae, 0x1c // It ge; add ge r6 r5 2
};
const byte kInstruction_add_pl_r2_r7_1[] = {
  0x58, 0xbf, 0x7a, 0x1c // It pl; add pl r2 r7 1
};
const byte kInstruction_add_gt_r1_r5_0[] = {
  0xc8, 0xbf, 0x29, 0x1c // It gt; add gt r1 r5 0
};
const byte kInstruction_add_hi_r4_r2_6[] = {
  0x88, 0xbf, 0x94, 0x1d // It hi; add hi r4 r2 6
};
const byte kInstruction_add_ls_r4_r6_1[] = {
  0x98, 0xbf, 0x74, 0x1c // It ls; add ls r4 r6 1
};
const byte kInstruction_add_vc_r4_r6_5[] = {
  0x78, 0xbf, 0x74, 0x1d // It vc; add vc r4 r6 5
};
const byte kInstruction_add_ne_r0_r2_7[] = {
  0x18, 0xbf, 0xd0, 0x1d // It ne; add ne r0 r2 7
};
const byte kInstruction_add_ne_r1_r7_3[] = {
  0x18, 0xbf, 0xf9, 0x1c // It ne; add ne r1 r7 3
};
const byte kInstruction_add_pl_r2_r1_2[] = {
  0x58, 0xbf, 0x8a, 0x1c // It pl; add pl r2 r1 2
};
const byte kInstruction_add_le_r6_r2_2[] = {
  0xd8, 0xbf, 0x96, 0x1c // It le; add le r6 r2 2
};
const byte kInstruction_add_ne_r3_r6_0[] = {
  0x18, 0xbf, 0x33, 0x1c // It ne; add ne r3 r6 0
};
const byte kInstruction_add_lt_r5_r6_6[] = {
  0xb8, 0xbf, 0xb5, 0x1d // It lt; add lt r5 r6 6
};
const byte kInstruction_add_eq_r3_r2_2[] = {
  0x08, 0xbf, 0x93, 0x1c // It eq; add eq r3 r2 2
};
const byte kInstruction_add_pl_r1_r6_7[] = {
  0x58, 0xbf, 0xf1, 0x1d // It pl; add pl r1 r6 7
};
const byte kInstruction_add_mi_r7_r0_6[] = {
  0x48, 0xbf, 0x87, 0x1d // It mi; add mi r7 r0 6
};
const byte kInstruction_add_le_r7_r2_7[] = {
  0xd8, 0xbf, 0xd7, 0x1d // It le; add le r7 r2 7
};
const byte kInstruction_add_hi_r0_r3_1[] = {
  0x88, 0xbf, 0x58, 0x1c // It hi; add hi r0 r3 1
};
const byte kInstruction_add_ls_r3_r6_4[] = {
  0x98, 0xbf, 0x33, 0x1d // It ls; add ls r3 r6 4
};
const byte kInstruction_add_eq_r6_r0_7[] = {
  0x08, 0xbf, 0xc6, 0x1d // It eq; add eq r6 r0 7
};
const byte kInstruction_add_ne_r1_r4_5[] = {
  0x18, 0xbf, 0x61, 0x1d // It ne; add ne r1 r4 5
};
const byte kInstruction_add_le_r1_r5_3[] = {
  0xd8, 0xbf, 0xe9, 0x1c // It le; add le r1 r5 3
};
const byte kInstruction_add_cc_r6_r1_7[] = {
  0x38, 0xbf, 0xce, 0x1d // It cc; add cc r6 r1 7
};
const byte kInstruction_add_hi_r2_r4_1[] = {
  0x88, 0xbf, 0x62, 0x1c // It hi; add hi r2 r4 1
};
const byte kInstruction_add_vs_r3_r7_2[] = {
  0x68, 0xbf, 0xbb, 0x1c // It vs; add vs r3 r7 2
};
const byte kInstruction_add_pl_r7_r6_1[] = {
  0x58, 0xbf, 0x77, 0x1c // It pl; add pl r7 r6 1
};
const byte kInstruction_add_ne_r7_r2_1[] = {
  0x18, 0xbf, 0x57, 0x1c // It ne; add ne r7 r2 1
};
const byte kInstruction_add_hi_r1_r0_0[] = {
  0x88, 0xbf, 0x01, 0x1c // It hi; add hi r1 r0 0
};
const byte kInstruction_add_vs_r2_r1_7[] = {
  0x68, 0xbf, 0xca, 0x1d // It vs; add vs r2 r1 7
};
const byte kInstruction_add_lt_r5_r4_2[] = {
  0xb8, 0xbf, 0xa5, 0x1c // It lt; add lt r5 r4 2
};
const byte kInstruction_add_mi_r3_r1_0[] = {
  0x48, 0xbf, 0x0b, 0x1c // It mi; add mi r3 r1 0
};
const byte kInstruction_add_ls_r2_r6_1[] = {
  0x98, 0xbf, 0x72, 0x1c // It ls; add ls r2 r6 1
};
const byte kInstruction_add_cc_r2_r2_1[] = {
  0x38, 0xbf, 0x52, 0x1c // It cc; add cc r2 r2 1
};
const byte kInstruction_add_pl_r4_r0_7[] = {
  0x58, 0xbf, 0xc4, 0x1d // It pl; add pl r4 r0 7
};
const byte kInstruction_add_cc_r6_r4_5[] = {
  0x38, 0xbf, 0x66, 0x1d // It cc; add cc r6 r4 5
};
const byte kInstruction_add_cc_r1_r3_6[] = {
  0x38, 0xbf, 0x99, 0x1d // It cc; add cc r1 r3 6
};
const byte kInstruction_add_lt_r0_r3_2[] = {
  0xb8, 0xbf, 0x98, 0x1c // It lt; add lt r0 r3 2
};
const byte kInstruction_add_ge_r5_r4_3[] = {
  0xa8, 0xbf, 0xe5, 0x1c // It ge; add ge r5 r4 3
};
const byte kInstruction_add_lt_r4_r4_2[] = {
  0xb8, 0xbf, 0xa4, 0x1c // It lt; add lt r4 r4 2
};
const byte kInstruction_add_gt_r3_r1_6[] = {
  0xc8, 0xbf, 0x8b, 0x1d // It gt; add gt r3 r1 6
};
const byte kInstruction_add_pl_r4_r2_1[] = {
  0x58, 0xbf, 0x54, 0x1c // It pl; add pl r4 r2 1
};
const byte kInstruction_add_cc_r3_r1_7[] = {
  0x38, 0xbf, 0xcb, 0x1d // It cc; add cc r3 r1 7
};
const byte kInstruction_add_mi_r1_r3_7[] = {
  0x48, 0xbf, 0xd9, 0x1d // It mi; add mi r1 r3 7
};
const byte kInstruction_add_cc_r2_r5_7[] = {
  0x38, 0xbf, 0xea, 0x1d // It cc; add cc r2 r5 7
};
const byte kInstruction_add_hi_r6_r5_0[] = {
  0x88, 0xbf, 0x2e, 0x1c // It hi; add hi r6 r5 0
};
const byte kInstruction_add_vc_r1_r0_7[] = {
  0x78, 0xbf, 0xc1, 0x1d // It vc; add vc r1 r0 7
};
const byte kInstruction_add_cs_r3_r0_5[] = {
  0x28, 0xbf, 0x43, 0x1d // It cs; add cs r3 r0 5
};
const byte kInstruction_add_pl_r1_r2_3[] = {
  0x58, 0xbf, 0xd1, 0x1c // It pl; add pl r1 r2 3
};
const byte kInstruction_add_pl_r2_r6_3[] = {
  0x58, 0xbf, 0xf2, 0x1c // It pl; add pl r2 r6 3
};
const byte kInstruction_add_ne_r7_r1_5[] = {
  0x18, 0xbf, 0x4f, 0x1d // It ne; add ne r7 r1 5
};
const byte kInstruction_add_hi_r5_r0_4[] = {
  0x88, 0xbf, 0x05, 0x1d // It hi; add hi r5 r0 4
};
const byte kInstruction_add_vc_r6_r1_0[] = {
  0x78, 0xbf, 0x0e, 0x1c // It vc; add vc r6 r1 0
};
const byte kInstruction_add_vc_r7_r3_2[] = {
  0x78, 0xbf, 0x9f, 0x1c // It vc; add vc r7 r3 2
};
const byte kInstruction_add_ge_r6_r6_6[] = {
  0xa8, 0xbf, 0xb6, 0x1d // It ge; add ge r6 r6 6
};
const byte kInstruction_add_ne_r6_r1_0[] = {
  0x18, 0xbf, 0x0e, 0x1c // It ne; add ne r6 r1 0
};
const byte kInstruction_add_eq_r5_r1_2[] = {
  0x08, 0xbf, 0x8d, 0x1c // It eq; add eq r5 r1 2
};
const byte kInstruction_add_hi_r2_r0_6[] = {
  0x88, 0xbf, 0x82, 0x1d // It hi; add hi r2 r0 6
};
const byte kInstruction_add_ne_r7_r4_5[] = {
  0x18, 0xbf, 0x67, 0x1d // It ne; add ne r7 r4 5
};
const byte kInstruction_add_cc_r0_r0_1[] = {
  0x38, 0xbf, 0x40, 0x1c // It cc; add cc r0 r0 1
};
const byte kInstruction_add_mi_r7_r5_1[] = {
  0x48, 0xbf, 0x6f, 0x1c // It mi; add mi r7 r5 1
};
const byte kInstruction_add_mi_r7_r0_0[] = {
  0x48, 0xbf, 0x07, 0x1c // It mi; add mi r7 r0 0
};
const byte kInstruction_add_ls_r0_r2_0[] = {
  0x98, 0xbf, 0x10, 0x1c // It ls; add ls r0 r2 0
};
const byte kInstruction_add_vs_r4_r7_3[] = {
  0x68, 0xbf, 0xfc, 0x1c // It vs; add vs r4 r7 3
};
const byte kInstruction_add_hi_r6_r2_4[] = {
  0x88, 0xbf, 0x16, 0x1d // It hi; add hi r6 r2 4
};
const byte kInstruction_add_eq_r4_r1_1[] = {
  0x08, 0xbf, 0x4c, 0x1c // It eq; add eq r4 r1 1
};
const byte kInstruction_add_vs_r7_r5_1[] = {
  0x68, 0xbf, 0x6f, 0x1c // It vs; add vs r7 r5 1
};
const byte kInstruction_add_ne_r0_r4_7[] = {
  0x18, 0xbf, 0xe0, 0x1d // It ne; add ne r0 r4 7
};
const byte kInstruction_add_mi_r2_r7_7[] = {
  0x48, 0xbf, 0xfa, 0x1d // It mi; add mi r2 r7 7
};
const byte kInstruction_add_mi_r3_r2_5[] = {
  0x48, 0xbf, 0x53, 0x1d // It mi; add mi r3 r2 5
};
const byte kInstruction_add_le_r7_r5_5[] = {
  0xd8, 0xbf, 0x6f, 0x1d // It le; add le r7 r5 5
};
const byte kInstruction_add_gt_r0_r5_7[] = {
  0xc8, 0xbf, 0xe8, 0x1d // It gt; add gt r0 r5 7
};
const byte kInstruction_add_vc_r0_r4_7[] = {
  0x78, 0xbf, 0xe0, 0x1d // It vc; add vc r0 r4 7
};
const byte kInstruction_add_ls_r5_r7_2[] = {
  0x98, 0xbf, 0xbd, 0x1c // It ls; add ls r5 r7 2
};
const byte kInstruction_add_hi_r0_r6_1[] = {
  0x88, 0xbf, 0x70, 0x1c // It hi; add hi r0 r6 1
};
const byte kInstruction_add_cc_r1_r4_5[] = {
  0x38, 0xbf, 0x61, 0x1d // It cc; add cc r1 r4 5
};
const byte kInstruction_add_vs_r2_r2_7[] = {
  0x68, 0xbf, 0xd2, 0x1d // It vs; add vs r2 r2 7
};
const byte kInstruction_add_ge_r6_r6_1[] = {
  0xa8, 0xbf, 0x76, 0x1c // It ge; add ge r6 r6 1
};
const byte kInstruction_add_hi_r7_r1_1[] = {
  0x88, 0xbf, 0x4f, 0x1c // It hi; add hi r7 r1 1
};
const byte kInstruction_add_le_r3_r4_3[] = {
  0xd8, 0xbf, 0xe3, 0x1c // It le; add le r3 r4 3
};
const byte kInstruction_add_mi_r2_r1_3[] = {
  0x48, 0xbf, 0xca, 0x1c // It mi; add mi r2 r1 3
};
const byte kInstruction_add_cs_r7_r7_4[] = {
  0x28, 0xbf, 0x3f, 0x1d // It cs; add cs r7 r7 4
};
const byte kInstruction_add_cs_r2_r5_0[] = {
  0x28, 0xbf, 0x2a, 0x1c // It cs; add cs r2 r5 0
};
const byte kInstruction_add_hi_r7_r6_4[] = {
  0x88, 0xbf, 0x37, 0x1d // It hi; add hi r7 r6 4
};
const byte kInstruction_add_lt_r7_r5_1[] = {
  0xb8, 0xbf, 0x6f, 0x1c // It lt; add lt r7 r5 1
};
const byte kInstruction_add_gt_r0_r0_1[] = {
  0xc8, 0xbf, 0x40, 0x1c // It gt; add gt r0 r0 1
};
const byte kInstruction_add_vs_r1_r2_6[] = {
  0x68, 0xbf, 0x91, 0x1d // It vs; add vs r1 r2 6
};
const byte kInstruction_add_vs_r3_r3_1[] = {
  0x68, 0xbf, 0x5b, 0x1c // It vs; add vs r3 r3 1
};
const byte kInstruction_add_ls_r4_r2_2[] = {
  0x98, 0xbf, 0x94, 0x1c // It ls; add ls r4 r2 2
};
const byte kInstruction_add_hi_r6_r3_2[] = {
  0x88, 0xbf, 0x9e, 0x1c // It hi; add hi r6 r3 2
};
const byte kInstruction_add_gt_r7_r5_3[] = {
  0xc8, 0xbf, 0xef, 0x1c // It gt; add gt r7 r5 3
};
const byte kInstruction_add_vc_r1_r6_7[] = {
  0x78, 0xbf, 0xf1, 0x1d // It vc; add vc r1 r6 7
};
const byte kInstruction_add_eq_r2_r2_0[] = {
  0x08, 0xbf, 0x12, 0x1c // It eq; add eq r2 r2 0
};
const byte kInstruction_add_hi_r7_r6_1[] = {
  0x88, 0xbf, 0x77, 0x1c // It hi; add hi r7 r6 1
};
const byte kInstruction_add_gt_r4_r6_7[] = {
  0xc8, 0xbf, 0xf4, 0x1d // It gt; add gt r4 r6 7
};
const byte kInstruction_add_le_r7_r0_6[] = {
  0xd8, 0xbf, 0x87, 0x1d // It le; add le r7 r0 6
};
const byte kInstruction_add_eq_r7_r1_7[] = {
  0x08, 0xbf, 0xcf, 0x1d // It eq; add eq r7 r1 7
};
const byte kInstruction_add_mi_r0_r6_7[] = {
  0x48, 0xbf, 0xf0, 0x1d // It mi; add mi r0 r6 7
};
const TestResult kReferenceadd[] = {
  {
    ARRAY_SIZE(kInstruction_add_pl_r0_r0_7),
    kInstruction_add_pl_r0_r0_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r0_r2_5),
    kInstruction_add_cs_r0_r2_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r1_r6_0),
    kInstruction_add_ls_r1_r6_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r5_r1_6),
    kInstruction_add_cc_r5_r1_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r2_0),
    kInstruction_add_cs_r5_r2_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r5_r6_7),
    kInstruction_add_vs_r5_r6_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r2_r0_5),
    kInstruction_add_ls_r2_r0_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r3_r4_5),
    kInstruction_add_eq_r3_r4_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r4_r5_7),
    kInstruction_add_lt_r4_r5_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r6_r7_1),
    kInstruction_add_lt_r6_r7_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r1_r0_3),
    kInstruction_add_eq_r1_r0_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r4_r1_0),
    kInstruction_add_le_r4_r1_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r5_r2_5),
    kInstruction_add_vc_r5_r2_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r1_r1_5),
    kInstruction_add_cs_r1_r1_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r5_4),
    kInstruction_add_cs_r2_r5_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r7_r5_7),
    kInstruction_add_ge_r7_r5_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r3_r4_4),
    kInstruction_add_eq_r3_r4_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r1_r0_2),
    kInstruction_add_ls_r1_r0_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r1_r2_0),
    kInstruction_add_vc_r1_r2_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r6_r6_4),
    kInstruction_add_ls_r6_r6_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r3_r1_0),
    kInstruction_add_hi_r3_r1_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r7_r1_0),
    kInstruction_add_ge_r7_r1_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r2_r0_0),
    kInstruction_add_ge_r2_r0_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r1_r7_0),
    kInstruction_add_ge_r1_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r2_0),
    kInstruction_add_lt_r5_r2_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r6_r3_0),
    kInstruction_add_ls_r6_r3_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r5_r7_4),
    kInstruction_add_ne_r5_r7_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r5_r3_2),
    kInstruction_add_le_r5_r3_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r6_r1_4),
    kInstruction_add_eq_r6_r1_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r6_r0_1),
    kInstruction_add_gt_r6_r0_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r5_r2_2),
    kInstruction_add_pl_r5_r2_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r7_r2_0),
    kInstruction_add_ls_r7_r2_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r4_r7_6),
    kInstruction_add_hi_r4_r7_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r1_r2_5),
    kInstruction_add_hi_r1_r2_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r3_r0_3),
    kInstruction_add_ls_r3_r0_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r5_r5_7),
    kInstruction_add_le_r5_r5_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r5_r0_2),
    kInstruction_add_pl_r5_r0_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r3_r1_7),
    kInstruction_add_cs_r3_r1_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r4_r6_6),
    kInstruction_add_le_r4_r6_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r7_r5_2),
    kInstruction_add_lt_r7_r5_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r5_r4_3),
    kInstruction_add_le_r5_r4_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r7_0),
    kInstruction_add_ne_r4_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r3_5),
    kInstruction_add_gt_r0_r3_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r0_r4_5),
    kInstruction_add_hi_r0_r4_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r4_r0_6),
    kInstruction_add_cs_r4_r0_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r6_r3_0),
    kInstruction_add_ne_r6_r3_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r0_r5_3),
    kInstruction_add_ge_r0_r5_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r4_r2_7),
    kInstruction_add_vc_r4_r2_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r3_r0_6),
    kInstruction_add_le_r3_r0_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r4_5),
    kInstruction_add_lt_r0_r4_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r7_r0_4),
    kInstruction_add_cs_r7_r0_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r0_r0_6),
    kInstruction_add_ge_r0_r0_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r2_r6_0),
    kInstruction_add_hi_r2_r6_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r1_r4_0),
    kInstruction_add_hi_r1_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r1_r5_5),
    kInstruction_add_lt_r1_r5_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r0_r6_7),
    kInstruction_add_eq_r0_r6_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r2_r4_6),
    kInstruction_add_hi_r2_r4_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r4_r4_2),
    kInstruction_add_ls_r4_r4_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r1_r5_5),
    kInstruction_add_gt_r1_r5_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r5_r4_2),
    kInstruction_add_gt_r5_r4_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r3_6),
    kInstruction_add_gt_r0_r3_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r6_r2_6),
    kInstruction_add_hi_r6_r2_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r7_5),
    kInstruction_add_le_r0_r7_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r2_r7_6),
    kInstruction_add_mi_r2_r7_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r5_r6_2),
    kInstruction_add_le_r5_r6_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r0_r4_5),
    kInstruction_add_eq_r0_r4_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r4_r5_2),
    kInstruction_add_hi_r4_r5_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r7_0),
    kInstruction_add_gt_r0_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r6_r4_0),
    kInstruction_add_eq_r6_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r2_r2_4),
    kInstruction_add_mi_r2_r2_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r7_r4_2),
    kInstruction_add_vs_r7_r4_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r5_r3_6),
    kInstruction_add_ne_r5_r3_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r0_r3_2),
    kInstruction_add_ge_r0_r3_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r1_r5_5),
    kInstruction_add_cc_r1_r5_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r7_r3_5),
    kInstruction_add_ge_r7_r3_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r6_r4_0),
    kInstruction_add_ls_r6_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r1_4),
    kInstruction_add_cs_r5_r1_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r6_r7_0),
    kInstruction_add_ls_r6_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r5_r7_5),
    kInstruction_add_cc_r5_r7_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r0_r2_5),
    kInstruction_add_eq_r0_r2_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r1_r6_6),
    kInstruction_add_hi_r1_r6_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r4_r6_7),
    kInstruction_add_vs_r4_r6_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r1_r5_4),
    kInstruction_add_lt_r1_r5_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r1_r2_7),
    kInstruction_add_gt_r1_r2_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r4_r5_4),
    kInstruction_add_cc_r4_r5_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r6_r5_2),
    kInstruction_add_pl_r6_r5_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r1_0),
    kInstruction_add_lt_r5_r1_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r1_r7_1),
    kInstruction_add_vc_r1_r7_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r1_r2_6),
    kInstruction_add_eq_r1_r2_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r0_r4_2),
    kInstruction_add_ls_r0_r4_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r2_r6_0),
    kInstruction_add_vs_r2_r6_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r6_r3_6),
    kInstruction_add_le_r6_r3_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r6_r7_1),
    kInstruction_add_mi_r6_r7_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r2_r0_5),
    kInstruction_add_lt_r2_r0_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r6_1),
    kInstruction_add_le_r0_r6_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r6_r6_6),
    kInstruction_add_hi_r6_r6_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r5_r6_1),
    kInstruction_add_eq_r5_r6_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r2_r1_4),
    kInstruction_add_eq_r2_r1_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r5_r6_5),
    kInstruction_add_gt_r5_r6_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r3_r4_3),
    kInstruction_add_vc_r3_r4_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r0_r1_4),
    kInstruction_add_cc_r0_r1_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r3_r1_7),
    kInstruction_add_hi_r3_r1_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r2_r3_5),
    kInstruction_add_ge_r2_r3_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r1_r0_5),
    kInstruction_add_mi_r1_r0_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r3_r0_3),
    kInstruction_add_vc_r3_r0_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r0_r5_7),
    kInstruction_add_vs_r0_r5_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r2_r1_5),
    kInstruction_add_eq_r2_r1_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r3_r0_0),
    kInstruction_add_ne_r3_r0_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r0_r6_1),
    kInstruction_add_vs_r0_r6_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r6_r1_4),
    kInstruction_add_cs_r6_r1_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r2_4),
    kInstruction_add_cs_r5_r2_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r4_r4_7),
    kInstruction_add_cs_r4_r4_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r2_4),
    kInstruction_add_vs_r1_r2_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r1_6),
    kInstruction_add_ne_r0_r1_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r2_r4_0),
    kInstruction_add_ge_r2_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r0_r4_4),
    kInstruction_add_pl_r0_r4_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r6_r2_6),
    kInstruction_add_vs_r6_r2_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r4_r0_6),
    kInstruction_add_lt_r4_r0_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r3_r1_2),
    kInstruction_add_cc_r3_r1_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r6_r1_2),
    kInstruction_add_hi_r6_r1_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r0_r5_4),
    kInstruction_add_cc_r0_r5_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r3_r3_2),
    kInstruction_add_lt_r3_r3_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r0_r2_3),
    kInstruction_add_vc_r0_r2_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r5_r3_5),
    kInstruction_add_ne_r5_r3_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r4_r3_6),
    kInstruction_add_gt_r4_r3_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r5_r5_7),
    kInstruction_add_eq_r5_r5_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r6_r1_6),
    kInstruction_add_cc_r6_r1_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r4_r4_4),
    kInstruction_add_ls_r4_r4_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r7_r3_2),
    kInstruction_add_vs_r7_r3_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r0_r4_1),
    kInstruction_add_cc_r0_r4_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r4_r2_3),
    kInstruction_add_ge_r4_r2_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r1_r6_0),
    kInstruction_add_pl_r1_r6_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r1_r5_4),
    kInstruction_add_eq_r1_r5_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r0_r1_7),
    kInstruction_add_pl_r0_r1_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r6_2),
    kInstruction_add_cs_r5_r6_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r1_r2_3),
    kInstruction_add_mi_r1_r2_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r3_r4_0),
    kInstruction_add_mi_r3_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r5_r1_1),
    kInstruction_add_ne_r5_r1_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r3_r5_4),
    kInstruction_add_cc_r3_r5_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r1_4),
    kInstruction_add_ne_r4_r1_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r4_r4_4),
    kInstruction_add_vs_r4_r4_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r4_r2_6),
    kInstruction_add_mi_r4_r2_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r4_r5_3),
    kInstruction_add_hi_r4_r5_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r2_r5_7),
    kInstruction_add_pl_r2_r5_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r1_r4_1),
    kInstruction_add_cs_r1_r4_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r7_r5_0),
    kInstruction_add_hi_r7_r5_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r5_r6_4),
    kInstruction_add_mi_r5_r6_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r3_r2_1),
    kInstruction_add_pl_r3_r2_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r1_r5_2),
    kInstruction_add_le_r1_r5_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r1_r0_0),
    kInstruction_add_le_r1_r0_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r5_r5_4),
    kInstruction_add_vc_r5_r5_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r7_r2_5),
    kInstruction_add_vc_r7_r2_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r5_r4_2),
    kInstruction_add_hi_r5_r4_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r4_r4_1),
    kInstruction_add_ls_r4_r4_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r6_r6_4),
    kInstruction_add_cc_r6_r6_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r6_r7_0),
    kInstruction_add_mi_r6_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r0_r1_2),
    kInstruction_add_eq_r0_r1_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r7_r2_5),
    kInstruction_add_cc_r7_r2_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r6_r7_7),
    kInstruction_add_ls_r6_r7_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r1_r1_3),
    kInstruction_add_cs_r1_r1_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r7_r1_2),
    kInstruction_add_ls_r7_r1_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r3_r1_3),
    kInstruction_add_gt_r3_r1_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r4_r7_0),
    kInstruction_add_gt_r4_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r4_r5_2),
    kInstruction_add_pl_r4_r5_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r4_r7_6),
    kInstruction_add_vs_r4_r7_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r5_r1_6),
    kInstruction_add_vc_r5_r1_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r6_r5_5),
    kInstruction_add_vs_r6_r5_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r5_r0_1),
    kInstruction_add_eq_r5_r0_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r4_r6_1),
    kInstruction_add_mi_r4_r6_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r1_r5_7),
    kInstruction_add_cs_r1_r5_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r3_6),
    kInstruction_add_lt_r0_r3_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r7_r0_6),
    kInstruction_add_eq_r7_r0_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r3_r5_3),
    kInstruction_add_vc_r3_r5_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r4_r7_7),
    kInstruction_add_gt_r4_r7_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r1_1),
    kInstruction_add_ne_r4_r1_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r7_4),
    kInstruction_add_ge_r6_r7_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r2_r0_2),
    kInstruction_add_le_r2_r0_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r7_r5_7),
    kInstruction_add_mi_r7_r5_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r3_6),
    kInstruction_add_lt_r5_r3_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r4_7),
    kInstruction_add_vs_r1_r4_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r2_4),
    kInstruction_add_cs_r2_r2_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r1_r0_1),
    kInstruction_add_cs_r1_r0_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r7_4),
    kInstruction_add_ne_r0_r7_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r6_r0_4),
    kInstruction_add_vs_r6_r0_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r7_r1_2),
    kInstruction_add_vc_r7_r1_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r5_r0_4),
    kInstruction_add_ls_r5_r0_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r1_1),
    kInstruction_add_le_r0_r1_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r7_r3_1),
    kInstruction_add_cs_r7_r3_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r7_r5_3),
    kInstruction_add_cs_r7_r5_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r3_r7_2),
    kInstruction_add_gt_r3_r7_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r3_r4_1),
    kInstruction_add_eq_r3_r4_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r3_r4_6),
    kInstruction_add_mi_r3_r4_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r0_r4_3),
    kInstruction_add_eq_r0_r4_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r7_6),
    kInstruction_add_le_r0_r7_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r5_r6_3),
    kInstruction_add_le_r5_r6_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r5_3),
    kInstruction_add_vs_r3_r5_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r3_r4_3),
    kInstruction_add_pl_r3_r4_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r2_r3_0),
    kInstruction_add_le_r2_r3_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r5_r4_4),
    kInstruction_add_vc_r5_r4_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r5_r0_6),
    kInstruction_add_le_r5_r0_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r6_r5_6),
    kInstruction_add_vs_r6_r5_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r2_r2_3),
    kInstruction_add_ge_r2_r2_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r5_r4_5),
    kInstruction_add_vs_r5_r4_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r4_2),
    kInstruction_add_vc_r6_r4_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r2_2),
    kInstruction_add_ne_r4_r2_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r5_r7_7),
    kInstruction_add_ge_r5_r7_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r1_r7_7),
    kInstruction_add_ne_r1_r7_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r1_2),
    kInstruction_add_vc_r6_r1_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r2_r4_0),
    kInstruction_add_mi_r2_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r6_r3_1),
    kInstruction_add_hi_r6_r3_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r5_r1_5),
    kInstruction_add_cc_r5_r1_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r3_r5_1),
    kInstruction_add_mi_r3_r5_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r0_r5_3),
    kInstruction_add_vs_r0_r5_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r1_4),
    kInstruction_add_gt_r0_r1_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r7_r5_4),
    kInstruction_add_vs_r7_r5_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r1_r5_3),
    kInstruction_add_mi_r1_r5_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r6_r6_0),
    kInstruction_add_hi_r6_r6_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r6_r4_3),
    kInstruction_add_gt_r6_r4_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r3_r2_6),
    kInstruction_add_gt_r3_r2_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r3_r7_7),
    kInstruction_add_mi_r3_r7_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r4_r6_0),
    kInstruction_add_gt_r4_r6_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r6_r4_0),
    kInstruction_add_cs_r6_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r6_r2_0),
    kInstruction_add_ne_r6_r2_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r3_r3_0),
    kInstruction_add_ne_r3_r3_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r2_r3_5),
    kInstruction_add_gt_r2_r3_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r5_r0_5),
    kInstruction_add_ne_r5_r0_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r7_r7_4),
    kInstruction_add_le_r7_r7_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r7_r3_0),
    kInstruction_add_ge_r7_r3_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r4_r5_6),
    kInstruction_add_le_r4_r5_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r3_r2_6),
    kInstruction_add_ls_r3_r2_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r4_r5_7),
    kInstruction_add_gt_r4_r5_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r2_r1_3),
    kInstruction_add_eq_r2_r1_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r3_r0_6),
    kInstruction_add_lt_r3_r0_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r0_2),
    kInstruction_add_ne_r4_r0_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r7_r6_5),
    kInstruction_add_lt_r7_r6_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r6_r0_6),
    kInstruction_add_hi_r6_r0_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r6_7),
    kInstruction_add_vs_r1_r6_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r0_r6_6),
    kInstruction_add_cc_r0_r6_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r1_r7_6),
    kInstruction_add_ne_r1_r7_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r3_1),
    kInstruction_add_cs_r2_r3_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r3_r7_7),
    kInstruction_add_lt_r3_r7_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r3_r3_0),
    kInstruction_add_cc_r3_r3_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r4_r3_6),
    kInstruction_add_eq_r4_r3_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r0_r5_7),
    kInstruction_add_cs_r0_r5_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r1_r2_4),
    kInstruction_add_gt_r1_r2_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r2_r1_5),
    kInstruction_add_cc_r2_r1_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r7_r5_5),
    kInstruction_add_pl_r7_r5_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r5_r7_6),
    kInstruction_add_pl_r5_r7_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r2_7),
    kInstruction_add_vc_r2_r2_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r4_r4_0),
    kInstruction_add_vs_r4_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r6_5),
    kInstruction_add_vc_r2_r6_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r1_r7_0),
    kInstruction_add_gt_r1_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r4_r1_7),
    kInstruction_add_pl_r4_r1_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r4_r3_2),
    kInstruction_add_lt_r4_r3_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r0_r2_4),
    kInstruction_add_ge_r0_r2_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r7_r5_7),
    kInstruction_add_ne_r7_r5_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r2_r1_6),
    kInstruction_add_gt_r2_r1_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r4_r1_6),
    kInstruction_add_eq_r4_r1_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r7_3),
    kInstruction_add_vs_r3_r7_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r0_r7_3),
    kInstruction_add_mi_r0_r7_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r3_r5_6),
    kInstruction_add_eq_r3_r5_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r6_r2_3),
    kInstruction_add_lt_r6_r2_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r7_r5_5),
    kInstruction_add_lt_r7_r5_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r5_6),
    kInstruction_add_vc_r6_r5_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r0_7),
    kInstruction_add_cs_r2_r0_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r4_r2_3),
    kInstruction_add_hi_r4_r2_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r5_r4_7),
    kInstruction_add_vs_r5_r4_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r7_r1_2),
    kInstruction_add_ge_r7_r1_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r6_r4_6),
    kInstruction_add_cc_r6_r4_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r4_r1_0),
    kInstruction_add_ls_r4_r1_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r1_r1_1),
    kInstruction_add_cc_r1_r1_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r2_r3_5),
    kInstruction_add_ls_r2_r3_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r2_r0_6),
    kInstruction_add_cc_r2_r0_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r6_r5_4),
    kInstruction_add_gt_r6_r5_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r3_r3_3),
    kInstruction_add_le_r3_r3_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r4_r5_7),
    kInstruction_add_ls_r4_r5_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r3_r7_0),
    kInstruction_add_ls_r3_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r4_1),
    kInstruction_add_lt_r5_r4_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r0_r2_3),
    kInstruction_add_hi_r0_r2_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r7_r2_2),
    kInstruction_add_ne_r7_r2_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r6_r4_4),
    kInstruction_add_gt_r6_r4_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r5_r1_0),
    kInstruction_add_cc_r5_r1_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r6_r5_3),
    kInstruction_add_le_r6_r5_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r3_r4_4),
    kInstruction_add_cs_r3_r4_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r4_r7_5),
    kInstruction_add_mi_r4_r7_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r1_r3_6),
    kInstruction_add_mi_r1_r3_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r0_r7_5),
    kInstruction_add_hi_r0_r7_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r4_r3_2),
    kInstruction_add_ls_r4_r3_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r7_6),
    kInstruction_add_ne_r4_r7_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r4_r5_5),
    kInstruction_add_cs_r4_r5_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r3_r5_0),
    kInstruction_add_mi_r3_r5_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r0_1),
    kInstruction_add_cs_r5_r0_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r3_r2_6),
    kInstruction_add_pl_r3_r2_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r6_5),
    kInstruction_add_vs_r1_r6_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r4_r0_2),
    kInstruction_add_le_r4_r0_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r3_r7_7),
    kInstruction_add_eq_r3_r7_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r6_r4_7),
    kInstruction_add_lt_r6_r4_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r2_r0_5),
    kInstruction_add_hi_r2_r0_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r1_r3_3),
    kInstruction_add_ge_r1_r3_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r6_r3_0),
    kInstruction_add_pl_r6_r3_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r1_r0_4),
    kInstruction_add_hi_r1_r0_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r3_r5_3),
    kInstruction_add_lt_r3_r5_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r2_r7_3),
    kInstruction_add_ge_r2_r7_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r0_r7_4),
    kInstruction_add_mi_r0_r7_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r7_r5_1),
    kInstruction_add_pl_r7_r5_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r3_4),
    kInstruction_add_lt_r0_r3_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r6_r4_2),
    kInstruction_add_mi_r6_r4_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r4_r4_1),
    kInstruction_add_vc_r4_r4_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r1_r5_3),
    kInstruction_add_gt_r1_r5_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r2_r0_7),
    kInstruction_add_cc_r2_r0_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r6_r4_0),
    kInstruction_add_le_r6_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r6_2),
    kInstruction_add_le_r0_r6_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r1_r1_7),
    kInstruction_add_le_r1_r1_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r1_5),
    kInstruction_add_vs_r1_r1_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r6_r0_2),
    kInstruction_add_cs_r6_r0_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r0_7),
    kInstruction_add_vs_r3_r0_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_4),
    kInstruction_add_vs_r3_r3_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r4_r2_7),
    kInstruction_add_cs_r4_r2_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r4_r0_1),
    kInstruction_add_mi_r4_r0_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r2_r4_1),
    kInstruction_add_gt_r2_r4_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r6_0),
    kInstruction_add_ne_r4_r6_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r4_r1_5),
    kInstruction_add_cs_r4_r1_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r2_r6_3),
    kInstruction_add_cc_r2_r6_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r1_0),
    kInstruction_add_ge_r6_r1_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r2_r5_3),
    kInstruction_add_pl_r2_r5_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r1_r7_4),
    kInstruction_add_ne_r1_r7_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r2_r6_0),
    kInstruction_add_le_r2_r6_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r7_r1_6),
    kInstruction_add_gt_r7_r1_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r0_r3_7),
    kInstruction_add_cs_r0_r3_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r1_r1_0),
    kInstruction_add_mi_r1_r1_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r4_r0_5),
    kInstruction_add_eq_r4_r0_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r0_6),
    kInstruction_add_lt_r5_r0_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r6_6),
    kInstruction_add_vc_r2_r6_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r1_r2_7),
    kInstruction_add_cs_r1_r2_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r6_r6_3),
    kInstruction_add_ls_r6_r6_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r0_4),
    kInstruction_add_lt_r5_r0_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r1_r6_0),
    kInstruction_add_gt_r1_r6_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r7_r2_2),
    kInstruction_add_vs_r7_r2_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r1_r3_5),
    kInstruction_add_gt_r1_r3_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r2_r6_5),
    kInstruction_add_mi_r2_r6_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r3_r6_2),
    kInstruction_add_cs_r3_r6_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r6_2),
    kInstruction_add_cs_r2_r6_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r7_r3_6),
    kInstruction_add_cs_r7_r3_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r7_r7_6),
    kInstruction_add_cs_r7_r7_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r1_r1_4),
    kInstruction_add_eq_r1_r1_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r3_r2_2),
    kInstruction_add_mi_r3_r2_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r5_r7_0),
    kInstruction_add_vc_r5_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r1_r6_7),
    kInstruction_add_hi_r1_r6_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r5_r7_3),
    kInstruction_add_vs_r5_r7_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r1_r7_2),
    kInstruction_add_gt_r1_r7_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r4_r5_0),
    kInstruction_add_vc_r4_r5_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r6_r7_0),
    kInstruction_add_le_r6_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r1_5),
    kInstruction_add_ge_r6_r1_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r0_r2_1),
    kInstruction_add_cc_r0_r2_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r7_r3_5),
    kInstruction_add_le_r7_r3_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r7_r1_2),
    kInstruction_add_hi_r7_r1_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r7_r5_0),
    kInstruction_add_cc_r7_r5_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r6_0),
    kInstruction_add_gt_r0_r6_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r6_r5_4),
    kInstruction_add_ne_r6_r5_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r7_r0_1),
    kInstruction_add_cc_r7_r0_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r6_r6_5),
    kInstruction_add_eq_r6_r6_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r4_r3_0),
    kInstruction_add_pl_r4_r3_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r2_r3_6),
    kInstruction_add_ne_r2_r3_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r2_1),
    kInstruction_add_le_r0_r2_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r0_r1_5),
    kInstruction_add_eq_r0_r1_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r5_r6_2),
    kInstruction_add_ls_r5_r6_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r3_r4_5),
    kInstruction_add_ne_r3_r4_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r1_r4_2),
    kInstruction_add_gt_r1_r4_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r3_r5_2),
    kInstruction_add_le_r3_r5_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r0_r5_5),
    kInstruction_add_vc_r0_r5_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r5_r3_6),
    kInstruction_add_pl_r5_r3_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r0_r1_4),
    kInstruction_add_eq_r0_r1_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r0_r0_7),
    kInstruction_add_cs_r0_r0_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r6_r5_6),
    kInstruction_add_lt_r6_r5_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r1_2),
    kInstruction_add_vs_r1_r1_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r4_r5_0),
    kInstruction_add_mi_r4_r5_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r3_r0_4),
    kInstruction_add_vc_r3_r0_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r2_r2_3),
    kInstruction_add_le_r2_r2_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r3_r7_5),
    kInstruction_add_gt_r3_r7_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r0_r4_0),
    kInstruction_add_cs_r0_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r5_r0_2),
    kInstruction_add_gt_r5_r0_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r4_r1_1),
    kInstruction_add_vs_r4_r1_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r7_r3_5),
    kInstruction_add_cc_r7_r3_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r5_r0_3),
    kInstruction_add_ls_r5_r0_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r2_r4_3),
    kInstruction_add_vs_r2_r4_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r3_r3_4),
    kInstruction_add_vc_r3_r3_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r6_1),
    kInstruction_add_gt_r0_r6_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r1_r7_1),
    kInstruction_add_hi_r1_r7_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r2_r4_5),
    kInstruction_add_ne_r2_r4_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r0_r7_0),
    kInstruction_add_hi_r0_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r1_r1_5),
    kInstruction_add_ne_r1_r1_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r1_r0_2),
    kInstruction_add_cc_r1_r0_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r7_r3_3),
    kInstruction_add_cc_r7_r3_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r0_3),
    kInstruction_add_lt_r0_r0_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r1_r1_1),
    kInstruction_add_hi_r1_r1_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r7_r2_4),
    kInstruction_add_vs_r7_r2_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r1_3),
    kInstruction_add_cs_r2_r1_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r4_r0_7),
    kInstruction_add_ge_r4_r0_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r2_2),
    kInstruction_add_gt_r0_r2_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r6_r4_4),
    kInstruction_add_lt_r6_r4_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r2_r4_1),
    kInstruction_add_lt_r2_r4_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r5_r5_3),
    kInstruction_add_cc_r5_r5_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r7_r3_0),
    kInstruction_add_eq_r7_r3_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r0_r2_2),
    kInstruction_add_cc_r0_r2_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r5_r7_7),
    kInstruction_add_mi_r5_r7_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r2_r4_5),
    kInstruction_add_eq_r2_r4_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r1_r7_0),
    kInstruction_add_pl_r1_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r6_r2_0),
    kInstruction_add_vs_r6_r2_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r0_r1_6),
    kInstruction_add_cc_r0_r1_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r2_5),
    kInstruction_add_cs_r2_r2_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r6_7),
    kInstruction_add_le_r0_r6_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r5_r0_7),
    kInstruction_add_hi_r5_r0_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r2_r5_1),
    kInstruction_add_gt_r2_r5_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r2_r1_2),
    kInstruction_add_gt_r2_r1_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r6_r2_5),
    kInstruction_add_le_r6_r2_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r4_r3_3),
    kInstruction_add_ls_r4_r3_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r2_6),
    kInstruction_add_le_r0_r2_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r0_r7_6),
    kInstruction_add_pl_r0_r7_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r4_r5_4),
    kInstruction_add_pl_r4_r5_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r2_2),
    kInstruction_add_vs_r3_r2_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r4_3),
    kInstruction_add_vc_r2_r4_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r1_r5_7),
    kInstruction_add_ge_r1_r5_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r1_r6_3),
    kInstruction_add_pl_r1_r6_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r3_r6_0),
    kInstruction_add_hi_r3_r6_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r4_3),
    kInstruction_add_ne_r0_r4_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r5_0),
    kInstruction_add_ne_r4_r5_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r7_r6_2),
    kInstruction_add_cc_r7_r6_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r2_r0_5),
    kInstruction_add_cc_r2_r0_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r3_r1_2),
    kInstruction_add_ge_r3_r1_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r1_r1_2),
    kInstruction_add_ge_r1_r1_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r5_r6_4),
    kInstruction_add_hi_r5_r6_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r6_r4_3),
    kInstruction_add_ne_r6_r4_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r3_r1_0),
    kInstruction_add_ne_r3_r1_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r3_r4_0),
    kInstruction_add_cs_r3_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r5_r0_6),
    kInstruction_add_eq_r5_r0_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r1_r5_1),
    kInstruction_add_pl_r1_r5_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r7_r7_4),
    kInstruction_add_ls_r7_r7_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r2_r2_2),
    kInstruction_add_le_r2_r2_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r6_r4_5),
    kInstruction_add_eq_r6_r4_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r1_r3_0),
    kInstruction_add_cc_r1_r3_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r3_r5_5),
    kInstruction_add_gt_r3_r5_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r7_r2_2),
    kInstruction_add_ge_r7_r2_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r2_r7_4),
    kInstruction_add_gt_r2_r7_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r7_r3_3),
    kInstruction_add_vs_r7_r3_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r2_r5_4),
    kInstruction_add_ge_r2_r5_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r0_r3_0),
    kInstruction_add_ge_r0_r3_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r3_r1_4),
    kInstruction_add_hi_r3_r1_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r7_r7_2),
    kInstruction_add_pl_r7_r7_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r6_r0_0),
    kInstruction_add_pl_r6_r0_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r0_r6_4),
    kInstruction_add_ls_r0_r6_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r3_r4_0),
    kInstruction_add_pl_r3_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r5_r3_5),
    kInstruction_add_ge_r5_r3_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r3_r2_5),
    kInstruction_add_ls_r3_r2_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r2_4),
    kInstruction_add_vs_r3_r2_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r7_r6_7),
    kInstruction_add_gt_r7_r6_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r0_r2_6),
    kInstruction_add_hi_r0_r2_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r2_r6_4),
    kInstruction_add_lt_r2_r6_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r1_r0_3),
    kInstruction_add_mi_r1_r0_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r5_r4_7),
    kInstruction_add_le_r5_r4_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r0_r7_6),
    kInstruction_add_ge_r0_r7_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r7_r3_3),
    kInstruction_add_hi_r7_r3_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r4_r4_7),
    kInstruction_add_cc_r4_r4_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r4_r2_5),
    kInstruction_add_le_r4_r2_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r0_3),
    kInstruction_add_gt_r0_r0_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r0_r4_5),
    kInstruction_add_pl_r0_r4_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r0_r5_6),
    kInstruction_add_ge_r0_r5_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r7_r3_1),
    kInstruction_add_lt_r7_r3_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r4_r2_0),
    kInstruction_add_cs_r4_r2_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r2_r5_5),
    kInstruction_add_vs_r2_r5_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r5_r4_1),
    kInstruction_add_le_r5_r4_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r5_r5_1),
    kInstruction_add_ge_r5_r5_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r2_r4_6),
    kInstruction_add_ls_r2_r4_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r4_r4_0),
    kInstruction_add_lt_r4_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r4_r6_1),
    kInstruction_add_vs_r4_r6_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r4_r1_0),
    kInstruction_add_hi_r4_r1_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r0_r7_1),
    kInstruction_add_vc_r0_r7_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r2_r7_2),
    kInstruction_add_lt_r2_r7_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r7_r5_6),
    kInstruction_add_ls_r7_r5_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r2_5),
    kInstruction_add_vs_r3_r2_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r6_r3_4),
    kInstruction_add_vs_r6_r3_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r1_r2_6),
    kInstruction_add_le_r1_r2_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r3_r7_7),
    kInstruction_add_vc_r3_r7_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r7_r7_0),
    kInstruction_add_le_r7_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r0_6),
    kInstruction_add_lt_r0_r0_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r5_2),
    kInstruction_add_ne_r4_r5_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r4_r1_6),
    kInstruction_add_mi_r4_r1_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r1_r4_2),
    kInstruction_add_ge_r1_r4_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r4_r3_3),
    kInstruction_add_mi_r4_r3_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r2_r5_0),
    kInstruction_add_lt_r2_r5_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r3_r4_1),
    kInstruction_add_gt_r3_r4_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r6_r3_3),
    kInstruction_add_le_r6_r3_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r2_r6_6),
    kInstruction_add_ge_r2_r6_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r2_r1_5),
    kInstruction_add_ne_r2_r1_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r1_r6_0),
    kInstruction_add_cc_r1_r6_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r1_r6_1),
    kInstruction_add_lt_r1_r6_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r7_2),
    kInstruction_add_vc_r2_r7_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r5_r1_1),
    kInstruction_add_le_r5_r1_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r1_r7_3),
    kInstruction_add_ge_r1_r7_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r5_r3_5),
    kInstruction_add_gt_r5_r3_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r1_r5_3),
    kInstruction_add_ne_r1_r5_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r5_r6_4),
    kInstruction_add_gt_r5_r6_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r3_r2_0),
    kInstruction_add_eq_r3_r2_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r0_1),
    kInstruction_add_lt_r0_r0_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r0_r3_1),
    kInstruction_add_cc_r0_r3_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r0_r1_5),
    kInstruction_add_cc_r0_r1_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r4_0),
    kInstruction_add_ne_r4_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r0_r6_4),
    kInstruction_add_ge_r0_r6_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r2_r6_0),
    kInstruction_add_lt_r2_r6_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r0_r6_5),
    kInstruction_add_cs_r0_r6_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r3_r5_1),
    kInstruction_add_lt_r3_r5_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r7_r1_0),
    kInstruction_add_lt_r7_r1_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r6_r3_6),
    kInstruction_add_cc_r6_r3_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r1_r0_0),
    kInstruction_add_gt_r1_r0_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r0_r7_5),
    kInstruction_add_ls_r0_r7_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r5_r2_6),
    kInstruction_add_hi_r5_r2_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r5_r2_7),
    kInstruction_add_pl_r5_r2_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r7_r0_5),
    kInstruction_add_mi_r7_r0_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r3_r0_1),
    kInstruction_add_cc_r3_r0_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r2_r2_0),
    kInstruction_add_lt_r2_r2_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r1_6),
    kInstruction_add_vc_r2_r1_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r2_r3_3),
    kInstruction_add_le_r2_r3_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r1_r3_1),
    kInstruction_add_le_r1_r3_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r5_r6_4),
    kInstruction_add_ls_r5_r6_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r5_r1_0),
    kInstruction_add_ne_r5_r1_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r0_5),
    kInstruction_add_le_r0_r0_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r4_r7_0),
    kInstruction_add_cc_r4_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r1_r4_0),
    kInstruction_add_ne_r1_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r5_r3_7),
    kInstruction_add_hi_r5_r3_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r3_1),
    kInstruction_add_ne_r0_r3_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r3_0),
    kInstruction_add_vs_r1_r3_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r6_r5_7),
    kInstruction_add_gt_r6_r5_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r3_r0_1),
    kInstruction_add_cs_r3_r0_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r7_r0_5),
    kInstruction_add_cc_r7_r0_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r5_r2_5),
    kInstruction_add_pl_r5_r2_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r5_r1_6),
    kInstruction_add_vs_r5_r1_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r1_r1_0),
    kInstruction_add_ge_r1_r1_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r6_r1_2),
    kInstruction_add_eq_r6_r1_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r5_5),
    kInstruction_add_lt_r0_r5_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r2_1),
    kInstruction_add_vc_r2_r2_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r0_r4_4),
    kInstruction_add_vc_r0_r4_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r2_r6_4),
    kInstruction_add_pl_r2_r6_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r1_r3_2),
    kInstruction_add_hi_r1_r3_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r7_r2_1),
    kInstruction_add_hi_r7_r2_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r1_4),
    kInstruction_add_lt_r0_r1_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r6_r2_0),
    kInstruction_add_cc_r6_r2_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r5_r4_2),
    kInstruction_add_vs_r5_r4_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r0_r5_3),
    kInstruction_add_pl_r0_r5_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r3_4),
    kInstruction_add_le_r0_r3_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r6_r2_5),
    kInstruction_add_cc_r6_r2_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r6_r1_3),
    kInstruction_add_ls_r6_r1_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r6_r6_0),
    kInstruction_add_ne_r6_r6_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r1_r3_1),
    kInstruction_add_vc_r1_r3_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r1_r7_6),
    kInstruction_add_lt_r1_r7_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r5_r3_5),
    kInstruction_add_hi_r5_r3_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r6_r4_6),
    kInstruction_add_gt_r6_r4_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r6_r6_3),
    kInstruction_add_ne_r6_r6_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r2_r4_1),
    kInstruction_add_pl_r2_r4_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r3_r6_7),
    kInstruction_add_cc_r3_r6_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r3_r7_0),
    kInstruction_add_cs_r3_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r7_r7_5),
    kInstruction_add_cc_r7_r7_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r0_7),
    kInstruction_add_vc_r6_r0_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r2_r0_3),
    kInstruction_add_lt_r2_r0_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r5_r4_7),
    kInstruction_add_ne_r5_r4_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r7_6),
    kInstruction_add_vc_r2_r7_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r4_r3_2),
    kInstruction_add_cc_r4_r3_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r3_r7_6),
    kInstruction_add_lt_r3_r7_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r3_r7_7),
    kInstruction_add_gt_r3_r7_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r7_5),
    kInstruction_add_ge_r6_r7_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r1_r4_5),
    kInstruction_add_ls_r1_r4_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r1_r7_3),
    kInstruction_add_eq_r1_r7_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r1_r4_6),
    kInstruction_add_le_r1_r4_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r3_1),
    kInstruction_add_ne_r4_r3_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r7_r3_6),
    kInstruction_add_pl_r7_r3_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r7_r7_0),
    kInstruction_add_ne_r7_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r1_r2_6),
    kInstruction_add_lt_r1_r2_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r7_2),
    kInstruction_add_le_r0_r7_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r2_r2_0),
    kInstruction_add_pl_r2_r2_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r2_r6_2),
    kInstruction_add_vs_r2_r6_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r0_r7_7),
    kInstruction_add_pl_r0_r7_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r6_r2_5),
    kInstruction_add_gt_r6_r2_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r4_r3_1),
    kInstruction_add_cs_r4_r3_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r3_r5_4),
    kInstruction_add_ne_r3_r5_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r0_r0_0),
    kInstruction_add_mi_r0_r0_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r2_r0_1),
    kInstruction_add_ge_r2_r0_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r7_r3_7),
    kInstruction_add_mi_r7_r3_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r7_2),
    kInstruction_add_ne_r0_r7_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r0_r6_2),
    kInstruction_add_ge_r0_r6_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r2_r7_2),
    kInstruction_add_hi_r2_r7_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r0_r4_0),
    kInstruction_add_vs_r0_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r2_r1_0),
    kInstruction_add_hi_r2_r1_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r7_r0_3),
    kInstruction_add_hi_r7_r0_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r5_r4_0),
    kInstruction_add_ge_r5_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r4_r1_3),
    kInstruction_add_mi_r4_r1_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r1_r6_3),
    kInstruction_add_le_r1_r6_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r7_r6_2),
    kInstruction_add_le_r7_r6_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r6_r3_4),
    kInstruction_add_mi_r6_r3_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r4_r4_0),
    kInstruction_add_cs_r4_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r2_r3_2),
    kInstruction_add_cc_r2_r3_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r1_r5_4),
    kInstruction_add_mi_r1_r5_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r5_r3_2),
    kInstruction_add_cc_r5_r3_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r2_r0_2),
    kInstruction_add_gt_r2_r0_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r2_r4_7),
    kInstruction_add_le_r2_r4_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r3_3),
    kInstruction_add_vc_r2_r3_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r4_r2_3),
    kInstruction_add_eq_r4_r2_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r0_7),
    kInstruction_add_ge_r6_r0_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r0_r7_7),
    kInstruction_add_eq_r0_r7_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r5_r6_2),
    kInstruction_add_gt_r5_r6_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r6_r0_7),
    kInstruction_add_ne_r6_r0_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r7_r0_4),
    kInstruction_add_ne_r7_r0_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r7_r7_1),
    kInstruction_add_eq_r7_r7_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r6_r5_0),
    kInstruction_add_le_r6_r5_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r3_r6_7),
    kInstruction_add_vc_r3_r6_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r0_r3_7),
    kInstruction_add_pl_r0_r3_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r5_r5_0),
    kInstruction_add_ls_r5_r5_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r3_7),
    kInstruction_add_ge_r6_r3_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r3_r2_6),
    kInstruction_add_eq_r3_r2_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r3_7),
    kInstruction_add_cs_r5_r3_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r3_r5_7),
    kInstruction_add_cc_r3_r5_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r1_4),
    kInstruction_add_ge_r6_r1_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r5_r4_5),
    kInstruction_add_mi_r5_r4_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r7_r6_6),
    kInstruction_add_eq_r7_r6_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r2_r2_2),
    kInstruction_add_gt_r2_r2_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r1_r0_2),
    kInstruction_add_mi_r1_r0_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r2_r2_1),
    kInstruction_add_eq_r2_r2_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r1_r2_1),
    kInstruction_add_lt_r1_r2_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r5_r1_3),
    kInstruction_add_cc_r5_r1_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r1_r7_5),
    kInstruction_add_ge_r1_r7_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r6_r3_3),
    kInstruction_add_lt_r6_r3_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r3_r1_7),
    kInstruction_add_vc_r3_r1_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r7_r2_7),
    kInstruction_add_mi_r7_r2_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r3_1),
    kInstruction_add_lt_r0_r3_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r1_r7_1),
    kInstruction_add_cs_r1_r7_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r3_r4_5),
    kInstruction_add_cc_r3_r4_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r0_2),
    kInstruction_add_lt_r5_r0_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r1_r5_7),
    kInstruction_add_hi_r1_r5_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r6_5),
    kInstruction_add_ne_r4_r6_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r3_6),
    kInstruction_add_vc_r2_r3_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r1_r7_0),
    kInstruction_add_mi_r1_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r0_r2_4),
    kInstruction_add_hi_r0_r2_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r2_5),
    kInstruction_add_cs_r5_r2_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r2_r6_3),
    kInstruction_add_ge_r2_r6_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r7_5),
    kInstruction_add_cs_r2_r7_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r1_r6_5),
    kInstruction_add_eq_r1_r6_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r7_r0_6),
    kInstruction_add_vc_r7_r0_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r0_2),
    kInstruction_add_vs_r1_r0_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r3_r3_5),
    kInstruction_add_eq_r3_r3_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r0_r4_5),
    kInstruction_add_cc_r0_r4_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r7_r2_5),
    kInstruction_add_lt_r7_r2_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r1_r1_0),
    kInstruction_add_pl_r1_r1_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r7_r2_5),
    kInstruction_add_hi_r7_r2_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r0_r4_0),
    kInstruction_add_cc_r0_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r0_r3_7),
    kInstruction_add_eq_r0_r3_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r4_r5_0),
    kInstruction_add_lt_r4_r5_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r5_r4_7),
    kInstruction_add_vc_r5_r4_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r7_r7_0),
    kInstruction_add_gt_r7_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r1_r2_7),
    kInstruction_add_le_r1_r2_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r0_r4_6),
    kInstruction_add_cc_r0_r4_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r1_r5_5),
    kInstruction_add_eq_r1_r5_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r1_r5_2),
    kInstruction_add_hi_r1_r5_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r2_r4_1),
    kInstruction_add_ne_r2_r4_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r3_r6_6),
    kInstruction_add_cc_r3_r6_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r7_r7_6),
    kInstruction_add_eq_r7_r7_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r5_r4_4),
    kInstruction_add_ne_r5_r4_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r2_r4_2),
    kInstruction_add_eq_r2_r4_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r1_2),
    kInstruction_add_vc_r2_r1_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r5_r1_3),
    kInstruction_add_ge_r5_r1_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r0_r0_5),
    kInstruction_add_hi_r0_r0_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r6_r2_6),
    kInstruction_add_ne_r6_r2_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r5_0),
    kInstruction_add_vc_r2_r5_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r1_r3_3),
    kInstruction_add_le_r1_r3_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r0_r2_0),
    kInstruction_add_mi_r0_r2_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r1_r2_7),
    kInstruction_add_mi_r1_r2_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r5_5),
    kInstruction_add_vc_r6_r5_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r2_r0_3),
    kInstruction_add_ge_r2_r0_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r5_r0_7),
    kInstruction_add_le_r5_r0_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r1_r4_0),
    kInstruction_add_lt_r1_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r4_r4_3),
    kInstruction_add_le_r4_r4_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r0_r6_5),
    kInstruction_add_vs_r0_r6_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r0_r0_2),
    kInstruction_add_vs_r0_r0_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r3_3),
    kInstruction_add_ne_r4_r3_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r0_r6_5),
    kInstruction_add_ge_r0_r6_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r6_r0_5),
    kInstruction_add_pl_r6_r0_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r5_r3_4),
    kInstruction_add_ls_r5_r3_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r4_5),
    kInstruction_add_vs_r3_r4_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r1_r4_6),
    kInstruction_add_gt_r1_r4_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r1_r1_2),
    kInstruction_add_vc_r1_r1_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r7_r1_5),
    kInstruction_add_lt_r7_r1_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r0_r4_1),
    kInstruction_add_eq_r0_r4_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r4_2),
    kInstruction_add_le_r0_r4_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r0_r0_4),
    kInstruction_add_vc_r0_r0_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r4_r6_3),
    kInstruction_add_eq_r4_r6_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r7_r1_4),
    kInstruction_add_eq_r7_r1_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r5_r4_7),
    kInstruction_add_pl_r5_r4_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r0_3),
    kInstruction_add_ge_r6_r0_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r5_r5_4),
    kInstruction_add_gt_r5_r5_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r6_r0_2),
    kInstruction_add_mi_r6_r0_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r7_r5_2),
    kInstruction_add_gt_r7_r5_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r6_5),
    kInstruction_add_le_r0_r6_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r1_r5_6),
    kInstruction_add_vc_r1_r5_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r5_r5_5),
    kInstruction_add_eq_r5_r5_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r2_r6_2),
    kInstruction_add_ge_r2_r6_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r4_r4_7),
    kInstruction_add_eq_r4_r4_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r5_r5_4),
    kInstruction_add_ne_r5_r5_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r2_r1_0),
    kInstruction_add_lt_r2_r1_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r4_r0_4),
    kInstruction_add_gt_r4_r0_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r0_1),
    kInstruction_add_le_r0_r0_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r1_r3_1),
    kInstruction_add_pl_r1_r3_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r2_0),
    kInstruction_add_ge_r6_r2_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r2_r6_0),
    kInstruction_add_ne_r2_r6_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r3_r2_5),
    kInstruction_add_cs_r3_r2_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r3_r3_2),
    kInstruction_add_eq_r3_r3_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r6_r6_5),
    kInstruction_add_ne_r6_r6_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r5_r2_2),
    kInstruction_add_vc_r5_r2_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r3_r6_4),
    kInstruction_add_pl_r3_r6_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r5_r7_6),
    kInstruction_add_cs_r5_r7_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r1_7),
    kInstruction_add_cs_r2_r1_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r4_r5_5),
    kInstruction_add_pl_r4_r5_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r1_r5_7),
    kInstruction_add_pl_r1_r5_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r4_r3_5),
    kInstruction_add_mi_r4_r3_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r0_r4_0),
    kInstruction_add_vc_r0_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r2_r7_0),
    kInstruction_add_le_r2_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r7_r2_6),
    kInstruction_add_cc_r7_r2_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r7_r1_1),
    kInstruction_add_vs_r7_r1_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r3_r3_5),
    kInstruction_add_gt_r3_r3_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r7_r1_1),
    kInstruction_add_eq_r7_r1_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r5_r7_5),
    kInstruction_add_pl_r5_r7_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r5_r6_1),
    kInstruction_add_hi_r5_r6_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r2_r0_0),
    kInstruction_add_cc_r2_r0_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r7_r7_2),
    kInstruction_add_cs_r7_r7_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r3_6),
    kInstruction_add_ne_r0_r3_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r4_r7_2),
    kInstruction_add_mi_r4_r7_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r4_r4_7),
    kInstruction_add_le_r4_r4_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r1_r1_0),
    kInstruction_add_gt_r1_r1_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r2_r5_1),
    kInstruction_add_vs_r2_r5_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r7_r2_4),
    kInstruction_add_ls_r7_r2_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r2_r1_4),
    kInstruction_add_vs_r2_r1_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r5_r6_5),
    kInstruction_add_eq_r5_r6_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r4_r3_4),
    kInstruction_add_vs_r4_r3_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r2_r1_2),
    kInstruction_add_vs_r2_r1_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r0_r3_3),
    kInstruction_add_hi_r0_r3_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r4_r1_5),
    kInstruction_add_hi_r4_r1_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r6_r6_7),
    kInstruction_add_pl_r6_r6_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r5_0),
    kInstruction_add_gt_r0_r5_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r5_r0_7),
    kInstruction_add_cc_r5_r0_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r7_r7_7),
    kInstruction_add_ls_r7_r7_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r3_0),
    kInstruction_add_vc_r6_r3_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r6_r4_4),
    kInstruction_add_cc_r6_r4_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r6_r5_0),
    kInstruction_add_cc_r6_r5_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r2_r0_6),
    kInstruction_add_le_r2_r0_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r5_r6_3),
    kInstruction_add_gt_r5_r6_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r4_0),
    kInstruction_add_vs_r3_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r4_r4_1),
    kInstruction_add_le_r4_r4_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r4_r0_6),
    kInstruction_add_ge_r4_r0_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r2_r3_2),
    kInstruction_add_eq_r2_r3_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r1_6),
    kInstruction_add_lt_r5_r1_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r7_r2_0),
    kInstruction_add_le_r7_r2_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r2_r4_6),
    kInstruction_add_mi_r2_r4_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r5_4),
    kInstruction_add_vc_r6_r5_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r3_r6_5),
    kInstruction_add_gt_r3_r6_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r3_r2_0),
    kInstruction_add_pl_r3_r2_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r3_r2_2),
    kInstruction_add_ls_r3_r2_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r2_r7_7),
    kInstruction_add_ge_r2_r7_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r3_r5_0),
    kInstruction_add_vc_r3_r5_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r4_r0_7),
    kInstruction_add_cc_r4_r0_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r3_r5_2),
    kInstruction_add_ls_r3_r5_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r7_6),
    kInstruction_add_vc_r6_r7_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r0_r3_6),
    kInstruction_add_ls_r0_r3_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r6_r0_1),
    kInstruction_add_hi_r6_r0_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r5_r0_6),
    kInstruction_add_mi_r5_r0_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r2_r5_3),
    kInstruction_add_mi_r2_r5_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r0_r7_0),
    kInstruction_add_ge_r0_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r7_r3_3),
    kInstruction_add_vc_r7_r3_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r0_r2_5),
    kInstruction_add_mi_r0_r2_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r4_r3_5),
    kInstruction_add_cc_r4_r3_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r0_r3_0),
    kInstruction_add_pl_r0_r3_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r6_r3_1),
    kInstruction_add_gt_r6_r3_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r3_r3_0),
    kInstruction_add_lt_r3_r3_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r1_r5_6),
    kInstruction_add_hi_r1_r5_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r4_r7_0),
    kInstruction_add_hi_r4_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r2_2),
    kInstruction_add_ge_r6_r2_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r7_r4_4),
    kInstruction_add_pl_r7_r4_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r5_r5_5),
    kInstruction_add_cc_r5_r5_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r6_r1_1),
    kInstruction_add_pl_r6_r1_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r7_r6_3),
    kInstruction_add_le_r7_r6_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r4_r7_1),
    kInstruction_add_lt_r4_r7_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r2_r4_7),
    kInstruction_add_pl_r2_r4_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r1_r3_0),
    kInstruction_add_vc_r1_r3_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r7_r7_3),
    kInstruction_add_pl_r7_r7_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r4_r1_3),
    kInstruction_add_ls_r4_r1_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r6_r6_2),
    kInstruction_add_lt_r6_r6_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r4_r2_4),
    kInstruction_add_mi_r4_r2_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r3_r4_7),
    kInstruction_add_pl_r3_r4_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r2_r6_6),
    kInstruction_add_ls_r2_r6_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r5_r7_6),
    kInstruction_add_hi_r5_r7_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r4_1),
    kInstruction_add_lt_r0_r4_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r5_r5_6),
    kInstruction_add_ge_r5_r5_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r1_r7_3),
    kInstruction_add_pl_r1_r7_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r6_r2_1),
    kInstruction_add_cs_r6_r2_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r1_r3_0),
    kInstruction_add_ne_r1_r3_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r6_r5_7),
    kInstruction_add_mi_r6_r5_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r2_r2_0),
    kInstruction_add_gt_r2_r2_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r6_r4_6),
    kInstruction_add_ls_r6_r4_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r2_r6_7),
    kInstruction_add_pl_r2_r6_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r1_r5_4),
    kInstruction_add_ne_r1_r5_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r0_r0_5),
    kInstruction_add_mi_r0_r0_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r5_r5_2),
    kInstruction_add_eq_r5_r5_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r7_r7_3),
    kInstruction_add_mi_r7_r7_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r7_r7_1),
    kInstruction_add_vc_r7_r7_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r5_r4_3),
    kInstruction_add_vc_r5_r4_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r5_r7_6),
    kInstruction_add_mi_r5_r7_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r2_r3_0),
    kInstruction_add_vs_r2_r3_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r2_r4_2),
    kInstruction_add_ne_r2_r4_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r6_r7_3),
    kInstruction_add_eq_r6_r7_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r1_5),
    kInstruction_add_ne_r0_r1_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r3_r2_6),
    kInstruction_add_lt_r3_r2_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r4_r2_1),
    kInstruction_add_gt_r4_r2_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r6_r1_0),
    kInstruction_add_ls_r6_r1_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r6_r2_3),
    kInstruction_add_cc_r6_r2_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r3_r2_7),
    kInstruction_add_hi_r3_r2_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r6_r7_3),
    kInstruction_add_cc_r6_r7_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r7_r6_4),
    kInstruction_add_vs_r7_r6_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r5_r4_5),
    kInstruction_add_gt_r5_r4_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r5_r7_6),
    kInstruction_add_vc_r5_r7_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r0_r6_5),
    kInstruction_add_hi_r0_r6_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r2_r7_3),
    kInstruction_add_eq_r2_r7_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r6_r5_7),
    kInstruction_add_le_r6_r5_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r0_r5_5),
    kInstruction_add_ge_r0_r5_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r1_r6_4),
    kInstruction_add_pl_r1_r6_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r3_r0_6),
    kInstruction_add_mi_r3_r0_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r7_r6_1),
    kInstruction_add_cs_r7_r6_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r3_r6_3),
    kInstruction_add_vc_r3_r6_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r5_r1_4),
    kInstruction_add_ls_r5_r1_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r6_r2_3),
    kInstruction_add_le_r6_r2_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r1_r6_4),
    kInstruction_add_cc_r1_r6_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r0_r7_0),
    kInstruction_add_mi_r0_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r2_2),
    kInstruction_add_vc_r6_r2_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r3_r0_4),
    kInstruction_add_gt_r3_r0_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r4_r0_5),
    kInstruction_add_cs_r4_r0_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r4_r4_6),
    kInstruction_add_hi_r4_r4_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r5_r5_1),
    kInstruction_add_cc_r5_r5_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r6_r3_4),
    kInstruction_add_pl_r6_r3_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r7_r4_0),
    kInstruction_add_ge_r7_r4_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r2_r1_3),
    kInstruction_add_hi_r2_r1_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r4_r3_1),
    kInstruction_add_le_r4_r3_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r0_7),
    kInstruction_add_gt_r0_r0_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r2_r0_6),
    kInstruction_add_gt_r2_r0_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r4_r3_5),
    kInstruction_add_ge_r4_r3_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r2_r0_7),
    kInstruction_add_le_r2_r0_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r7_r7_4),
    kInstruction_add_vc_r7_r7_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r6_r5_1),
    kInstruction_add_lt_r6_r5_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r0_4),
    kInstruction_add_vs_r1_r0_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r1_6),
    kInstruction_add_vs_r1_r1_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r1_r0_6),
    kInstruction_add_ge_r1_r0_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r1_r6_0),
    kInstruction_add_eq_r1_r6_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r0_r5_5),
    kInstruction_add_cs_r0_r5_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r7_r6_5),
    kInstruction_add_mi_r7_r6_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r1_r4_3),
    kInstruction_add_vc_r1_r4_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r1_r3_4),
    kInstruction_add_hi_r1_r3_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r6_3),
    kInstruction_add_vs_r3_r6_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r1_r6_1),
    kInstruction_add_vc_r1_r6_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r3_r7_1),
    kInstruction_add_mi_r3_r7_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r7_r0_1),
    kInstruction_add_eq_r7_r0_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r3_r7_7),
    kInstruction_add_hi_r3_r7_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r3_r4_3),
    kInstruction_add_mi_r3_r4_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r3_r1_2),
    kInstruction_add_le_r3_r1_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r1_r3_5),
    kInstruction_add_mi_r1_r3_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r2_4),
    kInstruction_add_vc_r2_r2_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r4_r7_2),
    kInstruction_add_vc_r4_r7_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r7_0),
    kInstruction_add_vs_r3_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r7_r1_3),
    kInstruction_add_gt_r7_r1_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r7_r4_7),
    kInstruction_add_hi_r7_r4_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r1_r0_3),
    kInstruction_add_hi_r1_r0_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r4_5),
    kInstruction_add_ne_r0_r4_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r7_r6_3),
    kInstruction_add_hi_r7_r6_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r3_r4_4),
    kInstruction_add_ne_r3_r4_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r2_r5_5),
    kInstruction_add_pl_r2_r5_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r7_0),
    kInstruction_add_ge_r6_r7_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r7_r3_4),
    kInstruction_add_ne_r7_r3_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r5_r2_6),
    kInstruction_add_ls_r5_r2_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r6_r3_2),
    kInstruction_add_cs_r6_r3_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r0_r1_6),
    kInstruction_add_vs_r0_r1_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r3_7),
    kInstruction_add_vc_r6_r3_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r3_r0_5),
    kInstruction_add_eq_r3_r0_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r4_r5_0),
    kInstruction_add_cc_r4_r5_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r3_r1_2),
    kInstruction_add_hi_r3_r1_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r2_r5_5),
    kInstruction_add_lt_r2_r5_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r0_r3_3),
    kInstruction_add_le_r0_r3_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r2_r5_7),
    kInstruction_add_vc_r2_r5_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r7_r1_7),
    kInstruction_add_gt_r7_r1_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r2_r1_5),
    kInstruction_add_le_r2_r1_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r4_r2_0),
    kInstruction_add_mi_r4_r2_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r4_r3_2),
    kInstruction_add_ne_r4_r3_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r3_r0_7),
    kInstruction_add_le_r3_r0_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r5_7),
    kInstruction_add_vc_r6_r5_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r5_0),
    kInstruction_add_lt_r5_r5_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r5_r7_2),
    kInstruction_add_le_r5_r7_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r2_r6_4),
    kInstruction_add_mi_r2_r6_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r5_r4_5),
    kInstruction_add_ne_r5_r4_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r5_r7_2),
    kInstruction_add_mi_r5_r7_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r7_r1_1),
    kInstruction_add_lt_r7_r1_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r3_r0_5),
    kInstruction_add_cc_r3_r0_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r2_r2_7),
    kInstruction_add_pl_r2_r2_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r3_r5_0),
    kInstruction_add_lt_r3_r5_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r7_r7_5),
    kInstruction_add_cs_r7_r7_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r5_2),
    kInstruction_add_ge_r6_r5_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r2_r7_1),
    kInstruction_add_pl_r2_r7_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r1_r5_0),
    kInstruction_add_gt_r1_r5_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r4_r2_6),
    kInstruction_add_hi_r4_r2_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r4_r6_1),
    kInstruction_add_ls_r4_r6_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r4_r6_5),
    kInstruction_add_vc_r4_r6_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r2_7),
    kInstruction_add_ne_r0_r2_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r1_r7_3),
    kInstruction_add_ne_r1_r7_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r2_r1_2),
    kInstruction_add_pl_r2_r1_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r6_r2_2),
    kInstruction_add_le_r6_r2_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r3_r6_0),
    kInstruction_add_ne_r3_r6_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r6_6),
    kInstruction_add_lt_r5_r6_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r3_r2_2),
    kInstruction_add_eq_r3_r2_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r1_r6_7),
    kInstruction_add_pl_r1_r6_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r7_r0_6),
    kInstruction_add_mi_r7_r0_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r7_r2_7),
    kInstruction_add_le_r7_r2_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r0_r3_1),
    kInstruction_add_hi_r0_r3_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r3_r6_4),
    kInstruction_add_ls_r3_r6_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r6_r0_7),
    kInstruction_add_eq_r6_r0_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r1_r4_5),
    kInstruction_add_ne_r1_r4_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r1_r5_3),
    kInstruction_add_le_r1_r5_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r6_r1_7),
    kInstruction_add_cc_r6_r1_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r2_r4_1),
    kInstruction_add_hi_r2_r4_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r7_2),
    kInstruction_add_vs_r3_r7_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r7_r6_1),
    kInstruction_add_pl_r7_r6_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r7_r2_1),
    kInstruction_add_ne_r7_r2_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r1_r0_0),
    kInstruction_add_hi_r1_r0_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r2_r1_7),
    kInstruction_add_vs_r2_r1_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r5_r4_2),
    kInstruction_add_lt_r5_r4_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r3_r1_0),
    kInstruction_add_mi_r3_r1_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r2_r6_1),
    kInstruction_add_ls_r2_r6_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r2_r2_1),
    kInstruction_add_cc_r2_r2_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r4_r0_7),
    kInstruction_add_pl_r4_r0_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r6_r4_5),
    kInstruction_add_cc_r6_r4_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r1_r3_6),
    kInstruction_add_cc_r1_r3_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r0_r3_2),
    kInstruction_add_lt_r0_r3_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r5_r4_3),
    kInstruction_add_ge_r5_r4_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r4_r4_2),
    kInstruction_add_lt_r4_r4_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r3_r1_6),
    kInstruction_add_gt_r3_r1_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r4_r2_1),
    kInstruction_add_pl_r4_r2_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r3_r1_7),
    kInstruction_add_cc_r3_r1_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r1_r3_7),
    kInstruction_add_mi_r1_r3_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r2_r5_7),
    kInstruction_add_cc_r2_r5_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r6_r5_0),
    kInstruction_add_hi_r6_r5_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r1_r0_7),
    kInstruction_add_vc_r1_r0_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r3_r0_5),
    kInstruction_add_cs_r3_r0_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r1_r2_3),
    kInstruction_add_pl_r1_r2_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_pl_r2_r6_3),
    kInstruction_add_pl_r2_r6_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r7_r1_5),
    kInstruction_add_ne_r7_r1_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r5_r0_4),
    kInstruction_add_hi_r5_r0_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r6_r1_0),
    kInstruction_add_vc_r6_r1_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r7_r3_2),
    kInstruction_add_vc_r7_r3_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r6_6),
    kInstruction_add_ge_r6_r6_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r6_r1_0),
    kInstruction_add_ne_r6_r1_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r5_r1_2),
    kInstruction_add_eq_r5_r1_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r2_r0_6),
    kInstruction_add_hi_r2_r0_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r7_r4_5),
    kInstruction_add_ne_r7_r4_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r0_r0_1),
    kInstruction_add_cc_r0_r0_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r7_r5_1),
    kInstruction_add_mi_r7_r5_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r7_r0_0),
    kInstruction_add_mi_r7_r0_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r0_r2_0),
    kInstruction_add_ls_r0_r2_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r4_r7_3),
    kInstruction_add_vs_r4_r7_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r6_r2_4),
    kInstruction_add_hi_r6_r2_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r4_r1_1),
    kInstruction_add_eq_r4_r1_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r7_r5_1),
    kInstruction_add_vs_r7_r5_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ne_r0_r4_7),
    kInstruction_add_ne_r0_r4_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r2_r7_7),
    kInstruction_add_mi_r2_r7_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r3_r2_5),
    kInstruction_add_mi_r3_r2_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r7_r5_5),
    kInstruction_add_le_r7_r5_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r5_7),
    kInstruction_add_gt_r0_r5_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r0_r4_7),
    kInstruction_add_vc_r0_r4_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r5_r7_2),
    kInstruction_add_ls_r5_r7_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r0_r6_1),
    kInstruction_add_hi_r0_r6_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_cc_r1_r4_5),
    kInstruction_add_cc_r1_r4_5,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r2_r2_7),
    kInstruction_add_vs_r2_r2_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_ge_r6_r6_1),
    kInstruction_add_ge_r6_r6_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r7_r1_1),
    kInstruction_add_hi_r7_r1_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r3_r4_3),
    kInstruction_add_le_r3_r4_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r2_r1_3),
    kInstruction_add_mi_r2_r1_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r7_r7_4),
    kInstruction_add_cs_r7_r7_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_cs_r2_r5_0),
    kInstruction_add_cs_r2_r5_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r7_r6_4),
    kInstruction_add_hi_r7_r6_4,
  },
  {
    ARRAY_SIZE(kInstruction_add_lt_r7_r5_1),
    kInstruction_add_lt_r7_r5_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r0_r0_1),
    kInstruction_add_gt_r0_r0_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r1_r2_6),
    kInstruction_add_vs_r1_r2_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_vs_r3_r3_1),
    kInstruction_add_vs_r3_r3_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_ls_r4_r2_2),
    kInstruction_add_ls_r4_r2_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r6_r3_2),
    kInstruction_add_hi_r6_r3_2,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r7_r5_3),
    kInstruction_add_gt_r7_r5_3,
  },
  {
    ARRAY_SIZE(kInstruction_add_vc_r1_r6_7),
    kInstruction_add_vc_r1_r6_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r2_r2_0),
    kInstruction_add_eq_r2_r2_0,
  },
  {
    ARRAY_SIZE(kInstruction_add_hi_r7_r6_1),
    kInstruction_add_hi_r7_r6_1,
  },
  {
    ARRAY_SIZE(kInstruction_add_gt_r4_r6_7),
    kInstruction_add_gt_r4_r6_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_le_r7_r0_6),
    kInstruction_add_le_r7_r0_6,
  },
  {
    ARRAY_SIZE(kInstruction_add_eq_r7_r1_7),
    kInstruction_add_eq_r7_r1_7,
  },
  {
    ARRAY_SIZE(kInstruction_add_mi_r0_r6_7),
    kInstruction_add_mi_r0_r6_7,
  },
};

#endif  // VIXL_ASSEMBLER_COND_RDLOW_RNLOW_OPERAND_IMMEDIATE_IMM3_IN_IT_BLOCK_ADD_T32_H_
