// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_ASSEMBLER_COND_RD_OPERAND_RN_ROR_AMOUNT_SXTB_T32_H_
#define VIXL_ASSEMBLER_COND_RD_OPERAND_RN_ROR_AMOUNT_SXTB_T32_H_

const byte kInstruction_sxtb_al_r0_r0_ROR_0[] = {
  0x40, 0xb2 // sxtb al r0 r0 ROR 0
};
const byte kInstruction_sxtb_al_r0_r0_ROR_8[] = {
  0x4f, 0xfa, 0x90, 0xf0 // sxtb al r0 r0 ROR 8
};
const byte kInstruction_sxtb_al_r0_r0_ROR_16[] = {
  0x4f, 0xfa, 0xa0, 0xf0 // sxtb al r0 r0 ROR 16
};
const byte kInstruction_sxtb_al_r0_r0_ROR_24[] = {
  0x4f, 0xfa, 0xb0, 0xf0 // sxtb al r0 r0 ROR 24
};
const byte kInstruction_sxtb_al_r0_r1_ROR_0[] = {
  0x48, 0xb2 // sxtb al r0 r1 ROR 0
};
const byte kInstruction_sxtb_al_r0_r1_ROR_8[] = {
  0x4f, 0xfa, 0x91, 0xf0 // sxtb al r0 r1 ROR 8
};
const byte kInstruction_sxtb_al_r0_r1_ROR_16[] = {
  0x4f, 0xfa, 0xa1, 0xf0 // sxtb al r0 r1 ROR 16
};
const byte kInstruction_sxtb_al_r0_r1_ROR_24[] = {
  0x4f, 0xfa, 0xb1, 0xf0 // sxtb al r0 r1 ROR 24
};
const byte kInstruction_sxtb_al_r0_r2_ROR_0[] = {
  0x50, 0xb2 // sxtb al r0 r2 ROR 0
};
const byte kInstruction_sxtb_al_r0_r2_ROR_8[] = {
  0x4f, 0xfa, 0x92, 0xf0 // sxtb al r0 r2 ROR 8
};
const byte kInstruction_sxtb_al_r0_r2_ROR_16[] = {
  0x4f, 0xfa, 0xa2, 0xf0 // sxtb al r0 r2 ROR 16
};
const byte kInstruction_sxtb_al_r0_r2_ROR_24[] = {
  0x4f, 0xfa, 0xb2, 0xf0 // sxtb al r0 r2 ROR 24
};
const byte kInstruction_sxtb_al_r0_r3_ROR_0[] = {
  0x58, 0xb2 // sxtb al r0 r3 ROR 0
};
const byte kInstruction_sxtb_al_r0_r3_ROR_8[] = {
  0x4f, 0xfa, 0x93, 0xf0 // sxtb al r0 r3 ROR 8
};
const byte kInstruction_sxtb_al_r0_r3_ROR_16[] = {
  0x4f, 0xfa, 0xa3, 0xf0 // sxtb al r0 r3 ROR 16
};
const byte kInstruction_sxtb_al_r0_r3_ROR_24[] = {
  0x4f, 0xfa, 0xb3, 0xf0 // sxtb al r0 r3 ROR 24
};
const byte kInstruction_sxtb_al_r0_r4_ROR_0[] = {
  0x60, 0xb2 // sxtb al r0 r4 ROR 0
};
const byte kInstruction_sxtb_al_r0_r4_ROR_8[] = {
  0x4f, 0xfa, 0x94, 0xf0 // sxtb al r0 r4 ROR 8
};
const byte kInstruction_sxtb_al_r0_r4_ROR_16[] = {
  0x4f, 0xfa, 0xa4, 0xf0 // sxtb al r0 r4 ROR 16
};
const byte kInstruction_sxtb_al_r0_r4_ROR_24[] = {
  0x4f, 0xfa, 0xb4, 0xf0 // sxtb al r0 r4 ROR 24
};
const byte kInstruction_sxtb_al_r0_r5_ROR_0[] = {
  0x68, 0xb2 // sxtb al r0 r5 ROR 0
};
const byte kInstruction_sxtb_al_r0_r5_ROR_8[] = {
  0x4f, 0xfa, 0x95, 0xf0 // sxtb al r0 r5 ROR 8
};
const byte kInstruction_sxtb_al_r0_r5_ROR_16[] = {
  0x4f, 0xfa, 0xa5, 0xf0 // sxtb al r0 r5 ROR 16
};
const byte kInstruction_sxtb_al_r0_r5_ROR_24[] = {
  0x4f, 0xfa, 0xb5, 0xf0 // sxtb al r0 r5 ROR 24
};
const byte kInstruction_sxtb_al_r0_r6_ROR_0[] = {
  0x70, 0xb2 // sxtb al r0 r6 ROR 0
};
const byte kInstruction_sxtb_al_r0_r6_ROR_8[] = {
  0x4f, 0xfa, 0x96, 0xf0 // sxtb al r0 r6 ROR 8
};
const byte kInstruction_sxtb_al_r0_r6_ROR_16[] = {
  0x4f, 0xfa, 0xa6, 0xf0 // sxtb al r0 r6 ROR 16
};
const byte kInstruction_sxtb_al_r0_r6_ROR_24[] = {
  0x4f, 0xfa, 0xb6, 0xf0 // sxtb al r0 r6 ROR 24
};
const byte kInstruction_sxtb_al_r0_r7_ROR_0[] = {
  0x78, 0xb2 // sxtb al r0 r7 ROR 0
};
const byte kInstruction_sxtb_al_r0_r7_ROR_8[] = {
  0x4f, 0xfa, 0x97, 0xf0 // sxtb al r0 r7 ROR 8
};
const byte kInstruction_sxtb_al_r0_r7_ROR_16[] = {
  0x4f, 0xfa, 0xa7, 0xf0 // sxtb al r0 r7 ROR 16
};
const byte kInstruction_sxtb_al_r0_r7_ROR_24[] = {
  0x4f, 0xfa, 0xb7, 0xf0 // sxtb al r0 r7 ROR 24
};
const byte kInstruction_sxtb_al_r0_r8_ROR_0[] = {
  0x4f, 0xfa, 0x88, 0xf0 // sxtb al r0 r8 ROR 0
};
const byte kInstruction_sxtb_al_r0_r8_ROR_8[] = {
  0x4f, 0xfa, 0x98, 0xf0 // sxtb al r0 r8 ROR 8
};
const byte kInstruction_sxtb_al_r0_r8_ROR_16[] = {
  0x4f, 0xfa, 0xa8, 0xf0 // sxtb al r0 r8 ROR 16
};
const byte kInstruction_sxtb_al_r0_r8_ROR_24[] = {
  0x4f, 0xfa, 0xb8, 0xf0 // sxtb al r0 r8 ROR 24
};
const byte kInstruction_sxtb_al_r0_r9_ROR_0[] = {
  0x4f, 0xfa, 0x89, 0xf0 // sxtb al r0 r9 ROR 0
};
const byte kInstruction_sxtb_al_r0_r9_ROR_8[] = {
  0x4f, 0xfa, 0x99, 0xf0 // sxtb al r0 r9 ROR 8
};
const byte kInstruction_sxtb_al_r0_r9_ROR_16[] = {
  0x4f, 0xfa, 0xa9, 0xf0 // sxtb al r0 r9 ROR 16
};
const byte kInstruction_sxtb_al_r0_r9_ROR_24[] = {
  0x4f, 0xfa, 0xb9, 0xf0 // sxtb al r0 r9 ROR 24
};
const byte kInstruction_sxtb_al_r0_r10_ROR_0[] = {
  0x4f, 0xfa, 0x8a, 0xf0 // sxtb al r0 r10 ROR 0
};
const byte kInstruction_sxtb_al_r0_r10_ROR_8[] = {
  0x4f, 0xfa, 0x9a, 0xf0 // sxtb al r0 r10 ROR 8
};
const byte kInstruction_sxtb_al_r0_r10_ROR_16[] = {
  0x4f, 0xfa, 0xaa, 0xf0 // sxtb al r0 r10 ROR 16
};
const byte kInstruction_sxtb_al_r0_r10_ROR_24[] = {
  0x4f, 0xfa, 0xba, 0xf0 // sxtb al r0 r10 ROR 24
};
const byte kInstruction_sxtb_al_r0_r11_ROR_0[] = {
  0x4f, 0xfa, 0x8b, 0xf0 // sxtb al r0 r11 ROR 0
};
const byte kInstruction_sxtb_al_r0_r11_ROR_8[] = {
  0x4f, 0xfa, 0x9b, 0xf0 // sxtb al r0 r11 ROR 8
};
const byte kInstruction_sxtb_al_r0_r11_ROR_16[] = {
  0x4f, 0xfa, 0xab, 0xf0 // sxtb al r0 r11 ROR 16
};
const byte kInstruction_sxtb_al_r0_r11_ROR_24[] = {
  0x4f, 0xfa, 0xbb, 0xf0 // sxtb al r0 r11 ROR 24
};
const byte kInstruction_sxtb_al_r0_r12_ROR_0[] = {
  0x4f, 0xfa, 0x8c, 0xf0 // sxtb al r0 r12 ROR 0
};
const byte kInstruction_sxtb_al_r0_r12_ROR_8[] = {
  0x4f, 0xfa, 0x9c, 0xf0 // sxtb al r0 r12 ROR 8
};
const byte kInstruction_sxtb_al_r0_r12_ROR_16[] = {
  0x4f, 0xfa, 0xac, 0xf0 // sxtb al r0 r12 ROR 16
};
const byte kInstruction_sxtb_al_r0_r12_ROR_24[] = {
  0x4f, 0xfa, 0xbc, 0xf0 // sxtb al r0 r12 ROR 24
};
const byte kInstruction_sxtb_al_r0_r13_ROR_0[] = {
  0x4f, 0xfa, 0x8d, 0xf0 // sxtb al r0 r13 ROR 0
};
const byte kInstruction_sxtb_al_r0_r13_ROR_8[] = {
  0x4f, 0xfa, 0x9d, 0xf0 // sxtb al r0 r13 ROR 8
};
const byte kInstruction_sxtb_al_r0_r13_ROR_16[] = {
  0x4f, 0xfa, 0xad, 0xf0 // sxtb al r0 r13 ROR 16
};
const byte kInstruction_sxtb_al_r0_r13_ROR_24[] = {
  0x4f, 0xfa, 0xbd, 0xf0 // sxtb al r0 r13 ROR 24
};
const byte kInstruction_sxtb_al_r0_r14_ROR_0[] = {
  0x4f, 0xfa, 0x8e, 0xf0 // sxtb al r0 r14 ROR 0
};
const byte kInstruction_sxtb_al_r0_r14_ROR_8[] = {
  0x4f, 0xfa, 0x9e, 0xf0 // sxtb al r0 r14 ROR 8
};
const byte kInstruction_sxtb_al_r0_r14_ROR_16[] = {
  0x4f, 0xfa, 0xae, 0xf0 // sxtb al r0 r14 ROR 16
};
const byte kInstruction_sxtb_al_r0_r14_ROR_24[] = {
  0x4f, 0xfa, 0xbe, 0xf0 // sxtb al r0 r14 ROR 24
};
const byte kInstruction_sxtb_al_r1_r0_ROR_0[] = {
  0x41, 0xb2 // sxtb al r1 r0 ROR 0
};
const byte kInstruction_sxtb_al_r1_r0_ROR_8[] = {
  0x4f, 0xfa, 0x90, 0xf1 // sxtb al r1 r0 ROR 8
};
const byte kInstruction_sxtb_al_r1_r0_ROR_16[] = {
  0x4f, 0xfa, 0xa0, 0xf1 // sxtb al r1 r0 ROR 16
};
const byte kInstruction_sxtb_al_r1_r0_ROR_24[] = {
  0x4f, 0xfa, 0xb0, 0xf1 // sxtb al r1 r0 ROR 24
};
const byte kInstruction_sxtb_al_r1_r1_ROR_0[] = {
  0x49, 0xb2 // sxtb al r1 r1 ROR 0
};
const byte kInstruction_sxtb_al_r1_r1_ROR_8[] = {
  0x4f, 0xfa, 0x91, 0xf1 // sxtb al r1 r1 ROR 8
};
const byte kInstruction_sxtb_al_r1_r1_ROR_16[] = {
  0x4f, 0xfa, 0xa1, 0xf1 // sxtb al r1 r1 ROR 16
};
const byte kInstruction_sxtb_al_r1_r1_ROR_24[] = {
  0x4f, 0xfa, 0xb1, 0xf1 // sxtb al r1 r1 ROR 24
};
const byte kInstruction_sxtb_al_r1_r2_ROR_0[] = {
  0x51, 0xb2 // sxtb al r1 r2 ROR 0
};
const byte kInstruction_sxtb_al_r1_r2_ROR_8[] = {
  0x4f, 0xfa, 0x92, 0xf1 // sxtb al r1 r2 ROR 8
};
const byte kInstruction_sxtb_al_r1_r2_ROR_16[] = {
  0x4f, 0xfa, 0xa2, 0xf1 // sxtb al r1 r2 ROR 16
};
const byte kInstruction_sxtb_al_r1_r2_ROR_24[] = {
  0x4f, 0xfa, 0xb2, 0xf1 // sxtb al r1 r2 ROR 24
};
const byte kInstruction_sxtb_al_r1_r3_ROR_0[] = {
  0x59, 0xb2 // sxtb al r1 r3 ROR 0
};
const byte kInstruction_sxtb_al_r1_r3_ROR_8[] = {
  0x4f, 0xfa, 0x93, 0xf1 // sxtb al r1 r3 ROR 8
};
const byte kInstruction_sxtb_al_r1_r3_ROR_16[] = {
  0x4f, 0xfa, 0xa3, 0xf1 // sxtb al r1 r3 ROR 16
};
const byte kInstruction_sxtb_al_r1_r3_ROR_24[] = {
  0x4f, 0xfa, 0xb3, 0xf1 // sxtb al r1 r3 ROR 24
};
const byte kInstruction_sxtb_al_r1_r4_ROR_0[] = {
  0x61, 0xb2 // sxtb al r1 r4 ROR 0
};
const byte kInstruction_sxtb_al_r1_r4_ROR_8[] = {
  0x4f, 0xfa, 0x94, 0xf1 // sxtb al r1 r4 ROR 8
};
const byte kInstruction_sxtb_al_r1_r4_ROR_16[] = {
  0x4f, 0xfa, 0xa4, 0xf1 // sxtb al r1 r4 ROR 16
};
const byte kInstruction_sxtb_al_r1_r4_ROR_24[] = {
  0x4f, 0xfa, 0xb4, 0xf1 // sxtb al r1 r4 ROR 24
};
const byte kInstruction_sxtb_al_r1_r5_ROR_0[] = {
  0x69, 0xb2 // sxtb al r1 r5 ROR 0
};
const byte kInstruction_sxtb_al_r1_r5_ROR_8[] = {
  0x4f, 0xfa, 0x95, 0xf1 // sxtb al r1 r5 ROR 8
};
const byte kInstruction_sxtb_al_r1_r5_ROR_16[] = {
  0x4f, 0xfa, 0xa5, 0xf1 // sxtb al r1 r5 ROR 16
};
const byte kInstruction_sxtb_al_r1_r5_ROR_24[] = {
  0x4f, 0xfa, 0xb5, 0xf1 // sxtb al r1 r5 ROR 24
};
const byte kInstruction_sxtb_al_r1_r6_ROR_0[] = {
  0x71, 0xb2 // sxtb al r1 r6 ROR 0
};
const byte kInstruction_sxtb_al_r1_r6_ROR_8[] = {
  0x4f, 0xfa, 0x96, 0xf1 // sxtb al r1 r6 ROR 8
};
const byte kInstruction_sxtb_al_r1_r6_ROR_16[] = {
  0x4f, 0xfa, 0xa6, 0xf1 // sxtb al r1 r6 ROR 16
};
const byte kInstruction_sxtb_al_r1_r6_ROR_24[] = {
  0x4f, 0xfa, 0xb6, 0xf1 // sxtb al r1 r6 ROR 24
};
const byte kInstruction_sxtb_al_r1_r7_ROR_0[] = {
  0x79, 0xb2 // sxtb al r1 r7 ROR 0
};
const byte kInstruction_sxtb_al_r1_r7_ROR_8[] = {
  0x4f, 0xfa, 0x97, 0xf1 // sxtb al r1 r7 ROR 8
};
const byte kInstruction_sxtb_al_r1_r7_ROR_16[] = {
  0x4f, 0xfa, 0xa7, 0xf1 // sxtb al r1 r7 ROR 16
};
const byte kInstruction_sxtb_al_r1_r7_ROR_24[] = {
  0x4f, 0xfa, 0xb7, 0xf1 // sxtb al r1 r7 ROR 24
};
const byte kInstruction_sxtb_al_r1_r8_ROR_0[] = {
  0x4f, 0xfa, 0x88, 0xf1 // sxtb al r1 r8 ROR 0
};
const byte kInstruction_sxtb_al_r1_r8_ROR_8[] = {
  0x4f, 0xfa, 0x98, 0xf1 // sxtb al r1 r8 ROR 8
};
const byte kInstruction_sxtb_al_r1_r8_ROR_16[] = {
  0x4f, 0xfa, 0xa8, 0xf1 // sxtb al r1 r8 ROR 16
};
const byte kInstruction_sxtb_al_r1_r8_ROR_24[] = {
  0x4f, 0xfa, 0xb8, 0xf1 // sxtb al r1 r8 ROR 24
};
const byte kInstruction_sxtb_al_r1_r9_ROR_0[] = {
  0x4f, 0xfa, 0x89, 0xf1 // sxtb al r1 r9 ROR 0
};
const byte kInstruction_sxtb_al_r1_r9_ROR_8[] = {
  0x4f, 0xfa, 0x99, 0xf1 // sxtb al r1 r9 ROR 8
};
const byte kInstruction_sxtb_al_r1_r9_ROR_16[] = {
  0x4f, 0xfa, 0xa9, 0xf1 // sxtb al r1 r9 ROR 16
};
const byte kInstruction_sxtb_al_r1_r9_ROR_24[] = {
  0x4f, 0xfa, 0xb9, 0xf1 // sxtb al r1 r9 ROR 24
};
const byte kInstruction_sxtb_al_r1_r10_ROR_0[] = {
  0x4f, 0xfa, 0x8a, 0xf1 // sxtb al r1 r10 ROR 0
};
const byte kInstruction_sxtb_al_r1_r10_ROR_8[] = {
  0x4f, 0xfa, 0x9a, 0xf1 // sxtb al r1 r10 ROR 8
};
const byte kInstruction_sxtb_al_r1_r10_ROR_16[] = {
  0x4f, 0xfa, 0xaa, 0xf1 // sxtb al r1 r10 ROR 16
};
const byte kInstruction_sxtb_al_r1_r10_ROR_24[] = {
  0x4f, 0xfa, 0xba, 0xf1 // sxtb al r1 r10 ROR 24
};
const byte kInstruction_sxtb_al_r1_r11_ROR_0[] = {
  0x4f, 0xfa, 0x8b, 0xf1 // sxtb al r1 r11 ROR 0
};
const byte kInstruction_sxtb_al_r1_r11_ROR_8[] = {
  0x4f, 0xfa, 0x9b, 0xf1 // sxtb al r1 r11 ROR 8
};
const byte kInstruction_sxtb_al_r1_r11_ROR_16[] = {
  0x4f, 0xfa, 0xab, 0xf1 // sxtb al r1 r11 ROR 16
};
const byte kInstruction_sxtb_al_r1_r11_ROR_24[] = {
  0x4f, 0xfa, 0xbb, 0xf1 // sxtb al r1 r11 ROR 24
};
const byte kInstruction_sxtb_al_r1_r12_ROR_0[] = {
  0x4f, 0xfa, 0x8c, 0xf1 // sxtb al r1 r12 ROR 0
};
const byte kInstruction_sxtb_al_r1_r12_ROR_8[] = {
  0x4f, 0xfa, 0x9c, 0xf1 // sxtb al r1 r12 ROR 8
};
const byte kInstruction_sxtb_al_r1_r12_ROR_16[] = {
  0x4f, 0xfa, 0xac, 0xf1 // sxtb al r1 r12 ROR 16
};
const byte kInstruction_sxtb_al_r1_r12_ROR_24[] = {
  0x4f, 0xfa, 0xbc, 0xf1 // sxtb al r1 r12 ROR 24
};
const byte kInstruction_sxtb_al_r1_r13_ROR_0[] = {
  0x4f, 0xfa, 0x8d, 0xf1 // sxtb al r1 r13 ROR 0
};
const byte kInstruction_sxtb_al_r1_r13_ROR_8[] = {
  0x4f, 0xfa, 0x9d, 0xf1 // sxtb al r1 r13 ROR 8
};
const byte kInstruction_sxtb_al_r1_r13_ROR_16[] = {
  0x4f, 0xfa, 0xad, 0xf1 // sxtb al r1 r13 ROR 16
};
const byte kInstruction_sxtb_al_r1_r13_ROR_24[] = {
  0x4f, 0xfa, 0xbd, 0xf1 // sxtb al r1 r13 ROR 24
};
const byte kInstruction_sxtb_al_r1_r14_ROR_0[] = {
  0x4f, 0xfa, 0x8e, 0xf1 // sxtb al r1 r14 ROR 0
};
const byte kInstruction_sxtb_al_r1_r14_ROR_8[] = {
  0x4f, 0xfa, 0x9e, 0xf1 // sxtb al r1 r14 ROR 8
};
const byte kInstruction_sxtb_al_r1_r14_ROR_16[] = {
  0x4f, 0xfa, 0xae, 0xf1 // sxtb al r1 r14 ROR 16
};
const byte kInstruction_sxtb_al_r1_r14_ROR_24[] = {
  0x4f, 0xfa, 0xbe, 0xf1 // sxtb al r1 r14 ROR 24
};
const byte kInstruction_sxtb_al_r2_r0_ROR_0[] = {
  0x42, 0xb2 // sxtb al r2 r0 ROR 0
};
const byte kInstruction_sxtb_al_r2_r0_ROR_8[] = {
  0x4f, 0xfa, 0x90, 0xf2 // sxtb al r2 r0 ROR 8
};
const byte kInstruction_sxtb_al_r2_r0_ROR_16[] = {
  0x4f, 0xfa, 0xa0, 0xf2 // sxtb al r2 r0 ROR 16
};
const byte kInstruction_sxtb_al_r2_r0_ROR_24[] = {
  0x4f, 0xfa, 0xb0, 0xf2 // sxtb al r2 r0 ROR 24
};
const byte kInstruction_sxtb_al_r2_r1_ROR_0[] = {
  0x4a, 0xb2 // sxtb al r2 r1 ROR 0
};
const byte kInstruction_sxtb_al_r2_r1_ROR_8[] = {
  0x4f, 0xfa, 0x91, 0xf2 // sxtb al r2 r1 ROR 8
};
const byte kInstruction_sxtb_al_r2_r1_ROR_16[] = {
  0x4f, 0xfa, 0xa1, 0xf2 // sxtb al r2 r1 ROR 16
};
const byte kInstruction_sxtb_al_r2_r1_ROR_24[] = {
  0x4f, 0xfa, 0xb1, 0xf2 // sxtb al r2 r1 ROR 24
};
const byte kInstruction_sxtb_al_r2_r2_ROR_0[] = {
  0x52, 0xb2 // sxtb al r2 r2 ROR 0
};
const byte kInstruction_sxtb_al_r2_r2_ROR_8[] = {
  0x4f, 0xfa, 0x92, 0xf2 // sxtb al r2 r2 ROR 8
};
const byte kInstruction_sxtb_al_r2_r2_ROR_16[] = {
  0x4f, 0xfa, 0xa2, 0xf2 // sxtb al r2 r2 ROR 16
};
const byte kInstruction_sxtb_al_r2_r2_ROR_24[] = {
  0x4f, 0xfa, 0xb2, 0xf2 // sxtb al r2 r2 ROR 24
};
const byte kInstruction_sxtb_al_r2_r3_ROR_0[] = {
  0x5a, 0xb2 // sxtb al r2 r3 ROR 0
};
const byte kInstruction_sxtb_al_r2_r3_ROR_8[] = {
  0x4f, 0xfa, 0x93, 0xf2 // sxtb al r2 r3 ROR 8
};
const byte kInstruction_sxtb_al_r2_r3_ROR_16[] = {
  0x4f, 0xfa, 0xa3, 0xf2 // sxtb al r2 r3 ROR 16
};
const byte kInstruction_sxtb_al_r2_r3_ROR_24[] = {
  0x4f, 0xfa, 0xb3, 0xf2 // sxtb al r2 r3 ROR 24
};
const byte kInstruction_sxtb_al_r2_r4_ROR_0[] = {
  0x62, 0xb2 // sxtb al r2 r4 ROR 0
};
const byte kInstruction_sxtb_al_r2_r4_ROR_8[] = {
  0x4f, 0xfa, 0x94, 0xf2 // sxtb al r2 r4 ROR 8
};
const byte kInstruction_sxtb_al_r2_r4_ROR_16[] = {
  0x4f, 0xfa, 0xa4, 0xf2 // sxtb al r2 r4 ROR 16
};
const byte kInstruction_sxtb_al_r2_r4_ROR_24[] = {
  0x4f, 0xfa, 0xb4, 0xf2 // sxtb al r2 r4 ROR 24
};
const byte kInstruction_sxtb_al_r2_r5_ROR_0[] = {
  0x6a, 0xb2 // sxtb al r2 r5 ROR 0
};
const byte kInstruction_sxtb_al_r2_r5_ROR_8[] = {
  0x4f, 0xfa, 0x95, 0xf2 // sxtb al r2 r5 ROR 8
};
const byte kInstruction_sxtb_al_r2_r5_ROR_16[] = {
  0x4f, 0xfa, 0xa5, 0xf2 // sxtb al r2 r5 ROR 16
};
const byte kInstruction_sxtb_al_r2_r5_ROR_24[] = {
  0x4f, 0xfa, 0xb5, 0xf2 // sxtb al r2 r5 ROR 24
};
const byte kInstruction_sxtb_al_r2_r6_ROR_0[] = {
  0x72, 0xb2 // sxtb al r2 r6 ROR 0
};
const byte kInstruction_sxtb_al_r2_r6_ROR_8[] = {
  0x4f, 0xfa, 0x96, 0xf2 // sxtb al r2 r6 ROR 8
};
const byte kInstruction_sxtb_al_r2_r6_ROR_16[] = {
  0x4f, 0xfa, 0xa6, 0xf2 // sxtb al r2 r6 ROR 16
};
const byte kInstruction_sxtb_al_r2_r6_ROR_24[] = {
  0x4f, 0xfa, 0xb6, 0xf2 // sxtb al r2 r6 ROR 24
};
const byte kInstruction_sxtb_al_r2_r7_ROR_0[] = {
  0x7a, 0xb2 // sxtb al r2 r7 ROR 0
};
const byte kInstruction_sxtb_al_r2_r7_ROR_8[] = {
  0x4f, 0xfa, 0x97, 0xf2 // sxtb al r2 r7 ROR 8
};
const byte kInstruction_sxtb_al_r2_r7_ROR_16[] = {
  0x4f, 0xfa, 0xa7, 0xf2 // sxtb al r2 r7 ROR 16
};
const byte kInstruction_sxtb_al_r2_r7_ROR_24[] = {
  0x4f, 0xfa, 0xb7, 0xf2 // sxtb al r2 r7 ROR 24
};
const byte kInstruction_sxtb_al_r2_r8_ROR_0[] = {
  0x4f, 0xfa, 0x88, 0xf2 // sxtb al r2 r8 ROR 0
};
const byte kInstruction_sxtb_al_r2_r8_ROR_8[] = {
  0x4f, 0xfa, 0x98, 0xf2 // sxtb al r2 r8 ROR 8
};
const byte kInstruction_sxtb_al_r2_r8_ROR_16[] = {
  0x4f, 0xfa, 0xa8, 0xf2 // sxtb al r2 r8 ROR 16
};
const byte kInstruction_sxtb_al_r2_r8_ROR_24[] = {
  0x4f, 0xfa, 0xb8, 0xf2 // sxtb al r2 r8 ROR 24
};
const byte kInstruction_sxtb_al_r2_r9_ROR_0[] = {
  0x4f, 0xfa, 0x89, 0xf2 // sxtb al r2 r9 ROR 0
};
const byte kInstruction_sxtb_al_r2_r9_ROR_8[] = {
  0x4f, 0xfa, 0x99, 0xf2 // sxtb al r2 r9 ROR 8
};
const byte kInstruction_sxtb_al_r2_r9_ROR_16[] = {
  0x4f, 0xfa, 0xa9, 0xf2 // sxtb al r2 r9 ROR 16
};
const byte kInstruction_sxtb_al_r2_r9_ROR_24[] = {
  0x4f, 0xfa, 0xb9, 0xf2 // sxtb al r2 r9 ROR 24
};
const byte kInstruction_sxtb_al_r2_r10_ROR_0[] = {
  0x4f, 0xfa, 0x8a, 0xf2 // sxtb al r2 r10 ROR 0
};
const byte kInstruction_sxtb_al_r2_r10_ROR_8[] = {
  0x4f, 0xfa, 0x9a, 0xf2 // sxtb al r2 r10 ROR 8
};
const byte kInstruction_sxtb_al_r2_r10_ROR_16[] = {
  0x4f, 0xfa, 0xaa, 0xf2 // sxtb al r2 r10 ROR 16
};
const byte kInstruction_sxtb_al_r2_r10_ROR_24[] = {
  0x4f, 0xfa, 0xba, 0xf2 // sxtb al r2 r10 ROR 24
};
const byte kInstruction_sxtb_al_r2_r11_ROR_0[] = {
  0x4f, 0xfa, 0x8b, 0xf2 // sxtb al r2 r11 ROR 0
};
const byte kInstruction_sxtb_al_r2_r11_ROR_8[] = {
  0x4f, 0xfa, 0x9b, 0xf2 // sxtb al r2 r11 ROR 8
};
const byte kInstruction_sxtb_al_r2_r11_ROR_16[] = {
  0x4f, 0xfa, 0xab, 0xf2 // sxtb al r2 r11 ROR 16
};
const byte kInstruction_sxtb_al_r2_r11_ROR_24[] = {
  0x4f, 0xfa, 0xbb, 0xf2 // sxtb al r2 r11 ROR 24
};
const byte kInstruction_sxtb_al_r2_r12_ROR_0[] = {
  0x4f, 0xfa, 0x8c, 0xf2 // sxtb al r2 r12 ROR 0
};
const byte kInstruction_sxtb_al_r2_r12_ROR_8[] = {
  0x4f, 0xfa, 0x9c, 0xf2 // sxtb al r2 r12 ROR 8
};
const byte kInstruction_sxtb_al_r2_r12_ROR_16[] = {
  0x4f, 0xfa, 0xac, 0xf2 // sxtb al r2 r12 ROR 16
};
const byte kInstruction_sxtb_al_r2_r12_ROR_24[] = {
  0x4f, 0xfa, 0xbc, 0xf2 // sxtb al r2 r12 ROR 24
};
const byte kInstruction_sxtb_al_r2_r13_ROR_0[] = {
  0x4f, 0xfa, 0x8d, 0xf2 // sxtb al r2 r13 ROR 0
};
const byte kInstruction_sxtb_al_r2_r13_ROR_8[] = {
  0x4f, 0xfa, 0x9d, 0xf2 // sxtb al r2 r13 ROR 8
};
const byte kInstruction_sxtb_al_r2_r13_ROR_16[] = {
  0x4f, 0xfa, 0xad, 0xf2 // sxtb al r2 r13 ROR 16
};
const byte kInstruction_sxtb_al_r2_r13_ROR_24[] = {
  0x4f, 0xfa, 0xbd, 0xf2 // sxtb al r2 r13 ROR 24
};
const byte kInstruction_sxtb_al_r2_r14_ROR_0[] = {
  0x4f, 0xfa, 0x8e, 0xf2 // sxtb al r2 r14 ROR 0
};
const byte kInstruction_sxtb_al_r2_r14_ROR_8[] = {
  0x4f, 0xfa, 0x9e, 0xf2 // sxtb al r2 r14 ROR 8
};
const byte kInstruction_sxtb_al_r2_r14_ROR_16[] = {
  0x4f, 0xfa, 0xae, 0xf2 // sxtb al r2 r14 ROR 16
};
const byte kInstruction_sxtb_al_r2_r14_ROR_24[] = {
  0x4f, 0xfa, 0xbe, 0xf2 // sxtb al r2 r14 ROR 24
};
const byte kInstruction_sxtb_al_r3_r0_ROR_0[] = {
  0x43, 0xb2 // sxtb al r3 r0 ROR 0
};
const byte kInstruction_sxtb_al_r3_r0_ROR_8[] = {
  0x4f, 0xfa, 0x90, 0xf3 // sxtb al r3 r0 ROR 8
};
const byte kInstruction_sxtb_al_r3_r0_ROR_16[] = {
  0x4f, 0xfa, 0xa0, 0xf3 // sxtb al r3 r0 ROR 16
};
const byte kInstruction_sxtb_al_r3_r0_ROR_24[] = {
  0x4f, 0xfa, 0xb0, 0xf3 // sxtb al r3 r0 ROR 24
};
const byte kInstruction_sxtb_al_r3_r1_ROR_0[] = {
  0x4b, 0xb2 // sxtb al r3 r1 ROR 0
};
const byte kInstruction_sxtb_al_r3_r1_ROR_8[] = {
  0x4f, 0xfa, 0x91, 0xf3 // sxtb al r3 r1 ROR 8
};
const byte kInstruction_sxtb_al_r3_r1_ROR_16[] = {
  0x4f, 0xfa, 0xa1, 0xf3 // sxtb al r3 r1 ROR 16
};
const byte kInstruction_sxtb_al_r3_r1_ROR_24[] = {
  0x4f, 0xfa, 0xb1, 0xf3 // sxtb al r3 r1 ROR 24
};
const byte kInstruction_sxtb_al_r3_r2_ROR_0[] = {
  0x53, 0xb2 // sxtb al r3 r2 ROR 0
};
const byte kInstruction_sxtb_al_r3_r2_ROR_8[] = {
  0x4f, 0xfa, 0x92, 0xf3 // sxtb al r3 r2 ROR 8
};
const byte kInstruction_sxtb_al_r3_r2_ROR_16[] = {
  0x4f, 0xfa, 0xa2, 0xf3 // sxtb al r3 r2 ROR 16
};
const byte kInstruction_sxtb_al_r3_r2_ROR_24[] = {
  0x4f, 0xfa, 0xb2, 0xf3 // sxtb al r3 r2 ROR 24
};
const byte kInstruction_sxtb_al_r3_r3_ROR_0[] = {
  0x5b, 0xb2 // sxtb al r3 r3 ROR 0
};
const byte kInstruction_sxtb_al_r3_r3_ROR_8[] = {
  0x4f, 0xfa, 0x93, 0xf3 // sxtb al r3 r3 ROR 8
};
const byte kInstruction_sxtb_al_r3_r3_ROR_16[] = {
  0x4f, 0xfa, 0xa3, 0xf3 // sxtb al r3 r3 ROR 16
};
const byte kInstruction_sxtb_al_r3_r3_ROR_24[] = {
  0x4f, 0xfa, 0xb3, 0xf3 // sxtb al r3 r3 ROR 24
};
const byte kInstruction_sxtb_al_r3_r4_ROR_0[] = {
  0x63, 0xb2 // sxtb al r3 r4 ROR 0
};
const byte kInstruction_sxtb_al_r3_r4_ROR_8[] = {
  0x4f, 0xfa, 0x94, 0xf3 // sxtb al r3 r4 ROR 8
};
const byte kInstruction_sxtb_al_r3_r4_ROR_16[] = {
  0x4f, 0xfa, 0xa4, 0xf3 // sxtb al r3 r4 ROR 16
};
const byte kInstruction_sxtb_al_r3_r4_ROR_24[] = {
  0x4f, 0xfa, 0xb4, 0xf3 // sxtb al r3 r4 ROR 24
};
const byte kInstruction_sxtb_al_r3_r5_ROR_0[] = {
  0x6b, 0xb2 // sxtb al r3 r5 ROR 0
};
const byte kInstruction_sxtb_al_r3_r5_ROR_8[] = {
  0x4f, 0xfa, 0x95, 0xf3 // sxtb al r3 r5 ROR 8
};
const byte kInstruction_sxtb_al_r3_r5_ROR_16[] = {
  0x4f, 0xfa, 0xa5, 0xf3 // sxtb al r3 r5 ROR 16
};
const byte kInstruction_sxtb_al_r3_r5_ROR_24[] = {
  0x4f, 0xfa, 0xb5, 0xf3 // sxtb al r3 r5 ROR 24
};
const byte kInstruction_sxtb_al_r3_r6_ROR_0[] = {
  0x73, 0xb2 // sxtb al r3 r6 ROR 0
};
const byte kInstruction_sxtb_al_r3_r6_ROR_8[] = {
  0x4f, 0xfa, 0x96, 0xf3 // sxtb al r3 r6 ROR 8
};
const byte kInstruction_sxtb_al_r3_r6_ROR_16[] = {
  0x4f, 0xfa, 0xa6, 0xf3 // sxtb al r3 r6 ROR 16
};
const byte kInstruction_sxtb_al_r3_r6_ROR_24[] = {
  0x4f, 0xfa, 0xb6, 0xf3 // sxtb al r3 r6 ROR 24
};
const byte kInstruction_sxtb_al_r3_r7_ROR_0[] = {
  0x7b, 0xb2 // sxtb al r3 r7 ROR 0
};
const byte kInstruction_sxtb_al_r3_r7_ROR_8[] = {
  0x4f, 0xfa, 0x97, 0xf3 // sxtb al r3 r7 ROR 8
};
const byte kInstruction_sxtb_al_r3_r7_ROR_16[] = {
  0x4f, 0xfa, 0xa7, 0xf3 // sxtb al r3 r7 ROR 16
};
const byte kInstruction_sxtb_al_r3_r7_ROR_24[] = {
  0x4f, 0xfa, 0xb7, 0xf3 // sxtb al r3 r7 ROR 24
};
const byte kInstruction_sxtb_al_r3_r8_ROR_0[] = {
  0x4f, 0xfa, 0x88, 0xf3 // sxtb al r3 r8 ROR 0
};
const byte kInstruction_sxtb_al_r3_r8_ROR_8[] = {
  0x4f, 0xfa, 0x98, 0xf3 // sxtb al r3 r8 ROR 8
};
const byte kInstruction_sxtb_al_r3_r8_ROR_16[] = {
  0x4f, 0xfa, 0xa8, 0xf3 // sxtb al r3 r8 ROR 16
};
const byte kInstruction_sxtb_al_r3_r8_ROR_24[] = {
  0x4f, 0xfa, 0xb8, 0xf3 // sxtb al r3 r8 ROR 24
};
const byte kInstruction_sxtb_al_r3_r9_ROR_0[] = {
  0x4f, 0xfa, 0x89, 0xf3 // sxtb al r3 r9 ROR 0
};
const byte kInstruction_sxtb_al_r3_r9_ROR_8[] = {
  0x4f, 0xfa, 0x99, 0xf3 // sxtb al r3 r9 ROR 8
};
const byte kInstruction_sxtb_al_r3_r9_ROR_16[] = {
  0x4f, 0xfa, 0xa9, 0xf3 // sxtb al r3 r9 ROR 16
};
const byte kInstruction_sxtb_al_r3_r9_ROR_24[] = {
  0x4f, 0xfa, 0xb9, 0xf3 // sxtb al r3 r9 ROR 24
};
const byte kInstruction_sxtb_al_r3_r10_ROR_0[] = {
  0x4f, 0xfa, 0x8a, 0xf3 // sxtb al r3 r10 ROR 0
};
const byte kInstruction_sxtb_al_r3_r10_ROR_8[] = {
  0x4f, 0xfa, 0x9a, 0xf3 // sxtb al r3 r10 ROR 8
};
const byte kInstruction_sxtb_al_r3_r10_ROR_16[] = {
  0x4f, 0xfa, 0xaa, 0xf3 // sxtb al r3 r10 ROR 16
};
const byte kInstruction_sxtb_al_r3_r10_ROR_24[] = {
  0x4f, 0xfa, 0xba, 0xf3 // sxtb al r3 r10 ROR 24
};
const byte kInstruction_sxtb_al_r3_r11_ROR_0[] = {
  0x4f, 0xfa, 0x8b, 0xf3 // sxtb al r3 r11 ROR 0
};
const byte kInstruction_sxtb_al_r3_r11_ROR_8[] = {
  0x4f, 0xfa, 0x9b, 0xf3 // sxtb al r3 r11 ROR 8
};
const byte kInstruction_sxtb_al_r3_r11_ROR_16[] = {
  0x4f, 0xfa, 0xab, 0xf3 // sxtb al r3 r11 ROR 16
};
const byte kInstruction_sxtb_al_r3_r11_ROR_24[] = {
  0x4f, 0xfa, 0xbb, 0xf3 // sxtb al r3 r11 ROR 24
};
const byte kInstruction_sxtb_al_r3_r12_ROR_0[] = {
  0x4f, 0xfa, 0x8c, 0xf3 // sxtb al r3 r12 ROR 0
};
const byte kInstruction_sxtb_al_r3_r12_ROR_8[] = {
  0x4f, 0xfa, 0x9c, 0xf3 // sxtb al r3 r12 ROR 8
};
const byte kInstruction_sxtb_al_r3_r12_ROR_16[] = {
  0x4f, 0xfa, 0xac, 0xf3 // sxtb al r3 r12 ROR 16
};
const byte kInstruction_sxtb_al_r3_r12_ROR_24[] = {
  0x4f, 0xfa, 0xbc, 0xf3 // sxtb al r3 r12 ROR 24
};
const byte kInstruction_sxtb_al_r3_r13_ROR_0[] = {
  0x4f, 0xfa, 0x8d, 0xf3 // sxtb al r3 r13 ROR 0
};
const byte kInstruction_sxtb_al_r3_r13_ROR_8[] = {
  0x4f, 0xfa, 0x9d, 0xf3 // sxtb al r3 r13 ROR 8
};
const byte kInstruction_sxtb_al_r3_r13_ROR_16[] = {
  0x4f, 0xfa, 0xad, 0xf3 // sxtb al r3 r13 ROR 16
};
const byte kInstruction_sxtb_al_r3_r13_ROR_24[] = {
  0x4f, 0xfa, 0xbd, 0xf3 // sxtb al r3 r13 ROR 24
};
const byte kInstruction_sxtb_al_r3_r14_ROR_0[] = {
  0x4f, 0xfa, 0x8e, 0xf3 // sxtb al r3 r14 ROR 0
};
const byte kInstruction_sxtb_al_r3_r14_ROR_8[] = {
  0x4f, 0xfa, 0x9e, 0xf3 // sxtb al r3 r14 ROR 8
};
const byte kInstruction_sxtb_al_r3_r14_ROR_16[] = {
  0x4f, 0xfa, 0xae, 0xf3 // sxtb al r3 r14 ROR 16
};
const byte kInstruction_sxtb_al_r3_r14_ROR_24[] = {
  0x4f, 0xfa, 0xbe, 0xf3 // sxtb al r3 r14 ROR 24
};
const byte kInstruction_sxtb_al_r4_r0_ROR_0[] = {
  0x44, 0xb2 // sxtb al r4 r0 ROR 0
};
const byte kInstruction_sxtb_al_r4_r0_ROR_8[] = {
  0x4f, 0xfa, 0x90, 0xf4 // sxtb al r4 r0 ROR 8
};
const byte kInstruction_sxtb_al_r4_r0_ROR_16[] = {
  0x4f, 0xfa, 0xa0, 0xf4 // sxtb al r4 r0 ROR 16
};
const byte kInstruction_sxtb_al_r4_r0_ROR_24[] = {
  0x4f, 0xfa, 0xb0, 0xf4 // sxtb al r4 r0 ROR 24
};
const byte kInstruction_sxtb_al_r4_r1_ROR_0[] = {
  0x4c, 0xb2 // sxtb al r4 r1 ROR 0
};
const byte kInstruction_sxtb_al_r4_r1_ROR_8[] = {
  0x4f, 0xfa, 0x91, 0xf4 // sxtb al r4 r1 ROR 8
};
const byte kInstruction_sxtb_al_r4_r1_ROR_16[] = {
  0x4f, 0xfa, 0xa1, 0xf4 // sxtb al r4 r1 ROR 16
};
const byte kInstruction_sxtb_al_r4_r1_ROR_24[] = {
  0x4f, 0xfa, 0xb1, 0xf4 // sxtb al r4 r1 ROR 24
};
const byte kInstruction_sxtb_al_r4_r2_ROR_0[] = {
  0x54, 0xb2 // sxtb al r4 r2 ROR 0
};
const byte kInstruction_sxtb_al_r4_r2_ROR_8[] = {
  0x4f, 0xfa, 0x92, 0xf4 // sxtb al r4 r2 ROR 8
};
const byte kInstruction_sxtb_al_r4_r2_ROR_16[] = {
  0x4f, 0xfa, 0xa2, 0xf4 // sxtb al r4 r2 ROR 16
};
const byte kInstruction_sxtb_al_r4_r2_ROR_24[] = {
  0x4f, 0xfa, 0xb2, 0xf4 // sxtb al r4 r2 ROR 24
};
const byte kInstruction_sxtb_al_r4_r3_ROR_0[] = {
  0x5c, 0xb2 // sxtb al r4 r3 ROR 0
};
const byte kInstruction_sxtb_al_r4_r3_ROR_8[] = {
  0x4f, 0xfa, 0x93, 0xf4 // sxtb al r4 r3 ROR 8
};
const byte kInstruction_sxtb_al_r4_r3_ROR_16[] = {
  0x4f, 0xfa, 0xa3, 0xf4 // sxtb al r4 r3 ROR 16
};
const byte kInstruction_sxtb_al_r4_r3_ROR_24[] = {
  0x4f, 0xfa, 0xb3, 0xf4 // sxtb al r4 r3 ROR 24
};
const byte kInstruction_sxtb_al_r4_r4_ROR_0[] = {
  0x64, 0xb2 // sxtb al r4 r4 ROR 0
};
const byte kInstruction_sxtb_al_r4_r4_ROR_8[] = {
  0x4f, 0xfa, 0x94, 0xf4 // sxtb al r4 r4 ROR 8
};
const byte kInstruction_sxtb_al_r4_r4_ROR_16[] = {
  0x4f, 0xfa, 0xa4, 0xf4 // sxtb al r4 r4 ROR 16
};
const byte kInstruction_sxtb_al_r4_r4_ROR_24[] = {
  0x4f, 0xfa, 0xb4, 0xf4 // sxtb al r4 r4 ROR 24
};
const byte kInstruction_sxtb_al_r4_r5_ROR_0[] = {
  0x6c, 0xb2 // sxtb al r4 r5 ROR 0
};
const byte kInstruction_sxtb_al_r4_r5_ROR_8[] = {
  0x4f, 0xfa, 0x95, 0xf4 // sxtb al r4 r5 ROR 8
};
const byte kInstruction_sxtb_al_r4_r5_ROR_16[] = {
  0x4f, 0xfa, 0xa5, 0xf4 // sxtb al r4 r5 ROR 16
};
const byte kInstruction_sxtb_al_r4_r5_ROR_24[] = {
  0x4f, 0xfa, 0xb5, 0xf4 // sxtb al r4 r5 ROR 24
};
const byte kInstruction_sxtb_al_r4_r6_ROR_0[] = {
  0x74, 0xb2 // sxtb al r4 r6 ROR 0
};
const byte kInstruction_sxtb_al_r4_r6_ROR_8[] = {
  0x4f, 0xfa, 0x96, 0xf4 // sxtb al r4 r6 ROR 8
};
const byte kInstruction_sxtb_al_r4_r6_ROR_16[] = {
  0x4f, 0xfa, 0xa6, 0xf4 // sxtb al r4 r6 ROR 16
};
const byte kInstruction_sxtb_al_r4_r6_ROR_24[] = {
  0x4f, 0xfa, 0xb6, 0xf4 // sxtb al r4 r6 ROR 24
};
const byte kInstruction_sxtb_al_r4_r7_ROR_0[] = {
  0x7c, 0xb2 // sxtb al r4 r7 ROR 0
};
const byte kInstruction_sxtb_al_r4_r7_ROR_8[] = {
  0x4f, 0xfa, 0x97, 0xf4 // sxtb al r4 r7 ROR 8
};
const byte kInstruction_sxtb_al_r4_r7_ROR_16[] = {
  0x4f, 0xfa, 0xa7, 0xf4 // sxtb al r4 r7 ROR 16
};
const byte kInstruction_sxtb_al_r4_r7_ROR_24[] = {
  0x4f, 0xfa, 0xb7, 0xf4 // sxtb al r4 r7 ROR 24
};
const byte kInstruction_sxtb_al_r4_r8_ROR_0[] = {
  0x4f, 0xfa, 0x88, 0xf4 // sxtb al r4 r8 ROR 0
};
const byte kInstruction_sxtb_al_r4_r8_ROR_8[] = {
  0x4f, 0xfa, 0x98, 0xf4 // sxtb al r4 r8 ROR 8
};
const byte kInstruction_sxtb_al_r4_r8_ROR_16[] = {
  0x4f, 0xfa, 0xa8, 0xf4 // sxtb al r4 r8 ROR 16
};
const byte kInstruction_sxtb_al_r4_r8_ROR_24[] = {
  0x4f, 0xfa, 0xb8, 0xf4 // sxtb al r4 r8 ROR 24
};
const byte kInstruction_sxtb_al_r4_r9_ROR_0[] = {
  0x4f, 0xfa, 0x89, 0xf4 // sxtb al r4 r9 ROR 0
};
const byte kInstruction_sxtb_al_r4_r9_ROR_8[] = {
  0x4f, 0xfa, 0x99, 0xf4 // sxtb al r4 r9 ROR 8
};
const byte kInstruction_sxtb_al_r4_r9_ROR_16[] = {
  0x4f, 0xfa, 0xa9, 0xf4 // sxtb al r4 r9 ROR 16
};
const byte kInstruction_sxtb_al_r4_r9_ROR_24[] = {
  0x4f, 0xfa, 0xb9, 0xf4 // sxtb al r4 r9 ROR 24
};
const byte kInstruction_sxtb_al_r4_r10_ROR_0[] = {
  0x4f, 0xfa, 0x8a, 0xf4 // sxtb al r4 r10 ROR 0
};
const byte kInstruction_sxtb_al_r4_r10_ROR_8[] = {
  0x4f, 0xfa, 0x9a, 0xf4 // sxtb al r4 r10 ROR 8
};
const byte kInstruction_sxtb_al_r4_r10_ROR_16[] = {
  0x4f, 0xfa, 0xaa, 0xf4 // sxtb al r4 r10 ROR 16
};
const byte kInstruction_sxtb_al_r4_r10_ROR_24[] = {
  0x4f, 0xfa, 0xba, 0xf4 // sxtb al r4 r10 ROR 24
};
const byte kInstruction_sxtb_al_r4_r11_ROR_0[] = {
  0x4f, 0xfa, 0x8b, 0xf4 // sxtb al r4 r11 ROR 0
};
const byte kInstruction_sxtb_al_r4_r11_ROR_8[] = {
  0x4f, 0xfa, 0x9b, 0xf4 // sxtb al r4 r11 ROR 8
};
const byte kInstruction_sxtb_al_r4_r11_ROR_16[] = {
  0x4f, 0xfa, 0xab, 0xf4 // sxtb al r4 r11 ROR 16
};
const byte kInstruction_sxtb_al_r4_r11_ROR_24[] = {
  0x4f, 0xfa, 0xbb, 0xf4 // sxtb al r4 r11 ROR 24
};
const byte kInstruction_sxtb_al_r4_r12_ROR_0[] = {
  0x4f, 0xfa, 0x8c, 0xf4 // sxtb al r4 r12 ROR 0
};
const byte kInstruction_sxtb_al_r4_r12_ROR_8[] = {
  0x4f, 0xfa, 0x9c, 0xf4 // sxtb al r4 r12 ROR 8
};
const byte kInstruction_sxtb_al_r4_r12_ROR_16[] = {
  0x4f, 0xfa, 0xac, 0xf4 // sxtb al r4 r12 ROR 16
};
const byte kInstruction_sxtb_al_r4_r12_ROR_24[] = {
  0x4f, 0xfa, 0xbc, 0xf4 // sxtb al r4 r12 ROR 24
};
const byte kInstruction_sxtb_al_r4_r13_ROR_0[] = {
  0x4f, 0xfa, 0x8d, 0xf4 // sxtb al r4 r13 ROR 0
};
const byte kInstruction_sxtb_al_r4_r13_ROR_8[] = {
  0x4f, 0xfa, 0x9d, 0xf4 // sxtb al r4 r13 ROR 8
};
const byte kInstruction_sxtb_al_r4_r13_ROR_16[] = {
  0x4f, 0xfa, 0xad, 0xf4 // sxtb al r4 r13 ROR 16
};
const byte kInstruction_sxtb_al_r4_r13_ROR_24[] = {
  0x4f, 0xfa, 0xbd, 0xf4 // sxtb al r4 r13 ROR 24
};
const byte kInstruction_sxtb_al_r4_r14_ROR_0[] = {
  0x4f, 0xfa, 0x8e, 0xf4 // sxtb al r4 r14 ROR 0
};
const byte kInstruction_sxtb_al_r4_r14_ROR_8[] = {
  0x4f, 0xfa, 0x9e, 0xf4 // sxtb al r4 r14 ROR 8
};
const byte kInstruction_sxtb_al_r4_r14_ROR_16[] = {
  0x4f, 0xfa, 0xae, 0xf4 // sxtb al r4 r14 ROR 16
};
const byte kInstruction_sxtb_al_r4_r14_ROR_24[] = {
  0x4f, 0xfa, 0xbe, 0xf4 // sxtb al r4 r14 ROR 24
};
const byte kInstruction_sxtb_al_r5_r0_ROR_0[] = {
  0x45, 0xb2 // sxtb al r5 r0 ROR 0
};
const byte kInstruction_sxtb_al_r5_r0_ROR_8[] = {
  0x4f, 0xfa, 0x90, 0xf5 // sxtb al r5 r0 ROR 8
};
const byte kInstruction_sxtb_al_r5_r0_ROR_16[] = {
  0x4f, 0xfa, 0xa0, 0xf5 // sxtb al r5 r0 ROR 16
};
const byte kInstruction_sxtb_al_r5_r0_ROR_24[] = {
  0x4f, 0xfa, 0xb0, 0xf5 // sxtb al r5 r0 ROR 24
};
const byte kInstruction_sxtb_al_r5_r1_ROR_0[] = {
  0x4d, 0xb2 // sxtb al r5 r1 ROR 0
};
const byte kInstruction_sxtb_al_r5_r1_ROR_8[] = {
  0x4f, 0xfa, 0x91, 0xf5 // sxtb al r5 r1 ROR 8
};
const byte kInstruction_sxtb_al_r5_r1_ROR_16[] = {
  0x4f, 0xfa, 0xa1, 0xf5 // sxtb al r5 r1 ROR 16
};
const byte kInstruction_sxtb_al_r5_r1_ROR_24[] = {
  0x4f, 0xfa, 0xb1, 0xf5 // sxtb al r5 r1 ROR 24
};
const byte kInstruction_sxtb_al_r5_r2_ROR_0[] = {
  0x55, 0xb2 // sxtb al r5 r2 ROR 0
};
const byte kInstruction_sxtb_al_r5_r2_ROR_8[] = {
  0x4f, 0xfa, 0x92, 0xf5 // sxtb al r5 r2 ROR 8
};
const byte kInstruction_sxtb_al_r5_r2_ROR_16[] = {
  0x4f, 0xfa, 0xa2, 0xf5 // sxtb al r5 r2 ROR 16
};
const byte kInstruction_sxtb_al_r5_r2_ROR_24[] = {
  0x4f, 0xfa, 0xb2, 0xf5 // sxtb al r5 r2 ROR 24
};
const byte kInstruction_sxtb_al_r5_r3_ROR_0[] = {
  0x5d, 0xb2 // sxtb al r5 r3 ROR 0
};
const byte kInstruction_sxtb_al_r5_r3_ROR_8[] = {
  0x4f, 0xfa, 0x93, 0xf5 // sxtb al r5 r3 ROR 8
};
const byte kInstruction_sxtb_al_r5_r3_ROR_16[] = {
  0x4f, 0xfa, 0xa3, 0xf5 // sxtb al r5 r3 ROR 16
};
const byte kInstruction_sxtb_al_r5_r3_ROR_24[] = {
  0x4f, 0xfa, 0xb3, 0xf5 // sxtb al r5 r3 ROR 24
};
const byte kInstruction_sxtb_al_r5_r4_ROR_0[] = {
  0x65, 0xb2 // sxtb al r5 r4 ROR 0
};
const byte kInstruction_sxtb_al_r5_r4_ROR_8[] = {
  0x4f, 0xfa, 0x94, 0xf5 // sxtb al r5 r4 ROR 8
};
const byte kInstruction_sxtb_al_r5_r4_ROR_16[] = {
  0x4f, 0xfa, 0xa4, 0xf5 // sxtb al r5 r4 ROR 16
};
const byte kInstruction_sxtb_al_r5_r4_ROR_24[] = {
  0x4f, 0xfa, 0xb4, 0xf5 // sxtb al r5 r4 ROR 24
};
const byte kInstruction_sxtb_al_r5_r5_ROR_0[] = {
  0x6d, 0xb2 // sxtb al r5 r5 ROR 0
};
const byte kInstruction_sxtb_al_r5_r5_ROR_8[] = {
  0x4f, 0xfa, 0x95, 0xf5 // sxtb al r5 r5 ROR 8
};
const byte kInstruction_sxtb_al_r5_r5_ROR_16[] = {
  0x4f, 0xfa, 0xa5, 0xf5 // sxtb al r5 r5 ROR 16
};
const byte kInstruction_sxtb_al_r5_r5_ROR_24[] = {
  0x4f, 0xfa, 0xb5, 0xf5 // sxtb al r5 r5 ROR 24
};
const byte kInstruction_sxtb_al_r5_r6_ROR_0[] = {
  0x75, 0xb2 // sxtb al r5 r6 ROR 0
};
const byte kInstruction_sxtb_al_r5_r6_ROR_8[] = {
  0x4f, 0xfa, 0x96, 0xf5 // sxtb al r5 r6 ROR 8
};
const byte kInstruction_sxtb_al_r5_r6_ROR_16[] = {
  0x4f, 0xfa, 0xa6, 0xf5 // sxtb al r5 r6 ROR 16
};
const byte kInstruction_sxtb_al_r5_r6_ROR_24[] = {
  0x4f, 0xfa, 0xb6, 0xf5 // sxtb al r5 r6 ROR 24
};
const byte kInstruction_sxtb_al_r5_r7_ROR_0[] = {
  0x7d, 0xb2 // sxtb al r5 r7 ROR 0
};
const byte kInstruction_sxtb_al_r5_r7_ROR_8[] = {
  0x4f, 0xfa, 0x97, 0xf5 // sxtb al r5 r7 ROR 8
};
const byte kInstruction_sxtb_al_r5_r7_ROR_16[] = {
  0x4f, 0xfa, 0xa7, 0xf5 // sxtb al r5 r7 ROR 16
};
const byte kInstruction_sxtb_al_r5_r7_ROR_24[] = {
  0x4f, 0xfa, 0xb7, 0xf5 // sxtb al r5 r7 ROR 24
};
const byte kInstruction_sxtb_al_r5_r8_ROR_0[] = {
  0x4f, 0xfa, 0x88, 0xf5 // sxtb al r5 r8 ROR 0
};
const byte kInstruction_sxtb_al_r5_r8_ROR_8[] = {
  0x4f, 0xfa, 0x98, 0xf5 // sxtb al r5 r8 ROR 8
};
const byte kInstruction_sxtb_al_r5_r8_ROR_16[] = {
  0x4f, 0xfa, 0xa8, 0xf5 // sxtb al r5 r8 ROR 16
};
const byte kInstruction_sxtb_al_r5_r8_ROR_24[] = {
  0x4f, 0xfa, 0xb8, 0xf5 // sxtb al r5 r8 ROR 24
};
const byte kInstruction_sxtb_al_r5_r9_ROR_0[] = {
  0x4f, 0xfa, 0x89, 0xf5 // sxtb al r5 r9 ROR 0
};
const byte kInstruction_sxtb_al_r5_r9_ROR_8[] = {
  0x4f, 0xfa, 0x99, 0xf5 // sxtb al r5 r9 ROR 8
};
const byte kInstruction_sxtb_al_r5_r9_ROR_16[] = {
  0x4f, 0xfa, 0xa9, 0xf5 // sxtb al r5 r9 ROR 16
};
const byte kInstruction_sxtb_al_r5_r9_ROR_24[] = {
  0x4f, 0xfa, 0xb9, 0xf5 // sxtb al r5 r9 ROR 24
};
const byte kInstruction_sxtb_al_r5_r10_ROR_0[] = {
  0x4f, 0xfa, 0x8a, 0xf5 // sxtb al r5 r10 ROR 0
};
const byte kInstruction_sxtb_al_r5_r10_ROR_8[] = {
  0x4f, 0xfa, 0x9a, 0xf5 // sxtb al r5 r10 ROR 8
};
const byte kInstruction_sxtb_al_r5_r10_ROR_16[] = {
  0x4f, 0xfa, 0xaa, 0xf5 // sxtb al r5 r10 ROR 16
};
const byte kInstruction_sxtb_al_r5_r10_ROR_24[] = {
  0x4f, 0xfa, 0xba, 0xf5 // sxtb al r5 r10 ROR 24
};
const byte kInstruction_sxtb_al_r5_r11_ROR_0[] = {
  0x4f, 0xfa, 0x8b, 0xf5 // sxtb al r5 r11 ROR 0
};
const byte kInstruction_sxtb_al_r5_r11_ROR_8[] = {
  0x4f, 0xfa, 0x9b, 0xf5 // sxtb al r5 r11 ROR 8
};
const byte kInstruction_sxtb_al_r5_r11_ROR_16[] = {
  0x4f, 0xfa, 0xab, 0xf5 // sxtb al r5 r11 ROR 16
};
const byte kInstruction_sxtb_al_r5_r11_ROR_24[] = {
  0x4f, 0xfa, 0xbb, 0xf5 // sxtb al r5 r11 ROR 24
};
const byte kInstruction_sxtb_al_r5_r12_ROR_0[] = {
  0x4f, 0xfa, 0x8c, 0xf5 // sxtb al r5 r12 ROR 0
};
const byte kInstruction_sxtb_al_r5_r12_ROR_8[] = {
  0x4f, 0xfa, 0x9c, 0xf5 // sxtb al r5 r12 ROR 8
};
const byte kInstruction_sxtb_al_r5_r12_ROR_16[] = {
  0x4f, 0xfa, 0xac, 0xf5 // sxtb al r5 r12 ROR 16
};
const byte kInstruction_sxtb_al_r5_r12_ROR_24[] = {
  0x4f, 0xfa, 0xbc, 0xf5 // sxtb al r5 r12 ROR 24
};
const byte kInstruction_sxtb_al_r5_r13_ROR_0[] = {
  0x4f, 0xfa, 0x8d, 0xf5 // sxtb al r5 r13 ROR 0
};
const byte kInstruction_sxtb_al_r5_r13_ROR_8[] = {
  0x4f, 0xfa, 0x9d, 0xf5 // sxtb al r5 r13 ROR 8
};
const byte kInstruction_sxtb_al_r5_r13_ROR_16[] = {
  0x4f, 0xfa, 0xad, 0xf5 // sxtb al r5 r13 ROR 16
};
const byte kInstruction_sxtb_al_r5_r13_ROR_24[] = {
  0x4f, 0xfa, 0xbd, 0xf5 // sxtb al r5 r13 ROR 24
};
const byte kInstruction_sxtb_al_r5_r14_ROR_0[] = {
  0x4f, 0xfa, 0x8e, 0xf5 // sxtb al r5 r14 ROR 0
};
const byte kInstruction_sxtb_al_r5_r14_ROR_8[] = {
  0x4f, 0xfa, 0x9e, 0xf5 // sxtb al r5 r14 ROR 8
};
const byte kInstruction_sxtb_al_r5_r14_ROR_16[] = {
  0x4f, 0xfa, 0xae, 0xf5 // sxtb al r5 r14 ROR 16
};
const byte kInstruction_sxtb_al_r5_r14_ROR_24[] = {
  0x4f, 0xfa, 0xbe, 0xf5 // sxtb al r5 r14 ROR 24
};
const byte kInstruction_sxtb_al_r6_r0_ROR_0[] = {
  0x46, 0xb2 // sxtb al r6 r0 ROR 0
};
const byte kInstruction_sxtb_al_r6_r0_ROR_8[] = {
  0x4f, 0xfa, 0x90, 0xf6 // sxtb al r6 r0 ROR 8
};
const byte kInstruction_sxtb_al_r6_r0_ROR_16[] = {
  0x4f, 0xfa, 0xa0, 0xf6 // sxtb al r6 r0 ROR 16
};
const byte kInstruction_sxtb_al_r6_r0_ROR_24[] = {
  0x4f, 0xfa, 0xb0, 0xf6 // sxtb al r6 r0 ROR 24
};
const byte kInstruction_sxtb_al_r6_r1_ROR_0[] = {
  0x4e, 0xb2 // sxtb al r6 r1 ROR 0
};
const byte kInstruction_sxtb_al_r6_r1_ROR_8[] = {
  0x4f, 0xfa, 0x91, 0xf6 // sxtb al r6 r1 ROR 8
};
const byte kInstruction_sxtb_al_r6_r1_ROR_16[] = {
  0x4f, 0xfa, 0xa1, 0xf6 // sxtb al r6 r1 ROR 16
};
const byte kInstruction_sxtb_al_r6_r1_ROR_24[] = {
  0x4f, 0xfa, 0xb1, 0xf6 // sxtb al r6 r1 ROR 24
};
const byte kInstruction_sxtb_al_r6_r2_ROR_0[] = {
  0x56, 0xb2 // sxtb al r6 r2 ROR 0
};
const byte kInstruction_sxtb_al_r6_r2_ROR_8[] = {
  0x4f, 0xfa, 0x92, 0xf6 // sxtb al r6 r2 ROR 8
};
const byte kInstruction_sxtb_al_r6_r2_ROR_16[] = {
  0x4f, 0xfa, 0xa2, 0xf6 // sxtb al r6 r2 ROR 16
};
const byte kInstruction_sxtb_al_r6_r2_ROR_24[] = {
  0x4f, 0xfa, 0xb2, 0xf6 // sxtb al r6 r2 ROR 24
};
const byte kInstruction_sxtb_al_r6_r3_ROR_0[] = {
  0x5e, 0xb2 // sxtb al r6 r3 ROR 0
};
const byte kInstruction_sxtb_al_r6_r3_ROR_8[] = {
  0x4f, 0xfa, 0x93, 0xf6 // sxtb al r6 r3 ROR 8
};
const byte kInstruction_sxtb_al_r6_r3_ROR_16[] = {
  0x4f, 0xfa, 0xa3, 0xf6 // sxtb al r6 r3 ROR 16
};
const byte kInstruction_sxtb_al_r6_r3_ROR_24[] = {
  0x4f, 0xfa, 0xb3, 0xf6 // sxtb al r6 r3 ROR 24
};
const byte kInstruction_sxtb_al_r6_r4_ROR_0[] = {
  0x66, 0xb2 // sxtb al r6 r4 ROR 0
};
const byte kInstruction_sxtb_al_r6_r4_ROR_8[] = {
  0x4f, 0xfa, 0x94, 0xf6 // sxtb al r6 r4 ROR 8
};
const byte kInstruction_sxtb_al_r6_r4_ROR_16[] = {
  0x4f, 0xfa, 0xa4, 0xf6 // sxtb al r6 r4 ROR 16
};
const byte kInstruction_sxtb_al_r6_r4_ROR_24[] = {
  0x4f, 0xfa, 0xb4, 0xf6 // sxtb al r6 r4 ROR 24
};
const byte kInstruction_sxtb_al_r6_r5_ROR_0[] = {
  0x6e, 0xb2 // sxtb al r6 r5 ROR 0
};
const byte kInstruction_sxtb_al_r6_r5_ROR_8[] = {
  0x4f, 0xfa, 0x95, 0xf6 // sxtb al r6 r5 ROR 8
};
const byte kInstruction_sxtb_al_r6_r5_ROR_16[] = {
  0x4f, 0xfa, 0xa5, 0xf6 // sxtb al r6 r5 ROR 16
};
const byte kInstruction_sxtb_al_r6_r5_ROR_24[] = {
  0x4f, 0xfa, 0xb5, 0xf6 // sxtb al r6 r5 ROR 24
};
const byte kInstruction_sxtb_al_r6_r6_ROR_0[] = {
  0x76, 0xb2 // sxtb al r6 r6 ROR 0
};
const byte kInstruction_sxtb_al_r6_r6_ROR_8[] = {
  0x4f, 0xfa, 0x96, 0xf6 // sxtb al r6 r6 ROR 8
};
const byte kInstruction_sxtb_al_r6_r6_ROR_16[] = {
  0x4f, 0xfa, 0xa6, 0xf6 // sxtb al r6 r6 ROR 16
};
const byte kInstruction_sxtb_al_r6_r6_ROR_24[] = {
  0x4f, 0xfa, 0xb6, 0xf6 // sxtb al r6 r6 ROR 24
};
const byte kInstruction_sxtb_al_r6_r7_ROR_0[] = {
  0x7e, 0xb2 // sxtb al r6 r7 ROR 0
};
const byte kInstruction_sxtb_al_r6_r7_ROR_8[] = {
  0x4f, 0xfa, 0x97, 0xf6 // sxtb al r6 r7 ROR 8
};
const byte kInstruction_sxtb_al_r6_r7_ROR_16[] = {
  0x4f, 0xfa, 0xa7, 0xf6 // sxtb al r6 r7 ROR 16
};
const byte kInstruction_sxtb_al_r6_r7_ROR_24[] = {
  0x4f, 0xfa, 0xb7, 0xf6 // sxtb al r6 r7 ROR 24
};
const byte kInstruction_sxtb_al_r6_r8_ROR_0[] = {
  0x4f, 0xfa, 0x88, 0xf6 // sxtb al r6 r8 ROR 0
};
const byte kInstruction_sxtb_al_r6_r8_ROR_8[] = {
  0x4f, 0xfa, 0x98, 0xf6 // sxtb al r6 r8 ROR 8
};
const byte kInstruction_sxtb_al_r6_r8_ROR_16[] = {
  0x4f, 0xfa, 0xa8, 0xf6 // sxtb al r6 r8 ROR 16
};
const byte kInstruction_sxtb_al_r6_r8_ROR_24[] = {
  0x4f, 0xfa, 0xb8, 0xf6 // sxtb al r6 r8 ROR 24
};
const byte kInstruction_sxtb_al_r6_r9_ROR_0[] = {
  0x4f, 0xfa, 0x89, 0xf6 // sxtb al r6 r9 ROR 0
};
const byte kInstruction_sxtb_al_r6_r9_ROR_8[] = {
  0x4f, 0xfa, 0x99, 0xf6 // sxtb al r6 r9 ROR 8
};
const byte kInstruction_sxtb_al_r6_r9_ROR_16[] = {
  0x4f, 0xfa, 0xa9, 0xf6 // sxtb al r6 r9 ROR 16
};
const byte kInstruction_sxtb_al_r6_r9_ROR_24[] = {
  0x4f, 0xfa, 0xb9, 0xf6 // sxtb al r6 r9 ROR 24
};
const byte kInstruction_sxtb_al_r6_r10_ROR_0[] = {
  0x4f, 0xfa, 0x8a, 0xf6 // sxtb al r6 r10 ROR 0
};
const byte kInstruction_sxtb_al_r6_r10_ROR_8[] = {
  0x4f, 0xfa, 0x9a, 0xf6 // sxtb al r6 r10 ROR 8
};
const byte kInstruction_sxtb_al_r6_r10_ROR_16[] = {
  0x4f, 0xfa, 0xaa, 0xf6 // sxtb al r6 r10 ROR 16
};
const byte kInstruction_sxtb_al_r6_r10_ROR_24[] = {
  0x4f, 0xfa, 0xba, 0xf6 // sxtb al r6 r10 ROR 24
};
const byte kInstruction_sxtb_al_r6_r11_ROR_0[] = {
  0x4f, 0xfa, 0x8b, 0xf6 // sxtb al r6 r11 ROR 0
};
const byte kInstruction_sxtb_al_r6_r11_ROR_8[] = {
  0x4f, 0xfa, 0x9b, 0xf6 // sxtb al r6 r11 ROR 8
};
const byte kInstruction_sxtb_al_r6_r11_ROR_16[] = {
  0x4f, 0xfa, 0xab, 0xf6 // sxtb al r6 r11 ROR 16
};
const byte kInstruction_sxtb_al_r6_r11_ROR_24[] = {
  0x4f, 0xfa, 0xbb, 0xf6 // sxtb al r6 r11 ROR 24
};
const byte kInstruction_sxtb_al_r6_r12_ROR_0[] = {
  0x4f, 0xfa, 0x8c, 0xf6 // sxtb al r6 r12 ROR 0
};
const byte kInstruction_sxtb_al_r6_r12_ROR_8[] = {
  0x4f, 0xfa, 0x9c, 0xf6 // sxtb al r6 r12 ROR 8
};
const byte kInstruction_sxtb_al_r6_r12_ROR_16[] = {
  0x4f, 0xfa, 0xac, 0xf6 // sxtb al r6 r12 ROR 16
};
const byte kInstruction_sxtb_al_r6_r12_ROR_24[] = {
  0x4f, 0xfa, 0xbc, 0xf6 // sxtb al r6 r12 ROR 24
};
const byte kInstruction_sxtb_al_r6_r13_ROR_0[] = {
  0x4f, 0xfa, 0x8d, 0xf6 // sxtb al r6 r13 ROR 0
};
const byte kInstruction_sxtb_al_r6_r13_ROR_8[] = {
  0x4f, 0xfa, 0x9d, 0xf6 // sxtb al r6 r13 ROR 8
};
const byte kInstruction_sxtb_al_r6_r13_ROR_16[] = {
  0x4f, 0xfa, 0xad, 0xf6 // sxtb al r6 r13 ROR 16
};
const byte kInstruction_sxtb_al_r6_r13_ROR_24[] = {
  0x4f, 0xfa, 0xbd, 0xf6 // sxtb al r6 r13 ROR 24
};
const byte kInstruction_sxtb_al_r6_r14_ROR_0[] = {
  0x4f, 0xfa, 0x8e, 0xf6 // sxtb al r6 r14 ROR 0
};
const byte kInstruction_sxtb_al_r6_r14_ROR_8[] = {
  0x4f, 0xfa, 0x9e, 0xf6 // sxtb al r6 r14 ROR 8
};
const byte kInstruction_sxtb_al_r6_r14_ROR_16[] = {
  0x4f, 0xfa, 0xae, 0xf6 // sxtb al r6 r14 ROR 16
};
const byte kInstruction_sxtb_al_r6_r14_ROR_24[] = {
  0x4f, 0xfa, 0xbe, 0xf6 // sxtb al r6 r14 ROR 24
};
const byte kInstruction_sxtb_al_r7_r0_ROR_0[] = {
  0x47, 0xb2 // sxtb al r7 r0 ROR 0
};
const byte kInstruction_sxtb_al_r7_r0_ROR_8[] = {
  0x4f, 0xfa, 0x90, 0xf7 // sxtb al r7 r0 ROR 8
};
const byte kInstruction_sxtb_al_r7_r0_ROR_16[] = {
  0x4f, 0xfa, 0xa0, 0xf7 // sxtb al r7 r0 ROR 16
};
const byte kInstruction_sxtb_al_r7_r0_ROR_24[] = {
  0x4f, 0xfa, 0xb0, 0xf7 // sxtb al r7 r0 ROR 24
};
const byte kInstruction_sxtb_al_r7_r1_ROR_0[] = {
  0x4f, 0xb2 // sxtb al r7 r1 ROR 0
};
const byte kInstruction_sxtb_al_r7_r1_ROR_8[] = {
  0x4f, 0xfa, 0x91, 0xf7 // sxtb al r7 r1 ROR 8
};
const byte kInstruction_sxtb_al_r7_r1_ROR_16[] = {
  0x4f, 0xfa, 0xa1, 0xf7 // sxtb al r7 r1 ROR 16
};
const byte kInstruction_sxtb_al_r7_r1_ROR_24[] = {
  0x4f, 0xfa, 0xb1, 0xf7 // sxtb al r7 r1 ROR 24
};
const byte kInstruction_sxtb_al_r7_r2_ROR_0[] = {
  0x57, 0xb2 // sxtb al r7 r2 ROR 0
};
const byte kInstruction_sxtb_al_r7_r2_ROR_8[] = {
  0x4f, 0xfa, 0x92, 0xf7 // sxtb al r7 r2 ROR 8
};
const byte kInstruction_sxtb_al_r7_r2_ROR_16[] = {
  0x4f, 0xfa, 0xa2, 0xf7 // sxtb al r7 r2 ROR 16
};
const byte kInstruction_sxtb_al_r7_r2_ROR_24[] = {
  0x4f, 0xfa, 0xb2, 0xf7 // sxtb al r7 r2 ROR 24
};
const byte kInstruction_sxtb_al_r7_r3_ROR_0[] = {
  0x5f, 0xb2 // sxtb al r7 r3 ROR 0
};
const byte kInstruction_sxtb_al_r7_r3_ROR_8[] = {
  0x4f, 0xfa, 0x93, 0xf7 // sxtb al r7 r3 ROR 8
};
const byte kInstruction_sxtb_al_r7_r3_ROR_16[] = {
  0x4f, 0xfa, 0xa3, 0xf7 // sxtb al r7 r3 ROR 16
};
const byte kInstruction_sxtb_al_r7_r3_ROR_24[] = {
  0x4f, 0xfa, 0xb3, 0xf7 // sxtb al r7 r3 ROR 24
};
const byte kInstruction_sxtb_al_r7_r4_ROR_0[] = {
  0x67, 0xb2 // sxtb al r7 r4 ROR 0
};
const byte kInstruction_sxtb_al_r7_r4_ROR_8[] = {
  0x4f, 0xfa, 0x94, 0xf7 // sxtb al r7 r4 ROR 8
};
const byte kInstruction_sxtb_al_r7_r4_ROR_16[] = {
  0x4f, 0xfa, 0xa4, 0xf7 // sxtb al r7 r4 ROR 16
};
const byte kInstruction_sxtb_al_r7_r4_ROR_24[] = {
  0x4f, 0xfa, 0xb4, 0xf7 // sxtb al r7 r4 ROR 24
};
const byte kInstruction_sxtb_al_r7_r5_ROR_0[] = {
  0x6f, 0xb2 // sxtb al r7 r5 ROR 0
};
const byte kInstruction_sxtb_al_r7_r5_ROR_8[] = {
  0x4f, 0xfa, 0x95, 0xf7 // sxtb al r7 r5 ROR 8
};
const byte kInstruction_sxtb_al_r7_r5_ROR_16[] = {
  0x4f, 0xfa, 0xa5, 0xf7 // sxtb al r7 r5 ROR 16
};
const byte kInstruction_sxtb_al_r7_r5_ROR_24[] = {
  0x4f, 0xfa, 0xb5, 0xf7 // sxtb al r7 r5 ROR 24
};
const byte kInstruction_sxtb_al_r7_r6_ROR_0[] = {
  0x77, 0xb2 // sxtb al r7 r6 ROR 0
};
const byte kInstruction_sxtb_al_r7_r6_ROR_8[] = {
  0x4f, 0xfa, 0x96, 0xf7 // sxtb al r7 r6 ROR 8
};
const byte kInstruction_sxtb_al_r7_r6_ROR_16[] = {
  0x4f, 0xfa, 0xa6, 0xf7 // sxtb al r7 r6 ROR 16
};
const byte kInstruction_sxtb_al_r7_r6_ROR_24[] = {
  0x4f, 0xfa, 0xb6, 0xf7 // sxtb al r7 r6 ROR 24
};
const byte kInstruction_sxtb_al_r7_r7_ROR_0[] = {
  0x7f, 0xb2 // sxtb al r7 r7 ROR 0
};
const byte kInstruction_sxtb_al_r7_r7_ROR_8[] = {
  0x4f, 0xfa, 0x97, 0xf7 // sxtb al r7 r7 ROR 8
};
const byte kInstruction_sxtb_al_r7_r7_ROR_16[] = {
  0x4f, 0xfa, 0xa7, 0xf7 // sxtb al r7 r7 ROR 16
};
const byte kInstruction_sxtb_al_r7_r7_ROR_24[] = {
  0x4f, 0xfa, 0xb7, 0xf7 // sxtb al r7 r7 ROR 24
};
const byte kInstruction_sxtb_al_r7_r8_ROR_0[] = {
  0x4f, 0xfa, 0x88, 0xf7 // sxtb al r7 r8 ROR 0
};
const byte kInstruction_sxtb_al_r7_r8_ROR_8[] = {
  0x4f, 0xfa, 0x98, 0xf7 // sxtb al r7 r8 ROR 8
};
const byte kInstruction_sxtb_al_r7_r8_ROR_16[] = {
  0x4f, 0xfa, 0xa8, 0xf7 // sxtb al r7 r8 ROR 16
};
const byte kInstruction_sxtb_al_r7_r8_ROR_24[] = {
  0x4f, 0xfa, 0xb8, 0xf7 // sxtb al r7 r8 ROR 24
};
const byte kInstruction_sxtb_al_r7_r9_ROR_0[] = {
  0x4f, 0xfa, 0x89, 0xf7 // sxtb al r7 r9 ROR 0
};
const byte kInstruction_sxtb_al_r7_r9_ROR_8[] = {
  0x4f, 0xfa, 0x99, 0xf7 // sxtb al r7 r9 ROR 8
};
const byte kInstruction_sxtb_al_r7_r9_ROR_16[] = {
  0x4f, 0xfa, 0xa9, 0xf7 // sxtb al r7 r9 ROR 16
};
const byte kInstruction_sxtb_al_r7_r9_ROR_24[] = {
  0x4f, 0xfa, 0xb9, 0xf7 // sxtb al r7 r9 ROR 24
};
const byte kInstruction_sxtb_al_r7_r10_ROR_0[] = {
  0x4f, 0xfa, 0x8a, 0xf7 // sxtb al r7 r10 ROR 0
};
const byte kInstruction_sxtb_al_r7_r10_ROR_8[] = {
  0x4f, 0xfa, 0x9a, 0xf7 // sxtb al r7 r10 ROR 8
};
const byte kInstruction_sxtb_al_r7_r10_ROR_16[] = {
  0x4f, 0xfa, 0xaa, 0xf7 // sxtb al r7 r10 ROR 16
};
const byte kInstruction_sxtb_al_r7_r10_ROR_24[] = {
  0x4f, 0xfa, 0xba, 0xf7 // sxtb al r7 r10 ROR 24
};
const byte kInstruction_sxtb_al_r7_r11_ROR_0[] = {
  0x4f, 0xfa, 0x8b, 0xf7 // sxtb al r7 r11 ROR 0
};
const byte kInstruction_sxtb_al_r7_r11_ROR_8[] = {
  0x4f, 0xfa, 0x9b, 0xf7 // sxtb al r7 r11 ROR 8
};
const byte kInstruction_sxtb_al_r7_r11_ROR_16[] = {
  0x4f, 0xfa, 0xab, 0xf7 // sxtb al r7 r11 ROR 16
};
const byte kInstruction_sxtb_al_r7_r11_ROR_24[] = {
  0x4f, 0xfa, 0xbb, 0xf7 // sxtb al r7 r11 ROR 24
};
const byte kInstruction_sxtb_al_r7_r12_ROR_0[] = {
  0x4f, 0xfa, 0x8c, 0xf7 // sxtb al r7 r12 ROR 0
};
const byte kInstruction_sxtb_al_r7_r12_ROR_8[] = {
  0x4f, 0xfa, 0x9c, 0xf7 // sxtb al r7 r12 ROR 8
};
const byte kInstruction_sxtb_al_r7_r12_ROR_16[] = {
  0x4f, 0xfa, 0xac, 0xf7 // sxtb al r7 r12 ROR 16
};
const byte kInstruction_sxtb_al_r7_r12_ROR_24[] = {
  0x4f, 0xfa, 0xbc, 0xf7 // sxtb al r7 r12 ROR 24
};
const byte kInstruction_sxtb_al_r7_r13_ROR_0[] = {
  0x4f, 0xfa, 0x8d, 0xf7 // sxtb al r7 r13 ROR 0
};
const byte kInstruction_sxtb_al_r7_r13_ROR_8[] = {
  0x4f, 0xfa, 0x9d, 0xf7 // sxtb al r7 r13 ROR 8
};
const byte kInstruction_sxtb_al_r7_r13_ROR_16[] = {
  0x4f, 0xfa, 0xad, 0xf7 // sxtb al r7 r13 ROR 16
};
const byte kInstruction_sxtb_al_r7_r13_ROR_24[] = {
  0x4f, 0xfa, 0xbd, 0xf7 // sxtb al r7 r13 ROR 24
};
const byte kInstruction_sxtb_al_r7_r14_ROR_0[] = {
  0x4f, 0xfa, 0x8e, 0xf7 // sxtb al r7 r14 ROR 0
};
const byte kInstruction_sxtb_al_r7_r14_ROR_8[] = {
  0x4f, 0xfa, 0x9e, 0xf7 // sxtb al r7 r14 ROR 8
};
const byte kInstruction_sxtb_al_r7_r14_ROR_16[] = {
  0x4f, 0xfa, 0xae, 0xf7 // sxtb al r7 r14 ROR 16
};
const byte kInstruction_sxtb_al_r7_r14_ROR_24[] = {
  0x4f, 0xfa, 0xbe, 0xf7 // sxtb al r7 r14 ROR 24
};
const byte kInstruction_sxtb_al_r8_r0_ROR_0[] = {
  0x4f, 0xfa, 0x80, 0xf8 // sxtb al r8 r0 ROR 0
};
const byte kInstruction_sxtb_al_r8_r0_ROR_8[] = {
  0x4f, 0xfa, 0x90, 0xf8 // sxtb al r8 r0 ROR 8
};
const byte kInstruction_sxtb_al_r8_r0_ROR_16[] = {
  0x4f, 0xfa, 0xa0, 0xf8 // sxtb al r8 r0 ROR 16
};
const byte kInstruction_sxtb_al_r8_r0_ROR_24[] = {
  0x4f, 0xfa, 0xb0, 0xf8 // sxtb al r8 r0 ROR 24
};
const byte kInstruction_sxtb_al_r8_r1_ROR_0[] = {
  0x4f, 0xfa, 0x81, 0xf8 // sxtb al r8 r1 ROR 0
};
const byte kInstruction_sxtb_al_r8_r1_ROR_8[] = {
  0x4f, 0xfa, 0x91, 0xf8 // sxtb al r8 r1 ROR 8
};
const byte kInstruction_sxtb_al_r8_r1_ROR_16[] = {
  0x4f, 0xfa, 0xa1, 0xf8 // sxtb al r8 r1 ROR 16
};
const byte kInstruction_sxtb_al_r8_r1_ROR_24[] = {
  0x4f, 0xfa, 0xb1, 0xf8 // sxtb al r8 r1 ROR 24
};
const byte kInstruction_sxtb_al_r8_r2_ROR_0[] = {
  0x4f, 0xfa, 0x82, 0xf8 // sxtb al r8 r2 ROR 0
};
const byte kInstruction_sxtb_al_r8_r2_ROR_8[] = {
  0x4f, 0xfa, 0x92, 0xf8 // sxtb al r8 r2 ROR 8
};
const byte kInstruction_sxtb_al_r8_r2_ROR_16[] = {
  0x4f, 0xfa, 0xa2, 0xf8 // sxtb al r8 r2 ROR 16
};
const byte kInstruction_sxtb_al_r8_r2_ROR_24[] = {
  0x4f, 0xfa, 0xb2, 0xf8 // sxtb al r8 r2 ROR 24
};
const byte kInstruction_sxtb_al_r8_r3_ROR_0[] = {
  0x4f, 0xfa, 0x83, 0xf8 // sxtb al r8 r3 ROR 0
};
const byte kInstruction_sxtb_al_r8_r3_ROR_8[] = {
  0x4f, 0xfa, 0x93, 0xf8 // sxtb al r8 r3 ROR 8
};
const byte kInstruction_sxtb_al_r8_r3_ROR_16[] = {
  0x4f, 0xfa, 0xa3, 0xf8 // sxtb al r8 r3 ROR 16
};
const byte kInstruction_sxtb_al_r8_r3_ROR_24[] = {
  0x4f, 0xfa, 0xb3, 0xf8 // sxtb al r8 r3 ROR 24
};
const byte kInstruction_sxtb_al_r8_r4_ROR_0[] = {
  0x4f, 0xfa, 0x84, 0xf8 // sxtb al r8 r4 ROR 0
};
const byte kInstruction_sxtb_al_r8_r4_ROR_8[] = {
  0x4f, 0xfa, 0x94, 0xf8 // sxtb al r8 r4 ROR 8
};
const byte kInstruction_sxtb_al_r8_r4_ROR_16[] = {
  0x4f, 0xfa, 0xa4, 0xf8 // sxtb al r8 r4 ROR 16
};
const byte kInstruction_sxtb_al_r8_r4_ROR_24[] = {
  0x4f, 0xfa, 0xb4, 0xf8 // sxtb al r8 r4 ROR 24
};
const byte kInstruction_sxtb_al_r8_r5_ROR_0[] = {
  0x4f, 0xfa, 0x85, 0xf8 // sxtb al r8 r5 ROR 0
};
const byte kInstruction_sxtb_al_r8_r5_ROR_8[] = {
  0x4f, 0xfa, 0x95, 0xf8 // sxtb al r8 r5 ROR 8
};
const byte kInstruction_sxtb_al_r8_r5_ROR_16[] = {
  0x4f, 0xfa, 0xa5, 0xf8 // sxtb al r8 r5 ROR 16
};
const byte kInstruction_sxtb_al_r8_r5_ROR_24[] = {
  0x4f, 0xfa, 0xb5, 0xf8 // sxtb al r8 r5 ROR 24
};
const byte kInstruction_sxtb_al_r8_r6_ROR_0[] = {
  0x4f, 0xfa, 0x86, 0xf8 // sxtb al r8 r6 ROR 0
};
const byte kInstruction_sxtb_al_r8_r6_ROR_8[] = {
  0x4f, 0xfa, 0x96, 0xf8 // sxtb al r8 r6 ROR 8
};
const byte kInstruction_sxtb_al_r8_r6_ROR_16[] = {
  0x4f, 0xfa, 0xa6, 0xf8 // sxtb al r8 r6 ROR 16
};
const byte kInstruction_sxtb_al_r8_r6_ROR_24[] = {
  0x4f, 0xfa, 0xb6, 0xf8 // sxtb al r8 r6 ROR 24
};
const byte kInstruction_sxtb_al_r8_r7_ROR_0[] = {
  0x4f, 0xfa, 0x87, 0xf8 // sxtb al r8 r7 ROR 0
};
const byte kInstruction_sxtb_al_r8_r7_ROR_8[] = {
  0x4f, 0xfa, 0x97, 0xf8 // sxtb al r8 r7 ROR 8
};
const byte kInstruction_sxtb_al_r8_r7_ROR_16[] = {
  0x4f, 0xfa, 0xa7, 0xf8 // sxtb al r8 r7 ROR 16
};
const byte kInstruction_sxtb_al_r8_r7_ROR_24[] = {
  0x4f, 0xfa, 0xb7, 0xf8 // sxtb al r8 r7 ROR 24
};
const byte kInstruction_sxtb_al_r8_r8_ROR_0[] = {
  0x4f, 0xfa, 0x88, 0xf8 // sxtb al r8 r8 ROR 0
};
const byte kInstruction_sxtb_al_r8_r8_ROR_8[] = {
  0x4f, 0xfa, 0x98, 0xf8 // sxtb al r8 r8 ROR 8
};
const byte kInstruction_sxtb_al_r8_r8_ROR_16[] = {
  0x4f, 0xfa, 0xa8, 0xf8 // sxtb al r8 r8 ROR 16
};
const byte kInstruction_sxtb_al_r8_r8_ROR_24[] = {
  0x4f, 0xfa, 0xb8, 0xf8 // sxtb al r8 r8 ROR 24
};
const byte kInstruction_sxtb_al_r8_r9_ROR_0[] = {
  0x4f, 0xfa, 0x89, 0xf8 // sxtb al r8 r9 ROR 0
};
const byte kInstruction_sxtb_al_r8_r9_ROR_8[] = {
  0x4f, 0xfa, 0x99, 0xf8 // sxtb al r8 r9 ROR 8
};
const byte kInstruction_sxtb_al_r8_r9_ROR_16[] = {
  0x4f, 0xfa, 0xa9, 0xf8 // sxtb al r8 r9 ROR 16
};
const byte kInstruction_sxtb_al_r8_r9_ROR_24[] = {
  0x4f, 0xfa, 0xb9, 0xf8 // sxtb al r8 r9 ROR 24
};
const byte kInstruction_sxtb_al_r8_r10_ROR_0[] = {
  0x4f, 0xfa, 0x8a, 0xf8 // sxtb al r8 r10 ROR 0
};
const byte kInstruction_sxtb_al_r8_r10_ROR_8[] = {
  0x4f, 0xfa, 0x9a, 0xf8 // sxtb al r8 r10 ROR 8
};
const byte kInstruction_sxtb_al_r8_r10_ROR_16[] = {
  0x4f, 0xfa, 0xaa, 0xf8 // sxtb al r8 r10 ROR 16
};
const byte kInstruction_sxtb_al_r8_r10_ROR_24[] = {
  0x4f, 0xfa, 0xba, 0xf8 // sxtb al r8 r10 ROR 24
};
const byte kInstruction_sxtb_al_r8_r11_ROR_0[] = {
  0x4f, 0xfa, 0x8b, 0xf8 // sxtb al r8 r11 ROR 0
};
const byte kInstruction_sxtb_al_r8_r11_ROR_8[] = {
  0x4f, 0xfa, 0x9b, 0xf8 // sxtb al r8 r11 ROR 8
};
const byte kInstruction_sxtb_al_r8_r11_ROR_16[] = {
  0x4f, 0xfa, 0xab, 0xf8 // sxtb al r8 r11 ROR 16
};
const byte kInstruction_sxtb_al_r8_r11_ROR_24[] = {
  0x4f, 0xfa, 0xbb, 0xf8 // sxtb al r8 r11 ROR 24
};
const byte kInstruction_sxtb_al_r8_r12_ROR_0[] = {
  0x4f, 0xfa, 0x8c, 0xf8 // sxtb al r8 r12 ROR 0
};
const byte kInstruction_sxtb_al_r8_r12_ROR_8[] = {
  0x4f, 0xfa, 0x9c, 0xf8 // sxtb al r8 r12 ROR 8
};
const byte kInstruction_sxtb_al_r8_r12_ROR_16[] = {
  0x4f, 0xfa, 0xac, 0xf8 // sxtb al r8 r12 ROR 16
};
const byte kInstruction_sxtb_al_r8_r12_ROR_24[] = {
  0x4f, 0xfa, 0xbc, 0xf8 // sxtb al r8 r12 ROR 24
};
const byte kInstruction_sxtb_al_r8_r13_ROR_0[] = {
  0x4f, 0xfa, 0x8d, 0xf8 // sxtb al r8 r13 ROR 0
};
const byte kInstruction_sxtb_al_r8_r13_ROR_8[] = {
  0x4f, 0xfa, 0x9d, 0xf8 // sxtb al r8 r13 ROR 8
};
const byte kInstruction_sxtb_al_r8_r13_ROR_16[] = {
  0x4f, 0xfa, 0xad, 0xf8 // sxtb al r8 r13 ROR 16
};
const byte kInstruction_sxtb_al_r8_r13_ROR_24[] = {
  0x4f, 0xfa, 0xbd, 0xf8 // sxtb al r8 r13 ROR 24
};
const byte kInstruction_sxtb_al_r8_r14_ROR_0[] = {
  0x4f, 0xfa, 0x8e, 0xf8 // sxtb al r8 r14 ROR 0
};
const byte kInstruction_sxtb_al_r8_r14_ROR_8[] = {
  0x4f, 0xfa, 0x9e, 0xf8 // sxtb al r8 r14 ROR 8
};
const byte kInstruction_sxtb_al_r8_r14_ROR_16[] = {
  0x4f, 0xfa, 0xae, 0xf8 // sxtb al r8 r14 ROR 16
};
const byte kInstruction_sxtb_al_r8_r14_ROR_24[] = {
  0x4f, 0xfa, 0xbe, 0xf8 // sxtb al r8 r14 ROR 24
};
const byte kInstruction_sxtb_al_r9_r0_ROR_0[] = {
  0x4f, 0xfa, 0x80, 0xf9 // sxtb al r9 r0 ROR 0
};
const byte kInstruction_sxtb_al_r9_r0_ROR_8[] = {
  0x4f, 0xfa, 0x90, 0xf9 // sxtb al r9 r0 ROR 8
};
const byte kInstruction_sxtb_al_r9_r0_ROR_16[] = {
  0x4f, 0xfa, 0xa0, 0xf9 // sxtb al r9 r0 ROR 16
};
const byte kInstruction_sxtb_al_r9_r0_ROR_24[] = {
  0x4f, 0xfa, 0xb0, 0xf9 // sxtb al r9 r0 ROR 24
};
const byte kInstruction_sxtb_al_r9_r1_ROR_0[] = {
  0x4f, 0xfa, 0x81, 0xf9 // sxtb al r9 r1 ROR 0
};
const byte kInstruction_sxtb_al_r9_r1_ROR_8[] = {
  0x4f, 0xfa, 0x91, 0xf9 // sxtb al r9 r1 ROR 8
};
const byte kInstruction_sxtb_al_r9_r1_ROR_16[] = {
  0x4f, 0xfa, 0xa1, 0xf9 // sxtb al r9 r1 ROR 16
};
const byte kInstruction_sxtb_al_r9_r1_ROR_24[] = {
  0x4f, 0xfa, 0xb1, 0xf9 // sxtb al r9 r1 ROR 24
};
const byte kInstruction_sxtb_al_r9_r2_ROR_0[] = {
  0x4f, 0xfa, 0x82, 0xf9 // sxtb al r9 r2 ROR 0
};
const byte kInstruction_sxtb_al_r9_r2_ROR_8[] = {
  0x4f, 0xfa, 0x92, 0xf9 // sxtb al r9 r2 ROR 8
};
const byte kInstruction_sxtb_al_r9_r2_ROR_16[] = {
  0x4f, 0xfa, 0xa2, 0xf9 // sxtb al r9 r2 ROR 16
};
const byte kInstruction_sxtb_al_r9_r2_ROR_24[] = {
  0x4f, 0xfa, 0xb2, 0xf9 // sxtb al r9 r2 ROR 24
};
const byte kInstruction_sxtb_al_r9_r3_ROR_0[] = {
  0x4f, 0xfa, 0x83, 0xf9 // sxtb al r9 r3 ROR 0
};
const byte kInstruction_sxtb_al_r9_r3_ROR_8[] = {
  0x4f, 0xfa, 0x93, 0xf9 // sxtb al r9 r3 ROR 8
};
const byte kInstruction_sxtb_al_r9_r3_ROR_16[] = {
  0x4f, 0xfa, 0xa3, 0xf9 // sxtb al r9 r3 ROR 16
};
const byte kInstruction_sxtb_al_r9_r3_ROR_24[] = {
  0x4f, 0xfa, 0xb3, 0xf9 // sxtb al r9 r3 ROR 24
};
const byte kInstruction_sxtb_al_r9_r4_ROR_0[] = {
  0x4f, 0xfa, 0x84, 0xf9 // sxtb al r9 r4 ROR 0
};
const byte kInstruction_sxtb_al_r9_r4_ROR_8[] = {
  0x4f, 0xfa, 0x94, 0xf9 // sxtb al r9 r4 ROR 8
};
const byte kInstruction_sxtb_al_r9_r4_ROR_16[] = {
  0x4f, 0xfa, 0xa4, 0xf9 // sxtb al r9 r4 ROR 16
};
const byte kInstruction_sxtb_al_r9_r4_ROR_24[] = {
  0x4f, 0xfa, 0xb4, 0xf9 // sxtb al r9 r4 ROR 24
};
const byte kInstruction_sxtb_al_r9_r5_ROR_0[] = {
  0x4f, 0xfa, 0x85, 0xf9 // sxtb al r9 r5 ROR 0
};
const byte kInstruction_sxtb_al_r9_r5_ROR_8[] = {
  0x4f, 0xfa, 0x95, 0xf9 // sxtb al r9 r5 ROR 8
};
const byte kInstruction_sxtb_al_r9_r5_ROR_16[] = {
  0x4f, 0xfa, 0xa5, 0xf9 // sxtb al r9 r5 ROR 16
};
const byte kInstruction_sxtb_al_r9_r5_ROR_24[] = {
  0x4f, 0xfa, 0xb5, 0xf9 // sxtb al r9 r5 ROR 24
};
const byte kInstruction_sxtb_al_r9_r6_ROR_0[] = {
  0x4f, 0xfa, 0x86, 0xf9 // sxtb al r9 r6 ROR 0
};
const byte kInstruction_sxtb_al_r9_r6_ROR_8[] = {
  0x4f, 0xfa, 0x96, 0xf9 // sxtb al r9 r6 ROR 8
};
const byte kInstruction_sxtb_al_r9_r6_ROR_16[] = {
  0x4f, 0xfa, 0xa6, 0xf9 // sxtb al r9 r6 ROR 16
};
const byte kInstruction_sxtb_al_r9_r6_ROR_24[] = {
  0x4f, 0xfa, 0xb6, 0xf9 // sxtb al r9 r6 ROR 24
};
const byte kInstruction_sxtb_al_r9_r7_ROR_0[] = {
  0x4f, 0xfa, 0x87, 0xf9 // sxtb al r9 r7 ROR 0
};
const byte kInstruction_sxtb_al_r9_r7_ROR_8[] = {
  0x4f, 0xfa, 0x97, 0xf9 // sxtb al r9 r7 ROR 8
};
const byte kInstruction_sxtb_al_r9_r7_ROR_16[] = {
  0x4f, 0xfa, 0xa7, 0xf9 // sxtb al r9 r7 ROR 16
};
const byte kInstruction_sxtb_al_r9_r7_ROR_24[] = {
  0x4f, 0xfa, 0xb7, 0xf9 // sxtb al r9 r7 ROR 24
};
const byte kInstruction_sxtb_al_r9_r8_ROR_0[] = {
  0x4f, 0xfa, 0x88, 0xf9 // sxtb al r9 r8 ROR 0
};
const byte kInstruction_sxtb_al_r9_r8_ROR_8[] = {
  0x4f, 0xfa, 0x98, 0xf9 // sxtb al r9 r8 ROR 8
};
const byte kInstruction_sxtb_al_r9_r8_ROR_16[] = {
  0x4f, 0xfa, 0xa8, 0xf9 // sxtb al r9 r8 ROR 16
};
const byte kInstruction_sxtb_al_r9_r8_ROR_24[] = {
  0x4f, 0xfa, 0xb8, 0xf9 // sxtb al r9 r8 ROR 24
};
const byte kInstruction_sxtb_al_r9_r9_ROR_0[] = {
  0x4f, 0xfa, 0x89, 0xf9 // sxtb al r9 r9 ROR 0
};
const byte kInstruction_sxtb_al_r9_r9_ROR_8[] = {
  0x4f, 0xfa, 0x99, 0xf9 // sxtb al r9 r9 ROR 8
};
const byte kInstruction_sxtb_al_r9_r9_ROR_16[] = {
  0x4f, 0xfa, 0xa9, 0xf9 // sxtb al r9 r9 ROR 16
};
const byte kInstruction_sxtb_al_r9_r9_ROR_24[] = {
  0x4f, 0xfa, 0xb9, 0xf9 // sxtb al r9 r9 ROR 24
};
const byte kInstruction_sxtb_al_r9_r10_ROR_0[] = {
  0x4f, 0xfa, 0x8a, 0xf9 // sxtb al r9 r10 ROR 0
};
const byte kInstruction_sxtb_al_r9_r10_ROR_8[] = {
  0x4f, 0xfa, 0x9a, 0xf9 // sxtb al r9 r10 ROR 8
};
const byte kInstruction_sxtb_al_r9_r10_ROR_16[] = {
  0x4f, 0xfa, 0xaa, 0xf9 // sxtb al r9 r10 ROR 16
};
const byte kInstruction_sxtb_al_r9_r10_ROR_24[] = {
  0x4f, 0xfa, 0xba, 0xf9 // sxtb al r9 r10 ROR 24
};
const byte kInstruction_sxtb_al_r9_r11_ROR_0[] = {
  0x4f, 0xfa, 0x8b, 0xf9 // sxtb al r9 r11 ROR 0
};
const byte kInstruction_sxtb_al_r9_r11_ROR_8[] = {
  0x4f, 0xfa, 0x9b, 0xf9 // sxtb al r9 r11 ROR 8
};
const byte kInstruction_sxtb_al_r9_r11_ROR_16[] = {
  0x4f, 0xfa, 0xab, 0xf9 // sxtb al r9 r11 ROR 16
};
const byte kInstruction_sxtb_al_r9_r11_ROR_24[] = {
  0x4f, 0xfa, 0xbb, 0xf9 // sxtb al r9 r11 ROR 24
};
const byte kInstruction_sxtb_al_r9_r12_ROR_0[] = {
  0x4f, 0xfa, 0x8c, 0xf9 // sxtb al r9 r12 ROR 0
};
const byte kInstruction_sxtb_al_r9_r12_ROR_8[] = {
  0x4f, 0xfa, 0x9c, 0xf9 // sxtb al r9 r12 ROR 8
};
const byte kInstruction_sxtb_al_r9_r12_ROR_16[] = {
  0x4f, 0xfa, 0xac, 0xf9 // sxtb al r9 r12 ROR 16
};
const byte kInstruction_sxtb_al_r9_r12_ROR_24[] = {
  0x4f, 0xfa, 0xbc, 0xf9 // sxtb al r9 r12 ROR 24
};
const byte kInstruction_sxtb_al_r9_r13_ROR_0[] = {
  0x4f, 0xfa, 0x8d, 0xf9 // sxtb al r9 r13 ROR 0
};
const byte kInstruction_sxtb_al_r9_r13_ROR_8[] = {
  0x4f, 0xfa, 0x9d, 0xf9 // sxtb al r9 r13 ROR 8
};
const byte kInstruction_sxtb_al_r9_r13_ROR_16[] = {
  0x4f, 0xfa, 0xad, 0xf9 // sxtb al r9 r13 ROR 16
};
const byte kInstruction_sxtb_al_r9_r13_ROR_24[] = {
  0x4f, 0xfa, 0xbd, 0xf9 // sxtb al r9 r13 ROR 24
};
const byte kInstruction_sxtb_al_r9_r14_ROR_0[] = {
  0x4f, 0xfa, 0x8e, 0xf9 // sxtb al r9 r14 ROR 0
};
const byte kInstruction_sxtb_al_r9_r14_ROR_8[] = {
  0x4f, 0xfa, 0x9e, 0xf9 // sxtb al r9 r14 ROR 8
};
const byte kInstruction_sxtb_al_r9_r14_ROR_16[] = {
  0x4f, 0xfa, 0xae, 0xf9 // sxtb al r9 r14 ROR 16
};
const byte kInstruction_sxtb_al_r9_r14_ROR_24[] = {
  0x4f, 0xfa, 0xbe, 0xf9 // sxtb al r9 r14 ROR 24
};
const byte kInstruction_sxtb_al_r10_r0_ROR_0[] = {
  0x4f, 0xfa, 0x80, 0xfa // sxtb al r10 r0 ROR 0
};
const byte kInstruction_sxtb_al_r10_r0_ROR_8[] = {
  0x4f, 0xfa, 0x90, 0xfa // sxtb al r10 r0 ROR 8
};
const byte kInstruction_sxtb_al_r10_r0_ROR_16[] = {
  0x4f, 0xfa, 0xa0, 0xfa // sxtb al r10 r0 ROR 16
};
const byte kInstruction_sxtb_al_r10_r0_ROR_24[] = {
  0x4f, 0xfa, 0xb0, 0xfa // sxtb al r10 r0 ROR 24
};
const byte kInstruction_sxtb_al_r10_r1_ROR_0[] = {
  0x4f, 0xfa, 0x81, 0xfa // sxtb al r10 r1 ROR 0
};
const byte kInstruction_sxtb_al_r10_r1_ROR_8[] = {
  0x4f, 0xfa, 0x91, 0xfa // sxtb al r10 r1 ROR 8
};
const byte kInstruction_sxtb_al_r10_r1_ROR_16[] = {
  0x4f, 0xfa, 0xa1, 0xfa // sxtb al r10 r1 ROR 16
};
const byte kInstruction_sxtb_al_r10_r1_ROR_24[] = {
  0x4f, 0xfa, 0xb1, 0xfa // sxtb al r10 r1 ROR 24
};
const byte kInstruction_sxtb_al_r10_r2_ROR_0[] = {
  0x4f, 0xfa, 0x82, 0xfa // sxtb al r10 r2 ROR 0
};
const byte kInstruction_sxtb_al_r10_r2_ROR_8[] = {
  0x4f, 0xfa, 0x92, 0xfa // sxtb al r10 r2 ROR 8
};
const byte kInstruction_sxtb_al_r10_r2_ROR_16[] = {
  0x4f, 0xfa, 0xa2, 0xfa // sxtb al r10 r2 ROR 16
};
const byte kInstruction_sxtb_al_r10_r2_ROR_24[] = {
  0x4f, 0xfa, 0xb2, 0xfa // sxtb al r10 r2 ROR 24
};
const byte kInstruction_sxtb_al_r10_r3_ROR_0[] = {
  0x4f, 0xfa, 0x83, 0xfa // sxtb al r10 r3 ROR 0
};
const byte kInstruction_sxtb_al_r10_r3_ROR_8[] = {
  0x4f, 0xfa, 0x93, 0xfa // sxtb al r10 r3 ROR 8
};
const byte kInstruction_sxtb_al_r10_r3_ROR_16[] = {
  0x4f, 0xfa, 0xa3, 0xfa // sxtb al r10 r3 ROR 16
};
const byte kInstruction_sxtb_al_r10_r3_ROR_24[] = {
  0x4f, 0xfa, 0xb3, 0xfa // sxtb al r10 r3 ROR 24
};
const byte kInstruction_sxtb_al_r10_r4_ROR_0[] = {
  0x4f, 0xfa, 0x84, 0xfa // sxtb al r10 r4 ROR 0
};
const byte kInstruction_sxtb_al_r10_r4_ROR_8[] = {
  0x4f, 0xfa, 0x94, 0xfa // sxtb al r10 r4 ROR 8
};
const byte kInstruction_sxtb_al_r10_r4_ROR_16[] = {
  0x4f, 0xfa, 0xa4, 0xfa // sxtb al r10 r4 ROR 16
};
const byte kInstruction_sxtb_al_r10_r4_ROR_24[] = {
  0x4f, 0xfa, 0xb4, 0xfa // sxtb al r10 r4 ROR 24
};
const byte kInstruction_sxtb_al_r10_r5_ROR_0[] = {
  0x4f, 0xfa, 0x85, 0xfa // sxtb al r10 r5 ROR 0
};
const byte kInstruction_sxtb_al_r10_r5_ROR_8[] = {
  0x4f, 0xfa, 0x95, 0xfa // sxtb al r10 r5 ROR 8
};
const byte kInstruction_sxtb_al_r10_r5_ROR_16[] = {
  0x4f, 0xfa, 0xa5, 0xfa // sxtb al r10 r5 ROR 16
};
const byte kInstruction_sxtb_al_r10_r5_ROR_24[] = {
  0x4f, 0xfa, 0xb5, 0xfa // sxtb al r10 r5 ROR 24
};
const byte kInstruction_sxtb_al_r10_r6_ROR_0[] = {
  0x4f, 0xfa, 0x86, 0xfa // sxtb al r10 r6 ROR 0
};
const byte kInstruction_sxtb_al_r10_r6_ROR_8[] = {
  0x4f, 0xfa, 0x96, 0xfa // sxtb al r10 r6 ROR 8
};
const byte kInstruction_sxtb_al_r10_r6_ROR_16[] = {
  0x4f, 0xfa, 0xa6, 0xfa // sxtb al r10 r6 ROR 16
};
const byte kInstruction_sxtb_al_r10_r6_ROR_24[] = {
  0x4f, 0xfa, 0xb6, 0xfa // sxtb al r10 r6 ROR 24
};
const byte kInstruction_sxtb_al_r10_r7_ROR_0[] = {
  0x4f, 0xfa, 0x87, 0xfa // sxtb al r10 r7 ROR 0
};
const byte kInstruction_sxtb_al_r10_r7_ROR_8[] = {
  0x4f, 0xfa, 0x97, 0xfa // sxtb al r10 r7 ROR 8
};
const byte kInstruction_sxtb_al_r10_r7_ROR_16[] = {
  0x4f, 0xfa, 0xa7, 0xfa // sxtb al r10 r7 ROR 16
};
const byte kInstruction_sxtb_al_r10_r7_ROR_24[] = {
  0x4f, 0xfa, 0xb7, 0xfa // sxtb al r10 r7 ROR 24
};
const byte kInstruction_sxtb_al_r10_r8_ROR_0[] = {
  0x4f, 0xfa, 0x88, 0xfa // sxtb al r10 r8 ROR 0
};
const byte kInstruction_sxtb_al_r10_r8_ROR_8[] = {
  0x4f, 0xfa, 0x98, 0xfa // sxtb al r10 r8 ROR 8
};
const byte kInstruction_sxtb_al_r10_r8_ROR_16[] = {
  0x4f, 0xfa, 0xa8, 0xfa // sxtb al r10 r8 ROR 16
};
const byte kInstruction_sxtb_al_r10_r8_ROR_24[] = {
  0x4f, 0xfa, 0xb8, 0xfa // sxtb al r10 r8 ROR 24
};
const byte kInstruction_sxtb_al_r10_r9_ROR_0[] = {
  0x4f, 0xfa, 0x89, 0xfa // sxtb al r10 r9 ROR 0
};
const byte kInstruction_sxtb_al_r10_r9_ROR_8[] = {
  0x4f, 0xfa, 0x99, 0xfa // sxtb al r10 r9 ROR 8
};
const byte kInstruction_sxtb_al_r10_r9_ROR_16[] = {
  0x4f, 0xfa, 0xa9, 0xfa // sxtb al r10 r9 ROR 16
};
const byte kInstruction_sxtb_al_r10_r9_ROR_24[] = {
  0x4f, 0xfa, 0xb9, 0xfa // sxtb al r10 r9 ROR 24
};
const byte kInstruction_sxtb_al_r10_r10_ROR_0[] = {
  0x4f, 0xfa, 0x8a, 0xfa // sxtb al r10 r10 ROR 0
};
const byte kInstruction_sxtb_al_r10_r10_ROR_8[] = {
  0x4f, 0xfa, 0x9a, 0xfa // sxtb al r10 r10 ROR 8
};
const byte kInstruction_sxtb_al_r10_r10_ROR_16[] = {
  0x4f, 0xfa, 0xaa, 0xfa // sxtb al r10 r10 ROR 16
};
const byte kInstruction_sxtb_al_r10_r10_ROR_24[] = {
  0x4f, 0xfa, 0xba, 0xfa // sxtb al r10 r10 ROR 24
};
const byte kInstruction_sxtb_al_r10_r11_ROR_0[] = {
  0x4f, 0xfa, 0x8b, 0xfa // sxtb al r10 r11 ROR 0
};
const byte kInstruction_sxtb_al_r10_r11_ROR_8[] = {
  0x4f, 0xfa, 0x9b, 0xfa // sxtb al r10 r11 ROR 8
};
const byte kInstruction_sxtb_al_r10_r11_ROR_16[] = {
  0x4f, 0xfa, 0xab, 0xfa // sxtb al r10 r11 ROR 16
};
const byte kInstruction_sxtb_al_r10_r11_ROR_24[] = {
  0x4f, 0xfa, 0xbb, 0xfa // sxtb al r10 r11 ROR 24
};
const byte kInstruction_sxtb_al_r10_r12_ROR_0[] = {
  0x4f, 0xfa, 0x8c, 0xfa // sxtb al r10 r12 ROR 0
};
const byte kInstruction_sxtb_al_r10_r12_ROR_8[] = {
  0x4f, 0xfa, 0x9c, 0xfa // sxtb al r10 r12 ROR 8
};
const byte kInstruction_sxtb_al_r10_r12_ROR_16[] = {
  0x4f, 0xfa, 0xac, 0xfa // sxtb al r10 r12 ROR 16
};
const byte kInstruction_sxtb_al_r10_r12_ROR_24[] = {
  0x4f, 0xfa, 0xbc, 0xfa // sxtb al r10 r12 ROR 24
};
const byte kInstruction_sxtb_al_r10_r13_ROR_0[] = {
  0x4f, 0xfa, 0x8d, 0xfa // sxtb al r10 r13 ROR 0
};
const byte kInstruction_sxtb_al_r10_r13_ROR_8[] = {
  0x4f, 0xfa, 0x9d, 0xfa // sxtb al r10 r13 ROR 8
};
const byte kInstruction_sxtb_al_r10_r13_ROR_16[] = {
  0x4f, 0xfa, 0xad, 0xfa // sxtb al r10 r13 ROR 16
};
const byte kInstruction_sxtb_al_r10_r13_ROR_24[] = {
  0x4f, 0xfa, 0xbd, 0xfa // sxtb al r10 r13 ROR 24
};
const byte kInstruction_sxtb_al_r10_r14_ROR_0[] = {
  0x4f, 0xfa, 0x8e, 0xfa // sxtb al r10 r14 ROR 0
};
const byte kInstruction_sxtb_al_r10_r14_ROR_8[] = {
  0x4f, 0xfa, 0x9e, 0xfa // sxtb al r10 r14 ROR 8
};
const byte kInstruction_sxtb_al_r10_r14_ROR_16[] = {
  0x4f, 0xfa, 0xae, 0xfa // sxtb al r10 r14 ROR 16
};
const byte kInstruction_sxtb_al_r10_r14_ROR_24[] = {
  0x4f, 0xfa, 0xbe, 0xfa // sxtb al r10 r14 ROR 24
};
const byte kInstruction_sxtb_al_r11_r0_ROR_0[] = {
  0x4f, 0xfa, 0x80, 0xfb // sxtb al r11 r0 ROR 0
};
const byte kInstruction_sxtb_al_r11_r0_ROR_8[] = {
  0x4f, 0xfa, 0x90, 0xfb // sxtb al r11 r0 ROR 8
};
const byte kInstruction_sxtb_al_r11_r0_ROR_16[] = {
  0x4f, 0xfa, 0xa0, 0xfb // sxtb al r11 r0 ROR 16
};
const byte kInstruction_sxtb_al_r11_r0_ROR_24[] = {
  0x4f, 0xfa, 0xb0, 0xfb // sxtb al r11 r0 ROR 24
};
const byte kInstruction_sxtb_al_r11_r1_ROR_0[] = {
  0x4f, 0xfa, 0x81, 0xfb // sxtb al r11 r1 ROR 0
};
const byte kInstruction_sxtb_al_r11_r1_ROR_8[] = {
  0x4f, 0xfa, 0x91, 0xfb // sxtb al r11 r1 ROR 8
};
const byte kInstruction_sxtb_al_r11_r1_ROR_16[] = {
  0x4f, 0xfa, 0xa1, 0xfb // sxtb al r11 r1 ROR 16
};
const byte kInstruction_sxtb_al_r11_r1_ROR_24[] = {
  0x4f, 0xfa, 0xb1, 0xfb // sxtb al r11 r1 ROR 24
};
const byte kInstruction_sxtb_al_r11_r2_ROR_0[] = {
  0x4f, 0xfa, 0x82, 0xfb // sxtb al r11 r2 ROR 0
};
const byte kInstruction_sxtb_al_r11_r2_ROR_8[] = {
  0x4f, 0xfa, 0x92, 0xfb // sxtb al r11 r2 ROR 8
};
const byte kInstruction_sxtb_al_r11_r2_ROR_16[] = {
  0x4f, 0xfa, 0xa2, 0xfb // sxtb al r11 r2 ROR 16
};
const byte kInstruction_sxtb_al_r11_r2_ROR_24[] = {
  0x4f, 0xfa, 0xb2, 0xfb // sxtb al r11 r2 ROR 24
};
const byte kInstruction_sxtb_al_r11_r3_ROR_0[] = {
  0x4f, 0xfa, 0x83, 0xfb // sxtb al r11 r3 ROR 0
};
const byte kInstruction_sxtb_al_r11_r3_ROR_8[] = {
  0x4f, 0xfa, 0x93, 0xfb // sxtb al r11 r3 ROR 8
};
const byte kInstruction_sxtb_al_r11_r3_ROR_16[] = {
  0x4f, 0xfa, 0xa3, 0xfb // sxtb al r11 r3 ROR 16
};
const byte kInstruction_sxtb_al_r11_r3_ROR_24[] = {
  0x4f, 0xfa, 0xb3, 0xfb // sxtb al r11 r3 ROR 24
};
const byte kInstruction_sxtb_al_r11_r4_ROR_0[] = {
  0x4f, 0xfa, 0x84, 0xfb // sxtb al r11 r4 ROR 0
};
const byte kInstruction_sxtb_al_r11_r4_ROR_8[] = {
  0x4f, 0xfa, 0x94, 0xfb // sxtb al r11 r4 ROR 8
};
const byte kInstruction_sxtb_al_r11_r4_ROR_16[] = {
  0x4f, 0xfa, 0xa4, 0xfb // sxtb al r11 r4 ROR 16
};
const byte kInstruction_sxtb_al_r11_r4_ROR_24[] = {
  0x4f, 0xfa, 0xb4, 0xfb // sxtb al r11 r4 ROR 24
};
const byte kInstruction_sxtb_al_r11_r5_ROR_0[] = {
  0x4f, 0xfa, 0x85, 0xfb // sxtb al r11 r5 ROR 0
};
const byte kInstruction_sxtb_al_r11_r5_ROR_8[] = {
  0x4f, 0xfa, 0x95, 0xfb // sxtb al r11 r5 ROR 8
};
const byte kInstruction_sxtb_al_r11_r5_ROR_16[] = {
  0x4f, 0xfa, 0xa5, 0xfb // sxtb al r11 r5 ROR 16
};
const byte kInstruction_sxtb_al_r11_r5_ROR_24[] = {
  0x4f, 0xfa, 0xb5, 0xfb // sxtb al r11 r5 ROR 24
};
const byte kInstruction_sxtb_al_r11_r6_ROR_0[] = {
  0x4f, 0xfa, 0x86, 0xfb // sxtb al r11 r6 ROR 0
};
const byte kInstruction_sxtb_al_r11_r6_ROR_8[] = {
  0x4f, 0xfa, 0x96, 0xfb // sxtb al r11 r6 ROR 8
};
const byte kInstruction_sxtb_al_r11_r6_ROR_16[] = {
  0x4f, 0xfa, 0xa6, 0xfb // sxtb al r11 r6 ROR 16
};
const byte kInstruction_sxtb_al_r11_r6_ROR_24[] = {
  0x4f, 0xfa, 0xb6, 0xfb // sxtb al r11 r6 ROR 24
};
const byte kInstruction_sxtb_al_r11_r7_ROR_0[] = {
  0x4f, 0xfa, 0x87, 0xfb // sxtb al r11 r7 ROR 0
};
const byte kInstruction_sxtb_al_r11_r7_ROR_8[] = {
  0x4f, 0xfa, 0x97, 0xfb // sxtb al r11 r7 ROR 8
};
const byte kInstruction_sxtb_al_r11_r7_ROR_16[] = {
  0x4f, 0xfa, 0xa7, 0xfb // sxtb al r11 r7 ROR 16
};
const byte kInstruction_sxtb_al_r11_r7_ROR_24[] = {
  0x4f, 0xfa, 0xb7, 0xfb // sxtb al r11 r7 ROR 24
};
const byte kInstruction_sxtb_al_r11_r8_ROR_0[] = {
  0x4f, 0xfa, 0x88, 0xfb // sxtb al r11 r8 ROR 0
};
const byte kInstruction_sxtb_al_r11_r8_ROR_8[] = {
  0x4f, 0xfa, 0x98, 0xfb // sxtb al r11 r8 ROR 8
};
const byte kInstruction_sxtb_al_r11_r8_ROR_16[] = {
  0x4f, 0xfa, 0xa8, 0xfb // sxtb al r11 r8 ROR 16
};
const byte kInstruction_sxtb_al_r11_r8_ROR_24[] = {
  0x4f, 0xfa, 0xb8, 0xfb // sxtb al r11 r8 ROR 24
};
const byte kInstruction_sxtb_al_r11_r9_ROR_0[] = {
  0x4f, 0xfa, 0x89, 0xfb // sxtb al r11 r9 ROR 0
};
const byte kInstruction_sxtb_al_r11_r9_ROR_8[] = {
  0x4f, 0xfa, 0x99, 0xfb // sxtb al r11 r9 ROR 8
};
const byte kInstruction_sxtb_al_r11_r9_ROR_16[] = {
  0x4f, 0xfa, 0xa9, 0xfb // sxtb al r11 r9 ROR 16
};
const byte kInstruction_sxtb_al_r11_r9_ROR_24[] = {
  0x4f, 0xfa, 0xb9, 0xfb // sxtb al r11 r9 ROR 24
};
const byte kInstruction_sxtb_al_r11_r10_ROR_0[] = {
  0x4f, 0xfa, 0x8a, 0xfb // sxtb al r11 r10 ROR 0
};
const byte kInstruction_sxtb_al_r11_r10_ROR_8[] = {
  0x4f, 0xfa, 0x9a, 0xfb // sxtb al r11 r10 ROR 8
};
const byte kInstruction_sxtb_al_r11_r10_ROR_16[] = {
  0x4f, 0xfa, 0xaa, 0xfb // sxtb al r11 r10 ROR 16
};
const byte kInstruction_sxtb_al_r11_r10_ROR_24[] = {
  0x4f, 0xfa, 0xba, 0xfb // sxtb al r11 r10 ROR 24
};
const byte kInstruction_sxtb_al_r11_r11_ROR_0[] = {
  0x4f, 0xfa, 0x8b, 0xfb // sxtb al r11 r11 ROR 0
};
const byte kInstruction_sxtb_al_r11_r11_ROR_8[] = {
  0x4f, 0xfa, 0x9b, 0xfb // sxtb al r11 r11 ROR 8
};
const byte kInstruction_sxtb_al_r11_r11_ROR_16[] = {
  0x4f, 0xfa, 0xab, 0xfb // sxtb al r11 r11 ROR 16
};
const byte kInstruction_sxtb_al_r11_r11_ROR_24[] = {
  0x4f, 0xfa, 0xbb, 0xfb // sxtb al r11 r11 ROR 24
};
const byte kInstruction_sxtb_al_r11_r12_ROR_0[] = {
  0x4f, 0xfa, 0x8c, 0xfb // sxtb al r11 r12 ROR 0
};
const byte kInstruction_sxtb_al_r11_r12_ROR_8[] = {
  0x4f, 0xfa, 0x9c, 0xfb // sxtb al r11 r12 ROR 8
};
const byte kInstruction_sxtb_al_r11_r12_ROR_16[] = {
  0x4f, 0xfa, 0xac, 0xfb // sxtb al r11 r12 ROR 16
};
const byte kInstruction_sxtb_al_r11_r12_ROR_24[] = {
  0x4f, 0xfa, 0xbc, 0xfb // sxtb al r11 r12 ROR 24
};
const byte kInstruction_sxtb_al_r11_r13_ROR_0[] = {
  0x4f, 0xfa, 0x8d, 0xfb // sxtb al r11 r13 ROR 0
};
const byte kInstruction_sxtb_al_r11_r13_ROR_8[] = {
  0x4f, 0xfa, 0x9d, 0xfb // sxtb al r11 r13 ROR 8
};
const byte kInstruction_sxtb_al_r11_r13_ROR_16[] = {
  0x4f, 0xfa, 0xad, 0xfb // sxtb al r11 r13 ROR 16
};
const byte kInstruction_sxtb_al_r11_r13_ROR_24[] = {
  0x4f, 0xfa, 0xbd, 0xfb // sxtb al r11 r13 ROR 24
};
const byte kInstruction_sxtb_al_r11_r14_ROR_0[] = {
  0x4f, 0xfa, 0x8e, 0xfb // sxtb al r11 r14 ROR 0
};
const byte kInstruction_sxtb_al_r11_r14_ROR_8[] = {
  0x4f, 0xfa, 0x9e, 0xfb // sxtb al r11 r14 ROR 8
};
const byte kInstruction_sxtb_al_r11_r14_ROR_16[] = {
  0x4f, 0xfa, 0xae, 0xfb // sxtb al r11 r14 ROR 16
};
const byte kInstruction_sxtb_al_r11_r14_ROR_24[] = {
  0x4f, 0xfa, 0xbe, 0xfb // sxtb al r11 r14 ROR 24
};
const byte kInstruction_sxtb_al_r12_r0_ROR_0[] = {
  0x4f, 0xfa, 0x80, 0xfc // sxtb al r12 r0 ROR 0
};
const byte kInstruction_sxtb_al_r12_r0_ROR_8[] = {
  0x4f, 0xfa, 0x90, 0xfc // sxtb al r12 r0 ROR 8
};
const byte kInstruction_sxtb_al_r12_r0_ROR_16[] = {
  0x4f, 0xfa, 0xa0, 0xfc // sxtb al r12 r0 ROR 16
};
const byte kInstruction_sxtb_al_r12_r0_ROR_24[] = {
  0x4f, 0xfa, 0xb0, 0xfc // sxtb al r12 r0 ROR 24
};
const byte kInstruction_sxtb_al_r12_r1_ROR_0[] = {
  0x4f, 0xfa, 0x81, 0xfc // sxtb al r12 r1 ROR 0
};
const byte kInstruction_sxtb_al_r12_r1_ROR_8[] = {
  0x4f, 0xfa, 0x91, 0xfc // sxtb al r12 r1 ROR 8
};
const byte kInstruction_sxtb_al_r12_r1_ROR_16[] = {
  0x4f, 0xfa, 0xa1, 0xfc // sxtb al r12 r1 ROR 16
};
const byte kInstruction_sxtb_al_r12_r1_ROR_24[] = {
  0x4f, 0xfa, 0xb1, 0xfc // sxtb al r12 r1 ROR 24
};
const byte kInstruction_sxtb_al_r12_r2_ROR_0[] = {
  0x4f, 0xfa, 0x82, 0xfc // sxtb al r12 r2 ROR 0
};
const byte kInstruction_sxtb_al_r12_r2_ROR_8[] = {
  0x4f, 0xfa, 0x92, 0xfc // sxtb al r12 r2 ROR 8
};
const byte kInstruction_sxtb_al_r12_r2_ROR_16[] = {
  0x4f, 0xfa, 0xa2, 0xfc // sxtb al r12 r2 ROR 16
};
const byte kInstruction_sxtb_al_r12_r2_ROR_24[] = {
  0x4f, 0xfa, 0xb2, 0xfc // sxtb al r12 r2 ROR 24
};
const byte kInstruction_sxtb_al_r12_r3_ROR_0[] = {
  0x4f, 0xfa, 0x83, 0xfc // sxtb al r12 r3 ROR 0
};
const byte kInstruction_sxtb_al_r12_r3_ROR_8[] = {
  0x4f, 0xfa, 0x93, 0xfc // sxtb al r12 r3 ROR 8
};
const byte kInstruction_sxtb_al_r12_r3_ROR_16[] = {
  0x4f, 0xfa, 0xa3, 0xfc // sxtb al r12 r3 ROR 16
};
const byte kInstruction_sxtb_al_r12_r3_ROR_24[] = {
  0x4f, 0xfa, 0xb3, 0xfc // sxtb al r12 r3 ROR 24
};
const byte kInstruction_sxtb_al_r12_r4_ROR_0[] = {
  0x4f, 0xfa, 0x84, 0xfc // sxtb al r12 r4 ROR 0
};
const byte kInstruction_sxtb_al_r12_r4_ROR_8[] = {
  0x4f, 0xfa, 0x94, 0xfc // sxtb al r12 r4 ROR 8
};
const byte kInstruction_sxtb_al_r12_r4_ROR_16[] = {
  0x4f, 0xfa, 0xa4, 0xfc // sxtb al r12 r4 ROR 16
};
const byte kInstruction_sxtb_al_r12_r4_ROR_24[] = {
  0x4f, 0xfa, 0xb4, 0xfc // sxtb al r12 r4 ROR 24
};
const byte kInstruction_sxtb_al_r12_r5_ROR_0[] = {
  0x4f, 0xfa, 0x85, 0xfc // sxtb al r12 r5 ROR 0
};
const byte kInstruction_sxtb_al_r12_r5_ROR_8[] = {
  0x4f, 0xfa, 0x95, 0xfc // sxtb al r12 r5 ROR 8
};
const byte kInstruction_sxtb_al_r12_r5_ROR_16[] = {
  0x4f, 0xfa, 0xa5, 0xfc // sxtb al r12 r5 ROR 16
};
const byte kInstruction_sxtb_al_r12_r5_ROR_24[] = {
  0x4f, 0xfa, 0xb5, 0xfc // sxtb al r12 r5 ROR 24
};
const byte kInstruction_sxtb_al_r12_r6_ROR_0[] = {
  0x4f, 0xfa, 0x86, 0xfc // sxtb al r12 r6 ROR 0
};
const byte kInstruction_sxtb_al_r12_r6_ROR_8[] = {
  0x4f, 0xfa, 0x96, 0xfc // sxtb al r12 r6 ROR 8
};
const byte kInstruction_sxtb_al_r12_r6_ROR_16[] = {
  0x4f, 0xfa, 0xa6, 0xfc // sxtb al r12 r6 ROR 16
};
const byte kInstruction_sxtb_al_r12_r6_ROR_24[] = {
  0x4f, 0xfa, 0xb6, 0xfc // sxtb al r12 r6 ROR 24
};
const byte kInstruction_sxtb_al_r12_r7_ROR_0[] = {
  0x4f, 0xfa, 0x87, 0xfc // sxtb al r12 r7 ROR 0
};
const byte kInstruction_sxtb_al_r12_r7_ROR_8[] = {
  0x4f, 0xfa, 0x97, 0xfc // sxtb al r12 r7 ROR 8
};
const byte kInstruction_sxtb_al_r12_r7_ROR_16[] = {
  0x4f, 0xfa, 0xa7, 0xfc // sxtb al r12 r7 ROR 16
};
const byte kInstruction_sxtb_al_r12_r7_ROR_24[] = {
  0x4f, 0xfa, 0xb7, 0xfc // sxtb al r12 r7 ROR 24
};
const byte kInstruction_sxtb_al_r12_r8_ROR_0[] = {
  0x4f, 0xfa, 0x88, 0xfc // sxtb al r12 r8 ROR 0
};
const byte kInstruction_sxtb_al_r12_r8_ROR_8[] = {
  0x4f, 0xfa, 0x98, 0xfc // sxtb al r12 r8 ROR 8
};
const byte kInstruction_sxtb_al_r12_r8_ROR_16[] = {
  0x4f, 0xfa, 0xa8, 0xfc // sxtb al r12 r8 ROR 16
};
const byte kInstruction_sxtb_al_r12_r8_ROR_24[] = {
  0x4f, 0xfa, 0xb8, 0xfc // sxtb al r12 r8 ROR 24
};
const byte kInstruction_sxtb_al_r12_r9_ROR_0[] = {
  0x4f, 0xfa, 0x89, 0xfc // sxtb al r12 r9 ROR 0
};
const byte kInstruction_sxtb_al_r12_r9_ROR_8[] = {
  0x4f, 0xfa, 0x99, 0xfc // sxtb al r12 r9 ROR 8
};
const byte kInstruction_sxtb_al_r12_r9_ROR_16[] = {
  0x4f, 0xfa, 0xa9, 0xfc // sxtb al r12 r9 ROR 16
};
const byte kInstruction_sxtb_al_r12_r9_ROR_24[] = {
  0x4f, 0xfa, 0xb9, 0xfc // sxtb al r12 r9 ROR 24
};
const byte kInstruction_sxtb_al_r12_r10_ROR_0[] = {
  0x4f, 0xfa, 0x8a, 0xfc // sxtb al r12 r10 ROR 0
};
const byte kInstruction_sxtb_al_r12_r10_ROR_8[] = {
  0x4f, 0xfa, 0x9a, 0xfc // sxtb al r12 r10 ROR 8
};
const byte kInstruction_sxtb_al_r12_r10_ROR_16[] = {
  0x4f, 0xfa, 0xaa, 0xfc // sxtb al r12 r10 ROR 16
};
const byte kInstruction_sxtb_al_r12_r10_ROR_24[] = {
  0x4f, 0xfa, 0xba, 0xfc // sxtb al r12 r10 ROR 24
};
const byte kInstruction_sxtb_al_r12_r11_ROR_0[] = {
  0x4f, 0xfa, 0x8b, 0xfc // sxtb al r12 r11 ROR 0
};
const byte kInstruction_sxtb_al_r12_r11_ROR_8[] = {
  0x4f, 0xfa, 0x9b, 0xfc // sxtb al r12 r11 ROR 8
};
const byte kInstruction_sxtb_al_r12_r11_ROR_16[] = {
  0x4f, 0xfa, 0xab, 0xfc // sxtb al r12 r11 ROR 16
};
const byte kInstruction_sxtb_al_r12_r11_ROR_24[] = {
  0x4f, 0xfa, 0xbb, 0xfc // sxtb al r12 r11 ROR 24
};
const byte kInstruction_sxtb_al_r12_r12_ROR_0[] = {
  0x4f, 0xfa, 0x8c, 0xfc // sxtb al r12 r12 ROR 0
};
const byte kInstruction_sxtb_al_r12_r12_ROR_8[] = {
  0x4f, 0xfa, 0x9c, 0xfc // sxtb al r12 r12 ROR 8
};
const byte kInstruction_sxtb_al_r12_r12_ROR_16[] = {
  0x4f, 0xfa, 0xac, 0xfc // sxtb al r12 r12 ROR 16
};
const byte kInstruction_sxtb_al_r12_r12_ROR_24[] = {
  0x4f, 0xfa, 0xbc, 0xfc // sxtb al r12 r12 ROR 24
};
const byte kInstruction_sxtb_al_r12_r13_ROR_0[] = {
  0x4f, 0xfa, 0x8d, 0xfc // sxtb al r12 r13 ROR 0
};
const byte kInstruction_sxtb_al_r12_r13_ROR_8[] = {
  0x4f, 0xfa, 0x9d, 0xfc // sxtb al r12 r13 ROR 8
};
const byte kInstruction_sxtb_al_r12_r13_ROR_16[] = {
  0x4f, 0xfa, 0xad, 0xfc // sxtb al r12 r13 ROR 16
};
const byte kInstruction_sxtb_al_r12_r13_ROR_24[] = {
  0x4f, 0xfa, 0xbd, 0xfc // sxtb al r12 r13 ROR 24
};
const byte kInstruction_sxtb_al_r12_r14_ROR_0[] = {
  0x4f, 0xfa, 0x8e, 0xfc // sxtb al r12 r14 ROR 0
};
const byte kInstruction_sxtb_al_r12_r14_ROR_8[] = {
  0x4f, 0xfa, 0x9e, 0xfc // sxtb al r12 r14 ROR 8
};
const byte kInstruction_sxtb_al_r12_r14_ROR_16[] = {
  0x4f, 0xfa, 0xae, 0xfc // sxtb al r12 r14 ROR 16
};
const byte kInstruction_sxtb_al_r12_r14_ROR_24[] = {
  0x4f, 0xfa, 0xbe, 0xfc // sxtb al r12 r14 ROR 24
};
const byte kInstruction_sxtb_al_r13_r0_ROR_0[] = {
  0x4f, 0xfa, 0x80, 0xfd // sxtb al r13 r0 ROR 0
};
const byte kInstruction_sxtb_al_r13_r0_ROR_8[] = {
  0x4f, 0xfa, 0x90, 0xfd // sxtb al r13 r0 ROR 8
};
const byte kInstruction_sxtb_al_r13_r0_ROR_16[] = {
  0x4f, 0xfa, 0xa0, 0xfd // sxtb al r13 r0 ROR 16
};
const byte kInstruction_sxtb_al_r13_r0_ROR_24[] = {
  0x4f, 0xfa, 0xb0, 0xfd // sxtb al r13 r0 ROR 24
};
const byte kInstruction_sxtb_al_r13_r1_ROR_0[] = {
  0x4f, 0xfa, 0x81, 0xfd // sxtb al r13 r1 ROR 0
};
const byte kInstruction_sxtb_al_r13_r1_ROR_8[] = {
  0x4f, 0xfa, 0x91, 0xfd // sxtb al r13 r1 ROR 8
};
const byte kInstruction_sxtb_al_r13_r1_ROR_16[] = {
  0x4f, 0xfa, 0xa1, 0xfd // sxtb al r13 r1 ROR 16
};
const byte kInstruction_sxtb_al_r13_r1_ROR_24[] = {
  0x4f, 0xfa, 0xb1, 0xfd // sxtb al r13 r1 ROR 24
};
const byte kInstruction_sxtb_al_r13_r2_ROR_0[] = {
  0x4f, 0xfa, 0x82, 0xfd // sxtb al r13 r2 ROR 0
};
const byte kInstruction_sxtb_al_r13_r2_ROR_8[] = {
  0x4f, 0xfa, 0x92, 0xfd // sxtb al r13 r2 ROR 8
};
const byte kInstruction_sxtb_al_r13_r2_ROR_16[] = {
  0x4f, 0xfa, 0xa2, 0xfd // sxtb al r13 r2 ROR 16
};
const byte kInstruction_sxtb_al_r13_r2_ROR_24[] = {
  0x4f, 0xfa, 0xb2, 0xfd // sxtb al r13 r2 ROR 24
};
const byte kInstruction_sxtb_al_r13_r3_ROR_0[] = {
  0x4f, 0xfa, 0x83, 0xfd // sxtb al r13 r3 ROR 0
};
const byte kInstruction_sxtb_al_r13_r3_ROR_8[] = {
  0x4f, 0xfa, 0x93, 0xfd // sxtb al r13 r3 ROR 8
};
const byte kInstruction_sxtb_al_r13_r3_ROR_16[] = {
  0x4f, 0xfa, 0xa3, 0xfd // sxtb al r13 r3 ROR 16
};
const byte kInstruction_sxtb_al_r13_r3_ROR_24[] = {
  0x4f, 0xfa, 0xb3, 0xfd // sxtb al r13 r3 ROR 24
};
const byte kInstruction_sxtb_al_r13_r4_ROR_0[] = {
  0x4f, 0xfa, 0x84, 0xfd // sxtb al r13 r4 ROR 0
};
const byte kInstruction_sxtb_al_r13_r4_ROR_8[] = {
  0x4f, 0xfa, 0x94, 0xfd // sxtb al r13 r4 ROR 8
};
const byte kInstruction_sxtb_al_r13_r4_ROR_16[] = {
  0x4f, 0xfa, 0xa4, 0xfd // sxtb al r13 r4 ROR 16
};
const byte kInstruction_sxtb_al_r13_r4_ROR_24[] = {
  0x4f, 0xfa, 0xb4, 0xfd // sxtb al r13 r4 ROR 24
};
const byte kInstruction_sxtb_al_r13_r5_ROR_0[] = {
  0x4f, 0xfa, 0x85, 0xfd // sxtb al r13 r5 ROR 0
};
const byte kInstruction_sxtb_al_r13_r5_ROR_8[] = {
  0x4f, 0xfa, 0x95, 0xfd // sxtb al r13 r5 ROR 8
};
const byte kInstruction_sxtb_al_r13_r5_ROR_16[] = {
  0x4f, 0xfa, 0xa5, 0xfd // sxtb al r13 r5 ROR 16
};
const byte kInstruction_sxtb_al_r13_r5_ROR_24[] = {
  0x4f, 0xfa, 0xb5, 0xfd // sxtb al r13 r5 ROR 24
};
const byte kInstruction_sxtb_al_r13_r6_ROR_0[] = {
  0x4f, 0xfa, 0x86, 0xfd // sxtb al r13 r6 ROR 0
};
const byte kInstruction_sxtb_al_r13_r6_ROR_8[] = {
  0x4f, 0xfa, 0x96, 0xfd // sxtb al r13 r6 ROR 8
};
const byte kInstruction_sxtb_al_r13_r6_ROR_16[] = {
  0x4f, 0xfa, 0xa6, 0xfd // sxtb al r13 r6 ROR 16
};
const byte kInstruction_sxtb_al_r13_r6_ROR_24[] = {
  0x4f, 0xfa, 0xb6, 0xfd // sxtb al r13 r6 ROR 24
};
const byte kInstruction_sxtb_al_r13_r7_ROR_0[] = {
  0x4f, 0xfa, 0x87, 0xfd // sxtb al r13 r7 ROR 0
};
const byte kInstruction_sxtb_al_r13_r7_ROR_8[] = {
  0x4f, 0xfa, 0x97, 0xfd // sxtb al r13 r7 ROR 8
};
const byte kInstruction_sxtb_al_r13_r7_ROR_16[] = {
  0x4f, 0xfa, 0xa7, 0xfd // sxtb al r13 r7 ROR 16
};
const byte kInstruction_sxtb_al_r13_r7_ROR_24[] = {
  0x4f, 0xfa, 0xb7, 0xfd // sxtb al r13 r7 ROR 24
};
const byte kInstruction_sxtb_al_r13_r8_ROR_0[] = {
  0x4f, 0xfa, 0x88, 0xfd // sxtb al r13 r8 ROR 0
};
const byte kInstruction_sxtb_al_r13_r8_ROR_8[] = {
  0x4f, 0xfa, 0x98, 0xfd // sxtb al r13 r8 ROR 8
};
const byte kInstruction_sxtb_al_r13_r8_ROR_16[] = {
  0x4f, 0xfa, 0xa8, 0xfd // sxtb al r13 r8 ROR 16
};
const byte kInstruction_sxtb_al_r13_r8_ROR_24[] = {
  0x4f, 0xfa, 0xb8, 0xfd // sxtb al r13 r8 ROR 24
};
const byte kInstruction_sxtb_al_r13_r9_ROR_0[] = {
  0x4f, 0xfa, 0x89, 0xfd // sxtb al r13 r9 ROR 0
};
const byte kInstruction_sxtb_al_r13_r9_ROR_8[] = {
  0x4f, 0xfa, 0x99, 0xfd // sxtb al r13 r9 ROR 8
};
const byte kInstruction_sxtb_al_r13_r9_ROR_16[] = {
  0x4f, 0xfa, 0xa9, 0xfd // sxtb al r13 r9 ROR 16
};
const byte kInstruction_sxtb_al_r13_r9_ROR_24[] = {
  0x4f, 0xfa, 0xb9, 0xfd // sxtb al r13 r9 ROR 24
};
const byte kInstruction_sxtb_al_r13_r10_ROR_0[] = {
  0x4f, 0xfa, 0x8a, 0xfd // sxtb al r13 r10 ROR 0
};
const byte kInstruction_sxtb_al_r13_r10_ROR_8[] = {
  0x4f, 0xfa, 0x9a, 0xfd // sxtb al r13 r10 ROR 8
};
const byte kInstruction_sxtb_al_r13_r10_ROR_16[] = {
  0x4f, 0xfa, 0xaa, 0xfd // sxtb al r13 r10 ROR 16
};
const byte kInstruction_sxtb_al_r13_r10_ROR_24[] = {
  0x4f, 0xfa, 0xba, 0xfd // sxtb al r13 r10 ROR 24
};
const byte kInstruction_sxtb_al_r13_r11_ROR_0[] = {
  0x4f, 0xfa, 0x8b, 0xfd // sxtb al r13 r11 ROR 0
};
const byte kInstruction_sxtb_al_r13_r11_ROR_8[] = {
  0x4f, 0xfa, 0x9b, 0xfd // sxtb al r13 r11 ROR 8
};
const byte kInstruction_sxtb_al_r13_r11_ROR_16[] = {
  0x4f, 0xfa, 0xab, 0xfd // sxtb al r13 r11 ROR 16
};
const byte kInstruction_sxtb_al_r13_r11_ROR_24[] = {
  0x4f, 0xfa, 0xbb, 0xfd // sxtb al r13 r11 ROR 24
};
const byte kInstruction_sxtb_al_r13_r12_ROR_0[] = {
  0x4f, 0xfa, 0x8c, 0xfd // sxtb al r13 r12 ROR 0
};
const byte kInstruction_sxtb_al_r13_r12_ROR_8[] = {
  0x4f, 0xfa, 0x9c, 0xfd // sxtb al r13 r12 ROR 8
};
const byte kInstruction_sxtb_al_r13_r12_ROR_16[] = {
  0x4f, 0xfa, 0xac, 0xfd // sxtb al r13 r12 ROR 16
};
const byte kInstruction_sxtb_al_r13_r12_ROR_24[] = {
  0x4f, 0xfa, 0xbc, 0xfd // sxtb al r13 r12 ROR 24
};
const byte kInstruction_sxtb_al_r13_r13_ROR_0[] = {
  0x4f, 0xfa, 0x8d, 0xfd // sxtb al r13 r13 ROR 0
};
const byte kInstruction_sxtb_al_r13_r13_ROR_8[] = {
  0x4f, 0xfa, 0x9d, 0xfd // sxtb al r13 r13 ROR 8
};
const byte kInstruction_sxtb_al_r13_r13_ROR_16[] = {
  0x4f, 0xfa, 0xad, 0xfd // sxtb al r13 r13 ROR 16
};
const byte kInstruction_sxtb_al_r13_r13_ROR_24[] = {
  0x4f, 0xfa, 0xbd, 0xfd // sxtb al r13 r13 ROR 24
};
const byte kInstruction_sxtb_al_r13_r14_ROR_0[] = {
  0x4f, 0xfa, 0x8e, 0xfd // sxtb al r13 r14 ROR 0
};
const byte kInstruction_sxtb_al_r13_r14_ROR_8[] = {
  0x4f, 0xfa, 0x9e, 0xfd // sxtb al r13 r14 ROR 8
};
const byte kInstruction_sxtb_al_r13_r14_ROR_16[] = {
  0x4f, 0xfa, 0xae, 0xfd // sxtb al r13 r14 ROR 16
};
const byte kInstruction_sxtb_al_r13_r14_ROR_24[] = {
  0x4f, 0xfa, 0xbe, 0xfd // sxtb al r13 r14 ROR 24
};
const byte kInstruction_sxtb_al_r14_r0_ROR_0[] = {
  0x4f, 0xfa, 0x80, 0xfe // sxtb al r14 r0 ROR 0
};
const byte kInstruction_sxtb_al_r14_r0_ROR_8[] = {
  0x4f, 0xfa, 0x90, 0xfe // sxtb al r14 r0 ROR 8
};
const byte kInstruction_sxtb_al_r14_r0_ROR_16[] = {
  0x4f, 0xfa, 0xa0, 0xfe // sxtb al r14 r0 ROR 16
};
const byte kInstruction_sxtb_al_r14_r0_ROR_24[] = {
  0x4f, 0xfa, 0xb0, 0xfe // sxtb al r14 r0 ROR 24
};
const byte kInstruction_sxtb_al_r14_r1_ROR_0[] = {
  0x4f, 0xfa, 0x81, 0xfe // sxtb al r14 r1 ROR 0
};
const byte kInstruction_sxtb_al_r14_r1_ROR_8[] = {
  0x4f, 0xfa, 0x91, 0xfe // sxtb al r14 r1 ROR 8
};
const byte kInstruction_sxtb_al_r14_r1_ROR_16[] = {
  0x4f, 0xfa, 0xa1, 0xfe // sxtb al r14 r1 ROR 16
};
const byte kInstruction_sxtb_al_r14_r1_ROR_24[] = {
  0x4f, 0xfa, 0xb1, 0xfe // sxtb al r14 r1 ROR 24
};
const byte kInstruction_sxtb_al_r14_r2_ROR_0[] = {
  0x4f, 0xfa, 0x82, 0xfe // sxtb al r14 r2 ROR 0
};
const byte kInstruction_sxtb_al_r14_r2_ROR_8[] = {
  0x4f, 0xfa, 0x92, 0xfe // sxtb al r14 r2 ROR 8
};
const byte kInstruction_sxtb_al_r14_r2_ROR_16[] = {
  0x4f, 0xfa, 0xa2, 0xfe // sxtb al r14 r2 ROR 16
};
const byte kInstruction_sxtb_al_r14_r2_ROR_24[] = {
  0x4f, 0xfa, 0xb2, 0xfe // sxtb al r14 r2 ROR 24
};
const byte kInstruction_sxtb_al_r14_r3_ROR_0[] = {
  0x4f, 0xfa, 0x83, 0xfe // sxtb al r14 r3 ROR 0
};
const byte kInstruction_sxtb_al_r14_r3_ROR_8[] = {
  0x4f, 0xfa, 0x93, 0xfe // sxtb al r14 r3 ROR 8
};
const byte kInstruction_sxtb_al_r14_r3_ROR_16[] = {
  0x4f, 0xfa, 0xa3, 0xfe // sxtb al r14 r3 ROR 16
};
const byte kInstruction_sxtb_al_r14_r3_ROR_24[] = {
  0x4f, 0xfa, 0xb3, 0xfe // sxtb al r14 r3 ROR 24
};
const byte kInstruction_sxtb_al_r14_r4_ROR_0[] = {
  0x4f, 0xfa, 0x84, 0xfe // sxtb al r14 r4 ROR 0
};
const byte kInstruction_sxtb_al_r14_r4_ROR_8[] = {
  0x4f, 0xfa, 0x94, 0xfe // sxtb al r14 r4 ROR 8
};
const byte kInstruction_sxtb_al_r14_r4_ROR_16[] = {
  0x4f, 0xfa, 0xa4, 0xfe // sxtb al r14 r4 ROR 16
};
const byte kInstruction_sxtb_al_r14_r4_ROR_24[] = {
  0x4f, 0xfa, 0xb4, 0xfe // sxtb al r14 r4 ROR 24
};
const byte kInstruction_sxtb_al_r14_r5_ROR_0[] = {
  0x4f, 0xfa, 0x85, 0xfe // sxtb al r14 r5 ROR 0
};
const byte kInstruction_sxtb_al_r14_r5_ROR_8[] = {
  0x4f, 0xfa, 0x95, 0xfe // sxtb al r14 r5 ROR 8
};
const byte kInstruction_sxtb_al_r14_r5_ROR_16[] = {
  0x4f, 0xfa, 0xa5, 0xfe // sxtb al r14 r5 ROR 16
};
const byte kInstruction_sxtb_al_r14_r5_ROR_24[] = {
  0x4f, 0xfa, 0xb5, 0xfe // sxtb al r14 r5 ROR 24
};
const byte kInstruction_sxtb_al_r14_r6_ROR_0[] = {
  0x4f, 0xfa, 0x86, 0xfe // sxtb al r14 r6 ROR 0
};
const byte kInstruction_sxtb_al_r14_r6_ROR_8[] = {
  0x4f, 0xfa, 0x96, 0xfe // sxtb al r14 r6 ROR 8
};
const byte kInstruction_sxtb_al_r14_r6_ROR_16[] = {
  0x4f, 0xfa, 0xa6, 0xfe // sxtb al r14 r6 ROR 16
};
const byte kInstruction_sxtb_al_r14_r6_ROR_24[] = {
  0x4f, 0xfa, 0xb6, 0xfe // sxtb al r14 r6 ROR 24
};
const byte kInstruction_sxtb_al_r14_r7_ROR_0[] = {
  0x4f, 0xfa, 0x87, 0xfe // sxtb al r14 r7 ROR 0
};
const byte kInstruction_sxtb_al_r14_r7_ROR_8[] = {
  0x4f, 0xfa, 0x97, 0xfe // sxtb al r14 r7 ROR 8
};
const byte kInstruction_sxtb_al_r14_r7_ROR_16[] = {
  0x4f, 0xfa, 0xa7, 0xfe // sxtb al r14 r7 ROR 16
};
const byte kInstruction_sxtb_al_r14_r7_ROR_24[] = {
  0x4f, 0xfa, 0xb7, 0xfe // sxtb al r14 r7 ROR 24
};
const byte kInstruction_sxtb_al_r14_r8_ROR_0[] = {
  0x4f, 0xfa, 0x88, 0xfe // sxtb al r14 r8 ROR 0
};
const byte kInstruction_sxtb_al_r14_r8_ROR_8[] = {
  0x4f, 0xfa, 0x98, 0xfe // sxtb al r14 r8 ROR 8
};
const byte kInstruction_sxtb_al_r14_r8_ROR_16[] = {
  0x4f, 0xfa, 0xa8, 0xfe // sxtb al r14 r8 ROR 16
};
const byte kInstruction_sxtb_al_r14_r8_ROR_24[] = {
  0x4f, 0xfa, 0xb8, 0xfe // sxtb al r14 r8 ROR 24
};
const byte kInstruction_sxtb_al_r14_r9_ROR_0[] = {
  0x4f, 0xfa, 0x89, 0xfe // sxtb al r14 r9 ROR 0
};
const byte kInstruction_sxtb_al_r14_r9_ROR_8[] = {
  0x4f, 0xfa, 0x99, 0xfe // sxtb al r14 r9 ROR 8
};
const byte kInstruction_sxtb_al_r14_r9_ROR_16[] = {
  0x4f, 0xfa, 0xa9, 0xfe // sxtb al r14 r9 ROR 16
};
const byte kInstruction_sxtb_al_r14_r9_ROR_24[] = {
  0x4f, 0xfa, 0xb9, 0xfe // sxtb al r14 r9 ROR 24
};
const byte kInstruction_sxtb_al_r14_r10_ROR_0[] = {
  0x4f, 0xfa, 0x8a, 0xfe // sxtb al r14 r10 ROR 0
};
const byte kInstruction_sxtb_al_r14_r10_ROR_8[] = {
  0x4f, 0xfa, 0x9a, 0xfe // sxtb al r14 r10 ROR 8
};
const byte kInstruction_sxtb_al_r14_r10_ROR_16[] = {
  0x4f, 0xfa, 0xaa, 0xfe // sxtb al r14 r10 ROR 16
};
const byte kInstruction_sxtb_al_r14_r10_ROR_24[] = {
  0x4f, 0xfa, 0xba, 0xfe // sxtb al r14 r10 ROR 24
};
const byte kInstruction_sxtb_al_r14_r11_ROR_0[] = {
  0x4f, 0xfa, 0x8b, 0xfe // sxtb al r14 r11 ROR 0
};
const byte kInstruction_sxtb_al_r14_r11_ROR_8[] = {
  0x4f, 0xfa, 0x9b, 0xfe // sxtb al r14 r11 ROR 8
};
const byte kInstruction_sxtb_al_r14_r11_ROR_16[] = {
  0x4f, 0xfa, 0xab, 0xfe // sxtb al r14 r11 ROR 16
};
const byte kInstruction_sxtb_al_r14_r11_ROR_24[] = {
  0x4f, 0xfa, 0xbb, 0xfe // sxtb al r14 r11 ROR 24
};
const byte kInstruction_sxtb_al_r14_r12_ROR_0[] = {
  0x4f, 0xfa, 0x8c, 0xfe // sxtb al r14 r12 ROR 0
};
const byte kInstruction_sxtb_al_r14_r12_ROR_8[] = {
  0x4f, 0xfa, 0x9c, 0xfe // sxtb al r14 r12 ROR 8
};
const byte kInstruction_sxtb_al_r14_r12_ROR_16[] = {
  0x4f, 0xfa, 0xac, 0xfe // sxtb al r14 r12 ROR 16
};
const byte kInstruction_sxtb_al_r14_r12_ROR_24[] = {
  0x4f, 0xfa, 0xbc, 0xfe // sxtb al r14 r12 ROR 24
};
const byte kInstruction_sxtb_al_r14_r13_ROR_0[] = {
  0x4f, 0xfa, 0x8d, 0xfe // sxtb al r14 r13 ROR 0
};
const byte kInstruction_sxtb_al_r14_r13_ROR_8[] = {
  0x4f, 0xfa, 0x9d, 0xfe // sxtb al r14 r13 ROR 8
};
const byte kInstruction_sxtb_al_r14_r13_ROR_16[] = {
  0x4f, 0xfa, 0xad, 0xfe // sxtb al r14 r13 ROR 16
};
const byte kInstruction_sxtb_al_r14_r13_ROR_24[] = {
  0x4f, 0xfa, 0xbd, 0xfe // sxtb al r14 r13 ROR 24
};
const byte kInstruction_sxtb_al_r14_r14_ROR_0[] = {
  0x4f, 0xfa, 0x8e, 0xfe // sxtb al r14 r14 ROR 0
};
const byte kInstruction_sxtb_al_r14_r14_ROR_8[] = {
  0x4f, 0xfa, 0x9e, 0xfe // sxtb al r14 r14 ROR 8
};
const byte kInstruction_sxtb_al_r14_r14_ROR_16[] = {
  0x4f, 0xfa, 0xae, 0xfe // sxtb al r14 r14 ROR 16
};
const byte kInstruction_sxtb_al_r14_r14_ROR_24[] = {
  0x4f, 0xfa, 0xbe, 0xfe // sxtb al r14 r14 ROR 24
};
const TestResult kReferencesxtb[] = {
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r0_ROR_0),
    kInstruction_sxtb_al_r0_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r0_ROR_8),
    kInstruction_sxtb_al_r0_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r0_ROR_16),
    kInstruction_sxtb_al_r0_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r0_ROR_24),
    kInstruction_sxtb_al_r0_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r1_ROR_0),
    kInstruction_sxtb_al_r0_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r1_ROR_8),
    kInstruction_sxtb_al_r0_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r1_ROR_16),
    kInstruction_sxtb_al_r0_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r1_ROR_24),
    kInstruction_sxtb_al_r0_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r2_ROR_0),
    kInstruction_sxtb_al_r0_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r2_ROR_8),
    kInstruction_sxtb_al_r0_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r2_ROR_16),
    kInstruction_sxtb_al_r0_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r2_ROR_24),
    kInstruction_sxtb_al_r0_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r3_ROR_0),
    kInstruction_sxtb_al_r0_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r3_ROR_8),
    kInstruction_sxtb_al_r0_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r3_ROR_16),
    kInstruction_sxtb_al_r0_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r3_ROR_24),
    kInstruction_sxtb_al_r0_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r4_ROR_0),
    kInstruction_sxtb_al_r0_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r4_ROR_8),
    kInstruction_sxtb_al_r0_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r4_ROR_16),
    kInstruction_sxtb_al_r0_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r4_ROR_24),
    kInstruction_sxtb_al_r0_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r5_ROR_0),
    kInstruction_sxtb_al_r0_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r5_ROR_8),
    kInstruction_sxtb_al_r0_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r5_ROR_16),
    kInstruction_sxtb_al_r0_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r5_ROR_24),
    kInstruction_sxtb_al_r0_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r6_ROR_0),
    kInstruction_sxtb_al_r0_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r6_ROR_8),
    kInstruction_sxtb_al_r0_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r6_ROR_16),
    kInstruction_sxtb_al_r0_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r6_ROR_24),
    kInstruction_sxtb_al_r0_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r7_ROR_0),
    kInstruction_sxtb_al_r0_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r7_ROR_8),
    kInstruction_sxtb_al_r0_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r7_ROR_16),
    kInstruction_sxtb_al_r0_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r7_ROR_24),
    kInstruction_sxtb_al_r0_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r8_ROR_0),
    kInstruction_sxtb_al_r0_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r8_ROR_8),
    kInstruction_sxtb_al_r0_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r8_ROR_16),
    kInstruction_sxtb_al_r0_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r8_ROR_24),
    kInstruction_sxtb_al_r0_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r9_ROR_0),
    kInstruction_sxtb_al_r0_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r9_ROR_8),
    kInstruction_sxtb_al_r0_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r9_ROR_16),
    kInstruction_sxtb_al_r0_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r9_ROR_24),
    kInstruction_sxtb_al_r0_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r10_ROR_0),
    kInstruction_sxtb_al_r0_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r10_ROR_8),
    kInstruction_sxtb_al_r0_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r10_ROR_16),
    kInstruction_sxtb_al_r0_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r10_ROR_24),
    kInstruction_sxtb_al_r0_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r11_ROR_0),
    kInstruction_sxtb_al_r0_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r11_ROR_8),
    kInstruction_sxtb_al_r0_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r11_ROR_16),
    kInstruction_sxtb_al_r0_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r11_ROR_24),
    kInstruction_sxtb_al_r0_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r12_ROR_0),
    kInstruction_sxtb_al_r0_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r12_ROR_8),
    kInstruction_sxtb_al_r0_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r12_ROR_16),
    kInstruction_sxtb_al_r0_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r12_ROR_24),
    kInstruction_sxtb_al_r0_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r13_ROR_0),
    kInstruction_sxtb_al_r0_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r13_ROR_8),
    kInstruction_sxtb_al_r0_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r13_ROR_16),
    kInstruction_sxtb_al_r0_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r13_ROR_24),
    kInstruction_sxtb_al_r0_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r14_ROR_0),
    kInstruction_sxtb_al_r0_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r14_ROR_8),
    kInstruction_sxtb_al_r0_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r14_ROR_16),
    kInstruction_sxtb_al_r0_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r0_r14_ROR_24),
    kInstruction_sxtb_al_r0_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r0_ROR_0),
    kInstruction_sxtb_al_r1_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r0_ROR_8),
    kInstruction_sxtb_al_r1_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r0_ROR_16),
    kInstruction_sxtb_al_r1_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r0_ROR_24),
    kInstruction_sxtb_al_r1_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r1_ROR_0),
    kInstruction_sxtb_al_r1_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r1_ROR_8),
    kInstruction_sxtb_al_r1_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r1_ROR_16),
    kInstruction_sxtb_al_r1_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r1_ROR_24),
    kInstruction_sxtb_al_r1_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r2_ROR_0),
    kInstruction_sxtb_al_r1_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r2_ROR_8),
    kInstruction_sxtb_al_r1_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r2_ROR_16),
    kInstruction_sxtb_al_r1_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r2_ROR_24),
    kInstruction_sxtb_al_r1_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r3_ROR_0),
    kInstruction_sxtb_al_r1_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r3_ROR_8),
    kInstruction_sxtb_al_r1_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r3_ROR_16),
    kInstruction_sxtb_al_r1_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r3_ROR_24),
    kInstruction_sxtb_al_r1_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r4_ROR_0),
    kInstruction_sxtb_al_r1_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r4_ROR_8),
    kInstruction_sxtb_al_r1_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r4_ROR_16),
    kInstruction_sxtb_al_r1_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r4_ROR_24),
    kInstruction_sxtb_al_r1_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r5_ROR_0),
    kInstruction_sxtb_al_r1_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r5_ROR_8),
    kInstruction_sxtb_al_r1_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r5_ROR_16),
    kInstruction_sxtb_al_r1_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r5_ROR_24),
    kInstruction_sxtb_al_r1_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r6_ROR_0),
    kInstruction_sxtb_al_r1_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r6_ROR_8),
    kInstruction_sxtb_al_r1_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r6_ROR_16),
    kInstruction_sxtb_al_r1_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r6_ROR_24),
    kInstruction_sxtb_al_r1_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r7_ROR_0),
    kInstruction_sxtb_al_r1_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r7_ROR_8),
    kInstruction_sxtb_al_r1_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r7_ROR_16),
    kInstruction_sxtb_al_r1_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r7_ROR_24),
    kInstruction_sxtb_al_r1_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r8_ROR_0),
    kInstruction_sxtb_al_r1_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r8_ROR_8),
    kInstruction_sxtb_al_r1_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r8_ROR_16),
    kInstruction_sxtb_al_r1_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r8_ROR_24),
    kInstruction_sxtb_al_r1_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r9_ROR_0),
    kInstruction_sxtb_al_r1_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r9_ROR_8),
    kInstruction_sxtb_al_r1_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r9_ROR_16),
    kInstruction_sxtb_al_r1_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r9_ROR_24),
    kInstruction_sxtb_al_r1_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r10_ROR_0),
    kInstruction_sxtb_al_r1_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r10_ROR_8),
    kInstruction_sxtb_al_r1_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r10_ROR_16),
    kInstruction_sxtb_al_r1_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r10_ROR_24),
    kInstruction_sxtb_al_r1_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r11_ROR_0),
    kInstruction_sxtb_al_r1_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r11_ROR_8),
    kInstruction_sxtb_al_r1_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r11_ROR_16),
    kInstruction_sxtb_al_r1_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r11_ROR_24),
    kInstruction_sxtb_al_r1_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r12_ROR_0),
    kInstruction_sxtb_al_r1_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r12_ROR_8),
    kInstruction_sxtb_al_r1_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r12_ROR_16),
    kInstruction_sxtb_al_r1_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r12_ROR_24),
    kInstruction_sxtb_al_r1_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r13_ROR_0),
    kInstruction_sxtb_al_r1_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r13_ROR_8),
    kInstruction_sxtb_al_r1_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r13_ROR_16),
    kInstruction_sxtb_al_r1_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r13_ROR_24),
    kInstruction_sxtb_al_r1_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r14_ROR_0),
    kInstruction_sxtb_al_r1_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r14_ROR_8),
    kInstruction_sxtb_al_r1_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r14_ROR_16),
    kInstruction_sxtb_al_r1_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r1_r14_ROR_24),
    kInstruction_sxtb_al_r1_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r0_ROR_0),
    kInstruction_sxtb_al_r2_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r0_ROR_8),
    kInstruction_sxtb_al_r2_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r0_ROR_16),
    kInstruction_sxtb_al_r2_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r0_ROR_24),
    kInstruction_sxtb_al_r2_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r1_ROR_0),
    kInstruction_sxtb_al_r2_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r1_ROR_8),
    kInstruction_sxtb_al_r2_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r1_ROR_16),
    kInstruction_sxtb_al_r2_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r1_ROR_24),
    kInstruction_sxtb_al_r2_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r2_ROR_0),
    kInstruction_sxtb_al_r2_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r2_ROR_8),
    kInstruction_sxtb_al_r2_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r2_ROR_16),
    kInstruction_sxtb_al_r2_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r2_ROR_24),
    kInstruction_sxtb_al_r2_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r3_ROR_0),
    kInstruction_sxtb_al_r2_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r3_ROR_8),
    kInstruction_sxtb_al_r2_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r3_ROR_16),
    kInstruction_sxtb_al_r2_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r3_ROR_24),
    kInstruction_sxtb_al_r2_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r4_ROR_0),
    kInstruction_sxtb_al_r2_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r4_ROR_8),
    kInstruction_sxtb_al_r2_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r4_ROR_16),
    kInstruction_sxtb_al_r2_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r4_ROR_24),
    kInstruction_sxtb_al_r2_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r5_ROR_0),
    kInstruction_sxtb_al_r2_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r5_ROR_8),
    kInstruction_sxtb_al_r2_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r5_ROR_16),
    kInstruction_sxtb_al_r2_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r5_ROR_24),
    kInstruction_sxtb_al_r2_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r6_ROR_0),
    kInstruction_sxtb_al_r2_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r6_ROR_8),
    kInstruction_sxtb_al_r2_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r6_ROR_16),
    kInstruction_sxtb_al_r2_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r6_ROR_24),
    kInstruction_sxtb_al_r2_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r7_ROR_0),
    kInstruction_sxtb_al_r2_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r7_ROR_8),
    kInstruction_sxtb_al_r2_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r7_ROR_16),
    kInstruction_sxtb_al_r2_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r7_ROR_24),
    kInstruction_sxtb_al_r2_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r8_ROR_0),
    kInstruction_sxtb_al_r2_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r8_ROR_8),
    kInstruction_sxtb_al_r2_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r8_ROR_16),
    kInstruction_sxtb_al_r2_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r8_ROR_24),
    kInstruction_sxtb_al_r2_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r9_ROR_0),
    kInstruction_sxtb_al_r2_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r9_ROR_8),
    kInstruction_sxtb_al_r2_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r9_ROR_16),
    kInstruction_sxtb_al_r2_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r9_ROR_24),
    kInstruction_sxtb_al_r2_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r10_ROR_0),
    kInstruction_sxtb_al_r2_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r10_ROR_8),
    kInstruction_sxtb_al_r2_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r10_ROR_16),
    kInstruction_sxtb_al_r2_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r10_ROR_24),
    kInstruction_sxtb_al_r2_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r11_ROR_0),
    kInstruction_sxtb_al_r2_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r11_ROR_8),
    kInstruction_sxtb_al_r2_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r11_ROR_16),
    kInstruction_sxtb_al_r2_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r11_ROR_24),
    kInstruction_sxtb_al_r2_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r12_ROR_0),
    kInstruction_sxtb_al_r2_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r12_ROR_8),
    kInstruction_sxtb_al_r2_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r12_ROR_16),
    kInstruction_sxtb_al_r2_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r12_ROR_24),
    kInstruction_sxtb_al_r2_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r13_ROR_0),
    kInstruction_sxtb_al_r2_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r13_ROR_8),
    kInstruction_sxtb_al_r2_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r13_ROR_16),
    kInstruction_sxtb_al_r2_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r13_ROR_24),
    kInstruction_sxtb_al_r2_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r14_ROR_0),
    kInstruction_sxtb_al_r2_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r14_ROR_8),
    kInstruction_sxtb_al_r2_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r14_ROR_16),
    kInstruction_sxtb_al_r2_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r2_r14_ROR_24),
    kInstruction_sxtb_al_r2_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r0_ROR_0),
    kInstruction_sxtb_al_r3_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r0_ROR_8),
    kInstruction_sxtb_al_r3_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r0_ROR_16),
    kInstruction_sxtb_al_r3_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r0_ROR_24),
    kInstruction_sxtb_al_r3_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r1_ROR_0),
    kInstruction_sxtb_al_r3_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r1_ROR_8),
    kInstruction_sxtb_al_r3_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r1_ROR_16),
    kInstruction_sxtb_al_r3_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r1_ROR_24),
    kInstruction_sxtb_al_r3_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r2_ROR_0),
    kInstruction_sxtb_al_r3_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r2_ROR_8),
    kInstruction_sxtb_al_r3_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r2_ROR_16),
    kInstruction_sxtb_al_r3_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r2_ROR_24),
    kInstruction_sxtb_al_r3_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r3_ROR_0),
    kInstruction_sxtb_al_r3_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r3_ROR_8),
    kInstruction_sxtb_al_r3_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r3_ROR_16),
    kInstruction_sxtb_al_r3_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r3_ROR_24),
    kInstruction_sxtb_al_r3_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r4_ROR_0),
    kInstruction_sxtb_al_r3_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r4_ROR_8),
    kInstruction_sxtb_al_r3_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r4_ROR_16),
    kInstruction_sxtb_al_r3_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r4_ROR_24),
    kInstruction_sxtb_al_r3_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r5_ROR_0),
    kInstruction_sxtb_al_r3_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r5_ROR_8),
    kInstruction_sxtb_al_r3_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r5_ROR_16),
    kInstruction_sxtb_al_r3_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r5_ROR_24),
    kInstruction_sxtb_al_r3_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r6_ROR_0),
    kInstruction_sxtb_al_r3_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r6_ROR_8),
    kInstruction_sxtb_al_r3_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r6_ROR_16),
    kInstruction_sxtb_al_r3_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r6_ROR_24),
    kInstruction_sxtb_al_r3_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r7_ROR_0),
    kInstruction_sxtb_al_r3_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r7_ROR_8),
    kInstruction_sxtb_al_r3_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r7_ROR_16),
    kInstruction_sxtb_al_r3_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r7_ROR_24),
    kInstruction_sxtb_al_r3_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r8_ROR_0),
    kInstruction_sxtb_al_r3_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r8_ROR_8),
    kInstruction_sxtb_al_r3_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r8_ROR_16),
    kInstruction_sxtb_al_r3_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r8_ROR_24),
    kInstruction_sxtb_al_r3_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r9_ROR_0),
    kInstruction_sxtb_al_r3_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r9_ROR_8),
    kInstruction_sxtb_al_r3_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r9_ROR_16),
    kInstruction_sxtb_al_r3_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r9_ROR_24),
    kInstruction_sxtb_al_r3_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r10_ROR_0),
    kInstruction_sxtb_al_r3_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r10_ROR_8),
    kInstruction_sxtb_al_r3_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r10_ROR_16),
    kInstruction_sxtb_al_r3_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r10_ROR_24),
    kInstruction_sxtb_al_r3_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r11_ROR_0),
    kInstruction_sxtb_al_r3_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r11_ROR_8),
    kInstruction_sxtb_al_r3_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r11_ROR_16),
    kInstruction_sxtb_al_r3_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r11_ROR_24),
    kInstruction_sxtb_al_r3_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r12_ROR_0),
    kInstruction_sxtb_al_r3_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r12_ROR_8),
    kInstruction_sxtb_al_r3_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r12_ROR_16),
    kInstruction_sxtb_al_r3_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r12_ROR_24),
    kInstruction_sxtb_al_r3_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r13_ROR_0),
    kInstruction_sxtb_al_r3_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r13_ROR_8),
    kInstruction_sxtb_al_r3_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r13_ROR_16),
    kInstruction_sxtb_al_r3_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r13_ROR_24),
    kInstruction_sxtb_al_r3_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r14_ROR_0),
    kInstruction_sxtb_al_r3_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r14_ROR_8),
    kInstruction_sxtb_al_r3_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r14_ROR_16),
    kInstruction_sxtb_al_r3_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r3_r14_ROR_24),
    kInstruction_sxtb_al_r3_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r0_ROR_0),
    kInstruction_sxtb_al_r4_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r0_ROR_8),
    kInstruction_sxtb_al_r4_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r0_ROR_16),
    kInstruction_sxtb_al_r4_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r0_ROR_24),
    kInstruction_sxtb_al_r4_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r1_ROR_0),
    kInstruction_sxtb_al_r4_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r1_ROR_8),
    kInstruction_sxtb_al_r4_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r1_ROR_16),
    kInstruction_sxtb_al_r4_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r1_ROR_24),
    kInstruction_sxtb_al_r4_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r2_ROR_0),
    kInstruction_sxtb_al_r4_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r2_ROR_8),
    kInstruction_sxtb_al_r4_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r2_ROR_16),
    kInstruction_sxtb_al_r4_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r2_ROR_24),
    kInstruction_sxtb_al_r4_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r3_ROR_0),
    kInstruction_sxtb_al_r4_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r3_ROR_8),
    kInstruction_sxtb_al_r4_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r3_ROR_16),
    kInstruction_sxtb_al_r4_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r3_ROR_24),
    kInstruction_sxtb_al_r4_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r4_ROR_0),
    kInstruction_sxtb_al_r4_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r4_ROR_8),
    kInstruction_sxtb_al_r4_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r4_ROR_16),
    kInstruction_sxtb_al_r4_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r4_ROR_24),
    kInstruction_sxtb_al_r4_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r5_ROR_0),
    kInstruction_sxtb_al_r4_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r5_ROR_8),
    kInstruction_sxtb_al_r4_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r5_ROR_16),
    kInstruction_sxtb_al_r4_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r5_ROR_24),
    kInstruction_sxtb_al_r4_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r6_ROR_0),
    kInstruction_sxtb_al_r4_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r6_ROR_8),
    kInstruction_sxtb_al_r4_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r6_ROR_16),
    kInstruction_sxtb_al_r4_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r6_ROR_24),
    kInstruction_sxtb_al_r4_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r7_ROR_0),
    kInstruction_sxtb_al_r4_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r7_ROR_8),
    kInstruction_sxtb_al_r4_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r7_ROR_16),
    kInstruction_sxtb_al_r4_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r7_ROR_24),
    kInstruction_sxtb_al_r4_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r8_ROR_0),
    kInstruction_sxtb_al_r4_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r8_ROR_8),
    kInstruction_sxtb_al_r4_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r8_ROR_16),
    kInstruction_sxtb_al_r4_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r8_ROR_24),
    kInstruction_sxtb_al_r4_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r9_ROR_0),
    kInstruction_sxtb_al_r4_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r9_ROR_8),
    kInstruction_sxtb_al_r4_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r9_ROR_16),
    kInstruction_sxtb_al_r4_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r9_ROR_24),
    kInstruction_sxtb_al_r4_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r10_ROR_0),
    kInstruction_sxtb_al_r4_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r10_ROR_8),
    kInstruction_sxtb_al_r4_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r10_ROR_16),
    kInstruction_sxtb_al_r4_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r10_ROR_24),
    kInstruction_sxtb_al_r4_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r11_ROR_0),
    kInstruction_sxtb_al_r4_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r11_ROR_8),
    kInstruction_sxtb_al_r4_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r11_ROR_16),
    kInstruction_sxtb_al_r4_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r11_ROR_24),
    kInstruction_sxtb_al_r4_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r12_ROR_0),
    kInstruction_sxtb_al_r4_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r12_ROR_8),
    kInstruction_sxtb_al_r4_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r12_ROR_16),
    kInstruction_sxtb_al_r4_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r12_ROR_24),
    kInstruction_sxtb_al_r4_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r13_ROR_0),
    kInstruction_sxtb_al_r4_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r13_ROR_8),
    kInstruction_sxtb_al_r4_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r13_ROR_16),
    kInstruction_sxtb_al_r4_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r13_ROR_24),
    kInstruction_sxtb_al_r4_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r14_ROR_0),
    kInstruction_sxtb_al_r4_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r14_ROR_8),
    kInstruction_sxtb_al_r4_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r14_ROR_16),
    kInstruction_sxtb_al_r4_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r4_r14_ROR_24),
    kInstruction_sxtb_al_r4_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r0_ROR_0),
    kInstruction_sxtb_al_r5_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r0_ROR_8),
    kInstruction_sxtb_al_r5_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r0_ROR_16),
    kInstruction_sxtb_al_r5_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r0_ROR_24),
    kInstruction_sxtb_al_r5_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r1_ROR_0),
    kInstruction_sxtb_al_r5_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r1_ROR_8),
    kInstruction_sxtb_al_r5_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r1_ROR_16),
    kInstruction_sxtb_al_r5_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r1_ROR_24),
    kInstruction_sxtb_al_r5_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r2_ROR_0),
    kInstruction_sxtb_al_r5_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r2_ROR_8),
    kInstruction_sxtb_al_r5_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r2_ROR_16),
    kInstruction_sxtb_al_r5_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r2_ROR_24),
    kInstruction_sxtb_al_r5_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r3_ROR_0),
    kInstruction_sxtb_al_r5_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r3_ROR_8),
    kInstruction_sxtb_al_r5_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r3_ROR_16),
    kInstruction_sxtb_al_r5_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r3_ROR_24),
    kInstruction_sxtb_al_r5_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r4_ROR_0),
    kInstruction_sxtb_al_r5_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r4_ROR_8),
    kInstruction_sxtb_al_r5_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r4_ROR_16),
    kInstruction_sxtb_al_r5_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r4_ROR_24),
    kInstruction_sxtb_al_r5_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r5_ROR_0),
    kInstruction_sxtb_al_r5_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r5_ROR_8),
    kInstruction_sxtb_al_r5_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r5_ROR_16),
    kInstruction_sxtb_al_r5_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r5_ROR_24),
    kInstruction_sxtb_al_r5_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r6_ROR_0),
    kInstruction_sxtb_al_r5_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r6_ROR_8),
    kInstruction_sxtb_al_r5_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r6_ROR_16),
    kInstruction_sxtb_al_r5_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r6_ROR_24),
    kInstruction_sxtb_al_r5_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r7_ROR_0),
    kInstruction_sxtb_al_r5_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r7_ROR_8),
    kInstruction_sxtb_al_r5_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r7_ROR_16),
    kInstruction_sxtb_al_r5_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r7_ROR_24),
    kInstruction_sxtb_al_r5_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r8_ROR_0),
    kInstruction_sxtb_al_r5_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r8_ROR_8),
    kInstruction_sxtb_al_r5_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r8_ROR_16),
    kInstruction_sxtb_al_r5_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r8_ROR_24),
    kInstruction_sxtb_al_r5_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r9_ROR_0),
    kInstruction_sxtb_al_r5_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r9_ROR_8),
    kInstruction_sxtb_al_r5_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r9_ROR_16),
    kInstruction_sxtb_al_r5_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r9_ROR_24),
    kInstruction_sxtb_al_r5_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r10_ROR_0),
    kInstruction_sxtb_al_r5_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r10_ROR_8),
    kInstruction_sxtb_al_r5_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r10_ROR_16),
    kInstruction_sxtb_al_r5_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r10_ROR_24),
    kInstruction_sxtb_al_r5_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r11_ROR_0),
    kInstruction_sxtb_al_r5_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r11_ROR_8),
    kInstruction_sxtb_al_r5_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r11_ROR_16),
    kInstruction_sxtb_al_r5_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r11_ROR_24),
    kInstruction_sxtb_al_r5_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r12_ROR_0),
    kInstruction_sxtb_al_r5_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r12_ROR_8),
    kInstruction_sxtb_al_r5_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r12_ROR_16),
    kInstruction_sxtb_al_r5_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r12_ROR_24),
    kInstruction_sxtb_al_r5_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r13_ROR_0),
    kInstruction_sxtb_al_r5_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r13_ROR_8),
    kInstruction_sxtb_al_r5_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r13_ROR_16),
    kInstruction_sxtb_al_r5_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r13_ROR_24),
    kInstruction_sxtb_al_r5_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r14_ROR_0),
    kInstruction_sxtb_al_r5_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r14_ROR_8),
    kInstruction_sxtb_al_r5_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r14_ROR_16),
    kInstruction_sxtb_al_r5_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r5_r14_ROR_24),
    kInstruction_sxtb_al_r5_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r0_ROR_0),
    kInstruction_sxtb_al_r6_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r0_ROR_8),
    kInstruction_sxtb_al_r6_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r0_ROR_16),
    kInstruction_sxtb_al_r6_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r0_ROR_24),
    kInstruction_sxtb_al_r6_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r1_ROR_0),
    kInstruction_sxtb_al_r6_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r1_ROR_8),
    kInstruction_sxtb_al_r6_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r1_ROR_16),
    kInstruction_sxtb_al_r6_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r1_ROR_24),
    kInstruction_sxtb_al_r6_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r2_ROR_0),
    kInstruction_sxtb_al_r6_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r2_ROR_8),
    kInstruction_sxtb_al_r6_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r2_ROR_16),
    kInstruction_sxtb_al_r6_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r2_ROR_24),
    kInstruction_sxtb_al_r6_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r3_ROR_0),
    kInstruction_sxtb_al_r6_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r3_ROR_8),
    kInstruction_sxtb_al_r6_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r3_ROR_16),
    kInstruction_sxtb_al_r6_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r3_ROR_24),
    kInstruction_sxtb_al_r6_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r4_ROR_0),
    kInstruction_sxtb_al_r6_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r4_ROR_8),
    kInstruction_sxtb_al_r6_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r4_ROR_16),
    kInstruction_sxtb_al_r6_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r4_ROR_24),
    kInstruction_sxtb_al_r6_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r5_ROR_0),
    kInstruction_sxtb_al_r6_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r5_ROR_8),
    kInstruction_sxtb_al_r6_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r5_ROR_16),
    kInstruction_sxtb_al_r6_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r5_ROR_24),
    kInstruction_sxtb_al_r6_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r6_ROR_0),
    kInstruction_sxtb_al_r6_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r6_ROR_8),
    kInstruction_sxtb_al_r6_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r6_ROR_16),
    kInstruction_sxtb_al_r6_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r6_ROR_24),
    kInstruction_sxtb_al_r6_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r7_ROR_0),
    kInstruction_sxtb_al_r6_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r7_ROR_8),
    kInstruction_sxtb_al_r6_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r7_ROR_16),
    kInstruction_sxtb_al_r6_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r7_ROR_24),
    kInstruction_sxtb_al_r6_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r8_ROR_0),
    kInstruction_sxtb_al_r6_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r8_ROR_8),
    kInstruction_sxtb_al_r6_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r8_ROR_16),
    kInstruction_sxtb_al_r6_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r8_ROR_24),
    kInstruction_sxtb_al_r6_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r9_ROR_0),
    kInstruction_sxtb_al_r6_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r9_ROR_8),
    kInstruction_sxtb_al_r6_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r9_ROR_16),
    kInstruction_sxtb_al_r6_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r9_ROR_24),
    kInstruction_sxtb_al_r6_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r10_ROR_0),
    kInstruction_sxtb_al_r6_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r10_ROR_8),
    kInstruction_sxtb_al_r6_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r10_ROR_16),
    kInstruction_sxtb_al_r6_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r10_ROR_24),
    kInstruction_sxtb_al_r6_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r11_ROR_0),
    kInstruction_sxtb_al_r6_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r11_ROR_8),
    kInstruction_sxtb_al_r6_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r11_ROR_16),
    kInstruction_sxtb_al_r6_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r11_ROR_24),
    kInstruction_sxtb_al_r6_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r12_ROR_0),
    kInstruction_sxtb_al_r6_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r12_ROR_8),
    kInstruction_sxtb_al_r6_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r12_ROR_16),
    kInstruction_sxtb_al_r6_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r12_ROR_24),
    kInstruction_sxtb_al_r6_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r13_ROR_0),
    kInstruction_sxtb_al_r6_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r13_ROR_8),
    kInstruction_sxtb_al_r6_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r13_ROR_16),
    kInstruction_sxtb_al_r6_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r13_ROR_24),
    kInstruction_sxtb_al_r6_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r14_ROR_0),
    kInstruction_sxtb_al_r6_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r14_ROR_8),
    kInstruction_sxtb_al_r6_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r14_ROR_16),
    kInstruction_sxtb_al_r6_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r6_r14_ROR_24),
    kInstruction_sxtb_al_r6_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r0_ROR_0),
    kInstruction_sxtb_al_r7_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r0_ROR_8),
    kInstruction_sxtb_al_r7_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r0_ROR_16),
    kInstruction_sxtb_al_r7_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r0_ROR_24),
    kInstruction_sxtb_al_r7_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r1_ROR_0),
    kInstruction_sxtb_al_r7_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r1_ROR_8),
    kInstruction_sxtb_al_r7_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r1_ROR_16),
    kInstruction_sxtb_al_r7_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r1_ROR_24),
    kInstruction_sxtb_al_r7_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r2_ROR_0),
    kInstruction_sxtb_al_r7_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r2_ROR_8),
    kInstruction_sxtb_al_r7_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r2_ROR_16),
    kInstruction_sxtb_al_r7_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r2_ROR_24),
    kInstruction_sxtb_al_r7_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r3_ROR_0),
    kInstruction_sxtb_al_r7_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r3_ROR_8),
    kInstruction_sxtb_al_r7_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r3_ROR_16),
    kInstruction_sxtb_al_r7_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r3_ROR_24),
    kInstruction_sxtb_al_r7_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r4_ROR_0),
    kInstruction_sxtb_al_r7_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r4_ROR_8),
    kInstruction_sxtb_al_r7_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r4_ROR_16),
    kInstruction_sxtb_al_r7_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r4_ROR_24),
    kInstruction_sxtb_al_r7_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r5_ROR_0),
    kInstruction_sxtb_al_r7_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r5_ROR_8),
    kInstruction_sxtb_al_r7_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r5_ROR_16),
    kInstruction_sxtb_al_r7_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r5_ROR_24),
    kInstruction_sxtb_al_r7_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r6_ROR_0),
    kInstruction_sxtb_al_r7_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r6_ROR_8),
    kInstruction_sxtb_al_r7_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r6_ROR_16),
    kInstruction_sxtb_al_r7_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r6_ROR_24),
    kInstruction_sxtb_al_r7_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r7_ROR_0),
    kInstruction_sxtb_al_r7_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r7_ROR_8),
    kInstruction_sxtb_al_r7_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r7_ROR_16),
    kInstruction_sxtb_al_r7_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r7_ROR_24),
    kInstruction_sxtb_al_r7_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r8_ROR_0),
    kInstruction_sxtb_al_r7_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r8_ROR_8),
    kInstruction_sxtb_al_r7_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r8_ROR_16),
    kInstruction_sxtb_al_r7_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r8_ROR_24),
    kInstruction_sxtb_al_r7_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r9_ROR_0),
    kInstruction_sxtb_al_r7_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r9_ROR_8),
    kInstruction_sxtb_al_r7_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r9_ROR_16),
    kInstruction_sxtb_al_r7_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r9_ROR_24),
    kInstruction_sxtb_al_r7_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r10_ROR_0),
    kInstruction_sxtb_al_r7_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r10_ROR_8),
    kInstruction_sxtb_al_r7_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r10_ROR_16),
    kInstruction_sxtb_al_r7_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r10_ROR_24),
    kInstruction_sxtb_al_r7_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r11_ROR_0),
    kInstruction_sxtb_al_r7_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r11_ROR_8),
    kInstruction_sxtb_al_r7_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r11_ROR_16),
    kInstruction_sxtb_al_r7_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r11_ROR_24),
    kInstruction_sxtb_al_r7_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r12_ROR_0),
    kInstruction_sxtb_al_r7_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r12_ROR_8),
    kInstruction_sxtb_al_r7_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r12_ROR_16),
    kInstruction_sxtb_al_r7_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r12_ROR_24),
    kInstruction_sxtb_al_r7_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r13_ROR_0),
    kInstruction_sxtb_al_r7_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r13_ROR_8),
    kInstruction_sxtb_al_r7_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r13_ROR_16),
    kInstruction_sxtb_al_r7_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r13_ROR_24),
    kInstruction_sxtb_al_r7_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r14_ROR_0),
    kInstruction_sxtb_al_r7_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r14_ROR_8),
    kInstruction_sxtb_al_r7_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r14_ROR_16),
    kInstruction_sxtb_al_r7_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r7_r14_ROR_24),
    kInstruction_sxtb_al_r7_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r0_ROR_0),
    kInstruction_sxtb_al_r8_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r0_ROR_8),
    kInstruction_sxtb_al_r8_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r0_ROR_16),
    kInstruction_sxtb_al_r8_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r0_ROR_24),
    kInstruction_sxtb_al_r8_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r1_ROR_0),
    kInstruction_sxtb_al_r8_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r1_ROR_8),
    kInstruction_sxtb_al_r8_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r1_ROR_16),
    kInstruction_sxtb_al_r8_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r1_ROR_24),
    kInstruction_sxtb_al_r8_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r2_ROR_0),
    kInstruction_sxtb_al_r8_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r2_ROR_8),
    kInstruction_sxtb_al_r8_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r2_ROR_16),
    kInstruction_sxtb_al_r8_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r2_ROR_24),
    kInstruction_sxtb_al_r8_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r3_ROR_0),
    kInstruction_sxtb_al_r8_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r3_ROR_8),
    kInstruction_sxtb_al_r8_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r3_ROR_16),
    kInstruction_sxtb_al_r8_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r3_ROR_24),
    kInstruction_sxtb_al_r8_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r4_ROR_0),
    kInstruction_sxtb_al_r8_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r4_ROR_8),
    kInstruction_sxtb_al_r8_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r4_ROR_16),
    kInstruction_sxtb_al_r8_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r4_ROR_24),
    kInstruction_sxtb_al_r8_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r5_ROR_0),
    kInstruction_sxtb_al_r8_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r5_ROR_8),
    kInstruction_sxtb_al_r8_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r5_ROR_16),
    kInstruction_sxtb_al_r8_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r5_ROR_24),
    kInstruction_sxtb_al_r8_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r6_ROR_0),
    kInstruction_sxtb_al_r8_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r6_ROR_8),
    kInstruction_sxtb_al_r8_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r6_ROR_16),
    kInstruction_sxtb_al_r8_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r6_ROR_24),
    kInstruction_sxtb_al_r8_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r7_ROR_0),
    kInstruction_sxtb_al_r8_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r7_ROR_8),
    kInstruction_sxtb_al_r8_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r7_ROR_16),
    kInstruction_sxtb_al_r8_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r7_ROR_24),
    kInstruction_sxtb_al_r8_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r8_ROR_0),
    kInstruction_sxtb_al_r8_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r8_ROR_8),
    kInstruction_sxtb_al_r8_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r8_ROR_16),
    kInstruction_sxtb_al_r8_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r8_ROR_24),
    kInstruction_sxtb_al_r8_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r9_ROR_0),
    kInstruction_sxtb_al_r8_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r9_ROR_8),
    kInstruction_sxtb_al_r8_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r9_ROR_16),
    kInstruction_sxtb_al_r8_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r9_ROR_24),
    kInstruction_sxtb_al_r8_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r10_ROR_0),
    kInstruction_sxtb_al_r8_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r10_ROR_8),
    kInstruction_sxtb_al_r8_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r10_ROR_16),
    kInstruction_sxtb_al_r8_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r10_ROR_24),
    kInstruction_sxtb_al_r8_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r11_ROR_0),
    kInstruction_sxtb_al_r8_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r11_ROR_8),
    kInstruction_sxtb_al_r8_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r11_ROR_16),
    kInstruction_sxtb_al_r8_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r11_ROR_24),
    kInstruction_sxtb_al_r8_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r12_ROR_0),
    kInstruction_sxtb_al_r8_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r12_ROR_8),
    kInstruction_sxtb_al_r8_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r12_ROR_16),
    kInstruction_sxtb_al_r8_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r12_ROR_24),
    kInstruction_sxtb_al_r8_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r13_ROR_0),
    kInstruction_sxtb_al_r8_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r13_ROR_8),
    kInstruction_sxtb_al_r8_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r13_ROR_16),
    kInstruction_sxtb_al_r8_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r13_ROR_24),
    kInstruction_sxtb_al_r8_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r14_ROR_0),
    kInstruction_sxtb_al_r8_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r14_ROR_8),
    kInstruction_sxtb_al_r8_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r14_ROR_16),
    kInstruction_sxtb_al_r8_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r8_r14_ROR_24),
    kInstruction_sxtb_al_r8_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r0_ROR_0),
    kInstruction_sxtb_al_r9_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r0_ROR_8),
    kInstruction_sxtb_al_r9_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r0_ROR_16),
    kInstruction_sxtb_al_r9_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r0_ROR_24),
    kInstruction_sxtb_al_r9_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r1_ROR_0),
    kInstruction_sxtb_al_r9_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r1_ROR_8),
    kInstruction_sxtb_al_r9_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r1_ROR_16),
    kInstruction_sxtb_al_r9_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r1_ROR_24),
    kInstruction_sxtb_al_r9_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r2_ROR_0),
    kInstruction_sxtb_al_r9_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r2_ROR_8),
    kInstruction_sxtb_al_r9_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r2_ROR_16),
    kInstruction_sxtb_al_r9_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r2_ROR_24),
    kInstruction_sxtb_al_r9_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r3_ROR_0),
    kInstruction_sxtb_al_r9_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r3_ROR_8),
    kInstruction_sxtb_al_r9_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r3_ROR_16),
    kInstruction_sxtb_al_r9_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r3_ROR_24),
    kInstruction_sxtb_al_r9_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r4_ROR_0),
    kInstruction_sxtb_al_r9_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r4_ROR_8),
    kInstruction_sxtb_al_r9_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r4_ROR_16),
    kInstruction_sxtb_al_r9_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r4_ROR_24),
    kInstruction_sxtb_al_r9_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r5_ROR_0),
    kInstruction_sxtb_al_r9_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r5_ROR_8),
    kInstruction_sxtb_al_r9_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r5_ROR_16),
    kInstruction_sxtb_al_r9_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r5_ROR_24),
    kInstruction_sxtb_al_r9_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r6_ROR_0),
    kInstruction_sxtb_al_r9_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r6_ROR_8),
    kInstruction_sxtb_al_r9_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r6_ROR_16),
    kInstruction_sxtb_al_r9_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r6_ROR_24),
    kInstruction_sxtb_al_r9_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r7_ROR_0),
    kInstruction_sxtb_al_r9_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r7_ROR_8),
    kInstruction_sxtb_al_r9_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r7_ROR_16),
    kInstruction_sxtb_al_r9_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r7_ROR_24),
    kInstruction_sxtb_al_r9_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r8_ROR_0),
    kInstruction_sxtb_al_r9_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r8_ROR_8),
    kInstruction_sxtb_al_r9_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r8_ROR_16),
    kInstruction_sxtb_al_r9_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r8_ROR_24),
    kInstruction_sxtb_al_r9_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r9_ROR_0),
    kInstruction_sxtb_al_r9_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r9_ROR_8),
    kInstruction_sxtb_al_r9_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r9_ROR_16),
    kInstruction_sxtb_al_r9_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r9_ROR_24),
    kInstruction_sxtb_al_r9_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r10_ROR_0),
    kInstruction_sxtb_al_r9_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r10_ROR_8),
    kInstruction_sxtb_al_r9_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r10_ROR_16),
    kInstruction_sxtb_al_r9_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r10_ROR_24),
    kInstruction_sxtb_al_r9_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r11_ROR_0),
    kInstruction_sxtb_al_r9_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r11_ROR_8),
    kInstruction_sxtb_al_r9_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r11_ROR_16),
    kInstruction_sxtb_al_r9_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r11_ROR_24),
    kInstruction_sxtb_al_r9_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r12_ROR_0),
    kInstruction_sxtb_al_r9_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r12_ROR_8),
    kInstruction_sxtb_al_r9_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r12_ROR_16),
    kInstruction_sxtb_al_r9_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r12_ROR_24),
    kInstruction_sxtb_al_r9_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r13_ROR_0),
    kInstruction_sxtb_al_r9_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r13_ROR_8),
    kInstruction_sxtb_al_r9_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r13_ROR_16),
    kInstruction_sxtb_al_r9_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r13_ROR_24),
    kInstruction_sxtb_al_r9_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r14_ROR_0),
    kInstruction_sxtb_al_r9_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r14_ROR_8),
    kInstruction_sxtb_al_r9_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r14_ROR_16),
    kInstruction_sxtb_al_r9_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r9_r14_ROR_24),
    kInstruction_sxtb_al_r9_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r0_ROR_0),
    kInstruction_sxtb_al_r10_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r0_ROR_8),
    kInstruction_sxtb_al_r10_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r0_ROR_16),
    kInstruction_sxtb_al_r10_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r0_ROR_24),
    kInstruction_sxtb_al_r10_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r1_ROR_0),
    kInstruction_sxtb_al_r10_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r1_ROR_8),
    kInstruction_sxtb_al_r10_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r1_ROR_16),
    kInstruction_sxtb_al_r10_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r1_ROR_24),
    kInstruction_sxtb_al_r10_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r2_ROR_0),
    kInstruction_sxtb_al_r10_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r2_ROR_8),
    kInstruction_sxtb_al_r10_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r2_ROR_16),
    kInstruction_sxtb_al_r10_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r2_ROR_24),
    kInstruction_sxtb_al_r10_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r3_ROR_0),
    kInstruction_sxtb_al_r10_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r3_ROR_8),
    kInstruction_sxtb_al_r10_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r3_ROR_16),
    kInstruction_sxtb_al_r10_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r3_ROR_24),
    kInstruction_sxtb_al_r10_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r4_ROR_0),
    kInstruction_sxtb_al_r10_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r4_ROR_8),
    kInstruction_sxtb_al_r10_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r4_ROR_16),
    kInstruction_sxtb_al_r10_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r4_ROR_24),
    kInstruction_sxtb_al_r10_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r5_ROR_0),
    kInstruction_sxtb_al_r10_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r5_ROR_8),
    kInstruction_sxtb_al_r10_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r5_ROR_16),
    kInstruction_sxtb_al_r10_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r5_ROR_24),
    kInstruction_sxtb_al_r10_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r6_ROR_0),
    kInstruction_sxtb_al_r10_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r6_ROR_8),
    kInstruction_sxtb_al_r10_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r6_ROR_16),
    kInstruction_sxtb_al_r10_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r6_ROR_24),
    kInstruction_sxtb_al_r10_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r7_ROR_0),
    kInstruction_sxtb_al_r10_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r7_ROR_8),
    kInstruction_sxtb_al_r10_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r7_ROR_16),
    kInstruction_sxtb_al_r10_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r7_ROR_24),
    kInstruction_sxtb_al_r10_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r8_ROR_0),
    kInstruction_sxtb_al_r10_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r8_ROR_8),
    kInstruction_sxtb_al_r10_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r8_ROR_16),
    kInstruction_sxtb_al_r10_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r8_ROR_24),
    kInstruction_sxtb_al_r10_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r9_ROR_0),
    kInstruction_sxtb_al_r10_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r9_ROR_8),
    kInstruction_sxtb_al_r10_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r9_ROR_16),
    kInstruction_sxtb_al_r10_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r9_ROR_24),
    kInstruction_sxtb_al_r10_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r10_ROR_0),
    kInstruction_sxtb_al_r10_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r10_ROR_8),
    kInstruction_sxtb_al_r10_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r10_ROR_16),
    kInstruction_sxtb_al_r10_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r10_ROR_24),
    kInstruction_sxtb_al_r10_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r11_ROR_0),
    kInstruction_sxtb_al_r10_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r11_ROR_8),
    kInstruction_sxtb_al_r10_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r11_ROR_16),
    kInstruction_sxtb_al_r10_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r11_ROR_24),
    kInstruction_sxtb_al_r10_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r12_ROR_0),
    kInstruction_sxtb_al_r10_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r12_ROR_8),
    kInstruction_sxtb_al_r10_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r12_ROR_16),
    kInstruction_sxtb_al_r10_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r12_ROR_24),
    kInstruction_sxtb_al_r10_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r13_ROR_0),
    kInstruction_sxtb_al_r10_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r13_ROR_8),
    kInstruction_sxtb_al_r10_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r13_ROR_16),
    kInstruction_sxtb_al_r10_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r13_ROR_24),
    kInstruction_sxtb_al_r10_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r14_ROR_0),
    kInstruction_sxtb_al_r10_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r14_ROR_8),
    kInstruction_sxtb_al_r10_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r14_ROR_16),
    kInstruction_sxtb_al_r10_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r10_r14_ROR_24),
    kInstruction_sxtb_al_r10_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r0_ROR_0),
    kInstruction_sxtb_al_r11_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r0_ROR_8),
    kInstruction_sxtb_al_r11_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r0_ROR_16),
    kInstruction_sxtb_al_r11_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r0_ROR_24),
    kInstruction_sxtb_al_r11_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r1_ROR_0),
    kInstruction_sxtb_al_r11_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r1_ROR_8),
    kInstruction_sxtb_al_r11_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r1_ROR_16),
    kInstruction_sxtb_al_r11_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r1_ROR_24),
    kInstruction_sxtb_al_r11_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r2_ROR_0),
    kInstruction_sxtb_al_r11_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r2_ROR_8),
    kInstruction_sxtb_al_r11_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r2_ROR_16),
    kInstruction_sxtb_al_r11_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r2_ROR_24),
    kInstruction_sxtb_al_r11_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r3_ROR_0),
    kInstruction_sxtb_al_r11_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r3_ROR_8),
    kInstruction_sxtb_al_r11_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r3_ROR_16),
    kInstruction_sxtb_al_r11_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r3_ROR_24),
    kInstruction_sxtb_al_r11_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r4_ROR_0),
    kInstruction_sxtb_al_r11_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r4_ROR_8),
    kInstruction_sxtb_al_r11_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r4_ROR_16),
    kInstruction_sxtb_al_r11_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r4_ROR_24),
    kInstruction_sxtb_al_r11_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r5_ROR_0),
    kInstruction_sxtb_al_r11_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r5_ROR_8),
    kInstruction_sxtb_al_r11_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r5_ROR_16),
    kInstruction_sxtb_al_r11_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r5_ROR_24),
    kInstruction_sxtb_al_r11_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r6_ROR_0),
    kInstruction_sxtb_al_r11_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r6_ROR_8),
    kInstruction_sxtb_al_r11_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r6_ROR_16),
    kInstruction_sxtb_al_r11_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r6_ROR_24),
    kInstruction_sxtb_al_r11_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r7_ROR_0),
    kInstruction_sxtb_al_r11_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r7_ROR_8),
    kInstruction_sxtb_al_r11_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r7_ROR_16),
    kInstruction_sxtb_al_r11_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r7_ROR_24),
    kInstruction_sxtb_al_r11_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r8_ROR_0),
    kInstruction_sxtb_al_r11_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r8_ROR_8),
    kInstruction_sxtb_al_r11_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r8_ROR_16),
    kInstruction_sxtb_al_r11_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r8_ROR_24),
    kInstruction_sxtb_al_r11_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r9_ROR_0),
    kInstruction_sxtb_al_r11_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r9_ROR_8),
    kInstruction_sxtb_al_r11_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r9_ROR_16),
    kInstruction_sxtb_al_r11_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r9_ROR_24),
    kInstruction_sxtb_al_r11_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r10_ROR_0),
    kInstruction_sxtb_al_r11_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r10_ROR_8),
    kInstruction_sxtb_al_r11_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r10_ROR_16),
    kInstruction_sxtb_al_r11_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r10_ROR_24),
    kInstruction_sxtb_al_r11_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r11_ROR_0),
    kInstruction_sxtb_al_r11_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r11_ROR_8),
    kInstruction_sxtb_al_r11_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r11_ROR_16),
    kInstruction_sxtb_al_r11_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r11_ROR_24),
    kInstruction_sxtb_al_r11_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r12_ROR_0),
    kInstruction_sxtb_al_r11_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r12_ROR_8),
    kInstruction_sxtb_al_r11_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r12_ROR_16),
    kInstruction_sxtb_al_r11_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r12_ROR_24),
    kInstruction_sxtb_al_r11_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r13_ROR_0),
    kInstruction_sxtb_al_r11_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r13_ROR_8),
    kInstruction_sxtb_al_r11_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r13_ROR_16),
    kInstruction_sxtb_al_r11_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r13_ROR_24),
    kInstruction_sxtb_al_r11_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r14_ROR_0),
    kInstruction_sxtb_al_r11_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r14_ROR_8),
    kInstruction_sxtb_al_r11_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r14_ROR_16),
    kInstruction_sxtb_al_r11_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r11_r14_ROR_24),
    kInstruction_sxtb_al_r11_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r0_ROR_0),
    kInstruction_sxtb_al_r12_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r0_ROR_8),
    kInstruction_sxtb_al_r12_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r0_ROR_16),
    kInstruction_sxtb_al_r12_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r0_ROR_24),
    kInstruction_sxtb_al_r12_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r1_ROR_0),
    kInstruction_sxtb_al_r12_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r1_ROR_8),
    kInstruction_sxtb_al_r12_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r1_ROR_16),
    kInstruction_sxtb_al_r12_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r1_ROR_24),
    kInstruction_sxtb_al_r12_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r2_ROR_0),
    kInstruction_sxtb_al_r12_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r2_ROR_8),
    kInstruction_sxtb_al_r12_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r2_ROR_16),
    kInstruction_sxtb_al_r12_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r2_ROR_24),
    kInstruction_sxtb_al_r12_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r3_ROR_0),
    kInstruction_sxtb_al_r12_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r3_ROR_8),
    kInstruction_sxtb_al_r12_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r3_ROR_16),
    kInstruction_sxtb_al_r12_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r3_ROR_24),
    kInstruction_sxtb_al_r12_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r4_ROR_0),
    kInstruction_sxtb_al_r12_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r4_ROR_8),
    kInstruction_sxtb_al_r12_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r4_ROR_16),
    kInstruction_sxtb_al_r12_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r4_ROR_24),
    kInstruction_sxtb_al_r12_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r5_ROR_0),
    kInstruction_sxtb_al_r12_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r5_ROR_8),
    kInstruction_sxtb_al_r12_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r5_ROR_16),
    kInstruction_sxtb_al_r12_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r5_ROR_24),
    kInstruction_sxtb_al_r12_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r6_ROR_0),
    kInstruction_sxtb_al_r12_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r6_ROR_8),
    kInstruction_sxtb_al_r12_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r6_ROR_16),
    kInstruction_sxtb_al_r12_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r6_ROR_24),
    kInstruction_sxtb_al_r12_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r7_ROR_0),
    kInstruction_sxtb_al_r12_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r7_ROR_8),
    kInstruction_sxtb_al_r12_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r7_ROR_16),
    kInstruction_sxtb_al_r12_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r7_ROR_24),
    kInstruction_sxtb_al_r12_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r8_ROR_0),
    kInstruction_sxtb_al_r12_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r8_ROR_8),
    kInstruction_sxtb_al_r12_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r8_ROR_16),
    kInstruction_sxtb_al_r12_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r8_ROR_24),
    kInstruction_sxtb_al_r12_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r9_ROR_0),
    kInstruction_sxtb_al_r12_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r9_ROR_8),
    kInstruction_sxtb_al_r12_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r9_ROR_16),
    kInstruction_sxtb_al_r12_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r9_ROR_24),
    kInstruction_sxtb_al_r12_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r10_ROR_0),
    kInstruction_sxtb_al_r12_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r10_ROR_8),
    kInstruction_sxtb_al_r12_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r10_ROR_16),
    kInstruction_sxtb_al_r12_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r10_ROR_24),
    kInstruction_sxtb_al_r12_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r11_ROR_0),
    kInstruction_sxtb_al_r12_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r11_ROR_8),
    kInstruction_sxtb_al_r12_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r11_ROR_16),
    kInstruction_sxtb_al_r12_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r11_ROR_24),
    kInstruction_sxtb_al_r12_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r12_ROR_0),
    kInstruction_sxtb_al_r12_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r12_ROR_8),
    kInstruction_sxtb_al_r12_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r12_ROR_16),
    kInstruction_sxtb_al_r12_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r12_ROR_24),
    kInstruction_sxtb_al_r12_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r13_ROR_0),
    kInstruction_sxtb_al_r12_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r13_ROR_8),
    kInstruction_sxtb_al_r12_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r13_ROR_16),
    kInstruction_sxtb_al_r12_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r13_ROR_24),
    kInstruction_sxtb_al_r12_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r14_ROR_0),
    kInstruction_sxtb_al_r12_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r14_ROR_8),
    kInstruction_sxtb_al_r12_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r14_ROR_16),
    kInstruction_sxtb_al_r12_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r12_r14_ROR_24),
    kInstruction_sxtb_al_r12_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r0_ROR_0),
    kInstruction_sxtb_al_r13_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r0_ROR_8),
    kInstruction_sxtb_al_r13_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r0_ROR_16),
    kInstruction_sxtb_al_r13_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r0_ROR_24),
    kInstruction_sxtb_al_r13_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r1_ROR_0),
    kInstruction_sxtb_al_r13_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r1_ROR_8),
    kInstruction_sxtb_al_r13_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r1_ROR_16),
    kInstruction_sxtb_al_r13_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r1_ROR_24),
    kInstruction_sxtb_al_r13_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r2_ROR_0),
    kInstruction_sxtb_al_r13_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r2_ROR_8),
    kInstruction_sxtb_al_r13_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r2_ROR_16),
    kInstruction_sxtb_al_r13_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r2_ROR_24),
    kInstruction_sxtb_al_r13_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r3_ROR_0),
    kInstruction_sxtb_al_r13_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r3_ROR_8),
    kInstruction_sxtb_al_r13_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r3_ROR_16),
    kInstruction_sxtb_al_r13_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r3_ROR_24),
    kInstruction_sxtb_al_r13_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r4_ROR_0),
    kInstruction_sxtb_al_r13_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r4_ROR_8),
    kInstruction_sxtb_al_r13_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r4_ROR_16),
    kInstruction_sxtb_al_r13_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r4_ROR_24),
    kInstruction_sxtb_al_r13_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r5_ROR_0),
    kInstruction_sxtb_al_r13_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r5_ROR_8),
    kInstruction_sxtb_al_r13_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r5_ROR_16),
    kInstruction_sxtb_al_r13_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r5_ROR_24),
    kInstruction_sxtb_al_r13_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r6_ROR_0),
    kInstruction_sxtb_al_r13_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r6_ROR_8),
    kInstruction_sxtb_al_r13_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r6_ROR_16),
    kInstruction_sxtb_al_r13_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r6_ROR_24),
    kInstruction_sxtb_al_r13_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r7_ROR_0),
    kInstruction_sxtb_al_r13_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r7_ROR_8),
    kInstruction_sxtb_al_r13_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r7_ROR_16),
    kInstruction_sxtb_al_r13_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r7_ROR_24),
    kInstruction_sxtb_al_r13_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r8_ROR_0),
    kInstruction_sxtb_al_r13_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r8_ROR_8),
    kInstruction_sxtb_al_r13_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r8_ROR_16),
    kInstruction_sxtb_al_r13_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r8_ROR_24),
    kInstruction_sxtb_al_r13_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r9_ROR_0),
    kInstruction_sxtb_al_r13_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r9_ROR_8),
    kInstruction_sxtb_al_r13_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r9_ROR_16),
    kInstruction_sxtb_al_r13_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r9_ROR_24),
    kInstruction_sxtb_al_r13_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r10_ROR_0),
    kInstruction_sxtb_al_r13_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r10_ROR_8),
    kInstruction_sxtb_al_r13_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r10_ROR_16),
    kInstruction_sxtb_al_r13_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r10_ROR_24),
    kInstruction_sxtb_al_r13_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r11_ROR_0),
    kInstruction_sxtb_al_r13_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r11_ROR_8),
    kInstruction_sxtb_al_r13_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r11_ROR_16),
    kInstruction_sxtb_al_r13_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r11_ROR_24),
    kInstruction_sxtb_al_r13_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r12_ROR_0),
    kInstruction_sxtb_al_r13_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r12_ROR_8),
    kInstruction_sxtb_al_r13_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r12_ROR_16),
    kInstruction_sxtb_al_r13_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r12_ROR_24),
    kInstruction_sxtb_al_r13_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r13_ROR_0),
    kInstruction_sxtb_al_r13_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r13_ROR_8),
    kInstruction_sxtb_al_r13_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r13_ROR_16),
    kInstruction_sxtb_al_r13_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r13_ROR_24),
    kInstruction_sxtb_al_r13_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r14_ROR_0),
    kInstruction_sxtb_al_r13_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r14_ROR_8),
    kInstruction_sxtb_al_r13_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r14_ROR_16),
    kInstruction_sxtb_al_r13_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r13_r14_ROR_24),
    kInstruction_sxtb_al_r13_r14_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r0_ROR_0),
    kInstruction_sxtb_al_r14_r0_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r0_ROR_8),
    kInstruction_sxtb_al_r14_r0_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r0_ROR_16),
    kInstruction_sxtb_al_r14_r0_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r0_ROR_24),
    kInstruction_sxtb_al_r14_r0_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r1_ROR_0),
    kInstruction_sxtb_al_r14_r1_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r1_ROR_8),
    kInstruction_sxtb_al_r14_r1_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r1_ROR_16),
    kInstruction_sxtb_al_r14_r1_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r1_ROR_24),
    kInstruction_sxtb_al_r14_r1_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r2_ROR_0),
    kInstruction_sxtb_al_r14_r2_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r2_ROR_8),
    kInstruction_sxtb_al_r14_r2_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r2_ROR_16),
    kInstruction_sxtb_al_r14_r2_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r2_ROR_24),
    kInstruction_sxtb_al_r14_r2_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r3_ROR_0),
    kInstruction_sxtb_al_r14_r3_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r3_ROR_8),
    kInstruction_sxtb_al_r14_r3_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r3_ROR_16),
    kInstruction_sxtb_al_r14_r3_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r3_ROR_24),
    kInstruction_sxtb_al_r14_r3_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r4_ROR_0),
    kInstruction_sxtb_al_r14_r4_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r4_ROR_8),
    kInstruction_sxtb_al_r14_r4_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r4_ROR_16),
    kInstruction_sxtb_al_r14_r4_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r4_ROR_24),
    kInstruction_sxtb_al_r14_r4_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r5_ROR_0),
    kInstruction_sxtb_al_r14_r5_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r5_ROR_8),
    kInstruction_sxtb_al_r14_r5_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r5_ROR_16),
    kInstruction_sxtb_al_r14_r5_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r5_ROR_24),
    kInstruction_sxtb_al_r14_r5_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r6_ROR_0),
    kInstruction_sxtb_al_r14_r6_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r6_ROR_8),
    kInstruction_sxtb_al_r14_r6_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r6_ROR_16),
    kInstruction_sxtb_al_r14_r6_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r6_ROR_24),
    kInstruction_sxtb_al_r14_r6_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r7_ROR_0),
    kInstruction_sxtb_al_r14_r7_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r7_ROR_8),
    kInstruction_sxtb_al_r14_r7_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r7_ROR_16),
    kInstruction_sxtb_al_r14_r7_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r7_ROR_24),
    kInstruction_sxtb_al_r14_r7_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r8_ROR_0),
    kInstruction_sxtb_al_r14_r8_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r8_ROR_8),
    kInstruction_sxtb_al_r14_r8_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r8_ROR_16),
    kInstruction_sxtb_al_r14_r8_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r8_ROR_24),
    kInstruction_sxtb_al_r14_r8_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r9_ROR_0),
    kInstruction_sxtb_al_r14_r9_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r9_ROR_8),
    kInstruction_sxtb_al_r14_r9_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r9_ROR_16),
    kInstruction_sxtb_al_r14_r9_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r9_ROR_24),
    kInstruction_sxtb_al_r14_r9_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r10_ROR_0),
    kInstruction_sxtb_al_r14_r10_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r10_ROR_8),
    kInstruction_sxtb_al_r14_r10_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r10_ROR_16),
    kInstruction_sxtb_al_r14_r10_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r10_ROR_24),
    kInstruction_sxtb_al_r14_r10_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r11_ROR_0),
    kInstruction_sxtb_al_r14_r11_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r11_ROR_8),
    kInstruction_sxtb_al_r14_r11_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r11_ROR_16),
    kInstruction_sxtb_al_r14_r11_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r11_ROR_24),
    kInstruction_sxtb_al_r14_r11_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r12_ROR_0),
    kInstruction_sxtb_al_r14_r12_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r12_ROR_8),
    kInstruction_sxtb_al_r14_r12_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r12_ROR_16),
    kInstruction_sxtb_al_r14_r12_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r12_ROR_24),
    kInstruction_sxtb_al_r14_r12_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r13_ROR_0),
    kInstruction_sxtb_al_r14_r13_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r13_ROR_8),
    kInstruction_sxtb_al_r14_r13_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r13_ROR_16),
    kInstruction_sxtb_al_r14_r13_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r13_ROR_24),
    kInstruction_sxtb_al_r14_r13_ROR_24,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r14_ROR_0),
    kInstruction_sxtb_al_r14_r14_ROR_0,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r14_ROR_8),
    kInstruction_sxtb_al_r14_r14_ROR_8,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r14_ROR_16),
    kInstruction_sxtb_al_r14_r14_ROR_16,
  },
  {
    ARRAY_SIZE(kInstruction_sxtb_al_r14_r14_ROR_24),
    kInstruction_sxtb_al_r14_r14_ROR_24,
  },
};

#endif  // VIXL_ASSEMBLER_COND_RD_OPERAND_RN_ROR_AMOUNT_SXTB_T32_H_
