// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_ASSEMBLER_COND_RD_MEMOP_RS_SHIFT_AMOUNT_1TO31_STR_A32_H_
#define VIXL_ASSEMBLER_COND_RD_MEMOP_RS_SHIFT_AMOUNT_1TO31_STR_A32_H_

const byte kInstruction_str_pl_r8_r11_plus_r6_LSL_1_Offset[] = {
  0x86, 0x80, 0x8b, 0x57 // str pl r8 r11 plus r6 LSL 1 Offset
};
const byte kInstruction_str_le_r4_r8_plus_r5_LSL_1_Offset[] = {
  0x85, 0x40, 0x88, 0xd7 // str le r4 r8 plus r5 LSL 1 Offset
};
const byte kInstruction_str_vs_r2_r6_plus_r14_LSL_1_Offset[] = {
  0x8e, 0x20, 0x86, 0x67 // str vs r2 r6 plus r14 LSL 1 Offset
};
const byte kInstruction_str_ls_r1_r7_plus_r8_LSL_1_Offset[] = {
  0x88, 0x10, 0x87, 0x97 // str ls r1 r7 plus r8 LSL 1 Offset
};
const byte kInstruction_str_ge_r14_r6_plus_r14_LSL_1_Offset[] = {
  0x8e, 0xe0, 0x86, 0xa7 // str ge r14 r6 plus r14 LSL 1 Offset
};
const byte kInstruction_str_cs_r7_r0_plus_r7_LSL_1_Offset[] = {
  0x87, 0x70, 0x80, 0x27 // str cs r7 r0 plus r7 LSL 1 Offset
};
const byte kInstruction_str_ge_r11_r0_plus_r9_LSL_1_Offset[] = {
  0x89, 0xb0, 0x80, 0xa7 // str ge r11 r0 plus r9 LSL 1 Offset
};
const byte kInstruction_str_eq_r7_r10_plus_r4_LSL_1_Offset[] = {
  0x84, 0x70, 0x8a, 0x07 // str eq r7 r10 plus r4 LSL 1 Offset
};
const byte kInstruction_str_al_r9_r2_plus_r3_LSL_1_Offset[] = {
  0x83, 0x90, 0x82, 0xe7 // str al r9 r2 plus r3 LSL 1 Offset
};
const byte kInstruction_str_cc_r11_r10_plus_r6_LSL_1_Offset[] = {
  0x86, 0xb0, 0x8a, 0x37 // str cc r11 r10 plus r6 LSL 1 Offset
};
const byte kInstruction_str_lt_r9_r6_plus_r1_LSL_1_Offset[] = {
  0x81, 0x90, 0x86, 0xb7 // str lt r9 r6 plus r1 LSL 1 Offset
};
const byte kInstruction_str_ge_r4_r0_plus_r4_LSL_1_Offset[] = {
  0x84, 0x40, 0x80, 0xa7 // str ge r4 r0 plus r4 LSL 1 Offset
};
const byte kInstruction_str_al_r9_r13_plus_r14_LSL_1_Offset[] = {
  0x8e, 0x90, 0x8d, 0xe7 // str al r9 r13 plus r14 LSL 1 Offset
};
const byte kInstruction_str_cc_r9_r0_plus_r7_LSL_1_Offset[] = {
  0x87, 0x90, 0x80, 0x37 // str cc r9 r0 plus r7 LSL 1 Offset
};
const byte kInstruction_str_cs_r11_r7_plus_r8_LSL_1_Offset[] = {
  0x88, 0xb0, 0x87, 0x27 // str cs r11 r7 plus r8 LSL 1 Offset
};
const byte kInstruction_str_eq_r8_r0_plus_r8_LSL_1_Offset[] = {
  0x88, 0x80, 0x80, 0x07 // str eq r8 r0 plus r8 LSL 1 Offset
};
const byte kInstruction_str_hi_r9_r10_plus_r8_LSL_1_Offset[] = {
  0x88, 0x90, 0x8a, 0x87 // str hi r9 r10 plus r8 LSL 1 Offset
};
const byte kInstruction_str_le_r8_r0_plus_r0_LSL_1_Offset[] = {
  0x80, 0x80, 0x80, 0xd7 // str le r8 r0 plus r0 LSL 1 Offset
};
const byte kInstruction_str_cs_r4_r5_plus_r2_LSL_1_Offset[] = {
  0x82, 0x40, 0x85, 0x27 // str cs r4 r5 plus r2 LSL 1 Offset
};
const byte kInstruction_str_eq_r14_r11_plus_r8_LSL_1_Offset[] = {
  0x88, 0xe0, 0x8b, 0x07 // str eq r14 r11 plus r8 LSL 1 Offset
};
const byte kInstruction_str_cs_r7_r6_plus_r14_LSL_1_Offset[] = {
  0x8e, 0x70, 0x86, 0x27 // str cs r7 r6 plus r14 LSL 1 Offset
};
const byte kInstruction_str_ne_r0_r9_plus_r7_LSL_1_Offset[] = {
  0x87, 0x00, 0x89, 0x17 // str ne r0 r9 plus r7 LSL 1 Offset
};
const byte kInstruction_str_le_r7_r3_plus_r11_LSL_1_Offset[] = {
  0x8b, 0x70, 0x83, 0xd7 // str le r7 r3 plus r11 LSL 1 Offset
};
const byte kInstruction_str_gt_r7_r9_plus_r9_LSL_1_Offset[] = {
  0x89, 0x70, 0x89, 0xc7 // str gt r7 r9 plus r9 LSL 1 Offset
};
const byte kInstruction_str_le_r9_r12_plus_r9_LSL_1_Offset[] = {
  0x89, 0x90, 0x8c, 0xd7 // str le r9 r12 plus r9 LSL 1 Offset
};
const byte kInstruction_str_pl_r2_r11_plus_r14_LSL_1_Offset[] = {
  0x8e, 0x20, 0x8b, 0x57 // str pl r2 r11 plus r14 LSL 1 Offset
};
const byte kInstruction_str_cs_r1_r7_plus_r2_LSL_1_Offset[] = {
  0x82, 0x10, 0x87, 0x27 // str cs r1 r7 plus r2 LSL 1 Offset
};
const byte kInstruction_str_al_r5_r1_plus_r12_LSL_1_Offset[] = {
  0x8c, 0x50, 0x81, 0xe7 // str al r5 r1 plus r12 LSL 1 Offset
};
const byte kInstruction_str_eq_r10_r13_plus_r12_LSL_1_Offset[] = {
  0x8c, 0xa0, 0x8d, 0x07 // str eq r10 r13 plus r12 LSL 1 Offset
};
const byte kInstruction_str_eq_r10_r11_plus_r3_LSL_1_Offset[] = {
  0x83, 0xa0, 0x8b, 0x07 // str eq r10 r11 plus r3 LSL 1 Offset
};
const byte kInstruction_str_al_r9_r12_plus_r0_LSL_1_Offset[] = {
  0x80, 0x90, 0x8c, 0xe7 // str al r9 r12 plus r0 LSL 1 Offset
};
const byte kInstruction_str_ls_r3_r14_plus_r1_LSL_1_Offset[] = {
  0x81, 0x30, 0x8e, 0x97 // str ls r3 r14 plus r1 LSL 1 Offset
};
const byte kInstruction_str_le_r13_r4_plus_r9_LSL_1_Offset[] = {
  0x89, 0xd0, 0x84, 0xd7 // str le r13 r4 plus r9 LSL 1 Offset
};
const byte kInstruction_str_ls_r3_r14_plus_r3_LSL_1_Offset[] = {
  0x83, 0x30, 0x8e, 0x97 // str ls r3 r14 plus r3 LSL 1 Offset
};
const byte kInstruction_str_cc_r7_r4_plus_r4_LSL_1_Offset[] = {
  0x84, 0x70, 0x84, 0x37 // str cc r7 r4 plus r4 LSL 1 Offset
};
const byte kInstruction_str_ls_r5_r9_plus_r5_LSL_1_Offset[] = {
  0x85, 0x50, 0x89, 0x97 // str ls r5 r9 plus r5 LSL 1 Offset
};
const byte kInstruction_str_ge_r6_r4_plus_r13_LSL_1_Offset[] = {
  0x8d, 0x60, 0x84, 0xa7 // str ge r6 r4 plus r13 LSL 1 Offset
};
const byte kInstruction_str_al_r1_r11_plus_r12_LSL_1_Offset[] = {
  0x8c, 0x10, 0x8b, 0xe7 // str al r1 r11 plus r12 LSL 1 Offset
};
const byte kInstruction_str_le_r6_r11_plus_r3_LSL_1_Offset[] = {
  0x83, 0x60, 0x8b, 0xd7 // str le r6 r11 plus r3 LSL 1 Offset
};
const byte kInstruction_str_al_r6_r6_plus_r13_LSL_1_Offset[] = {
  0x8d, 0x60, 0x86, 0xe7 // str al r6 r6 plus r13 LSL 1 Offset
};
const byte kInstruction_str_gt_r14_r11_plus_r2_LSL_1_Offset[] = {
  0x82, 0xe0, 0x8b, 0xc7 // str gt r14 r11 plus r2 LSL 1 Offset
};
const byte kInstruction_str_vs_r5_r8_plus_r12_LSL_1_Offset[] = {
  0x8c, 0x50, 0x88, 0x67 // str vs r5 r8 plus r12 LSL 1 Offset
};
const byte kInstruction_str_ls_r8_r12_plus_r6_LSL_1_Offset[] = {
  0x86, 0x80, 0x8c, 0x97 // str ls r8 r12 plus r6 LSL 1 Offset
};
const byte kInstruction_str_vs_r6_r12_plus_r14_LSL_1_Offset[] = {
  0x8e, 0x60, 0x8c, 0x67 // str vs r6 r12 plus r14 LSL 1 Offset
};
const byte kInstruction_str_pl_r7_r2_plus_r7_LSL_1_Offset[] = {
  0x87, 0x70, 0x82, 0x57 // str pl r7 r2 plus r7 LSL 1 Offset
};
const byte kInstruction_str_gt_r5_r4_plus_r13_LSL_1_Offset[] = {
  0x8d, 0x50, 0x84, 0xc7 // str gt r5 r4 plus r13 LSL 1 Offset
};
const byte kInstruction_str_hi_r2_r7_plus_r12_LSL_1_Offset[] = {
  0x8c, 0x20, 0x87, 0x87 // str hi r2 r7 plus r12 LSL 1 Offset
};
const byte kInstruction_str_ne_r2_r3_plus_r10_LSL_1_Offset[] = {
  0x8a, 0x20, 0x83, 0x17 // str ne r2 r3 plus r10 LSL 1 Offset
};
const byte kInstruction_str_lt_r13_r2_plus_r6_LSL_1_Offset[] = {
  0x86, 0xd0, 0x82, 0xb7 // str lt r13 r2 plus r6 LSL 1 Offset
};
const byte kInstruction_str_hi_r14_r10_plus_r11_LSL_1_Offset[] = {
  0x8b, 0xe0, 0x8a, 0x87 // str hi r14 r10 plus r11 LSL 1 Offset
};
const byte kInstruction_str_hi_r7_r10_plus_r12_LSL_1_Offset[] = {
  0x8c, 0x70, 0x8a, 0x87 // str hi r7 r10 plus r12 LSL 1 Offset
};
const byte kInstruction_str_cs_r3_r12_plus_r2_LSL_1_Offset[] = {
  0x82, 0x30, 0x8c, 0x27 // str cs r3 r12 plus r2 LSL 1 Offset
};
const byte kInstruction_str_ls_r11_r5_plus_r1_LSL_1_Offset[] = {
  0x81, 0xb0, 0x85, 0x97 // str ls r11 r5 plus r1 LSL 1 Offset
};
const byte kInstruction_str_hi_r5_r2_plus_r11_LSL_1_Offset[] = {
  0x8b, 0x50, 0x82, 0x87 // str hi r5 r2 plus r11 LSL 1 Offset
};
const byte kInstruction_str_cc_r11_r8_plus_r6_LSL_1_Offset[] = {
  0x86, 0xb0, 0x88, 0x37 // str cc r11 r8 plus r6 LSL 1 Offset
};
const byte kInstruction_str_lt_r11_r3_plus_r9_LSL_1_Offset[] = {
  0x89, 0xb0, 0x83, 0xb7 // str lt r11 r3 plus r9 LSL 1 Offset
};
const byte kInstruction_str_ne_r11_r9_plus_r8_LSL_1_Offset[] = {
  0x88, 0xb0, 0x89, 0x17 // str ne r11 r9 plus r8 LSL 1 Offset
};
const byte kInstruction_str_lt_r7_r0_plus_r14_LSL_1_Offset[] = {
  0x8e, 0x70, 0x80, 0xb7 // str lt r7 r0 plus r14 LSL 1 Offset
};
const byte kInstruction_str_ge_r12_r3_plus_r1_LSL_1_Offset[] = {
  0x81, 0xc0, 0x83, 0xa7 // str ge r12 r3 plus r1 LSL 1 Offset
};
const byte kInstruction_str_le_r0_r8_plus_r13_LSL_1_Offset[] = {
  0x8d, 0x00, 0x88, 0xd7 // str le r0 r8 plus r13 LSL 1 Offset
};
const byte kInstruction_str_vc_r5_r13_plus_r7_LSL_1_Offset[] = {
  0x87, 0x50, 0x8d, 0x77 // str vc r5 r13 plus r7 LSL 1 Offset
};
const byte kInstruction_str_ge_r7_r5_plus_r11_LSL_1_Offset[] = {
  0x8b, 0x70, 0x85, 0xa7 // str ge r7 r5 plus r11 LSL 1 Offset
};
const byte kInstruction_str_ge_r10_r9_plus_r3_LSL_1_Offset[] = {
  0x83, 0xa0, 0x89, 0xa7 // str ge r10 r9 plus r3 LSL 1 Offset
};
const byte kInstruction_str_vs_r9_r5_plus_r2_LSL_1_Offset[] = {
  0x82, 0x90, 0x85, 0x67 // str vs r9 r5 plus r2 LSL 1 Offset
};
const byte kInstruction_str_hi_r2_r10_plus_r4_LSL_1_Offset[] = {
  0x84, 0x20, 0x8a, 0x87 // str hi r2 r10 plus r4 LSL 1 Offset
};
const byte kInstruction_str_ge_r1_r11_plus_r1_LSL_1_Offset[] = {
  0x81, 0x10, 0x8b, 0xa7 // str ge r1 r11 plus r1 LSL 1 Offset
};
const byte kInstruction_str_ls_r9_r14_plus_r12_LSL_1_Offset[] = {
  0x8c, 0x90, 0x8e, 0x97 // str ls r9 r14 plus r12 LSL 1 Offset
};
const byte kInstruction_str_mi_r1_r9_plus_r4_LSL_1_Offset[] = {
  0x84, 0x10, 0x89, 0x47 // str mi r1 r9 plus r4 LSL 1 Offset
};
const byte kInstruction_str_mi_r7_r10_plus_r4_LSL_1_Offset[] = {
  0x84, 0x70, 0x8a, 0x47 // str mi r7 r10 plus r4 LSL 1 Offset
};
const byte kInstruction_str_gt_r6_r2_plus_r5_LSL_1_Offset[] = {
  0x85, 0x60, 0x82, 0xc7 // str gt r6 r2 plus r5 LSL 1 Offset
};
const byte kInstruction_str_eq_r10_r8_plus_r11_LSL_1_Offset[] = {
  0x8b, 0xa0, 0x88, 0x07 // str eq r10 r8 plus r11 LSL 1 Offset
};
const byte kInstruction_str_le_r10_r1_plus_r7_LSL_1_Offset[] = {
  0x87, 0xa0, 0x81, 0xd7 // str le r10 r1 plus r7 LSL 1 Offset
};
const byte kInstruction_str_pl_r8_r14_plus_r8_LSL_1_Offset[] = {
  0x88, 0x80, 0x8e, 0x57 // str pl r8 r14 plus r8 LSL 1 Offset
};
const byte kInstruction_str_gt_r5_r1_plus_r3_LSL_1_Offset[] = {
  0x83, 0x50, 0x81, 0xc7 // str gt r5 r1 plus r3 LSL 1 Offset
};
const byte kInstruction_str_cs_r10_r4_plus_r13_LSL_1_Offset[] = {
  0x8d, 0xa0, 0x84, 0x27 // str cs r10 r4 plus r13 LSL 1 Offset
};
const byte kInstruction_str_mi_r10_r5_plus_r2_LSL_1_Offset[] = {
  0x82, 0xa0, 0x85, 0x47 // str mi r10 r5 plus r2 LSL 1 Offset
};
const byte kInstruction_str_cc_r6_r0_plus_r11_LSL_1_Offset[] = {
  0x8b, 0x60, 0x80, 0x37 // str cc r6 r0 plus r11 LSL 1 Offset
};
const byte kInstruction_str_eq_r3_r9_plus_r2_LSL_1_Offset[] = {
  0x82, 0x30, 0x89, 0x07 // str eq r3 r9 plus r2 LSL 1 Offset
};
const byte kInstruction_str_al_r1_r0_plus_r3_LSL_1_Offset[] = {
  0x83, 0x10, 0x80, 0xe7 // str al r1 r0 plus r3 LSL 1 Offset
};
const byte kInstruction_str_mi_r1_r0_plus_r13_LSL_1_Offset[] = {
  0x8d, 0x10, 0x80, 0x47 // str mi r1 r0 plus r13 LSL 1 Offset
};
const byte kInstruction_str_cc_r4_r4_plus_r5_LSL_1_Offset[] = {
  0x85, 0x40, 0x84, 0x37 // str cc r4 r4 plus r5 LSL 1 Offset
};
const byte kInstruction_str_al_r2_r6_plus_r11_LSL_1_Offset[] = {
  0x8b, 0x20, 0x86, 0xe7 // str al r2 r6 plus r11 LSL 1 Offset
};
const byte kInstruction_str_ls_r1_r5_plus_r4_LSL_1_Offset[] = {
  0x84, 0x10, 0x85, 0x97 // str ls r1 r5 plus r4 LSL 1 Offset
};
const byte kInstruction_str_eq_r0_r3_plus_r4_LSL_1_Offset[] = {
  0x84, 0x00, 0x83, 0x07 // str eq r0 r3 plus r4 LSL 1 Offset
};
const byte kInstruction_str_lt_r7_r11_plus_r10_LSL_1_Offset[] = {
  0x8a, 0x70, 0x8b, 0xb7 // str lt r7 r11 plus r10 LSL 1 Offset
};
const byte kInstruction_str_vc_r3_r0_plus_r13_LSL_1_Offset[] = {
  0x8d, 0x30, 0x80, 0x77 // str vc r3 r0 plus r13 LSL 1 Offset
};
const byte kInstruction_str_ls_r3_r3_plus_r7_LSL_1_Offset[] = {
  0x87, 0x30, 0x83, 0x97 // str ls r3 r3 plus r7 LSL 1 Offset
};
const byte kInstruction_str_al_r5_r14_plus_r3_LSL_1_Offset[] = {
  0x83, 0x50, 0x8e, 0xe7 // str al r5 r14 plus r3 LSL 1 Offset
};
const byte kInstruction_str_ne_r2_r13_plus_r9_LSL_1_Offset[] = {
  0x89, 0x20, 0x8d, 0x17 // str ne r2 r13 plus r9 LSL 1 Offset
};
const byte kInstruction_str_lt_r6_r2_plus_r11_LSL_1_Offset[] = {
  0x8b, 0x60, 0x82, 0xb7 // str lt r6 r2 plus r11 LSL 1 Offset
};
const byte kInstruction_str_vc_r9_r2_plus_r13_LSL_1_Offset[] = {
  0x8d, 0x90, 0x82, 0x77 // str vc r9 r2 plus r13 LSL 1 Offset
};
const byte kInstruction_str_ne_r5_r9_plus_r6_LSL_1_Offset[] = {
  0x86, 0x50, 0x89, 0x17 // str ne r5 r9 plus r6 LSL 1 Offset
};
const byte kInstruction_str_vc_r4_r9_plus_r8_LSL_1_Offset[] = {
  0x88, 0x40, 0x89, 0x77 // str vc r4 r9 plus r8 LSL 1 Offset
};
const byte kInstruction_str_cc_r6_r9_plus_r2_LSL_1_Offset[] = {
  0x82, 0x60, 0x89, 0x37 // str cc r6 r9 plus r2 LSL 1 Offset
};
const byte kInstruction_str_vs_r8_r7_plus_r13_LSL_1_Offset[] = {
  0x8d, 0x80, 0x87, 0x67 // str vs r8 r7 plus r13 LSL 1 Offset
};
const byte kInstruction_str_pl_r14_r14_plus_r7_LSL_1_Offset[] = {
  0x87, 0xe0, 0x8e, 0x57 // str pl r14 r14 plus r7 LSL 1 Offset
};
const byte kInstruction_str_eq_r8_r2_plus_r6_LSL_1_Offset[] = {
  0x86, 0x80, 0x82, 0x07 // str eq r8 r2 plus r6 LSL 1 Offset
};
const byte kInstruction_str_al_r13_r13_plus_r13_LSL_1_Offset[] = {
  0x8d, 0xd0, 0x8d, 0xe7 // str al r13 r13 plus r13 LSL 1 Offset
};
const byte kInstruction_str_gt_r12_r14_plus_r5_LSL_1_Offset[] = {
  0x85, 0xc0, 0x8e, 0xc7 // str gt r12 r14 plus r5 LSL 1 Offset
};
const byte kInstruction_str_lt_r14_r7_plus_r1_LSL_1_Offset[] = {
  0x81, 0xe0, 0x87, 0xb7 // str lt r14 r7 plus r1 LSL 1 Offset
};
const byte kInstruction_str_al_r0_r2_plus_r14_ROR_9_Offset[] = {
  0xee, 0x04, 0x82, 0xe7 // str al r0 r2 plus r14 ROR 9 Offset
};
const byte kInstruction_str_al_r0_r4_plus_r8_LSL_19_Offset[] = {
  0x88, 0x09, 0x84, 0xe7 // str al r0 r4 plus r8 LSL 19 Offset
};
const byte kInstruction_str_al_r0_r13_minus_r1_LSL_8_Offset[] = {
  0x01, 0x04, 0x0d, 0xe7 // str al r0 r13 minus r1 LSL 8 Offset
};
const byte kInstruction_str_al_r0_r4_plus_r12_ROR_13_Offset[] = {
  0xec, 0x06, 0x84, 0xe7 // str al r0 r4 plus r12 ROR 13 Offset
};
const byte kInstruction_str_al_r0_r9_minus_r11_LSL_10_Offset[] = {
  0x0b, 0x05, 0x09, 0xe7 // str al r0 r9 minus r11 LSL 10 Offset
};
const byte kInstruction_str_al_r0_r7_plus_r0_ROR_25_Offset[] = {
  0xe0, 0x0c, 0x87, 0xe7 // str al r0 r7 plus r0 ROR 25 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r14_LSL_9_Offset[] = {
  0x8e, 0x04, 0x80, 0xe7 // str al r0 r0 plus r14 LSL 9 Offset
};
const byte kInstruction_str_al_r0_r10_plus_r2_LSL_19_Offset[] = {
  0x82, 0x09, 0x8a, 0xe7 // str al r0 r10 plus r2 LSL 19 Offset
};
const byte kInstruction_str_al_r0_r14_plus_r4_LSL_9_Offset[] = {
  0x84, 0x04, 0x8e, 0xe7 // str al r0 r14 plus r4 LSL 9 Offset
};
const byte kInstruction_str_al_r0_r7_plus_r12_ROR_11_Offset[] = {
  0xec, 0x05, 0x87, 0xe7 // str al r0 r7 plus r12 ROR 11 Offset
};
const byte kInstruction_str_al_r0_r3_minus_r10_LSL_2_Offset[] = {
  0x0a, 0x01, 0x03, 0xe7 // str al r0 r3 minus r10 LSL 2 Offset
};
const byte kInstruction_str_al_r0_r7_plus_r12_LSL_23_Offset[] = {
  0x8c, 0x0b, 0x87, 0xe7 // str al r0 r7 plus r12 LSL 23 Offset
};
const byte kInstruction_str_al_r0_r2_minus_r8_LSL_25_Offset[] = {
  0x88, 0x0c, 0x02, 0xe7 // str al r0 r2 minus r8 LSL 25 Offset
};
const byte kInstruction_str_al_r0_r11_plus_r3_LSL_12_Offset[] = {
  0x03, 0x06, 0x8b, 0xe7 // str al r0 r11 plus r3 LSL 12 Offset
};
const byte kInstruction_str_al_r0_r0_minus_r2_LSL_12_Offset[] = {
  0x02, 0x06, 0x00, 0xe7 // str al r0 r0 minus r2 LSL 12 Offset
};
const byte kInstruction_str_al_r0_r8_minus_r7_ROR_30_Offset[] = {
  0x67, 0x0f, 0x08, 0xe7 // str al r0 r8 minus r7 ROR 30 Offset
};
const byte kInstruction_str_al_r0_r4_minus_r11_ROR_19_Offset[] = {
  0xeb, 0x09, 0x04, 0xe7 // str al r0 r4 minus r11 ROR 19 Offset
};
const byte kInstruction_str_al_r0_r9_minus_r11_ROR_23_Offset[] = {
  0xeb, 0x0b, 0x09, 0xe7 // str al r0 r9 minus r11 ROR 23 Offset
};
const byte kInstruction_str_al_r0_r12_plus_r12_ROR_28_Offset[] = {
  0x6c, 0x0e, 0x8c, 0xe7 // str al r0 r12 plus r12 ROR 28 Offset
};
const byte kInstruction_str_al_r0_r9_plus_r12_ROR_25_Offset[] = {
  0xec, 0x0c, 0x89, 0xe7 // str al r0 r9 plus r12 ROR 25 Offset
};
const byte kInstruction_str_al_r0_r6_minus_r5_LSL_26_Offset[] = {
  0x05, 0x0d, 0x06, 0xe7 // str al r0 r6 minus r5 LSL 26 Offset
};
const byte kInstruction_str_al_r0_r6_minus_r12_ROR_24_Offset[] = {
  0x6c, 0x0c, 0x06, 0xe7 // str al r0 r6 minus r12 ROR 24 Offset
};
const byte kInstruction_str_al_r0_r12_minus_r5_ROR_7_Offset[] = {
  0xe5, 0x03, 0x0c, 0xe7 // str al r0 r12 minus r5 ROR 7 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r7_ROR_15_Offset[] = {
  0xe7, 0x07, 0x80, 0xe7 // str al r0 r0 plus r7 ROR 15 Offset
};
const byte kInstruction_str_al_r0_r10_minus_r12_ROR_6_Offset[] = {
  0x6c, 0x03, 0x0a, 0xe7 // str al r0 r10 minus r12 ROR 6 Offset
};
const byte kInstruction_str_al_r0_r2_plus_r3_LSL_31_Offset[] = {
  0x83, 0x0f, 0x82, 0xe7 // str al r0 r2 plus r3 LSL 31 Offset
};
const byte kInstruction_str_al_r0_r14_plus_r9_LSL_28_Offset[] = {
  0x09, 0x0e, 0x8e, 0xe7 // str al r0 r14 plus r9 LSL 28 Offset
};
const byte kInstruction_str_al_r0_r8_minus_r4_LSL_16_Offset[] = {
  0x04, 0x08, 0x08, 0xe7 // str al r0 r8 minus r4 LSL 16 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r8_ROR_18_Offset[] = {
  0x68, 0x09, 0x80, 0xe7 // str al r0 r0 plus r8 ROR 18 Offset
};
const byte kInstruction_str_al_r0_r4_minus_r11_LSL_18_Offset[] = {
  0x0b, 0x09, 0x04, 0xe7 // str al r0 r4 minus r11 LSL 18 Offset
};
const byte kInstruction_str_al_r0_r2_minus_r4_LSL_23_Offset[] = {
  0x84, 0x0b, 0x02, 0xe7 // str al r0 r2 minus r4 LSL 23 Offset
};
const byte kInstruction_str_al_r0_r12_minus_r3_LSL_9_Offset[] = {
  0x83, 0x04, 0x0c, 0xe7 // str al r0 r12 minus r3 LSL 9 Offset
};
const byte kInstruction_str_al_r0_r6_minus_r3_ROR_31_Offset[] = {
  0xe3, 0x0f, 0x06, 0xe7 // str al r0 r6 minus r3 ROR 31 Offset
};
const byte kInstruction_str_al_r0_r9_minus_r7_LSL_23_Offset[] = {
  0x87, 0x0b, 0x09, 0xe7 // str al r0 r9 minus r7 LSL 23 Offset
};
const byte kInstruction_str_al_r0_r6_plus_r12_LSL_26_Offset[] = {
  0x0c, 0x0d, 0x86, 0xe7 // str al r0 r6 plus r12 LSL 26 Offset
};
const byte kInstruction_str_al_r0_r4_minus_r14_LSL_24_Offset[] = {
  0x0e, 0x0c, 0x04, 0xe7 // str al r0 r4 minus r14 LSL 24 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r7_ROR_21_Offset[] = {
  0xe7, 0x0a, 0x80, 0xe7 // str al r0 r0 plus r7 ROR 21 Offset
};
const byte kInstruction_str_al_r0_r14_minus_r7_LSL_29_Offset[] = {
  0x87, 0x0e, 0x0e, 0xe7 // str al r0 r14 minus r7 LSL 29 Offset
};
const byte kInstruction_str_al_r0_r10_minus_r1_ROR_29_Offset[] = {
  0xe1, 0x0e, 0x0a, 0xe7 // str al r0 r10 minus r1 ROR 29 Offset
};
const byte kInstruction_str_al_r0_r11_plus_r13_ROR_28_Offset[] = {
  0x6d, 0x0e, 0x8b, 0xe7 // str al r0 r11 plus r13 ROR 28 Offset
};
const byte kInstruction_str_al_r0_r13_minus_r1_LSL_5_Offset[] = {
  0x81, 0x02, 0x0d, 0xe7 // str al r0 r13 minus r1 LSL 5 Offset
};
const byte kInstruction_str_al_r0_r4_plus_r7_ROR_12_Offset[] = {
  0x67, 0x06, 0x84, 0xe7 // str al r0 r4 plus r7 ROR 12 Offset
};
const byte kInstruction_str_al_r0_r4_minus_r6_LSL_28_Offset[] = {
  0x06, 0x0e, 0x04, 0xe7 // str al r0 r4 minus r6 LSL 28 Offset
};
const byte kInstruction_str_al_r0_r14_plus_r5_LSL_19_Offset[] = {
  0x85, 0x09, 0x8e, 0xe7 // str al r0 r14 plus r5 LSL 19 Offset
};
const byte kInstruction_str_al_r0_r11_plus_r4_ROR_4_Offset[] = {
  0x64, 0x02, 0x8b, 0xe7 // str al r0 r11 plus r4 ROR 4 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r0_LSL_10_Offset[] = {
  0x00, 0x05, 0x80, 0xe7 // str al r0 r0 plus r0 LSL 10 Offset
};
const byte kInstruction_str_al_r0_r5_minus_r6_ROR_14_Offset[] = {
  0x66, 0x07, 0x05, 0xe7 // str al r0 r5 minus r6 ROR 14 Offset
};
const byte kInstruction_str_al_r0_r3_plus_r13_ROR_4_Offset[] = {
  0x6d, 0x02, 0x83, 0xe7 // str al r0 r3 plus r13 ROR 4 Offset
};
const byte kInstruction_str_al_r0_r1_plus_r13_LSL_12_Offset[] = {
  0x0d, 0x06, 0x81, 0xe7 // str al r0 r1 plus r13 LSL 12 Offset
};
const byte kInstruction_str_al_r0_r4_minus_r11_ROR_1_Offset[] = {
  0xeb, 0x00, 0x04, 0xe7 // str al r0 r4 minus r11 ROR 1 Offset
};
const byte kInstruction_str_al_r0_r4_plus_r14_ROR_10_Offset[] = {
  0x6e, 0x05, 0x84, 0xe7 // str al r0 r4 plus r14 ROR 10 Offset
};
const byte kInstruction_str_al_r0_r4_plus_r7_LSL_8_Offset[] = {
  0x07, 0x04, 0x84, 0xe7 // str al r0 r4 plus r7 LSL 8 Offset
};
const byte kInstruction_str_al_r0_r5_plus_r8_LSL_2_Offset[] = {
  0x08, 0x01, 0x85, 0xe7 // str al r0 r5 plus r8 LSL 2 Offset
};
const byte kInstruction_str_al_r0_r3_minus_r13_LSL_30_Offset[] = {
  0x0d, 0x0f, 0x03, 0xe7 // str al r0 r3 minus r13 LSL 30 Offset
};
const byte kInstruction_str_al_r0_r3_minus_r14_ROR_4_Offset[] = {
  0x6e, 0x02, 0x03, 0xe7 // str al r0 r3 minus r14 ROR 4 Offset
};
const byte kInstruction_str_al_r0_r9_minus_r13_ROR_23_Offset[] = {
  0xed, 0x0b, 0x09, 0xe7 // str al r0 r9 minus r13 ROR 23 Offset
};
const byte kInstruction_str_al_r0_r4_plus_r11_LSL_29_Offset[] = {
  0x8b, 0x0e, 0x84, 0xe7 // str al r0 r4 plus r11 LSL 29 Offset
};
const byte kInstruction_str_al_r0_r4_plus_r13_ROR_28_Offset[] = {
  0x6d, 0x0e, 0x84, 0xe7 // str al r0 r4 plus r13 ROR 28 Offset
};
const byte kInstruction_str_al_r0_r2_plus_r3_ROR_13_Offset[] = {
  0xe3, 0x06, 0x82, 0xe7 // str al r0 r2 plus r3 ROR 13 Offset
};
const byte kInstruction_str_al_r0_r7_minus_r11_LSL_6_Offset[] = {
  0x0b, 0x03, 0x07, 0xe7 // str al r0 r7 minus r11 LSL 6 Offset
};
const byte kInstruction_str_al_r0_r6_minus_r14_LSL_15_Offset[] = {
  0x8e, 0x07, 0x06, 0xe7 // str al r0 r6 minus r14 LSL 15 Offset
};
const byte kInstruction_str_al_r0_r3_minus_r3_LSL_17_Offset[] = {
  0x83, 0x08, 0x03, 0xe7 // str al r0 r3 minus r3 LSL 17 Offset
};
const byte kInstruction_str_al_r0_r5_plus_r13_LSL_20_Offset[] = {
  0x0d, 0x0a, 0x85, 0xe7 // str al r0 r5 plus r13 LSL 20 Offset
};
const byte kInstruction_str_al_r0_r6_plus_r1_ROR_21_Offset[] = {
  0xe1, 0x0a, 0x86, 0xe7 // str al r0 r6 plus r1 ROR 21 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r12_LSL_6_Offset[] = {
  0x0c, 0x03, 0x80, 0xe7 // str al r0 r0 plus r12 LSL 6 Offset
};
const byte kInstruction_str_al_r0_r9_minus_r6_LSL_21_Offset[] = {
  0x86, 0x0a, 0x09, 0xe7 // str al r0 r9 minus r6 LSL 21 Offset
};
const byte kInstruction_str_al_r0_r3_minus_r11_LSL_11_Offset[] = {
  0x8b, 0x05, 0x03, 0xe7 // str al r0 r3 minus r11 LSL 11 Offset
};
const byte kInstruction_str_al_r0_r4_minus_r12_ROR_11_Offset[] = {
  0xec, 0x05, 0x04, 0xe7 // str al r0 r4 minus r12 ROR 11 Offset
};
const byte kInstruction_str_al_r0_r5_plus_r4_ROR_1_Offset[] = {
  0xe4, 0x00, 0x85, 0xe7 // str al r0 r5 plus r4 ROR 1 Offset
};
const byte kInstruction_str_al_r0_r1_plus_r2_ROR_11_Offset[] = {
  0xe2, 0x05, 0x81, 0xe7 // str al r0 r1 plus r2 ROR 11 Offset
};
const byte kInstruction_str_al_r0_r5_plus_r12_LSL_28_Offset[] = {
  0x0c, 0x0e, 0x85, 0xe7 // str al r0 r5 plus r12 LSL 28 Offset
};
const byte kInstruction_str_al_r0_r10_plus_r11_ROR_25_Offset[] = {
  0xeb, 0x0c, 0x8a, 0xe7 // str al r0 r10 plus r11 ROR 25 Offset
};
const byte kInstruction_str_al_r0_r14_plus_r1_ROR_8_Offset[] = {
  0x61, 0x04, 0x8e, 0xe7 // str al r0 r14 plus r1 ROR 8 Offset
};
const byte kInstruction_str_al_r0_r9_plus_r1_ROR_21_Offset[] = {
  0xe1, 0x0a, 0x89, 0xe7 // str al r0 r9 plus r1 ROR 21 Offset
};
const byte kInstruction_str_al_r0_r12_minus_r7_LSL_6_Offset[] = {
  0x07, 0x03, 0x0c, 0xe7 // str al r0 r12 minus r7 LSL 6 Offset
};
const byte kInstruction_str_al_r0_r3_minus_r0_ROR_12_Offset[] = {
  0x60, 0x06, 0x03, 0xe7 // str al r0 r3 minus r0 ROR 12 Offset
};
const byte kInstruction_str_al_r0_r2_minus_r0_ROR_27_Offset[] = {
  0xe0, 0x0d, 0x02, 0xe7 // str al r0 r2 minus r0 ROR 27 Offset
};
const byte kInstruction_str_al_r0_r6_minus_r2_ROR_3_Offset[] = {
  0xe2, 0x01, 0x06, 0xe7 // str al r0 r6 minus r2 ROR 3 Offset
};
const byte kInstruction_str_al_r0_r1_plus_r6_LSL_16_Offset[] = {
  0x06, 0x08, 0x81, 0xe7 // str al r0 r1 plus r6 LSL 16 Offset
};
const byte kInstruction_str_al_r0_r6_minus_r14_ROR_15_Offset[] = {
  0xee, 0x07, 0x06, 0xe7 // str al r0 r6 minus r14 ROR 15 Offset
};
const byte kInstruction_str_al_r0_r4_plus_r10_LSL_20_Offset[] = {
  0x0a, 0x0a, 0x84, 0xe7 // str al r0 r4 plus r10 LSL 20 Offset
};
const byte kInstruction_str_al_r0_r7_minus_r4_LSL_16_Offset[] = {
  0x04, 0x08, 0x07, 0xe7 // str al r0 r7 minus r4 LSL 16 Offset
};
const byte kInstruction_str_al_r0_r12_minus_r5_ROR_4_Offset[] = {
  0x65, 0x02, 0x0c, 0xe7 // str al r0 r12 minus r5 ROR 4 Offset
};
const byte kInstruction_str_al_r0_r11_minus_r12_LSL_27_Offset[] = {
  0x8c, 0x0d, 0x0b, 0xe7 // str al r0 r11 minus r12 LSL 27 Offset
};
const byte kInstruction_str_al_r0_r4_minus_r4_LSL_30_Offset[] = {
  0x04, 0x0f, 0x04, 0xe7 // str al r0 r4 minus r4 LSL 30 Offset
};
const byte kInstruction_str_al_r0_r6_plus_r4_ROR_19_Offset[] = {
  0xe4, 0x09, 0x86, 0xe7 // str al r0 r6 plus r4 ROR 19 Offset
};
const byte kInstruction_str_al_r0_r11_plus_r3_ROR_19_Offset[] = {
  0xe3, 0x09, 0x8b, 0xe7 // str al r0 r11 plus r3 ROR 19 Offset
};
const byte kInstruction_str_al_r0_r9_plus_r6_LSL_12_Offset[] = {
  0x06, 0x06, 0x89, 0xe7 // str al r0 r9 plus r6 LSL 12 Offset
};
const byte kInstruction_str_al_r0_r3_minus_r8_LSL_29_Offset[] = {
  0x88, 0x0e, 0x03, 0xe7 // str al r0 r3 minus r8 LSL 29 Offset
};
const byte kInstruction_str_al_r0_r13_plus_r5_ROR_16_Offset[] = {
  0x65, 0x08, 0x8d, 0xe7 // str al r0 r13 plus r5 ROR 16 Offset
};
const byte kInstruction_str_al_r0_r11_plus_r8_ROR_10_Offset[] = {
  0x68, 0x05, 0x8b, 0xe7 // str al r0 r11 plus r8 ROR 10 Offset
};
const byte kInstruction_str_al_r0_r0_minus_r9_LSL_10_Offset[] = {
  0x09, 0x05, 0x00, 0xe7 // str al r0 r0 minus r9 LSL 10 Offset
};
const byte kInstruction_str_al_r0_r4_minus_r5_ROR_11_Offset[] = {
  0xe5, 0x05, 0x04, 0xe7 // str al r0 r4 minus r5 ROR 11 Offset
};
const byte kInstruction_str_al_r0_r3_plus_r7_LSL_28_Offset[] = {
  0x07, 0x0e, 0x83, 0xe7 // str al r0 r3 plus r7 LSL 28 Offset
};
const byte kInstruction_str_al_r0_r6_plus_r7_LSL_17_Offset[] = {
  0x87, 0x08, 0x86, 0xe7 // str al r0 r6 plus r7 LSL 17 Offset
};
const byte kInstruction_str_al_r0_r6_plus_r4_LSL_9_Offset[] = {
  0x84, 0x04, 0x86, 0xe7 // str al r0 r6 plus r4 LSL 9 Offset
};
const byte kInstruction_str_al_r0_r6_minus_r9_LSL_15_Offset[] = {
  0x89, 0x07, 0x06, 0xe7 // str al r0 r6 minus r9 LSL 15 Offset
};
const byte kInstruction_str_al_r0_r14_plus_r1_LSL_31_Offset[] = {
  0x81, 0x0f, 0x8e, 0xe7 // str al r0 r14 plus r1 LSL 31 Offset
};
const byte kInstruction_str_al_r0_r13_plus_r4_ROR_31_Offset[] = {
  0xe4, 0x0f, 0x8d, 0xe7 // str al r0 r13 plus r4 ROR 31 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r0_ROR_20_Offset[] = {
  0x60, 0x0a, 0x80, 0xe7 // str al r0 r0 plus r0 ROR 20 Offset
};
const byte kInstruction_str_al_r0_r9_minus_r5_LSL_8_Offset[] = {
  0x05, 0x04, 0x09, 0xe7 // str al r0 r9 minus r5 LSL 8 Offset
};
const byte kInstruction_str_al_r0_r11_plus_r8_ROR_1_Offset[] = {
  0xe8, 0x00, 0x8b, 0xe7 // str al r0 r11 plus r8 ROR 1 Offset
};
const byte kInstruction_str_al_r0_r1_minus_r4_ROR_18_Offset[] = {
  0x64, 0x09, 0x01, 0xe7 // str al r0 r1 minus r4 ROR 18 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r3_LSL_11_Offset[] = {
  0x83, 0x05, 0x80, 0xe7 // str al r0 r0 plus r3 LSL 11 Offset
};
const byte kInstruction_str_al_r0_r2_plus_r11_ROR_31_Offset[] = {
  0xeb, 0x0f, 0x82, 0xe7 // str al r0 r2 plus r11 ROR 31 Offset
};
const byte kInstruction_str_al_r0_r7_minus_r11_LSL_22_Offset[] = {
  0x0b, 0x0b, 0x07, 0xe7 // str al r0 r7 minus r11 LSL 22 Offset
};
const byte kInstruction_str_al_r0_r10_minus_r12_LSL_13_Offset[] = {
  0x8c, 0x06, 0x0a, 0xe7 // str al r0 r10 minus r12 LSL 13 Offset
};
const byte kInstruction_str_al_r0_r12_plus_r3_ROR_17_Offset[] = {
  0xe3, 0x08, 0x8c, 0xe7 // str al r0 r12 plus r3 ROR 17 Offset
};
const byte kInstruction_str_al_r0_r0_minus_r4_ROR_15_Offset[] = {
  0xe4, 0x07, 0x00, 0xe7 // str al r0 r0 minus r4 ROR 15 Offset
};
const byte kInstruction_str_al_r0_r3_plus_r4_LSL_1_Offset[] = {
  0x84, 0x00, 0x83, 0xe7 // str al r0 r3 plus r4 LSL 1 Offset
};
const byte kInstruction_str_al_r0_r2_plus_r10_ROR_20_Offset[] = {
  0x6a, 0x0a, 0x82, 0xe7 // str al r0 r2 plus r10 ROR 20 Offset
};
const byte kInstruction_str_al_r0_r11_minus_r8_ROR_10_Offset[] = {
  0x68, 0x05, 0x0b, 0xe7 // str al r0 r11 minus r8 ROR 10 Offset
};
const byte kInstruction_str_al_r0_r14_minus_r5_LSL_3_Offset[] = {
  0x85, 0x01, 0x0e, 0xe7 // str al r0 r14 minus r5 LSL 3 Offset
};
const byte kInstruction_str_al_r0_r0_minus_r7_LSL_24_Offset[] = {
  0x07, 0x0c, 0x00, 0xe7 // str al r0 r0 minus r7 LSL 24 Offset
};
const byte kInstruction_str_al_r0_r2_minus_r11_LSL_4_Offset[] = {
  0x0b, 0x02, 0x02, 0xe7 // str al r0 r2 minus r11 LSL 4 Offset
};
const byte kInstruction_str_al_r0_r3_plus_r0_ROR_25_Offset[] = {
  0xe0, 0x0c, 0x83, 0xe7 // str al r0 r3 plus r0 ROR 25 Offset
};
const byte kInstruction_str_al_r0_r12_plus_r8_ROR_7_Offset[] = {
  0xe8, 0x03, 0x8c, 0xe7 // str al r0 r12 plus r8 ROR 7 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r9_ROR_31_Offset[] = {
  0xe9, 0x0f, 0x80, 0xe7 // str al r0 r0 plus r9 ROR 31 Offset
};
const byte kInstruction_str_al_r0_r5_minus_r0_LSL_3_Offset[] = {
  0x80, 0x01, 0x05, 0xe7 // str al r0 r5 minus r0 LSL 3 Offset
};
const byte kInstruction_str_al_r0_r0_minus_r9_ROR_8_Offset[] = {
  0x69, 0x04, 0x00, 0xe7 // str al r0 r0 minus r9 ROR 8 Offset
};
const byte kInstruction_str_al_r0_r8_plus_r12_ROR_22_Offset[] = {
  0x6c, 0x0b, 0x88, 0xe7 // str al r0 r8 plus r12 ROR 22 Offset
};
const byte kInstruction_str_al_r0_r0_minus_r1_ROR_26_Offset[] = {
  0x61, 0x0d, 0x00, 0xe7 // str al r0 r0 minus r1 ROR 26 Offset
};
const byte kInstruction_str_al_r0_r1_minus_r1_ROR_22_Offset[] = {
  0x61, 0x0b, 0x01, 0xe7 // str al r0 r1 minus r1 ROR 22 Offset
};
const byte kInstruction_str_al_r0_r2_minus_r0_ROR_23_Offset[] = {
  0xe0, 0x0b, 0x02, 0xe7 // str al r0 r2 minus r0 ROR 23 Offset
};
const byte kInstruction_str_al_r0_r9_plus_r11_ROR_29_Offset[] = {
  0xeb, 0x0e, 0x89, 0xe7 // str al r0 r9 plus r11 ROR 29 Offset
};
const byte kInstruction_str_al_r0_r5_minus_r9_ROR_18_Offset[] = {
  0x69, 0x09, 0x05, 0xe7 // str al r0 r5 minus r9 ROR 18 Offset
};
const byte kInstruction_str_al_r0_r2_minus_r8_LSL_4_Offset[] = {
  0x08, 0x02, 0x02, 0xe7 // str al r0 r2 minus r8 LSL 4 Offset
};
const byte kInstruction_str_al_r0_r2_plus_r10_ROR_7_Offset[] = {
  0xea, 0x03, 0x82, 0xe7 // str al r0 r2 plus r10 ROR 7 Offset
};
const byte kInstruction_str_al_r0_r6_plus_r1_LSL_22_Offset[] = {
  0x01, 0x0b, 0x86, 0xe7 // str al r0 r6 plus r1 LSL 22 Offset
};
const byte kInstruction_str_al_r0_r14_minus_r14_ROR_24_Offset[] = {
  0x6e, 0x0c, 0x0e, 0xe7 // str al r0 r14 minus r14 ROR 24 Offset
};
const byte kInstruction_str_al_r0_r5_plus_r11_ROR_4_Offset[] = {
  0x6b, 0x02, 0x85, 0xe7 // str al r0 r5 plus r11 ROR 4 Offset
};
const byte kInstruction_str_al_r0_r3_minus_r7_LSL_16_Offset[] = {
  0x07, 0x08, 0x03, 0xe7 // str al r0 r3 minus r7 LSL 16 Offset
};
const byte kInstruction_str_al_r0_r12_plus_r11_ROR_17_Offset[] = {
  0xeb, 0x08, 0x8c, 0xe7 // str al r0 r12 plus r11 ROR 17 Offset
};
const byte kInstruction_str_al_r0_r11_minus_r1_LSL_25_Offset[] = {
  0x81, 0x0c, 0x0b, 0xe7 // str al r0 r11 minus r1 LSL 25 Offset
};
const byte kInstruction_str_al_r0_r4_plus_r0_ROR_4_Offset[] = {
  0x60, 0x02, 0x84, 0xe7 // str al r0 r4 plus r0 ROR 4 Offset
};
const byte kInstruction_str_al_r0_r4_minus_r9_ROR_28_Offset[] = {
  0x69, 0x0e, 0x04, 0xe7 // str al r0 r4 minus r9 ROR 28 Offset
};
const byte kInstruction_str_al_r0_r3_plus_r8_ROR_22_Offset[] = {
  0x68, 0x0b, 0x83, 0xe7 // str al r0 r3 plus r8 ROR 22 Offset
};
const byte kInstruction_str_al_r0_r8_plus_r13_LSL_12_Offset[] = {
  0x0d, 0x06, 0x88, 0xe7 // str al r0 r8 plus r13 LSL 12 Offset
};
const byte kInstruction_str_al_r0_r8_minus_r4_ROR_31_Offset[] = {
  0xe4, 0x0f, 0x08, 0xe7 // str al r0 r8 minus r4 ROR 31 Offset
};
const byte kInstruction_str_al_r0_r10_minus_r7_ROR_3_Offset[] = {
  0xe7, 0x01, 0x0a, 0xe7 // str al r0 r10 minus r7 ROR 3 Offset
};
const byte kInstruction_str_al_r0_r8_minus_r11_ROR_30_Offset[] = {
  0x6b, 0x0f, 0x08, 0xe7 // str al r0 r8 minus r11 ROR 30 Offset
};
const byte kInstruction_str_al_r0_r11_minus_r5_ROR_8_Offset[] = {
  0x65, 0x04, 0x0b, 0xe7 // str al r0 r11 minus r5 ROR 8 Offset
};
const byte kInstruction_str_al_r0_r10_minus_r2_LSL_16_Offset[] = {
  0x02, 0x08, 0x0a, 0xe7 // str al r0 r10 minus r2 LSL 16 Offset
};
const byte kInstruction_str_al_r0_r10_minus_r2_ROR_8_Offset[] = {
  0x62, 0x04, 0x0a, 0xe7 // str al r0 r10 minus r2 ROR 8 Offset
};
const byte kInstruction_str_al_r0_r7_minus_r5_ROR_2_Offset[] = {
  0x65, 0x01, 0x07, 0xe7 // str al r0 r7 minus r5 ROR 2 Offset
};
const byte kInstruction_str_al_r0_r12_minus_r13_LSL_24_Offset[] = {
  0x0d, 0x0c, 0x0c, 0xe7 // str al r0 r12 minus r13 LSL 24 Offset
};
const byte kInstruction_str_al_r0_r2_minus_r4_ROR_3_Offset[] = {
  0xe4, 0x01, 0x02, 0xe7 // str al r0 r2 minus r4 ROR 3 Offset
};
const byte kInstruction_str_al_r0_r14_plus_r12_LSL_19_Offset[] = {
  0x8c, 0x09, 0x8e, 0xe7 // str al r0 r14 plus r12 LSL 19 Offset
};
const byte kInstruction_str_al_r0_r2_plus_r6_LSL_17_Offset[] = {
  0x86, 0x08, 0x82, 0xe7 // str al r0 r2 plus r6 LSL 17 Offset
};
const byte kInstruction_str_al_r0_r9_plus_r10_LSL_16_Offset[] = {
  0x0a, 0x08, 0x89, 0xe7 // str al r0 r9 plus r10 LSL 16 Offset
};
const byte kInstruction_str_al_r0_r3_plus_r9_ROR_28_Offset[] = {
  0x69, 0x0e, 0x83, 0xe7 // str al r0 r3 plus r9 ROR 28 Offset
};
const byte kInstruction_str_al_r0_r2_plus_r3_LSL_3_Offset[] = {
  0x83, 0x01, 0x82, 0xe7 // str al r0 r2 plus r3 LSL 3 Offset
};
const byte kInstruction_str_al_r0_r0_minus_r12_LSL_28_Offset[] = {
  0x0c, 0x0e, 0x00, 0xe7 // str al r0 r0 minus r12 LSL 28 Offset
};
const byte kInstruction_str_al_r0_r2_minus_r12_ROR_14_Offset[] = {
  0x6c, 0x07, 0x02, 0xe7 // str al r0 r2 minus r12 ROR 14 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r11_LSL_28_Offset[] = {
  0x0b, 0x0e, 0x80, 0xe7 // str al r0 r0 plus r11 LSL 28 Offset
};
const byte kInstruction_str_al_r0_r11_plus_r6_ROR_15_Offset[] = {
  0xe6, 0x07, 0x8b, 0xe7 // str al r0 r11 plus r6 ROR 15 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r14_ROR_21_Offset[] = {
  0xee, 0x0a, 0x80, 0xe7 // str al r0 r0 plus r14 ROR 21 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r2_LSL_12_Offset[] = {
  0x02, 0x06, 0x80, 0xe7 // str al r0 r0 plus r2 LSL 12 Offset
};
const byte kInstruction_str_al_r0_r14_plus_r11_LSL_8_Offset[] = {
  0x0b, 0x04, 0x8e, 0xe7 // str al r0 r14 plus r11 LSL 8 Offset
};
const byte kInstruction_str_al_r0_r2_plus_r4_LSL_12_Offset[] = {
  0x04, 0x06, 0x82, 0xe7 // str al r0 r2 plus r4 LSL 12 Offset
};
const byte kInstruction_str_al_r0_r12_minus_r7_LSL_26_Offset[] = {
  0x07, 0x0d, 0x0c, 0xe7 // str al r0 r12 minus r7 LSL 26 Offset
};
const byte kInstruction_str_al_r0_r8_minus_r8_ROR_24_Offset[] = {
  0x68, 0x0c, 0x08, 0xe7 // str al r0 r8 minus r8 ROR 24 Offset
};
const byte kInstruction_str_al_r0_r10_plus_r0_LSL_28_Offset[] = {
  0x00, 0x0e, 0x8a, 0xe7 // str al r0 r10 plus r0 LSL 28 Offset
};
const byte kInstruction_str_al_r0_r3_minus_r2_ROR_26_Offset[] = {
  0x62, 0x0d, 0x03, 0xe7 // str al r0 r3 minus r2 ROR 26 Offset
};
const byte kInstruction_str_al_r0_r7_minus_r0_LSL_7_Offset[] = {
  0x80, 0x03, 0x07, 0xe7 // str al r0 r7 minus r0 LSL 7 Offset
};
const byte kInstruction_str_al_r0_r0_minus_r14_ROR_12_Offset[] = {
  0x6e, 0x06, 0x00, 0xe7 // str al r0 r0 minus r14 ROR 12 Offset
};
const byte kInstruction_str_al_r0_r3_minus_r10_ROR_9_Offset[] = {
  0xea, 0x04, 0x03, 0xe7 // str al r0 r3 minus r10 ROR 9 Offset
};
const byte kInstruction_str_al_r0_r9_plus_r2_LSL_15_Offset[] = {
  0x82, 0x07, 0x89, 0xe7 // str al r0 r9 plus r2 LSL 15 Offset
};
const byte kInstruction_str_al_r0_r13_minus_r10_ROR_28_Offset[] = {
  0x6a, 0x0e, 0x0d, 0xe7 // str al r0 r13 minus r10 ROR 28 Offset
};
const byte kInstruction_str_al_r0_r10_plus_r14_LSL_10_Offset[] = {
  0x0e, 0x05, 0x8a, 0xe7 // str al r0 r10 plus r14 LSL 10 Offset
};
const byte kInstruction_str_al_r0_r2_plus_r3_ROR_23_Offset[] = {
  0xe3, 0x0b, 0x82, 0xe7 // str al r0 r2 plus r3 ROR 23 Offset
};
const byte kInstruction_str_al_r0_r3_plus_r4_LSL_2_Offset[] = {
  0x04, 0x01, 0x83, 0xe7 // str al r0 r3 plus r4 LSL 2 Offset
};
const byte kInstruction_str_al_r0_r7_plus_r12_LSL_28_Offset[] = {
  0x0c, 0x0e, 0x87, 0xe7 // str al r0 r7 plus r12 LSL 28 Offset
};
const byte kInstruction_str_al_r0_r12_minus_r3_LSL_18_Offset[] = {
  0x03, 0x09, 0x0c, 0xe7 // str al r0 r12 minus r3 LSL 18 Offset
};
const byte kInstruction_str_al_r0_r13_minus_r11_ROR_7_Offset[] = {
  0xeb, 0x03, 0x0d, 0xe7 // str al r0 r13 minus r11 ROR 7 Offset
};
const byte kInstruction_str_al_r0_r14_minus_r14_LSL_6_Offset[] = {
  0x0e, 0x03, 0x0e, 0xe7 // str al r0 r14 minus r14 LSL 6 Offset
};
const byte kInstruction_str_al_r0_r7_minus_r11_ROR_31_Offset[] = {
  0xeb, 0x0f, 0x07, 0xe7 // str al r0 r7 minus r11 ROR 31 Offset
};
const byte kInstruction_str_al_r0_r9_minus_r7_ROR_14_Offset[] = {
  0x67, 0x07, 0x09, 0xe7 // str al r0 r9 minus r7 ROR 14 Offset
};
const byte kInstruction_str_al_r0_r8_minus_r2_LSL_7_Offset[] = {
  0x82, 0x03, 0x08, 0xe7 // str al r0 r8 minus r2 LSL 7 Offset
};
const byte kInstruction_str_al_r0_r3_plus_r14_LSL_23_Offset[] = {
  0x8e, 0x0b, 0x83, 0xe7 // str al r0 r3 plus r14 LSL 23 Offset
};
const byte kInstruction_str_al_r0_r5_plus_r7_LSL_5_Offset[] = {
  0x87, 0x02, 0x85, 0xe7 // str al r0 r5 plus r7 LSL 5 Offset
};
const byte kInstruction_str_al_r0_r10_minus_r14_LSL_30_Offset[] = {
  0x0e, 0x0f, 0x0a, 0xe7 // str al r0 r10 minus r14 LSL 30 Offset
};
const byte kInstruction_str_al_r0_r0_minus_r1_LSL_25_Offset[] = {
  0x81, 0x0c, 0x00, 0xe7 // str al r0 r0 minus r1 LSL 25 Offset
};
const byte kInstruction_str_al_r0_r14_plus_r8_ROR_31_Offset[] = {
  0xe8, 0x0f, 0x8e, 0xe7 // str al r0 r14 plus r8 ROR 31 Offset
};
const byte kInstruction_str_al_r0_r11_plus_r11_ROR_31_Offset[] = {
  0xeb, 0x0f, 0x8b, 0xe7 // str al r0 r11 plus r11 ROR 31 Offset
};
const byte kInstruction_str_al_r0_r14_plus_r8_LSL_27_Offset[] = {
  0x88, 0x0d, 0x8e, 0xe7 // str al r0 r14 plus r8 LSL 27 Offset
};
const byte kInstruction_str_al_r0_r6_minus_r0_ROR_9_Offset[] = {
  0xe0, 0x04, 0x06, 0xe7 // str al r0 r6 minus r0 ROR 9 Offset
};
const byte kInstruction_str_al_r0_r4_minus_r12_LSL_29_Offset[] = {
  0x8c, 0x0e, 0x04, 0xe7 // str al r0 r4 minus r12 LSL 29 Offset
};
const byte kInstruction_str_al_r0_r14_plus_r8_ROR_4_Offset[] = {
  0x68, 0x02, 0x8e, 0xe7 // str al r0 r14 plus r8 ROR 4 Offset
};
const byte kInstruction_str_al_r0_r9_minus_r9_LSL_27_Offset[] = {
  0x89, 0x0d, 0x09, 0xe7 // str al r0 r9 minus r9 LSL 27 Offset
};
const byte kInstruction_str_al_r0_r3_plus_r8_ROR_28_Offset[] = {
  0x68, 0x0e, 0x83, 0xe7 // str al r0 r3 plus r8 ROR 28 Offset
};
const byte kInstruction_str_al_r0_r12_plus_r0_ROR_13_Offset[] = {
  0xe0, 0x06, 0x8c, 0xe7 // str al r0 r12 plus r0 ROR 13 Offset
};
const byte kInstruction_str_al_r0_r8_minus_r4_ROR_20_Offset[] = {
  0x64, 0x0a, 0x08, 0xe7 // str al r0 r8 minus r4 ROR 20 Offset
};
const byte kInstruction_str_al_r0_r1_plus_r14_ROR_4_Offset[] = {
  0x6e, 0x02, 0x81, 0xe7 // str al r0 r1 plus r14 ROR 4 Offset
};
const byte kInstruction_str_al_r0_r11_plus_r2_ROR_10_Offset[] = {
  0x62, 0x05, 0x8b, 0xe7 // str al r0 r11 plus r2 ROR 10 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r10_ROR_18_Offset[] = {
  0x6a, 0x09, 0x80, 0xe7 // str al r0 r0 plus r10 ROR 18 Offset
};
const byte kInstruction_str_al_r0_r11_minus_r7_ROR_3_Offset[] = {
  0xe7, 0x01, 0x0b, 0xe7 // str al r0 r11 minus r7 ROR 3 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r14_LSL_10_Offset[] = {
  0x0e, 0x05, 0x80, 0xe7 // str al r0 r0 plus r14 LSL 10 Offset
};
const byte kInstruction_str_al_r0_r12_minus_r9_ROR_5_Offset[] = {
  0xe9, 0x02, 0x0c, 0xe7 // str al r0 r12 minus r9 ROR 5 Offset
};
const byte kInstruction_str_al_r0_r11_plus_r14_ROR_26_Offset[] = {
  0x6e, 0x0d, 0x8b, 0xe7 // str al r0 r11 plus r14 ROR 26 Offset
};
const byte kInstruction_str_al_r8_r3_minus_r4_ROR_30_PreIndex[] = {
  0x64, 0x8f, 0x23, 0xe7 // str al r8 r3 minus r4 ROR 30 PreIndex
};
const byte kInstruction_str_al_r1_r0_plus_r5_LSL_13_PreIndex[] = {
  0x85, 0x16, 0xa0, 0xe7 // str al r1 r0 plus r5 LSL 13 PreIndex
};
const byte kInstruction_str_al_r13_r3_minus_r1_ROR_31_PostIndex[] = {
  0xe1, 0xdf, 0x03, 0xe6 // str al r13 r3 minus r1 ROR 31 PostIndex
};
const byte kInstruction_str_al_r9_r7_minus_r1_LSL_8_PostIndex[] = {
  0x01, 0x94, 0x07, 0xe6 // str al r9 r7 minus r1 LSL 8 PostIndex
};
const byte kInstruction_str_al_r2_r6_plus_r7_ROR_18_PreIndex[] = {
  0x67, 0x29, 0xa6, 0xe7 // str al r2 r6 plus r7 ROR 18 PreIndex
};
const byte kInstruction_str_al_r0_r11_minus_r11_ROR_26_PreIndex[] = {
  0x6b, 0x0d, 0x2b, 0xe7 // str al r0 r11 minus r11 ROR 26 PreIndex
};
const byte kInstruction_str_al_r14_r4_plus_r14_LSL_12_PreIndex[] = {
  0x0e, 0xe6, 0xa4, 0xe7 // str al r14 r4 plus r14 LSL 12 PreIndex
};
const byte kInstruction_str_al_r10_r6_minus_r13_LSL_15_PreIndex[] = {
  0x8d, 0xa7, 0x26, 0xe7 // str al r10 r6 minus r13 LSL 15 PreIndex
};
const byte kInstruction_str_al_r0_r2_minus_r13_ROR_30_PreIndex[] = {
  0x6d, 0x0f, 0x22, 0xe7 // str al r0 r2 minus r13 ROR 30 PreIndex
};
const byte kInstruction_str_al_r0_r14_plus_r2_LSL_10_PostIndex[] = {
  0x02, 0x05, 0x8e, 0xe6 // str al r0 r14 plus r2 LSL 10 PostIndex
};
const byte kInstruction_str_al_r8_r3_plus_r8_ROR_26_PreIndex[] = {
  0x68, 0x8d, 0xa3, 0xe7 // str al r8 r3 plus r8 ROR 26 PreIndex
};
const byte kInstruction_str_al_r0_r4_minus_r12_LSL_31_PostIndex[] = {
  0x8c, 0x0f, 0x04, 0xe6 // str al r0 r4 minus r12 LSL 31 PostIndex
};
const byte kInstruction_str_al_r11_r12_plus_r7_LSL_25_PostIndex[] = {
  0x87, 0xbc, 0x8c, 0xe6 // str al r11 r12 plus r7 LSL 25 PostIndex
};
const byte kInstruction_str_al_r2_r10_minus_r7_ROR_22_PostIndex[] = {
  0x67, 0x2b, 0x0a, 0xe6 // str al r2 r10 minus r7 ROR 22 PostIndex
};
const byte kInstruction_str_al_r3_r6_plus_r8_LSL_28_PreIndex[] = {
  0x08, 0x3e, 0xa6, 0xe7 // str al r3 r6 plus r8 LSL 28 PreIndex
};
const byte kInstruction_str_al_r9_r3_plus_r12_ROR_21_PreIndex[] = {
  0xec, 0x9a, 0xa3, 0xe7 // str al r9 r3 plus r12 ROR 21 PreIndex
};
const byte kInstruction_str_al_r11_r5_plus_r3_ROR_29_PostIndex[] = {
  0xe3, 0xbe, 0x85, 0xe6 // str al r11 r5 plus r3 ROR 29 PostIndex
};
const byte kInstruction_str_al_r4_r9_plus_r3_LSL_4_PreIndex[] = {
  0x03, 0x42, 0xa9, 0xe7 // str al r4 r9 plus r3 LSL 4 PreIndex
};
const byte kInstruction_str_al_r8_r11_plus_r12_LSL_14_PreIndex[] = {
  0x0c, 0x87, 0xab, 0xe7 // str al r8 r11 plus r12 LSL 14 PreIndex
};
const byte kInstruction_str_al_r4_r7_plus_r14_LSL_24_PreIndex[] = {
  0x0e, 0x4c, 0xa7, 0xe7 // str al r4 r7 plus r14 LSL 24 PreIndex
};
const byte kInstruction_str_al_r5_r14_plus_r14_ROR_15_PostIndex[] = {
  0xee, 0x57, 0x8e, 0xe6 // str al r5 r14 plus r14 ROR 15 PostIndex
};
const byte kInstruction_str_al_r12_r10_plus_r9_ROR_24_PreIndex[] = {
  0x69, 0xcc, 0xaa, 0xe7 // str al r12 r10 plus r9 ROR 24 PreIndex
};
const byte kInstruction_str_al_r6_r0_plus_r5_ROR_11_PreIndex[] = {
  0xe5, 0x65, 0xa0, 0xe7 // str al r6 r0 plus r5 ROR 11 PreIndex
};
const byte kInstruction_str_al_r8_r7_plus_r6_ROR_27_PostIndex[] = {
  0xe6, 0x8d, 0x87, 0xe6 // str al r8 r7 plus r6 ROR 27 PostIndex
};
const byte kInstruction_str_al_r2_r12_plus_r12_LSL_8_PreIndex[] = {
  0x0c, 0x24, 0xac, 0xe7 // str al r2 r12 plus r12 LSL 8 PreIndex
};
const byte kInstruction_str_al_r6_r5_plus_r13_ROR_7_PreIndex[] = {
  0xed, 0x63, 0xa5, 0xe7 // str al r6 r5 plus r13 ROR 7 PreIndex
};
const byte kInstruction_str_al_r10_r7_plus_r3_LSL_14_PreIndex[] = {
  0x03, 0xa7, 0xa7, 0xe7 // str al r10 r7 plus r3 LSL 14 PreIndex
};
const byte kInstruction_str_al_r7_r0_plus_r0_ROR_23_PostIndex[] = {
  0xe0, 0x7b, 0x80, 0xe6 // str al r7 r0 plus r0 ROR 23 PostIndex
};
const byte kInstruction_str_al_r12_r0_plus_r3_LSL_15_PostIndex[] = {
  0x83, 0xc7, 0x80, 0xe6 // str al r12 r0 plus r3 LSL 15 PostIndex
};
const byte kInstruction_str_al_r2_r4_plus_r11_ROR_17_PostIndex[] = {
  0xeb, 0x28, 0x84, 0xe6 // str al r2 r4 plus r11 ROR 17 PostIndex
};
const byte kInstruction_str_al_r1_r7_plus_r9_ROR_18_PostIndex[] = {
  0x69, 0x19, 0x87, 0xe6 // str al r1 r7 plus r9 ROR 18 PostIndex
};
const byte kInstruction_str_al_r8_r10_plus_r14_ROR_3_PreIndex[] = {
  0xee, 0x81, 0xaa, 0xe7 // str al r8 r10 plus r14 ROR 3 PreIndex
};
const byte kInstruction_str_al_r2_r3_minus_r13_LSL_26_PostIndex[] = {
  0x0d, 0x2d, 0x03, 0xe6 // str al r2 r3 minus r13 LSL 26 PostIndex
};
const byte kInstruction_str_al_r8_r1_plus_r4_LSL_20_PostIndex[] = {
  0x04, 0x8a, 0x81, 0xe6 // str al r8 r1 plus r4 LSL 20 PostIndex
};
const byte kInstruction_str_al_r8_r10_minus_r6_ROR_31_PreIndex[] = {
  0xe6, 0x8f, 0x2a, 0xe7 // str al r8 r10 minus r6 ROR 31 PreIndex
};
const byte kInstruction_str_al_r8_r7_minus_r8_LSL_9_PreIndex[] = {
  0x88, 0x84, 0x27, 0xe7 // str al r8 r7 minus r8 LSL 9 PreIndex
};
const byte kInstruction_str_al_r2_r1_minus_r13_LSL_3_PostIndex[] = {
  0x8d, 0x21, 0x01, 0xe6 // str al r2 r1 minus r13 LSL 3 PostIndex
};
const byte kInstruction_str_al_r7_r1_minus_r12_LSL_8_PostIndex[] = {
  0x0c, 0x74, 0x01, 0xe6 // str al r7 r1 minus r12 LSL 8 PostIndex
};
const byte kInstruction_str_al_r3_r8_plus_r5_LSL_28_PostIndex[] = {
  0x05, 0x3e, 0x88, 0xe6 // str al r3 r8 plus r5 LSL 28 PostIndex
};
const byte kInstruction_str_al_r13_r0_plus_r0_ROR_9_PreIndex[] = {
  0xe0, 0xd4, 0xa0, 0xe7 // str al r13 r0 plus r0 ROR 9 PreIndex
};
const byte kInstruction_str_al_r2_r5_plus_r4_ROR_22_PreIndex[] = {
  0x64, 0x2b, 0xa5, 0xe7 // str al r2 r5 plus r4 ROR 22 PreIndex
};
const byte kInstruction_str_al_r4_r0_plus_r0_LSL_12_PreIndex[] = {
  0x00, 0x46, 0xa0, 0xe7 // str al r4 r0 plus r0 LSL 12 PreIndex
};
const byte kInstruction_str_al_r11_r7_plus_r0_LSL_10_PostIndex[] = {
  0x00, 0xb5, 0x87, 0xe6 // str al r11 r7 plus r0 LSL 10 PostIndex
};
const byte kInstruction_str_al_r13_r0_minus_r12_LSL_25_PostIndex[] = {
  0x8c, 0xdc, 0x00, 0xe6 // str al r13 r0 minus r12 LSL 25 PostIndex
};
const byte kInstruction_str_al_r1_r3_minus_r12_ROR_22_PreIndex[] = {
  0x6c, 0x1b, 0x23, 0xe7 // str al r1 r3 minus r12 ROR 22 PreIndex
};
const byte kInstruction_str_al_r10_r6_minus_r5_LSL_26_PostIndex[] = {
  0x05, 0xad, 0x06, 0xe6 // str al r10 r6 minus r5 LSL 26 PostIndex
};
const byte kInstruction_str_al_r0_r7_plus_r11_LSL_17_PreIndex[] = {
  0x8b, 0x08, 0xa7, 0xe7 // str al r0 r7 plus r11 LSL 17 PreIndex
};
const byte kInstruction_str_al_r12_r13_minus_r9_ROR_28_PreIndex[] = {
  0x69, 0xce, 0x2d, 0xe7 // str al r12 r13 minus r9 ROR 28 PreIndex
};
const byte kInstruction_str_al_r11_r1_plus_r6_ROR_10_PostIndex[] = {
  0x66, 0xb5, 0x81, 0xe6 // str al r11 r1 plus r6 ROR 10 PostIndex
};
const byte kInstruction_str_al_r3_r0_minus_r8_ROR_31_PreIndex[] = {
  0xe8, 0x3f, 0x20, 0xe7 // str al r3 r0 minus r8 ROR 31 PreIndex
};
const byte kInstruction_str_al_r13_r6_plus_r12_ROR_29_PreIndex[] = {
  0xec, 0xde, 0xa6, 0xe7 // str al r13 r6 plus r12 ROR 29 PreIndex
};
const byte kInstruction_str_al_r2_r5_minus_r9_LSL_10_PostIndex[] = {
  0x09, 0x25, 0x05, 0xe6 // str al r2 r5 minus r9 LSL 10 PostIndex
};
const byte kInstruction_str_al_r11_r3_minus_r6_ROR_5_PostIndex[] = {
  0xe6, 0xb2, 0x03, 0xe6 // str al r11 r3 minus r6 ROR 5 PostIndex
};
const byte kInstruction_str_al_r6_r2_minus_r14_LSL_10_PostIndex[] = {
  0x0e, 0x65, 0x02, 0xe6 // str al r6 r2 minus r14 LSL 10 PostIndex
};
const byte kInstruction_str_al_r3_r10_minus_r10_LSL_24_PreIndex[] = {
  0x0a, 0x3c, 0x2a, 0xe7 // str al r3 r10 minus r10 LSL 24 PreIndex
};
const byte kInstruction_str_al_r4_r3_plus_r11_LSL_12_PreIndex[] = {
  0x0b, 0x46, 0xa3, 0xe7 // str al r4 r3 plus r11 LSL 12 PreIndex
};
const byte kInstruction_str_al_r4_r13_minus_r0_ROR_22_PostIndex[] = {
  0x60, 0x4b, 0x0d, 0xe6 // str al r4 r13 minus r0 ROR 22 PostIndex
};
const byte kInstruction_str_al_r2_r13_plus_r1_ROR_2_PostIndex[] = {
  0x61, 0x21, 0x8d, 0xe6 // str al r2 r13 plus r1 ROR 2 PostIndex
};
const byte kInstruction_str_al_r11_r9_plus_r8_LSL_3_PreIndex[] = {
  0x88, 0xb1, 0xa9, 0xe7 // str al r11 r9 plus r8 LSL 3 PreIndex
};
const byte kInstruction_str_al_r10_r6_minus_r6_ROR_23_PostIndex[] = {
  0xe6, 0xab, 0x06, 0xe6 // str al r10 r6 minus r6 ROR 23 PostIndex
};
const byte kInstruction_str_al_r4_r0_minus_r4_LSL_8_PreIndex[] = {
  0x04, 0x44, 0x20, 0xe7 // str al r4 r0 minus r4 LSL 8 PreIndex
};
const byte kInstruction_str_al_r14_r9_plus_r2_ROR_6_PreIndex[] = {
  0x62, 0xe3, 0xa9, 0xe7 // str al r14 r9 plus r2 ROR 6 PreIndex
};
const byte kInstruction_str_al_r8_r10_plus_r9_LSL_3_PostIndex[] = {
  0x89, 0x81, 0x8a, 0xe6 // str al r8 r10 plus r9 LSL 3 PostIndex
};
const byte kInstruction_str_al_r9_r14_plus_r13_ROR_25_PostIndex[] = {
  0xed, 0x9c, 0x8e, 0xe6 // str al r9 r14 plus r13 ROR 25 PostIndex
};
const byte kInstruction_str_al_r13_r3_plus_r6_LSL_3_PostIndex[] = {
  0x86, 0xd1, 0x83, 0xe6 // str al r13 r3 plus r6 LSL 3 PostIndex
};
const byte kInstruction_str_al_r9_r14_plus_r4_LSL_24_PreIndex[] = {
  0x04, 0x9c, 0xae, 0xe7 // str al r9 r14 plus r4 LSL 24 PreIndex
};
const byte kInstruction_str_al_r5_r10_plus_r4_LSL_29_PreIndex[] = {
  0x84, 0x5e, 0xaa, 0xe7 // str al r5 r10 plus r4 LSL 29 PreIndex
};
const byte kInstruction_str_al_r7_r5_minus_r11_LSL_13_PreIndex[] = {
  0x8b, 0x76, 0x25, 0xe7 // str al r7 r5 minus r11 LSL 13 PreIndex
};
const byte kInstruction_str_al_r8_r1_plus_r2_ROR_27_PreIndex[] = {
  0xe2, 0x8d, 0xa1, 0xe7 // str al r8 r1 plus r2 ROR 27 PreIndex
};
const byte kInstruction_str_al_r14_r11_minus_r8_ROR_1_PreIndex[] = {
  0xe8, 0xe0, 0x2b, 0xe7 // str al r14 r11 minus r8 ROR 1 PreIndex
};
const byte kInstruction_str_al_r14_r6_plus_r4_ROR_29_PostIndex[] = {
  0xe4, 0xee, 0x86, 0xe6 // str al r14 r6 plus r4 ROR 29 PostIndex
};
const byte kInstruction_str_al_r13_r0_plus_r1_LSL_21_PreIndex[] = {
  0x81, 0xda, 0xa0, 0xe7 // str al r13 r0 plus r1 LSL 21 PreIndex
};
const byte kInstruction_str_al_r1_r8_plus_r2_LSL_22_PostIndex[] = {
  0x02, 0x1b, 0x88, 0xe6 // str al r1 r8 plus r2 LSL 22 PostIndex
};
const byte kInstruction_str_al_r11_r8_plus_r5_LSL_19_PreIndex[] = {
  0x85, 0xb9, 0xa8, 0xe7 // str al r11 r8 plus r5 LSL 19 PreIndex
};
const byte kInstruction_str_al_r12_r10_plus_r7_ROR_15_PostIndex[] = {
  0xe7, 0xc7, 0x8a, 0xe6 // str al r12 r10 plus r7 ROR 15 PostIndex
};
const byte kInstruction_str_al_r2_r6_plus_r14_ROR_4_PostIndex[] = {
  0x6e, 0x22, 0x86, 0xe6 // str al r2 r6 plus r14 ROR 4 PostIndex
};
const byte kInstruction_str_al_r3_r6_plus_r12_LSL_9_PostIndex[] = {
  0x8c, 0x34, 0x86, 0xe6 // str al r3 r6 plus r12 LSL 9 PostIndex
};
const byte kInstruction_str_al_r1_r0_minus_r6_LSL_24_PreIndex[] = {
  0x06, 0x1c, 0x20, 0xe7 // str al r1 r0 minus r6 LSL 24 PreIndex
};
const byte kInstruction_str_al_r5_r12_plus_r2_ROR_30_PostIndex[] = {
  0x62, 0x5f, 0x8c, 0xe6 // str al r5 r12 plus r2 ROR 30 PostIndex
};
const byte kInstruction_str_al_r14_r3_plus_r8_ROR_6_PostIndex[] = {
  0x68, 0xe3, 0x83, 0xe6 // str al r14 r3 plus r8 ROR 6 PostIndex
};
const byte kInstruction_str_al_r8_r14_minus_r10_ROR_25_PostIndex[] = {
  0xea, 0x8c, 0x0e, 0xe6 // str al r8 r14 minus r10 ROR 25 PostIndex
};
const byte kInstruction_str_al_r7_r12_plus_r2_ROR_3_PreIndex[] = {
  0xe2, 0x71, 0xac, 0xe7 // str al r7 r12 plus r2 ROR 3 PreIndex
};
const byte kInstruction_str_al_r9_r13_plus_r11_ROR_29_PreIndex[] = {
  0xeb, 0x9e, 0xad, 0xe7 // str al r9 r13 plus r11 ROR 29 PreIndex
};
const byte kInstruction_str_al_r5_r2_plus_r10_ROR_21_PostIndex[] = {
  0xea, 0x5a, 0x82, 0xe6 // str al r5 r2 plus r10 ROR 21 PostIndex
};
const byte kInstruction_str_al_r13_r8_minus_r2_LSL_21_PreIndex[] = {
  0x82, 0xda, 0x28, 0xe7 // str al r13 r8 minus r2 LSL 21 PreIndex
};
const byte kInstruction_str_al_r1_r13_plus_r5_LSL_20_PostIndex[] = {
  0x05, 0x1a, 0x8d, 0xe6 // str al r1 r13 plus r5 LSL 20 PostIndex
};
const byte kInstruction_str_al_r6_r0_plus_r6_LSL_15_PostIndex[] = {
  0x86, 0x67, 0x80, 0xe6 // str al r6 r0 plus r6 LSL 15 PostIndex
};
const byte kInstruction_str_al_r7_r3_minus_r6_LSL_31_PreIndex[] = {
  0x86, 0x7f, 0x23, 0xe7 // str al r7 r3 minus r6 LSL 31 PreIndex
};
const byte kInstruction_str_al_r5_r10_minus_r11_ROR_6_PostIndex[] = {
  0x6b, 0x53, 0x0a, 0xe6 // str al r5 r10 minus r11 ROR 6 PostIndex
};
const byte kInstruction_str_al_r7_r3_minus_r13_ROR_29_PreIndex[] = {
  0xed, 0x7e, 0x23, 0xe7 // str al r7 r3 minus r13 ROR 29 PreIndex
};
const byte kInstruction_str_al_r4_r6_plus_r3_LSL_10_PreIndex[] = {
  0x03, 0x45, 0xa6, 0xe7 // str al r4 r6 plus r3 LSL 10 PreIndex
};
const byte kInstruction_str_al_r7_r13_plus_r4_ROR_8_PreIndex[] = {
  0x64, 0x74, 0xad, 0xe7 // str al r7 r13 plus r4 ROR 8 PreIndex
};
const byte kInstruction_str_al_r2_r5_plus_r0_LSL_30_PreIndex[] = {
  0x00, 0x2f, 0xa5, 0xe7 // str al r2 r5 plus r0 LSL 30 PreIndex
};
const byte kInstruction_str_al_r10_r12_plus_r5_LSL_8_PreIndex[] = {
  0x05, 0xa4, 0xac, 0xe7 // str al r10 r12 plus r5 LSL 8 PreIndex
};
const byte kInstruction_str_al_r4_r10_minus_r11_ROR_2_PreIndex[] = {
  0x6b, 0x41, 0x2a, 0xe7 // str al r4 r10 minus r11 ROR 2 PreIndex
};
const byte kInstruction_str_al_r4_r11_plus_r12_LSL_14_PreIndex[] = {
  0x0c, 0x47, 0xab, 0xe7 // str al r4 r11 plus r12 LSL 14 PreIndex
};
const byte kInstruction_str_al_r1_r6_plus_r1_LSL_10_PostIndex[] = {
  0x01, 0x15, 0x86, 0xe6 // str al r1 r6 plus r1 LSL 10 PostIndex
};
const byte kInstruction_str_al_r8_r11_plus_r11_LSL_11_PreIndex[] = {
  0x8b, 0x85, 0xab, 0xe7 // str al r8 r11 plus r11 LSL 11 PreIndex
};
const byte kInstruction_str_al_r12_r8_minus_r7_ROR_10_PostIndex[] = {
  0x67, 0xc5, 0x08, 0xe6 // str al r12 r8 minus r7 ROR 10 PostIndex
};
const byte kInstruction_str_al_r13_r11_plus_r1_LSL_11_PreIndex[] = {
  0x81, 0xd5, 0xab, 0xe7 // str al r13 r11 plus r1 LSL 11 PreIndex
};
const byte kInstruction_str_al_r5_r3_plus_r9_ROR_9_PreIndex[] = {
  0xe9, 0x54, 0xa3, 0xe7 // str al r5 r3 plus r9 ROR 9 PreIndex
};
const byte kInstruction_str_al_r6_r10_plus_r6_LSL_18_PreIndex[] = {
  0x06, 0x69, 0xaa, 0xe7 // str al r6 r10 plus r6 LSL 18 PreIndex
};
const byte kInstruction_str_al_r6_r1_minus_r14_LSL_14_PostIndex[] = {
  0x0e, 0x67, 0x01, 0xe6 // str al r6 r1 minus r14 LSL 14 PostIndex
};
const byte kInstruction_str_al_r10_r3_minus_r6_LSL_5_PreIndex[] = {
  0x86, 0xa2, 0x23, 0xe7 // str al r10 r3 minus r6 LSL 5 PreIndex
};
const byte kInstruction_str_al_r6_r7_minus_r13_ROR_26_PreIndex[] = {
  0x6d, 0x6d, 0x27, 0xe7 // str al r6 r7 minus r13 ROR 26 PreIndex
};
const byte kInstruction_str_al_r8_r12_minus_r7_ROR_30_PreIndex[] = {
  0x67, 0x8f, 0x2c, 0xe7 // str al r8 r12 minus r7 ROR 30 PreIndex
};
const byte kInstruction_str_al_r9_r7_plus_r5_LSL_27_PostIndex[] = {
  0x85, 0x9d, 0x87, 0xe6 // str al r9 r7 plus r5 LSL 27 PostIndex
};
const byte kInstruction_str_al_r8_r10_minus_r14_LSL_26_PreIndex[] = {
  0x0e, 0x8d, 0x2a, 0xe7 // str al r8 r10 minus r14 LSL 26 PreIndex
};
const byte kInstruction_str_al_r9_r13_minus_r2_LSL_30_PostIndex[] = {
  0x02, 0x9f, 0x0d, 0xe6 // str al r9 r13 minus r2 LSL 30 PostIndex
};
const byte kInstruction_str_al_r12_r9_minus_r8_ROR_7_PostIndex[] = {
  0xe8, 0xc3, 0x09, 0xe6 // str al r12 r9 minus r8 ROR 7 PostIndex
};
const byte kInstruction_str_al_r4_r9_minus_r5_ROR_2_PreIndex[] = {
  0x65, 0x41, 0x29, 0xe7 // str al r4 r9 minus r5 ROR 2 PreIndex
};
const byte kInstruction_str_al_r1_r5_plus_r9_ROR_16_PostIndex[] = {
  0x69, 0x18, 0x85, 0xe6 // str al r1 r5 plus r9 ROR 16 PostIndex
};
const byte kInstruction_str_al_r3_r2_minus_r9_LSL_25_PostIndex[] = {
  0x89, 0x3c, 0x02, 0xe6 // str al r3 r2 minus r9 LSL 25 PostIndex
};
const byte kInstruction_str_al_r14_r0_minus_r4_LSL_22_PreIndex[] = {
  0x04, 0xeb, 0x20, 0xe7 // str al r14 r0 minus r4 LSL 22 PreIndex
};
const byte kInstruction_str_al_r1_r4_plus_r2_LSL_7_PostIndex[] = {
  0x82, 0x13, 0x84, 0xe6 // str al r1 r4 plus r2 LSL 7 PostIndex
};
const byte kInstruction_str_al_r13_r9_minus_r4_LSL_31_PostIndex[] = {
  0x84, 0xdf, 0x09, 0xe6 // str al r13 r9 minus r4 LSL 31 PostIndex
};
const byte kInstruction_str_al_r11_r14_minus_r2_LSL_13_PostIndex[] = {
  0x82, 0xb6, 0x0e, 0xe6 // str al r11 r14 minus r2 LSL 13 PostIndex
};
const byte kInstruction_str_al_r10_r5_plus_r1_LSL_22_PreIndex[] = {
  0x01, 0xab, 0xa5, 0xe7 // str al r10 r5 plus r1 LSL 22 PreIndex
};
const byte kInstruction_str_al_r13_r2_minus_r10_LSL_25_PreIndex[] = {
  0x8a, 0xdc, 0x22, 0xe7 // str al r13 r2 minus r10 LSL 25 PreIndex
};
const byte kInstruction_str_al_r12_r2_plus_r0_ROR_27_PostIndex[] = {
  0xe0, 0xcd, 0x82, 0xe6 // str al r12 r2 plus r0 ROR 27 PostIndex
};
const byte kInstruction_str_al_r5_r13_plus_r13_LSL_5_PostIndex[] = {
  0x8d, 0x52, 0x8d, 0xe6 // str al r5 r13 plus r13 LSL 5 PostIndex
};
const byte kInstruction_str_al_r14_r0_minus_r11_ROR_3_PreIndex[] = {
  0xeb, 0xe1, 0x20, 0xe7 // str al r14 r0 minus r11 ROR 3 PreIndex
};
const byte kInstruction_str_al_r1_r10_minus_r8_LSL_4_PreIndex[] = {
  0x08, 0x12, 0x2a, 0xe7 // str al r1 r10 minus r8 LSL 4 PreIndex
};
const byte kInstruction_str_al_r11_r4_plus_r3_ROR_23_PostIndex[] = {
  0xe3, 0xbb, 0x84, 0xe6 // str al r11 r4 plus r3 ROR 23 PostIndex
};
const byte kInstruction_str_al_r1_r4_plus_r11_LSL_29_PreIndex[] = {
  0x8b, 0x1e, 0xa4, 0xe7 // str al r1 r4 plus r11 LSL 29 PreIndex
};
const byte kInstruction_str_al_r12_r4_plus_r7_ROR_4_PreIndex[] = {
  0x67, 0xc2, 0xa4, 0xe7 // str al r12 r4 plus r7 ROR 4 PreIndex
};
const byte kInstruction_str_al_r4_r11_plus_r14_ROR_17_PostIndex[] = {
  0xee, 0x48, 0x8b, 0xe6 // str al r4 r11 plus r14 ROR 17 PostIndex
};
const byte kInstruction_str_al_r3_r10_plus_r0_LSL_2_PostIndex[] = {
  0x00, 0x31, 0x8a, 0xe6 // str al r3 r10 plus r0 LSL 2 PostIndex
};
const byte kInstruction_str_al_r11_r13_minus_r8_ROR_26_PostIndex[] = {
  0x68, 0xbd, 0x0d, 0xe6 // str al r11 r13 minus r8 ROR 26 PostIndex
};
const byte kInstruction_str_al_r0_r8_plus_r14_ROR_21_PostIndex[] = {
  0xee, 0x0a, 0x88, 0xe6 // str al r0 r8 plus r14 ROR 21 PostIndex
};
const byte kInstruction_str_al_r8_r5_plus_r7_LSL_9_PreIndex[] = {
  0x87, 0x84, 0xa5, 0xe7 // str al r8 r5 plus r7 LSL 9 PreIndex
};
const byte kInstruction_str_al_r9_r10_plus_r14_LSL_14_PostIndex[] = {
  0x0e, 0x97, 0x8a, 0xe6 // str al r9 r10 plus r14 LSL 14 PostIndex
};
const byte kInstruction_str_al_r1_r9_minus_r4_LSL_26_PostIndex[] = {
  0x04, 0x1d, 0x09, 0xe6 // str al r1 r9 minus r4 LSL 26 PostIndex
};
const byte kInstruction_str_al_r8_r4_plus_r1_LSL_8_PreIndex[] = {
  0x01, 0x84, 0xa4, 0xe7 // str al r8 r4 plus r1 LSL 8 PreIndex
};
const byte kInstruction_str_al_r13_r6_minus_r7_ROR_23_PostIndex[] = {
  0xe7, 0xdb, 0x06, 0xe6 // str al r13 r6 minus r7 ROR 23 PostIndex
};
const byte kInstruction_str_al_r12_r9_plus_r4_LSL_15_PreIndex[] = {
  0x84, 0xc7, 0xa9, 0xe7 // str al r12 r9 plus r4 LSL 15 PreIndex
};
const byte kInstruction_str_al_r7_r8_plus_r1_ROR_24_PostIndex[] = {
  0x61, 0x7c, 0x88, 0xe6 // str al r7 r8 plus r1 ROR 24 PostIndex
};
const byte kInstruction_str_al_r5_r7_minus_r2_LSL_20_PreIndex[] = {
  0x02, 0x5a, 0x27, 0xe7 // str al r5 r7 minus r2 LSL 20 PreIndex
};
const byte kInstruction_str_al_r3_r11_minus_r6_LSL_16_PostIndex[] = {
  0x06, 0x38, 0x0b, 0xe6 // str al r3 r11 minus r6 LSL 16 PostIndex
};
const byte kInstruction_str_al_r6_r14_minus_r8_ROR_15_PostIndex[] = {
  0xe8, 0x67, 0x0e, 0xe6 // str al r6 r14 minus r8 ROR 15 PostIndex
};
const byte kInstruction_str_al_r13_r14_minus_r7_LSL_28_PreIndex[] = {
  0x07, 0xde, 0x2e, 0xe7 // str al r13 r14 minus r7 LSL 28 PreIndex
};
const byte kInstruction_str_al_r3_r12_plus_r9_LSL_3_PreIndex[] = {
  0x89, 0x31, 0xac, 0xe7 // str al r3 r12 plus r9 LSL 3 PreIndex
};
const byte kInstruction_str_al_r1_r3_plus_r10_ROR_14_PreIndex[] = {
  0x6a, 0x17, 0xa3, 0xe7 // str al r1 r3 plus r10 ROR 14 PreIndex
};
const byte kInstruction_str_al_r1_r3_minus_r12_LSL_9_PreIndex[] = {
  0x8c, 0x14, 0x23, 0xe7 // str al r1 r3 minus r12 LSL 9 PreIndex
};
const byte kInstruction_str_al_r3_r6_plus_r8_ROR_21_PostIndex[] = {
  0xe8, 0x3a, 0x86, 0xe6 // str al r3 r6 plus r8 ROR 21 PostIndex
};
const byte kInstruction_str_al_r14_r3_plus_r9_LSL_7_PreIndex[] = {
  0x89, 0xe3, 0xa3, 0xe7 // str al r14 r3 plus r9 LSL 7 PreIndex
};
const byte kInstruction_str_al_r4_r8_plus_r1_ROR_27_PreIndex[] = {
  0xe1, 0x4d, 0xa8, 0xe7 // str al r4 r8 plus r1 ROR 27 PreIndex
};
const byte kInstruction_str_al_r1_r13_minus_r5_ROR_19_PreIndex[] = {
  0xe5, 0x19, 0x2d, 0xe7 // str al r1 r13 minus r5 ROR 19 PreIndex
};
const byte kInstruction_str_al_r5_r1_plus_r14_LSL_15_PostIndex[] = {
  0x8e, 0x57, 0x81, 0xe6 // str al r5 r1 plus r14 LSL 15 PostIndex
};
const byte kInstruction_str_al_r12_r3_minus_r11_ROR_28_PreIndex[] = {
  0x6b, 0xce, 0x23, 0xe7 // str al r12 r3 minus r11 ROR 28 PreIndex
};
const byte kInstruction_str_al_r14_r3_plus_r4_LSL_20_PostIndex[] = {
  0x04, 0xea, 0x83, 0xe6 // str al r14 r3 plus r4 LSL 20 PostIndex
};
const byte kInstruction_str_al_r6_r2_plus_r10_ROR_14_PreIndex[] = {
  0x6a, 0x67, 0xa2, 0xe7 // str al r6 r2 plus r10 ROR 14 PreIndex
};
const byte kInstruction_str_al_r8_r12_plus_r13_ROR_26_PreIndex[] = {
  0x6d, 0x8d, 0xac, 0xe7 // str al r8 r12 plus r13 ROR 26 PreIndex
};
const byte kInstruction_str_al_r12_r13_minus_r13_LSL_11_PostIndex[] = {
  0x8d, 0xc5, 0x0d, 0xe6 // str al r12 r13 minus r13 LSL 11 PostIndex
};
const byte kInstruction_str_al_r0_r6_minus_r10_LSL_24_PostIndex[] = {
  0x0a, 0x0c, 0x06, 0xe6 // str al r0 r6 minus r10 LSL 24 PostIndex
};
const byte kInstruction_str_al_r6_r0_minus_r13_LSL_29_PreIndex[] = {
  0x8d, 0x6e, 0x20, 0xe7 // str al r6 r0 minus r13 LSL 29 PreIndex
};
const byte kInstruction_str_al_r4_r2_plus_r10_LSL_15_PostIndex[] = {
  0x8a, 0x47, 0x82, 0xe6 // str al r4 r2 plus r10 LSL 15 PostIndex
};
const byte kInstruction_str_al_r13_r9_plus_r14_ROR_21_PreIndex[] = {
  0xee, 0xda, 0xa9, 0xe7 // str al r13 r9 plus r14 ROR 21 PreIndex
};
const byte kInstruction_str_al_r4_r1_minus_r0_ROR_26_PostIndex[] = {
  0x60, 0x4d, 0x01, 0xe6 // str al r4 r1 minus r0 ROR 26 PostIndex
};
const byte kInstruction_str_al_r12_r7_minus_r12_ROR_28_PreIndex[] = {
  0x6c, 0xce, 0x27, 0xe7 // str al r12 r7 minus r12 ROR 28 PreIndex
};
const byte kInstruction_str_al_r10_r0_minus_r5_LSL_7_PreIndex[] = {
  0x85, 0xa3, 0x20, 0xe7 // str al r10 r0 minus r5 LSL 7 PreIndex
};
const byte kInstruction_str_al_r4_r1_plus_r2_ROR_23_PreIndex[] = {
  0xe2, 0x4b, 0xa1, 0xe7 // str al r4 r1 plus r2 ROR 23 PreIndex
};
const byte kInstruction_str_al_r12_r6_minus_r1_LSL_31_PostIndex[] = {
  0x81, 0xcf, 0x06, 0xe6 // str al r12 r6 minus r1 LSL 31 PostIndex
};
const byte kInstruction_str_al_r4_r8_plus_r3_LSL_28_PreIndex[] = {
  0x03, 0x4e, 0xa8, 0xe7 // str al r4 r8 plus r3 LSL 28 PreIndex
};
const byte kInstruction_str_al_r11_r7_minus_r5_ROR_3_PreIndex[] = {
  0xe5, 0xb1, 0x27, 0xe7 // str al r11 r7 minus r5 ROR 3 PreIndex
};
const byte kInstruction_str_al_r4_r3_plus_r11_LSL_16_PostIndex[] = {
  0x0b, 0x48, 0x83, 0xe6 // str al r4 r3 plus r11 LSL 16 PostIndex
};
const byte kInstruction_str_al_r12_r11_plus_r12_ROR_11_PreIndex[] = {
  0xec, 0xc5, 0xab, 0xe7 // str al r12 r11 plus r12 ROR 11 PreIndex
};
const byte kInstruction_str_al_r11_r5_minus_r0_ROR_29_PreIndex[] = {
  0xe0, 0xbe, 0x25, 0xe7 // str al r11 r5 minus r0 ROR 29 PreIndex
};
const byte kInstruction_str_al_r11_r7_plus_r12_LSL_24_PostIndex[] = {
  0x0c, 0xbc, 0x87, 0xe6 // str al r11 r7 plus r12 LSL 24 PostIndex
};
const byte kInstruction_str_al_r13_r4_plus_r4_ROR_9_PostIndex[] = {
  0xe4, 0xd4, 0x84, 0xe6 // str al r13 r4 plus r4 ROR 9 PostIndex
};
const byte kInstruction_str_al_r0_r9_minus_r11_LSL_17_PostIndex[] = {
  0x8b, 0x08, 0x09, 0xe6 // str al r0 r9 minus r11 LSL 17 PostIndex
};
const byte kInstruction_str_al_r1_r11_plus_r5_ROR_27_PostIndex[] = {
  0xe5, 0x1d, 0x8b, 0xe6 // str al r1 r11 plus r5 ROR 27 PostIndex
};
const byte kInstruction_str_al_r6_r0_plus_r5_ROR_16_PostIndex[] = {
  0x65, 0x68, 0x80, 0xe6 // str al r6 r0 plus r5 ROR 16 PostIndex
};
const byte kInstruction_str_al_r14_r8_plus_r8_LSL_23_PreIndex[] = {
  0x88, 0xeb, 0xa8, 0xe7 // str al r14 r8 plus r8 LSL 23 PreIndex
};
const byte kInstruction_str_al_r7_r0_plus_r12_LSL_29_PostIndex[] = {
  0x8c, 0x7e, 0x80, 0xe6 // str al r7 r0 plus r12 LSL 29 PostIndex
};
const byte kInstruction_str_al_r5_r4_minus_r12_ROR_28_PreIndex[] = {
  0x6c, 0x5e, 0x24, 0xe7 // str al r5 r4 minus r12 ROR 28 PreIndex
};
const byte kInstruction_str_al_r1_r13_plus_r1_LSL_12_PreIndex[] = {
  0x01, 0x16, 0xad, 0xe7 // str al r1 r13 plus r1 LSL 12 PreIndex
};
const byte kInstruction_str_al_r7_r9_minus_r11_ROR_24_PostIndex[] = {
  0x6b, 0x7c, 0x09, 0xe6 // str al r7 r9 minus r11 ROR 24 PostIndex
};
const byte kInstruction_str_al_r4_r8_minus_r0_ROR_6_PreIndex[] = {
  0x60, 0x43, 0x28, 0xe7 // str al r4 r8 minus r0 ROR 6 PreIndex
};
const byte kInstruction_str_al_r6_r9_plus_r9_ROR_30_PreIndex[] = {
  0x69, 0x6f, 0xa9, 0xe7 // str al r6 r9 plus r9 ROR 30 PreIndex
};
const byte kInstruction_str_al_r1_r14_minus_r10_LSL_4_PreIndex[] = {
  0x0a, 0x12, 0x2e, 0xe7 // str al r1 r14 minus r10 LSL 4 PreIndex
};
const byte kInstruction_str_al_r6_r0_minus_r13_ROR_12_PreIndex[] = {
  0x6d, 0x66, 0x20, 0xe7 // str al r6 r0 minus r13 ROR 12 PreIndex
};
const byte kInstruction_str_al_r1_r4_minus_r2_LSL_23_PreIndex[] = {
  0x82, 0x1b, 0x24, 0xe7 // str al r1 r4 minus r2 LSL 23 PreIndex
};
const byte kInstruction_str_al_r0_r12_minus_r11_ROR_26_PostIndex[] = {
  0x6b, 0x0d, 0x0c, 0xe6 // str al r0 r12 minus r11 ROR 26 PostIndex
};
const byte kInstruction_str_al_r5_r8_plus_r4_LSL_31_PostIndex[] = {
  0x84, 0x5f, 0x88, 0xe6 // str al r5 r8 plus r4 LSL 31 PostIndex
};
const byte kInstruction_str_al_r14_r11_plus_r10_LSL_8_PreIndex[] = {
  0x0a, 0xe4, 0xab, 0xe7 // str al r14 r11 plus r10 LSL 8 PreIndex
};
const byte kInstruction_str_al_r4_r14_minus_r2_LSL_31_PostIndex[] = {
  0x82, 0x4f, 0x0e, 0xe6 // str al r4 r14 minus r2 LSL 31 PostIndex
};
const byte kInstruction_str_al_r9_r6_plus_r11_LSL_30_PostIndex[] = {
  0x0b, 0x9f, 0x86, 0xe6 // str al r9 r6 plus r11 LSL 30 PostIndex
};
const byte kInstruction_str_al_r14_r5_minus_r1_ROR_9_PostIndex[] = {
  0xe1, 0xe4, 0x05, 0xe6 // str al r14 r5 minus r1 ROR 9 PostIndex
};
const byte kInstruction_str_al_r2_r8_minus_r14_ROR_28_PreIndex[] = {
  0x6e, 0x2e, 0x28, 0xe7 // str al r2 r8 minus r14 ROR 28 PreIndex
};
const byte kInstruction_str_al_r0_r4_plus_r12_ROR_8_PostIndex[] = {
  0x6c, 0x04, 0x84, 0xe6 // str al r0 r4 plus r12 ROR 8 PostIndex
};
const byte kInstruction_str_al_r6_r12_minus_r10_LSL_19_PostIndex[] = {
  0x8a, 0x69, 0x0c, 0xe6 // str al r6 r12 minus r10 LSL 19 PostIndex
};
const byte kInstruction_str_al_r7_r8_minus_r11_LSL_9_PostIndex[] = {
  0x8b, 0x74, 0x08, 0xe6 // str al r7 r8 minus r11 LSL 9 PostIndex
};
const byte kInstruction_str_al_r6_r4_plus_r4_ROR_6_PostIndex[] = {
  0x64, 0x63, 0x84, 0xe6 // str al r6 r4 plus r4 ROR 6 PostIndex
};
const byte kInstruction_str_al_r12_r14_minus_r3_LSL_19_PostIndex[] = {
  0x83, 0xc9, 0x0e, 0xe6 // str al r12 r14 minus r3 LSL 19 PostIndex
};
const byte kInstruction_str_al_r14_r4_plus_r14_ROR_22_PostIndex[] = {
  0x6e, 0xeb, 0x84, 0xe6 // str al r14 r4 plus r14 ROR 22 PostIndex
};
const byte kInstruction_str_al_r2_r5_minus_r2_LSL_15_PostIndex[] = {
  0x82, 0x27, 0x05, 0xe6 // str al r2 r5 minus r2 LSL 15 PostIndex
};
const byte kInstruction_str_al_r6_r5_minus_r12_ROR_24_PreIndex[] = {
  0x6c, 0x6c, 0x25, 0xe7 // str al r6 r5 minus r12 ROR 24 PreIndex
};
const byte kInstruction_str_al_r0_r1_plus_r0_LSL_8_PostIndex[] = {
  0x00, 0x04, 0x81, 0xe6 // str al r0 r1 plus r0 LSL 8 PostIndex
};
const byte kInstruction_str_al_r6_r3_plus_r10_LSL_23_PostIndex[] = {
  0x8a, 0x6b, 0x83, 0xe6 // str al r6 r3 plus r10 LSL 23 PostIndex
};
const byte kInstruction_str_al_r10_r1_plus_r0_ROR_15_PostIndex[] = {
  0xe0, 0xa7, 0x81, 0xe6 // str al r10 r1 plus r0 ROR 15 PostIndex
};
const byte kInstruction_str_al_r1_r14_plus_r8_ROR_31_PostIndex[] = {
  0xe8, 0x1f, 0x8e, 0xe6 // str al r1 r14 plus r8 ROR 31 PostIndex
};
const byte kInstruction_str_al_r14_r10_plus_r6_LSL_6_PreIndex[] = {
  0x06, 0xe3, 0xaa, 0xe7 // str al r14 r10 plus r6 LSL 6 PreIndex
};
const byte kInstruction_str_al_r12_r2_plus_r10_ROR_18_PostIndex[] = {
  0x6a, 0xc9, 0x82, 0xe6 // str al r12 r2 plus r10 ROR 18 PostIndex
};
const byte kInstruction_str_al_r4_r8_minus_r0_LSL_9_PostIndex[] = {
  0x80, 0x44, 0x08, 0xe6 // str al r4 r8 minus r0 LSL 9 PostIndex
};
const byte kInstruction_str_al_r9_r3_plus_r3_ROR_16_PreIndex[] = {
  0x63, 0x98, 0xa3, 0xe7 // str al r9 r3 plus r3 ROR 16 PreIndex
};
const byte kInstruction_str_al_r1_r7_minus_r3_LSL_2_PostIndex[] = {
  0x03, 0x11, 0x07, 0xe6 // str al r1 r7 minus r3 LSL 2 PostIndex
};
const byte kInstruction_str_al_r8_r13_plus_r14_ROR_16_PreIndex[] = {
  0x6e, 0x88, 0xad, 0xe7 // str al r8 r13 plus r14 ROR 16 PreIndex
};
const byte kInstruction_str_al_r3_r0_minus_r3_LSL_30_PreIndex[] = {
  0x03, 0x3f, 0x20, 0xe7 // str al r3 r0 minus r3 LSL 30 PreIndex
};
const byte kInstruction_str_al_r1_r11_plus_r13_ROR_26_PostIndex[] = {
  0x6d, 0x1d, 0x8b, 0xe6 // str al r1 r11 plus r13 ROR 26 PostIndex
};
const byte kInstruction_str_al_r3_r1_plus_r9_ROR_3_PostIndex[] = {
  0xe9, 0x31, 0x81, 0xe6 // str al r3 r1 plus r9 ROR 3 PostIndex
};
const byte kInstruction_str_al_r14_r11_minus_r4_ROR_22_PostIndex[] = {
  0x64, 0xeb, 0x0b, 0xe6 // str al r14 r11 minus r4 ROR 22 PostIndex
};
const byte kInstruction_str_al_r3_r1_plus_r10_LSL_29_PostIndex[] = {
  0x8a, 0x3e, 0x81, 0xe6 // str al r3 r1 plus r10 LSL 29 PostIndex
};
const byte kInstruction_str_al_r12_r6_plus_r13_ROR_6_PostIndex[] = {
  0x6d, 0xc3, 0x86, 0xe6 // str al r12 r6 plus r13 ROR 6 PostIndex
};
const byte kInstruction_str_al_r12_r11_plus_r2_LSL_19_PostIndex[] = {
  0x82, 0xc9, 0x8b, 0xe6 // str al r12 r11 plus r2 LSL 19 PostIndex
};
const byte kInstruction_str_al_r14_r10_plus_r4_LSL_31_PostIndex[] = {
  0x84, 0xef, 0x8a, 0xe6 // str al r14 r10 plus r4 LSL 31 PostIndex
};
const byte kInstruction_str_al_r1_r4_minus_r2_ROR_28_PostIndex[] = {
  0x62, 0x1e, 0x04, 0xe6 // str al r1 r4 minus r2 ROR 28 PostIndex
};
const byte kInstruction_str_al_r3_r4_minus_r6_ROR_20_PreIndex[] = {
  0x66, 0x3a, 0x24, 0xe7 // str al r3 r4 minus r6 ROR 20 PreIndex
};
const byte kInstruction_str_al_r6_r8_plus_r13_ROR_11_PostIndex[] = {
  0xed, 0x65, 0x88, 0xe6 // str al r6 r8 plus r13 ROR 11 PostIndex
};
const byte kInstruction_str_al_r8_r0_plus_r14_LSL_27_PreIndex[] = {
  0x8e, 0x8d, 0xa0, 0xe7 // str al r8 r0 plus r14 LSL 27 PreIndex
};
const byte kInstruction_str_al_r10_r2_minus_r9_ROR_3_PreIndex[] = {
  0xe9, 0xa1, 0x22, 0xe7 // str al r10 r2 minus r9 ROR 3 PreIndex
};
const byte kInstruction_str_al_r7_r9_minus_r3_ROR_11_PostIndex[] = {
  0xe3, 0x75, 0x09, 0xe6 // str al r7 r9 minus r3 ROR 11 PostIndex
};
const byte kInstruction_str_al_r12_r14_minus_r3_LSL_24_PostIndex[] = {
  0x03, 0xcc, 0x0e, 0xe6 // str al r12 r14 minus r3 LSL 24 PostIndex
};
const byte kInstruction_str_al_r1_r6_plus_r8_LSL_26_PreIndex[] = {
  0x08, 0x1d, 0xa6, 0xe7 // str al r1 r6 plus r8 LSL 26 PreIndex
};
const byte kInstruction_str_al_r12_r10_minus_r7_LSL_11_PostIndex[] = {
  0x87, 0xc5, 0x0a, 0xe6 // str al r12 r10 minus r7 LSL 11 PostIndex
};
const byte kInstruction_str_al_r11_r2_minus_r8_LSL_13_PreIndex[] = {
  0x88, 0xb6, 0x22, 0xe7 // str al r11 r2 minus r8 LSL 13 PreIndex
};
const byte kInstruction_str_al_r13_r8_plus_r3_ROR_9_PreIndex[] = {
  0xe3, 0xd4, 0xa8, 0xe7 // str al r13 r8 plus r3 ROR 9 PreIndex
};
const byte kInstruction_str_al_r7_r4_minus_r10_ROR_19_PostIndex[] = {
  0xea, 0x79, 0x04, 0xe6 // str al r7 r4 minus r10 ROR 19 PostIndex
};
const byte kInstruction_str_al_r7_r4_plus_r11_LSL_8_PostIndex[] = {
  0x0b, 0x74, 0x84, 0xe6 // str al r7 r4 plus r11 LSL 8 PostIndex
};
const byte kInstruction_str_al_r7_r3_plus_r8_ROR_24_PostIndex[] = {
  0x68, 0x7c, 0x83, 0xe6 // str al r7 r3 plus r8 ROR 24 PostIndex
};
const byte kInstruction_str_al_r5_r14_plus_r12_LSL_9_PostIndex[] = {
  0x8c, 0x54, 0x8e, 0xe6 // str al r5 r14 plus r12 LSL 9 PostIndex
};
const byte kInstruction_str_al_r10_r1_minus_r5_LSL_17_PostIndex[] = {
  0x85, 0xa8, 0x01, 0xe6 // str al r10 r1 minus r5 LSL 17 PostIndex
};
const byte kInstruction_str_al_r10_r5_plus_r11_LSL_26_PostIndex[] = {
  0x0b, 0xad, 0x85, 0xe6 // str al r10 r5 plus r11 LSL 26 PostIndex
};
const byte kInstruction_str_al_r6_r0_plus_r2_ROR_29_PostIndex[] = {
  0xe2, 0x6e, 0x80, 0xe6 // str al r6 r0 plus r2 ROR 29 PostIndex
};
const byte kInstruction_str_al_r14_r4_plus_r11_LSL_4_PreIndex[] = {
  0x0b, 0xe2, 0xa4, 0xe7 // str al r14 r4 plus r11 LSL 4 PreIndex
};
const byte kInstruction_str_al_r5_r0_minus_r14_LSL_6_PreIndex[] = {
  0x0e, 0x53, 0x20, 0xe7 // str al r5 r0 minus r14 LSL 6 PreIndex
};
const byte kInstruction_str_al_r2_r14_plus_r6_LSL_17_PostIndex[] = {
  0x86, 0x28, 0x8e, 0xe6 // str al r2 r14 plus r6 LSL 17 PostIndex
};
const byte kInstruction_str_al_r14_r0_minus_r0_LSL_16_PostIndex[] = {
  0x00, 0xe8, 0x00, 0xe6 // str al r14 r0 minus r0 LSL 16 PostIndex
};
const byte kInstruction_str_al_r6_r9_minus_r9_ROR_7_PostIndex[] = {
  0xe9, 0x63, 0x09, 0xe6 // str al r6 r9 minus r9 ROR 7 PostIndex
};
const byte kInstruction_str_al_r10_r1_minus_r13_LSL_3_PreIndex[] = {
  0x8d, 0xa1, 0x21, 0xe7 // str al r10 r1 minus r13 LSL 3 PreIndex
};
const byte kInstruction_str_al_r13_r14_minus_r9_LSL_21_PreIndex[] = {
  0x89, 0xda, 0x2e, 0xe7 // str al r13 r14 minus r9 LSL 21 PreIndex
};
const byte kInstruction_str_al_r14_r11_plus_r8_LSL_20_PostIndex[] = {
  0x08, 0xea, 0x8b, 0xe6 // str al r14 r11 plus r8 LSL 20 PostIndex
};
const byte kInstruction_str_al_r1_r4_minus_r11_LSL_4_PreIndex[] = {
  0x0b, 0x12, 0x24, 0xe7 // str al r1 r4 minus r11 LSL 4 PreIndex
};
const byte kInstruction_str_al_r11_r14_minus_r5_LSL_24_PostIndex[] = {
  0x05, 0xbc, 0x0e, 0xe6 // str al r11 r14 minus r5 LSL 24 PostIndex
};
const byte kInstruction_str_al_r3_r1_minus_r0_LSL_8_PreIndex[] = {
  0x00, 0x34, 0x21, 0xe7 // str al r3 r1 minus r0 LSL 8 PreIndex
};
const byte kInstruction_str_al_r9_r13_plus_r12_LSL_3_PostIndex[] = {
  0x8c, 0x91, 0x8d, 0xe6 // str al r9 r13 plus r12 LSL 3 PostIndex
};
const byte kInstruction_str_al_r6_r13_plus_r5_ROR_7_PostIndex[] = {
  0xe5, 0x63, 0x8d, 0xe6 // str al r6 r13 plus r5 ROR 7 PostIndex
};
const byte kInstruction_str_al_r13_r4_minus_r7_ROR_19_PreIndex[] = {
  0xe7, 0xd9, 0x24, 0xe7 // str al r13 r4 minus r7 ROR 19 PreIndex
};
const byte kInstruction_str_al_r12_r7_minus_r9_ROR_2_PostIndex[] = {
  0x69, 0xc1, 0x07, 0xe6 // str al r12 r7 minus r9 ROR 2 PostIndex
};
const byte kInstruction_str_al_r12_r4_plus_r4_ROR_7_PreIndex[] = {
  0xe4, 0xc3, 0xa4, 0xe7 // str al r12 r4 plus r4 ROR 7 PreIndex
};
const byte kInstruction_str_al_r6_r8_minus_r7_LSL_1_PreIndex[] = {
  0x87, 0x60, 0x28, 0xe7 // str al r6 r8 minus r7 LSL 1 PreIndex
};
const byte kInstruction_str_al_r3_r7_minus_r2_LSL_16_PostIndex[] = {
  0x02, 0x38, 0x07, 0xe6 // str al r3 r7 minus r2 LSL 16 PostIndex
};
const byte kInstruction_str_al_r9_r6_plus_r4_ROR_16_PreIndex[] = {
  0x64, 0x98, 0xa6, 0xe7 // str al r9 r6 plus r4 ROR 16 PreIndex
};
const byte kInstruction_str_al_r8_r9_minus_r2_LSL_15_PostIndex[] = {
  0x82, 0x87, 0x09, 0xe6 // str al r8 r9 minus r2 LSL 15 PostIndex
};
const byte kInstruction_str_al_r0_r12_minus_r0_ROR_31_PostIndex[] = {
  0xe0, 0x0f, 0x0c, 0xe6 // str al r0 r12 minus r0 ROR 31 PostIndex
};
const byte kInstruction_str_al_r6_r9_plus_r0_LSL_15_PostIndex[] = {
  0x80, 0x67, 0x89, 0xe6 // str al r6 r9 plus r0 LSL 15 PostIndex
};
const byte kInstruction_str_al_r0_r2_minus_r13_ROR_5_PostIndex[] = {
  0xed, 0x02, 0x02, 0xe6 // str al r0 r2 minus r13 ROR 5 PostIndex
};
const byte kInstruction_str_al_r11_r4_minus_r5_ROR_30_PreIndex[] = {
  0x65, 0xbf, 0x24, 0xe7 // str al r11 r4 minus r5 ROR 30 PreIndex
};
const byte kInstruction_str_al_r2_r11_minus_r5_LSL_16_PostIndex[] = {
  0x05, 0x28, 0x0b, 0xe6 // str al r2 r11 minus r5 LSL 16 PostIndex
};
const byte kInstruction_str_al_r8_r2_minus_r0_ROR_25_PreIndex[] = {
  0xe0, 0x8c, 0x22, 0xe7 // str al r8 r2 minus r0 ROR 25 PreIndex
};
const byte kInstruction_str_al_r6_r1_minus_r7_LSL_11_PreIndex[] = {
  0x87, 0x65, 0x21, 0xe7 // str al r6 r1 minus r7 LSL 11 PreIndex
};
const byte kInstruction_str_al_r7_r11_minus_r5_LSL_4_PreIndex[] = {
  0x05, 0x72, 0x2b, 0xe7 // str al r7 r11 minus r5 LSL 4 PreIndex
};
const byte kInstruction_str_al_r6_r11_plus_r5_LSL_17_PostIndex[] = {
  0x85, 0x68, 0x8b, 0xe6 // str al r6 r11 plus r5 LSL 17 PostIndex
};
const byte kInstruction_str_al_r9_r4_minus_r2_LSL_29_PostIndex[] = {
  0x82, 0x9e, 0x04, 0xe6 // str al r9 r4 minus r2 LSL 29 PostIndex
};
const byte kInstruction_str_al_r14_r5_minus_r0_ROR_26_PostIndex[] = {
  0x60, 0xed, 0x05, 0xe6 // str al r14 r5 minus r0 ROR 26 PostIndex
};
const byte kInstruction_str_al_r5_r11_plus_r0_ROR_27_PostIndex[] = {
  0xe0, 0x5d, 0x8b, 0xe6 // str al r5 r11 plus r0 ROR 27 PostIndex
};
const byte kInstruction_str_al_r1_r9_minus_r11_ROR_8_PostIndex[] = {
  0x6b, 0x14, 0x09, 0xe6 // str al r1 r9 minus r11 ROR 8 PostIndex
};
const byte kInstruction_str_al_r7_r0_plus_r14_ROR_11_PostIndex[] = {
  0xee, 0x75, 0x80, 0xe6 // str al r7 r0 plus r14 ROR 11 PostIndex
};
const byte kInstruction_str_al_r7_r0_minus_r9_LSL_9_PreIndex[] = {
  0x89, 0x74, 0x20, 0xe7 // str al r7 r0 minus r9 LSL 9 PreIndex
};
const byte kInstruction_str_al_r4_r3_plus_r10_LSL_6_PostIndex[] = {
  0x0a, 0x43, 0x83, 0xe6 // str al r4 r3 plus r10 LSL 6 PostIndex
};
const byte kInstruction_str_al_r10_r0_minus_r5_ROR_14_PostIndex[] = {
  0x65, 0xa7, 0x00, 0xe6 // str al r10 r0 minus r5 ROR 14 PostIndex
};
const byte kInstruction_str_al_r1_r2_plus_r13_LSL_17_PostIndex[] = {
  0x8d, 0x18, 0x82, 0xe6 // str al r1 r2 plus r13 LSL 17 PostIndex
};
const byte kInstruction_str_al_r11_r2_plus_r9_ROR_22_PostIndex[] = {
  0x69, 0xbb, 0x82, 0xe6 // str al r11 r2 plus r9 ROR 22 PostIndex
};
const byte kInstruction_str_al_r2_r9_plus_r3_LSL_19_PostIndex[] = {
  0x83, 0x29, 0x89, 0xe6 // str al r2 r9 plus r3 LSL 19 PostIndex
};
const byte kInstruction_str_al_r7_r3_plus_r4_ROR_23_PostIndex[] = {
  0xe4, 0x7b, 0x83, 0xe6 // str al r7 r3 plus r4 ROR 23 PostIndex
};
const byte kInstruction_str_al_r2_r9_plus_r11_LSL_8_PostIndex[] = {
  0x0b, 0x24, 0x89, 0xe6 // str al r2 r9 plus r11 LSL 8 PostIndex
};
const byte kInstruction_str_al_r5_r13_minus_r1_ROR_15_PreIndex[] = {
  0xe1, 0x57, 0x2d, 0xe7 // str al r5 r13 minus r1 ROR 15 PreIndex
};
const byte kInstruction_str_al_r8_r12_plus_r2_LSL_15_PreIndex[] = {
  0x82, 0x87, 0xac, 0xe7 // str al r8 r12 plus r2 LSL 15 PreIndex
};
const byte kInstruction_str_al_r7_r1_plus_r13_LSL_24_PreIndex[] = {
  0x0d, 0x7c, 0xa1, 0xe7 // str al r7 r1 plus r13 LSL 24 PreIndex
};
const byte kInstruction_str_al_r0_r9_minus_r1_ROR_13_PostIndex[] = {
  0xe1, 0x06, 0x09, 0xe6 // str al r0 r9 minus r1 ROR 13 PostIndex
};
const byte kInstruction_str_al_r5_r9_minus_r1_LSL_1_PostIndex[] = {
  0x81, 0x50, 0x09, 0xe6 // str al r5 r9 minus r1 LSL 1 PostIndex
};
const byte kInstruction_str_al_r3_r4_minus_r2_LSL_4_PreIndex[] = {
  0x02, 0x32, 0x24, 0xe7 // str al r3 r4 minus r2 LSL 4 PreIndex
};
const byte kInstruction_str_al_r3_r10_minus_r6_ROR_5_PreIndex[] = {
  0xe6, 0x32, 0x2a, 0xe7 // str al r3 r10 minus r6 ROR 5 PreIndex
};
const byte kInstruction_str_al_r2_r14_minus_r6_ROR_16_PreIndex[] = {
  0x66, 0x28, 0x2e, 0xe7 // str al r2 r14 minus r6 ROR 16 PreIndex
};
const byte kInstruction_str_al_r2_r1_plus_r14_ROR_3_PostIndex[] = {
  0xee, 0x21, 0x81, 0xe6 // str al r2 r1 plus r14 ROR 3 PostIndex
};
const byte kInstruction_str_al_r6_r2_plus_r2_LSL_22_PreIndex[] = {
  0x02, 0x6b, 0xa2, 0xe7 // str al r6 r2 plus r2 LSL 22 PreIndex
};
const byte kInstruction_str_al_r5_r7_plus_r4_ROR_3_PostIndex[] = {
  0xe4, 0x51, 0x87, 0xe6 // str al r5 r7 plus r4 ROR 3 PostIndex
};
const byte kInstruction_str_al_r12_r9_plus_r6_ROR_3_PreIndex[] = {
  0xe6, 0xc1, 0xa9, 0xe7 // str al r12 r9 plus r6 ROR 3 PreIndex
};
const byte kInstruction_str_al_r3_r7_plus_r10_ROR_29_PreIndex[] = {
  0xea, 0x3e, 0xa7, 0xe7 // str al r3 r7 plus r10 ROR 29 PreIndex
};
const byte kInstruction_str_al_r2_r0_minus_r9_ROR_9_PreIndex[] = {
  0xe9, 0x24, 0x20, 0xe7 // str al r2 r0 minus r9 ROR 9 PreIndex
};
const byte kInstruction_str_al_r7_r12_plus_r7_LSL_3_PreIndex[] = {
  0x87, 0x71, 0xac, 0xe7 // str al r7 r12 plus r7 LSL 3 PreIndex
};
const byte kInstruction_str_al_r7_r5_plus_r10_LSL_15_PostIndex[] = {
  0x8a, 0x77, 0x85, 0xe6 // str al r7 r5 plus r10 LSL 15 PostIndex
};
const byte kInstruction_str_al_r14_r4_minus_r6_ROR_13_PreIndex[] = {
  0xe6, 0xe6, 0x24, 0xe7 // str al r14 r4 minus r6 ROR 13 PreIndex
};
const byte kInstruction_str_al_r0_r6_plus_r7_ROR_27_PreIndex[] = {
  0xe7, 0x0d, 0xa6, 0xe7 // str al r0 r6 plus r7 ROR 27 PreIndex
};
const byte kInstruction_str_al_r0_r14_plus_r13_ROR_23_PreIndex[] = {
  0xed, 0x0b, 0xae, 0xe7 // str al r0 r14 plus r13 ROR 23 PreIndex
};
const byte kInstruction_str_al_r9_r5_plus_r1_LSL_8_PreIndex[] = {
  0x01, 0x94, 0xa5, 0xe7 // str al r9 r5 plus r1 LSL 8 PreIndex
};
const byte kInstruction_str_al_r8_r3_plus_r7_LSL_12_PostIndex[] = {
  0x07, 0x86, 0x83, 0xe6 // str al r8 r3 plus r7 LSL 12 PostIndex
};
const byte kInstruction_str_al_r2_r5_plus_r3_LSL_16_PostIndex[] = {
  0x03, 0x28, 0x85, 0xe6 // str al r2 r5 plus r3 LSL 16 PostIndex
};
const byte kInstruction_str_al_r4_r12_minus_r7_LSL_8_PostIndex[] = {
  0x07, 0x44, 0x0c, 0xe6 // str al r4 r12 minus r7 LSL 8 PostIndex
};
const byte kInstruction_str_al_r0_r7_minus_r5_LSL_6_PostIndex[] = {
  0x05, 0x03, 0x07, 0xe6 // str al r0 r7 minus r5 LSL 6 PostIndex
};
const byte kInstruction_str_al_r9_r4_plus_r12_ROR_14_PreIndex[] = {
  0x6c, 0x97, 0xa4, 0xe7 // str al r9 r4 plus r12 ROR 14 PreIndex
};
const byte kInstruction_str_al_r6_r2_minus_r14_LSL_8_PostIndex[] = {
  0x0e, 0x64, 0x02, 0xe6 // str al r6 r2 minus r14 LSL 8 PostIndex
};
const byte kInstruction_str_al_r6_r12_minus_r9_ROR_14_PostIndex[] = {
  0x69, 0x67, 0x0c, 0xe6 // str al r6 r12 minus r9 ROR 14 PostIndex
};
const byte kInstruction_str_al_r13_r7_minus_r9_LSL_3_PreIndex[] = {
  0x89, 0xd1, 0x27, 0xe7 // str al r13 r7 minus r9 LSL 3 PreIndex
};
const byte kInstruction_str_al_r13_r1_plus_r13_ROR_20_PostIndex[] = {
  0x6d, 0xda, 0x81, 0xe6 // str al r13 r1 plus r13 ROR 20 PostIndex
};
const byte kInstruction_str_al_r8_r11_minus_r6_ROR_4_PreIndex[] = {
  0x66, 0x82, 0x2b, 0xe7 // str al r8 r11 minus r6 ROR 4 PreIndex
};
const byte kInstruction_str_al_r0_r14_plus_r8_ROR_17_PostIndex[] = {
  0xe8, 0x08, 0x8e, 0xe6 // str al r0 r14 plus r8 ROR 17 PostIndex
};
const byte kInstruction_str_al_r12_r9_plus_r10_LSL_9_PreIndex[] = {
  0x8a, 0xc4, 0xa9, 0xe7 // str al r12 r9 plus r10 LSL 9 PreIndex
};
const byte kInstruction_str_al_r10_r13_plus_r1_LSL_18_PostIndex[] = {
  0x01, 0xa9, 0x8d, 0xe6 // str al r10 r13 plus r1 LSL 18 PostIndex
};
const byte kInstruction_str_al_r10_r9_plus_r3_LSL_13_PreIndex[] = {
  0x83, 0xa6, 0xa9, 0xe7 // str al r10 r9 plus r3 LSL 13 PreIndex
};
const byte kInstruction_str_al_r8_r14_plus_r5_LSL_19_PreIndex[] = {
  0x85, 0x89, 0xae, 0xe7 // str al r8 r14 plus r5 LSL 19 PreIndex
};
const byte kInstruction_str_al_r2_r10_minus_r6_ROR_21_PostIndex[] = {
  0xe6, 0x2a, 0x0a, 0xe6 // str al r2 r10 minus r6 ROR 21 PostIndex
};
const byte kInstruction_str_al_r9_r6_plus_r5_LSL_3_PreIndex[] = {
  0x85, 0x91, 0xa6, 0xe7 // str al r9 r6 plus r5 LSL 3 PreIndex
};
const byte kInstruction_str_al_r13_r10_plus_r4_ROR_29_PreIndex[] = {
  0xe4, 0xde, 0xaa, 0xe7 // str al r13 r10 plus r4 ROR 29 PreIndex
};
const byte kInstruction_str_al_r3_r4_plus_r5_LSL_22_PostIndex[] = {
  0x05, 0x3b, 0x84, 0xe6 // str al r3 r4 plus r5 LSL 22 PostIndex
};
const byte kInstruction_str_al_r0_r5_minus_r10_LSL_11_PreIndex[] = {
  0x8a, 0x05, 0x25, 0xe7 // str al r0 r5 minus r10 LSL 11 PreIndex
};
const byte kInstruction_str_al_r11_r12_minus_r14_ROR_30_PostIndex[] = {
  0x6e, 0xbf, 0x0c, 0xe6 // str al r11 r12 minus r14 ROR 30 PostIndex
};
const byte kInstruction_str_al_r12_r14_minus_r10_LSL_23_PostIndex[] = {
  0x8a, 0xcb, 0x0e, 0xe6 // str al r12 r14 minus r10 LSL 23 PostIndex
};
const byte kInstruction_str_al_r3_r7_plus_r5_ROR_5_PostIndex[] = {
  0xe5, 0x32, 0x87, 0xe6 // str al r3 r7 plus r5 ROR 5 PostIndex
};
const byte kInstruction_str_al_r1_r0_minus_r14_LSL_23_PostIndex[] = {
  0x8e, 0x1b, 0x00, 0xe6 // str al r1 r0 minus r14 LSL 23 PostIndex
};
const byte kInstruction_str_al_r14_r0_plus_r13_LSL_16_PostIndex[] = {
  0x0d, 0xe8, 0x80, 0xe6 // str al r14 r0 plus r13 LSL 16 PostIndex
};
const byte kInstruction_str_al_r1_r7_minus_r10_ROR_31_PreIndex[] = {
  0xea, 0x1f, 0x27, 0xe7 // str al r1 r7 minus r10 ROR 31 PreIndex
};
const byte kInstruction_str_al_r14_r10_minus_r12_LSL_12_PostIndex[] = {
  0x0c, 0xe6, 0x0a, 0xe6 // str al r14 r10 minus r12 LSL 12 PostIndex
};
const byte kInstruction_str_al_r14_r9_minus_r9_ROR_18_PreIndex[] = {
  0x69, 0xe9, 0x29, 0xe7 // str al r14 r9 minus r9 ROR 18 PreIndex
};
const byte kInstruction_str_al_r10_r12_plus_r2_ROR_15_PreIndex[] = {
  0xe2, 0xa7, 0xac, 0xe7 // str al r10 r12 plus r2 ROR 15 PreIndex
};
const byte kInstruction_str_al_r13_r14_minus_r0_LSL_11_PreIndex[] = {
  0x80, 0xd5, 0x2e, 0xe7 // str al r13 r14 minus r0 LSL 11 PreIndex
};
const byte kInstruction_str_al_r3_r5_plus_r6_ROR_2_PreIndex[] = {
  0x66, 0x31, 0xa5, 0xe7 // str al r3 r5 plus r6 ROR 2 PreIndex
};
const byte kInstruction_str_al_r9_r3_minus_r14_ROR_27_PostIndex[] = {
  0xee, 0x9d, 0x03, 0xe6 // str al r9 r3 minus r14 ROR 27 PostIndex
};
const byte kInstruction_str_al_r2_r10_minus_r7_ROR_19_PostIndex[] = {
  0xe7, 0x29, 0x0a, 0xe6 // str al r2 r10 minus r7 ROR 19 PostIndex
};
const byte kInstruction_str_al_r12_r5_plus_r11_LSL_29_PostIndex[] = {
  0x8b, 0xce, 0x85, 0xe6 // str al r12 r5 plus r11 LSL 29 PostIndex
};
const byte kInstruction_str_al_r2_r9_minus_r12_ROR_13_PreIndex[] = {
  0xec, 0x26, 0x29, 0xe7 // str al r2 r9 minus r12 ROR 13 PreIndex
};
const byte kInstruction_str_al_r11_r1_plus_r13_LSL_26_PreIndex[] = {
  0x0d, 0xbd, 0xa1, 0xe7 // str al r11 r1 plus r13 LSL 26 PreIndex
};
const byte kInstruction_str_al_r12_r0_minus_r1_ROR_3_PreIndex[] = {
  0xe1, 0xc1, 0x20, 0xe7 // str al r12 r0 minus r1 ROR 3 PreIndex
};
const byte kInstruction_str_al_r11_r14_plus_r4_LSL_19_PreIndex[] = {
  0x84, 0xb9, 0xae, 0xe7 // str al r11 r14 plus r4 LSL 19 PreIndex
};
const byte kInstruction_str_al_r8_r12_plus_r3_ROR_30_PreIndex[] = {
  0x63, 0x8f, 0xac, 0xe7 // str al r8 r12 plus r3 ROR 30 PreIndex
};
const byte kInstruction_str_al_r6_r0_plus_r6_LSL_24_PostIndex[] = {
  0x06, 0x6c, 0x80, 0xe6 // str al r6 r0 plus r6 LSL 24 PostIndex
};
const byte kInstruction_str_al_r4_r6_minus_r11_ROR_16_PreIndex[] = {
  0x6b, 0x48, 0x26, 0xe7 // str al r4 r6 minus r11 ROR 16 PreIndex
};
const byte kInstruction_str_al_r8_r0_plus_r11_ROR_8_PostIndex[] = {
  0x6b, 0x84, 0x80, 0xe6 // str al r8 r0 plus r11 ROR 8 PostIndex
};
const byte kInstruction_str_al_r7_r5_plus_r1_ROR_25_PostIndex[] = {
  0xe1, 0x7c, 0x85, 0xe6 // str al r7 r5 plus r1 ROR 25 PostIndex
};
const byte kInstruction_str_al_r1_r12_plus_r11_ROR_12_PreIndex[] = {
  0x6b, 0x16, 0xac, 0xe7 // str al r1 r12 plus r11 ROR 12 PreIndex
};
const byte kInstruction_str_al_r4_r2_plus_r14_ROR_8_PostIndex[] = {
  0x6e, 0x44, 0x82, 0xe6 // str al r4 r2 plus r14 ROR 8 PostIndex
};
const byte kInstruction_str_al_r5_r0_plus_r1_LSL_10_PostIndex[] = {
  0x01, 0x55, 0x80, 0xe6 // str al r5 r0 plus r1 LSL 10 PostIndex
};
const byte kInstruction_str_al_r0_r11_minus_r4_ROR_30_PostIndex[] = {
  0x64, 0x0f, 0x0b, 0xe6 // str al r0 r11 minus r4 ROR 30 PostIndex
};
const byte kInstruction_str_al_r14_r3_minus_r7_LSL_2_PostIndex[] = {
  0x07, 0xe1, 0x03, 0xe6 // str al r14 r3 minus r7 LSL 2 PostIndex
};
const byte kInstruction_str_al_r4_r8_plus_r10_LSL_26_PostIndex[] = {
  0x0a, 0x4d, 0x88, 0xe6 // str al r4 r8 plus r10 LSL 26 PostIndex
};
const byte kInstruction_str_al_r12_r2_minus_r8_LSL_21_PreIndex[] = {
  0x88, 0xca, 0x22, 0xe7 // str al r12 r2 minus r8 LSL 21 PreIndex
};
const byte kInstruction_str_al_r9_r14_minus_r14_LSL_24_PreIndex[] = {
  0x0e, 0x9c, 0x2e, 0xe7 // str al r9 r14 minus r14 LSL 24 PreIndex
};
const byte kInstruction_str_al_r7_r13_minus_r11_LSL_2_PostIndex[] = {
  0x0b, 0x71, 0x0d, 0xe6 // str al r7 r13 minus r11 LSL 2 PostIndex
};
const byte kInstruction_str_al_r0_r2_plus_r0_ROR_30_PreIndex[] = {
  0x60, 0x0f, 0xa2, 0xe7 // str al r0 r2 plus r0 ROR 30 PreIndex
};
const byte kInstruction_str_al_r1_r6_minus_r0_ROR_1_PreIndex[] = {
  0xe0, 0x10, 0x26, 0xe7 // str al r1 r6 minus r0 ROR 1 PreIndex
};
const byte kInstruction_str_al_r6_r5_plus_r12_LSL_4_PostIndex[] = {
  0x0c, 0x62, 0x85, 0xe6 // str al r6 r5 plus r12 LSL 4 PostIndex
};
const byte kInstruction_str_al_r7_r9_plus_r13_LSL_11_PostIndex[] = {
  0x8d, 0x75, 0x89, 0xe6 // str al r7 r9 plus r13 LSL 11 PostIndex
};
const byte kInstruction_str_al_r5_r12_minus_r2_ROR_5_PostIndex[] = {
  0xe2, 0x52, 0x0c, 0xe6 // str al r5 r12 minus r2 ROR 5 PostIndex
};
const byte kInstruction_str_al_r6_r7_plus_r14_LSL_30_PreIndex[] = {
  0x0e, 0x6f, 0xa7, 0xe7 // str al r6 r7 plus r14 LSL 30 PreIndex
};
const byte kInstruction_str_al_r2_r11_minus_r0_ROR_20_PostIndex[] = {
  0x60, 0x2a, 0x0b, 0xe6 // str al r2 r11 minus r0 ROR 20 PostIndex
};
const byte kInstruction_str_al_r6_r2_minus_r10_ROR_11_PostIndex[] = {
  0xea, 0x65, 0x02, 0xe6 // str al r6 r2 minus r10 ROR 11 PostIndex
};
const byte kInstruction_str_al_r3_r2_plus_r0_ROR_4_PostIndex[] = {
  0x60, 0x32, 0x82, 0xe6 // str al r3 r2 plus r0 ROR 4 PostIndex
};
const byte kInstruction_str_al_r8_r6_plus_r12_LSL_11_PostIndex[] = {
  0x8c, 0x85, 0x86, 0xe6 // str al r8 r6 plus r12 LSL 11 PostIndex
};
const byte kInstruction_str_al_r0_r6_minus_r4_ROR_8_PreIndex[] = {
  0x64, 0x04, 0x26, 0xe7 // str al r0 r6 minus r4 ROR 8 PreIndex
};
const byte kInstruction_str_al_r10_r1_minus_r3_ROR_5_PostIndex[] = {
  0xe3, 0xa2, 0x01, 0xe6 // str al r10 r1 minus r3 ROR 5 PostIndex
};
const byte kInstruction_str_al_r10_r4_plus_r12_LSL_9_PreIndex[] = {
  0x8c, 0xa4, 0xa4, 0xe7 // str al r10 r4 plus r12 LSL 9 PreIndex
};
const byte kInstruction_str_al_r1_r9_minus_r5_ROR_7_PostIndex[] = {
  0xe5, 0x13, 0x09, 0xe6 // str al r1 r9 minus r5 ROR 7 PostIndex
};
const byte kInstruction_str_al_r5_r7_plus_r1_ROR_14_PreIndex[] = {
  0x61, 0x57, 0xa7, 0xe7 // str al r5 r7 plus r1 ROR 14 PreIndex
};
const byte kInstruction_str_al_r9_r1_plus_r2_LSL_20_PreIndex[] = {
  0x02, 0x9a, 0xa1, 0xe7 // str al r9 r1 plus r2 LSL 20 PreIndex
};
const byte kInstruction_str_al_r6_r8_minus_r5_ROR_13_PostIndex[] = {
  0xe5, 0x66, 0x08, 0xe6 // str al r6 r8 minus r5 ROR 13 PostIndex
};
const byte kInstruction_str_al_r4_r8_plus_r1_LSL_17_PreIndex[] = {
  0x81, 0x48, 0xa8, 0xe7 // str al r4 r8 plus r1 LSL 17 PreIndex
};
const byte kInstruction_str_al_r11_r13_plus_r6_LSL_31_PostIndex[] = {
  0x86, 0xbf, 0x8d, 0xe6 // str al r11 r13 plus r6 LSL 31 PostIndex
};
const byte kInstruction_str_al_r9_r0_minus_r4_LSL_7_PostIndex[] = {
  0x84, 0x93, 0x00, 0xe6 // str al r9 r0 minus r4 LSL 7 PostIndex
};
const byte kInstruction_str_al_r4_r1_minus_r6_LSL_11_PreIndex[] = {
  0x86, 0x45, 0x21, 0xe7 // str al r4 r1 minus r6 LSL 11 PreIndex
};
const byte kInstruction_str_al_r10_r13_plus_r11_ROR_14_PostIndex[] = {
  0x6b, 0xa7, 0x8d, 0xe6 // str al r10 r13 plus r11 ROR 14 PostIndex
};
const byte kInstruction_str_al_r12_r11_plus_r9_ROR_31_PreIndex[] = {
  0xe9, 0xcf, 0xab, 0xe7 // str al r12 r11 plus r9 ROR 31 PreIndex
};
const byte kInstruction_str_al_r9_r1_minus_r2_ROR_5_PreIndex[] = {
  0xe2, 0x92, 0x21, 0xe7 // str al r9 r1 minus r2 ROR 5 PreIndex
};
const byte kInstruction_str_al_r1_r10_minus_r4_LSL_18_PostIndex[] = {
  0x04, 0x19, 0x0a, 0xe6 // str al r1 r10 minus r4 LSL 18 PostIndex
};
const byte kInstruction_str_al_r5_r14_plus_r0_ROR_24_PostIndex[] = {
  0x60, 0x5c, 0x8e, 0xe6 // str al r5 r14 plus r0 ROR 24 PostIndex
};
const byte kInstruction_str_al_r9_r10_plus_r1_ROR_22_PreIndex[] = {
  0x61, 0x9b, 0xaa, 0xe7 // str al r9 r10 plus r1 ROR 22 PreIndex
};
const byte kInstruction_str_al_r12_r0_plus_r9_LSL_9_PostIndex[] = {
  0x89, 0xc4, 0x80, 0xe6 // str al r12 r0 plus r9 LSL 9 PostIndex
};
const byte kInstruction_str_al_r3_r11_plus_r11_ROR_22_PreIndex[] = {
  0x6b, 0x3b, 0xab, 0xe7 // str al r3 r11 plus r11 ROR 22 PreIndex
};
const byte kInstruction_str_al_r0_r4_minus_r12_LSL_23_PostIndex[] = {
  0x8c, 0x0b, 0x04, 0xe6 // str al r0 r4 minus r12 LSL 23 PostIndex
};
const byte kInstruction_str_al_r13_r3_minus_r6_ROR_6_PostIndex[] = {
  0x66, 0xd3, 0x03, 0xe6 // str al r13 r3 minus r6 ROR 6 PostIndex
};
const byte kInstruction_str_al_r10_r2_plus_r8_ROR_10_PostIndex[] = {
  0x68, 0xa5, 0x82, 0xe6 // str al r10 r2 plus r8 ROR 10 PostIndex
};
const byte kInstruction_str_al_r6_r13_minus_r6_ROR_19_PostIndex[] = {
  0xe6, 0x69, 0x0d, 0xe6 // str al r6 r13 minus r6 ROR 19 PostIndex
};
const byte kInstruction_str_al_r4_r3_plus_r5_LSL_20_PostIndex[] = {
  0x05, 0x4a, 0x83, 0xe6 // str al r4 r3 plus r5 LSL 20 PostIndex
};
const byte kInstruction_str_al_r1_r12_minus_r6_ROR_28_PostIndex[] = {
  0x66, 0x1e, 0x0c, 0xe6 // str al r1 r12 minus r6 ROR 28 PostIndex
};
const byte kInstruction_str_al_r2_r8_plus_r7_ROR_17_PreIndex[] = {
  0xe7, 0x28, 0xa8, 0xe7 // str al r2 r8 plus r7 ROR 17 PreIndex
};
const byte kInstruction_str_al_r2_r5_minus_r10_ROR_3_PreIndex[] = {
  0xea, 0x21, 0x25, 0xe7 // str al r2 r5 minus r10 ROR 3 PreIndex
};
const byte kInstruction_str_al_r9_r14_minus_r0_LSL_16_PreIndex[] = {
  0x00, 0x98, 0x2e, 0xe7 // str al r9 r14 minus r0 LSL 16 PreIndex
};
const byte kInstruction_str_al_r6_r3_minus_r10_ROR_23_PreIndex[] = {
  0xea, 0x6b, 0x23, 0xe7 // str al r6 r3 minus r10 ROR 23 PreIndex
};
const byte kInstruction_str_al_r0_r9_minus_r8_LSL_11_PostIndex[] = {
  0x88, 0x05, 0x09, 0xe6 // str al r0 r9 minus r8 LSL 11 PostIndex
};
const byte kInstruction_str_al_r9_r2_plus_r14_LSL_9_PreIndex[] = {
  0x8e, 0x94, 0xa2, 0xe7 // str al r9 r2 plus r14 LSL 9 PreIndex
};
const byte kInstruction_str_al_r13_r7_minus_r6_LSL_19_PreIndex[] = {
  0x86, 0xd9, 0x27, 0xe7 // str al r13 r7 minus r6 LSL 19 PreIndex
};
const byte kInstruction_str_al_r13_r4_minus_r2_LSL_10_PostIndex[] = {
  0x02, 0xd5, 0x04, 0xe6 // str al r13 r4 minus r2 LSL 10 PostIndex
};
const byte kInstruction_str_al_r9_r11_minus_r0_LSL_3_PostIndex[] = {
  0x80, 0x91, 0x0b, 0xe6 // str al r9 r11 minus r0 LSL 3 PostIndex
};
const byte kInstruction_str_al_r11_r13_plus_r8_LSL_28_PreIndex[] = {
  0x08, 0xbe, 0xad, 0xe7 // str al r11 r13 plus r8 LSL 28 PreIndex
};
const byte kInstruction_str_al_r7_r9_minus_r7_ROR_13_PreIndex[] = {
  0xe7, 0x76, 0x29, 0xe7 // str al r7 r9 minus r7 ROR 13 PreIndex
};
const byte kInstruction_str_al_r13_r7_plus_r5_LSL_3_PostIndex[] = {
  0x85, 0xd1, 0x87, 0xe6 // str al r13 r7 plus r5 LSL 3 PostIndex
};
const byte kInstruction_str_al_r14_r10_plus_r14_ROR_1_PostIndex[] = {
  0xee, 0xe0, 0x8a, 0xe6 // str al r14 r10 plus r14 ROR 1 PostIndex
};
const byte kInstruction_str_al_r1_r6_plus_r14_ROR_3_PostIndex[] = {
  0xee, 0x11, 0x86, 0xe6 // str al r1 r6 plus r14 ROR 3 PostIndex
};
const byte kInstruction_str_al_r12_r14_minus_r11_ROR_17_PreIndex[] = {
  0xeb, 0xc8, 0x2e, 0xe7 // str al r12 r14 minus r11 ROR 17 PreIndex
};
const byte kInstruction_str_al_r4_r5_minus_r10_LSL_3_PostIndex[] = {
  0x8a, 0x41, 0x05, 0xe6 // str al r4 r5 minus r10 LSL 3 PostIndex
};
const byte kInstruction_str_al_r0_r14_minus_r5_ROR_16_PreIndex[] = {
  0x65, 0x08, 0x2e, 0xe7 // str al r0 r14 minus r5 ROR 16 PreIndex
};
const TestResult kReferencestr[] = {
  {
    ARRAY_SIZE(kInstruction_str_pl_r8_r11_plus_r6_LSL_1_Offset),
    kInstruction_str_pl_r8_r11_plus_r6_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_le_r4_r8_plus_r5_LSL_1_Offset),
    kInstruction_str_le_r4_r8_plus_r5_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_vs_r2_r6_plus_r14_LSL_1_Offset),
    kInstruction_str_vs_r2_r6_plus_r14_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ls_r1_r7_plus_r8_LSL_1_Offset),
    kInstruction_str_ls_r1_r7_plus_r8_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ge_r14_r6_plus_r14_LSL_1_Offset),
    kInstruction_str_ge_r14_r6_plus_r14_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cs_r7_r0_plus_r7_LSL_1_Offset),
    kInstruction_str_cs_r7_r0_plus_r7_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ge_r11_r0_plus_r9_LSL_1_Offset),
    kInstruction_str_ge_r11_r0_plus_r9_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_eq_r7_r10_plus_r4_LSL_1_Offset),
    kInstruction_str_eq_r7_r10_plus_r4_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r2_plus_r3_LSL_1_Offset),
    kInstruction_str_al_r9_r2_plus_r3_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cc_r11_r10_plus_r6_LSL_1_Offset),
    kInstruction_str_cc_r11_r10_plus_r6_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_lt_r9_r6_plus_r1_LSL_1_Offset),
    kInstruction_str_lt_r9_r6_plus_r1_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ge_r4_r0_plus_r4_LSL_1_Offset),
    kInstruction_str_ge_r4_r0_plus_r4_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r13_plus_r14_LSL_1_Offset),
    kInstruction_str_al_r9_r13_plus_r14_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cc_r9_r0_plus_r7_LSL_1_Offset),
    kInstruction_str_cc_r9_r0_plus_r7_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cs_r11_r7_plus_r8_LSL_1_Offset),
    kInstruction_str_cs_r11_r7_plus_r8_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_eq_r8_r0_plus_r8_LSL_1_Offset),
    kInstruction_str_eq_r8_r0_plus_r8_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_hi_r9_r10_plus_r8_LSL_1_Offset),
    kInstruction_str_hi_r9_r10_plus_r8_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_le_r8_r0_plus_r0_LSL_1_Offset),
    kInstruction_str_le_r8_r0_plus_r0_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cs_r4_r5_plus_r2_LSL_1_Offset),
    kInstruction_str_cs_r4_r5_plus_r2_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_eq_r14_r11_plus_r8_LSL_1_Offset),
    kInstruction_str_eq_r14_r11_plus_r8_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cs_r7_r6_plus_r14_LSL_1_Offset),
    kInstruction_str_cs_r7_r6_plus_r14_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ne_r0_r9_plus_r7_LSL_1_Offset),
    kInstruction_str_ne_r0_r9_plus_r7_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_le_r7_r3_plus_r11_LSL_1_Offset),
    kInstruction_str_le_r7_r3_plus_r11_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_gt_r7_r9_plus_r9_LSL_1_Offset),
    kInstruction_str_gt_r7_r9_plus_r9_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_le_r9_r12_plus_r9_LSL_1_Offset),
    kInstruction_str_le_r9_r12_plus_r9_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_pl_r2_r11_plus_r14_LSL_1_Offset),
    kInstruction_str_pl_r2_r11_plus_r14_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cs_r1_r7_plus_r2_LSL_1_Offset),
    kInstruction_str_cs_r1_r7_plus_r2_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r1_plus_r12_LSL_1_Offset),
    kInstruction_str_al_r5_r1_plus_r12_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_eq_r10_r13_plus_r12_LSL_1_Offset),
    kInstruction_str_eq_r10_r13_plus_r12_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_eq_r10_r11_plus_r3_LSL_1_Offset),
    kInstruction_str_eq_r10_r11_plus_r3_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r12_plus_r0_LSL_1_Offset),
    kInstruction_str_al_r9_r12_plus_r0_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ls_r3_r14_plus_r1_LSL_1_Offset),
    kInstruction_str_ls_r3_r14_plus_r1_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_le_r13_r4_plus_r9_LSL_1_Offset),
    kInstruction_str_le_r13_r4_plus_r9_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ls_r3_r14_plus_r3_LSL_1_Offset),
    kInstruction_str_ls_r3_r14_plus_r3_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cc_r7_r4_plus_r4_LSL_1_Offset),
    kInstruction_str_cc_r7_r4_plus_r4_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ls_r5_r9_plus_r5_LSL_1_Offset),
    kInstruction_str_ls_r5_r9_plus_r5_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ge_r6_r4_plus_r13_LSL_1_Offset),
    kInstruction_str_ge_r6_r4_plus_r13_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r11_plus_r12_LSL_1_Offset),
    kInstruction_str_al_r1_r11_plus_r12_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_le_r6_r11_plus_r3_LSL_1_Offset),
    kInstruction_str_le_r6_r11_plus_r3_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r6_plus_r13_LSL_1_Offset),
    kInstruction_str_al_r6_r6_plus_r13_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_gt_r14_r11_plus_r2_LSL_1_Offset),
    kInstruction_str_gt_r14_r11_plus_r2_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_vs_r5_r8_plus_r12_LSL_1_Offset),
    kInstruction_str_vs_r5_r8_plus_r12_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ls_r8_r12_plus_r6_LSL_1_Offset),
    kInstruction_str_ls_r8_r12_plus_r6_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_vs_r6_r12_plus_r14_LSL_1_Offset),
    kInstruction_str_vs_r6_r12_plus_r14_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_pl_r7_r2_plus_r7_LSL_1_Offset),
    kInstruction_str_pl_r7_r2_plus_r7_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_gt_r5_r4_plus_r13_LSL_1_Offset),
    kInstruction_str_gt_r5_r4_plus_r13_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_hi_r2_r7_plus_r12_LSL_1_Offset),
    kInstruction_str_hi_r2_r7_plus_r12_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ne_r2_r3_plus_r10_LSL_1_Offset),
    kInstruction_str_ne_r2_r3_plus_r10_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_lt_r13_r2_plus_r6_LSL_1_Offset),
    kInstruction_str_lt_r13_r2_plus_r6_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_hi_r14_r10_plus_r11_LSL_1_Offset),
    kInstruction_str_hi_r14_r10_plus_r11_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_hi_r7_r10_plus_r12_LSL_1_Offset),
    kInstruction_str_hi_r7_r10_plus_r12_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cs_r3_r12_plus_r2_LSL_1_Offset),
    kInstruction_str_cs_r3_r12_plus_r2_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ls_r11_r5_plus_r1_LSL_1_Offset),
    kInstruction_str_ls_r11_r5_plus_r1_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_hi_r5_r2_plus_r11_LSL_1_Offset),
    kInstruction_str_hi_r5_r2_plus_r11_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cc_r11_r8_plus_r6_LSL_1_Offset),
    kInstruction_str_cc_r11_r8_plus_r6_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_lt_r11_r3_plus_r9_LSL_1_Offset),
    kInstruction_str_lt_r11_r3_plus_r9_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ne_r11_r9_plus_r8_LSL_1_Offset),
    kInstruction_str_ne_r11_r9_plus_r8_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_lt_r7_r0_plus_r14_LSL_1_Offset),
    kInstruction_str_lt_r7_r0_plus_r14_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ge_r12_r3_plus_r1_LSL_1_Offset),
    kInstruction_str_ge_r12_r3_plus_r1_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_le_r0_r8_plus_r13_LSL_1_Offset),
    kInstruction_str_le_r0_r8_plus_r13_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_vc_r5_r13_plus_r7_LSL_1_Offset),
    kInstruction_str_vc_r5_r13_plus_r7_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ge_r7_r5_plus_r11_LSL_1_Offset),
    kInstruction_str_ge_r7_r5_plus_r11_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ge_r10_r9_plus_r3_LSL_1_Offset),
    kInstruction_str_ge_r10_r9_plus_r3_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_vs_r9_r5_plus_r2_LSL_1_Offset),
    kInstruction_str_vs_r9_r5_plus_r2_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_hi_r2_r10_plus_r4_LSL_1_Offset),
    kInstruction_str_hi_r2_r10_plus_r4_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ge_r1_r11_plus_r1_LSL_1_Offset),
    kInstruction_str_ge_r1_r11_plus_r1_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ls_r9_r14_plus_r12_LSL_1_Offset),
    kInstruction_str_ls_r9_r14_plus_r12_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_mi_r1_r9_plus_r4_LSL_1_Offset),
    kInstruction_str_mi_r1_r9_plus_r4_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_mi_r7_r10_plus_r4_LSL_1_Offset),
    kInstruction_str_mi_r7_r10_plus_r4_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_gt_r6_r2_plus_r5_LSL_1_Offset),
    kInstruction_str_gt_r6_r2_plus_r5_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_eq_r10_r8_plus_r11_LSL_1_Offset),
    kInstruction_str_eq_r10_r8_plus_r11_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_le_r10_r1_plus_r7_LSL_1_Offset),
    kInstruction_str_le_r10_r1_plus_r7_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_pl_r8_r14_plus_r8_LSL_1_Offset),
    kInstruction_str_pl_r8_r14_plus_r8_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_gt_r5_r1_plus_r3_LSL_1_Offset),
    kInstruction_str_gt_r5_r1_plus_r3_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cs_r10_r4_plus_r13_LSL_1_Offset),
    kInstruction_str_cs_r10_r4_plus_r13_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_mi_r10_r5_plus_r2_LSL_1_Offset),
    kInstruction_str_mi_r10_r5_plus_r2_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cc_r6_r0_plus_r11_LSL_1_Offset),
    kInstruction_str_cc_r6_r0_plus_r11_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_eq_r3_r9_plus_r2_LSL_1_Offset),
    kInstruction_str_eq_r3_r9_plus_r2_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r0_plus_r3_LSL_1_Offset),
    kInstruction_str_al_r1_r0_plus_r3_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_mi_r1_r0_plus_r13_LSL_1_Offset),
    kInstruction_str_mi_r1_r0_plus_r13_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cc_r4_r4_plus_r5_LSL_1_Offset),
    kInstruction_str_cc_r4_r4_plus_r5_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r6_plus_r11_LSL_1_Offset),
    kInstruction_str_al_r2_r6_plus_r11_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ls_r1_r5_plus_r4_LSL_1_Offset),
    kInstruction_str_ls_r1_r5_plus_r4_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_eq_r0_r3_plus_r4_LSL_1_Offset),
    kInstruction_str_eq_r0_r3_plus_r4_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_lt_r7_r11_plus_r10_LSL_1_Offset),
    kInstruction_str_lt_r7_r11_plus_r10_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_vc_r3_r0_plus_r13_LSL_1_Offset),
    kInstruction_str_vc_r3_r0_plus_r13_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ls_r3_r3_plus_r7_LSL_1_Offset),
    kInstruction_str_ls_r3_r3_plus_r7_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r14_plus_r3_LSL_1_Offset),
    kInstruction_str_al_r5_r14_plus_r3_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ne_r2_r13_plus_r9_LSL_1_Offset),
    kInstruction_str_ne_r2_r13_plus_r9_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_lt_r6_r2_plus_r11_LSL_1_Offset),
    kInstruction_str_lt_r6_r2_plus_r11_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_vc_r9_r2_plus_r13_LSL_1_Offset),
    kInstruction_str_vc_r9_r2_plus_r13_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ne_r5_r9_plus_r6_LSL_1_Offset),
    kInstruction_str_ne_r5_r9_plus_r6_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_vc_r4_r9_plus_r8_LSL_1_Offset),
    kInstruction_str_vc_r4_r9_plus_r8_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cc_r6_r9_plus_r2_LSL_1_Offset),
    kInstruction_str_cc_r6_r9_plus_r2_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_vs_r8_r7_plus_r13_LSL_1_Offset),
    kInstruction_str_vs_r8_r7_plus_r13_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_pl_r14_r14_plus_r7_LSL_1_Offset),
    kInstruction_str_pl_r14_r14_plus_r7_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_eq_r8_r2_plus_r6_LSL_1_Offset),
    kInstruction_str_eq_r8_r2_plus_r6_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r13_plus_r13_LSL_1_Offset),
    kInstruction_str_al_r13_r13_plus_r13_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_gt_r12_r14_plus_r5_LSL_1_Offset),
    kInstruction_str_gt_r12_r14_plus_r5_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_lt_r14_r7_plus_r1_LSL_1_Offset),
    kInstruction_str_lt_r14_r7_plus_r1_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r14_ROR_9_Offset),
    kInstruction_str_al_r0_r2_plus_r14_ROR_9_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_plus_r8_LSL_19_Offset),
    kInstruction_str_al_r0_r4_plus_r8_LSL_19_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r13_minus_r1_LSL_8_Offset),
    kInstruction_str_al_r0_r13_minus_r1_LSL_8_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_plus_r12_ROR_13_Offset),
    kInstruction_str_al_r0_r4_plus_r12_ROR_13_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_minus_r11_LSL_10_Offset),
    kInstruction_str_al_r0_r9_minus_r11_LSL_10_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r7_plus_r0_ROR_25_Offset),
    kInstruction_str_al_r0_r7_plus_r0_ROR_25_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r14_LSL_9_Offset),
    kInstruction_str_al_r0_r0_plus_r14_LSL_9_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_plus_r2_LSL_19_Offset),
    kInstruction_str_al_r0_r10_plus_r2_LSL_19_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_plus_r4_LSL_9_Offset),
    kInstruction_str_al_r0_r14_plus_r4_LSL_9_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r7_plus_r12_ROR_11_Offset),
    kInstruction_str_al_r0_r7_plus_r12_ROR_11_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_minus_r10_LSL_2_Offset),
    kInstruction_str_al_r0_r3_minus_r10_LSL_2_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r7_plus_r12_LSL_23_Offset),
    kInstruction_str_al_r0_r7_plus_r12_LSL_23_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_minus_r8_LSL_25_Offset),
    kInstruction_str_al_r0_r2_minus_r8_LSL_25_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_plus_r3_LSL_12_Offset),
    kInstruction_str_al_r0_r11_plus_r3_LSL_12_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_minus_r2_LSL_12_Offset),
    kInstruction_str_al_r0_r0_minus_r2_LSL_12_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r8_minus_r7_ROR_30_Offset),
    kInstruction_str_al_r0_r8_minus_r7_ROR_30_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r11_ROR_19_Offset),
    kInstruction_str_al_r0_r4_minus_r11_ROR_19_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_minus_r11_ROR_23_Offset),
    kInstruction_str_al_r0_r9_minus_r11_ROR_23_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_plus_r12_ROR_28_Offset),
    kInstruction_str_al_r0_r12_plus_r12_ROR_28_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_plus_r12_ROR_25_Offset),
    kInstruction_str_al_r0_r9_plus_r12_ROR_25_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_minus_r5_LSL_26_Offset),
    kInstruction_str_al_r0_r6_minus_r5_LSL_26_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_minus_r12_ROR_24_Offset),
    kInstruction_str_al_r0_r6_minus_r12_ROR_24_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_minus_r5_ROR_7_Offset),
    kInstruction_str_al_r0_r12_minus_r5_ROR_7_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r7_ROR_15_Offset),
    kInstruction_str_al_r0_r0_plus_r7_ROR_15_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_minus_r12_ROR_6_Offset),
    kInstruction_str_al_r0_r10_minus_r12_ROR_6_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r3_LSL_31_Offset),
    kInstruction_str_al_r0_r2_plus_r3_LSL_31_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_plus_r9_LSL_28_Offset),
    kInstruction_str_al_r0_r14_plus_r9_LSL_28_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r8_minus_r4_LSL_16_Offset),
    kInstruction_str_al_r0_r8_minus_r4_LSL_16_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r8_ROR_18_Offset),
    kInstruction_str_al_r0_r0_plus_r8_ROR_18_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r11_LSL_18_Offset),
    kInstruction_str_al_r0_r4_minus_r11_LSL_18_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_minus_r4_LSL_23_Offset),
    kInstruction_str_al_r0_r2_minus_r4_LSL_23_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_minus_r3_LSL_9_Offset),
    kInstruction_str_al_r0_r12_minus_r3_LSL_9_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_minus_r3_ROR_31_Offset),
    kInstruction_str_al_r0_r6_minus_r3_ROR_31_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_minus_r7_LSL_23_Offset),
    kInstruction_str_al_r0_r9_minus_r7_LSL_23_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_plus_r12_LSL_26_Offset),
    kInstruction_str_al_r0_r6_plus_r12_LSL_26_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r14_LSL_24_Offset),
    kInstruction_str_al_r0_r4_minus_r14_LSL_24_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r7_ROR_21_Offset),
    kInstruction_str_al_r0_r0_plus_r7_ROR_21_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_minus_r7_LSL_29_Offset),
    kInstruction_str_al_r0_r14_minus_r7_LSL_29_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_minus_r1_ROR_29_Offset),
    kInstruction_str_al_r0_r10_minus_r1_ROR_29_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_plus_r13_ROR_28_Offset),
    kInstruction_str_al_r0_r11_plus_r13_ROR_28_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r13_minus_r1_LSL_5_Offset),
    kInstruction_str_al_r0_r13_minus_r1_LSL_5_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_plus_r7_ROR_12_Offset),
    kInstruction_str_al_r0_r4_plus_r7_ROR_12_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r6_LSL_28_Offset),
    kInstruction_str_al_r0_r4_minus_r6_LSL_28_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_plus_r5_LSL_19_Offset),
    kInstruction_str_al_r0_r14_plus_r5_LSL_19_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_plus_r4_ROR_4_Offset),
    kInstruction_str_al_r0_r11_plus_r4_ROR_4_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r0_LSL_10_Offset),
    kInstruction_str_al_r0_r0_plus_r0_LSL_10_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r5_minus_r6_ROR_14_Offset),
    kInstruction_str_al_r0_r5_minus_r6_ROR_14_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_plus_r13_ROR_4_Offset),
    kInstruction_str_al_r0_r3_plus_r13_ROR_4_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r1_plus_r13_LSL_12_Offset),
    kInstruction_str_al_r0_r1_plus_r13_LSL_12_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r11_ROR_1_Offset),
    kInstruction_str_al_r0_r4_minus_r11_ROR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_plus_r14_ROR_10_Offset),
    kInstruction_str_al_r0_r4_plus_r14_ROR_10_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_plus_r7_LSL_8_Offset),
    kInstruction_str_al_r0_r4_plus_r7_LSL_8_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r5_plus_r8_LSL_2_Offset),
    kInstruction_str_al_r0_r5_plus_r8_LSL_2_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_minus_r13_LSL_30_Offset),
    kInstruction_str_al_r0_r3_minus_r13_LSL_30_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_minus_r14_ROR_4_Offset),
    kInstruction_str_al_r0_r3_minus_r14_ROR_4_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_minus_r13_ROR_23_Offset),
    kInstruction_str_al_r0_r9_minus_r13_ROR_23_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_plus_r11_LSL_29_Offset),
    kInstruction_str_al_r0_r4_plus_r11_LSL_29_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_plus_r13_ROR_28_Offset),
    kInstruction_str_al_r0_r4_plus_r13_ROR_28_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r3_ROR_13_Offset),
    kInstruction_str_al_r0_r2_plus_r3_ROR_13_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r7_minus_r11_LSL_6_Offset),
    kInstruction_str_al_r0_r7_minus_r11_LSL_6_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_minus_r14_LSL_15_Offset),
    kInstruction_str_al_r0_r6_minus_r14_LSL_15_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_minus_r3_LSL_17_Offset),
    kInstruction_str_al_r0_r3_minus_r3_LSL_17_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r5_plus_r13_LSL_20_Offset),
    kInstruction_str_al_r0_r5_plus_r13_LSL_20_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_plus_r1_ROR_21_Offset),
    kInstruction_str_al_r0_r6_plus_r1_ROR_21_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r12_LSL_6_Offset),
    kInstruction_str_al_r0_r0_plus_r12_LSL_6_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_minus_r6_LSL_21_Offset),
    kInstruction_str_al_r0_r9_minus_r6_LSL_21_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_minus_r11_LSL_11_Offset),
    kInstruction_str_al_r0_r3_minus_r11_LSL_11_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r12_ROR_11_Offset),
    kInstruction_str_al_r0_r4_minus_r12_ROR_11_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r5_plus_r4_ROR_1_Offset),
    kInstruction_str_al_r0_r5_plus_r4_ROR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r1_plus_r2_ROR_11_Offset),
    kInstruction_str_al_r0_r1_plus_r2_ROR_11_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r5_plus_r12_LSL_28_Offset),
    kInstruction_str_al_r0_r5_plus_r12_LSL_28_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_plus_r11_ROR_25_Offset),
    kInstruction_str_al_r0_r10_plus_r11_ROR_25_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_plus_r1_ROR_8_Offset),
    kInstruction_str_al_r0_r14_plus_r1_ROR_8_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_plus_r1_ROR_21_Offset),
    kInstruction_str_al_r0_r9_plus_r1_ROR_21_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_minus_r7_LSL_6_Offset),
    kInstruction_str_al_r0_r12_minus_r7_LSL_6_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_minus_r0_ROR_12_Offset),
    kInstruction_str_al_r0_r3_minus_r0_ROR_12_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_minus_r0_ROR_27_Offset),
    kInstruction_str_al_r0_r2_minus_r0_ROR_27_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_minus_r2_ROR_3_Offset),
    kInstruction_str_al_r0_r6_minus_r2_ROR_3_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r1_plus_r6_LSL_16_Offset),
    kInstruction_str_al_r0_r1_plus_r6_LSL_16_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_minus_r14_ROR_15_Offset),
    kInstruction_str_al_r0_r6_minus_r14_ROR_15_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_plus_r10_LSL_20_Offset),
    kInstruction_str_al_r0_r4_plus_r10_LSL_20_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r7_minus_r4_LSL_16_Offset),
    kInstruction_str_al_r0_r7_minus_r4_LSL_16_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_minus_r5_ROR_4_Offset),
    kInstruction_str_al_r0_r12_minus_r5_ROR_4_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_minus_r12_LSL_27_Offset),
    kInstruction_str_al_r0_r11_minus_r12_LSL_27_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r4_LSL_30_Offset),
    kInstruction_str_al_r0_r4_minus_r4_LSL_30_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_plus_r4_ROR_19_Offset),
    kInstruction_str_al_r0_r6_plus_r4_ROR_19_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_plus_r3_ROR_19_Offset),
    kInstruction_str_al_r0_r11_plus_r3_ROR_19_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_plus_r6_LSL_12_Offset),
    kInstruction_str_al_r0_r9_plus_r6_LSL_12_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_minus_r8_LSL_29_Offset),
    kInstruction_str_al_r0_r3_minus_r8_LSL_29_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r13_plus_r5_ROR_16_Offset),
    kInstruction_str_al_r0_r13_plus_r5_ROR_16_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_plus_r8_ROR_10_Offset),
    kInstruction_str_al_r0_r11_plus_r8_ROR_10_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_minus_r9_LSL_10_Offset),
    kInstruction_str_al_r0_r0_minus_r9_LSL_10_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r5_ROR_11_Offset),
    kInstruction_str_al_r0_r4_minus_r5_ROR_11_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_plus_r7_LSL_28_Offset),
    kInstruction_str_al_r0_r3_plus_r7_LSL_28_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_plus_r7_LSL_17_Offset),
    kInstruction_str_al_r0_r6_plus_r7_LSL_17_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_plus_r4_LSL_9_Offset),
    kInstruction_str_al_r0_r6_plus_r4_LSL_9_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_minus_r9_LSL_15_Offset),
    kInstruction_str_al_r0_r6_minus_r9_LSL_15_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_plus_r1_LSL_31_Offset),
    kInstruction_str_al_r0_r14_plus_r1_LSL_31_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r13_plus_r4_ROR_31_Offset),
    kInstruction_str_al_r0_r13_plus_r4_ROR_31_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r0_ROR_20_Offset),
    kInstruction_str_al_r0_r0_plus_r0_ROR_20_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_minus_r5_LSL_8_Offset),
    kInstruction_str_al_r0_r9_minus_r5_LSL_8_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_plus_r8_ROR_1_Offset),
    kInstruction_str_al_r0_r11_plus_r8_ROR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r1_minus_r4_ROR_18_Offset),
    kInstruction_str_al_r0_r1_minus_r4_ROR_18_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r3_LSL_11_Offset),
    kInstruction_str_al_r0_r0_plus_r3_LSL_11_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r11_ROR_31_Offset),
    kInstruction_str_al_r0_r2_plus_r11_ROR_31_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r7_minus_r11_LSL_22_Offset),
    kInstruction_str_al_r0_r7_minus_r11_LSL_22_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_minus_r12_LSL_13_Offset),
    kInstruction_str_al_r0_r10_minus_r12_LSL_13_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_plus_r3_ROR_17_Offset),
    kInstruction_str_al_r0_r12_plus_r3_ROR_17_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_minus_r4_ROR_15_Offset),
    kInstruction_str_al_r0_r0_minus_r4_ROR_15_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_plus_r4_LSL_1_Offset),
    kInstruction_str_al_r0_r3_plus_r4_LSL_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r10_ROR_20_Offset),
    kInstruction_str_al_r0_r2_plus_r10_ROR_20_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_minus_r8_ROR_10_Offset),
    kInstruction_str_al_r0_r11_minus_r8_ROR_10_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_minus_r5_LSL_3_Offset),
    kInstruction_str_al_r0_r14_minus_r5_LSL_3_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_minus_r7_LSL_24_Offset),
    kInstruction_str_al_r0_r0_minus_r7_LSL_24_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_minus_r11_LSL_4_Offset),
    kInstruction_str_al_r0_r2_minus_r11_LSL_4_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_plus_r0_ROR_25_Offset),
    kInstruction_str_al_r0_r3_plus_r0_ROR_25_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_plus_r8_ROR_7_Offset),
    kInstruction_str_al_r0_r12_plus_r8_ROR_7_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r9_ROR_31_Offset),
    kInstruction_str_al_r0_r0_plus_r9_ROR_31_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r5_minus_r0_LSL_3_Offset),
    kInstruction_str_al_r0_r5_minus_r0_LSL_3_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_minus_r9_ROR_8_Offset),
    kInstruction_str_al_r0_r0_minus_r9_ROR_8_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r8_plus_r12_ROR_22_Offset),
    kInstruction_str_al_r0_r8_plus_r12_ROR_22_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_minus_r1_ROR_26_Offset),
    kInstruction_str_al_r0_r0_minus_r1_ROR_26_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r1_minus_r1_ROR_22_Offset),
    kInstruction_str_al_r0_r1_minus_r1_ROR_22_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_minus_r0_ROR_23_Offset),
    kInstruction_str_al_r0_r2_minus_r0_ROR_23_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_plus_r11_ROR_29_Offset),
    kInstruction_str_al_r0_r9_plus_r11_ROR_29_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r5_minus_r9_ROR_18_Offset),
    kInstruction_str_al_r0_r5_minus_r9_ROR_18_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_minus_r8_LSL_4_Offset),
    kInstruction_str_al_r0_r2_minus_r8_LSL_4_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r10_ROR_7_Offset),
    kInstruction_str_al_r0_r2_plus_r10_ROR_7_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_plus_r1_LSL_22_Offset),
    kInstruction_str_al_r0_r6_plus_r1_LSL_22_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_minus_r14_ROR_24_Offset),
    kInstruction_str_al_r0_r14_minus_r14_ROR_24_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r5_plus_r11_ROR_4_Offset),
    kInstruction_str_al_r0_r5_plus_r11_ROR_4_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_minus_r7_LSL_16_Offset),
    kInstruction_str_al_r0_r3_minus_r7_LSL_16_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_plus_r11_ROR_17_Offset),
    kInstruction_str_al_r0_r12_plus_r11_ROR_17_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_minus_r1_LSL_25_Offset),
    kInstruction_str_al_r0_r11_minus_r1_LSL_25_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_plus_r0_ROR_4_Offset),
    kInstruction_str_al_r0_r4_plus_r0_ROR_4_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r9_ROR_28_Offset),
    kInstruction_str_al_r0_r4_minus_r9_ROR_28_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_plus_r8_ROR_22_Offset),
    kInstruction_str_al_r0_r3_plus_r8_ROR_22_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r8_plus_r13_LSL_12_Offset),
    kInstruction_str_al_r0_r8_plus_r13_LSL_12_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r8_minus_r4_ROR_31_Offset),
    kInstruction_str_al_r0_r8_minus_r4_ROR_31_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_minus_r7_ROR_3_Offset),
    kInstruction_str_al_r0_r10_minus_r7_ROR_3_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r8_minus_r11_ROR_30_Offset),
    kInstruction_str_al_r0_r8_minus_r11_ROR_30_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_minus_r5_ROR_8_Offset),
    kInstruction_str_al_r0_r11_minus_r5_ROR_8_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_minus_r2_LSL_16_Offset),
    kInstruction_str_al_r0_r10_minus_r2_LSL_16_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_minus_r2_ROR_8_Offset),
    kInstruction_str_al_r0_r10_minus_r2_ROR_8_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r7_minus_r5_ROR_2_Offset),
    kInstruction_str_al_r0_r7_minus_r5_ROR_2_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_minus_r13_LSL_24_Offset),
    kInstruction_str_al_r0_r12_minus_r13_LSL_24_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_minus_r4_ROR_3_Offset),
    kInstruction_str_al_r0_r2_minus_r4_ROR_3_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_plus_r12_LSL_19_Offset),
    kInstruction_str_al_r0_r14_plus_r12_LSL_19_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r6_LSL_17_Offset),
    kInstruction_str_al_r0_r2_plus_r6_LSL_17_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_plus_r10_LSL_16_Offset),
    kInstruction_str_al_r0_r9_plus_r10_LSL_16_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_plus_r9_ROR_28_Offset),
    kInstruction_str_al_r0_r3_plus_r9_ROR_28_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r3_LSL_3_Offset),
    kInstruction_str_al_r0_r2_plus_r3_LSL_3_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_minus_r12_LSL_28_Offset),
    kInstruction_str_al_r0_r0_minus_r12_LSL_28_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_minus_r12_ROR_14_Offset),
    kInstruction_str_al_r0_r2_minus_r12_ROR_14_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r11_LSL_28_Offset),
    kInstruction_str_al_r0_r0_plus_r11_LSL_28_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_plus_r6_ROR_15_Offset),
    kInstruction_str_al_r0_r11_plus_r6_ROR_15_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r14_ROR_21_Offset),
    kInstruction_str_al_r0_r0_plus_r14_ROR_21_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r2_LSL_12_Offset),
    kInstruction_str_al_r0_r0_plus_r2_LSL_12_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_plus_r11_LSL_8_Offset),
    kInstruction_str_al_r0_r14_plus_r11_LSL_8_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r4_LSL_12_Offset),
    kInstruction_str_al_r0_r2_plus_r4_LSL_12_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_minus_r7_LSL_26_Offset),
    kInstruction_str_al_r0_r12_minus_r7_LSL_26_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r8_minus_r8_ROR_24_Offset),
    kInstruction_str_al_r0_r8_minus_r8_ROR_24_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_plus_r0_LSL_28_Offset),
    kInstruction_str_al_r0_r10_plus_r0_LSL_28_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_minus_r2_ROR_26_Offset),
    kInstruction_str_al_r0_r3_minus_r2_ROR_26_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r7_minus_r0_LSL_7_Offset),
    kInstruction_str_al_r0_r7_minus_r0_LSL_7_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_minus_r14_ROR_12_Offset),
    kInstruction_str_al_r0_r0_minus_r14_ROR_12_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_minus_r10_ROR_9_Offset),
    kInstruction_str_al_r0_r3_minus_r10_ROR_9_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_plus_r2_LSL_15_Offset),
    kInstruction_str_al_r0_r9_plus_r2_LSL_15_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r13_minus_r10_ROR_28_Offset),
    kInstruction_str_al_r0_r13_minus_r10_ROR_28_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_plus_r14_LSL_10_Offset),
    kInstruction_str_al_r0_r10_plus_r14_LSL_10_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r3_ROR_23_Offset),
    kInstruction_str_al_r0_r2_plus_r3_ROR_23_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_plus_r4_LSL_2_Offset),
    kInstruction_str_al_r0_r3_plus_r4_LSL_2_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r7_plus_r12_LSL_28_Offset),
    kInstruction_str_al_r0_r7_plus_r12_LSL_28_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_minus_r3_LSL_18_Offset),
    kInstruction_str_al_r0_r12_minus_r3_LSL_18_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r13_minus_r11_ROR_7_Offset),
    kInstruction_str_al_r0_r13_minus_r11_ROR_7_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_minus_r14_LSL_6_Offset),
    kInstruction_str_al_r0_r14_minus_r14_LSL_6_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r7_minus_r11_ROR_31_Offset),
    kInstruction_str_al_r0_r7_minus_r11_ROR_31_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_minus_r7_ROR_14_Offset),
    kInstruction_str_al_r0_r9_minus_r7_ROR_14_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r8_minus_r2_LSL_7_Offset),
    kInstruction_str_al_r0_r8_minus_r2_LSL_7_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_plus_r14_LSL_23_Offset),
    kInstruction_str_al_r0_r3_plus_r14_LSL_23_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r5_plus_r7_LSL_5_Offset),
    kInstruction_str_al_r0_r5_plus_r7_LSL_5_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_minus_r14_LSL_30_Offset),
    kInstruction_str_al_r0_r10_minus_r14_LSL_30_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_minus_r1_LSL_25_Offset),
    kInstruction_str_al_r0_r0_minus_r1_LSL_25_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_plus_r8_ROR_31_Offset),
    kInstruction_str_al_r0_r14_plus_r8_ROR_31_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_plus_r11_ROR_31_Offset),
    kInstruction_str_al_r0_r11_plus_r11_ROR_31_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_plus_r8_LSL_27_Offset),
    kInstruction_str_al_r0_r14_plus_r8_LSL_27_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_minus_r0_ROR_9_Offset),
    kInstruction_str_al_r0_r6_minus_r0_ROR_9_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r12_LSL_29_Offset),
    kInstruction_str_al_r0_r4_minus_r12_LSL_29_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_plus_r8_ROR_4_Offset),
    kInstruction_str_al_r0_r14_plus_r8_ROR_4_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_minus_r9_LSL_27_Offset),
    kInstruction_str_al_r0_r9_minus_r9_LSL_27_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_plus_r8_ROR_28_Offset),
    kInstruction_str_al_r0_r3_plus_r8_ROR_28_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_plus_r0_ROR_13_Offset),
    kInstruction_str_al_r0_r12_plus_r0_ROR_13_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r8_minus_r4_ROR_20_Offset),
    kInstruction_str_al_r0_r8_minus_r4_ROR_20_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r1_plus_r14_ROR_4_Offset),
    kInstruction_str_al_r0_r1_plus_r14_ROR_4_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_plus_r2_ROR_10_Offset),
    kInstruction_str_al_r0_r11_plus_r2_ROR_10_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r10_ROR_18_Offset),
    kInstruction_str_al_r0_r0_plus_r10_ROR_18_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_minus_r7_ROR_3_Offset),
    kInstruction_str_al_r0_r11_minus_r7_ROR_3_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r14_LSL_10_Offset),
    kInstruction_str_al_r0_r0_plus_r14_LSL_10_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_minus_r9_ROR_5_Offset),
    kInstruction_str_al_r0_r12_minus_r9_ROR_5_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_plus_r14_ROR_26_Offset),
    kInstruction_str_al_r0_r11_plus_r14_ROR_26_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r3_minus_r4_ROR_30_PreIndex),
    kInstruction_str_al_r8_r3_minus_r4_ROR_30_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r0_plus_r5_LSL_13_PreIndex),
    kInstruction_str_al_r1_r0_plus_r5_LSL_13_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r3_minus_r1_ROR_31_PostIndex),
    kInstruction_str_al_r13_r3_minus_r1_ROR_31_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r7_minus_r1_LSL_8_PostIndex),
    kInstruction_str_al_r9_r7_minus_r1_LSL_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r6_plus_r7_ROR_18_PreIndex),
    kInstruction_str_al_r2_r6_plus_r7_ROR_18_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_minus_r11_ROR_26_PreIndex),
    kInstruction_str_al_r0_r11_minus_r11_ROR_26_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r4_plus_r14_LSL_12_PreIndex),
    kInstruction_str_al_r14_r4_plus_r14_LSL_12_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r6_minus_r13_LSL_15_PreIndex),
    kInstruction_str_al_r10_r6_minus_r13_LSL_15_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_minus_r13_ROR_30_PreIndex),
    kInstruction_str_al_r0_r2_minus_r13_ROR_30_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_plus_r2_LSL_10_PostIndex),
    kInstruction_str_al_r0_r14_plus_r2_LSL_10_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r3_plus_r8_ROR_26_PreIndex),
    kInstruction_str_al_r8_r3_plus_r8_ROR_26_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r12_LSL_31_PostIndex),
    kInstruction_str_al_r0_r4_minus_r12_LSL_31_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r12_plus_r7_LSL_25_PostIndex),
    kInstruction_str_al_r11_r12_plus_r7_LSL_25_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r10_minus_r7_ROR_22_PostIndex),
    kInstruction_str_al_r2_r10_minus_r7_ROR_22_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r6_plus_r8_LSL_28_PreIndex),
    kInstruction_str_al_r3_r6_plus_r8_LSL_28_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r3_plus_r12_ROR_21_PreIndex),
    kInstruction_str_al_r9_r3_plus_r12_ROR_21_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r5_plus_r3_ROR_29_PostIndex),
    kInstruction_str_al_r11_r5_plus_r3_ROR_29_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r9_plus_r3_LSL_4_PreIndex),
    kInstruction_str_al_r4_r9_plus_r3_LSL_4_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r11_plus_r12_LSL_14_PreIndex),
    kInstruction_str_al_r8_r11_plus_r12_LSL_14_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r7_plus_r14_LSL_24_PreIndex),
    kInstruction_str_al_r4_r7_plus_r14_LSL_24_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r14_plus_r14_ROR_15_PostIndex),
    kInstruction_str_al_r5_r14_plus_r14_ROR_15_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r10_plus_r9_ROR_24_PreIndex),
    kInstruction_str_al_r12_r10_plus_r9_ROR_24_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r0_plus_r5_ROR_11_PreIndex),
    kInstruction_str_al_r6_r0_plus_r5_ROR_11_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r7_plus_r6_ROR_27_PostIndex),
    kInstruction_str_al_r8_r7_plus_r6_ROR_27_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r12_plus_r12_LSL_8_PreIndex),
    kInstruction_str_al_r2_r12_plus_r12_LSL_8_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r5_plus_r13_ROR_7_PreIndex),
    kInstruction_str_al_r6_r5_plus_r13_ROR_7_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r7_plus_r3_LSL_14_PreIndex),
    kInstruction_str_al_r10_r7_plus_r3_LSL_14_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r0_plus_r0_ROR_23_PostIndex),
    kInstruction_str_al_r7_r0_plus_r0_ROR_23_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r0_plus_r3_LSL_15_PostIndex),
    kInstruction_str_al_r12_r0_plus_r3_LSL_15_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r4_plus_r11_ROR_17_PostIndex),
    kInstruction_str_al_r2_r4_plus_r11_ROR_17_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r7_plus_r9_ROR_18_PostIndex),
    kInstruction_str_al_r1_r7_plus_r9_ROR_18_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r10_plus_r14_ROR_3_PreIndex),
    kInstruction_str_al_r8_r10_plus_r14_ROR_3_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r3_minus_r13_LSL_26_PostIndex),
    kInstruction_str_al_r2_r3_minus_r13_LSL_26_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r1_plus_r4_LSL_20_PostIndex),
    kInstruction_str_al_r8_r1_plus_r4_LSL_20_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r10_minus_r6_ROR_31_PreIndex),
    kInstruction_str_al_r8_r10_minus_r6_ROR_31_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r7_minus_r8_LSL_9_PreIndex),
    kInstruction_str_al_r8_r7_minus_r8_LSL_9_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r1_minus_r13_LSL_3_PostIndex),
    kInstruction_str_al_r2_r1_minus_r13_LSL_3_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r1_minus_r12_LSL_8_PostIndex),
    kInstruction_str_al_r7_r1_minus_r12_LSL_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r8_plus_r5_LSL_28_PostIndex),
    kInstruction_str_al_r3_r8_plus_r5_LSL_28_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r0_plus_r0_ROR_9_PreIndex),
    kInstruction_str_al_r13_r0_plus_r0_ROR_9_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r5_plus_r4_ROR_22_PreIndex),
    kInstruction_str_al_r2_r5_plus_r4_ROR_22_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r0_plus_r0_LSL_12_PreIndex),
    kInstruction_str_al_r4_r0_plus_r0_LSL_12_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r7_plus_r0_LSL_10_PostIndex),
    kInstruction_str_al_r11_r7_plus_r0_LSL_10_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r0_minus_r12_LSL_25_PostIndex),
    kInstruction_str_al_r13_r0_minus_r12_LSL_25_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r3_minus_r12_ROR_22_PreIndex),
    kInstruction_str_al_r1_r3_minus_r12_ROR_22_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r6_minus_r5_LSL_26_PostIndex),
    kInstruction_str_al_r10_r6_minus_r5_LSL_26_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r7_plus_r11_LSL_17_PreIndex),
    kInstruction_str_al_r0_r7_plus_r11_LSL_17_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r13_minus_r9_ROR_28_PreIndex),
    kInstruction_str_al_r12_r13_minus_r9_ROR_28_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r1_plus_r6_ROR_10_PostIndex),
    kInstruction_str_al_r11_r1_plus_r6_ROR_10_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r0_minus_r8_ROR_31_PreIndex),
    kInstruction_str_al_r3_r0_minus_r8_ROR_31_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r6_plus_r12_ROR_29_PreIndex),
    kInstruction_str_al_r13_r6_plus_r12_ROR_29_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r5_minus_r9_LSL_10_PostIndex),
    kInstruction_str_al_r2_r5_minus_r9_LSL_10_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r3_minus_r6_ROR_5_PostIndex),
    kInstruction_str_al_r11_r3_minus_r6_ROR_5_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r2_minus_r14_LSL_10_PostIndex),
    kInstruction_str_al_r6_r2_minus_r14_LSL_10_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r10_minus_r10_LSL_24_PreIndex),
    kInstruction_str_al_r3_r10_minus_r10_LSL_24_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r3_plus_r11_LSL_12_PreIndex),
    kInstruction_str_al_r4_r3_plus_r11_LSL_12_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r13_minus_r0_ROR_22_PostIndex),
    kInstruction_str_al_r4_r13_minus_r0_ROR_22_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r13_plus_r1_ROR_2_PostIndex),
    kInstruction_str_al_r2_r13_plus_r1_ROR_2_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r9_plus_r8_LSL_3_PreIndex),
    kInstruction_str_al_r11_r9_plus_r8_LSL_3_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r6_minus_r6_ROR_23_PostIndex),
    kInstruction_str_al_r10_r6_minus_r6_ROR_23_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r0_minus_r4_LSL_8_PreIndex),
    kInstruction_str_al_r4_r0_minus_r4_LSL_8_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r9_plus_r2_ROR_6_PreIndex),
    kInstruction_str_al_r14_r9_plus_r2_ROR_6_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r10_plus_r9_LSL_3_PostIndex),
    kInstruction_str_al_r8_r10_plus_r9_LSL_3_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r14_plus_r13_ROR_25_PostIndex),
    kInstruction_str_al_r9_r14_plus_r13_ROR_25_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r3_plus_r6_LSL_3_PostIndex),
    kInstruction_str_al_r13_r3_plus_r6_LSL_3_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r14_plus_r4_LSL_24_PreIndex),
    kInstruction_str_al_r9_r14_plus_r4_LSL_24_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r10_plus_r4_LSL_29_PreIndex),
    kInstruction_str_al_r5_r10_plus_r4_LSL_29_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r5_minus_r11_LSL_13_PreIndex),
    kInstruction_str_al_r7_r5_minus_r11_LSL_13_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r1_plus_r2_ROR_27_PreIndex),
    kInstruction_str_al_r8_r1_plus_r2_ROR_27_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r11_minus_r8_ROR_1_PreIndex),
    kInstruction_str_al_r14_r11_minus_r8_ROR_1_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r6_plus_r4_ROR_29_PostIndex),
    kInstruction_str_al_r14_r6_plus_r4_ROR_29_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r0_plus_r1_LSL_21_PreIndex),
    kInstruction_str_al_r13_r0_plus_r1_LSL_21_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r8_plus_r2_LSL_22_PostIndex),
    kInstruction_str_al_r1_r8_plus_r2_LSL_22_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r8_plus_r5_LSL_19_PreIndex),
    kInstruction_str_al_r11_r8_plus_r5_LSL_19_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r10_plus_r7_ROR_15_PostIndex),
    kInstruction_str_al_r12_r10_plus_r7_ROR_15_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r6_plus_r14_ROR_4_PostIndex),
    kInstruction_str_al_r2_r6_plus_r14_ROR_4_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r6_plus_r12_LSL_9_PostIndex),
    kInstruction_str_al_r3_r6_plus_r12_LSL_9_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r0_minus_r6_LSL_24_PreIndex),
    kInstruction_str_al_r1_r0_minus_r6_LSL_24_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r12_plus_r2_ROR_30_PostIndex),
    kInstruction_str_al_r5_r12_plus_r2_ROR_30_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r3_plus_r8_ROR_6_PostIndex),
    kInstruction_str_al_r14_r3_plus_r8_ROR_6_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r14_minus_r10_ROR_25_PostIndex),
    kInstruction_str_al_r8_r14_minus_r10_ROR_25_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r12_plus_r2_ROR_3_PreIndex),
    kInstruction_str_al_r7_r12_plus_r2_ROR_3_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r13_plus_r11_ROR_29_PreIndex),
    kInstruction_str_al_r9_r13_plus_r11_ROR_29_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r2_plus_r10_ROR_21_PostIndex),
    kInstruction_str_al_r5_r2_plus_r10_ROR_21_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r8_minus_r2_LSL_21_PreIndex),
    kInstruction_str_al_r13_r8_minus_r2_LSL_21_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r13_plus_r5_LSL_20_PostIndex),
    kInstruction_str_al_r1_r13_plus_r5_LSL_20_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r0_plus_r6_LSL_15_PostIndex),
    kInstruction_str_al_r6_r0_plus_r6_LSL_15_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r3_minus_r6_LSL_31_PreIndex),
    kInstruction_str_al_r7_r3_minus_r6_LSL_31_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r10_minus_r11_ROR_6_PostIndex),
    kInstruction_str_al_r5_r10_minus_r11_ROR_6_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r3_minus_r13_ROR_29_PreIndex),
    kInstruction_str_al_r7_r3_minus_r13_ROR_29_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r6_plus_r3_LSL_10_PreIndex),
    kInstruction_str_al_r4_r6_plus_r3_LSL_10_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r13_plus_r4_ROR_8_PreIndex),
    kInstruction_str_al_r7_r13_plus_r4_ROR_8_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r5_plus_r0_LSL_30_PreIndex),
    kInstruction_str_al_r2_r5_plus_r0_LSL_30_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r12_plus_r5_LSL_8_PreIndex),
    kInstruction_str_al_r10_r12_plus_r5_LSL_8_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r10_minus_r11_ROR_2_PreIndex),
    kInstruction_str_al_r4_r10_minus_r11_ROR_2_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r11_plus_r12_LSL_14_PreIndex),
    kInstruction_str_al_r4_r11_plus_r12_LSL_14_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r6_plus_r1_LSL_10_PostIndex),
    kInstruction_str_al_r1_r6_plus_r1_LSL_10_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r11_plus_r11_LSL_11_PreIndex),
    kInstruction_str_al_r8_r11_plus_r11_LSL_11_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r8_minus_r7_ROR_10_PostIndex),
    kInstruction_str_al_r12_r8_minus_r7_ROR_10_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r11_plus_r1_LSL_11_PreIndex),
    kInstruction_str_al_r13_r11_plus_r1_LSL_11_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r3_plus_r9_ROR_9_PreIndex),
    kInstruction_str_al_r5_r3_plus_r9_ROR_9_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r10_plus_r6_LSL_18_PreIndex),
    kInstruction_str_al_r6_r10_plus_r6_LSL_18_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r1_minus_r14_LSL_14_PostIndex),
    kInstruction_str_al_r6_r1_minus_r14_LSL_14_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r3_minus_r6_LSL_5_PreIndex),
    kInstruction_str_al_r10_r3_minus_r6_LSL_5_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r7_minus_r13_ROR_26_PreIndex),
    kInstruction_str_al_r6_r7_minus_r13_ROR_26_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r12_minus_r7_ROR_30_PreIndex),
    kInstruction_str_al_r8_r12_minus_r7_ROR_30_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r7_plus_r5_LSL_27_PostIndex),
    kInstruction_str_al_r9_r7_plus_r5_LSL_27_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r10_minus_r14_LSL_26_PreIndex),
    kInstruction_str_al_r8_r10_minus_r14_LSL_26_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r13_minus_r2_LSL_30_PostIndex),
    kInstruction_str_al_r9_r13_minus_r2_LSL_30_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r9_minus_r8_ROR_7_PostIndex),
    kInstruction_str_al_r12_r9_minus_r8_ROR_7_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r9_minus_r5_ROR_2_PreIndex),
    kInstruction_str_al_r4_r9_minus_r5_ROR_2_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r5_plus_r9_ROR_16_PostIndex),
    kInstruction_str_al_r1_r5_plus_r9_ROR_16_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r2_minus_r9_LSL_25_PostIndex),
    kInstruction_str_al_r3_r2_minus_r9_LSL_25_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r0_minus_r4_LSL_22_PreIndex),
    kInstruction_str_al_r14_r0_minus_r4_LSL_22_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r4_plus_r2_LSL_7_PostIndex),
    kInstruction_str_al_r1_r4_plus_r2_LSL_7_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r9_minus_r4_LSL_31_PostIndex),
    kInstruction_str_al_r13_r9_minus_r4_LSL_31_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r14_minus_r2_LSL_13_PostIndex),
    kInstruction_str_al_r11_r14_minus_r2_LSL_13_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r5_plus_r1_LSL_22_PreIndex),
    kInstruction_str_al_r10_r5_plus_r1_LSL_22_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r2_minus_r10_LSL_25_PreIndex),
    kInstruction_str_al_r13_r2_minus_r10_LSL_25_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r2_plus_r0_ROR_27_PostIndex),
    kInstruction_str_al_r12_r2_plus_r0_ROR_27_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r13_plus_r13_LSL_5_PostIndex),
    kInstruction_str_al_r5_r13_plus_r13_LSL_5_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r0_minus_r11_ROR_3_PreIndex),
    kInstruction_str_al_r14_r0_minus_r11_ROR_3_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r10_minus_r8_LSL_4_PreIndex),
    kInstruction_str_al_r1_r10_minus_r8_LSL_4_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r4_plus_r3_ROR_23_PostIndex),
    kInstruction_str_al_r11_r4_plus_r3_ROR_23_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r4_plus_r11_LSL_29_PreIndex),
    kInstruction_str_al_r1_r4_plus_r11_LSL_29_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r4_plus_r7_ROR_4_PreIndex),
    kInstruction_str_al_r12_r4_plus_r7_ROR_4_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r11_plus_r14_ROR_17_PostIndex),
    kInstruction_str_al_r4_r11_plus_r14_ROR_17_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r10_plus_r0_LSL_2_PostIndex),
    kInstruction_str_al_r3_r10_plus_r0_LSL_2_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r13_minus_r8_ROR_26_PostIndex),
    kInstruction_str_al_r11_r13_minus_r8_ROR_26_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r8_plus_r14_ROR_21_PostIndex),
    kInstruction_str_al_r0_r8_plus_r14_ROR_21_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r5_plus_r7_LSL_9_PreIndex),
    kInstruction_str_al_r8_r5_plus_r7_LSL_9_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r10_plus_r14_LSL_14_PostIndex),
    kInstruction_str_al_r9_r10_plus_r14_LSL_14_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r9_minus_r4_LSL_26_PostIndex),
    kInstruction_str_al_r1_r9_minus_r4_LSL_26_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r4_plus_r1_LSL_8_PreIndex),
    kInstruction_str_al_r8_r4_plus_r1_LSL_8_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r6_minus_r7_ROR_23_PostIndex),
    kInstruction_str_al_r13_r6_minus_r7_ROR_23_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r9_plus_r4_LSL_15_PreIndex),
    kInstruction_str_al_r12_r9_plus_r4_LSL_15_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r8_plus_r1_ROR_24_PostIndex),
    kInstruction_str_al_r7_r8_plus_r1_ROR_24_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r7_minus_r2_LSL_20_PreIndex),
    kInstruction_str_al_r5_r7_minus_r2_LSL_20_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r11_minus_r6_LSL_16_PostIndex),
    kInstruction_str_al_r3_r11_minus_r6_LSL_16_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r14_minus_r8_ROR_15_PostIndex),
    kInstruction_str_al_r6_r14_minus_r8_ROR_15_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r14_minus_r7_LSL_28_PreIndex),
    kInstruction_str_al_r13_r14_minus_r7_LSL_28_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r12_plus_r9_LSL_3_PreIndex),
    kInstruction_str_al_r3_r12_plus_r9_LSL_3_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r3_plus_r10_ROR_14_PreIndex),
    kInstruction_str_al_r1_r3_plus_r10_ROR_14_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r3_minus_r12_LSL_9_PreIndex),
    kInstruction_str_al_r1_r3_minus_r12_LSL_9_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r6_plus_r8_ROR_21_PostIndex),
    kInstruction_str_al_r3_r6_plus_r8_ROR_21_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r3_plus_r9_LSL_7_PreIndex),
    kInstruction_str_al_r14_r3_plus_r9_LSL_7_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r8_plus_r1_ROR_27_PreIndex),
    kInstruction_str_al_r4_r8_plus_r1_ROR_27_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r13_minus_r5_ROR_19_PreIndex),
    kInstruction_str_al_r1_r13_minus_r5_ROR_19_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r1_plus_r14_LSL_15_PostIndex),
    kInstruction_str_al_r5_r1_plus_r14_LSL_15_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r3_minus_r11_ROR_28_PreIndex),
    kInstruction_str_al_r12_r3_minus_r11_ROR_28_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r3_plus_r4_LSL_20_PostIndex),
    kInstruction_str_al_r14_r3_plus_r4_LSL_20_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r2_plus_r10_ROR_14_PreIndex),
    kInstruction_str_al_r6_r2_plus_r10_ROR_14_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r12_plus_r13_ROR_26_PreIndex),
    kInstruction_str_al_r8_r12_plus_r13_ROR_26_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r13_minus_r13_LSL_11_PostIndex),
    kInstruction_str_al_r12_r13_minus_r13_LSL_11_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_minus_r10_LSL_24_PostIndex),
    kInstruction_str_al_r0_r6_minus_r10_LSL_24_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r0_minus_r13_LSL_29_PreIndex),
    kInstruction_str_al_r6_r0_minus_r13_LSL_29_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r2_plus_r10_LSL_15_PostIndex),
    kInstruction_str_al_r4_r2_plus_r10_LSL_15_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r9_plus_r14_ROR_21_PreIndex),
    kInstruction_str_al_r13_r9_plus_r14_ROR_21_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r1_minus_r0_ROR_26_PostIndex),
    kInstruction_str_al_r4_r1_minus_r0_ROR_26_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r7_minus_r12_ROR_28_PreIndex),
    kInstruction_str_al_r12_r7_minus_r12_ROR_28_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r0_minus_r5_LSL_7_PreIndex),
    kInstruction_str_al_r10_r0_minus_r5_LSL_7_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r1_plus_r2_ROR_23_PreIndex),
    kInstruction_str_al_r4_r1_plus_r2_ROR_23_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r6_minus_r1_LSL_31_PostIndex),
    kInstruction_str_al_r12_r6_minus_r1_LSL_31_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r8_plus_r3_LSL_28_PreIndex),
    kInstruction_str_al_r4_r8_plus_r3_LSL_28_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r7_minus_r5_ROR_3_PreIndex),
    kInstruction_str_al_r11_r7_minus_r5_ROR_3_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r3_plus_r11_LSL_16_PostIndex),
    kInstruction_str_al_r4_r3_plus_r11_LSL_16_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r11_plus_r12_ROR_11_PreIndex),
    kInstruction_str_al_r12_r11_plus_r12_ROR_11_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r5_minus_r0_ROR_29_PreIndex),
    kInstruction_str_al_r11_r5_minus_r0_ROR_29_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r7_plus_r12_LSL_24_PostIndex),
    kInstruction_str_al_r11_r7_plus_r12_LSL_24_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r4_plus_r4_ROR_9_PostIndex),
    kInstruction_str_al_r13_r4_plus_r4_ROR_9_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_minus_r11_LSL_17_PostIndex),
    kInstruction_str_al_r0_r9_minus_r11_LSL_17_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r11_plus_r5_ROR_27_PostIndex),
    kInstruction_str_al_r1_r11_plus_r5_ROR_27_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r0_plus_r5_ROR_16_PostIndex),
    kInstruction_str_al_r6_r0_plus_r5_ROR_16_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r8_plus_r8_LSL_23_PreIndex),
    kInstruction_str_al_r14_r8_plus_r8_LSL_23_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r0_plus_r12_LSL_29_PostIndex),
    kInstruction_str_al_r7_r0_plus_r12_LSL_29_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r4_minus_r12_ROR_28_PreIndex),
    kInstruction_str_al_r5_r4_minus_r12_ROR_28_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r13_plus_r1_LSL_12_PreIndex),
    kInstruction_str_al_r1_r13_plus_r1_LSL_12_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r9_minus_r11_ROR_24_PostIndex),
    kInstruction_str_al_r7_r9_minus_r11_ROR_24_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r8_minus_r0_ROR_6_PreIndex),
    kInstruction_str_al_r4_r8_minus_r0_ROR_6_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r9_plus_r9_ROR_30_PreIndex),
    kInstruction_str_al_r6_r9_plus_r9_ROR_30_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r14_minus_r10_LSL_4_PreIndex),
    kInstruction_str_al_r1_r14_minus_r10_LSL_4_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r0_minus_r13_ROR_12_PreIndex),
    kInstruction_str_al_r6_r0_minus_r13_ROR_12_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r4_minus_r2_LSL_23_PreIndex),
    kInstruction_str_al_r1_r4_minus_r2_LSL_23_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_minus_r11_ROR_26_PostIndex),
    kInstruction_str_al_r0_r12_minus_r11_ROR_26_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r8_plus_r4_LSL_31_PostIndex),
    kInstruction_str_al_r5_r8_plus_r4_LSL_31_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r11_plus_r10_LSL_8_PreIndex),
    kInstruction_str_al_r14_r11_plus_r10_LSL_8_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r14_minus_r2_LSL_31_PostIndex),
    kInstruction_str_al_r4_r14_minus_r2_LSL_31_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r6_plus_r11_LSL_30_PostIndex),
    kInstruction_str_al_r9_r6_plus_r11_LSL_30_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r5_minus_r1_ROR_9_PostIndex),
    kInstruction_str_al_r14_r5_minus_r1_ROR_9_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r8_minus_r14_ROR_28_PreIndex),
    kInstruction_str_al_r2_r8_minus_r14_ROR_28_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_plus_r12_ROR_8_PostIndex),
    kInstruction_str_al_r0_r4_plus_r12_ROR_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r12_minus_r10_LSL_19_PostIndex),
    kInstruction_str_al_r6_r12_minus_r10_LSL_19_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r8_minus_r11_LSL_9_PostIndex),
    kInstruction_str_al_r7_r8_minus_r11_LSL_9_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r4_plus_r4_ROR_6_PostIndex),
    kInstruction_str_al_r6_r4_plus_r4_ROR_6_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r14_minus_r3_LSL_19_PostIndex),
    kInstruction_str_al_r12_r14_minus_r3_LSL_19_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r4_plus_r14_ROR_22_PostIndex),
    kInstruction_str_al_r14_r4_plus_r14_ROR_22_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r5_minus_r2_LSL_15_PostIndex),
    kInstruction_str_al_r2_r5_minus_r2_LSL_15_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r5_minus_r12_ROR_24_PreIndex),
    kInstruction_str_al_r6_r5_minus_r12_ROR_24_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r1_plus_r0_LSL_8_PostIndex),
    kInstruction_str_al_r0_r1_plus_r0_LSL_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r3_plus_r10_LSL_23_PostIndex),
    kInstruction_str_al_r6_r3_plus_r10_LSL_23_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r1_plus_r0_ROR_15_PostIndex),
    kInstruction_str_al_r10_r1_plus_r0_ROR_15_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r14_plus_r8_ROR_31_PostIndex),
    kInstruction_str_al_r1_r14_plus_r8_ROR_31_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r10_plus_r6_LSL_6_PreIndex),
    kInstruction_str_al_r14_r10_plus_r6_LSL_6_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r2_plus_r10_ROR_18_PostIndex),
    kInstruction_str_al_r12_r2_plus_r10_ROR_18_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r8_minus_r0_LSL_9_PostIndex),
    kInstruction_str_al_r4_r8_minus_r0_LSL_9_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r3_plus_r3_ROR_16_PreIndex),
    kInstruction_str_al_r9_r3_plus_r3_ROR_16_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r7_minus_r3_LSL_2_PostIndex),
    kInstruction_str_al_r1_r7_minus_r3_LSL_2_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r13_plus_r14_ROR_16_PreIndex),
    kInstruction_str_al_r8_r13_plus_r14_ROR_16_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r0_minus_r3_LSL_30_PreIndex),
    kInstruction_str_al_r3_r0_minus_r3_LSL_30_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r11_plus_r13_ROR_26_PostIndex),
    kInstruction_str_al_r1_r11_plus_r13_ROR_26_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r1_plus_r9_ROR_3_PostIndex),
    kInstruction_str_al_r3_r1_plus_r9_ROR_3_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r11_minus_r4_ROR_22_PostIndex),
    kInstruction_str_al_r14_r11_minus_r4_ROR_22_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r1_plus_r10_LSL_29_PostIndex),
    kInstruction_str_al_r3_r1_plus_r10_LSL_29_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r6_plus_r13_ROR_6_PostIndex),
    kInstruction_str_al_r12_r6_plus_r13_ROR_6_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r11_plus_r2_LSL_19_PostIndex),
    kInstruction_str_al_r12_r11_plus_r2_LSL_19_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r10_plus_r4_LSL_31_PostIndex),
    kInstruction_str_al_r14_r10_plus_r4_LSL_31_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r4_minus_r2_ROR_28_PostIndex),
    kInstruction_str_al_r1_r4_minus_r2_ROR_28_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r4_minus_r6_ROR_20_PreIndex),
    kInstruction_str_al_r3_r4_minus_r6_ROR_20_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r8_plus_r13_ROR_11_PostIndex),
    kInstruction_str_al_r6_r8_plus_r13_ROR_11_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r0_plus_r14_LSL_27_PreIndex),
    kInstruction_str_al_r8_r0_plus_r14_LSL_27_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r2_minus_r9_ROR_3_PreIndex),
    kInstruction_str_al_r10_r2_minus_r9_ROR_3_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r9_minus_r3_ROR_11_PostIndex),
    kInstruction_str_al_r7_r9_minus_r3_ROR_11_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r14_minus_r3_LSL_24_PostIndex),
    kInstruction_str_al_r12_r14_minus_r3_LSL_24_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r6_plus_r8_LSL_26_PreIndex),
    kInstruction_str_al_r1_r6_plus_r8_LSL_26_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r10_minus_r7_LSL_11_PostIndex),
    kInstruction_str_al_r12_r10_minus_r7_LSL_11_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r2_minus_r8_LSL_13_PreIndex),
    kInstruction_str_al_r11_r2_minus_r8_LSL_13_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r8_plus_r3_ROR_9_PreIndex),
    kInstruction_str_al_r13_r8_plus_r3_ROR_9_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r4_minus_r10_ROR_19_PostIndex),
    kInstruction_str_al_r7_r4_minus_r10_ROR_19_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r4_plus_r11_LSL_8_PostIndex),
    kInstruction_str_al_r7_r4_plus_r11_LSL_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r3_plus_r8_ROR_24_PostIndex),
    kInstruction_str_al_r7_r3_plus_r8_ROR_24_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r14_plus_r12_LSL_9_PostIndex),
    kInstruction_str_al_r5_r14_plus_r12_LSL_9_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r1_minus_r5_LSL_17_PostIndex),
    kInstruction_str_al_r10_r1_minus_r5_LSL_17_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r5_plus_r11_LSL_26_PostIndex),
    kInstruction_str_al_r10_r5_plus_r11_LSL_26_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r0_plus_r2_ROR_29_PostIndex),
    kInstruction_str_al_r6_r0_plus_r2_ROR_29_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r4_plus_r11_LSL_4_PreIndex),
    kInstruction_str_al_r14_r4_plus_r11_LSL_4_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r0_minus_r14_LSL_6_PreIndex),
    kInstruction_str_al_r5_r0_minus_r14_LSL_6_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r14_plus_r6_LSL_17_PostIndex),
    kInstruction_str_al_r2_r14_plus_r6_LSL_17_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r0_minus_r0_LSL_16_PostIndex),
    kInstruction_str_al_r14_r0_minus_r0_LSL_16_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r9_minus_r9_ROR_7_PostIndex),
    kInstruction_str_al_r6_r9_minus_r9_ROR_7_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r1_minus_r13_LSL_3_PreIndex),
    kInstruction_str_al_r10_r1_minus_r13_LSL_3_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r14_minus_r9_LSL_21_PreIndex),
    kInstruction_str_al_r13_r14_minus_r9_LSL_21_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r11_plus_r8_LSL_20_PostIndex),
    kInstruction_str_al_r14_r11_plus_r8_LSL_20_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r4_minus_r11_LSL_4_PreIndex),
    kInstruction_str_al_r1_r4_minus_r11_LSL_4_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r14_minus_r5_LSL_24_PostIndex),
    kInstruction_str_al_r11_r14_minus_r5_LSL_24_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r1_minus_r0_LSL_8_PreIndex),
    kInstruction_str_al_r3_r1_minus_r0_LSL_8_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r13_plus_r12_LSL_3_PostIndex),
    kInstruction_str_al_r9_r13_plus_r12_LSL_3_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r13_plus_r5_ROR_7_PostIndex),
    kInstruction_str_al_r6_r13_plus_r5_ROR_7_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r4_minus_r7_ROR_19_PreIndex),
    kInstruction_str_al_r13_r4_minus_r7_ROR_19_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r7_minus_r9_ROR_2_PostIndex),
    kInstruction_str_al_r12_r7_minus_r9_ROR_2_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r4_plus_r4_ROR_7_PreIndex),
    kInstruction_str_al_r12_r4_plus_r4_ROR_7_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r8_minus_r7_LSL_1_PreIndex),
    kInstruction_str_al_r6_r8_minus_r7_LSL_1_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r7_minus_r2_LSL_16_PostIndex),
    kInstruction_str_al_r3_r7_minus_r2_LSL_16_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r6_plus_r4_ROR_16_PreIndex),
    kInstruction_str_al_r9_r6_plus_r4_ROR_16_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r9_minus_r2_LSL_15_PostIndex),
    kInstruction_str_al_r8_r9_minus_r2_LSL_15_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_minus_r0_ROR_31_PostIndex),
    kInstruction_str_al_r0_r12_minus_r0_ROR_31_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r9_plus_r0_LSL_15_PostIndex),
    kInstruction_str_al_r6_r9_plus_r0_LSL_15_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_minus_r13_ROR_5_PostIndex),
    kInstruction_str_al_r0_r2_minus_r13_ROR_5_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r4_minus_r5_ROR_30_PreIndex),
    kInstruction_str_al_r11_r4_minus_r5_ROR_30_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r11_minus_r5_LSL_16_PostIndex),
    kInstruction_str_al_r2_r11_minus_r5_LSL_16_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r2_minus_r0_ROR_25_PreIndex),
    kInstruction_str_al_r8_r2_minus_r0_ROR_25_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r1_minus_r7_LSL_11_PreIndex),
    kInstruction_str_al_r6_r1_minus_r7_LSL_11_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r11_minus_r5_LSL_4_PreIndex),
    kInstruction_str_al_r7_r11_minus_r5_LSL_4_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r11_plus_r5_LSL_17_PostIndex),
    kInstruction_str_al_r6_r11_plus_r5_LSL_17_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r4_minus_r2_LSL_29_PostIndex),
    kInstruction_str_al_r9_r4_minus_r2_LSL_29_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r5_minus_r0_ROR_26_PostIndex),
    kInstruction_str_al_r14_r5_minus_r0_ROR_26_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r11_plus_r0_ROR_27_PostIndex),
    kInstruction_str_al_r5_r11_plus_r0_ROR_27_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r9_minus_r11_ROR_8_PostIndex),
    kInstruction_str_al_r1_r9_minus_r11_ROR_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r0_plus_r14_ROR_11_PostIndex),
    kInstruction_str_al_r7_r0_plus_r14_ROR_11_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r0_minus_r9_LSL_9_PreIndex),
    kInstruction_str_al_r7_r0_minus_r9_LSL_9_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r3_plus_r10_LSL_6_PostIndex),
    kInstruction_str_al_r4_r3_plus_r10_LSL_6_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r0_minus_r5_ROR_14_PostIndex),
    kInstruction_str_al_r10_r0_minus_r5_ROR_14_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r2_plus_r13_LSL_17_PostIndex),
    kInstruction_str_al_r1_r2_plus_r13_LSL_17_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r2_plus_r9_ROR_22_PostIndex),
    kInstruction_str_al_r11_r2_plus_r9_ROR_22_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r9_plus_r3_LSL_19_PostIndex),
    kInstruction_str_al_r2_r9_plus_r3_LSL_19_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r3_plus_r4_ROR_23_PostIndex),
    kInstruction_str_al_r7_r3_plus_r4_ROR_23_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r9_plus_r11_LSL_8_PostIndex),
    kInstruction_str_al_r2_r9_plus_r11_LSL_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r13_minus_r1_ROR_15_PreIndex),
    kInstruction_str_al_r5_r13_minus_r1_ROR_15_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r12_plus_r2_LSL_15_PreIndex),
    kInstruction_str_al_r8_r12_plus_r2_LSL_15_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r1_plus_r13_LSL_24_PreIndex),
    kInstruction_str_al_r7_r1_plus_r13_LSL_24_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_minus_r1_ROR_13_PostIndex),
    kInstruction_str_al_r0_r9_minus_r1_ROR_13_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r9_minus_r1_LSL_1_PostIndex),
    kInstruction_str_al_r5_r9_minus_r1_LSL_1_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r4_minus_r2_LSL_4_PreIndex),
    kInstruction_str_al_r3_r4_minus_r2_LSL_4_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r10_minus_r6_ROR_5_PreIndex),
    kInstruction_str_al_r3_r10_minus_r6_ROR_5_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r14_minus_r6_ROR_16_PreIndex),
    kInstruction_str_al_r2_r14_minus_r6_ROR_16_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r1_plus_r14_ROR_3_PostIndex),
    kInstruction_str_al_r2_r1_plus_r14_ROR_3_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r2_plus_r2_LSL_22_PreIndex),
    kInstruction_str_al_r6_r2_plus_r2_LSL_22_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r7_plus_r4_ROR_3_PostIndex),
    kInstruction_str_al_r5_r7_plus_r4_ROR_3_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r9_plus_r6_ROR_3_PreIndex),
    kInstruction_str_al_r12_r9_plus_r6_ROR_3_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r7_plus_r10_ROR_29_PreIndex),
    kInstruction_str_al_r3_r7_plus_r10_ROR_29_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r0_minus_r9_ROR_9_PreIndex),
    kInstruction_str_al_r2_r0_minus_r9_ROR_9_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r12_plus_r7_LSL_3_PreIndex),
    kInstruction_str_al_r7_r12_plus_r7_LSL_3_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r5_plus_r10_LSL_15_PostIndex),
    kInstruction_str_al_r7_r5_plus_r10_LSL_15_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r4_minus_r6_ROR_13_PreIndex),
    kInstruction_str_al_r14_r4_minus_r6_ROR_13_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_plus_r7_ROR_27_PreIndex),
    kInstruction_str_al_r0_r6_plus_r7_ROR_27_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_plus_r13_ROR_23_PreIndex),
    kInstruction_str_al_r0_r14_plus_r13_ROR_23_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r5_plus_r1_LSL_8_PreIndex),
    kInstruction_str_al_r9_r5_plus_r1_LSL_8_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r3_plus_r7_LSL_12_PostIndex),
    kInstruction_str_al_r8_r3_plus_r7_LSL_12_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r5_plus_r3_LSL_16_PostIndex),
    kInstruction_str_al_r2_r5_plus_r3_LSL_16_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r12_minus_r7_LSL_8_PostIndex),
    kInstruction_str_al_r4_r12_minus_r7_LSL_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r7_minus_r5_LSL_6_PostIndex),
    kInstruction_str_al_r0_r7_minus_r5_LSL_6_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r4_plus_r12_ROR_14_PreIndex),
    kInstruction_str_al_r9_r4_plus_r12_ROR_14_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r2_minus_r14_LSL_8_PostIndex),
    kInstruction_str_al_r6_r2_minus_r14_LSL_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r12_minus_r9_ROR_14_PostIndex),
    kInstruction_str_al_r6_r12_minus_r9_ROR_14_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r7_minus_r9_LSL_3_PreIndex),
    kInstruction_str_al_r13_r7_minus_r9_LSL_3_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r1_plus_r13_ROR_20_PostIndex),
    kInstruction_str_al_r13_r1_plus_r13_ROR_20_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r11_minus_r6_ROR_4_PreIndex),
    kInstruction_str_al_r8_r11_minus_r6_ROR_4_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_plus_r8_ROR_17_PostIndex),
    kInstruction_str_al_r0_r14_plus_r8_ROR_17_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r9_plus_r10_LSL_9_PreIndex),
    kInstruction_str_al_r12_r9_plus_r10_LSL_9_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r13_plus_r1_LSL_18_PostIndex),
    kInstruction_str_al_r10_r13_plus_r1_LSL_18_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r9_plus_r3_LSL_13_PreIndex),
    kInstruction_str_al_r10_r9_plus_r3_LSL_13_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r14_plus_r5_LSL_19_PreIndex),
    kInstruction_str_al_r8_r14_plus_r5_LSL_19_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r10_minus_r6_ROR_21_PostIndex),
    kInstruction_str_al_r2_r10_minus_r6_ROR_21_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r6_plus_r5_LSL_3_PreIndex),
    kInstruction_str_al_r9_r6_plus_r5_LSL_3_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r10_plus_r4_ROR_29_PreIndex),
    kInstruction_str_al_r13_r10_plus_r4_ROR_29_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r4_plus_r5_LSL_22_PostIndex),
    kInstruction_str_al_r3_r4_plus_r5_LSL_22_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r5_minus_r10_LSL_11_PreIndex),
    kInstruction_str_al_r0_r5_minus_r10_LSL_11_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r12_minus_r14_ROR_30_PostIndex),
    kInstruction_str_al_r11_r12_minus_r14_ROR_30_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r14_minus_r10_LSL_23_PostIndex),
    kInstruction_str_al_r12_r14_minus_r10_LSL_23_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r7_plus_r5_ROR_5_PostIndex),
    kInstruction_str_al_r3_r7_plus_r5_ROR_5_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r0_minus_r14_LSL_23_PostIndex),
    kInstruction_str_al_r1_r0_minus_r14_LSL_23_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r0_plus_r13_LSL_16_PostIndex),
    kInstruction_str_al_r14_r0_plus_r13_LSL_16_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r7_minus_r10_ROR_31_PreIndex),
    kInstruction_str_al_r1_r7_minus_r10_ROR_31_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r10_minus_r12_LSL_12_PostIndex),
    kInstruction_str_al_r14_r10_minus_r12_LSL_12_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r9_minus_r9_ROR_18_PreIndex),
    kInstruction_str_al_r14_r9_minus_r9_ROR_18_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r12_plus_r2_ROR_15_PreIndex),
    kInstruction_str_al_r10_r12_plus_r2_ROR_15_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r14_minus_r0_LSL_11_PreIndex),
    kInstruction_str_al_r13_r14_minus_r0_LSL_11_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r5_plus_r6_ROR_2_PreIndex),
    kInstruction_str_al_r3_r5_plus_r6_ROR_2_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r3_minus_r14_ROR_27_PostIndex),
    kInstruction_str_al_r9_r3_minus_r14_ROR_27_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r10_minus_r7_ROR_19_PostIndex),
    kInstruction_str_al_r2_r10_minus_r7_ROR_19_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r5_plus_r11_LSL_29_PostIndex),
    kInstruction_str_al_r12_r5_plus_r11_LSL_29_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r9_minus_r12_ROR_13_PreIndex),
    kInstruction_str_al_r2_r9_minus_r12_ROR_13_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r1_plus_r13_LSL_26_PreIndex),
    kInstruction_str_al_r11_r1_plus_r13_LSL_26_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r0_minus_r1_ROR_3_PreIndex),
    kInstruction_str_al_r12_r0_minus_r1_ROR_3_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r14_plus_r4_LSL_19_PreIndex),
    kInstruction_str_al_r11_r14_plus_r4_LSL_19_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r12_plus_r3_ROR_30_PreIndex),
    kInstruction_str_al_r8_r12_plus_r3_ROR_30_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r0_plus_r6_LSL_24_PostIndex),
    kInstruction_str_al_r6_r0_plus_r6_LSL_24_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r6_minus_r11_ROR_16_PreIndex),
    kInstruction_str_al_r4_r6_minus_r11_ROR_16_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r0_plus_r11_ROR_8_PostIndex),
    kInstruction_str_al_r8_r0_plus_r11_ROR_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r5_plus_r1_ROR_25_PostIndex),
    kInstruction_str_al_r7_r5_plus_r1_ROR_25_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r12_plus_r11_ROR_12_PreIndex),
    kInstruction_str_al_r1_r12_plus_r11_ROR_12_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r2_plus_r14_ROR_8_PostIndex),
    kInstruction_str_al_r4_r2_plus_r14_ROR_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r0_plus_r1_LSL_10_PostIndex),
    kInstruction_str_al_r5_r0_plus_r1_LSL_10_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_minus_r4_ROR_30_PostIndex),
    kInstruction_str_al_r0_r11_minus_r4_ROR_30_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r3_minus_r7_LSL_2_PostIndex),
    kInstruction_str_al_r14_r3_minus_r7_LSL_2_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r8_plus_r10_LSL_26_PostIndex),
    kInstruction_str_al_r4_r8_plus_r10_LSL_26_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r2_minus_r8_LSL_21_PreIndex),
    kInstruction_str_al_r12_r2_minus_r8_LSL_21_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r14_minus_r14_LSL_24_PreIndex),
    kInstruction_str_al_r9_r14_minus_r14_LSL_24_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r13_minus_r11_LSL_2_PostIndex),
    kInstruction_str_al_r7_r13_minus_r11_LSL_2_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r0_ROR_30_PreIndex),
    kInstruction_str_al_r0_r2_plus_r0_ROR_30_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r6_minus_r0_ROR_1_PreIndex),
    kInstruction_str_al_r1_r6_minus_r0_ROR_1_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r5_plus_r12_LSL_4_PostIndex),
    kInstruction_str_al_r6_r5_plus_r12_LSL_4_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r9_plus_r13_LSL_11_PostIndex),
    kInstruction_str_al_r7_r9_plus_r13_LSL_11_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r12_minus_r2_ROR_5_PostIndex),
    kInstruction_str_al_r5_r12_minus_r2_ROR_5_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r7_plus_r14_LSL_30_PreIndex),
    kInstruction_str_al_r6_r7_plus_r14_LSL_30_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r11_minus_r0_ROR_20_PostIndex),
    kInstruction_str_al_r2_r11_minus_r0_ROR_20_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r2_minus_r10_ROR_11_PostIndex),
    kInstruction_str_al_r6_r2_minus_r10_ROR_11_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r2_plus_r0_ROR_4_PostIndex),
    kInstruction_str_al_r3_r2_plus_r0_ROR_4_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r6_plus_r12_LSL_11_PostIndex),
    kInstruction_str_al_r8_r6_plus_r12_LSL_11_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_minus_r4_ROR_8_PreIndex),
    kInstruction_str_al_r0_r6_minus_r4_ROR_8_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r1_minus_r3_ROR_5_PostIndex),
    kInstruction_str_al_r10_r1_minus_r3_ROR_5_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r4_plus_r12_LSL_9_PreIndex),
    kInstruction_str_al_r10_r4_plus_r12_LSL_9_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r9_minus_r5_ROR_7_PostIndex),
    kInstruction_str_al_r1_r9_minus_r5_ROR_7_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r7_plus_r1_ROR_14_PreIndex),
    kInstruction_str_al_r5_r7_plus_r1_ROR_14_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r1_plus_r2_LSL_20_PreIndex),
    kInstruction_str_al_r9_r1_plus_r2_LSL_20_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r8_minus_r5_ROR_13_PostIndex),
    kInstruction_str_al_r6_r8_minus_r5_ROR_13_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r8_plus_r1_LSL_17_PreIndex),
    kInstruction_str_al_r4_r8_plus_r1_LSL_17_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r13_plus_r6_LSL_31_PostIndex),
    kInstruction_str_al_r11_r13_plus_r6_LSL_31_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r0_minus_r4_LSL_7_PostIndex),
    kInstruction_str_al_r9_r0_minus_r4_LSL_7_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r1_minus_r6_LSL_11_PreIndex),
    kInstruction_str_al_r4_r1_minus_r6_LSL_11_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r13_plus_r11_ROR_14_PostIndex),
    kInstruction_str_al_r10_r13_plus_r11_ROR_14_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r11_plus_r9_ROR_31_PreIndex),
    kInstruction_str_al_r12_r11_plus_r9_ROR_31_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r1_minus_r2_ROR_5_PreIndex),
    kInstruction_str_al_r9_r1_minus_r2_ROR_5_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r10_minus_r4_LSL_18_PostIndex),
    kInstruction_str_al_r1_r10_minus_r4_LSL_18_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r14_plus_r0_ROR_24_PostIndex),
    kInstruction_str_al_r5_r14_plus_r0_ROR_24_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r10_plus_r1_ROR_22_PreIndex),
    kInstruction_str_al_r9_r10_plus_r1_ROR_22_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r0_plus_r9_LSL_9_PostIndex),
    kInstruction_str_al_r12_r0_plus_r9_LSL_9_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r11_plus_r11_ROR_22_PreIndex),
    kInstruction_str_al_r3_r11_plus_r11_ROR_22_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r12_LSL_23_PostIndex),
    kInstruction_str_al_r0_r4_minus_r12_LSL_23_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r3_minus_r6_ROR_6_PostIndex),
    kInstruction_str_al_r13_r3_minus_r6_ROR_6_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r2_plus_r8_ROR_10_PostIndex),
    kInstruction_str_al_r10_r2_plus_r8_ROR_10_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r13_minus_r6_ROR_19_PostIndex),
    kInstruction_str_al_r6_r13_minus_r6_ROR_19_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r3_plus_r5_LSL_20_PostIndex),
    kInstruction_str_al_r4_r3_plus_r5_LSL_20_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r12_minus_r6_ROR_28_PostIndex),
    kInstruction_str_al_r1_r12_minus_r6_ROR_28_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r8_plus_r7_ROR_17_PreIndex),
    kInstruction_str_al_r2_r8_plus_r7_ROR_17_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r5_minus_r10_ROR_3_PreIndex),
    kInstruction_str_al_r2_r5_minus_r10_ROR_3_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r14_minus_r0_LSL_16_PreIndex),
    kInstruction_str_al_r9_r14_minus_r0_LSL_16_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r3_minus_r10_ROR_23_PreIndex),
    kInstruction_str_al_r6_r3_minus_r10_ROR_23_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_minus_r8_LSL_11_PostIndex),
    kInstruction_str_al_r0_r9_minus_r8_LSL_11_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r2_plus_r14_LSL_9_PreIndex),
    kInstruction_str_al_r9_r2_plus_r14_LSL_9_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r7_minus_r6_LSL_19_PreIndex),
    kInstruction_str_al_r13_r7_minus_r6_LSL_19_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r4_minus_r2_LSL_10_PostIndex),
    kInstruction_str_al_r13_r4_minus_r2_LSL_10_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r11_minus_r0_LSL_3_PostIndex),
    kInstruction_str_al_r9_r11_minus_r0_LSL_3_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r13_plus_r8_LSL_28_PreIndex),
    kInstruction_str_al_r11_r13_plus_r8_LSL_28_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r9_minus_r7_ROR_13_PreIndex),
    kInstruction_str_al_r7_r9_minus_r7_ROR_13_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r7_plus_r5_LSL_3_PostIndex),
    kInstruction_str_al_r13_r7_plus_r5_LSL_3_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r10_plus_r14_ROR_1_PostIndex),
    kInstruction_str_al_r14_r10_plus_r14_ROR_1_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r6_plus_r14_ROR_3_PostIndex),
    kInstruction_str_al_r1_r6_plus_r14_ROR_3_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r14_minus_r11_ROR_17_PreIndex),
    kInstruction_str_al_r12_r14_minus_r11_ROR_17_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r5_minus_r10_LSL_3_PostIndex),
    kInstruction_str_al_r4_r5_minus_r10_LSL_3_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_minus_r5_ROR_16_PreIndex),
    kInstruction_str_al_r0_r14_minus_r5_ROR_16_PreIndex,
  },
};

#endif  // VIXL_ASSEMBLER_COND_RD_MEMOP_RS_SHIFT_AMOUNT_1TO31_STR_A32_H_
