// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_ASSEMBLER_COND_RD_MEMOP_RS_SHIFT_AMOUNT_1TO32_STR_A32_H_
#define VIXL_ASSEMBLER_COND_RD_MEMOP_RS_SHIFT_AMOUNT_1TO32_STR_A32_H_

const byte kInstruction_str_pl_r8_r11_plus_r6_LSR_1_Offset[] = {
  0xa6, 0x80, 0x8b, 0x57 // str pl r8 r11 plus r6 LSR 1 Offset
};
const byte kInstruction_str_le_r4_r8_plus_r5_LSR_1_Offset[] = {
  0xa5, 0x40, 0x88, 0xd7 // str le r4 r8 plus r5 LSR 1 Offset
};
const byte kInstruction_str_vs_r2_r6_plus_r14_LSR_1_Offset[] = {
  0xae, 0x20, 0x86, 0x67 // str vs r2 r6 plus r14 LSR 1 Offset
};
const byte kInstruction_str_ls_r1_r7_plus_r8_LSR_1_Offset[] = {
  0xa8, 0x10, 0x87, 0x97 // str ls r1 r7 plus r8 LSR 1 Offset
};
const byte kInstruction_str_ge_r14_r6_plus_r14_LSR_1_Offset[] = {
  0xae, 0xe0, 0x86, 0xa7 // str ge r14 r6 plus r14 LSR 1 Offset
};
const byte kInstruction_str_cs_r7_r0_plus_r7_LSR_1_Offset[] = {
  0xa7, 0x70, 0x80, 0x27 // str cs r7 r0 plus r7 LSR 1 Offset
};
const byte kInstruction_str_ge_r11_r0_plus_r9_LSR_1_Offset[] = {
  0xa9, 0xb0, 0x80, 0xa7 // str ge r11 r0 plus r9 LSR 1 Offset
};
const byte kInstruction_str_eq_r7_r10_plus_r4_LSR_1_Offset[] = {
  0xa4, 0x70, 0x8a, 0x07 // str eq r7 r10 plus r4 LSR 1 Offset
};
const byte kInstruction_str_al_r9_r2_plus_r3_LSR_1_Offset[] = {
  0xa3, 0x90, 0x82, 0xe7 // str al r9 r2 plus r3 LSR 1 Offset
};
const byte kInstruction_str_cc_r11_r10_plus_r6_LSR_1_Offset[] = {
  0xa6, 0xb0, 0x8a, 0x37 // str cc r11 r10 plus r6 LSR 1 Offset
};
const byte kInstruction_str_lt_r9_r6_plus_r1_LSR_1_Offset[] = {
  0xa1, 0x90, 0x86, 0xb7 // str lt r9 r6 plus r1 LSR 1 Offset
};
const byte kInstruction_str_ge_r4_r0_plus_r4_LSR_1_Offset[] = {
  0xa4, 0x40, 0x80, 0xa7 // str ge r4 r0 plus r4 LSR 1 Offset
};
const byte kInstruction_str_al_r9_r13_plus_r14_LSR_1_Offset[] = {
  0xae, 0x90, 0x8d, 0xe7 // str al r9 r13 plus r14 LSR 1 Offset
};
const byte kInstruction_str_cc_r9_r0_plus_r7_LSR_1_Offset[] = {
  0xa7, 0x90, 0x80, 0x37 // str cc r9 r0 plus r7 LSR 1 Offset
};
const byte kInstruction_str_cs_r11_r7_plus_r8_LSR_1_Offset[] = {
  0xa8, 0xb0, 0x87, 0x27 // str cs r11 r7 plus r8 LSR 1 Offset
};
const byte kInstruction_str_eq_r8_r0_plus_r8_LSR_1_Offset[] = {
  0xa8, 0x80, 0x80, 0x07 // str eq r8 r0 plus r8 LSR 1 Offset
};
const byte kInstruction_str_hi_r9_r10_plus_r8_LSR_1_Offset[] = {
  0xa8, 0x90, 0x8a, 0x87 // str hi r9 r10 plus r8 LSR 1 Offset
};
const byte kInstruction_str_le_r8_r0_plus_r0_LSR_1_Offset[] = {
  0xa0, 0x80, 0x80, 0xd7 // str le r8 r0 plus r0 LSR 1 Offset
};
const byte kInstruction_str_cs_r4_r5_plus_r2_LSR_1_Offset[] = {
  0xa2, 0x40, 0x85, 0x27 // str cs r4 r5 plus r2 LSR 1 Offset
};
const byte kInstruction_str_eq_r14_r11_plus_r8_LSR_1_Offset[] = {
  0xa8, 0xe0, 0x8b, 0x07 // str eq r14 r11 plus r8 LSR 1 Offset
};
const byte kInstruction_str_cs_r7_r6_plus_r14_LSR_1_Offset[] = {
  0xae, 0x70, 0x86, 0x27 // str cs r7 r6 plus r14 LSR 1 Offset
};
const byte kInstruction_str_ne_r0_r9_plus_r7_LSR_1_Offset[] = {
  0xa7, 0x00, 0x89, 0x17 // str ne r0 r9 plus r7 LSR 1 Offset
};
const byte kInstruction_str_le_r7_r3_plus_r11_LSR_1_Offset[] = {
  0xab, 0x70, 0x83, 0xd7 // str le r7 r3 plus r11 LSR 1 Offset
};
const byte kInstruction_str_gt_r7_r9_plus_r9_LSR_1_Offset[] = {
  0xa9, 0x70, 0x89, 0xc7 // str gt r7 r9 plus r9 LSR 1 Offset
};
const byte kInstruction_str_le_r9_r12_plus_r9_LSR_1_Offset[] = {
  0xa9, 0x90, 0x8c, 0xd7 // str le r9 r12 plus r9 LSR 1 Offset
};
const byte kInstruction_str_pl_r2_r11_plus_r14_LSR_1_Offset[] = {
  0xae, 0x20, 0x8b, 0x57 // str pl r2 r11 plus r14 LSR 1 Offset
};
const byte kInstruction_str_cs_r1_r7_plus_r2_LSR_1_Offset[] = {
  0xa2, 0x10, 0x87, 0x27 // str cs r1 r7 plus r2 LSR 1 Offset
};
const byte kInstruction_str_al_r5_r1_plus_r12_LSR_1_Offset[] = {
  0xac, 0x50, 0x81, 0xe7 // str al r5 r1 plus r12 LSR 1 Offset
};
const byte kInstruction_str_eq_r10_r13_plus_r12_LSR_1_Offset[] = {
  0xac, 0xa0, 0x8d, 0x07 // str eq r10 r13 plus r12 LSR 1 Offset
};
const byte kInstruction_str_eq_r10_r11_plus_r3_LSR_1_Offset[] = {
  0xa3, 0xa0, 0x8b, 0x07 // str eq r10 r11 plus r3 LSR 1 Offset
};
const byte kInstruction_str_al_r9_r12_plus_r0_LSR_1_Offset[] = {
  0xa0, 0x90, 0x8c, 0xe7 // str al r9 r12 plus r0 LSR 1 Offset
};
const byte kInstruction_str_ls_r3_r14_plus_r1_LSR_1_Offset[] = {
  0xa1, 0x30, 0x8e, 0x97 // str ls r3 r14 plus r1 LSR 1 Offset
};
const byte kInstruction_str_le_r13_r4_plus_r9_LSR_1_Offset[] = {
  0xa9, 0xd0, 0x84, 0xd7 // str le r13 r4 plus r9 LSR 1 Offset
};
const byte kInstruction_str_ls_r3_r14_plus_r3_LSR_1_Offset[] = {
  0xa3, 0x30, 0x8e, 0x97 // str ls r3 r14 plus r3 LSR 1 Offset
};
const byte kInstruction_str_cc_r7_r4_plus_r4_LSR_1_Offset[] = {
  0xa4, 0x70, 0x84, 0x37 // str cc r7 r4 plus r4 LSR 1 Offset
};
const byte kInstruction_str_ls_r5_r9_plus_r5_LSR_1_Offset[] = {
  0xa5, 0x50, 0x89, 0x97 // str ls r5 r9 plus r5 LSR 1 Offset
};
const byte kInstruction_str_ge_r6_r4_plus_r13_LSR_1_Offset[] = {
  0xad, 0x60, 0x84, 0xa7 // str ge r6 r4 plus r13 LSR 1 Offset
};
const byte kInstruction_str_al_r1_r11_plus_r12_LSR_1_Offset[] = {
  0xac, 0x10, 0x8b, 0xe7 // str al r1 r11 plus r12 LSR 1 Offset
};
const byte kInstruction_str_le_r6_r11_plus_r3_LSR_1_Offset[] = {
  0xa3, 0x60, 0x8b, 0xd7 // str le r6 r11 plus r3 LSR 1 Offset
};
const byte kInstruction_str_al_r6_r6_plus_r13_LSR_1_Offset[] = {
  0xad, 0x60, 0x86, 0xe7 // str al r6 r6 plus r13 LSR 1 Offset
};
const byte kInstruction_str_gt_r14_r11_plus_r2_LSR_1_Offset[] = {
  0xa2, 0xe0, 0x8b, 0xc7 // str gt r14 r11 plus r2 LSR 1 Offset
};
const byte kInstruction_str_vs_r5_r8_plus_r12_LSR_1_Offset[] = {
  0xac, 0x50, 0x88, 0x67 // str vs r5 r8 plus r12 LSR 1 Offset
};
const byte kInstruction_str_ls_r8_r12_plus_r6_LSR_1_Offset[] = {
  0xa6, 0x80, 0x8c, 0x97 // str ls r8 r12 plus r6 LSR 1 Offset
};
const byte kInstruction_str_vs_r6_r12_plus_r14_LSR_1_Offset[] = {
  0xae, 0x60, 0x8c, 0x67 // str vs r6 r12 plus r14 LSR 1 Offset
};
const byte kInstruction_str_pl_r7_r2_plus_r7_LSR_1_Offset[] = {
  0xa7, 0x70, 0x82, 0x57 // str pl r7 r2 plus r7 LSR 1 Offset
};
const byte kInstruction_str_gt_r5_r4_plus_r13_LSR_1_Offset[] = {
  0xad, 0x50, 0x84, 0xc7 // str gt r5 r4 plus r13 LSR 1 Offset
};
const byte kInstruction_str_hi_r2_r7_plus_r12_LSR_1_Offset[] = {
  0xac, 0x20, 0x87, 0x87 // str hi r2 r7 plus r12 LSR 1 Offset
};
const byte kInstruction_str_ne_r2_r3_plus_r10_LSR_1_Offset[] = {
  0xaa, 0x20, 0x83, 0x17 // str ne r2 r3 plus r10 LSR 1 Offset
};
const byte kInstruction_str_lt_r13_r2_plus_r6_LSR_1_Offset[] = {
  0xa6, 0xd0, 0x82, 0xb7 // str lt r13 r2 plus r6 LSR 1 Offset
};
const byte kInstruction_str_hi_r14_r10_plus_r11_LSR_1_Offset[] = {
  0xab, 0xe0, 0x8a, 0x87 // str hi r14 r10 plus r11 LSR 1 Offset
};
const byte kInstruction_str_hi_r7_r10_plus_r12_LSR_1_Offset[] = {
  0xac, 0x70, 0x8a, 0x87 // str hi r7 r10 plus r12 LSR 1 Offset
};
const byte kInstruction_str_cs_r3_r12_plus_r2_LSR_1_Offset[] = {
  0xa2, 0x30, 0x8c, 0x27 // str cs r3 r12 plus r2 LSR 1 Offset
};
const byte kInstruction_str_ls_r11_r5_plus_r1_LSR_1_Offset[] = {
  0xa1, 0xb0, 0x85, 0x97 // str ls r11 r5 plus r1 LSR 1 Offset
};
const byte kInstruction_str_hi_r5_r2_plus_r11_LSR_1_Offset[] = {
  0xab, 0x50, 0x82, 0x87 // str hi r5 r2 plus r11 LSR 1 Offset
};
const byte kInstruction_str_cc_r11_r8_plus_r6_LSR_1_Offset[] = {
  0xa6, 0xb0, 0x88, 0x37 // str cc r11 r8 plus r6 LSR 1 Offset
};
const byte kInstruction_str_lt_r11_r3_plus_r9_LSR_1_Offset[] = {
  0xa9, 0xb0, 0x83, 0xb7 // str lt r11 r3 plus r9 LSR 1 Offset
};
const byte kInstruction_str_ne_r11_r9_plus_r8_LSR_1_Offset[] = {
  0xa8, 0xb0, 0x89, 0x17 // str ne r11 r9 plus r8 LSR 1 Offset
};
const byte kInstruction_str_lt_r7_r0_plus_r14_LSR_1_Offset[] = {
  0xae, 0x70, 0x80, 0xb7 // str lt r7 r0 plus r14 LSR 1 Offset
};
const byte kInstruction_str_ge_r12_r3_plus_r1_LSR_1_Offset[] = {
  0xa1, 0xc0, 0x83, 0xa7 // str ge r12 r3 plus r1 LSR 1 Offset
};
const byte kInstruction_str_le_r0_r8_plus_r13_LSR_1_Offset[] = {
  0xad, 0x00, 0x88, 0xd7 // str le r0 r8 plus r13 LSR 1 Offset
};
const byte kInstruction_str_vc_r5_r13_plus_r7_LSR_1_Offset[] = {
  0xa7, 0x50, 0x8d, 0x77 // str vc r5 r13 plus r7 LSR 1 Offset
};
const byte kInstruction_str_ge_r7_r5_plus_r11_LSR_1_Offset[] = {
  0xab, 0x70, 0x85, 0xa7 // str ge r7 r5 plus r11 LSR 1 Offset
};
const byte kInstruction_str_ge_r10_r9_plus_r3_LSR_1_Offset[] = {
  0xa3, 0xa0, 0x89, 0xa7 // str ge r10 r9 plus r3 LSR 1 Offset
};
const byte kInstruction_str_vs_r9_r5_plus_r2_LSR_1_Offset[] = {
  0xa2, 0x90, 0x85, 0x67 // str vs r9 r5 plus r2 LSR 1 Offset
};
const byte kInstruction_str_hi_r2_r10_plus_r4_LSR_1_Offset[] = {
  0xa4, 0x20, 0x8a, 0x87 // str hi r2 r10 plus r4 LSR 1 Offset
};
const byte kInstruction_str_ge_r1_r11_plus_r1_LSR_1_Offset[] = {
  0xa1, 0x10, 0x8b, 0xa7 // str ge r1 r11 plus r1 LSR 1 Offset
};
const byte kInstruction_str_ls_r9_r14_plus_r12_LSR_1_Offset[] = {
  0xac, 0x90, 0x8e, 0x97 // str ls r9 r14 plus r12 LSR 1 Offset
};
const byte kInstruction_str_mi_r1_r9_plus_r4_LSR_1_Offset[] = {
  0xa4, 0x10, 0x89, 0x47 // str mi r1 r9 plus r4 LSR 1 Offset
};
const byte kInstruction_str_mi_r7_r10_plus_r4_LSR_1_Offset[] = {
  0xa4, 0x70, 0x8a, 0x47 // str mi r7 r10 plus r4 LSR 1 Offset
};
const byte kInstruction_str_gt_r6_r2_plus_r5_LSR_1_Offset[] = {
  0xa5, 0x60, 0x82, 0xc7 // str gt r6 r2 plus r5 LSR 1 Offset
};
const byte kInstruction_str_eq_r10_r8_plus_r11_LSR_1_Offset[] = {
  0xab, 0xa0, 0x88, 0x07 // str eq r10 r8 plus r11 LSR 1 Offset
};
const byte kInstruction_str_le_r10_r1_plus_r7_LSR_1_Offset[] = {
  0xa7, 0xa0, 0x81, 0xd7 // str le r10 r1 plus r7 LSR 1 Offset
};
const byte kInstruction_str_pl_r8_r14_plus_r8_LSR_1_Offset[] = {
  0xa8, 0x80, 0x8e, 0x57 // str pl r8 r14 plus r8 LSR 1 Offset
};
const byte kInstruction_str_gt_r5_r1_plus_r3_LSR_1_Offset[] = {
  0xa3, 0x50, 0x81, 0xc7 // str gt r5 r1 plus r3 LSR 1 Offset
};
const byte kInstruction_str_cs_r10_r4_plus_r13_LSR_1_Offset[] = {
  0xad, 0xa0, 0x84, 0x27 // str cs r10 r4 plus r13 LSR 1 Offset
};
const byte kInstruction_str_mi_r10_r5_plus_r2_LSR_1_Offset[] = {
  0xa2, 0xa0, 0x85, 0x47 // str mi r10 r5 plus r2 LSR 1 Offset
};
const byte kInstruction_str_cc_r6_r0_plus_r11_LSR_1_Offset[] = {
  0xab, 0x60, 0x80, 0x37 // str cc r6 r0 plus r11 LSR 1 Offset
};
const byte kInstruction_str_eq_r3_r9_plus_r2_LSR_1_Offset[] = {
  0xa2, 0x30, 0x89, 0x07 // str eq r3 r9 plus r2 LSR 1 Offset
};
const byte kInstruction_str_al_r1_r0_plus_r3_LSR_1_Offset[] = {
  0xa3, 0x10, 0x80, 0xe7 // str al r1 r0 plus r3 LSR 1 Offset
};
const byte kInstruction_str_mi_r1_r0_plus_r13_LSR_1_Offset[] = {
  0xad, 0x10, 0x80, 0x47 // str mi r1 r0 plus r13 LSR 1 Offset
};
const byte kInstruction_str_cc_r4_r4_plus_r5_LSR_1_Offset[] = {
  0xa5, 0x40, 0x84, 0x37 // str cc r4 r4 plus r5 LSR 1 Offset
};
const byte kInstruction_str_al_r2_r6_plus_r11_LSR_1_Offset[] = {
  0xab, 0x20, 0x86, 0xe7 // str al r2 r6 plus r11 LSR 1 Offset
};
const byte kInstruction_str_ls_r1_r5_plus_r4_LSR_1_Offset[] = {
  0xa4, 0x10, 0x85, 0x97 // str ls r1 r5 plus r4 LSR 1 Offset
};
const byte kInstruction_str_eq_r0_r3_plus_r4_LSR_1_Offset[] = {
  0xa4, 0x00, 0x83, 0x07 // str eq r0 r3 plus r4 LSR 1 Offset
};
const byte kInstruction_str_lt_r7_r11_plus_r10_LSR_1_Offset[] = {
  0xaa, 0x70, 0x8b, 0xb7 // str lt r7 r11 plus r10 LSR 1 Offset
};
const byte kInstruction_str_vc_r3_r0_plus_r13_LSR_1_Offset[] = {
  0xad, 0x30, 0x80, 0x77 // str vc r3 r0 plus r13 LSR 1 Offset
};
const byte kInstruction_str_ls_r3_r3_plus_r7_LSR_1_Offset[] = {
  0xa7, 0x30, 0x83, 0x97 // str ls r3 r3 plus r7 LSR 1 Offset
};
const byte kInstruction_str_al_r5_r14_plus_r3_LSR_1_Offset[] = {
  0xa3, 0x50, 0x8e, 0xe7 // str al r5 r14 plus r3 LSR 1 Offset
};
const byte kInstruction_str_ne_r2_r13_plus_r9_LSR_1_Offset[] = {
  0xa9, 0x20, 0x8d, 0x17 // str ne r2 r13 plus r9 LSR 1 Offset
};
const byte kInstruction_str_lt_r6_r2_plus_r11_LSR_1_Offset[] = {
  0xab, 0x60, 0x82, 0xb7 // str lt r6 r2 plus r11 LSR 1 Offset
};
const byte kInstruction_str_vc_r9_r2_plus_r13_LSR_1_Offset[] = {
  0xad, 0x90, 0x82, 0x77 // str vc r9 r2 plus r13 LSR 1 Offset
};
const byte kInstruction_str_ne_r5_r9_plus_r6_LSR_1_Offset[] = {
  0xa6, 0x50, 0x89, 0x17 // str ne r5 r9 plus r6 LSR 1 Offset
};
const byte kInstruction_str_vc_r4_r9_plus_r8_LSR_1_Offset[] = {
  0xa8, 0x40, 0x89, 0x77 // str vc r4 r9 plus r8 LSR 1 Offset
};
const byte kInstruction_str_cc_r6_r9_plus_r2_LSR_1_Offset[] = {
  0xa2, 0x60, 0x89, 0x37 // str cc r6 r9 plus r2 LSR 1 Offset
};
const byte kInstruction_str_vs_r8_r7_plus_r13_LSR_1_Offset[] = {
  0xad, 0x80, 0x87, 0x67 // str vs r8 r7 plus r13 LSR 1 Offset
};
const byte kInstruction_str_pl_r14_r14_plus_r7_LSR_1_Offset[] = {
  0xa7, 0xe0, 0x8e, 0x57 // str pl r14 r14 plus r7 LSR 1 Offset
};
const byte kInstruction_str_eq_r8_r2_plus_r6_LSR_1_Offset[] = {
  0xa6, 0x80, 0x82, 0x07 // str eq r8 r2 plus r6 LSR 1 Offset
};
const byte kInstruction_str_al_r13_r13_plus_r13_LSR_1_Offset[] = {
  0xad, 0xd0, 0x8d, 0xe7 // str al r13 r13 plus r13 LSR 1 Offset
};
const byte kInstruction_str_gt_r12_r14_plus_r5_LSR_1_Offset[] = {
  0xa5, 0xc0, 0x8e, 0xc7 // str gt r12 r14 plus r5 LSR 1 Offset
};
const byte kInstruction_str_lt_r14_r7_plus_r1_LSR_1_Offset[] = {
  0xa1, 0xe0, 0x87, 0xb7 // str lt r14 r7 plus r1 LSR 1 Offset
};
const byte kInstruction_str_al_r0_r2_plus_r12_LSR_20_Offset[] = {
  0x2c, 0x0a, 0x82, 0xe7 // str al r0 r2 plus r12 LSR 20 Offset
};
const byte kInstruction_str_al_r0_r4_plus_r4_LSR_19_Offset[] = {
  0xa4, 0x09, 0x84, 0xe7 // str al r0 r4 plus r4 LSR 19 Offset
};
const byte kInstruction_str_al_r0_r13_plus_r3_LSR_28_Offset[] = {
  0x23, 0x0e, 0x8d, 0xe7 // str al r0 r13 plus r3 LSR 28 Offset
};
const byte kInstruction_str_al_r0_r14_minus_r1_ASR_1_Offset[] = {
  0xc1, 0x00, 0x0e, 0xe7 // str al r0 r14 minus r1 ASR 1 Offset
};
const byte kInstruction_str_al_r0_r4_plus_r8_ASR_4_Offset[] = {
  0x48, 0x02, 0x84, 0xe7 // str al r0 r4 plus r8 ASR 4 Offset
};
const byte kInstruction_str_al_r0_r9_minus_r1_ASR_26_Offset[] = {
  0x41, 0x0d, 0x09, 0xe7 // str al r0 r9 minus r1 ASR 26 Offset
};
const byte kInstruction_str_al_r0_r6_minus_r9_LSR_20_Offset[] = {
  0x29, 0x0a, 0x06, 0xe7 // str al r0 r6 minus r9 LSR 20 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r13_ASR_13_Offset[] = {
  0xcd, 0x06, 0x80, 0xe7 // str al r0 r0 plus r13 ASR 13 Offset
};
const byte kInstruction_str_al_r0_r9_minus_r7_ASR_23_Offset[] = {
  0xc7, 0x0b, 0x09, 0xe7 // str al r0 r9 minus r7 ASR 23 Offset
};
const byte kInstruction_str_al_r0_r13_minus_r5_ASR_25_Offset[] = {
  0xc5, 0x0c, 0x0d, 0xe7 // str al r0 r13 minus r5 ASR 25 Offset
};
const byte kInstruction_str_al_r0_r7_plus_r5_ASR_14_Offset[] = {
  0x45, 0x07, 0x87, 0xe7 // str al r0 r7 plus r5 ASR 14 Offset
};
const byte kInstruction_str_al_r0_r3_minus_r6_LSR_28_Offset[] = {
  0x26, 0x0e, 0x03, 0xe7 // str al r0 r3 minus r6 LSR 28 Offset
};
const byte kInstruction_str_al_r0_r7_plus_r5_LSR_27_Offset[] = {
  0xa5, 0x0d, 0x87, 0xe7 // str al r0 r7 plus r5 LSR 27 Offset
};
const byte kInstruction_str_al_r0_r2_minus_r5_ASR_19_Offset[] = {
  0xc5, 0x09, 0x02, 0xe7 // str al r0 r2 minus r5 ASR 19 Offset
};
const byte kInstruction_str_al_r0_r10_minus_r7_ASR_18_Offset[] = {
  0x47, 0x09, 0x0a, 0xe7 // str al r0 r10 minus r7 ASR 18 Offset
};
const byte kInstruction_str_al_r0_r0_minus_r1_ASR_10_Offset[] = {
  0x41, 0x05, 0x00, 0xe7 // str al r0 r0 minus r1 ASR 10 Offset
};
const byte kInstruction_str_al_r0_r8_plus_r14_ASR_17_Offset[] = {
  0xce, 0x08, 0x88, 0xe7 // str al r0 r8 plus r14 ASR 17 Offset
};
const byte kInstruction_str_al_r0_r4_minus_r7_LSR_14_Offset[] = {
  0x27, 0x07, 0x04, 0xe7 // str al r0 r4 minus r7 LSR 14 Offset
};
const byte kInstruction_str_al_r0_r14_minus_r4_ASR_13_Offset[] = {
  0xc4, 0x06, 0x0e, 0xe7 // str al r0 r14 minus r4 ASR 13 Offset
};
const byte kInstruction_str_al_r0_r9_minus_r2_ASR_6_Offset[] = {
  0x42, 0x03, 0x09, 0xe7 // str al r0 r9 minus r2 ASR 6 Offset
};
const byte kInstruction_str_al_r0_r12_plus_r1_LSR_19_Offset[] = {
  0xa1, 0x09, 0x8c, 0xe7 // str al r0 r12 plus r1 LSR 19 Offset
};
const byte kInstruction_str_al_r0_r9_plus_r4_LSR_4_Offset[] = {
  0x24, 0x02, 0x89, 0xe7 // str al r0 r9 plus r4 LSR 4 Offset
};
const byte kInstruction_str_al_r0_r6_plus_r14_LSR_10_Offset[] = {
  0x2e, 0x05, 0x86, 0xe7 // str al r0 r6 plus r14 LSR 10 Offset
};
const byte kInstruction_str_al_r0_r6_minus_r6_LSR_25_Offset[] = {
  0xa6, 0x0c, 0x06, 0xe7 // str al r0 r6 minus r6 LSR 25 Offset
};
const byte kInstruction_str_al_r0_r12_plus_r8_ASR_14_Offset[] = {
  0x48, 0x07, 0x8c, 0xe7 // str al r0 r12 plus r8 ASR 14 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r7_LSR_32_Offset[] = {
  0x27, 0x00, 0x80, 0xe7 // str al r0 r0 plus r7 LSR 32 Offset
};
const byte kInstruction_str_al_r0_r10_minus_r2_LSR_23_Offset[] = {
  0xa2, 0x0b, 0x0a, 0xe7 // str al r0 r10 minus r2 LSR 23 Offset
};
const byte kInstruction_str_al_r0_r2_plus_r1_ASR_1_Offset[] = {
  0xc1, 0x00, 0x82, 0xe7 // str al r0 r2 plus r1 ASR 1 Offset
};
const byte kInstruction_str_al_r0_r13_minus_r11_LSR_2_Offset[] = {
  0x2b, 0x01, 0x0d, 0xe7 // str al r0 r13 minus r11 LSR 2 Offset
};
const byte kInstruction_str_al_r0_r8_plus_r11_LSR_10_Offset[] = {
  0x2b, 0x05, 0x88, 0xe7 // str al r0 r8 plus r11 LSR 10 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r8_ASR_1_Offset[] = {
  0xc8, 0x00, 0x80, 0xe7 // str al r0 r0 plus r8 ASR 1 Offset
};
const byte kInstruction_str_al_r0_r4_minus_r6_ASR_14_Offset[] = {
  0x46, 0x07, 0x04, 0xe7 // str al r0 r4 minus r6 ASR 14 Offset
};
const byte kInstruction_str_al_r0_r2_minus_r1_ASR_25_Offset[] = {
  0xc1, 0x0c, 0x02, 0xe7 // str al r0 r2 minus r1 ASR 25 Offset
};
const byte kInstruction_str_al_r0_r12_plus_r6_LSR_21_Offset[] = {
  0xa6, 0x0a, 0x8c, 0xe7 // str al r0 r12 plus r6 LSR 21 Offset
};
const byte kInstruction_str_al_r0_r6_plus_r12_ASR_18_Offset[] = {
  0x4c, 0x09, 0x86, 0xe7 // str al r0 r6 plus r12 ASR 18 Offset
};
const byte kInstruction_str_al_r0_r9_plus_r13_LSR_15_Offset[] = {
  0xad, 0x07, 0x89, 0xe7 // str al r0 r9 plus r13 LSR 15 Offset
};
const byte kInstruction_str_al_r0_r6_plus_r6_LSR_26_Offset[] = {
  0x26, 0x0d, 0x86, 0xe7 // str al r0 r6 plus r6 LSR 26 Offset
};
const byte kInstruction_str_al_r0_r4_minus_r9_ASR_14_Offset[] = {
  0x49, 0x07, 0x04, 0xe7 // str al r0 r4 minus r9 ASR 14 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r7_ASR_6_Offset[] = {
  0x47, 0x03, 0x80, 0xe7 // str al r0 r0 plus r7 ASR 6 Offset
};
const byte kInstruction_str_al_r0_r14_plus_r8_ASR_9_Offset[] = {
  0xc8, 0x04, 0x8e, 0xe7 // str al r0 r14 plus r8 ASR 9 Offset
};
const byte kInstruction_str_al_r0_r10_plus_r7_LSR_4_Offset[] = {
  0x27, 0x02, 0x8a, 0xe7 // str al r0 r10 plus r7 LSR 4 Offset
};
const byte kInstruction_str_al_r0_r11_plus_r3_LSR_13_Offset[] = {
  0xa3, 0x06, 0x8b, 0xe7 // str al r0 r11 plus r3 LSR 13 Offset
};
const byte kInstruction_str_al_r0_r13_plus_r3_LSR_25_Offset[] = {
  0xa3, 0x0c, 0x8d, 0xe7 // str al r0 r13 plus r3 LSR 25 Offset
};
const byte kInstruction_str_al_r0_r4_plus_r3_ASR_13_Offset[] = {
  0xc3, 0x06, 0x84, 0xe7 // str al r0 r4 plus r3 ASR 13 Offset
};
const byte kInstruction_str_al_r0_r4_minus_r2_LSR_2_Offset[] = {
  0x22, 0x01, 0x04, 0xe7 // str al r0 r4 minus r2 LSR 2 Offset
};
const byte kInstruction_str_al_r0_r13_minus_r7_LSR_1_Offset[] = {
  0xa7, 0x00, 0x0d, 0xe7 // str al r0 r13 minus r7 LSR 1 Offset
};
const byte kInstruction_str_al_r0_r10_minus_r9_LSR_7_Offset[] = {
  0xa9, 0x03, 0x0a, 0xe7 // str al r0 r10 minus r9 LSR 7 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r0_LSR_10_Offset[] = {
  0x20, 0x05, 0x80, 0xe7 // str al r0 r0 plus r0 LSR 10 Offset
};
const byte kInstruction_str_al_r0_r5_minus_r1_LSR_23_Offset[] = {
  0xa1, 0x0b, 0x05, 0xe7 // str al r0 r5 minus r1 LSR 23 Offset
};
const byte kInstruction_str_al_r0_r3_plus_r10_LSR_21_Offset[] = {
  0xaa, 0x0a, 0x83, 0xe7 // str al r0 r3 plus r10 LSR 21 Offset
};
const byte kInstruction_str_al_r0_r1_plus_r11_ASR_22_Offset[] = {
  0x4b, 0x0b, 0x81, 0xe7 // str al r0 r1 plus r11 ASR 22 Offset
};
const byte kInstruction_str_al_r0_r4_minus_r6_ASR_28_Offset[] = {
  0x46, 0x0e, 0x04, 0xe7 // str al r0 r4 minus r6 ASR 28 Offset
};
const byte kInstruction_str_al_r0_r4_plus_r10_LSR_29_Offset[] = {
  0xaa, 0x0e, 0x84, 0xe7 // str al r0 r4 plus r10 LSR 29 Offset
};
const byte kInstruction_str_al_r0_r4_plus_r3_LSR_10_Offset[] = {
  0x23, 0x05, 0x84, 0xe7 // str al r0 r4 plus r3 LSR 10 Offset
};
const byte kInstruction_str_al_r0_r14_minus_r12_LSR_11_Offset[] = {
  0xac, 0x05, 0x0e, 0xe7 // str al r0 r14 minus r12 LSR 11 Offset
};
const byte kInstruction_str_al_r0_r5_plus_r3_LSR_6_Offset[] = {
  0x23, 0x03, 0x85, 0xe7 // str al r0 r5 plus r3 LSR 6 Offset
};
const byte kInstruction_str_al_r0_r3_minus_r9_ASR_18_Offset[] = {
  0x49, 0x09, 0x03, 0xe7 // str al r0 r3 minus r9 ASR 18 Offset
};
const byte kInstruction_str_al_r0_r3_minus_r10_ASR_21_Offset[] = {
  0xca, 0x0a, 0x03, 0xe7 // str al r0 r3 minus r10 ASR 21 Offset
};
const byte kInstruction_str_al_r0_r9_minus_r4_ASR_2_Offset[] = {
  0x44, 0x01, 0x09, 0xe7 // str al r0 r9 minus r4 ASR 2 Offset
};
const byte kInstruction_str_al_r0_r4_plus_r7_LSR_23_Offset[] = {
  0xa7, 0x0b, 0x84, 0xe7 // str al r0 r4 plus r7 LSR 23 Offset
};
const byte kInstruction_str_al_r0_r4_plus_r9_ASR_17_Offset[] = {
  0xc9, 0x08, 0x84, 0xe7 // str al r0 r4 plus r9 ASR 17 Offset
};
const byte kInstruction_str_al_r0_r2_plus_r1_ASR_14_Offset[] = {
  0x41, 0x07, 0x82, 0xe7 // str al r0 r2 plus r1 ASR 14 Offset
};
const byte kInstruction_str_al_r0_r7_minus_r3_ASR_14_Offset[] = {
  0x43, 0x07, 0x07, 0xe7 // str al r0 r7 minus r3 ASR 14 Offset
};
const byte kInstruction_str_al_r0_r6_minus_r7_ASR_13_Offset[] = {
  0xc7, 0x06, 0x06, 0xe7 // str al r0 r6 minus r7 ASR 13 Offset
};
const byte kInstruction_str_al_r0_r3_plus_r14_ASR_25_Offset[] = {
  0xce, 0x0c, 0x83, 0xe7 // str al r0 r3 plus r14 ASR 25 Offset
};
const byte kInstruction_str_al_r0_r5_plus_r8_LSR_14_Offset[] = {
  0x28, 0x07, 0x85, 0xe7 // str al r0 r5 plus r8 LSR 14 Offset
};
const byte kInstruction_str_al_r0_r5_minus_r11_LSR_10_Offset[] = {
  0x2b, 0x05, 0x05, 0xe7 // str al r0 r5 minus r11 LSR 10 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r11_ASR_14_Offset[] = {
  0x4b, 0x07, 0x80, 0xe7 // str al r0 r0 plus r11 ASR 14 Offset
};
const byte kInstruction_str_al_r0_r9_plus_r12_LSR_15_Offset[] = {
  0xac, 0x07, 0x89, 0xe7 // str al r0 r9 plus r12 LSR 15 Offset
};
const byte kInstruction_str_al_r0_r3_minus_r7_ASR_3_Offset[] = {
  0xc7, 0x01, 0x03, 0xe7 // str al r0 r3 minus r7 ASR 3 Offset
};
const byte kInstruction_str_al_r0_r4_minus_r8_LSR_4_Offset[] = {
  0x28, 0x02, 0x04, 0xe7 // str al r0 r4 minus r8 LSR 4 Offset
};
const byte kInstruction_str_al_r0_r4_minus_r14_ASR_12_Offset[] = {
  0x4e, 0x06, 0x04, 0xe7 // str al r0 r4 minus r14 ASR 12 Offset
};
const byte kInstruction_str_al_r0_r1_plus_r1_ASR_10_Offset[] = {
  0x41, 0x05, 0x81, 0xe7 // str al r0 r1 plus r1 ASR 10 Offset
};
const byte kInstruction_str_al_r0_r5_plus_r7_LSR_24_Offset[] = {
  0x27, 0x0c, 0x85, 0xe7 // str al r0 r5 plus r7 LSR 24 Offset
};
const byte kInstruction_str_al_r0_r10_plus_r2_LSR_10_Offset[] = {
  0x22, 0x05, 0x8a, 0xe7 // str al r0 r10 plus r2 LSR 10 Offset
};
const byte kInstruction_str_al_r0_r13_minus_r3_LSR_29_Offset[] = {
  0xa3, 0x0e, 0x0d, 0xe7 // str al r0 r13 minus r3 LSR 29 Offset
};
const byte kInstruction_str_al_r0_r8_minus_r8_LSR_22_Offset[] = {
  0x28, 0x0b, 0x08, 0xe7 // str al r0 r8 minus r8 LSR 22 Offset
};
const byte kInstruction_str_al_r0_r14_minus_r10_LSR_20_Offset[] = {
  0x2a, 0x0a, 0x0e, 0xe7 // str al r0 r14 minus r10 LSR 20 Offset
};
const byte kInstruction_str_al_r0_r12_plus_r10_LSR_10_Offset[] = {
  0x2a, 0x05, 0x8c, 0xe7 // str al r0 r12 plus r10 LSR 10 Offset
};
const byte kInstruction_str_al_r0_r3_plus_r12_LSR_25_Offset[] = {
  0xac, 0x0c, 0x83, 0xe7 // str al r0 r3 plus r12 LSR 25 Offset
};
const byte kInstruction_str_al_r0_r2_plus_r13_ASR_4_Offset[] = {
  0x4d, 0x02, 0x82, 0xe7 // str al r0 r2 plus r13 ASR 4 Offset
};
const byte kInstruction_str_al_r0_r6_plus_r11_LSR_24_Offset[] = {
  0x2b, 0x0c, 0x86, 0xe7 // str al r0 r6 plus r11 LSR 24 Offset
};
const byte kInstruction_str_al_r0_r1_plus_r5_LSR_8_Offset[] = {
  0x25, 0x04, 0x81, 0xe7 // str al r0 r1 plus r5 LSR 8 Offset
};
const byte kInstruction_str_al_r0_r6_minus_r8_LSR_12_Offset[] = {
  0x28, 0x06, 0x06, 0xe7 // str al r0 r6 minus r8 LSR 12 Offset
};
const byte kInstruction_str_al_r0_r4_plus_r6_LSR_16_Offset[] = {
  0x26, 0x08, 0x84, 0xe7 // str al r0 r4 plus r6 LSR 16 Offset
};
const byte kInstruction_str_al_r0_r7_plus_r12_LSR_6_Offset[] = {
  0x2c, 0x03, 0x87, 0xe7 // str al r0 r7 plus r12 LSR 6 Offset
};
const byte kInstruction_str_al_r0_r12_plus_r8_ASR_11_Offset[] = {
  0xc8, 0x05, 0x8c, 0xe7 // str al r0 r12 plus r8 ASR 11 Offset
};
const byte kInstruction_str_al_r0_r11_minus_r1_LSR_17_Offset[] = {
  0xa1, 0x08, 0x0b, 0xe7 // str al r0 r11 minus r1 LSR 17 Offset
};
const byte kInstruction_str_al_r0_r4_minus_r0_LSR_8_Offset[] = {
  0x20, 0x04, 0x04, 0xe7 // str al r0 r4 minus r0 LSR 8 Offset
};
const byte kInstruction_str_al_r0_r5_minus_r14_LSR_2_Offset[] = {
  0x2e, 0x01, 0x05, 0xe7 // str al r0 r5 minus r14 LSR 2 Offset
};
const byte kInstruction_str_al_r0_r10_minus_r8_LSR_24_Offset[] = {
  0x28, 0x0c, 0x0a, 0xe7 // str al r0 r10 minus r8 LSR 24 Offset
};
const byte kInstruction_str_al_r0_r8_minus_r12_ASR_4_Offset[] = {
  0x4c, 0x02, 0x08, 0xe7 // str al r0 r8 minus r12 ASR 4 Offset
};
const byte kInstruction_str_al_r0_r3_minus_r4_ASR_27_Offset[] = {
  0xc4, 0x0d, 0x03, 0xe7 // str al r0 r3 minus r4 ASR 27 Offset
};
const byte kInstruction_str_al_r0_r12_minus_r8_LSR_25_Offset[] = {
  0xa8, 0x0c, 0x0c, 0xe7 // str al r0 r12 minus r8 LSR 25 Offset
};
const byte kInstruction_str_al_r0_r10_minus_r13_LSR_5_Offset[] = {
  0xad, 0x02, 0x0a, 0xe7 // str al r0 r10 minus r13 LSR 5 Offset
};
const byte kInstruction_str_al_r0_r0_minus_r8_LSR_26_Offset[] = {
  0x28, 0x0d, 0x00, 0xe7 // str al r0 r0 minus r8 LSR 26 Offset
};
const byte kInstruction_str_al_r0_r4_minus_r1_LSR_18_Offset[] = {
  0x21, 0x09, 0x04, 0xe7 // str al r0 r4 minus r1 LSR 18 Offset
};
const byte kInstruction_str_al_r0_r3_plus_r4_LSR_26_Offset[] = {
  0x24, 0x0d, 0x83, 0xe7 // str al r0 r3 plus r4 LSR 26 Offset
};
const byte kInstruction_str_al_r0_r6_plus_r1_LSR_27_Offset[] = {
  0xa1, 0x0d, 0x86, 0xe7 // str al r0 r6 plus r1 LSR 27 Offset
};
const byte kInstruction_str_al_r0_r14_minus_r11_ASR_9_Offset[] = {
  0xcb, 0x04, 0x0e, 0xe7 // str al r0 r14 minus r11 ASR 9 Offset
};
const byte kInstruction_str_al_r0_r5_minus_r13_LSR_25_Offset[] = {
  0xad, 0x0c, 0x05, 0xe7 // str al r0 r5 minus r13 LSR 25 Offset
};
const byte kInstruction_str_al_r0_r6_minus_r2_ASR_23_Offset[] = {
  0xc2, 0x0b, 0x06, 0xe7 // str al r0 r6 minus r2 ASR 23 Offset
};
const byte kInstruction_str_al_r0_r13_minus_r3_LSR_21_Offset[] = {
  0xa3, 0x0a, 0x0d, 0xe7 // str al r0 r13 minus r3 LSR 21 Offset
};
const byte kInstruction_str_al_r0_r12_minus_r7_ASR_10_Offset[] = {
  0x47, 0x05, 0x0c, 0xe7 // str al r0 r12 minus r7 ASR 10 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r0_ASR_19_Offset[] = {
  0xc0, 0x09, 0x80, 0xe7 // str al r0 r0 plus r0 ASR 19 Offset
};
const byte kInstruction_str_al_r0_r14_minus_r14_ASR_32_Offset[] = {
  0x4e, 0x00, 0x0e, 0xe7 // str al r0 r14 minus r14 ASR 32 Offset
};
const byte kInstruction_str_al_r0_r9_plus_r11_LSR_4_Offset[] = {
  0x2b, 0x02, 0x89, 0xe7 // str al r0 r9 plus r11 LSR 4 Offset
};
const byte kInstruction_str_al_r0_r10_minus_r12_ASR_28_Offset[] = {
  0x4c, 0x0e, 0x0a, 0xe7 // str al r0 r10 minus r12 ASR 28 Offset
};
const byte kInstruction_str_al_r0_r1_minus_r3_LSR_15_Offset[] = {
  0xa3, 0x07, 0x01, 0xe7 // str al r0 r1 minus r3 LSR 15 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r3_LSR_5_Offset[] = {
  0xa3, 0x02, 0x80, 0xe7 // str al r0 r0 plus r3 LSR 5 Offset
};
const byte kInstruction_str_al_r0_r2_plus_r9_ASR_16_Offset[] = {
  0x49, 0x08, 0x82, 0xe7 // str al r0 r2 plus r9 ASR 16 Offset
};
const byte kInstruction_str_al_r0_r7_minus_r3_ASR_30_Offset[] = {
  0x43, 0x0f, 0x07, 0xe7 // str al r0 r7 minus r3 ASR 30 Offset
};
const byte kInstruction_str_al_r0_r10_minus_r1_ASR_31_Offset[] = {
  0xc1, 0x0f, 0x0a, 0xe7 // str al r0 r10 minus r1 ASR 31 Offset
};
const byte kInstruction_str_al_r0_r11_minus_r7_LSR_26_Offset[] = {
  0x27, 0x0d, 0x0b, 0xe7 // str al r0 r11 minus r7 LSR 26 Offset
};
const byte kInstruction_str_al_r0_r0_minus_r4_LSR_8_Offset[] = {
  0x24, 0x04, 0x00, 0xe7 // str al r0 r0 minus r4 LSR 8 Offset
};
const byte kInstruction_str_al_r0_r3_plus_r1_LSR_5_Offset[] = {
  0xa1, 0x02, 0x83, 0xe7 // str al r0 r3 plus r1 LSR 5 Offset
};
const byte kInstruction_str_al_r0_r14_minus_r1_LSR_15_Offset[] = {
  0xa1, 0x07, 0x0e, 0xe7 // str al r0 r14 minus r1 LSR 15 Offset
};
const byte kInstruction_str_al_r0_r2_plus_r8_ASR_7_Offset[] = {
  0xc8, 0x03, 0x82, 0xe7 // str al r0 r2 plus r8 ASR 7 Offset
};
const byte kInstruction_str_al_r0_r11_plus_r12_ASR_7_Offset[] = {
  0xcc, 0x03, 0x8b, 0xe7 // str al r0 r11 plus r12 ASR 7 Offset
};
const byte kInstruction_str_al_r0_r14_plus_r6_LSR_19_Offset[] = {
  0xa6, 0x09, 0x8e, 0xe7 // str al r0 r14 plus r6 LSR 19 Offset
};
const byte kInstruction_str_al_r0_r0_minus_r6_ASR_12_Offset[] = {
  0x46, 0x06, 0x00, 0xe7 // str al r0 r0 minus r6 ASR 12 Offset
};
const byte kInstruction_str_al_r0_r2_minus_r8_LSR_24_Offset[] = {
  0x28, 0x0c, 0x02, 0xe7 // str al r0 r2 minus r8 LSR 24 Offset
};
const byte kInstruction_str_al_r0_r2_minus_r13_LSR_4_Offset[] = {
  0x2d, 0x02, 0x02, 0xe7 // str al r0 r2 minus r13 LSR 4 Offset
};
const byte kInstruction_str_al_r0_r11_minus_r12_LSR_6_Offset[] = {
  0x2c, 0x03, 0x0b, 0xe7 // str al r0 r11 minus r12 LSR 6 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r9_ASR_12_Offset[] = {
  0x49, 0x06, 0x80, 0xe7 // str al r0 r0 plus r9 ASR 12 Offset
};
const byte kInstruction_str_al_r0_r5_plus_r9_ASR_25_Offset[] = {
  0xc9, 0x0c, 0x85, 0xe7 // str al r0 r5 plus r9 ASR 25 Offset
};
const byte kInstruction_str_al_r0_r0_minus_r8_ASR_23_Offset[] = {
  0xc8, 0x0b, 0x00, 0xe7 // str al r0 r0 minus r8 ASR 23 Offset
};
const byte kInstruction_str_al_r0_r8_plus_r4_ASR_29_Offset[] = {
  0xc4, 0x0e, 0x88, 0xe7 // str al r0 r8 plus r4 ASR 29 Offset
};
const byte kInstruction_str_al_r0_r0_minus_r1_LSR_25_Offset[] = {
  0xa1, 0x0c, 0x00, 0xe7 // str al r0 r0 minus r1 LSR 25 Offset
};
const byte kInstruction_str_al_r0_r1_minus_r0_LSR_25_Offset[] = {
  0xa0, 0x0c, 0x01, 0xe7 // str al r0 r1 minus r0 LSR 25 Offset
};
const byte kInstruction_str_al_r0_r2_plus_r13_LSR_32_Offset[] = {
  0x2d, 0x00, 0x82, 0xe7 // str al r0 r2 plus r13 LSR 32 Offset
};
const byte kInstruction_str_al_r0_r9_plus_r3_LSR_10_Offset[] = {
  0x23, 0x05, 0x89, 0xe7 // str al r0 r9 plus r3 LSR 10 Offset
};
const byte kInstruction_str_al_r0_r5_minus_r4_LSR_21_Offset[] = {
  0xa4, 0x0a, 0x05, 0xe7 // str al r0 r5 minus r4 LSR 21 Offset
};
const byte kInstruction_str_al_r0_r2_minus_r5_LSR_30_Offset[] = {
  0x25, 0x0f, 0x02, 0xe7 // str al r0 r2 minus r5 LSR 30 Offset
};
const byte kInstruction_str_al_r0_r2_plus_r8_LSR_26_Offset[] = {
  0x28, 0x0d, 0x82, 0xe7 // str al r0 r2 plus r8 LSR 26 Offset
};
const byte kInstruction_str_al_r0_r5_minus_r10_ASR_12_Offset[] = {
  0x4a, 0x06, 0x05, 0xe7 // str al r0 r5 minus r10 ASR 12 Offset
};
const byte kInstruction_str_al_r0_r14_minus_r0_ASR_21_Offset[] = {
  0xc0, 0x0a, 0x0e, 0xe7 // str al r0 r14 minus r0 ASR 21 Offset
};
const byte kInstruction_str_al_r0_r5_plus_r6_ASR_1_Offset[] = {
  0xc6, 0x00, 0x85, 0xe7 // str al r0 r5 plus r6 ASR 1 Offset
};
const byte kInstruction_str_al_r0_r3_minus_r3_ASR_16_Offset[] = {
  0x43, 0x08, 0x03, 0xe7 // str al r0 r3 minus r3 ASR 16 Offset
};
const byte kInstruction_str_al_r0_r12_plus_r0_LSR_10_Offset[] = {
  0x20, 0x05, 0x8c, 0xe7 // str al r0 r12 plus r0 LSR 10 Offset
};
const byte kInstruction_str_al_r0_r11_plus_r5_ASR_5_Offset[] = {
  0xc5, 0x02, 0x8b, 0xe7 // str al r0 r11 plus r5 ASR 5 Offset
};
const byte kInstruction_str_al_r0_r3_minus_r11_ASR_19_Offset[] = {
  0xcb, 0x09, 0x03, 0xe7 // str al r0 r3 minus r11 ASR 19 Offset
};
const byte kInstruction_str_al_r0_r4_minus_r5_LSR_27_Offset[] = {
  0xa5, 0x0d, 0x04, 0xe7 // str al r0 r4 minus r5 LSR 27 Offset
};
const byte kInstruction_str_al_r0_r3_plus_r5_ASR_17_Offset[] = {
  0xc5, 0x08, 0x83, 0xe7 // str al r0 r3 plus r5 ASR 17 Offset
};
const byte kInstruction_str_al_r0_r8_plus_r5_LSR_18_Offset[] = {
  0x25, 0x09, 0x88, 0xe7 // str al r0 r8 plus r5 LSR 18 Offset
};
const byte kInstruction_str_al_r0_r8_plus_r11_ASR_24_Offset[] = {
  0x4b, 0x0c, 0x88, 0xe7 // str al r0 r8 plus r11 ASR 24 Offset
};
const byte kInstruction_str_al_r0_r10_plus_r12_LSR_30_Offset[] = {
  0x2c, 0x0f, 0x8a, 0xe7 // str al r0 r10 plus r12 LSR 30 Offset
};
const byte kInstruction_str_al_r0_r8_minus_r3_ASR_9_Offset[] = {
  0xc3, 0x04, 0x08, 0xe7 // str al r0 r8 minus r3 ASR 9 Offset
};
const byte kInstruction_str_al_r0_r11_plus_r9_ASR_11_Offset[] = {
  0xc9, 0x05, 0x8b, 0xe7 // str al r0 r11 plus r9 ASR 11 Offset
};
const byte kInstruction_str_al_r0_r10_plus_r7_LSR_22_Offset[] = {
  0x27, 0x0b, 0x8a, 0xe7 // str al r0 r10 plus r7 LSR 22 Offset
};
const byte kInstruction_str_al_r0_r10_plus_r7_ASR_13_Offset[] = {
  0xc7, 0x06, 0x8a, 0xe7 // str al r0 r10 plus r7 ASR 13 Offset
};
const byte kInstruction_str_al_r0_r7_plus_r13_LSR_21_Offset[] = {
  0xad, 0x0a, 0x87, 0xe7 // str al r0 r7 plus r13 LSR 21 Offset
};
const byte kInstruction_str_al_r0_r12_minus_r1_LSR_16_Offset[] = {
  0x21, 0x08, 0x0c, 0xe7 // str al r0 r12 minus r1 LSR 16 Offset
};
const byte kInstruction_str_al_r0_r2_minus_r2_LSR_4_Offset[] = {
  0x22, 0x02, 0x02, 0xe7 // str al r0 r2 minus r2 LSR 4 Offset
};
const byte kInstruction_str_al_r0_r13_minus_r13_ASR_19_Offset[] = {
  0xcd, 0x09, 0x0d, 0xe7 // str al r0 r13 minus r13 ASR 19 Offset
};
const byte kInstruction_str_al_r0_r2_plus_r4_LSR_13_Offset[] = {
  0xa4, 0x06, 0x82, 0xe7 // str al r0 r2 plus r4 LSR 13 Offset
};
const byte kInstruction_str_al_r0_r9_plus_r1_LSR_32_Offset[] = {
  0x21, 0x00, 0x89, 0xe7 // str al r0 r9 plus r1 LSR 32 Offset
};
const byte kInstruction_str_al_r0_r3_plus_r6_ASR_21_Offset[] = {
  0xc6, 0x0a, 0x83, 0xe7 // str al r0 r3 plus r6 ASR 21 Offset
};
const byte kInstruction_str_al_r0_r2_plus_r1_LSR_5_Offset[] = {
  0xa1, 0x02, 0x82, 0xe7 // str al r0 r2 plus r1 LSR 5 Offset
};
const byte kInstruction_str_al_r0_r0_minus_r11_ASR_6_Offset[] = {
  0x4b, 0x03, 0x00, 0xe7 // str al r0 r0 minus r11 ASR 6 Offset
};
const byte kInstruction_str_al_r0_r2_minus_r9_ASR_31_Offset[] = {
  0xc9, 0x0f, 0x02, 0xe7 // str al r0 r2 minus r9 ASR 31 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r11_LSR_6_Offset[] = {
  0x2b, 0x03, 0x80, 0xe7 // str al r0 r0 plus r11 LSR 6 Offset
};
const byte kInstruction_str_al_r0_r10_minus_r11_LSR_14_Offset[] = {
  0x2b, 0x07, 0x0a, 0xe7 // str al r0 r10 minus r11 LSR 14 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r14_LSR_24_Offset[] = {
  0x2e, 0x0c, 0x80, 0xe7 // str al r0 r0 plus r14 LSR 24 Offset
};
const byte kInstruction_str_al_r0_r0_plus_r2_LSR_8_Offset[] = {
  0x22, 0x04, 0x80, 0xe7 // str al r0 r0 plus r2 LSR 8 Offset
};
const byte kInstruction_str_al_r0_r13_minus_r12_ASR_10_Offset[] = {
  0x4c, 0x05, 0x0d, 0xe7 // str al r0 r13 minus r12 ASR 10 Offset
};
const byte kInstruction_str_al_r0_r2_plus_r2_LSR_12_Offset[] = {
  0x22, 0x06, 0x82, 0xe7 // str al r0 r2 plus r2 LSR 12 Offset
};
const byte kInstruction_str_al_r0_r12_plus_r10_LSR_30_Offset[] = {
  0x2a, 0x0f, 0x8c, 0xe7 // str al r0 r12 plus r10 LSR 30 Offset
};
const byte kInstruction_str_al_r0_r8_minus_r0_ASR_9_Offset[] = {
  0xc0, 0x04, 0x08, 0xe7 // str al r0 r8 minus r0 ASR 9 Offset
};
const byte kInstruction_str_al_r0_r9_minus_r6_LSR_4_Offset[] = {
  0x26, 0x02, 0x09, 0xe7 // str al r0 r9 minus r6 LSR 4 Offset
};
const byte kInstruction_str_al_r0_r3_plus_r14_ASR_3_Offset[] = {
  0xce, 0x01, 0x83, 0xe7 // str al r0 r3 plus r14 ASR 3 Offset
};
const byte kInstruction_str_al_r0_r14_minus_r10_ASR_4_Offset[] = {
  0x4a, 0x02, 0x0e, 0xe7 // str al r0 r14 minus r10 ASR 4 Offset
};
const byte kInstruction_str_al_r0_r7_plus_r8_LSR_5_Offset[] = {
  0xa8, 0x02, 0x87, 0xe7 // str al r0 r7 plus r8 LSR 5 Offset
};
const byte kInstruction_str_al_r0_r0_minus_r13_ASR_17_Offset[] = {
  0xcd, 0x08, 0x00, 0xe7 // str al r0 r0 minus r13 ASR 17 Offset
};
const byte kInstruction_str_al_r0_r3_minus_r7_LSR_2_Offset[] = {
  0x27, 0x01, 0x03, 0xe7 // str al r0 r3 minus r7 LSR 2 Offset
};
const byte kInstruction_str_al_r0_r8_minus_r8_ASR_15_Offset[] = {
  0xc8, 0x07, 0x08, 0xe7 // str al r0 r8 minus r8 ASR 15 Offset
};
const byte kInstruction_str_al_r0_r13_plus_r12_ASR_29_Offset[] = {
  0xcc, 0x0e, 0x8d, 0xe7 // str al r0 r13 plus r12 ASR 29 Offset
};
const byte kInstruction_str_al_r0_r10_plus_r4_LSR_22_Offset[] = {
  0x24, 0x0b, 0x8a, 0xe7 // str al r0 r10 plus r4 LSR 22 Offset
};
const byte kInstruction_str_al_r0_r2_plus_r1_ASR_24_Offset[] = {
  0x41, 0x0c, 0x82, 0xe7 // str al r0 r2 plus r1 ASR 24 Offset
};
const byte kInstruction_str_al_r0_r14_minus_r13_LSR_29_Offset[] = {
  0xad, 0x0e, 0x0e, 0xe7 // str al r0 r14 minus r13 LSR 29 Offset
};
const byte kInstruction_str_al_r0_r3_plus_r1_LSR_6_Offset[] = {
  0x21, 0x03, 0x83, 0xe7 // str al r0 r3 plus r1 LSR 6 Offset
};
const byte kInstruction_str_al_r0_r7_plus_r5_LSR_32_Offset[] = {
  0x25, 0x00, 0x87, 0xe7 // str al r0 r7 plus r5 LSR 32 Offset
};
const byte kInstruction_str_al_r0_r12_plus_r6_LSR_30_Offset[] = {
  0x26, 0x0f, 0x8c, 0xe7 // str al r0 r12 plus r6 LSR 30 Offset
};
const byte kInstruction_str_al_r0_r13_plus_r13_ASR_6_Offset[] = {
  0x4d, 0x03, 0x8d, 0xe7 // str al r0 r13 plus r13 ASR 6 Offset
};
const byte kInstruction_str_al_r0_r14_minus_r0_LSR_4_Offset[] = {
  0x20, 0x02, 0x0e, 0xe7 // str al r0 r14 minus r0 LSR 4 Offset
};
const byte kInstruction_str_al_r0_r7_minus_r4_ASR_6_Offset[] = {
  0x44, 0x03, 0x07, 0xe7 // str al r0 r7 minus r4 ASR 6 Offset
};
const byte kInstruction_str_al_r0_r9_plus_r13_ASR_5_Offset[] = {
  0xcd, 0x02, 0x89, 0xe7 // str al r0 r9 plus r13 ASR 5 Offset
};
const byte kInstruction_str_al_r0_r8_plus_r9_LSR_5_Offset[] = {
  0xa9, 0x02, 0x88, 0xe7 // str al r0 r8 plus r9 LSR 5 Offset
};
const byte kInstruction_str_al_r0_r3_plus_r11_LSR_7_Offset[] = {
  0xab, 0x03, 0x83, 0xe7 // str al r0 r3 plus r11 LSR 7 Offset
};
const byte kInstruction_str_al_r0_r5_plus_r2_LSR_11_Offset[] = {
  0xa2, 0x05, 0x85, 0xe7 // str al r0 r5 plus r2 LSR 11 Offset
};
const byte kInstruction_str_al_r0_r10_minus_r4_LSR_12_Offset[] = {
  0x24, 0x06, 0x0a, 0xe7 // str al r0 r10 minus r4 LSR 12 Offset
};
const byte kInstruction_str_al_r0_r0_minus_r0_ASR_25_Offset[] = {
  0xc0, 0x0c, 0x00, 0xe7 // str al r0 r0 minus r0 ASR 25 Offset
};
const byte kInstruction_str_al_r0_r13_minus_r10_ASR_6_Offset[] = {
  0x4a, 0x03, 0x0d, 0xe7 // str al r0 r13 minus r10 ASR 6 Offset
};
const byte kInstruction_str_al_r0_r11_plus_r1_LSR_20_Offset[] = {
  0x21, 0x0a, 0x8b, 0xe7 // str al r0 r11 plus r1 LSR 20 Offset
};
const byte kInstruction_str_al_r0_r13_minus_r10_LSR_3_Offset[] = {
  0xaa, 0x01, 0x0d, 0xe7 // str al r0 r13 minus r10 LSR 3 Offset
};
const byte kInstruction_str_al_r0_r6_plus_r9_ASR_2_Offset[] = {
  0x49, 0x01, 0x86, 0xe7 // str al r0 r6 plus r9 ASR 2 Offset
};
const byte kInstruction_str_al_r0_r4_minus_r7_ASR_23_Offset[] = {
  0xc7, 0x0b, 0x04, 0xe7 // str al r0 r4 minus r7 ASR 23 Offset
};
const byte kInstruction_str_al_r0_r13_minus_r10_LSR_11_Offset[] = {
  0xaa, 0x05, 0x0d, 0xe7 // str al r0 r13 minus r10 LSR 11 Offset
};
const byte kInstruction_str_al_r0_r9_minus_r0_LSR_15_Offset[] = {
  0xa0, 0x07, 0x09, 0xe7 // str al r0 r9 minus r0 LSR 15 Offset
};
const byte kInstruction_str_al_r0_r3_plus_r5_ASR_23_Offset[] = {
  0xc5, 0x0b, 0x83, 0xe7 // str al r0 r3 plus r5 ASR 23 Offset
};
const byte kInstruction_str_al_r8_r0_plus_r1_ASR_3_PreIndex[] = {
  0xc1, 0x81, 0xa0, 0xe7 // str al r8 r0 plus r1 ASR 3 PreIndex
};
const byte kInstruction_str_al_r0_r14_minus_r6_ASR_27_PreIndex[] = {
  0xc6, 0x0d, 0x2e, 0xe7 // str al r0 r14 minus r6 ASR 27 PreIndex
};
const byte kInstruction_str_al_r12_r11_minus_r8_LSR_2_PostIndex[] = {
  0x28, 0xc1, 0x0b, 0xe6 // str al r12 r11 minus r8 LSR 2 PostIndex
};
const byte kInstruction_str_al_r9_r3_plus_r10_ASR_28_PostIndex[] = {
  0x4a, 0x9e, 0x83, 0xe6 // str al r9 r3 plus r10 ASR 28 PostIndex
};
const byte kInstruction_str_al_r2_r5_plus_r6_ASR_7_PreIndex[] = {
  0xc6, 0x23, 0xa5, 0xe7 // str al r2 r5 plus r6 ASR 7 PreIndex
};
const byte kInstruction_str_al_r14_r11_minus_r2_ASR_2_PostIndex[] = {
  0x42, 0xe1, 0x0b, 0xe6 // str al r14 r11 minus r2 ASR 2 PostIndex
};
const byte kInstruction_str_al_r0_r11_minus_r1_ASR_13_PreIndex[] = {
  0xc1, 0x06, 0x2b, 0xe7 // str al r0 r11 minus r1 ASR 13 PreIndex
};
const byte kInstruction_str_al_r13_r12_plus_r6_LSR_16_PreIndex[] = {
  0x26, 0xd8, 0xac, 0xe7 // str al r13 r12 plus r6 LSR 16 PreIndex
};
const byte kInstruction_str_al_r10_r2_plus_r10_LSR_31_PreIndex[] = {
  0xaa, 0xaf, 0xa2, 0xe7 // str al r10 r2 plus r10 LSR 31 PreIndex
};
const byte kInstruction_str_al_r14_r11_plus_r14_LSR_9_PreIndex[] = {
  0xae, 0xe4, 0xab, 0xe7 // str al r14 r11 plus r14 LSR 9 PreIndex
};
const byte kInstruction_str_al_r0_r2_minus_r12_LSR_9_PreIndex[] = {
  0xac, 0x04, 0x22, 0xe7 // str al r0 r2 minus r12 LSR 9 PreIndex
};
const byte kInstruction_str_al_r0_r13_minus_r4_ASR_26_PostIndex[] = {
  0x44, 0x0d, 0x0d, 0xe6 // str al r0 r13 minus r4 ASR 26 PostIndex
};
const byte kInstruction_str_al_r7_r14_minus_r5_ASR_21_PreIndex[] = {
  0xc5, 0x7a, 0x2e, 0xe7 // str al r7 r14 minus r5 ASR 21 PreIndex
};
const byte kInstruction_str_al_r0_r4_minus_r8_ASR_21_PostIndex[] = {
  0xc8, 0x0a, 0x04, 0xe6 // str al r0 r4 minus r8 ASR 21 PostIndex
};
const byte kInstruction_str_al_r11_r6_plus_r2_LSR_31_PostIndex[] = {
  0xa2, 0xbf, 0x86, 0xe6 // str al r11 r6 plus r2 LSR 31 PostIndex
};
const byte kInstruction_str_al_r2_r9_minus_r2_LSR_29_PostIndex[] = {
  0xa2, 0x2e, 0x09, 0xe6 // str al r2 r9 minus r2 LSR 29 PostIndex
};
const byte kInstruction_str_al_r3_r4_minus_r9_LSR_8_PreIndex[] = {
  0x29, 0x34, 0x24, 0xe7 // str al r3 r4 minus r9 LSR 8 PreIndex
};
const byte kInstruction_str_al_r8_r14_plus_r11_LSR_32_PreIndex[] = {
  0x2b, 0x80, 0xae, 0xe7 // str al r8 r14 plus r11 LSR 32 PreIndex
};
const byte kInstruction_str_al_r11_r0_plus_r4_ASR_18_PostIndex[] = {
  0x44, 0xb9, 0x80, 0xe6 // str al r11 r0 plus r4 ASR 18 PostIndex
};
const byte kInstruction_str_al_r4_r7_plus_r2_ASR_30_PreIndex[] = {
  0x42, 0x4f, 0xa7, 0xe7 // str al r4 r7 plus r2 ASR 30 PreIndex
};
const byte kInstruction_str_al_r8_r6_minus_r2_LSR_30_PreIndex[] = {
  0x22, 0x8f, 0x26, 0xe7 // str al r8 r6 minus r2 LSR 30 PreIndex
};
const byte kInstruction_str_al_r4_r5_minus_r0_ASR_20_PreIndex[] = {
  0x40, 0x4a, 0x25, 0xe7 // str al r4 r5 minus r0 ASR 20 PreIndex
};
const byte kInstruction_str_al_r5_r11_minus_r11_LSR_30_PostIndex[] = {
  0x2b, 0x5f, 0x0b, 0xe6 // str al r5 r11 minus r11 LSR 30 PostIndex
};
const byte kInstruction_str_al_r12_r4_minus_r7_ASR_13_PreIndex[] = {
  0xc7, 0xc6, 0x24, 0xe7 // str al r12 r4 minus r7 ASR 13 PreIndex
};
const byte kInstruction_str_al_r5_r12_minus_r1_ASR_16_PreIndex[] = {
  0x41, 0x58, 0x2c, 0xe7 // str al r5 r12 minus r1 ASR 16 PreIndex
};
const byte kInstruction_str_al_r8_r3_minus_r0_LSR_10_PostIndex[] = {
  0x20, 0x85, 0x03, 0xe6 // str al r8 r3 minus r0 LSR 10 PostIndex
};
const byte kInstruction_str_al_r2_r11_plus_r5_LSR_12_PreIndex[] = {
  0x25, 0x26, 0xab, 0xe7 // str al r2 r11 plus r5 LSR 12 PreIndex
};
const byte kInstruction_str_al_r6_r2_minus_r4_ASR_16_PreIndex[] = {
  0x44, 0x68, 0x22, 0xe7 // str al r6 r2 minus r4 ASR 16 PreIndex
};
const byte kInstruction_str_al_r10_r2_minus_r0_LSR_20_PreIndex[] = {
  0x20, 0xaa, 0x22, 0xe7 // str al r10 r2 minus r0 LSR 20 PreIndex
};
const byte kInstruction_str_al_r6_r11_minus_r13_ASR_30_PostIndex[] = {
  0x4d, 0x6f, 0x0b, 0xe6 // str al r6 r11 minus r13 ASR 30 PostIndex
};
const byte kInstruction_str_al_r11_r8_minus_r10_ASR_9_PostIndex[] = {
  0xca, 0xb4, 0x08, 0xe6 // str al r11 r8 minus r10 ASR 9 PostIndex
};
const byte kInstruction_str_al_r2_r3_plus_r12_LSR_22_PostIndex[] = {
  0x2c, 0x2b, 0x83, 0xe6 // str al r2 r3 plus r12 LSR 22 PostIndex
};
const byte kInstruction_str_al_r1_r6_minus_r5_ASR_15_PostIndex[] = {
  0xc5, 0x17, 0x06, 0xe6 // str al r1 r6 minus r5 ASR 15 PostIndex
};
const byte kInstruction_str_al_r8_r5_minus_r5_ASR_10_PreIndex[] = {
  0x45, 0x85, 0x25, 0xe7 // str al r8 r5 minus r5 ASR 10 PreIndex
};
const byte kInstruction_str_al_r2_r1_minus_r14_LSR_26_PostIndex[] = {
  0x2e, 0x2d, 0x01, 0xe6 // str al r2 r1 minus r14 LSR 26 PostIndex
};
const byte kInstruction_str_al_r7_r12_minus_r3_LSR_16_PostIndex[] = {
  0x23, 0x78, 0x0c, 0xe6 // str al r7 r12 minus r3 LSR 16 PostIndex
};
const byte kInstruction_str_al_r8_r5_minus_r12_ASR_24_PreIndex[] = {
  0x4c, 0x8c, 0x25, 0xe7 // str al r8 r5 minus r12 ASR 24 PreIndex
};
const byte kInstruction_str_al_r8_r4_plus_r0_ASR_23_PreIndex[] = {
  0xc0, 0x8b, 0xa4, 0xe7 // str al r8 r4 plus r0 ASR 23 PreIndex
};
const byte kInstruction_str_al_r2_r0_minus_r14_ASR_31_PostIndex[] = {
  0xce, 0x2f, 0x00, 0xe6 // str al r2 r0 minus r14 ASR 31 PostIndex
};
const byte kInstruction_str_al_r6_r13_minus_r8_LSR_30_PostIndex[] = {
  0x28, 0x6f, 0x0d, 0xe6 // str al r6 r13 minus r8 LSR 30 PostIndex
};
const byte kInstruction_str_al_r3_r6_minus_r4_LSR_22_PostIndex[] = {
  0x24, 0x3b, 0x06, 0xe6 // str al r3 r6 minus r4 LSR 22 PostIndex
};
const byte kInstruction_str_al_r12_r8_plus_r9_ASR_32_PreIndex[] = {
  0x49, 0xc0, 0xa8, 0xe7 // str al r12 r8 plus r9 ASR 32 PreIndex
};
const byte kInstruction_str_al_r2_r4_plus_r4_ASR_13_PreIndex[] = {
  0xc4, 0x26, 0xa4, 0xe7 // str al r2 r4 plus r4 ASR 13 PreIndex
};
const byte kInstruction_str_al_r3_r13_plus_r7_ASR_12_PreIndex[] = {
  0x47, 0x36, 0xad, 0xe7 // str al r3 r13 plus r7 ASR 12 PreIndex
};
const byte kInstruction_str_al_r11_r1_minus_r14_LSR_14_PostIndex[] = {
  0x2e, 0xb7, 0x01, 0xe6 // str al r11 r1 minus r14 LSR 14 PostIndex
};
const byte kInstruction_str_al_r12_r9_plus_r5_ASR_27_PostIndex[] = {
  0xc5, 0xcd, 0x89, 0xe6 // str al r12 r9 plus r5 ASR 27 PostIndex
};
const byte kInstruction_str_al_r1_r3_plus_r11_ASR_31_PreIndex[] = {
  0xcb, 0x1f, 0xa3, 0xe7 // str al r1 r3 plus r11 ASR 31 PreIndex
};
const byte kInstruction_str_al_r10_r2_plus_r2_ASR_26_PostIndex[] = {
  0x42, 0xad, 0x82, 0xe6 // str al r10 r2 plus r2 ASR 26 PostIndex
};
const byte kInstruction_str_al_r0_r7_plus_r5_LSR_19_PreIndex[] = {
  0xa5, 0x09, 0xa7, 0xe7 // str al r0 r7 plus r5 LSR 19 PreIndex
};
const byte kInstruction_str_al_r12_r7_plus_r5_LSR_27_PreIndex[] = {
  0xa5, 0xcd, 0xa7, 0xe7 // str al r12 r7 plus r5 LSR 27 PreIndex
};
const byte kInstruction_str_al_r10_r11_plus_r11_LSR_9_PostIndex[] = {
  0xab, 0xa4, 0x8b, 0xe6 // str al r10 r11 plus r11 LSR 9 PostIndex
};
const byte kInstruction_str_al_r2_r14_plus_r13_ASR_24_PreIndex[] = {
  0x4d, 0x2c, 0xae, 0xe7 // str al r2 r14 plus r13 ASR 24 PreIndex
};
const byte kInstruction_str_al_r13_r0_minus_r1_LSR_20_PreIndex[] = {
  0x21, 0xda, 0x20, 0xe7 // str al r13 r0 minus r1 LSR 20 PreIndex
};
const byte kInstruction_str_al_r2_r4_minus_r8_LSR_26_PostIndex[] = {
  0x28, 0x2d, 0x04, 0xe6 // str al r2 r4 minus r8 LSR 26 PostIndex
};
const byte kInstruction_str_al_r10_r13_minus_r8_ASR_14_PostIndex[] = {
  0x48, 0xa7, 0x0d, 0xe6 // str al r10 r13 minus r8 ASR 14 PostIndex
};
const byte kInstruction_str_al_r6_r0_plus_r7_ASR_8_PostIndex[] = {
  0x47, 0x64, 0x80, 0xe6 // str al r6 r0 plus r7 ASR 8 PostIndex
};
const byte kInstruction_str_al_r3_r9_plus_r6_ASR_18_PreIndex[] = {
  0x46, 0x39, 0xa9, 0xe7 // str al r3 r9 plus r6 ASR 18 PreIndex
};
const byte kInstruction_str_al_r4_r1_minus_r0_ASR_2_PreIndex[] = {
  0x40, 0x41, 0x21, 0xe7 // str al r4 r1 minus r0 ASR 2 PreIndex
};
const byte kInstruction_str_al_r4_r11_plus_r11_ASR_7_PostIndex[] = {
  0xcb, 0x43, 0x8b, 0xe6 // str al r4 r11 plus r11 ASR 7 PostIndex
};
const byte kInstruction_str_al_r14_r9_minus_r11_ASR_3_PostIndex[] = {
  0xcb, 0xe1, 0x09, 0xe6 // str al r14 r9 minus r11 ASR 3 PostIndex
};
const byte kInstruction_str_al_r2_r11_minus_r8_ASR_31_PostIndex[] = {
  0xc8, 0x2f, 0x0b, 0xe6 // str al r2 r11 minus r8 ASR 31 PostIndex
};
const byte kInstruction_str_al_r11_r4_plus_r4_ASR_31_PreIndex[] = {
  0xc4, 0xbf, 0xa4, 0xe7 // str al r11 r4 plus r4 ASR 31 PreIndex
};
const byte kInstruction_str_al_r10_r2_plus_r4_LSR_20_PostIndex[] = {
  0x24, 0xaa, 0x82, 0xe6 // str al r10 r2 plus r4 LSR 20 PostIndex
};
const byte kInstruction_str_al_r3_r13_minus_r11_LSR_2_PreIndex[] = {
  0x2b, 0x31, 0x2d, 0xe7 // str al r3 r13 minus r11 LSR 2 PreIndex
};
const byte kInstruction_str_al_r14_r2_minus_r5_LSR_21_PreIndex[] = {
  0xa5, 0xea, 0x22, 0xe7 // str al r14 r2 minus r5 LSR 21 PreIndex
};
const byte kInstruction_str_al_r8_r5_minus_r0_LSR_21_PostIndex[] = {
  0xa0, 0x8a, 0x05, 0xe6 // str al r8 r5 minus r0 LSR 21 PostIndex
};
const byte kInstruction_str_al_r9_r10_plus_r3_LSR_10_PostIndex[] = {
  0x23, 0x95, 0x8a, 0xe6 // str al r9 r10 plus r3 LSR 10 PostIndex
};
const byte kInstruction_str_al_r12_r11_plus_r12_LSR_27_PostIndex[] = {
  0xac, 0xcd, 0x8b, 0xe6 // str al r12 r11 plus r12 LSR 27 PostIndex
};
const byte kInstruction_str_al_r9_r8_minus_r8_ASR_28_PreIndex[] = {
  0x48, 0x9e, 0x28, 0xe7 // str al r9 r8 minus r8 ASR 28 PreIndex
};
const byte kInstruction_str_al_r5_r7_minus_r5_LSR_17_PreIndex[] = {
  0xa5, 0x58, 0x27, 0xe7 // str al r5 r7 minus r5 LSR 17 PreIndex
};
const byte kInstruction_str_al_r7_r2_minus_r3_ASR_21_PreIndex[] = {
  0xc3, 0x7a, 0x22, 0xe7 // str al r7 r2 minus r3 ASR 21 PreIndex
};
const byte kInstruction_str_al_r7_r12_minus_r1_ASR_26_PreIndex[] = {
  0x41, 0x7d, 0x2c, 0xe7 // str al r7 r12 minus r1 ASR 26 PreIndex
};
const byte kInstruction_str_al_r14_r5_plus_r8_ASR_14_PreIndex[] = {
  0x48, 0xe7, 0xa5, 0xe7 // str al r14 r5 plus r8 ASR 14 PreIndex
};
const byte kInstruction_str_al_r13_r14_minus_r10_LSR_28_PostIndex[] = {
  0x2a, 0xde, 0x0e, 0xe6 // str al r13 r14 minus r10 LSR 28 PostIndex
};
const byte kInstruction_str_al_r12_r8_plus_r10_ASR_11_PreIndex[] = {
  0xca, 0xc5, 0xa8, 0xe7 // str al r12 r8 plus r10 ASR 11 PreIndex
};
const byte kInstruction_str_al_r1_r7_plus_r12_ASR_6_PostIndex[] = {
  0x4c, 0x13, 0x87, 0xe6 // str al r1 r7 plus r12 ASR 6 PostIndex
};
const byte kInstruction_str_al_r11_r3_plus_r3_LSR_17_PreIndex[] = {
  0xa3, 0xb8, 0xa3, 0xe7 // str al r11 r3 plus r3 LSR 17 PreIndex
};
const byte kInstruction_str_al_r12_r4_minus_r5_ASR_8_PostIndex[] = {
  0x45, 0xc4, 0x04, 0xe6 // str al r12 r4 minus r5 ASR 8 PostIndex
};
const byte kInstruction_str_al_r2_r5_plus_r13_LSR_11_PostIndex[] = {
  0xad, 0x25, 0x85, 0xe6 // str al r2 r5 plus r13 LSR 11 PostIndex
};
const byte kInstruction_str_al_r3_r4_minus_r12_ASR_13_PostIndex[] = {
  0xcc, 0x36, 0x04, 0xe6 // str al r3 r4 minus r12 ASR 13 PostIndex
};
const byte kInstruction_str_al_r1_r0_plus_r7_ASR_6_PreIndex[] = {
  0x47, 0x13, 0xa0, 0xe7 // str al r1 r0 plus r7 ASR 6 PreIndex
};
const byte kInstruction_str_al_r5_r9_minus_r1_ASR_29_PostIndex[] = {
  0xc1, 0x5e, 0x09, 0xe6 // str al r5 r9 minus r1 ASR 29 PostIndex
};
const byte kInstruction_str_al_r13_r11_plus_r1_ASR_17_PostIndex[] = {
  0xc1, 0xd8, 0x8b, 0xe6 // str al r13 r11 plus r1 ASR 17 PostIndex
};
const byte kInstruction_str_al_r8_r10_minus_r12_ASR_26_PostIndex[] = {
  0x4c, 0x8d, 0x0a, 0xe6 // str al r8 r10 minus r12 ASR 26 PostIndex
};
const byte kInstruction_str_al_r7_r8_minus_r5_LSR_18_PreIndex[] = {
  0x25, 0x79, 0x28, 0xe7 // str al r7 r8 minus r5 LSR 18 PreIndex
};
const byte kInstruction_str_al_r9_r8_plus_r2_LSR_14_PreIndex[] = {
  0x22, 0x97, 0xa8, 0xe7 // str al r9 r8 plus r2 LSR 14 PreIndex
};
const byte kInstruction_str_al_r5_r0_plus_r2_ASR_32_PostIndex[] = {
  0x42, 0x50, 0x80, 0xe6 // str al r5 r0 plus r2 ASR 32 PostIndex
};
const byte kInstruction_str_al_r13_r2_minus_r3_ASR_11_PreIndex[] = {
  0xc3, 0xd5, 0x22, 0xe7 // str al r13 r2 minus r3 ASR 11 PreIndex
};
const byte kInstruction_str_al_r1_r12_plus_r10_ASR_18_PostIndex[] = {
  0x4a, 0x19, 0x8c, 0xe6 // str al r1 r12 plus r10 ASR 18 PostIndex
};
const byte kInstruction_str_al_r5_r12_minus_r2_LSR_19_PostIndex[] = {
  0xa2, 0x59, 0x0c, 0xe6 // str al r5 r12 minus r2 LSR 19 PostIndex
};
const byte kInstruction_str_al_r7_r0_minus_r1_LSR_9_PreIndex[] = {
  0xa1, 0x74, 0x20, 0xe7 // str al r7 r0 minus r1 LSR 9 PreIndex
};
const byte kInstruction_str_al_r5_r8_plus_r11_ASR_13_PostIndex[] = {
  0xcb, 0x56, 0x88, 0xe6 // str al r5 r8 plus r11 ASR 13 PostIndex
};
const byte kInstruction_str_al_r7_r0_minus_r8_LSR_24_PreIndex[] = {
  0x28, 0x7c, 0x20, 0xe7 // str al r7 r0 minus r8 LSR 24 PreIndex
};
const byte kInstruction_str_al_r4_r3_plus_r5_ASR_24_PreIndex[] = {
  0x45, 0x4c, 0xa3, 0xe7 // str al r4 r3 plus r5 ASR 24 PreIndex
};
const byte kInstruction_str_al_r7_r9_minus_r6_LSR_23_PreIndex[] = {
  0xa6, 0x7b, 0x29, 0xe7 // str al r7 r9 minus r6 LSR 23 PreIndex
};
const byte kInstruction_str_al_r2_r4_plus_r0_LSR_30_PreIndex[] = {
  0x20, 0x2f, 0xa4, 0xe7 // str al r2 r4 plus r0 LSR 30 PreIndex
};
const byte kInstruction_str_al_r10_r6_plus_r13_LSR_26_PreIndex[] = {
  0x2d, 0xad, 0xa6, 0xe7 // str al r10 r6 plus r13 LSR 26 PreIndex
};
const byte kInstruction_str_al_r4_r8_minus_r9_ASR_17_PreIndex[] = {
  0xc9, 0x48, 0x28, 0xe7 // str al r4 r8 minus r9 ASR 17 PreIndex
};
const byte kInstruction_str_al_r4_r9_plus_r9_ASR_30_PreIndex[] = {
  0x49, 0x4f, 0xa9, 0xe7 // str al r4 r9 plus r9 ASR 30 PreIndex
};
const byte kInstruction_str_al_r1_r5_plus_r13_LSR_20_PostIndex[] = {
  0x2d, 0x1a, 0x85, 0xe6 // str al r1 r5 plus r13 LSR 20 PostIndex
};
const byte kInstruction_str_al_r8_r6_minus_r1_LSR_29_PreIndex[] = {
  0xa1, 0x8e, 0x26, 0xe7 // str al r8 r6 minus r1 LSR 29 PreIndex
};
const byte kInstruction_str_al_r12_r3_plus_r6_ASR_29_PostIndex[] = {
  0xc6, 0xce, 0x83, 0xe6 // str al r12 r3 plus r6 ASR 29 PostIndex
};
const byte kInstruction_str_al_r13_r5_plus_r0_LSR_13_PreIndex[] = {
  0xa0, 0xd6, 0xa5, 0xe7 // str al r13 r5 plus r0 LSR 13 PreIndex
};
const byte kInstruction_str_al_r5_r1_plus_r0_ASR_26_PreIndex[] = {
  0x40, 0x5d, 0xa1, 0xe7 // str al r5 r1 plus r0 ASR 26 PreIndex
};
const byte kInstruction_str_al_r6_r7_plus_r8_ASR_26_PreIndex[] = {
  0x48, 0x6d, 0xa7, 0xe7 // str al r6 r7 plus r8 ASR 26 PreIndex
};
const byte kInstruction_str_al_r5_r14_plus_r8_ASR_8_PostIndex[] = {
  0x48, 0x54, 0x8e, 0xe6 // str al r5 r14 plus r8 ASR 8 PostIndex
};
const byte kInstruction_str_al_r9_r14_plus_r6_LSR_23_PreIndex[] = {
  0xa6, 0x9b, 0xae, 0xe7 // str al r9 r14 plus r6 LSR 23 PreIndex
};
const byte kInstruction_str_al_r6_r4_plus_r3_ASR_9_PreIndex[] = {
  0xc3, 0x64, 0xa4, 0xe7 // str al r6 r4 plus r3 ASR 9 PreIndex
};
const byte kInstruction_str_al_r8_r7_minus_r11_ASR_29_PreIndex[] = {
  0xcb, 0x8e, 0x27, 0xe7 // str al r8 r7 minus r11 ASR 29 PreIndex
};
const byte kInstruction_str_al_r9_r3_plus_r0_ASR_5_PostIndex[] = {
  0xc0, 0x92, 0x83, 0xe6 // str al r9 r3 plus r0 ASR 5 PostIndex
};
const byte kInstruction_str_al_r8_r6_plus_r5_LSR_4_PreIndex[] = {
  0x25, 0x82, 0xa6, 0xe7 // str al r8 r6 plus r5 LSR 4 PreIndex
};
const byte kInstruction_str_al_r9_r8_plus_r7_ASR_4_PostIndex[] = {
  0x47, 0x92, 0x88, 0xe6 // str al r9 r8 plus r7 ASR 4 PostIndex
};
const byte kInstruction_str_al_r12_r4_plus_r6_ASR_28_PostIndex[] = {
  0x46, 0xce, 0x84, 0xe6 // str al r12 r4 plus r6 ASR 28 PostIndex
};
const byte kInstruction_str_al_r4_r7_minus_r4_ASR_25_PreIndex[] = {
  0xc4, 0x4c, 0x27, 0xe7 // str al r4 r7 minus r4 ASR 25 PreIndex
};
const byte kInstruction_str_al_r1_r4_minus_r7_ASR_5_PostIndex[] = {
  0xc7, 0x12, 0x04, 0xe6 // str al r1 r4 minus r7 ASR 5 PostIndex
};
const byte kInstruction_str_al_r3_r1_plus_r12_LSR_25_PostIndex[] = {
  0xac, 0x3c, 0x81, 0xe6 // str al r3 r1 plus r12 LSR 25 PostIndex
};
const byte kInstruction_str_al_r13_r8_plus_r14_ASR_32_PreIndex[] = {
  0x4e, 0xd0, 0xa8, 0xe7 // str al r13 r8 plus r14 ASR 32 PreIndex
};
const byte kInstruction_str_al_r1_r3_minus_r1_LSR_7_PostIndex[] = {
  0xa1, 0x13, 0x03, 0xe6 // str al r1 r3 minus r1 LSR 7 PostIndex
};
const byte kInstruction_str_al_r13_r3_minus_r4_ASR_21_PostIndex[] = {
  0xc4, 0xda, 0x03, 0xe6 // str al r13 r3 minus r4 ASR 21 PostIndex
};
const byte kInstruction_str_al_r11_r8_plus_r10_LSR_7_PostIndex[] = {
  0xaa, 0xb3, 0x88, 0xe6 // str al r11 r8 plus r10 LSR 7 PostIndex
};
const byte kInstruction_str_al_r10_r0_minus_r0_LSR_24_PreIndex[] = {
  0x20, 0xac, 0x20, 0xe7 // str al r10 r0 minus r0 LSR 24 PreIndex
};
const byte kInstruction_str_al_r12_r11_plus_r2_LSR_7_PreIndex[] = {
  0xa2, 0xc3, 0xab, 0xe7 // str al r12 r11 plus r2 LSR 7 PreIndex
};
const byte kInstruction_str_al_r11_r10_minus_r6_ASR_2_PostIndex[] = {
  0x46, 0xb1, 0x0a, 0xe6 // str al r11 r10 minus r6 ASR 2 PostIndex
};
const byte kInstruction_str_al_r5_r10_minus_r10_ASR_19_PostIndex[] = {
  0xca, 0x59, 0x0a, 0xe6 // str al r5 r10 minus r10 ASR 19 PostIndex
};
const byte kInstruction_str_al_r13_r8_minus_r6_ASR_30_PreIndex[] = {
  0x46, 0xdf, 0x28, 0xe7 // str al r13 r8 minus r6 ASR 30 PreIndex
};
const byte kInstruction_str_al_r1_r10_plus_r0_ASR_18_PreIndex[] = {
  0x40, 0x19, 0xaa, 0xe7 // str al r1 r10 plus r0 ASR 18 PreIndex
};
const byte kInstruction_str_al_r10_r14_plus_r5_ASR_8_PostIndex[] = {
  0x45, 0xa4, 0x8e, 0xe6 // str al r10 r14 plus r5 ASR 8 PostIndex
};
const byte kInstruction_str_al_r1_r3_minus_r10_LSR_11_PreIndex[] = {
  0xaa, 0x15, 0x23, 0xe7 // str al r1 r3 minus r10 LSR 11 PreIndex
};
const byte kInstruction_str_al_r11_r13_minus_r11_LSR_5_PreIndex[] = {
  0xab, 0xb2, 0x2d, 0xe7 // str al r11 r13 minus r11 LSR 5 PreIndex
};
const byte kInstruction_str_al_r4_r9_plus_r12_LSR_28_PostIndex[] = {
  0x2c, 0x4e, 0x89, 0xe6 // str al r4 r9 plus r12 LSR 28 PostIndex
};
const byte kInstruction_str_al_r3_r8_plus_r12_LSR_14_PostIndex[] = {
  0x2c, 0x37, 0x88, 0xe6 // str al r3 r8 plus r12 LSR 14 PostIndex
};
const byte kInstruction_str_al_r11_r7_minus_r2_ASR_3_PostIndex[] = {
  0xc2, 0xb1, 0x07, 0xe6 // str al r11 r7 minus r2 ASR 3 PostIndex
};
const byte kInstruction_str_al_r0_r8_plus_r7_ASR_20_PostIndex[] = {
  0x47, 0x0a, 0x88, 0xe6 // str al r0 r8 plus r7 ASR 20 PostIndex
};
const byte kInstruction_str_al_r8_r1_minus_r2_LSR_15_PreIndex[] = {
  0xa2, 0x87, 0x21, 0xe7 // str al r8 r1 minus r2 LSR 15 PreIndex
};
const byte kInstruction_str_al_r9_r5_plus_r7_LSR_14_PostIndex[] = {
  0x27, 0x97, 0x85, 0xe6 // str al r9 r5 plus r7 LSR 14 PostIndex
};
const byte kInstruction_str_al_r1_r8_minus_r13_LSR_12_PostIndex[] = {
  0x2d, 0x16, 0x08, 0xe6 // str al r1 r8 minus r13 LSR 12 PostIndex
};
const byte kInstruction_str_al_r8_r0_plus_r12_LSR_22_PreIndex[] = {
  0x2c, 0x8b, 0xa0, 0xe7 // str al r8 r0 plus r12 LSR 22 PreIndex
};
const byte kInstruction_str_al_r13_r0_minus_r10_ASR_26_PostIndex[] = {
  0x4a, 0xdd, 0x00, 0xe6 // str al r13 r0 minus r10 ASR 26 PostIndex
};
const byte kInstruction_str_al_r12_r3_minus_r3_LSR_11_PreIndex[] = {
  0xa3, 0xc5, 0x23, 0xe7 // str al r12 r3 minus r3 LSR 11 PreIndex
};
const byte kInstruction_str_al_r7_r3_minus_r8_LSR_25_PostIndex[] = {
  0xa8, 0x7c, 0x03, 0xe6 // str al r7 r3 minus r8 LSR 25 PostIndex
};
const byte kInstruction_str_al_r5_r4_plus_r5_ASR_2_PreIndex[] = {
  0x45, 0x51, 0xa4, 0xe7 // str al r5 r4 plus r5 ASR 2 PreIndex
};
const byte kInstruction_str_al_r3_r10_plus_r1_ASR_22_PostIndex[] = {
  0x41, 0x3b, 0x8a, 0xe6 // str al r3 r10 plus r1 ASR 22 PostIndex
};
const byte kInstruction_str_al_r6_r11_minus_r7_LSR_4_PostIndex[] = {
  0x27, 0x62, 0x0b, 0xe6 // str al r6 r11 minus r7 LSR 4 PostIndex
};
const byte kInstruction_str_al_r13_r7_minus_r3_ASR_28_PreIndex[] = {
  0x43, 0xde, 0x27, 0xe7 // str al r13 r7 minus r3 ASR 28 PreIndex
};
const byte kInstruction_str_al_r3_r10_minus_r4_LSR_5_PreIndex[] = {
  0xa4, 0x32, 0x2a, 0xe7 // str al r3 r10 minus r4 LSR 5 PreIndex
};
const byte kInstruction_str_al_r1_r2_minus_r10_LSR_25_PreIndex[] = {
  0xaa, 0x1c, 0x22, 0xe7 // str al r1 r2 minus r10 LSR 25 PreIndex
};
const byte kInstruction_str_al_r1_r3_plus_r11_LSR_19_PreIndex[] = {
  0xab, 0x19, 0xa3, 0xe7 // str al r1 r3 plus r11 LSR 19 PreIndex
};
const byte kInstruction_str_al_r3_r4_minus_r9_LSR_32_PostIndex[] = {
  0x29, 0x30, 0x04, 0xe6 // str al r3 r4 minus r9 LSR 32 PostIndex
};
const byte kInstruction_str_al_r13_r11_plus_r2_LSR_17_PreIndex[] = {
  0xa2, 0xd8, 0xab, 0xe7 // str al r13 r11 plus r2 LSR 17 PreIndex
};
const byte kInstruction_str_al_r4_r6_plus_r2_ASR_20_PreIndex[] = {
  0x42, 0x4a, 0xa6, 0xe7 // str al r4 r6 plus r2 ASR 20 PreIndex
};
const byte kInstruction_str_al_r1_r12_minus_r10_ASR_18_PreIndex[] = {
  0x4a, 0x19, 0x2c, 0xe7 // str al r1 r12 minus r10 ASR 18 PreIndex
};
const byte kInstruction_str_al_r4_r14_plus_r7_LSR_15_PostIndex[] = {
  0xa7, 0x47, 0x8e, 0xe6 // str al r4 r14 plus r7 LSR 15 PostIndex
};
const byte kInstruction_str_al_r11_r13_minus_r0_ASR_19_PreIndex[] = {
  0xc0, 0xb9, 0x2d, 0xe7 // str al r11 r13 minus r0 ASR 19 PreIndex
};
const byte kInstruction_str_al_r13_r10_minus_r12_ASR_8_PostIndex[] = {
  0x4c, 0xd4, 0x0a, 0xe6 // str al r13 r10 minus r12 ASR 8 PostIndex
};
const byte kInstruction_str_al_r5_r14_minus_r4_ASR_17_PreIndex[] = {
  0xc4, 0x58, 0x2e, 0xe7 // str al r5 r14 minus r4 ASR 17 PreIndex
};
const byte kInstruction_str_al_r8_r7_minus_r3_LSR_11_PreIndex[] = {
  0xa3, 0x85, 0x27, 0xe7 // str al r8 r7 minus r3 LSR 11 PreIndex
};
const byte kInstruction_str_al_r12_r7_plus_r8_ASR_3_PostIndex[] = {
  0xc8, 0xc1, 0x87, 0xe6 // str al r12 r7 plus r8 ASR 3 PostIndex
};
const byte kInstruction_str_al_r0_r6_minus_r4_ASR_26_PostIndex[] = {
  0x44, 0x0d, 0x06, 0xe6 // str al r0 r6 minus r4 ASR 26 PostIndex
};
const byte kInstruction_str_al_r5_r13_plus_r8_ASR_21_PreIndex[] = {
  0xc8, 0x5a, 0xad, 0xe7 // str al r5 r13 plus r8 ASR 21 PreIndex
};
const byte kInstruction_str_al_r4_r0_minus_r0_ASR_3_PostIndex[] = {
  0xc0, 0x41, 0x00, 0xe6 // str al r4 r0 minus r0 ASR 3 PostIndex
};
const byte kInstruction_str_al_r13_r3_minus_r0_LSR_20_PreIndex[] = {
  0x20, 0xda, 0x23, 0xe7 // str al r13 r3 minus r0 LSR 20 PreIndex
};
const byte kInstruction_str_al_r3_r14_minus_r6_ASR_31_PostIndex[] = {
  0xc6, 0x3f, 0x0e, 0xe6 // str al r3 r14 minus r6 ASR 31 PostIndex
};
const byte kInstruction_str_al_r12_r2_plus_r13_LSR_1_PreIndex[] = {
  0xad, 0xc0, 0xa2, 0xe7 // str al r12 r2 plus r13 LSR 1 PreIndex
};
const byte kInstruction_str_al_r9_r11_plus_r8_LSR_15_PreIndex[] = {
  0xa8, 0x97, 0xab, 0xe7 // str al r9 r11 plus r8 LSR 15 PreIndex
};
const byte kInstruction_str_al_r3_r14_plus_r9_LSR_22_PreIndex[] = {
  0x29, 0x3b, 0xae, 0xe7 // str al r3 r14 plus r9 LSR 22 PreIndex
};
const byte kInstruction_str_al_r12_r1_plus_r2_ASR_23_PostIndex[] = {
  0xc2, 0xcb, 0x81, 0xe6 // str al r12 r1 plus r2 ASR 23 PostIndex
};
const byte kInstruction_str_al_r4_r6_plus_r4_LSR_18_PreIndex[] = {
  0x24, 0x49, 0xa6, 0xe7 // str al r4 r6 plus r4 LSR 18 PreIndex
};
const byte kInstruction_str_al_r11_r2_minus_r3_ASR_30_PreIndex[] = {
  0x43, 0xbf, 0x22, 0xe7 // str al r11 r2 minus r3 ASR 30 PreIndex
};
const byte kInstruction_str_al_r4_r1_minus_r0_ASR_6_PostIndex[] = {
  0x40, 0x43, 0x01, 0xe6 // str al r4 r1 minus r0 ASR 6 PostIndex
};
const byte kInstruction_str_al_r12_r5_minus_r9_LSR_30_PreIndex[] = {
  0x29, 0xcf, 0x25, 0xe7 // str al r12 r5 minus r9 LSR 30 PreIndex
};
const byte kInstruction_str_al_r11_r0_minus_r1_LSR_26_PreIndex[] = {
  0x21, 0xbd, 0x20, 0xe7 // str al r11 r0 minus r1 LSR 26 PreIndex
};
const byte kInstruction_str_al_r11_r2_plus_r11_LSR_4_PostIndex[] = {
  0x2b, 0xb2, 0x82, 0xe6 // str al r11 r2 plus r11 LSR 4 PostIndex
};
const byte kInstruction_str_al_r12_r13_plus_r10_LSR_8_PostIndex[] = {
  0x2a, 0xc4, 0x8d, 0xe6 // str al r12 r13 plus r10 LSR 8 PostIndex
};
const byte kInstruction_str_al_r0_r9_minus_r2_ASR_29_PostIndex[] = {
  0xc2, 0x0e, 0x09, 0xe6 // str al r0 r9 minus r2 ASR 29 PostIndex
};
const byte kInstruction_str_al_r1_r10_plus_r13_LSR_16_PostIndex[] = {
  0x2d, 0x18, 0x8a, 0xe6 // str al r1 r10 plus r13 LSR 16 PostIndex
};
const byte kInstruction_str_al_r5_r12_minus_r1_ASR_21_PostIndex[] = {
  0xc1, 0x5a, 0x0c, 0xe6 // str al r5 r12 minus r1 ASR 21 PostIndex
};
const byte kInstruction_str_al_r14_r1_minus_r11_ASR_23_PreIndex[] = {
  0xcb, 0xeb, 0x21, 0xe7 // str al r14 r1 minus r11 ASR 23 PreIndex
};
const byte kInstruction_str_al_r6_r12_plus_r10_LSR_13_PostIndex[] = {
  0xaa, 0x66, 0x8c, 0xe6 // str al r6 r12 plus r10 LSR 13 PostIndex
};
const byte kInstruction_str_al_r5_r2_minus_r2_ASR_13_PreIndex[] = {
  0xc2, 0x56, 0x22, 0xe7 // str al r5 r2 minus r2 ASR 13 PreIndex
};
const byte kInstruction_str_al_r1_r12_plus_r6_ASR_18_PreIndex[] = {
  0x46, 0x19, 0xac, 0xe7 // str al r1 r12 plus r6 ASR 18 PreIndex
};
const byte kInstruction_str_al_r7_r5_minus_r1_ASR_11_PostIndex[] = {
  0xc1, 0x75, 0x05, 0xe6 // str al r7 r5 minus r1 ASR 11 PostIndex
};
const byte kInstruction_str_al_r4_r6_minus_r1_LSR_3_PreIndex[] = {
  0xa1, 0x41, 0x26, 0xe7 // str al r4 r6 minus r1 LSR 3 PreIndex
};
const byte kInstruction_str_al_r6_r5_plus_r13_LSR_27_PreIndex[] = {
  0xad, 0x6d, 0xa5, 0xe7 // str al r6 r5 plus r13 LSR 27 PreIndex
};
const byte kInstruction_str_al_r1_r13_minus_r13_ASR_30_PreIndex[] = {
  0x4d, 0x1f, 0x2d, 0xe7 // str al r1 r13 minus r13 ASR 30 PreIndex
};
const byte kInstruction_str_al_r5_r13_plus_r9_LSR_3_PreIndex[] = {
  0xa9, 0x51, 0xad, 0xe7 // str al r5 r13 plus r9 LSR 3 PreIndex
};
const byte kInstruction_str_al_r1_r4_plus_r0_ASR_25_PreIndex[] = {
  0xc0, 0x1c, 0xa4, 0xe7 // str al r1 r4 plus r0 ASR 25 PreIndex
};
const byte kInstruction_str_al_r0_r12_minus_r0_ASR_17_PostIndex[] = {
  0xc0, 0x08, 0x0c, 0xe6 // str al r0 r12 minus r0 ASR 17 PostIndex
};
const byte kInstruction_str_al_r5_r4_minus_r7_LSR_11_PostIndex[] = {
  0xa7, 0x55, 0x04, 0xe6 // str al r5 r4 minus r7 LSR 11 PostIndex
};
const byte kInstruction_str_al_r14_r4_minus_r10_ASR_16_PreIndex[] = {
  0x4a, 0xe8, 0x24, 0xe7 // str al r14 r4 minus r10 ASR 16 PreIndex
};
const byte kInstruction_str_al_r4_r12_plus_r12_LSR_17_PostIndex[] = {
  0xac, 0x48, 0x8c, 0xe6 // str al r4 r12 plus r12 LSR 17 PostIndex
};
const byte kInstruction_str_al_r9_r2_plus_r7_LSR_24_PostIndex[] = {
  0x27, 0x9c, 0x82, 0xe6 // str al r9 r2 plus r7 LSR 24 PostIndex
};
const byte kInstruction_str_al_r13_r14_plus_r7_ASR_12_PostIndex[] = {
  0x47, 0xd6, 0x8e, 0xe6 // str al r13 r14 plus r7 ASR 12 PostIndex
};
const byte kInstruction_str_al_r2_r7_minus_r11_LSR_13_PreIndex[] = {
  0xab, 0x26, 0x27, 0xe7 // str al r2 r7 minus r11 LSR 13 PreIndex
};
const byte kInstruction_str_al_r0_r4_plus_r9_LSR_27_PostIndex[] = {
  0xa9, 0x0d, 0x84, 0xe6 // str al r0 r4 plus r9 LSR 27 PostIndex
};
const byte kInstruction_str_al_r6_r9_minus_r10_LSR_29_PostIndex[] = {
  0xaa, 0x6e, 0x09, 0xe6 // str al r6 r9 minus r10 LSR 29 PostIndex
};
const byte kInstruction_str_al_r7_r4_minus_r1_ASR_25_PostIndex[] = {
  0xc1, 0x7c, 0x04, 0xe6 // str al r7 r4 minus r1 ASR 25 PostIndex
};
const byte kInstruction_str_al_r6_r1_plus_r11_ASR_29_PostIndex[] = {
  0xcb, 0x6e, 0x81, 0xe6 // str al r6 r1 plus r11 ASR 29 PostIndex
};
const byte kInstruction_str_al_r12_r7_minus_r13_LSR_3_PostIndex[] = {
  0xad, 0xc1, 0x07, 0xe6 // str al r12 r7 minus r13 LSR 3 PostIndex
};
const byte kInstruction_str_al_r13_r12_plus_r6_ASR_25_PostIndex[] = {
  0xc6, 0xdc, 0x8c, 0xe6 // str al r13 r12 plus r6 ASR 25 PostIndex
};
const byte kInstruction_str_al_r2_r4_minus_r1_ASR_13_PostIndex[] = {
  0xc1, 0x26, 0x04, 0xe6 // str al r2 r4 minus r1 ASR 13 PostIndex
};
const byte kInstruction_str_al_r6_r3_plus_r3_ASR_5_PreIndex[] = {
  0xc3, 0x62, 0xa3, 0xe7 // str al r6 r3 plus r3 ASR 5 PreIndex
};
const byte kInstruction_str_al_r0_r1_plus_r0_LSR_8_PostIndex[] = {
  0x20, 0x04, 0x81, 0xe6 // str al r0 r1 plus r0 LSR 8 PostIndex
};
const byte kInstruction_str_al_r6_r0_minus_r3_LSR_31_PostIndex[] = {
  0xa3, 0x6f, 0x00, 0xe6 // str al r6 r0 minus r3 LSR 31 PostIndex
};
const byte kInstruction_str_al_r9_r11_minus_r3_ASR_2_PostIndex[] = {
  0x43, 0x91, 0x0b, 0xe6 // str al r9 r11 minus r3 ASR 2 PostIndex
};
const byte kInstruction_str_al_r1_r13_plus_r13_LSR_26_PostIndex[] = {
  0x2d, 0x1d, 0x8d, 0xe6 // str al r1 r13 plus r13 LSR 26 PostIndex
};
const byte kInstruction_str_al_r14_r3_minus_r7_ASR_18_PreIndex[] = {
  0x47, 0xe9, 0x23, 0xe7 // str al r14 r3 minus r7 ASR 18 PreIndex
};
const byte kInstruction_str_al_r11_r12_plus_r1_LSR_5_PostIndex[] = {
  0xa1, 0xb2, 0x8c, 0xe6 // str al r11 r12 plus r1 LSR 5 PostIndex
};
const byte kInstruction_str_al_r4_r6_minus_r0_ASR_7_PostIndex[] = {
  0xc0, 0x43, 0x06, 0xe6 // str al r4 r6 minus r0 ASR 7 PostIndex
};
const byte kInstruction_str_al_r8_r14_plus_r2_ASR_13_PreIndex[] = {
  0xc2, 0x86, 0xae, 0xe7 // str al r8 r14 plus r2 ASR 13 PreIndex
};
const byte kInstruction_str_al_r1_r6_minus_r13_ASR_14_PostIndex[] = {
  0x4d, 0x17, 0x06, 0xe6 // str al r1 r6 minus r13 ASR 14 PostIndex
};
const byte kInstruction_str_al_r8_r9_minus_r3_LSR_3_PreIndex[] = {
  0xa3, 0x81, 0x29, 0xe7 // str al r8 r9 minus r3 LSR 3 PreIndex
};
const byte kInstruction_str_al_r2_r14_plus_r8_ASR_2_PreIndex[] = {
  0x48, 0x21, 0xae, 0xe7 // str al r2 r14 plus r8 ASR 2 PreIndex
};
const byte kInstruction_str_al_r1_r10_minus_r5_ASR_31_PostIndex[] = {
  0xc5, 0x1f, 0x0a, 0xe6 // str al r1 r10 minus r5 ASR 31 PostIndex
};
const byte kInstruction_str_al_r2_r14_minus_r13_ASR_28_PostIndex[] = {
  0x4d, 0x2e, 0x0e, 0xe6 // str al r2 r14 minus r13 ASR 28 PostIndex
};
const byte kInstruction_str_al_r14_r5_plus_r5_LSR_11_PostIndex[] = {
  0xa5, 0xe5, 0x85, 0xe6 // str al r14 r5 plus r5 LSR 11 PostIndex
};
const byte kInstruction_str_al_r2_r14_minus_r14_ASR_21_PostIndex[] = {
  0xce, 0x2a, 0x0e, 0xe6 // str al r2 r14 minus r14 ASR 21 PostIndex
};
const byte kInstruction_str_al_r12_r1_plus_r0_LSR_3_PostIndex[] = {
  0xa0, 0xc1, 0x81, 0xe6 // str al r12 r1 plus r0 LSR 3 PostIndex
};
const byte kInstruction_str_al_r12_r5_plus_r14_LSR_27_PostIndex[] = {
  0xae, 0xcd, 0x85, 0xe6 // str al r12 r5 plus r14 LSR 27 PostIndex
};
const byte kInstruction_str_al_r14_r3_minus_r6_LSR_15_PostIndex[] = {
  0xa6, 0xe7, 0x03, 0xe6 // str al r14 r3 minus r6 LSR 15 PostIndex
};
const byte kInstruction_str_al_r1_r4_plus_r1_LSR_29_PostIndex[] = {
  0xa1, 0x1e, 0x84, 0xe6 // str al r1 r4 plus r1 LSR 29 PostIndex
};
const byte kInstruction_str_al_r3_r2_plus_r8_ASR_29_PreIndex[] = {
  0xc8, 0x3e, 0xa2, 0xe7 // str al r3 r2 plus r8 ASR 29 PreIndex
};
const byte kInstruction_str_al_r6_r4_minus_r2_ASR_28_PostIndex[] = {
  0x42, 0x6e, 0x04, 0xe6 // str al r6 r4 minus r2 ASR 28 PostIndex
};
const byte kInstruction_str_al_r7_r11_minus_r13_ASR_31_PreIndex[] = {
  0xcd, 0x7f, 0x2b, 0xe7 // str al r7 r11 minus r13 ASR 31 PreIndex
};
const byte kInstruction_str_al_r9_r13_plus_r10_ASR_10_PreIndex[] = {
  0x4a, 0x95, 0xad, 0xe7 // str al r9 r13 plus r10 ASR 10 PreIndex
};
const byte kInstruction_str_al_r7_r5_plus_r8_ASR_14_PostIndex[] = {
  0x48, 0x77, 0x85, 0xe6 // str al r7 r5 plus r8 ASR 14 PostIndex
};
const byte kInstruction_str_al_r12_r7_minus_r13_LSR_8_PostIndex[] = {
  0x2d, 0xc4, 0x07, 0xe6 // str al r12 r7 minus r13 LSR 8 PostIndex
};
const byte kInstruction_str_al_r14_r11_plus_r10_ASR_6_PreIndex[] = {
  0x4a, 0xe3, 0xab, 0xe7 // str al r14 r11 plus r10 ASR 6 PreIndex
};
const byte kInstruction_str_al_r1_r5_minus_r5_LSR_22_PreIndex[] = {
  0x25, 0x1b, 0x25, 0xe7 // str al r1 r5 minus r5 LSR 22 PreIndex
};
const byte kInstruction_str_al_r12_r5_plus_r4_ASR_7_PostIndex[] = {
  0xc4, 0xc3, 0x85, 0xe6 // str al r12 r5 plus r4 ASR 7 PostIndex
};
const byte kInstruction_str_al_r10_r12_minus_r11_LSR_15_PreIndex[] = {
  0xab, 0xa7, 0x2c, 0xe7 // str al r10 r12 minus r11 LSR 15 PreIndex
};
const byte kInstruction_str_al_r13_r2_plus_r5_LSR_26_PreIndex[] = {
  0x25, 0xdd, 0xa2, 0xe7 // str al r13 r2 plus r5 LSR 26 PreIndex
};
const byte kInstruction_str_al_r7_r1_minus_r4_LSR_24_PostIndex[] = {
  0x24, 0x7c, 0x01, 0xe6 // str al r7 r1 minus r4 LSR 24 PostIndex
};
const byte kInstruction_str_al_r7_r1_plus_r5_LSR_10_PostIndex[] = {
  0x25, 0x75, 0x81, 0xe6 // str al r7 r1 plus r5 LSR 10 PostIndex
};
const byte kInstruction_str_al_r7_r0_plus_r3_ASR_27_PostIndex[] = {
  0xc3, 0x7d, 0x80, 0xe6 // str al r7 r0 plus r3 ASR 27 PostIndex
};
const byte kInstruction_str_al_r5_r11_minus_r8_ASR_29_PostIndex[] = {
  0xc8, 0x5e, 0x0b, 0xe6 // str al r5 r11 minus r8 ASR 29 PostIndex
};
const byte kInstruction_str_al_r9_r12_plus_r7_LSR_29_PostIndex[] = {
  0xa7, 0x9e, 0x8c, 0xe6 // str al r9 r12 plus r7 LSR 29 PostIndex
};
const byte kInstruction_str_al_r10_r0_minus_r10_LSR_8_PostIndex[] = {
  0x2a, 0xa4, 0x00, 0xe6 // str al r10 r0 minus r10 LSR 8 PostIndex
};
const byte kInstruction_str_al_r5_r12_plus_r14_LSR_8_PostIndex[] = {
  0x2e, 0x54, 0x8c, 0xe6 // str al r5 r12 plus r14 LSR 8 PostIndex
};
const byte kInstruction_str_al_r13_r12_plus_r3_LSR_14_PreIndex[] = {
  0x23, 0xd7, 0xac, 0xe7 // str al r13 r12 plus r3 LSR 14 PreIndex
};
const byte kInstruction_str_al_r4_r13_minus_r7_ASR_4_PreIndex[] = {
  0x47, 0x42, 0x2d, 0xe7 // str al r4 r13 minus r7 ASR 4 PreIndex
};
const byte kInstruction_str_al_r2_r12_minus_r12_ASR_9_PostIndex[] = {
  0xcc, 0x24, 0x0c, 0xe6 // str al r2 r12 minus r12 ASR 9 PostIndex
};
const byte kInstruction_str_al_r13_r8_plus_r11_LSR_2_PostIndex[] = {
  0x2b, 0xd1, 0x88, 0xe6 // str al r13 r8 plus r11 LSR 2 PostIndex
};
const byte kInstruction_str_al_r6_r5_minus_r12_ASR_6_PostIndex[] = {
  0x4c, 0x63, 0x05, 0xe6 // str al r6 r5 minus r12 ASR 6 PostIndex
};
const byte kInstruction_str_al_r9_r12_plus_r14_ASR_31_PreIndex[] = {
  0xce, 0x9f, 0xac, 0xe7 // str al r9 r12 plus r14 ASR 31 PreIndex
};
const byte kInstruction_str_al_r13_r7_minus_r5_ASR_17_PreIndex[] = {
  0xc5, 0xd8, 0x27, 0xe7 // str al r13 r7 minus r5 ASR 17 PreIndex
};
const byte kInstruction_str_al_r14_r4_minus_r8_ASR_32_PostIndex[] = {
  0x48, 0xe0, 0x04, 0xe6 // str al r14 r4 minus r8 ASR 32 PostIndex
};
const byte kInstruction_str_al_r1_r4_plus_r9_LSR_20_PreIndex[] = {
  0x29, 0x1a, 0xa4, 0xe7 // str al r1 r4 plus r9 LSR 20 PreIndex
};
const byte kInstruction_str_al_r11_r8_plus_r13_LSR_12_PostIndex[] = {
  0x2d, 0xb6, 0x88, 0xe6 // str al r11 r8 plus r13 LSR 12 PostIndex
};
const byte kInstruction_str_al_r3_r0_plus_r4_LSR_22_PreIndex[] = {
  0x24, 0x3b, 0xa0, 0xe7 // str al r3 r0 plus r4 LSR 22 PreIndex
};
const byte kInstruction_str_al_r9_r8_plus_r2_LSR_19_PostIndex[] = {
  0xa2, 0x99, 0x88, 0xe6 // str al r9 r8 plus r2 LSR 19 PostIndex
};
const byte kInstruction_str_al_r14_r11_plus_r8_ASR_28_PreIndex[] = {
  0x48, 0xee, 0xab, 0xe7 // str al r14 r11 plus r8 ASR 28 PreIndex
};
const byte kInstruction_str_al_r6_r10_plus_r5_LSR_28_PostIndex[] = {
  0x25, 0x6e, 0x8a, 0xe6 // str al r6 r10 plus r5 LSR 28 PostIndex
};
const byte kInstruction_str_al_r12_r13_minus_r12_ASR_14_PreIndex[] = {
  0x4c, 0xc7, 0x2d, 0xe7 // str al r12 r13 minus r12 ASR 14 PreIndex
};
const byte kInstruction_str_al_r12_r2_plus_r9_ASR_13_PostIndex[] = {
  0xc9, 0xc6, 0x82, 0xe6 // str al r12 r2 plus r9 ASR 13 PostIndex
};
const byte kInstruction_str_al_r11_r13_minus_r8_LSR_14_PreIndex[] = {
  0x28, 0xb7, 0x2d, 0xe7 // str al r11 r13 minus r8 LSR 14 PreIndex
};
const byte kInstruction_str_al_r6_r4_minus_r11_LSR_1_PreIndex[] = {
  0xab, 0x60, 0x24, 0xe7 // str al r6 r4 minus r11 LSR 1 PreIndex
};
const byte kInstruction_str_al_r3_r6_plus_r1_ASR_14_PostIndex[] = {
  0x41, 0x37, 0x86, 0xe6 // str al r3 r6 plus r1 ASR 14 PostIndex
};
const byte kInstruction_str_al_r14_r9_minus_r0_ASR_23_PostIndex[] = {
  0xc0, 0xeb, 0x09, 0xe6 // str al r14 r9 minus r0 ASR 23 PostIndex
};
const byte kInstruction_str_al_r9_r2_plus_r0_ASR_23_PreIndex[] = {
  0xc0, 0x9b, 0xa2, 0xe7 // str al r9 r2 plus r0 ASR 23 PreIndex
};
const byte kInstruction_str_al_r8_r4_minus_r9_LSR_13_PostIndex[] = {
  0xa9, 0x86, 0x04, 0xe6 // str al r8 r4 minus r9 LSR 13 PostIndex
};
const byte kInstruction_str_al_r0_r12_plus_r5_LSR_12_PostIndex[] = {
  0x25, 0x06, 0x8c, 0xe6 // str al r0 r12 plus r5 LSR 12 PostIndex
};
const byte kInstruction_str_al_r6_r5_plus_r3_ASR_31_PostIndex[] = {
  0xc3, 0x6f, 0x85, 0xe6 // str al r6 r5 plus r3 ASR 31 PostIndex
};
const byte kInstruction_str_al_r0_r2_minus_r11_ASR_16_PostIndex[] = {
  0x4b, 0x08, 0x02, 0xe6 // str al r0 r2 minus r11 ASR 16 PostIndex
};
const byte kInstruction_str_al_r14_r10_plus_r12_ASR_32_PreIndex[] = {
  0x4c, 0xe0, 0xaa, 0xe7 // str al r14 r10 plus r12 ASR 32 PreIndex
};
const byte kInstruction_str_al_r10_r14_minus_r7_LSR_13_PreIndex[] = {
  0xa7, 0xa6, 0x2e, 0xe7 // str al r10 r14 minus r7 LSR 13 PreIndex
};
const byte kInstruction_str_al_r2_r10_plus_r13_ASR_32_PostIndex[] = {
  0x4d, 0x20, 0x8a, 0xe6 // str al r2 r10 plus r13 ASR 32 PostIndex
};
const byte kInstruction_str_al_r7_r13_minus_r13_ASR_2_PreIndex[] = {
  0x4d, 0x71, 0x2d, 0xe7 // str al r7 r13 minus r13 ASR 2 PreIndex
};
const byte kInstruction_str_al_r5_r14_plus_r1_ASR_19_PreIndex[] = {
  0xc1, 0x59, 0xae, 0xe7 // str al r5 r14 plus r1 ASR 19 PreIndex
};
const byte kInstruction_str_al_r7_r8_plus_r8_LSR_12_PreIndex[] = {
  0x28, 0x76, 0xa8, 0xe7 // str al r7 r8 plus r8 LSR 12 PreIndex
};
const byte kInstruction_str_al_r6_r8_plus_r6_ASR_31_PostIndex[] = {
  0xc6, 0x6f, 0x88, 0xe6 // str al r6 r8 plus r6 ASR 31 PostIndex
};
const byte kInstruction_str_al_r9_r0_minus_r0_LSR_3_PostIndex[] = {
  0xa0, 0x91, 0x00, 0xe6 // str al r9 r0 minus r0 LSR 3 PostIndex
};
const byte kInstruction_str_al_r13_r14_plus_r6_ASR_31_PostIndex[] = {
  0xc6, 0xdf, 0x8e, 0xe6 // str al r13 r14 plus r6 ASR 31 PostIndex
};
const byte kInstruction_str_al_r5_r8_minus_r0_ASR_26_PostIndex[] = {
  0x40, 0x5d, 0x08, 0xe6 // str al r5 r8 minus r0 ASR 26 PostIndex
};
const byte kInstruction_str_al_r1_r9_plus_r5_LSR_11_PostIndex[] = {
  0xa5, 0x15, 0x89, 0xe6 // str al r1 r9 plus r5 LSR 11 PostIndex
};
const byte kInstruction_str_al_r6_r12_plus_r12_LSR_22_PostIndex[] = {
  0x2c, 0x6b, 0x8c, 0xe6 // str al r6 r12 plus r12 LSR 22 PostIndex
};
const byte kInstruction_str_al_r6_r12_minus_r6_ASR_1_PreIndex[] = {
  0xc6, 0x60, 0x2c, 0xe7 // str al r6 r12 minus r6 ASR 1 PreIndex
};
const byte kInstruction_str_al_r4_r1_plus_r14_LSR_30_PostIndex[] = {
  0x2e, 0x4f, 0x81, 0xe6 // str al r4 r1 plus r14 LSR 30 PostIndex
};
const byte kInstruction_str_al_r9_r11_plus_r8_ASR_21_PostIndex[] = {
  0xc8, 0x9a, 0x8b, 0xe6 // str al r9 r11 plus r8 ASR 21 PostIndex
};
const byte kInstruction_str_al_r1_r0_minus_r13_ASR_19_PostIndex[] = {
  0xcd, 0x19, 0x00, 0xe6 // str al r1 r0 minus r13 ASR 19 PostIndex
};
const byte kInstruction_str_al_r10_r12_plus_r13_LSR_19_PostIndex[] = {
  0xad, 0xa9, 0x8c, 0xe6 // str al r10 r12 plus r13 LSR 19 PostIndex
};
const byte kInstruction_str_al_r2_r7_minus_r14_LSR_29_PostIndex[] = {
  0xae, 0x2e, 0x07, 0xe6 // str al r2 r7 minus r14 LSR 29 PostIndex
};
const byte kInstruction_str_al_r7_r0_plus_r0_LSR_2_PostIndex[] = {
  0x20, 0x71, 0x80, 0xe6 // str al r7 r0 plus r0 LSR 2 PostIndex
};
const byte kInstruction_str_al_r2_r8_plus_r7_LSR_2_PostIndex[] = {
  0x27, 0x21, 0x88, 0xe6 // str al r2 r8 plus r7 LSR 2 PostIndex
};
const byte kInstruction_str_al_r5_r10_minus_r14_LSR_22_PreIndex[] = {
  0x2e, 0x5b, 0x2a, 0xe7 // str al r5 r10 minus r14 LSR 22 PreIndex
};
const byte kInstruction_str_al_r8_r7_plus_r6_ASR_23_PreIndex[] = {
  0xc6, 0x8b, 0xa7, 0xe7 // str al r8 r7 plus r6 ASR 23 PreIndex
};
const byte kInstruction_str_al_r6_r13_plus_r10_LSR_10_PreIndex[] = {
  0x2a, 0x65, 0xad, 0xe7 // str al r6 r13 plus r10 LSR 10 PreIndex
};
const byte kInstruction_str_al_r0_r9_plus_r8_ASR_12_PostIndex[] = {
  0x48, 0x06, 0x89, 0xe6 // str al r0 r9 plus r8 ASR 12 PostIndex
};
const byte kInstruction_str_al_r14_r13_plus_r5_LSR_20_PreIndex[] = {
  0x25, 0xea, 0xad, 0xe7 // str al r14 r13 plus r5 LSR 20 PreIndex
};
const byte kInstruction_str_al_r5_r7_plus_r2_LSR_25_PostIndex[] = {
  0xa2, 0x5c, 0x87, 0xe6 // str al r5 r7 plus r2 LSR 25 PostIndex
};
const byte kInstruction_str_al_r3_r2_plus_r4_LSR_22_PreIndex[] = {
  0x24, 0x3b, 0xa2, 0xe7 // str al r3 r2 plus r4 LSR 22 PreIndex
};
const byte kInstruction_str_al_r14_r10_plus_r5_LSR_27_PreIndex[] = {
  0xa5, 0xed, 0xaa, 0xe7 // str al r14 r10 plus r5 LSR 27 PreIndex
};
const byte kInstruction_str_al_r3_r9_plus_r3_LSR_6_PreIndex[] = {
  0x23, 0x33, 0xa9, 0xe7 // str al r3 r9 plus r3 LSR 6 PreIndex
};
const byte kInstruction_str_al_r2_r13_plus_r12_ASR_9_PreIndex[] = {
  0xcc, 0x24, 0xad, 0xe7 // str al r2 r13 plus r12 ASR 9 PreIndex
};
const byte kInstruction_str_al_r2_r0_minus_r1_ASR_26_PostIndex[] = {
  0x41, 0x2d, 0x00, 0xe6 // str al r2 r0 minus r1 ASR 26 PostIndex
};
const byte kInstruction_str_al_r5_r14_plus_r11_ASR_10_PreIndex[] = {
  0x4b, 0x55, 0xae, 0xe7 // str al r5 r14 plus r11 ASR 10 PreIndex
};
const byte kInstruction_str_al_r5_r3_minus_r8_LSR_10_PostIndex[] = {
  0x28, 0x55, 0x03, 0xe6 // str al r5 r3 minus r8 LSR 10 PostIndex
};
const byte kInstruction_str_al_r12_r3_minus_r5_LSR_26_PreIndex[] = {
  0x25, 0xcd, 0x23, 0xe7 // str al r12 r3 minus r5 LSR 26 PreIndex
};
const byte kInstruction_str_al_r3_r5_minus_r10_ASR_8_PreIndex[] = {
  0x4a, 0x34, 0x25, 0xe7 // str al r3 r5 minus r10 ASR 8 PreIndex
};
const byte kInstruction_str_al_r1_r14_minus_r12_ASR_8_PreIndex[] = {
  0x4c, 0x14, 0x2e, 0xe7 // str al r1 r14 minus r12 ASR 8 PreIndex
};
const byte kInstruction_str_al_r7_r8_minus_r9_ASR_9_PreIndex[] = {
  0xc9, 0x74, 0x28, 0xe7 // str al r7 r8 minus r9 ASR 9 PreIndex
};
const byte kInstruction_str_al_r7_r2_plus_r3_LSR_23_PostIndex[] = {
  0xa3, 0x7b, 0x82, 0xe6 // str al r7 r2 plus r3 LSR 23 PostIndex
};
const byte kInstruction_str_al_r13_r12_plus_r13_ASR_2_PreIndex[] = {
  0x4d, 0xd1, 0xac, 0xe7 // str al r13 r12 plus r13 ASR 2 PreIndex
};
const byte kInstruction_str_al_r0_r6_plus_r2_ASR_32_PreIndex[] = {
  0x42, 0x00, 0xa6, 0xe7 // str al r0 r6 plus r2 ASR 32 PreIndex
};
const byte kInstruction_str_al_r0_r14_plus_r1_LSR_16_PreIndex[] = {
  0x21, 0x08, 0xae, 0xe7 // str al r0 r14 plus r1 LSR 16 PreIndex
};
const byte kInstruction_str_al_r9_r0_minus_r13_LSR_18_PreIndex[] = {
  0x2d, 0x99, 0x20, 0xe7 // str al r9 r0 minus r13 LSR 18 PreIndex
};
const byte kInstruction_str_al_r7_r14_minus_r4_LSR_10_PostIndex[] = {
  0x24, 0x75, 0x0e, 0xe6 // str al r7 r14 minus r4 LSR 10 PostIndex
};
const byte kInstruction_str_al_r2_r4_plus_r3_LSR_10_PostIndex[] = {
  0x23, 0x25, 0x84, 0xe6 // str al r2 r4 plus r3 LSR 10 PostIndex
};
const byte kInstruction_str_al_r4_r10_minus_r3_ASR_8_PostIndex[] = {
  0x43, 0x44, 0x0a, 0xe6 // str al r4 r10 minus r3 ASR 8 PostIndex
};
const byte kInstruction_str_al_r0_r7_plus_r13_ASR_22_PostIndex[] = {
  0x4d, 0x0b, 0x87, 0xe6 // str al r0 r7 plus r13 ASR 22 PostIndex
};
const byte kInstruction_str_al_r9_r0_plus_r10_LSR_29_PreIndex[] = {
  0xaa, 0x9e, 0xa0, 0xe7 // str al r9 r0 plus r10 LSR 29 PreIndex
};
const byte kInstruction_str_al_r6_r0_plus_r7_ASR_6_PostIndex[] = {
  0x47, 0x63, 0x80, 0xe6 // str al r6 r0 plus r7 ASR 6 PostIndex
};
const byte kInstruction_str_al_r14_r11_minus_r2_ASR_6_PreIndex[] = {
  0x42, 0xe3, 0x2b, 0xe7 // str al r14 r11 minus r2 ASR 6 PreIndex
};
const byte kInstruction_str_al_r6_r9_minus_r9_ASR_25_PostIndex[] = {
  0xc9, 0x6c, 0x09, 0xe6 // str al r6 r9 minus r9 ASR 25 PostIndex
};
const byte kInstruction_str_al_r13_r1_minus_r11_LSR_7_PreIndex[] = {
  0xab, 0xd3, 0x21, 0xe7 // str al r13 r1 minus r11 LSR 7 PreIndex
};
const byte kInstruction_str_al_r12_r9_minus_r6_ASR_21_PostIndex[] = {
  0xc6, 0xca, 0x09, 0xe6 // str al r12 r9 minus r6 ASR 21 PostIndex
};
const byte kInstruction_str_al_r8_r6_minus_r11_ASR_1_PreIndex[] = {
  0xcb, 0x80, 0x26, 0xe7 // str al r8 r6 minus r11 ASR 1 PreIndex
};
const byte kInstruction_str_al_r0_r13_minus_r11_LSR_20_PostIndex[] = {
  0x2b, 0x0a, 0x0d, 0xe6 // str al r0 r13 minus r11 LSR 20 PostIndex
};
const byte kInstruction_str_al_r12_r3_minus_r8_ASR_25_PreIndex[] = {
  0xc8, 0xcc, 0x23, 0xe7 // str al r12 r3 minus r8 ASR 25 PreIndex
};
const byte kInstruction_str_al_r10_r7_plus_r8_ASR_16_PostIndex[] = {
  0x48, 0xa8, 0x87, 0xe6 // str al r10 r7 plus r8 ASR 16 PostIndex
};
const byte kInstruction_str_al_r10_r4_plus_r13_LSR_27_PreIndex[] = {
  0xad, 0xad, 0xa4, 0xe7 // str al r10 r4 plus r13 LSR 27 PreIndex
};
const byte kInstruction_str_al_r8_r10_plus_r7_ASR_29_PreIndex[] = {
  0xc7, 0x8e, 0xaa, 0xe7 // str al r8 r10 plus r7 ASR 29 PreIndex
};
const byte kInstruction_str_al_r2_r9_minus_r1_LSR_30_PostIndex[] = {
  0x21, 0x2f, 0x09, 0xe6 // str al r2 r9 minus r1 LSR 30 PostIndex
};
const byte kInstruction_str_al_r9_r2_plus_r1_LSR_9_PreIndex[] = {
  0xa1, 0x94, 0xa2, 0xe7 // str al r9 r2 plus r1 LSR 9 PreIndex
};
const byte kInstruction_str_al_r13_r4_plus_r4_ASR_20_PreIndex[] = {
  0x44, 0xda, 0xa4, 0xe7 // str al r13 r4 plus r4 ASR 20 PreIndex
};
const byte kInstruction_str_al_r3_r1_minus_r7_ASR_32_PostIndex[] = {
  0x47, 0x30, 0x01, 0xe6 // str al r3 r1 minus r7 ASR 32 PostIndex
};
const byte kInstruction_str_al_r0_r5_minus_r5_ASR_9_PreIndex[] = {
  0xc5, 0x04, 0x25, 0xe7 // str al r0 r5 minus r5 ASR 9 PreIndex
};
const byte kInstruction_str_al_r11_r6_minus_r9_LSR_23_PostIndex[] = {
  0xa9, 0xbb, 0x06, 0xe6 // str al r11 r6 minus r9 LSR 23 PostIndex
};
const byte kInstruction_str_al_r12_r8_plus_r4_ASR_25_PostIndex[] = {
  0xc4, 0xcc, 0x88, 0xe6 // str al r12 r8 plus r4 ASR 25 PostIndex
};
const byte kInstruction_str_al_r3_r5_minus_r5_LSR_26_PostIndex[] = {
  0x25, 0x3d, 0x05, 0xe6 // str al r3 r5 minus r5 LSR 26 PostIndex
};
const byte kInstruction_str_al_r1_r0_minus_r0_LSR_21_PostIndex[] = {
  0xa0, 0x1a, 0x00, 0xe6 // str al r1 r0 minus r0 LSR 21 PostIndex
};
const byte kInstruction_str_al_r13_r8_plus_r9_LSR_6_PostIndex[] = {
  0x29, 0xd3, 0x88, 0xe6 // str al r13 r8 plus r9 LSR 6 PostIndex
};
const byte kInstruction_str_al_r1_r7_plus_r6_LSR_28_PreIndex[] = {
  0x26, 0x1e, 0xa7, 0xe7 // str al r1 r7 plus r6 LSR 28 PreIndex
};
const byte kInstruction_str_al_r14_r4_plus_r13_LSR_14_PostIndex[] = {
  0x2d, 0xe7, 0x84, 0xe6 // str al r14 r4 plus r13 LSR 14 PostIndex
};
const byte kInstruction_str_al_r14_r3_plus_r11_ASR_21_PreIndex[] = {
  0xcb, 0xea, 0xa3, 0xe7 // str al r14 r3 plus r11 ASR 21 PreIndex
};
const byte kInstruction_str_al_r10_r6_plus_r11_LSR_6_PreIndex[] = {
  0x2b, 0xa3, 0xa6, 0xe7 // str al r10 r6 plus r11 LSR 6 PreIndex
};
const byte kInstruction_str_al_r13_r7_plus_r11_ASR_25_PreIndex[] = {
  0xcb, 0xdc, 0xa7, 0xe7 // str al r13 r7 plus r11 ASR 25 PreIndex
};
const byte kInstruction_str_al_r3_r2_minus_r8_LSR_13_PreIndex[] = {
  0xa8, 0x36, 0x22, 0xe7 // str al r3 r2 minus r8 LSR 13 PreIndex
};
const byte kInstruction_str_al_r8_r14_minus_r13_LSR_4_PostIndex[] = {
  0x2d, 0x82, 0x0e, 0xe6 // str al r8 r14 minus r13 LSR 4 PostIndex
};
const byte kInstruction_str_al_r2_r9_minus_r2_LSR_26_PostIndex[] = {
  0x22, 0x2d, 0x09, 0xe6 // str al r2 r9 minus r2 LSR 26 PostIndex
};
const byte kInstruction_str_al_r11_r14_minus_r13_ASR_27_PostIndex[] = {
  0xcd, 0xbd, 0x0e, 0xe6 // str al r11 r14 minus r13 ASR 27 PostIndex
};
const byte kInstruction_str_al_r2_r8_minus_r8_LSR_6_PreIndex[] = {
  0x28, 0x23, 0x28, 0xe7 // str al r2 r8 minus r8 LSR 6 PreIndex
};
const byte kInstruction_str_al_r10_r11_minus_r2_ASR_12_PreIndex[] = {
  0x42, 0xa6, 0x2b, 0xe7 // str al r10 r11 minus r2 ASR 12 PreIndex
};
const byte kInstruction_str_al_r11_r9_plus_r8_ASR_2_PreIndex[] = {
  0x48, 0xb1, 0xa9, 0xe7 // str al r11 r9 plus r8 ASR 2 PreIndex
};
const byte kInstruction_str_al_r11_r7_minus_r12_ASR_7_PreIndex[] = {
  0xcc, 0xb3, 0x27, 0xe7 // str al r11 r7 minus r12 ASR 7 PreIndex
};
const byte kInstruction_str_al_r8_r7_plus_r8_ASR_3_PreIndex[] = {
  0xc8, 0x81, 0xa7, 0xe7 // str al r8 r7 plus r8 ASR 3 PreIndex
};
const byte kInstruction_str_al_r5_r12_minus_r2_LSR_28_PostIndex[] = {
  0x22, 0x5e, 0x0c, 0xe6 // str al r5 r12 minus r2 LSR 28 PostIndex
};
const byte kInstruction_str_al_r4_r3_minus_r13_ASR_15_PreIndex[] = {
  0xcd, 0x47, 0x23, 0xe7 // str al r4 r3 minus r13 ASR 15 PreIndex
};
const byte kInstruction_str_al_r7_r11_minus_r11_LSR_17_PostIndex[] = {
  0xab, 0x78, 0x0b, 0xe6 // str al r7 r11 minus r11 LSR 17 PostIndex
};
const byte kInstruction_str_al_r7_r1_minus_r10_LSR_18_PostIndex[] = {
  0x2a, 0x79, 0x01, 0xe6 // str al r7 r1 minus r10 LSR 18 PostIndex
};
const byte kInstruction_str_al_r1_r11_minus_r2_ASR_25_PreIndex[] = {
  0xc2, 0x1c, 0x2b, 0xe7 // str al r1 r11 minus r2 ASR 25 PreIndex
};
const byte kInstruction_str_al_r4_r0_minus_r4_ASR_19_PostIndex[] = {
  0xc4, 0x49, 0x00, 0xe6 // str al r4 r0 minus r4 ASR 19 PostIndex
};
const byte kInstruction_str_al_r4_r12_minus_r10_LSR_32_PostIndex[] = {
  0x2a, 0x40, 0x0c, 0xe6 // str al r4 r12 minus r10 LSR 32 PostIndex
};
const byte kInstruction_str_al_r0_r11_plus_r9_ASR_31_PostIndex[] = {
  0xc9, 0x0f, 0x8b, 0xe6 // str al r0 r11 plus r9 ASR 31 PostIndex
};
const byte kInstruction_str_al_r13_r11_plus_r14_ASR_18_PostIndex[] = {
  0x4e, 0xd9, 0x8b, 0xe6 // str al r13 r11 plus r14 ASR 18 PostIndex
};
const byte kInstruction_str_al_r4_r6_plus_r11_LSR_2_PostIndex[] = {
  0x2b, 0x41, 0x86, 0xe6 // str al r4 r6 plus r11 LSR 2 PostIndex
};
const byte kInstruction_str_al_r11_r12_plus_r13_LSR_15_PreIndex[] = {
  0xad, 0xb7, 0xac, 0xe7 // str al r11 r12 plus r13 LSR 15 PreIndex
};
const byte kInstruction_str_al_r9_r10_minus_r3_LSR_10_PreIndex[] = {
  0x23, 0x95, 0x2a, 0xe7 // str al r9 r10 minus r3 LSR 10 PreIndex
};
const byte kInstruction_str_al_r7_r10_plus_r12_LSR_6_PostIndex[] = {
  0x2c, 0x73, 0x8a, 0xe6 // str al r7 r10 plus r12 LSR 6 PostIndex
};
const byte kInstruction_str_al_r0_r2_plus_r0_LSR_1_PreIndex[] = {
  0xa0, 0x00, 0xa2, 0xe7 // str al r0 r2 plus r0 LSR 1 PreIndex
};
const byte kInstruction_str_al_r1_r5_minus_r12_LSR_14_PreIndex[] = {
  0x2c, 0x17, 0x25, 0xe7 // str al r1 r5 minus r12 LSR 14 PreIndex
};
const byte kInstruction_str_al_r6_r2_minus_r3_LSR_16_PostIndex[] = {
  0x23, 0x68, 0x02, 0xe6 // str al r6 r2 minus r3 LSR 16 PostIndex
};
const byte kInstruction_str_al_r7_r5_plus_r3_LSR_25_PostIndex[] = {
  0xa3, 0x7c, 0x85, 0xe6 // str al r7 r5 plus r3 LSR 25 PostIndex
};
const byte kInstruction_str_al_r5_r10_plus_r1_LSR_6_PostIndex[] = {
  0x21, 0x53, 0x8a, 0xe6 // str al r5 r10 plus r1 LSR 6 PostIndex
};
const byte kInstruction_str_al_r6_r3_minus_r4_ASR_10_PreIndex[] = {
  0x44, 0x65, 0x23, 0xe7 // str al r6 r3 minus r4 ASR 10 PreIndex
};
const byte kInstruction_str_al_r2_r10_plus_r9_ASR_13_PostIndex[] = {
  0xc9, 0x26, 0x8a, 0xe6 // str al r2 r10 plus r9 ASR 13 PostIndex
};
const byte kInstruction_str_al_r6_r0_plus_r4_LSR_16_PostIndex[] = {
  0x24, 0x68, 0x80, 0xe6 // str al r6 r0 plus r4 LSR 16 PostIndex
};
const byte kInstruction_str_al_r3_r0_minus_r4_LSR_19_PostIndex[] = {
  0xa4, 0x39, 0x00, 0xe6 // str al r3 r0 minus r4 LSR 19 PostIndex
};
const byte kInstruction_str_al_r8_r2_minus_r6_LSR_11_PostIndex[] = {
  0xa6, 0x85, 0x02, 0xe6 // str al r8 r2 minus r6 LSR 11 PostIndex
};
const byte kInstruction_str_al_r0_r6_plus_r14_LSR_21_PreIndex[] = {
  0xae, 0x0a, 0xa6, 0xe7 // str al r0 r6 plus r14 LSR 21 PreIndex
};
const byte kInstruction_str_al_r9_r12_plus_r5_ASR_20_PostIndex[] = {
  0x45, 0x9a, 0x8c, 0xe6 // str al r9 r12 plus r5 ASR 20 PostIndex
};
const byte kInstruction_str_al_r9_r14_minus_r11_ASR_17_PreIndex[] = {
  0xcb, 0x98, 0x2e, 0xe7 // str al r9 r14 minus r11 ASR 17 PreIndex
};
const byte kInstruction_str_al_r1_r8_minus_r14_LSR_22_PostIndex[] = {
  0x2e, 0x1b, 0x08, 0xe6 // str al r1 r8 minus r14 LSR 22 PostIndex
};
const byte kInstruction_str_al_r5_r3_minus_r5_LSR_27_PreIndex[] = {
  0xa5, 0x5d, 0x23, 0xe7 // str al r5 r3 minus r5 LSR 27 PreIndex
};
const byte kInstruction_str_al_r14_r9_plus_r4_LSR_31_PreIndex[] = {
  0xa4, 0xef, 0xa9, 0xe7 // str al r14 r9 plus r4 LSR 31 PreIndex
};
const byte kInstruction_str_al_r8_r12_plus_r3_LSR_12_PreIndex[] = {
  0x23, 0x86, 0xac, 0xe7 // str al r8 r12 plus r3 LSR 12 PreIndex
};
const byte kInstruction_str_al_r6_r4_minus_r9_ASR_16_PostIndex[] = {
  0x49, 0x68, 0x04, 0xe6 // str al r6 r4 minus r9 ASR 16 PostIndex
};
const byte kInstruction_str_al_r4_r6_plus_r2_LSR_11_PreIndex[] = {
  0xa2, 0x45, 0xa6, 0xe7 // str al r4 r6 plus r2 LSR 11 PreIndex
};
const byte kInstruction_str_al_r11_r7_plus_r0_ASR_11_PostIndex[] = {
  0xc0, 0xb5, 0x87, 0xe6 // str al r11 r7 plus r0 ASR 11 PostIndex
};
const byte kInstruction_str_al_r8_r11_minus_r5_LSR_25_PostIndex[] = {
  0xa5, 0x8c, 0x0b, 0xe6 // str al r8 r11 minus r5 LSR 25 PostIndex
};
const byte kInstruction_str_al_r3_r14_minus_r12_LSR_5_PreIndex[] = {
  0xac, 0x32, 0x2e, 0xe7 // str al r3 r14 minus r12 LSR 5 PreIndex
};
const byte kInstruction_str_al_r10_r7_minus_r3_ASR_23_PostIndex[] = {
  0xc3, 0xab, 0x07, 0xe6 // str al r10 r7 minus r3 ASR 23 PostIndex
};
const byte kInstruction_str_al_r12_r5_minus_r6_ASR_24_PreIndex[] = {
  0x46, 0xcc, 0x25, 0xe7 // str al r12 r5 minus r6 ASR 24 PreIndex
};
const byte kInstruction_str_al_r8_r12_minus_r2_ASR_30_PreIndex[] = {
  0x42, 0x8f, 0x2c, 0xe7 // str al r8 r12 minus r2 ASR 30 PreIndex
};
const byte kInstruction_str_al_r1_r9_minus_r12_LSR_8_PostIndex[] = {
  0x2c, 0x14, 0x09, 0xe6 // str al r1 r9 minus r12 LSR 8 PostIndex
};
const byte kInstruction_str_al_r5_r11_plus_r13_LSR_3_PostIndex[] = {
  0xad, 0x51, 0x8b, 0xe6 // str al r5 r11 plus r13 LSR 3 PostIndex
};
const byte kInstruction_str_al_r14_r7_plus_r13_LSR_21_PostIndex[] = {
  0xad, 0xea, 0x87, 0xe6 // str al r14 r7 plus r13 LSR 21 PostIndex
};
const byte kInstruction_str_al_r9_r4_minus_r10_LSR_15_PreIndex[] = {
  0xaa, 0x97, 0x24, 0xe7 // str al r9 r4 minus r10 LSR 15 PreIndex
};
const byte kInstruction_str_al_r11_r9_plus_r1_LSR_23_PostIndex[] = {
  0xa1, 0xbb, 0x89, 0xe6 // str al r11 r9 plus r1 LSR 23 PostIndex
};
const byte kInstruction_str_al_r3_r9_minus_r7_ASR_15_PreIndex[] = {
  0xc7, 0x37, 0x29, 0xe7 // str al r3 r9 minus r7 ASR 15 PreIndex
};
const byte kInstruction_str_al_r0_r4_minus_r8_ASR_13_PostIndex[] = {
  0xc8, 0x06, 0x04, 0xe6 // str al r0 r4 minus r8 ASR 13 PostIndex
};
const byte kInstruction_str_al_r12_r11_minus_r12_LSR_31_PostIndex[] = {
  0xac, 0xcf, 0x0b, 0xe6 // str al r12 r11 minus r12 LSR 31 PostIndex
};
const byte kInstruction_str_al_r9_r12_minus_r10_LSR_17_PostIndex[] = {
  0xaa, 0x98, 0x0c, 0xe6 // str al r9 r12 minus r10 LSR 17 PostIndex
};
const byte kInstruction_str_al_r6_r10_minus_r6_LSR_8_PostIndex[] = {
  0x26, 0x64, 0x0a, 0xe6 // str al r6 r10 minus r6 LSR 8 PostIndex
};
const byte kInstruction_str_al_r4_r1_plus_r9_ASR_22_PostIndex[] = {
  0x49, 0x4b, 0x81, 0xe6 // str al r4 r1 plus r9 ASR 22 PostIndex
};
const byte kInstruction_str_al_r1_r11_minus_r12_ASR_21_PostIndex[] = {
  0xcc, 0x1a, 0x0b, 0xe6 // str al r1 r11 minus r12 ASR 21 PostIndex
};
const byte kInstruction_str_al_r2_r7_plus_r4_ASR_14_PreIndex[] = {
  0x44, 0x27, 0xa7, 0xe7 // str al r2 r7 plus r4 ASR 14 PreIndex
};
const byte kInstruction_str_al_r2_r4_minus_r9_ASR_16_PreIndex[] = {
  0x49, 0x28, 0x24, 0xe7 // str al r2 r4 minus r9 ASR 16 PreIndex
};
const byte kInstruction_str_al_r9_r10_plus_r4_LSR_30_PreIndex[] = {
  0x24, 0x9f, 0xaa, 0xe7 // str al r9 r10 plus r4 LSR 30 PreIndex
};
const byte kInstruction_str_al_r6_r1_plus_r3_LSR_32_PreIndex[] = {
  0x23, 0x60, 0xa1, 0xe7 // str al r6 r1 plus r3 LSR 32 PreIndex
};
const byte kInstruction_str_al_r0_r9_plus_r14_ASR_29_PostIndex[] = {
  0xce, 0x0e, 0x89, 0xe6 // str al r0 r9 plus r14 ASR 29 PostIndex
};
const TestResult kReferencestr[] = {
  {
    ARRAY_SIZE(kInstruction_str_pl_r8_r11_plus_r6_LSR_1_Offset),
    kInstruction_str_pl_r8_r11_plus_r6_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_le_r4_r8_plus_r5_LSR_1_Offset),
    kInstruction_str_le_r4_r8_plus_r5_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_vs_r2_r6_plus_r14_LSR_1_Offset),
    kInstruction_str_vs_r2_r6_plus_r14_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ls_r1_r7_plus_r8_LSR_1_Offset),
    kInstruction_str_ls_r1_r7_plus_r8_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ge_r14_r6_plus_r14_LSR_1_Offset),
    kInstruction_str_ge_r14_r6_plus_r14_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cs_r7_r0_plus_r7_LSR_1_Offset),
    kInstruction_str_cs_r7_r0_plus_r7_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ge_r11_r0_plus_r9_LSR_1_Offset),
    kInstruction_str_ge_r11_r0_plus_r9_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_eq_r7_r10_plus_r4_LSR_1_Offset),
    kInstruction_str_eq_r7_r10_plus_r4_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r2_plus_r3_LSR_1_Offset),
    kInstruction_str_al_r9_r2_plus_r3_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cc_r11_r10_plus_r6_LSR_1_Offset),
    kInstruction_str_cc_r11_r10_plus_r6_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_lt_r9_r6_plus_r1_LSR_1_Offset),
    kInstruction_str_lt_r9_r6_plus_r1_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ge_r4_r0_plus_r4_LSR_1_Offset),
    kInstruction_str_ge_r4_r0_plus_r4_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r13_plus_r14_LSR_1_Offset),
    kInstruction_str_al_r9_r13_plus_r14_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cc_r9_r0_plus_r7_LSR_1_Offset),
    kInstruction_str_cc_r9_r0_plus_r7_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cs_r11_r7_plus_r8_LSR_1_Offset),
    kInstruction_str_cs_r11_r7_plus_r8_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_eq_r8_r0_plus_r8_LSR_1_Offset),
    kInstruction_str_eq_r8_r0_plus_r8_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_hi_r9_r10_plus_r8_LSR_1_Offset),
    kInstruction_str_hi_r9_r10_plus_r8_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_le_r8_r0_plus_r0_LSR_1_Offset),
    kInstruction_str_le_r8_r0_plus_r0_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cs_r4_r5_plus_r2_LSR_1_Offset),
    kInstruction_str_cs_r4_r5_plus_r2_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_eq_r14_r11_plus_r8_LSR_1_Offset),
    kInstruction_str_eq_r14_r11_plus_r8_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cs_r7_r6_plus_r14_LSR_1_Offset),
    kInstruction_str_cs_r7_r6_plus_r14_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ne_r0_r9_plus_r7_LSR_1_Offset),
    kInstruction_str_ne_r0_r9_plus_r7_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_le_r7_r3_plus_r11_LSR_1_Offset),
    kInstruction_str_le_r7_r3_plus_r11_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_gt_r7_r9_plus_r9_LSR_1_Offset),
    kInstruction_str_gt_r7_r9_plus_r9_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_le_r9_r12_plus_r9_LSR_1_Offset),
    kInstruction_str_le_r9_r12_plus_r9_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_pl_r2_r11_plus_r14_LSR_1_Offset),
    kInstruction_str_pl_r2_r11_plus_r14_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cs_r1_r7_plus_r2_LSR_1_Offset),
    kInstruction_str_cs_r1_r7_plus_r2_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r1_plus_r12_LSR_1_Offset),
    kInstruction_str_al_r5_r1_plus_r12_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_eq_r10_r13_plus_r12_LSR_1_Offset),
    kInstruction_str_eq_r10_r13_plus_r12_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_eq_r10_r11_plus_r3_LSR_1_Offset),
    kInstruction_str_eq_r10_r11_plus_r3_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r12_plus_r0_LSR_1_Offset),
    kInstruction_str_al_r9_r12_plus_r0_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ls_r3_r14_plus_r1_LSR_1_Offset),
    kInstruction_str_ls_r3_r14_plus_r1_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_le_r13_r4_plus_r9_LSR_1_Offset),
    kInstruction_str_le_r13_r4_plus_r9_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ls_r3_r14_plus_r3_LSR_1_Offset),
    kInstruction_str_ls_r3_r14_plus_r3_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cc_r7_r4_plus_r4_LSR_1_Offset),
    kInstruction_str_cc_r7_r4_plus_r4_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ls_r5_r9_plus_r5_LSR_1_Offset),
    kInstruction_str_ls_r5_r9_plus_r5_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ge_r6_r4_plus_r13_LSR_1_Offset),
    kInstruction_str_ge_r6_r4_plus_r13_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r11_plus_r12_LSR_1_Offset),
    kInstruction_str_al_r1_r11_plus_r12_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_le_r6_r11_plus_r3_LSR_1_Offset),
    kInstruction_str_le_r6_r11_plus_r3_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r6_plus_r13_LSR_1_Offset),
    kInstruction_str_al_r6_r6_plus_r13_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_gt_r14_r11_plus_r2_LSR_1_Offset),
    kInstruction_str_gt_r14_r11_plus_r2_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_vs_r5_r8_plus_r12_LSR_1_Offset),
    kInstruction_str_vs_r5_r8_plus_r12_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ls_r8_r12_plus_r6_LSR_1_Offset),
    kInstruction_str_ls_r8_r12_plus_r6_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_vs_r6_r12_plus_r14_LSR_1_Offset),
    kInstruction_str_vs_r6_r12_plus_r14_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_pl_r7_r2_plus_r7_LSR_1_Offset),
    kInstruction_str_pl_r7_r2_plus_r7_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_gt_r5_r4_plus_r13_LSR_1_Offset),
    kInstruction_str_gt_r5_r4_plus_r13_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_hi_r2_r7_plus_r12_LSR_1_Offset),
    kInstruction_str_hi_r2_r7_plus_r12_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ne_r2_r3_plus_r10_LSR_1_Offset),
    kInstruction_str_ne_r2_r3_plus_r10_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_lt_r13_r2_plus_r6_LSR_1_Offset),
    kInstruction_str_lt_r13_r2_plus_r6_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_hi_r14_r10_plus_r11_LSR_1_Offset),
    kInstruction_str_hi_r14_r10_plus_r11_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_hi_r7_r10_plus_r12_LSR_1_Offset),
    kInstruction_str_hi_r7_r10_plus_r12_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cs_r3_r12_plus_r2_LSR_1_Offset),
    kInstruction_str_cs_r3_r12_plus_r2_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ls_r11_r5_plus_r1_LSR_1_Offset),
    kInstruction_str_ls_r11_r5_plus_r1_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_hi_r5_r2_plus_r11_LSR_1_Offset),
    kInstruction_str_hi_r5_r2_plus_r11_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cc_r11_r8_plus_r6_LSR_1_Offset),
    kInstruction_str_cc_r11_r8_plus_r6_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_lt_r11_r3_plus_r9_LSR_1_Offset),
    kInstruction_str_lt_r11_r3_plus_r9_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ne_r11_r9_plus_r8_LSR_1_Offset),
    kInstruction_str_ne_r11_r9_plus_r8_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_lt_r7_r0_plus_r14_LSR_1_Offset),
    kInstruction_str_lt_r7_r0_plus_r14_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ge_r12_r3_plus_r1_LSR_1_Offset),
    kInstruction_str_ge_r12_r3_plus_r1_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_le_r0_r8_plus_r13_LSR_1_Offset),
    kInstruction_str_le_r0_r8_plus_r13_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_vc_r5_r13_plus_r7_LSR_1_Offset),
    kInstruction_str_vc_r5_r13_plus_r7_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ge_r7_r5_plus_r11_LSR_1_Offset),
    kInstruction_str_ge_r7_r5_plus_r11_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ge_r10_r9_plus_r3_LSR_1_Offset),
    kInstruction_str_ge_r10_r9_plus_r3_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_vs_r9_r5_plus_r2_LSR_1_Offset),
    kInstruction_str_vs_r9_r5_plus_r2_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_hi_r2_r10_plus_r4_LSR_1_Offset),
    kInstruction_str_hi_r2_r10_plus_r4_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ge_r1_r11_plus_r1_LSR_1_Offset),
    kInstruction_str_ge_r1_r11_plus_r1_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ls_r9_r14_plus_r12_LSR_1_Offset),
    kInstruction_str_ls_r9_r14_plus_r12_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_mi_r1_r9_plus_r4_LSR_1_Offset),
    kInstruction_str_mi_r1_r9_plus_r4_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_mi_r7_r10_plus_r4_LSR_1_Offset),
    kInstruction_str_mi_r7_r10_plus_r4_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_gt_r6_r2_plus_r5_LSR_1_Offset),
    kInstruction_str_gt_r6_r2_plus_r5_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_eq_r10_r8_plus_r11_LSR_1_Offset),
    kInstruction_str_eq_r10_r8_plus_r11_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_le_r10_r1_plus_r7_LSR_1_Offset),
    kInstruction_str_le_r10_r1_plus_r7_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_pl_r8_r14_plus_r8_LSR_1_Offset),
    kInstruction_str_pl_r8_r14_plus_r8_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_gt_r5_r1_plus_r3_LSR_1_Offset),
    kInstruction_str_gt_r5_r1_plus_r3_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cs_r10_r4_plus_r13_LSR_1_Offset),
    kInstruction_str_cs_r10_r4_plus_r13_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_mi_r10_r5_plus_r2_LSR_1_Offset),
    kInstruction_str_mi_r10_r5_plus_r2_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cc_r6_r0_plus_r11_LSR_1_Offset),
    kInstruction_str_cc_r6_r0_plus_r11_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_eq_r3_r9_plus_r2_LSR_1_Offset),
    kInstruction_str_eq_r3_r9_plus_r2_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r0_plus_r3_LSR_1_Offset),
    kInstruction_str_al_r1_r0_plus_r3_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_mi_r1_r0_plus_r13_LSR_1_Offset),
    kInstruction_str_mi_r1_r0_plus_r13_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cc_r4_r4_plus_r5_LSR_1_Offset),
    kInstruction_str_cc_r4_r4_plus_r5_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r6_plus_r11_LSR_1_Offset),
    kInstruction_str_al_r2_r6_plus_r11_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ls_r1_r5_plus_r4_LSR_1_Offset),
    kInstruction_str_ls_r1_r5_plus_r4_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_eq_r0_r3_plus_r4_LSR_1_Offset),
    kInstruction_str_eq_r0_r3_plus_r4_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_lt_r7_r11_plus_r10_LSR_1_Offset),
    kInstruction_str_lt_r7_r11_plus_r10_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_vc_r3_r0_plus_r13_LSR_1_Offset),
    kInstruction_str_vc_r3_r0_plus_r13_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ls_r3_r3_plus_r7_LSR_1_Offset),
    kInstruction_str_ls_r3_r3_plus_r7_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r14_plus_r3_LSR_1_Offset),
    kInstruction_str_al_r5_r14_plus_r3_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ne_r2_r13_plus_r9_LSR_1_Offset),
    kInstruction_str_ne_r2_r13_plus_r9_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_lt_r6_r2_plus_r11_LSR_1_Offset),
    kInstruction_str_lt_r6_r2_plus_r11_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_vc_r9_r2_plus_r13_LSR_1_Offset),
    kInstruction_str_vc_r9_r2_plus_r13_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_ne_r5_r9_plus_r6_LSR_1_Offset),
    kInstruction_str_ne_r5_r9_plus_r6_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_vc_r4_r9_plus_r8_LSR_1_Offset),
    kInstruction_str_vc_r4_r9_plus_r8_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_cc_r6_r9_plus_r2_LSR_1_Offset),
    kInstruction_str_cc_r6_r9_plus_r2_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_vs_r8_r7_plus_r13_LSR_1_Offset),
    kInstruction_str_vs_r8_r7_plus_r13_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_pl_r14_r14_plus_r7_LSR_1_Offset),
    kInstruction_str_pl_r14_r14_plus_r7_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_eq_r8_r2_plus_r6_LSR_1_Offset),
    kInstruction_str_eq_r8_r2_plus_r6_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r13_plus_r13_LSR_1_Offset),
    kInstruction_str_al_r13_r13_plus_r13_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_gt_r12_r14_plus_r5_LSR_1_Offset),
    kInstruction_str_gt_r12_r14_plus_r5_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_lt_r14_r7_plus_r1_LSR_1_Offset),
    kInstruction_str_lt_r14_r7_plus_r1_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r12_LSR_20_Offset),
    kInstruction_str_al_r0_r2_plus_r12_LSR_20_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_plus_r4_LSR_19_Offset),
    kInstruction_str_al_r0_r4_plus_r4_LSR_19_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r13_plus_r3_LSR_28_Offset),
    kInstruction_str_al_r0_r13_plus_r3_LSR_28_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_minus_r1_ASR_1_Offset),
    kInstruction_str_al_r0_r14_minus_r1_ASR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_plus_r8_ASR_4_Offset),
    kInstruction_str_al_r0_r4_plus_r8_ASR_4_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_minus_r1_ASR_26_Offset),
    kInstruction_str_al_r0_r9_minus_r1_ASR_26_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_minus_r9_LSR_20_Offset),
    kInstruction_str_al_r0_r6_minus_r9_LSR_20_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r13_ASR_13_Offset),
    kInstruction_str_al_r0_r0_plus_r13_ASR_13_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_minus_r7_ASR_23_Offset),
    kInstruction_str_al_r0_r9_minus_r7_ASR_23_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r13_minus_r5_ASR_25_Offset),
    kInstruction_str_al_r0_r13_minus_r5_ASR_25_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r7_plus_r5_ASR_14_Offset),
    kInstruction_str_al_r0_r7_plus_r5_ASR_14_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_minus_r6_LSR_28_Offset),
    kInstruction_str_al_r0_r3_minus_r6_LSR_28_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r7_plus_r5_LSR_27_Offset),
    kInstruction_str_al_r0_r7_plus_r5_LSR_27_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_minus_r5_ASR_19_Offset),
    kInstruction_str_al_r0_r2_minus_r5_ASR_19_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_minus_r7_ASR_18_Offset),
    kInstruction_str_al_r0_r10_minus_r7_ASR_18_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_minus_r1_ASR_10_Offset),
    kInstruction_str_al_r0_r0_minus_r1_ASR_10_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r8_plus_r14_ASR_17_Offset),
    kInstruction_str_al_r0_r8_plus_r14_ASR_17_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r7_LSR_14_Offset),
    kInstruction_str_al_r0_r4_minus_r7_LSR_14_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_minus_r4_ASR_13_Offset),
    kInstruction_str_al_r0_r14_minus_r4_ASR_13_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_minus_r2_ASR_6_Offset),
    kInstruction_str_al_r0_r9_minus_r2_ASR_6_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_plus_r1_LSR_19_Offset),
    kInstruction_str_al_r0_r12_plus_r1_LSR_19_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_plus_r4_LSR_4_Offset),
    kInstruction_str_al_r0_r9_plus_r4_LSR_4_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_plus_r14_LSR_10_Offset),
    kInstruction_str_al_r0_r6_plus_r14_LSR_10_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_minus_r6_LSR_25_Offset),
    kInstruction_str_al_r0_r6_minus_r6_LSR_25_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_plus_r8_ASR_14_Offset),
    kInstruction_str_al_r0_r12_plus_r8_ASR_14_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r7_LSR_32_Offset),
    kInstruction_str_al_r0_r0_plus_r7_LSR_32_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_minus_r2_LSR_23_Offset),
    kInstruction_str_al_r0_r10_minus_r2_LSR_23_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r1_ASR_1_Offset),
    kInstruction_str_al_r0_r2_plus_r1_ASR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r13_minus_r11_LSR_2_Offset),
    kInstruction_str_al_r0_r13_minus_r11_LSR_2_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r8_plus_r11_LSR_10_Offset),
    kInstruction_str_al_r0_r8_plus_r11_LSR_10_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r8_ASR_1_Offset),
    kInstruction_str_al_r0_r0_plus_r8_ASR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r6_ASR_14_Offset),
    kInstruction_str_al_r0_r4_minus_r6_ASR_14_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_minus_r1_ASR_25_Offset),
    kInstruction_str_al_r0_r2_minus_r1_ASR_25_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_plus_r6_LSR_21_Offset),
    kInstruction_str_al_r0_r12_plus_r6_LSR_21_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_plus_r12_ASR_18_Offset),
    kInstruction_str_al_r0_r6_plus_r12_ASR_18_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_plus_r13_LSR_15_Offset),
    kInstruction_str_al_r0_r9_plus_r13_LSR_15_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_plus_r6_LSR_26_Offset),
    kInstruction_str_al_r0_r6_plus_r6_LSR_26_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r9_ASR_14_Offset),
    kInstruction_str_al_r0_r4_minus_r9_ASR_14_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r7_ASR_6_Offset),
    kInstruction_str_al_r0_r0_plus_r7_ASR_6_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_plus_r8_ASR_9_Offset),
    kInstruction_str_al_r0_r14_plus_r8_ASR_9_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_plus_r7_LSR_4_Offset),
    kInstruction_str_al_r0_r10_plus_r7_LSR_4_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_plus_r3_LSR_13_Offset),
    kInstruction_str_al_r0_r11_plus_r3_LSR_13_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r13_plus_r3_LSR_25_Offset),
    kInstruction_str_al_r0_r13_plus_r3_LSR_25_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_plus_r3_ASR_13_Offset),
    kInstruction_str_al_r0_r4_plus_r3_ASR_13_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r2_LSR_2_Offset),
    kInstruction_str_al_r0_r4_minus_r2_LSR_2_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r13_minus_r7_LSR_1_Offset),
    kInstruction_str_al_r0_r13_minus_r7_LSR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_minus_r9_LSR_7_Offset),
    kInstruction_str_al_r0_r10_minus_r9_LSR_7_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r0_LSR_10_Offset),
    kInstruction_str_al_r0_r0_plus_r0_LSR_10_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r5_minus_r1_LSR_23_Offset),
    kInstruction_str_al_r0_r5_minus_r1_LSR_23_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_plus_r10_LSR_21_Offset),
    kInstruction_str_al_r0_r3_plus_r10_LSR_21_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r1_plus_r11_ASR_22_Offset),
    kInstruction_str_al_r0_r1_plus_r11_ASR_22_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r6_ASR_28_Offset),
    kInstruction_str_al_r0_r4_minus_r6_ASR_28_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_plus_r10_LSR_29_Offset),
    kInstruction_str_al_r0_r4_plus_r10_LSR_29_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_plus_r3_LSR_10_Offset),
    kInstruction_str_al_r0_r4_plus_r3_LSR_10_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_minus_r12_LSR_11_Offset),
    kInstruction_str_al_r0_r14_minus_r12_LSR_11_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r5_plus_r3_LSR_6_Offset),
    kInstruction_str_al_r0_r5_plus_r3_LSR_6_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_minus_r9_ASR_18_Offset),
    kInstruction_str_al_r0_r3_minus_r9_ASR_18_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_minus_r10_ASR_21_Offset),
    kInstruction_str_al_r0_r3_minus_r10_ASR_21_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_minus_r4_ASR_2_Offset),
    kInstruction_str_al_r0_r9_minus_r4_ASR_2_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_plus_r7_LSR_23_Offset),
    kInstruction_str_al_r0_r4_plus_r7_LSR_23_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_plus_r9_ASR_17_Offset),
    kInstruction_str_al_r0_r4_plus_r9_ASR_17_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r1_ASR_14_Offset),
    kInstruction_str_al_r0_r2_plus_r1_ASR_14_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r7_minus_r3_ASR_14_Offset),
    kInstruction_str_al_r0_r7_minus_r3_ASR_14_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_minus_r7_ASR_13_Offset),
    kInstruction_str_al_r0_r6_minus_r7_ASR_13_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_plus_r14_ASR_25_Offset),
    kInstruction_str_al_r0_r3_plus_r14_ASR_25_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r5_plus_r8_LSR_14_Offset),
    kInstruction_str_al_r0_r5_plus_r8_LSR_14_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r5_minus_r11_LSR_10_Offset),
    kInstruction_str_al_r0_r5_minus_r11_LSR_10_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r11_ASR_14_Offset),
    kInstruction_str_al_r0_r0_plus_r11_ASR_14_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_plus_r12_LSR_15_Offset),
    kInstruction_str_al_r0_r9_plus_r12_LSR_15_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_minus_r7_ASR_3_Offset),
    kInstruction_str_al_r0_r3_minus_r7_ASR_3_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r8_LSR_4_Offset),
    kInstruction_str_al_r0_r4_minus_r8_LSR_4_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r14_ASR_12_Offset),
    kInstruction_str_al_r0_r4_minus_r14_ASR_12_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r1_plus_r1_ASR_10_Offset),
    kInstruction_str_al_r0_r1_plus_r1_ASR_10_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r5_plus_r7_LSR_24_Offset),
    kInstruction_str_al_r0_r5_plus_r7_LSR_24_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_plus_r2_LSR_10_Offset),
    kInstruction_str_al_r0_r10_plus_r2_LSR_10_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r13_minus_r3_LSR_29_Offset),
    kInstruction_str_al_r0_r13_minus_r3_LSR_29_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r8_minus_r8_LSR_22_Offset),
    kInstruction_str_al_r0_r8_minus_r8_LSR_22_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_minus_r10_LSR_20_Offset),
    kInstruction_str_al_r0_r14_minus_r10_LSR_20_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_plus_r10_LSR_10_Offset),
    kInstruction_str_al_r0_r12_plus_r10_LSR_10_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_plus_r12_LSR_25_Offset),
    kInstruction_str_al_r0_r3_plus_r12_LSR_25_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r13_ASR_4_Offset),
    kInstruction_str_al_r0_r2_plus_r13_ASR_4_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_plus_r11_LSR_24_Offset),
    kInstruction_str_al_r0_r6_plus_r11_LSR_24_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r1_plus_r5_LSR_8_Offset),
    kInstruction_str_al_r0_r1_plus_r5_LSR_8_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_minus_r8_LSR_12_Offset),
    kInstruction_str_al_r0_r6_minus_r8_LSR_12_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_plus_r6_LSR_16_Offset),
    kInstruction_str_al_r0_r4_plus_r6_LSR_16_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r7_plus_r12_LSR_6_Offset),
    kInstruction_str_al_r0_r7_plus_r12_LSR_6_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_plus_r8_ASR_11_Offset),
    kInstruction_str_al_r0_r12_plus_r8_ASR_11_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_minus_r1_LSR_17_Offset),
    kInstruction_str_al_r0_r11_minus_r1_LSR_17_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r0_LSR_8_Offset),
    kInstruction_str_al_r0_r4_minus_r0_LSR_8_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r5_minus_r14_LSR_2_Offset),
    kInstruction_str_al_r0_r5_minus_r14_LSR_2_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_minus_r8_LSR_24_Offset),
    kInstruction_str_al_r0_r10_minus_r8_LSR_24_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r8_minus_r12_ASR_4_Offset),
    kInstruction_str_al_r0_r8_minus_r12_ASR_4_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_minus_r4_ASR_27_Offset),
    kInstruction_str_al_r0_r3_minus_r4_ASR_27_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_minus_r8_LSR_25_Offset),
    kInstruction_str_al_r0_r12_minus_r8_LSR_25_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_minus_r13_LSR_5_Offset),
    kInstruction_str_al_r0_r10_minus_r13_LSR_5_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_minus_r8_LSR_26_Offset),
    kInstruction_str_al_r0_r0_minus_r8_LSR_26_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r1_LSR_18_Offset),
    kInstruction_str_al_r0_r4_minus_r1_LSR_18_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_plus_r4_LSR_26_Offset),
    kInstruction_str_al_r0_r3_plus_r4_LSR_26_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_plus_r1_LSR_27_Offset),
    kInstruction_str_al_r0_r6_plus_r1_LSR_27_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_minus_r11_ASR_9_Offset),
    kInstruction_str_al_r0_r14_minus_r11_ASR_9_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r5_minus_r13_LSR_25_Offset),
    kInstruction_str_al_r0_r5_minus_r13_LSR_25_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_minus_r2_ASR_23_Offset),
    kInstruction_str_al_r0_r6_minus_r2_ASR_23_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r13_minus_r3_LSR_21_Offset),
    kInstruction_str_al_r0_r13_minus_r3_LSR_21_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_minus_r7_ASR_10_Offset),
    kInstruction_str_al_r0_r12_minus_r7_ASR_10_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r0_ASR_19_Offset),
    kInstruction_str_al_r0_r0_plus_r0_ASR_19_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_minus_r14_ASR_32_Offset),
    kInstruction_str_al_r0_r14_minus_r14_ASR_32_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_plus_r11_LSR_4_Offset),
    kInstruction_str_al_r0_r9_plus_r11_LSR_4_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_minus_r12_ASR_28_Offset),
    kInstruction_str_al_r0_r10_minus_r12_ASR_28_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r1_minus_r3_LSR_15_Offset),
    kInstruction_str_al_r0_r1_minus_r3_LSR_15_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r3_LSR_5_Offset),
    kInstruction_str_al_r0_r0_plus_r3_LSR_5_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r9_ASR_16_Offset),
    kInstruction_str_al_r0_r2_plus_r9_ASR_16_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r7_minus_r3_ASR_30_Offset),
    kInstruction_str_al_r0_r7_minus_r3_ASR_30_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_minus_r1_ASR_31_Offset),
    kInstruction_str_al_r0_r10_minus_r1_ASR_31_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_minus_r7_LSR_26_Offset),
    kInstruction_str_al_r0_r11_minus_r7_LSR_26_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_minus_r4_LSR_8_Offset),
    kInstruction_str_al_r0_r0_minus_r4_LSR_8_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_plus_r1_LSR_5_Offset),
    kInstruction_str_al_r0_r3_plus_r1_LSR_5_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_minus_r1_LSR_15_Offset),
    kInstruction_str_al_r0_r14_minus_r1_LSR_15_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r8_ASR_7_Offset),
    kInstruction_str_al_r0_r2_plus_r8_ASR_7_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_plus_r12_ASR_7_Offset),
    kInstruction_str_al_r0_r11_plus_r12_ASR_7_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_plus_r6_LSR_19_Offset),
    kInstruction_str_al_r0_r14_plus_r6_LSR_19_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_minus_r6_ASR_12_Offset),
    kInstruction_str_al_r0_r0_minus_r6_ASR_12_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_minus_r8_LSR_24_Offset),
    kInstruction_str_al_r0_r2_minus_r8_LSR_24_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_minus_r13_LSR_4_Offset),
    kInstruction_str_al_r0_r2_minus_r13_LSR_4_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_minus_r12_LSR_6_Offset),
    kInstruction_str_al_r0_r11_minus_r12_LSR_6_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r9_ASR_12_Offset),
    kInstruction_str_al_r0_r0_plus_r9_ASR_12_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r5_plus_r9_ASR_25_Offset),
    kInstruction_str_al_r0_r5_plus_r9_ASR_25_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_minus_r8_ASR_23_Offset),
    kInstruction_str_al_r0_r0_minus_r8_ASR_23_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r8_plus_r4_ASR_29_Offset),
    kInstruction_str_al_r0_r8_plus_r4_ASR_29_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_minus_r1_LSR_25_Offset),
    kInstruction_str_al_r0_r0_minus_r1_LSR_25_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r1_minus_r0_LSR_25_Offset),
    kInstruction_str_al_r0_r1_minus_r0_LSR_25_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r13_LSR_32_Offset),
    kInstruction_str_al_r0_r2_plus_r13_LSR_32_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_plus_r3_LSR_10_Offset),
    kInstruction_str_al_r0_r9_plus_r3_LSR_10_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r5_minus_r4_LSR_21_Offset),
    kInstruction_str_al_r0_r5_minus_r4_LSR_21_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_minus_r5_LSR_30_Offset),
    kInstruction_str_al_r0_r2_minus_r5_LSR_30_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r8_LSR_26_Offset),
    kInstruction_str_al_r0_r2_plus_r8_LSR_26_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r5_minus_r10_ASR_12_Offset),
    kInstruction_str_al_r0_r5_minus_r10_ASR_12_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_minus_r0_ASR_21_Offset),
    kInstruction_str_al_r0_r14_minus_r0_ASR_21_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r5_plus_r6_ASR_1_Offset),
    kInstruction_str_al_r0_r5_plus_r6_ASR_1_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_minus_r3_ASR_16_Offset),
    kInstruction_str_al_r0_r3_minus_r3_ASR_16_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_plus_r0_LSR_10_Offset),
    kInstruction_str_al_r0_r12_plus_r0_LSR_10_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_plus_r5_ASR_5_Offset),
    kInstruction_str_al_r0_r11_plus_r5_ASR_5_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_minus_r11_ASR_19_Offset),
    kInstruction_str_al_r0_r3_minus_r11_ASR_19_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r5_LSR_27_Offset),
    kInstruction_str_al_r0_r4_minus_r5_LSR_27_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_plus_r5_ASR_17_Offset),
    kInstruction_str_al_r0_r3_plus_r5_ASR_17_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r8_plus_r5_LSR_18_Offset),
    kInstruction_str_al_r0_r8_plus_r5_LSR_18_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r8_plus_r11_ASR_24_Offset),
    kInstruction_str_al_r0_r8_plus_r11_ASR_24_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_plus_r12_LSR_30_Offset),
    kInstruction_str_al_r0_r10_plus_r12_LSR_30_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r8_minus_r3_ASR_9_Offset),
    kInstruction_str_al_r0_r8_minus_r3_ASR_9_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_plus_r9_ASR_11_Offset),
    kInstruction_str_al_r0_r11_plus_r9_ASR_11_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_plus_r7_LSR_22_Offset),
    kInstruction_str_al_r0_r10_plus_r7_LSR_22_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_plus_r7_ASR_13_Offset),
    kInstruction_str_al_r0_r10_plus_r7_ASR_13_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r7_plus_r13_LSR_21_Offset),
    kInstruction_str_al_r0_r7_plus_r13_LSR_21_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_minus_r1_LSR_16_Offset),
    kInstruction_str_al_r0_r12_minus_r1_LSR_16_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_minus_r2_LSR_4_Offset),
    kInstruction_str_al_r0_r2_minus_r2_LSR_4_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r13_minus_r13_ASR_19_Offset),
    kInstruction_str_al_r0_r13_minus_r13_ASR_19_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r4_LSR_13_Offset),
    kInstruction_str_al_r0_r2_plus_r4_LSR_13_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_plus_r1_LSR_32_Offset),
    kInstruction_str_al_r0_r9_plus_r1_LSR_32_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_plus_r6_ASR_21_Offset),
    kInstruction_str_al_r0_r3_plus_r6_ASR_21_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r1_LSR_5_Offset),
    kInstruction_str_al_r0_r2_plus_r1_LSR_5_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_minus_r11_ASR_6_Offset),
    kInstruction_str_al_r0_r0_minus_r11_ASR_6_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_minus_r9_ASR_31_Offset),
    kInstruction_str_al_r0_r2_minus_r9_ASR_31_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r11_LSR_6_Offset),
    kInstruction_str_al_r0_r0_plus_r11_LSR_6_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_minus_r11_LSR_14_Offset),
    kInstruction_str_al_r0_r10_minus_r11_LSR_14_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r14_LSR_24_Offset),
    kInstruction_str_al_r0_r0_plus_r14_LSR_24_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_plus_r2_LSR_8_Offset),
    kInstruction_str_al_r0_r0_plus_r2_LSR_8_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r13_minus_r12_ASR_10_Offset),
    kInstruction_str_al_r0_r13_minus_r12_ASR_10_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r2_LSR_12_Offset),
    kInstruction_str_al_r0_r2_plus_r2_LSR_12_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_plus_r10_LSR_30_Offset),
    kInstruction_str_al_r0_r12_plus_r10_LSR_30_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r8_minus_r0_ASR_9_Offset),
    kInstruction_str_al_r0_r8_minus_r0_ASR_9_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_minus_r6_LSR_4_Offset),
    kInstruction_str_al_r0_r9_minus_r6_LSR_4_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_plus_r14_ASR_3_Offset),
    kInstruction_str_al_r0_r3_plus_r14_ASR_3_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_minus_r10_ASR_4_Offset),
    kInstruction_str_al_r0_r14_minus_r10_ASR_4_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r7_plus_r8_LSR_5_Offset),
    kInstruction_str_al_r0_r7_plus_r8_LSR_5_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_minus_r13_ASR_17_Offset),
    kInstruction_str_al_r0_r0_minus_r13_ASR_17_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_minus_r7_LSR_2_Offset),
    kInstruction_str_al_r0_r3_minus_r7_LSR_2_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r8_minus_r8_ASR_15_Offset),
    kInstruction_str_al_r0_r8_minus_r8_ASR_15_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r13_plus_r12_ASR_29_Offset),
    kInstruction_str_al_r0_r13_plus_r12_ASR_29_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_plus_r4_LSR_22_Offset),
    kInstruction_str_al_r0_r10_plus_r4_LSR_22_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r1_ASR_24_Offset),
    kInstruction_str_al_r0_r2_plus_r1_ASR_24_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_minus_r13_LSR_29_Offset),
    kInstruction_str_al_r0_r14_minus_r13_LSR_29_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_plus_r1_LSR_6_Offset),
    kInstruction_str_al_r0_r3_plus_r1_LSR_6_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r7_plus_r5_LSR_32_Offset),
    kInstruction_str_al_r0_r7_plus_r5_LSR_32_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_plus_r6_LSR_30_Offset),
    kInstruction_str_al_r0_r12_plus_r6_LSR_30_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r13_plus_r13_ASR_6_Offset),
    kInstruction_str_al_r0_r13_plus_r13_ASR_6_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_minus_r0_LSR_4_Offset),
    kInstruction_str_al_r0_r14_minus_r0_LSR_4_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r7_minus_r4_ASR_6_Offset),
    kInstruction_str_al_r0_r7_minus_r4_ASR_6_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_plus_r13_ASR_5_Offset),
    kInstruction_str_al_r0_r9_plus_r13_ASR_5_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r8_plus_r9_LSR_5_Offset),
    kInstruction_str_al_r0_r8_plus_r9_LSR_5_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_plus_r11_LSR_7_Offset),
    kInstruction_str_al_r0_r3_plus_r11_LSR_7_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r5_plus_r2_LSR_11_Offset),
    kInstruction_str_al_r0_r5_plus_r2_LSR_11_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r10_minus_r4_LSR_12_Offset),
    kInstruction_str_al_r0_r10_minus_r4_LSR_12_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r0_minus_r0_ASR_25_Offset),
    kInstruction_str_al_r0_r0_minus_r0_ASR_25_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r13_minus_r10_ASR_6_Offset),
    kInstruction_str_al_r0_r13_minus_r10_ASR_6_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_plus_r1_LSR_20_Offset),
    kInstruction_str_al_r0_r11_plus_r1_LSR_20_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r13_minus_r10_LSR_3_Offset),
    kInstruction_str_al_r0_r13_minus_r10_LSR_3_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_plus_r9_ASR_2_Offset),
    kInstruction_str_al_r0_r6_plus_r9_ASR_2_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r7_ASR_23_Offset),
    kInstruction_str_al_r0_r4_minus_r7_ASR_23_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r13_minus_r10_LSR_11_Offset),
    kInstruction_str_al_r0_r13_minus_r10_LSR_11_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_minus_r0_LSR_15_Offset),
    kInstruction_str_al_r0_r9_minus_r0_LSR_15_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r3_plus_r5_ASR_23_Offset),
    kInstruction_str_al_r0_r3_plus_r5_ASR_23_Offset,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r0_plus_r1_ASR_3_PreIndex),
    kInstruction_str_al_r8_r0_plus_r1_ASR_3_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_minus_r6_ASR_27_PreIndex),
    kInstruction_str_al_r0_r14_minus_r6_ASR_27_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r11_minus_r8_LSR_2_PostIndex),
    kInstruction_str_al_r12_r11_minus_r8_LSR_2_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r3_plus_r10_ASR_28_PostIndex),
    kInstruction_str_al_r9_r3_plus_r10_ASR_28_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r5_plus_r6_ASR_7_PreIndex),
    kInstruction_str_al_r2_r5_plus_r6_ASR_7_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r11_minus_r2_ASR_2_PostIndex),
    kInstruction_str_al_r14_r11_minus_r2_ASR_2_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_minus_r1_ASR_13_PreIndex),
    kInstruction_str_al_r0_r11_minus_r1_ASR_13_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r12_plus_r6_LSR_16_PreIndex),
    kInstruction_str_al_r13_r12_plus_r6_LSR_16_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r2_plus_r10_LSR_31_PreIndex),
    kInstruction_str_al_r10_r2_plus_r10_LSR_31_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r11_plus_r14_LSR_9_PreIndex),
    kInstruction_str_al_r14_r11_plus_r14_LSR_9_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_minus_r12_LSR_9_PreIndex),
    kInstruction_str_al_r0_r2_minus_r12_LSR_9_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r13_minus_r4_ASR_26_PostIndex),
    kInstruction_str_al_r0_r13_minus_r4_ASR_26_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r14_minus_r5_ASR_21_PreIndex),
    kInstruction_str_al_r7_r14_minus_r5_ASR_21_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r8_ASR_21_PostIndex),
    kInstruction_str_al_r0_r4_minus_r8_ASR_21_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r6_plus_r2_LSR_31_PostIndex),
    kInstruction_str_al_r11_r6_plus_r2_LSR_31_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r9_minus_r2_LSR_29_PostIndex),
    kInstruction_str_al_r2_r9_minus_r2_LSR_29_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r4_minus_r9_LSR_8_PreIndex),
    kInstruction_str_al_r3_r4_minus_r9_LSR_8_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r14_plus_r11_LSR_32_PreIndex),
    kInstruction_str_al_r8_r14_plus_r11_LSR_32_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r0_plus_r4_ASR_18_PostIndex),
    kInstruction_str_al_r11_r0_plus_r4_ASR_18_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r7_plus_r2_ASR_30_PreIndex),
    kInstruction_str_al_r4_r7_plus_r2_ASR_30_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r6_minus_r2_LSR_30_PreIndex),
    kInstruction_str_al_r8_r6_minus_r2_LSR_30_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r5_minus_r0_ASR_20_PreIndex),
    kInstruction_str_al_r4_r5_minus_r0_ASR_20_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r11_minus_r11_LSR_30_PostIndex),
    kInstruction_str_al_r5_r11_minus_r11_LSR_30_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r4_minus_r7_ASR_13_PreIndex),
    kInstruction_str_al_r12_r4_minus_r7_ASR_13_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r12_minus_r1_ASR_16_PreIndex),
    kInstruction_str_al_r5_r12_minus_r1_ASR_16_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r3_minus_r0_LSR_10_PostIndex),
    kInstruction_str_al_r8_r3_minus_r0_LSR_10_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r11_plus_r5_LSR_12_PreIndex),
    kInstruction_str_al_r2_r11_plus_r5_LSR_12_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r2_minus_r4_ASR_16_PreIndex),
    kInstruction_str_al_r6_r2_minus_r4_ASR_16_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r2_minus_r0_LSR_20_PreIndex),
    kInstruction_str_al_r10_r2_minus_r0_LSR_20_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r11_minus_r13_ASR_30_PostIndex),
    kInstruction_str_al_r6_r11_minus_r13_ASR_30_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r8_minus_r10_ASR_9_PostIndex),
    kInstruction_str_al_r11_r8_minus_r10_ASR_9_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r3_plus_r12_LSR_22_PostIndex),
    kInstruction_str_al_r2_r3_plus_r12_LSR_22_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r6_minus_r5_ASR_15_PostIndex),
    kInstruction_str_al_r1_r6_minus_r5_ASR_15_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r5_minus_r5_ASR_10_PreIndex),
    kInstruction_str_al_r8_r5_minus_r5_ASR_10_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r1_minus_r14_LSR_26_PostIndex),
    kInstruction_str_al_r2_r1_minus_r14_LSR_26_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r12_minus_r3_LSR_16_PostIndex),
    kInstruction_str_al_r7_r12_minus_r3_LSR_16_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r5_minus_r12_ASR_24_PreIndex),
    kInstruction_str_al_r8_r5_minus_r12_ASR_24_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r4_plus_r0_ASR_23_PreIndex),
    kInstruction_str_al_r8_r4_plus_r0_ASR_23_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r0_minus_r14_ASR_31_PostIndex),
    kInstruction_str_al_r2_r0_minus_r14_ASR_31_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r13_minus_r8_LSR_30_PostIndex),
    kInstruction_str_al_r6_r13_minus_r8_LSR_30_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r6_minus_r4_LSR_22_PostIndex),
    kInstruction_str_al_r3_r6_minus_r4_LSR_22_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r8_plus_r9_ASR_32_PreIndex),
    kInstruction_str_al_r12_r8_plus_r9_ASR_32_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r4_plus_r4_ASR_13_PreIndex),
    kInstruction_str_al_r2_r4_plus_r4_ASR_13_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r13_plus_r7_ASR_12_PreIndex),
    kInstruction_str_al_r3_r13_plus_r7_ASR_12_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r1_minus_r14_LSR_14_PostIndex),
    kInstruction_str_al_r11_r1_minus_r14_LSR_14_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r9_plus_r5_ASR_27_PostIndex),
    kInstruction_str_al_r12_r9_plus_r5_ASR_27_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r3_plus_r11_ASR_31_PreIndex),
    kInstruction_str_al_r1_r3_plus_r11_ASR_31_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r2_plus_r2_ASR_26_PostIndex),
    kInstruction_str_al_r10_r2_plus_r2_ASR_26_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r7_plus_r5_LSR_19_PreIndex),
    kInstruction_str_al_r0_r7_plus_r5_LSR_19_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r7_plus_r5_LSR_27_PreIndex),
    kInstruction_str_al_r12_r7_plus_r5_LSR_27_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r11_plus_r11_LSR_9_PostIndex),
    kInstruction_str_al_r10_r11_plus_r11_LSR_9_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r14_plus_r13_ASR_24_PreIndex),
    kInstruction_str_al_r2_r14_plus_r13_ASR_24_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r0_minus_r1_LSR_20_PreIndex),
    kInstruction_str_al_r13_r0_minus_r1_LSR_20_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r4_minus_r8_LSR_26_PostIndex),
    kInstruction_str_al_r2_r4_minus_r8_LSR_26_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r13_minus_r8_ASR_14_PostIndex),
    kInstruction_str_al_r10_r13_minus_r8_ASR_14_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r0_plus_r7_ASR_8_PostIndex),
    kInstruction_str_al_r6_r0_plus_r7_ASR_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r9_plus_r6_ASR_18_PreIndex),
    kInstruction_str_al_r3_r9_plus_r6_ASR_18_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r1_minus_r0_ASR_2_PreIndex),
    kInstruction_str_al_r4_r1_minus_r0_ASR_2_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r11_plus_r11_ASR_7_PostIndex),
    kInstruction_str_al_r4_r11_plus_r11_ASR_7_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r9_minus_r11_ASR_3_PostIndex),
    kInstruction_str_al_r14_r9_minus_r11_ASR_3_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r11_minus_r8_ASR_31_PostIndex),
    kInstruction_str_al_r2_r11_minus_r8_ASR_31_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r4_plus_r4_ASR_31_PreIndex),
    kInstruction_str_al_r11_r4_plus_r4_ASR_31_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r2_plus_r4_LSR_20_PostIndex),
    kInstruction_str_al_r10_r2_plus_r4_LSR_20_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r13_minus_r11_LSR_2_PreIndex),
    kInstruction_str_al_r3_r13_minus_r11_LSR_2_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r2_minus_r5_LSR_21_PreIndex),
    kInstruction_str_al_r14_r2_minus_r5_LSR_21_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r5_minus_r0_LSR_21_PostIndex),
    kInstruction_str_al_r8_r5_minus_r0_LSR_21_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r10_plus_r3_LSR_10_PostIndex),
    kInstruction_str_al_r9_r10_plus_r3_LSR_10_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r11_plus_r12_LSR_27_PostIndex),
    kInstruction_str_al_r12_r11_plus_r12_LSR_27_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r8_minus_r8_ASR_28_PreIndex),
    kInstruction_str_al_r9_r8_minus_r8_ASR_28_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r7_minus_r5_LSR_17_PreIndex),
    kInstruction_str_al_r5_r7_minus_r5_LSR_17_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r2_minus_r3_ASR_21_PreIndex),
    kInstruction_str_al_r7_r2_minus_r3_ASR_21_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r12_minus_r1_ASR_26_PreIndex),
    kInstruction_str_al_r7_r12_minus_r1_ASR_26_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r5_plus_r8_ASR_14_PreIndex),
    kInstruction_str_al_r14_r5_plus_r8_ASR_14_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r14_minus_r10_LSR_28_PostIndex),
    kInstruction_str_al_r13_r14_minus_r10_LSR_28_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r8_plus_r10_ASR_11_PreIndex),
    kInstruction_str_al_r12_r8_plus_r10_ASR_11_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r7_plus_r12_ASR_6_PostIndex),
    kInstruction_str_al_r1_r7_plus_r12_ASR_6_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r3_plus_r3_LSR_17_PreIndex),
    kInstruction_str_al_r11_r3_plus_r3_LSR_17_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r4_minus_r5_ASR_8_PostIndex),
    kInstruction_str_al_r12_r4_minus_r5_ASR_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r5_plus_r13_LSR_11_PostIndex),
    kInstruction_str_al_r2_r5_plus_r13_LSR_11_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r4_minus_r12_ASR_13_PostIndex),
    kInstruction_str_al_r3_r4_minus_r12_ASR_13_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r0_plus_r7_ASR_6_PreIndex),
    kInstruction_str_al_r1_r0_plus_r7_ASR_6_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r9_minus_r1_ASR_29_PostIndex),
    kInstruction_str_al_r5_r9_minus_r1_ASR_29_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r11_plus_r1_ASR_17_PostIndex),
    kInstruction_str_al_r13_r11_plus_r1_ASR_17_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r10_minus_r12_ASR_26_PostIndex),
    kInstruction_str_al_r8_r10_minus_r12_ASR_26_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r8_minus_r5_LSR_18_PreIndex),
    kInstruction_str_al_r7_r8_minus_r5_LSR_18_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r8_plus_r2_LSR_14_PreIndex),
    kInstruction_str_al_r9_r8_plus_r2_LSR_14_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r0_plus_r2_ASR_32_PostIndex),
    kInstruction_str_al_r5_r0_plus_r2_ASR_32_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r2_minus_r3_ASR_11_PreIndex),
    kInstruction_str_al_r13_r2_minus_r3_ASR_11_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r12_plus_r10_ASR_18_PostIndex),
    kInstruction_str_al_r1_r12_plus_r10_ASR_18_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r12_minus_r2_LSR_19_PostIndex),
    kInstruction_str_al_r5_r12_minus_r2_LSR_19_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r0_minus_r1_LSR_9_PreIndex),
    kInstruction_str_al_r7_r0_minus_r1_LSR_9_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r8_plus_r11_ASR_13_PostIndex),
    kInstruction_str_al_r5_r8_plus_r11_ASR_13_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r0_minus_r8_LSR_24_PreIndex),
    kInstruction_str_al_r7_r0_minus_r8_LSR_24_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r3_plus_r5_ASR_24_PreIndex),
    kInstruction_str_al_r4_r3_plus_r5_ASR_24_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r9_minus_r6_LSR_23_PreIndex),
    kInstruction_str_al_r7_r9_minus_r6_LSR_23_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r4_plus_r0_LSR_30_PreIndex),
    kInstruction_str_al_r2_r4_plus_r0_LSR_30_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r6_plus_r13_LSR_26_PreIndex),
    kInstruction_str_al_r10_r6_plus_r13_LSR_26_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r8_minus_r9_ASR_17_PreIndex),
    kInstruction_str_al_r4_r8_minus_r9_ASR_17_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r9_plus_r9_ASR_30_PreIndex),
    kInstruction_str_al_r4_r9_plus_r9_ASR_30_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r5_plus_r13_LSR_20_PostIndex),
    kInstruction_str_al_r1_r5_plus_r13_LSR_20_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r6_minus_r1_LSR_29_PreIndex),
    kInstruction_str_al_r8_r6_minus_r1_LSR_29_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r3_plus_r6_ASR_29_PostIndex),
    kInstruction_str_al_r12_r3_plus_r6_ASR_29_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r5_plus_r0_LSR_13_PreIndex),
    kInstruction_str_al_r13_r5_plus_r0_LSR_13_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r1_plus_r0_ASR_26_PreIndex),
    kInstruction_str_al_r5_r1_plus_r0_ASR_26_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r7_plus_r8_ASR_26_PreIndex),
    kInstruction_str_al_r6_r7_plus_r8_ASR_26_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r14_plus_r8_ASR_8_PostIndex),
    kInstruction_str_al_r5_r14_plus_r8_ASR_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r14_plus_r6_LSR_23_PreIndex),
    kInstruction_str_al_r9_r14_plus_r6_LSR_23_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r4_plus_r3_ASR_9_PreIndex),
    kInstruction_str_al_r6_r4_plus_r3_ASR_9_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r7_minus_r11_ASR_29_PreIndex),
    kInstruction_str_al_r8_r7_minus_r11_ASR_29_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r3_plus_r0_ASR_5_PostIndex),
    kInstruction_str_al_r9_r3_plus_r0_ASR_5_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r6_plus_r5_LSR_4_PreIndex),
    kInstruction_str_al_r8_r6_plus_r5_LSR_4_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r8_plus_r7_ASR_4_PostIndex),
    kInstruction_str_al_r9_r8_plus_r7_ASR_4_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r4_plus_r6_ASR_28_PostIndex),
    kInstruction_str_al_r12_r4_plus_r6_ASR_28_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r7_minus_r4_ASR_25_PreIndex),
    kInstruction_str_al_r4_r7_minus_r4_ASR_25_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r4_minus_r7_ASR_5_PostIndex),
    kInstruction_str_al_r1_r4_minus_r7_ASR_5_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r1_plus_r12_LSR_25_PostIndex),
    kInstruction_str_al_r3_r1_plus_r12_LSR_25_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r8_plus_r14_ASR_32_PreIndex),
    kInstruction_str_al_r13_r8_plus_r14_ASR_32_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r3_minus_r1_LSR_7_PostIndex),
    kInstruction_str_al_r1_r3_minus_r1_LSR_7_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r3_minus_r4_ASR_21_PostIndex),
    kInstruction_str_al_r13_r3_minus_r4_ASR_21_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r8_plus_r10_LSR_7_PostIndex),
    kInstruction_str_al_r11_r8_plus_r10_LSR_7_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r0_minus_r0_LSR_24_PreIndex),
    kInstruction_str_al_r10_r0_minus_r0_LSR_24_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r11_plus_r2_LSR_7_PreIndex),
    kInstruction_str_al_r12_r11_plus_r2_LSR_7_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r10_minus_r6_ASR_2_PostIndex),
    kInstruction_str_al_r11_r10_minus_r6_ASR_2_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r10_minus_r10_ASR_19_PostIndex),
    kInstruction_str_al_r5_r10_minus_r10_ASR_19_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r8_minus_r6_ASR_30_PreIndex),
    kInstruction_str_al_r13_r8_minus_r6_ASR_30_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r10_plus_r0_ASR_18_PreIndex),
    kInstruction_str_al_r1_r10_plus_r0_ASR_18_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r14_plus_r5_ASR_8_PostIndex),
    kInstruction_str_al_r10_r14_plus_r5_ASR_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r3_minus_r10_LSR_11_PreIndex),
    kInstruction_str_al_r1_r3_minus_r10_LSR_11_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r13_minus_r11_LSR_5_PreIndex),
    kInstruction_str_al_r11_r13_minus_r11_LSR_5_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r9_plus_r12_LSR_28_PostIndex),
    kInstruction_str_al_r4_r9_plus_r12_LSR_28_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r8_plus_r12_LSR_14_PostIndex),
    kInstruction_str_al_r3_r8_plus_r12_LSR_14_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r7_minus_r2_ASR_3_PostIndex),
    kInstruction_str_al_r11_r7_minus_r2_ASR_3_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r8_plus_r7_ASR_20_PostIndex),
    kInstruction_str_al_r0_r8_plus_r7_ASR_20_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r1_minus_r2_LSR_15_PreIndex),
    kInstruction_str_al_r8_r1_minus_r2_LSR_15_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r5_plus_r7_LSR_14_PostIndex),
    kInstruction_str_al_r9_r5_plus_r7_LSR_14_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r8_minus_r13_LSR_12_PostIndex),
    kInstruction_str_al_r1_r8_minus_r13_LSR_12_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r0_plus_r12_LSR_22_PreIndex),
    kInstruction_str_al_r8_r0_plus_r12_LSR_22_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r0_minus_r10_ASR_26_PostIndex),
    kInstruction_str_al_r13_r0_minus_r10_ASR_26_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r3_minus_r3_LSR_11_PreIndex),
    kInstruction_str_al_r12_r3_minus_r3_LSR_11_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r3_minus_r8_LSR_25_PostIndex),
    kInstruction_str_al_r7_r3_minus_r8_LSR_25_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r4_plus_r5_ASR_2_PreIndex),
    kInstruction_str_al_r5_r4_plus_r5_ASR_2_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r10_plus_r1_ASR_22_PostIndex),
    kInstruction_str_al_r3_r10_plus_r1_ASR_22_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r11_minus_r7_LSR_4_PostIndex),
    kInstruction_str_al_r6_r11_minus_r7_LSR_4_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r7_minus_r3_ASR_28_PreIndex),
    kInstruction_str_al_r13_r7_minus_r3_ASR_28_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r10_minus_r4_LSR_5_PreIndex),
    kInstruction_str_al_r3_r10_minus_r4_LSR_5_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r2_minus_r10_LSR_25_PreIndex),
    kInstruction_str_al_r1_r2_minus_r10_LSR_25_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r3_plus_r11_LSR_19_PreIndex),
    kInstruction_str_al_r1_r3_plus_r11_LSR_19_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r4_minus_r9_LSR_32_PostIndex),
    kInstruction_str_al_r3_r4_minus_r9_LSR_32_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r11_plus_r2_LSR_17_PreIndex),
    kInstruction_str_al_r13_r11_plus_r2_LSR_17_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r6_plus_r2_ASR_20_PreIndex),
    kInstruction_str_al_r4_r6_plus_r2_ASR_20_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r12_minus_r10_ASR_18_PreIndex),
    kInstruction_str_al_r1_r12_minus_r10_ASR_18_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r14_plus_r7_LSR_15_PostIndex),
    kInstruction_str_al_r4_r14_plus_r7_LSR_15_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r13_minus_r0_ASR_19_PreIndex),
    kInstruction_str_al_r11_r13_minus_r0_ASR_19_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r10_minus_r12_ASR_8_PostIndex),
    kInstruction_str_al_r13_r10_minus_r12_ASR_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r14_minus_r4_ASR_17_PreIndex),
    kInstruction_str_al_r5_r14_minus_r4_ASR_17_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r7_minus_r3_LSR_11_PreIndex),
    kInstruction_str_al_r8_r7_minus_r3_LSR_11_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r7_plus_r8_ASR_3_PostIndex),
    kInstruction_str_al_r12_r7_plus_r8_ASR_3_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_minus_r4_ASR_26_PostIndex),
    kInstruction_str_al_r0_r6_minus_r4_ASR_26_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r13_plus_r8_ASR_21_PreIndex),
    kInstruction_str_al_r5_r13_plus_r8_ASR_21_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r0_minus_r0_ASR_3_PostIndex),
    kInstruction_str_al_r4_r0_minus_r0_ASR_3_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r3_minus_r0_LSR_20_PreIndex),
    kInstruction_str_al_r13_r3_minus_r0_LSR_20_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r14_minus_r6_ASR_31_PostIndex),
    kInstruction_str_al_r3_r14_minus_r6_ASR_31_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r2_plus_r13_LSR_1_PreIndex),
    kInstruction_str_al_r12_r2_plus_r13_LSR_1_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r11_plus_r8_LSR_15_PreIndex),
    kInstruction_str_al_r9_r11_plus_r8_LSR_15_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r14_plus_r9_LSR_22_PreIndex),
    kInstruction_str_al_r3_r14_plus_r9_LSR_22_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r1_plus_r2_ASR_23_PostIndex),
    kInstruction_str_al_r12_r1_plus_r2_ASR_23_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r6_plus_r4_LSR_18_PreIndex),
    kInstruction_str_al_r4_r6_plus_r4_LSR_18_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r2_minus_r3_ASR_30_PreIndex),
    kInstruction_str_al_r11_r2_minus_r3_ASR_30_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r1_minus_r0_ASR_6_PostIndex),
    kInstruction_str_al_r4_r1_minus_r0_ASR_6_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r5_minus_r9_LSR_30_PreIndex),
    kInstruction_str_al_r12_r5_minus_r9_LSR_30_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r0_minus_r1_LSR_26_PreIndex),
    kInstruction_str_al_r11_r0_minus_r1_LSR_26_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r2_plus_r11_LSR_4_PostIndex),
    kInstruction_str_al_r11_r2_plus_r11_LSR_4_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r13_plus_r10_LSR_8_PostIndex),
    kInstruction_str_al_r12_r13_plus_r10_LSR_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_minus_r2_ASR_29_PostIndex),
    kInstruction_str_al_r0_r9_minus_r2_ASR_29_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r10_plus_r13_LSR_16_PostIndex),
    kInstruction_str_al_r1_r10_plus_r13_LSR_16_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r12_minus_r1_ASR_21_PostIndex),
    kInstruction_str_al_r5_r12_minus_r1_ASR_21_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r1_minus_r11_ASR_23_PreIndex),
    kInstruction_str_al_r14_r1_minus_r11_ASR_23_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r12_plus_r10_LSR_13_PostIndex),
    kInstruction_str_al_r6_r12_plus_r10_LSR_13_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r2_minus_r2_ASR_13_PreIndex),
    kInstruction_str_al_r5_r2_minus_r2_ASR_13_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r12_plus_r6_ASR_18_PreIndex),
    kInstruction_str_al_r1_r12_plus_r6_ASR_18_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r5_minus_r1_ASR_11_PostIndex),
    kInstruction_str_al_r7_r5_minus_r1_ASR_11_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r6_minus_r1_LSR_3_PreIndex),
    kInstruction_str_al_r4_r6_minus_r1_LSR_3_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r5_plus_r13_LSR_27_PreIndex),
    kInstruction_str_al_r6_r5_plus_r13_LSR_27_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r13_minus_r13_ASR_30_PreIndex),
    kInstruction_str_al_r1_r13_minus_r13_ASR_30_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r13_plus_r9_LSR_3_PreIndex),
    kInstruction_str_al_r5_r13_plus_r9_LSR_3_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r4_plus_r0_ASR_25_PreIndex),
    kInstruction_str_al_r1_r4_plus_r0_ASR_25_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_minus_r0_ASR_17_PostIndex),
    kInstruction_str_al_r0_r12_minus_r0_ASR_17_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r4_minus_r7_LSR_11_PostIndex),
    kInstruction_str_al_r5_r4_minus_r7_LSR_11_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r4_minus_r10_ASR_16_PreIndex),
    kInstruction_str_al_r14_r4_minus_r10_ASR_16_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r12_plus_r12_LSR_17_PostIndex),
    kInstruction_str_al_r4_r12_plus_r12_LSR_17_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r2_plus_r7_LSR_24_PostIndex),
    kInstruction_str_al_r9_r2_plus_r7_LSR_24_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r14_plus_r7_ASR_12_PostIndex),
    kInstruction_str_al_r13_r14_plus_r7_ASR_12_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r7_minus_r11_LSR_13_PreIndex),
    kInstruction_str_al_r2_r7_minus_r11_LSR_13_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_plus_r9_LSR_27_PostIndex),
    kInstruction_str_al_r0_r4_plus_r9_LSR_27_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r9_minus_r10_LSR_29_PostIndex),
    kInstruction_str_al_r6_r9_minus_r10_LSR_29_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r4_minus_r1_ASR_25_PostIndex),
    kInstruction_str_al_r7_r4_minus_r1_ASR_25_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r1_plus_r11_ASR_29_PostIndex),
    kInstruction_str_al_r6_r1_plus_r11_ASR_29_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r7_minus_r13_LSR_3_PostIndex),
    kInstruction_str_al_r12_r7_minus_r13_LSR_3_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r12_plus_r6_ASR_25_PostIndex),
    kInstruction_str_al_r13_r12_plus_r6_ASR_25_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r4_minus_r1_ASR_13_PostIndex),
    kInstruction_str_al_r2_r4_minus_r1_ASR_13_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r3_plus_r3_ASR_5_PreIndex),
    kInstruction_str_al_r6_r3_plus_r3_ASR_5_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r1_plus_r0_LSR_8_PostIndex),
    kInstruction_str_al_r0_r1_plus_r0_LSR_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r0_minus_r3_LSR_31_PostIndex),
    kInstruction_str_al_r6_r0_minus_r3_LSR_31_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r11_minus_r3_ASR_2_PostIndex),
    kInstruction_str_al_r9_r11_minus_r3_ASR_2_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r13_plus_r13_LSR_26_PostIndex),
    kInstruction_str_al_r1_r13_plus_r13_LSR_26_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r3_minus_r7_ASR_18_PreIndex),
    kInstruction_str_al_r14_r3_minus_r7_ASR_18_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r12_plus_r1_LSR_5_PostIndex),
    kInstruction_str_al_r11_r12_plus_r1_LSR_5_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r6_minus_r0_ASR_7_PostIndex),
    kInstruction_str_al_r4_r6_minus_r0_ASR_7_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r14_plus_r2_ASR_13_PreIndex),
    kInstruction_str_al_r8_r14_plus_r2_ASR_13_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r6_minus_r13_ASR_14_PostIndex),
    kInstruction_str_al_r1_r6_minus_r13_ASR_14_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r9_minus_r3_LSR_3_PreIndex),
    kInstruction_str_al_r8_r9_minus_r3_LSR_3_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r14_plus_r8_ASR_2_PreIndex),
    kInstruction_str_al_r2_r14_plus_r8_ASR_2_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r10_minus_r5_ASR_31_PostIndex),
    kInstruction_str_al_r1_r10_minus_r5_ASR_31_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r14_minus_r13_ASR_28_PostIndex),
    kInstruction_str_al_r2_r14_minus_r13_ASR_28_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r5_plus_r5_LSR_11_PostIndex),
    kInstruction_str_al_r14_r5_plus_r5_LSR_11_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r14_minus_r14_ASR_21_PostIndex),
    kInstruction_str_al_r2_r14_minus_r14_ASR_21_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r1_plus_r0_LSR_3_PostIndex),
    kInstruction_str_al_r12_r1_plus_r0_LSR_3_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r5_plus_r14_LSR_27_PostIndex),
    kInstruction_str_al_r12_r5_plus_r14_LSR_27_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r3_minus_r6_LSR_15_PostIndex),
    kInstruction_str_al_r14_r3_minus_r6_LSR_15_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r4_plus_r1_LSR_29_PostIndex),
    kInstruction_str_al_r1_r4_plus_r1_LSR_29_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r2_plus_r8_ASR_29_PreIndex),
    kInstruction_str_al_r3_r2_plus_r8_ASR_29_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r4_minus_r2_ASR_28_PostIndex),
    kInstruction_str_al_r6_r4_minus_r2_ASR_28_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r11_minus_r13_ASR_31_PreIndex),
    kInstruction_str_al_r7_r11_minus_r13_ASR_31_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r13_plus_r10_ASR_10_PreIndex),
    kInstruction_str_al_r9_r13_plus_r10_ASR_10_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r5_plus_r8_ASR_14_PostIndex),
    kInstruction_str_al_r7_r5_plus_r8_ASR_14_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r7_minus_r13_LSR_8_PostIndex),
    kInstruction_str_al_r12_r7_minus_r13_LSR_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r11_plus_r10_ASR_6_PreIndex),
    kInstruction_str_al_r14_r11_plus_r10_ASR_6_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r5_minus_r5_LSR_22_PreIndex),
    kInstruction_str_al_r1_r5_minus_r5_LSR_22_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r5_plus_r4_ASR_7_PostIndex),
    kInstruction_str_al_r12_r5_plus_r4_ASR_7_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r12_minus_r11_LSR_15_PreIndex),
    kInstruction_str_al_r10_r12_minus_r11_LSR_15_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r2_plus_r5_LSR_26_PreIndex),
    kInstruction_str_al_r13_r2_plus_r5_LSR_26_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r1_minus_r4_LSR_24_PostIndex),
    kInstruction_str_al_r7_r1_minus_r4_LSR_24_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r1_plus_r5_LSR_10_PostIndex),
    kInstruction_str_al_r7_r1_plus_r5_LSR_10_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r0_plus_r3_ASR_27_PostIndex),
    kInstruction_str_al_r7_r0_plus_r3_ASR_27_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r11_minus_r8_ASR_29_PostIndex),
    kInstruction_str_al_r5_r11_minus_r8_ASR_29_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r12_plus_r7_LSR_29_PostIndex),
    kInstruction_str_al_r9_r12_plus_r7_LSR_29_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r0_minus_r10_LSR_8_PostIndex),
    kInstruction_str_al_r10_r0_minus_r10_LSR_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r12_plus_r14_LSR_8_PostIndex),
    kInstruction_str_al_r5_r12_plus_r14_LSR_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r12_plus_r3_LSR_14_PreIndex),
    kInstruction_str_al_r13_r12_plus_r3_LSR_14_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r13_minus_r7_ASR_4_PreIndex),
    kInstruction_str_al_r4_r13_minus_r7_ASR_4_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r12_minus_r12_ASR_9_PostIndex),
    kInstruction_str_al_r2_r12_minus_r12_ASR_9_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r8_plus_r11_LSR_2_PostIndex),
    kInstruction_str_al_r13_r8_plus_r11_LSR_2_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r5_minus_r12_ASR_6_PostIndex),
    kInstruction_str_al_r6_r5_minus_r12_ASR_6_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r12_plus_r14_ASR_31_PreIndex),
    kInstruction_str_al_r9_r12_plus_r14_ASR_31_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r7_minus_r5_ASR_17_PreIndex),
    kInstruction_str_al_r13_r7_minus_r5_ASR_17_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r4_minus_r8_ASR_32_PostIndex),
    kInstruction_str_al_r14_r4_minus_r8_ASR_32_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r4_plus_r9_LSR_20_PreIndex),
    kInstruction_str_al_r1_r4_plus_r9_LSR_20_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r8_plus_r13_LSR_12_PostIndex),
    kInstruction_str_al_r11_r8_plus_r13_LSR_12_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r0_plus_r4_LSR_22_PreIndex),
    kInstruction_str_al_r3_r0_plus_r4_LSR_22_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r8_plus_r2_LSR_19_PostIndex),
    kInstruction_str_al_r9_r8_plus_r2_LSR_19_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r11_plus_r8_ASR_28_PreIndex),
    kInstruction_str_al_r14_r11_plus_r8_ASR_28_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r10_plus_r5_LSR_28_PostIndex),
    kInstruction_str_al_r6_r10_plus_r5_LSR_28_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r13_minus_r12_ASR_14_PreIndex),
    kInstruction_str_al_r12_r13_minus_r12_ASR_14_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r2_plus_r9_ASR_13_PostIndex),
    kInstruction_str_al_r12_r2_plus_r9_ASR_13_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r13_minus_r8_LSR_14_PreIndex),
    kInstruction_str_al_r11_r13_minus_r8_LSR_14_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r4_minus_r11_LSR_1_PreIndex),
    kInstruction_str_al_r6_r4_minus_r11_LSR_1_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r6_plus_r1_ASR_14_PostIndex),
    kInstruction_str_al_r3_r6_plus_r1_ASR_14_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r9_minus_r0_ASR_23_PostIndex),
    kInstruction_str_al_r14_r9_minus_r0_ASR_23_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r2_plus_r0_ASR_23_PreIndex),
    kInstruction_str_al_r9_r2_plus_r0_ASR_23_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r4_minus_r9_LSR_13_PostIndex),
    kInstruction_str_al_r8_r4_minus_r9_LSR_13_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r12_plus_r5_LSR_12_PostIndex),
    kInstruction_str_al_r0_r12_plus_r5_LSR_12_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r5_plus_r3_ASR_31_PostIndex),
    kInstruction_str_al_r6_r5_plus_r3_ASR_31_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_minus_r11_ASR_16_PostIndex),
    kInstruction_str_al_r0_r2_minus_r11_ASR_16_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r10_plus_r12_ASR_32_PreIndex),
    kInstruction_str_al_r14_r10_plus_r12_ASR_32_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r14_minus_r7_LSR_13_PreIndex),
    kInstruction_str_al_r10_r14_minus_r7_LSR_13_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r10_plus_r13_ASR_32_PostIndex),
    kInstruction_str_al_r2_r10_plus_r13_ASR_32_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r13_minus_r13_ASR_2_PreIndex),
    kInstruction_str_al_r7_r13_minus_r13_ASR_2_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r14_plus_r1_ASR_19_PreIndex),
    kInstruction_str_al_r5_r14_plus_r1_ASR_19_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r8_plus_r8_LSR_12_PreIndex),
    kInstruction_str_al_r7_r8_plus_r8_LSR_12_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r8_plus_r6_ASR_31_PostIndex),
    kInstruction_str_al_r6_r8_plus_r6_ASR_31_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r0_minus_r0_LSR_3_PostIndex),
    kInstruction_str_al_r9_r0_minus_r0_LSR_3_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r14_plus_r6_ASR_31_PostIndex),
    kInstruction_str_al_r13_r14_plus_r6_ASR_31_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r8_minus_r0_ASR_26_PostIndex),
    kInstruction_str_al_r5_r8_minus_r0_ASR_26_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r9_plus_r5_LSR_11_PostIndex),
    kInstruction_str_al_r1_r9_plus_r5_LSR_11_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r12_plus_r12_LSR_22_PostIndex),
    kInstruction_str_al_r6_r12_plus_r12_LSR_22_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r12_minus_r6_ASR_1_PreIndex),
    kInstruction_str_al_r6_r12_minus_r6_ASR_1_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r1_plus_r14_LSR_30_PostIndex),
    kInstruction_str_al_r4_r1_plus_r14_LSR_30_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r11_plus_r8_ASR_21_PostIndex),
    kInstruction_str_al_r9_r11_plus_r8_ASR_21_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r0_minus_r13_ASR_19_PostIndex),
    kInstruction_str_al_r1_r0_minus_r13_ASR_19_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r12_plus_r13_LSR_19_PostIndex),
    kInstruction_str_al_r10_r12_plus_r13_LSR_19_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r7_minus_r14_LSR_29_PostIndex),
    kInstruction_str_al_r2_r7_minus_r14_LSR_29_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r0_plus_r0_LSR_2_PostIndex),
    kInstruction_str_al_r7_r0_plus_r0_LSR_2_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r8_plus_r7_LSR_2_PostIndex),
    kInstruction_str_al_r2_r8_plus_r7_LSR_2_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r10_minus_r14_LSR_22_PreIndex),
    kInstruction_str_al_r5_r10_minus_r14_LSR_22_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r7_plus_r6_ASR_23_PreIndex),
    kInstruction_str_al_r8_r7_plus_r6_ASR_23_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r13_plus_r10_LSR_10_PreIndex),
    kInstruction_str_al_r6_r13_plus_r10_LSR_10_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_plus_r8_ASR_12_PostIndex),
    kInstruction_str_al_r0_r9_plus_r8_ASR_12_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r13_plus_r5_LSR_20_PreIndex),
    kInstruction_str_al_r14_r13_plus_r5_LSR_20_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r7_plus_r2_LSR_25_PostIndex),
    kInstruction_str_al_r5_r7_plus_r2_LSR_25_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r2_plus_r4_LSR_22_PreIndex),
    kInstruction_str_al_r3_r2_plus_r4_LSR_22_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r10_plus_r5_LSR_27_PreIndex),
    kInstruction_str_al_r14_r10_plus_r5_LSR_27_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r9_plus_r3_LSR_6_PreIndex),
    kInstruction_str_al_r3_r9_plus_r3_LSR_6_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r13_plus_r12_ASR_9_PreIndex),
    kInstruction_str_al_r2_r13_plus_r12_ASR_9_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r0_minus_r1_ASR_26_PostIndex),
    kInstruction_str_al_r2_r0_minus_r1_ASR_26_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r14_plus_r11_ASR_10_PreIndex),
    kInstruction_str_al_r5_r14_plus_r11_ASR_10_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r3_minus_r8_LSR_10_PostIndex),
    kInstruction_str_al_r5_r3_minus_r8_LSR_10_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r3_minus_r5_LSR_26_PreIndex),
    kInstruction_str_al_r12_r3_minus_r5_LSR_26_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r5_minus_r10_ASR_8_PreIndex),
    kInstruction_str_al_r3_r5_minus_r10_ASR_8_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r14_minus_r12_ASR_8_PreIndex),
    kInstruction_str_al_r1_r14_minus_r12_ASR_8_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r8_minus_r9_ASR_9_PreIndex),
    kInstruction_str_al_r7_r8_minus_r9_ASR_9_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r2_plus_r3_LSR_23_PostIndex),
    kInstruction_str_al_r7_r2_plus_r3_LSR_23_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r12_plus_r13_ASR_2_PreIndex),
    kInstruction_str_al_r13_r12_plus_r13_ASR_2_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_plus_r2_ASR_32_PreIndex),
    kInstruction_str_al_r0_r6_plus_r2_ASR_32_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r14_plus_r1_LSR_16_PreIndex),
    kInstruction_str_al_r0_r14_plus_r1_LSR_16_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r0_minus_r13_LSR_18_PreIndex),
    kInstruction_str_al_r9_r0_minus_r13_LSR_18_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r14_minus_r4_LSR_10_PostIndex),
    kInstruction_str_al_r7_r14_minus_r4_LSR_10_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r4_plus_r3_LSR_10_PostIndex),
    kInstruction_str_al_r2_r4_plus_r3_LSR_10_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r10_minus_r3_ASR_8_PostIndex),
    kInstruction_str_al_r4_r10_minus_r3_ASR_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r7_plus_r13_ASR_22_PostIndex),
    kInstruction_str_al_r0_r7_plus_r13_ASR_22_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r0_plus_r10_LSR_29_PreIndex),
    kInstruction_str_al_r9_r0_plus_r10_LSR_29_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r0_plus_r7_ASR_6_PostIndex),
    kInstruction_str_al_r6_r0_plus_r7_ASR_6_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r11_minus_r2_ASR_6_PreIndex),
    kInstruction_str_al_r14_r11_minus_r2_ASR_6_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r9_minus_r9_ASR_25_PostIndex),
    kInstruction_str_al_r6_r9_minus_r9_ASR_25_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r1_minus_r11_LSR_7_PreIndex),
    kInstruction_str_al_r13_r1_minus_r11_LSR_7_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r9_minus_r6_ASR_21_PostIndex),
    kInstruction_str_al_r12_r9_minus_r6_ASR_21_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r6_minus_r11_ASR_1_PreIndex),
    kInstruction_str_al_r8_r6_minus_r11_ASR_1_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r13_minus_r11_LSR_20_PostIndex),
    kInstruction_str_al_r0_r13_minus_r11_LSR_20_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r3_minus_r8_ASR_25_PreIndex),
    kInstruction_str_al_r12_r3_minus_r8_ASR_25_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r7_plus_r8_ASR_16_PostIndex),
    kInstruction_str_al_r10_r7_plus_r8_ASR_16_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r4_plus_r13_LSR_27_PreIndex),
    kInstruction_str_al_r10_r4_plus_r13_LSR_27_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r10_plus_r7_ASR_29_PreIndex),
    kInstruction_str_al_r8_r10_plus_r7_ASR_29_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r9_minus_r1_LSR_30_PostIndex),
    kInstruction_str_al_r2_r9_minus_r1_LSR_30_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r2_plus_r1_LSR_9_PreIndex),
    kInstruction_str_al_r9_r2_plus_r1_LSR_9_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r4_plus_r4_ASR_20_PreIndex),
    kInstruction_str_al_r13_r4_plus_r4_ASR_20_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r1_minus_r7_ASR_32_PostIndex),
    kInstruction_str_al_r3_r1_minus_r7_ASR_32_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r5_minus_r5_ASR_9_PreIndex),
    kInstruction_str_al_r0_r5_minus_r5_ASR_9_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r6_minus_r9_LSR_23_PostIndex),
    kInstruction_str_al_r11_r6_minus_r9_LSR_23_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r8_plus_r4_ASR_25_PostIndex),
    kInstruction_str_al_r12_r8_plus_r4_ASR_25_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r5_minus_r5_LSR_26_PostIndex),
    kInstruction_str_al_r3_r5_minus_r5_LSR_26_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r0_minus_r0_LSR_21_PostIndex),
    kInstruction_str_al_r1_r0_minus_r0_LSR_21_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r8_plus_r9_LSR_6_PostIndex),
    kInstruction_str_al_r13_r8_plus_r9_LSR_6_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r7_plus_r6_LSR_28_PreIndex),
    kInstruction_str_al_r1_r7_plus_r6_LSR_28_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r4_plus_r13_LSR_14_PostIndex),
    kInstruction_str_al_r14_r4_plus_r13_LSR_14_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r3_plus_r11_ASR_21_PreIndex),
    kInstruction_str_al_r14_r3_plus_r11_ASR_21_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r6_plus_r11_LSR_6_PreIndex),
    kInstruction_str_al_r10_r6_plus_r11_LSR_6_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r7_plus_r11_ASR_25_PreIndex),
    kInstruction_str_al_r13_r7_plus_r11_ASR_25_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r2_minus_r8_LSR_13_PreIndex),
    kInstruction_str_al_r3_r2_minus_r8_LSR_13_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r14_minus_r13_LSR_4_PostIndex),
    kInstruction_str_al_r8_r14_minus_r13_LSR_4_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r9_minus_r2_LSR_26_PostIndex),
    kInstruction_str_al_r2_r9_minus_r2_LSR_26_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r14_minus_r13_ASR_27_PostIndex),
    kInstruction_str_al_r11_r14_minus_r13_ASR_27_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r8_minus_r8_LSR_6_PreIndex),
    kInstruction_str_al_r2_r8_minus_r8_LSR_6_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r11_minus_r2_ASR_12_PreIndex),
    kInstruction_str_al_r10_r11_minus_r2_ASR_12_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r9_plus_r8_ASR_2_PreIndex),
    kInstruction_str_al_r11_r9_plus_r8_ASR_2_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r7_minus_r12_ASR_7_PreIndex),
    kInstruction_str_al_r11_r7_minus_r12_ASR_7_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r7_plus_r8_ASR_3_PreIndex),
    kInstruction_str_al_r8_r7_plus_r8_ASR_3_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r12_minus_r2_LSR_28_PostIndex),
    kInstruction_str_al_r5_r12_minus_r2_LSR_28_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r3_minus_r13_ASR_15_PreIndex),
    kInstruction_str_al_r4_r3_minus_r13_ASR_15_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r11_minus_r11_LSR_17_PostIndex),
    kInstruction_str_al_r7_r11_minus_r11_LSR_17_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r1_minus_r10_LSR_18_PostIndex),
    kInstruction_str_al_r7_r1_minus_r10_LSR_18_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r11_minus_r2_ASR_25_PreIndex),
    kInstruction_str_al_r1_r11_minus_r2_ASR_25_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r0_minus_r4_ASR_19_PostIndex),
    kInstruction_str_al_r4_r0_minus_r4_ASR_19_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r12_minus_r10_LSR_32_PostIndex),
    kInstruction_str_al_r4_r12_minus_r10_LSR_32_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r11_plus_r9_ASR_31_PostIndex),
    kInstruction_str_al_r0_r11_plus_r9_ASR_31_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r13_r11_plus_r14_ASR_18_PostIndex),
    kInstruction_str_al_r13_r11_plus_r14_ASR_18_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r6_plus_r11_LSR_2_PostIndex),
    kInstruction_str_al_r4_r6_plus_r11_LSR_2_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r12_plus_r13_LSR_15_PreIndex),
    kInstruction_str_al_r11_r12_plus_r13_LSR_15_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r10_minus_r3_LSR_10_PreIndex),
    kInstruction_str_al_r9_r10_minus_r3_LSR_10_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r10_plus_r12_LSR_6_PostIndex),
    kInstruction_str_al_r7_r10_plus_r12_LSR_6_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r2_plus_r0_LSR_1_PreIndex),
    kInstruction_str_al_r0_r2_plus_r0_LSR_1_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r5_minus_r12_LSR_14_PreIndex),
    kInstruction_str_al_r1_r5_minus_r12_LSR_14_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r2_minus_r3_LSR_16_PostIndex),
    kInstruction_str_al_r6_r2_minus_r3_LSR_16_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r7_r5_plus_r3_LSR_25_PostIndex),
    kInstruction_str_al_r7_r5_plus_r3_LSR_25_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r10_plus_r1_LSR_6_PostIndex),
    kInstruction_str_al_r5_r10_plus_r1_LSR_6_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r3_minus_r4_ASR_10_PreIndex),
    kInstruction_str_al_r6_r3_minus_r4_ASR_10_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r10_plus_r9_ASR_13_PostIndex),
    kInstruction_str_al_r2_r10_plus_r9_ASR_13_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r0_plus_r4_LSR_16_PostIndex),
    kInstruction_str_al_r6_r0_plus_r4_LSR_16_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r0_minus_r4_LSR_19_PostIndex),
    kInstruction_str_al_r3_r0_minus_r4_LSR_19_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r2_minus_r6_LSR_11_PostIndex),
    kInstruction_str_al_r8_r2_minus_r6_LSR_11_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r6_plus_r14_LSR_21_PreIndex),
    kInstruction_str_al_r0_r6_plus_r14_LSR_21_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r12_plus_r5_ASR_20_PostIndex),
    kInstruction_str_al_r9_r12_plus_r5_ASR_20_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r14_minus_r11_ASR_17_PreIndex),
    kInstruction_str_al_r9_r14_minus_r11_ASR_17_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r8_minus_r14_LSR_22_PostIndex),
    kInstruction_str_al_r1_r8_minus_r14_LSR_22_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r3_minus_r5_LSR_27_PreIndex),
    kInstruction_str_al_r5_r3_minus_r5_LSR_27_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r9_plus_r4_LSR_31_PreIndex),
    kInstruction_str_al_r14_r9_plus_r4_LSR_31_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r12_plus_r3_LSR_12_PreIndex),
    kInstruction_str_al_r8_r12_plus_r3_LSR_12_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r4_minus_r9_ASR_16_PostIndex),
    kInstruction_str_al_r6_r4_minus_r9_ASR_16_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r6_plus_r2_LSR_11_PreIndex),
    kInstruction_str_al_r4_r6_plus_r2_LSR_11_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r7_plus_r0_ASR_11_PostIndex),
    kInstruction_str_al_r11_r7_plus_r0_ASR_11_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r11_minus_r5_LSR_25_PostIndex),
    kInstruction_str_al_r8_r11_minus_r5_LSR_25_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r14_minus_r12_LSR_5_PreIndex),
    kInstruction_str_al_r3_r14_minus_r12_LSR_5_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r10_r7_minus_r3_ASR_23_PostIndex),
    kInstruction_str_al_r10_r7_minus_r3_ASR_23_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r5_minus_r6_ASR_24_PreIndex),
    kInstruction_str_al_r12_r5_minus_r6_ASR_24_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r8_r12_minus_r2_ASR_30_PreIndex),
    kInstruction_str_al_r8_r12_minus_r2_ASR_30_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r9_minus_r12_LSR_8_PostIndex),
    kInstruction_str_al_r1_r9_minus_r12_LSR_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r5_r11_plus_r13_LSR_3_PostIndex),
    kInstruction_str_al_r5_r11_plus_r13_LSR_3_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r14_r7_plus_r13_LSR_21_PostIndex),
    kInstruction_str_al_r14_r7_plus_r13_LSR_21_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r4_minus_r10_LSR_15_PreIndex),
    kInstruction_str_al_r9_r4_minus_r10_LSR_15_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r11_r9_plus_r1_LSR_23_PostIndex),
    kInstruction_str_al_r11_r9_plus_r1_LSR_23_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r3_r9_minus_r7_ASR_15_PreIndex),
    kInstruction_str_al_r3_r9_minus_r7_ASR_15_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r4_minus_r8_ASR_13_PostIndex),
    kInstruction_str_al_r0_r4_minus_r8_ASR_13_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r12_r11_minus_r12_LSR_31_PostIndex),
    kInstruction_str_al_r12_r11_minus_r12_LSR_31_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r12_minus_r10_LSR_17_PostIndex),
    kInstruction_str_al_r9_r12_minus_r10_LSR_17_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r10_minus_r6_LSR_8_PostIndex),
    kInstruction_str_al_r6_r10_minus_r6_LSR_8_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r4_r1_plus_r9_ASR_22_PostIndex),
    kInstruction_str_al_r4_r1_plus_r9_ASR_22_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r1_r11_minus_r12_ASR_21_PostIndex),
    kInstruction_str_al_r1_r11_minus_r12_ASR_21_PostIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r7_plus_r4_ASR_14_PreIndex),
    kInstruction_str_al_r2_r7_plus_r4_ASR_14_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r2_r4_minus_r9_ASR_16_PreIndex),
    kInstruction_str_al_r2_r4_minus_r9_ASR_16_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r9_r10_plus_r4_LSR_30_PreIndex),
    kInstruction_str_al_r9_r10_plus_r4_LSR_30_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r6_r1_plus_r3_LSR_32_PreIndex),
    kInstruction_str_al_r6_r1_plus_r3_LSR_32_PreIndex,
  },
  {
    ARRAY_SIZE(kInstruction_str_al_r0_r9_plus_r14_ASR_29_PostIndex),
    kInstruction_str_al_r0_r9_plus_r14_ASR_29_PostIndex,
  },
};

#endif  // VIXL_ASSEMBLER_COND_RD_MEMOP_RS_SHIFT_AMOUNT_1TO32_STR_A32_H_
