// Copyright 2015, VIXL authors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//   * Neither the name of ARM Limited nor the names of its contributors may be
//     used to endorse or promote products derived from this software without
//     specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// ---------------------------------------------------------------------
// This file is auto generated using tools/generate_simulator_traces.py.
//
// PLEASE DO NOT EDIT.
// ---------------------------------------------------------------------

#ifndef VIXL_ASSEMBLER_COND_RD_OPERAND_RN_SHIFT_AMOUNT_1TO31_IN_IT_BLOCK_MOV_T32_H_
#define VIXL_ASSEMBLER_COND_RD_OPERAND_RN_SHIFT_AMOUNT_1TO31_IN_IT_BLOCK_MOV_T32_H_

const byte kInstruction_mov_hi_r2_r4_LSL_10[] = {
  0x88, 0xbf, 0xa2, 0x02 // It hi; mov hi r2 r4 LSL 10
};
const byte kInstruction_mov_cs_r6_r2_LSL_8[] = {
  0x28, 0xbf, 0x16, 0x02 // It cs; mov cs r6 r2 LSL 8
};
const byte kInstruction_mov_lt_r5_r3_LSL_21[] = {
  0xb8, 0xbf, 0x5d, 0x05 // It lt; mov lt r5 r3 LSL 21
};
const byte kInstruction_mov_ge_r5_r0_LSL_3[] = {
  0xa8, 0xbf, 0xc5, 0x00 // It ge; mov ge r5 r0 LSL 3
};
const byte kInstruction_mov_mi_r4_r1_LSL_4[] = {
  0x48, 0xbf, 0x0c, 0x01 // It mi; mov mi r4 r1 LSL 4
};
const byte kInstruction_mov_ge_r6_r6_LSL_28[] = {
  0xa8, 0xbf, 0x36, 0x07 // It ge; mov ge r6 r6 LSL 28
};
const byte kInstruction_mov_lt_r4_r2_LSL_31[] = {
  0xb8, 0xbf, 0xd4, 0x07 // It lt; mov lt r4 r2 LSL 31
};
const byte kInstruction_mov_eq_r0_r3_LSL_17[] = {
  0x08, 0xbf, 0x58, 0x04 // It eq; mov eq r0 r3 LSL 17
};
const byte kInstruction_mov_vc_r2_r2_LSL_22[] = {
  0x78, 0xbf, 0x92, 0x05 // It vc; mov vc r2 r2 LSL 22
};
const byte kInstruction_mov_ge_r0_r1_LSL_6[] = {
  0xa8, 0xbf, 0x88, 0x01 // It ge; mov ge r0 r1 LSL 6
};
const byte kInstruction_mov_vs_r6_r0_LSL_26[] = {
  0x68, 0xbf, 0x86, 0x06 // It vs; mov vs r6 r0 LSL 26
};
const byte kInstruction_mov_vs_r3_r1_LSL_26[] = {
  0x68, 0xbf, 0x8b, 0x06 // It vs; mov vs r3 r1 LSL 26
};
const byte kInstruction_mov_cs_r7_r4_LSL_28[] = {
  0x28, 0xbf, 0x27, 0x07 // It cs; mov cs r7 r4 LSL 28
};
const byte kInstruction_mov_le_r3_r0_LSL_28[] = {
  0xd8, 0xbf, 0x03, 0x07 // It le; mov le r3 r0 LSL 28
};
const byte kInstruction_mov_le_r4_r0_LSL_23[] = {
  0xd8, 0xbf, 0xc4, 0x05 // It le; mov le r4 r0 LSL 23
};
const byte kInstruction_mov_gt_r5_r3_LSL_1[] = {
  0xc8, 0xbf, 0x5d, 0x00 // It gt; mov gt r5 r3 LSL 1
};
const byte kInstruction_mov_hi_r7_r1_LSL_14[] = {
  0x88, 0xbf, 0x8f, 0x03 // It hi; mov hi r7 r1 LSL 14
};
const byte kInstruction_mov_cs_r1_r4_LSL_27[] = {
  0x28, 0xbf, 0xe1, 0x06 // It cs; mov cs r1 r4 LSL 27
};
const byte kInstruction_mov_cc_r1_r0_LSL_2[] = {
  0x38, 0xbf, 0x81, 0x00 // It cc; mov cc r1 r0 LSL 2
};
const byte kInstruction_mov_le_r5_r2_LSL_1[] = {
  0xd8, 0xbf, 0x55, 0x00 // It le; mov le r5 r2 LSL 1
};
const byte kInstruction_mov_gt_r6_r4_LSL_23[] = {
  0xc8, 0xbf, 0xe6, 0x05 // It gt; mov gt r6 r4 LSL 23
};
const byte kInstruction_mov_lt_r0_r0_LSL_27[] = {
  0xb8, 0xbf, 0xc0, 0x06 // It lt; mov lt r0 r0 LSL 27
};
const byte kInstruction_mov_le_r5_r4_LSL_6[] = {
  0xd8, 0xbf, 0xa5, 0x01 // It le; mov le r5 r4 LSL 6
};
const byte kInstruction_mov_ls_r2_r0_LSL_14[] = {
  0x98, 0xbf, 0x82, 0x03 // It ls; mov ls r2 r0 LSL 14
};
const byte kInstruction_mov_mi_r3_r5_LSL_31[] = {
  0x48, 0xbf, 0xeb, 0x07 // It mi; mov mi r3 r5 LSL 31
};
const byte kInstruction_mov_le_r6_r6_LSL_11[] = {
  0xd8, 0xbf, 0xf6, 0x02 // It le; mov le r6 r6 LSL 11
};
const byte kInstruction_mov_le_r1_r4_LSL_30[] = {
  0xd8, 0xbf, 0xa1, 0x07 // It le; mov le r1 r4 LSL 30
};
const byte kInstruction_mov_le_r3_r1_LSL_16[] = {
  0xd8, 0xbf, 0x0b, 0x04 // It le; mov le r3 r1 LSL 16
};
const byte kInstruction_mov_eq_r3_r6_LSL_12[] = {
  0x08, 0xbf, 0x33, 0x03 // It eq; mov eq r3 r6 LSL 12
};
const byte kInstruction_mov_lt_r4_r5_LSL_25[] = {
  0xb8, 0xbf, 0x6c, 0x06 // It lt; mov lt r4 r5 LSL 25
};
const byte kInstruction_mov_ge_r1_r3_LSL_10[] = {
  0xa8, 0xbf, 0x99, 0x02 // It ge; mov ge r1 r3 LSL 10
};
const byte kInstruction_mov_vc_r0_r5_LSL_20[] = {
  0x78, 0xbf, 0x28, 0x05 // It vc; mov vc r0 r5 LSL 20
};
const byte kInstruction_mov_cc_r0_r7_LSL_6[] = {
  0x38, 0xbf, 0xb8, 0x01 // It cc; mov cc r0 r7 LSL 6
};
const byte kInstruction_mov_vc_r6_r0_LSL_13[] = {
  0x78, 0xbf, 0x46, 0x03 // It vc; mov vc r6 r0 LSL 13
};
const byte kInstruction_mov_ls_r7_r7_LSL_11[] = {
  0x98, 0xbf, 0xff, 0x02 // It ls; mov ls r7 r7 LSL 11
};
const byte kInstruction_mov_ls_r2_r3_LSL_28[] = {
  0x98, 0xbf, 0x1a, 0x07 // It ls; mov ls r2 r3 LSL 28
};
const byte kInstruction_mov_le_r2_r1_LSL_20[] = {
  0xd8, 0xbf, 0x0a, 0x05 // It le; mov le r2 r1 LSL 20
};
const byte kInstruction_mov_ne_r4_r5_LSL_8[] = {
  0x18, 0xbf, 0x2c, 0x02 // It ne; mov ne r4 r5 LSL 8
};
const byte kInstruction_mov_vc_r5_r6_LSL_5[] = {
  0x78, 0xbf, 0x75, 0x01 // It vc; mov vc r5 r6 LSL 5
};
const byte kInstruction_mov_cc_r6_r1_LSL_14[] = {
  0x38, 0xbf, 0x8e, 0x03 // It cc; mov cc r6 r1 LSL 14
};
const byte kInstruction_mov_cc_r1_r4_LSL_8[] = {
  0x38, 0xbf, 0x21, 0x02 // It cc; mov cc r1 r4 LSL 8
};
const byte kInstruction_mov_ls_r0_r4_LSL_24[] = {
  0x98, 0xbf, 0x20, 0x06 // It ls; mov ls r0 r4 LSL 24
};
const byte kInstruction_mov_ls_r4_r4_LSL_12[] = {
  0x98, 0xbf, 0x24, 0x03 // It ls; mov ls r4 r4 LSL 12
};
const byte kInstruction_mov_eq_r6_r2_LSL_19[] = {
  0x08, 0xbf, 0xd6, 0x04 // It eq; mov eq r6 r2 LSL 19
};
const byte kInstruction_mov_cc_r7_r2_LSL_16[] = {
  0x38, 0xbf, 0x17, 0x04 // It cc; mov cc r7 r2 LSL 16
};
const byte kInstruction_mov_lt_r7_r3_LSL_28[] = {
  0xb8, 0xbf, 0x1f, 0x07 // It lt; mov lt r7 r3 LSL 28
};
const byte kInstruction_mov_hi_r2_r1_LSL_5[] = {
  0x88, 0xbf, 0x4a, 0x01 // It hi; mov hi r2 r1 LSL 5
};
const byte kInstruction_mov_ge_r5_r3_LSL_26[] = {
  0xa8, 0xbf, 0x9d, 0x06 // It ge; mov ge r5 r3 LSL 26
};
const byte kInstruction_mov_mi_r0_r0_LSL_1[] = {
  0x48, 0xbf, 0x40, 0x00 // It mi; mov mi r0 r0 LSL 1
};
const byte kInstruction_mov_vc_r6_r3_LSL_17[] = {
  0x78, 0xbf, 0x5e, 0x04 // It vc; mov vc r6 r3 LSL 17
};
const byte kInstruction_mov_mi_r1_r4_LSL_16[] = {
  0x48, 0xbf, 0x21, 0x04 // It mi; mov mi r1 r4 LSL 16
};
const byte kInstruction_mov_ge_r1_r3_LSL_20[] = {
  0xa8, 0xbf, 0x19, 0x05 // It ge; mov ge r1 r3 LSL 20
};
const byte kInstruction_mov_eq_r1_r3_LSL_9[] = {
  0x08, 0xbf, 0x59, 0x02 // It eq; mov eq r1 r3 LSL 9
};
const byte kInstruction_mov_ne_r0_r4_LSL_27[] = {
  0x18, 0xbf, 0xe0, 0x06 // It ne; mov ne r0 r4 LSL 27
};
const byte kInstruction_mov_le_r4_r1_LSL_12[] = {
  0xd8, 0xbf, 0x0c, 0x03 // It le; mov le r4 r1 LSL 12
};
const byte kInstruction_mov_ne_r7_r7_LSL_26[] = {
  0x18, 0xbf, 0xbf, 0x06 // It ne; mov ne r7 r7 LSL 26
};
const byte kInstruction_mov_hi_r5_r2_LSL_12[] = {
  0x88, 0xbf, 0x15, 0x03 // It hi; mov hi r5 r2 LSL 12
};
const byte kInstruction_mov_pl_r3_r4_LSL_9[] = {
  0x58, 0xbf, 0x63, 0x02 // It pl; mov pl r3 r4 LSL 9
};
const byte kInstruction_mov_cs_r5_r3_LSL_12[] = {
  0x28, 0xbf, 0x1d, 0x03 // It cs; mov cs r5 r3 LSL 12
};
const byte kInstruction_mov_vs_r1_r1_LSL_28[] = {
  0x68, 0xbf, 0x09, 0x07 // It vs; mov vs r1 r1 LSL 28
};
const byte kInstruction_mov_lt_r4_r6_LSL_19[] = {
  0xb8, 0xbf, 0xf4, 0x04 // It lt; mov lt r4 r6 LSL 19
};
const byte kInstruction_mov_vc_r6_r6_LSL_31[] = {
  0x78, 0xbf, 0xf6, 0x07 // It vc; mov vc r6 r6 LSL 31
};
const byte kInstruction_mov_cs_r4_r5_LSL_27[] = {
  0x28, 0xbf, 0xec, 0x06 // It cs; mov cs r4 r5 LSL 27
};
const byte kInstruction_mov_gt_r2_r6_LSL_2[] = {
  0xc8, 0xbf, 0xb2, 0x00 // It gt; mov gt r2 r6 LSL 2
};
const byte kInstruction_mov_vc_r6_r4_LSL_20[] = {
  0x78, 0xbf, 0x26, 0x05 // It vc; mov vc r6 r4 LSL 20
};
const byte kInstruction_mov_ne_r1_r5_LSL_13[] = {
  0x18, 0xbf, 0x69, 0x03 // It ne; mov ne r1 r5 LSL 13
};
const byte kInstruction_mov_vs_r6_r2_LSL_4[] = {
  0x68, 0xbf, 0x16, 0x01 // It vs; mov vs r6 r2 LSL 4
};
const byte kInstruction_mov_lt_r5_r3_LSL_20[] = {
  0xb8, 0xbf, 0x1d, 0x05 // It lt; mov lt r5 r3 LSL 20
};
const byte kInstruction_mov_le_r3_r0_LSL_18[] = {
  0xd8, 0xbf, 0x83, 0x04 // It le; mov le r3 r0 LSL 18
};
const byte kInstruction_mov_ge_r1_r3_LSL_25[] = {
  0xa8, 0xbf, 0x59, 0x06 // It ge; mov ge r1 r3 LSL 25
};
const byte kInstruction_mov_cs_r4_r5_LSL_23[] = {
  0x28, 0xbf, 0xec, 0x05 // It cs; mov cs r4 r5 LSL 23
};
const byte kInstruction_mov_mi_r0_r3_LSL_30[] = {
  0x48, 0xbf, 0x98, 0x07 // It mi; mov mi r0 r3 LSL 30
};
const byte kInstruction_mov_ge_r4_r5_LSL_19[] = {
  0xa8, 0xbf, 0xec, 0x04 // It ge; mov ge r4 r5 LSL 19
};
const byte kInstruction_mov_gt_r3_r1_LSL_20[] = {
  0xc8, 0xbf, 0x0b, 0x05 // It gt; mov gt r3 r1 LSL 20
};
const byte kInstruction_mov_le_r0_r5_LSL_20[] = {
  0xd8, 0xbf, 0x28, 0x05 // It le; mov le r0 r5 LSL 20
};
const byte kInstruction_mov_vs_r0_r6_LSL_24[] = {
  0x68, 0xbf, 0x30, 0x06 // It vs; mov vs r0 r6 LSL 24
};
const byte kInstruction_mov_le_r5_r6_LSL_30[] = {
  0xd8, 0xbf, 0xb5, 0x07 // It le; mov le r5 r6 LSL 30
};
const byte kInstruction_mov_ne_r6_r5_LSL_2[] = {
  0x18, 0xbf, 0xae, 0x00 // It ne; mov ne r6 r5 LSL 2
};
const byte kInstruction_mov_gt_r6_r3_LSL_2[] = {
  0xc8, 0xbf, 0x9e, 0x00 // It gt; mov gt r6 r3 LSL 2
};
const byte kInstruction_mov_ls_r4_r4_LSL_8[] = {
  0x98, 0xbf, 0x24, 0x02 // It ls; mov ls r4 r4 LSL 8
};
const byte kInstruction_mov_cs_r5_r3_LSL_5[] = {
  0x28, 0xbf, 0x5d, 0x01 // It cs; mov cs r5 r3 LSL 5
};
const byte kInstruction_mov_ne_r5_r4_LSL_23[] = {
  0x18, 0xbf, 0xe5, 0x05 // It ne; mov ne r5 r4 LSL 23
};
const byte kInstruction_mov_lt_r0_r0_LSL_14[] = {
  0xb8, 0xbf, 0x80, 0x03 // It lt; mov lt r0 r0 LSL 14
};
const byte kInstruction_mov_cc_r3_r4_LSL_12[] = {
  0x38, 0xbf, 0x23, 0x03 // It cc; mov cc r3 r4 LSL 12
};
const byte kInstruction_mov_vc_r3_r7_LSL_25[] = {
  0x78, 0xbf, 0x7b, 0x06 // It vc; mov vc r3 r7 LSL 25
};
const byte kInstruction_mov_ge_r2_r3_LSL_21[] = {
  0xa8, 0xbf, 0x5a, 0x05 // It ge; mov ge r2 r3 LSL 21
};
const byte kInstruction_mov_gt_r3_r3_LSL_17[] = {
  0xc8, 0xbf, 0x5b, 0x04 // It gt; mov gt r3 r3 LSL 17
};
const byte kInstruction_mov_mi_r3_r5_LSL_21[] = {
  0x48, 0xbf, 0x6b, 0x05 // It mi; mov mi r3 r5 LSL 21
};
const byte kInstruction_mov_hi_r2_r5_LSL_30[] = {
  0x88, 0xbf, 0xaa, 0x07 // It hi; mov hi r2 r5 LSL 30
};
const byte kInstruction_mov_ls_r2_r5_LSL_12[] = {
  0x98, 0xbf, 0x2a, 0x03 // It ls; mov ls r2 r5 LSL 12
};
const byte kInstruction_mov_cc_r4_r6_LSL_1[] = {
  0x38, 0xbf, 0x74, 0x00 // It cc; mov cc r4 r6 LSL 1
};
const byte kInstruction_mov_cc_r1_r1_LSL_24[] = {
  0x38, 0xbf, 0x09, 0x06 // It cc; mov cc r1 r1 LSL 24
};
const byte kInstruction_mov_vc_r3_r2_LSL_24[] = {
  0x78, 0xbf, 0x13, 0x06 // It vc; mov vc r3 r2 LSL 24
};
const byte kInstruction_mov_le_r0_r5_LSL_18[] = {
  0xd8, 0xbf, 0xa8, 0x04 // It le; mov le r0 r5 LSL 18
};
const byte kInstruction_mov_eq_r7_r1_LSL_23[] = {
  0x08, 0xbf, 0xcf, 0x05 // It eq; mov eq r7 r1 LSL 23
};
const byte kInstruction_mov_lt_r2_r3_LSL_3[] = {
  0xb8, 0xbf, 0xda, 0x00 // It lt; mov lt r2 r3 LSL 3
};
const byte kInstruction_mov_pl_r2_r0_LSL_27[] = {
  0x58, 0xbf, 0xc2, 0x06 // It pl; mov pl r2 r0 LSL 27
};
const byte kInstruction_mov_mi_r5_r4_LSL_19[] = {
  0x48, 0xbf, 0xe5, 0x04 // It mi; mov mi r5 r4 LSL 19
};
const byte kInstruction_mov_gt_r3_r2_LSL_2[] = {
  0xc8, 0xbf, 0x93, 0x00 // It gt; mov gt r3 r2 LSL 2
};
const byte kInstruction_mov_lt_r5_r0_LSL_14[] = {
  0xb8, 0xbf, 0x85, 0x03 // It lt; mov lt r5 r0 LSL 14
};
const byte kInstruction_mov_cc_r4_r3_LSL_25[] = {
  0x38, 0xbf, 0x5c, 0x06 // It cc; mov cc r4 r3 LSL 25
};
const byte kInstruction_mov_cs_r4_r7_LSL_15[] = {
  0x28, 0xbf, 0xfc, 0x03 // It cs; mov cs r4 r7 LSL 15
};
const byte kInstruction_mov_ls_r2_r5_LSL_10[] = {
  0x98, 0xbf, 0xaa, 0x02 // It ls; mov ls r2 r5 LSL 10
};
const byte kInstruction_mov_cc_r2_r6_LSL_17[] = {
  0x38, 0xbf, 0x72, 0x04 // It cc; mov cc r2 r6 LSL 17
};
const byte kInstruction_mov_hi_r6_r3_LSL_12[] = {
  0x88, 0xbf, 0x1e, 0x03 // It hi; mov hi r6 r3 LSL 12
};
const byte kInstruction_mov_cc_r4_r0_LSL_26[] = {
  0x38, 0xbf, 0x84, 0x06 // It cc; mov cc r4 r0 LSL 26
};
const byte kInstruction_mov_ls_r4_r2_LSL_29[] = {
  0x98, 0xbf, 0x54, 0x07 // It ls; mov ls r4 r2 LSL 29
};
const byte kInstruction_mov_ls_r4_r2_LSL_8[] = {
  0x98, 0xbf, 0x14, 0x02 // It ls; mov ls r4 r2 LSL 8
};
const byte kInstruction_mov_vs_r7_r5_LSL_1[] = {
  0x68, 0xbf, 0x6f, 0x00 // It vs; mov vs r7 r5 LSL 1
};
const byte kInstruction_mov_eq_r1_r5_LSL_11[] = {
  0x08, 0xbf, 0xe9, 0x02 // It eq; mov eq r1 r5 LSL 11
};
const byte kInstruction_mov_vc_r4_r2_LSL_24[] = {
  0x78, 0xbf, 0x14, 0x06 // It vc; mov vc r4 r2 LSL 24
};
const byte kInstruction_mov_cs_r7_r2_LSL_15[] = {
  0x28, 0xbf, 0xd7, 0x03 // It cs; mov cs r7 r2 LSL 15
};
const byte kInstruction_mov_vs_r7_r1_LSL_9[] = {
  0x68, 0xbf, 0x4f, 0x02 // It vs; mov vs r7 r1 LSL 9
};
const byte kInstruction_mov_pl_r7_r4_LSL_11[] = {
  0x58, 0xbf, 0xe7, 0x02 // It pl; mov pl r7 r4 LSL 11
};
const byte kInstruction_mov_ne_r3_r6_LSL_19[] = {
  0x18, 0xbf, 0xf3, 0x04 // It ne; mov ne r3 r6 LSL 19
};
const byte kInstruction_mov_ne_r6_r4_LSL_9[] = {
  0x18, 0xbf, 0x66, 0x02 // It ne; mov ne r6 r4 LSL 9
};
const byte kInstruction_mov_le_r1_r7_LSL_13[] = {
  0xd8, 0xbf, 0x79, 0x03 // It le; mov le r1 r7 LSL 13
};
const byte kInstruction_mov_vc_r2_r3_LSL_19[] = {
  0x78, 0xbf, 0xda, 0x04 // It vc; mov vc r2 r3 LSL 19
};
const byte kInstruction_mov_mi_r2_r6_LSL_3[] = {
  0x48, 0xbf, 0xf2, 0x00 // It mi; mov mi r2 r6 LSL 3
};
const byte kInstruction_mov_eq_r3_r0_LSL_30[] = {
  0x08, 0xbf, 0x83, 0x07 // It eq; mov eq r3 r0 LSL 30
};
const byte kInstruction_mov_le_r6_r2_LSL_3[] = {
  0xd8, 0xbf, 0xd6, 0x00 // It le; mov le r6 r2 LSL 3
};
const byte kInstruction_mov_ne_r5_r7_LSL_22[] = {
  0x18, 0xbf, 0xbd, 0x05 // It ne; mov ne r5 r7 LSL 22
};
const byte kInstruction_mov_vs_r7_r4_LSL_19[] = {
  0x68, 0xbf, 0xe7, 0x04 // It vs; mov vs r7 r4 LSL 19
};
const byte kInstruction_mov_eq_r6_r7_LSL_20[] = {
  0x08, 0xbf, 0x3e, 0x05 // It eq; mov eq r6 r7 LSL 20
};
const byte kInstruction_mov_cs_r2_r7_LSL_19[] = {
  0x28, 0xbf, 0xfa, 0x04 // It cs; mov cs r2 r7 LSL 19
};
const byte kInstruction_mov_cs_r3_r1_LSL_17[] = {
  0x28, 0xbf, 0x4b, 0x04 // It cs; mov cs r3 r1 LSL 17
};
const byte kInstruction_mov_vs_r5_r3_LSL_11[] = {
  0x68, 0xbf, 0xdd, 0x02 // It vs; mov vs r5 r3 LSL 11
};
const byte kInstruction_mov_le_r5_r6_LSL_27[] = {
  0xd8, 0xbf, 0xf5, 0x06 // It le; mov le r5 r6 LSL 27
};
const byte kInstruction_mov_vs_r7_r0_LSL_27[] = {
  0x68, 0xbf, 0xc7, 0x06 // It vs; mov vs r7 r0 LSL 27
};
const byte kInstruction_mov_lt_r2_r0_LSL_29[] = {
  0xb8, 0xbf, 0x42, 0x07 // It lt; mov lt r2 r0 LSL 29
};
const byte kInstruction_mov_vc_r0_r0_LSL_11[] = {
  0x78, 0xbf, 0xc0, 0x02 // It vc; mov vc r0 r0 LSL 11
};
const byte kInstruction_mov_cc_r6_r3_LSL_16[] = {
  0x38, 0xbf, 0x1e, 0x04 // It cc; mov cc r6 r3 LSL 16
};
const byte kInstruction_mov_cs_r0_r0_LSL_16[] = {
  0x28, 0xbf, 0x00, 0x04 // It cs; mov cs r0 r0 LSL 16
};
const byte kInstruction_mov_pl_r4_r4_LSL_24[] = {
  0x58, 0xbf, 0x24, 0x06 // It pl; mov pl r4 r4 LSL 24
};
const byte kInstruction_mov_mi_r6_r7_LSL_22[] = {
  0x48, 0xbf, 0xbe, 0x05 // It mi; mov mi r6 r7 LSL 22
};
const byte kInstruction_mov_vc_r1_r6_LSL_15[] = {
  0x78, 0xbf, 0xf1, 0x03 // It vc; mov vc r1 r6 LSL 15
};
const byte kInstruction_mov_ls_r6_r7_LSL_18[] = {
  0x98, 0xbf, 0xbe, 0x04 // It ls; mov ls r6 r7 LSL 18
};
const byte kInstruction_mov_vc_r1_r1_LSL_22[] = {
  0x78, 0xbf, 0x89, 0x05 // It vc; mov vc r1 r1 LSL 22
};
const byte kInstruction_mov_lt_r3_r0_LSL_13[] = {
  0xb8, 0xbf, 0x43, 0x03 // It lt; mov lt r3 r0 LSL 13
};
const byte kInstruction_mov_pl_r2_r2_LSL_25[] = {
  0x58, 0xbf, 0x52, 0x06 // It pl; mov pl r2 r2 LSL 25
};
const byte kInstruction_mov_hi_r3_r2_LSL_30[] = {
  0x88, 0xbf, 0x93, 0x07 // It hi; mov hi r3 r2 LSL 30
};
const byte kInstruction_mov_vs_r5_r0_LSL_1[] = {
  0x68, 0xbf, 0x45, 0x00 // It vs; mov vs r5 r0 LSL 1
};
const byte kInstruction_mov_gt_r5_r0_LSL_6[] = {
  0xc8, 0xbf, 0x85, 0x01 // It gt; mov gt r5 r0 LSL 6
};
const byte kInstruction_mov_cs_r7_r0_LSL_3[] = {
  0x28, 0xbf, 0xc7, 0x00 // It cs; mov cs r7 r0 LSL 3
};
const byte kInstruction_mov_ne_r4_r6_LSL_9[] = {
  0x18, 0xbf, 0x74, 0x02 // It ne; mov ne r4 r6 LSL 9
};
const byte kInstruction_mov_ge_r2_r0_LSL_4[] = {
  0xa8, 0xbf, 0x02, 0x01 // It ge; mov ge r2 r0 LSL 4
};
const byte kInstruction_mov_pl_r6_r5_LSL_16[] = {
  0x58, 0xbf, 0x2e, 0x04 // It pl; mov pl r6 r5 LSL 16
};
const byte kInstruction_mov_lt_r5_r6_LSL_8[] = {
  0xb8, 0xbf, 0x35, 0x02 // It lt; mov lt r5 r6 LSL 8
};
const byte kInstruction_mov_hi_r2_r2_LSL_2[] = {
  0x88, 0xbf, 0x92, 0x00 // It hi; mov hi r2 r2 LSL 2
};
const byte kInstruction_mov_pl_r6_r2_LSL_16[] = {
  0x58, 0xbf, 0x16, 0x04 // It pl; mov pl r6 r2 LSL 16
};
const byte kInstruction_mov_cs_r2_r2_LSL_14[] = {
  0x28, 0xbf, 0x92, 0x03 // It cs; mov cs r2 r2 LSL 14
};
const byte kInstruction_mov_mi_r2_r4_LSL_25[] = {
  0x48, 0xbf, 0x62, 0x06 // It mi; mov mi r2 r4 LSL 25
};
const byte kInstruction_mov_mi_r0_r5_LSL_4[] = {
  0x48, 0xbf, 0x28, 0x01 // It mi; mov mi r0 r5 LSL 4
};
const byte kInstruction_mov_hi_r0_r6_LSL_6[] = {
  0x88, 0xbf, 0xb0, 0x01 // It hi; mov hi r0 r6 LSL 6
};
const byte kInstruction_mov_cs_r1_r3_LSL_16[] = {
  0x28, 0xbf, 0x19, 0x04 // It cs; mov cs r1 r3 LSL 16
};
const byte kInstruction_mov_le_r0_r6_LSL_25[] = {
  0xd8, 0xbf, 0x70, 0x06 // It le; mov le r0 r6 LSL 25
};
const byte kInstruction_mov_gt_r6_r1_LSL_22[] = {
  0xc8, 0xbf, 0x8e, 0x05 // It gt; mov gt r6 r1 LSL 22
};
const byte kInstruction_mov_cc_r4_r2_LSL_6[] = {
  0x38, 0xbf, 0x94, 0x01 // It cc; mov cc r4 r2 LSL 6
};
const byte kInstruction_mov_eq_r6_r2_LSL_23[] = {
  0x08, 0xbf, 0xd6, 0x05 // It eq; mov eq r6 r2 LSL 23
};
const byte kInstruction_mov_hi_r2_r5_LSL_11[] = {
  0x88, 0xbf, 0xea, 0x02 // It hi; mov hi r2 r5 LSL 11
};
const byte kInstruction_mov_eq_r0_r6_LSL_12[] = {
  0x08, 0xbf, 0x30, 0x03 // It eq; mov eq r0 r6 LSL 12
};
const byte kInstruction_mov_pl_r1_r7_LSL_16[] = {
  0x58, 0xbf, 0x39, 0x04 // It pl; mov pl r1 r7 LSL 16
};
const byte kInstruction_mov_ne_r4_r3_LSL_11[] = {
  0x18, 0xbf, 0xdc, 0x02 // It ne; mov ne r4 r3 LSL 11
};
const byte kInstruction_mov_pl_r1_r1_LSL_25[] = {
  0x58, 0xbf, 0x49, 0x06 // It pl; mov pl r1 r1 LSL 25
};
const byte kInstruction_mov_gt_r3_r0_LSL_1[] = {
  0xc8, 0xbf, 0x43, 0x00 // It gt; mov gt r3 r0 LSL 1
};
const byte kInstruction_mov_cs_r1_r2_LSL_25[] = {
  0x28, 0xbf, 0x51, 0x06 // It cs; mov cs r1 r2 LSL 25
};
const byte kInstruction_mov_pl_r4_r1_LSL_13[] = {
  0x58, 0xbf, 0x4c, 0x03 // It pl; mov pl r4 r1 LSL 13
};
const byte kInstruction_mov_lt_r0_r7_LSL_30[] = {
  0xb8, 0xbf, 0xb8, 0x07 // It lt; mov lt r0 r7 LSL 30
};
const byte kInstruction_mov_vc_r3_r6_LSL_17[] = {
  0x78, 0xbf, 0x73, 0x04 // It vc; mov vc r3 r6 LSL 17
};
const byte kInstruction_mov_ne_r7_r6_LSL_14[] = {
  0x18, 0xbf, 0xb7, 0x03 // It ne; mov ne r7 r6 LSL 14
};
const byte kInstruction_mov_hi_r0_r2_LSL_2[] = {
  0x88, 0xbf, 0x90, 0x00 // It hi; mov hi r0 r2 LSL 2
};
const byte kInstruction_mov_hi_r2_r3_LSL_19[] = {
  0x88, 0xbf, 0xda, 0x04 // It hi; mov hi r2 r3 LSL 19
};
const byte kInstruction_mov_cs_r1_r5_LSL_6[] = {
  0x28, 0xbf, 0xa9, 0x01 // It cs; mov cs r1 r5 LSL 6
};
const byte kInstruction_mov_mi_r6_r7_LSL_9[] = {
  0x48, 0xbf, 0x7e, 0x02 // It mi; mov mi r6 r7 LSL 9
};
const byte kInstruction_mov_ls_r5_r4_LSL_20[] = {
  0x98, 0xbf, 0x25, 0x05 // It ls; mov ls r5 r4 LSL 20
};
const byte kInstruction_mov_gt_r5_r7_LSL_14[] = {
  0xc8, 0xbf, 0xbd, 0x03 // It gt; mov gt r5 r7 LSL 14
};
const byte kInstruction_mov_pl_r0_r3_LSL_2[] = {
  0x58, 0xbf, 0x98, 0x00 // It pl; mov pl r0 r3 LSL 2
};
const byte kInstruction_mov_ls_r0_r3_LSL_16[] = {
  0x98, 0xbf, 0x18, 0x04 // It ls; mov ls r0 r3 LSL 16
};
const byte kInstruction_mov_gt_r7_r7_LSL_26[] = {
  0xc8, 0xbf, 0xbf, 0x06 // It gt; mov gt r7 r7 LSL 26
};
const byte kInstruction_mov_vc_r1_r7_LSL_6[] = {
  0x78, 0xbf, 0xb9, 0x01 // It vc; mov vc r1 r7 LSL 6
};
const byte kInstruction_mov_cs_r6_r4_LSL_31[] = {
  0x28, 0xbf, 0xe6, 0x07 // It cs; mov cs r6 r4 LSL 31
};
const byte kInstruction_mov_ls_r7_r1_LSL_8[] = {
  0x98, 0xbf, 0x0f, 0x02 // It ls; mov ls r7 r1 LSL 8
};
const byte kInstruction_mov_eq_r6_r4_LSL_7[] = {
  0x08, 0xbf, 0xe6, 0x01 // It eq; mov eq r6 r4 LSL 7
};
const byte kInstruction_mov_le_r0_r3_LSL_12[] = {
  0xd8, 0xbf, 0x18, 0x03 // It le; mov le r0 r3 LSL 12
};
const byte kInstruction_mov_eq_r7_r1_LSL_5[] = {
  0x08, 0xbf, 0x4f, 0x01 // It eq; mov eq r7 r1 LSL 5
};
const byte kInstruction_mov_mi_r4_r4_LSL_23[] = {
  0x48, 0xbf, 0xe4, 0x05 // It mi; mov mi r4 r4 LSL 23
};
const byte kInstruction_mov_vc_r6_r2_LSL_20[] = {
  0x78, 0xbf, 0x16, 0x05 // It vc; mov vc r6 r2 LSL 20
};
const byte kInstruction_mov_eq_r1_r1_LSL_1[] = {
  0x08, 0xbf, 0x49, 0x00 // It eq; mov eq r1 r1 LSL 1
};
const byte kInstruction_mov_cc_r0_r0_LSL_20[] = {
  0x38, 0xbf, 0x00, 0x05 // It cc; mov cc r0 r0 LSL 20
};
const byte kInstruction_mov_pl_r5_r4_LSL_26[] = {
  0x58, 0xbf, 0xa5, 0x06 // It pl; mov pl r5 r4 LSL 26
};
const byte kInstruction_mov_vc_r2_r5_LSL_6[] = {
  0x78, 0xbf, 0xaa, 0x01 // It vc; mov vc r2 r5 LSL 6
};
const byte kInstruction_mov_le_r5_r5_LSL_25[] = {
  0xd8, 0xbf, 0x6d, 0x06 // It le; mov le r5 r5 LSL 25
};
const byte kInstruction_mov_le_r5_r0_LSL_26[] = {
  0xd8, 0xbf, 0x85, 0x06 // It le; mov le r5 r0 LSL 26
};
const byte kInstruction_mov_le_r0_r3_LSL_17[] = {
  0xd8, 0xbf, 0x58, 0x04 // It le; mov le r0 r3 LSL 17
};
const byte kInstruction_mov_mi_r5_r5_LSL_29[] = {
  0x48, 0xbf, 0x6d, 0x07 // It mi; mov mi r5 r5 LSL 29
};
const byte kInstruction_mov_vc_r5_r4_LSL_8[] = {
  0x78, 0xbf, 0x25, 0x02 // It vc; mov vc r5 r4 LSL 8
};
const byte kInstruction_mov_ge_r0_r2_LSL_7[] = {
  0xa8, 0xbf, 0xd0, 0x01 // It ge; mov ge r0 r2 LSL 7
};
const byte kInstruction_mov_ne_r5_r5_LSL_29[] = {
  0x18, 0xbf, 0x6d, 0x07 // It ne; mov ne r5 r5 LSL 29
};
const byte kInstruction_mov_ge_r1_r7_LSL_24[] = {
  0xa8, 0xbf, 0x39, 0x06 // It ge; mov ge r1 r7 LSL 24
};
const byte kInstruction_mov_vs_r6_r1_LSL_24[] = {
  0x68, 0xbf, 0x0e, 0x06 // It vs; mov vs r6 r1 LSL 24
};
const byte kInstruction_mov_vs_r5_r7_LSL_31[] = {
  0x68, 0xbf, 0xfd, 0x07 // It vs; mov vs r5 r7 LSL 31
};
const byte kInstruction_mov_eq_r0_r3_LSL_9[] = {
  0x08, 0xbf, 0x58, 0x02 // It eq; mov eq r0 r3 LSL 9
};
const byte kInstruction_mov_mi_r0_r1_LSL_28[] = {
  0x48, 0xbf, 0x08, 0x07 // It mi; mov mi r0 r1 LSL 28
};
const byte kInstruction_mov_vc_r1_r5_LSL_2[] = {
  0x78, 0xbf, 0xa9, 0x00 // It vc; mov vc r1 r5 LSL 2
};
const byte kInstruction_mov_lt_r5_r7_LSL_26[] = {
  0xb8, 0xbf, 0xbd, 0x06 // It lt; mov lt r5 r7 LSL 26
};
const byte kInstruction_mov_eq_r5_r2_LSL_18[] = {
  0x08, 0xbf, 0x95, 0x04 // It eq; mov eq r5 r2 LSL 18
};
const byte kInstruction_mov_lt_r4_r0_LSL_6[] = {
  0xb8, 0xbf, 0x84, 0x01 // It lt; mov lt r4 r0 LSL 6
};
const byte kInstruction_mov_hi_r6_r2_LSL_9[] = {
  0x88, 0xbf, 0x56, 0x02 // It hi; mov hi r6 r2 LSL 9
};
const byte kInstruction_mov_vc_r6_r1_LSL_25[] = {
  0x78, 0xbf, 0x4e, 0x06 // It vc; mov vc r6 r1 LSL 25
};
const byte kInstruction_mov_le_r2_r7_LSL_10[] = {
  0xd8, 0xbf, 0xba, 0x02 // It le; mov le r2 r7 LSL 10
};
const byte kInstruction_mov_vs_r4_r0_LSL_16[] = {
  0x68, 0xbf, 0x04, 0x04 // It vs; mov vs r4 r0 LSL 16
};
const byte kInstruction_mov_ne_r6_r7_LSL_10[] = {
  0x18, 0xbf, 0xbe, 0x02 // It ne; mov ne r6 r7 LSL 10
};
const byte kInstruction_mov_ne_r4_r3_LSL_26[] = {
  0x18, 0xbf, 0x9c, 0x06 // It ne; mov ne r4 r3 LSL 26
};
const byte kInstruction_mov_le_r6_r5_LSL_9[] = {
  0xd8, 0xbf, 0x6e, 0x02 // It le; mov le r6 r5 LSL 9
};
const byte kInstruction_mov_vc_r2_r5_LSL_1[] = {
  0x78, 0xbf, 0x6a, 0x00 // It vc; mov vc r2 r5 LSL 1
};
const byte kInstruction_mov_vc_r7_r5_LSL_28[] = {
  0x78, 0xbf, 0x2f, 0x07 // It vc; mov vc r7 r5 LSL 28
};
const byte kInstruction_mov_ne_r0_r3_LSL_4[] = {
  0x18, 0xbf, 0x18, 0x01 // It ne; mov ne r0 r3 LSL 4
};
const byte kInstruction_mov_ne_r5_r4_LSL_22[] = {
  0x18, 0xbf, 0xa5, 0x05 // It ne; mov ne r5 r4 LSL 22
};
const byte kInstruction_mov_vc_r1_r4_LSL_30[] = {
  0x78, 0xbf, 0xa1, 0x07 // It vc; mov vc r1 r4 LSL 30
};
const byte kInstruction_mov_gt_r5_r4_LSL_7[] = {
  0xc8, 0xbf, 0xe5, 0x01 // It gt; mov gt r5 r4 LSL 7
};
const byte kInstruction_mov_vs_r4_r5_LSL_24[] = {
  0x68, 0xbf, 0x2c, 0x06 // It vs; mov vs r4 r5 LSL 24
};
const byte kInstruction_mov_hi_r3_r5_LSL_13[] = {
  0x88, 0xbf, 0x6b, 0x03 // It hi; mov hi r3 r5 LSL 13
};
const byte kInstruction_mov_ge_r6_r7_LSL_9[] = {
  0xa8, 0xbf, 0x7e, 0x02 // It ge; mov ge r6 r7 LSL 9
};
const byte kInstruction_mov_eq_r6_r3_LSL_24[] = {
  0x08, 0xbf, 0x1e, 0x06 // It eq; mov eq r6 r3 LSL 24
};
const byte kInstruction_mov_le_r3_r5_LSL_14[] = {
  0xd8, 0xbf, 0xab, 0x03 // It le; mov le r3 r5 LSL 14
};
const byte kInstruction_mov_lt_r0_r7_LSL_1[] = {
  0xb8, 0xbf, 0x78, 0x00 // It lt; mov lt r0 r7 LSL 1
};
const byte kInstruction_mov_cc_r0_r5_LSL_25[] = {
  0x38, 0xbf, 0x68, 0x06 // It cc; mov cc r0 r5 LSL 25
};
const byte kInstruction_mov_ne_r0_r2_LSL_21[] = {
  0x18, 0xbf, 0x50, 0x05 // It ne; mov ne r0 r2 LSL 21
};
const byte kInstruction_mov_eq_r4_r2_LSL_20[] = {
  0x08, 0xbf, 0x14, 0x05 // It eq; mov eq r4 r2 LSL 20
};
const byte kInstruction_mov_vc_r5_r4_LSL_12[] = {
  0x78, 0xbf, 0x25, 0x03 // It vc; mov vc r5 r4 LSL 12
};
const byte kInstruction_mov_ne_r3_r1_LSL_11[] = {
  0x18, 0xbf, 0xcb, 0x02 // It ne; mov ne r3 r1 LSL 11
};
const byte kInstruction_mov_mi_r3_r7_LSL_27[] = {
  0x48, 0xbf, 0xfb, 0x06 // It mi; mov mi r3 r7 LSL 27
};
const byte kInstruction_mov_ls_r1_r7_LSL_10[] = {
  0x98, 0xbf, 0xb9, 0x02 // It ls; mov ls r1 r7 LSL 10
};
const byte kInstruction_mov_le_r7_r1_LSL_30[] = {
  0xd8, 0xbf, 0x8f, 0x07 // It le; mov le r7 r1 LSL 30
};
const byte kInstruction_mov_pl_r5_r2_LSL_4[] = {
  0x58, 0xbf, 0x15, 0x01 // It pl; mov pl r5 r2 LSL 4
};
const byte kInstruction_mov_cs_r5_r4_LSL_7[] = {
  0x28, 0xbf, 0xe5, 0x01 // It cs; mov cs r5 r4 LSL 7
};
const byte kInstruction_mov_mi_r4_r0_LSL_25[] = {
  0x48, 0xbf, 0x44, 0x06 // It mi; mov mi r4 r0 LSL 25
};
const byte kInstruction_mov_vs_r3_r7_LSL_12[] = {
  0x68, 0xbf, 0x3b, 0x03 // It vs; mov vs r3 r7 LSL 12
};
const byte kInstruction_mov_gt_r4_r6_LSL_7[] = {
  0xc8, 0xbf, 0xf4, 0x01 // It gt; mov gt r4 r6 LSL 7
};
const byte kInstruction_mov_vs_r2_r1_LSL_4[] = {
  0x68, 0xbf, 0x0a, 0x01 // It vs; mov vs r2 r1 LSL 4
};
const byte kInstruction_mov_ne_r0_r0_LSL_7[] = {
  0x18, 0xbf, 0xc0, 0x01 // It ne; mov ne r0 r0 LSL 7
};
const byte kInstruction_mov_hi_r5_r1_LSL_8[] = {
  0x88, 0xbf, 0x0d, 0x02 // It hi; mov hi r5 r1 LSL 8
};
const byte kInstruction_mov_eq_r0_r0_LSL_9[] = {
  0x08, 0xbf, 0x40, 0x02 // It eq; mov eq r0 r0 LSL 9
};
const byte kInstruction_mov_cc_r7_r3_LSL_31[] = {
  0x38, 0xbf, 0xdf, 0x07 // It cc; mov cc r7 r3 LSL 31
};
const byte kInstruction_mov_ne_r0_r5_LSL_18[] = {
  0x18, 0xbf, 0xa8, 0x04 // It ne; mov ne r0 r5 LSL 18
};
const byte kInstruction_mov_eq_r6_r3_LSL_23[] = {
  0x08, 0xbf, 0xde, 0x05 // It eq; mov eq r6 r3 LSL 23
};
const byte kInstruction_mov_cc_r5_r7_LSL_14[] = {
  0x38, 0xbf, 0xbd, 0x03 // It cc; mov cc r5 r7 LSL 14
};
const byte kInstruction_mov_ne_r4_r0_LSL_27[] = {
  0x18, 0xbf, 0xc4, 0x06 // It ne; mov ne r4 r0 LSL 27
};
const byte kInstruction_mov_mi_r7_r0_LSL_2[] = {
  0x48, 0xbf, 0x87, 0x00 // It mi; mov mi r7 r0 LSL 2
};
const byte kInstruction_mov_hi_r6_r5_LSL_24[] = {
  0x88, 0xbf, 0x2e, 0x06 // It hi; mov hi r6 r5 LSL 24
};
const byte kInstruction_mov_hi_r5_r7_LSL_20[] = {
  0x88, 0xbf, 0x3d, 0x05 // It hi; mov hi r5 r7 LSL 20
};
const byte kInstruction_mov_vc_r1_r2_LSL_23[] = {
  0x78, 0xbf, 0xd1, 0x05 // It vc; mov vc r1 r2 LSL 23
};
const byte kInstruction_mov_mi_r2_r0_LSL_27[] = {
  0x48, 0xbf, 0xc2, 0x06 // It mi; mov mi r2 r0 LSL 27
};
const byte kInstruction_mov_ls_r1_r2_LSL_17[] = {
  0x98, 0xbf, 0x51, 0x04 // It ls; mov ls r1 r2 LSL 17
};
const byte kInstruction_mov_pl_r0_r6_LSL_1[] = {
  0x58, 0xbf, 0x70, 0x00 // It pl; mov pl r0 r6 LSL 1
};
const byte kInstruction_mov_vs_r6_r3_LSL_16[] = {
  0x68, 0xbf, 0x1e, 0x04 // It vs; mov vs r6 r3 LSL 16
};
const byte kInstruction_mov_mi_r6_r6_LSL_23[] = {
  0x48, 0xbf, 0xf6, 0x05 // It mi; mov mi r6 r6 LSL 23
};
const byte kInstruction_mov_ge_r7_r1_LSL_27[] = {
  0xa8, 0xbf, 0xcf, 0x06 // It ge; mov ge r7 r1 LSL 27
};
const byte kInstruction_mov_gt_r0_r5_LSL_16[] = {
  0xc8, 0xbf, 0x28, 0x04 // It gt; mov gt r0 r5 LSL 16
};
const byte kInstruction_mov_le_r6_r4_LSL_20[] = {
  0xd8, 0xbf, 0x26, 0x05 // It le; mov le r6 r4 LSL 20
};
const byte kInstruction_mov_gt_r6_r0_LSL_20[] = {
  0xc8, 0xbf, 0x06, 0x05 // It gt; mov gt r6 r0 LSL 20
};
const byte kInstruction_mov_gt_r7_r2_LSL_28[] = {
  0xc8, 0xbf, 0x17, 0x07 // It gt; mov gt r7 r2 LSL 28
};
const byte kInstruction_mov_le_r6_r7_LSL_9[] = {
  0xd8, 0xbf, 0x7e, 0x02 // It le; mov le r6 r7 LSL 9
};
const byte kInstruction_mov_gt_r4_r4_LSL_2[] = {
  0xc8, 0xbf, 0xa4, 0x00 // It gt; mov gt r4 r4 LSL 2
};
const byte kInstruction_mov_mi_r6_r5_LSL_13[] = {
  0x48, 0xbf, 0x6e, 0x03 // It mi; mov mi r6 r5 LSL 13
};
const byte kInstruction_mov_le_r0_r0_LSL_2[] = {
  0xd8, 0xbf, 0x80, 0x00 // It le; mov le r0 r0 LSL 2
};
const byte kInstruction_mov_ge_r2_r3_LSL_2[] = {
  0xa8, 0xbf, 0x9a, 0x00 // It ge; mov ge r2 r3 LSL 2
};
const byte kInstruction_mov_cs_r1_r4_LSL_28[] = {
  0x28, 0xbf, 0x21, 0x07 // It cs; mov cs r1 r4 LSL 28
};
const byte kInstruction_mov_vc_r1_r4_LSL_6[] = {
  0x78, 0xbf, 0xa1, 0x01 // It vc; mov vc r1 r4 LSL 6
};
const byte kInstruction_mov_vs_r3_r6_LSL_26[] = {
  0x68, 0xbf, 0xb3, 0x06 // It vs; mov vs r3 r6 LSL 26
};
const byte kInstruction_mov_cs_r2_r6_LSL_23[] = {
  0x28, 0xbf, 0xf2, 0x05 // It cs; mov cs r2 r6 LSL 23
};
const byte kInstruction_mov_eq_r1_r3_LSL_13[] = {
  0x08, 0xbf, 0x59, 0x03 // It eq; mov eq r1 r3 LSL 13
};
const byte kInstruction_mov_ne_r7_r1_LSL_23[] = {
  0x18, 0xbf, 0xcf, 0x05 // It ne; mov ne r7 r1 LSL 23
};
const byte kInstruction_mov_pl_r7_r4_LSL_14[] = {
  0x58, 0xbf, 0xa7, 0x03 // It pl; mov pl r7 r4 LSL 14
};
const byte kInstruction_mov_vs_r3_r1_LSL_22[] = {
  0x68, 0xbf, 0x8b, 0x05 // It vs; mov vs r3 r1 LSL 22
};
const byte kInstruction_mov_eq_r5_r4_LSL_24[] = {
  0x08, 0xbf, 0x25, 0x06 // It eq; mov eq r5 r4 LSL 24
};
const byte kInstruction_mov_cc_r4_r1_LSL_9[] = {
  0x38, 0xbf, 0x4c, 0x02 // It cc; mov cc r4 r1 LSL 9
};
const byte kInstruction_mov_mi_r2_r1_LSL_23[] = {
  0x48, 0xbf, 0xca, 0x05 // It mi; mov mi r2 r1 LSL 23
};
const byte kInstruction_mov_gt_r1_r3_LSL_19[] = {
  0xc8, 0xbf, 0xd9, 0x04 // It gt; mov gt r1 r3 LSL 19
};
const byte kInstruction_mov_lt_r7_r0_LSL_3[] = {
  0xb8, 0xbf, 0xc7, 0x00 // It lt; mov lt r7 r0 LSL 3
};
const byte kInstruction_mov_le_r2_r1_LSL_13[] = {
  0xd8, 0xbf, 0x4a, 0x03 // It le; mov le r2 r1 LSL 13
};
const byte kInstruction_mov_ge_r2_r1_LSL_2[] = {
  0xa8, 0xbf, 0x8a, 0x00 // It ge; mov ge r2 r1 LSL 2
};
const byte kInstruction_mov_hi_r5_r0_LSL_21[] = {
  0x88, 0xbf, 0x45, 0x05 // It hi; mov hi r5 r0 LSL 21
};
const byte kInstruction_mov_gt_r7_r5_LSL_19[] = {
  0xc8, 0xbf, 0xef, 0x04 // It gt; mov gt r7 r5 LSL 19
};
const byte kInstruction_mov_eq_r2_r3_LSL_19[] = {
  0x08, 0xbf, 0xda, 0x04 // It eq; mov eq r2 r3 LSL 19
};
const byte kInstruction_mov_vs_r0_r5_LSL_4[] = {
  0x68, 0xbf, 0x28, 0x01 // It vs; mov vs r0 r5 LSL 4
};
const byte kInstruction_mov_pl_r7_r2_LSL_30[] = {
  0x58, 0xbf, 0x97, 0x07 // It pl; mov pl r7 r2 LSL 30
};
const byte kInstruction_mov_vc_r2_r3_LSL_28[] = {
  0x78, 0xbf, 0x1a, 0x07 // It vc; mov vc r2 r3 LSL 28
};
const byte kInstruction_mov_pl_r2_r7_LSL_25[] = {
  0x58, 0xbf, 0x7a, 0x06 // It pl; mov pl r2 r7 LSL 25
};
const byte kInstruction_mov_eq_r3_r6_LSL_23[] = {
  0x08, 0xbf, 0xf3, 0x05 // It eq; mov eq r3 r6 LSL 23
};
const byte kInstruction_mov_eq_r3_r0_LSL_8[] = {
  0x08, 0xbf, 0x03, 0x02 // It eq; mov eq r3 r0 LSL 8
};
const byte kInstruction_mov_pl_r3_r2_LSL_22[] = {
  0x58, 0xbf, 0x93, 0x05 // It pl; mov pl r3 r2 LSL 22
};
const byte kInstruction_mov_cc_r4_r3_LSL_26[] = {
  0x38, 0xbf, 0x9c, 0x06 // It cc; mov cc r4 r3 LSL 26
};
const byte kInstruction_mov_cs_r5_r2_LSL_1[] = {
  0x28, 0xbf, 0x55, 0x00 // It cs; mov cs r5 r2 LSL 1
};
const byte kInstruction_mov_cc_r1_r1_LSL_6[] = {
  0x38, 0xbf, 0x89, 0x01 // It cc; mov cc r1 r1 LSL 6
};
const byte kInstruction_mov_mi_r3_r5_LSL_22[] = {
  0x48, 0xbf, 0xab, 0x05 // It mi; mov mi r3 r5 LSL 22
};
const byte kInstruction_mov_cs_r0_r3_LSL_21[] = {
  0x28, 0xbf, 0x58, 0x05 // It cs; mov cs r0 r3 LSL 21
};
const byte kInstruction_mov_vs_r7_r7_LSL_13[] = {
  0x68, 0xbf, 0x7f, 0x03 // It vs; mov vs r7 r7 LSL 13
};
const byte kInstruction_mov_pl_r5_r6_LSL_30[] = {
  0x58, 0xbf, 0xb5, 0x07 // It pl; mov pl r5 r6 LSL 30
};
const byte kInstruction_mov_ge_r3_r2_LSL_9[] = {
  0xa8, 0xbf, 0x53, 0x02 // It ge; mov ge r3 r2 LSL 9
};
const byte kInstruction_mov_pl_r1_r3_LSL_11[] = {
  0x58, 0xbf, 0xd9, 0x02 // It pl; mov pl r1 r3 LSL 11
};
const byte kInstruction_mov_eq_r5_r7_LSL_25[] = {
  0x08, 0xbf, 0x7d, 0x06 // It eq; mov eq r5 r7 LSL 25
};
const byte kInstruction_mov_mi_r6_r4_LSL_8[] = {
  0x48, 0xbf, 0x26, 0x02 // It mi; mov mi r6 r4 LSL 8
};
const byte kInstruction_mov_ls_r4_r5_LSL_2[] = {
  0x98, 0xbf, 0xac, 0x00 // It ls; mov ls r4 r5 LSL 2
};
const byte kInstruction_mov_ls_r3_r4_LSL_28[] = {
  0x98, 0xbf, 0x23, 0x07 // It ls; mov ls r3 r4 LSL 28
};
const byte kInstruction_mov_ls_r5_r6_LSL_31[] = {
  0x98, 0xbf, 0xf5, 0x07 // It ls; mov ls r5 r6 LSL 31
};
const byte kInstruction_mov_pl_r6_r0_LSL_11[] = {
  0x58, 0xbf, 0xc6, 0x02 // It pl; mov pl r6 r0 LSL 11
};
const byte kInstruction_mov_ls_r2_r6_LSL_13[] = {
  0x98, 0xbf, 0x72, 0x03 // It ls; mov ls r2 r6 LSL 13
};
const byte kInstruction_mov_gt_r1_r4_LSL_31[] = {
  0xc8, 0xbf, 0xe1, 0x07 // It gt; mov gt r1 r4 LSL 31
};
const byte kInstruction_mov_vc_r2_r3_LSL_20[] = {
  0x78, 0xbf, 0x1a, 0x05 // It vc; mov vc r2 r3 LSL 20
};
const byte kInstruction_mov_eq_r0_r1_LSL_16[] = {
  0x08, 0xbf, 0x08, 0x04 // It eq; mov eq r0 r1 LSL 16
};
const byte kInstruction_mov_pl_r4_r1_LSL_25[] = {
  0x58, 0xbf, 0x4c, 0x06 // It pl; mov pl r4 r1 LSL 25
};
const byte kInstruction_mov_vc_r0_r5_LSL_17[] = {
  0x78, 0xbf, 0x68, 0x04 // It vc; mov vc r0 r5 LSL 17
};
const byte kInstruction_mov_ne_r4_r3_LSL_8[] = {
  0x18, 0xbf, 0x1c, 0x02 // It ne; mov ne r4 r3 LSL 8
};
const byte kInstruction_mov_le_r2_r5_LSL_10[] = {
  0xd8, 0xbf, 0xaa, 0x02 // It le; mov le r2 r5 LSL 10
};
const byte kInstruction_mov_mi_r0_r2_LSL_4[] = {
  0x48, 0xbf, 0x10, 0x01 // It mi; mov mi r0 r2 LSL 4
};
const byte kInstruction_mov_cc_r3_r1_LSL_6[] = {
  0x38, 0xbf, 0x8b, 0x01 // It cc; mov cc r3 r1 LSL 6
};
const byte kInstruction_mov_vs_r4_r7_LSL_30[] = {
  0x68, 0xbf, 0xbc, 0x07 // It vs; mov vs r4 r7 LSL 30
};
const byte kInstruction_mov_ls_r0_r1_LSL_7[] = {
  0x98, 0xbf, 0xc8, 0x01 // It ls; mov ls r0 r1 LSL 7
};
const byte kInstruction_mov_eq_r0_r0_LSL_30[] = {
  0x08, 0xbf, 0x80, 0x07 // It eq; mov eq r0 r0 LSL 30
};
const byte kInstruction_mov_ne_r2_r6_LSL_31[] = {
  0x18, 0xbf, 0xf2, 0x07 // It ne; mov ne r2 r6 LSL 31
};
const byte kInstruction_mov_vc_r7_r5_LSL_15[] = {
  0x78, 0xbf, 0xef, 0x03 // It vc; mov vc r7 r5 LSL 15
};
const byte kInstruction_mov_gt_r0_r2_LSL_22[] = {
  0xc8, 0xbf, 0x90, 0x05 // It gt; mov gt r0 r2 LSL 22
};
const byte kInstruction_mov_ge_r7_r4_LSL_13[] = {
  0xa8, 0xbf, 0x67, 0x03 // It ge; mov ge r7 r4 LSL 13
};
const byte kInstruction_mov_lt_r4_r3_LSL_3[] = {
  0xb8, 0xbf, 0xdc, 0x00 // It lt; mov lt r4 r3 LSL 3
};
const byte kInstruction_mov_vs_r3_r0_LSL_8[] = {
  0x68, 0xbf, 0x03, 0x02 // It vs; mov vs r3 r0 LSL 8
};
const byte kInstruction_mov_gt_r2_r2_LSL_9[] = {
  0xc8, 0xbf, 0x52, 0x02 // It gt; mov gt r2 r2 LSL 9
};
const byte kInstruction_mov_vc_r1_r5_LSL_1[] = {
  0x78, 0xbf, 0x69, 0x00 // It vc; mov vc r1 r5 LSL 1
};
const byte kInstruction_mov_pl_r2_r4_LSL_18[] = {
  0x58, 0xbf, 0xa2, 0x04 // It pl; mov pl r2 r4 LSL 18
};
const byte kInstruction_mov_lt_r5_r2_LSL_27[] = {
  0xb8, 0xbf, 0xd5, 0x06 // It lt; mov lt r5 r2 LSL 27
};
const byte kInstruction_mov_ge_r2_r6_LSL_8[] = {
  0xa8, 0xbf, 0x32, 0x02 // It ge; mov ge r2 r6 LSL 8
};
const byte kInstruction_mov_le_r0_r3_LSL_15[] = {
  0xd8, 0xbf, 0xd8, 0x03 // It le; mov le r0 r3 LSL 15
};
const byte kInstruction_mov_mi_r4_r5_LSL_7[] = {
  0x48, 0xbf, 0xec, 0x01 // It mi; mov mi r4 r5 LSL 7
};
const byte kInstruction_mov_vc_r0_r4_LSL_12[] = {
  0x78, 0xbf, 0x20, 0x03 // It vc; mov vc r0 r4 LSL 12
};
const byte kInstruction_mov_eq_r7_r6_LSL_26[] = {
  0x08, 0xbf, 0xb7, 0x06 // It eq; mov eq r7 r6 LSL 26
};
const byte kInstruction_mov_vc_r1_r0_LSL_24[] = {
  0x78, 0xbf, 0x01, 0x06 // It vc; mov vc r1 r0 LSL 24
};
const byte kInstruction_mov_gt_r3_r6_LSL_28[] = {
  0xc8, 0xbf, 0x33, 0x07 // It gt; mov gt r3 r6 LSL 28
};
const byte kInstruction_mov_vc_r3_r6_LSL_6[] = {
  0x78, 0xbf, 0xb3, 0x01 // It vc; mov vc r3 r6 LSL 6
};
const byte kInstruction_mov_ls_r1_r5_LSL_16[] = {
  0x98, 0xbf, 0x29, 0x04 // It ls; mov ls r1 r5 LSL 16
};
const byte kInstruction_mov_lt_r7_r7_LSL_30[] = {
  0xb8, 0xbf, 0xbf, 0x07 // It lt; mov lt r7 r7 LSL 30
};
const byte kInstruction_mov_eq_r7_r2_LSL_25[] = {
  0x08, 0xbf, 0x57, 0x06 // It eq; mov eq r7 r2 LSL 25
};
const byte kInstruction_mov_ne_r0_r1_LSL_24[] = {
  0x18, 0xbf, 0x08, 0x06 // It ne; mov ne r0 r1 LSL 24
};
const byte kInstruction_mov_ge_r1_r3_LSL_17[] = {
  0xa8, 0xbf, 0x59, 0x04 // It ge; mov ge r1 r3 LSL 17
};
const byte kInstruction_mov_vs_r1_r4_LSL_6[] = {
  0x68, 0xbf, 0xa1, 0x01 // It vs; mov vs r1 r4 LSL 6
};
const byte kInstruction_mov_vs_r7_r3_LSL_30[] = {
  0x68, 0xbf, 0x9f, 0x07 // It vs; mov vs r7 r3 LSL 30
};
const byte kInstruction_mov_le_r0_r4_LSL_16[] = {
  0xd8, 0xbf, 0x20, 0x04 // It le; mov le r0 r4 LSL 16
};
const byte kInstruction_mov_hi_r6_r6_LSL_4[] = {
  0x88, 0xbf, 0x36, 0x01 // It hi; mov hi r6 r6 LSL 4
};
const byte kInstruction_mov_ls_r6_r2_LSL_12[] = {
  0x98, 0xbf, 0x16, 0x03 // It ls; mov ls r6 r2 LSL 12
};
const byte kInstruction_mov_le_r5_r0_LSL_23[] = {
  0xd8, 0xbf, 0xc5, 0x05 // It le; mov le r5 r0 LSL 23
};
const byte kInstruction_mov_cc_r7_r1_LSL_5[] = {
  0x38, 0xbf, 0x4f, 0x01 // It cc; mov cc r7 r1 LSL 5
};
const byte kInstruction_mov_vc_r4_r3_LSL_23[] = {
  0x78, 0xbf, 0xdc, 0x05 // It vc; mov vc r4 r3 LSL 23
};
const byte kInstruction_mov_cc_r6_r1_LSL_30[] = {
  0x38, 0xbf, 0x8e, 0x07 // It cc; mov cc r6 r1 LSL 30
};
const byte kInstruction_mov_lt_r7_r3_LSL_30[] = {
  0xb8, 0xbf, 0x9f, 0x07 // It lt; mov lt r7 r3 LSL 30
};
const byte kInstruction_mov_eq_r0_r6_LSL_18[] = {
  0x08, 0xbf, 0xb0, 0x04 // It eq; mov eq r0 r6 LSL 18
};
const byte kInstruction_mov_le_r2_r2_LSL_20[] = {
  0xd8, 0xbf, 0x12, 0x05 // It le; mov le r2 r2 LSL 20
};
const byte kInstruction_mov_ls_r4_r0_LSL_11[] = {
  0x98, 0xbf, 0xc4, 0x02 // It ls; mov ls r4 r0 LSL 11
};
const byte kInstruction_mov_hi_r2_r6_LSL_26[] = {
  0x88, 0xbf, 0xb2, 0x06 // It hi; mov hi r2 r6 LSL 26
};
const byte kInstruction_mov_vs_r0_r6_LSL_22[] = {
  0x68, 0xbf, 0xb0, 0x05 // It vs; mov vs r0 r6 LSL 22
};
const byte kInstruction_mov_cs_r0_r4_LSL_29[] = {
  0x28, 0xbf, 0x60, 0x07 // It cs; mov cs r0 r4 LSL 29
};
const byte kInstruction_mov_ge_r3_r5_LSL_17[] = {
  0xa8, 0xbf, 0x6b, 0x04 // It ge; mov ge r3 r5 LSL 17
};
const byte kInstruction_mov_lt_r5_r7_LSL_28[] = {
  0xb8, 0xbf, 0x3d, 0x07 // It lt; mov lt r5 r7 LSL 28
};
const byte kInstruction_mov_pl_r7_r3_LSL_13[] = {
  0x58, 0xbf, 0x5f, 0x03 // It pl; mov pl r7 r3 LSL 13
};
const byte kInstruction_mov_mi_r5_r5_LSL_4[] = {
  0x48, 0xbf, 0x2d, 0x01 // It mi; mov mi r5 r5 LSL 4
};
const byte kInstruction_mov_vc_r2_r2_LSL_18[] = {
  0x78, 0xbf, 0x92, 0x04 // It vc; mov vc r2 r2 LSL 18
};
const byte kInstruction_mov_ne_r6_r3_LSL_12[] = {
  0x18, 0xbf, 0x1e, 0x03 // It ne; mov ne r6 r3 LSL 12
};
const byte kInstruction_mov_ne_r6_r6_LSL_31[] = {
  0x18, 0xbf, 0xf6, 0x07 // It ne; mov ne r6 r6 LSL 31
};
const byte kInstruction_mov_mi_r4_r4_LSL_15[] = {
  0x48, 0xbf, 0xe4, 0x03 // It mi; mov mi r4 r4 LSL 15
};
const byte kInstruction_mov_cs_r3_r7_LSL_18[] = {
  0x28, 0xbf, 0xbb, 0x04 // It cs; mov cs r3 r7 LSL 18
};
const byte kInstruction_mov_vs_r4_r5_LSL_13[] = {
  0x68, 0xbf, 0x6c, 0x03 // It vs; mov vs r4 r5 LSL 13
};
const byte kInstruction_mov_ls_r7_r5_LSL_25[] = {
  0x98, 0xbf, 0x6f, 0x06 // It ls; mov ls r7 r5 LSL 25
};
const byte kInstruction_mov_cc_r6_r0_LSL_22[] = {
  0x38, 0xbf, 0x86, 0x05 // It cc; mov cc r6 r0 LSL 22
};
const byte kInstruction_mov_ne_r0_r1_LSL_17[] = {
  0x18, 0xbf, 0x48, 0x04 // It ne; mov ne r0 r1 LSL 17
};
const byte kInstruction_mov_mi_r7_r5_LSL_25[] = {
  0x48, 0xbf, 0x6f, 0x06 // It mi; mov mi r7 r5 LSL 25
};
const byte kInstruction_mov_lt_r5_r6_LSL_24[] = {
  0xb8, 0xbf, 0x35, 0x06 // It lt; mov lt r5 r6 LSL 24
};
const byte kInstruction_mov_le_r6_r4_LSL_31[] = {
  0xd8, 0xbf, 0xe6, 0x07 // It le; mov le r6 r4 LSL 31
};
const byte kInstruction_mov_hi_r4_r4_LSL_16[] = {
  0x88, 0xbf, 0x24, 0x04 // It hi; mov hi r4 r4 LSL 16
};
const byte kInstruction_mov_pl_r5_r4_LSL_7[] = {
  0x58, 0xbf, 0xe5, 0x01 // It pl; mov pl r5 r4 LSL 7
};
const byte kInstruction_mov_ge_r2_r6_LSL_30[] = {
  0xa8, 0xbf, 0xb2, 0x07 // It ge; mov ge r2 r6 LSL 30
};
const byte kInstruction_mov_vs_r2_r5_LSL_31[] = {
  0x68, 0xbf, 0xea, 0x07 // It vs; mov vs r2 r5 LSL 31
};
const byte kInstruction_mov_eq_r5_r2_LSL_21[] = {
  0x08, 0xbf, 0x55, 0x05 // It eq; mov eq r5 r2 LSL 21
};
const byte kInstruction_mov_mi_r7_r6_LSL_18[] = {
  0x48, 0xbf, 0xb7, 0x04 // It mi; mov mi r7 r6 LSL 18
};
const byte kInstruction_mov_ls_r2_r7_LSL_4[] = {
  0x98, 0xbf, 0x3a, 0x01 // It ls; mov ls r2 r7 LSL 4
};
const byte kInstruction_mov_ne_r6_r0_LSL_22[] = {
  0x18, 0xbf, 0x86, 0x05 // It ne; mov ne r6 r0 LSL 22
};
const byte kInstruction_mov_le_r4_r4_LSL_15[] = {
  0xd8, 0xbf, 0xe4, 0x03 // It le; mov le r4 r4 LSL 15
};
const byte kInstruction_mov_ne_r1_r2_LSL_28[] = {
  0x18, 0xbf, 0x11, 0x07 // It ne; mov ne r1 r2 LSL 28
};
const byte kInstruction_mov_hi_r7_r2_LSL_28[] = {
  0x88, 0xbf, 0x17, 0x07 // It hi; mov hi r7 r2 LSL 28
};
const byte kInstruction_mov_lt_r2_r1_LSL_22[] = {
  0xb8, 0xbf, 0x8a, 0x05 // It lt; mov lt r2 r1 LSL 22
};
const byte kInstruction_mov_cs_r4_r2_LSL_30[] = {
  0x28, 0xbf, 0x94, 0x07 // It cs; mov cs r4 r2 LSL 30
};
const byte kInstruction_mov_ls_r7_r7_LSL_1[] = {
  0x98, 0xbf, 0x7f, 0x00 // It ls; mov ls r7 r7 LSL 1
};
const byte kInstruction_mov_mi_r1_r6_LSL_16[] = {
  0x48, 0xbf, 0x31, 0x04 // It mi; mov mi r1 r6 LSL 16
};
const byte kInstruction_mov_cc_r4_r5_LSL_7[] = {
  0x38, 0xbf, 0xec, 0x01 // It cc; mov cc r4 r5 LSL 7
};
const byte kInstruction_mov_vc_r7_r5_LSL_7[] = {
  0x78, 0xbf, 0xef, 0x01 // It vc; mov vc r7 r5 LSL 7
};
const byte kInstruction_mov_eq_r0_r5_LSL_8[] = {
  0x08, 0xbf, 0x28, 0x02 // It eq; mov eq r0 r5 LSL 8
};
const byte kInstruction_mov_eq_r1_r6_LSL_30[] = {
  0x08, 0xbf, 0xb1, 0x07 // It eq; mov eq r1 r6 LSL 30
};
const byte kInstruction_mov_vs_r3_r6_LSL_7[] = {
  0x68, 0xbf, 0xf3, 0x01 // It vs; mov vs r3 r6 LSL 7
};
const byte kInstruction_mov_ge_r3_r6_LSL_17[] = {
  0xa8, 0xbf, 0x73, 0x04 // It ge; mov ge r3 r6 LSL 17
};
const byte kInstruction_mov_vs_r4_r4_LSL_9[] = {
  0x68, 0xbf, 0x64, 0x02 // It vs; mov vs r4 r4 LSL 9
};
const byte kInstruction_mov_cc_r4_r3_LSL_4[] = {
  0x38, 0xbf, 0x1c, 0x01 // It cc; mov cc r4 r3 LSL 4
};
const byte kInstruction_mov_mi_r7_r1_LSL_26[] = {
  0x48, 0xbf, 0x8f, 0x06 // It mi; mov mi r7 r1 LSL 26
};
const byte kInstruction_mov_le_r7_r6_LSL_11[] = {
  0xd8, 0xbf, 0xf7, 0x02 // It le; mov le r7 r6 LSL 11
};
const byte kInstruction_mov_mi_r7_r5_LSL_5[] = {
  0x48, 0xbf, 0x6f, 0x01 // It mi; mov mi r7 r5 LSL 5
};
const byte kInstruction_mov_cs_r0_r4_LSL_9[] = {
  0x28, 0xbf, 0x60, 0x02 // It cs; mov cs r0 r4 LSL 9
};
const byte kInstruction_mov_ge_r1_r1_LSL_28[] = {
  0xa8, 0xbf, 0x09, 0x07 // It ge; mov ge r1 r1 LSL 28
};
const byte kInstruction_mov_pl_r4_r6_LSL_30[] = {
  0x58, 0xbf, 0xb4, 0x07 // It pl; mov pl r4 r6 LSL 30
};
const byte kInstruction_mov_eq_r5_r1_LSL_30[] = {
  0x08, 0xbf, 0x8d, 0x07 // It eq; mov eq r5 r1 LSL 30
};
const byte kInstruction_mov_pl_r5_r6_LSL_31[] = {
  0x58, 0xbf, 0xf5, 0x07 // It pl; mov pl r5 r6 LSL 31
};
const byte kInstruction_mov_hi_r3_r0_LSL_16[] = {
  0x88, 0xbf, 0x03, 0x04 // It hi; mov hi r3 r0 LSL 16
};
const byte kInstruction_mov_eq_r3_r2_LSL_24[] = {
  0x08, 0xbf, 0x13, 0x06 // It eq; mov eq r3 r2 LSL 24
};
const byte kInstruction_mov_mi_r3_r5_LSL_25[] = {
  0x48, 0xbf, 0x6b, 0x06 // It mi; mov mi r3 r5 LSL 25
};
const byte kInstruction_mov_vs_r4_r1_LSL_25[] = {
  0x68, 0xbf, 0x4c, 0x06 // It vs; mov vs r4 r1 LSL 25
};
const byte kInstruction_mov_cc_r5_r3_LSL_24[] = {
  0x38, 0xbf, 0x1d, 0x06 // It cc; mov cc r5 r3 LSL 24
};
const byte kInstruction_mov_gt_r0_r6_LSL_1[] = {
  0xc8, 0xbf, 0x70, 0x00 // It gt; mov gt r0 r6 LSL 1
};
const byte kInstruction_mov_cs_r0_r4_LSL_11[] = {
  0x28, 0xbf, 0xe0, 0x02 // It cs; mov cs r0 r4 LSL 11
};
const byte kInstruction_mov_hi_r0_r2_LSL_19[] = {
  0x88, 0xbf, 0xd0, 0x04 // It hi; mov hi r0 r2 LSL 19
};
const byte kInstruction_mov_ne_r1_r6_LSL_4[] = {
  0x18, 0xbf, 0x31, 0x01 // It ne; mov ne r1 r6 LSL 4
};
const byte kInstruction_mov_vc_r4_r4_LSL_5[] = {
  0x78, 0xbf, 0x64, 0x01 // It vc; mov vc r4 r4 LSL 5
};
const byte kInstruction_mov_hi_r4_r0_LSL_18[] = {
  0x88, 0xbf, 0x84, 0x04 // It hi; mov hi r4 r0 LSL 18
};
const byte kInstruction_mov_cs_r3_r0_LSL_11[] = {
  0x28, 0xbf, 0xc3, 0x02 // It cs; mov cs r3 r0 LSL 11
};
const byte kInstruction_mov_ls_r6_r1_LSL_30[] = {
  0x98, 0xbf, 0x8e, 0x07 // It ls; mov ls r6 r1 LSL 30
};
const byte kInstruction_mov_ne_r4_r0_LSL_3[] = {
  0x18, 0xbf, 0xc4, 0x00 // It ne; mov ne r4 r0 LSL 3
};
const byte kInstruction_mov_ls_r6_r4_LSL_13[] = {
  0x98, 0xbf, 0x66, 0x03 // It ls; mov ls r6 r4 LSL 13
};
const byte kInstruction_mov_gt_r0_r4_LSL_31[] = {
  0xc8, 0xbf, 0xe0, 0x07 // It gt; mov gt r0 r4 LSL 31
};
const byte kInstruction_mov_eq_r6_r3_LSL_10[] = {
  0x08, 0xbf, 0x9e, 0x02 // It eq; mov eq r6 r3 LSL 10
};
const byte kInstruction_mov_ge_r2_r6_LSL_11[] = {
  0xa8, 0xbf, 0xf2, 0x02 // It ge; mov ge r2 r6 LSL 11
};
const byte kInstruction_mov_vs_r2_r4_LSL_14[] = {
  0x68, 0xbf, 0xa2, 0x03 // It vs; mov vs r2 r4 LSL 14
};
const byte kInstruction_mov_gt_r2_r5_LSL_27[] = {
  0xc8, 0xbf, 0xea, 0x06 // It gt; mov gt r2 r5 LSL 27
};
const byte kInstruction_mov_gt_r0_r5_LSL_28[] = {
  0xc8, 0xbf, 0x28, 0x07 // It gt; mov gt r0 r5 LSL 28
};
const byte kInstruction_mov_vs_r4_r6_LSL_1[] = {
  0x68, 0xbf, 0x74, 0x00 // It vs; mov vs r4 r6 LSL 1
};
const byte kInstruction_mov_ls_r0_r3_LSL_29[] = {
  0x98, 0xbf, 0x58, 0x07 // It ls; mov ls r0 r3 LSL 29
};
const byte kInstruction_mov_lt_r4_r4_LSL_16[] = {
  0xb8, 0xbf, 0x24, 0x04 // It lt; mov lt r4 r4 LSL 16
};
const byte kInstruction_mov_cc_r6_r1_LSL_11[] = {
  0x38, 0xbf, 0xce, 0x02 // It cc; mov cc r6 r1 LSL 11
};
const byte kInstruction_mov_vc_r6_r5_LSL_3[] = {
  0x78, 0xbf, 0xee, 0x00 // It vc; mov vc r6 r5 LSL 3
};
const byte kInstruction_mov_eq_r6_r7_LSL_30[] = {
  0x08, 0xbf, 0xbe, 0x07 // It eq; mov eq r6 r7 LSL 30
};
const byte kInstruction_mov_ls_r0_r1_LSL_29[] = {
  0x98, 0xbf, 0x48, 0x07 // It ls; mov ls r0 r1 LSL 29
};
const byte kInstruction_mov_ls_r2_r2_LSL_8[] = {
  0x98, 0xbf, 0x12, 0x02 // It ls; mov ls r2 r2 LSL 8
};
const byte kInstruction_mov_gt_r2_r6_LSL_12[] = {
  0xc8, 0xbf, 0x32, 0x03 // It gt; mov gt r2 r6 LSL 12
};
const byte kInstruction_mov_ge_r5_r1_LSL_25[] = {
  0xa8, 0xbf, 0x4d, 0x06 // It ge; mov ge r5 r1 LSL 25
};
const byte kInstruction_mov_gt_r3_r4_LSL_23[] = {
  0xc8, 0xbf, 0xe3, 0x05 // It gt; mov gt r3 r4 LSL 23
};
const byte kInstruction_mov_eq_r6_r3_LSL_13[] = {
  0x08, 0xbf, 0x5e, 0x03 // It eq; mov eq r6 r3 LSL 13
};
const byte kInstruction_mov_ne_r7_r2_LSL_2[] = {
  0x18, 0xbf, 0x97, 0x00 // It ne; mov ne r7 r2 LSL 2
};
const byte kInstruction_mov_pl_r6_r6_LSL_24[] = {
  0x58, 0xbf, 0x36, 0x06 // It pl; mov pl r6 r6 LSL 24
};
const byte kInstruction_mov_mi_r5_r0_LSL_21[] = {
  0x48, 0xbf, 0x45, 0x05 // It mi; mov mi r5 r0 LSL 21
};
const byte kInstruction_mov_le_r6_r5_LSL_10[] = {
  0xd8, 0xbf, 0xae, 0x02 // It le; mov le r6 r5 LSL 10
};
const byte kInstruction_mov_vc_r2_r6_LSL_5[] = {
  0x78, 0xbf, 0x72, 0x01 // It vc; mov vc r2 r6 LSL 5
};
const byte kInstruction_mov_vc_r6_r2_LSL_9[] = {
  0x78, 0xbf, 0x56, 0x02 // It vc; mov vc r6 r2 LSL 9
};
const byte kInstruction_mov_mi_r5_r6_LSL_8[] = {
  0x48, 0xbf, 0x35, 0x02 // It mi; mov mi r5 r6 LSL 8
};
const byte kInstruction_mov_cs_r2_r5_LSL_16[] = {
  0x28, 0xbf, 0x2a, 0x04 // It cs; mov cs r2 r5 LSL 16
};
const byte kInstruction_mov_cs_r4_r2_LSL_6[] = {
  0x28, 0xbf, 0x94, 0x01 // It cs; mov cs r4 r2 LSL 6
};
const byte kInstruction_mov_lt_r7_r4_LSL_1[] = {
  0xb8, 0xbf, 0x67, 0x00 // It lt; mov lt r7 r4 LSL 1
};
const byte kInstruction_mov_ls_r3_r4_LSL_14[] = {
  0x98, 0xbf, 0xa3, 0x03 // It ls; mov ls r3 r4 LSL 14
};
const byte kInstruction_mov_cs_r6_r4_LSL_9[] = {
  0x28, 0xbf, 0x66, 0x02 // It cs; mov cs r6 r4 LSL 9
};
const byte kInstruction_mov_pl_r6_r5_LSL_3[] = {
  0x58, 0xbf, 0xee, 0x00 // It pl; mov pl r6 r5 LSL 3
};
const byte kInstruction_mov_hi_r3_r2_LSL_17[] = {
  0x88, 0xbf, 0x53, 0x04 // It hi; mov hi r3 r2 LSL 17
};
const byte kInstruction_mov_le_r0_r5_LSL_1[] = {
  0xd8, 0xbf, 0x68, 0x00 // It le; mov le r0 r5 LSL 1
};
const byte kInstruction_mov_pl_r0_r0_LSL_12[] = {
  0x58, 0xbf, 0x00, 0x03 // It pl; mov pl r0 r0 LSL 12
};
const byte kInstruction_mov_ne_r5_r0_LSL_9[] = {
  0x18, 0xbf, 0x45, 0x02 // It ne; mov ne r5 r0 LSL 9
};
const byte kInstruction_mov_ls_r1_r6_LSL_6[] = {
  0x98, 0xbf, 0xb1, 0x01 // It ls; mov ls r1 r6 LSL 6
};
const byte kInstruction_mov_mi_r1_r2_LSL_15[] = {
  0x48, 0xbf, 0xd1, 0x03 // It mi; mov mi r1 r2 LSL 15
};
const byte kInstruction_mov_hi_r6_r4_LSL_22[] = {
  0x88, 0xbf, 0xa6, 0x05 // It hi; mov hi r6 r4 LSL 22
};
const byte kInstruction_mov_ls_r0_r4_LSL_29[] = {
  0x98, 0xbf, 0x60, 0x07 // It ls; mov ls r0 r4 LSL 29
};
const byte kInstruction_mov_vs_r1_r5_LSL_1[] = {
  0x68, 0xbf, 0x69, 0x00 // It vs; mov vs r1 r5 LSL 1
};
const byte kInstruction_mov_vs_r3_r2_LSL_24[] = {
  0x68, 0xbf, 0x13, 0x06 // It vs; mov vs r3 r2 LSL 24
};
const byte kInstruction_mov_cc_r6_r7_LSL_7[] = {
  0x38, 0xbf, 0xfe, 0x01 // It cc; mov cc r6 r7 LSL 7
};
const byte kInstruction_mov_vc_r4_r0_LSL_27[] = {
  0x78, 0xbf, 0xc4, 0x06 // It vc; mov vc r4 r0 LSL 27
};
const byte kInstruction_mov_vs_r0_r1_LSL_13[] = {
  0x68, 0xbf, 0x48, 0x03 // It vs; mov vs r0 r1 LSL 13
};
const byte kInstruction_mov_lt_r1_r5_LSL_16[] = {
  0xb8, 0xbf, 0x29, 0x04 // It lt; mov lt r1 r5 LSL 16
};
const byte kInstruction_mov_cc_r1_r3_LSL_10[] = {
  0x38, 0xbf, 0x99, 0x02 // It cc; mov cc r1 r3 LSL 10
};
const byte kInstruction_mov_eq_r0_r3_LSL_13[] = {
  0x08, 0xbf, 0x58, 0x03 // It eq; mov eq r0 r3 LSL 13
};
const byte kInstruction_mov_ls_r0_r7_LSL_3[] = {
  0x98, 0xbf, 0xf8, 0x00 // It ls; mov ls r0 r7 LSL 3
};
const byte kInstruction_mov_mi_r7_r7_LSL_16[] = {
  0x48, 0xbf, 0x3f, 0x04 // It mi; mov mi r7 r7 LSL 16
};
const byte kInstruction_mov_le_r5_r2_LSL_7[] = {
  0xd8, 0xbf, 0xd5, 0x01 // It le; mov le r5 r2 LSL 7
};
const byte kInstruction_mov_ne_r6_r4_LSL_22[] = {
  0x18, 0xbf, 0xa6, 0x05 // It ne; mov ne r6 r4 LSL 22
};
const byte kInstruction_mov_eq_r0_r0_LSL_14[] = {
  0x08, 0xbf, 0x80, 0x03 // It eq; mov eq r0 r0 LSL 14
};
const byte kInstruction_mov_ne_r5_r2_LSL_3[] = {
  0x18, 0xbf, 0xd5, 0x00 // It ne; mov ne r5 r2 LSL 3
};
const byte kInstruction_mov_eq_r0_r6_LSL_29[] = {
  0x08, 0xbf, 0x70, 0x07 // It eq; mov eq r0 r6 LSL 29
};
const byte kInstruction_mov_vs_r4_r2_LSL_23[] = {
  0x68, 0xbf, 0xd4, 0x05 // It vs; mov vs r4 r2 LSL 23
};
const byte kInstruction_mov_ne_r1_r5_LSL_15[] = {
  0x18, 0xbf, 0xe9, 0x03 // It ne; mov ne r1 r5 LSL 15
};
const byte kInstruction_mov_ne_r3_r6_LSL_30[] = {
  0x18, 0xbf, 0xb3, 0x07 // It ne; mov ne r3 r6 LSL 30
};
const byte kInstruction_mov_hi_r7_r2_LSL_17[] = {
  0x88, 0xbf, 0x57, 0x04 // It hi; mov hi r7 r2 LSL 17
};
const byte kInstruction_mov_vs_r4_r0_LSL_7[] = {
  0x68, 0xbf, 0xc4, 0x01 // It vs; mov vs r4 r0 LSL 7
};
const byte kInstruction_mov_vc_r5_r0_LSL_28[] = {
  0x78, 0xbf, 0x05, 0x07 // It vc; mov vc r5 r0 LSL 28
};
const byte kInstruction_mov_lt_r6_r5_LSL_20[] = {
  0xb8, 0xbf, 0x2e, 0x05 // It lt; mov lt r6 r5 LSL 20
};
const byte kInstruction_mov_cs_r3_r0_LSL_16[] = {
  0x28, 0xbf, 0x03, 0x04 // It cs; mov cs r3 r0 LSL 16
};
const byte kInstruction_mov_cs_r0_r5_LSL_19[] = {
  0x28, 0xbf, 0xe8, 0x04 // It cs; mov cs r0 r5 LSL 19
};
const byte kInstruction_mov_ge_r1_r1_LSL_13[] = {
  0xa8, 0xbf, 0x49, 0x03 // It ge; mov ge r1 r1 LSL 13
};
const byte kInstruction_mov_cc_r5_r4_LSL_14[] = {
  0x38, 0xbf, 0xa5, 0x03 // It cc; mov cc r5 r4 LSL 14
};
const byte kInstruction_mov_hi_r3_r5_LSL_8[] = {
  0x88, 0xbf, 0x2b, 0x02 // It hi; mov hi r3 r5 LSL 8
};
const byte kInstruction_mov_vc_r7_r5_LSL_1[] = {
  0x78, 0xbf, 0x6f, 0x00 // It vc; mov vc r7 r5 LSL 1
};
const byte kInstruction_mov_cc_r4_r3_LSL_1[] = {
  0x38, 0xbf, 0x5c, 0x00 // It cc; mov cc r4 r3 LSL 1
};
const byte kInstruction_mov_cs_r3_r6_LSL_8[] = {
  0x28, 0xbf, 0x33, 0x02 // It cs; mov cs r3 r6 LSL 8
};
const byte kInstruction_mov_pl_r3_r1_LSL_30[] = {
  0x58, 0xbf, 0x8b, 0x07 // It pl; mov pl r3 r1 LSL 30
};
const byte kInstruction_mov_cc_r3_r1_LSL_31[] = {
  0x38, 0xbf, 0xcb, 0x07 // It cc; mov cc r3 r1 LSL 31
};
const byte kInstruction_mov_ne_r3_r1_LSL_9[] = {
  0x18, 0xbf, 0x4b, 0x02 // It ne; mov ne r3 r1 LSL 9
};
const byte kInstruction_mov_cc_r0_r7_LSL_9[] = {
  0x38, 0xbf, 0x78, 0x02 // It cc; mov cc r0 r7 LSL 9
};
const byte kInstruction_mov_mi_r6_r4_LSL_13[] = {
  0x48, 0xbf, 0x66, 0x03 // It mi; mov mi r6 r4 LSL 13
};
const byte kInstruction_mov_le_r3_r0_LSL_7[] = {
  0xd8, 0xbf, 0xc3, 0x01 // It le; mov le r3 r0 LSL 7
};
const byte kInstruction_mov_hi_r6_r7_LSL_18[] = {
  0x88, 0xbf, 0xbe, 0x04 // It hi; mov hi r6 r7 LSL 18
};
const byte kInstruction_mov_mi_r3_r3_LSL_22[] = {
  0x48, 0xbf, 0x9b, 0x05 // It mi; mov mi r3 r3 LSL 22
};
const byte kInstruction_mov_gt_r4_r5_LSL_2[] = {
  0xc8, 0xbf, 0xac, 0x00 // It gt; mov gt r4 r5 LSL 2
};
const byte kInstruction_mov_le_r4_r0_LSL_13[] = {
  0xd8, 0xbf, 0x44, 0x03 // It le; mov le r4 r0 LSL 13
};
const byte kInstruction_mov_lt_r4_r1_LSL_14[] = {
  0xb8, 0xbf, 0x8c, 0x03 // It lt; mov lt r4 r1 LSL 14
};
const byte kInstruction_mov_lt_r5_r5_LSL_4[] = {
  0xb8, 0xbf, 0x2d, 0x01 // It lt; mov lt r5 r5 LSL 4
};
const byte kInstruction_mov_gt_r7_r7_LSL_29[] = {
  0xc8, 0xbf, 0x7f, 0x07 // It gt; mov gt r7 r7 LSL 29
};
const byte kInstruction_mov_le_r7_r3_LSL_12[] = {
  0xd8, 0xbf, 0x1f, 0x03 // It le; mov le r7 r3 LSL 12
};
const byte kInstruction_mov_vc_r0_r0_LSL_15[] = {
  0x78, 0xbf, 0xc0, 0x03 // It vc; mov vc r0 r0 LSL 15
};
const byte kInstruction_mov_ne_r2_r6_LSL_11[] = {
  0x18, 0xbf, 0xf2, 0x02 // It ne; mov ne r2 r6 LSL 11
};
const byte kInstruction_mov_hi_r2_r0_LSL_29[] = {
  0x88, 0xbf, 0x42, 0x07 // It hi; mov hi r2 r0 LSL 29
};
const byte kInstruction_mov_eq_r0_r4_LSL_22[] = {
  0x08, 0xbf, 0xa0, 0x05 // It eq; mov eq r0 r4 LSL 22
};
const byte kInstruction_mov_cs_r3_r4_LSL_30[] = {
  0x28, 0xbf, 0xa3, 0x07 // It cs; mov cs r3 r4 LSL 30
};
const byte kInstruction_mov_hi_r1_r2_LSL_19[] = {
  0x88, 0xbf, 0xd1, 0x04 // It hi; mov hi r1 r2 LSL 19
};
const byte kInstruction_mov_ne_r7_r2_LSL_31[] = {
  0x18, 0xbf, 0xd7, 0x07 // It ne; mov ne r7 r2 LSL 31
};
const byte kInstruction_mov_pl_r5_r7_LSL_9[] = {
  0x58, 0xbf, 0x7d, 0x02 // It pl; mov pl r5 r7 LSL 9
};
const byte kInstruction_mov_mi_r2_r2_LSL_28[] = {
  0x48, 0xbf, 0x12, 0x07 // It mi; mov mi r2 r2 LSL 28
};
const byte kInstruction_mov_ls_r3_r0_LSL_16[] = {
  0x98, 0xbf, 0x03, 0x04 // It ls; mov ls r3 r0 LSL 16
};
const byte kInstruction_mov_ge_r5_r4_LSL_11[] = {
  0xa8, 0xbf, 0xe5, 0x02 // It ge; mov ge r5 r4 LSL 11
};
const byte kInstruction_mov_gt_r0_r1_LSL_15[] = {
  0xc8, 0xbf, 0xc8, 0x03 // It gt; mov gt r0 r1 LSL 15
};
const byte kInstruction_mov_ls_r7_r4_LSL_29[] = {
  0x98, 0xbf, 0x67, 0x07 // It ls; mov ls r7 r4 LSL 29
};
const byte kInstruction_mov_ne_r4_r6_LSL_21[] = {
  0x18, 0xbf, 0x74, 0x05 // It ne; mov ne r4 r6 LSL 21
};
const byte kInstruction_mov_eq_r2_r5_LSL_2[] = {
  0x08, 0xbf, 0xaa, 0x00 // It eq; mov eq r2 r5 LSL 2
};
const byte kInstruction_mov_le_r3_r2_LSL_15[] = {
  0xd8, 0xbf, 0xd3, 0x03 // It le; mov le r3 r2 LSL 15
};
const byte kInstruction_mov_cc_r2_r7_LSL_5[] = {
  0x38, 0xbf, 0x7a, 0x01 // It cc; mov cc r2 r7 LSL 5
};
const byte kInstruction_mov_lt_r4_r7_LSL_4[] = {
  0xb8, 0xbf, 0x3c, 0x01 // It lt; mov lt r4 r7 LSL 4
};
const byte kInstruction_mov_cc_r6_r5_LSL_28[] = {
  0x38, 0xbf, 0x2e, 0x07 // It cc; mov cc r6 r5 LSL 28
};
const byte kInstruction_mov_pl_r4_r1_LSL_27[] = {
  0x58, 0xbf, 0xcc, 0x06 // It pl; mov pl r4 r1 LSL 27
};
const byte kInstruction_mov_vs_r2_r5_LSL_22[] = {
  0x68, 0xbf, 0xaa, 0x05 // It vs; mov vs r2 r5 LSL 22
};
const byte kInstruction_mov_eq_r2_r7_LSL_25[] = {
  0x08, 0xbf, 0x7a, 0x06 // It eq; mov eq r2 r7 LSL 25
};
const byte kInstruction_mov_pl_r1_r2_LSL_8[] = {
  0x58, 0xbf, 0x11, 0x02 // It pl; mov pl r1 r2 LSL 8
};
const byte kInstruction_mov_ge_r4_r0_LSL_15[] = {
  0xa8, 0xbf, 0xc4, 0x03 // It ge; mov ge r4 r0 LSL 15
};
const byte kInstruction_mov_vc_r7_r5_LSL_3[] = {
  0x78, 0xbf, 0xef, 0x00 // It vc; mov vc r7 r5 LSL 3
};
const byte kInstruction_mov_vc_r2_r7_LSL_19[] = {
  0x78, 0xbf, 0xfa, 0x04 // It vc; mov vc r2 r7 LSL 19
};
const byte kInstruction_mov_pl_r4_r0_LSL_31[] = {
  0x58, 0xbf, 0xc4, 0x07 // It pl; mov pl r4 r0 LSL 31
};
const byte kInstruction_mov_le_r3_r1_LSL_21[] = {
  0xd8, 0xbf, 0x4b, 0x05 // It le; mov le r3 r1 LSL 21
};
const byte kInstruction_mov_cs_r3_r7_LSL_5[] = {
  0x28, 0xbf, 0x7b, 0x01 // It cs; mov cs r3 r7 LSL 5
};
const byte kInstruction_mov_cs_r5_r0_LSL_24[] = {
  0x28, 0xbf, 0x05, 0x06 // It cs; mov cs r5 r0 LSL 24
};
const byte kInstruction_mov_vc_r5_r2_LSL_21[] = {
  0x78, 0xbf, 0x55, 0x05 // It vc; mov vc r5 r2 LSL 21
};
const byte kInstruction_mov_vc_r1_r3_LSL_8[] = {
  0x78, 0xbf, 0x19, 0x02 // It vc; mov vc r1 r3 LSL 8
};
const byte kInstruction_mov_ne_r1_r3_LSL_8[] = {
  0x18, 0xbf, 0x19, 0x02 // It ne; mov ne r1 r3 LSL 8
};
const byte kInstruction_mov_pl_r7_r0_LSL_28[] = {
  0x58, 0xbf, 0x07, 0x07 // It pl; mov pl r7 r0 LSL 28
};
const byte kInstruction_mov_ge_r1_r0_LSL_15[] = {
  0xa8, 0xbf, 0xc1, 0x03 // It ge; mov ge r1 r0 LSL 15
};
const byte kInstruction_mov_vc_r4_r6_LSL_13[] = {
  0x78, 0xbf, 0x74, 0x03 // It vc; mov vc r4 r6 LSL 13
};
const byte kInstruction_mov_le_r4_r2_LSL_16[] = {
  0xd8, 0xbf, 0x14, 0x04 // It le; mov le r4 r2 LSL 16
};
const byte kInstruction_mov_vc_r6_r0_LSL_5[] = {
  0x78, 0xbf, 0x46, 0x01 // It vc; mov vc r6 r0 LSL 5
};
const byte kInstruction_mov_cs_r0_r2_LSL_16[] = {
  0x28, 0xbf, 0x10, 0x04 // It cs; mov cs r0 r2 LSL 16
};
const byte kInstruction_mov_ne_r7_r0_LSL_11[] = {
  0x18, 0xbf, 0xc7, 0x02 // It ne; mov ne r7 r0 LSL 11
};
const byte kInstruction_mov_lt_r3_r4_LSL_18[] = {
  0xb8, 0xbf, 0xa3, 0x04 // It lt; mov lt r3 r4 LSL 18
};
const byte kInstruction_mov_le_r5_r4_LSL_23[] = {
  0xd8, 0xbf, 0xe5, 0x05 // It le; mov le r5 r4 LSL 23
};
const byte kInstruction_mov_hi_r4_r3_LSL_9[] = {
  0x88, 0xbf, 0x5c, 0x02 // It hi; mov hi r4 r3 LSL 9
};
const byte kInstruction_mov_cs_r5_r3_LSL_20[] = {
  0x28, 0xbf, 0x1d, 0x05 // It cs; mov cs r5 r3 LSL 20
};
const byte kInstruction_mov_le_r4_r0_LSL_18[] = {
  0xd8, 0xbf, 0x84, 0x04 // It le; mov le r4 r0 LSL 18
};
const byte kInstruction_mov_ls_r3_r5_LSL_1[] = {
  0x98, 0xbf, 0x6b, 0x00 // It ls; mov ls r3 r5 LSL 1
};
const byte kInstruction_mov_ge_r2_r5_LSL_7[] = {
  0xa8, 0xbf, 0xea, 0x01 // It ge; mov ge r2 r5 LSL 7
};
const byte kInstruction_mov_vs_r3_r6_LSL_12[] = {
  0x68, 0xbf, 0x33, 0x03 // It vs; mov vs r3 r6 LSL 12
};
const byte kInstruction_mov_gt_r5_r3_LSL_15[] = {
  0xc8, 0xbf, 0xdd, 0x03 // It gt; mov gt r5 r3 LSL 15
};
const byte kInstruction_mov_mi_r4_r0_LSL_5[] = {
  0x48, 0xbf, 0x44, 0x01 // It mi; mov mi r4 r0 LSL 5
};
const byte kInstruction_mov_vc_r4_r7_LSL_5[] = {
  0x78, 0xbf, 0x7c, 0x01 // It vc; mov vc r4 r7 LSL 5
};
const byte kInstruction_mov_cs_r0_r5_LSL_24[] = {
  0x28, 0xbf, 0x28, 0x06 // It cs; mov cs r0 r5 LSL 24
};
const byte kInstruction_mov_vs_r7_r7_LSL_19[] = {
  0x68, 0xbf, 0xff, 0x04 // It vs; mov vs r7 r7 LSL 19
};
const byte kInstruction_mov_mi_r5_r4_LSL_27[] = {
  0x48, 0xbf, 0xe5, 0x06 // It mi; mov mi r5 r4 LSL 27
};
const byte kInstruction_mov_ne_r3_r7_LSL_1[] = {
  0x18, 0xbf, 0x7b, 0x00 // It ne; mov ne r3 r7 LSL 1
};
const byte kInstruction_mov_ne_r4_r7_LSL_8[] = {
  0x18, 0xbf, 0x3c, 0x02 // It ne; mov ne r4 r7 LSL 8
};
const byte kInstruction_mov_vs_r5_r4_LSL_21[] = {
  0x68, 0xbf, 0x65, 0x05 // It vs; mov vs r5 r4 LSL 21
};
const byte kInstruction_mov_ls_r0_r0_LSL_20[] = {
  0x98, 0xbf, 0x00, 0x05 // It ls; mov ls r0 r0 LSL 20
};
const byte kInstruction_mov_gt_r5_r1_LSL_30[] = {
  0xc8, 0xbf, 0x8d, 0x07 // It gt; mov gt r5 r1 LSL 30
};
const byte kInstruction_mov_lt_r4_r5_LSL_4[] = {
  0xb8, 0xbf, 0x2c, 0x01 // It lt; mov lt r4 r5 LSL 4
};
const byte kInstruction_mov_ge_r7_r6_LSL_10[] = {
  0xa8, 0xbf, 0xb7, 0x02 // It ge; mov ge r7 r6 LSL 10
};
const byte kInstruction_mov_cc_r1_r6_LSL_10[] = {
  0x38, 0xbf, 0xb1, 0x02 // It cc; mov cc r1 r6 LSL 10
};
const byte kInstruction_mov_ls_r0_r5_LSL_23[] = {
  0x98, 0xbf, 0xe8, 0x05 // It ls; mov ls r0 r5 LSL 23
};
const byte kInstruction_mov_ge_r4_r2_LSL_19[] = {
  0xa8, 0xbf, 0xd4, 0x04 // It ge; mov ge r4 r2 LSL 19
};
const byte kInstruction_mov_cc_r0_r7_LSL_12[] = {
  0x38, 0xbf, 0x38, 0x03 // It cc; mov cc r0 r7 LSL 12
};
const byte kInstruction_mov_ls_r0_r2_LSL_27[] = {
  0x98, 0xbf, 0xd0, 0x06 // It ls; mov ls r0 r2 LSL 27
};
const byte kInstruction_mov_eq_r2_r7_LSL_13[] = {
  0x08, 0xbf, 0x7a, 0x03 // It eq; mov eq r2 r7 LSL 13
};
const byte kInstruction_mov_cc_r6_r7_LSL_18[] = {
  0x38, 0xbf, 0xbe, 0x04 // It cc; mov cc r6 r7 LSL 18
};
const byte kInstruction_mov_cc_r6_r1_LSL_18[] = {
  0x38, 0xbf, 0x8e, 0x04 // It cc; mov cc r6 r1 LSL 18
};
const byte kInstruction_mov_gt_r4_r6_LSL_3[] = {
  0xc8, 0xbf, 0xf4, 0x00 // It gt; mov gt r4 r6 LSL 3
};
const byte kInstruction_mov_cc_r2_r4_LSL_26[] = {
  0x38, 0xbf, 0xa2, 0x06 // It cc; mov cc r2 r4 LSL 26
};
const byte kInstruction_mov_mi_r6_r5_LSL_31[] = {
  0x48, 0xbf, 0xee, 0x07 // It mi; mov mi r6 r5 LSL 31
};
const byte kInstruction_mov_cs_r5_r3_LSL_24[] = {
  0x28, 0xbf, 0x1d, 0x06 // It cs; mov cs r5 r3 LSL 24
};
const byte kInstruction_mov_gt_r2_r4_LSL_7[] = {
  0xc8, 0xbf, 0xe2, 0x01 // It gt; mov gt r2 r4 LSL 7
};
const byte kInstruction_mov_mi_r5_r3_LSL_6[] = {
  0x48, 0xbf, 0x9d, 0x01 // It mi; mov mi r5 r3 LSL 6
};
const byte kInstruction_mov_lt_r2_r5_LSL_23[] = {
  0xb8, 0xbf, 0xea, 0x05 // It lt; mov lt r2 r5 LSL 23
};
const byte kInstruction_mov_ls_r6_r4_LSL_7[] = {
  0x98, 0xbf, 0xe6, 0x01 // It ls; mov ls r6 r4 LSL 7
};
const byte kInstruction_mov_eq_r0_r6_LSL_24[] = {
  0x08, 0xbf, 0x30, 0x06 // It eq; mov eq r0 r6 LSL 24
};
const byte kInstruction_mov_hi_r2_r6_LSL_2[] = {
  0x88, 0xbf, 0xb2, 0x00 // It hi; mov hi r2 r6 LSL 2
};
const byte kInstruction_mov_eq_r1_r4_LSL_29[] = {
  0x08, 0xbf, 0x61, 0x07 // It eq; mov eq r1 r4 LSL 29
};
const byte kInstruction_mov_cs_r2_r0_LSL_10[] = {
  0x28, 0xbf, 0x82, 0x02 // It cs; mov cs r2 r0 LSL 10
};
const byte kInstruction_mov_cs_r3_r4_LSL_1[] = {
  0x28, 0xbf, 0x63, 0x00 // It cs; mov cs r3 r4 LSL 1
};
const byte kInstruction_mov_vs_r3_r7_LSL_21[] = {
  0x68, 0xbf, 0x7b, 0x05 // It vs; mov vs r3 r7 LSL 21
};
const byte kInstruction_mov_lt_r3_r2_LSL_3[] = {
  0xb8, 0xbf, 0xd3, 0x00 // It lt; mov lt r3 r2 LSL 3
};
const byte kInstruction_mov_le_r1_r3_LSL_14[] = {
  0xd8, 0xbf, 0x99, 0x03 // It le; mov le r1 r3 LSL 14
};
const byte kInstruction_mov_vc_r2_r3_LSL_23[] = {
  0x78, 0xbf, 0xda, 0x05 // It vc; mov vc r2 r3 LSL 23
};
const byte kInstruction_mov_pl_r2_r0_LSL_12[] = {
  0x58, 0xbf, 0x02, 0x03 // It pl; mov pl r2 r0 LSL 12
};
const byte kInstruction_mov_ge_r2_r6_LSL_27[] = {
  0xa8, 0xbf, 0xf2, 0x06 // It ge; mov ge r2 r6 LSL 27
};
const byte kInstruction_mov_ls_r6_r3_LSL_8[] = {
  0x98, 0xbf, 0x1e, 0x02 // It ls; mov ls r6 r3 LSL 8
};
const byte kInstruction_mov_lt_r4_r1_LSL_31[] = {
  0xb8, 0xbf, 0xcc, 0x07 // It lt; mov lt r4 r1 LSL 31
};
const byte kInstruction_mov_le_r1_r7_LSL_24[] = {
  0xd8, 0xbf, 0x39, 0x06 // It le; mov le r1 r7 LSL 24
};
const byte kInstruction_mov_mi_r2_r2_LSL_11[] = {
  0x48, 0xbf, 0xd2, 0x02 // It mi; mov mi r2 r2 LSL 11
};
const byte kInstruction_mov_cc_r4_r1_LSL_27[] = {
  0x38, 0xbf, 0xcc, 0x06 // It cc; mov cc r4 r1 LSL 27
};
const byte kInstruction_mov_ne_r7_r2_LSL_15[] = {
  0x18, 0xbf, 0xd7, 0x03 // It ne; mov ne r7 r2 LSL 15
};
const byte kInstruction_mov_cc_r2_r4_LSL_10[] = {
  0x38, 0xbf, 0xa2, 0x02 // It cc; mov cc r2 r4 LSL 10
};
const byte kInstruction_mov_lt_r3_r6_LSL_28[] = {
  0xb8, 0xbf, 0x33, 0x07 // It lt; mov lt r3 r6 LSL 28
};
const byte kInstruction_mov_vc_r5_r1_LSL_1[] = {
  0x78, 0xbf, 0x4d, 0x00 // It vc; mov vc r5 r1 LSL 1
};
const byte kInstruction_mov_ls_r7_r1_LSL_21[] = {
  0x98, 0xbf, 0x4f, 0x05 // It ls; mov ls r7 r1 LSL 21
};
const byte kInstruction_mov_eq_r0_r0_LSL_20[] = {
  0x08, 0xbf, 0x00, 0x05 // It eq; mov eq r0 r0 LSL 20
};
const byte kInstruction_mov_ne_r7_r0_LSL_16[] = {
  0x18, 0xbf, 0x07, 0x04 // It ne; mov ne r7 r0 LSL 16
};
const byte kInstruction_mov_hi_r6_r1_LSL_16[] = {
  0x88, 0xbf, 0x0e, 0x04 // It hi; mov hi r6 r1 LSL 16
};
const byte kInstruction_mov_mi_r6_r5_LSL_15[] = {
  0x48, 0xbf, 0xee, 0x03 // It mi; mov mi r6 r5 LSL 15
};
const byte kInstruction_mov_ge_r5_r3_LSL_1[] = {
  0xa8, 0xbf, 0x5d, 0x00 // It ge; mov ge r5 r3 LSL 1
};
const byte kInstruction_mov_cs_r3_r6_LSL_10[] = {
  0x28, 0xbf, 0xb3, 0x02 // It cs; mov cs r3 r6 LSL 10
};
const byte kInstruction_mov_vs_r1_r3_LSL_7[] = {
  0x68, 0xbf, 0xd9, 0x01 // It vs; mov vs r1 r3 LSL 7
};
const byte kInstruction_mov_hi_r3_r2_LSL_31[] = {
  0x88, 0xbf, 0xd3, 0x07 // It hi; mov hi r3 r2 LSL 31
};
const byte kInstruction_mov_vc_r3_r0_LSL_22[] = {
  0x78, 0xbf, 0x83, 0x05 // It vc; mov vc r3 r0 LSL 22
};
const byte kInstruction_mov_cc_r0_r7_LSL_13[] = {
  0x38, 0xbf, 0x78, 0x03 // It cc; mov cc r0 r7 LSL 13
};
const byte kInstruction_mov_mi_r0_r1_LSL_26[] = {
  0x48, 0xbf, 0x88, 0x06 // It mi; mov mi r0 r1 LSL 26
};
const byte kInstruction_mov_eq_r1_r5_LSL_2[] = {
  0x08, 0xbf, 0xa9, 0x00 // It eq; mov eq r1 r5 LSL 2
};
const byte kInstruction_mov_eq_r2_r4_LSL_22[] = {
  0x08, 0xbf, 0xa2, 0x05 // It eq; mov eq r2 r4 LSL 22
};
const byte kInstruction_mov_gt_r4_r2_LSL_7[] = {
  0xc8, 0xbf, 0xd4, 0x01 // It gt; mov gt r4 r2 LSL 7
};
const byte kInstruction_mov_eq_r4_r3_LSL_13[] = {
  0x08, 0xbf, 0x5c, 0x03 // It eq; mov eq r4 r3 LSL 13
};
const byte kInstruction_mov_pl_r0_r2_LSL_6[] = {
  0x58, 0xbf, 0x90, 0x01 // It pl; mov pl r0 r2 LSL 6
};
const byte kInstruction_mov_eq_r1_r4_LSL_11[] = {
  0x08, 0xbf, 0xe1, 0x02 // It eq; mov eq r1 r4 LSL 11
};
const byte kInstruction_mov_vs_r1_r7_LSL_15[] = {
  0x68, 0xbf, 0xf9, 0x03 // It vs; mov vs r1 r7 LSL 15
};
const byte kInstruction_mov_eq_r3_r2_LSL_2[] = {
  0x08, 0xbf, 0x93, 0x00 // It eq; mov eq r3 r2 LSL 2
};
const byte kInstruction_mov_ge_r1_r1_LSL_8[] = {
  0xa8, 0xbf, 0x09, 0x02 // It ge; mov ge r1 r1 LSL 8
};
const byte kInstruction_mov_vc_r2_r6_LSL_7[] = {
  0x78, 0xbf, 0xf2, 0x01 // It vc; mov vc r2 r6 LSL 7
};
const byte kInstruction_mov_pl_r0_r4_LSL_28[] = {
  0x58, 0xbf, 0x20, 0x07 // It pl; mov pl r0 r4 LSL 28
};
const byte kInstruction_mov_le_r7_r6_LSL_20[] = {
  0xd8, 0xbf, 0x37, 0x05 // It le; mov le r7 r6 LSL 20
};
const byte kInstruction_mov_hi_r2_r3_LSL_14[] = {
  0x88, 0xbf, 0x9a, 0x03 // It hi; mov hi r2 r3 LSL 14
};
const byte kInstruction_mov_mi_r1_r3_LSL_5[] = {
  0x48, 0xbf, 0x59, 0x01 // It mi; mov mi r1 r3 LSL 5
};
const byte kInstruction_mov_vc_r7_r0_LSL_13[] = {
  0x78, 0xbf, 0x47, 0x03 // It vc; mov vc r7 r0 LSL 13
};
const byte kInstruction_mov_cc_r6_r2_LSL_21[] = {
  0x38, 0xbf, 0x56, 0x05 // It cc; mov cc r6 r2 LSL 21
};
const byte kInstruction_mov_le_r5_r3_LSL_6[] = {
  0xd8, 0xbf, 0x9d, 0x01 // It le; mov le r5 r3 LSL 6
};
const byte kInstruction_mov_eq_r3_r7_LSL_1[] = {
  0x08, 0xbf, 0x7b, 0x00 // It eq; mov eq r3 r7 LSL 1
};
const byte kInstruction_mov_ge_r1_r0_LSL_28[] = {
  0xa8, 0xbf, 0x01, 0x07 // It ge; mov ge r1 r0 LSL 28
};
const byte kInstruction_mov_mi_r3_r4_LSL_7[] = {
  0x48, 0xbf, 0xe3, 0x01 // It mi; mov mi r3 r4 LSL 7
};
const byte kInstruction_mov_pl_r0_r4_LSL_13[] = {
  0x58, 0xbf, 0x60, 0x03 // It pl; mov pl r0 r4 LSL 13
};
const byte kInstruction_mov_lt_r0_r4_LSL_5[] = {
  0xb8, 0xbf, 0x60, 0x01 // It lt; mov lt r0 r4 LSL 5
};
const byte kInstruction_mov_cc_r2_r2_LSL_1[] = {
  0x38, 0xbf, 0x52, 0x00 // It cc; mov cc r2 r2 LSL 1
};
const byte kInstruction_mov_pl_r6_r2_LSL_30[] = {
  0x58, 0xbf, 0x96, 0x07 // It pl; mov pl r6 r2 LSL 30
};
const byte kInstruction_mov_ls_r7_r0_LSL_24[] = {
  0x98, 0xbf, 0x07, 0x06 // It ls; mov ls r7 r0 LSL 24
};
const byte kInstruction_mov_cs_r4_r4_LSL_3[] = {
  0x28, 0xbf, 0xe4, 0x00 // It cs; mov cs r4 r4 LSL 3
};
const byte kInstruction_mov_vs_r7_r0_LSL_19[] = {
  0x68, 0xbf, 0xc7, 0x04 // It vs; mov vs r7 r0 LSL 19
};
const byte kInstruction_mov_cc_r2_r3_LSL_12[] = {
  0x38, 0xbf, 0x1a, 0x03 // It cc; mov cc r2 r3 LSL 12
};
const byte kInstruction_mov_lt_r3_r7_LSL_7[] = {
  0xb8, 0xbf, 0xfb, 0x01 // It lt; mov lt r3 r7 LSL 7
};
const byte kInstruction_mov_mi_r5_r5_LSL_3[] = {
  0x48, 0xbf, 0xed, 0x00 // It mi; mov mi r5 r5 LSL 3
};
const byte kInstruction_mov_vc_r5_r5_LSL_18[] = {
  0x78, 0xbf, 0xad, 0x04 // It vc; mov vc r5 r5 LSL 18
};
const byte kInstruction_mov_vc_r5_r0_LSL_26[] = {
  0x78, 0xbf, 0x85, 0x06 // It vc; mov vc r5 r0 LSL 26
};
const byte kInstruction_mov_ls_r2_r5_LSL_6[] = {
  0x98, 0xbf, 0xaa, 0x01 // It ls; mov ls r2 r5 LSL 6
};
const byte kInstruction_mov_ge_r1_r7_LSL_30[] = {
  0xa8, 0xbf, 0xb9, 0x07 // It ge; mov ge r1 r7 LSL 30
};
const byte kInstruction_mov_ls_r1_r2_LSL_23[] = {
  0x98, 0xbf, 0xd1, 0x05 // It ls; mov ls r1 r2 LSL 23
};
const byte kInstruction_mov_hi_r3_r7_LSL_25[] = {
  0x88, 0xbf, 0x7b, 0x06 // It hi; mov hi r3 r7 LSL 25
};
const byte kInstruction_mov_lt_r7_r0_LSL_26[] = {
  0xb8, 0xbf, 0x87, 0x06 // It lt; mov lt r7 r0 LSL 26
};
const byte kInstruction_mov_vs_r4_r2_LSL_31[] = {
  0x68, 0xbf, 0xd4, 0x07 // It vs; mov vs r4 r2 LSL 31
};
const byte kInstruction_mov_gt_r7_r2_LSL_9[] = {
  0xc8, 0xbf, 0x57, 0x02 // It gt; mov gt r7 r2 LSL 9
};
const byte kInstruction_mov_le_r2_r5_LSL_16[] = {
  0xd8, 0xbf, 0x2a, 0x04 // It le; mov le r2 r5 LSL 16
};
const byte kInstruction_mov_ls_r6_r6_LSL_23[] = {
  0x98, 0xbf, 0xf6, 0x05 // It ls; mov ls r6 r6 LSL 23
};
const byte kInstruction_mov_pl_r5_r5_LSL_5[] = {
  0x58, 0xbf, 0x6d, 0x01 // It pl; mov pl r5 r5 LSL 5
};
const byte kInstruction_mov_gt_r3_r2_LSL_16[] = {
  0xc8, 0xbf, 0x13, 0x04 // It gt; mov gt r3 r2 LSL 16
};
const byte kInstruction_mov_vs_r2_r3_LSL_11[] = {
  0x68, 0xbf, 0xda, 0x02 // It vs; mov vs r2 r3 LSL 11
};
const byte kInstruction_mov_cc_r5_r6_LSL_15[] = {
  0x38, 0xbf, 0xf5, 0x03 // It cc; mov cc r5 r6 LSL 15
};
const byte kInstruction_mov_vc_r0_r0_LSL_29[] = {
  0x78, 0xbf, 0x40, 0x07 // It vc; mov vc r0 r0 LSL 29
};
const byte kInstruction_mov_vs_r5_r0_LSL_20[] = {
  0x68, 0xbf, 0x05, 0x05 // It vs; mov vs r5 r0 LSL 20
};
const byte kInstruction_mov_ls_r6_r3_LSL_6[] = {
  0x98, 0xbf, 0x9e, 0x01 // It ls; mov ls r6 r3 LSL 6
};
const byte kInstruction_mov_cc_r6_r7_LSL_2[] = {
  0x38, 0xbf, 0xbe, 0x00 // It cc; mov cc r6 r7 LSL 2
};
const byte kInstruction_mov_eq_r2_r2_LSL_14[] = {
  0x08, 0xbf, 0x92, 0x03 // It eq; mov eq r2 r2 LSL 14
};
const byte kInstruction_mov_ne_r5_r3_LSL_12[] = {
  0x18, 0xbf, 0x1d, 0x03 // It ne; mov ne r5 r3 LSL 12
};
const byte kInstruction_mov_cc_r7_r2_LSL_20[] = {
  0x38, 0xbf, 0x17, 0x05 // It cc; mov cc r7 r2 LSL 20
};
const byte kInstruction_mov_lt_r5_r0_LSL_24[] = {
  0xb8, 0xbf, 0x05, 0x06 // It lt; mov lt r5 r0 LSL 24
};
const byte kInstruction_mov_ge_r6_r7_LSL_21[] = {
  0xa8, 0xbf, 0x7e, 0x05 // It ge; mov ge r6 r7 LSL 21
};
const byte kInstruction_mov_ls_r6_r7_LSL_3[] = {
  0x98, 0xbf, 0xfe, 0x00 // It ls; mov ls r6 r7 LSL 3
};
const byte kInstruction_mov_cs_r3_r6_LSL_7[] = {
  0x28, 0xbf, 0xf3, 0x01 // It cs; mov cs r3 r6 LSL 7
};
const byte kInstruction_mov_ls_r0_r5_LSL_26[] = {
  0x98, 0xbf, 0xa8, 0x06 // It ls; mov ls r0 r5 LSL 26
};
const byte kInstruction_mov_ne_r3_r0_LSL_6[] = {
  0x18, 0xbf, 0x83, 0x01 // It ne; mov ne r3 r0 LSL 6
};
const byte kInstruction_mov_gt_r3_r7_LSL_29[] = {
  0xc8, 0xbf, 0x7b, 0x07 // It gt; mov gt r3 r7 LSL 29
};
const byte kInstruction_mov_ne_r7_r3_LSL_5[] = {
  0x18, 0xbf, 0x5f, 0x01 // It ne; mov ne r7 r3 LSL 5
};
const byte kInstruction_mov_vs_r2_r1_LSL_31[] = {
  0x68, 0xbf, 0xca, 0x07 // It vs; mov vs r2 r1 LSL 31
};
const byte kInstruction_mov_ne_r1_r2_LSL_27[] = {
  0x18, 0xbf, 0xd1, 0x06 // It ne; mov ne r1 r2 LSL 27
};
const byte kInstruction_mov_pl_r5_r0_LSL_25[] = {
  0x58, 0xbf, 0x45, 0x06 // It pl; mov pl r5 r0 LSL 25
};
const byte kInstruction_mov_hi_r5_r0_LSL_14[] = {
  0x88, 0xbf, 0x85, 0x03 // It hi; mov hi r5 r0 LSL 14
};
const byte kInstruction_mov_vs_r5_r5_LSL_19[] = {
  0x68, 0xbf, 0xed, 0x04 // It vs; mov vs r5 r5 LSL 19
};
const byte kInstruction_mov_le_r6_r6_LSL_10[] = {
  0xd8, 0xbf, 0xb6, 0x02 // It le; mov le r6 r6 LSL 10
};
const byte kInstruction_mov_ne_r2_r1_LSL_3[] = {
  0x18, 0xbf, 0xca, 0x00 // It ne; mov ne r2 r1 LSL 3
};
const byte kInstruction_mov_gt_r2_r7_LSL_7[] = {
  0xc8, 0xbf, 0xfa, 0x01 // It gt; mov gt r2 r7 LSL 7
};
const byte kInstruction_mov_vs_r0_r6_LSL_31[] = {
  0x68, 0xbf, 0xf0, 0x07 // It vs; mov vs r0 r6 LSL 31
};
const byte kInstruction_mov_ne_r4_r4_LSL_28[] = {
  0x18, 0xbf, 0x24, 0x07 // It ne; mov ne r4 r4 LSL 28
};
const byte kInstruction_mov_mi_r1_r0_LSL_14[] = {
  0x48, 0xbf, 0x81, 0x03 // It mi; mov mi r1 r0 LSL 14
};
const byte kInstruction_mov_lt_r0_r6_LSL_30[] = {
  0xb8, 0xbf, 0xb0, 0x07 // It lt; mov lt r0 r6 LSL 30
};
const byte kInstruction_mov_lt_r2_r4_LSL_10[] = {
  0xb8, 0xbf, 0xa2, 0x02 // It lt; mov lt r2 r4 LSL 10
};
const byte kInstruction_mov_lt_r0_r3_LSL_22[] = {
  0xb8, 0xbf, 0x98, 0x05 // It lt; mov lt r0 r3 LSL 22
};
const byte kInstruction_mov_ls_r5_r5_LSL_28[] = {
  0x98, 0xbf, 0x2d, 0x07 // It ls; mov ls r5 r5 LSL 28
};
const byte kInstruction_mov_pl_r4_r1_LSL_20[] = {
  0x58, 0xbf, 0x0c, 0x05 // It pl; mov pl r4 r1 LSL 20
};
const byte kInstruction_mov_le_r0_r1_LSL_14[] = {
  0xd8, 0xbf, 0x88, 0x03 // It le; mov le r0 r1 LSL 14
};
const byte kInstruction_mov_le_r2_r1_LSL_8[] = {
  0xd8, 0xbf, 0x0a, 0x02 // It le; mov le r2 r1 LSL 8
};
const byte kInstruction_mov_ls_r2_r1_LSL_20[] = {
  0x98, 0xbf, 0x0a, 0x05 // It ls; mov ls r2 r1 LSL 20
};
const byte kInstruction_mov_le_r4_r0_LSL_14[] = {
  0xd8, 0xbf, 0x84, 0x03 // It le; mov le r4 r0 LSL 14
};
const byte kInstruction_mov_lt_r7_r4_LSL_17[] = {
  0xb8, 0xbf, 0x67, 0x04 // It lt; mov lt r7 r4 LSL 17
};
const byte kInstruction_mov_cc_r6_r1_LSL_3[] = {
  0x38, 0xbf, 0xce, 0x00 // It cc; mov cc r6 r1 LSL 3
};
const byte kInstruction_mov_pl_r6_r3_LSL_1[] = {
  0x58, 0xbf, 0x5e, 0x00 // It pl; mov pl r6 r3 LSL 1
};
const byte kInstruction_mov_cc_r2_r2_LSL_3[] = {
  0x38, 0xbf, 0xd2, 0x00 // It cc; mov cc r2 r2 LSL 3
};
const byte kInstruction_mov_ge_r2_r6_LSL_22[] = {
  0xa8, 0xbf, 0xb2, 0x05 // It ge; mov ge r2 r6 LSL 22
};
const byte kInstruction_mov_gt_r6_r0_LSL_23[] = {
  0xc8, 0xbf, 0xc6, 0x05 // It gt; mov gt r6 r0 LSL 23
};
const byte kInstruction_mov_ge_r5_r1_LSL_18[] = {
  0xa8, 0xbf, 0x8d, 0x04 // It ge; mov ge r5 r1 LSL 18
};
const byte kInstruction_mov_eq_r7_r0_LSL_11[] = {
  0x08, 0xbf, 0xc7, 0x02 // It eq; mov eq r7 r0 LSL 11
};
const byte kInstruction_mov_mi_r2_r7_LSL_8[] = {
  0x48, 0xbf, 0x3a, 0x02 // It mi; mov mi r2 r7 LSL 8
};
const byte kInstruction_mov_pl_r2_r7_LSL_19[] = {
  0x58, 0xbf, 0xfa, 0x04 // It pl; mov pl r2 r7 LSL 19
};
const byte kInstruction_mov_mi_r3_r6_LSL_1[] = {
  0x48, 0xbf, 0x73, 0x00 // It mi; mov mi r3 r6 LSL 1
};
const byte kInstruction_mov_lt_r6_r5_LSL_29[] = {
  0xb8, 0xbf, 0x6e, 0x07 // It lt; mov lt r6 r5 LSL 29
};
const byte kInstruction_mov_eq_r0_r4_LSL_9[] = {
  0x08, 0xbf, 0x60, 0x02 // It eq; mov eq r0 r4 LSL 9
};
const byte kInstruction_mov_hi_r3_r1_LSL_7[] = {
  0x88, 0xbf, 0xcb, 0x01 // It hi; mov hi r3 r1 LSL 7
};
const byte kInstruction_mov_cs_r5_r5_LSL_12[] = {
  0x28, 0xbf, 0x2d, 0x03 // It cs; mov cs r5 r5 LSL 12
};
const byte kInstruction_mov_lt_r1_r5_LSL_24[] = {
  0xb8, 0xbf, 0x29, 0x06 // It lt; mov lt r1 r5 LSL 24
};
const byte kInstruction_mov_cc_r6_r0_LSL_4[] = {
  0x38, 0xbf, 0x06, 0x01 // It cc; mov cc r6 r0 LSL 4
};
const byte kInstruction_mov_hi_r3_r0_LSL_15[] = {
  0x88, 0xbf, 0xc3, 0x03 // It hi; mov hi r3 r0 LSL 15
};
const byte kInstruction_mov_gt_r7_r2_LSL_17[] = {
  0xc8, 0xbf, 0x57, 0x04 // It gt; mov gt r7 r2 LSL 17
};
const byte kInstruction_mov_hi_r4_r0_LSL_4[] = {
  0x88, 0xbf, 0x04, 0x01 // It hi; mov hi r4 r0 LSL 4
};
const byte kInstruction_mov_cc_r5_r4_LSL_22[] = {
  0x38, 0xbf, 0xa5, 0x05 // It cc; mov cc r5 r4 LSL 22
};
const byte kInstruction_mov_lt_r3_r1_LSL_13[] = {
  0xb8, 0xbf, 0x4b, 0x03 // It lt; mov lt r3 r1 LSL 13
};
const byte kInstruction_mov_le_r4_r7_LSL_5[] = {
  0xd8, 0xbf, 0x7c, 0x01 // It le; mov le r4 r7 LSL 5
};
const byte kInstruction_mov_cs_r0_r5_LSL_22[] = {
  0x28, 0xbf, 0xa8, 0x05 // It cs; mov cs r0 r5 LSL 22
};
const byte kInstruction_mov_eq_r3_r6_LSL_1[] = {
  0x08, 0xbf, 0x73, 0x00 // It eq; mov eq r3 r6 LSL 1
};
const byte kInstruction_mov_vs_r3_r3_LSL_8[] = {
  0x68, 0xbf, 0x1b, 0x02 // It vs; mov vs r3 r3 LSL 8
};
const byte kInstruction_mov_vs_r4_r4_LSL_17[] = {
  0x68, 0xbf, 0x64, 0x04 // It vs; mov vs r4 r4 LSL 17
};
const byte kInstruction_mov_ne_r0_r3_LSL_16[] = {
  0x18, 0xbf, 0x18, 0x04 // It ne; mov ne r0 r3 LSL 16
};
const byte kInstruction_mov_le_r4_r7_LSL_4[] = {
  0xd8, 0xbf, 0x3c, 0x01 // It le; mov le r4 r7 LSL 4
};
const byte kInstruction_mov_vs_r4_r6_LSL_2[] = {
  0x68, 0xbf, 0xb4, 0x00 // It vs; mov vs r4 r6 LSL 2
};
const byte kInstruction_mov_pl_r7_r4_LSL_8[] = {
  0x58, 0xbf, 0x27, 0x02 // It pl; mov pl r7 r4 LSL 8
};
const byte kInstruction_mov_ls_r6_r2_LSL_2[] = {
  0x98, 0xbf, 0x96, 0x00 // It ls; mov ls r6 r2 LSL 2
};
const byte kInstruction_mov_cc_r2_r2_LSL_13[] = {
  0x38, 0xbf, 0x52, 0x03 // It cc; mov cc r2 r2 LSL 13
};
const byte kInstruction_mov_pl_r7_r5_LSL_7[] = {
  0x58, 0xbf, 0xef, 0x01 // It pl; mov pl r7 r5 LSL 7
};
const byte kInstruction_mov_pl_r3_r6_LSL_6[] = {
  0x58, 0xbf, 0xb3, 0x01 // It pl; mov pl r3 r6 LSL 6
};
const byte kInstruction_mov_ne_r4_r5_LSL_3[] = {
  0x18, 0xbf, 0xec, 0x00 // It ne; mov ne r4 r5 LSL 3
};
const byte kInstruction_mov_lt_r5_r5_LSL_16[] = {
  0xb8, 0xbf, 0x2d, 0x04 // It lt; mov lt r5 r5 LSL 16
};
const byte kInstruction_mov_gt_r1_r2_LSL_23[] = {
  0xc8, 0xbf, 0xd1, 0x05 // It gt; mov gt r1 r2 LSL 23
};
const byte kInstruction_mov_vc_r1_r6_LSL_27[] = {
  0x78, 0xbf, 0xf1, 0x06 // It vc; mov vc r1 r6 LSL 27
};
const byte kInstruction_mov_vc_r6_r0_LSL_3[] = {
  0x78, 0xbf, 0xc6, 0x00 // It vc; mov vc r6 r0 LSL 3
};
const byte kInstruction_mov_vc_r1_r6_LSL_10[] = {
  0x78, 0xbf, 0xb1, 0x02 // It vc; mov vc r1 r6 LSL 10
};
const byte kInstruction_mov_vc_r3_r7_LSL_21[] = {
  0x78, 0xbf, 0x7b, 0x05 // It vc; mov vc r3 r7 LSL 21
};
const byte kInstruction_mov_lt_r5_r7_LSL_31[] = {
  0xb8, 0xbf, 0xfd, 0x07 // It lt; mov lt r5 r7 LSL 31
};
const byte kInstruction_mov_vs_r6_r1_LSL_11[] = {
  0x68, 0xbf, 0xce, 0x02 // It vs; mov vs r6 r1 LSL 11
};
const byte kInstruction_mov_gt_r7_r4_LSL_19[] = {
  0xc8, 0xbf, 0xe7, 0x04 // It gt; mov gt r7 r4 LSL 19
};
const byte kInstruction_mov_hi_r0_r4_LSL_22[] = {
  0x88, 0xbf, 0xa0, 0x05 // It hi; mov hi r0 r4 LSL 22
};
const byte kInstruction_mov_hi_r2_r0_LSL_18[] = {
  0x88, 0xbf, 0x82, 0x04 // It hi; mov hi r2 r0 LSL 18
};
const byte kInstruction_mov_ls_r1_r3_LSL_12[] = {
  0x98, 0xbf, 0x19, 0x03 // It ls; mov ls r1 r3 LSL 12
};
const byte kInstruction_mov_hi_r0_r6_LSL_23[] = {
  0x88, 0xbf, 0xf0, 0x05 // It hi; mov hi r0 r6 LSL 23
};
const byte kInstruction_mov_ne_r3_r5_LSL_22[] = {
  0x18, 0xbf, 0xab, 0x05 // It ne; mov ne r3 r5 LSL 22
};
const byte kInstruction_mov_ge_r4_r4_LSL_4[] = {
  0xa8, 0xbf, 0x24, 0x01 // It ge; mov ge r4 r4 LSL 4
};
const byte kInstruction_mov_cc_r1_r6_LSL_25[] = {
  0x38, 0xbf, 0x71, 0x06 // It cc; mov cc r1 r6 LSL 25
};
const byte kInstruction_mov_vs_r2_r1_LSL_13[] = {
  0x68, 0xbf, 0x4a, 0x03 // It vs; mov vs r2 r1 LSL 13
};
const byte kInstruction_mov_lt_r6_r7_LSL_6[] = {
  0xb8, 0xbf, 0xbe, 0x01 // It lt; mov lt r6 r7 LSL 6
};
const byte kInstruction_mov_gt_r5_r6_LSL_7[] = {
  0xc8, 0xbf, 0xf5, 0x01 // It gt; mov gt r5 r6 LSL 7
};
const byte kInstruction_mov_ge_r5_r1_LSL_27[] = {
  0xa8, 0xbf, 0xcd, 0x06 // It ge; mov ge r5 r1 LSL 27
};
const byte kInstruction_mov_vs_r2_r3_LSL_8[] = {
  0x68, 0xbf, 0x1a, 0x02 // It vs; mov vs r2 r3 LSL 8
};
const byte kInstruction_mov_gt_r0_r3_LSL_4[] = {
  0xc8, 0xbf, 0x18, 0x01 // It gt; mov gt r0 r3 LSL 4
};
const byte kInstruction_mov_vc_r5_r6_LSL_17[] = {
  0x78, 0xbf, 0x75, 0x04 // It vc; mov vc r5 r6 LSL 17
};
const byte kInstruction_mov_mi_r5_r2_LSL_23[] = {
  0x48, 0xbf, 0xd5, 0x05 // It mi; mov mi r5 r2 LSL 23
};
const byte kInstruction_mov_cc_r3_r3_LSL_10[] = {
  0x38, 0xbf, 0x9b, 0x02 // It cc; mov cc r3 r3 LSL 10
};
const byte kInstruction_mov_le_r5_r3_LSL_18[] = {
  0xd8, 0xbf, 0x9d, 0x04 // It le; mov le r5 r3 LSL 18
};
const byte kInstruction_mov_le_r4_r0_LSL_9[] = {
  0xd8, 0xbf, 0x44, 0x02 // It le; mov le r4 r0 LSL 9
};
const byte kInstruction_mov_pl_r0_r1_LSL_10[] = {
  0x58, 0xbf, 0x88, 0x02 // It pl; mov pl r0 r1 LSL 10
};
const byte kInstruction_mov_ls_r6_r4_LSL_2[] = {
  0x98, 0xbf, 0xa6, 0x00 // It ls; mov ls r6 r4 LSL 2
};
const byte kInstruction_mov_cc_r6_r6_LSL_5[] = {
  0x38, 0xbf, 0x76, 0x01 // It cc; mov cc r6 r6 LSL 5
};
const byte kInstruction_mov_pl_r2_r5_LSL_2[] = {
  0x58, 0xbf, 0xaa, 0x00 // It pl; mov pl r2 r5 LSL 2
};
const byte kInstruction_mov_vs_r7_r0_LSL_3[] = {
  0x68, 0xbf, 0xc7, 0x00 // It vs; mov vs r7 r0 LSL 3
};
const byte kInstruction_mov_vs_r7_r7_LSL_8[] = {
  0x68, 0xbf, 0x3f, 0x02 // It vs; mov vs r7 r7 LSL 8
};
const byte kInstruction_mov_ge_r0_r7_LSL_5[] = {
  0xa8, 0xbf, 0x78, 0x01 // It ge; mov ge r0 r7 LSL 5
};
const byte kInstruction_mov_gt_r3_r7_LSL_16[] = {
  0xc8, 0xbf, 0x3b, 0x04 // It gt; mov gt r3 r7 LSL 16
};
const byte kInstruction_mov_mi_r4_r2_LSL_22[] = {
  0x48, 0xbf, 0x94, 0x05 // It mi; mov mi r4 r2 LSL 22
};
const byte kInstruction_mov_le_r1_r2_LSL_4[] = {
  0xd8, 0xbf, 0x11, 0x01 // It le; mov le r1 r2 LSL 4
};
const byte kInstruction_mov_mi_r5_r4_LSL_3[] = {
  0x48, 0xbf, 0xe5, 0x00 // It mi; mov mi r5 r4 LSL 3
};
const byte kInstruction_mov_hi_r2_r3_LSL_26[] = {
  0x88, 0xbf, 0x9a, 0x06 // It hi; mov hi r2 r3 LSL 26
};
const byte kInstruction_mov_ne_r6_r1_LSL_29[] = {
  0x18, 0xbf, 0x4e, 0x07 // It ne; mov ne r6 r1 LSL 29
};
const byte kInstruction_mov_gt_r6_r0_LSL_29[] = {
  0xc8, 0xbf, 0x46, 0x07 // It gt; mov gt r6 r0 LSL 29
};
const byte kInstruction_mov_pl_r3_r2_LSL_19[] = {
  0x58, 0xbf, 0xd3, 0x04 // It pl; mov pl r3 r2 LSL 19
};
const byte kInstruction_mov_ls_r5_r2_LSL_5[] = {
  0x98, 0xbf, 0x55, 0x01 // It ls; mov ls r5 r2 LSL 5
};
const byte kInstruction_mov_ne_r4_r5_LSL_16[] = {
  0x18, 0xbf, 0x2c, 0x04 // It ne; mov ne r4 r5 LSL 16
};
const byte kInstruction_mov_vc_r2_r1_LSL_2[] = {
  0x78, 0xbf, 0x8a, 0x00 // It vc; mov vc r2 r1 LSL 2
};
const byte kInstruction_mov_ne_r0_r7_LSL_27[] = {
  0x18, 0xbf, 0xf8, 0x06 // It ne; mov ne r0 r7 LSL 27
};
const byte kInstruction_mov_eq_r4_r2_LSL_30[] = {
  0x08, 0xbf, 0x94, 0x07 // It eq; mov eq r4 r2 LSL 30
};
const byte kInstruction_mov_hi_r7_r2_LSL_18[] = {
  0x88, 0xbf, 0x97, 0x04 // It hi; mov hi r7 r2 LSL 18
};
const byte kInstruction_mov_eq_r2_r0_LSL_21[] = {
  0x08, 0xbf, 0x42, 0x05 // It eq; mov eq r2 r0 LSL 21
};
const byte kInstruction_mov_hi_r1_r1_LSL_9[] = {
  0x88, 0xbf, 0x49, 0x02 // It hi; mov hi r1 r1 LSL 9
};
const byte kInstruction_mov_lt_r6_r6_LSL_10[] = {
  0xb8, 0xbf, 0xb6, 0x02 // It lt; mov lt r6 r6 LSL 10
};
const byte kInstruction_mov_vc_r4_r5_LSL_15[] = {
  0x78, 0xbf, 0xec, 0x03 // It vc; mov vc r4 r5 LSL 15
};
const byte kInstruction_mov_cc_r1_r1_LSL_12[] = {
  0x38, 0xbf, 0x09, 0x03 // It cc; mov cc r1 r1 LSL 12
};
const byte kInstruction_mov_le_r0_r4_LSL_29[] = {
  0xd8, 0xbf, 0x60, 0x07 // It le; mov le r0 r4 LSL 29
};
const byte kInstruction_mov_cs_r6_r2_LSL_12[] = {
  0x28, 0xbf, 0x16, 0x03 // It cs; mov cs r6 r2 LSL 12
};
const byte kInstruction_mov_pl_r6_r2_LSL_21[] = {
  0x58, 0xbf, 0x56, 0x05 // It pl; mov pl r6 r2 LSL 21
};
const byte kInstruction_mov_le_r4_r4_LSL_31[] = {
  0xd8, 0xbf, 0xe4, 0x07 // It le; mov le r4 r4 LSL 31
};
const byte kInstruction_mov_cs_r3_r6_LSL_26[] = {
  0x28, 0xbf, 0xb3, 0x06 // It cs; mov cs r3 r6 LSL 26
};
const byte kInstruction_mov_ne_r1_r2_LSL_2[] = {
  0x18, 0xbf, 0x91, 0x00 // It ne; mov ne r1 r2 LSL 2
};
const byte kInstruction_mov_vc_r0_r4_LSL_24[] = {
  0x78, 0xbf, 0x20, 0x06 // It vc; mov vc r0 r4 LSL 24
};
const byte kInstruction_mov_mi_r3_r4_LSL_9[] = {
  0x48, 0xbf, 0x63, 0x02 // It mi; mov mi r3 r4 LSL 9
};
const byte kInstruction_mov_pl_r5_r1_LSL_23[] = {
  0x58, 0xbf, 0xcd, 0x05 // It pl; mov pl r5 r1 LSL 23
};
const byte kInstruction_mov_ls_r1_r6_LSL_31[] = {
  0x98, 0xbf, 0xf1, 0x07 // It ls; mov ls r1 r6 LSL 31
};
const byte kInstruction_mov_mi_r6_r2_LSL_6[] = {
  0x48, 0xbf, 0x96, 0x01 // It mi; mov mi r6 r2 LSL 6
};
const byte kInstruction_mov_vc_r1_r3_LSL_27[] = {
  0x78, 0xbf, 0xd9, 0x06 // It vc; mov vc r1 r3 LSL 27
};
const byte kInstruction_mov_vc_r6_r7_LSL_26[] = {
  0x78, 0xbf, 0xbe, 0x06 // It vc; mov vc r6 r7 LSL 26
};
const byte kInstruction_mov_vs_r2_r6_LSL_10[] = {
  0x68, 0xbf, 0xb2, 0x02 // It vs; mov vs r2 r6 LSL 10
};
const byte kInstruction_mov_ge_r5_r6_LSL_5[] = {
  0xa8, 0xbf, 0x75, 0x01 // It ge; mov ge r5 r6 LSL 5
};
const byte kInstruction_mov_hi_r0_r5_LSL_19[] = {
  0x88, 0xbf, 0xe8, 0x04 // It hi; mov hi r0 r5 LSL 19
};
const byte kInstruction_mov_eq_r4_r1_LSL_13[] = {
  0x08, 0xbf, 0x4c, 0x03 // It eq; mov eq r4 r1 LSL 13
};
const byte kInstruction_mov_eq_r1_r3_LSL_24[] = {
  0x08, 0xbf, 0x19, 0x06 // It eq; mov eq r1 r3 LSL 24
};
const byte kInstruction_mov_hi_r1_r0_LSL_30[] = {
  0x88, 0xbf, 0x81, 0x07 // It hi; mov hi r1 r0 LSL 30
};
const byte kInstruction_mov_lt_r7_r0_LSL_11[] = {
  0xb8, 0xbf, 0xc7, 0x02 // It lt; mov lt r7 r0 LSL 11
};
const byte kInstruction_mov_le_r1_r5_LSL_3[] = {
  0xd8, 0xbf, 0xe9, 0x00 // It le; mov le r1 r5 LSL 3
};
const byte kInstruction_mov_gt_r2_r5_LSL_1[] = {
  0xc8, 0xbf, 0x6a, 0x00 // It gt; mov gt r2 r5 LSL 1
};
const byte kInstruction_mov_mi_r1_r0_LSL_6[] = {
  0x48, 0xbf, 0x81, 0x01 // It mi; mov mi r1 r0 LSL 6
};
const byte kInstruction_mov_le_r1_r6_LSL_24[] = {
  0xd8, 0xbf, 0x31, 0x06 // It le; mov le r1 r6 LSL 24
};
const byte kInstruction_mov_vs_r4_r2_LSL_17[] = {
  0x68, 0xbf, 0x54, 0x04 // It vs; mov vs r4 r2 LSL 17
};
const byte kInstruction_mov_ls_r2_r3_LSL_20[] = {
  0x98, 0xbf, 0x1a, 0x05 // It ls; mov ls r2 r3 LSL 20
};
const byte kInstruction_mov_pl_r0_r7_LSL_8[] = {
  0x58, 0xbf, 0x38, 0x02 // It pl; mov pl r0 r7 LSL 8
};
const byte kInstruction_mov_ne_r1_r3_LSL_19[] = {
  0x18, 0xbf, 0xd9, 0x04 // It ne; mov ne r1 r3 LSL 19
};
const byte kInstruction_mov_gt_r4_r6_LSL_2[] = {
  0xc8, 0xbf, 0xb4, 0x00 // It gt; mov gt r4 r6 LSL 2
};
const byte kInstruction_mov_vc_r5_r0_LSL_1[] = {
  0x78, 0xbf, 0x45, 0x00 // It vc; mov vc r5 r0 LSL 1
};
const byte kInstruction_mov_le_r1_r1_LSL_28[] = {
  0xd8, 0xbf, 0x09, 0x07 // It le; mov le r1 r1 LSL 28
};
const byte kInstruction_mov_vc_r2_r4_LSL_17[] = {
  0x78, 0xbf, 0x62, 0x04 // It vc; mov vc r2 r4 LSL 17
};
const byte kInstruction_mov_le_r5_r2_LSL_25[] = {
  0xd8, 0xbf, 0x55, 0x06 // It le; mov le r5 r2 LSL 25
};
const byte kInstruction_mov_cc_r6_r3_LSL_22[] = {
  0x38, 0xbf, 0x9e, 0x05 // It cc; mov cc r6 r3 LSL 22
};
const byte kInstruction_mov_cc_r3_r3_LSL_20[] = {
  0x38, 0xbf, 0x1b, 0x05 // It cc; mov cc r3 r3 LSL 20
};
const byte kInstruction_mov_pl_r4_r5_LSL_20[] = {
  0x58, 0xbf, 0x2c, 0x05 // It pl; mov pl r4 r5 LSL 20
};
const byte kInstruction_mov_cc_r2_r3_LSL_14[] = {
  0x38, 0xbf, 0x9a, 0x03 // It cc; mov cc r2 r3 LSL 14
};
const byte kInstruction_mov_eq_r4_r0_LSL_8[] = {
  0x08, 0xbf, 0x04, 0x02 // It eq; mov eq r4 r0 LSL 8
};
const byte kInstruction_mov_lt_r5_r0_LSL_15[] = {
  0xb8, 0xbf, 0xc5, 0x03 // It lt; mov lt r5 r0 LSL 15
};
const byte kInstruction_mov_pl_r5_r6_LSL_25[] = {
  0x58, 0xbf, 0x75, 0x06 // It pl; mov pl r5 r6 LSL 25
};
const byte kInstruction_mov_ls_r5_r3_LSL_6[] = {
  0x98, 0xbf, 0x9d, 0x01 // It ls; mov ls r5 r3 LSL 6
};
const byte kInstruction_mov_ge_r7_r0_LSL_7[] = {
  0xa8, 0xbf, 0xc7, 0x01 // It ge; mov ge r7 r0 LSL 7
};
const byte kInstruction_mov_le_r0_r7_LSL_12[] = {
  0xd8, 0xbf, 0x38, 0x03 // It le; mov le r0 r7 LSL 12
};
const byte kInstruction_mov_hi_r4_r4_LSL_31[] = {
  0x88, 0xbf, 0xe4, 0x07 // It hi; mov hi r4 r4 LSL 31
};
const byte kInstruction_mov_pl_r5_r1_LSL_22[] = {
  0x58, 0xbf, 0x8d, 0x05 // It pl; mov pl r5 r1 LSL 22
};
const byte kInstruction_mov_cc_r0_r3_LSL_11[] = {
  0x38, 0xbf, 0xd8, 0x02 // It cc; mov cc r0 r3 LSL 11
};
const byte kInstruction_mov_le_r2_r4_LSL_9[] = {
  0xd8, 0xbf, 0x62, 0x02 // It le; mov le r2 r4 LSL 9
};
const byte kInstruction_mov_ne_r6_r2_LSL_17[] = {
  0x18, 0xbf, 0x56, 0x04 // It ne; mov ne r6 r2 LSL 17
};
const byte kInstruction_mov_vs_r1_r1_LSL_7[] = {
  0x68, 0xbf, 0xc9, 0x01 // It vs; mov vs r1 r1 LSL 7
};
const byte kInstruction_mov_cc_r5_r7_LSL_2[] = {
  0x38, 0xbf, 0xbd, 0x00 // It cc; mov cc r5 r7 LSL 2
};
const byte kInstruction_mov_le_r7_r3_LSL_15[] = {
  0xd8, 0xbf, 0xdf, 0x03 // It le; mov le r7 r3 LSL 15
};
const byte kInstruction_mov_ge_r7_r0_LSL_20[] = {
  0xa8, 0xbf, 0x07, 0x05 // It ge; mov ge r7 r0 LSL 20
};
const byte kInstruction_mov_vc_r3_r4_LSL_2[] = {
  0x78, 0xbf, 0xa3, 0x00 // It vc; mov vc r3 r4 LSL 2
};
const byte kInstruction_mov_eq_r5_r0_LSL_10[] = {
  0x08, 0xbf, 0x85, 0x02 // It eq; mov eq r5 r0 LSL 10
};
const byte kInstruction_mov_cc_r5_r6_LSL_4[] = {
  0x38, 0xbf, 0x35, 0x01 // It cc; mov cc r5 r6 LSL 4
};
const byte kInstruction_mov_ge_r4_r1_LSL_23[] = {
  0xa8, 0xbf, 0xcc, 0x05 // It ge; mov ge r4 r1 LSL 23
};
const byte kInstruction_mov_cs_r2_r1_LSL_18[] = {
  0x28, 0xbf, 0x8a, 0x04 // It cs; mov cs r2 r1 LSL 18
};
const byte kInstruction_mov_gt_r1_r3_LSL_22[] = {
  0xc8, 0xbf, 0x99, 0x05 // It gt; mov gt r1 r3 LSL 22
};
const byte kInstruction_mov_cc_r0_r3_LSL_7[] = {
  0x38, 0xbf, 0xd8, 0x01 // It cc; mov cc r0 r3 LSL 7
};
const byte kInstruction_mov_cs_r1_r6_LSL_3[] = {
  0x28, 0xbf, 0xf1, 0x00 // It cs; mov cs r1 r6 LSL 3
};
const byte kInstruction_mov_ne_r0_r4_LSL_14[] = {
  0x18, 0xbf, 0xa0, 0x03 // It ne; mov ne r0 r4 LSL 14
};
const byte kInstruction_mov_ls_r0_r1_LSL_31[] = {
  0x98, 0xbf, 0xc8, 0x07 // It ls; mov ls r0 r1 LSL 31
};
const byte kInstruction_mov_ge_r1_r2_LSL_4[] = {
  0xa8, 0xbf, 0x11, 0x01 // It ge; mov ge r1 r2 LSL 4
};
const byte kInstruction_mov_ne_r1_r4_LSL_6[] = {
  0x18, 0xbf, 0xa1, 0x01 // It ne; mov ne r1 r4 LSL 6
};
const byte kInstruction_mov_eq_r2_r6_LSL_3[] = {
  0x08, 0xbf, 0xf2, 0x00 // It eq; mov eq r2 r6 LSL 3
};
const byte kInstruction_mov_gt_r3_r7_LSL_7[] = {
  0xc8, 0xbf, 0xfb, 0x01 // It gt; mov gt r3 r7 LSL 7
};
const byte kInstruction_mov_ls_r3_r6_LSL_25[] = {
  0x98, 0xbf, 0x73, 0x06 // It ls; mov ls r3 r6 LSL 25
};
const byte kInstruction_mov_lt_r2_r4_LSL_4[] = {
  0xb8, 0xbf, 0x22, 0x01 // It lt; mov lt r2 r4 LSL 4
};
const byte kInstruction_mov_cs_r5_r6_LSL_24[] = {
  0x28, 0xbf, 0x35, 0x06 // It cs; mov cs r5 r6 LSL 24
};
const byte kInstruction_mov_pl_r6_r6_LSL_19[] = {
  0x58, 0xbf, 0xf6, 0x04 // It pl; mov pl r6 r6 LSL 19
};
const byte kInstruction_mov_vs_r1_r0_LSL_10[] = {
  0x68, 0xbf, 0x81, 0x02 // It vs; mov vs r1 r0 LSL 10
};
const byte kInstruction_mov_cs_r6_r4_LSL_17[] = {
  0x28, 0xbf, 0x66, 0x04 // It cs; mov cs r6 r4 LSL 17
};
const byte kInstruction_mov_lt_r4_r3_LSL_5[] = {
  0xb8, 0xbf, 0x5c, 0x01 // It lt; mov lt r4 r3 LSL 5
};
const byte kInstruction_mov_cs_r6_r5_LSL_30[] = {
  0x28, 0xbf, 0xae, 0x07 // It cs; mov cs r6 r5 LSL 30
};
const byte kInstruction_mov_cs_r7_r4_LSL_27[] = {
  0x28, 0xbf, 0xe7, 0x06 // It cs; mov cs r7 r4 LSL 27
};
const byte kInstruction_mov_lt_r2_r6_LSL_4[] = {
  0xb8, 0xbf, 0x32, 0x01 // It lt; mov lt r2 r6 LSL 4
};
const byte kInstruction_mov_gt_r7_r7_LSL_16[] = {
  0xc8, 0xbf, 0x3f, 0x04 // It gt; mov gt r7 r7 LSL 16
};
const byte kInstruction_mov_mi_r1_r4_LSL_11[] = {
  0x48, 0xbf, 0xe1, 0x02 // It mi; mov mi r1 r4 LSL 11
};
const byte kInstruction_mov_ls_r5_r6_LSL_21[] = {
  0x98, 0xbf, 0x75, 0x05 // It ls; mov ls r5 r6 LSL 21
};
const byte kInstruction_mov_le_r1_r4_LSL_25[] = {
  0xd8, 0xbf, 0x61, 0x06 // It le; mov le r1 r4 LSL 25
};
const byte kInstruction_mov_hi_r7_r1_LSL_5[] = {
  0x88, 0xbf, 0x4f, 0x01 // It hi; mov hi r7 r1 LSL 5
};
const byte kInstruction_mov_ls_r5_r1_LSL_15[] = {
  0x98, 0xbf, 0xcd, 0x03 // It ls; mov ls r5 r1 LSL 15
};
const byte kInstruction_mov_ne_r7_r3_LSL_3[] = {
  0x18, 0xbf, 0xdf, 0x00 // It ne; mov ne r7 r3 LSL 3
};
const byte kInstruction_mov_mi_r1_r0_LSL_1[] = {
  0x48, 0xbf, 0x41, 0x00 // It mi; mov mi r1 r0 LSL 1
};
const byte kInstruction_mov_ls_r5_r5_LSL_21[] = {
  0x98, 0xbf, 0x6d, 0x05 // It ls; mov ls r5 r5 LSL 21
};
const byte kInstruction_mov_ls_r1_r0_LSL_11[] = {
  0x98, 0xbf, 0xc1, 0x02 // It ls; mov ls r1 r0 LSL 11
};
const byte kInstruction_mov_cs_r1_r5_LSL_30[] = {
  0x28, 0xbf, 0xa9, 0x07 // It cs; mov cs r1 r5 LSL 30
};
const byte kInstruction_mov_ge_r4_r1_LSL_19[] = {
  0xa8, 0xbf, 0xcc, 0x04 // It ge; mov ge r4 r1 LSL 19
};
const byte kInstruction_mov_pl_r0_r2_LSL_15[] = {
  0x58, 0xbf, 0xd0, 0x03 // It pl; mov pl r0 r2 LSL 15
};
const byte kInstruction_mov_hi_r4_r5_LSL_10[] = {
  0x88, 0xbf, 0xac, 0x02 // It hi; mov hi r4 r5 LSL 10
};
const byte kInstruction_mov_pl_r3_r0_LSL_1[] = {
  0x58, 0xbf, 0x43, 0x00 // It pl; mov pl r3 r0 LSL 1
};
const byte kInstruction_mov_ge_r3_r0_LSL_20[] = {
  0xa8, 0xbf, 0x03, 0x05 // It ge; mov ge r3 r0 LSL 20
};
const byte kInstruction_mov_pl_r6_r4_LSL_20[] = {
  0x58, 0xbf, 0x26, 0x05 // It pl; mov pl r6 r4 LSL 20
};
const byte kInstruction_mov_le_r6_r0_LSL_16[] = {
  0xd8, 0xbf, 0x06, 0x04 // It le; mov le r6 r0 LSL 16
};
const byte kInstruction_mov_vc_r6_r5_LSL_14[] = {
  0x78, 0xbf, 0xae, 0x03 // It vc; mov vc r6 r5 LSL 14
};
const byte kInstruction_mov_mi_r6_r2_LSL_20[] = {
  0x48, 0xbf, 0x16, 0x05 // It mi; mov mi r6 r2 LSL 20
};
const byte kInstruction_mov_lt_r1_r3_LSL_18[] = {
  0xb8, 0xbf, 0x99, 0x04 // It lt; mov lt r1 r3 LSL 18
};
const byte kInstruction_mov_eq_r1_r2_LSL_5[] = {
  0x08, 0xbf, 0x51, 0x01 // It eq; mov eq r1 r2 LSL 5
};
const byte kInstruction_mov_ge_r1_r0_LSL_23[] = {
  0xa8, 0xbf, 0xc1, 0x05 // It ge; mov ge r1 r0 LSL 23
};
const byte kInstruction_mov_cc_r2_r5_LSL_7[] = {
  0x38, 0xbf, 0xea, 0x01 // It cc; mov cc r2 r5 LSL 7
};
const byte kInstruction_mov_cc_r1_r7_LSL_3[] = {
  0x38, 0xbf, 0xf9, 0x00 // It cc; mov cc r1 r7 LSL 3
};
const byte kInstruction_mov_mi_r2_r5_LSL_29[] = {
  0x48, 0xbf, 0x6a, 0x07 // It mi; mov mi r2 r5 LSL 29
};
const byte kInstruction_mov_cc_r5_r5_LSL_9[] = {
  0x38, 0xbf, 0x6d, 0x02 // It cc; mov cc r5 r5 LSL 9
};
const byte kInstruction_mov_cs_r3_r2_LSL_1[] = {
  0x28, 0xbf, 0x53, 0x00 // It cs; mov cs r3 r2 LSL 1
};
const byte kInstruction_mov_ge_r5_r3_LSL_15[] = {
  0xa8, 0xbf, 0xdd, 0x03 // It ge; mov ge r5 r3 LSL 15
};
const byte kInstruction_mov_gt_r7_r1_LSL_10[] = {
  0xc8, 0xbf, 0x8f, 0x02 // It gt; mov gt r7 r1 LSL 10
};
const byte kInstruction_mov_ls_r4_r6_LSL_15[] = {
  0x98, 0xbf, 0xf4, 0x03 // It ls; mov ls r4 r6 LSL 15
};
const byte kInstruction_mov_gt_r3_r1_LSL_15[] = {
  0xc8, 0xbf, 0xcb, 0x03 // It gt; mov gt r3 r1 LSL 15
};
const byte kInstruction_mov_cs_r5_r5_LSL_30[] = {
  0x28, 0xbf, 0xad, 0x07 // It cs; mov cs r5 r5 LSL 30
};
const byte kInstruction_mov_eq_r4_r2_LSL_19[] = {
  0x08, 0xbf, 0xd4, 0x04 // It eq; mov eq r4 r2 LSL 19
};
const byte kInstruction_mov_vc_r0_r5_LSL_21[] = {
  0x78, 0xbf, 0x68, 0x05 // It vc; mov vc r0 r5 LSL 21
};
const byte kInstruction_mov_cs_r2_r0_LSL_3[] = {
  0x28, 0xbf, 0xc2, 0x00 // It cs; mov cs r2 r0 LSL 3
};
const byte kInstruction_mov_ne_r1_r2_LSL_20[] = {
  0x18, 0xbf, 0x11, 0x05 // It ne; mov ne r1 r2 LSL 20
};
const byte kInstruction_mov_ne_r6_r0_LSL_28[] = {
  0x18, 0xbf, 0x06, 0x07 // It ne; mov ne r6 r0 LSL 28
};
const byte kInstruction_mov_ne_r7_r3_LSL_7[] = {
  0x18, 0xbf, 0xdf, 0x01 // It ne; mov ne r7 r3 LSL 7
};
const byte kInstruction_mov_mi_r5_r4_LSL_29[] = {
  0x48, 0xbf, 0x65, 0x07 // It mi; mov mi r5 r4 LSL 29
};
const byte kInstruction_mov_vs_r3_r5_LSL_18[] = {
  0x68, 0xbf, 0xab, 0x04 // It vs; mov vs r3 r5 LSL 18
};
const byte kInstruction_mov_mi_r1_r2_LSL_1[] = {
  0x48, 0xbf, 0x51, 0x00 // It mi; mov mi r1 r2 LSL 1
};
const byte kInstruction_mov_hi_r4_r7_LSL_21[] = {
  0x88, 0xbf, 0x7c, 0x05 // It hi; mov hi r4 r7 LSL 21
};
const byte kInstruction_mov_ge_r6_r1_LSL_21[] = {
  0xa8, 0xbf, 0x4e, 0x05 // It ge; mov ge r6 r1 LSL 21
};
const byte kInstruction_mov_lt_r4_r5_LSL_20[] = {
  0xb8, 0xbf, 0x2c, 0x05 // It lt; mov lt r4 r5 LSL 20
};
const byte kInstruction_mov_vs_r6_r7_LSL_2[] = {
  0x68, 0xbf, 0xbe, 0x00 // It vs; mov vs r6 r7 LSL 2
};
const byte kInstruction_mov_hi_r1_r6_LSL_30[] = {
  0x88, 0xbf, 0xb1, 0x07 // It hi; mov hi r1 r6 LSL 30
};
const byte kInstruction_mov_ls_r0_r2_LSL_7[] = {
  0x98, 0xbf, 0xd0, 0x01 // It ls; mov ls r0 r2 LSL 7
};
const byte kInstruction_mov_cc_r7_r3_LSL_22[] = {
  0x38, 0xbf, 0x9f, 0x05 // It cc; mov cc r7 r3 LSL 22
};
const byte kInstruction_mov_vs_r1_r1_LSL_5[] = {
  0x68, 0xbf, 0x49, 0x01 // It vs; mov vs r1 r1 LSL 5
};
const byte kInstruction_mov_ls_r5_r4_LSL_19[] = {
  0x98, 0xbf, 0xe5, 0x04 // It ls; mov ls r5 r4 LSL 19
};
const byte kInstruction_mov_pl_r6_r2_LSL_14[] = {
  0x58, 0xbf, 0x96, 0x03 // It pl; mov pl r6 r2 LSL 14
};
const byte kInstruction_mov_cs_r1_r0_LSL_1[] = {
  0x28, 0xbf, 0x41, 0x00 // It cs; mov cs r1 r0 LSL 1
};
const byte kInstruction_mov_mi_r6_r1_LSL_2[] = {
  0x48, 0xbf, 0x8e, 0x00 // It mi; mov mi r6 r1 LSL 2
};
const byte kInstruction_mov_mi_r5_r1_LSL_26[] = {
  0x48, 0xbf, 0x8d, 0x06 // It mi; mov mi r5 r1 LSL 26
};
const byte kInstruction_mov_ne_r0_r7_LSL_15[] = {
  0x18, 0xbf, 0xf8, 0x03 // It ne; mov ne r0 r7 LSL 15
};
const byte kInstruction_mov_cs_r2_r7_LSL_15[] = {
  0x28, 0xbf, 0xfa, 0x03 // It cs; mov cs r2 r7 LSL 15
};
const byte kInstruction_mov_ne_r7_r2_LSL_24[] = {
  0x18, 0xbf, 0x17, 0x06 // It ne; mov ne r7 r2 LSL 24
};
const byte kInstruction_mov_le_r1_r2_LSL_8[] = {
  0xd8, 0xbf, 0x11, 0x02 // It le; mov le r1 r2 LSL 8
};
const byte kInstruction_mov_eq_r3_r3_LSL_3[] = {
  0x08, 0xbf, 0xdb, 0x00 // It eq; mov eq r3 r3 LSL 3
};
const byte kInstruction_mov_mi_r1_r4_LSL_23[] = {
  0x48, 0xbf, 0xe1, 0x05 // It mi; mov mi r1 r4 LSL 23
};
const byte kInstruction_mov_pl_r3_r4_LSL_26[] = {
  0x58, 0xbf, 0xa3, 0x06 // It pl; mov pl r3 r4 LSL 26
};
const byte kInstruction_mov_lt_r3_r2_LSL_14[] = {
  0xb8, 0xbf, 0x93, 0x03 // It lt; mov lt r3 r2 LSL 14
};
const byte kInstruction_mov_ne_r7_r0_LSL_6[] = {
  0x18, 0xbf, 0x87, 0x01 // It ne; mov ne r7 r0 LSL 6
};
const byte kInstruction_mov_pl_r2_r0_LSL_26[] = {
  0x58, 0xbf, 0x82, 0x06 // It pl; mov pl r2 r0 LSL 26
};
const byte kInstruction_mov_cc_r1_r0_LSL_22[] = {
  0x38, 0xbf, 0x81, 0x05 // It cc; mov cc r1 r0 LSL 22
};
const byte kInstruction_mov_mi_r7_r3_LSL_30[] = {
  0x48, 0xbf, 0x9f, 0x07 // It mi; mov mi r7 r3 LSL 30
};
const byte kInstruction_mov_ge_r0_r4_LSL_30[] = {
  0xa8, 0xbf, 0xa0, 0x07 // It ge; mov ge r0 r4 LSL 30
};
const byte kInstruction_mov_gt_r0_r4_LSL_14[] = {
  0xc8, 0xbf, 0xa0, 0x03 // It gt; mov gt r0 r4 LSL 14
};
const byte kInstruction_mov_cc_r7_r1_LSL_21[] = {
  0x38, 0xbf, 0x4f, 0x05 // It cc; mov cc r7 r1 LSL 21
};
const byte kInstruction_mov_lt_r7_r1_LSL_4[] = {
  0xb8, 0xbf, 0x0f, 0x01 // It lt; mov lt r7 r1 LSL 4
};
const byte kInstruction_mov_ls_r0_r2_LSL_23[] = {
  0x98, 0xbf, 0xd0, 0x05 // It ls; mov ls r0 r2 LSL 23
};
const byte kInstruction_mov_vs_r6_r6_LSL_9[] = {
  0x68, 0xbf, 0x76, 0x02 // It vs; mov vs r6 r6 LSL 9
};
const byte kInstruction_mov_eq_r4_r6_LSL_14[] = {
  0x08, 0xbf, 0xb4, 0x03 // It eq; mov eq r4 r6 LSL 14
};
const byte kInstruction_mov_gt_r2_r0_LSL_2[] = {
  0xc8, 0xbf, 0x82, 0x00 // It gt; mov gt r2 r0 LSL 2
};
const byte kInstruction_mov_lt_r2_r4_LSL_27[] = {
  0xb8, 0xbf, 0xe2, 0x06 // It lt; mov lt r2 r4 LSL 27
};
const byte kInstruction_mov_cs_r7_r2_LSL_14[] = {
  0x28, 0xbf, 0x97, 0x03 // It cs; mov cs r7 r2 LSL 14
};
const byte kInstruction_mov_vc_r7_r1_LSL_29[] = {
  0x78, 0xbf, 0x4f, 0x07 // It vc; mov vc r7 r1 LSL 29
};
const byte kInstruction_mov_cc_r1_r2_LSL_10[] = {
  0x38, 0xbf, 0x91, 0x02 // It cc; mov cc r1 r2 LSL 10
};
const byte kInstruction_mov_lt_r3_r6_LSL_11[] = {
  0xb8, 0xbf, 0xf3, 0x02 // It lt; mov lt r3 r6 LSL 11
};
const byte kInstruction_mov_le_r7_r6_LSL_6[] = {
  0xd8, 0xbf, 0xb7, 0x01 // It le; mov le r7 r6 LSL 6
};
const byte kInstruction_mov_lt_r6_r1_LSL_16[] = {
  0xb8, 0xbf, 0x0e, 0x04 // It lt; mov lt r6 r1 LSL 16
};
const byte kInstruction_mov_pl_r5_r3_LSL_21[] = {
  0x58, 0xbf, 0x5d, 0x05 // It pl; mov pl r5 r3 LSL 21
};
const byte kInstruction_mov_cs_r1_r7_LSL_20[] = {
  0x28, 0xbf, 0x39, 0x05 // It cs; mov cs r1 r7 LSL 20
};
const byte kInstruction_mov_le_r6_r1_LSL_31[] = {
  0xd8, 0xbf, 0xce, 0x07 // It le; mov le r6 r1 LSL 31
};
const byte kInstruction_mov_cc_r7_r7_LSL_2[] = {
  0x38, 0xbf, 0xbf, 0x00 // It cc; mov cc r7 r7 LSL 2
};
const byte kInstruction_mov_pl_r5_r7_LSL_28[] = {
  0x58, 0xbf, 0x3d, 0x07 // It pl; mov pl r5 r7 LSL 28
};
const byte kInstruction_mov_pl_r6_r4_LSL_5[] = {
  0x58, 0xbf, 0x66, 0x01 // It pl; mov pl r6 r4 LSL 5
};
const byte kInstruction_mov_eq_r1_r1_LSL_20[] = {
  0x08, 0xbf, 0x09, 0x05 // It eq; mov eq r1 r1 LSL 20
};
const byte kInstruction_mov_ls_r1_r1_LSL_11[] = {
  0x98, 0xbf, 0xc9, 0x02 // It ls; mov ls r1 r1 LSL 11
};
const byte kInstruction_mov_eq_r2_r7_LSL_30[] = {
  0x08, 0xbf, 0xba, 0x07 // It eq; mov eq r2 r7 LSL 30
};
const byte kInstruction_mov_ls_r4_r2_LSL_25[] = {
  0x98, 0xbf, 0x54, 0x06 // It ls; mov ls r4 r2 LSL 25
};
const byte kInstruction_mov_hi_r5_r6_LSL_30[] = {
  0x88, 0xbf, 0xb5, 0x07 // It hi; mov hi r5 r6 LSL 30
};
const byte kInstruction_mov_ls_r5_r7_LSL_9[] = {
  0x98, 0xbf, 0x7d, 0x02 // It ls; mov ls r5 r7 LSL 9
};
const byte kInstruction_mov_lt_r7_r7_LSL_16[] = {
  0xb8, 0xbf, 0x3f, 0x04 // It lt; mov lt r7 r7 LSL 16
};
const byte kInstruction_mov_mi_r0_r4_LSL_6[] = {
  0x48, 0xbf, 0xa0, 0x01 // It mi; mov mi r0 r4 LSL 6
};
const byte kInstruction_mov_vs_r3_r0_LSL_3[] = {
  0x68, 0xbf, 0xc3, 0x00 // It vs; mov vs r3 r0 LSL 3
};
const byte kInstruction_mov_lt_r2_r0_LSL_14[] = {
  0xb8, 0xbf, 0x82, 0x03 // It lt; mov lt r2 r0 LSL 14
};
const byte kInstruction_mov_ls_r6_r5_LSL_13[] = {
  0x98, 0xbf, 0x6e, 0x03 // It ls; mov ls r6 r5 LSL 13
};
const byte kInstruction_mov_eq_r6_r0_LSL_7[] = {
  0x08, 0xbf, 0xc6, 0x01 // It eq; mov eq r6 r0 LSL 7
};
const byte kInstruction_mov_ge_r2_r0_LSL_18[] = {
  0xa8, 0xbf, 0x82, 0x04 // It ge; mov ge r2 r0 LSL 18
};
const byte kInstruction_mov_mi_r5_r3_LSL_30[] = {
  0x48, 0xbf, 0x9d, 0x07 // It mi; mov mi r5 r3 LSL 30
};
const byte kInstruction_mov_mi_r1_r5_LSL_4[] = {
  0x48, 0xbf, 0x29, 0x01 // It mi; mov mi r1 r5 LSL 4
};
const byte kInstruction_mov_pl_r4_r1_LSL_15[] = {
  0x58, 0xbf, 0xcc, 0x03 // It pl; mov pl r4 r1 LSL 15
};
const byte kInstruction_mov_ls_r7_r6_LSL_7[] = {
  0x98, 0xbf, 0xf7, 0x01 // It ls; mov ls r7 r6 LSL 7
};
const byte kInstruction_mov_eq_r2_r4_LSL_16[] = {
  0x08, 0xbf, 0x22, 0x04 // It eq; mov eq r2 r4 LSL 16
};
const byte kInstruction_mov_eq_r5_r3_LSL_23[] = {
  0x08, 0xbf, 0xdd, 0x05 // It eq; mov eq r5 r3 LSL 23
};
const byte kInstruction_mov_le_r4_r6_LSL_23[] = {
  0xd8, 0xbf, 0xf4, 0x05 // It le; mov le r4 r6 LSL 23
};
const byte kInstruction_mov_ls_r3_r3_LSL_13[] = {
  0x98, 0xbf, 0x5b, 0x03 // It ls; mov ls r3 r3 LSL 13
};
const byte kInstruction_mov_hi_r2_r5_LSL_28[] = {
  0x88, 0xbf, 0x2a, 0x07 // It hi; mov hi r2 r5 LSL 28
};
const byte kInstruction_mov_gt_r1_r2_LSL_8[] = {
  0xc8, 0xbf, 0x11, 0x02 // It gt; mov gt r1 r2 LSL 8
};
const byte kInstruction_mov_cs_r1_r2_LSL_11[] = {
  0x28, 0xbf, 0xd1, 0x02 // It cs; mov cs r1 r2 LSL 11
};
const byte kInstruction_mov_vs_r3_r4_LSL_23[] = {
  0x68, 0xbf, 0xe3, 0x05 // It vs; mov vs r3 r4 LSL 23
};
const byte kInstruction_mov_cs_r4_r0_LSL_12[] = {
  0x28, 0xbf, 0x04, 0x03 // It cs; mov cs r4 r0 LSL 12
};
const byte kInstruction_mov_hi_r5_r6_LSL_5[] = {
  0x88, 0xbf, 0x75, 0x01 // It hi; mov hi r5 r6 LSL 5
};
const byte kInstruction_mov_cs_r0_r3_LSL_10[] = {
  0x28, 0xbf, 0x98, 0x02 // It cs; mov cs r0 r3 LSL 10
};
const byte kInstruction_mov_lt_r2_r2_LSL_30[] = {
  0xb8, 0xbf, 0x92, 0x07 // It lt; mov lt r2 r2 LSL 30
};
const byte kInstruction_mov_eq_r4_r2_LSL_21[] = {
  0x08, 0xbf, 0x54, 0x05 // It eq; mov eq r4 r2 LSL 21
};
const byte kInstruction_mov_mi_r0_r2_LSL_10[] = {
  0x48, 0xbf, 0x90, 0x02 // It mi; mov mi r0 r2 LSL 10
};
const byte kInstruction_mov_pl_r5_r0_LSL_11[] = {
  0x58, 0xbf, 0xc5, 0x02 // It pl; mov pl r5 r0 LSL 11
};
const byte kInstruction_mov_hi_r1_r7_LSL_19[] = {
  0x88, 0xbf, 0xf9, 0x04 // It hi; mov hi r1 r7 LSL 19
};
const byte kInstruction_mov_le_r3_r5_LSL_7[] = {
  0xd8, 0xbf, 0xeb, 0x01 // It le; mov le r3 r5 LSL 7
};
const byte kInstruction_mov_hi_r3_r5_LSL_19[] = {
  0x88, 0xbf, 0xeb, 0x04 // It hi; mov hi r3 r5 LSL 19
};
const byte kInstruction_mov_ne_r5_r3_LSL_14[] = {
  0x18, 0xbf, 0x9d, 0x03 // It ne; mov ne r5 r3 LSL 14
};
const byte kInstruction_mov_ne_r3_r5_LSL_1[] = {
  0x18, 0xbf, 0x6b, 0x00 // It ne; mov ne r3 r5 LSL 1
};
const byte kInstruction_mov_gt_r7_r4_LSL_23[] = {
  0xc8, 0xbf, 0xe7, 0x05 // It gt; mov gt r7 r4 LSL 23
};
const byte kInstruction_mov_cc_r2_r3_LSL_3[] = {
  0x38, 0xbf, 0xda, 0x00 // It cc; mov cc r2 r3 LSL 3
};
const byte kInstruction_mov_cc_r7_r7_LSL_22[] = {
  0x38, 0xbf, 0xbf, 0x05 // It cc; mov cc r7 r7 LSL 22
};
const byte kInstruction_mov_cs_r7_r0_LSL_10[] = {
  0x28, 0xbf, 0x87, 0x02 // It cs; mov cs r7 r0 LSL 10
};
const byte kInstruction_mov_le_r2_r7_LSL_7[] = {
  0xd8, 0xbf, 0xfa, 0x01 // It le; mov le r2 r7 LSL 7
};
const byte kInstruction_mov_cc_r5_r6_LSL_7[] = {
  0x38, 0xbf, 0xf5, 0x01 // It cc; mov cc r5 r6 LSL 7
};
const byte kInstruction_mov_ne_r0_r1_LSL_1[] = {
  0x18, 0xbf, 0x48, 0x00 // It ne; mov ne r0 r1 LSL 1
};
const byte kInstruction_mov_mi_r3_r6_LSL_8[] = {
  0x48, 0xbf, 0x33, 0x02 // It mi; mov mi r3 r6 LSL 8
};
const byte kInstruction_mov_mi_r0_r7_LSL_14[] = {
  0x48, 0xbf, 0xb8, 0x03 // It mi; mov mi r0 r7 LSL 14
};
const byte kInstruction_mov_hi_r0_r5_LSL_17[] = {
  0x88, 0xbf, 0x68, 0x04 // It hi; mov hi r0 r5 LSL 17
};
const byte kInstruction_mov_lt_r2_r5_LSL_7[] = {
  0xb8, 0xbf, 0xea, 0x01 // It lt; mov lt r2 r5 LSL 7
};
const byte kInstruction_mov_mi_r1_r1_LSL_18[] = {
  0x48, 0xbf, 0x89, 0x04 // It mi; mov mi r1 r1 LSL 18
};
const byte kInstruction_mov_pl_r3_r2_LSL_13[] = {
  0x58, 0xbf, 0x53, 0x03 // It pl; mov pl r3 r2 LSL 13
};
const byte kInstruction_mov_gt_r6_r1_LSL_13[] = {
  0xc8, 0xbf, 0x4e, 0x03 // It gt; mov gt r6 r1 LSL 13
};
const byte kInstruction_mov_eq_r2_r6_LSL_11[] = {
  0x08, 0xbf, 0xf2, 0x02 // It eq; mov eq r2 r6 LSL 11
};
const byte kInstruction_mov_vs_r2_r6_LSL_24[] = {
  0x68, 0xbf, 0x32, 0x06 // It vs; mov vs r2 r6 LSL 24
};
const byte kInstruction_mov_mi_r5_r0_LSL_15[] = {
  0x48, 0xbf, 0xc5, 0x03 // It mi; mov mi r5 r0 LSL 15
};
const TestResult kReferencemov[] = {
  {
    ARRAY_SIZE(kInstruction_mov_hi_r2_r4_LSL_10),
    kInstruction_mov_hi_r2_r4_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r6_r2_LSL_8),
    kInstruction_mov_cs_r6_r2_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r5_r3_LSL_21),
    kInstruction_mov_lt_r5_r3_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r5_r0_LSL_3),
    kInstruction_mov_ge_r5_r0_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r4_r1_LSL_4),
    kInstruction_mov_mi_r4_r1_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r6_r6_LSL_28),
    kInstruction_mov_ge_r6_r6_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r4_r2_LSL_31),
    kInstruction_mov_lt_r4_r2_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r0_r3_LSL_17),
    kInstruction_mov_eq_r0_r3_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r2_r2_LSL_22),
    kInstruction_mov_vc_r2_r2_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r0_r1_LSL_6),
    kInstruction_mov_ge_r0_r1_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r6_r0_LSL_26),
    kInstruction_mov_vs_r6_r0_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r3_r1_LSL_26),
    kInstruction_mov_vs_r3_r1_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r7_r4_LSL_28),
    kInstruction_mov_cs_r7_r4_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r3_r0_LSL_28),
    kInstruction_mov_le_r3_r0_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r4_r0_LSL_23),
    kInstruction_mov_le_r4_r0_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r5_r3_LSL_1),
    kInstruction_mov_gt_r5_r3_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r7_r1_LSL_14),
    kInstruction_mov_hi_r7_r1_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r1_r4_LSL_27),
    kInstruction_mov_cs_r1_r4_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r1_r0_LSL_2),
    kInstruction_mov_cc_r1_r0_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r5_r2_LSL_1),
    kInstruction_mov_le_r5_r2_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r6_r4_LSL_23),
    kInstruction_mov_gt_r6_r4_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r0_r0_LSL_27),
    kInstruction_mov_lt_r0_r0_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r5_r4_LSL_6),
    kInstruction_mov_le_r5_r4_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r2_r0_LSL_14),
    kInstruction_mov_ls_r2_r0_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r3_r5_LSL_31),
    kInstruction_mov_mi_r3_r5_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r6_r6_LSL_11),
    kInstruction_mov_le_r6_r6_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r1_r4_LSL_30),
    kInstruction_mov_le_r1_r4_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r3_r1_LSL_16),
    kInstruction_mov_le_r3_r1_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r3_r6_LSL_12),
    kInstruction_mov_eq_r3_r6_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r4_r5_LSL_25),
    kInstruction_mov_lt_r4_r5_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r1_r3_LSL_10),
    kInstruction_mov_ge_r1_r3_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r0_r5_LSL_20),
    kInstruction_mov_vc_r0_r5_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r0_r7_LSL_6),
    kInstruction_mov_cc_r0_r7_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r6_r0_LSL_13),
    kInstruction_mov_vc_r6_r0_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r7_r7_LSL_11),
    kInstruction_mov_ls_r7_r7_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r2_r3_LSL_28),
    kInstruction_mov_ls_r2_r3_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r2_r1_LSL_20),
    kInstruction_mov_le_r2_r1_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r4_r5_LSL_8),
    kInstruction_mov_ne_r4_r5_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r5_r6_LSL_5),
    kInstruction_mov_vc_r5_r6_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r6_r1_LSL_14),
    kInstruction_mov_cc_r6_r1_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r1_r4_LSL_8),
    kInstruction_mov_cc_r1_r4_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r0_r4_LSL_24),
    kInstruction_mov_ls_r0_r4_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r4_r4_LSL_12),
    kInstruction_mov_ls_r4_r4_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r6_r2_LSL_19),
    kInstruction_mov_eq_r6_r2_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r7_r2_LSL_16),
    kInstruction_mov_cc_r7_r2_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r7_r3_LSL_28),
    kInstruction_mov_lt_r7_r3_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r2_r1_LSL_5),
    kInstruction_mov_hi_r2_r1_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r5_r3_LSL_26),
    kInstruction_mov_ge_r5_r3_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r0_r0_LSL_1),
    kInstruction_mov_mi_r0_r0_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r6_r3_LSL_17),
    kInstruction_mov_vc_r6_r3_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r1_r4_LSL_16),
    kInstruction_mov_mi_r1_r4_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r1_r3_LSL_20),
    kInstruction_mov_ge_r1_r3_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r1_r3_LSL_9),
    kInstruction_mov_eq_r1_r3_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r0_r4_LSL_27),
    kInstruction_mov_ne_r0_r4_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r4_r1_LSL_12),
    kInstruction_mov_le_r4_r1_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r7_r7_LSL_26),
    kInstruction_mov_ne_r7_r7_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r5_r2_LSL_12),
    kInstruction_mov_hi_r5_r2_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r3_r4_LSL_9),
    kInstruction_mov_pl_r3_r4_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r5_r3_LSL_12),
    kInstruction_mov_cs_r5_r3_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r1_r1_LSL_28),
    kInstruction_mov_vs_r1_r1_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r4_r6_LSL_19),
    kInstruction_mov_lt_r4_r6_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r6_r6_LSL_31),
    kInstruction_mov_vc_r6_r6_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r4_r5_LSL_27),
    kInstruction_mov_cs_r4_r5_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r2_r6_LSL_2),
    kInstruction_mov_gt_r2_r6_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r6_r4_LSL_20),
    kInstruction_mov_vc_r6_r4_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r1_r5_LSL_13),
    kInstruction_mov_ne_r1_r5_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r6_r2_LSL_4),
    kInstruction_mov_vs_r6_r2_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r5_r3_LSL_20),
    kInstruction_mov_lt_r5_r3_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r3_r0_LSL_18),
    kInstruction_mov_le_r3_r0_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r1_r3_LSL_25),
    kInstruction_mov_ge_r1_r3_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r4_r5_LSL_23),
    kInstruction_mov_cs_r4_r5_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r0_r3_LSL_30),
    kInstruction_mov_mi_r0_r3_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r4_r5_LSL_19),
    kInstruction_mov_ge_r4_r5_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r3_r1_LSL_20),
    kInstruction_mov_gt_r3_r1_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r0_r5_LSL_20),
    kInstruction_mov_le_r0_r5_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r0_r6_LSL_24),
    kInstruction_mov_vs_r0_r6_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r5_r6_LSL_30),
    kInstruction_mov_le_r5_r6_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r6_r5_LSL_2),
    kInstruction_mov_ne_r6_r5_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r6_r3_LSL_2),
    kInstruction_mov_gt_r6_r3_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r4_r4_LSL_8),
    kInstruction_mov_ls_r4_r4_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r5_r3_LSL_5),
    kInstruction_mov_cs_r5_r3_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r5_r4_LSL_23),
    kInstruction_mov_ne_r5_r4_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r0_r0_LSL_14),
    kInstruction_mov_lt_r0_r0_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r3_r4_LSL_12),
    kInstruction_mov_cc_r3_r4_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r3_r7_LSL_25),
    kInstruction_mov_vc_r3_r7_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r2_r3_LSL_21),
    kInstruction_mov_ge_r2_r3_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r3_r3_LSL_17),
    kInstruction_mov_gt_r3_r3_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r3_r5_LSL_21),
    kInstruction_mov_mi_r3_r5_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r2_r5_LSL_30),
    kInstruction_mov_hi_r2_r5_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r2_r5_LSL_12),
    kInstruction_mov_ls_r2_r5_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r4_r6_LSL_1),
    kInstruction_mov_cc_r4_r6_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r1_r1_LSL_24),
    kInstruction_mov_cc_r1_r1_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r3_r2_LSL_24),
    kInstruction_mov_vc_r3_r2_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r0_r5_LSL_18),
    kInstruction_mov_le_r0_r5_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r7_r1_LSL_23),
    kInstruction_mov_eq_r7_r1_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r2_r3_LSL_3),
    kInstruction_mov_lt_r2_r3_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r2_r0_LSL_27),
    kInstruction_mov_pl_r2_r0_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r5_r4_LSL_19),
    kInstruction_mov_mi_r5_r4_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r3_r2_LSL_2),
    kInstruction_mov_gt_r3_r2_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r5_r0_LSL_14),
    kInstruction_mov_lt_r5_r0_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r4_r3_LSL_25),
    kInstruction_mov_cc_r4_r3_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r4_r7_LSL_15),
    kInstruction_mov_cs_r4_r7_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r2_r5_LSL_10),
    kInstruction_mov_ls_r2_r5_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r2_r6_LSL_17),
    kInstruction_mov_cc_r2_r6_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r6_r3_LSL_12),
    kInstruction_mov_hi_r6_r3_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r4_r0_LSL_26),
    kInstruction_mov_cc_r4_r0_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r4_r2_LSL_29),
    kInstruction_mov_ls_r4_r2_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r4_r2_LSL_8),
    kInstruction_mov_ls_r4_r2_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r7_r5_LSL_1),
    kInstruction_mov_vs_r7_r5_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r1_r5_LSL_11),
    kInstruction_mov_eq_r1_r5_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r4_r2_LSL_24),
    kInstruction_mov_vc_r4_r2_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r7_r2_LSL_15),
    kInstruction_mov_cs_r7_r2_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r7_r1_LSL_9),
    kInstruction_mov_vs_r7_r1_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r7_r4_LSL_11),
    kInstruction_mov_pl_r7_r4_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r3_r6_LSL_19),
    kInstruction_mov_ne_r3_r6_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r6_r4_LSL_9),
    kInstruction_mov_ne_r6_r4_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r1_r7_LSL_13),
    kInstruction_mov_le_r1_r7_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r2_r3_LSL_19),
    kInstruction_mov_vc_r2_r3_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r2_r6_LSL_3),
    kInstruction_mov_mi_r2_r6_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r3_r0_LSL_30),
    kInstruction_mov_eq_r3_r0_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r6_r2_LSL_3),
    kInstruction_mov_le_r6_r2_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r5_r7_LSL_22),
    kInstruction_mov_ne_r5_r7_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r7_r4_LSL_19),
    kInstruction_mov_vs_r7_r4_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r6_r7_LSL_20),
    kInstruction_mov_eq_r6_r7_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r2_r7_LSL_19),
    kInstruction_mov_cs_r2_r7_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r3_r1_LSL_17),
    kInstruction_mov_cs_r3_r1_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r5_r3_LSL_11),
    kInstruction_mov_vs_r5_r3_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r5_r6_LSL_27),
    kInstruction_mov_le_r5_r6_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r7_r0_LSL_27),
    kInstruction_mov_vs_r7_r0_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r2_r0_LSL_29),
    kInstruction_mov_lt_r2_r0_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r0_r0_LSL_11),
    kInstruction_mov_vc_r0_r0_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r6_r3_LSL_16),
    kInstruction_mov_cc_r6_r3_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r0_r0_LSL_16),
    kInstruction_mov_cs_r0_r0_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r4_r4_LSL_24),
    kInstruction_mov_pl_r4_r4_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r6_r7_LSL_22),
    kInstruction_mov_mi_r6_r7_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r1_r6_LSL_15),
    kInstruction_mov_vc_r1_r6_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r6_r7_LSL_18),
    kInstruction_mov_ls_r6_r7_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r1_r1_LSL_22),
    kInstruction_mov_vc_r1_r1_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r3_r0_LSL_13),
    kInstruction_mov_lt_r3_r0_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r2_r2_LSL_25),
    kInstruction_mov_pl_r2_r2_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r3_r2_LSL_30),
    kInstruction_mov_hi_r3_r2_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r5_r0_LSL_1),
    kInstruction_mov_vs_r5_r0_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r5_r0_LSL_6),
    kInstruction_mov_gt_r5_r0_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r7_r0_LSL_3),
    kInstruction_mov_cs_r7_r0_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r4_r6_LSL_9),
    kInstruction_mov_ne_r4_r6_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r2_r0_LSL_4),
    kInstruction_mov_ge_r2_r0_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r6_r5_LSL_16),
    kInstruction_mov_pl_r6_r5_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r5_r6_LSL_8),
    kInstruction_mov_lt_r5_r6_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r2_r2_LSL_2),
    kInstruction_mov_hi_r2_r2_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r6_r2_LSL_16),
    kInstruction_mov_pl_r6_r2_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r2_r2_LSL_14),
    kInstruction_mov_cs_r2_r2_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r2_r4_LSL_25),
    kInstruction_mov_mi_r2_r4_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r0_r5_LSL_4),
    kInstruction_mov_mi_r0_r5_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r0_r6_LSL_6),
    kInstruction_mov_hi_r0_r6_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r1_r3_LSL_16),
    kInstruction_mov_cs_r1_r3_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r0_r6_LSL_25),
    kInstruction_mov_le_r0_r6_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r6_r1_LSL_22),
    kInstruction_mov_gt_r6_r1_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r4_r2_LSL_6),
    kInstruction_mov_cc_r4_r2_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r6_r2_LSL_23),
    kInstruction_mov_eq_r6_r2_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r2_r5_LSL_11),
    kInstruction_mov_hi_r2_r5_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r0_r6_LSL_12),
    kInstruction_mov_eq_r0_r6_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r1_r7_LSL_16),
    kInstruction_mov_pl_r1_r7_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r4_r3_LSL_11),
    kInstruction_mov_ne_r4_r3_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r1_r1_LSL_25),
    kInstruction_mov_pl_r1_r1_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r3_r0_LSL_1),
    kInstruction_mov_gt_r3_r0_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r1_r2_LSL_25),
    kInstruction_mov_cs_r1_r2_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r4_r1_LSL_13),
    kInstruction_mov_pl_r4_r1_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r0_r7_LSL_30),
    kInstruction_mov_lt_r0_r7_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r3_r6_LSL_17),
    kInstruction_mov_vc_r3_r6_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r7_r6_LSL_14),
    kInstruction_mov_ne_r7_r6_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r0_r2_LSL_2),
    kInstruction_mov_hi_r0_r2_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r2_r3_LSL_19),
    kInstruction_mov_hi_r2_r3_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r1_r5_LSL_6),
    kInstruction_mov_cs_r1_r5_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r6_r7_LSL_9),
    kInstruction_mov_mi_r6_r7_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r5_r4_LSL_20),
    kInstruction_mov_ls_r5_r4_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r5_r7_LSL_14),
    kInstruction_mov_gt_r5_r7_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r0_r3_LSL_2),
    kInstruction_mov_pl_r0_r3_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r0_r3_LSL_16),
    kInstruction_mov_ls_r0_r3_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r7_r7_LSL_26),
    kInstruction_mov_gt_r7_r7_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r1_r7_LSL_6),
    kInstruction_mov_vc_r1_r7_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r6_r4_LSL_31),
    kInstruction_mov_cs_r6_r4_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r7_r1_LSL_8),
    kInstruction_mov_ls_r7_r1_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r6_r4_LSL_7),
    kInstruction_mov_eq_r6_r4_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r0_r3_LSL_12),
    kInstruction_mov_le_r0_r3_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r7_r1_LSL_5),
    kInstruction_mov_eq_r7_r1_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r4_r4_LSL_23),
    kInstruction_mov_mi_r4_r4_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r6_r2_LSL_20),
    kInstruction_mov_vc_r6_r2_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r1_r1_LSL_1),
    kInstruction_mov_eq_r1_r1_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r0_r0_LSL_20),
    kInstruction_mov_cc_r0_r0_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r5_r4_LSL_26),
    kInstruction_mov_pl_r5_r4_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r2_r5_LSL_6),
    kInstruction_mov_vc_r2_r5_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r5_r5_LSL_25),
    kInstruction_mov_le_r5_r5_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r5_r0_LSL_26),
    kInstruction_mov_le_r5_r0_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r0_r3_LSL_17),
    kInstruction_mov_le_r0_r3_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r5_r5_LSL_29),
    kInstruction_mov_mi_r5_r5_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r5_r4_LSL_8),
    kInstruction_mov_vc_r5_r4_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r0_r2_LSL_7),
    kInstruction_mov_ge_r0_r2_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r5_r5_LSL_29),
    kInstruction_mov_ne_r5_r5_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r1_r7_LSL_24),
    kInstruction_mov_ge_r1_r7_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r6_r1_LSL_24),
    kInstruction_mov_vs_r6_r1_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r5_r7_LSL_31),
    kInstruction_mov_vs_r5_r7_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r0_r3_LSL_9),
    kInstruction_mov_eq_r0_r3_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r0_r1_LSL_28),
    kInstruction_mov_mi_r0_r1_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r1_r5_LSL_2),
    kInstruction_mov_vc_r1_r5_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r5_r7_LSL_26),
    kInstruction_mov_lt_r5_r7_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r5_r2_LSL_18),
    kInstruction_mov_eq_r5_r2_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r4_r0_LSL_6),
    kInstruction_mov_lt_r4_r0_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r6_r2_LSL_9),
    kInstruction_mov_hi_r6_r2_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r6_r1_LSL_25),
    kInstruction_mov_vc_r6_r1_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r2_r7_LSL_10),
    kInstruction_mov_le_r2_r7_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r4_r0_LSL_16),
    kInstruction_mov_vs_r4_r0_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r6_r7_LSL_10),
    kInstruction_mov_ne_r6_r7_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r4_r3_LSL_26),
    kInstruction_mov_ne_r4_r3_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r6_r5_LSL_9),
    kInstruction_mov_le_r6_r5_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r2_r5_LSL_1),
    kInstruction_mov_vc_r2_r5_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r7_r5_LSL_28),
    kInstruction_mov_vc_r7_r5_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r0_r3_LSL_4),
    kInstruction_mov_ne_r0_r3_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r5_r4_LSL_22),
    kInstruction_mov_ne_r5_r4_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r1_r4_LSL_30),
    kInstruction_mov_vc_r1_r4_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r5_r4_LSL_7),
    kInstruction_mov_gt_r5_r4_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r4_r5_LSL_24),
    kInstruction_mov_vs_r4_r5_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r3_r5_LSL_13),
    kInstruction_mov_hi_r3_r5_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r6_r7_LSL_9),
    kInstruction_mov_ge_r6_r7_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r6_r3_LSL_24),
    kInstruction_mov_eq_r6_r3_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r3_r5_LSL_14),
    kInstruction_mov_le_r3_r5_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r0_r7_LSL_1),
    kInstruction_mov_lt_r0_r7_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r0_r5_LSL_25),
    kInstruction_mov_cc_r0_r5_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r0_r2_LSL_21),
    kInstruction_mov_ne_r0_r2_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r4_r2_LSL_20),
    kInstruction_mov_eq_r4_r2_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r5_r4_LSL_12),
    kInstruction_mov_vc_r5_r4_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r3_r1_LSL_11),
    kInstruction_mov_ne_r3_r1_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r3_r7_LSL_27),
    kInstruction_mov_mi_r3_r7_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r1_r7_LSL_10),
    kInstruction_mov_ls_r1_r7_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r7_r1_LSL_30),
    kInstruction_mov_le_r7_r1_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r5_r2_LSL_4),
    kInstruction_mov_pl_r5_r2_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r5_r4_LSL_7),
    kInstruction_mov_cs_r5_r4_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r4_r0_LSL_25),
    kInstruction_mov_mi_r4_r0_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r3_r7_LSL_12),
    kInstruction_mov_vs_r3_r7_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r4_r6_LSL_7),
    kInstruction_mov_gt_r4_r6_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r2_r1_LSL_4),
    kInstruction_mov_vs_r2_r1_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r0_r0_LSL_7),
    kInstruction_mov_ne_r0_r0_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r5_r1_LSL_8),
    kInstruction_mov_hi_r5_r1_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r0_r0_LSL_9),
    kInstruction_mov_eq_r0_r0_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r7_r3_LSL_31),
    kInstruction_mov_cc_r7_r3_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r0_r5_LSL_18),
    kInstruction_mov_ne_r0_r5_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r6_r3_LSL_23),
    kInstruction_mov_eq_r6_r3_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r5_r7_LSL_14),
    kInstruction_mov_cc_r5_r7_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r4_r0_LSL_27),
    kInstruction_mov_ne_r4_r0_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r7_r0_LSL_2),
    kInstruction_mov_mi_r7_r0_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r6_r5_LSL_24),
    kInstruction_mov_hi_r6_r5_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r5_r7_LSL_20),
    kInstruction_mov_hi_r5_r7_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r1_r2_LSL_23),
    kInstruction_mov_vc_r1_r2_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r2_r0_LSL_27),
    kInstruction_mov_mi_r2_r0_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r1_r2_LSL_17),
    kInstruction_mov_ls_r1_r2_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r0_r6_LSL_1),
    kInstruction_mov_pl_r0_r6_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r6_r3_LSL_16),
    kInstruction_mov_vs_r6_r3_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r6_r6_LSL_23),
    kInstruction_mov_mi_r6_r6_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r7_r1_LSL_27),
    kInstruction_mov_ge_r7_r1_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r0_r5_LSL_16),
    kInstruction_mov_gt_r0_r5_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r6_r4_LSL_20),
    kInstruction_mov_le_r6_r4_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r6_r0_LSL_20),
    kInstruction_mov_gt_r6_r0_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r7_r2_LSL_28),
    kInstruction_mov_gt_r7_r2_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r6_r7_LSL_9),
    kInstruction_mov_le_r6_r7_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r4_r4_LSL_2),
    kInstruction_mov_gt_r4_r4_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r6_r5_LSL_13),
    kInstruction_mov_mi_r6_r5_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r0_r0_LSL_2),
    kInstruction_mov_le_r0_r0_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r2_r3_LSL_2),
    kInstruction_mov_ge_r2_r3_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r1_r4_LSL_28),
    kInstruction_mov_cs_r1_r4_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r1_r4_LSL_6),
    kInstruction_mov_vc_r1_r4_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r3_r6_LSL_26),
    kInstruction_mov_vs_r3_r6_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r2_r6_LSL_23),
    kInstruction_mov_cs_r2_r6_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r1_r3_LSL_13),
    kInstruction_mov_eq_r1_r3_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r7_r1_LSL_23),
    kInstruction_mov_ne_r7_r1_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r7_r4_LSL_14),
    kInstruction_mov_pl_r7_r4_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r3_r1_LSL_22),
    kInstruction_mov_vs_r3_r1_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r5_r4_LSL_24),
    kInstruction_mov_eq_r5_r4_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r4_r1_LSL_9),
    kInstruction_mov_cc_r4_r1_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r2_r1_LSL_23),
    kInstruction_mov_mi_r2_r1_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r1_r3_LSL_19),
    kInstruction_mov_gt_r1_r3_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r7_r0_LSL_3),
    kInstruction_mov_lt_r7_r0_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r2_r1_LSL_13),
    kInstruction_mov_le_r2_r1_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r2_r1_LSL_2),
    kInstruction_mov_ge_r2_r1_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r5_r0_LSL_21),
    kInstruction_mov_hi_r5_r0_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r7_r5_LSL_19),
    kInstruction_mov_gt_r7_r5_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r2_r3_LSL_19),
    kInstruction_mov_eq_r2_r3_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r0_r5_LSL_4),
    kInstruction_mov_vs_r0_r5_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r7_r2_LSL_30),
    kInstruction_mov_pl_r7_r2_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r2_r3_LSL_28),
    kInstruction_mov_vc_r2_r3_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r2_r7_LSL_25),
    kInstruction_mov_pl_r2_r7_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r3_r6_LSL_23),
    kInstruction_mov_eq_r3_r6_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r3_r0_LSL_8),
    kInstruction_mov_eq_r3_r0_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r3_r2_LSL_22),
    kInstruction_mov_pl_r3_r2_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r4_r3_LSL_26),
    kInstruction_mov_cc_r4_r3_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r5_r2_LSL_1),
    kInstruction_mov_cs_r5_r2_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r1_r1_LSL_6),
    kInstruction_mov_cc_r1_r1_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r3_r5_LSL_22),
    kInstruction_mov_mi_r3_r5_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r0_r3_LSL_21),
    kInstruction_mov_cs_r0_r3_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r7_r7_LSL_13),
    kInstruction_mov_vs_r7_r7_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r5_r6_LSL_30),
    kInstruction_mov_pl_r5_r6_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r3_r2_LSL_9),
    kInstruction_mov_ge_r3_r2_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r1_r3_LSL_11),
    kInstruction_mov_pl_r1_r3_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r5_r7_LSL_25),
    kInstruction_mov_eq_r5_r7_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r6_r4_LSL_8),
    kInstruction_mov_mi_r6_r4_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r4_r5_LSL_2),
    kInstruction_mov_ls_r4_r5_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r3_r4_LSL_28),
    kInstruction_mov_ls_r3_r4_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r5_r6_LSL_31),
    kInstruction_mov_ls_r5_r6_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r6_r0_LSL_11),
    kInstruction_mov_pl_r6_r0_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r2_r6_LSL_13),
    kInstruction_mov_ls_r2_r6_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r1_r4_LSL_31),
    kInstruction_mov_gt_r1_r4_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r2_r3_LSL_20),
    kInstruction_mov_vc_r2_r3_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r0_r1_LSL_16),
    kInstruction_mov_eq_r0_r1_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r4_r1_LSL_25),
    kInstruction_mov_pl_r4_r1_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r0_r5_LSL_17),
    kInstruction_mov_vc_r0_r5_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r4_r3_LSL_8),
    kInstruction_mov_ne_r4_r3_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r2_r5_LSL_10),
    kInstruction_mov_le_r2_r5_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r0_r2_LSL_4),
    kInstruction_mov_mi_r0_r2_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r3_r1_LSL_6),
    kInstruction_mov_cc_r3_r1_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r4_r7_LSL_30),
    kInstruction_mov_vs_r4_r7_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r0_r1_LSL_7),
    kInstruction_mov_ls_r0_r1_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r0_r0_LSL_30),
    kInstruction_mov_eq_r0_r0_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r2_r6_LSL_31),
    kInstruction_mov_ne_r2_r6_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r7_r5_LSL_15),
    kInstruction_mov_vc_r7_r5_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r0_r2_LSL_22),
    kInstruction_mov_gt_r0_r2_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r7_r4_LSL_13),
    kInstruction_mov_ge_r7_r4_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r4_r3_LSL_3),
    kInstruction_mov_lt_r4_r3_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r3_r0_LSL_8),
    kInstruction_mov_vs_r3_r0_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r2_r2_LSL_9),
    kInstruction_mov_gt_r2_r2_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r1_r5_LSL_1),
    kInstruction_mov_vc_r1_r5_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r2_r4_LSL_18),
    kInstruction_mov_pl_r2_r4_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r5_r2_LSL_27),
    kInstruction_mov_lt_r5_r2_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r2_r6_LSL_8),
    kInstruction_mov_ge_r2_r6_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r0_r3_LSL_15),
    kInstruction_mov_le_r0_r3_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r4_r5_LSL_7),
    kInstruction_mov_mi_r4_r5_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r0_r4_LSL_12),
    kInstruction_mov_vc_r0_r4_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r7_r6_LSL_26),
    kInstruction_mov_eq_r7_r6_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r1_r0_LSL_24),
    kInstruction_mov_vc_r1_r0_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r3_r6_LSL_28),
    kInstruction_mov_gt_r3_r6_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r3_r6_LSL_6),
    kInstruction_mov_vc_r3_r6_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r1_r5_LSL_16),
    kInstruction_mov_ls_r1_r5_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r7_r7_LSL_30),
    kInstruction_mov_lt_r7_r7_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r7_r2_LSL_25),
    kInstruction_mov_eq_r7_r2_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r0_r1_LSL_24),
    kInstruction_mov_ne_r0_r1_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r1_r3_LSL_17),
    kInstruction_mov_ge_r1_r3_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r1_r4_LSL_6),
    kInstruction_mov_vs_r1_r4_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r7_r3_LSL_30),
    kInstruction_mov_vs_r7_r3_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r0_r4_LSL_16),
    kInstruction_mov_le_r0_r4_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r6_r6_LSL_4),
    kInstruction_mov_hi_r6_r6_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r6_r2_LSL_12),
    kInstruction_mov_ls_r6_r2_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r5_r0_LSL_23),
    kInstruction_mov_le_r5_r0_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r7_r1_LSL_5),
    kInstruction_mov_cc_r7_r1_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r4_r3_LSL_23),
    kInstruction_mov_vc_r4_r3_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r6_r1_LSL_30),
    kInstruction_mov_cc_r6_r1_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r7_r3_LSL_30),
    kInstruction_mov_lt_r7_r3_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r0_r6_LSL_18),
    kInstruction_mov_eq_r0_r6_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r2_r2_LSL_20),
    kInstruction_mov_le_r2_r2_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r4_r0_LSL_11),
    kInstruction_mov_ls_r4_r0_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r2_r6_LSL_26),
    kInstruction_mov_hi_r2_r6_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r0_r6_LSL_22),
    kInstruction_mov_vs_r0_r6_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r0_r4_LSL_29),
    kInstruction_mov_cs_r0_r4_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r3_r5_LSL_17),
    kInstruction_mov_ge_r3_r5_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r5_r7_LSL_28),
    kInstruction_mov_lt_r5_r7_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r7_r3_LSL_13),
    kInstruction_mov_pl_r7_r3_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r5_r5_LSL_4),
    kInstruction_mov_mi_r5_r5_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r2_r2_LSL_18),
    kInstruction_mov_vc_r2_r2_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r6_r3_LSL_12),
    kInstruction_mov_ne_r6_r3_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r6_r6_LSL_31),
    kInstruction_mov_ne_r6_r6_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r4_r4_LSL_15),
    kInstruction_mov_mi_r4_r4_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r3_r7_LSL_18),
    kInstruction_mov_cs_r3_r7_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r4_r5_LSL_13),
    kInstruction_mov_vs_r4_r5_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r7_r5_LSL_25),
    kInstruction_mov_ls_r7_r5_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r6_r0_LSL_22),
    kInstruction_mov_cc_r6_r0_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r0_r1_LSL_17),
    kInstruction_mov_ne_r0_r1_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r7_r5_LSL_25),
    kInstruction_mov_mi_r7_r5_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r5_r6_LSL_24),
    kInstruction_mov_lt_r5_r6_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r6_r4_LSL_31),
    kInstruction_mov_le_r6_r4_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r4_r4_LSL_16),
    kInstruction_mov_hi_r4_r4_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r5_r4_LSL_7),
    kInstruction_mov_pl_r5_r4_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r2_r6_LSL_30),
    kInstruction_mov_ge_r2_r6_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r2_r5_LSL_31),
    kInstruction_mov_vs_r2_r5_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r5_r2_LSL_21),
    kInstruction_mov_eq_r5_r2_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r7_r6_LSL_18),
    kInstruction_mov_mi_r7_r6_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r2_r7_LSL_4),
    kInstruction_mov_ls_r2_r7_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r6_r0_LSL_22),
    kInstruction_mov_ne_r6_r0_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r4_r4_LSL_15),
    kInstruction_mov_le_r4_r4_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r1_r2_LSL_28),
    kInstruction_mov_ne_r1_r2_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r7_r2_LSL_28),
    kInstruction_mov_hi_r7_r2_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r2_r1_LSL_22),
    kInstruction_mov_lt_r2_r1_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r4_r2_LSL_30),
    kInstruction_mov_cs_r4_r2_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r7_r7_LSL_1),
    kInstruction_mov_ls_r7_r7_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r1_r6_LSL_16),
    kInstruction_mov_mi_r1_r6_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r4_r5_LSL_7),
    kInstruction_mov_cc_r4_r5_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r7_r5_LSL_7),
    kInstruction_mov_vc_r7_r5_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r0_r5_LSL_8),
    kInstruction_mov_eq_r0_r5_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r1_r6_LSL_30),
    kInstruction_mov_eq_r1_r6_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r3_r6_LSL_7),
    kInstruction_mov_vs_r3_r6_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r3_r6_LSL_17),
    kInstruction_mov_ge_r3_r6_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r4_r4_LSL_9),
    kInstruction_mov_vs_r4_r4_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r4_r3_LSL_4),
    kInstruction_mov_cc_r4_r3_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r7_r1_LSL_26),
    kInstruction_mov_mi_r7_r1_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r7_r6_LSL_11),
    kInstruction_mov_le_r7_r6_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r7_r5_LSL_5),
    kInstruction_mov_mi_r7_r5_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r0_r4_LSL_9),
    kInstruction_mov_cs_r0_r4_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r1_r1_LSL_28),
    kInstruction_mov_ge_r1_r1_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r4_r6_LSL_30),
    kInstruction_mov_pl_r4_r6_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r5_r1_LSL_30),
    kInstruction_mov_eq_r5_r1_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r5_r6_LSL_31),
    kInstruction_mov_pl_r5_r6_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r3_r0_LSL_16),
    kInstruction_mov_hi_r3_r0_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r3_r2_LSL_24),
    kInstruction_mov_eq_r3_r2_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r3_r5_LSL_25),
    kInstruction_mov_mi_r3_r5_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r4_r1_LSL_25),
    kInstruction_mov_vs_r4_r1_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r5_r3_LSL_24),
    kInstruction_mov_cc_r5_r3_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r0_r6_LSL_1),
    kInstruction_mov_gt_r0_r6_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r0_r4_LSL_11),
    kInstruction_mov_cs_r0_r4_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r0_r2_LSL_19),
    kInstruction_mov_hi_r0_r2_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r1_r6_LSL_4),
    kInstruction_mov_ne_r1_r6_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r4_r4_LSL_5),
    kInstruction_mov_vc_r4_r4_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r4_r0_LSL_18),
    kInstruction_mov_hi_r4_r0_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r3_r0_LSL_11),
    kInstruction_mov_cs_r3_r0_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r6_r1_LSL_30),
    kInstruction_mov_ls_r6_r1_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r4_r0_LSL_3),
    kInstruction_mov_ne_r4_r0_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r6_r4_LSL_13),
    kInstruction_mov_ls_r6_r4_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r0_r4_LSL_31),
    kInstruction_mov_gt_r0_r4_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r6_r3_LSL_10),
    kInstruction_mov_eq_r6_r3_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r2_r6_LSL_11),
    kInstruction_mov_ge_r2_r6_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r2_r4_LSL_14),
    kInstruction_mov_vs_r2_r4_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r2_r5_LSL_27),
    kInstruction_mov_gt_r2_r5_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r0_r5_LSL_28),
    kInstruction_mov_gt_r0_r5_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r4_r6_LSL_1),
    kInstruction_mov_vs_r4_r6_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r0_r3_LSL_29),
    kInstruction_mov_ls_r0_r3_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r4_r4_LSL_16),
    kInstruction_mov_lt_r4_r4_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r6_r1_LSL_11),
    kInstruction_mov_cc_r6_r1_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r6_r5_LSL_3),
    kInstruction_mov_vc_r6_r5_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r6_r7_LSL_30),
    kInstruction_mov_eq_r6_r7_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r0_r1_LSL_29),
    kInstruction_mov_ls_r0_r1_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r2_r2_LSL_8),
    kInstruction_mov_ls_r2_r2_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r2_r6_LSL_12),
    kInstruction_mov_gt_r2_r6_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r5_r1_LSL_25),
    kInstruction_mov_ge_r5_r1_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r3_r4_LSL_23),
    kInstruction_mov_gt_r3_r4_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r6_r3_LSL_13),
    kInstruction_mov_eq_r6_r3_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r7_r2_LSL_2),
    kInstruction_mov_ne_r7_r2_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r6_r6_LSL_24),
    kInstruction_mov_pl_r6_r6_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r5_r0_LSL_21),
    kInstruction_mov_mi_r5_r0_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r6_r5_LSL_10),
    kInstruction_mov_le_r6_r5_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r2_r6_LSL_5),
    kInstruction_mov_vc_r2_r6_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r6_r2_LSL_9),
    kInstruction_mov_vc_r6_r2_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r5_r6_LSL_8),
    kInstruction_mov_mi_r5_r6_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r2_r5_LSL_16),
    kInstruction_mov_cs_r2_r5_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r4_r2_LSL_6),
    kInstruction_mov_cs_r4_r2_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r7_r4_LSL_1),
    kInstruction_mov_lt_r7_r4_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r3_r4_LSL_14),
    kInstruction_mov_ls_r3_r4_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r6_r4_LSL_9),
    kInstruction_mov_cs_r6_r4_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r6_r5_LSL_3),
    kInstruction_mov_pl_r6_r5_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r3_r2_LSL_17),
    kInstruction_mov_hi_r3_r2_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r0_r5_LSL_1),
    kInstruction_mov_le_r0_r5_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r0_r0_LSL_12),
    kInstruction_mov_pl_r0_r0_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r5_r0_LSL_9),
    kInstruction_mov_ne_r5_r0_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r1_r6_LSL_6),
    kInstruction_mov_ls_r1_r6_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r1_r2_LSL_15),
    kInstruction_mov_mi_r1_r2_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r6_r4_LSL_22),
    kInstruction_mov_hi_r6_r4_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r0_r4_LSL_29),
    kInstruction_mov_ls_r0_r4_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r1_r5_LSL_1),
    kInstruction_mov_vs_r1_r5_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r3_r2_LSL_24),
    kInstruction_mov_vs_r3_r2_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r6_r7_LSL_7),
    kInstruction_mov_cc_r6_r7_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r4_r0_LSL_27),
    kInstruction_mov_vc_r4_r0_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r0_r1_LSL_13),
    kInstruction_mov_vs_r0_r1_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r1_r5_LSL_16),
    kInstruction_mov_lt_r1_r5_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r1_r3_LSL_10),
    kInstruction_mov_cc_r1_r3_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r0_r3_LSL_13),
    kInstruction_mov_eq_r0_r3_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r0_r7_LSL_3),
    kInstruction_mov_ls_r0_r7_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r7_r7_LSL_16),
    kInstruction_mov_mi_r7_r7_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r5_r2_LSL_7),
    kInstruction_mov_le_r5_r2_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r6_r4_LSL_22),
    kInstruction_mov_ne_r6_r4_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r0_r0_LSL_14),
    kInstruction_mov_eq_r0_r0_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r5_r2_LSL_3),
    kInstruction_mov_ne_r5_r2_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r0_r6_LSL_29),
    kInstruction_mov_eq_r0_r6_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r4_r2_LSL_23),
    kInstruction_mov_vs_r4_r2_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r1_r5_LSL_15),
    kInstruction_mov_ne_r1_r5_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r3_r6_LSL_30),
    kInstruction_mov_ne_r3_r6_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r7_r2_LSL_17),
    kInstruction_mov_hi_r7_r2_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r4_r0_LSL_7),
    kInstruction_mov_vs_r4_r0_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r5_r0_LSL_28),
    kInstruction_mov_vc_r5_r0_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r6_r5_LSL_20),
    kInstruction_mov_lt_r6_r5_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r3_r0_LSL_16),
    kInstruction_mov_cs_r3_r0_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r0_r5_LSL_19),
    kInstruction_mov_cs_r0_r5_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r1_r1_LSL_13),
    kInstruction_mov_ge_r1_r1_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r5_r4_LSL_14),
    kInstruction_mov_cc_r5_r4_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r3_r5_LSL_8),
    kInstruction_mov_hi_r3_r5_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r7_r5_LSL_1),
    kInstruction_mov_vc_r7_r5_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r4_r3_LSL_1),
    kInstruction_mov_cc_r4_r3_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r3_r6_LSL_8),
    kInstruction_mov_cs_r3_r6_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r3_r1_LSL_30),
    kInstruction_mov_pl_r3_r1_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r3_r1_LSL_31),
    kInstruction_mov_cc_r3_r1_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r3_r1_LSL_9),
    kInstruction_mov_ne_r3_r1_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r0_r7_LSL_9),
    kInstruction_mov_cc_r0_r7_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r6_r4_LSL_13),
    kInstruction_mov_mi_r6_r4_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r3_r0_LSL_7),
    kInstruction_mov_le_r3_r0_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r6_r7_LSL_18),
    kInstruction_mov_hi_r6_r7_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r3_r3_LSL_22),
    kInstruction_mov_mi_r3_r3_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r4_r5_LSL_2),
    kInstruction_mov_gt_r4_r5_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r4_r0_LSL_13),
    kInstruction_mov_le_r4_r0_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r4_r1_LSL_14),
    kInstruction_mov_lt_r4_r1_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r5_r5_LSL_4),
    kInstruction_mov_lt_r5_r5_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r7_r7_LSL_29),
    kInstruction_mov_gt_r7_r7_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r7_r3_LSL_12),
    kInstruction_mov_le_r7_r3_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r0_r0_LSL_15),
    kInstruction_mov_vc_r0_r0_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r2_r6_LSL_11),
    kInstruction_mov_ne_r2_r6_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r2_r0_LSL_29),
    kInstruction_mov_hi_r2_r0_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r0_r4_LSL_22),
    kInstruction_mov_eq_r0_r4_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r3_r4_LSL_30),
    kInstruction_mov_cs_r3_r4_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r1_r2_LSL_19),
    kInstruction_mov_hi_r1_r2_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r7_r2_LSL_31),
    kInstruction_mov_ne_r7_r2_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r5_r7_LSL_9),
    kInstruction_mov_pl_r5_r7_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r2_r2_LSL_28),
    kInstruction_mov_mi_r2_r2_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r3_r0_LSL_16),
    kInstruction_mov_ls_r3_r0_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r5_r4_LSL_11),
    kInstruction_mov_ge_r5_r4_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r0_r1_LSL_15),
    kInstruction_mov_gt_r0_r1_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r7_r4_LSL_29),
    kInstruction_mov_ls_r7_r4_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r4_r6_LSL_21),
    kInstruction_mov_ne_r4_r6_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r2_r5_LSL_2),
    kInstruction_mov_eq_r2_r5_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r3_r2_LSL_15),
    kInstruction_mov_le_r3_r2_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r2_r7_LSL_5),
    kInstruction_mov_cc_r2_r7_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r4_r7_LSL_4),
    kInstruction_mov_lt_r4_r7_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r6_r5_LSL_28),
    kInstruction_mov_cc_r6_r5_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r4_r1_LSL_27),
    kInstruction_mov_pl_r4_r1_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r2_r5_LSL_22),
    kInstruction_mov_vs_r2_r5_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r2_r7_LSL_25),
    kInstruction_mov_eq_r2_r7_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r1_r2_LSL_8),
    kInstruction_mov_pl_r1_r2_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r4_r0_LSL_15),
    kInstruction_mov_ge_r4_r0_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r7_r5_LSL_3),
    kInstruction_mov_vc_r7_r5_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r2_r7_LSL_19),
    kInstruction_mov_vc_r2_r7_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r4_r0_LSL_31),
    kInstruction_mov_pl_r4_r0_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r3_r1_LSL_21),
    kInstruction_mov_le_r3_r1_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r3_r7_LSL_5),
    kInstruction_mov_cs_r3_r7_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r5_r0_LSL_24),
    kInstruction_mov_cs_r5_r0_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r5_r2_LSL_21),
    kInstruction_mov_vc_r5_r2_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r1_r3_LSL_8),
    kInstruction_mov_vc_r1_r3_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r1_r3_LSL_8),
    kInstruction_mov_ne_r1_r3_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r7_r0_LSL_28),
    kInstruction_mov_pl_r7_r0_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r1_r0_LSL_15),
    kInstruction_mov_ge_r1_r0_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r4_r6_LSL_13),
    kInstruction_mov_vc_r4_r6_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r4_r2_LSL_16),
    kInstruction_mov_le_r4_r2_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r6_r0_LSL_5),
    kInstruction_mov_vc_r6_r0_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r0_r2_LSL_16),
    kInstruction_mov_cs_r0_r2_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r7_r0_LSL_11),
    kInstruction_mov_ne_r7_r0_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r3_r4_LSL_18),
    kInstruction_mov_lt_r3_r4_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r5_r4_LSL_23),
    kInstruction_mov_le_r5_r4_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r4_r3_LSL_9),
    kInstruction_mov_hi_r4_r3_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r5_r3_LSL_20),
    kInstruction_mov_cs_r5_r3_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r4_r0_LSL_18),
    kInstruction_mov_le_r4_r0_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r3_r5_LSL_1),
    kInstruction_mov_ls_r3_r5_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r2_r5_LSL_7),
    kInstruction_mov_ge_r2_r5_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r3_r6_LSL_12),
    kInstruction_mov_vs_r3_r6_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r5_r3_LSL_15),
    kInstruction_mov_gt_r5_r3_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r4_r0_LSL_5),
    kInstruction_mov_mi_r4_r0_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r4_r7_LSL_5),
    kInstruction_mov_vc_r4_r7_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r0_r5_LSL_24),
    kInstruction_mov_cs_r0_r5_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r7_r7_LSL_19),
    kInstruction_mov_vs_r7_r7_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r5_r4_LSL_27),
    kInstruction_mov_mi_r5_r4_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r3_r7_LSL_1),
    kInstruction_mov_ne_r3_r7_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r4_r7_LSL_8),
    kInstruction_mov_ne_r4_r7_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r5_r4_LSL_21),
    kInstruction_mov_vs_r5_r4_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r0_r0_LSL_20),
    kInstruction_mov_ls_r0_r0_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r5_r1_LSL_30),
    kInstruction_mov_gt_r5_r1_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r4_r5_LSL_4),
    kInstruction_mov_lt_r4_r5_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r7_r6_LSL_10),
    kInstruction_mov_ge_r7_r6_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r1_r6_LSL_10),
    kInstruction_mov_cc_r1_r6_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r0_r5_LSL_23),
    kInstruction_mov_ls_r0_r5_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r4_r2_LSL_19),
    kInstruction_mov_ge_r4_r2_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r0_r7_LSL_12),
    kInstruction_mov_cc_r0_r7_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r0_r2_LSL_27),
    kInstruction_mov_ls_r0_r2_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r2_r7_LSL_13),
    kInstruction_mov_eq_r2_r7_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r6_r7_LSL_18),
    kInstruction_mov_cc_r6_r7_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r6_r1_LSL_18),
    kInstruction_mov_cc_r6_r1_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r4_r6_LSL_3),
    kInstruction_mov_gt_r4_r6_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r2_r4_LSL_26),
    kInstruction_mov_cc_r2_r4_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r6_r5_LSL_31),
    kInstruction_mov_mi_r6_r5_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r5_r3_LSL_24),
    kInstruction_mov_cs_r5_r3_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r2_r4_LSL_7),
    kInstruction_mov_gt_r2_r4_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r5_r3_LSL_6),
    kInstruction_mov_mi_r5_r3_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r2_r5_LSL_23),
    kInstruction_mov_lt_r2_r5_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r6_r4_LSL_7),
    kInstruction_mov_ls_r6_r4_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r0_r6_LSL_24),
    kInstruction_mov_eq_r0_r6_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r2_r6_LSL_2),
    kInstruction_mov_hi_r2_r6_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r1_r4_LSL_29),
    kInstruction_mov_eq_r1_r4_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r2_r0_LSL_10),
    kInstruction_mov_cs_r2_r0_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r3_r4_LSL_1),
    kInstruction_mov_cs_r3_r4_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r3_r7_LSL_21),
    kInstruction_mov_vs_r3_r7_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r3_r2_LSL_3),
    kInstruction_mov_lt_r3_r2_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r1_r3_LSL_14),
    kInstruction_mov_le_r1_r3_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r2_r3_LSL_23),
    kInstruction_mov_vc_r2_r3_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r2_r0_LSL_12),
    kInstruction_mov_pl_r2_r0_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r2_r6_LSL_27),
    kInstruction_mov_ge_r2_r6_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r6_r3_LSL_8),
    kInstruction_mov_ls_r6_r3_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r4_r1_LSL_31),
    kInstruction_mov_lt_r4_r1_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r1_r7_LSL_24),
    kInstruction_mov_le_r1_r7_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r2_r2_LSL_11),
    kInstruction_mov_mi_r2_r2_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r4_r1_LSL_27),
    kInstruction_mov_cc_r4_r1_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r7_r2_LSL_15),
    kInstruction_mov_ne_r7_r2_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r2_r4_LSL_10),
    kInstruction_mov_cc_r2_r4_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r3_r6_LSL_28),
    kInstruction_mov_lt_r3_r6_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r5_r1_LSL_1),
    kInstruction_mov_vc_r5_r1_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r7_r1_LSL_21),
    kInstruction_mov_ls_r7_r1_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r0_r0_LSL_20),
    kInstruction_mov_eq_r0_r0_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r7_r0_LSL_16),
    kInstruction_mov_ne_r7_r0_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r6_r1_LSL_16),
    kInstruction_mov_hi_r6_r1_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r6_r5_LSL_15),
    kInstruction_mov_mi_r6_r5_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r5_r3_LSL_1),
    kInstruction_mov_ge_r5_r3_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r3_r6_LSL_10),
    kInstruction_mov_cs_r3_r6_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r1_r3_LSL_7),
    kInstruction_mov_vs_r1_r3_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r3_r2_LSL_31),
    kInstruction_mov_hi_r3_r2_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r3_r0_LSL_22),
    kInstruction_mov_vc_r3_r0_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r0_r7_LSL_13),
    kInstruction_mov_cc_r0_r7_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r0_r1_LSL_26),
    kInstruction_mov_mi_r0_r1_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r1_r5_LSL_2),
    kInstruction_mov_eq_r1_r5_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r2_r4_LSL_22),
    kInstruction_mov_eq_r2_r4_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r4_r2_LSL_7),
    kInstruction_mov_gt_r4_r2_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r4_r3_LSL_13),
    kInstruction_mov_eq_r4_r3_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r0_r2_LSL_6),
    kInstruction_mov_pl_r0_r2_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r1_r4_LSL_11),
    kInstruction_mov_eq_r1_r4_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r1_r7_LSL_15),
    kInstruction_mov_vs_r1_r7_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r3_r2_LSL_2),
    kInstruction_mov_eq_r3_r2_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r1_r1_LSL_8),
    kInstruction_mov_ge_r1_r1_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r2_r6_LSL_7),
    kInstruction_mov_vc_r2_r6_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r0_r4_LSL_28),
    kInstruction_mov_pl_r0_r4_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r7_r6_LSL_20),
    kInstruction_mov_le_r7_r6_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r2_r3_LSL_14),
    kInstruction_mov_hi_r2_r3_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r1_r3_LSL_5),
    kInstruction_mov_mi_r1_r3_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r7_r0_LSL_13),
    kInstruction_mov_vc_r7_r0_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r6_r2_LSL_21),
    kInstruction_mov_cc_r6_r2_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r5_r3_LSL_6),
    kInstruction_mov_le_r5_r3_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r3_r7_LSL_1),
    kInstruction_mov_eq_r3_r7_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r1_r0_LSL_28),
    kInstruction_mov_ge_r1_r0_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r3_r4_LSL_7),
    kInstruction_mov_mi_r3_r4_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r0_r4_LSL_13),
    kInstruction_mov_pl_r0_r4_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r0_r4_LSL_5),
    kInstruction_mov_lt_r0_r4_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r2_r2_LSL_1),
    kInstruction_mov_cc_r2_r2_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r6_r2_LSL_30),
    kInstruction_mov_pl_r6_r2_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r7_r0_LSL_24),
    kInstruction_mov_ls_r7_r0_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r4_r4_LSL_3),
    kInstruction_mov_cs_r4_r4_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r7_r0_LSL_19),
    kInstruction_mov_vs_r7_r0_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r2_r3_LSL_12),
    kInstruction_mov_cc_r2_r3_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r3_r7_LSL_7),
    kInstruction_mov_lt_r3_r7_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r5_r5_LSL_3),
    kInstruction_mov_mi_r5_r5_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r5_r5_LSL_18),
    kInstruction_mov_vc_r5_r5_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r5_r0_LSL_26),
    kInstruction_mov_vc_r5_r0_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r2_r5_LSL_6),
    kInstruction_mov_ls_r2_r5_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r1_r7_LSL_30),
    kInstruction_mov_ge_r1_r7_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r1_r2_LSL_23),
    kInstruction_mov_ls_r1_r2_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r3_r7_LSL_25),
    kInstruction_mov_hi_r3_r7_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r7_r0_LSL_26),
    kInstruction_mov_lt_r7_r0_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r4_r2_LSL_31),
    kInstruction_mov_vs_r4_r2_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r7_r2_LSL_9),
    kInstruction_mov_gt_r7_r2_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r2_r5_LSL_16),
    kInstruction_mov_le_r2_r5_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r6_r6_LSL_23),
    kInstruction_mov_ls_r6_r6_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r5_r5_LSL_5),
    kInstruction_mov_pl_r5_r5_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r3_r2_LSL_16),
    kInstruction_mov_gt_r3_r2_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r2_r3_LSL_11),
    kInstruction_mov_vs_r2_r3_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r5_r6_LSL_15),
    kInstruction_mov_cc_r5_r6_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r0_r0_LSL_29),
    kInstruction_mov_vc_r0_r0_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r5_r0_LSL_20),
    kInstruction_mov_vs_r5_r0_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r6_r3_LSL_6),
    kInstruction_mov_ls_r6_r3_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r6_r7_LSL_2),
    kInstruction_mov_cc_r6_r7_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r2_r2_LSL_14),
    kInstruction_mov_eq_r2_r2_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r5_r3_LSL_12),
    kInstruction_mov_ne_r5_r3_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r7_r2_LSL_20),
    kInstruction_mov_cc_r7_r2_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r5_r0_LSL_24),
    kInstruction_mov_lt_r5_r0_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r6_r7_LSL_21),
    kInstruction_mov_ge_r6_r7_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r6_r7_LSL_3),
    kInstruction_mov_ls_r6_r7_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r3_r6_LSL_7),
    kInstruction_mov_cs_r3_r6_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r0_r5_LSL_26),
    kInstruction_mov_ls_r0_r5_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r3_r0_LSL_6),
    kInstruction_mov_ne_r3_r0_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r3_r7_LSL_29),
    kInstruction_mov_gt_r3_r7_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r7_r3_LSL_5),
    kInstruction_mov_ne_r7_r3_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r2_r1_LSL_31),
    kInstruction_mov_vs_r2_r1_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r1_r2_LSL_27),
    kInstruction_mov_ne_r1_r2_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r5_r0_LSL_25),
    kInstruction_mov_pl_r5_r0_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r5_r0_LSL_14),
    kInstruction_mov_hi_r5_r0_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r5_r5_LSL_19),
    kInstruction_mov_vs_r5_r5_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r6_r6_LSL_10),
    kInstruction_mov_le_r6_r6_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r2_r1_LSL_3),
    kInstruction_mov_ne_r2_r1_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r2_r7_LSL_7),
    kInstruction_mov_gt_r2_r7_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r0_r6_LSL_31),
    kInstruction_mov_vs_r0_r6_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r4_r4_LSL_28),
    kInstruction_mov_ne_r4_r4_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r1_r0_LSL_14),
    kInstruction_mov_mi_r1_r0_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r0_r6_LSL_30),
    kInstruction_mov_lt_r0_r6_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r2_r4_LSL_10),
    kInstruction_mov_lt_r2_r4_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r0_r3_LSL_22),
    kInstruction_mov_lt_r0_r3_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r5_r5_LSL_28),
    kInstruction_mov_ls_r5_r5_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r4_r1_LSL_20),
    kInstruction_mov_pl_r4_r1_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r0_r1_LSL_14),
    kInstruction_mov_le_r0_r1_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r2_r1_LSL_8),
    kInstruction_mov_le_r2_r1_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r2_r1_LSL_20),
    kInstruction_mov_ls_r2_r1_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r4_r0_LSL_14),
    kInstruction_mov_le_r4_r0_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r7_r4_LSL_17),
    kInstruction_mov_lt_r7_r4_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r6_r1_LSL_3),
    kInstruction_mov_cc_r6_r1_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r6_r3_LSL_1),
    kInstruction_mov_pl_r6_r3_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r2_r2_LSL_3),
    kInstruction_mov_cc_r2_r2_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r2_r6_LSL_22),
    kInstruction_mov_ge_r2_r6_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r6_r0_LSL_23),
    kInstruction_mov_gt_r6_r0_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r5_r1_LSL_18),
    kInstruction_mov_ge_r5_r1_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r7_r0_LSL_11),
    kInstruction_mov_eq_r7_r0_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r2_r7_LSL_8),
    kInstruction_mov_mi_r2_r7_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r2_r7_LSL_19),
    kInstruction_mov_pl_r2_r7_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r3_r6_LSL_1),
    kInstruction_mov_mi_r3_r6_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r6_r5_LSL_29),
    kInstruction_mov_lt_r6_r5_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r0_r4_LSL_9),
    kInstruction_mov_eq_r0_r4_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r3_r1_LSL_7),
    kInstruction_mov_hi_r3_r1_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r5_r5_LSL_12),
    kInstruction_mov_cs_r5_r5_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r1_r5_LSL_24),
    kInstruction_mov_lt_r1_r5_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r6_r0_LSL_4),
    kInstruction_mov_cc_r6_r0_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r3_r0_LSL_15),
    kInstruction_mov_hi_r3_r0_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r7_r2_LSL_17),
    kInstruction_mov_gt_r7_r2_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r4_r0_LSL_4),
    kInstruction_mov_hi_r4_r0_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r5_r4_LSL_22),
    kInstruction_mov_cc_r5_r4_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r3_r1_LSL_13),
    kInstruction_mov_lt_r3_r1_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r4_r7_LSL_5),
    kInstruction_mov_le_r4_r7_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r0_r5_LSL_22),
    kInstruction_mov_cs_r0_r5_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r3_r6_LSL_1),
    kInstruction_mov_eq_r3_r6_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r3_r3_LSL_8),
    kInstruction_mov_vs_r3_r3_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r4_r4_LSL_17),
    kInstruction_mov_vs_r4_r4_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r0_r3_LSL_16),
    kInstruction_mov_ne_r0_r3_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r4_r7_LSL_4),
    kInstruction_mov_le_r4_r7_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r4_r6_LSL_2),
    kInstruction_mov_vs_r4_r6_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r7_r4_LSL_8),
    kInstruction_mov_pl_r7_r4_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r6_r2_LSL_2),
    kInstruction_mov_ls_r6_r2_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r2_r2_LSL_13),
    kInstruction_mov_cc_r2_r2_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r7_r5_LSL_7),
    kInstruction_mov_pl_r7_r5_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r3_r6_LSL_6),
    kInstruction_mov_pl_r3_r6_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r4_r5_LSL_3),
    kInstruction_mov_ne_r4_r5_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r5_r5_LSL_16),
    kInstruction_mov_lt_r5_r5_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r1_r2_LSL_23),
    kInstruction_mov_gt_r1_r2_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r1_r6_LSL_27),
    kInstruction_mov_vc_r1_r6_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r6_r0_LSL_3),
    kInstruction_mov_vc_r6_r0_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r1_r6_LSL_10),
    kInstruction_mov_vc_r1_r6_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r3_r7_LSL_21),
    kInstruction_mov_vc_r3_r7_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r5_r7_LSL_31),
    kInstruction_mov_lt_r5_r7_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r6_r1_LSL_11),
    kInstruction_mov_vs_r6_r1_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r7_r4_LSL_19),
    kInstruction_mov_gt_r7_r4_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r0_r4_LSL_22),
    kInstruction_mov_hi_r0_r4_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r2_r0_LSL_18),
    kInstruction_mov_hi_r2_r0_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r1_r3_LSL_12),
    kInstruction_mov_ls_r1_r3_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r0_r6_LSL_23),
    kInstruction_mov_hi_r0_r6_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r3_r5_LSL_22),
    kInstruction_mov_ne_r3_r5_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r4_r4_LSL_4),
    kInstruction_mov_ge_r4_r4_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r1_r6_LSL_25),
    kInstruction_mov_cc_r1_r6_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r2_r1_LSL_13),
    kInstruction_mov_vs_r2_r1_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r6_r7_LSL_6),
    kInstruction_mov_lt_r6_r7_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r5_r6_LSL_7),
    kInstruction_mov_gt_r5_r6_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r5_r1_LSL_27),
    kInstruction_mov_ge_r5_r1_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r2_r3_LSL_8),
    kInstruction_mov_vs_r2_r3_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r0_r3_LSL_4),
    kInstruction_mov_gt_r0_r3_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r5_r6_LSL_17),
    kInstruction_mov_vc_r5_r6_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r5_r2_LSL_23),
    kInstruction_mov_mi_r5_r2_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r3_r3_LSL_10),
    kInstruction_mov_cc_r3_r3_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r5_r3_LSL_18),
    kInstruction_mov_le_r5_r3_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r4_r0_LSL_9),
    kInstruction_mov_le_r4_r0_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r0_r1_LSL_10),
    kInstruction_mov_pl_r0_r1_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r6_r4_LSL_2),
    kInstruction_mov_ls_r6_r4_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r6_r6_LSL_5),
    kInstruction_mov_cc_r6_r6_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r2_r5_LSL_2),
    kInstruction_mov_pl_r2_r5_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r7_r0_LSL_3),
    kInstruction_mov_vs_r7_r0_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r7_r7_LSL_8),
    kInstruction_mov_vs_r7_r7_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r0_r7_LSL_5),
    kInstruction_mov_ge_r0_r7_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r3_r7_LSL_16),
    kInstruction_mov_gt_r3_r7_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r4_r2_LSL_22),
    kInstruction_mov_mi_r4_r2_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r1_r2_LSL_4),
    kInstruction_mov_le_r1_r2_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r5_r4_LSL_3),
    kInstruction_mov_mi_r5_r4_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r2_r3_LSL_26),
    kInstruction_mov_hi_r2_r3_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r6_r1_LSL_29),
    kInstruction_mov_ne_r6_r1_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r6_r0_LSL_29),
    kInstruction_mov_gt_r6_r0_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r3_r2_LSL_19),
    kInstruction_mov_pl_r3_r2_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r5_r2_LSL_5),
    kInstruction_mov_ls_r5_r2_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r4_r5_LSL_16),
    kInstruction_mov_ne_r4_r5_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r2_r1_LSL_2),
    kInstruction_mov_vc_r2_r1_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r0_r7_LSL_27),
    kInstruction_mov_ne_r0_r7_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r4_r2_LSL_30),
    kInstruction_mov_eq_r4_r2_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r7_r2_LSL_18),
    kInstruction_mov_hi_r7_r2_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r2_r0_LSL_21),
    kInstruction_mov_eq_r2_r0_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r1_r1_LSL_9),
    kInstruction_mov_hi_r1_r1_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r6_r6_LSL_10),
    kInstruction_mov_lt_r6_r6_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r4_r5_LSL_15),
    kInstruction_mov_vc_r4_r5_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r1_r1_LSL_12),
    kInstruction_mov_cc_r1_r1_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r0_r4_LSL_29),
    kInstruction_mov_le_r0_r4_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r6_r2_LSL_12),
    kInstruction_mov_cs_r6_r2_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r6_r2_LSL_21),
    kInstruction_mov_pl_r6_r2_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r4_r4_LSL_31),
    kInstruction_mov_le_r4_r4_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r3_r6_LSL_26),
    kInstruction_mov_cs_r3_r6_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r1_r2_LSL_2),
    kInstruction_mov_ne_r1_r2_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r0_r4_LSL_24),
    kInstruction_mov_vc_r0_r4_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r3_r4_LSL_9),
    kInstruction_mov_mi_r3_r4_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r5_r1_LSL_23),
    kInstruction_mov_pl_r5_r1_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r1_r6_LSL_31),
    kInstruction_mov_ls_r1_r6_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r6_r2_LSL_6),
    kInstruction_mov_mi_r6_r2_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r1_r3_LSL_27),
    kInstruction_mov_vc_r1_r3_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r6_r7_LSL_26),
    kInstruction_mov_vc_r6_r7_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r2_r6_LSL_10),
    kInstruction_mov_vs_r2_r6_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r5_r6_LSL_5),
    kInstruction_mov_ge_r5_r6_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r0_r5_LSL_19),
    kInstruction_mov_hi_r0_r5_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r4_r1_LSL_13),
    kInstruction_mov_eq_r4_r1_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r1_r3_LSL_24),
    kInstruction_mov_eq_r1_r3_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r1_r0_LSL_30),
    kInstruction_mov_hi_r1_r0_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r7_r0_LSL_11),
    kInstruction_mov_lt_r7_r0_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r1_r5_LSL_3),
    kInstruction_mov_le_r1_r5_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r2_r5_LSL_1),
    kInstruction_mov_gt_r2_r5_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r1_r0_LSL_6),
    kInstruction_mov_mi_r1_r0_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r1_r6_LSL_24),
    kInstruction_mov_le_r1_r6_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r4_r2_LSL_17),
    kInstruction_mov_vs_r4_r2_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r2_r3_LSL_20),
    kInstruction_mov_ls_r2_r3_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r0_r7_LSL_8),
    kInstruction_mov_pl_r0_r7_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r1_r3_LSL_19),
    kInstruction_mov_ne_r1_r3_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r4_r6_LSL_2),
    kInstruction_mov_gt_r4_r6_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r5_r0_LSL_1),
    kInstruction_mov_vc_r5_r0_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r1_r1_LSL_28),
    kInstruction_mov_le_r1_r1_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r2_r4_LSL_17),
    kInstruction_mov_vc_r2_r4_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r5_r2_LSL_25),
    kInstruction_mov_le_r5_r2_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r6_r3_LSL_22),
    kInstruction_mov_cc_r6_r3_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r3_r3_LSL_20),
    kInstruction_mov_cc_r3_r3_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r4_r5_LSL_20),
    kInstruction_mov_pl_r4_r5_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r2_r3_LSL_14),
    kInstruction_mov_cc_r2_r3_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r4_r0_LSL_8),
    kInstruction_mov_eq_r4_r0_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r5_r0_LSL_15),
    kInstruction_mov_lt_r5_r0_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r5_r6_LSL_25),
    kInstruction_mov_pl_r5_r6_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r5_r3_LSL_6),
    kInstruction_mov_ls_r5_r3_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r7_r0_LSL_7),
    kInstruction_mov_ge_r7_r0_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r0_r7_LSL_12),
    kInstruction_mov_le_r0_r7_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r4_r4_LSL_31),
    kInstruction_mov_hi_r4_r4_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r5_r1_LSL_22),
    kInstruction_mov_pl_r5_r1_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r0_r3_LSL_11),
    kInstruction_mov_cc_r0_r3_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r2_r4_LSL_9),
    kInstruction_mov_le_r2_r4_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r6_r2_LSL_17),
    kInstruction_mov_ne_r6_r2_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r1_r1_LSL_7),
    kInstruction_mov_vs_r1_r1_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r5_r7_LSL_2),
    kInstruction_mov_cc_r5_r7_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r7_r3_LSL_15),
    kInstruction_mov_le_r7_r3_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r7_r0_LSL_20),
    kInstruction_mov_ge_r7_r0_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r3_r4_LSL_2),
    kInstruction_mov_vc_r3_r4_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r5_r0_LSL_10),
    kInstruction_mov_eq_r5_r0_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r5_r6_LSL_4),
    kInstruction_mov_cc_r5_r6_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r4_r1_LSL_23),
    kInstruction_mov_ge_r4_r1_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r2_r1_LSL_18),
    kInstruction_mov_cs_r2_r1_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r1_r3_LSL_22),
    kInstruction_mov_gt_r1_r3_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r0_r3_LSL_7),
    kInstruction_mov_cc_r0_r3_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r1_r6_LSL_3),
    kInstruction_mov_cs_r1_r6_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r0_r4_LSL_14),
    kInstruction_mov_ne_r0_r4_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r0_r1_LSL_31),
    kInstruction_mov_ls_r0_r1_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r1_r2_LSL_4),
    kInstruction_mov_ge_r1_r2_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r1_r4_LSL_6),
    kInstruction_mov_ne_r1_r4_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r2_r6_LSL_3),
    kInstruction_mov_eq_r2_r6_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r3_r7_LSL_7),
    kInstruction_mov_gt_r3_r7_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r3_r6_LSL_25),
    kInstruction_mov_ls_r3_r6_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r2_r4_LSL_4),
    kInstruction_mov_lt_r2_r4_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r5_r6_LSL_24),
    kInstruction_mov_cs_r5_r6_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r6_r6_LSL_19),
    kInstruction_mov_pl_r6_r6_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r1_r0_LSL_10),
    kInstruction_mov_vs_r1_r0_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r6_r4_LSL_17),
    kInstruction_mov_cs_r6_r4_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r4_r3_LSL_5),
    kInstruction_mov_lt_r4_r3_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r6_r5_LSL_30),
    kInstruction_mov_cs_r6_r5_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r7_r4_LSL_27),
    kInstruction_mov_cs_r7_r4_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r2_r6_LSL_4),
    kInstruction_mov_lt_r2_r6_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r7_r7_LSL_16),
    kInstruction_mov_gt_r7_r7_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r1_r4_LSL_11),
    kInstruction_mov_mi_r1_r4_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r5_r6_LSL_21),
    kInstruction_mov_ls_r5_r6_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r1_r4_LSL_25),
    kInstruction_mov_le_r1_r4_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r7_r1_LSL_5),
    kInstruction_mov_hi_r7_r1_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r5_r1_LSL_15),
    kInstruction_mov_ls_r5_r1_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r7_r3_LSL_3),
    kInstruction_mov_ne_r7_r3_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r1_r0_LSL_1),
    kInstruction_mov_mi_r1_r0_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r5_r5_LSL_21),
    kInstruction_mov_ls_r5_r5_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r1_r0_LSL_11),
    kInstruction_mov_ls_r1_r0_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r1_r5_LSL_30),
    kInstruction_mov_cs_r1_r5_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r4_r1_LSL_19),
    kInstruction_mov_ge_r4_r1_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r0_r2_LSL_15),
    kInstruction_mov_pl_r0_r2_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r4_r5_LSL_10),
    kInstruction_mov_hi_r4_r5_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r3_r0_LSL_1),
    kInstruction_mov_pl_r3_r0_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r3_r0_LSL_20),
    kInstruction_mov_ge_r3_r0_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r6_r4_LSL_20),
    kInstruction_mov_pl_r6_r4_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r6_r0_LSL_16),
    kInstruction_mov_le_r6_r0_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r6_r5_LSL_14),
    kInstruction_mov_vc_r6_r5_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r6_r2_LSL_20),
    kInstruction_mov_mi_r6_r2_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r1_r3_LSL_18),
    kInstruction_mov_lt_r1_r3_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r1_r2_LSL_5),
    kInstruction_mov_eq_r1_r2_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r1_r0_LSL_23),
    kInstruction_mov_ge_r1_r0_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r2_r5_LSL_7),
    kInstruction_mov_cc_r2_r5_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r1_r7_LSL_3),
    kInstruction_mov_cc_r1_r7_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r2_r5_LSL_29),
    kInstruction_mov_mi_r2_r5_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r5_r5_LSL_9),
    kInstruction_mov_cc_r5_r5_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r3_r2_LSL_1),
    kInstruction_mov_cs_r3_r2_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r5_r3_LSL_15),
    kInstruction_mov_ge_r5_r3_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r7_r1_LSL_10),
    kInstruction_mov_gt_r7_r1_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r4_r6_LSL_15),
    kInstruction_mov_ls_r4_r6_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r3_r1_LSL_15),
    kInstruction_mov_gt_r3_r1_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r5_r5_LSL_30),
    kInstruction_mov_cs_r5_r5_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r4_r2_LSL_19),
    kInstruction_mov_eq_r4_r2_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r0_r5_LSL_21),
    kInstruction_mov_vc_r0_r5_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r2_r0_LSL_3),
    kInstruction_mov_cs_r2_r0_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r1_r2_LSL_20),
    kInstruction_mov_ne_r1_r2_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r6_r0_LSL_28),
    kInstruction_mov_ne_r6_r0_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r7_r3_LSL_7),
    kInstruction_mov_ne_r7_r3_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r5_r4_LSL_29),
    kInstruction_mov_mi_r5_r4_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r3_r5_LSL_18),
    kInstruction_mov_vs_r3_r5_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r1_r2_LSL_1),
    kInstruction_mov_mi_r1_r2_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r4_r7_LSL_21),
    kInstruction_mov_hi_r4_r7_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r6_r1_LSL_21),
    kInstruction_mov_ge_r6_r1_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r4_r5_LSL_20),
    kInstruction_mov_lt_r4_r5_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r6_r7_LSL_2),
    kInstruction_mov_vs_r6_r7_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r1_r6_LSL_30),
    kInstruction_mov_hi_r1_r6_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r0_r2_LSL_7),
    kInstruction_mov_ls_r0_r2_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r7_r3_LSL_22),
    kInstruction_mov_cc_r7_r3_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r1_r1_LSL_5),
    kInstruction_mov_vs_r1_r1_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r5_r4_LSL_19),
    kInstruction_mov_ls_r5_r4_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r6_r2_LSL_14),
    kInstruction_mov_pl_r6_r2_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r1_r0_LSL_1),
    kInstruction_mov_cs_r1_r0_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r6_r1_LSL_2),
    kInstruction_mov_mi_r6_r1_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r5_r1_LSL_26),
    kInstruction_mov_mi_r5_r1_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r0_r7_LSL_15),
    kInstruction_mov_ne_r0_r7_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r2_r7_LSL_15),
    kInstruction_mov_cs_r2_r7_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r7_r2_LSL_24),
    kInstruction_mov_ne_r7_r2_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r1_r2_LSL_8),
    kInstruction_mov_le_r1_r2_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r3_r3_LSL_3),
    kInstruction_mov_eq_r3_r3_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r1_r4_LSL_23),
    kInstruction_mov_mi_r1_r4_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r3_r4_LSL_26),
    kInstruction_mov_pl_r3_r4_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r3_r2_LSL_14),
    kInstruction_mov_lt_r3_r2_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r7_r0_LSL_6),
    kInstruction_mov_ne_r7_r0_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r2_r0_LSL_26),
    kInstruction_mov_pl_r2_r0_LSL_26,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r1_r0_LSL_22),
    kInstruction_mov_cc_r1_r0_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r7_r3_LSL_30),
    kInstruction_mov_mi_r7_r3_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r0_r4_LSL_30),
    kInstruction_mov_ge_r0_r4_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r0_r4_LSL_14),
    kInstruction_mov_gt_r0_r4_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r7_r1_LSL_21),
    kInstruction_mov_cc_r7_r1_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r7_r1_LSL_4),
    kInstruction_mov_lt_r7_r1_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r0_r2_LSL_23),
    kInstruction_mov_ls_r0_r2_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r6_r6_LSL_9),
    kInstruction_mov_vs_r6_r6_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r4_r6_LSL_14),
    kInstruction_mov_eq_r4_r6_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r2_r0_LSL_2),
    kInstruction_mov_gt_r2_r0_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r2_r4_LSL_27),
    kInstruction_mov_lt_r2_r4_LSL_27,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r7_r2_LSL_14),
    kInstruction_mov_cs_r7_r2_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vc_r7_r1_LSL_29),
    kInstruction_mov_vc_r7_r1_LSL_29,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r1_r2_LSL_10),
    kInstruction_mov_cc_r1_r2_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r3_r6_LSL_11),
    kInstruction_mov_lt_r3_r6_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r7_r6_LSL_6),
    kInstruction_mov_le_r7_r6_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r6_r1_LSL_16),
    kInstruction_mov_lt_r6_r1_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r5_r3_LSL_21),
    kInstruction_mov_pl_r5_r3_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r1_r7_LSL_20),
    kInstruction_mov_cs_r1_r7_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r6_r1_LSL_31),
    kInstruction_mov_le_r6_r1_LSL_31,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r7_r7_LSL_2),
    kInstruction_mov_cc_r7_r7_LSL_2,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r5_r7_LSL_28),
    kInstruction_mov_pl_r5_r7_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r6_r4_LSL_5),
    kInstruction_mov_pl_r6_r4_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r1_r1_LSL_20),
    kInstruction_mov_eq_r1_r1_LSL_20,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r1_r1_LSL_11),
    kInstruction_mov_ls_r1_r1_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r2_r7_LSL_30),
    kInstruction_mov_eq_r2_r7_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r4_r2_LSL_25),
    kInstruction_mov_ls_r4_r2_LSL_25,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r5_r6_LSL_30),
    kInstruction_mov_hi_r5_r6_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r5_r7_LSL_9),
    kInstruction_mov_ls_r5_r7_LSL_9,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r7_r7_LSL_16),
    kInstruction_mov_lt_r7_r7_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r0_r4_LSL_6),
    kInstruction_mov_mi_r0_r4_LSL_6,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r3_r0_LSL_3),
    kInstruction_mov_vs_r3_r0_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r2_r0_LSL_14),
    kInstruction_mov_lt_r2_r0_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r6_r5_LSL_13),
    kInstruction_mov_ls_r6_r5_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r6_r0_LSL_7),
    kInstruction_mov_eq_r6_r0_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ge_r2_r0_LSL_18),
    kInstruction_mov_ge_r2_r0_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r5_r3_LSL_30),
    kInstruction_mov_mi_r5_r3_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r1_r5_LSL_4),
    kInstruction_mov_mi_r1_r5_LSL_4,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r4_r1_LSL_15),
    kInstruction_mov_pl_r4_r1_LSL_15,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r7_r6_LSL_7),
    kInstruction_mov_ls_r7_r6_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r2_r4_LSL_16),
    kInstruction_mov_eq_r2_r4_LSL_16,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r5_r3_LSL_23),
    kInstruction_mov_eq_r5_r3_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r4_r6_LSL_23),
    kInstruction_mov_le_r4_r6_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ls_r3_r3_LSL_13),
    kInstruction_mov_ls_r3_r3_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r2_r5_LSL_28),
    kInstruction_mov_hi_r2_r5_LSL_28,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r1_r2_LSL_8),
    kInstruction_mov_gt_r1_r2_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r1_r2_LSL_11),
    kInstruction_mov_cs_r1_r2_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r3_r4_LSL_23),
    kInstruction_mov_vs_r3_r4_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r4_r0_LSL_12),
    kInstruction_mov_cs_r4_r0_LSL_12,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r5_r6_LSL_5),
    kInstruction_mov_hi_r5_r6_LSL_5,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r0_r3_LSL_10),
    kInstruction_mov_cs_r0_r3_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r2_r2_LSL_30),
    kInstruction_mov_lt_r2_r2_LSL_30,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r4_r2_LSL_21),
    kInstruction_mov_eq_r4_r2_LSL_21,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r0_r2_LSL_10),
    kInstruction_mov_mi_r0_r2_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r5_r0_LSL_11),
    kInstruction_mov_pl_r5_r0_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r1_r7_LSL_19),
    kInstruction_mov_hi_r1_r7_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r3_r5_LSL_7),
    kInstruction_mov_le_r3_r5_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r3_r5_LSL_19),
    kInstruction_mov_hi_r3_r5_LSL_19,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r5_r3_LSL_14),
    kInstruction_mov_ne_r5_r3_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r3_r5_LSL_1),
    kInstruction_mov_ne_r3_r5_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r7_r4_LSL_23),
    kInstruction_mov_gt_r7_r4_LSL_23,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r2_r3_LSL_3),
    kInstruction_mov_cc_r2_r3_LSL_3,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r7_r7_LSL_22),
    kInstruction_mov_cc_r7_r7_LSL_22,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cs_r7_r0_LSL_10),
    kInstruction_mov_cs_r7_r0_LSL_10,
  },
  {
    ARRAY_SIZE(kInstruction_mov_le_r2_r7_LSL_7),
    kInstruction_mov_le_r2_r7_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_cc_r5_r6_LSL_7),
    kInstruction_mov_cc_r5_r6_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_ne_r0_r1_LSL_1),
    kInstruction_mov_ne_r0_r1_LSL_1,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r3_r6_LSL_8),
    kInstruction_mov_mi_r3_r6_LSL_8,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r0_r7_LSL_14),
    kInstruction_mov_mi_r0_r7_LSL_14,
  },
  {
    ARRAY_SIZE(kInstruction_mov_hi_r0_r5_LSL_17),
    kInstruction_mov_hi_r0_r5_LSL_17,
  },
  {
    ARRAY_SIZE(kInstruction_mov_lt_r2_r5_LSL_7),
    kInstruction_mov_lt_r2_r5_LSL_7,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r1_r1_LSL_18),
    kInstruction_mov_mi_r1_r1_LSL_18,
  },
  {
    ARRAY_SIZE(kInstruction_mov_pl_r3_r2_LSL_13),
    kInstruction_mov_pl_r3_r2_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_gt_r6_r1_LSL_13),
    kInstruction_mov_gt_r6_r1_LSL_13,
  },
  {
    ARRAY_SIZE(kInstruction_mov_eq_r2_r6_LSL_11),
    kInstruction_mov_eq_r2_r6_LSL_11,
  },
  {
    ARRAY_SIZE(kInstruction_mov_vs_r2_r6_LSL_24),
    kInstruction_mov_vs_r2_r6_LSL_24,
  },
  {
    ARRAY_SIZE(kInstruction_mov_mi_r5_r0_LSL_15),
    kInstruction_mov_mi_r5_r0_LSL_15,
  },
};

#endif  // VIXL_ASSEMBLER_COND_RD_OPERAND_RN_SHIFT_AMOUNT_1TO31_IN_IT_BLOCK_MOV_T32_H_
